/* Capstone Disassembly Engine, https://www.capstone-engine.org */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2022, */
/*    Rot127 <unisono@quyllur.org> 2022-2023 */
/* Automatically generated file by Capstone's LLVM TableGen Disassembler Backend. */

/* LLVM-commit: 083d57d0731afc1746680d828bdfe2fa41f62a61 */
/* LLVM-tag: llvmorg-3.0.0-2-g083d57d0731a */

/* Do not edit. */

/* Capstone's LLVM TableGen Backends: */
/* https://github.com/capstone-engine/llvm-capstone */

#ifdef GET_INSTRINFO_ENUM
#undef GET_INSTRINFO_ENUM

  enum {
    Alpha_PHI	= 0,
    Alpha_INLINEASM	= 1,
    Alpha_INLINEASM_BR	= 2,
    Alpha_CFI_INSTRUCTION	= 3,
    Alpha_EH_LABEL	= 4,
    Alpha_GC_LABEL	= 5,
    Alpha_ANNOTATION_LABEL	= 6,
    Alpha_KILL	= 7,
    Alpha_EXTRACT_SUBREG	= 8,
    Alpha_INSERT_SUBREG	= 9,
    Alpha_IMPLICIT_DEF	= 10,
    Alpha_SUBREG_TO_REG	= 11,
    Alpha_COPY_TO_REGCLASS	= 12,
    Alpha_DBG_VALUE	= 13,
    Alpha_DBG_VALUE_LIST	= 14,
    Alpha_DBG_INSTR_REF	= 15,
    Alpha_DBG_PHI	= 16,
    Alpha_DBG_LABEL	= 17,
    Alpha_REG_SEQUENCE	= 18,
    Alpha_COPY	= 19,
    Alpha_BUNDLE	= 20,
    Alpha_LIFETIME_START	= 21,
    Alpha_LIFETIME_END	= 22,
    Alpha_PSEUDO_PROBE	= 23,
    Alpha_ARITH_FENCE	= 24,
    Alpha_STACKMAP	= 25,
    Alpha_FENTRY_CALL	= 26,
    Alpha_PATCHPOINT	= 27,
    Alpha_LOAD_STACK_GUARD	= 28,
    Alpha_PREALLOCATED_SETUP	= 29,
    Alpha_PREALLOCATED_ARG	= 30,
    Alpha_STATEPOINT	= 31,
    Alpha_LOCAL_ESCAPE	= 32,
    Alpha_FAULTING_OP	= 33,
    Alpha_PATCHABLE_OP	= 34,
    Alpha_PATCHABLE_FUNCTION_ENTER	= 35,
    Alpha_PATCHABLE_RET	= 36,
    Alpha_PATCHABLE_FUNCTION_EXIT	= 37,
    Alpha_PATCHABLE_TAIL_CALL	= 38,
    Alpha_PATCHABLE_EVENT_CALL	= 39,
    Alpha_PATCHABLE_TYPED_EVENT_CALL	= 40,
    Alpha_ICALL_BRANCH_FUNNEL	= 41,
    Alpha_MEMBARRIER	= 42,
    Alpha_G_ASSERT_SEXT	= 43,
    Alpha_G_ASSERT_ZEXT	= 44,
    Alpha_G_ASSERT_ALIGN	= 45,
    Alpha_G_ADD	= 46,
    Alpha_G_SUB	= 47,
    Alpha_G_MUL	= 48,
    Alpha_G_SDIV	= 49,
    Alpha_G_UDIV	= 50,
    Alpha_G_SREM	= 51,
    Alpha_G_UREM	= 52,
    Alpha_G_SDIVREM	= 53,
    Alpha_G_UDIVREM	= 54,
    Alpha_G_AND	= 55,
    Alpha_G_OR	= 56,
    Alpha_G_XOR	= 57,
    Alpha_G_IMPLICIT_DEF	= 58,
    Alpha_G_PHI	= 59,
    Alpha_G_FRAME_INDEX	= 60,
    Alpha_G_GLOBAL_VALUE	= 61,
    Alpha_G_EXTRACT	= 62,
    Alpha_G_UNMERGE_VALUES	= 63,
    Alpha_G_INSERT	= 64,
    Alpha_G_MERGE_VALUES	= 65,
    Alpha_G_BUILD_VECTOR	= 66,
    Alpha_G_BUILD_VECTOR_TRUNC	= 67,
    Alpha_G_CONCAT_VECTORS	= 68,
    Alpha_G_PTRTOINT	= 69,
    Alpha_G_INTTOPTR	= 70,
    Alpha_G_BITCAST	= 71,
    Alpha_G_FREEZE	= 72,
    Alpha_G_INTRINSIC_FPTRUNC_ROUND	= 73,
    Alpha_G_INTRINSIC_TRUNC	= 74,
    Alpha_G_INTRINSIC_ROUND	= 75,
    Alpha_G_INTRINSIC_LRINT	= 76,
    Alpha_G_INTRINSIC_ROUNDEVEN	= 77,
    Alpha_G_READCYCLECOUNTER	= 78,
    Alpha_G_LOAD	= 79,
    Alpha_G_SEXTLOAD	= 80,
    Alpha_G_ZEXTLOAD	= 81,
    Alpha_G_INDEXED_LOAD	= 82,
    Alpha_G_INDEXED_SEXTLOAD	= 83,
    Alpha_G_INDEXED_ZEXTLOAD	= 84,
    Alpha_G_STORE	= 85,
    Alpha_G_INDEXED_STORE	= 86,
    Alpha_G_ATOMIC_CMPXCHG_WITH_SUCCESS	= 87,
    Alpha_G_ATOMIC_CMPXCHG	= 88,
    Alpha_G_ATOMICRMW_XCHG	= 89,
    Alpha_G_ATOMICRMW_ADD	= 90,
    Alpha_G_ATOMICRMW_SUB	= 91,
    Alpha_G_ATOMICRMW_AND	= 92,
    Alpha_G_ATOMICRMW_NAND	= 93,
    Alpha_G_ATOMICRMW_OR	= 94,
    Alpha_G_ATOMICRMW_XOR	= 95,
    Alpha_G_ATOMICRMW_MAX	= 96,
    Alpha_G_ATOMICRMW_MIN	= 97,
    Alpha_G_ATOMICRMW_UMAX	= 98,
    Alpha_G_ATOMICRMW_UMIN	= 99,
    Alpha_G_ATOMICRMW_FADD	= 100,
    Alpha_G_ATOMICRMW_FSUB	= 101,
    Alpha_G_ATOMICRMW_FMAX	= 102,
    Alpha_G_ATOMICRMW_FMIN	= 103,
    Alpha_G_ATOMICRMW_UINC_WRAP	= 104,
    Alpha_G_ATOMICRMW_UDEC_WRAP	= 105,
    Alpha_G_FENCE	= 106,
    Alpha_G_BRCOND	= 107,
    Alpha_G_BRINDIRECT	= 108,
    Alpha_G_INVOKE_REGION_START	= 109,
    Alpha_G_INTRINSIC	= 110,
    Alpha_G_INTRINSIC_W_SIDE_EFFECTS	= 111,
    Alpha_G_ANYEXT	= 112,
    Alpha_G_TRUNC	= 113,
    Alpha_G_CONSTANT	= 114,
    Alpha_G_FCONSTANT	= 115,
    Alpha_G_VASTART	= 116,
    Alpha_G_VAARG	= 117,
    Alpha_G_SEXT	= 118,
    Alpha_G_SEXT_INREG	= 119,
    Alpha_G_ZEXT	= 120,
    Alpha_G_SHL	= 121,
    Alpha_G_LSHR	= 122,
    Alpha_G_ASHR	= 123,
    Alpha_G_FSHL	= 124,
    Alpha_G_FSHR	= 125,
    Alpha_G_ROTR	= 126,
    Alpha_G_ROTL	= 127,
    Alpha_G_ICMP	= 128,
    Alpha_G_FCMP	= 129,
    Alpha_G_SELECT	= 130,
    Alpha_G_UADDO	= 131,
    Alpha_G_UADDE	= 132,
    Alpha_G_USUBO	= 133,
    Alpha_G_USUBE	= 134,
    Alpha_G_SADDO	= 135,
    Alpha_G_SADDE	= 136,
    Alpha_G_SSUBO	= 137,
    Alpha_G_SSUBE	= 138,
    Alpha_G_UMULO	= 139,
    Alpha_G_SMULO	= 140,
    Alpha_G_UMULH	= 141,
    Alpha_G_SMULH	= 142,
    Alpha_G_UADDSAT	= 143,
    Alpha_G_SADDSAT	= 144,
    Alpha_G_USUBSAT	= 145,
    Alpha_G_SSUBSAT	= 146,
    Alpha_G_USHLSAT	= 147,
    Alpha_G_SSHLSAT	= 148,
    Alpha_G_SMULFIX	= 149,
    Alpha_G_UMULFIX	= 150,
    Alpha_G_SMULFIXSAT	= 151,
    Alpha_G_UMULFIXSAT	= 152,
    Alpha_G_SDIVFIX	= 153,
    Alpha_G_UDIVFIX	= 154,
    Alpha_G_SDIVFIXSAT	= 155,
    Alpha_G_UDIVFIXSAT	= 156,
    Alpha_G_FADD	= 157,
    Alpha_G_FSUB	= 158,
    Alpha_G_FMUL	= 159,
    Alpha_G_FMA	= 160,
    Alpha_G_FMAD	= 161,
    Alpha_G_FDIV	= 162,
    Alpha_G_FREM	= 163,
    Alpha_G_FPOW	= 164,
    Alpha_G_FPOWI	= 165,
    Alpha_G_FEXP	= 166,
    Alpha_G_FEXP2	= 167,
    Alpha_G_FLOG	= 168,
    Alpha_G_FLOG2	= 169,
    Alpha_G_FLOG10	= 170,
    Alpha_G_FNEG	= 171,
    Alpha_G_FPEXT	= 172,
    Alpha_G_FPTRUNC	= 173,
    Alpha_G_FPTOSI	= 174,
    Alpha_G_FPTOUI	= 175,
    Alpha_G_SITOFP	= 176,
    Alpha_G_UITOFP	= 177,
    Alpha_G_FABS	= 178,
    Alpha_G_FCOPYSIGN	= 179,
    Alpha_G_IS_FPCLASS	= 180,
    Alpha_G_FCANONICALIZE	= 181,
    Alpha_G_FMINNUM	= 182,
    Alpha_G_FMAXNUM	= 183,
    Alpha_G_FMINNUM_IEEE	= 184,
    Alpha_G_FMAXNUM_IEEE	= 185,
    Alpha_G_FMINIMUM	= 186,
    Alpha_G_FMAXIMUM	= 187,
    Alpha_G_PTR_ADD	= 188,
    Alpha_G_PTRMASK	= 189,
    Alpha_G_SMIN	= 190,
    Alpha_G_SMAX	= 191,
    Alpha_G_UMIN	= 192,
    Alpha_G_UMAX	= 193,
    Alpha_G_ABS	= 194,
    Alpha_G_LROUND	= 195,
    Alpha_G_LLROUND	= 196,
    Alpha_G_BR	= 197,
    Alpha_G_BRJT	= 198,
    Alpha_G_INSERT_VECTOR_ELT	= 199,
    Alpha_G_EXTRACT_VECTOR_ELT	= 200,
    Alpha_G_SHUFFLE_VECTOR	= 201,
    Alpha_G_CTTZ	= 202,
    Alpha_G_CTTZ_ZERO_UNDEF	= 203,
    Alpha_G_CTLZ	= 204,
    Alpha_G_CTLZ_ZERO_UNDEF	= 205,
    Alpha_G_CTPOP	= 206,
    Alpha_G_BSWAP	= 207,
    Alpha_G_BITREVERSE	= 208,
    Alpha_G_FCEIL	= 209,
    Alpha_G_FCOS	= 210,
    Alpha_G_FSIN	= 211,
    Alpha_G_FSQRT	= 212,
    Alpha_G_FFLOOR	= 213,
    Alpha_G_FRINT	= 214,
    Alpha_G_FNEARBYINT	= 215,
    Alpha_G_ADDRSPACE_CAST	= 216,
    Alpha_G_BLOCK_ADDR	= 217,
    Alpha_G_JUMP_TABLE	= 218,
    Alpha_G_DYN_STACKALLOC	= 219,
    Alpha_G_STRICT_FADD	= 220,
    Alpha_G_STRICT_FSUB	= 221,
    Alpha_G_STRICT_FMUL	= 222,
    Alpha_G_STRICT_FDIV	= 223,
    Alpha_G_STRICT_FREM	= 224,
    Alpha_G_STRICT_FMA	= 225,
    Alpha_G_STRICT_FSQRT	= 226,
    Alpha_G_READ_REGISTER	= 227,
    Alpha_G_WRITE_REGISTER	= 228,
    Alpha_G_MEMCPY	= 229,
    Alpha_G_MEMCPY_INLINE	= 230,
    Alpha_G_MEMMOVE	= 231,
    Alpha_G_MEMSET	= 232,
    Alpha_G_BZERO	= 233,
    Alpha_G_VECREDUCE_SEQ_FADD	= 234,
    Alpha_G_VECREDUCE_SEQ_FMUL	= 235,
    Alpha_G_VECREDUCE_FADD	= 236,
    Alpha_G_VECREDUCE_FMUL	= 237,
    Alpha_G_VECREDUCE_FMAX	= 238,
    Alpha_G_VECREDUCE_FMIN	= 239,
    Alpha_G_VECREDUCE_ADD	= 240,
    Alpha_G_VECREDUCE_MUL	= 241,
    Alpha_G_VECREDUCE_AND	= 242,
    Alpha_G_VECREDUCE_OR	= 243,
    Alpha_G_VECREDUCE_XOR	= 244,
    Alpha_G_VECREDUCE_SMAX	= 245,
    Alpha_G_VECREDUCE_SMIN	= 246,
    Alpha_G_VECREDUCE_UMAX	= 247,
    Alpha_G_VECREDUCE_UMIN	= 248,
    Alpha_G_SBFX	= 249,
    Alpha_G_UBFX	= 250,
    Alpha_ADJUSTSTACKDOWN	= 251,
    Alpha_ADJUSTSTACKUP	= 252,
    Alpha_ALTENT	= 253,
    Alpha_CAS32	= 254,
    Alpha_CAS64	= 255,
    Alpha_LAS32	= 256,
    Alpha_LAS64	= 257,
    Alpha_MEMLABEL	= 258,
    Alpha_PCLABEL	= 259,
    Alpha_SWAP32	= 260,
    Alpha_SWAP64	= 261,
    Alpha_WTF	= 262,
    Alpha_ADDLi	= 263,
    Alpha_ADDLr	= 264,
    Alpha_ADDQi	= 265,
    Alpha_ADDQr	= 266,
    Alpha_ADDS	= 267,
    Alpha_ADDT	= 268,
    Alpha_ANDi	= 269,
    Alpha_ANDr	= 270,
    Alpha_BEQ	= 271,
    Alpha_BGE	= 272,
    Alpha_BGT	= 273,
    Alpha_BICi	= 274,
    Alpha_BICr	= 275,
    Alpha_BISi	= 276,
    Alpha_BISr	= 277,
    Alpha_BLBC	= 278,
    Alpha_BLBS	= 279,
    Alpha_BLE	= 280,
    Alpha_BLT	= 281,
    Alpha_BNE	= 282,
    Alpha_BR	= 283,
    Alpha_BSR	= 284,
    Alpha_CMOVEQi	= 285,
    Alpha_CMOVEQr	= 286,
    Alpha_CMOVGEi	= 287,
    Alpha_CMOVGEr	= 288,
    Alpha_CMOVGTi	= 289,
    Alpha_CMOVGTr	= 290,
    Alpha_CMOVLBCi	= 291,
    Alpha_CMOVLBCr	= 292,
    Alpha_CMOVLBSi	= 293,
    Alpha_CMOVLBSr	= 294,
    Alpha_CMOVLEi	= 295,
    Alpha_CMOVLEr	= 296,
    Alpha_CMOVLTi	= 297,
    Alpha_CMOVLTr	= 298,
    Alpha_CMOVNEi	= 299,
    Alpha_CMOVNEr	= 300,
    Alpha_CMPBGE	= 301,
    Alpha_CMPBGEi	= 302,
    Alpha_CMPEQ	= 303,
    Alpha_CMPEQi	= 304,
    Alpha_CMPLE	= 305,
    Alpha_CMPLEi	= 306,
    Alpha_CMPLT	= 307,
    Alpha_CMPLTi	= 308,
    Alpha_CMPTEQ	= 309,
    Alpha_CMPTLE	= 310,
    Alpha_CMPTLT	= 311,
    Alpha_CMPTUN	= 312,
    Alpha_CMPULE	= 313,
    Alpha_CMPULEi	= 314,
    Alpha_CMPULT	= 315,
    Alpha_CMPULTi	= 316,
    Alpha_COND_BRANCH_F	= 317,
    Alpha_COND_BRANCH_I	= 318,
    Alpha_CPYSES	= 319,
    Alpha_CPYSESt	= 320,
    Alpha_CPYSET	= 321,
    Alpha_CPYSNS	= 322,
    Alpha_CPYSNSt	= 323,
    Alpha_CPYSNT	= 324,
    Alpha_CPYSNTs	= 325,
    Alpha_CPYSS	= 326,
    Alpha_CPYSSt	= 327,
    Alpha_CPYST	= 328,
    Alpha_CPYSTs	= 329,
    Alpha_CTLZ	= 330,
    Alpha_CTPOP	= 331,
    Alpha_CTTZ	= 332,
    Alpha_CVTQS	= 333,
    Alpha_CVTQT	= 334,
    Alpha_CVTST	= 335,
    Alpha_CVTTQ	= 336,
    Alpha_CVTTS	= 337,
    Alpha_DIVS	= 338,
    Alpha_DIVT	= 339,
    Alpha_ECB	= 340,
    Alpha_EQVi	= 341,
    Alpha_EQVr	= 342,
    Alpha_EXCB	= 343,
    Alpha_EXTBL	= 344,
    Alpha_EXTBLi	= 345,
    Alpha_EXTLH	= 346,
    Alpha_EXTLHi	= 347,
    Alpha_EXTLL	= 348,
    Alpha_EXTLLi	= 349,
    Alpha_EXTQH	= 350,
    Alpha_EXTQHi	= 351,
    Alpha_EXTQL	= 352,
    Alpha_EXTQLi	= 353,
    Alpha_EXTWH	= 354,
    Alpha_EXTWHi	= 355,
    Alpha_EXTWL	= 356,
    Alpha_EXTWLi	= 357,
    Alpha_FBEQ	= 358,
    Alpha_FBGE	= 359,
    Alpha_FBGT	= 360,
    Alpha_FBLE	= 361,
    Alpha_FBLT	= 362,
    Alpha_FBNE	= 363,
    Alpha_FCMOVEQS	= 364,
    Alpha_FCMOVEQT	= 365,
    Alpha_FCMOVGES	= 366,
    Alpha_FCMOVGET	= 367,
    Alpha_FCMOVGTS	= 368,
    Alpha_FCMOVGTT	= 369,
    Alpha_FCMOVLES	= 370,
    Alpha_FCMOVLET	= 371,
    Alpha_FCMOVLTS	= 372,
    Alpha_FCMOVLTT	= 373,
    Alpha_FCMOVNES	= 374,
    Alpha_FCMOVNET	= 375,
    Alpha_FETCH	= 376,
    Alpha_FETCH_M	= 377,
    Alpha_FTOIS	= 378,
    Alpha_FTOIT	= 379,
    Alpha_INSBL	= 380,
    Alpha_INSBLi	= 381,
    Alpha_INSLH	= 382,
    Alpha_INSLHi	= 383,
    Alpha_INSLL	= 384,
    Alpha_INSLLi	= 385,
    Alpha_INSQH	= 386,
    Alpha_INSQHi	= 387,
    Alpha_INSQL	= 388,
    Alpha_INSQLi	= 389,
    Alpha_INSWH	= 390,
    Alpha_INSWHi	= 391,
    Alpha_INSWL	= 392,
    Alpha_INSWLi	= 393,
    Alpha_ITOFS	= 394,
    Alpha_ITOFT	= 395,
    Alpha_JMP	= 396,
    Alpha_JSR	= 397,
    Alpha_JSR_COROUTINE	= 398,
    Alpha_JSRs	= 399,
    Alpha_LDA	= 400,
    Alpha_LDAH	= 401,
    Alpha_LDAHg	= 402,
    Alpha_LDAHr	= 403,
    Alpha_LDAg	= 404,
    Alpha_LDAr	= 405,
    Alpha_LDBU	= 406,
    Alpha_LDBUr	= 407,
    Alpha_LDL	= 408,
    Alpha_LDL_L	= 409,
    Alpha_LDLr	= 410,
    Alpha_LDQ	= 411,
    Alpha_LDQ_L	= 412,
    Alpha_LDQ_U	= 413,
    Alpha_LDQl	= 414,
    Alpha_LDQr	= 415,
    Alpha_LDS	= 416,
    Alpha_LDSr	= 417,
    Alpha_LDT	= 418,
    Alpha_LDTr	= 419,
    Alpha_LDWU	= 420,
    Alpha_LDWUr	= 421,
    Alpha_MB	= 422,
    Alpha_MSKBL	= 423,
    Alpha_MSKBLi	= 424,
    Alpha_MSKLH	= 425,
    Alpha_MSKLHi	= 426,
    Alpha_MSKLL	= 427,
    Alpha_MSKLLi	= 428,
    Alpha_MSKQH	= 429,
    Alpha_MSKQHi	= 430,
    Alpha_MSKQL	= 431,
    Alpha_MSKQLi	= 432,
    Alpha_MSKWH	= 433,
    Alpha_MSKWHi	= 434,
    Alpha_MSKWL	= 435,
    Alpha_MSKWLi	= 436,
    Alpha_MULLi	= 437,
    Alpha_MULLr	= 438,
    Alpha_MULQi	= 439,
    Alpha_MULQr	= 440,
    Alpha_MULS	= 441,
    Alpha_MULT	= 442,
    Alpha_ORNOTi	= 443,
    Alpha_ORNOTr	= 444,
    Alpha_RC	= 445,
    Alpha_RETDAG	= 446,
    Alpha_RETDAGp	= 447,
    Alpha_RPCC	= 448,
    Alpha_RS	= 449,
    Alpha_S4ADDLi	= 450,
    Alpha_S4ADDLr	= 451,
    Alpha_S4ADDQi	= 452,
    Alpha_S4ADDQr	= 453,
    Alpha_S4SUBLi	= 454,
    Alpha_S4SUBLr	= 455,
    Alpha_S4SUBQi	= 456,
    Alpha_S4SUBQr	= 457,
    Alpha_S8ADDLi	= 458,
    Alpha_S8ADDLr	= 459,
    Alpha_S8ADDQi	= 460,
    Alpha_S8ADDQr	= 461,
    Alpha_S8SUBLi	= 462,
    Alpha_S8SUBLr	= 463,
    Alpha_S8SUBQi	= 464,
    Alpha_S8SUBQr	= 465,
    Alpha_SEXTB	= 466,
    Alpha_SEXTW	= 467,
    Alpha_SLi	= 468,
    Alpha_SLr	= 469,
    Alpha_SQRTS	= 470,
    Alpha_SQRTT	= 471,
    Alpha_SRAi	= 472,
    Alpha_SRAr	= 473,
    Alpha_SRLi	= 474,
    Alpha_SRLr	= 475,
    Alpha_STB	= 476,
    Alpha_STBr	= 477,
    Alpha_STL	= 478,
    Alpha_STL_C	= 479,
    Alpha_STLr	= 480,
    Alpha_STQ	= 481,
    Alpha_STQ_C	= 482,
    Alpha_STQ_U	= 483,
    Alpha_STQr	= 484,
    Alpha_STS	= 485,
    Alpha_STSr	= 486,
    Alpha_STT	= 487,
    Alpha_STTr	= 488,
    Alpha_STW	= 489,
    Alpha_STWr	= 490,
    Alpha_SUBLi	= 491,
    Alpha_SUBLr	= 492,
    Alpha_SUBQi	= 493,
    Alpha_SUBQr	= 494,
    Alpha_SUBS	= 495,
    Alpha_SUBT	= 496,
    Alpha_TRAPB	= 497,
    Alpha_UMULHi	= 498,
    Alpha_UMULHr	= 499,
    Alpha_WH64	= 500,
    Alpha_WH64EN	= 501,
    Alpha_WMB	= 502,
    Alpha_XORi	= 503,
    Alpha_XORr	= 504,
    Alpha_ZAPNOTi	= 505,
    INSTRUCTION_LIST_END = 506
  };

#endif // GET_INSTRINFO_ENUM

#ifdef GET_INSTRINFO_MC_DESC
#undef GET_INSTRINFO_MC_DESC


static const MCOperandInfo OperandInfo2[] = { { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo3[] = { { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo4[] = { { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo5[] = { { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo6[] = { { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo7[] = { { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo8[] = { { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo9[] = { { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, CONSTRAINT_MCOI_TIED_TO(0) }, };
static const MCOperandInfo OperandInfo10[] = { { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo11[] = { { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo12[] = { { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo13[] = { { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo14[] = { { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo15[] = { { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo16[] = { { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo17[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 }, };
static const MCOperandInfo OperandInfo18[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, };
static const MCOperandInfo OperandInfo19[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, };
static const MCOperandInfo OperandInfo20[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, };
static const MCOperandInfo OperandInfo21[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo22[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 }, };
static const MCOperandInfo OperandInfo23[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, };
static const MCOperandInfo OperandInfo24[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 }, };
static const MCOperandInfo OperandInfo25[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, };
static const MCOperandInfo OperandInfo26[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo27[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo28[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo29[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, };
static const MCOperandInfo OperandInfo30[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, };
static const MCOperandInfo OperandInfo31[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, };
static const MCOperandInfo OperandInfo32[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo33[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, };
static const MCOperandInfo OperandInfo34[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, };
static const MCOperandInfo OperandInfo35[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, };
static const MCOperandInfo OperandInfo36[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, };
static const MCOperandInfo OperandInfo37[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 }, };
static const MCOperandInfo OperandInfo38[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, };
static const MCOperandInfo OperandInfo39[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, };
static const MCOperandInfo OperandInfo40[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, };
static const MCOperandInfo OperandInfo41[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, };
static const MCOperandInfo OperandInfo42[] = { { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, };
static const MCOperandInfo OperandInfo43[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 }, };
static const MCOperandInfo OperandInfo44[] = { { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, };
static const MCOperandInfo OperandInfo45[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo46[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo47[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo48[] = { { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo49[] = { { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo50[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 }, };
static const MCOperandInfo OperandInfo51[] = { { -1, 0, MCOI_OPERAND_PCREL, 0 }, };
static const MCOperandInfo OperandInfo52[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, };
static const MCOperandInfo OperandInfo53[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, };
static const MCOperandInfo OperandInfo54[] = { { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 }, };
static const MCOperandInfo OperandInfo55[] = { { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 }, };
static const MCOperandInfo OperandInfo56[] = { { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo57[] = { { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo58[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo59[] = { { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo60[] = { { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo61[] = { { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo62[] = { { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 }, };
static const MCOperandInfo OperandInfo63[] = { { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo64[] = { { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo65[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo66[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo67[] = { { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo68[] = { { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo69[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo70[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo71[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, };
static const MCOperandInfo OperandInfo72[] = { { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo73[] = { { Alpha_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo74[] = { { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };
static const MCOperandInfo OperandInfo75[] = { { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { Alpha_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, };

static const MCInstrDesc AlphaInsts[] = {
  { 1, OperandInfo2 },  // Inst #0 = PHI
  { 0, 0 },  // Inst #1 = INLINEASM
  { 0, 0 },  // Inst #2 = INLINEASM_BR
  { 1, OperandInfo3 },  // Inst #3 = CFI_INSTRUCTION
  { 1, OperandInfo3 },  // Inst #4 = EH_LABEL
  { 1, OperandInfo3 },  // Inst #5 = GC_LABEL
  { 1, OperandInfo3 },  // Inst #6 = ANNOTATION_LABEL
  { 0, 0 },  // Inst #7 = KILL
  { 3, OperandInfo4 },  // Inst #8 = EXTRACT_SUBREG
  { 4, OperandInfo5 },  // Inst #9 = INSERT_SUBREG
  { 1, OperandInfo2 },  // Inst #10 = IMPLICIT_DEF
  { 4, OperandInfo6 },  // Inst #11 = SUBREG_TO_REG
  { 3, OperandInfo4 },  // Inst #12 = COPY_TO_REGCLASS
  { 0, 0 },  // Inst #13 = DBG_VALUE
  { 0, 0 },  // Inst #14 = DBG_VALUE_LIST
  { 0, 0 },  // Inst #15 = DBG_INSTR_REF
  { 0, 0 },  // Inst #16 = DBG_PHI
  { 1, OperandInfo2 },  // Inst #17 = DBG_LABEL
  { 2, OperandInfo7 },  // Inst #18 = REG_SEQUENCE
  { 2, OperandInfo7 },  // Inst #19 = COPY
  { 0, 0 },  // Inst #20 = BUNDLE
  { 1, OperandInfo3 },  // Inst #21 = LIFETIME_START
  { 1, OperandInfo3 },  // Inst #22 = LIFETIME_END
  { 4, OperandInfo8 },  // Inst #23 = PSEUDO_PROBE
  { 2, OperandInfo9 },  // Inst #24 = ARITH_FENCE
  { 2, OperandInfo10 },  // Inst #25 = STACKMAP
  { 0, 0 },  // Inst #26 = FENTRY_CALL
  { 6, OperandInfo11 },  // Inst #27 = PATCHPOINT
  { 1, OperandInfo12 },  // Inst #28 = LOAD_STACK_GUARD
  { 1, OperandInfo3 },  // Inst #29 = PREALLOCATED_SETUP
  { 3, OperandInfo13 },  // Inst #30 = PREALLOCATED_ARG
  { 0, 0 },  // Inst #31 = STATEPOINT
  { 2, OperandInfo14 },  // Inst #32 = LOCAL_ESCAPE
  { 1, OperandInfo2 },  // Inst #33 = FAULTING_OP
  { 0, 0 },  // Inst #34 = PATCHABLE_OP
  { 0, 0 },  // Inst #35 = PATCHABLE_FUNCTION_ENTER
  { 0, 0 },  // Inst #36 = PATCHABLE_RET
  { 0, 0 },  // Inst #37 = PATCHABLE_FUNCTION_EXIT
  { 0, 0 },  // Inst #38 = PATCHABLE_TAIL_CALL
  { 2, OperandInfo15 },  // Inst #39 = PATCHABLE_EVENT_CALL
  { 3, OperandInfo16 },  // Inst #40 = PATCHABLE_TYPED_EVENT_CALL
  { 0, 0 },  // Inst #41 = ICALL_BRANCH_FUNNEL
  { 0, 0 },  // Inst #42 = MEMBARRIER
  { 3, OperandInfo17 },  // Inst #43 = G_ASSERT_SEXT
  { 3, OperandInfo17 },  // Inst #44 = G_ASSERT_ZEXT
  { 3, OperandInfo17 },  // Inst #45 = G_ASSERT_ALIGN
  { 3, OperandInfo18 },  // Inst #46 = G_ADD
  { 3, OperandInfo18 },  // Inst #47 = G_SUB
  { 3, OperandInfo18 },  // Inst #48 = G_MUL
  { 3, OperandInfo18 },  // Inst #49 = G_SDIV
  { 3, OperandInfo18 },  // Inst #50 = G_UDIV
  { 3, OperandInfo18 },  // Inst #51 = G_SREM
  { 3, OperandInfo18 },  // Inst #52 = G_UREM
  { 4, OperandInfo19 },  // Inst #53 = G_SDIVREM
  { 4, OperandInfo19 },  // Inst #54 = G_UDIVREM
  { 3, OperandInfo18 },  // Inst #55 = G_AND
  { 3, OperandInfo18 },  // Inst #56 = G_OR
  { 3, OperandInfo18 },  // Inst #57 = G_XOR
  { 1, OperandInfo20 },  // Inst #58 = G_IMPLICIT_DEF
  { 1, OperandInfo20 },  // Inst #59 = G_PHI
  { 2, OperandInfo21 },  // Inst #60 = G_FRAME_INDEX
  { 2, OperandInfo21 },  // Inst #61 = G_GLOBAL_VALUE
  { 3, OperandInfo22 },  // Inst #62 = G_EXTRACT
  { 2, OperandInfo23 },  // Inst #63 = G_UNMERGE_VALUES
  { 4, OperandInfo24 },  // Inst #64 = G_INSERT
  { 2, OperandInfo23 },  // Inst #65 = G_MERGE_VALUES
  { 2, OperandInfo23 },  // Inst #66 = G_BUILD_VECTOR
  { 2, OperandInfo23 },  // Inst #67 = G_BUILD_VECTOR_TRUNC
  { 2, OperandInfo23 },  // Inst #68 = G_CONCAT_VECTORS
  { 2, OperandInfo23 },  // Inst #69 = G_PTRTOINT
  { 2, OperandInfo23 },  // Inst #70 = G_INTTOPTR
  { 2, OperandInfo23 },  // Inst #71 = G_BITCAST
  { 2, OperandInfo25 },  // Inst #72 = G_FREEZE
  { 3, OperandInfo26 },  // Inst #73 = G_INTRINSIC_FPTRUNC_ROUND
  { 2, OperandInfo25 },  // Inst #74 = G_INTRINSIC_TRUNC
  { 2, OperandInfo25 },  // Inst #75 = G_INTRINSIC_ROUND
  { 2, OperandInfo23 },  // Inst #76 = G_INTRINSIC_LRINT
  { 2, OperandInfo25 },  // Inst #77 = G_INTRINSIC_ROUNDEVEN
  { 1, OperandInfo20 },  // Inst #78 = G_READCYCLECOUNTER
  { 2, OperandInfo23 },  // Inst #79 = G_LOAD
  { 2, OperandInfo23 },  // Inst #80 = G_SEXTLOAD
  { 2, OperandInfo23 },  // Inst #81 = G_ZEXTLOAD
  { 5, OperandInfo27 },  // Inst #82 = G_INDEXED_LOAD
  { 5, OperandInfo27 },  // Inst #83 = G_INDEXED_SEXTLOAD
  { 5, OperandInfo27 },  // Inst #84 = G_INDEXED_ZEXTLOAD
  { 2, OperandInfo23 },  // Inst #85 = G_STORE
  { 5, OperandInfo28 },  // Inst #86 = G_INDEXED_STORE
  { 5, OperandInfo29 },  // Inst #87 = G_ATOMIC_CMPXCHG_WITH_SUCCESS
  { 4, OperandInfo30 },  // Inst #88 = G_ATOMIC_CMPXCHG
  { 3, OperandInfo31 },  // Inst #89 = G_ATOMICRMW_XCHG
  { 3, OperandInfo31 },  // Inst #90 = G_ATOMICRMW_ADD
  { 3, OperandInfo31 },  // Inst #91 = G_ATOMICRMW_SUB
  { 3, OperandInfo31 },  // Inst #92 = G_ATOMICRMW_AND
  { 3, OperandInfo31 },  // Inst #93 = G_ATOMICRMW_NAND
  { 3, OperandInfo31 },  // Inst #94 = G_ATOMICRMW_OR
  { 3, OperandInfo31 },  // Inst #95 = G_ATOMICRMW_XOR
  { 3, OperandInfo31 },  // Inst #96 = G_ATOMICRMW_MAX
  { 3, OperandInfo31 },  // Inst #97 = G_ATOMICRMW_MIN
  { 3, OperandInfo31 },  // Inst #98 = G_ATOMICRMW_UMAX
  { 3, OperandInfo31 },  // Inst #99 = G_ATOMICRMW_UMIN
  { 3, OperandInfo31 },  // Inst #100 = G_ATOMICRMW_FADD
  { 3, OperandInfo31 },  // Inst #101 = G_ATOMICRMW_FSUB
  { 3, OperandInfo31 },  // Inst #102 = G_ATOMICRMW_FMAX
  { 3, OperandInfo31 },  // Inst #103 = G_ATOMICRMW_FMIN
  { 3, OperandInfo31 },  // Inst #104 = G_ATOMICRMW_UINC_WRAP
  { 3, OperandInfo31 },  // Inst #105 = G_ATOMICRMW_UDEC_WRAP
  { 2, OperandInfo10 },  // Inst #106 = G_FENCE
  { 2, OperandInfo21 },  // Inst #107 = G_BRCOND
  { 1, OperandInfo20 },  // Inst #108 = G_BRINDIRECT
  { 0, 0 },  // Inst #109 = G_INVOKE_REGION_START
  { 1, OperandInfo2 },  // Inst #110 = G_INTRINSIC
  { 1, OperandInfo2 },  // Inst #111 = G_INTRINSIC_W_SIDE_EFFECTS
  { 2, OperandInfo23 },  // Inst #112 = G_ANYEXT
  { 2, OperandInfo23 },  // Inst #113 = G_TRUNC
  { 2, OperandInfo21 },  // Inst #114 = G_CONSTANT
  { 2, OperandInfo21 },  // Inst #115 = G_FCONSTANT
  { 1, OperandInfo20 },  // Inst #116 = G_VASTART
  { 3, OperandInfo32 },  // Inst #117 = G_VAARG
  { 2, OperandInfo23 },  // Inst #118 = G_SEXT
  { 3, OperandInfo17 },  // Inst #119 = G_SEXT_INREG
  { 2, OperandInfo23 },  // Inst #120 = G_ZEXT
  { 3, OperandInfo33 },  // Inst #121 = G_SHL
  { 3, OperandInfo33 },  // Inst #122 = G_LSHR
  { 3, OperandInfo33 },  // Inst #123 = G_ASHR
  { 4, OperandInfo34 },  // Inst #124 = G_FSHL
  { 4, OperandInfo34 },  // Inst #125 = G_FSHR
  { 3, OperandInfo33 },  // Inst #126 = G_ROTR
  { 3, OperandInfo33 },  // Inst #127 = G_ROTL
  { 4, OperandInfo35 },  // Inst #128 = G_ICMP
  { 4, OperandInfo35 },  // Inst #129 = G_FCMP
  { 4, OperandInfo30 },  // Inst #130 = G_SELECT
  { 4, OperandInfo30 },  // Inst #131 = G_UADDO
  { 5, OperandInfo36 },  // Inst #132 = G_UADDE
  { 4, OperandInfo30 },  // Inst #133 = G_USUBO
  { 5, OperandInfo36 },  // Inst #134 = G_USUBE
  { 4, OperandInfo30 },  // Inst #135 = G_SADDO
  { 5, OperandInfo36 },  // Inst #136 = G_SADDE
  { 4, OperandInfo30 },  // Inst #137 = G_SSUBO
  { 5, OperandInfo36 },  // Inst #138 = G_SSUBE
  { 4, OperandInfo30 },  // Inst #139 = G_UMULO
  { 4, OperandInfo30 },  // Inst #140 = G_SMULO
  { 3, OperandInfo18 },  // Inst #141 = G_UMULH
  { 3, OperandInfo18 },  // Inst #142 = G_SMULH
  { 3, OperandInfo18 },  // Inst #143 = G_UADDSAT
  { 3, OperandInfo18 },  // Inst #144 = G_SADDSAT
  { 3, OperandInfo18 },  // Inst #145 = G_USUBSAT
  { 3, OperandInfo18 },  // Inst #146 = G_SSUBSAT
  { 3, OperandInfo33 },  // Inst #147 = G_USHLSAT
  { 3, OperandInfo33 },  // Inst #148 = G_SSHLSAT
  { 4, OperandInfo37 },  // Inst #149 = G_SMULFIX
  { 4, OperandInfo37 },  // Inst #150 = G_UMULFIX
  { 4, OperandInfo37 },  // Inst #151 = G_SMULFIXSAT
  { 4, OperandInfo37 },  // Inst #152 = G_UMULFIXSAT
  { 4, OperandInfo37 },  // Inst #153 = G_SDIVFIX
  { 4, OperandInfo37 },  // Inst #154 = G_UDIVFIX
  { 4, OperandInfo37 },  // Inst #155 = G_SDIVFIXSAT
  { 4, OperandInfo37 },  // Inst #156 = G_UDIVFIXSAT
  { 3, OperandInfo18 },  // Inst #157 = G_FADD
  { 3, OperandInfo18 },  // Inst #158 = G_FSUB
  { 3, OperandInfo18 },  // Inst #159 = G_FMUL
  { 4, OperandInfo19 },  // Inst #160 = G_FMA
  { 4, OperandInfo19 },  // Inst #161 = G_FMAD
  { 3, OperandInfo18 },  // Inst #162 = G_FDIV
  { 3, OperandInfo18 },  // Inst #163 = G_FREM
  { 3, OperandInfo18 },  // Inst #164 = G_FPOW
  { 3, OperandInfo33 },  // Inst #165 = G_FPOWI
  { 2, OperandInfo25 },  // Inst #166 = G_FEXP
  { 2, OperandInfo25 },  // Inst #167 = G_FEXP2
  { 2, OperandInfo25 },  // Inst #168 = G_FLOG
  { 2, OperandInfo25 },  // Inst #169 = G_FLOG2
  { 2, OperandInfo25 },  // Inst #170 = G_FLOG10
  { 2, OperandInfo25 },  // Inst #171 = G_FNEG
  { 2, OperandInfo23 },  // Inst #172 = G_FPEXT
  { 2, OperandInfo23 },  // Inst #173 = G_FPTRUNC
  { 2, OperandInfo23 },  // Inst #174 = G_FPTOSI
  { 2, OperandInfo23 },  // Inst #175 = G_FPTOUI
  { 2, OperandInfo23 },  // Inst #176 = G_SITOFP
  { 2, OperandInfo23 },  // Inst #177 = G_UITOFP
  { 2, OperandInfo25 },  // Inst #178 = G_FABS
  { 3, OperandInfo33 },  // Inst #179 = G_FCOPYSIGN
  { 3, OperandInfo32 },  // Inst #180 = G_IS_FPCLASS
  { 2, OperandInfo25 },  // Inst #181 = G_FCANONICALIZE
  { 3, OperandInfo18 },  // Inst #182 = G_FMINNUM
  { 3, OperandInfo18 },  // Inst #183 = G_FMAXNUM
  { 3, OperandInfo18 },  // Inst #184 = G_FMINNUM_IEEE
  { 3, OperandInfo18 },  // Inst #185 = G_FMAXNUM_IEEE
  { 3, OperandInfo18 },  // Inst #186 = G_FMINIMUM
  { 3, OperandInfo18 },  // Inst #187 = G_FMAXIMUM
  { 3, OperandInfo33 },  // Inst #188 = G_PTR_ADD
  { 3, OperandInfo33 },  // Inst #189 = G_PTRMASK
  { 3, OperandInfo18 },  // Inst #190 = G_SMIN
  { 3, OperandInfo18 },  // Inst #191 = G_SMAX
  { 3, OperandInfo18 },  // Inst #192 = G_UMIN
  { 3, OperandInfo18 },  // Inst #193 = G_UMAX
  { 2, OperandInfo25 },  // Inst #194 = G_ABS
  { 2, OperandInfo23 },  // Inst #195 = G_LROUND
  { 2, OperandInfo23 },  // Inst #196 = G_LLROUND
  { 1, OperandInfo2 },  // Inst #197 = G_BR
  { 3, OperandInfo38 },  // Inst #198 = G_BRJT
  { 4, OperandInfo39 },  // Inst #199 = G_INSERT_VECTOR_ELT
  { 3, OperandInfo40 },  // Inst #200 = G_EXTRACT_VECTOR_ELT
  { 4, OperandInfo41 },  // Inst #201 = G_SHUFFLE_VECTOR
  { 2, OperandInfo23 },  // Inst #202 = G_CTTZ
  { 2, OperandInfo23 },  // Inst #203 = G_CTTZ_ZERO_UNDEF
  { 2, OperandInfo23 },  // Inst #204 = G_CTLZ
  { 2, OperandInfo23 },  // Inst #205 = G_CTLZ_ZERO_UNDEF
  { 2, OperandInfo23 },  // Inst #206 = G_CTPOP
  { 2, OperandInfo25 },  // Inst #207 = G_BSWAP
  { 2, OperandInfo25 },  // Inst #208 = G_BITREVERSE
  { 2, OperandInfo25 },  // Inst #209 = G_FCEIL
  { 2, OperandInfo25 },  // Inst #210 = G_FCOS
  { 2, OperandInfo25 },  // Inst #211 = G_FSIN
  { 2, OperandInfo25 },  // Inst #212 = G_FSQRT
  { 2, OperandInfo25 },  // Inst #213 = G_FFLOOR
  { 2, OperandInfo25 },  // Inst #214 = G_FRINT
  { 2, OperandInfo25 },  // Inst #215 = G_FNEARBYINT
  { 2, OperandInfo23 },  // Inst #216 = G_ADDRSPACE_CAST
  { 2, OperandInfo21 },  // Inst #217 = G_BLOCK_ADDR
  { 2, OperandInfo21 },  // Inst #218 = G_JUMP_TABLE
  { 3, OperandInfo26 },  // Inst #219 = G_DYN_STACKALLOC
  { 3, OperandInfo18 },  // Inst #220 = G_STRICT_FADD
  { 3, OperandInfo18 },  // Inst #221 = G_STRICT_FSUB
  { 3, OperandInfo18 },  // Inst #222 = G_STRICT_FMUL
  { 3, OperandInfo18 },  // Inst #223 = G_STRICT_FDIV
  { 3, OperandInfo18 },  // Inst #224 = G_STRICT_FREM
  { 4, OperandInfo19 },  // Inst #225 = G_STRICT_FMA
  { 2, OperandInfo25 },  // Inst #226 = G_STRICT_FSQRT
  { 2, OperandInfo21 },  // Inst #227 = G_READ_REGISTER
  { 2, OperandInfo42 },  // Inst #228 = G_WRITE_REGISTER
  { 4, OperandInfo43 },  // Inst #229 = G_MEMCPY
  { 3, OperandInfo40 },  // Inst #230 = G_MEMCPY_INLINE
  { 4, OperandInfo43 },  // Inst #231 = G_MEMMOVE
  { 4, OperandInfo43 },  // Inst #232 = G_MEMSET
  { 3, OperandInfo22 },  // Inst #233 = G_BZERO
  { 3, OperandInfo40 },  // Inst #234 = G_VECREDUCE_SEQ_FADD
  { 3, OperandInfo40 },  // Inst #235 = G_VECREDUCE_SEQ_FMUL
  { 2, OperandInfo23 },  // Inst #236 = G_VECREDUCE_FADD
  { 2, OperandInfo23 },  // Inst #237 = G_VECREDUCE_FMUL
  { 2, OperandInfo23 },  // Inst #238 = G_VECREDUCE_FMAX
  { 2, OperandInfo23 },  // Inst #239 = G_VECREDUCE_FMIN
  { 2, OperandInfo23 },  // Inst #240 = G_VECREDUCE_ADD
  { 2, OperandInfo23 },  // Inst #241 = G_VECREDUCE_MUL
  { 2, OperandInfo23 },  // Inst #242 = G_VECREDUCE_AND
  { 2, OperandInfo23 },  // Inst #243 = G_VECREDUCE_OR
  { 2, OperandInfo23 },  // Inst #244 = G_VECREDUCE_XOR
  { 2, OperandInfo23 },  // Inst #245 = G_VECREDUCE_SMAX
  { 2, OperandInfo23 },  // Inst #246 = G_VECREDUCE_SMIN
  { 2, OperandInfo23 },  // Inst #247 = G_VECREDUCE_UMAX
  { 2, OperandInfo23 },  // Inst #248 = G_VECREDUCE_UMIN
  { 4, OperandInfo44 },  // Inst #249 = G_SBFX
  { 4, OperandInfo44 },  // Inst #250 = G_UBFX
  { 2, OperandInfo10 },  // Inst #251 = ADJUSTSTACKDOWN
  { 2, OperandInfo10 },  // Inst #252 = ADJUSTSTACKUP
  { 1, OperandInfo3 },  // Inst #253 = ALTENT
  { 4, OperandInfo45 },  // Inst #254 = CAS32
  { 4, OperandInfo45 },  // Inst #255 = CAS64
  { 3, OperandInfo46 },  // Inst #256 = LAS32
  { 3, OperandInfo46 },  // Inst #257 = LAS64
  { 4, OperandInfo8 },  // Inst #258 = MEMLABEL
  { 1, OperandInfo3 },  // Inst #259 = PCLABEL
  { 3, OperandInfo46 },  // Inst #260 = SWAP32
  { 3, OperandInfo46 },  // Inst #261 = SWAP64
  { 0, 0 },  // Inst #262 = WTF
  { 3, OperandInfo47 },  // Inst #263 = ADDLi
  { 3, OperandInfo46 },  // Inst #264 = ADDLr
  { 3, OperandInfo47 },  // Inst #265 = ADDQi
  { 3, OperandInfo46 },  // Inst #266 = ADDQr
  { 3, OperandInfo48 },  // Inst #267 = ADDS
  { 3, OperandInfo49 },  // Inst #268 = ADDT
  { 3, OperandInfo47 },  // Inst #269 = ANDi
  { 3, OperandInfo46 },  // Inst #270 = ANDr
  { 2, OperandInfo50 },  // Inst #271 = BEQ
  { 2, OperandInfo50 },  // Inst #272 = BGE
  { 2, OperandInfo50 },  // Inst #273 = BGT
  { 3, OperandInfo47 },  // Inst #274 = BICi
  { 3, OperandInfo46 },  // Inst #275 = BICr
  { 3, OperandInfo47 },  // Inst #276 = BISi
  { 3, OperandInfo46 },  // Inst #277 = BISr
  { 2, OperandInfo50 },  // Inst #278 = BLBC
  { 2, OperandInfo50 },  // Inst #279 = BLBS
  { 2, OperandInfo50 },  // Inst #280 = BLE
  { 2, OperandInfo50 },  // Inst #281 = BLT
  { 2, OperandInfo50 },  // Inst #282 = BNE
  { 1, OperandInfo51 },  // Inst #283 = BR
  { 1, OperandInfo51 },  // Inst #284 = BSR
  { 4, OperandInfo52 },  // Inst #285 = CMOVEQi
  { 4, OperandInfo53 },  // Inst #286 = CMOVEQr
  { 4, OperandInfo52 },  // Inst #287 = CMOVGEi
  { 4, OperandInfo53 },  // Inst #288 = CMOVGEr
  { 4, OperandInfo52 },  // Inst #289 = CMOVGTi
  { 4, OperandInfo53 },  // Inst #290 = CMOVGTr
  { 4, OperandInfo52 },  // Inst #291 = CMOVLBCi
  { 4, OperandInfo53 },  // Inst #292 = CMOVLBCr
  { 4, OperandInfo52 },  // Inst #293 = CMOVLBSi
  { 4, OperandInfo53 },  // Inst #294 = CMOVLBSr
  { 4, OperandInfo52 },  // Inst #295 = CMOVLEi
  { 4, OperandInfo53 },  // Inst #296 = CMOVLEr
  { 4, OperandInfo52 },  // Inst #297 = CMOVLTi
  { 4, OperandInfo53 },  // Inst #298 = CMOVLTr
  { 4, OperandInfo52 },  // Inst #299 = CMOVNEi
  { 4, OperandInfo53 },  // Inst #300 = CMOVNEr
  { 3, OperandInfo46 },  // Inst #301 = CMPBGE
  { 3, OperandInfo47 },  // Inst #302 = CMPBGEi
  { 3, OperandInfo46 },  // Inst #303 = CMPEQ
  { 3, OperandInfo47 },  // Inst #304 = CMPEQi
  { 3, OperandInfo46 },  // Inst #305 = CMPLE
  { 3, OperandInfo47 },  // Inst #306 = CMPLEi
  { 3, OperandInfo46 },  // Inst #307 = CMPLT
  { 3, OperandInfo47 },  // Inst #308 = CMPLTi
  { 3, OperandInfo49 },  // Inst #309 = CMPTEQ
  { 3, OperandInfo49 },  // Inst #310 = CMPTLE
  { 3, OperandInfo49 },  // Inst #311 = CMPTLT
  { 3, OperandInfo49 },  // Inst #312 = CMPTUN
  { 3, OperandInfo46 },  // Inst #313 = CMPULE
  { 3, OperandInfo47 },  // Inst #314 = CMPULEi
  { 3, OperandInfo46 },  // Inst #315 = CMPULT
  { 3, OperandInfo47 },  // Inst #316 = CMPULTi
  { 3, OperandInfo54 },  // Inst #317 = COND_BRANCH_F
  { 3, OperandInfo55 },  // Inst #318 = COND_BRANCH_I
  { 3, OperandInfo48 },  // Inst #319 = CPYSES
  { 3, OperandInfo56 },  // Inst #320 = CPYSESt
  { 3, OperandInfo49 },  // Inst #321 = CPYSET
  { 3, OperandInfo48 },  // Inst #322 = CPYSNS
  { 3, OperandInfo56 },  // Inst #323 = CPYSNSt
  { 3, OperandInfo49 },  // Inst #324 = CPYSNT
  { 3, OperandInfo57 },  // Inst #325 = CPYSNTs
  { 3, OperandInfo48 },  // Inst #326 = CPYSS
  { 3, OperandInfo56 },  // Inst #327 = CPYSSt
  { 3, OperandInfo49 },  // Inst #328 = CPYST
  { 3, OperandInfo57 },  // Inst #329 = CPYSTs
  { 2, OperandInfo58 },  // Inst #330 = CTLZ
  { 2, OperandInfo58 },  // Inst #331 = CTPOP
  { 2, OperandInfo58 },  // Inst #332 = CTTZ
  { 2, OperandInfo59 },  // Inst #333 = CVTQS
  { 2, OperandInfo60 },  // Inst #334 = CVTQT
  { 2, OperandInfo61 },  // Inst #335 = CVTST
  { 2, OperandInfo60 },  // Inst #336 = CVTTQ
  { 2, OperandInfo59 },  // Inst #337 = CVTTS
  { 3, OperandInfo48 },  // Inst #338 = DIVS
  { 3, OperandInfo49 },  // Inst #339 = DIVT
  { 2, OperandInfo58 },  // Inst #340 = ECB
  { 3, OperandInfo47 },  // Inst #341 = EQVi
  { 3, OperandInfo46 },  // Inst #342 = EQVr
  { 0, 0 },  // Inst #343 = EXCB
  { 3, OperandInfo46 },  // Inst #344 = EXTBL
  { 3, OperandInfo47 },  // Inst #345 = EXTBLi
  { 3, OperandInfo46 },  // Inst #346 = EXTLH
  { 3, OperandInfo47 },  // Inst #347 = EXTLHi
  { 3, OperandInfo46 },  // Inst #348 = EXTLL
  { 3, OperandInfo47 },  // Inst #349 = EXTLLi
  { 3, OperandInfo46 },  // Inst #350 = EXTQH
  { 3, OperandInfo47 },  // Inst #351 = EXTQHi
  { 3, OperandInfo46 },  // Inst #352 = EXTQL
  { 3, OperandInfo47 },  // Inst #353 = EXTQLi
  { 3, OperandInfo46 },  // Inst #354 = EXTWH
  { 3, OperandInfo47 },  // Inst #355 = EXTWHi
  { 3, OperandInfo46 },  // Inst #356 = EXTWL
  { 3, OperandInfo47 },  // Inst #357 = EXTWLi
  { 2, OperandInfo62 },  // Inst #358 = FBEQ
  { 2, OperandInfo62 },  // Inst #359 = FBGE
  { 2, OperandInfo62 },  // Inst #360 = FBGT
  { 2, OperandInfo62 },  // Inst #361 = FBLE
  { 2, OperandInfo62 },  // Inst #362 = FBLT
  { 2, OperandInfo62 },  // Inst #363 = FBNE
  { 4, OperandInfo63 },  // Inst #364 = FCMOVEQS
  { 4, OperandInfo64 },  // Inst #365 = FCMOVEQT
  { 4, OperandInfo63 },  // Inst #366 = FCMOVGES
  { 4, OperandInfo64 },  // Inst #367 = FCMOVGET
  { 4, OperandInfo63 },  // Inst #368 = FCMOVGTS
  { 4, OperandInfo64 },  // Inst #369 = FCMOVGTT
  { 4, OperandInfo63 },  // Inst #370 = FCMOVLES
  { 4, OperandInfo64 },  // Inst #371 = FCMOVLET
  { 4, OperandInfo63 },  // Inst #372 = FCMOVLTS
  { 4, OperandInfo64 },  // Inst #373 = FCMOVLTT
  { 4, OperandInfo63 },  // Inst #374 = FCMOVNES
  { 4, OperandInfo64 },  // Inst #375 = FCMOVNET
  { 2, OperandInfo58 },  // Inst #376 = FETCH
  { 2, OperandInfo58 },  // Inst #377 = FETCH_M
  { 2, OperandInfo65 },  // Inst #378 = FTOIS
  { 2, OperandInfo66 },  // Inst #379 = FTOIT
  { 3, OperandInfo46 },  // Inst #380 = INSBL
  { 3, OperandInfo47 },  // Inst #381 = INSBLi
  { 3, OperandInfo46 },  // Inst #382 = INSLH
  { 3, OperandInfo47 },  // Inst #383 = INSLHi
  { 3, OperandInfo46 },  // Inst #384 = INSLL
  { 3, OperandInfo47 },  // Inst #385 = INSLLi
  { 3, OperandInfo46 },  // Inst #386 = INSQH
  { 3, OperandInfo47 },  // Inst #387 = INSQHi
  { 3, OperandInfo46 },  // Inst #388 = INSQL
  { 3, OperandInfo47 },  // Inst #389 = INSQLi
  { 3, OperandInfo46 },  // Inst #390 = INSWH
  { 3, OperandInfo47 },  // Inst #391 = INSWHi
  { 3, OperandInfo46 },  // Inst #392 = INSWL
  { 3, OperandInfo47 },  // Inst #393 = INSWLi
  { 2, OperandInfo67 },  // Inst #394 = ITOFS
  { 2, OperandInfo68 },  // Inst #395 = ITOFT
  { 1, OperandInfo69 },  // Inst #396 = JMP
  { 0, 0 },  // Inst #397 = JSR
  { 3, OperandInfo47 },  // Inst #398 = JSR_COROUTINE
  { 0, 0 },  // Inst #399 = JSRs
  { 3, OperandInfo70 },  // Inst #400 = LDA
  { 3, OperandInfo70 },  // Inst #401 = LDAH
  { 4, OperandInfo71 },  // Inst #402 = LDAHg
  { 3, OperandInfo70 },  // Inst #403 = LDAHr
  { 4, OperandInfo71 },  // Inst #404 = LDAg
  { 3, OperandInfo70 },  // Inst #405 = LDAr
  { 3, OperandInfo70 },  // Inst #406 = LDBU
  { 3, OperandInfo70 },  // Inst #407 = LDBUr
  { 3, OperandInfo70 },  // Inst #408 = LDL
  { 3, OperandInfo70 },  // Inst #409 = LDL_L
  { 3, OperandInfo70 },  // Inst #410 = LDLr
  { 3, OperandInfo70 },  // Inst #411 = LDQ
  { 3, OperandInfo70 },  // Inst #412 = LDQ_L
  { 3, OperandInfo70 },  // Inst #413 = LDQ_U
  { 3, OperandInfo70 },  // Inst #414 = LDQl
  { 3, OperandInfo70 },  // Inst #415 = LDQr
  { 3, OperandInfo72 },  // Inst #416 = LDS
  { 3, OperandInfo72 },  // Inst #417 = LDSr
  { 3, OperandInfo73 },  // Inst #418 = LDT
  { 3, OperandInfo73 },  // Inst #419 = LDTr
  { 3, OperandInfo70 },  // Inst #420 = LDWU
  { 3, OperandInfo70 },  // Inst #421 = LDWUr
  { 0, 0 },  // Inst #422 = MB
  { 3, OperandInfo46 },  // Inst #423 = MSKBL
  { 3, OperandInfo47 },  // Inst #424 = MSKBLi
  { 3, OperandInfo46 },  // Inst #425 = MSKLH
  { 3, OperandInfo47 },  // Inst #426 = MSKLHi
  { 3, OperandInfo46 },  // Inst #427 = MSKLL
  { 3, OperandInfo47 },  // Inst #428 = MSKLLi
  { 3, OperandInfo46 },  // Inst #429 = MSKQH
  { 3, OperandInfo47 },  // Inst #430 = MSKQHi
  { 3, OperandInfo46 },  // Inst #431 = MSKQL
  { 3, OperandInfo47 },  // Inst #432 = MSKQLi
  { 3, OperandInfo46 },  // Inst #433 = MSKWH
  { 3, OperandInfo47 },  // Inst #434 = MSKWHi
  { 3, OperandInfo46 },  // Inst #435 = MSKWL
  { 3, OperandInfo47 },  // Inst #436 = MSKWLi
  { 3, OperandInfo47 },  // Inst #437 = MULLi
  { 3, OperandInfo46 },  // Inst #438 = MULLr
  { 3, OperandInfo47 },  // Inst #439 = MULQi
  { 3, OperandInfo46 },  // Inst #440 = MULQr
  { 3, OperandInfo48 },  // Inst #441 = MULS
  { 3, OperandInfo49 },  // Inst #442 = MULT
  { 3, OperandInfo47 },  // Inst #443 = ORNOTi
  { 3, OperandInfo46 },  // Inst #444 = ORNOTr
  { 1, OperandInfo69 },  // Inst #445 = RC
  { 0, 0 },  // Inst #446 = RETDAG
  { 0, 0 },  // Inst #447 = RETDAGp
  { 2, OperandInfo58 },  // Inst #448 = RPCC
  { 1, OperandInfo69 },  // Inst #449 = RS
  { 3, OperandInfo47 },  // Inst #450 = S4ADDLi
  { 3, OperandInfo46 },  // Inst #451 = S4ADDLr
  { 3, OperandInfo47 },  // Inst #452 = S4ADDQi
  { 3, OperandInfo46 },  // Inst #453 = S4ADDQr
  { 3, OperandInfo47 },  // Inst #454 = S4SUBLi
  { 3, OperandInfo46 },  // Inst #455 = S4SUBLr
  { 3, OperandInfo47 },  // Inst #456 = S4SUBQi
  { 3, OperandInfo46 },  // Inst #457 = S4SUBQr
  { 3, OperandInfo47 },  // Inst #458 = S8ADDLi
  { 3, OperandInfo46 },  // Inst #459 = S8ADDLr
  { 3, OperandInfo47 },  // Inst #460 = S8ADDQi
  { 3, OperandInfo46 },  // Inst #461 = S8ADDQr
  { 3, OperandInfo47 },  // Inst #462 = S8SUBLi
  { 3, OperandInfo46 },  // Inst #463 = S8SUBLr
  { 3, OperandInfo47 },  // Inst #464 = S8SUBQi
  { 3, OperandInfo46 },  // Inst #465 = S8SUBQr
  { 2, OperandInfo58 },  // Inst #466 = SEXTB
  { 2, OperandInfo58 },  // Inst #467 = SEXTW
  { 3, OperandInfo47 },  // Inst #468 = SLi
  { 3, OperandInfo46 },  // Inst #469 = SLr
  { 2, OperandInfo74 },  // Inst #470 = SQRTS
  { 2, OperandInfo60 },  // Inst #471 = SQRTT
  { 3, OperandInfo47 },  // Inst #472 = SRAi
  { 3, OperandInfo46 },  // Inst #473 = SRAr
  { 3, OperandInfo47 },  // Inst #474 = SRLi
  { 3, OperandInfo46 },  // Inst #475 = SRLr
  { 3, OperandInfo70 },  // Inst #476 = STB
  { 3, OperandInfo70 },  // Inst #477 = STBr
  { 3, OperandInfo70 },  // Inst #478 = STL
  { 4, OperandInfo75 },  // Inst #479 = STL_C
  { 3, OperandInfo70 },  // Inst #480 = STLr
  { 3, OperandInfo70 },  // Inst #481 = STQ
  { 4, OperandInfo75 },  // Inst #482 = STQ_C
  { 3, OperandInfo70 },  // Inst #483 = STQ_U
  { 3, OperandInfo70 },  // Inst #484 = STQr
  { 3, OperandInfo72 },  // Inst #485 = STS
  { 3, OperandInfo72 },  // Inst #486 = STSr
  { 3, OperandInfo73 },  // Inst #487 = STT
  { 3, OperandInfo73 },  // Inst #488 = STTr
  { 3, OperandInfo70 },  // Inst #489 = STW
  { 3, OperandInfo70 },  // Inst #490 = STWr
  { 3, OperandInfo47 },  // Inst #491 = SUBLi
  { 3, OperandInfo46 },  // Inst #492 = SUBLr
  { 3, OperandInfo47 },  // Inst #493 = SUBQi
  { 3, OperandInfo46 },  // Inst #494 = SUBQr
  { 3, OperandInfo48 },  // Inst #495 = SUBS
  { 3, OperandInfo49 },  // Inst #496 = SUBT
  { 0, 0 },  // Inst #497 = TRAPB
  { 3, OperandInfo47 },  // Inst #498 = UMULHi
  { 3, OperandInfo46 },  // Inst #499 = UMULHr
  { 2, OperandInfo58 },  // Inst #500 = WH64
  { 2, OperandInfo58 },  // Inst #501 = WH64EN
  { 0, 0 },  // Inst #502 = WMB
  { 3, OperandInfo47 },  // Inst #503 = XORi
  { 3, OperandInfo46 },  // Inst #504 = XORr
  { 3, OperandInfo47 },  // Inst #505 = ZAPNOTi
};

#endif // GET_INSTRINFO_MC_DESC


