/* Capstone Disassembly Engine, https://www.capstone-engine.org */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2022, */
/*    Rot127 <unisono@quyllur.org> 2022-2024 */
/* Automatically generated file by Capstone's LLVM TableGen Disassembler Backend. */

/* LLVM-commit: <commit> */
/* LLVM-tag: <tag> */

/* Do not edit. */

/* Capstone's LLVM TableGen Backends: */
/* https://github.com/capstone-engine/llvm-capstone */

#ifdef GET_SUBTARGETINFO_ENUM
#undef GET_SUBTARGETINFO_ENUM

enum {
  SystemZ_FeatureBEAREnhancement = 0,
  SystemZ_FeatureBackChain = 1,
  SystemZ_FeatureDFPPackedConversion = 2,
  SystemZ_FeatureDFPZonedConversion = 3,
  SystemZ_FeatureDeflateConversion = 4,
  SystemZ_FeatureDistinctOps = 5,
  SystemZ_FeatureEnhancedDAT2 = 6,
  SystemZ_FeatureEnhancedSort = 7,
  SystemZ_FeatureExecutionHint = 8,
  SystemZ_FeatureFPExtension = 9,
  SystemZ_FeatureFastSerialization = 10,
  SystemZ_FeatureGuardedStorage = 11,
  SystemZ_FeatureHighWord = 12,
  SystemZ_FeatureInsertReferenceBitsMultiple = 13,
  SystemZ_FeatureInterlockedAccess1 = 14,
  SystemZ_FeatureLoadAndTrap = 15,
  SystemZ_FeatureLoadAndZeroRightmostByte = 16,
  SystemZ_FeatureLoadStoreOnCond = 17,
  SystemZ_FeatureLoadStoreOnCond2 = 18,
  SystemZ_FeatureMessageSecurityAssist3 = 19,
  SystemZ_FeatureMessageSecurityAssist4 = 20,
  SystemZ_FeatureMessageSecurityAssist5 = 21,
  SystemZ_FeatureMessageSecurityAssist7 = 22,
  SystemZ_FeatureMessageSecurityAssist8 = 23,
  SystemZ_FeatureMessageSecurityAssist9 = 24,
  SystemZ_FeatureMiscellaneousExtensions = 25,
  SystemZ_FeatureMiscellaneousExtensions2 = 26,
  SystemZ_FeatureMiscellaneousExtensions3 = 27,
  SystemZ_FeatureNNPAssist = 28,
  SystemZ_FeaturePopulationCount = 29,
  SystemZ_FeatureProcessorActivityInstrumentation = 30,
  SystemZ_FeatureProcessorAssist = 31,
  SystemZ_FeatureResetDATProtection = 32,
  SystemZ_FeatureResetReferenceBitsMultiple = 33,
  SystemZ_FeatureSoftFloat = 34,
  SystemZ_FeatureTransactionalExecution = 35,
  SystemZ_FeatureVector = 36,
  SystemZ_FeatureVectorEnhancements1 = 37,
  SystemZ_FeatureVectorEnhancements2 = 38,
  SystemZ_FeatureVectorPackedDecimal = 39,
  SystemZ_FeatureVectorPackedDecimalEnhancement = 40,
  SystemZ_FeatureVectorPackedDecimalEnhancement2 = 41,
  SystemZ_NumSubtargetFeatures = 42
};
#endif // GET_SUBTARGETINFO_ENUM



