/* Capstone Disassembly Engine, https://www.capstone-engine.org */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2022, */
/*    Rot127 <unisono@quyllur.org> 2022-2024 */
/* Automatically generated file by Capstone's LLVM TableGen Disassembler Backend. */

/* LLVM-commit: <commit> */
/* LLVM-tag: <tag> */

/* Do not edit. */

/* Capstone's LLVM TableGen Backends: */
/* https://github.com/capstone-engine/llvm-capstone */

#ifdef GET_SUBTARGETINFO_ENUM
#undef GET_SUBTARGETINFO_ENUM

enum {
  Sparc_DetectRoundChange = 0,
  Sparc_FeatureHardQuad = 1,
  Sparc_FeatureLeon = 2,
  Sparc_FeatureNoFMULS = 3,
  Sparc_FeatureNoFSMULD = 4,
  Sparc_FeaturePWRPSR = 5,
  Sparc_FeatureReserveG1 = 6,
  Sparc_FeatureReserveG2 = 7,
  Sparc_FeatureReserveG3 = 8,
  Sparc_FeatureReserveG4 = 9,
  Sparc_FeatureReserveG5 = 10,
  Sparc_FeatureReserveG6 = 11,
  Sparc_FeatureReserveG7 = 12,
  Sparc_FeatureReserveI0 = 13,
  Sparc_FeatureReserveI1 = 14,
  Sparc_FeatureReserveI2 = 15,
  Sparc_FeatureReserveI3 = 16,
  Sparc_FeatureReserveI4 = 17,
  Sparc_FeatureReserveI5 = 18,
  Sparc_FeatureReserveL0 = 19,
  Sparc_FeatureReserveL1 = 20,
  Sparc_FeatureReserveL2 = 21,
  Sparc_FeatureReserveL3 = 22,
  Sparc_FeatureReserveL4 = 23,
  Sparc_FeatureReserveL5 = 24,
  Sparc_FeatureReserveL6 = 25,
  Sparc_FeatureReserveL7 = 26,
  Sparc_FeatureReserveO0 = 27,
  Sparc_FeatureReserveO1 = 28,
  Sparc_FeatureReserveO2 = 29,
  Sparc_FeatureReserveO3 = 30,
  Sparc_FeatureReserveO4 = 31,
  Sparc_FeatureReserveO5 = 32,
  Sparc_FeatureSoftFloat = 33,
  Sparc_FeatureSoftMulDiv = 34,
  Sparc_FeatureV8Deprecated = 35,
  Sparc_FeatureV9 = 36,
  Sparc_FeatureVIS = 37,
  Sparc_FeatureVIS2 = 38,
  Sparc_FeatureVIS3 = 39,
  Sparc_FixAllFDIVSQRT = 40,
  Sparc_InsertNOPLoad = 41,
  Sparc_LeonCASA = 42,
  Sparc_LeonCycleCounter = 43,
  Sparc_TuneSlowRDPC = 44,
  Sparc_UMACSMACSupport = 45,
  Sparc_UsePopc = 46,
  Sparc_NumSubtargetFeatures = 47
};
#endif // GET_SUBTARGETINFO_ENUM



