/* Capstone Disassembly Engine, https://www.capstone-engine.org */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2022, */
/*    Rot127 <unisono@quyllur.org> 2022-2024 */
/* Automatically generated file by Capstone's LLVM TableGen Disassembler Backend. */

/* LLVM-commit: <commit> */
/* LLVM-tag: <tag> */

/* Do not edit. */

/* Capstone's LLVM TableGen Backends: */
/* https://github.com/capstone-engine/llvm-capstone */

#ifdef GET_SUBTARGETINFO_ENUM
#undef GET_SUBTARGETINFO_ENUM

enum {
  ARM_ARMv4 = 0,
  ARM_ARMv4t = 1,
  ARM_ARMv5t = 2,
  ARM_ARMv5te = 3,
  ARM_ARMv5tej = 4,
  ARM_ARMv6 = 5,
  ARM_ARMv6j = 6,
  ARM_ARMv6k = 7,
  ARM_ARMv6kz = 8,
  ARM_ARMv6m = 9,
  ARM_ARMv6sm = 10,
  ARM_ARMv6t2 = 11,
  ARM_ARMv7a = 12,
  ARM_ARMv7em = 13,
  ARM_ARMv7k = 14,
  ARM_ARMv7m = 15,
  ARM_ARMv7r = 16,
  ARM_ARMv7s = 17,
  ARM_ARMv7ve = 18,
  ARM_ARMv8a = 19,
  ARM_ARMv8mBaseline = 20,
  ARM_ARMv8mMainline = 21,
  ARM_ARMv8r = 22,
  ARM_ARMv9a = 23,
  ARM_ARMv81a = 24,
  ARM_ARMv81mMainline = 25,
  ARM_ARMv82a = 26,
  ARM_ARMv83a = 27,
  ARM_ARMv84a = 28,
  ARM_ARMv85a = 29,
  ARM_ARMv86a = 30,
  ARM_ARMv87a = 31,
  ARM_ARMv88a = 32,
  ARM_ARMv89a = 33,
  ARM_ARMv91a = 34,
  ARM_ARMv92a = 35,
  ARM_ARMv93a = 36,
  ARM_ARMv94a = 37,
  ARM_ARMv95a = 38,
  ARM_Feature8MSecExt = 39,
  ARM_FeatureAAPCSFrameChain = 40,
  ARM_FeatureAAPCSFrameChainLeaf = 41,
  ARM_FeatureAClass = 42,
  ARM_FeatureAES = 43,
  ARM_FeatureAcquireRelease = 44,
  ARM_FeatureAtomics32 = 45,
  ARM_FeatureAvoidMOVsShOp = 46,
  ARM_FeatureAvoidPartialCPSR = 47,
  ARM_FeatureBF16 = 48,
  ARM_FeatureCLRBHB = 49,
  ARM_FeatureCRC = 50,
  ARM_FeatureCheapPredicableCPSR = 51,
  ARM_FeatureCheckVLDnAlign = 52,
  ARM_FeatureCoprocCDE0 = 53,
  ARM_FeatureCoprocCDE1 = 54,
  ARM_FeatureCoprocCDE2 = 55,
  ARM_FeatureCoprocCDE3 = 56,
  ARM_FeatureCoprocCDE4 = 57,
  ARM_FeatureCoprocCDE5 = 58,
  ARM_FeatureCoprocCDE6 = 59,
  ARM_FeatureCoprocCDE7 = 60,
  ARM_FeatureCrypto = 61,
  ARM_FeatureD32 = 62,
  ARM_FeatureDB = 63,
  ARM_FeatureDFB = 64,
  ARM_FeatureDSP = 65,
  ARM_FeatureDontWidenVMOVS = 66,
  ARM_FeatureDotProd = 67,
  ARM_FeatureExecuteOnly = 68,
  ARM_FeatureExpandMLx = 69,
  ARM_FeatureFP16 = 70,
  ARM_FeatureFP16FML = 71,
  ARM_FeatureFP64 = 72,
  ARM_FeatureFPAO = 73,
  ARM_FeatureFPARMv8 = 74,
  ARM_FeatureFPARMv8_D16 = 75,
  ARM_FeatureFPARMv8_D16_SP = 76,
  ARM_FeatureFPARMv8_SP = 77,
  ARM_FeatureFPRegs = 78,
  ARM_FeatureFPRegs16 = 79,
  ARM_FeatureFPRegs64 = 80,
  ARM_FeatureFixCMSE_CVE_2021_35465 = 81,
  ARM_FeatureFixCortexA57AES1742098 = 82,
  ARM_FeatureFullFP16 = 83,
  ARM_FeatureFuseAES = 84,
  ARM_FeatureFuseLiterals = 85,
  ARM_FeatureHWDivARM = 86,
  ARM_FeatureHWDivThumb = 87,
  ARM_FeatureHardenSlsBlr = 88,
  ARM_FeatureHardenSlsNoComdat = 89,
  ARM_FeatureHardenSlsRetBr = 90,
  ARM_FeatureHasNoBranchPredictor = 91,
  ARM_FeatureHasRetAddrStack = 92,
  ARM_FeatureHasSlowFPVFMx = 93,
  ARM_FeatureHasSlowFPVMLx = 94,
  ARM_FeatureHasVMLxHazards = 95,
  ARM_FeatureLOB = 96,
  ARM_FeatureLongCalls = 97,
  ARM_FeatureMClass = 98,
  ARM_FeatureMP = 99,
  ARM_FeatureMVEVectorCostFactor1 = 100,
  ARM_FeatureMVEVectorCostFactor2 = 101,
  ARM_FeatureMVEVectorCostFactor4 = 102,
  ARM_FeatureMatMulInt8 = 103,
  ARM_FeatureMuxedUnits = 104,
  ARM_FeatureNEON = 105,
  ARM_FeatureNEONForFP = 106,
  ARM_FeatureNEONForFPMovs = 107,
  ARM_FeatureNaClTrap = 108,
  ARM_FeatureNoARM = 109,
  ARM_FeatureNoBTIAtReturnTwice = 110,
  ARM_FeatureNoMovt = 111,
  ARM_FeatureNoNegativeImmediates = 112,
  ARM_FeatureNoPostRASched = 113,
  ARM_FeatureNonpipelinedVFP = 114,
  ARM_FeaturePACBTI = 115,
  ARM_FeaturePerfMon = 116,
  ARM_FeaturePref32BitThumb = 117,
  ARM_FeaturePrefISHSTBarrier = 118,
  ARM_FeaturePrefLoopAlign32 = 119,
  ARM_FeaturePreferVMOVSR = 120,
  ARM_FeatureProfUnpredicate = 121,
  ARM_FeatureRAS = 122,
  ARM_FeatureRClass = 123,
  ARM_FeatureReadTpTPIDRPRW = 124,
  ARM_FeatureReadTpTPIDRURO = 125,
  ARM_FeatureReadTpTPIDRURW = 126,
  ARM_FeatureReserveR9 = 127,
  ARM_FeatureSB = 128,
  ARM_FeatureSHA2 = 129,
  ARM_FeatureSlowFPBrcc = 130,
  ARM_FeatureSlowLoadDSubreg = 131,
  ARM_FeatureSlowOddRegister = 132,
  ARM_FeatureSlowVDUP32 = 133,
  ARM_FeatureSlowVGETLNi32 = 134,
  ARM_FeatureSplatVFPToNeon = 135,
  ARM_FeatureStrictAlign = 136,
  ARM_FeatureThumb2 = 137,
  ARM_FeatureTrustZone = 138,
  ARM_FeatureUseMIPipeliner = 139,
  ARM_FeatureUseMISched = 140,
  ARM_FeatureUseWideStrideVFP = 141,
  ARM_FeatureV7Clrex = 142,
  ARM_FeatureVFP2 = 143,
  ARM_FeatureVFP2_SP = 144,
  ARM_FeatureVFP3 = 145,
  ARM_FeatureVFP3_D16 = 146,
  ARM_FeatureVFP3_D16_SP = 147,
  ARM_FeatureVFP3_SP = 148,
  ARM_FeatureVFP4 = 149,
  ARM_FeatureVFP4_D16 = 150,
  ARM_FeatureVFP4_D16_SP = 151,
  ARM_FeatureVFP4_SP = 152,
  ARM_FeatureVMLxForwarding = 153,
  ARM_FeatureVirtualization = 154,
  ARM_FeatureZCZeroing = 155,
  ARM_HasCDEOps = 156,
  ARM_HasMVEFloatOps = 157,
  ARM_HasMVEIntegerOps = 158,
  ARM_HasV4TOps = 159,
  ARM_HasV5TEOps = 160,
  ARM_HasV5TOps = 161,
  ARM_HasV6KOps = 162,
  ARM_HasV6MOps = 163,
  ARM_HasV6Ops = 164,
  ARM_HasV6T2Ops = 165,
  ARM_HasV7Ops = 166,
  ARM_HasV8MBaselineOps = 167,
  ARM_HasV8MMainlineOps = 168,
  ARM_HasV8Ops = 169,
  ARM_HasV8_1MMainlineOps = 170,
  ARM_HasV8_1aOps = 171,
  ARM_HasV8_2aOps = 172,
  ARM_HasV8_3aOps = 173,
  ARM_HasV8_4aOps = 174,
  ARM_HasV8_5aOps = 175,
  ARM_HasV8_6aOps = 176,
  ARM_HasV8_7aOps = 177,
  ARM_HasV8_8aOps = 178,
  ARM_HasV8_9aOps = 179,
  ARM_HasV9_0aOps = 180,
  ARM_HasV9_1aOps = 181,
  ARM_HasV9_2aOps = 182,
  ARM_HasV9_3aOps = 183,
  ARM_HasV9_4aOps = 184,
  ARM_HasV9_5aOps = 185,
  ARM_IWMMXT = 186,
  ARM_IWMMXT2 = 187,
  ARM_ModeBigEndianInstructions = 188,
  ARM_ModeSoftFloat = 189,
  ARM_ModeThumb = 190,
  ARM_ProcA5 = 191,
  ARM_ProcA7 = 192,
  ARM_ProcA8 = 193,
  ARM_ProcA9 = 194,
  ARM_ProcA12 = 195,
  ARM_ProcA15 = 196,
  ARM_ProcA17 = 197,
  ARM_ProcA32 = 198,
  ARM_ProcA35 = 199,
  ARM_ProcA53 = 200,
  ARM_ProcA55 = 201,
  ARM_ProcA57 = 202,
  ARM_ProcA72 = 203,
  ARM_ProcA73 = 204,
  ARM_ProcA75 = 205,
  ARM_ProcA76 = 206,
  ARM_ProcA77 = 207,
  ARM_ProcA78 = 208,
  ARM_ProcA78C = 209,
  ARM_ProcA710 = 210,
  ARM_ProcExynos = 211,
  ARM_ProcKrait = 212,
  ARM_ProcKryo = 213,
  ARM_ProcM3 = 214,
  ARM_ProcM7 = 215,
  ARM_ProcR4 = 216,
  ARM_ProcR5 = 217,
  ARM_ProcR7 = 218,
  ARM_ProcR52 = 219,
  ARM_ProcSwift = 220,
  ARM_ProcV1 = 221,
  ARM_ProcX1 = 222,
  ARM_ProcX1C = 223,
  ARM_XScale = 224,
  ARM_NumSubtargetFeatures = 225
};
#endif // GET_SUBTARGETINFO_ENUM



