/* Capstone Disassembly Engine, https://www.capstone-engine.org */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2022, */
/*    Rot127 <unisono@quyllur.org> 2022-2024 */
/* Automatically generated file by Capstone's LLVM TableGen Disassembler Backend. */

/* LLVM-commit: <commit> */
/* LLVM-tag: <tag> */

/* Do not edit. */

/* Capstone's LLVM TableGen Backends: */
/* https://github.com/capstone-engine/llvm-capstone */

#ifdef GET_INSTRINFO_ENUM
#undef GET_INSTRINFO_ENUM

  enum {
    PPC_PHI	= 0,
    PPC_INLINEASM	= 1,
    PPC_INLINEASM_BR	= 2,
    PPC_CFI_INSTRUCTION	= 3,
    PPC_EH_LABEL	= 4,
    PPC_GC_LABEL	= 5,
    PPC_ANNOTATION_LABEL	= 6,
    PPC_KILL	= 7,
    PPC_EXTRACT_SUBREG	= 8,
    PPC_INSERT_SUBREG	= 9,
    PPC_IMPLICIT_DEF	= 10,
    PPC_SUBREG_TO_REG	= 11,
    PPC_COPY_TO_REGCLASS	= 12,
    PPC_DBG_VALUE	= 13,
    PPC_DBG_VALUE_LIST	= 14,
    PPC_DBG_INSTR_REF	= 15,
    PPC_DBG_PHI	= 16,
    PPC_DBG_LABEL	= 17,
    PPC_REG_SEQUENCE	= 18,
    PPC_COPY	= 19,
    PPC_BUNDLE	= 20,
    PPC_LIFETIME_START	= 21,
    PPC_LIFETIME_END	= 22,
    PPC_PSEUDO_PROBE	= 23,
    PPC_ARITH_FENCE	= 24,
    PPC_STACKMAP	= 25,
    PPC_FENTRY_CALL	= 26,
    PPC_PATCHPOINT	= 27,
    PPC_LOAD_STACK_GUARD	= 28,
    PPC_PREALLOCATED_SETUP	= 29,
    PPC_PREALLOCATED_ARG	= 30,
    PPC_STATEPOINT	= 31,
    PPC_LOCAL_ESCAPE	= 32,
    PPC_FAULTING_OP	= 33,
    PPC_PATCHABLE_OP	= 34,
    PPC_PATCHABLE_FUNCTION_ENTER	= 35,
    PPC_PATCHABLE_RET	= 36,
    PPC_PATCHABLE_FUNCTION_EXIT	= 37,
    PPC_PATCHABLE_TAIL_CALL	= 38,
    PPC_PATCHABLE_EVENT_CALL	= 39,
    PPC_PATCHABLE_TYPED_EVENT_CALL	= 40,
    PPC_ICALL_BRANCH_FUNNEL	= 41,
    PPC_MEMBARRIER	= 42,
    PPC_JUMP_TABLE_DEBUG_INFO	= 43,
    PPC_G_ASSERT_SEXT	= 44,
    PPC_G_ASSERT_ZEXT	= 45,
    PPC_G_ASSERT_ALIGN	= 46,
    PPC_G_ADD	= 47,
    PPC_G_SUB	= 48,
    PPC_G_MUL	= 49,
    PPC_G_SDIV	= 50,
    PPC_G_UDIV	= 51,
    PPC_G_SREM	= 52,
    PPC_G_UREM	= 53,
    PPC_G_SDIVREM	= 54,
    PPC_G_UDIVREM	= 55,
    PPC_G_AND	= 56,
    PPC_G_OR	= 57,
    PPC_G_XOR	= 58,
    PPC_G_IMPLICIT_DEF	= 59,
    PPC_G_PHI	= 60,
    PPC_G_FRAME_INDEX	= 61,
    PPC_G_GLOBAL_VALUE	= 62,
    PPC_G_CONSTANT_POOL	= 63,
    PPC_G_EXTRACT	= 64,
    PPC_G_UNMERGE_VALUES	= 65,
    PPC_G_INSERT	= 66,
    PPC_G_MERGE_VALUES	= 67,
    PPC_G_BUILD_VECTOR	= 68,
    PPC_G_BUILD_VECTOR_TRUNC	= 69,
    PPC_G_CONCAT_VECTORS	= 70,
    PPC_G_PTRTOINT	= 71,
    PPC_G_INTTOPTR	= 72,
    PPC_G_BITCAST	= 73,
    PPC_G_FREEZE	= 74,
    PPC_G_CONSTANT_FOLD_BARRIER	= 75,
    PPC_G_INTRINSIC_FPTRUNC_ROUND	= 76,
    PPC_G_INTRINSIC_TRUNC	= 77,
    PPC_G_INTRINSIC_ROUND	= 78,
    PPC_G_INTRINSIC_LRINT	= 79,
    PPC_G_INTRINSIC_ROUNDEVEN	= 80,
    PPC_G_READCYCLECOUNTER	= 81,
    PPC_G_LOAD	= 82,
    PPC_G_SEXTLOAD	= 83,
    PPC_G_ZEXTLOAD	= 84,
    PPC_G_INDEXED_LOAD	= 85,
    PPC_G_INDEXED_SEXTLOAD	= 86,
    PPC_G_INDEXED_ZEXTLOAD	= 87,
    PPC_G_STORE	= 88,
    PPC_G_INDEXED_STORE	= 89,
    PPC_G_ATOMIC_CMPXCHG_WITH_SUCCESS	= 90,
    PPC_G_ATOMIC_CMPXCHG	= 91,
    PPC_G_ATOMICRMW_XCHG	= 92,
    PPC_G_ATOMICRMW_ADD	= 93,
    PPC_G_ATOMICRMW_SUB	= 94,
    PPC_G_ATOMICRMW_AND	= 95,
    PPC_G_ATOMICRMW_NAND	= 96,
    PPC_G_ATOMICRMW_OR	= 97,
    PPC_G_ATOMICRMW_XOR	= 98,
    PPC_G_ATOMICRMW_MAX	= 99,
    PPC_G_ATOMICRMW_MIN	= 100,
    PPC_G_ATOMICRMW_UMAX	= 101,
    PPC_G_ATOMICRMW_UMIN	= 102,
    PPC_G_ATOMICRMW_FADD	= 103,
    PPC_G_ATOMICRMW_FSUB	= 104,
    PPC_G_ATOMICRMW_FMAX	= 105,
    PPC_G_ATOMICRMW_FMIN	= 106,
    PPC_G_ATOMICRMW_UINC_WRAP	= 107,
    PPC_G_ATOMICRMW_UDEC_WRAP	= 108,
    PPC_G_FENCE	= 109,
    PPC_G_PREFETCH	= 110,
    PPC_G_BRCOND	= 111,
    PPC_G_BRINDIRECT	= 112,
    PPC_G_INVOKE_REGION_START	= 113,
    PPC_G_INTRINSIC	= 114,
    PPC_G_INTRINSIC_W_SIDE_EFFECTS	= 115,
    PPC_G_INTRINSIC_CONVERGENT	= 116,
    PPC_G_INTRINSIC_CONVERGENT_W_SIDE_EFFECTS	= 117,
    PPC_G_ANYEXT	= 118,
    PPC_G_TRUNC	= 119,
    PPC_G_CONSTANT	= 120,
    PPC_G_FCONSTANT	= 121,
    PPC_G_VASTART	= 122,
    PPC_G_VAARG	= 123,
    PPC_G_SEXT	= 124,
    PPC_G_SEXT_INREG	= 125,
    PPC_G_ZEXT	= 126,
    PPC_G_SHL	= 127,
    PPC_G_LSHR	= 128,
    PPC_G_ASHR	= 129,
    PPC_G_FSHL	= 130,
    PPC_G_FSHR	= 131,
    PPC_G_ROTR	= 132,
    PPC_G_ROTL	= 133,
    PPC_G_ICMP	= 134,
    PPC_G_FCMP	= 135,
    PPC_G_SELECT	= 136,
    PPC_G_UADDO	= 137,
    PPC_G_UADDE	= 138,
    PPC_G_USUBO	= 139,
    PPC_G_USUBE	= 140,
    PPC_G_SADDO	= 141,
    PPC_G_SADDE	= 142,
    PPC_G_SSUBO	= 143,
    PPC_G_SSUBE	= 144,
    PPC_G_UMULO	= 145,
    PPC_G_SMULO	= 146,
    PPC_G_UMULH	= 147,
    PPC_G_SMULH	= 148,
    PPC_G_UADDSAT	= 149,
    PPC_G_SADDSAT	= 150,
    PPC_G_USUBSAT	= 151,
    PPC_G_SSUBSAT	= 152,
    PPC_G_USHLSAT	= 153,
    PPC_G_SSHLSAT	= 154,
    PPC_G_SMULFIX	= 155,
    PPC_G_UMULFIX	= 156,
    PPC_G_SMULFIXSAT	= 157,
    PPC_G_UMULFIXSAT	= 158,
    PPC_G_SDIVFIX	= 159,
    PPC_G_UDIVFIX	= 160,
    PPC_G_SDIVFIXSAT	= 161,
    PPC_G_UDIVFIXSAT	= 162,
    PPC_G_FADD	= 163,
    PPC_G_FSUB	= 164,
    PPC_G_FMUL	= 165,
    PPC_G_FMA	= 166,
    PPC_G_FMAD	= 167,
    PPC_G_FDIV	= 168,
    PPC_G_FREM	= 169,
    PPC_G_FPOW	= 170,
    PPC_G_FPOWI	= 171,
    PPC_G_FEXP	= 172,
    PPC_G_FEXP2	= 173,
    PPC_G_FEXP10	= 174,
    PPC_G_FLOG	= 175,
    PPC_G_FLOG2	= 176,
    PPC_G_FLOG10	= 177,
    PPC_G_FLDEXP	= 178,
    PPC_G_FFREXP	= 179,
    PPC_G_FNEG	= 180,
    PPC_G_FPEXT	= 181,
    PPC_G_FPTRUNC	= 182,
    PPC_G_FPTOSI	= 183,
    PPC_G_FPTOUI	= 184,
    PPC_G_SITOFP	= 185,
    PPC_G_UITOFP	= 186,
    PPC_G_FABS	= 187,
    PPC_G_FCOPYSIGN	= 188,
    PPC_G_IS_FPCLASS	= 189,
    PPC_G_FCANONICALIZE	= 190,
    PPC_G_FMINNUM	= 191,
    PPC_G_FMAXNUM	= 192,
    PPC_G_FMINNUM_IEEE	= 193,
    PPC_G_FMAXNUM_IEEE	= 194,
    PPC_G_FMINIMUM	= 195,
    PPC_G_FMAXIMUM	= 196,
    PPC_G_GET_FPENV	= 197,
    PPC_G_SET_FPENV	= 198,
    PPC_G_RESET_FPENV	= 199,
    PPC_G_GET_FPMODE	= 200,
    PPC_G_SET_FPMODE	= 201,
    PPC_G_RESET_FPMODE	= 202,
    PPC_G_PTR_ADD	= 203,
    PPC_G_PTRMASK	= 204,
    PPC_G_SMIN	= 205,
    PPC_G_SMAX	= 206,
    PPC_G_UMIN	= 207,
    PPC_G_UMAX	= 208,
    PPC_G_ABS	= 209,
    PPC_G_LROUND	= 210,
    PPC_G_LLROUND	= 211,
    PPC_G_BR	= 212,
    PPC_G_BRJT	= 213,
    PPC_G_INSERT_VECTOR_ELT	= 214,
    PPC_G_EXTRACT_VECTOR_ELT	= 215,
    PPC_G_SHUFFLE_VECTOR	= 216,
    PPC_G_CTTZ	= 217,
    PPC_G_CTTZ_ZERO_UNDEF	= 218,
    PPC_G_CTLZ	= 219,
    PPC_G_CTLZ_ZERO_UNDEF	= 220,
    PPC_G_CTPOP	= 221,
    PPC_G_BSWAP	= 222,
    PPC_G_BITREVERSE	= 223,
    PPC_G_FCEIL	= 224,
    PPC_G_FCOS	= 225,
    PPC_G_FSIN	= 226,
    PPC_G_FSQRT	= 227,
    PPC_G_FFLOOR	= 228,
    PPC_G_FRINT	= 229,
    PPC_G_FNEARBYINT	= 230,
    PPC_G_ADDRSPACE_CAST	= 231,
    PPC_G_BLOCK_ADDR	= 232,
    PPC_G_JUMP_TABLE	= 233,
    PPC_G_DYN_STACKALLOC	= 234,
    PPC_G_STACKSAVE	= 235,
    PPC_G_STACKRESTORE	= 236,
    PPC_G_STRICT_FADD	= 237,
    PPC_G_STRICT_FSUB	= 238,
    PPC_G_STRICT_FMUL	= 239,
    PPC_G_STRICT_FDIV	= 240,
    PPC_G_STRICT_FREM	= 241,
    PPC_G_STRICT_FMA	= 242,
    PPC_G_STRICT_FSQRT	= 243,
    PPC_G_STRICT_FLDEXP	= 244,
    PPC_G_READ_REGISTER	= 245,
    PPC_G_WRITE_REGISTER	= 246,
    PPC_G_MEMCPY	= 247,
    PPC_G_MEMCPY_INLINE	= 248,
    PPC_G_MEMMOVE	= 249,
    PPC_G_MEMSET	= 250,
    PPC_G_BZERO	= 251,
    PPC_G_VECREDUCE_SEQ_FADD	= 252,
    PPC_G_VECREDUCE_SEQ_FMUL	= 253,
    PPC_G_VECREDUCE_FADD	= 254,
    PPC_G_VECREDUCE_FMUL	= 255,
    PPC_G_VECREDUCE_FMAX	= 256,
    PPC_G_VECREDUCE_FMIN	= 257,
    PPC_G_VECREDUCE_FMAXIMUM	= 258,
    PPC_G_VECREDUCE_FMINIMUM	= 259,
    PPC_G_VECREDUCE_ADD	= 260,
    PPC_G_VECREDUCE_MUL	= 261,
    PPC_G_VECREDUCE_AND	= 262,
    PPC_G_VECREDUCE_OR	= 263,
    PPC_G_VECREDUCE_XOR	= 264,
    PPC_G_VECREDUCE_SMAX	= 265,
    PPC_G_VECREDUCE_SMIN	= 266,
    PPC_G_VECREDUCE_UMAX	= 267,
    PPC_G_VECREDUCE_UMIN	= 268,
    PPC_G_SBFX	= 269,
    PPC_G_UBFX	= 270,
    PPC_ATOMIC_CMP_SWAP_I128	= 271,
    PPC_ATOMIC_LOAD_ADD_I128	= 272,
    PPC_ATOMIC_LOAD_AND_I128	= 273,
    PPC_ATOMIC_LOAD_NAND_I128	= 274,
    PPC_ATOMIC_LOAD_OR_I128	= 275,
    PPC_ATOMIC_LOAD_SUB_I128	= 276,
    PPC_ATOMIC_LOAD_XOR_I128	= 277,
    PPC_ATOMIC_SWAP_I128	= 278,
    PPC_BUILD_QUADWORD	= 279,
    PPC_BUILD_UACC	= 280,
    PPC_CFENCE	= 281,
    PPC_CFENCE8	= 282,
    PPC_CLRLSLDI	= 283,
    PPC_CLRLSLDI_rec	= 284,
    PPC_CLRLSLWI	= 285,
    PPC_CLRLSLWI_rec	= 286,
    PPC_CLRRDI	= 287,
    PPC_CLRRDI_rec	= 288,
    PPC_CLRRWI	= 289,
    PPC_CLRRWI_rec	= 290,
    PPC_DCBFL	= 291,
    PPC_DCBFLP	= 292,
    PPC_DCBFPS	= 293,
    PPC_DCBFx	= 294,
    PPC_DCBSTPS	= 295,
    PPC_DCBTCT	= 296,
    PPC_DCBTDS	= 297,
    PPC_DCBTSTCT	= 298,
    PPC_DCBTSTDS	= 299,
    PPC_DCBTSTT	= 300,
    PPC_DCBTSTx	= 301,
    PPC_DCBTT	= 302,
    PPC_DCBTx	= 303,
    PPC_DFLOADf32	= 304,
    PPC_DFLOADf64	= 305,
    PPC_DFSTOREf32	= 306,
    PPC_DFSTOREf64	= 307,
    PPC_EXTLDI	= 308,
    PPC_EXTLDI_rec	= 309,
    PPC_EXTLWI	= 310,
    PPC_EXTLWI_rec	= 311,
    PPC_EXTRDI	= 312,
    PPC_EXTRDI_rec	= 313,
    PPC_EXTRWI	= 314,
    PPC_EXTRWI_rec	= 315,
    PPC_INSLWI	= 316,
    PPC_INSLWI_rec	= 317,
    PPC_INSRDI	= 318,
    PPC_INSRDI_rec	= 319,
    PPC_INSRWI	= 320,
    PPC_INSRWI_rec	= 321,
    PPC_KILL_PAIR	= 322,
    PPC_LAx	= 323,
    PPC_LI	= 324,
    PPC_LI8	= 325,
    PPC_LIS	= 326,
    PPC_LIS8	= 327,
    PPC_LIWAX	= 328,
    PPC_LIWZX	= 329,
    PPC_PSUBI	= 330,
    PPC_RLWIMIbm	= 331,
    PPC_RLWIMIbm_rec	= 332,
    PPC_RLWINMbm	= 333,
    PPC_RLWINMbm_rec	= 334,
    PPC_RLWNMbm	= 335,
    PPC_RLWNMbm_rec	= 336,
    PPC_ROTRDI	= 337,
    PPC_ROTRDI_rec	= 338,
    PPC_ROTRWI	= 339,
    PPC_ROTRWI_rec	= 340,
    PPC_SLDI	= 341,
    PPC_SLDI_rec	= 342,
    PPC_SLWI	= 343,
    PPC_SLWI_rec	= 344,
    PPC_SPILLTOVSR_LD	= 345,
    PPC_SPILLTOVSR_LDX	= 346,
    PPC_SPILLTOVSR_ST	= 347,
    PPC_SPILLTOVSR_STX	= 348,
    PPC_SRDI	= 349,
    PPC_SRDI_rec	= 350,
    PPC_SRWI	= 351,
    PPC_SRWI_rec	= 352,
    PPC_STIWX	= 353,
    PPC_SUBI	= 354,
    PPC_SUBIC	= 355,
    PPC_SUBIC_rec	= 356,
    PPC_SUBIS	= 357,
    PPC_SUBPCIS	= 358,
    PPC_XFLOADf32	= 359,
    PPC_XFLOADf64	= 360,
    PPC_XFSTOREf32	= 361,
    PPC_XFSTOREf64	= 362,
    PPC_ADD4	= 363,
    PPC_ADD4O	= 364,
    PPC_ADD4O_rec	= 365,
    PPC_ADD4TLS	= 366,
    PPC_ADD4_rec	= 367,
    PPC_ADD8	= 368,
    PPC_ADD8O	= 369,
    PPC_ADD8O_rec	= 370,
    PPC_ADD8TLS	= 371,
    PPC_ADD8TLS_	= 372,
    PPC_ADD8_rec	= 373,
    PPC_ADDC	= 374,
    PPC_ADDC8	= 375,
    PPC_ADDC8O	= 376,
    PPC_ADDC8O_rec	= 377,
    PPC_ADDC8_rec	= 378,
    PPC_ADDCO	= 379,
    PPC_ADDCO_rec	= 380,
    PPC_ADDC_rec	= 381,
    PPC_ADDE	= 382,
    PPC_ADDE8	= 383,
    PPC_ADDE8O	= 384,
    PPC_ADDE8O_rec	= 385,
    PPC_ADDE8_rec	= 386,
    PPC_ADDEO	= 387,
    PPC_ADDEO_rec	= 388,
    PPC_ADDEX	= 389,
    PPC_ADDEX8	= 390,
    PPC_ADDE_rec	= 391,
    PPC_ADDG6S	= 392,
    PPC_ADDG6S8	= 393,
    PPC_ADDI	= 394,
    PPC_ADDI8	= 395,
    PPC_ADDIC	= 396,
    PPC_ADDIC8	= 397,
    PPC_ADDIC_rec	= 398,
    PPC_ADDIS	= 399,
    PPC_ADDIS8	= 400,
    PPC_ADDISdtprelHA	= 401,
    PPC_ADDISdtprelHA32	= 402,
    PPC_ADDISgotTprelHA	= 403,
    PPC_ADDIStlsgdHA	= 404,
    PPC_ADDIStlsldHA	= 405,
    PPC_ADDIStocHA	= 406,
    PPC_ADDIStocHA8	= 407,
    PPC_ADDIdtprelL	= 408,
    PPC_ADDIdtprelL32	= 409,
    PPC_ADDItlsgdL	= 410,
    PPC_ADDItlsgdL32	= 411,
    PPC_ADDItlsgdLADDR	= 412,
    PPC_ADDItlsgdLADDR32	= 413,
    PPC_ADDItlsldL	= 414,
    PPC_ADDItlsldL32	= 415,
    PPC_ADDItlsldLADDR	= 416,
    PPC_ADDItlsldLADDR32	= 417,
    PPC_ADDItoc	= 418,
    PPC_ADDItoc8	= 419,
    PPC_ADDItocL	= 420,
    PPC_ADDME	= 421,
    PPC_ADDME8	= 422,
    PPC_ADDME8O	= 423,
    PPC_ADDME8O_rec	= 424,
    PPC_ADDME8_rec	= 425,
    PPC_ADDMEO	= 426,
    PPC_ADDMEO_rec	= 427,
    PPC_ADDME_rec	= 428,
    PPC_ADDPCIS	= 429,
    PPC_ADDZE	= 430,
    PPC_ADDZE8	= 431,
    PPC_ADDZE8O	= 432,
    PPC_ADDZE8O_rec	= 433,
    PPC_ADDZE8_rec	= 434,
    PPC_ADDZEO	= 435,
    PPC_ADDZEO_rec	= 436,
    PPC_ADDZE_rec	= 437,
    PPC_ADJCALLSTACKDOWN	= 438,
    PPC_ADJCALLSTACKUP	= 439,
    PPC_AND	= 440,
    PPC_AND8	= 441,
    PPC_AND8_rec	= 442,
    PPC_ANDC	= 443,
    PPC_ANDC8	= 444,
    PPC_ANDC8_rec	= 445,
    PPC_ANDC_rec	= 446,
    PPC_ANDI8_rec	= 447,
    PPC_ANDIS8_rec	= 448,
    PPC_ANDIS_rec	= 449,
    PPC_ANDI_rec	= 450,
    PPC_ANDI_rec_1_EQ_BIT	= 451,
    PPC_ANDI_rec_1_EQ_BIT8	= 452,
    PPC_ANDI_rec_1_GT_BIT	= 453,
    PPC_ANDI_rec_1_GT_BIT8	= 454,
    PPC_AND_rec	= 455,
    PPC_ATOMIC_CMP_SWAP_I16	= 456,
    PPC_ATOMIC_CMP_SWAP_I32	= 457,
    PPC_ATOMIC_CMP_SWAP_I64	= 458,
    PPC_ATOMIC_CMP_SWAP_I8	= 459,
    PPC_ATOMIC_LOAD_ADD_I16	= 460,
    PPC_ATOMIC_LOAD_ADD_I32	= 461,
    PPC_ATOMIC_LOAD_ADD_I64	= 462,
    PPC_ATOMIC_LOAD_ADD_I8	= 463,
    PPC_ATOMIC_LOAD_AND_I16	= 464,
    PPC_ATOMIC_LOAD_AND_I32	= 465,
    PPC_ATOMIC_LOAD_AND_I64	= 466,
    PPC_ATOMIC_LOAD_AND_I8	= 467,
    PPC_ATOMIC_LOAD_MAX_I16	= 468,
    PPC_ATOMIC_LOAD_MAX_I32	= 469,
    PPC_ATOMIC_LOAD_MAX_I64	= 470,
    PPC_ATOMIC_LOAD_MAX_I8	= 471,
    PPC_ATOMIC_LOAD_MIN_I16	= 472,
    PPC_ATOMIC_LOAD_MIN_I32	= 473,
    PPC_ATOMIC_LOAD_MIN_I64	= 474,
    PPC_ATOMIC_LOAD_MIN_I8	= 475,
    PPC_ATOMIC_LOAD_NAND_I16	= 476,
    PPC_ATOMIC_LOAD_NAND_I32	= 477,
    PPC_ATOMIC_LOAD_NAND_I64	= 478,
    PPC_ATOMIC_LOAD_NAND_I8	= 479,
    PPC_ATOMIC_LOAD_OR_I16	= 480,
    PPC_ATOMIC_LOAD_OR_I32	= 481,
    PPC_ATOMIC_LOAD_OR_I64	= 482,
    PPC_ATOMIC_LOAD_OR_I8	= 483,
    PPC_ATOMIC_LOAD_SUB_I16	= 484,
    PPC_ATOMIC_LOAD_SUB_I32	= 485,
    PPC_ATOMIC_LOAD_SUB_I64	= 486,
    PPC_ATOMIC_LOAD_SUB_I8	= 487,
    PPC_ATOMIC_LOAD_UMAX_I16	= 488,
    PPC_ATOMIC_LOAD_UMAX_I32	= 489,
    PPC_ATOMIC_LOAD_UMAX_I64	= 490,
    PPC_ATOMIC_LOAD_UMAX_I8	= 491,
    PPC_ATOMIC_LOAD_UMIN_I16	= 492,
    PPC_ATOMIC_LOAD_UMIN_I32	= 493,
    PPC_ATOMIC_LOAD_UMIN_I64	= 494,
    PPC_ATOMIC_LOAD_UMIN_I8	= 495,
    PPC_ATOMIC_LOAD_XOR_I16	= 496,
    PPC_ATOMIC_LOAD_XOR_I32	= 497,
    PPC_ATOMIC_LOAD_XOR_I64	= 498,
    PPC_ATOMIC_LOAD_XOR_I8	= 499,
    PPC_ATOMIC_SWAP_I16	= 500,
    PPC_ATOMIC_SWAP_I32	= 501,
    PPC_ATOMIC_SWAP_I64	= 502,
    PPC_ATOMIC_SWAP_I8	= 503,
    PPC_ATTN	= 504,
    PPC_B	= 505,
    PPC_BA	= 506,
    PPC_BC	= 507,
    PPC_BCC	= 508,
    PPC_BCCA	= 509,
    PPC_BCCCTR	= 510,
    PPC_BCCCTR8	= 511,
    PPC_BCCCTRL	= 512,
    PPC_BCCCTRL8	= 513,
    PPC_BCCL	= 514,
    PPC_BCCLA	= 515,
    PPC_BCCLR	= 516,
    PPC_BCCLRL	= 517,
    PPC_BCCTR	= 518,
    PPC_BCCTR8	= 519,
    PPC_BCCTRL	= 520,
    PPC_BCCTRL8	= 521,
    PPC_BCDADD_rec	= 522,
    PPC_BCDCFN_rec	= 523,
    PPC_BCDCFSQ_rec	= 524,
    PPC_BCDCFZ_rec	= 525,
    PPC_BCDCPSGN_rec	= 526,
    PPC_BCDCTN_rec	= 527,
    PPC_BCDCTSQ_rec	= 528,
    PPC_BCDCTZ_rec	= 529,
    PPC_BCDSETSGN_rec	= 530,
    PPC_BCDSR_rec	= 531,
    PPC_BCDSUB_rec	= 532,
    PPC_BCDS_rec	= 533,
    PPC_BCDTRUNC_rec	= 534,
    PPC_BCDUS_rec	= 535,
    PPC_BCDUTRUNC_rec	= 536,
    PPC_BCL	= 537,
    PPC_BCLR	= 538,
    PPC_BCLRL	= 539,
    PPC_BCTR	= 540,
    PPC_BCTR8	= 541,
    PPC_BCTRL	= 542,
    PPC_BCTRL8	= 543,
    PPC_BCTRL8_LDinto_toc	= 544,
    PPC_BCTRL8_LDinto_toc_RM	= 545,
    PPC_BCTRL8_RM	= 546,
    PPC_BCTRL_LWZinto_toc	= 547,
    PPC_BCTRL_LWZinto_toc_RM	= 548,
    PPC_BCTRL_RM	= 549,
    PPC_BL	= 550,
    PPC_BL8	= 551,
    PPC_BL8_NOP	= 552,
    PPC_BL8_NOP_RM	= 553,
    PPC_BL8_NOP_TLS	= 554,
    PPC_BL8_NOTOC	= 555,
    PPC_BL8_NOTOC_RM	= 556,
    PPC_BL8_NOTOC_TLS	= 557,
    PPC_BL8_RM	= 558,
    PPC_BL8_TLS	= 559,
    PPC_BL8_TLS_	= 560,
    PPC_BLA	= 561,
    PPC_BLA8	= 562,
    PPC_BLA8_NOP	= 563,
    PPC_BLA8_NOP_RM	= 564,
    PPC_BLA8_RM	= 565,
    PPC_BLA_RM	= 566,
    PPC_BLR	= 567,
    PPC_BLR8	= 568,
    PPC_BLRL	= 569,
    PPC_BL_NOP	= 570,
    PPC_BL_NOP_RM	= 571,
    PPC_BL_RM	= 572,
    PPC_BL_TLS	= 573,
    PPC_BPERMD	= 574,
    PPC_BRD	= 575,
    PPC_BRH	= 576,
    PPC_BRH8	= 577,
    PPC_BRINC	= 578,
    PPC_BRW	= 579,
    PPC_BRW8	= 580,
    PPC_CBCDTD	= 581,
    PPC_CBCDTD8	= 582,
    PPC_CDTBCD	= 583,
    PPC_CDTBCD8	= 584,
    PPC_CFUGED	= 585,
    PPC_CLRBHRB	= 586,
    PPC_CMPB	= 587,
    PPC_CMPB8	= 588,
    PPC_CMPD	= 589,
    PPC_CMPDI	= 590,
    PPC_CMPEQB	= 591,
    PPC_CMPLD	= 592,
    PPC_CMPLDI	= 593,
    PPC_CMPLW	= 594,
    PPC_CMPLWI	= 595,
    PPC_CMPRB	= 596,
    PPC_CMPRB8	= 597,
    PPC_CMPW	= 598,
    PPC_CMPWI	= 599,
    PPC_CNTLZD	= 600,
    PPC_CNTLZDM	= 601,
    PPC_CNTLZD_rec	= 602,
    PPC_CNTLZW	= 603,
    PPC_CNTLZW8	= 604,
    PPC_CNTLZW8_rec	= 605,
    PPC_CNTLZW_rec	= 606,
    PPC_CNTTZD	= 607,
    PPC_CNTTZDM	= 608,
    PPC_CNTTZD_rec	= 609,
    PPC_CNTTZW	= 610,
    PPC_CNTTZW8	= 611,
    PPC_CNTTZW8_rec	= 612,
    PPC_CNTTZW_rec	= 613,
    PPC_CP_ABORT	= 614,
    PPC_CP_COPY	= 615,
    PPC_CP_COPY8	= 616,
    PPC_CP_PASTE8_rec	= 617,
    PPC_CP_PASTE_rec	= 618,
    PPC_CR6SET	= 619,
    PPC_CR6UNSET	= 620,
    PPC_CRAND	= 621,
    PPC_CRANDC	= 622,
    PPC_CREQV	= 623,
    PPC_CRNAND	= 624,
    PPC_CRNOR	= 625,
    PPC_CRNOT	= 626,
    PPC_CROR	= 627,
    PPC_CRORC	= 628,
    PPC_CRSET	= 629,
    PPC_CRUNSET	= 630,
    PPC_CRXOR	= 631,
    PPC_CTRL_DEP	= 632,
    PPC_DADD	= 633,
    PPC_DADDQ	= 634,
    PPC_DADDQ_rec	= 635,
    PPC_DADD_rec	= 636,
    PPC_DARN	= 637,
    PPC_DCBA	= 638,
    PPC_DCBF	= 639,
    PPC_DCBFEP	= 640,
    PPC_DCBI	= 641,
    PPC_DCBST	= 642,
    PPC_DCBSTEP	= 643,
    PPC_DCBT	= 644,
    PPC_DCBTEP	= 645,
    PPC_DCBTST	= 646,
    PPC_DCBTSTEP	= 647,
    PPC_DCBZ	= 648,
    PPC_DCBZEP	= 649,
    PPC_DCBZL	= 650,
    PPC_DCBZLEP	= 651,
    PPC_DCCCI	= 652,
    PPC_DCFFIX	= 653,
    PPC_DCFFIXQ	= 654,
    PPC_DCFFIXQQ	= 655,
    PPC_DCFFIXQ_rec	= 656,
    PPC_DCFFIX_rec	= 657,
    PPC_DCMPO	= 658,
    PPC_DCMPOQ	= 659,
    PPC_DCMPU	= 660,
    PPC_DCMPUQ	= 661,
    PPC_DCTDP	= 662,
    PPC_DCTDP_rec	= 663,
    PPC_DCTFIX	= 664,
    PPC_DCTFIXQ	= 665,
    PPC_DCTFIXQQ	= 666,
    PPC_DCTFIXQ_rec	= 667,
    PPC_DCTFIX_rec	= 668,
    PPC_DCTQPQ	= 669,
    PPC_DCTQPQ_rec	= 670,
    PPC_DDEDPD	= 671,
    PPC_DDEDPDQ	= 672,
    PPC_DDEDPDQ_rec	= 673,
    PPC_DDEDPD_rec	= 674,
    PPC_DDIV	= 675,
    PPC_DDIVQ	= 676,
    PPC_DDIVQ_rec	= 677,
    PPC_DDIV_rec	= 678,
    PPC_DENBCD	= 679,
    PPC_DENBCDQ	= 680,
    PPC_DENBCDQ_rec	= 681,
    PPC_DENBCD_rec	= 682,
    PPC_DIEX	= 683,
    PPC_DIEXQ	= 684,
    PPC_DIEXQ_rec	= 685,
    PPC_DIEX_rec	= 686,
    PPC_DIVD	= 687,
    PPC_DIVDE	= 688,
    PPC_DIVDEO	= 689,
    PPC_DIVDEO_rec	= 690,
    PPC_DIVDEU	= 691,
    PPC_DIVDEUO	= 692,
    PPC_DIVDEUO_rec	= 693,
    PPC_DIVDEU_rec	= 694,
    PPC_DIVDE_rec	= 695,
    PPC_DIVDO	= 696,
    PPC_DIVDO_rec	= 697,
    PPC_DIVDU	= 698,
    PPC_DIVDUO	= 699,
    PPC_DIVDUO_rec	= 700,
    PPC_DIVDU_rec	= 701,
    PPC_DIVD_rec	= 702,
    PPC_DIVW	= 703,
    PPC_DIVWE	= 704,
    PPC_DIVWEO	= 705,
    PPC_DIVWEO_rec	= 706,
    PPC_DIVWEU	= 707,
    PPC_DIVWEUO	= 708,
    PPC_DIVWEUO_rec	= 709,
    PPC_DIVWEU_rec	= 710,
    PPC_DIVWE_rec	= 711,
    PPC_DIVWO	= 712,
    PPC_DIVWO_rec	= 713,
    PPC_DIVWU	= 714,
    PPC_DIVWUO	= 715,
    PPC_DIVWUO_rec	= 716,
    PPC_DIVWU_rec	= 717,
    PPC_DIVW_rec	= 718,
    PPC_DMMR	= 719,
    PPC_DMSETDMRZ	= 720,
    PPC_DMUL	= 721,
    PPC_DMULQ	= 722,
    PPC_DMULQ_rec	= 723,
    PPC_DMUL_rec	= 724,
    PPC_DMXOR	= 725,
    PPC_DMXXEXTFDMR256	= 726,
    PPC_DMXXEXTFDMR512	= 727,
    PPC_DMXXEXTFDMR512_HI	= 728,
    PPC_DMXXINSTFDMR256	= 729,
    PPC_DMXXINSTFDMR512	= 730,
    PPC_DMXXINSTFDMR512_HI	= 731,
    PPC_DQUA	= 732,
    PPC_DQUAI	= 733,
    PPC_DQUAIQ	= 734,
    PPC_DQUAIQ_rec	= 735,
    PPC_DQUAI_rec	= 736,
    PPC_DQUAQ	= 737,
    PPC_DQUAQ_rec	= 738,
    PPC_DQUA_rec	= 739,
    PPC_DRDPQ	= 740,
    PPC_DRDPQ_rec	= 741,
    PPC_DRINTN	= 742,
    PPC_DRINTNQ	= 743,
    PPC_DRINTNQ_rec	= 744,
    PPC_DRINTN_rec	= 745,
    PPC_DRINTX	= 746,
    PPC_DRINTXQ	= 747,
    PPC_DRINTXQ_rec	= 748,
    PPC_DRINTX_rec	= 749,
    PPC_DRRND	= 750,
    PPC_DRRNDQ	= 751,
    PPC_DRRNDQ_rec	= 752,
    PPC_DRRND_rec	= 753,
    PPC_DRSP	= 754,
    PPC_DRSP_rec	= 755,
    PPC_DSCLI	= 756,
    PPC_DSCLIQ	= 757,
    PPC_DSCLIQ_rec	= 758,
    PPC_DSCLI_rec	= 759,
    PPC_DSCRI	= 760,
    PPC_DSCRIQ	= 761,
    PPC_DSCRIQ_rec	= 762,
    PPC_DSCRI_rec	= 763,
    PPC_DSS	= 764,
    PPC_DSSALL	= 765,
    PPC_DST	= 766,
    PPC_DST64	= 767,
    PPC_DSTST	= 768,
    PPC_DSTST64	= 769,
    PPC_DSTSTT	= 770,
    PPC_DSTSTT64	= 771,
    PPC_DSTT	= 772,
    PPC_DSTT64	= 773,
    PPC_DSUB	= 774,
    PPC_DSUBQ	= 775,
    PPC_DSUBQ_rec	= 776,
    PPC_DSUB_rec	= 777,
    PPC_DTSTDC	= 778,
    PPC_DTSTDCQ	= 779,
    PPC_DTSTDG	= 780,
    PPC_DTSTDGQ	= 781,
    PPC_DTSTEX	= 782,
    PPC_DTSTEXQ	= 783,
    PPC_DTSTSF	= 784,
    PPC_DTSTSFI	= 785,
    PPC_DTSTSFIQ	= 786,
    PPC_DTSTSFQ	= 787,
    PPC_DXEX	= 788,
    PPC_DXEXQ	= 789,
    PPC_DXEXQ_rec	= 790,
    PPC_DXEX_rec	= 791,
    PPC_DYNALLOC	= 792,
    PPC_DYNALLOC8	= 793,
    PPC_DYNAREAOFFSET	= 794,
    PPC_DYNAREAOFFSET8	= 795,
    PPC_DecreaseCTR8loop	= 796,
    PPC_DecreaseCTRloop	= 797,
    PPC_EFDABS	= 798,
    PPC_EFDADD	= 799,
    PPC_EFDCFS	= 800,
    PPC_EFDCFSF	= 801,
    PPC_EFDCFSI	= 802,
    PPC_EFDCFSID	= 803,
    PPC_EFDCFUF	= 804,
    PPC_EFDCFUI	= 805,
    PPC_EFDCFUID	= 806,
    PPC_EFDCMPEQ	= 807,
    PPC_EFDCMPGT	= 808,
    PPC_EFDCMPLT	= 809,
    PPC_EFDCTSF	= 810,
    PPC_EFDCTSI	= 811,
    PPC_EFDCTSIDZ	= 812,
    PPC_EFDCTSIZ	= 813,
    PPC_EFDCTUF	= 814,
    PPC_EFDCTUI	= 815,
    PPC_EFDCTUIDZ	= 816,
    PPC_EFDCTUIZ	= 817,
    PPC_EFDDIV	= 818,
    PPC_EFDMUL	= 819,
    PPC_EFDNABS	= 820,
    PPC_EFDNEG	= 821,
    PPC_EFDSUB	= 822,
    PPC_EFDTSTEQ	= 823,
    PPC_EFDTSTGT	= 824,
    PPC_EFDTSTLT	= 825,
    PPC_EFSABS	= 826,
    PPC_EFSADD	= 827,
    PPC_EFSCFD	= 828,
    PPC_EFSCFSF	= 829,
    PPC_EFSCFSI	= 830,
    PPC_EFSCFUF	= 831,
    PPC_EFSCFUI	= 832,
    PPC_EFSCMPEQ	= 833,
    PPC_EFSCMPGT	= 834,
    PPC_EFSCMPLT	= 835,
    PPC_EFSCTSF	= 836,
    PPC_EFSCTSI	= 837,
    PPC_EFSCTSIZ	= 838,
    PPC_EFSCTUF	= 839,
    PPC_EFSCTUI	= 840,
    PPC_EFSCTUIZ	= 841,
    PPC_EFSDIV	= 842,
    PPC_EFSMUL	= 843,
    PPC_EFSNABS	= 844,
    PPC_EFSNEG	= 845,
    PPC_EFSSUB	= 846,
    PPC_EFSTSTEQ	= 847,
    PPC_EFSTSTGT	= 848,
    PPC_EFSTSTLT	= 849,
    PPC_EH_SjLj_LongJmp32	= 850,
    PPC_EH_SjLj_LongJmp64	= 851,
    PPC_EH_SjLj_SetJmp32	= 852,
    PPC_EH_SjLj_SetJmp64	= 853,
    PPC_EH_SjLj_Setup	= 854,
    PPC_EQV	= 855,
    PPC_EQV8	= 856,
    PPC_EQV8_rec	= 857,
    PPC_EQV_rec	= 858,
    PPC_EVABS	= 859,
    PPC_EVADDIW	= 860,
    PPC_EVADDSMIAAW	= 861,
    PPC_EVADDSSIAAW	= 862,
    PPC_EVADDUMIAAW	= 863,
    PPC_EVADDUSIAAW	= 864,
    PPC_EVADDW	= 865,
    PPC_EVAND	= 866,
    PPC_EVANDC	= 867,
    PPC_EVCMPEQ	= 868,
    PPC_EVCMPGTS	= 869,
    PPC_EVCMPGTU	= 870,
    PPC_EVCMPLTS	= 871,
    PPC_EVCMPLTU	= 872,
    PPC_EVCNTLSW	= 873,
    PPC_EVCNTLZW	= 874,
    PPC_EVDIVWS	= 875,
    PPC_EVDIVWU	= 876,
    PPC_EVEQV	= 877,
    PPC_EVEXTSB	= 878,
    PPC_EVEXTSH	= 879,
    PPC_EVFSABS	= 880,
    PPC_EVFSADD	= 881,
    PPC_EVFSCFSF	= 882,
    PPC_EVFSCFSI	= 883,
    PPC_EVFSCFUF	= 884,
    PPC_EVFSCFUI	= 885,
    PPC_EVFSCMPEQ	= 886,
    PPC_EVFSCMPGT	= 887,
    PPC_EVFSCMPLT	= 888,
    PPC_EVFSCTSF	= 889,
    PPC_EVFSCTSI	= 890,
    PPC_EVFSCTSIZ	= 891,
    PPC_EVFSCTUF	= 892,
    PPC_EVFSCTUI	= 893,
    PPC_EVFSCTUIZ	= 894,
    PPC_EVFSDIV	= 895,
    PPC_EVFSMUL	= 896,
    PPC_EVFSNABS	= 897,
    PPC_EVFSNEG	= 898,
    PPC_EVFSSUB	= 899,
    PPC_EVFSTSTEQ	= 900,
    PPC_EVFSTSTGT	= 901,
    PPC_EVFSTSTLT	= 902,
    PPC_EVLDD	= 903,
    PPC_EVLDDX	= 904,
    PPC_EVLDH	= 905,
    PPC_EVLDHX	= 906,
    PPC_EVLDW	= 907,
    PPC_EVLDWX	= 908,
    PPC_EVLHHESPLAT	= 909,
    PPC_EVLHHESPLATX	= 910,
    PPC_EVLHHOSSPLAT	= 911,
    PPC_EVLHHOSSPLATX	= 912,
    PPC_EVLHHOUSPLAT	= 913,
    PPC_EVLHHOUSPLATX	= 914,
    PPC_EVLWHE	= 915,
    PPC_EVLWHEX	= 916,
    PPC_EVLWHOS	= 917,
    PPC_EVLWHOSX	= 918,
    PPC_EVLWHOU	= 919,
    PPC_EVLWHOUX	= 920,
    PPC_EVLWHSPLAT	= 921,
    PPC_EVLWHSPLATX	= 922,
    PPC_EVLWWSPLAT	= 923,
    PPC_EVLWWSPLATX	= 924,
    PPC_EVMERGEHI	= 925,
    PPC_EVMERGEHILO	= 926,
    PPC_EVMERGELO	= 927,
    PPC_EVMERGELOHI	= 928,
    PPC_EVMHEGSMFAA	= 929,
    PPC_EVMHEGSMFAN	= 930,
    PPC_EVMHEGSMIAA	= 931,
    PPC_EVMHEGSMIAN	= 932,
    PPC_EVMHEGUMIAA	= 933,
    PPC_EVMHEGUMIAN	= 934,
    PPC_EVMHESMF	= 935,
    PPC_EVMHESMFA	= 936,
    PPC_EVMHESMFAAW	= 937,
    PPC_EVMHESMFANW	= 938,
    PPC_EVMHESMI	= 939,
    PPC_EVMHESMIA	= 940,
    PPC_EVMHESMIAAW	= 941,
    PPC_EVMHESMIANW	= 942,
    PPC_EVMHESSF	= 943,
    PPC_EVMHESSFA	= 944,
    PPC_EVMHESSFAAW	= 945,
    PPC_EVMHESSFANW	= 946,
    PPC_EVMHESSIAAW	= 947,
    PPC_EVMHESSIANW	= 948,
    PPC_EVMHEUMI	= 949,
    PPC_EVMHEUMIA	= 950,
    PPC_EVMHEUMIAAW	= 951,
    PPC_EVMHEUMIANW	= 952,
    PPC_EVMHEUSIAAW	= 953,
    PPC_EVMHEUSIANW	= 954,
    PPC_EVMHOGSMFAA	= 955,
    PPC_EVMHOGSMFAN	= 956,
    PPC_EVMHOGSMIAA	= 957,
    PPC_EVMHOGSMIAN	= 958,
    PPC_EVMHOGUMIAA	= 959,
    PPC_EVMHOGUMIAN	= 960,
    PPC_EVMHOSMF	= 961,
    PPC_EVMHOSMFA	= 962,
    PPC_EVMHOSMFAAW	= 963,
    PPC_EVMHOSMFANW	= 964,
    PPC_EVMHOSMI	= 965,
    PPC_EVMHOSMIA	= 966,
    PPC_EVMHOSMIAAW	= 967,
    PPC_EVMHOSMIANW	= 968,
    PPC_EVMHOSSF	= 969,
    PPC_EVMHOSSFA	= 970,
    PPC_EVMHOSSFAAW	= 971,
    PPC_EVMHOSSFANW	= 972,
    PPC_EVMHOSSIAAW	= 973,
    PPC_EVMHOSSIANW	= 974,
    PPC_EVMHOUMI	= 975,
    PPC_EVMHOUMIA	= 976,
    PPC_EVMHOUMIAAW	= 977,
    PPC_EVMHOUMIANW	= 978,
    PPC_EVMHOUSIAAW	= 979,
    PPC_EVMHOUSIANW	= 980,
    PPC_EVMRA	= 981,
    PPC_EVMWHSMF	= 982,
    PPC_EVMWHSMFA	= 983,
    PPC_EVMWHSMI	= 984,
    PPC_EVMWHSMIA	= 985,
    PPC_EVMWHSSF	= 986,
    PPC_EVMWHSSFA	= 987,
    PPC_EVMWHUMI	= 988,
    PPC_EVMWHUMIA	= 989,
    PPC_EVMWLSMIAAW	= 990,
    PPC_EVMWLSMIANW	= 991,
    PPC_EVMWLSSIAAW	= 992,
    PPC_EVMWLSSIANW	= 993,
    PPC_EVMWLUMI	= 994,
    PPC_EVMWLUMIA	= 995,
    PPC_EVMWLUMIAAW	= 996,
    PPC_EVMWLUMIANW	= 997,
    PPC_EVMWLUSIAAW	= 998,
    PPC_EVMWLUSIANW	= 999,
    PPC_EVMWSMF	= 1000,
    PPC_EVMWSMFA	= 1001,
    PPC_EVMWSMFAA	= 1002,
    PPC_EVMWSMFAN	= 1003,
    PPC_EVMWSMI	= 1004,
    PPC_EVMWSMIA	= 1005,
    PPC_EVMWSMIAA	= 1006,
    PPC_EVMWSMIAN	= 1007,
    PPC_EVMWSSF	= 1008,
    PPC_EVMWSSFA	= 1009,
    PPC_EVMWSSFAA	= 1010,
    PPC_EVMWSSFAN	= 1011,
    PPC_EVMWUMI	= 1012,
    PPC_EVMWUMIA	= 1013,
    PPC_EVMWUMIAA	= 1014,
    PPC_EVMWUMIAN	= 1015,
    PPC_EVNAND	= 1016,
    PPC_EVNEG	= 1017,
    PPC_EVNOR	= 1018,
    PPC_EVOR	= 1019,
    PPC_EVORC	= 1020,
    PPC_EVRLW	= 1021,
    PPC_EVRLWI	= 1022,
    PPC_EVRNDW	= 1023,
    PPC_EVSEL	= 1024,
    PPC_EVSLW	= 1025,
    PPC_EVSLWI	= 1026,
    PPC_EVSPLATFI	= 1027,
    PPC_EVSPLATI	= 1028,
    PPC_EVSRWIS	= 1029,
    PPC_EVSRWIU	= 1030,
    PPC_EVSRWS	= 1031,
    PPC_EVSRWU	= 1032,
    PPC_EVSTDD	= 1033,
    PPC_EVSTDDX	= 1034,
    PPC_EVSTDH	= 1035,
    PPC_EVSTDHX	= 1036,
    PPC_EVSTDW	= 1037,
    PPC_EVSTDWX	= 1038,
    PPC_EVSTWHE	= 1039,
    PPC_EVSTWHEX	= 1040,
    PPC_EVSTWHO	= 1041,
    PPC_EVSTWHOX	= 1042,
    PPC_EVSTWWE	= 1043,
    PPC_EVSTWWEX	= 1044,
    PPC_EVSTWWO	= 1045,
    PPC_EVSTWWOX	= 1046,
    PPC_EVSUBFSMIAAW	= 1047,
    PPC_EVSUBFSSIAAW	= 1048,
    PPC_EVSUBFUMIAAW	= 1049,
    PPC_EVSUBFUSIAAW	= 1050,
    PPC_EVSUBFW	= 1051,
    PPC_EVSUBIFW	= 1052,
    PPC_EVXOR	= 1053,
    PPC_EXTSB	= 1054,
    PPC_EXTSB8	= 1055,
    PPC_EXTSB8_32_64	= 1056,
    PPC_EXTSB8_rec	= 1057,
    PPC_EXTSB_rec	= 1058,
    PPC_EXTSH	= 1059,
    PPC_EXTSH8	= 1060,
    PPC_EXTSH8_32_64	= 1061,
    PPC_EXTSH8_rec	= 1062,
    PPC_EXTSH_rec	= 1063,
    PPC_EXTSW	= 1064,
    PPC_EXTSWSLI	= 1065,
    PPC_EXTSWSLI_32_64	= 1066,
    PPC_EXTSWSLI_32_64_rec	= 1067,
    PPC_EXTSWSLI_rec	= 1068,
    PPC_EXTSW_32	= 1069,
    PPC_EXTSW_32_64	= 1070,
    PPC_EXTSW_32_64_rec	= 1071,
    PPC_EXTSW_rec	= 1072,
    PPC_EnforceIEIO	= 1073,
    PPC_FABSD	= 1074,
    PPC_FABSD_rec	= 1075,
    PPC_FABSS	= 1076,
    PPC_FABSS_rec	= 1077,
    PPC_FADD	= 1078,
    PPC_FADDS	= 1079,
    PPC_FADDS_rec	= 1080,
    PPC_FADD_rec	= 1081,
    PPC_FADDrtz	= 1082,
    PPC_FCFID	= 1083,
    PPC_FCFIDS	= 1084,
    PPC_FCFIDS_rec	= 1085,
    PPC_FCFIDU	= 1086,
    PPC_FCFIDUS	= 1087,
    PPC_FCFIDUS_rec	= 1088,
    PPC_FCFIDU_rec	= 1089,
    PPC_FCFID_rec	= 1090,
    PPC_FCMPOD	= 1091,
    PPC_FCMPOS	= 1092,
    PPC_FCMPUD	= 1093,
    PPC_FCMPUS	= 1094,
    PPC_FCPSGND	= 1095,
    PPC_FCPSGND_rec	= 1096,
    PPC_FCPSGNS	= 1097,
    PPC_FCPSGNS_rec	= 1098,
    PPC_FCTID	= 1099,
    PPC_FCTIDU	= 1100,
    PPC_FCTIDUZ	= 1101,
    PPC_FCTIDUZ_rec	= 1102,
    PPC_FCTIDU_rec	= 1103,
    PPC_FCTIDZ	= 1104,
    PPC_FCTIDZ_rec	= 1105,
    PPC_FCTID_rec	= 1106,
    PPC_FCTIW	= 1107,
    PPC_FCTIWU	= 1108,
    PPC_FCTIWUZ	= 1109,
    PPC_FCTIWUZ_rec	= 1110,
    PPC_FCTIWU_rec	= 1111,
    PPC_FCTIWZ	= 1112,
    PPC_FCTIWZ_rec	= 1113,
    PPC_FCTIW_rec	= 1114,
    PPC_FDIV	= 1115,
    PPC_FDIVS	= 1116,
    PPC_FDIVS_rec	= 1117,
    PPC_FDIV_rec	= 1118,
    PPC_FENCE	= 1119,
    PPC_FMADD	= 1120,
    PPC_FMADDS	= 1121,
    PPC_FMADDS_rec	= 1122,
    PPC_FMADD_rec	= 1123,
    PPC_FMR	= 1124,
    PPC_FMR_rec	= 1125,
    PPC_FMSUB	= 1126,
    PPC_FMSUBS	= 1127,
    PPC_FMSUBS_rec	= 1128,
    PPC_FMSUB_rec	= 1129,
    PPC_FMUL	= 1130,
    PPC_FMULS	= 1131,
    PPC_FMULS_rec	= 1132,
    PPC_FMUL_rec	= 1133,
    PPC_FNABSD	= 1134,
    PPC_FNABSD_rec	= 1135,
    PPC_FNABSS	= 1136,
    PPC_FNABSS_rec	= 1137,
    PPC_FNEGD	= 1138,
    PPC_FNEGD_rec	= 1139,
    PPC_FNEGS	= 1140,
    PPC_FNEGS_rec	= 1141,
    PPC_FNMADD	= 1142,
    PPC_FNMADDS	= 1143,
    PPC_FNMADDS_rec	= 1144,
    PPC_FNMADD_rec	= 1145,
    PPC_FNMSUB	= 1146,
    PPC_FNMSUBS	= 1147,
    PPC_FNMSUBS_rec	= 1148,
    PPC_FNMSUB_rec	= 1149,
    PPC_FRE	= 1150,
    PPC_FRES	= 1151,
    PPC_FRES_rec	= 1152,
    PPC_FRE_rec	= 1153,
    PPC_FRIMD	= 1154,
    PPC_FRIMD_rec	= 1155,
    PPC_FRIMS	= 1156,
    PPC_FRIMS_rec	= 1157,
    PPC_FRIND	= 1158,
    PPC_FRIND_rec	= 1159,
    PPC_FRINS	= 1160,
    PPC_FRINS_rec	= 1161,
    PPC_FRIPD	= 1162,
    PPC_FRIPD_rec	= 1163,
    PPC_FRIPS	= 1164,
    PPC_FRIPS_rec	= 1165,
    PPC_FRIZD	= 1166,
    PPC_FRIZD_rec	= 1167,
    PPC_FRIZS	= 1168,
    PPC_FRIZS_rec	= 1169,
    PPC_FRSP	= 1170,
    PPC_FRSP_rec	= 1171,
    PPC_FRSQRTE	= 1172,
    PPC_FRSQRTES	= 1173,
    PPC_FRSQRTES_rec	= 1174,
    PPC_FRSQRTE_rec	= 1175,
    PPC_FSELD	= 1176,
    PPC_FSELD_rec	= 1177,
    PPC_FSELS	= 1178,
    PPC_FSELS_rec	= 1179,
    PPC_FSQRT	= 1180,
    PPC_FSQRTS	= 1181,
    PPC_FSQRTS_rec	= 1182,
    PPC_FSQRT_rec	= 1183,
    PPC_FSUB	= 1184,
    PPC_FSUBS	= 1185,
    PPC_FSUBS_rec	= 1186,
    PPC_FSUB_rec	= 1187,
    PPC_FTDIV	= 1188,
    PPC_FTSQRT	= 1189,
    PPC_GETtlsADDR	= 1190,
    PPC_GETtlsADDR32	= 1191,
    PPC_GETtlsADDR32AIX	= 1192,
    PPC_GETtlsADDR64AIX	= 1193,
    PPC_GETtlsADDRPCREL	= 1194,
    PPC_GETtlsTpointer32AIX	= 1195,
    PPC_GETtlsldADDR	= 1196,
    PPC_GETtlsldADDR32	= 1197,
    PPC_GETtlsldADDRPCREL	= 1198,
    PPC_HASHCHK	= 1199,
    PPC_HASHCHK8	= 1200,
    PPC_HASHCHKP	= 1201,
    PPC_HASHCHKP8	= 1202,
    PPC_HASHST	= 1203,
    PPC_HASHST8	= 1204,
    PPC_HASHSTP	= 1205,
    PPC_HASHSTP8	= 1206,
    PPC_HRFID	= 1207,
    PPC_ICBI	= 1208,
    PPC_ICBIEP	= 1209,
    PPC_ICBLC	= 1210,
    PPC_ICBLQ	= 1211,
    PPC_ICBT	= 1212,
    PPC_ICBTLS	= 1213,
    PPC_ICCCI	= 1214,
    PPC_ISEL	= 1215,
    PPC_ISEL8	= 1216,
    PPC_ISYNC	= 1217,
    PPC_LA	= 1218,
    PPC_LA8	= 1219,
    PPC_LBARX	= 1220,
    PPC_LBARXL	= 1221,
    PPC_LBEPX	= 1222,
    PPC_LBZ	= 1223,
    PPC_LBZ8	= 1224,
    PPC_LBZCIX	= 1225,
    PPC_LBZU	= 1226,
    PPC_LBZU8	= 1227,
    PPC_LBZUX	= 1228,
    PPC_LBZUX8	= 1229,
    PPC_LBZX	= 1230,
    PPC_LBZX8	= 1231,
    PPC_LBZXTLS	= 1232,
    PPC_LBZXTLS_	= 1233,
    PPC_LBZXTLS_32	= 1234,
    PPC_LD	= 1235,
    PPC_LDARX	= 1236,
    PPC_LDARXL	= 1237,
    PPC_LDAT	= 1238,
    PPC_LDBRX	= 1239,
    PPC_LDCIX	= 1240,
    PPC_LDU	= 1241,
    PPC_LDUX	= 1242,
    PPC_LDX	= 1243,
    PPC_LDXTLS	= 1244,
    PPC_LDXTLS_	= 1245,
    PPC_LDgotTprelL	= 1246,
    PPC_LDgotTprelL32	= 1247,
    PPC_LDtoc	= 1248,
    PPC_LDtocBA	= 1249,
    PPC_LDtocCPT	= 1250,
    PPC_LDtocJTI	= 1251,
    PPC_LDtocL	= 1252,
    PPC_LFD	= 1253,
    PPC_LFDEPX	= 1254,
    PPC_LFDU	= 1255,
    PPC_LFDUX	= 1256,
    PPC_LFDX	= 1257,
    PPC_LFDXTLS	= 1258,
    PPC_LFDXTLS_	= 1259,
    PPC_LFIWAX	= 1260,
    PPC_LFIWZX	= 1261,
    PPC_LFS	= 1262,
    PPC_LFSU	= 1263,
    PPC_LFSUX	= 1264,
    PPC_LFSX	= 1265,
    PPC_LFSXTLS	= 1266,
    PPC_LFSXTLS_	= 1267,
    PPC_LHA	= 1268,
    PPC_LHA8	= 1269,
    PPC_LHARX	= 1270,
    PPC_LHARXL	= 1271,
    PPC_LHAU	= 1272,
    PPC_LHAU8	= 1273,
    PPC_LHAUX	= 1274,
    PPC_LHAUX8	= 1275,
    PPC_LHAX	= 1276,
    PPC_LHAX8	= 1277,
    PPC_LHAXTLS	= 1278,
    PPC_LHAXTLS_	= 1279,
    PPC_LHAXTLS_32	= 1280,
    PPC_LHBRX	= 1281,
    PPC_LHBRX8	= 1282,
    PPC_LHEPX	= 1283,
    PPC_LHZ	= 1284,
    PPC_LHZ8	= 1285,
    PPC_LHZCIX	= 1286,
    PPC_LHZU	= 1287,
    PPC_LHZU8	= 1288,
    PPC_LHZUX	= 1289,
    PPC_LHZUX8	= 1290,
    PPC_LHZX	= 1291,
    PPC_LHZX8	= 1292,
    PPC_LHZXTLS	= 1293,
    PPC_LHZXTLS_	= 1294,
    PPC_LHZXTLS_32	= 1295,
    PPC_LMW	= 1296,
    PPC_LQ	= 1297,
    PPC_LQARX	= 1298,
    PPC_LQARXL	= 1299,
    PPC_LQX_PSEUDO	= 1300,
    PPC_LSWI	= 1301,
    PPC_LVEBX	= 1302,
    PPC_LVEHX	= 1303,
    PPC_LVEWX	= 1304,
    PPC_LVSL	= 1305,
    PPC_LVSR	= 1306,
    PPC_LVX	= 1307,
    PPC_LVXL	= 1308,
    PPC_LWA	= 1309,
    PPC_LWARX	= 1310,
    PPC_LWARXL	= 1311,
    PPC_LWAT	= 1312,
    PPC_LWAUX	= 1313,
    PPC_LWAX	= 1314,
    PPC_LWAXTLS	= 1315,
    PPC_LWAXTLS_	= 1316,
    PPC_LWAXTLS_32	= 1317,
    PPC_LWAX_32	= 1318,
    PPC_LWA_32	= 1319,
    PPC_LWBRX	= 1320,
    PPC_LWBRX8	= 1321,
    PPC_LWEPX	= 1322,
    PPC_LWZ	= 1323,
    PPC_LWZ8	= 1324,
    PPC_LWZCIX	= 1325,
    PPC_LWZU	= 1326,
    PPC_LWZU8	= 1327,
    PPC_LWZUX	= 1328,
    PPC_LWZUX8	= 1329,
    PPC_LWZX	= 1330,
    PPC_LWZX8	= 1331,
    PPC_LWZXTLS	= 1332,
    PPC_LWZXTLS_	= 1333,
    PPC_LWZXTLS_32	= 1334,
    PPC_LWZtoc	= 1335,
    PPC_LWZtocL	= 1336,
    PPC_LXSD	= 1337,
    PPC_LXSDX	= 1338,
    PPC_LXSIBZX	= 1339,
    PPC_LXSIHZX	= 1340,
    PPC_LXSIWAX	= 1341,
    PPC_LXSIWZX	= 1342,
    PPC_LXSSP	= 1343,
    PPC_LXSSPX	= 1344,
    PPC_LXV	= 1345,
    PPC_LXVB16X	= 1346,
    PPC_LXVD2X	= 1347,
    PPC_LXVDSX	= 1348,
    PPC_LXVH8X	= 1349,
    PPC_LXVKQ	= 1350,
    PPC_LXVL	= 1351,
    PPC_LXVLL	= 1352,
    PPC_LXVP	= 1353,
    PPC_LXVPRL	= 1354,
    PPC_LXVPRLL	= 1355,
    PPC_LXVPX	= 1356,
    PPC_LXVRBX	= 1357,
    PPC_LXVRDX	= 1358,
    PPC_LXVRHX	= 1359,
    PPC_LXVRL	= 1360,
    PPC_LXVRLL	= 1361,
    PPC_LXVRWX	= 1362,
    PPC_LXVW4X	= 1363,
    PPC_LXVWSX	= 1364,
    PPC_LXVX	= 1365,
    PPC_MADDHD	= 1366,
    PPC_MADDHDU	= 1367,
    PPC_MADDLD	= 1368,
    PPC_MADDLD8	= 1369,
    PPC_MBAR	= 1370,
    PPC_MCRF	= 1371,
    PPC_MCRFS	= 1372,
    PPC_MCRXRX	= 1373,
    PPC_MFBHRBE	= 1374,
    PPC_MFCR	= 1375,
    PPC_MFCR8	= 1376,
    PPC_MFCTR	= 1377,
    PPC_MFCTR8	= 1378,
    PPC_MFDCR	= 1379,
    PPC_MFFS	= 1380,
    PPC_MFFSCDRN	= 1381,
    PPC_MFFSCDRNI	= 1382,
    PPC_MFFSCE	= 1383,
    PPC_MFFSCRN	= 1384,
    PPC_MFFSCRNI	= 1385,
    PPC_MFFSL	= 1386,
    PPC_MFFS_rec	= 1387,
    PPC_MFLR	= 1388,
    PPC_MFLR8	= 1389,
    PPC_MFMSR	= 1390,
    PPC_MFOCRF	= 1391,
    PPC_MFOCRF8	= 1392,
    PPC_MFPMR	= 1393,
    PPC_MFSPR	= 1394,
    PPC_MFSPR8	= 1395,
    PPC_MFSR	= 1396,
    PPC_MFSRIN	= 1397,
    PPC_MFTB	= 1398,
    PPC_MFTB8	= 1399,
    PPC_MFUDSCR	= 1400,
    PPC_MFVRD	= 1401,
    PPC_MFVRSAVE	= 1402,
    PPC_MFVRSAVEv	= 1403,
    PPC_MFVRWZ	= 1404,
    PPC_MFVSCR	= 1405,
    PPC_MFVSRD	= 1406,
    PPC_MFVSRLD	= 1407,
    PPC_MFVSRWZ	= 1408,
    PPC_MODSD	= 1409,
    PPC_MODSW	= 1410,
    PPC_MODUD	= 1411,
    PPC_MODUW	= 1412,
    PPC_MSGSYNC	= 1413,
    PPC_MSYNC	= 1414,
    PPC_MTCRF	= 1415,
    PPC_MTCRF8	= 1416,
    PPC_MTCTR	= 1417,
    PPC_MTCTR8	= 1418,
    PPC_MTCTR8loop	= 1419,
    PPC_MTCTRloop	= 1420,
    PPC_MTDCR	= 1421,
    PPC_MTFSB0	= 1422,
    PPC_MTFSB1	= 1423,
    PPC_MTFSF	= 1424,
    PPC_MTFSFI	= 1425,
    PPC_MTFSFI_rec	= 1426,
    PPC_MTFSFIb	= 1427,
    PPC_MTFSF_rec	= 1428,
    PPC_MTFSFb	= 1429,
    PPC_MTLR	= 1430,
    PPC_MTLR8	= 1431,
    PPC_MTMSR	= 1432,
    PPC_MTMSRD	= 1433,
    PPC_MTOCRF	= 1434,
    PPC_MTOCRF8	= 1435,
    PPC_MTPMR	= 1436,
    PPC_MTSPR	= 1437,
    PPC_MTSPR8	= 1438,
    PPC_MTSR	= 1439,
    PPC_MTSRIN	= 1440,
    PPC_MTUDSCR	= 1441,
    PPC_MTVRD	= 1442,
    PPC_MTVRSAVE	= 1443,
    PPC_MTVRSAVEv	= 1444,
    PPC_MTVRWA	= 1445,
    PPC_MTVRWZ	= 1446,
    PPC_MTVSCR	= 1447,
    PPC_MTVSRBM	= 1448,
    PPC_MTVSRBMI	= 1449,
    PPC_MTVSRD	= 1450,
    PPC_MTVSRDD	= 1451,
    PPC_MTVSRDM	= 1452,
    PPC_MTVSRHM	= 1453,
    PPC_MTVSRQM	= 1454,
    PPC_MTVSRWA	= 1455,
    PPC_MTVSRWM	= 1456,
    PPC_MTVSRWS	= 1457,
    PPC_MTVSRWZ	= 1458,
    PPC_MULHD	= 1459,
    PPC_MULHDU	= 1460,
    PPC_MULHDU_rec	= 1461,
    PPC_MULHD_rec	= 1462,
    PPC_MULHW	= 1463,
    PPC_MULHWU	= 1464,
    PPC_MULHWU_rec	= 1465,
    PPC_MULHW_rec	= 1466,
    PPC_MULLD	= 1467,
    PPC_MULLDO	= 1468,
    PPC_MULLDO_rec	= 1469,
    PPC_MULLD_rec	= 1470,
    PPC_MULLI	= 1471,
    PPC_MULLI8	= 1472,
    PPC_MULLW	= 1473,
    PPC_MULLWO	= 1474,
    PPC_MULLWO_rec	= 1475,
    PPC_MULLW_rec	= 1476,
    PPC_MoveGOTtoLR	= 1477,
    PPC_MovePCtoLR	= 1478,
    PPC_MovePCtoLR8	= 1479,
    PPC_NAND	= 1480,
    PPC_NAND8	= 1481,
    PPC_NAND8_rec	= 1482,
    PPC_NAND_rec	= 1483,
    PPC_NAP	= 1484,
    PPC_NEG	= 1485,
    PPC_NEG8	= 1486,
    PPC_NEG8O	= 1487,
    PPC_NEG8O_rec	= 1488,
    PPC_NEG8_rec	= 1489,
    PPC_NEGO	= 1490,
    PPC_NEGO_rec	= 1491,
    PPC_NEG_rec	= 1492,
    PPC_NOP	= 1493,
    PPC_NOP_GT_PWR6	= 1494,
    PPC_NOP_GT_PWR7	= 1495,
    PPC_NOR	= 1496,
    PPC_NOR8	= 1497,
    PPC_NOR8_rec	= 1498,
    PPC_NOR_rec	= 1499,
    PPC_OR	= 1500,
    PPC_OR8	= 1501,
    PPC_OR8_rec	= 1502,
    PPC_ORC	= 1503,
    PPC_ORC8	= 1504,
    PPC_ORC8_rec	= 1505,
    PPC_ORC_rec	= 1506,
    PPC_ORI	= 1507,
    PPC_ORI8	= 1508,
    PPC_ORIS	= 1509,
    PPC_ORIS8	= 1510,
    PPC_OR_rec	= 1511,
    PPC_PADDI	= 1512,
    PPC_PADDI8	= 1513,
    PPC_PADDI8pc	= 1514,
    PPC_PADDIdtprel	= 1515,
    PPC_PADDIpc	= 1516,
    PPC_PDEPD	= 1517,
    PPC_PEXTD	= 1518,
    PPC_PLA	= 1519,
    PPC_PLA8	= 1520,
    PPC_PLA8pc	= 1521,
    PPC_PLApc	= 1522,
    PPC_PLBZ	= 1523,
    PPC_PLBZ8	= 1524,
    PPC_PLBZ8nopc	= 1525,
    PPC_PLBZ8onlypc	= 1526,
    PPC_PLBZ8pc	= 1527,
    PPC_PLBZnopc	= 1528,
    PPC_PLBZonlypc	= 1529,
    PPC_PLBZpc	= 1530,
    PPC_PLD	= 1531,
    PPC_PLDnopc	= 1532,
    PPC_PLDonlypc	= 1533,
    PPC_PLDpc	= 1534,
    PPC_PLFD	= 1535,
    PPC_PLFDnopc	= 1536,
    PPC_PLFDonlypc	= 1537,
    PPC_PLFDpc	= 1538,
    PPC_PLFS	= 1539,
    PPC_PLFSnopc	= 1540,
    PPC_PLFSonlypc	= 1541,
    PPC_PLFSpc	= 1542,
    PPC_PLHA	= 1543,
    PPC_PLHA8	= 1544,
    PPC_PLHA8nopc	= 1545,
    PPC_PLHA8onlypc	= 1546,
    PPC_PLHA8pc	= 1547,
    PPC_PLHAnopc	= 1548,
    PPC_PLHAonlypc	= 1549,
    PPC_PLHApc	= 1550,
    PPC_PLHZ	= 1551,
    PPC_PLHZ8	= 1552,
    PPC_PLHZ8nopc	= 1553,
    PPC_PLHZ8onlypc	= 1554,
    PPC_PLHZ8pc	= 1555,
    PPC_PLHZnopc	= 1556,
    PPC_PLHZonlypc	= 1557,
    PPC_PLHZpc	= 1558,
    PPC_PLI	= 1559,
    PPC_PLI8	= 1560,
    PPC_PLWA	= 1561,
    PPC_PLWA8	= 1562,
    PPC_PLWA8nopc	= 1563,
    PPC_PLWA8onlypc	= 1564,
    PPC_PLWA8pc	= 1565,
    PPC_PLWAnopc	= 1566,
    PPC_PLWAonlypc	= 1567,
    PPC_PLWApc	= 1568,
    PPC_PLWZ	= 1569,
    PPC_PLWZ8	= 1570,
    PPC_PLWZ8nopc	= 1571,
    PPC_PLWZ8onlypc	= 1572,
    PPC_PLWZ8pc	= 1573,
    PPC_PLWZnopc	= 1574,
    PPC_PLWZonlypc	= 1575,
    PPC_PLWZpc	= 1576,
    PPC_PLXSD	= 1577,
    PPC_PLXSDnopc	= 1578,
    PPC_PLXSDonlypc	= 1579,
    PPC_PLXSDpc	= 1580,
    PPC_PLXSSP	= 1581,
    PPC_PLXSSPnopc	= 1582,
    PPC_PLXSSPonlypc	= 1583,
    PPC_PLXSSPpc	= 1584,
    PPC_PLXV	= 1585,
    PPC_PLXVP	= 1586,
    PPC_PLXVPnopc	= 1587,
    PPC_PLXVPonlypc	= 1588,
    PPC_PLXVPpc	= 1589,
    PPC_PLXVnopc	= 1590,
    PPC_PLXVonlypc	= 1591,
    PPC_PLXVpc	= 1592,
    PPC_PMXVBF16GER2	= 1593,
    PPC_PMXVBF16GER2NN	= 1594,
    PPC_PMXVBF16GER2NP	= 1595,
    PPC_PMXVBF16GER2PN	= 1596,
    PPC_PMXVBF16GER2PP	= 1597,
    PPC_PMXVBF16GER2W	= 1598,
    PPC_PMXVBF16GER2WNN	= 1599,
    PPC_PMXVBF16GER2WNP	= 1600,
    PPC_PMXVBF16GER2WPN	= 1601,
    PPC_PMXVBF16GER2WPP	= 1602,
    PPC_PMXVF16GER2	= 1603,
    PPC_PMXVF16GER2NN	= 1604,
    PPC_PMXVF16GER2NP	= 1605,
    PPC_PMXVF16GER2PN	= 1606,
    PPC_PMXVF16GER2PP	= 1607,
    PPC_PMXVF16GER2W	= 1608,
    PPC_PMXVF16GER2WNN	= 1609,
    PPC_PMXVF16GER2WNP	= 1610,
    PPC_PMXVF16GER2WPN	= 1611,
    PPC_PMXVF16GER2WPP	= 1612,
    PPC_PMXVF32GER	= 1613,
    PPC_PMXVF32GERNN	= 1614,
    PPC_PMXVF32GERNP	= 1615,
    PPC_PMXVF32GERPN	= 1616,
    PPC_PMXVF32GERPP	= 1617,
    PPC_PMXVF32GERW	= 1618,
    PPC_PMXVF32GERWNN	= 1619,
    PPC_PMXVF32GERWNP	= 1620,
    PPC_PMXVF32GERWPN	= 1621,
    PPC_PMXVF32GERWPP	= 1622,
    PPC_PMXVF64GER	= 1623,
    PPC_PMXVF64GERNN	= 1624,
    PPC_PMXVF64GERNP	= 1625,
    PPC_PMXVF64GERPN	= 1626,
    PPC_PMXVF64GERPP	= 1627,
    PPC_PMXVF64GERW	= 1628,
    PPC_PMXVF64GERWNN	= 1629,
    PPC_PMXVF64GERWNP	= 1630,
    PPC_PMXVF64GERWPN	= 1631,
    PPC_PMXVF64GERWPP	= 1632,
    PPC_PMXVI16GER2	= 1633,
    PPC_PMXVI16GER2PP	= 1634,
    PPC_PMXVI16GER2S	= 1635,
    PPC_PMXVI16GER2SPP	= 1636,
    PPC_PMXVI16GER2SW	= 1637,
    PPC_PMXVI16GER2SWPP	= 1638,
    PPC_PMXVI16GER2W	= 1639,
    PPC_PMXVI16GER2WPP	= 1640,
    PPC_PMXVI4GER8	= 1641,
    PPC_PMXVI4GER8PP	= 1642,
    PPC_PMXVI4GER8W	= 1643,
    PPC_PMXVI4GER8WPP	= 1644,
    PPC_PMXVI8GER4	= 1645,
    PPC_PMXVI8GER4PP	= 1646,
    PPC_PMXVI8GER4SPP	= 1647,
    PPC_PMXVI8GER4W	= 1648,
    PPC_PMXVI8GER4WPP	= 1649,
    PPC_PMXVI8GER4WSPP	= 1650,
    PPC_POPCNTB	= 1651,
    PPC_POPCNTB8	= 1652,
    PPC_POPCNTD	= 1653,
    PPC_POPCNTW	= 1654,
    PPC_PPC32GOT	= 1655,
    PPC_PPC32PICGOT	= 1656,
    PPC_PREPARE_PROBED_ALLOCA_32	= 1657,
    PPC_PREPARE_PROBED_ALLOCA_64	= 1658,
    PPC_PREPARE_PROBED_ALLOCA_NEGSIZE_SAME_REG_32	= 1659,
    PPC_PREPARE_PROBED_ALLOCA_NEGSIZE_SAME_REG_64	= 1660,
    PPC_PROBED_ALLOCA_32	= 1661,
    PPC_PROBED_ALLOCA_64	= 1662,
    PPC_PROBED_STACKALLOC_32	= 1663,
    PPC_PROBED_STACKALLOC_64	= 1664,
    PPC_PSC_DCBZL	= 1665,
    PPC_PSQ_L	= 1666,
    PPC_PSQ_LU	= 1667,
    PPC_PSQ_LUX	= 1668,
    PPC_PSQ_LX	= 1669,
    PPC_PSQ_ST	= 1670,
    PPC_PSQ_STU	= 1671,
    PPC_PSQ_STUX	= 1672,
    PPC_PSQ_STX	= 1673,
    PPC_PSTB	= 1674,
    PPC_PSTB8	= 1675,
    PPC_PSTB8nopc	= 1676,
    PPC_PSTB8onlypc	= 1677,
    PPC_PSTB8pc	= 1678,
    PPC_PSTBnopc	= 1679,
    PPC_PSTBonlypc	= 1680,
    PPC_PSTBpc	= 1681,
    PPC_PSTD	= 1682,
    PPC_PSTDnopc	= 1683,
    PPC_PSTDonlypc	= 1684,
    PPC_PSTDpc	= 1685,
    PPC_PSTFD	= 1686,
    PPC_PSTFDnopc	= 1687,
    PPC_PSTFDonlypc	= 1688,
    PPC_PSTFDpc	= 1689,
    PPC_PSTFS	= 1690,
    PPC_PSTFSnopc	= 1691,
    PPC_PSTFSonlypc	= 1692,
    PPC_PSTFSpc	= 1693,
    PPC_PSTH	= 1694,
    PPC_PSTH8	= 1695,
    PPC_PSTH8nopc	= 1696,
    PPC_PSTH8onlypc	= 1697,
    PPC_PSTH8pc	= 1698,
    PPC_PSTHnopc	= 1699,
    PPC_PSTHonlypc	= 1700,
    PPC_PSTHpc	= 1701,
    PPC_PSTW	= 1702,
    PPC_PSTW8	= 1703,
    PPC_PSTW8nopc	= 1704,
    PPC_PSTW8onlypc	= 1705,
    PPC_PSTW8pc	= 1706,
    PPC_PSTWnopc	= 1707,
    PPC_PSTWonlypc	= 1708,
    PPC_PSTWpc	= 1709,
    PPC_PSTXSD	= 1710,
    PPC_PSTXSDnopc	= 1711,
    PPC_PSTXSDonlypc	= 1712,
    PPC_PSTXSDpc	= 1713,
    PPC_PSTXSSP	= 1714,
    PPC_PSTXSSPnopc	= 1715,
    PPC_PSTXSSPonlypc	= 1716,
    PPC_PSTXSSPpc	= 1717,
    PPC_PSTXV	= 1718,
    PPC_PSTXVP	= 1719,
    PPC_PSTXVPnopc	= 1720,
    PPC_PSTXVPonlypc	= 1721,
    PPC_PSTXVPpc	= 1722,
    PPC_PSTXVnopc	= 1723,
    PPC_PSTXVonlypc	= 1724,
    PPC_PSTXVpc	= 1725,
    PPC_PS_ABS	= 1726,
    PPC_PS_ABSo	= 1727,
    PPC_PS_ADD	= 1728,
    PPC_PS_ADDo	= 1729,
    PPC_PS_CMPO0	= 1730,
    PPC_PS_CMPO1	= 1731,
    PPC_PS_CMPU0	= 1732,
    PPC_PS_CMPU1	= 1733,
    PPC_PS_DIV	= 1734,
    PPC_PS_DIVo	= 1735,
    PPC_PS_MADD	= 1736,
    PPC_PS_MADDS0	= 1737,
    PPC_PS_MADDS0o	= 1738,
    PPC_PS_MADDS1	= 1739,
    PPC_PS_MADDS1o	= 1740,
    PPC_PS_MADDo	= 1741,
    PPC_PS_MERGE00	= 1742,
    PPC_PS_MERGE00o	= 1743,
    PPC_PS_MERGE01	= 1744,
    PPC_PS_MERGE01o	= 1745,
    PPC_PS_MERGE10	= 1746,
    PPC_PS_MERGE10o	= 1747,
    PPC_PS_MERGE11	= 1748,
    PPC_PS_MERGE11o	= 1749,
    PPC_PS_MR	= 1750,
    PPC_PS_MRo	= 1751,
    PPC_PS_MSUB	= 1752,
    PPC_PS_MSUBo	= 1753,
    PPC_PS_MUL	= 1754,
    PPC_PS_MULS0	= 1755,
    PPC_PS_MULS0o	= 1756,
    PPC_PS_MULS1	= 1757,
    PPC_PS_MULS1o	= 1758,
    PPC_PS_MULo	= 1759,
    PPC_PS_NABS	= 1760,
    PPC_PS_NABSo	= 1761,
    PPC_PS_NEG	= 1762,
    PPC_PS_NEGo	= 1763,
    PPC_PS_NMADD	= 1764,
    PPC_PS_NMADDo	= 1765,
    PPC_PS_NMSUB	= 1766,
    PPC_PS_NMSUBo	= 1767,
    PPC_PS_RES	= 1768,
    PPC_PS_RESo	= 1769,
    PPC_PS_RSQRTE	= 1770,
    PPC_PS_RSQRTEo	= 1771,
    PPC_PS_SEL	= 1772,
    PPC_PS_SELo	= 1773,
    PPC_PS_SUB	= 1774,
    PPC_PS_SUBo	= 1775,
    PPC_PS_SUM0	= 1776,
    PPC_PS_SUM0o	= 1777,
    PPC_PS_SUM1	= 1778,
    PPC_PS_SUM1o	= 1779,
    PPC_PseudoEIEIO	= 1780,
    PPC_QVALIGNI	= 1781,
    PPC_QVALIGNIb	= 1782,
    PPC_QVALIGNIs	= 1783,
    PPC_QVESPLATI	= 1784,
    PPC_QVESPLATIb	= 1785,
    PPC_QVESPLATIs	= 1786,
    PPC_QVFABS	= 1787,
    PPC_QVFABSs	= 1788,
    PPC_QVFADD	= 1789,
    PPC_QVFADDS	= 1790,
    PPC_QVFADDSs	= 1791,
    PPC_QVFCFID	= 1792,
    PPC_QVFCFIDS	= 1793,
    PPC_QVFCFIDU	= 1794,
    PPC_QVFCFIDUS	= 1795,
    PPC_QVFCFIDb	= 1796,
    PPC_QVFCMPEQ	= 1797,
    PPC_QVFCMPEQb	= 1798,
    PPC_QVFCMPEQbs	= 1799,
    PPC_QVFCMPGT	= 1800,
    PPC_QVFCMPGTb	= 1801,
    PPC_QVFCMPGTbs	= 1802,
    PPC_QVFCMPLT	= 1803,
    PPC_QVFCMPLTb	= 1804,
    PPC_QVFCMPLTbs	= 1805,
    PPC_QVFCPSGN	= 1806,
    PPC_QVFCPSGNs	= 1807,
    PPC_QVFCTID	= 1808,
    PPC_QVFCTIDU	= 1809,
    PPC_QVFCTIDUZ	= 1810,
    PPC_QVFCTIDZ	= 1811,
    PPC_QVFCTIDb	= 1812,
    PPC_QVFCTIW	= 1813,
    PPC_QVFCTIWU	= 1814,
    PPC_QVFCTIWUZ	= 1815,
    PPC_QVFCTIWZ	= 1816,
    PPC_QVFLOGICAL	= 1817,
    PPC_QVFLOGICALb	= 1818,
    PPC_QVFLOGICALs	= 1819,
    PPC_QVFMADD	= 1820,
    PPC_QVFMADDS	= 1821,
    PPC_QVFMADDSs	= 1822,
    PPC_QVFMR	= 1823,
    PPC_QVFMRb	= 1824,
    PPC_QVFMRs	= 1825,
    PPC_QVFMSUB	= 1826,
    PPC_QVFMSUBS	= 1827,
    PPC_QVFMSUBSs	= 1828,
    PPC_QVFMUL	= 1829,
    PPC_QVFMULS	= 1830,
    PPC_QVFMULSs	= 1831,
    PPC_QVFNABS	= 1832,
    PPC_QVFNABSs	= 1833,
    PPC_QVFNEG	= 1834,
    PPC_QVFNEGs	= 1835,
    PPC_QVFNMADD	= 1836,
    PPC_QVFNMADDS	= 1837,
    PPC_QVFNMADDSs	= 1838,
    PPC_QVFNMSUB	= 1839,
    PPC_QVFNMSUBS	= 1840,
    PPC_QVFNMSUBSs	= 1841,
    PPC_QVFPERM	= 1842,
    PPC_QVFPERMs	= 1843,
    PPC_QVFRE	= 1844,
    PPC_QVFRES	= 1845,
    PPC_QVFRESs	= 1846,
    PPC_QVFRIM	= 1847,
    PPC_QVFRIMs	= 1848,
    PPC_QVFRIN	= 1849,
    PPC_QVFRINs	= 1850,
    PPC_QVFRIP	= 1851,
    PPC_QVFRIPs	= 1852,
    PPC_QVFRIZ	= 1853,
    PPC_QVFRIZs	= 1854,
    PPC_QVFRSP	= 1855,
    PPC_QVFRSPs	= 1856,
    PPC_QVFRSQRTE	= 1857,
    PPC_QVFRSQRTES	= 1858,
    PPC_QVFRSQRTESs	= 1859,
    PPC_QVFSEL	= 1860,
    PPC_QVFSELb	= 1861,
    PPC_QVFSELbb	= 1862,
    PPC_QVFSELbs	= 1863,
    PPC_QVFSUB	= 1864,
    PPC_QVFSUBS	= 1865,
    PPC_QVFSUBSs	= 1866,
    PPC_QVFTSTNAN	= 1867,
    PPC_QVFTSTNANb	= 1868,
    PPC_QVFTSTNANbs	= 1869,
    PPC_QVFXMADD	= 1870,
    PPC_QVFXMADDS	= 1871,
    PPC_QVFXMUL	= 1872,
    PPC_QVFXMULS	= 1873,
    PPC_QVFXXCPNMADD	= 1874,
    PPC_QVFXXCPNMADDS	= 1875,
    PPC_QVFXXMADD	= 1876,
    PPC_QVFXXMADDS	= 1877,
    PPC_QVFXXNPMADD	= 1878,
    PPC_QVFXXNPMADDS	= 1879,
    PPC_QVGPCI	= 1880,
    PPC_QVLFCDUX	= 1881,
    PPC_QVLFCDUXA	= 1882,
    PPC_QVLFCDX	= 1883,
    PPC_QVLFCDXA	= 1884,
    PPC_QVLFCSUX	= 1885,
    PPC_QVLFCSUXA	= 1886,
    PPC_QVLFCSX	= 1887,
    PPC_QVLFCSXA	= 1888,
    PPC_QVLFCSXs	= 1889,
    PPC_QVLFDUX	= 1890,
    PPC_QVLFDUXA	= 1891,
    PPC_QVLFDX	= 1892,
    PPC_QVLFDXA	= 1893,
    PPC_QVLFDXb	= 1894,
    PPC_QVLFIWAX	= 1895,
    PPC_QVLFIWAXA	= 1896,
    PPC_QVLFIWZX	= 1897,
    PPC_QVLFIWZXA	= 1898,
    PPC_QVLFSUX	= 1899,
    PPC_QVLFSUXA	= 1900,
    PPC_QVLFSX	= 1901,
    PPC_QVLFSXA	= 1902,
    PPC_QVLFSXb	= 1903,
    PPC_QVLFSXs	= 1904,
    PPC_QVLPCLDX	= 1905,
    PPC_QVLPCLSX	= 1906,
    PPC_QVLPCLSXint	= 1907,
    PPC_QVLPCRDX	= 1908,
    PPC_QVLPCRSX	= 1909,
    PPC_QVSTFCDUX	= 1910,
    PPC_QVSTFCDUXA	= 1911,
    PPC_QVSTFCDUXI	= 1912,
    PPC_QVSTFCDUXIA	= 1913,
    PPC_QVSTFCDX	= 1914,
    PPC_QVSTFCDXA	= 1915,
    PPC_QVSTFCDXI	= 1916,
    PPC_QVSTFCDXIA	= 1917,
    PPC_QVSTFCSUX	= 1918,
    PPC_QVSTFCSUXA	= 1919,
    PPC_QVSTFCSUXI	= 1920,
    PPC_QVSTFCSUXIA	= 1921,
    PPC_QVSTFCSX	= 1922,
    PPC_QVSTFCSXA	= 1923,
    PPC_QVSTFCSXI	= 1924,
    PPC_QVSTFCSXIA	= 1925,
    PPC_QVSTFCSXs	= 1926,
    PPC_QVSTFDUX	= 1927,
    PPC_QVSTFDUXA	= 1928,
    PPC_QVSTFDUXI	= 1929,
    PPC_QVSTFDUXIA	= 1930,
    PPC_QVSTFDX	= 1931,
    PPC_QVSTFDXA	= 1932,
    PPC_QVSTFDXI	= 1933,
    PPC_QVSTFDXIA	= 1934,
    PPC_QVSTFDXb	= 1935,
    PPC_QVSTFIWX	= 1936,
    PPC_QVSTFIWXA	= 1937,
    PPC_QVSTFSUX	= 1938,
    PPC_QVSTFSUXA	= 1939,
    PPC_QVSTFSUXI	= 1940,
    PPC_QVSTFSUXIA	= 1941,
    PPC_QVSTFSUXs	= 1942,
    PPC_QVSTFSX	= 1943,
    PPC_QVSTFSXA	= 1944,
    PPC_QVSTFSXI	= 1945,
    PPC_QVSTFSXIA	= 1946,
    PPC_QVSTFSXs	= 1947,
    PPC_RESTORE_ACC	= 1948,
    PPC_RESTORE_CR	= 1949,
    PPC_RESTORE_CRBIT	= 1950,
    PPC_RESTORE_QUADWORD	= 1951,
    PPC_RESTORE_UACC	= 1952,
    PPC_RESTORE_WACC	= 1953,
    PPC_RFCI	= 1954,
    PPC_RFDI	= 1955,
    PPC_RFEBB	= 1956,
    PPC_RFI	= 1957,
    PPC_RFID	= 1958,
    PPC_RFMCI	= 1959,
    PPC_RLDCL	= 1960,
    PPC_RLDCL_rec	= 1961,
    PPC_RLDCR	= 1962,
    PPC_RLDCR_rec	= 1963,
    PPC_RLDIC	= 1964,
    PPC_RLDICL	= 1965,
    PPC_RLDICL_32	= 1966,
    PPC_RLDICL_32_64	= 1967,
    PPC_RLDICL_32_rec	= 1968,
    PPC_RLDICL_rec	= 1969,
    PPC_RLDICR	= 1970,
    PPC_RLDICR_32	= 1971,
    PPC_RLDICR_rec	= 1972,
    PPC_RLDIC_rec	= 1973,
    PPC_RLDIMI	= 1974,
    PPC_RLDIMI_rec	= 1975,
    PPC_RLWIMI	= 1976,
    PPC_RLWIMI8	= 1977,
    PPC_RLWIMI8_rec	= 1978,
    PPC_RLWIMI_rec	= 1979,
    PPC_RLWINM	= 1980,
    PPC_RLWINM8	= 1981,
    PPC_RLWINM8_rec	= 1982,
    PPC_RLWINM_rec	= 1983,
    PPC_RLWNM	= 1984,
    PPC_RLWNM8	= 1985,
    PPC_RLWNM8_rec	= 1986,
    PPC_RLWNM_rec	= 1987,
    PPC_ReadTB	= 1988,
    PPC_SC	= 1989,
    PPC_SCV	= 1990,
    PPC_SELECT_CC_F16	= 1991,
    PPC_SELECT_CC_F4	= 1992,
    PPC_SELECT_CC_F8	= 1993,
    PPC_SELECT_CC_I4	= 1994,
    PPC_SELECT_CC_I8	= 1995,
    PPC_SELECT_CC_QBRC	= 1996,
    PPC_SELECT_CC_QFRC	= 1997,
    PPC_SELECT_CC_QSRC	= 1998,
    PPC_SELECT_CC_SPE	= 1999,
    PPC_SELECT_CC_SPE4	= 2000,
    PPC_SELECT_CC_VRRC	= 2001,
    PPC_SELECT_CC_VSFRC	= 2002,
    PPC_SELECT_CC_VSRC	= 2003,
    PPC_SELECT_CC_VSSRC	= 2004,
    PPC_SELECT_F16	= 2005,
    PPC_SELECT_F4	= 2006,
    PPC_SELECT_F8	= 2007,
    PPC_SELECT_I4	= 2008,
    PPC_SELECT_I8	= 2009,
    PPC_SELECT_QBRC	= 2010,
    PPC_SELECT_QFRC	= 2011,
    PPC_SELECT_QSRC	= 2012,
    PPC_SELECT_SPE	= 2013,
    PPC_SELECT_SPE4	= 2014,
    PPC_SELECT_VRRC	= 2015,
    PPC_SELECT_VSFRC	= 2016,
    PPC_SELECT_VSRC	= 2017,
    PPC_SELECT_VSSRC	= 2018,
    PPC_SETB	= 2019,
    PPC_SETB8	= 2020,
    PPC_SETBC	= 2021,
    PPC_SETBC8	= 2022,
    PPC_SETBCR	= 2023,
    PPC_SETBCR8	= 2024,
    PPC_SETFLM	= 2025,
    PPC_SETNBC	= 2026,
    PPC_SETNBC8	= 2027,
    PPC_SETNBCR	= 2028,
    PPC_SETNBCR8	= 2029,
    PPC_SETRND	= 2030,
    PPC_SETRNDi	= 2031,
    PPC_SLBFEE_rec	= 2032,
    PPC_SLBIA	= 2033,
    PPC_SLBIE	= 2034,
    PPC_SLBIEG	= 2035,
    PPC_SLBMFEE	= 2036,
    PPC_SLBMFEV	= 2037,
    PPC_SLBMTE	= 2038,
    PPC_SLBSYNC	= 2039,
    PPC_SLD	= 2040,
    PPC_SLD_rec	= 2041,
    PPC_SLW	= 2042,
    PPC_SLW8	= 2043,
    PPC_SLW8_rec	= 2044,
    PPC_SLW_rec	= 2045,
    PPC_SPELWZ	= 2046,
    PPC_SPELWZX	= 2047,
    PPC_SPESTW	= 2048,
    PPC_SPESTWX	= 2049,
    PPC_SPILL_ACC	= 2050,
    PPC_SPILL_CR	= 2051,
    PPC_SPILL_CRBIT	= 2052,
    PPC_SPILL_QUADWORD	= 2053,
    PPC_SPILL_UACC	= 2054,
    PPC_SPILL_WACC	= 2055,
    PPC_SPLIT_QUADWORD	= 2056,
    PPC_SRAD	= 2057,
    PPC_SRADI	= 2058,
    PPC_SRADI_32	= 2059,
    PPC_SRADI_rec	= 2060,
    PPC_SRAD_rec	= 2061,
    PPC_SRAW	= 2062,
    PPC_SRAWI	= 2063,
    PPC_SRAWI_rec	= 2064,
    PPC_SRAW_rec	= 2065,
    PPC_SRD	= 2066,
    PPC_SRD_rec	= 2067,
    PPC_SRW	= 2068,
    PPC_SRW8	= 2069,
    PPC_SRW8_rec	= 2070,
    PPC_SRW_rec	= 2071,
    PPC_STB	= 2072,
    PPC_STB8	= 2073,
    PPC_STBCIX	= 2074,
    PPC_STBCX	= 2075,
    PPC_STBEPX	= 2076,
    PPC_STBU	= 2077,
    PPC_STBU8	= 2078,
    PPC_STBUX	= 2079,
    PPC_STBUX8	= 2080,
    PPC_STBX	= 2081,
    PPC_STBX8	= 2082,
    PPC_STBXTLS	= 2083,
    PPC_STBXTLS_	= 2084,
    PPC_STBXTLS_32	= 2085,
    PPC_STD	= 2086,
    PPC_STDAT	= 2087,
    PPC_STDBRX	= 2088,
    PPC_STDCIX	= 2089,
    PPC_STDCX	= 2090,
    PPC_STDU	= 2091,
    PPC_STDUX	= 2092,
    PPC_STDX	= 2093,
    PPC_STDXTLS	= 2094,
    PPC_STDXTLS_	= 2095,
    PPC_STFD	= 2096,
    PPC_STFDEPX	= 2097,
    PPC_STFDU	= 2098,
    PPC_STFDUX	= 2099,
    PPC_STFDX	= 2100,
    PPC_STFDXTLS	= 2101,
    PPC_STFDXTLS_	= 2102,
    PPC_STFIWX	= 2103,
    PPC_STFS	= 2104,
    PPC_STFSU	= 2105,
    PPC_STFSUX	= 2106,
    PPC_STFSX	= 2107,
    PPC_STFSXTLS	= 2108,
    PPC_STFSXTLS_	= 2109,
    PPC_STH	= 2110,
    PPC_STH8	= 2111,
    PPC_STHBRX	= 2112,
    PPC_STHCIX	= 2113,
    PPC_STHCX	= 2114,
    PPC_STHEPX	= 2115,
    PPC_STHU	= 2116,
    PPC_STHU8	= 2117,
    PPC_STHUX	= 2118,
    PPC_STHUX8	= 2119,
    PPC_STHX	= 2120,
    PPC_STHX8	= 2121,
    PPC_STHXTLS	= 2122,
    PPC_STHXTLS_	= 2123,
    PPC_STHXTLS_32	= 2124,
    PPC_STMW	= 2125,
    PPC_STOP	= 2126,
    PPC_STQ	= 2127,
    PPC_STQCX	= 2128,
    PPC_STQX_PSEUDO	= 2129,
    PPC_STSWI	= 2130,
    PPC_STVEBX	= 2131,
    PPC_STVEHX	= 2132,
    PPC_STVEWX	= 2133,
    PPC_STVX	= 2134,
    PPC_STVXL	= 2135,
    PPC_STW	= 2136,
    PPC_STW8	= 2137,
    PPC_STWAT	= 2138,
    PPC_STWBRX	= 2139,
    PPC_STWCIX	= 2140,
    PPC_STWCX	= 2141,
    PPC_STWEPX	= 2142,
    PPC_STWU	= 2143,
    PPC_STWU8	= 2144,
    PPC_STWUX	= 2145,
    PPC_STWUX8	= 2146,
    PPC_STWX	= 2147,
    PPC_STWX8	= 2148,
    PPC_STWXTLS	= 2149,
    PPC_STWXTLS_	= 2150,
    PPC_STWXTLS_32	= 2151,
    PPC_STXSD	= 2152,
    PPC_STXSDX	= 2153,
    PPC_STXSIBX	= 2154,
    PPC_STXSIBXv	= 2155,
    PPC_STXSIHX	= 2156,
    PPC_STXSIHXv	= 2157,
    PPC_STXSIWX	= 2158,
    PPC_STXSSP	= 2159,
    PPC_STXSSPX	= 2160,
    PPC_STXV	= 2161,
    PPC_STXVB16X	= 2162,
    PPC_STXVD2X	= 2163,
    PPC_STXVH8X	= 2164,
    PPC_STXVL	= 2165,
    PPC_STXVLL	= 2166,
    PPC_STXVP	= 2167,
    PPC_STXVPRL	= 2168,
    PPC_STXVPRLL	= 2169,
    PPC_STXVPX	= 2170,
    PPC_STXVRBX	= 2171,
    PPC_STXVRDX	= 2172,
    PPC_STXVRHX	= 2173,
    PPC_STXVRL	= 2174,
    PPC_STXVRLL	= 2175,
    PPC_STXVRWX	= 2176,
    PPC_STXVW4X	= 2177,
    PPC_STXVX	= 2178,
    PPC_SUBF	= 2179,
    PPC_SUBF8	= 2180,
    PPC_SUBF8O	= 2181,
    PPC_SUBF8O_rec	= 2182,
    PPC_SUBF8_rec	= 2183,
    PPC_SUBFC	= 2184,
    PPC_SUBFC8	= 2185,
    PPC_SUBFC8O	= 2186,
    PPC_SUBFC8O_rec	= 2187,
    PPC_SUBFC8_rec	= 2188,
    PPC_SUBFCO	= 2189,
    PPC_SUBFCO_rec	= 2190,
    PPC_SUBFC_rec	= 2191,
    PPC_SUBFE	= 2192,
    PPC_SUBFE8	= 2193,
    PPC_SUBFE8O	= 2194,
    PPC_SUBFE8O_rec	= 2195,
    PPC_SUBFE8_rec	= 2196,
    PPC_SUBFEO	= 2197,
    PPC_SUBFEO_rec	= 2198,
    PPC_SUBFE_rec	= 2199,
    PPC_SUBFIC	= 2200,
    PPC_SUBFIC8	= 2201,
    PPC_SUBFME	= 2202,
    PPC_SUBFME8	= 2203,
    PPC_SUBFME8O	= 2204,
    PPC_SUBFME8O_rec	= 2205,
    PPC_SUBFME8_rec	= 2206,
    PPC_SUBFMEO	= 2207,
    PPC_SUBFMEO_rec	= 2208,
    PPC_SUBFME_rec	= 2209,
    PPC_SUBFO	= 2210,
    PPC_SUBFO_rec	= 2211,
    PPC_SUBFUS	= 2212,
    PPC_SUBFUS_rec	= 2213,
    PPC_SUBFZE	= 2214,
    PPC_SUBFZE8	= 2215,
    PPC_SUBFZE8O	= 2216,
    PPC_SUBFZE8O_rec	= 2217,
    PPC_SUBFZE8_rec	= 2218,
    PPC_SUBFZEO	= 2219,
    PPC_SUBFZEO_rec	= 2220,
    PPC_SUBFZE_rec	= 2221,
    PPC_SUBF_rec	= 2222,
    PPC_SYNC	= 2223,
    PPC_SYNCP10	= 2224,
    PPC_TABORT	= 2225,
    PPC_TABORTDC	= 2226,
    PPC_TABORTDCI	= 2227,
    PPC_TABORTWC	= 2228,
    PPC_TABORTWCI	= 2229,
    PPC_TAILB	= 2230,
    PPC_TAILB8	= 2231,
    PPC_TAILBA	= 2232,
    PPC_TAILBA8	= 2233,
    PPC_TAILBCTR	= 2234,
    PPC_TAILBCTR8	= 2235,
    PPC_TBEGIN	= 2236,
    PPC_TBEGIN_RET	= 2237,
    PPC_TCHECK	= 2238,
    PPC_TCHECK_RET	= 2239,
    PPC_TCRETURNai	= 2240,
    PPC_TCRETURNai8	= 2241,
    PPC_TCRETURNdi	= 2242,
    PPC_TCRETURNdi8	= 2243,
    PPC_TCRETURNri	= 2244,
    PPC_TCRETURNri8	= 2245,
    PPC_TD	= 2246,
    PPC_TDI	= 2247,
    PPC_TEND	= 2248,
    PPC_TLBIA	= 2249,
    PPC_TLBIE	= 2250,
    PPC_TLBIEL	= 2251,
    PPC_TLBILX	= 2252,
    PPC_TLBIVAX	= 2253,
    PPC_TLBLD	= 2254,
    PPC_TLBLI	= 2255,
    PPC_TLBRE	= 2256,
    PPC_TLBRE2	= 2257,
    PPC_TLBSX	= 2258,
    PPC_TLBSX2	= 2259,
    PPC_TLBSX2D	= 2260,
    PPC_TLBSYNC	= 2261,
    PPC_TLBWE	= 2262,
    PPC_TLBWE2	= 2263,
    PPC_TLSGDAIX	= 2264,
    PPC_TLSGDAIX8	= 2265,
    PPC_TRAP	= 2266,
    PPC_TRECHKPT	= 2267,
    PPC_TRECLAIM	= 2268,
    PPC_TSR	= 2269,
    PPC_TW	= 2270,
    PPC_TWI	= 2271,
    PPC_UNENCODED_NOP	= 2272,
    PPC_UpdateGBR	= 2273,
    PPC_VABSDUB	= 2274,
    PPC_VABSDUH	= 2275,
    PPC_VABSDUW	= 2276,
    PPC_VADDCUQ	= 2277,
    PPC_VADDCUW	= 2278,
    PPC_VADDECUQ	= 2279,
    PPC_VADDEUQM	= 2280,
    PPC_VADDFP	= 2281,
    PPC_VADDSBS	= 2282,
    PPC_VADDSHS	= 2283,
    PPC_VADDSWS	= 2284,
    PPC_VADDUBM	= 2285,
    PPC_VADDUBS	= 2286,
    PPC_VADDUDM	= 2287,
    PPC_VADDUHM	= 2288,
    PPC_VADDUHS	= 2289,
    PPC_VADDUQM	= 2290,
    PPC_VADDUWM	= 2291,
    PPC_VADDUWS	= 2292,
    PPC_VAND	= 2293,
    PPC_VANDC	= 2294,
    PPC_VAVGSB	= 2295,
    PPC_VAVGSH	= 2296,
    PPC_VAVGSW	= 2297,
    PPC_VAVGUB	= 2298,
    PPC_VAVGUH	= 2299,
    PPC_VAVGUW	= 2300,
    PPC_VBPERMD	= 2301,
    PPC_VBPERMQ	= 2302,
    PPC_VCFSX	= 2303,
    PPC_VCFSX_0	= 2304,
    PPC_VCFUGED	= 2305,
    PPC_VCFUX	= 2306,
    PPC_VCFUX_0	= 2307,
    PPC_VCIPHER	= 2308,
    PPC_VCIPHERLAST	= 2309,
    PPC_VCLRLB	= 2310,
    PPC_VCLRRB	= 2311,
    PPC_VCLZB	= 2312,
    PPC_VCLZD	= 2313,
    PPC_VCLZDM	= 2314,
    PPC_VCLZH	= 2315,
    PPC_VCLZLSBB	= 2316,
    PPC_VCLZW	= 2317,
    PPC_VCMPBFP	= 2318,
    PPC_VCMPBFP_rec	= 2319,
    PPC_VCMPEQFP	= 2320,
    PPC_VCMPEQFP_rec	= 2321,
    PPC_VCMPEQUB	= 2322,
    PPC_VCMPEQUB_rec	= 2323,
    PPC_VCMPEQUD	= 2324,
    PPC_VCMPEQUD_rec	= 2325,
    PPC_VCMPEQUH	= 2326,
    PPC_VCMPEQUH_rec	= 2327,
    PPC_VCMPEQUQ	= 2328,
    PPC_VCMPEQUQ_rec	= 2329,
    PPC_VCMPEQUW	= 2330,
    PPC_VCMPEQUW_rec	= 2331,
    PPC_VCMPGEFP	= 2332,
    PPC_VCMPGEFP_rec	= 2333,
    PPC_VCMPGTFP	= 2334,
    PPC_VCMPGTFP_rec	= 2335,
    PPC_VCMPGTSB	= 2336,
    PPC_VCMPGTSB_rec	= 2337,
    PPC_VCMPGTSD	= 2338,
    PPC_VCMPGTSD_rec	= 2339,
    PPC_VCMPGTSH	= 2340,
    PPC_VCMPGTSH_rec	= 2341,
    PPC_VCMPGTSQ	= 2342,
    PPC_VCMPGTSQ_rec	= 2343,
    PPC_VCMPGTSW	= 2344,
    PPC_VCMPGTSW_rec	= 2345,
    PPC_VCMPGTUB	= 2346,
    PPC_VCMPGTUB_rec	= 2347,
    PPC_VCMPGTUD	= 2348,
    PPC_VCMPGTUD_rec	= 2349,
    PPC_VCMPGTUH	= 2350,
    PPC_VCMPGTUH_rec	= 2351,
    PPC_VCMPGTUQ	= 2352,
    PPC_VCMPGTUQ_rec	= 2353,
    PPC_VCMPGTUW	= 2354,
    PPC_VCMPGTUW_rec	= 2355,
    PPC_VCMPNEB	= 2356,
    PPC_VCMPNEB_rec	= 2357,
    PPC_VCMPNEH	= 2358,
    PPC_VCMPNEH_rec	= 2359,
    PPC_VCMPNEW	= 2360,
    PPC_VCMPNEW_rec	= 2361,
    PPC_VCMPNEZB	= 2362,
    PPC_VCMPNEZB_rec	= 2363,
    PPC_VCMPNEZH	= 2364,
    PPC_VCMPNEZH_rec	= 2365,
    PPC_VCMPNEZW	= 2366,
    PPC_VCMPNEZW_rec	= 2367,
    PPC_VCMPSQ	= 2368,
    PPC_VCMPUQ	= 2369,
    PPC_VCNTMBB	= 2370,
    PPC_VCNTMBD	= 2371,
    PPC_VCNTMBH	= 2372,
    PPC_VCNTMBW	= 2373,
    PPC_VCTSXS	= 2374,
    PPC_VCTSXS_0	= 2375,
    PPC_VCTUXS	= 2376,
    PPC_VCTUXS_0	= 2377,
    PPC_VCTZB	= 2378,
    PPC_VCTZD	= 2379,
    PPC_VCTZDM	= 2380,
    PPC_VCTZH	= 2381,
    PPC_VCTZLSBB	= 2382,
    PPC_VCTZW	= 2383,
    PPC_VDIVESD	= 2384,
    PPC_VDIVESQ	= 2385,
    PPC_VDIVESW	= 2386,
    PPC_VDIVEUD	= 2387,
    PPC_VDIVEUQ	= 2388,
    PPC_VDIVEUW	= 2389,
    PPC_VDIVSD	= 2390,
    PPC_VDIVSQ	= 2391,
    PPC_VDIVSW	= 2392,
    PPC_VDIVUD	= 2393,
    PPC_VDIVUQ	= 2394,
    PPC_VDIVUW	= 2395,
    PPC_VEQV	= 2396,
    PPC_VEXPANDBM	= 2397,
    PPC_VEXPANDDM	= 2398,
    PPC_VEXPANDHM	= 2399,
    PPC_VEXPANDQM	= 2400,
    PPC_VEXPANDWM	= 2401,
    PPC_VEXPTEFP	= 2402,
    PPC_VEXTDDVLX	= 2403,
    PPC_VEXTDDVRX	= 2404,
    PPC_VEXTDUBVLX	= 2405,
    PPC_VEXTDUBVRX	= 2406,
    PPC_VEXTDUHVLX	= 2407,
    PPC_VEXTDUHVRX	= 2408,
    PPC_VEXTDUWVLX	= 2409,
    PPC_VEXTDUWVRX	= 2410,
    PPC_VEXTRACTBM	= 2411,
    PPC_VEXTRACTD	= 2412,
    PPC_VEXTRACTDM	= 2413,
    PPC_VEXTRACTHM	= 2414,
    PPC_VEXTRACTQM	= 2415,
    PPC_VEXTRACTUB	= 2416,
    PPC_VEXTRACTUH	= 2417,
    PPC_VEXTRACTUW	= 2418,
    PPC_VEXTRACTWM	= 2419,
    PPC_VEXTSB2D	= 2420,
    PPC_VEXTSB2Ds	= 2421,
    PPC_VEXTSB2W	= 2422,
    PPC_VEXTSB2Ws	= 2423,
    PPC_VEXTSD2Q	= 2424,
    PPC_VEXTSH2D	= 2425,
    PPC_VEXTSH2Ds	= 2426,
    PPC_VEXTSH2W	= 2427,
    PPC_VEXTSH2Ws	= 2428,
    PPC_VEXTSW2D	= 2429,
    PPC_VEXTSW2Ds	= 2430,
    PPC_VEXTUBLX	= 2431,
    PPC_VEXTUBRX	= 2432,
    PPC_VEXTUHLX	= 2433,
    PPC_VEXTUHRX	= 2434,
    PPC_VEXTUWLX	= 2435,
    PPC_VEXTUWRX	= 2436,
    PPC_VGBBD	= 2437,
    PPC_VGNB	= 2438,
    PPC_VINSBLX	= 2439,
    PPC_VINSBRX	= 2440,
    PPC_VINSBVLX	= 2441,
    PPC_VINSBVRX	= 2442,
    PPC_VINSD	= 2443,
    PPC_VINSDLX	= 2444,
    PPC_VINSDRX	= 2445,
    PPC_VINSERTB	= 2446,
    PPC_VINSERTD	= 2447,
    PPC_VINSERTH	= 2448,
    PPC_VINSERTW	= 2449,
    PPC_VINSHLX	= 2450,
    PPC_VINSHRX	= 2451,
    PPC_VINSHVLX	= 2452,
    PPC_VINSHVRX	= 2453,
    PPC_VINSW	= 2454,
    PPC_VINSWLX	= 2455,
    PPC_VINSWRX	= 2456,
    PPC_VINSWVLX	= 2457,
    PPC_VINSWVRX	= 2458,
    PPC_VLOGEFP	= 2459,
    PPC_VMADDFP	= 2460,
    PPC_VMAXFP	= 2461,
    PPC_VMAXSB	= 2462,
    PPC_VMAXSD	= 2463,
    PPC_VMAXSH	= 2464,
    PPC_VMAXSW	= 2465,
    PPC_VMAXUB	= 2466,
    PPC_VMAXUD	= 2467,
    PPC_VMAXUH	= 2468,
    PPC_VMAXUW	= 2469,
    PPC_VMHADDSHS	= 2470,
    PPC_VMHRADDSHS	= 2471,
    PPC_VMINFP	= 2472,
    PPC_VMINSB	= 2473,
    PPC_VMINSD	= 2474,
    PPC_VMINSH	= 2475,
    PPC_VMINSW	= 2476,
    PPC_VMINUB	= 2477,
    PPC_VMINUD	= 2478,
    PPC_VMINUH	= 2479,
    PPC_VMINUW	= 2480,
    PPC_VMLADDUHM	= 2481,
    PPC_VMODSD	= 2482,
    PPC_VMODSQ	= 2483,
    PPC_VMODSW	= 2484,
    PPC_VMODUD	= 2485,
    PPC_VMODUQ	= 2486,
    PPC_VMODUW	= 2487,
    PPC_VMRGEW	= 2488,
    PPC_VMRGHB	= 2489,
    PPC_VMRGHH	= 2490,
    PPC_VMRGHW	= 2491,
    PPC_VMRGLB	= 2492,
    PPC_VMRGLH	= 2493,
    PPC_VMRGLW	= 2494,
    PPC_VMRGOW	= 2495,
    PPC_VMSUMCUD	= 2496,
    PPC_VMSUMMBM	= 2497,
    PPC_VMSUMSHM	= 2498,
    PPC_VMSUMSHS	= 2499,
    PPC_VMSUMUBM	= 2500,
    PPC_VMSUMUDM	= 2501,
    PPC_VMSUMUHM	= 2502,
    PPC_VMSUMUHS	= 2503,
    PPC_VMUL10CUQ	= 2504,
    PPC_VMUL10ECUQ	= 2505,
    PPC_VMUL10EUQ	= 2506,
    PPC_VMUL10UQ	= 2507,
    PPC_VMULESB	= 2508,
    PPC_VMULESD	= 2509,
    PPC_VMULESH	= 2510,
    PPC_VMULESW	= 2511,
    PPC_VMULEUB	= 2512,
    PPC_VMULEUD	= 2513,
    PPC_VMULEUH	= 2514,
    PPC_VMULEUW	= 2515,
    PPC_VMULHSD	= 2516,
    PPC_VMULHSW	= 2517,
    PPC_VMULHUD	= 2518,
    PPC_VMULHUW	= 2519,
    PPC_VMULLD	= 2520,
    PPC_VMULOSB	= 2521,
    PPC_VMULOSD	= 2522,
    PPC_VMULOSH	= 2523,
    PPC_VMULOSW	= 2524,
    PPC_VMULOUB	= 2525,
    PPC_VMULOUD	= 2526,
    PPC_VMULOUH	= 2527,
    PPC_VMULOUW	= 2528,
    PPC_VMULUWM	= 2529,
    PPC_VNAND	= 2530,
    PPC_VNCIPHER	= 2531,
    PPC_VNCIPHERLAST	= 2532,
    PPC_VNEGD	= 2533,
    PPC_VNEGW	= 2534,
    PPC_VNMSUBFP	= 2535,
    PPC_VNOR	= 2536,
    PPC_VOR	= 2537,
    PPC_VORC	= 2538,
    PPC_VPDEPD	= 2539,
    PPC_VPERM	= 2540,
    PPC_VPERMR	= 2541,
    PPC_VPERMXOR	= 2542,
    PPC_VPEXTD	= 2543,
    PPC_VPKPX	= 2544,
    PPC_VPKSDSS	= 2545,
    PPC_VPKSDUS	= 2546,
    PPC_VPKSHSS	= 2547,
    PPC_VPKSHUS	= 2548,
    PPC_VPKSWSS	= 2549,
    PPC_VPKSWUS	= 2550,
    PPC_VPKUDUM	= 2551,
    PPC_VPKUDUS	= 2552,
    PPC_VPKUHUM	= 2553,
    PPC_VPKUHUS	= 2554,
    PPC_VPKUWUM	= 2555,
    PPC_VPKUWUS	= 2556,
    PPC_VPMSUMB	= 2557,
    PPC_VPMSUMD	= 2558,
    PPC_VPMSUMH	= 2559,
    PPC_VPMSUMW	= 2560,
    PPC_VPOPCNTB	= 2561,
    PPC_VPOPCNTD	= 2562,
    PPC_VPOPCNTH	= 2563,
    PPC_VPOPCNTW	= 2564,
    PPC_VPRTYBD	= 2565,
    PPC_VPRTYBQ	= 2566,
    PPC_VPRTYBW	= 2567,
    PPC_VREFP	= 2568,
    PPC_VRFIM	= 2569,
    PPC_VRFIN	= 2570,
    PPC_VRFIP	= 2571,
    PPC_VRFIZ	= 2572,
    PPC_VRLB	= 2573,
    PPC_VRLD	= 2574,
    PPC_VRLDMI	= 2575,
    PPC_VRLDNM	= 2576,
    PPC_VRLH	= 2577,
    PPC_VRLQ	= 2578,
    PPC_VRLQMI	= 2579,
    PPC_VRLQNM	= 2580,
    PPC_VRLW	= 2581,
    PPC_VRLWMI	= 2582,
    PPC_VRLWNM	= 2583,
    PPC_VRSQRTEFP	= 2584,
    PPC_VSBOX	= 2585,
    PPC_VSEL	= 2586,
    PPC_VSHASIGMAD	= 2587,
    PPC_VSHASIGMAW	= 2588,
    PPC_VSL	= 2589,
    PPC_VSLB	= 2590,
    PPC_VSLD	= 2591,
    PPC_VSLDBI	= 2592,
    PPC_VSLDOI	= 2593,
    PPC_VSLH	= 2594,
    PPC_VSLO	= 2595,
    PPC_VSLQ	= 2596,
    PPC_VSLV	= 2597,
    PPC_VSLW	= 2598,
    PPC_VSPLTB	= 2599,
    PPC_VSPLTBs	= 2600,
    PPC_VSPLTH	= 2601,
    PPC_VSPLTHs	= 2602,
    PPC_VSPLTISB	= 2603,
    PPC_VSPLTISH	= 2604,
    PPC_VSPLTISW	= 2605,
    PPC_VSPLTW	= 2606,
    PPC_VSR	= 2607,
    PPC_VSRAB	= 2608,
    PPC_VSRAD	= 2609,
    PPC_VSRAH	= 2610,
    PPC_VSRAQ	= 2611,
    PPC_VSRAW	= 2612,
    PPC_VSRB	= 2613,
    PPC_VSRD	= 2614,
    PPC_VSRDBI	= 2615,
    PPC_VSRH	= 2616,
    PPC_VSRO	= 2617,
    PPC_VSRQ	= 2618,
    PPC_VSRV	= 2619,
    PPC_VSRW	= 2620,
    PPC_VSTRIBL	= 2621,
    PPC_VSTRIBL_rec	= 2622,
    PPC_VSTRIBR	= 2623,
    PPC_VSTRIBR_rec	= 2624,
    PPC_VSTRIHL	= 2625,
    PPC_VSTRIHL_rec	= 2626,
    PPC_VSTRIHR	= 2627,
    PPC_VSTRIHR_rec	= 2628,
    PPC_VSUBCUQ	= 2629,
    PPC_VSUBCUW	= 2630,
    PPC_VSUBECUQ	= 2631,
    PPC_VSUBEUQM	= 2632,
    PPC_VSUBFP	= 2633,
    PPC_VSUBSBS	= 2634,
    PPC_VSUBSHS	= 2635,
    PPC_VSUBSWS	= 2636,
    PPC_VSUBUBM	= 2637,
    PPC_VSUBUBS	= 2638,
    PPC_VSUBUDM	= 2639,
    PPC_VSUBUHM	= 2640,
    PPC_VSUBUHS	= 2641,
    PPC_VSUBUQM	= 2642,
    PPC_VSUBUWM	= 2643,
    PPC_VSUBUWS	= 2644,
    PPC_VSUM2SWS	= 2645,
    PPC_VSUM4SBS	= 2646,
    PPC_VSUM4SHS	= 2647,
    PPC_VSUM4UBS	= 2648,
    PPC_VSUMSWS	= 2649,
    PPC_VUPKHPX	= 2650,
    PPC_VUPKHSB	= 2651,
    PPC_VUPKHSH	= 2652,
    PPC_VUPKHSW	= 2653,
    PPC_VUPKLPX	= 2654,
    PPC_VUPKLSB	= 2655,
    PPC_VUPKLSH	= 2656,
    PPC_VUPKLSW	= 2657,
    PPC_VXOR	= 2658,
    PPC_V_SET0	= 2659,
    PPC_V_SET0B	= 2660,
    PPC_V_SET0H	= 2661,
    PPC_V_SETALLONES	= 2662,
    PPC_V_SETALLONESB	= 2663,
    PPC_V_SETALLONESH	= 2664,
    PPC_WAIT	= 2665,
    PPC_WAITP10	= 2666,
    PPC_WRTEE	= 2667,
    PPC_WRTEEI	= 2668,
    PPC_XOR	= 2669,
    PPC_XOR8	= 2670,
    PPC_XOR8_rec	= 2671,
    PPC_XORI	= 2672,
    PPC_XORI8	= 2673,
    PPC_XORIS	= 2674,
    PPC_XORIS8	= 2675,
    PPC_XOR_rec	= 2676,
    PPC_XSABSDP	= 2677,
    PPC_XSABSQP	= 2678,
    PPC_XSADDDP	= 2679,
    PPC_XSADDQP	= 2680,
    PPC_XSADDQPO	= 2681,
    PPC_XSADDSP	= 2682,
    PPC_XSCMPEQDP	= 2683,
    PPC_XSCMPEQQP	= 2684,
    PPC_XSCMPEXPDP	= 2685,
    PPC_XSCMPEXPQP	= 2686,
    PPC_XSCMPGEDP	= 2687,
    PPC_XSCMPGEQP	= 2688,
    PPC_XSCMPGTDP	= 2689,
    PPC_XSCMPGTQP	= 2690,
    PPC_XSCMPODP	= 2691,
    PPC_XSCMPOQP	= 2692,
    PPC_XSCMPUDP	= 2693,
    PPC_XSCMPUQP	= 2694,
    PPC_XSCPSGNDP	= 2695,
    PPC_XSCPSGNQP	= 2696,
    PPC_XSCVDPHP	= 2697,
    PPC_XSCVDPQP	= 2698,
    PPC_XSCVDPSP	= 2699,
    PPC_XSCVDPSPN	= 2700,
    PPC_XSCVDPSXDS	= 2701,
    PPC_XSCVDPSXDSs	= 2702,
    PPC_XSCVDPSXWS	= 2703,
    PPC_XSCVDPSXWSs	= 2704,
    PPC_XSCVDPUXDS	= 2705,
    PPC_XSCVDPUXDSs	= 2706,
    PPC_XSCVDPUXWS	= 2707,
    PPC_XSCVDPUXWSs	= 2708,
    PPC_XSCVHPDP	= 2709,
    PPC_XSCVQPDP	= 2710,
    PPC_XSCVQPDPO	= 2711,
    PPC_XSCVQPSDZ	= 2712,
    PPC_XSCVQPSQZ	= 2713,
    PPC_XSCVQPSWZ	= 2714,
    PPC_XSCVQPUDZ	= 2715,
    PPC_XSCVQPUQZ	= 2716,
    PPC_XSCVQPUWZ	= 2717,
    PPC_XSCVSDQP	= 2718,
    PPC_XSCVSPDP	= 2719,
    PPC_XSCVSPDPN	= 2720,
    PPC_XSCVSQQP	= 2721,
    PPC_XSCVSXDDP	= 2722,
    PPC_XSCVSXDSP	= 2723,
    PPC_XSCVUDQP	= 2724,
    PPC_XSCVUQQP	= 2725,
    PPC_XSCVUXDDP	= 2726,
    PPC_XSCVUXDSP	= 2727,
    PPC_XSDIVDP	= 2728,
    PPC_XSDIVQP	= 2729,
    PPC_XSDIVQPO	= 2730,
    PPC_XSDIVSP	= 2731,
    PPC_XSIEXPDP	= 2732,
    PPC_XSIEXPQP	= 2733,
    PPC_XSMADDADP	= 2734,
    PPC_XSMADDASP	= 2735,
    PPC_XSMADDMDP	= 2736,
    PPC_XSMADDMSP	= 2737,
    PPC_XSMADDQP	= 2738,
    PPC_XSMADDQPO	= 2739,
    PPC_XSMAXCDP	= 2740,
    PPC_XSMAXCQP	= 2741,
    PPC_XSMAXDP	= 2742,
    PPC_XSMAXJDP	= 2743,
    PPC_XSMINCDP	= 2744,
    PPC_XSMINCQP	= 2745,
    PPC_XSMINDP	= 2746,
    PPC_XSMINJDP	= 2747,
    PPC_XSMSUBADP	= 2748,
    PPC_XSMSUBASP	= 2749,
    PPC_XSMSUBMDP	= 2750,
    PPC_XSMSUBMSP	= 2751,
    PPC_XSMSUBQP	= 2752,
    PPC_XSMSUBQPO	= 2753,
    PPC_XSMULDP	= 2754,
    PPC_XSMULQP	= 2755,
    PPC_XSMULQPO	= 2756,
    PPC_XSMULSP	= 2757,
    PPC_XSNABSDP	= 2758,
    PPC_XSNABSDPs	= 2759,
    PPC_XSNABSQP	= 2760,
    PPC_XSNEGDP	= 2761,
    PPC_XSNEGQP	= 2762,
    PPC_XSNMADDADP	= 2763,
    PPC_XSNMADDASP	= 2764,
    PPC_XSNMADDMDP	= 2765,
    PPC_XSNMADDMSP	= 2766,
    PPC_XSNMADDQP	= 2767,
    PPC_XSNMADDQPO	= 2768,
    PPC_XSNMSUBADP	= 2769,
    PPC_XSNMSUBASP	= 2770,
    PPC_XSNMSUBMDP	= 2771,
    PPC_XSNMSUBMSP	= 2772,
    PPC_XSNMSUBQP	= 2773,
    PPC_XSNMSUBQPO	= 2774,
    PPC_XSRDPI	= 2775,
    PPC_XSRDPIC	= 2776,
    PPC_XSRDPIM	= 2777,
    PPC_XSRDPIP	= 2778,
    PPC_XSRDPIZ	= 2779,
    PPC_XSREDP	= 2780,
    PPC_XSRESP	= 2781,
    PPC_XSRQPI	= 2782,
    PPC_XSRQPIX	= 2783,
    PPC_XSRQPXP	= 2784,
    PPC_XSRSP	= 2785,
    PPC_XSRSQRTEDP	= 2786,
    PPC_XSRSQRTESP	= 2787,
    PPC_XSSQRTDP	= 2788,
    PPC_XSSQRTQP	= 2789,
    PPC_XSSQRTQPO	= 2790,
    PPC_XSSQRTSP	= 2791,
    PPC_XSSUBDP	= 2792,
    PPC_XSSUBQP	= 2793,
    PPC_XSSUBQPO	= 2794,
    PPC_XSSUBSP	= 2795,
    PPC_XSTDIVDP	= 2796,
    PPC_XSTSQRTDP	= 2797,
    PPC_XSTSTDCDP	= 2798,
    PPC_XSTSTDCQP	= 2799,
    PPC_XSTSTDCSP	= 2800,
    PPC_XSXEXPDP	= 2801,
    PPC_XSXEXPQP	= 2802,
    PPC_XSXSIGDP	= 2803,
    PPC_XSXSIGQP	= 2804,
    PPC_XVABSDP	= 2805,
    PPC_XVABSSP	= 2806,
    PPC_XVADDDP	= 2807,
    PPC_XVADDSP	= 2808,
    PPC_XVBF16GER2	= 2809,
    PPC_XVBF16GER2NN	= 2810,
    PPC_XVBF16GER2NP	= 2811,
    PPC_XVBF16GER2PN	= 2812,
    PPC_XVBF16GER2PP	= 2813,
    PPC_XVBF16GER2W	= 2814,
    PPC_XVBF16GER2WNN	= 2815,
    PPC_XVBF16GER2WNP	= 2816,
    PPC_XVBF16GER2WPN	= 2817,
    PPC_XVBF16GER2WPP	= 2818,
    PPC_XVCMPEQDP	= 2819,
    PPC_XVCMPEQDP_rec	= 2820,
    PPC_XVCMPEQSP	= 2821,
    PPC_XVCMPEQSP_rec	= 2822,
    PPC_XVCMPGEDP	= 2823,
    PPC_XVCMPGEDP_rec	= 2824,
    PPC_XVCMPGESP	= 2825,
    PPC_XVCMPGESP_rec	= 2826,
    PPC_XVCMPGTDP	= 2827,
    PPC_XVCMPGTDP_rec	= 2828,
    PPC_XVCMPGTSP	= 2829,
    PPC_XVCMPGTSP_rec	= 2830,
    PPC_XVCPSGNDP	= 2831,
    PPC_XVCPSGNSP	= 2832,
    PPC_XVCVBF16SPN	= 2833,
    PPC_XVCVDPSP	= 2834,
    PPC_XVCVDPSXDS	= 2835,
    PPC_XVCVDPSXWS	= 2836,
    PPC_XVCVDPUXDS	= 2837,
    PPC_XVCVDPUXWS	= 2838,
    PPC_XVCVHPSP	= 2839,
    PPC_XVCVSPBF16	= 2840,
    PPC_XVCVSPDP	= 2841,
    PPC_XVCVSPHP	= 2842,
    PPC_XVCVSPSXDS	= 2843,
    PPC_XVCVSPSXWS	= 2844,
    PPC_XVCVSPUXDS	= 2845,
    PPC_XVCVSPUXWS	= 2846,
    PPC_XVCVSXDDP	= 2847,
    PPC_XVCVSXDSP	= 2848,
    PPC_XVCVSXWDP	= 2849,
    PPC_XVCVSXWSP	= 2850,
    PPC_XVCVUXDDP	= 2851,
    PPC_XVCVUXDSP	= 2852,
    PPC_XVCVUXWDP	= 2853,
    PPC_XVCVUXWSP	= 2854,
    PPC_XVDIVDP	= 2855,
    PPC_XVDIVSP	= 2856,
    PPC_XVF16GER2	= 2857,
    PPC_XVF16GER2NN	= 2858,
    PPC_XVF16GER2NP	= 2859,
    PPC_XVF16GER2PN	= 2860,
    PPC_XVF16GER2PP	= 2861,
    PPC_XVF16GER2W	= 2862,
    PPC_XVF16GER2WNN	= 2863,
    PPC_XVF16GER2WNP	= 2864,
    PPC_XVF16GER2WPN	= 2865,
    PPC_XVF16GER2WPP	= 2866,
    PPC_XVF32GER	= 2867,
    PPC_XVF32GERNN	= 2868,
    PPC_XVF32GERNP	= 2869,
    PPC_XVF32GERPN	= 2870,
    PPC_XVF32GERPP	= 2871,
    PPC_XVF32GERW	= 2872,
    PPC_XVF32GERWNN	= 2873,
    PPC_XVF32GERWNP	= 2874,
    PPC_XVF32GERWPN	= 2875,
    PPC_XVF32GERWPP	= 2876,
    PPC_XVF64GER	= 2877,
    PPC_XVF64GERNN	= 2878,
    PPC_XVF64GERNP	= 2879,
    PPC_XVF64GERPN	= 2880,
    PPC_XVF64GERPP	= 2881,
    PPC_XVF64GERW	= 2882,
    PPC_XVF64GERWNN	= 2883,
    PPC_XVF64GERWNP	= 2884,
    PPC_XVF64GERWPN	= 2885,
    PPC_XVF64GERWPP	= 2886,
    PPC_XVI16GER2	= 2887,
    PPC_XVI16GER2PP	= 2888,
    PPC_XVI16GER2S	= 2889,
    PPC_XVI16GER2SPP	= 2890,
    PPC_XVI16GER2SW	= 2891,
    PPC_XVI16GER2SWPP	= 2892,
    PPC_XVI16GER2W	= 2893,
    PPC_XVI16GER2WPP	= 2894,
    PPC_XVI4GER8	= 2895,
    PPC_XVI4GER8PP	= 2896,
    PPC_XVI4GER8W	= 2897,
    PPC_XVI4GER8WPP	= 2898,
    PPC_XVI8GER4	= 2899,
    PPC_XVI8GER4PP	= 2900,
    PPC_XVI8GER4SPP	= 2901,
    PPC_XVI8GER4W	= 2902,
    PPC_XVI8GER4WPP	= 2903,
    PPC_XVI8GER4WSPP	= 2904,
    PPC_XVIEXPDP	= 2905,
    PPC_XVIEXPSP	= 2906,
    PPC_XVMADDADP	= 2907,
    PPC_XVMADDASP	= 2908,
    PPC_XVMADDMDP	= 2909,
    PPC_XVMADDMSP	= 2910,
    PPC_XVMAXDP	= 2911,
    PPC_XVMAXSP	= 2912,
    PPC_XVMINDP	= 2913,
    PPC_XVMINSP	= 2914,
    PPC_XVMSUBADP	= 2915,
    PPC_XVMSUBASP	= 2916,
    PPC_XVMSUBMDP	= 2917,
    PPC_XVMSUBMSP	= 2918,
    PPC_XVMULDP	= 2919,
    PPC_XVMULSP	= 2920,
    PPC_XVNABSDP	= 2921,
    PPC_XVNABSSP	= 2922,
    PPC_XVNEGDP	= 2923,
    PPC_XVNEGSP	= 2924,
    PPC_XVNMADDADP	= 2925,
    PPC_XVNMADDASP	= 2926,
    PPC_XVNMADDMDP	= 2927,
    PPC_XVNMADDMSP	= 2928,
    PPC_XVNMSUBADP	= 2929,
    PPC_XVNMSUBASP	= 2930,
    PPC_XVNMSUBMDP	= 2931,
    PPC_XVNMSUBMSP	= 2932,
    PPC_XVRDPI	= 2933,
    PPC_XVRDPIC	= 2934,
    PPC_XVRDPIM	= 2935,
    PPC_XVRDPIP	= 2936,
    PPC_XVRDPIZ	= 2937,
    PPC_XVREDP	= 2938,
    PPC_XVRESP	= 2939,
    PPC_XVRSPI	= 2940,
    PPC_XVRSPIC	= 2941,
    PPC_XVRSPIM	= 2942,
    PPC_XVRSPIP	= 2943,
    PPC_XVRSPIZ	= 2944,
    PPC_XVRSQRTEDP	= 2945,
    PPC_XVRSQRTESP	= 2946,
    PPC_XVSQRTDP	= 2947,
    PPC_XVSQRTSP	= 2948,
    PPC_XVSUBDP	= 2949,
    PPC_XVSUBSP	= 2950,
    PPC_XVTDIVDP	= 2951,
    PPC_XVTDIVSP	= 2952,
    PPC_XVTLSBB	= 2953,
    PPC_XVTSQRTDP	= 2954,
    PPC_XVTSQRTSP	= 2955,
    PPC_XVTSTDCDP	= 2956,
    PPC_XVTSTDCSP	= 2957,
    PPC_XVXEXPDP	= 2958,
    PPC_XVXEXPSP	= 2959,
    PPC_XVXSIGDP	= 2960,
    PPC_XVXSIGSP	= 2961,
    PPC_XXBLENDVB	= 2962,
    PPC_XXBLENDVD	= 2963,
    PPC_XXBLENDVH	= 2964,
    PPC_XXBLENDVW	= 2965,
    PPC_XXBRD	= 2966,
    PPC_XXBRH	= 2967,
    PPC_XXBRQ	= 2968,
    PPC_XXBRW	= 2969,
    PPC_XXEVAL	= 2970,
    PPC_XXEXTRACTUW	= 2971,
    PPC_XXGENPCVBM	= 2972,
    PPC_XXGENPCVDM	= 2973,
    PPC_XXGENPCVHM	= 2974,
    PPC_XXGENPCVWM	= 2975,
    PPC_XXINSERTW	= 2976,
    PPC_XXLAND	= 2977,
    PPC_XXLANDC	= 2978,
    PPC_XXLEQV	= 2979,
    PPC_XXLEQVOnes	= 2980,
    PPC_XXLNAND	= 2981,
    PPC_XXLNOR	= 2982,
    PPC_XXLOR	= 2983,
    PPC_XXLORC	= 2984,
    PPC_XXLORf	= 2985,
    PPC_XXLXOR	= 2986,
    PPC_XXLXORdpz	= 2987,
    PPC_XXLXORspz	= 2988,
    PPC_XXLXORz	= 2989,
    PPC_XXMFACC	= 2990,
    PPC_XXMFACCW	= 2991,
    PPC_XXMRGHW	= 2992,
    PPC_XXMRGLW	= 2993,
    PPC_XXMTACC	= 2994,
    PPC_XXMTACCW	= 2995,
    PPC_XXPERM	= 2996,
    PPC_XXPERMDI	= 2997,
    PPC_XXPERMDIs	= 2998,
    PPC_XXPERMR	= 2999,
    PPC_XXPERMX	= 3000,
    PPC_XXSEL	= 3001,
    PPC_XXSETACCZ	= 3002,
    PPC_XXSETACCZW	= 3003,
    PPC_XXSLDWI	= 3004,
    PPC_XXSLDWIs	= 3005,
    PPC_XXSPLTI32DX	= 3006,
    PPC_XXSPLTIB	= 3007,
    PPC_XXSPLTIDP	= 3008,
    PPC_XXSPLTIW	= 3009,
    PPC_XXSPLTW	= 3010,
    PPC_XXSPLTWs	= 3011,
    PPC_gBC	= 3012,
    PPC_gBCA	= 3013,
    PPC_gBCAat	= 3014,
    PPC_gBCCTR	= 3015,
    PPC_gBCCTRL	= 3016,
    PPC_gBCL	= 3017,
    PPC_gBCLA	= 3018,
    PPC_gBCLAat	= 3019,
    PPC_gBCLR	= 3020,
    PPC_gBCLRL	= 3021,
    PPC_gBCLat	= 3022,
    PPC_gBCat	= 3023,
    INSTRUCTION_LIST_END = 3024
  };

#endif // GET_INSTRINFO_ENUM

#if defined(GET_INSTRINFO_MC_DESC) || defined(GET_INSTRINFO_CTOR_DTOR)
typedef struct PPCInstrTable {
  MCInstrDesc Insts[3024];
  MCOperandInfo OperandInfo[1401];
  MCPhysReg ImplicitOps[218];
} PPCInstrTable;

#endif // defined(GET_INSTRINFO_MC_DESC) || defined(GET_INSTRINFO_CTOR_DTOR)

#ifdef GET_INSTRINFO_MC_DESC
#undef GET_INSTRINFO_MC_DESC

static const unsigned PPCImpOpBase = sizeof(MCOperandInfo) / (sizeof(MCPhysReg));

static const PPCInstrTable PPCDescs = {
  {
  { 4, &PPCDescs.OperandInfo[1397] },  // Inst #3023 = gBCat
  { 4, &PPCDescs.OperandInfo[1397] },  // Inst #3022 = gBCLat
  { 3, &PPCDescs.OperandInfo[1390] },  // Inst #3021 = gBCLRL
  { 3, &PPCDescs.OperandInfo[1390] },  // Inst #3020 = gBCLR
  { 4, &PPCDescs.OperandInfo[1393] },  // Inst #3019 = gBCLAat
  { 3, &PPCDescs.OperandInfo[1390] },  // Inst #3018 = gBCLA
  { 3, &PPCDescs.OperandInfo[1387] },  // Inst #3017 = gBCL
  { 3, &PPCDescs.OperandInfo[1390] },  // Inst #3016 = gBCCTRL
  { 3, &PPCDescs.OperandInfo[1390] },  // Inst #3015 = gBCCTR
  { 4, &PPCDescs.OperandInfo[1393] },  // Inst #3014 = gBCAat
  { 3, &PPCDescs.OperandInfo[1390] },  // Inst #3013 = gBCA
  { 3, &PPCDescs.OperandInfo[1387] },  // Inst #3012 = gBC
  { 3, &PPCDescs.OperandInfo[1375] },  // Inst #3011 = XXSPLTWs
  { 3, &PPCDescs.OperandInfo[1384] },  // Inst #3010 = XXSPLTW
  { 2, &PPCDescs.OperandInfo[621] },  // Inst #3009 = XXSPLTIW
  { 2, &PPCDescs.OperandInfo[621] },  // Inst #3008 = XXSPLTIDP
  { 2, &PPCDescs.OperandInfo[621] },  // Inst #3007 = XXSPLTIB
  { 4, &PPCDescs.OperandInfo[1380] },  // Inst #3006 = XXSPLTI32DX
  { 3, &PPCDescs.OperandInfo[1375] },  // Inst #3005 = XXSLDWIs
  { 4, &PPCDescs.OperandInfo[1371] },  // Inst #3004 = XXSLDWI
  { 1, &PPCDescs.OperandInfo[1379] },  // Inst #3003 = XXSETACCZW
  { 1, &PPCDescs.OperandInfo[1378] },  // Inst #3002 = XXSETACCZ
  { 4, &PPCDescs.OperandInfo[1341] },  // Inst #3001 = XXSEL
  { 5, &PPCDescs.OperandInfo[1345] },  // Inst #3000 = XXPERMX
  { 4, &PPCDescs.OperandInfo[1367] },  // Inst #2999 = XXPERMR
  { 3, &PPCDescs.OperandInfo[1375] },  // Inst #2998 = XXPERMDIs
  { 4, &PPCDescs.OperandInfo[1371] },  // Inst #2997 = XXPERMDI
  { 4, &PPCDescs.OperandInfo[1367] },  // Inst #2996 = XXPERM
  { 2, &PPCDescs.OperandInfo[1365] },  // Inst #2995 = XXMTACCW
  { 2, &PPCDescs.OperandInfo[1363] },  // Inst #2994 = XXMTACC
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2993 = XXMRGLW
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2992 = XXMRGHW
  { 2, &PPCDescs.OperandInfo[1365] },  // Inst #2991 = XXMFACCW
  { 2, &PPCDescs.OperandInfo[1363] },  // Inst #2990 = XXMFACC
  { 1, &PPCDescs.OperandInfo[1360] },  // Inst #2989 = XXLXORz
  { 1, &PPCDescs.OperandInfo[1362] },  // Inst #2988 = XXLXORspz
  { 1, &PPCDescs.OperandInfo[1361] },  // Inst #2987 = XXLXORdpz
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2986 = XXLXOR
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2985 = XXLORf
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2984 = XXLORC
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2983 = XXLOR
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2982 = XXLNOR
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2981 = XXLNAND
  { 1, &PPCDescs.OperandInfo[1360] },  // Inst #2980 = XXLEQVOnes
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2979 = XXLEQV
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2978 = XXLANDC
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2977 = XXLAND
  { 4, &PPCDescs.OperandInfo[1356] },  // Inst #2976 = XXINSERTW
  { 3, &PPCDescs.OperandInfo[1353] },  // Inst #2975 = XXGENPCVWM
  { 3, &PPCDescs.OperandInfo[1353] },  // Inst #2974 = XXGENPCVHM
  { 3, &PPCDescs.OperandInfo[1353] },  // Inst #2973 = XXGENPCVDM
  { 3, &PPCDescs.OperandInfo[1353] },  // Inst #2972 = XXGENPCVBM
  { 3, &PPCDescs.OperandInfo[1350] },  // Inst #2971 = XXEXTRACTUW
  { 5, &PPCDescs.OperandInfo[1345] },  // Inst #2970 = XXEVAL
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2969 = XXBRW
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2968 = XXBRQ
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2967 = XXBRH
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2966 = XXBRD
  { 4, &PPCDescs.OperandInfo[1341] },  // Inst #2965 = XXBLENDVW
  { 4, &PPCDescs.OperandInfo[1341] },  // Inst #2964 = XXBLENDVH
  { 4, &PPCDescs.OperandInfo[1341] },  // Inst #2963 = XXBLENDVD
  { 4, &PPCDescs.OperandInfo[1341] },  // Inst #2962 = XXBLENDVB
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2961 = XVXSIGSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2960 = XVXSIGDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2959 = XVXEXPSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2958 = XVXEXPDP
  { 3, &PPCDescs.OperandInfo[1338] },  // Inst #2957 = XVTSTDCSP
  { 3, &PPCDescs.OperandInfo[1338] },  // Inst #2956 = XVTSTDCDP
  { 2, &PPCDescs.OperandInfo[1336] },  // Inst #2955 = XVTSQRTSP
  { 2, &PPCDescs.OperandInfo[1336] },  // Inst #2954 = XVTSQRTDP
  { 2, &PPCDescs.OperandInfo[1336] },  // Inst #2953 = XVTLSBB
  { 3, &PPCDescs.OperandInfo[1333] },  // Inst #2952 = XVTDIVSP
  { 3, &PPCDescs.OperandInfo[1333] },  // Inst #2951 = XVTDIVDP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2950 = XVSUBSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2949 = XVSUBDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2948 = XVSQRTSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2947 = XVSQRTDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2946 = XVRSQRTESP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2945 = XVRSQRTEDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2944 = XVRSPIZ
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2943 = XVRSPIP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2942 = XVRSPIM
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2941 = XVRSPIC
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2940 = XVRSPI
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2939 = XVRESP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2938 = XVREDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2937 = XVRDPIZ
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2936 = XVRDPIP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2935 = XVRDPIM
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2934 = XVRDPIC
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2933 = XVRDPI
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2932 = XVNMSUBMSP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2931 = XVNMSUBMDP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2930 = XVNMSUBASP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2929 = XVNMSUBADP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2928 = XVNMADDMSP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2927 = XVNMADDMDP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2926 = XVNMADDASP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2925 = XVNMADDADP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2924 = XVNEGSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2923 = XVNEGDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2922 = XVNABSSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2921 = XVNABSDP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2920 = XVMULSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2919 = XVMULDP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2918 = XVMSUBMSP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2917 = XVMSUBMDP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2916 = XVMSUBASP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2915 = XVMSUBADP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2914 = XVMINSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2913 = XVMINDP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2912 = XVMAXSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2911 = XVMAXDP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2910 = XVMADDMSP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2909 = XVMADDMDP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2908 = XVMADDASP
  { 4, &PPCDescs.OperandInfo[1329] },  // Inst #2907 = XVMADDADP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2906 = XVIEXPSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2905 = XVIEXPDP
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2904 = XVI8GER4WSPP
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2903 = XVI8GER4WPP
  { 3, &PPCDescs.OperandInfo[1308] },  // Inst #2902 = XVI8GER4W
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2901 = XVI8GER4SPP
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2900 = XVI8GER4PP
  { 3, &PPCDescs.OperandInfo[1301] },  // Inst #2899 = XVI8GER4
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2898 = XVI4GER8WPP
  { 3, &PPCDescs.OperandInfo[1308] },  // Inst #2897 = XVI4GER8W
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2896 = XVI4GER8PP
  { 3, &PPCDescs.OperandInfo[1301] },  // Inst #2895 = XVI4GER8
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2894 = XVI16GER2WPP
  { 3, &PPCDescs.OperandInfo[1308] },  // Inst #2893 = XVI16GER2W
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2892 = XVI16GER2SWPP
  { 3, &PPCDescs.OperandInfo[1308] },  // Inst #2891 = XVI16GER2SW
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2890 = XVI16GER2SPP
  { 3, &PPCDescs.OperandInfo[1301] },  // Inst #2889 = XVI16GER2S
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2888 = XVI16GER2PP
  { 3, &PPCDescs.OperandInfo[1301] },  // Inst #2887 = XVI16GER2
  { 4, &PPCDescs.OperandInfo[1325] },  // Inst #2886 = XVF64GERWPP
  { 4, &PPCDescs.OperandInfo[1325] },  // Inst #2885 = XVF64GERWPN
  { 4, &PPCDescs.OperandInfo[1325] },  // Inst #2884 = XVF64GERWNP
  { 4, &PPCDescs.OperandInfo[1325] },  // Inst #2883 = XVF64GERWNN
  { 3, &PPCDescs.OperandInfo[1322] },  // Inst #2882 = XVF64GERW
  { 4, &PPCDescs.OperandInfo[1318] },  // Inst #2881 = XVF64GERPP
  { 4, &PPCDescs.OperandInfo[1318] },  // Inst #2880 = XVF64GERPN
  { 4, &PPCDescs.OperandInfo[1318] },  // Inst #2879 = XVF64GERNP
  { 4, &PPCDescs.OperandInfo[1318] },  // Inst #2878 = XVF64GERNN
  { 3, &PPCDescs.OperandInfo[1315] },  // Inst #2877 = XVF64GER
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2876 = XVF32GERWPP
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2875 = XVF32GERWPN
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2874 = XVF32GERWNP
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2873 = XVF32GERWNN
  { 3, &PPCDescs.OperandInfo[1308] },  // Inst #2872 = XVF32GERW
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2871 = XVF32GERPP
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2870 = XVF32GERPN
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2869 = XVF32GERNP
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2868 = XVF32GERNN
  { 3, &PPCDescs.OperandInfo[1301] },  // Inst #2867 = XVF32GER
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2866 = XVF16GER2WPP
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2865 = XVF16GER2WPN
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2864 = XVF16GER2WNP
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2863 = XVF16GER2WNN
  { 3, &PPCDescs.OperandInfo[1308] },  // Inst #2862 = XVF16GER2W
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2861 = XVF16GER2PP
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2860 = XVF16GER2PN
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2859 = XVF16GER2NP
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2858 = XVF16GER2NN
  { 3, &PPCDescs.OperandInfo[1301] },  // Inst #2857 = XVF16GER2
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2856 = XVDIVSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2855 = XVDIVDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2854 = XVCVUXWSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2853 = XVCVUXWDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2852 = XVCVUXDSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2851 = XVCVUXDDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2850 = XVCVSXWSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2849 = XVCVSXWDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2848 = XVCVSXDSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2847 = XVCVSXDDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2846 = XVCVSPUXWS
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2845 = XVCVSPUXDS
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2844 = XVCVSPSXWS
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2843 = XVCVSPSXDS
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2842 = XVCVSPHP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2841 = XVCVSPDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2840 = XVCVSPBF16
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2839 = XVCVHPSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2838 = XVCVDPUXWS
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2837 = XVCVDPUXDS
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2836 = XVCVDPSXWS
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2835 = XVCVDPSXDS
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2834 = XVCVDPSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2833 = XVCVBF16SPN
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2832 = XVCPSGNSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2831 = XVCPSGNDP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2830 = XVCMPGTSP_rec
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2829 = XVCMPGTSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2828 = XVCMPGTDP_rec
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2827 = XVCMPGTDP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2826 = XVCMPGESP_rec
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2825 = XVCMPGESP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2824 = XVCMPGEDP_rec
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2823 = XVCMPGEDP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2822 = XVCMPEQSP_rec
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2821 = XVCMPEQSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2820 = XVCMPEQDP_rec
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2819 = XVCMPEQDP
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2818 = XVBF16GER2WPP
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2817 = XVBF16GER2WPN
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2816 = XVBF16GER2WNP
  { 4, &PPCDescs.OperandInfo[1311] },  // Inst #2815 = XVBF16GER2WNN
  { 3, &PPCDescs.OperandInfo[1308] },  // Inst #2814 = XVBF16GER2W
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2813 = XVBF16GER2PP
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2812 = XVBF16GER2PN
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2811 = XVBF16GER2NP
  { 4, &PPCDescs.OperandInfo[1304] },  // Inst #2810 = XVBF16GER2NN
  { 3, &PPCDescs.OperandInfo[1301] },  // Inst #2809 = XVBF16GER2
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2808 = XVADDSP
  { 3, &PPCDescs.OperandInfo[1298] },  // Inst #2807 = XVADDDP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2806 = XVABSSP
  { 2, &PPCDescs.OperandInfo[1296] },  // Inst #2805 = XVABSDP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2804 = XSXSIGQP
  { 2, &PPCDescs.OperandInfo[663] },  // Inst #2803 = XSXSIGDP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2802 = XSXEXPQP
  { 2, &PPCDescs.OperandInfo[663] },  // Inst #2801 = XSXEXPDP
  { 3, &PPCDescs.OperandInfo[1290] },  // Inst #2800 = XSTSTDCSP
  { 3, &PPCDescs.OperandInfo[1293] },  // Inst #2799 = XSTSTDCQP
  { 3, &PPCDescs.OperandInfo[1290] },  // Inst #2798 = XSTSTDCDP
  { 2, &PPCDescs.OperandInfo[1288] },  // Inst #2797 = XSTSQRTDP
  { 3, &PPCDescs.OperandInfo[1251] },  // Inst #2796 = XSTDIVDP
  { 3, &PPCDescs.OperandInfo[1245] },  // Inst #2795 = XSSUBSP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2794 = XSSUBQPO
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2793 = XSSUBQP
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2792 = XSSUBDP
  { 2, &PPCDescs.OperandInfo[1258] },  // Inst #2791 = XSSQRTSP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2790 = XSSQRTQPO
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2789 = XSSQRTQP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2788 = XSSQRTDP
  { 2, &PPCDescs.OperandInfo[1258] },  // Inst #2787 = XSRSQRTESP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2786 = XSRSQRTEDP
  { 2, &PPCDescs.OperandInfo[1264] },  // Inst #2785 = XSRSP
  { 4, &PPCDescs.OperandInfo[1284] },  // Inst #2784 = XSRQPXP
  { 4, &PPCDescs.OperandInfo[1284] },  // Inst #2783 = XSRQPIX
  { 4, &PPCDescs.OperandInfo[1284] },  // Inst #2782 = XSRQPI
  { 2, &PPCDescs.OperandInfo[1258] },  // Inst #2781 = XSRESP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2780 = XSREDP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2779 = XSRDPIZ
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2778 = XSRDPIP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2777 = XSRDPIM
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2776 = XSRDPIC
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2775 = XSRDPI
  { 4, &PPCDescs.OperandInfo[1280] },  // Inst #2774 = XSNMSUBQPO
  { 4, &PPCDescs.OperandInfo[1280] },  // Inst #2773 = XSNMSUBQP
  { 4, &PPCDescs.OperandInfo[1276] },  // Inst #2772 = XSNMSUBMSP
  { 4, &PPCDescs.OperandInfo[1272] },  // Inst #2771 = XSNMSUBMDP
  { 4, &PPCDescs.OperandInfo[1276] },  // Inst #2770 = XSNMSUBASP
  { 4, &PPCDescs.OperandInfo[1272] },  // Inst #2769 = XSNMSUBADP
  { 4, &PPCDescs.OperandInfo[1280] },  // Inst #2768 = XSNMADDQPO
  { 4, &PPCDescs.OperandInfo[1280] },  // Inst #2767 = XSNMADDQP
  { 4, &PPCDescs.OperandInfo[1276] },  // Inst #2766 = XSNMADDMSP
  { 4, &PPCDescs.OperandInfo[1272] },  // Inst #2765 = XSNMADDMDP
  { 4, &PPCDescs.OperandInfo[1276] },  // Inst #2764 = XSNMADDASP
  { 4, &PPCDescs.OperandInfo[1272] },  // Inst #2763 = XSNMADDADP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2762 = XSNEGQP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2761 = XSNEGDP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2760 = XSNABSQP
  { 2, &PPCDescs.OperandInfo[1258] },  // Inst #2759 = XSNABSDPs
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2758 = XSNABSDP
  { 3, &PPCDescs.OperandInfo[1245] },  // Inst #2757 = XSMULSP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2756 = XSMULQPO
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2755 = XSMULQP
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2754 = XSMULDP
  { 4, &PPCDescs.OperandInfo[1280] },  // Inst #2753 = XSMSUBQPO
  { 4, &PPCDescs.OperandInfo[1280] },  // Inst #2752 = XSMSUBQP
  { 4, &PPCDescs.OperandInfo[1276] },  // Inst #2751 = XSMSUBMSP
  { 4, &PPCDescs.OperandInfo[1272] },  // Inst #2750 = XSMSUBMDP
  { 4, &PPCDescs.OperandInfo[1276] },  // Inst #2749 = XSMSUBASP
  { 4, &PPCDescs.OperandInfo[1272] },  // Inst #2748 = XSMSUBADP
  { 3, &PPCDescs.OperandInfo[1248] },  // Inst #2747 = XSMINJDP
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2746 = XSMINDP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2745 = XSMINCQP
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2744 = XSMINCDP
  { 3, &PPCDescs.OperandInfo[1248] },  // Inst #2743 = XSMAXJDP
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2742 = XSMAXDP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2741 = XSMAXCQP
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2740 = XSMAXCDP
  { 4, &PPCDescs.OperandInfo[1280] },  // Inst #2739 = XSMADDQPO
  { 4, &PPCDescs.OperandInfo[1280] },  // Inst #2738 = XSMADDQP
  { 4, &PPCDescs.OperandInfo[1276] },  // Inst #2737 = XSMADDMSP
  { 4, &PPCDescs.OperandInfo[1272] },  // Inst #2736 = XSMADDMDP
  { 4, &PPCDescs.OperandInfo[1276] },  // Inst #2735 = XSMADDASP
  { 4, &PPCDescs.OperandInfo[1272] },  // Inst #2734 = XSMADDADP
  { 3, &PPCDescs.OperandInfo[1269] },  // Inst #2733 = XSIEXPQP
  { 3, &PPCDescs.OperandInfo[1266] },  // Inst #2732 = XSIEXPDP
  { 3, &PPCDescs.OperandInfo[1245] },  // Inst #2731 = XSDIVSP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2730 = XSDIVQPO
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2729 = XSDIVQP
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2728 = XSDIVDP
  { 2, &PPCDescs.OperandInfo[1264] },  // Inst #2727 = XSCVUXDSP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2726 = XSCVUXDDP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2725 = XSCVUQQP
  { 2, &PPCDescs.OperandInfo[1254] },  // Inst #2724 = XSCVUDQP
  { 2, &PPCDescs.OperandInfo[1264] },  // Inst #2723 = XSCVSXDSP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2722 = XSCVSXDDP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2721 = XSCVSQQP
  { 2, &PPCDescs.OperandInfo[1262] },  // Inst #2720 = XSCVSPDPN
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2719 = XSCVSPDP
  { 2, &PPCDescs.OperandInfo[1254] },  // Inst #2718 = XSCVSDQP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2717 = XSCVQPUWZ
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2716 = XSCVQPUQZ
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2715 = XSCVQPUDZ
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2714 = XSCVQPSWZ
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2713 = XSCVQPSQZ
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2712 = XSCVQPSDZ
  { 2, &PPCDescs.OperandInfo[1260] },  // Inst #2711 = XSCVQPDPO
  { 2, &PPCDescs.OperandInfo[1260] },  // Inst #2710 = XSCVQPDP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2709 = XSCVHPDP
  { 2, &PPCDescs.OperandInfo[1258] },  // Inst #2708 = XSCVDPUXWSs
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2707 = XSCVDPUXWS
  { 2, &PPCDescs.OperandInfo[1258] },  // Inst #2706 = XSCVDPUXDSs
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2705 = XSCVDPUXDS
  { 2, &PPCDescs.OperandInfo[1258] },  // Inst #2704 = XSCVDPSXWSs
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2703 = XSCVDPSXWS
  { 2, &PPCDescs.OperandInfo[1258] },  // Inst #2702 = XSCVDPSXDSs
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2701 = XSCVDPSXDS
  { 2, &PPCDescs.OperandInfo[1256] },  // Inst #2700 = XSCVDPSPN
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2699 = XSCVDPSP
  { 2, &PPCDescs.OperandInfo[1254] },  // Inst #2698 = XSCVDPQP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2697 = XSCVDPHP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2696 = XSCPSGNQP
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2695 = XSCPSGNDP
  { 3, &PPCDescs.OperandInfo[1190] },  // Inst #2694 = XSCMPUQP
  { 3, &PPCDescs.OperandInfo[1251] },  // Inst #2693 = XSCMPUDP
  { 3, &PPCDescs.OperandInfo[1190] },  // Inst #2692 = XSCMPOQP
  { 3, &PPCDescs.OperandInfo[1251] },  // Inst #2691 = XSCMPODP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2690 = XSCMPGTQP
  { 3, &PPCDescs.OperandInfo[1248] },  // Inst #2689 = XSCMPGTDP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2688 = XSCMPGEQP
  { 3, &PPCDescs.OperandInfo[1248] },  // Inst #2687 = XSCMPGEDP
  { 3, &PPCDescs.OperandInfo[1190] },  // Inst #2686 = XSCMPEXPQP
  { 3, &PPCDescs.OperandInfo[1251] },  // Inst #2685 = XSCMPEXPDP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2684 = XSCMPEQQP
  { 3, &PPCDescs.OperandInfo[1248] },  // Inst #2683 = XSCMPEQDP
  { 3, &PPCDescs.OperandInfo[1245] },  // Inst #2682 = XSADDSP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2681 = XSADDQPO
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2680 = XSADDQP
  { 3, &PPCDescs.OperandInfo[1242] },  // Inst #2679 = XSADDDP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2678 = XSABSQP
  { 2, &PPCDescs.OperandInfo[1240] },  // Inst #2677 = XSABSDP
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2676 = XOR_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #2675 = XORIS8
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2674 = XORIS
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #2673 = XORI8
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2672 = XORI
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2671 = XOR8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2670 = XOR8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2669 = XOR
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #2668 = WRTEEI
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #2667 = WRTEE
  { 2, &PPCDescs.OperandInfo[21] },  // Inst #2666 = WAITP10
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #2665 = WAIT
  { 1, &PPCDescs.OperandInfo[662] },  // Inst #2664 = V_SETALLONESH
  { 1, &PPCDescs.OperandInfo[662] },  // Inst #2663 = V_SETALLONESB
  { 1, &PPCDescs.OperandInfo[662] },  // Inst #2662 = V_SETALLONES
  { 1, &PPCDescs.OperandInfo[662] },  // Inst #2661 = V_SET0H
  { 1, &PPCDescs.OperandInfo[662] },  // Inst #2660 = V_SET0B
  { 1, &PPCDescs.OperandInfo[662] },  // Inst #2659 = V_SET0
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2658 = VXOR
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2657 = VUPKLSW
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2656 = VUPKLSH
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2655 = VUPKLSB
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2654 = VUPKLPX
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2653 = VUPKHSW
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2652 = VUPKHSH
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2651 = VUPKHSB
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2650 = VUPKHPX
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2649 = VSUMSWS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2648 = VSUM4UBS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2647 = VSUM4SHS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2646 = VSUM4SBS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2645 = VSUM2SWS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2644 = VSUBUWS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2643 = VSUBUWM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2642 = VSUBUQM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2641 = VSUBUHS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2640 = VSUBUHM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2639 = VSUBUDM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2638 = VSUBUBS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2637 = VSUBUBM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2636 = VSUBSWS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2635 = VSUBSHS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2634 = VSUBSBS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2633 = VSUBFP
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2632 = VSUBEUQM
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2631 = VSUBECUQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2630 = VSUBCUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2629 = VSUBCUQ
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2628 = VSTRIHR_rec
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2627 = VSTRIHR
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2626 = VSTRIHL_rec
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2625 = VSTRIHL
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2624 = VSTRIBR_rec
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2623 = VSTRIBR
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2622 = VSTRIBL_rec
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2621 = VSTRIBL
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2620 = VSRW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2619 = VSRV
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2618 = VSRQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2617 = VSRO
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2616 = VSRH
  { 4, &PPCDescs.OperandInfo[291] },  // Inst #2615 = VSRDBI
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2614 = VSRD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2613 = VSRB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2612 = VSRAW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2611 = VSRAQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2610 = VSRAH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2609 = VSRAD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2608 = VSRAB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2607 = VSR
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2606 = VSPLTW
  { 2, &PPCDescs.OperandInfo[692] },  // Inst #2605 = VSPLTISW
  { 2, &PPCDescs.OperandInfo[692] },  // Inst #2604 = VSPLTISH
  { 2, &PPCDescs.OperandInfo[692] },  // Inst #2603 = VSPLTISB
  { 3, &PPCDescs.OperandInfo[1237] },  // Inst #2602 = VSPLTHs
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2601 = VSPLTH
  { 3, &PPCDescs.OperandInfo[1237] },  // Inst #2600 = VSPLTBs
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2599 = VSPLTB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2598 = VSLW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2597 = VSLV
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2596 = VSLQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2595 = VSLO
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2594 = VSLH
  { 4, &PPCDescs.OperandInfo[291] },  // Inst #2593 = VSLDOI
  { 4, &PPCDescs.OperandInfo[291] },  // Inst #2592 = VSLDBI
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2591 = VSLD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2590 = VSLB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2589 = VSL
  { 4, &PPCDescs.OperandInfo[1233] },  // Inst #2588 = VSHASIGMAW
  { 4, &PPCDescs.OperandInfo[1233] },  // Inst #2587 = VSHASIGMAD
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2586 = VSEL
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2585 = VSBOX
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2584 = VRSQRTEFP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2583 = VRLWNM
  { 4, &PPCDescs.OperandInfo[1229] },  // Inst #2582 = VRLWMI
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2581 = VRLW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2580 = VRLQNM
  { 4, &PPCDescs.OperandInfo[1229] },  // Inst #2579 = VRLQMI
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2578 = VRLQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2577 = VRLH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2576 = VRLDNM
  { 4, &PPCDescs.OperandInfo[1229] },  // Inst #2575 = VRLDMI
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2574 = VRLD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2573 = VRLB
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2572 = VRFIZ
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2571 = VRFIP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2570 = VRFIN
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2569 = VRFIM
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2568 = VREFP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2567 = VPRTYBW
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2566 = VPRTYBQ
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2565 = VPRTYBD
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2564 = VPOPCNTW
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2563 = VPOPCNTH
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2562 = VPOPCNTD
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2561 = VPOPCNTB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2560 = VPMSUMW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2559 = VPMSUMH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2558 = VPMSUMD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2557 = VPMSUMB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2556 = VPKUWUS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2555 = VPKUWUM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2554 = VPKUHUS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2553 = VPKUHUM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2552 = VPKUDUS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2551 = VPKUDUM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2550 = VPKSWUS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2549 = VPKSWSS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2548 = VPKSHUS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2547 = VPKSHSS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2546 = VPKSDUS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2545 = VPKSDSS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2544 = VPKPX
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2543 = VPEXTD
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2542 = VPERMXOR
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2541 = VPERMR
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2540 = VPERM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2539 = VPDEPD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2538 = VORC
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2537 = VOR
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2536 = VNOR
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2535 = VNMSUBFP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2534 = VNEGW
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2533 = VNEGD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2532 = VNCIPHERLAST
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2531 = VNCIPHER
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2530 = VNAND
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2529 = VMULUWM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2528 = VMULOUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2527 = VMULOUH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2526 = VMULOUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2525 = VMULOUB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2524 = VMULOSW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2523 = VMULOSH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2522 = VMULOSD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2521 = VMULOSB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2520 = VMULLD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2519 = VMULHUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2518 = VMULHUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2517 = VMULHSW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2516 = VMULHSD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2515 = VMULEUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2514 = VMULEUH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2513 = VMULEUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2512 = VMULEUB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2511 = VMULESW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2510 = VMULESH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2509 = VMULESD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2508 = VMULESB
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2507 = VMUL10UQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2506 = VMUL10EUQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2505 = VMUL10ECUQ
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2504 = VMUL10CUQ
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2503 = VMSUMUHS
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2502 = VMSUMUHM
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2501 = VMSUMUDM
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2500 = VMSUMUBM
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2499 = VMSUMSHS
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2498 = VMSUMSHM
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2497 = VMSUMMBM
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2496 = VMSUMCUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2495 = VMRGOW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2494 = VMRGLW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2493 = VMRGLH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2492 = VMRGLB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2491 = VMRGHW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2490 = VMRGHH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2489 = VMRGHB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2488 = VMRGEW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2487 = VMODUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2486 = VMODUQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2485 = VMODUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2484 = VMODSW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2483 = VMODSQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2482 = VMODSD
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2481 = VMLADDUHM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2480 = VMINUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2479 = VMINUH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2478 = VMINUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2477 = VMINUB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2476 = VMINSW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2475 = VMINSH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2474 = VMINSD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2473 = VMINSB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2472 = VMINFP
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2471 = VMHRADDSHS
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2470 = VMHADDSHS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2469 = VMAXUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2468 = VMAXUH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2467 = VMAXUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2466 = VMAXUB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2465 = VMAXSW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2464 = VMAXSH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2463 = VMAXSD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2462 = VMAXSB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2461 = VMAXFP
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2460 = VMADDFP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2459 = VLOGEFP
  { 4, &PPCDescs.OperandInfo[1209] },  // Inst #2458 = VINSWVRX
  { 4, &PPCDescs.OperandInfo[1209] },  // Inst #2457 = VINSWVLX
  { 4, &PPCDescs.OperandInfo[1205] },  // Inst #2456 = VINSWRX
  { 4, &PPCDescs.OperandInfo[1205] },  // Inst #2455 = VINSWLX
  { 4, &PPCDescs.OperandInfo[1225] },  // Inst #2454 = VINSW
  { 4, &PPCDescs.OperandInfo[1209] },  // Inst #2453 = VINSHVRX
  { 4, &PPCDescs.OperandInfo[1209] },  // Inst #2452 = VINSHVLX
  { 4, &PPCDescs.OperandInfo[1205] },  // Inst #2451 = VINSHRX
  { 4, &PPCDescs.OperandInfo[1205] },  // Inst #2450 = VINSHLX
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2449 = VINSERTW
  { 4, &PPCDescs.OperandInfo[1221] },  // Inst #2448 = VINSERTH
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2447 = VINSERTD
  { 4, &PPCDescs.OperandInfo[1221] },  // Inst #2446 = VINSERTB
  { 4, &PPCDescs.OperandInfo[1217] },  // Inst #2445 = VINSDRX
  { 4, &PPCDescs.OperandInfo[1217] },  // Inst #2444 = VINSDLX
  { 4, &PPCDescs.OperandInfo[1213] },  // Inst #2443 = VINSD
  { 4, &PPCDescs.OperandInfo[1209] },  // Inst #2442 = VINSBVRX
  { 4, &PPCDescs.OperandInfo[1209] },  // Inst #2441 = VINSBVLX
  { 4, &PPCDescs.OperandInfo[1205] },  // Inst #2440 = VINSBRX
  { 4, &PPCDescs.OperandInfo[1205] },  // Inst #2439 = VINSBLX
  { 3, &PPCDescs.OperandInfo[1193] },  // Inst #2438 = VGNB
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2437 = VGBBD
  { 3, &PPCDescs.OperandInfo[1202] },  // Inst #2436 = VEXTUWRX
  { 3, &PPCDescs.OperandInfo[1202] },  // Inst #2435 = VEXTUWLX
  { 3, &PPCDescs.OperandInfo[1202] },  // Inst #2434 = VEXTUHRX
  { 3, &PPCDescs.OperandInfo[1202] },  // Inst #2433 = VEXTUHLX
  { 3, &PPCDescs.OperandInfo[1202] },  // Inst #2432 = VEXTUBRX
  { 3, &PPCDescs.OperandInfo[1202] },  // Inst #2431 = VEXTUBLX
  { 2, &PPCDescs.OperandInfo[1200] },  // Inst #2430 = VEXTSW2Ds
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2429 = VEXTSW2D
  { 2, &PPCDescs.OperandInfo[1200] },  // Inst #2428 = VEXTSH2Ws
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2427 = VEXTSH2W
  { 2, &PPCDescs.OperandInfo[1200] },  // Inst #2426 = VEXTSH2Ds
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2425 = VEXTSH2D
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2424 = VEXTSD2Q
  { 2, &PPCDescs.OperandInfo[1200] },  // Inst #2423 = VEXTSB2Ws
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2422 = VEXTSB2W
  { 2, &PPCDescs.OperandInfo[1200] },  // Inst #2421 = VEXTSB2Ds
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2420 = VEXTSB2D
  { 2, &PPCDescs.OperandInfo[1188] },  // Inst #2419 = VEXTRACTWM
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2418 = VEXTRACTUW
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2417 = VEXTRACTUH
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2416 = VEXTRACTUB
  { 2, &PPCDescs.OperandInfo[1188] },  // Inst #2415 = VEXTRACTQM
  { 2, &PPCDescs.OperandInfo[1188] },  // Inst #2414 = VEXTRACTHM
  { 2, &PPCDescs.OperandInfo[1188] },  // Inst #2413 = VEXTRACTDM
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2412 = VEXTRACTD
  { 2, &PPCDescs.OperandInfo[1188] },  // Inst #2411 = VEXTRACTBM
  { 4, &PPCDescs.OperandInfo[1196] },  // Inst #2410 = VEXTDUWVRX
  { 4, &PPCDescs.OperandInfo[1196] },  // Inst #2409 = VEXTDUWVLX
  { 4, &PPCDescs.OperandInfo[1196] },  // Inst #2408 = VEXTDUHVRX
  { 4, &PPCDescs.OperandInfo[1196] },  // Inst #2407 = VEXTDUHVLX
  { 4, &PPCDescs.OperandInfo[1196] },  // Inst #2406 = VEXTDUBVRX
  { 4, &PPCDescs.OperandInfo[1196] },  // Inst #2405 = VEXTDUBVLX
  { 4, &PPCDescs.OperandInfo[1196] },  // Inst #2404 = VEXTDDVRX
  { 4, &PPCDescs.OperandInfo[1196] },  // Inst #2403 = VEXTDDVLX
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2402 = VEXPTEFP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2401 = VEXPANDWM
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2400 = VEXPANDQM
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2399 = VEXPANDHM
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2398 = VEXPANDDM
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2397 = VEXPANDBM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2396 = VEQV
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2395 = VDIVUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2394 = VDIVUQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2393 = VDIVUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2392 = VDIVSW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2391 = VDIVSQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2390 = VDIVSD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2389 = VDIVEUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2388 = VDIVEUQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2387 = VDIVEUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2386 = VDIVESW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2385 = VDIVESQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2384 = VDIVESD
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2383 = VCTZW
  { 2, &PPCDescs.OperandInfo[1188] },  // Inst #2382 = VCTZLSBB
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2381 = VCTZH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2380 = VCTZDM
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2379 = VCTZD
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2378 = VCTZB
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2377 = VCTUXS_0
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2376 = VCTUXS
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2375 = VCTSXS_0
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2374 = VCTSXS
  { 3, &PPCDescs.OperandInfo[1193] },  // Inst #2373 = VCNTMBW
  { 3, &PPCDescs.OperandInfo[1193] },  // Inst #2372 = VCNTMBH
  { 3, &PPCDescs.OperandInfo[1193] },  // Inst #2371 = VCNTMBD
  { 3, &PPCDescs.OperandInfo[1193] },  // Inst #2370 = VCNTMBB
  { 3, &PPCDescs.OperandInfo[1190] },  // Inst #2369 = VCMPUQ
  { 3, &PPCDescs.OperandInfo[1190] },  // Inst #2368 = VCMPSQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2367 = VCMPNEZW_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2366 = VCMPNEZW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2365 = VCMPNEZH_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2364 = VCMPNEZH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2363 = VCMPNEZB_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2362 = VCMPNEZB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2361 = VCMPNEW_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2360 = VCMPNEW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2359 = VCMPNEH_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2358 = VCMPNEH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2357 = VCMPNEB_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2356 = VCMPNEB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2355 = VCMPGTUW_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2354 = VCMPGTUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2353 = VCMPGTUQ_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2352 = VCMPGTUQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2351 = VCMPGTUH_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2350 = VCMPGTUH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2349 = VCMPGTUD_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2348 = VCMPGTUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2347 = VCMPGTUB_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2346 = VCMPGTUB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2345 = VCMPGTSW_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2344 = VCMPGTSW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2343 = VCMPGTSQ_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2342 = VCMPGTSQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2341 = VCMPGTSH_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2340 = VCMPGTSH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2339 = VCMPGTSD_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2338 = VCMPGTSD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2337 = VCMPGTSB_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2336 = VCMPGTSB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2335 = VCMPGTFP_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2334 = VCMPGTFP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2333 = VCMPGEFP_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2332 = VCMPGEFP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2331 = VCMPEQUW_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2330 = VCMPEQUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2329 = VCMPEQUQ_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2328 = VCMPEQUQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2327 = VCMPEQUH_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2326 = VCMPEQUH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2325 = VCMPEQUD_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2324 = VCMPEQUD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2323 = VCMPEQUB_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2322 = VCMPEQUB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2321 = VCMPEQFP_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2320 = VCMPEQFP
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2319 = VCMPBFP_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2318 = VCMPBFP
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2317 = VCLZW
  { 2, &PPCDescs.OperandInfo[1188] },  // Inst #2316 = VCLZLSBB
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2315 = VCLZH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2314 = VCLZDM
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2313 = VCLZD
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2312 = VCLZB
  { 3, &PPCDescs.OperandInfo[1185] },  // Inst #2311 = VCLRRB
  { 3, &PPCDescs.OperandInfo[1185] },  // Inst #2310 = VCLRLB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2309 = VCIPHERLAST
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2308 = VCIPHER
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2307 = VCFUX_0
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2306 = VCFUX
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2305 = VCFUGED
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #2304 = VCFSX_0
  { 3, &PPCDescs.OperandInfo[1182] },  // Inst #2303 = VCFSX
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2302 = VBPERMQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2301 = VBPERMD
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2300 = VAVGUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2299 = VAVGUH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2298 = VAVGUB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2297 = VAVGSW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2296 = VAVGSH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2295 = VAVGSB
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2294 = VANDC
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2293 = VAND
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2292 = VADDUWS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2291 = VADDUWM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2290 = VADDUQM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2289 = VADDUHS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2288 = VADDUHM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2287 = VADDUDM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2286 = VADDUBS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2285 = VADDUBM
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2284 = VADDSWS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2283 = VADDSHS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2282 = VADDSBS
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2281 = VADDFP
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2280 = VADDEUQM
  { 4, &PPCDescs.OperandInfo[1178] },  // Inst #2279 = VADDECUQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2278 = VADDCUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2277 = VADDCUQ
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2276 = VABSDUW
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2275 = VABSDUH
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #2274 = VABSDUB
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2273 = UpdateGBR
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2272 = UNENCODED_NOP
  { 3, &PPCDescs.OperandInfo[1163] },  // Inst #2271 = TWI
  { 3, &PPCDescs.OperandInfo[416] },  // Inst #2270 = TW
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #2269 = TSR
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #2268 = TRECLAIM
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2267 = TRECHKPT
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2266 = TRAP
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2265 = TLSGDAIX8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2264 = TLSGDAIX
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2263 = TLBWE2
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2262 = TLBWE
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2261 = TLBSYNC
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2260 = TLBSX2D
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2259 = TLBSX2
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2258 = TLBSX
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2257 = TLBRE2
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2256 = TLBRE
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #2255 = TLBLI
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #2254 = TLBLD
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2253 = TLBIVAX
  { 3, &PPCDescs.OperandInfo[416] },  // Inst #2252 = TLBILX
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #2251 = TLBIEL
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2250 = TLBIE
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2249 = TLBIA
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #2248 = TEND
  { 3, &PPCDescs.OperandInfo[1175] },  // Inst #2247 = TDI
  { 3, &PPCDescs.OperandInfo[1172] },  // Inst #2246 = TD
  { 2, &PPCDescs.OperandInfo[1170] },  // Inst #2245 = TCRETURNri8
  { 2, &PPCDescs.OperandInfo[1168] },  // Inst #2244 = TCRETURNri
  { 2, &PPCDescs.OperandInfo[1166] },  // Inst #2243 = TCRETURNdi8
  { 2, &PPCDescs.OperandInfo[1166] },  // Inst #2242 = TCRETURNdi
  { 2, &PPCDescs.OperandInfo[21] },  // Inst #2241 = TCRETURNai8
  { 2, &PPCDescs.OperandInfo[21] },  // Inst #2240 = TCRETURNai
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #2239 = TCHECK_RET
  { 1, &PPCDescs.OperandInfo[645] },  // Inst #2238 = TCHECK
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #2237 = TBEGIN_RET
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #2236 = TBEGIN
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2235 = TAILBCTR8
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2234 = TAILBCTR
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #2233 = TAILBA8
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #2232 = TAILBA
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #2231 = TAILB8
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #2230 = TAILB
  { 3, &PPCDescs.OperandInfo[1163] },  // Inst #2229 = TABORTWCI
  { 3, &PPCDescs.OperandInfo[416] },  // Inst #2228 = TABORTWC
  { 3, &PPCDescs.OperandInfo[1163] },  // Inst #2227 = TABORTDCI
  { 3, &PPCDescs.OperandInfo[416] },  // Inst #2226 = TABORTDC
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #2225 = TABORT
  { 2, &PPCDescs.OperandInfo[21] },  // Inst #2224 = SYNCP10
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #2223 = SYNC
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2222 = SUBF_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2221 = SUBFZE_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2220 = SUBFZEO_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2219 = SUBFZEO
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #2218 = SUBFZE8_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #2217 = SUBFZE8O_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #2216 = SUBFZE8O
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #2215 = SUBFZE8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2214 = SUBFZE
  { 4, &PPCDescs.OperandInfo[229] },  // Inst #2213 = SUBFUS_rec
  { 4, &PPCDescs.OperandInfo[229] },  // Inst #2212 = SUBFUS
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2211 = SUBFO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2210 = SUBFO
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2209 = SUBFME_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2208 = SUBFMEO_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2207 = SUBFMEO
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #2206 = SUBFME8_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #2205 = SUBFME8O_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #2204 = SUBFME8O
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #2203 = SUBFME8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2202 = SUBFME
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #2201 = SUBFIC8
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2200 = SUBFIC
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2199 = SUBFE_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2198 = SUBFEO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2197 = SUBFEO
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2196 = SUBFE8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2195 = SUBFE8O_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2194 = SUBFE8O
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2193 = SUBFE8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2192 = SUBFE
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2191 = SUBFC_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2190 = SUBFCO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2189 = SUBFCO
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2188 = SUBFC8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2187 = SUBFC8O_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2186 = SUBFC8O
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2185 = SUBFC8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2184 = SUBFC
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2183 = SUBF8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2182 = SUBF8O_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2181 = SUBF8O
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2180 = SUBF8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2179 = SUBF
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2178 = STXVX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2177 = STXVW4X
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2176 = STXVRWX
  { 3, &PPCDescs.OperandInfo[623] },  // Inst #2175 = STXVRLL
  { 3, &PPCDescs.OperandInfo[623] },  // Inst #2174 = STXVRL
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2173 = STXVRHX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2172 = STXVRDX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2171 = STXVRBX
  { 3, &PPCDescs.OperandInfo[632] },  // Inst #2170 = STXVPX
  { 3, &PPCDescs.OperandInfo[629] },  // Inst #2169 = STXVPRLL
  { 3, &PPCDescs.OperandInfo[629] },  // Inst #2168 = STXVPRL
  { 3, &PPCDescs.OperandInfo[626] },  // Inst #2167 = STXVP
  { 3, &PPCDescs.OperandInfo[623] },  // Inst #2166 = STXVLL
  { 3, &PPCDescs.OperandInfo[623] },  // Inst #2165 = STXVL
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2164 = STXVH8X
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2163 = STXVD2X
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2162 = STXVB16X
  { 3, &PPCDescs.OperandInfo[615] },  // Inst #2161 = STXV
  { 3, &PPCDescs.OperandInfo[207] },  // Inst #2160 = STXSSPX
  { 3, &PPCDescs.OperandInfo[612] },  // Inst #2159 = STXSSP
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #2158 = STXSIWX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2157 = STXSIHXv
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #2156 = STXSIHX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #2155 = STXSIBXv
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #2154 = STXSIBX
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #2153 = STXSDX
  { 3, &PPCDescs.OperandInfo[612] },  // Inst #2152 = STXSD
  { 3, &PPCDescs.OperandInfo[554] },  // Inst #2151 = STWXTLS_32
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #2150 = STWXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #2149 = STWXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #2148 = STWX8
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2147 = STWX
  { 4, &PPCDescs.OperandInfo[1143] },  // Inst #2146 = STWUX8
  { 4, &PPCDescs.OperandInfo[1139] },  // Inst #2145 = STWUX
  { 4, &PPCDescs.OperandInfo[1135] },  // Inst #2144 = STWU8
  { 4, &PPCDescs.OperandInfo[1131] },  // Inst #2143 = STWU
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2142 = STWEPX
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2141 = STWCX
  { 3, &PPCDescs.OperandInfo[529] },  // Inst #2140 = STWCIX
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2139 = STWBRX
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2138 = STWAT
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #2137 = STW8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #2136 = STW
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #2135 = STVXL
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #2134 = STVX
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #2133 = STVEWX
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #2132 = STVEHX
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #2131 = STVEBX
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2130 = STSWI
  { 3, &PPCDescs.OperandInfo[603] },  // Inst #2129 = STQX_PSEUDO
  { 3, &PPCDescs.OperandInfo[603] },  // Inst #2128 = STQCX
  { 3, &PPCDescs.OperandInfo[962] },  // Inst #2127 = STQ
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2126 = STOP
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #2125 = STMW
  { 3, &PPCDescs.OperandInfo[554] },  // Inst #2124 = STHXTLS_32
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #2123 = STHXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #2122 = STHXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #2121 = STHX8
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2120 = STHX
  { 4, &PPCDescs.OperandInfo[1143] },  // Inst #2119 = STHUX8
  { 4, &PPCDescs.OperandInfo[1139] },  // Inst #2118 = STHUX
  { 4, &PPCDescs.OperandInfo[1135] },  // Inst #2117 = STHU8
  { 4, &PPCDescs.OperandInfo[1131] },  // Inst #2116 = STHU
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2115 = STHEPX
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2114 = STHCX
  { 3, &PPCDescs.OperandInfo[529] },  // Inst #2113 = STHCIX
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2112 = STHBRX
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #2111 = STH8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #2110 = STH
  { 3, &PPCDescs.OperandInfo[597] },  // Inst #2109 = STFSXTLS_
  { 3, &PPCDescs.OperandInfo[597] },  // Inst #2108 = STFSXTLS
  { 3, &PPCDescs.OperandInfo[594] },  // Inst #2107 = STFSX
  { 4, &PPCDescs.OperandInfo[1159] },  // Inst #2106 = STFSUX
  { 4, &PPCDescs.OperandInfo[1155] },  // Inst #2105 = STFSU
  { 3, &PPCDescs.OperandInfo[583] },  // Inst #2104 = STFS
  { 3, &PPCDescs.OperandInfo[569] },  // Inst #2103 = STFIWX
  { 3, &PPCDescs.OperandInfo[580] },  // Inst #2102 = STFDXTLS_
  { 3, &PPCDescs.OperandInfo[580] },  // Inst #2101 = STFDXTLS
  { 3, &PPCDescs.OperandInfo[569] },  // Inst #2100 = STFDX
  { 4, &PPCDescs.OperandInfo[1151] },  // Inst #2099 = STFDUX
  { 4, &PPCDescs.OperandInfo[1147] },  // Inst #2098 = STFDU
  { 3, &PPCDescs.OperandInfo[569] },  // Inst #2097 = STFDEPX
  { 3, &PPCDescs.OperandInfo[566] },  // Inst #2096 = STFD
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #2095 = STDXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #2094 = STDXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #2093 = STDX
  { 4, &PPCDescs.OperandInfo[1143] },  // Inst #2092 = STDUX
  { 4, &PPCDescs.OperandInfo[1135] },  // Inst #2091 = STDU
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #2090 = STDCX
  { 3, &PPCDescs.OperandInfo[529] },  // Inst #2089 = STDCIX
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #2088 = STDBRX
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #2087 = STDAT
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #2086 = STD
  { 3, &PPCDescs.OperandInfo[554] },  // Inst #2085 = STBXTLS_32
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #2084 = STBXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #2083 = STBXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #2082 = STBX8
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2081 = STBX
  { 4, &PPCDescs.OperandInfo[1143] },  // Inst #2080 = STBUX8
  { 4, &PPCDescs.OperandInfo[1139] },  // Inst #2079 = STBUX
  { 4, &PPCDescs.OperandInfo[1135] },  // Inst #2078 = STBU8
  { 4, &PPCDescs.OperandInfo[1131] },  // Inst #2077 = STBU
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2076 = STBEPX
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2075 = STBCX
  { 3, &PPCDescs.OperandInfo[529] },  // Inst #2074 = STBCIX
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #2073 = STB8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #2072 = STB
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2071 = SRW_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2070 = SRW8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2069 = SRW8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2068 = SRW
  { 3, &PPCDescs.OperandInfo[1125] },  // Inst #2067 = SRD_rec
  { 3, &PPCDescs.OperandInfo[1125] },  // Inst #2066 = SRD
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2065 = SRAW_rec
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2064 = SRAWI_rec
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2063 = SRAWI
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2062 = SRAW
  { 3, &PPCDescs.OperandInfo[1125] },  // Inst #2061 = SRAD_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #2060 = SRADI_rec
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #2059 = SRADI_32
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #2058 = SRADI
  { 3, &PPCDescs.OperandInfo[1125] },  // Inst #2057 = SRAD
  { 3, &PPCDescs.OperandInfo[1128] },  // Inst #2056 = SPLIT_QUADWORD
  { 3, &PPCDescs.OperandInfo[968] },  // Inst #2055 = SPILL_WACC
  { 3, &PPCDescs.OperandInfo[965] },  // Inst #2054 = SPILL_UACC
  { 3, &PPCDescs.OperandInfo[962] },  // Inst #2053 = SPILL_QUADWORD
  { 3, &PPCDescs.OperandInfo[959] },  // Inst #2052 = SPILL_CRBIT
  { 3, &PPCDescs.OperandInfo[956] },  // Inst #2051 = SPILL_CR
  { 3, &PPCDescs.OperandInfo[953] },  // Inst #2050 = SPILL_ACC
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2049 = SPESTWX
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #2048 = SPESTW
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #2047 = SPELWZX
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #2046 = SPELWZ
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2045 = SLW_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2044 = SLW8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #2043 = SLW8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #2042 = SLW
  { 3, &PPCDescs.OperandInfo[1125] },  // Inst #2041 = SLD_rec
  { 3, &PPCDescs.OperandInfo[1125] },  // Inst #2040 = SLD
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2039 = SLBSYNC
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2038 = SLBMTE
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2037 = SLBMFEV
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2036 = SLBMFEE
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2035 = SLBIEG
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #2034 = SLBIE
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2033 = SLBIA
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #2032 = SLBFEE_rec
  { 2, &PPCDescs.OperandInfo[650] },  // Inst #2031 = SETRNDi
  { 2, &PPCDescs.OperandInfo[1123] },  // Inst #2030 = SETRND
  { 2, &PPCDescs.OperandInfo[1121] },  // Inst #2029 = SETNBCR8
  { 2, &PPCDescs.OperandInfo[1119] },  // Inst #2028 = SETNBCR
  { 2, &PPCDescs.OperandInfo[1121] },  // Inst #2027 = SETNBC8
  { 2, &PPCDescs.OperandInfo[1119] },  // Inst #2026 = SETNBC
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #2025 = SETFLM
  { 2, &PPCDescs.OperandInfo[1121] },  // Inst #2024 = SETBCR8
  { 2, &PPCDescs.OperandInfo[1119] },  // Inst #2023 = SETBCR
  { 2, &PPCDescs.OperandInfo[1121] },  // Inst #2022 = SETBC8
  { 2, &PPCDescs.OperandInfo[1119] },  // Inst #2021 = SETBC
  { 2, &PPCDescs.OperandInfo[1117] },  // Inst #2020 = SETB8
  { 2, &PPCDescs.OperandInfo[1115] },  // Inst #2019 = SETB
  { 4, &PPCDescs.OperandInfo[1075] },  // Inst #2018 = SELECT_VSSRC
  { 4, &PPCDescs.OperandInfo[1111] },  // Inst #2017 = SELECT_VSRC
  { 4, &PPCDescs.OperandInfo[1079] },  // Inst #2016 = SELECT_VSFRC
  { 4, &PPCDescs.OperandInfo[1071] },  // Inst #2015 = SELECT_VRRC
  { 4, &PPCDescs.OperandInfo[1107] },  // Inst #2014 = SELECT_SPE4
  { 4, &PPCDescs.OperandInfo[1103] },  // Inst #2013 = SELECT_SPE
  { 4, &PPCDescs.OperandInfo[1099] },  // Inst #2012 = SELECT_QSRC
  { 4, &PPCDescs.OperandInfo[1095] },  // Inst #2011 = SELECT_QFRC
  { 4, &PPCDescs.OperandInfo[1091] },  // Inst #2010 = SELECT_QBRC
  { 4, &PPCDescs.OperandInfo[1087] },  // Inst #2009 = SELECT_I8
  { 4, &PPCDescs.OperandInfo[1083] },  // Inst #2008 = SELECT_I4
  { 4, &PPCDescs.OperandInfo[1079] },  // Inst #2007 = SELECT_F8
  { 4, &PPCDescs.OperandInfo[1075] },  // Inst #2006 = SELECT_F4
  { 4, &PPCDescs.OperandInfo[1071] },  // Inst #2005 = SELECT_F16
  { 5, &PPCDescs.OperandInfo[1021] },  // Inst #2004 = SELECT_CC_VSSRC
  { 5, &PPCDescs.OperandInfo[1066] },  // Inst #2003 = SELECT_CC_VSRC
  { 5, &PPCDescs.OperandInfo[1026] },  // Inst #2002 = SELECT_CC_VSFRC
  { 5, &PPCDescs.OperandInfo[1016] },  // Inst #2001 = SELECT_CC_VRRC
  { 5, &PPCDescs.OperandInfo[1061] },  // Inst #2000 = SELECT_CC_SPE4
  { 5, &PPCDescs.OperandInfo[1056] },  // Inst #1999 = SELECT_CC_SPE
  { 5, &PPCDescs.OperandInfo[1051] },  // Inst #1998 = SELECT_CC_QSRC
  { 5, &PPCDescs.OperandInfo[1046] },  // Inst #1997 = SELECT_CC_QFRC
  { 5, &PPCDescs.OperandInfo[1041] },  // Inst #1996 = SELECT_CC_QBRC
  { 5, &PPCDescs.OperandInfo[1036] },  // Inst #1995 = SELECT_CC_I8
  { 5, &PPCDescs.OperandInfo[1031] },  // Inst #1994 = SELECT_CC_I4
  { 5, &PPCDescs.OperandInfo[1026] },  // Inst #1993 = SELECT_CC_F8
  { 5, &PPCDescs.OperandInfo[1021] },  // Inst #1992 = SELECT_CC_F4
  { 5, &PPCDescs.OperandInfo[1016] },  // Inst #1991 = SELECT_CC_F16
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #1990 = SCV
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #1989 = SC
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1988 = ReadTB
  { 5, &PPCDescs.OperandInfo[1006] },  // Inst #1987 = RLWNM_rec
  { 5, &PPCDescs.OperandInfo[1011] },  // Inst #1986 = RLWNM8_rec
  { 5, &PPCDescs.OperandInfo[1011] },  // Inst #1985 = RLWNM8
  { 5, &PPCDescs.OperandInfo[1006] },  // Inst #1984 = RLWNM
  { 5, &PPCDescs.OperandInfo[996] },  // Inst #1983 = RLWINM_rec
  { 5, &PPCDescs.OperandInfo[1001] },  // Inst #1982 = RLWINM8_rec
  { 5, &PPCDescs.OperandInfo[1001] },  // Inst #1981 = RLWINM8
  { 5, &PPCDescs.OperandInfo[996] },  // Inst #1980 = RLWINM
  { 6, &PPCDescs.OperandInfo[984] },  // Inst #1979 = RLWIMI_rec
  { 6, &PPCDescs.OperandInfo[990] },  // Inst #1978 = RLWIMI8_rec
  { 6, &PPCDescs.OperandInfo[990] },  // Inst #1977 = RLWIMI8
  { 6, &PPCDescs.OperandInfo[984] },  // Inst #1976 = RLWIMI
  { 5, &PPCDescs.OperandInfo[979] },  // Inst #1975 = RLDIMI_rec
  { 5, &PPCDescs.OperandInfo[979] },  // Inst #1974 = RLDIMI
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #1973 = RLDIC_rec
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #1972 = RLDICR_rec
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #1971 = RLDICR_32
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #1970 = RLDICR
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #1969 = RLDICL_rec
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #1968 = RLDICL_32_rec
  { 4, &PPCDescs.OperandInfo[975] },  // Inst #1967 = RLDICL_32_64
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #1966 = RLDICL_32
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #1965 = RLDICL
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #1964 = RLDIC
  { 4, &PPCDescs.OperandInfo[971] },  // Inst #1963 = RLDCR_rec
  { 4, &PPCDescs.OperandInfo[971] },  // Inst #1962 = RLDCR
  { 4, &PPCDescs.OperandInfo[971] },  // Inst #1961 = RLDCL_rec
  { 4, &PPCDescs.OperandInfo[971] },  // Inst #1960 = RLDCL
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1959 = RFMCI
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1958 = RFID
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1957 = RFI
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #1956 = RFEBB
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1955 = RFDI
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1954 = RFCI
  { 3, &PPCDescs.OperandInfo[968] },  // Inst #1953 = RESTORE_WACC
  { 3, &PPCDescs.OperandInfo[965] },  // Inst #1952 = RESTORE_UACC
  { 3, &PPCDescs.OperandInfo[962] },  // Inst #1951 = RESTORE_QUADWORD
  { 3, &PPCDescs.OperandInfo[959] },  // Inst #1950 = RESTORE_CRBIT
  { 3, &PPCDescs.OperandInfo[956] },  // Inst #1949 = RESTORE_CR
  { 3, &PPCDescs.OperandInfo[953] },  // Inst #1948 = RESTORE_ACC
  { 3, &PPCDescs.OperandInfo[929] },  // Inst #1947 = QVSTFSXs
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1946 = QVSTFSXIA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1945 = QVSTFSXI
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1944 = QVSTFSXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1943 = QVSTFSX
  { 4, &PPCDescs.OperandInfo[945] },  // Inst #1942 = QVSTFSUXs
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1941 = QVSTFSUXIA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1940 = QVSTFSUXI
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1939 = QVSTFSUXA
  { 4, &PPCDescs.OperandInfo[949] },  // Inst #1938 = QVSTFSUX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1937 = QVSTFIWXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1936 = QVSTFIWX
  { 3, &PPCDescs.OperandInfo[936] },  // Inst #1935 = QVSTFDXb
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1934 = QVSTFDXIA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1933 = QVSTFDXI
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1932 = QVSTFDXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1931 = QVSTFDX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1930 = QVSTFDUXIA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1929 = QVSTFDUXI
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1928 = QVSTFDUXA
  { 4, &PPCDescs.OperandInfo[945] },  // Inst #1927 = QVSTFDUX
  { 3, &PPCDescs.OperandInfo[929] },  // Inst #1926 = QVSTFCSXs
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1925 = QVSTFCSXIA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1924 = QVSTFCSXI
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1923 = QVSTFCSXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1922 = QVSTFCSX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1921 = QVSTFCSUXIA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1920 = QVSTFCSUXI
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1919 = QVSTFCSUXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1918 = QVSTFCSUX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1917 = QVSTFCDXIA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1916 = QVSTFCDXI
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1915 = QVSTFCDXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1914 = QVSTFCDX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1913 = QVSTFCDUXIA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1912 = QVSTFCDUXI
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1911 = QVSTFCDUXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1910 = QVSTFCDUX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1909 = QVLPCRSX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1908 = QVLPCRDX
  { 2, &PPCDescs.OperandInfo[943] },  // Inst #1907 = QVLPCLSXint
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1906 = QVLPCLSX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1905 = QVLPCLDX
  { 3, &PPCDescs.OperandInfo[929] },  // Inst #1904 = QVLFSXs
  { 3, &PPCDescs.OperandInfo[936] },  // Inst #1903 = QVLFSXb
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1902 = QVLFSXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1901 = QVLFSX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1900 = QVLFSUXA
  { 4, &PPCDescs.OperandInfo[939] },  // Inst #1899 = QVLFSUX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1898 = QVLFIWZXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1897 = QVLFIWZX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1896 = QVLFIWAXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1895 = QVLFIWAX
  { 3, &PPCDescs.OperandInfo[936] },  // Inst #1894 = QVLFDXb
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1893 = QVLFDXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1892 = QVLFDX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1891 = QVLFDUXA
  { 4, &PPCDescs.OperandInfo[932] },  // Inst #1890 = QVLFDUX
  { 3, &PPCDescs.OperandInfo[929] },  // Inst #1889 = QVLFCSXs
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1888 = QVLFCSXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1887 = QVLFCSX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1886 = QVLFCSUXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1885 = QVLFCSUX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1884 = QVLFCDXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1883 = QVLFCDX
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1882 = QVLFCDUXA
  { 3, &PPCDescs.OperandInfo[926] },  // Inst #1881 = QVLFCDUX
  { 2, &PPCDescs.OperandInfo[924] },  // Inst #1880 = QVGPCI
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1879 = QVFXXNPMADDS
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1878 = QVFXXNPMADD
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1877 = QVFXXMADDS
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1876 = QVFXXMADD
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1875 = QVFXXCPNMADDS
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1874 = QVFXXCPNMADD
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1873 = QVFXMULS
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1872 = QVFXMUL
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1871 = QVFXMADDS
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1870 = QVFXMADD
  { 3, &PPCDescs.OperandInfo[895] },  // Inst #1869 = QVFTSTNANbs
  { 3, &PPCDescs.OperandInfo[892] },  // Inst #1868 = QVFTSTNANb
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1867 = QVFTSTNAN
  { 3, &PPCDescs.OperandInfo[887] },  // Inst #1866 = QVFSUBSs
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1865 = QVFSUBS
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1864 = QVFSUB
  { 4, &PPCDescs.OperandInfo[920] },  // Inst #1863 = QVFSELbs
  { 4, &PPCDescs.OperandInfo[916] },  // Inst #1862 = QVFSELbb
  { 4, &PPCDescs.OperandInfo[912] },  // Inst #1861 = QVFSELb
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1860 = QVFSEL
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1859 = QVFRSQRTESs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1858 = QVFRSQRTES
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1857 = QVFRSQRTE
  { 2, &PPCDescs.OperandInfo[910] },  // Inst #1856 = QVFRSPs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1855 = QVFRSP
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1854 = QVFRIZs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1853 = QVFRIZ
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1852 = QVFRIPs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1851 = QVFRIP
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1850 = QVFRINs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1849 = QVFRIN
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1848 = QVFRIMs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1847 = QVFRIM
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1846 = QVFRESs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1845 = QVFRES
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1844 = QVFRE
  { 4, &PPCDescs.OperandInfo[906] },  // Inst #1843 = QVFPERMs
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1842 = QVFPERM
  { 4, &PPCDescs.OperandInfo[902] },  // Inst #1841 = QVFNMSUBSs
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1840 = QVFNMSUBS
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1839 = QVFNMSUB
  { 4, &PPCDescs.OperandInfo[902] },  // Inst #1838 = QVFNMADDSs
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1837 = QVFNMADDS
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1836 = QVFNMADD
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1835 = QVFNEGs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1834 = QVFNEG
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1833 = QVFNABSs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1832 = QVFNABS
  { 3, &PPCDescs.OperandInfo[887] },  // Inst #1831 = QVFMULSs
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1830 = QVFMULS
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1829 = QVFMUL
  { 4, &PPCDescs.OperandInfo[902] },  // Inst #1828 = QVFMSUBSs
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1827 = QVFMSUBS
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1826 = QVFMSUB
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1825 = QVFMRs
  { 2, &PPCDescs.OperandInfo[890] },  // Inst #1824 = QVFMRb
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1823 = QVFMR
  { 4, &PPCDescs.OperandInfo[902] },  // Inst #1822 = QVFMADDSs
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1821 = QVFMADDS
  { 4, &PPCDescs.OperandInfo[898] },  // Inst #1820 = QVFMADD
  { 4, &PPCDescs.OperandInfo[863] },  // Inst #1819 = QVFLOGICALs
  { 4, &PPCDescs.OperandInfo[863] },  // Inst #1818 = QVFLOGICALb
  { 4, &PPCDescs.OperandInfo[859] },  // Inst #1817 = QVFLOGICAL
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1816 = QVFCTIWZ
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1815 = QVFCTIWUZ
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1814 = QVFCTIWU
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1813 = QVFCTIW
  { 2, &PPCDescs.OperandInfo[890] },  // Inst #1812 = QVFCTIDb
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1811 = QVFCTIDZ
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1810 = QVFCTIDUZ
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1809 = QVFCTIDU
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1808 = QVFCTID
  { 3, &PPCDescs.OperandInfo[887] },  // Inst #1807 = QVFCPSGNs
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1806 = QVFCPSGN
  { 3, &PPCDescs.OperandInfo[895] },  // Inst #1805 = QVFCMPLTbs
  { 3, &PPCDescs.OperandInfo[892] },  // Inst #1804 = QVFCMPLTb
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1803 = QVFCMPLT
  { 3, &PPCDescs.OperandInfo[895] },  // Inst #1802 = QVFCMPGTbs
  { 3, &PPCDescs.OperandInfo[892] },  // Inst #1801 = QVFCMPGTb
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1800 = QVFCMPGT
  { 3, &PPCDescs.OperandInfo[895] },  // Inst #1799 = QVFCMPEQbs
  { 3, &PPCDescs.OperandInfo[892] },  // Inst #1798 = QVFCMPEQb
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1797 = QVFCMPEQ
  { 2, &PPCDescs.OperandInfo[890] },  // Inst #1796 = QVFCFIDb
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1795 = QVFCFIDUS
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1794 = QVFCFIDU
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1793 = QVFCFIDS
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1792 = QVFCFID
  { 3, &PPCDescs.OperandInfo[887] },  // Inst #1791 = QVFADDSs
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1790 = QVFADDS
  { 3, &PPCDescs.OperandInfo[884] },  // Inst #1789 = QVFADD
  { 2, &PPCDescs.OperandInfo[882] },  // Inst #1788 = QVFABSs
  { 2, &PPCDescs.OperandInfo[880] },  // Inst #1787 = QVFABS
  { 3, &PPCDescs.OperandInfo[877] },  // Inst #1786 = QVESPLATIs
  { 3, &PPCDescs.OperandInfo[874] },  // Inst #1785 = QVESPLATIb
  { 3, &PPCDescs.OperandInfo[871] },  // Inst #1784 = QVESPLATI
  { 4, &PPCDescs.OperandInfo[867] },  // Inst #1783 = QVALIGNIs
  { 4, &PPCDescs.OperandInfo[863] },  // Inst #1782 = QVALIGNIb
  { 4, &PPCDescs.OperandInfo[859] },  // Inst #1781 = QVALIGNI
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1780 = PseudoEIEIO
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1779 = PS_SUM1o
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1778 = PS_SUM1
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1777 = PS_SUM0o
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1776 = PS_SUM0
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1775 = PS_SUBo
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1774 = PS_SUB
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1773 = PS_SELo
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1772 = PS_SEL
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1771 = PS_RSQRTEo
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1770 = PS_RSQRTE
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1769 = PS_RESo
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1768 = PS_RES
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1767 = PS_NMSUBo
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1766 = PS_NMSUB
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1765 = PS_NMADDo
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1764 = PS_NMADD
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1763 = PS_NEGo
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1762 = PS_NEG
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1761 = PS_NABSo
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1760 = PS_NABS
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1759 = PS_MULo
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1758 = PS_MULS1o
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1757 = PS_MULS1
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1756 = PS_MULS0o
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1755 = PS_MULS0
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1754 = PS_MUL
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1753 = PS_MSUBo
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1752 = PS_MSUB
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1751 = PS_MRo
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1750 = PS_MR
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1749 = PS_MERGE11o
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1748 = PS_MERGE11
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1747 = PS_MERGE10o
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1746 = PS_MERGE10
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1745 = PS_MERGE01o
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1744 = PS_MERGE01
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1743 = PS_MERGE00o
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1742 = PS_MERGE00
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1741 = PS_MADDo
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1740 = PS_MADDS1o
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1739 = PS_MADDS1
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1738 = PS_MADDS0o
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1737 = PS_MADDS0
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1736 = PS_MADD
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1735 = PS_DIVo
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1734 = PS_DIV
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #1733 = PS_CMPU1
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #1732 = PS_CMPU0
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #1731 = PS_CMPO1
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #1730 = PS_CMPO0
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1729 = PS_ADDo
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1728 = PS_ADD
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1727 = PS_ABSo
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1726 = PS_ABS
  { 3, &PPCDescs.OperandInfo[749] },  // Inst #1725 = PSTXVpc
  { 2, &PPCDescs.OperandInfo[621] },  // Inst #1724 = PSTXVonlypc
  { 3, &PPCDescs.OperandInfo[738] },  // Inst #1723 = PSTXVnopc
  { 3, &PPCDescs.OperandInfo[746] },  // Inst #1722 = PSTXVPpc
  { 2, &PPCDescs.OperandInfo[744] },  // Inst #1721 = PSTXVPonlypc
  { 3, &PPCDescs.OperandInfo[741] },  // Inst #1720 = PSTXVPnopc
  { 3, &PPCDescs.OperandInfo[741] },  // Inst #1719 = PSTXVP
  { 3, &PPCDescs.OperandInfo[738] },  // Inst #1718 = PSTXV
  { 3, &PPCDescs.OperandInfo[735] },  // Inst #1717 = PSTXSSPpc
  { 2, &PPCDescs.OperandInfo[733] },  // Inst #1716 = PSTXSSPonlypc
  { 3, &PPCDescs.OperandInfo[730] },  // Inst #1715 = PSTXSSPnopc
  { 3, &PPCDescs.OperandInfo[730] },  // Inst #1714 = PSTXSSP
  { 3, &PPCDescs.OperandInfo[735] },  // Inst #1713 = PSTXSDpc
  { 2, &PPCDescs.OperandInfo[733] },  // Inst #1712 = PSTXSDonlypc
  { 3, &PPCDescs.OperandInfo[730] },  // Inst #1711 = PSTXSDnopc
  { 3, &PPCDescs.OperandInfo[730] },  // Inst #1710 = PSTXSD
  { 3, &PPCDescs.OperandInfo[713] },  // Inst #1709 = PSTWpc
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1708 = PSTWonlypc
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1707 = PSTWnopc
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1706 = PSTW8pc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1705 = PSTW8onlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1704 = PSTW8nopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1703 = PSTW8
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1702 = PSTW
  { 3, &PPCDescs.OperandInfo[713] },  // Inst #1701 = PSTHpc
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1700 = PSTHonlypc
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1699 = PSTHnopc
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1698 = PSTH8pc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1697 = PSTH8onlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1696 = PSTH8nopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1695 = PSTH8
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1694 = PSTH
  { 3, &PPCDescs.OperandInfo[727] },  // Inst #1693 = PSTFSpc
  { 2, &PPCDescs.OperandInfo[725] },  // Inst #1692 = PSTFSonlypc
  { 3, &PPCDescs.OperandInfo[722] },  // Inst #1691 = PSTFSnopc
  { 3, &PPCDescs.OperandInfo[722] },  // Inst #1690 = PSTFS
  { 3, &PPCDescs.OperandInfo[719] },  // Inst #1689 = PSTFDpc
  { 2, &PPCDescs.OperandInfo[650] },  // Inst #1688 = PSTFDonlypc
  { 3, &PPCDescs.OperandInfo[716] },  // Inst #1687 = PSTFDnopc
  { 3, &PPCDescs.OperandInfo[716] },  // Inst #1686 = PSTFD
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1685 = PSTDpc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1684 = PSTDonlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1683 = PSTDnopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1682 = PSTD
  { 3, &PPCDescs.OperandInfo[713] },  // Inst #1681 = PSTBpc
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1680 = PSTBonlypc
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1679 = PSTBnopc
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1678 = PSTB8pc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1677 = PSTB8onlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1676 = PSTB8nopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1675 = PSTB8
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1674 = PSTB
  { 5, &PPCDescs.OperandInfo[854] },  // Inst #1673 = PSQ_STX
  { 5, &PPCDescs.OperandInfo[854] },  // Inst #1672 = PSQ_STUX
  { 5, &PPCDescs.OperandInfo[849] },  // Inst #1671 = PSQ_STU
  { 5, &PPCDescs.OperandInfo[849] },  // Inst #1670 = PSQ_ST
  { 5, &PPCDescs.OperandInfo[854] },  // Inst #1669 = PSQ_LX
  { 5, &PPCDescs.OperandInfo[854] },  // Inst #1668 = PSQ_LUX
  { 5, &PPCDescs.OperandInfo[849] },  // Inst #1667 = PSQ_LU
  { 5, &PPCDescs.OperandInfo[849] },  // Inst #1666 = PSQ_L
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1665 = PSC_DCBZL
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #1664 = PROBED_STACKALLOC_64
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #1663 = PROBED_STACKALLOC_32
  { 4, &PPCDescs.OperandInfo[441] },  // Inst #1662 = PROBED_ALLOCA_64
  { 4, &PPCDescs.OperandInfo[437] },  // Inst #1661 = PROBED_ALLOCA_32
  { 5, &PPCDescs.OperandInfo[844] },  // Inst #1660 = PREPARE_PROBED_ALLOCA_NEGSIZE_SAME_REG_64
  { 5, &PPCDescs.OperandInfo[839] },  // Inst #1659 = PREPARE_PROBED_ALLOCA_NEGSIZE_SAME_REG_32
  { 5, &PPCDescs.OperandInfo[834] },  // Inst #1658 = PREPARE_PROBED_ALLOCA_64
  { 5, &PPCDescs.OperandInfo[829] },  // Inst #1657 = PREPARE_PROBED_ALLOCA_32
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1656 = PPC32PICGOT
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1655 = PPC32GOT
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1654 = POPCNTW
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1653 = POPCNTD
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1652 = POPCNTB8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1651 = POPCNTB
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1650 = PMXVI8GER4WSPP
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1649 = PMXVI8GER4WPP
  { 6, &PPCDescs.OperandInfo[765] },  // Inst #1648 = PMXVI8GER4W
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1647 = PMXVI8GER4SPP
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1646 = PMXVI8GER4PP
  { 6, &PPCDescs.OperandInfo[752] },  // Inst #1645 = PMXVI8GER4
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1644 = PMXVI4GER8WPP
  { 6, &PPCDescs.OperandInfo[765] },  // Inst #1643 = PMXVI4GER8W
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1642 = PMXVI4GER8PP
  { 6, &PPCDescs.OperandInfo[752] },  // Inst #1641 = PMXVI4GER8
  { 7, &PPCDescs.OperandInfo[822] },  // Inst #1640 = PMXVI16GER2WPP
  { 6, &PPCDescs.OperandInfo[765] },  // Inst #1639 = PMXVI16GER2W
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1638 = PMXVI16GER2SWPP
  { 6, &PPCDescs.OperandInfo[765] },  // Inst #1637 = PMXVI16GER2SW
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1636 = PMXVI16GER2SPP
  { 6, &PPCDescs.OperandInfo[752] },  // Inst #1635 = PMXVI16GER2S
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1634 = PMXVI16GER2PP
  { 6, &PPCDescs.OperandInfo[752] },  // Inst #1633 = PMXVI16GER2
  { 6, &PPCDescs.OperandInfo[816] },  // Inst #1632 = PMXVF64GERWPP
  { 6, &PPCDescs.OperandInfo[816] },  // Inst #1631 = PMXVF64GERWPN
  { 6, &PPCDescs.OperandInfo[816] },  // Inst #1630 = PMXVF64GERWNP
  { 6, &PPCDescs.OperandInfo[816] },  // Inst #1629 = PMXVF64GERWNN
  { 5, &PPCDescs.OperandInfo[811] },  // Inst #1628 = PMXVF64GERW
  { 6, &PPCDescs.OperandInfo[805] },  // Inst #1627 = PMXVF64GERPP
  { 6, &PPCDescs.OperandInfo[805] },  // Inst #1626 = PMXVF64GERPN
  { 6, &PPCDescs.OperandInfo[805] },  // Inst #1625 = PMXVF64GERNP
  { 6, &PPCDescs.OperandInfo[805] },  // Inst #1624 = PMXVF64GERNN
  { 5, &PPCDescs.OperandInfo[800] },  // Inst #1623 = PMXVF64GER
  { 6, &PPCDescs.OperandInfo[794] },  // Inst #1622 = PMXVF32GERWPP
  { 6, &PPCDescs.OperandInfo[794] },  // Inst #1621 = PMXVF32GERWPN
  { 6, &PPCDescs.OperandInfo[794] },  // Inst #1620 = PMXVF32GERWNP
  { 6, &PPCDescs.OperandInfo[794] },  // Inst #1619 = PMXVF32GERWNN
  { 5, &PPCDescs.OperandInfo[789] },  // Inst #1618 = PMXVF32GERW
  { 6, &PPCDescs.OperandInfo[783] },  // Inst #1617 = PMXVF32GERPP
  { 6, &PPCDescs.OperandInfo[783] },  // Inst #1616 = PMXVF32GERPN
  { 6, &PPCDescs.OperandInfo[783] },  // Inst #1615 = PMXVF32GERNP
  { 6, &PPCDescs.OperandInfo[783] },  // Inst #1614 = PMXVF32GERNN
  { 5, &PPCDescs.OperandInfo[778] },  // Inst #1613 = PMXVF32GER
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1612 = PMXVF16GER2WPP
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1611 = PMXVF16GER2WPN
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1610 = PMXVF16GER2WNP
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1609 = PMXVF16GER2WNN
  { 6, &PPCDescs.OperandInfo[765] },  // Inst #1608 = PMXVF16GER2W
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1607 = PMXVF16GER2PP
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1606 = PMXVF16GER2PN
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1605 = PMXVF16GER2NP
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1604 = PMXVF16GER2NN
  { 6, &PPCDescs.OperandInfo[752] },  // Inst #1603 = PMXVF16GER2
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1602 = PMXVBF16GER2WPP
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1601 = PMXVBF16GER2WPN
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1600 = PMXVBF16GER2WNP
  { 7, &PPCDescs.OperandInfo[771] },  // Inst #1599 = PMXVBF16GER2WNN
  { 6, &PPCDescs.OperandInfo[765] },  // Inst #1598 = PMXVBF16GER2W
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1597 = PMXVBF16GER2PP
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1596 = PMXVBF16GER2PN
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1595 = PMXVBF16GER2NP
  { 7, &PPCDescs.OperandInfo[758] },  // Inst #1594 = PMXVBF16GER2NN
  { 6, &PPCDescs.OperandInfo[752] },  // Inst #1593 = PMXVBF16GER2
  { 3, &PPCDescs.OperandInfo[749] },  // Inst #1592 = PLXVpc
  { 2, &PPCDescs.OperandInfo[621] },  // Inst #1591 = PLXVonlypc
  { 3, &PPCDescs.OperandInfo[738] },  // Inst #1590 = PLXVnopc
  { 3, &PPCDescs.OperandInfo[746] },  // Inst #1589 = PLXVPpc
  { 2, &PPCDescs.OperandInfo[744] },  // Inst #1588 = PLXVPonlypc
  { 3, &PPCDescs.OperandInfo[741] },  // Inst #1587 = PLXVPnopc
  { 3, &PPCDescs.OperandInfo[741] },  // Inst #1586 = PLXVP
  { 3, &PPCDescs.OperandInfo[738] },  // Inst #1585 = PLXV
  { 3, &PPCDescs.OperandInfo[735] },  // Inst #1584 = PLXSSPpc
  { 2, &PPCDescs.OperandInfo[733] },  // Inst #1583 = PLXSSPonlypc
  { 3, &PPCDescs.OperandInfo[730] },  // Inst #1582 = PLXSSPnopc
  { 3, &PPCDescs.OperandInfo[730] },  // Inst #1581 = PLXSSP
  { 3, &PPCDescs.OperandInfo[735] },  // Inst #1580 = PLXSDpc
  { 2, &PPCDescs.OperandInfo[733] },  // Inst #1579 = PLXSDonlypc
  { 3, &PPCDescs.OperandInfo[730] },  // Inst #1578 = PLXSDnopc
  { 3, &PPCDescs.OperandInfo[730] },  // Inst #1577 = PLXSD
  { 3, &PPCDescs.OperandInfo[713] },  // Inst #1576 = PLWZpc
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1575 = PLWZonlypc
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1574 = PLWZnopc
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1573 = PLWZ8pc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1572 = PLWZ8onlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1571 = PLWZ8nopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1570 = PLWZ8
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1569 = PLWZ
  { 3, &PPCDescs.OperandInfo[713] },  // Inst #1568 = PLWApc
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1567 = PLWAonlypc
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1566 = PLWAnopc
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1565 = PLWA8pc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1564 = PLWA8onlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1563 = PLWA8nopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1562 = PLWA8
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1561 = PLWA
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1560 = PLI8
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1559 = PLI
  { 3, &PPCDescs.OperandInfo[713] },  // Inst #1558 = PLHZpc
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1557 = PLHZonlypc
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1556 = PLHZnopc
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1555 = PLHZ8pc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1554 = PLHZ8onlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1553 = PLHZ8nopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1552 = PLHZ8
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1551 = PLHZ
  { 3, &PPCDescs.OperandInfo[713] },  // Inst #1550 = PLHApc
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1549 = PLHAonlypc
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1548 = PLHAnopc
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1547 = PLHA8pc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1546 = PLHA8onlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1545 = PLHA8nopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1544 = PLHA8
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1543 = PLHA
  { 3, &PPCDescs.OperandInfo[727] },  // Inst #1542 = PLFSpc
  { 2, &PPCDescs.OperandInfo[725] },  // Inst #1541 = PLFSonlypc
  { 3, &PPCDescs.OperandInfo[722] },  // Inst #1540 = PLFSnopc
  { 3, &PPCDescs.OperandInfo[722] },  // Inst #1539 = PLFS
  { 3, &PPCDescs.OperandInfo[719] },  // Inst #1538 = PLFDpc
  { 2, &PPCDescs.OperandInfo[650] },  // Inst #1537 = PLFDonlypc
  { 3, &PPCDescs.OperandInfo[716] },  // Inst #1536 = PLFDnopc
  { 3, &PPCDescs.OperandInfo[716] },  // Inst #1535 = PLFD
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1534 = PLDpc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1533 = PLDonlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1532 = PLDnopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1531 = PLD
  { 3, &PPCDescs.OperandInfo[713] },  // Inst #1530 = PLBZpc
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1529 = PLBZonlypc
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1528 = PLBZnopc
  { 3, &PPCDescs.OperandInfo[710] },  // Inst #1527 = PLBZ8pc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1526 = PLBZ8onlypc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1525 = PLBZ8nopc
  { 3, &PPCDescs.OperandInfo[707] },  // Inst #1524 = PLBZ8
  { 3, &PPCDescs.OperandInfo[704] },  // Inst #1523 = PLBZ
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1522 = PLApc
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1521 = PLA8pc
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #1520 = PLA8
  { 3, &PPCDescs.OperandInfo[233] },  // Inst #1519 = PLA
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1518 = PEXTD
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1517 = PDEPD
  { 3, &PPCDescs.OperandInfo[646] },  // Inst #1516 = PADDIpc
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #1515 = PADDIdtprel
  { 3, &PPCDescs.OperandInfo[701] },  // Inst #1514 = PADDI8pc
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #1513 = PADDI8
  { 3, &PPCDescs.OperandInfo[233] },  // Inst #1512 = PADDI
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1511 = OR_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #1510 = ORIS8
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #1509 = ORIS
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #1508 = ORI8
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #1507 = ORI
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1506 = ORC_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1505 = ORC8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1504 = ORC8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1503 = ORC
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1502 = OR8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1501 = OR8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1500 = OR
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1499 = NOR_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1498 = NOR8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1497 = NOR8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1496 = NOR
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1495 = NOP_GT_PWR7
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1494 = NOP_GT_PWR6
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1493 = NOP
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1492 = NEG_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1491 = NEGO_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1490 = NEGO
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1489 = NEG8_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1488 = NEG8O_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1487 = NEG8O
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1486 = NEG8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1485 = NEG
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1484 = NAP
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1483 = NAND_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1482 = NAND8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1481 = NAND8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1480 = NAND
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1479 = MovePCtoLR8
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1478 = MovePCtoLR
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1477 = MoveGOTtoLR
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1476 = MULLW_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1475 = MULLWO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1474 = MULLWO
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1473 = MULLW
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #1472 = MULLI8
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #1471 = MULLI
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1470 = MULLD_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1469 = MULLDO_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1468 = MULLDO
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1467 = MULLD
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1466 = MULHW_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1465 = MULHWU_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1464 = MULHWU
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1463 = MULHW
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1462 = MULHD_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1461 = MULHDU_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1460 = MULHDU
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1459 = MULHD
  { 2, &PPCDescs.OperandInfo[699] },  // Inst #1458 = MTVSRWZ
  { 2, &PPCDescs.OperandInfo[688] },  // Inst #1457 = MTVSRWS
  { 2, &PPCDescs.OperandInfo[690] },  // Inst #1456 = MTVSRWM
  { 2, &PPCDescs.OperandInfo[699] },  // Inst #1455 = MTVSRWA
  { 2, &PPCDescs.OperandInfo[690] },  // Inst #1454 = MTVSRQM
  { 2, &PPCDescs.OperandInfo[690] },  // Inst #1453 = MTVSRHM
  { 2, &PPCDescs.OperandInfo[690] },  // Inst #1452 = MTVSRDM
  { 3, &PPCDescs.OperandInfo[696] },  // Inst #1451 = MTVSRDD
  { 2, &PPCDescs.OperandInfo[694] },  // Inst #1450 = MTVSRD
  { 2, &PPCDescs.OperandInfo[692] },  // Inst #1449 = MTVSRBMI
  { 2, &PPCDescs.OperandInfo[690] },  // Inst #1448 = MTVSRBM
  { 1, &PPCDescs.OperandInfo[662] },  // Inst #1447 = MTVSCR
  { 2, &PPCDescs.OperandInfo[688] },  // Inst #1446 = MTVRWZ
  { 2, &PPCDescs.OperandInfo[688] },  // Inst #1445 = MTVRWA
  { 2, &PPCDescs.OperandInfo[686] },  // Inst #1444 = MTVRSAVEv
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1443 = MTVRSAVE
  { 2, &PPCDescs.OperandInfo[684] },  // Inst #1442 = MTVRD
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1441 = MTUDSCR
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1440 = MTSRIN
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1439 = MTSR
  { 2, &PPCDescs.OperandInfo[669] },  // Inst #1438 = MTSPR8
  { 2, &PPCDescs.OperandInfo[667] },  // Inst #1437 = MTSPR
  { 2, &PPCDescs.OperandInfo[667] },  // Inst #1436 = MTPMR
  { 2, &PPCDescs.OperandInfo[682] },  // Inst #1435 = MTOCRF8
  { 2, &PPCDescs.OperandInfo[680] },  // Inst #1434 = MTOCRF
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1433 = MTMSRD
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1432 = MTMSR
  { 1, &PPCDescs.OperandInfo[160] },  // Inst #1431 = MTLR8
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1430 = MTLR
  { 2, &PPCDescs.OperandInfo[678] },  // Inst #1429 = MTFSFb
  { 4, &PPCDescs.OperandInfo[671] },  // Inst #1428 = MTFSF_rec
  { 2, &PPCDescs.OperandInfo[21] },  // Inst #1427 = MTFSFIb
  { 3, &PPCDescs.OperandInfo[675] },  // Inst #1426 = MTFSFI_rec
  { 3, &PPCDescs.OperandInfo[675] },  // Inst #1425 = MTFSFI
  { 4, &PPCDescs.OperandInfo[671] },  // Inst #1424 = MTFSF
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #1423 = MTFSB1
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #1422 = MTFSB0
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1421 = MTDCR
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1420 = MTCTRloop
  { 1, &PPCDescs.OperandInfo[160] },  // Inst #1419 = MTCTR8loop
  { 1, &PPCDescs.OperandInfo[160] },  // Inst #1418 = MTCTR8
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1417 = MTCTR
  { 2, &PPCDescs.OperandInfo[669] },  // Inst #1416 = MTCRF8
  { 2, &PPCDescs.OperandInfo[667] },  // Inst #1415 = MTCRF
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1414 = MSYNC
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1413 = MSGSYNC
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1412 = MODUW
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1411 = MODUD
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1410 = MODSW
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1409 = MODSD
  { 2, &PPCDescs.OperandInfo[665] },  // Inst #1408 = MFVSRWZ
  { 2, &PPCDescs.OperandInfo[656] },  // Inst #1407 = MFVSRLD
  { 2, &PPCDescs.OperandInfo[663] },  // Inst #1406 = MFVSRD
  { 1, &PPCDescs.OperandInfo[662] },  // Inst #1405 = MFVSCR
  { 2, &PPCDescs.OperandInfo[660] },  // Inst #1404 = MFVRWZ
  { 2, &PPCDescs.OperandInfo[658] },  // Inst #1403 = MFVRSAVEv
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1402 = MFVRSAVE
  { 2, &PPCDescs.OperandInfo[656] },  // Inst #1401 = MFVRD
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1400 = MFUDSCR
  { 1, &PPCDescs.OperandInfo[160] },  // Inst #1399 = MFTB8
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1398 = MFTB
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1397 = MFSRIN
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1396 = MFSR
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #1395 = MFSPR8
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1394 = MFSPR
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1393 = MFPMR
  { 2, &PPCDescs.OperandInfo[654] },  // Inst #1392 = MFOCRF8
  { 2, &PPCDescs.OperandInfo[652] },  // Inst #1391 = MFOCRF
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1390 = MFMSR
  { 1, &PPCDescs.OperandInfo[160] },  // Inst #1389 = MFLR8
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1388 = MFLR
  { 1, &PPCDescs.OperandInfo[649] },  // Inst #1387 = MFFS_rec
  { 1, &PPCDescs.OperandInfo[649] },  // Inst #1386 = MFFSL
  { 2, &PPCDescs.OperandInfo[650] },  // Inst #1385 = MFFSCRNI
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1384 = MFFSCRN
  { 1, &PPCDescs.OperandInfo[649] },  // Inst #1383 = MFFSCE
  { 2, &PPCDescs.OperandInfo[650] },  // Inst #1382 = MFFSCDRNI
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1381 = MFFSCDRN
  { 1, &PPCDescs.OperandInfo[649] },  // Inst #1380 = MFFS
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #1379 = MFDCR
  { 1, &PPCDescs.OperandInfo[160] },  // Inst #1378 = MFCTR8
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1377 = MFCTR
  { 1, &PPCDescs.OperandInfo[160] },  // Inst #1376 = MFCR8
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1375 = MFCR
  { 3, &PPCDescs.OperandInfo[646] },  // Inst #1374 = MFBHRBE
  { 1, &PPCDescs.OperandInfo[645] },  // Inst #1373 = MCRXRX
  { 2, &PPCDescs.OperandInfo[643] },  // Inst #1372 = MCRFS
  { 2, &PPCDescs.OperandInfo[643] },  // Inst #1371 = MCRF
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #1370 = MBAR
  { 4, &PPCDescs.OperandInfo[635] },  // Inst #1369 = MADDLD8
  { 4, &PPCDescs.OperandInfo[639] },  // Inst #1368 = MADDLD
  { 4, &PPCDescs.OperandInfo[635] },  // Inst #1367 = MADDHDU
  { 4, &PPCDescs.OperandInfo[635] },  // Inst #1366 = MADDHD
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1365 = LXVX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1364 = LXVWSX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1363 = LXVW4X
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1362 = LXVRWX
  { 3, &PPCDescs.OperandInfo[623] },  // Inst #1361 = LXVRLL
  { 3, &PPCDescs.OperandInfo[623] },  // Inst #1360 = LXVRL
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1359 = LXVRHX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1358 = LXVRDX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1357 = LXVRBX
  { 3, &PPCDescs.OperandInfo[632] },  // Inst #1356 = LXVPX
  { 3, &PPCDescs.OperandInfo[629] },  // Inst #1355 = LXVPRLL
  { 3, &PPCDescs.OperandInfo[629] },  // Inst #1354 = LXVPRL
  { 3, &PPCDescs.OperandInfo[626] },  // Inst #1353 = LXVP
  { 3, &PPCDescs.OperandInfo[623] },  // Inst #1352 = LXVLL
  { 3, &PPCDescs.OperandInfo[623] },  // Inst #1351 = LXVL
  { 2, &PPCDescs.OperandInfo[621] },  // Inst #1350 = LXVKQ
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1349 = LXVH8X
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1348 = LXVDSX
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1347 = LXVD2X
  { 3, &PPCDescs.OperandInfo[618] },  // Inst #1346 = LXVB16X
  { 3, &PPCDescs.OperandInfo[615] },  // Inst #1345 = LXV
  { 3, &PPCDescs.OperandInfo[207] },  // Inst #1344 = LXSSPX
  { 3, &PPCDescs.OperandInfo[612] },  // Inst #1343 = LXSSP
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #1342 = LXSIWZX
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #1341 = LXSIWAX
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #1340 = LXSIHZX
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #1339 = LXSIBZX
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #1338 = LXSDX
  { 3, &PPCDescs.OperandInfo[612] },  // Inst #1337 = LXSD
  { 3, &PPCDescs.OperandInfo[609] },  // Inst #1336 = LWZtocL
  { 3, &PPCDescs.OperandInfo[247] },  // Inst #1335 = LWZtoc
  { 3, &PPCDescs.OperandInfo[554] },  // Inst #1334 = LWZXTLS_32
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1333 = LWZXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1332 = LWZXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1331 = LWZX8
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1330 = LWZX
  { 4, &PPCDescs.OperandInfo[544] },  // Inst #1329 = LWZUX8
  { 4, &PPCDescs.OperandInfo[540] },  // Inst #1328 = LWZUX
  { 4, &PPCDescs.OperandInfo[536] },  // Inst #1327 = LWZU8
  { 4, &PPCDescs.OperandInfo[532] },  // Inst #1326 = LWZU
  { 3, &PPCDescs.OperandInfo[529] },  // Inst #1325 = LWZCIX
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1324 = LWZ8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1323 = LWZ
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1322 = LWEPX
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1321 = LWBRX8
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1320 = LWBRX
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1319 = LWA_32
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1318 = LWAX_32
  { 3, &PPCDescs.OperandInfo[554] },  // Inst #1317 = LWAXTLS_32
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1316 = LWAXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1315 = LWAXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1314 = LWAX
  { 4, &PPCDescs.OperandInfo[544] },  // Inst #1313 = LWAUX
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #1312 = LWAT
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1311 = LWARXL
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1310 = LWARX
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1309 = LWA
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #1308 = LVXL
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #1307 = LVX
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #1306 = LVSR
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #1305 = LVSL
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #1304 = LVEWX
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #1303 = LVEHX
  { 3, &PPCDescs.OperandInfo[606] },  // Inst #1302 = LVEBX
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #1301 = LSWI
  { 3, &PPCDescs.OperandInfo[603] },  // Inst #1300 = LQX_PSEUDO
  { 3, &PPCDescs.OperandInfo[603] },  // Inst #1299 = LQARXL
  { 3, &PPCDescs.OperandInfo[603] },  // Inst #1298 = LQARX
  { 3, &PPCDescs.OperandInfo[600] },  // Inst #1297 = LQ
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1296 = LMW
  { 3, &PPCDescs.OperandInfo[554] },  // Inst #1295 = LHZXTLS_32
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1294 = LHZXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1293 = LHZXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1292 = LHZX8
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1291 = LHZX
  { 4, &PPCDescs.OperandInfo[544] },  // Inst #1290 = LHZUX8
  { 4, &PPCDescs.OperandInfo[540] },  // Inst #1289 = LHZUX
  { 4, &PPCDescs.OperandInfo[536] },  // Inst #1288 = LHZU8
  { 4, &PPCDescs.OperandInfo[532] },  // Inst #1287 = LHZU
  { 3, &PPCDescs.OperandInfo[529] },  // Inst #1286 = LHZCIX
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1285 = LHZ8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1284 = LHZ
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1283 = LHEPX
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1282 = LHBRX8
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1281 = LHBRX
  { 3, &PPCDescs.OperandInfo[554] },  // Inst #1280 = LHAXTLS_32
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1279 = LHAXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1278 = LHAXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1277 = LHAX8
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1276 = LHAX
  { 4, &PPCDescs.OperandInfo[544] },  // Inst #1275 = LHAUX8
  { 4, &PPCDescs.OperandInfo[540] },  // Inst #1274 = LHAUX
  { 4, &PPCDescs.OperandInfo[536] },  // Inst #1273 = LHAU8
  { 4, &PPCDescs.OperandInfo[532] },  // Inst #1272 = LHAU
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1271 = LHARXL
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1270 = LHARX
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1269 = LHA8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1268 = LHA
  { 3, &PPCDescs.OperandInfo[597] },  // Inst #1267 = LFSXTLS_
  { 3, &PPCDescs.OperandInfo[597] },  // Inst #1266 = LFSXTLS
  { 3, &PPCDescs.OperandInfo[594] },  // Inst #1265 = LFSX
  { 4, &PPCDescs.OperandInfo[590] },  // Inst #1264 = LFSUX
  { 4, &PPCDescs.OperandInfo[586] },  // Inst #1263 = LFSU
  { 3, &PPCDescs.OperandInfo[583] },  // Inst #1262 = LFS
  { 3, &PPCDescs.OperandInfo[569] },  // Inst #1261 = LFIWZX
  { 3, &PPCDescs.OperandInfo[569] },  // Inst #1260 = LFIWAX
  { 3, &PPCDescs.OperandInfo[580] },  // Inst #1259 = LFDXTLS_
  { 3, &PPCDescs.OperandInfo[580] },  // Inst #1258 = LFDXTLS
  { 3, &PPCDescs.OperandInfo[569] },  // Inst #1257 = LFDX
  { 4, &PPCDescs.OperandInfo[576] },  // Inst #1256 = LFDUX
  { 4, &PPCDescs.OperandInfo[572] },  // Inst #1255 = LFDU
  { 3, &PPCDescs.OperandInfo[569] },  // Inst #1254 = LFDEPX
  { 3, &PPCDescs.OperandInfo[566] },  // Inst #1253 = LFD
  { 3, &PPCDescs.OperandInfo[250] },  // Inst #1252 = LDtocL
  { 3, &PPCDescs.OperandInfo[563] },  // Inst #1251 = LDtocJTI
  { 3, &PPCDescs.OperandInfo[563] },  // Inst #1250 = LDtocCPT
  { 3, &PPCDescs.OperandInfo[563] },  // Inst #1249 = LDtocBA
  { 3, &PPCDescs.OperandInfo[563] },  // Inst #1248 = LDtoc
  { 3, &PPCDescs.OperandInfo[560] },  // Inst #1247 = LDgotTprelL32
  { 3, &PPCDescs.OperandInfo[557] },  // Inst #1246 = LDgotTprelL
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1245 = LDXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1244 = LDXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1243 = LDX
  { 4, &PPCDescs.OperandInfo[544] },  // Inst #1242 = LDUX
  { 4, &PPCDescs.OperandInfo[536] },  // Inst #1241 = LDU
  { 3, &PPCDescs.OperandInfo[529] },  // Inst #1240 = LDCIX
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1239 = LDBRX
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #1238 = LDAT
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1237 = LDARXL
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1236 = LDARX
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1235 = LD
  { 3, &PPCDescs.OperandInfo[554] },  // Inst #1234 = LBZXTLS_32
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1233 = LBZXTLS_
  { 3, &PPCDescs.OperandInfo[551] },  // Inst #1232 = LBZXTLS
  { 3, &PPCDescs.OperandInfo[548] },  // Inst #1231 = LBZX8
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1230 = LBZX
  { 4, &PPCDescs.OperandInfo[544] },  // Inst #1229 = LBZUX8
  { 4, &PPCDescs.OperandInfo[540] },  // Inst #1228 = LBZUX
  { 4, &PPCDescs.OperandInfo[536] },  // Inst #1227 = LBZU8
  { 4, &PPCDescs.OperandInfo[532] },  // Inst #1226 = LBZU
  { 3, &PPCDescs.OperandInfo[529] },  // Inst #1225 = LBZCIX
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1224 = LBZ8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1223 = LBZ
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1222 = LBEPX
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1221 = LBARXL
  { 3, &PPCDescs.OperandInfo[526] },  // Inst #1220 = LBARX
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #1219 = LA8
  { 3, &PPCDescs.OperandInfo[233] },  // Inst #1218 = LA
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1217 = ISYNC
  { 4, &PPCDescs.OperandInfo[522] },  // Inst #1216 = ISEL8
  { 4, &PPCDescs.OperandInfo[518] },  // Inst #1215 = ISEL
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1214 = ICCCI
  { 3, &PPCDescs.OperandInfo[336] },  // Inst #1213 = ICBTLS
  { 3, &PPCDescs.OperandInfo[336] },  // Inst #1212 = ICBT
  { 3, &PPCDescs.OperandInfo[336] },  // Inst #1211 = ICBLQ
  { 3, &PPCDescs.OperandInfo[336] },  // Inst #1210 = ICBLC
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #1209 = ICBIEP
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #1208 = ICBI
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1207 = HRFID
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1206 = HASHSTP8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1205 = HASHSTP
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1204 = HASHST8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1203 = HASHST
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1202 = HASHCHKP8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1201 = HASHCHKP
  { 3, &PPCDescs.OperandInfo[515] },  // Inst #1200 = HASHCHK8
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #1199 = HASHCHK
  { 3, &PPCDescs.OperandInfo[222] },  // Inst #1198 = GETtlsldADDRPCREL
  { 3, &PPCDescs.OperandInfo[213] },  // Inst #1197 = GETtlsldADDR32
  { 3, &PPCDescs.OperandInfo[222] },  // Inst #1196 = GETtlsldADDR
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #1195 = GETtlsTpointer32AIX
  { 3, &PPCDescs.OperandInfo[222] },  // Inst #1194 = GETtlsADDRPCREL
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #1193 = GETtlsADDR64AIX
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #1192 = GETtlsADDR32AIX
  { 3, &PPCDescs.OperandInfo[213] },  // Inst #1191 = GETtlsADDR32
  { 3, &PPCDescs.OperandInfo[222] },  // Inst #1190 = GETtlsADDR
  { 2, &PPCDescs.OperandInfo[513] },  // Inst #1189 = FTSQRT
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #1188 = FTDIV
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1187 = FSUB_rec
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1186 = FSUBS_rec
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1185 = FSUBS
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1184 = FSUB
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1183 = FSQRT_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1182 = FSQRTS_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1181 = FSQRTS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1180 = FSQRT
  { 4, &PPCDescs.OperandInfo[509] },  // Inst #1179 = FSELS_rec
  { 4, &PPCDescs.OperandInfo[509] },  // Inst #1178 = FSELS
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1177 = FSELD_rec
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1176 = FSELD
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1175 = FRSQRTE_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1174 = FRSQRTES_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1173 = FRSQRTES
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1172 = FRSQRTE
  { 2, &PPCDescs.OperandInfo[496] },  // Inst #1171 = FRSP_rec
  { 2, &PPCDescs.OperandInfo[496] },  // Inst #1170 = FRSP
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1169 = FRIZS_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1168 = FRIZS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1167 = FRIZD_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1166 = FRIZD
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1165 = FRIPS_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1164 = FRIPS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1163 = FRIPD_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1162 = FRIPD
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1161 = FRINS_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1160 = FRINS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1159 = FRIND_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1158 = FRIND
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1157 = FRIMS_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1156 = FRIMS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1155 = FRIMD_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1154 = FRIMD
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1153 = FRE_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1152 = FRES_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1151 = FRES
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1150 = FRE
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1149 = FNMSUB_rec
  { 4, &PPCDescs.OperandInfo[505] },  // Inst #1148 = FNMSUBS_rec
  { 4, &PPCDescs.OperandInfo[505] },  // Inst #1147 = FNMSUBS
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1146 = FNMSUB
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1145 = FNMADD_rec
  { 4, &PPCDescs.OperandInfo[505] },  // Inst #1144 = FNMADDS_rec
  { 4, &PPCDescs.OperandInfo[505] },  // Inst #1143 = FNMADDS
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1142 = FNMADD
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1141 = FNEGS_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1140 = FNEGS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1139 = FNEGD_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1138 = FNEGD
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1137 = FNABSS_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1136 = FNABSS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1135 = FNABSD_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1134 = FNABSD
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1133 = FMUL_rec
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1132 = FMULS_rec
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1131 = FMULS
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1130 = FMUL
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1129 = FMSUB_rec
  { 4, &PPCDescs.OperandInfo[505] },  // Inst #1128 = FMSUBS_rec
  { 4, &PPCDescs.OperandInfo[505] },  // Inst #1127 = FMSUBS
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1126 = FMSUB
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1125 = FMR_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1124 = FMR
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1123 = FMADD_rec
  { 4, &PPCDescs.OperandInfo[505] },  // Inst #1122 = FMADDS_rec
  { 4, &PPCDescs.OperandInfo[505] },  // Inst #1121 = FMADDS
  { 4, &PPCDescs.OperandInfo[501] },  // Inst #1120 = FMADD
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1119 = FENCE
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1118 = FDIV_rec
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1117 = FDIVS_rec
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1116 = FDIVS
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1115 = FDIV
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1114 = FCTIW_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1113 = FCTIWZ_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1112 = FCTIWZ
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1111 = FCTIWU_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1110 = FCTIWUZ_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1109 = FCTIWUZ
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1108 = FCTIWU
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1107 = FCTIW
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1106 = FCTID_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1105 = FCTIDZ_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1104 = FCTIDZ
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1103 = FCTIDU_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1102 = FCTIDUZ_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1101 = FCTIDUZ
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1100 = FCTIDU
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1099 = FCTID
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1098 = FCPSGNS_rec
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1097 = FCPSGNS
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1096 = FCPSGND_rec
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1095 = FCPSGND
  { 3, &PPCDescs.OperandInfo[498] },  // Inst #1094 = FCMPUS
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #1093 = FCMPUD
  { 3, &PPCDescs.OperandInfo[498] },  // Inst #1092 = FCMPOS
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #1091 = FCMPOD
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1090 = FCFID_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1089 = FCFIDU_rec
  { 2, &PPCDescs.OperandInfo[496] },  // Inst #1088 = FCFIDUS_rec
  { 2, &PPCDescs.OperandInfo[496] },  // Inst #1087 = FCFIDUS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1086 = FCFIDU
  { 2, &PPCDescs.OperandInfo[496] },  // Inst #1085 = FCFIDS_rec
  { 2, &PPCDescs.OperandInfo[496] },  // Inst #1084 = FCFIDS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1083 = FCFID
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1082 = FADDrtz
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1081 = FADD_rec
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1080 = FADDS_rec
  { 3, &PPCDescs.OperandInfo[493] },  // Inst #1079 = FADDS
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #1078 = FADD
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1077 = FABSS_rec
  { 2, &PPCDescs.OperandInfo[491] },  // Inst #1076 = FABSS
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1075 = FABSD_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #1074 = FABSD
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1073 = EnforceIEIO
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1072 = EXTSW_rec
  { 2, &PPCDescs.OperandInfo[486] },  // Inst #1071 = EXTSW_32_64_rec
  { 2, &PPCDescs.OperandInfo[486] },  // Inst #1070 = EXTSW_32_64
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1069 = EXTSW_32
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #1068 = EXTSWSLI_rec
  { 3, &PPCDescs.OperandInfo[488] },  // Inst #1067 = EXTSWSLI_32_64_rec
  { 3, &PPCDescs.OperandInfo[488] },  // Inst #1066 = EXTSWSLI_32_64
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #1065 = EXTSWSLI
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1064 = EXTSW
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1063 = EXTSH_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1062 = EXTSH8_rec
  { 2, &PPCDescs.OperandInfo[486] },  // Inst #1061 = EXTSH8_32_64
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1060 = EXTSH8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1059 = EXTSH
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1058 = EXTSB_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1057 = EXTSB8_rec
  { 2, &PPCDescs.OperandInfo[486] },  // Inst #1056 = EXTSB8_32_64
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #1055 = EXTSB8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #1054 = EXTSB
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1053 = EVXOR
  { 3, &PPCDescs.OperandInfo[483] },  // Inst #1052 = EVSUBIFW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1051 = EVSUBFW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #1050 = EVSUBFUSIAAW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #1049 = EVSUBFUMIAAW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #1048 = EVSUBFSSIAAW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #1047 = EVSUBFSMIAAW
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #1046 = EVSTWWOX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #1045 = EVSTWWO
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #1044 = EVSTWWEX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #1043 = EVSTWWE
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #1042 = EVSTWHOX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #1041 = EVSTWHO
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #1040 = EVSTWHEX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #1039 = EVSTWHE
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #1038 = EVSTDWX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #1037 = EVSTDW
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #1036 = EVSTDHX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #1035 = EVSTDH
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #1034 = EVSTDDX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #1033 = EVSTDD
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1032 = EVSRWU
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1031 = EVSRWS
  { 3, &PPCDescs.OperandInfo[465] },  // Inst #1030 = EVSRWIU
  { 3, &PPCDescs.OperandInfo[465] },  // Inst #1029 = EVSRWIS
  { 2, &PPCDescs.OperandInfo[481] },  // Inst #1028 = EVSPLATI
  { 2, &PPCDescs.OperandInfo[481] },  // Inst #1027 = EVSPLATFI
  { 3, &PPCDescs.OperandInfo[465] },  // Inst #1026 = EVSLWI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1025 = EVSLW
  { 4, &PPCDescs.OperandInfo[477] },  // Inst #1024 = EVSEL
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #1023 = EVRNDW
  { 3, &PPCDescs.OperandInfo[465] },  // Inst #1022 = EVRLWI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1021 = EVRLW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1020 = EVORC
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1019 = EVOR
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1018 = EVNOR
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #1017 = EVNEG
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1016 = EVNAND
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1015 = EVMWUMIAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1014 = EVMWUMIAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1013 = EVMWUMIA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1012 = EVMWUMI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1011 = EVMWSSFAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1010 = EVMWSSFAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1009 = EVMWSSFA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1008 = EVMWSSF
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1007 = EVMWSMIAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1006 = EVMWSMIAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1005 = EVMWSMIA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1004 = EVMWSMI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1003 = EVMWSMFAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1002 = EVMWSMFAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1001 = EVMWSMFA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #1000 = EVMWSMF
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #999 = EVMWLUSIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #998 = EVMWLUSIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #997 = EVMWLUMIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #996 = EVMWLUMIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #995 = EVMWLUMIA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #994 = EVMWLUMI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #993 = EVMWLSSIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #992 = EVMWLSSIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #991 = EVMWLSMIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #990 = EVMWLSMIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #989 = EVMWHUMIA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #988 = EVMWHUMI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #987 = EVMWHSSFA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #986 = EVMWHSSF
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #985 = EVMWHSMIA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #984 = EVMWHSMI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #983 = EVMWHSMFA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #982 = EVMWHSMF
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #981 = EVMRA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #980 = EVMHOUSIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #979 = EVMHOUSIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #978 = EVMHOUMIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #977 = EVMHOUMIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #976 = EVMHOUMIA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #975 = EVMHOUMI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #974 = EVMHOSSIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #973 = EVMHOSSIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #972 = EVMHOSSFANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #971 = EVMHOSSFAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #970 = EVMHOSSFA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #969 = EVMHOSSF
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #968 = EVMHOSMIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #967 = EVMHOSMIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #966 = EVMHOSMIA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #965 = EVMHOSMI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #964 = EVMHOSMFANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #963 = EVMHOSMFAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #962 = EVMHOSMFA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #961 = EVMHOSMF
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #960 = EVMHOGUMIAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #959 = EVMHOGUMIAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #958 = EVMHOGSMIAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #957 = EVMHOGSMIAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #956 = EVMHOGSMFAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #955 = EVMHOGSMFAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #954 = EVMHEUSIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #953 = EVMHEUSIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #952 = EVMHEUMIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #951 = EVMHEUMIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #950 = EVMHEUMIA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #949 = EVMHEUMI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #948 = EVMHESSIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #947 = EVMHESSIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #946 = EVMHESSFANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #945 = EVMHESSFAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #944 = EVMHESSFA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #943 = EVMHESSF
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #942 = EVMHESMIANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #941 = EVMHESMIAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #940 = EVMHESMIA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #939 = EVMHESMI
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #938 = EVMHESMFANW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #937 = EVMHESMFAAW
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #936 = EVMHESMFA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #935 = EVMHESMF
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #934 = EVMHEGUMIAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #933 = EVMHEGUMIAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #932 = EVMHEGSMIAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #931 = EVMHEGSMIAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #930 = EVMHEGSMFAN
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #929 = EVMHEGSMFAA
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #928 = EVMERGELOHI
  { 3, &PPCDescs.OperandInfo[474] },  // Inst #927 = EVMERGELO
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #926 = EVMERGEHILO
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #925 = EVMERGEHI
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #924 = EVLWWSPLATX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #923 = EVLWWSPLAT
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #922 = EVLWHSPLATX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #921 = EVLWHSPLAT
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #920 = EVLWHOUX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #919 = EVLWHOU
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #918 = EVLWHOSX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #917 = EVLWHOS
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #916 = EVLWHEX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #915 = EVLWHE
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #914 = EVLHHOUSPLATX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #913 = EVLHHOUSPLAT
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #912 = EVLHHOSSPLATX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #911 = EVLHHOSSPLAT
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #910 = EVLHHESPLATX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #909 = EVLHHESPLAT
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #908 = EVLDWX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #907 = EVLDW
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #906 = EVLDHX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #905 = EVLDH
  { 3, &PPCDescs.OperandInfo[471] },  // Inst #904 = EVLDDX
  { 3, &PPCDescs.OperandInfo[468] },  // Inst #903 = EVLDD
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #902 = EVFSTSTLT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #901 = EVFSTSTGT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #900 = EVFSTSTEQ
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #899 = EVFSSUB
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #898 = EVFSNEG
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #897 = EVFSNABS
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #896 = EVFSMUL
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #895 = EVFSDIV
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #894 = EVFSCTUIZ
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #893 = EVFSCTUI
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #892 = EVFSCTUF
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #891 = EVFSCTSIZ
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #890 = EVFSCTSI
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #889 = EVFSCTSF
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #888 = EVFSCMPLT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #887 = EVFSCMPGT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #886 = EVFSCMPEQ
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #885 = EVFSCFUI
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #884 = EVFSCFUF
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #883 = EVFSCFSI
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #882 = EVFSCFSF
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #881 = EVFSADD
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #880 = EVFSABS
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #879 = EVEXTSH
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #878 = EVEXTSB
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #877 = EVEQV
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #876 = EVDIVWU
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #875 = EVDIVWS
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #874 = EVCNTLZW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #873 = EVCNTLSW
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #872 = EVCMPLTU
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #871 = EVCMPLTS
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #870 = EVCMPGTU
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #869 = EVCMPGTS
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #868 = EVCMPEQ
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #867 = EVANDC
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #866 = EVAND
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #865 = EVADDW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #864 = EVADDUSIAAW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #863 = EVADDUMIAAW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #862 = EVADDSSIAAW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #861 = EVADDSMIAAW
  { 3, &PPCDescs.OperandInfo[465] },  // Inst #860 = EVADDIW
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #859 = EVABS
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #858 = EQV_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #857 = EQV8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #856 = EQV8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #855 = EQV
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #854 = EH_SjLj_Setup
  { 2, &PPCDescs.OperandInfo[463] },  // Inst #853 = EH_SjLj_SetJmp64
  { 2, &PPCDescs.OperandInfo[463] },  // Inst #852 = EH_SjLj_SetJmp32
  { 1, &PPCDescs.OperandInfo[462] },  // Inst #851 = EH_SjLj_LongJmp64
  { 1, &PPCDescs.OperandInfo[462] },  // Inst #850 = EH_SjLj_LongJmp32
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #849 = EFSTSTLT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #848 = EFSTSTGT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #847 = EFSTSTEQ
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #846 = EFSSUB
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #845 = EFSNEG
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #844 = EFSNABS
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #843 = EFSMUL
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #842 = EFSDIV
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #841 = EFSCTUIZ
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #840 = EFSCTUI
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #839 = EFSCTUF
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #838 = EFSCTSIZ
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #837 = EFSCTSI
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #836 = EFSCTSF
  { 3, &PPCDescs.OperandInfo[311] },  // Inst #835 = EFSCMPLT
  { 3, &PPCDescs.OperandInfo[311] },  // Inst #834 = EFSCMPGT
  { 3, &PPCDescs.OperandInfo[311] },  // Inst #833 = EFSCMPEQ
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #832 = EFSCFUI
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #831 = EFSCFUF
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #830 = EFSCFSI
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #829 = EFSCFSF
  { 2, &PPCDescs.OperandInfo[460] },  // Inst #828 = EFSCFD
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #827 = EFSADD
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #826 = EFSABS
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #825 = EFDTSTLT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #824 = EFDTSTGT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #823 = EFDTSTEQ
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #822 = EFDSUB
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #821 = EFDNEG
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #820 = EFDNABS
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #819 = EFDMUL
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #818 = EFDDIV
  { 2, &PPCDescs.OperandInfo[460] },  // Inst #817 = EFDCTUIZ
  { 2, &PPCDescs.OperandInfo[460] },  // Inst #816 = EFDCTUIDZ
  { 2, &PPCDescs.OperandInfo[460] },  // Inst #815 = EFDCTUI
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #814 = EFDCTUF
  { 2, &PPCDescs.OperandInfo[460] },  // Inst #813 = EFDCTSIZ
  { 2, &PPCDescs.OperandInfo[460] },  // Inst #812 = EFDCTSIDZ
  { 2, &PPCDescs.OperandInfo[460] },  // Inst #811 = EFDCTSI
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #810 = EFDCTSF
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #809 = EFDCMPLT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #808 = EFDCMPGT
  { 3, &PPCDescs.OperandInfo[457] },  // Inst #807 = EFDCMPEQ
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #806 = EFDCFUID
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #805 = EFDCFUI
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #804 = EFDCFUF
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #803 = EFDCFSID
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #802 = EFDCFSI
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #801 = EFDCFSF
  { 2, &PPCDescs.OperandInfo[455] },  // Inst #800 = EFDCFS
  { 3, &PPCDescs.OperandInfo[452] },  // Inst #799 = EFDADD
  { 2, &PPCDescs.OperandInfo[450] },  // Inst #798 = EFDABS
  { 2, &PPCDescs.OperandInfo[448] },  // Inst #797 = DecreaseCTRloop
  { 2, &PPCDescs.OperandInfo[448] },  // Inst #796 = DecreaseCTR8loop
  { 3, &PPCDescs.OperandInfo[445] },  // Inst #795 = DYNAREAOFFSET8
  { 3, &PPCDescs.OperandInfo[445] },  // Inst #794 = DYNAREAOFFSET
  { 4, &PPCDescs.OperandInfo[441] },  // Inst #793 = DYNALLOC8
  { 4, &PPCDescs.OperandInfo[437] },  // Inst #792 = DYNALLOC
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #791 = DXEX_rec
  { 2, &PPCDescs.OperandInfo[351] },  // Inst #790 = DXEXQ_rec
  { 2, &PPCDescs.OperandInfo[351] },  // Inst #789 = DXEXQ
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #788 = DXEX
  { 3, &PPCDescs.OperandInfo[434] },  // Inst #787 = DTSTSFQ
  { 3, &PPCDescs.OperandInfo[431] },  // Inst #786 = DTSTSFIQ
  { 3, &PPCDescs.OperandInfo[428] },  // Inst #785 = DTSTSFI
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #784 = DTSTSF
  { 3, &PPCDescs.OperandInfo[348] },  // Inst #783 = DTSTEXQ
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #782 = DTSTEX
  { 3, &PPCDescs.OperandInfo[425] },  // Inst #781 = DTSTDGQ
  { 3, &PPCDescs.OperandInfo[422] },  // Inst #780 = DTSTDG
  { 3, &PPCDescs.OperandInfo[425] },  // Inst #779 = DTSTDCQ
  { 3, &PPCDescs.OperandInfo[422] },  // Inst #778 = DTSTDC
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #777 = DSUB_rec
  { 3, &PPCDescs.OperandInfo[333] },  // Inst #776 = DSUBQ_rec
  { 3, &PPCDescs.OperandInfo[333] },  // Inst #775 = DSUBQ
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #774 = DSUB
  { 3, &PPCDescs.OperandInfo[419] },  // Inst #773 = DSTT64
  { 3, &PPCDescs.OperandInfo[416] },  // Inst #772 = DSTT
  { 3, &PPCDescs.OperandInfo[419] },  // Inst #771 = DSTSTT64
  { 3, &PPCDescs.OperandInfo[416] },  // Inst #770 = DSTSTT
  { 3, &PPCDescs.OperandInfo[419] },  // Inst #769 = DSTST64
  { 3, &PPCDescs.OperandInfo[416] },  // Inst #768 = DSTST
  { 3, &PPCDescs.OperandInfo[419] },  // Inst #767 = DST64
  { 3, &PPCDescs.OperandInfo[416] },  // Inst #766 = DST
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #765 = DSSALL
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #764 = DSS
  { 3, &PPCDescs.OperandInfo[410] },  // Inst #763 = DSCRI_rec
  { 3, &PPCDescs.OperandInfo[413] },  // Inst #762 = DSCRIQ_rec
  { 3, &PPCDescs.OperandInfo[413] },  // Inst #761 = DSCRIQ
  { 3, &PPCDescs.OperandInfo[410] },  // Inst #760 = DSCRI
  { 3, &PPCDescs.OperandInfo[410] },  // Inst #759 = DSCLI_rec
  { 3, &PPCDescs.OperandInfo[413] },  // Inst #758 = DSCLIQ_rec
  { 3, &PPCDescs.OperandInfo[413] },  // Inst #757 = DSCLIQ
  { 3, &PPCDescs.OperandInfo[410] },  // Inst #756 = DSCLI
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #755 = DRSP_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #754 = DRSP
  { 4, &PPCDescs.OperandInfo[388] },  // Inst #753 = DRRND_rec
  { 4, &PPCDescs.OperandInfo[406] },  // Inst #752 = DRRNDQ_rec
  { 4, &PPCDescs.OperandInfo[406] },  // Inst #751 = DRRNDQ
  { 4, &PPCDescs.OperandInfo[388] },  // Inst #750 = DRRND
  { 4, &PPCDescs.OperandInfo[392] },  // Inst #749 = DRINTX_rec
  { 4, &PPCDescs.OperandInfo[396] },  // Inst #748 = DRINTXQ_rec
  { 4, &PPCDescs.OperandInfo[396] },  // Inst #747 = DRINTXQ
  { 4, &PPCDescs.OperandInfo[392] },  // Inst #746 = DRINTX
  { 4, &PPCDescs.OperandInfo[392] },  // Inst #745 = DRINTN_rec
  { 4, &PPCDescs.OperandInfo[396] },  // Inst #744 = DRINTNQ_rec
  { 4, &PPCDescs.OperandInfo[396] },  // Inst #743 = DRINTNQ
  { 4, &PPCDescs.OperandInfo[392] },  // Inst #742 = DRINTN
  { 2, &PPCDescs.OperandInfo[404] },  // Inst #741 = DRDPQ_rec
  { 2, &PPCDescs.OperandInfo[404] },  // Inst #740 = DRDPQ
  { 4, &PPCDescs.OperandInfo[388] },  // Inst #739 = DQUA_rec
  { 4, &PPCDescs.OperandInfo[400] },  // Inst #738 = DQUAQ_rec
  { 4, &PPCDescs.OperandInfo[400] },  // Inst #737 = DQUAQ
  { 4, &PPCDescs.OperandInfo[392] },  // Inst #736 = DQUAI_rec
  { 4, &PPCDescs.OperandInfo[396] },  // Inst #735 = DQUAIQ_rec
  { 4, &PPCDescs.OperandInfo[396] },  // Inst #734 = DQUAIQ
  { 4, &PPCDescs.OperandInfo[392] },  // Inst #733 = DQUAI
  { 4, &PPCDescs.OperandInfo[388] },  // Inst #732 = DQUA
  { 3, &PPCDescs.OperandInfo[385] },  // Inst #731 = DMXXINSTFDMR512_HI
  { 3, &PPCDescs.OperandInfo[382] },  // Inst #730 = DMXXINSTFDMR512
  { 3, &PPCDescs.OperandInfo[379] },  // Inst #729 = DMXXINSTFDMR256
  { 3, &PPCDescs.OperandInfo[376] },  // Inst #728 = DMXXEXTFDMR512_HI
  { 3, &PPCDescs.OperandInfo[373] },  // Inst #727 = DMXXEXTFDMR512
  { 3, &PPCDescs.OperandInfo[370] },  // Inst #726 = DMXXEXTFDMR256
  { 3, &PPCDescs.OperandInfo[367] },  // Inst #725 = DMXOR
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #724 = DMUL_rec
  { 3, &PPCDescs.OperandInfo[333] },  // Inst #723 = DMULQ_rec
  { 3, &PPCDescs.OperandInfo[333] },  // Inst #722 = DMULQ
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #721 = DMUL
  { 1, &PPCDescs.OperandInfo[366] },  // Inst #720 = DMSETDMRZ
  { 2, &PPCDescs.OperandInfo[364] },  // Inst #719 = DMMR
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #718 = DIVW_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #717 = DIVWU_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #716 = DIVWUO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #715 = DIVWUO
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #714 = DIVWU
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #713 = DIVWO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #712 = DIVWO
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #711 = DIVWE_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #710 = DIVWEU_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #709 = DIVWEUO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #708 = DIVWEUO
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #707 = DIVWEU
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #706 = DIVWEO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #705 = DIVWEO
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #704 = DIVWE
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #703 = DIVW
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #702 = DIVD_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #701 = DIVDU_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #700 = DIVDUO_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #699 = DIVDUO
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #698 = DIVDU
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #697 = DIVDO_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #696 = DIVDO
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #695 = DIVDE_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #694 = DIVDEU_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #693 = DIVDEUO_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #692 = DIVDEUO
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #691 = DIVDEU
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #690 = DIVDEO_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #689 = DIVDEO
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #688 = DIVDE
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #687 = DIVD
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #686 = DIEX_rec
  { 3, &PPCDescs.OperandInfo[361] },  // Inst #685 = DIEXQ_rec
  { 3, &PPCDescs.OperandInfo[361] },  // Inst #684 = DIEXQ
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #683 = DIEX
  { 3, &PPCDescs.OperandInfo[355] },  // Inst #682 = DENBCD_rec
  { 3, &PPCDescs.OperandInfo[358] },  // Inst #681 = DENBCDQ_rec
  { 3, &PPCDescs.OperandInfo[358] },  // Inst #680 = DENBCDQ
  { 3, &PPCDescs.OperandInfo[355] },  // Inst #679 = DENBCD
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #678 = DDIV_rec
  { 3, &PPCDescs.OperandInfo[333] },  // Inst #677 = DDIVQ_rec
  { 3, &PPCDescs.OperandInfo[333] },  // Inst #676 = DDIVQ
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #675 = DDIV
  { 3, &PPCDescs.OperandInfo[355] },  // Inst #674 = DDEDPD_rec
  { 3, &PPCDescs.OperandInfo[358] },  // Inst #673 = DDEDPDQ_rec
  { 3, &PPCDescs.OperandInfo[358] },  // Inst #672 = DDEDPDQ
  { 3, &PPCDescs.OperandInfo[355] },  // Inst #671 = DDEDPD
  { 2, &PPCDescs.OperandInfo[341] },  // Inst #670 = DCTQPQ_rec
  { 2, &PPCDescs.OperandInfo[341] },  // Inst #669 = DCTQPQ
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #668 = DCTFIX_rec
  { 2, &PPCDescs.OperandInfo[351] },  // Inst #667 = DCTFIXQ_rec
  { 2, &PPCDescs.OperandInfo[353] },  // Inst #666 = DCTFIXQQ
  { 2, &PPCDescs.OperandInfo[351] },  // Inst #665 = DCTFIXQ
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #664 = DCTFIX
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #663 = DCTDP_rec
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #662 = DCTDP
  { 3, &PPCDescs.OperandInfo[348] },  // Inst #661 = DCMPUQ
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #660 = DCMPU
  { 3, &PPCDescs.OperandInfo[348] },  // Inst #659 = DCMPOQ
  { 3, &PPCDescs.OperandInfo[345] },  // Inst #658 = DCMPO
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #657 = DCFFIX_rec
  { 2, &PPCDescs.OperandInfo[341] },  // Inst #656 = DCFFIXQ_rec
  { 2, &PPCDescs.OperandInfo[343] },  // Inst #655 = DCFFIXQQ
  { 2, &PPCDescs.OperandInfo[341] },  // Inst #654 = DCFFIXQ
  { 2, &PPCDescs.OperandInfo[339] },  // Inst #653 = DCFFIX
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #652 = DCCCI
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #651 = DCBZLEP
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #650 = DCBZL
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #649 = DCBZEP
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #648 = DCBZ
  { 3, &PPCDescs.OperandInfo[177] },  // Inst #647 = DCBTSTEP
  { 3, &PPCDescs.OperandInfo[336] },  // Inst #646 = DCBTST
  { 3, &PPCDescs.OperandInfo[177] },  // Inst #645 = DCBTEP
  { 3, &PPCDescs.OperandInfo[336] },  // Inst #644 = DCBT
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #643 = DCBSTEP
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #642 = DCBST
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #641 = DCBI
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #640 = DCBFEP
  { 3, &PPCDescs.OperandInfo[336] },  // Inst #639 = DCBF
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #638 = DCBA
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #637 = DARN
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #636 = DADD_rec
  { 3, &PPCDescs.OperandInfo[333] },  // Inst #635 = DADDQ_rec
  { 3, &PPCDescs.OperandInfo[333] },  // Inst #634 = DADDQ
  { 3, &PPCDescs.OperandInfo[330] },  // Inst #633 = DADD
  { 3, &PPCDescs.OperandInfo[282] },  // Inst #632 = CTRL_DEP
  { 3, &PPCDescs.OperandInfo[325] },  // Inst #631 = CRXOR
  { 1, &PPCDescs.OperandInfo[290] },  // Inst #630 = CRUNSET
  { 1, &PPCDescs.OperandInfo[290] },  // Inst #629 = CRSET
  { 3, &PPCDescs.OperandInfo[325] },  // Inst #628 = CRORC
  { 3, &PPCDescs.OperandInfo[325] },  // Inst #627 = CROR
  { 2, &PPCDescs.OperandInfo[328] },  // Inst #626 = CRNOT
  { 3, &PPCDescs.OperandInfo[325] },  // Inst #625 = CRNOR
  { 3, &PPCDescs.OperandInfo[325] },  // Inst #624 = CRNAND
  { 3, &PPCDescs.OperandInfo[325] },  // Inst #623 = CREQV
  { 3, &PPCDescs.OperandInfo[325] },  // Inst #622 = CRANDC
  { 3, &PPCDescs.OperandInfo[325] },  // Inst #621 = CRAND
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #620 = CR6UNSET
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #619 = CR6SET
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #618 = CP_PASTE_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #617 = CP_PASTE8_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #616 = CP_COPY8
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #615 = CP_COPY
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #614 = CP_ABORT
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #613 = CNTTZW_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #612 = CNTTZW8_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #611 = CNTTZW8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #610 = CNTTZW
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #609 = CNTTZD_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #608 = CNTTZDM
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #607 = CNTTZD
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #606 = CNTLZW_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #605 = CNTLZW8_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #604 = CNTLZW8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #603 = CNTLZW
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #602 = CNTLZD_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #601 = CNTLZDM
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #600 = CNTLZD
  { 3, &PPCDescs.OperandInfo[314] },  // Inst #599 = CMPWI
  { 3, &PPCDescs.OperandInfo[311] },  // Inst #598 = CMPW
  { 4, &PPCDescs.OperandInfo[321] },  // Inst #597 = CMPRB8
  { 4, &PPCDescs.OperandInfo[317] },  // Inst #596 = CMPRB
  { 3, &PPCDescs.OperandInfo[314] },  // Inst #595 = CMPLWI
  { 3, &PPCDescs.OperandInfo[311] },  // Inst #594 = CMPLW
  { 3, &PPCDescs.OperandInfo[308] },  // Inst #593 = CMPLDI
  { 3, &PPCDescs.OperandInfo[305] },  // Inst #592 = CMPLD
  { 3, &PPCDescs.OperandInfo[305] },  // Inst #591 = CMPEQB
  { 3, &PPCDescs.OperandInfo[308] },  // Inst #590 = CMPDI
  { 3, &PPCDescs.OperandInfo[305] },  // Inst #589 = CMPD
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #588 = CMPB8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #587 = CMPB
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #586 = CLRBHRB
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #585 = CFUGED
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #584 = CDTBCD8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #583 = CDTBCD
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #582 = CBCDTD8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #581 = CBCDTD
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #580 = BRW8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #579 = BRW
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #578 = BRINC
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #577 = BRH8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #576 = BRH
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #575 = BRD
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #574 = BPERMD
  { 2, &PPCDescs.OperandInfo[13] },  // Inst #573 = BL_TLS
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #572 = BL_RM
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #571 = BL_NOP_RM
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #570 = BL_NOP
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #569 = BLRL
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #568 = BLR8
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #567 = BLR
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #566 = BLA_RM
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #565 = BLA8_RM
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #564 = BLA8_NOP_RM
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #563 = BLA8_NOP
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #562 = BLA8
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #561 = BLA
  { 2, &PPCDescs.OperandInfo[13] },  // Inst #560 = BL8_TLS_
  { 2, &PPCDescs.OperandInfo[13] },  // Inst #559 = BL8_TLS
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #558 = BL8_RM
  { 2, &PPCDescs.OperandInfo[13] },  // Inst #557 = BL8_NOTOC_TLS
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #556 = BL8_NOTOC_RM
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #555 = BL8_NOTOC
  { 2, &PPCDescs.OperandInfo[13] },  // Inst #554 = BL8_NOP_TLS
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #553 = BL8_NOP_RM
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #552 = BL8_NOP
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #551 = BL8
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #550 = BL
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #549 = BCTRL_RM
  { 2, &PPCDescs.OperandInfo[303] },  // Inst #548 = BCTRL_LWZinto_toc_RM
  { 2, &PPCDescs.OperandInfo[303] },  // Inst #547 = BCTRL_LWZinto_toc
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #546 = BCTRL8_RM
  { 2, &PPCDescs.OperandInfo[303] },  // Inst #545 = BCTRL8_LDinto_toc_RM
  { 2, &PPCDescs.OperandInfo[303] },  // Inst #544 = BCTRL8_LDinto_toc
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #543 = BCTRL8
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #542 = BCTRL
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #541 = BCTR8
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #540 = BCTR
  { 1, &PPCDescs.OperandInfo[290] },  // Inst #539 = BCLRL
  { 1, &PPCDescs.OperandInfo[290] },  // Inst #538 = BCLR
  { 2, &PPCDescs.OperandInfo[280] },  // Inst #537 = BCL
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #536 = BCDUTRUNC_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #535 = BCDUS_rec
  { 4, &PPCDescs.OperandInfo[291] },  // Inst #534 = BCDTRUNC_rec
  { 4, &PPCDescs.OperandInfo[291] },  // Inst #533 = BCDS_rec
  { 4, &PPCDescs.OperandInfo[291] },  // Inst #532 = BCDSUB_rec
  { 4, &PPCDescs.OperandInfo[291] },  // Inst #531 = BCDSR_rec
  { 3, &PPCDescs.OperandInfo[295] },  // Inst #530 = BCDSETSGN_rec
  { 3, &PPCDescs.OperandInfo[295] },  // Inst #529 = BCDCTZ_rec
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #528 = BCDCTSQ_rec
  { 2, &PPCDescs.OperandInfo[301] },  // Inst #527 = BCDCTN_rec
  { 3, &PPCDescs.OperandInfo[298] },  // Inst #526 = BCDCPSGN_rec
  { 3, &PPCDescs.OperandInfo[295] },  // Inst #525 = BCDCFZ_rec
  { 3, &PPCDescs.OperandInfo[295] },  // Inst #524 = BCDCFSQ_rec
  { 3, &PPCDescs.OperandInfo[295] },  // Inst #523 = BCDCFN_rec
  { 4, &PPCDescs.OperandInfo[291] },  // Inst #522 = BCDADD_rec
  { 1, &PPCDescs.OperandInfo[290] },  // Inst #521 = BCCTRL8
  { 1, &PPCDescs.OperandInfo[290] },  // Inst #520 = BCCTRL
  { 1, &PPCDescs.OperandInfo[290] },  // Inst #519 = BCCTR8
  { 1, &PPCDescs.OperandInfo[290] },  // Inst #518 = BCCTR
  { 2, &PPCDescs.OperandInfo[288] },  // Inst #517 = BCCLRL
  { 2, &PPCDescs.OperandInfo[288] },  // Inst #516 = BCCLR
  { 3, &PPCDescs.OperandInfo[285] },  // Inst #515 = BCCLA
  { 3, &PPCDescs.OperandInfo[282] },  // Inst #514 = BCCL
  { 2, &PPCDescs.OperandInfo[288] },  // Inst #513 = BCCCTRL8
  { 2, &PPCDescs.OperandInfo[288] },  // Inst #512 = BCCCTRL
  { 2, &PPCDescs.OperandInfo[288] },  // Inst #511 = BCCCTR8
  { 2, &PPCDescs.OperandInfo[288] },  // Inst #510 = BCCCTR
  { 3, &PPCDescs.OperandInfo[285] },  // Inst #509 = BCCA
  { 3, &PPCDescs.OperandInfo[282] },  // Inst #508 = BCC
  { 2, &PPCDescs.OperandInfo[280] },  // Inst #507 = BC
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #506 = BA
  { 1, &PPCDescs.OperandInfo[279] },  // Inst #505 = B
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #504 = ATTN
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #503 = ATOMIC_SWAP_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #502 = ATOMIC_SWAP_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #501 = ATOMIC_SWAP_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #500 = ATOMIC_SWAP_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #499 = ATOMIC_LOAD_XOR_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #498 = ATOMIC_LOAD_XOR_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #497 = ATOMIC_LOAD_XOR_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #496 = ATOMIC_LOAD_XOR_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #495 = ATOMIC_LOAD_UMIN_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #494 = ATOMIC_LOAD_UMIN_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #493 = ATOMIC_LOAD_UMIN_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #492 = ATOMIC_LOAD_UMIN_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #491 = ATOMIC_LOAD_UMAX_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #490 = ATOMIC_LOAD_UMAX_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #489 = ATOMIC_LOAD_UMAX_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #488 = ATOMIC_LOAD_UMAX_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #487 = ATOMIC_LOAD_SUB_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #486 = ATOMIC_LOAD_SUB_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #485 = ATOMIC_LOAD_SUB_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #484 = ATOMIC_LOAD_SUB_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #483 = ATOMIC_LOAD_OR_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #482 = ATOMIC_LOAD_OR_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #481 = ATOMIC_LOAD_OR_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #480 = ATOMIC_LOAD_OR_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #479 = ATOMIC_LOAD_NAND_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #478 = ATOMIC_LOAD_NAND_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #477 = ATOMIC_LOAD_NAND_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #476 = ATOMIC_LOAD_NAND_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #475 = ATOMIC_LOAD_MIN_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #474 = ATOMIC_LOAD_MIN_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #473 = ATOMIC_LOAD_MIN_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #472 = ATOMIC_LOAD_MIN_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #471 = ATOMIC_LOAD_MAX_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #470 = ATOMIC_LOAD_MAX_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #469 = ATOMIC_LOAD_MAX_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #468 = ATOMIC_LOAD_MAX_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #467 = ATOMIC_LOAD_AND_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #466 = ATOMIC_LOAD_AND_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #465 = ATOMIC_LOAD_AND_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #464 = ATOMIC_LOAD_AND_I16
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #463 = ATOMIC_LOAD_ADD_I8
  { 4, &PPCDescs.OperandInfo[275] },  // Inst #462 = ATOMIC_LOAD_ADD_I64
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #461 = ATOMIC_LOAD_ADD_I32
  { 4, &PPCDescs.OperandInfo[271] },  // Inst #460 = ATOMIC_LOAD_ADD_I16
  { 5, &PPCDescs.OperandInfo[261] },  // Inst #459 = ATOMIC_CMP_SWAP_I8
  { 5, &PPCDescs.OperandInfo[266] },  // Inst #458 = ATOMIC_CMP_SWAP_I64
  { 5, &PPCDescs.OperandInfo[261] },  // Inst #457 = ATOMIC_CMP_SWAP_I32
  { 5, &PPCDescs.OperandInfo[261] },  // Inst #456 = ATOMIC_CMP_SWAP_I16
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #455 = AND_rec
  { 2, &PPCDescs.OperandInfo[259] },  // Inst #454 = ANDI_rec_1_GT_BIT8
  { 2, &PPCDescs.OperandInfo[257] },  // Inst #453 = ANDI_rec_1_GT_BIT
  { 2, &PPCDescs.OperandInfo[259] },  // Inst #452 = ANDI_rec_1_EQ_BIT8
  { 2, &PPCDescs.OperandInfo[257] },  // Inst #451 = ANDI_rec_1_EQ_BIT
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #450 = ANDI_rec
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #449 = ANDIS_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #448 = ANDIS8_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #447 = ANDI8_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #446 = ANDC_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #445 = ANDC8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #444 = ANDC8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #443 = ANDC
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #442 = AND8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #441 = AND8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #440 = AND
  { 2, &PPCDescs.OperandInfo[21] },  // Inst #439 = ADJCALLSTACKUP
  { 2, &PPCDescs.OperandInfo[21] },  // Inst #438 = ADJCALLSTACKDOWN
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #437 = ADDZE_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #436 = ADDZEO_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #435 = ADDZEO
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #434 = ADDZE8_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #433 = ADDZE8O_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #432 = ADDZE8O
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #431 = ADDZE8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #430 = ADDZE
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #429 = ADDPCIS
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #428 = ADDME_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #427 = ADDMEO_rec
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #426 = ADDMEO
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #425 = ADDME8_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #424 = ADDME8O_rec
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #423 = ADDME8O
  { 2, &PPCDescs.OperandInfo[255] },  // Inst #422 = ADDME8
  { 2, &PPCDescs.OperandInfo[253] },  // Inst #421 = ADDME
  { 3, &PPCDescs.OperandInfo[219] },  // Inst #420 = ADDItocL
  { 3, &PPCDescs.OperandInfo[250] },  // Inst #419 = ADDItoc8
  { 3, &PPCDescs.OperandInfo[247] },  // Inst #418 = ADDItoc
  { 4, &PPCDescs.OperandInfo[243] },  // Inst #417 = ADDItlsldLADDR32
  { 4, &PPCDescs.OperandInfo[239] },  // Inst #416 = ADDItlsldLADDR
  { 3, &PPCDescs.OperandInfo[233] },  // Inst #415 = ADDItlsldL32
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #414 = ADDItlsldL
  { 4, &PPCDescs.OperandInfo[243] },  // Inst #413 = ADDItlsgdLADDR32
  { 4, &PPCDescs.OperandInfo[239] },  // Inst #412 = ADDItlsgdLADDR
  { 3, &PPCDescs.OperandInfo[233] },  // Inst #411 = ADDItlsgdL32
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #410 = ADDItlsgdL
  { 3, &PPCDescs.OperandInfo[233] },  // Inst #409 = ADDIdtprelL32
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #408 = ADDIdtprelL
  { 3, &PPCDescs.OperandInfo[219] },  // Inst #407 = ADDIStocHA8
  { 3, &PPCDescs.OperandInfo[236] },  // Inst #406 = ADDIStocHA
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #405 = ADDIStlsldHA
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #404 = ADDIStlsgdHA
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #403 = ADDISgotTprelHA
  { 3, &PPCDescs.OperandInfo[233] },  // Inst #402 = ADDISdtprelHA32
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #401 = ADDISdtprelHA
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #400 = ADDIS8
  { 3, &PPCDescs.OperandInfo[233] },  // Inst #399 = ADDIS
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #398 = ADDIC_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #397 = ADDIC8
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #396 = ADDIC
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #395 = ADDI8
  { 3, &PPCDescs.OperandInfo[233] },  // Inst #394 = ADDI
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #393 = ADDG6S8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #392 = ADDG6S
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #391 = ADDE_rec
  { 4, &PPCDescs.OperandInfo[229] },  // Inst #390 = ADDEX8
  { 4, &PPCDescs.OperandInfo[225] },  // Inst #389 = ADDEX
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #388 = ADDEO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #387 = ADDEO
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #386 = ADDE8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #385 = ADDE8O_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #384 = ADDE8O
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #383 = ADDE8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #382 = ADDE
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #381 = ADDC_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #380 = ADDCO_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #379 = ADDCO
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #378 = ADDC8_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #377 = ADDC8O_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #376 = ADDC8O
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #375 = ADDC8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #374 = ADDC
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #373 = ADD8_rec
  { 3, &PPCDescs.OperandInfo[222] },  // Inst #372 = ADD8TLS_
  { 3, &PPCDescs.OperandInfo[219] },  // Inst #371 = ADD8TLS
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #370 = ADD8O_rec
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #369 = ADD8O
  { 3, &PPCDescs.OperandInfo[216] },  // Inst #368 = ADD8
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #367 = ADD4_rec
  { 3, &PPCDescs.OperandInfo[213] },  // Inst #366 = ADD4TLS
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #365 = ADD4O_rec
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #364 = ADD4O
  { 3, &PPCDescs.OperandInfo[210] },  // Inst #363 = ADD4
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #362 = XFSTOREf64
  { 3, &PPCDescs.OperandInfo[207] },  // Inst #361 = XFSTOREf32
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #360 = XFLOADf64
  { 3, &PPCDescs.OperandInfo[207] },  // Inst #359 = XFLOADf32
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #358 = SUBPCIS
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #357 = SUBIS
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #356 = SUBIC_rec
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #355 = SUBIC
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #354 = SUBI
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #353 = STIWX
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #352 = SRWI_rec
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #351 = SRWI
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #350 = SRDI_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #349 = SRDI
  { 3, &PPCDescs.OperandInfo[204] },  // Inst #348 = SPILLTOVSR_STX
  { 3, &PPCDescs.OperandInfo[201] },  // Inst #347 = SPILLTOVSR_ST
  { 3, &PPCDescs.OperandInfo[204] },  // Inst #346 = SPILLTOVSR_LDX
  { 3, &PPCDescs.OperandInfo[201] },  // Inst #345 = SPILLTOVSR_LD
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #344 = SLWI_rec
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #343 = SLWI
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #342 = SLDI_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #341 = SLDI
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #340 = ROTRWI_rec
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #339 = ROTRWI
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #338 = ROTRDI_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #337 = ROTRDI
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #336 = RLWNMbm_rec
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #335 = RLWNMbm
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #334 = RLWINMbm_rec
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #333 = RLWINMbm
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #332 = RLWIMIbm_rec
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #331 = RLWIMIbm
  { 3, &PPCDescs.OperandInfo[198] },  // Inst #330 = PSUBI
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #329 = LIWZX
  { 3, &PPCDescs.OperandInfo[195] },  // Inst #328 = LIWAX
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #327 = LIS8
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #326 = LIS
  { 2, &PPCDescs.OperandInfo[193] },  // Inst #325 = LI8
  { 2, &PPCDescs.OperandInfo[191] },  // Inst #324 = LI
  { 3, &PPCDescs.OperandInfo[188] },  // Inst #323 = LAx
  { 2, &PPCDescs.OperandInfo[186] },  // Inst #322 = KILL_PAIR
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #321 = INSRWI_rec
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #320 = INSRWI
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #319 = INSRDI_rec
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #318 = INSRDI
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #317 = INSLWI_rec
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #316 = INSLWI
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #315 = EXTRWI_rec
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #314 = EXTRWI
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #313 = EXTRDI_rec
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #312 = EXTRDI
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #311 = EXTLWI_rec
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #310 = EXTLWI
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #309 = EXTLDI_rec
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #308 = EXTLDI
  { 3, &PPCDescs.OperandInfo[183] },  // Inst #307 = DFSTOREf64
  { 3, &PPCDescs.OperandInfo[180] },  // Inst #306 = DFSTOREf32
  { 3, &PPCDescs.OperandInfo[183] },  // Inst #305 = DFLOADf64
  { 3, &PPCDescs.OperandInfo[180] },  // Inst #304 = DFLOADf32
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #303 = DCBTx
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #302 = DCBTT
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #301 = DCBTSTx
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #300 = DCBTSTT
  { 3, &PPCDescs.OperandInfo[177] },  // Inst #299 = DCBTSTDS
  { 3, &PPCDescs.OperandInfo[177] },  // Inst #298 = DCBTSTCT
  { 3, &PPCDescs.OperandInfo[177] },  // Inst #297 = DCBTDS
  { 3, &PPCDescs.OperandInfo[177] },  // Inst #296 = DCBTCT
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #295 = DCBSTPS
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #294 = DCBFx
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #293 = DCBFPS
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #292 = DCBFLP
  { 2, &PPCDescs.OperandInfo[175] },  // Inst #291 = DCBFL
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #290 = CLRRWI_rec
  { 3, &PPCDescs.OperandInfo[172] },  // Inst #289 = CLRRWI
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #288 = CLRRDI_rec
  { 3, &PPCDescs.OperandInfo[169] },  // Inst #287 = CLRRDI
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #286 = CLRLSLWI_rec
  { 4, &PPCDescs.OperandInfo[165] },  // Inst #285 = CLRLSLWI
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #284 = CLRLSLDI_rec
  { 4, &PPCDescs.OperandInfo[161] },  // Inst #283 = CLRLSLDI
  { 1, &PPCDescs.OperandInfo[160] },  // Inst #282 = CFENCE8
  { 1, &PPCDescs.OperandInfo[159] },  // Inst #281 = CFENCE
  { 2, &PPCDescs.OperandInfo[157] },  // Inst #280 = BUILD_UACC
  { 3, &PPCDescs.OperandInfo[154] },  // Inst #279 = BUILD_QUADWORD
  { 6, &PPCDescs.OperandInfo[148] },  // Inst #278 = ATOMIC_SWAP_I128
  { 6, &PPCDescs.OperandInfo[148] },  // Inst #277 = ATOMIC_LOAD_XOR_I128
  { 6, &PPCDescs.OperandInfo[148] },  // Inst #276 = ATOMIC_LOAD_SUB_I128
  { 6, &PPCDescs.OperandInfo[148] },  // Inst #275 = ATOMIC_LOAD_OR_I128
  { 6, &PPCDescs.OperandInfo[148] },  // Inst #274 = ATOMIC_LOAD_NAND_I128
  { 6, &PPCDescs.OperandInfo[148] },  // Inst #273 = ATOMIC_LOAD_AND_I128
  { 6, &PPCDescs.OperandInfo[148] },  // Inst #272 = ATOMIC_LOAD_ADD_I128
  { 8, &PPCDescs.OperandInfo[140] },  // Inst #271 = ATOMIC_CMP_SWAP_I128
  { 4, &PPCDescs.OperandInfo[136] },  // Inst #270 = G_UBFX
  { 4, &PPCDescs.OperandInfo[136] },  // Inst #269 = G_SBFX
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #268 = G_VECREDUCE_UMIN
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #267 = G_VECREDUCE_UMAX
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #266 = G_VECREDUCE_SMIN
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #265 = G_VECREDUCE_SMAX
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #264 = G_VECREDUCE_XOR
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #263 = G_VECREDUCE_OR
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #262 = G_VECREDUCE_AND
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #261 = G_VECREDUCE_MUL
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #260 = G_VECREDUCE_ADD
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #259 = G_VECREDUCE_FMINIMUM
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #258 = G_VECREDUCE_FMAXIMUM
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #257 = G_VECREDUCE_FMIN
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #256 = G_VECREDUCE_FMAX
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #255 = G_VECREDUCE_FMUL
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #254 = G_VECREDUCE_FADD
  { 3, &PPCDescs.OperandInfo[123] },  // Inst #253 = G_VECREDUCE_SEQ_FMUL
  { 3, &PPCDescs.OperandInfo[123] },  // Inst #252 = G_VECREDUCE_SEQ_FADD
  { 3, &PPCDescs.OperandInfo[53] },  // Inst #251 = G_BZERO
  { 4, &PPCDescs.OperandInfo[132] },  // Inst #250 = G_MEMSET
  { 4, &PPCDescs.OperandInfo[132] },  // Inst #249 = G_MEMMOVE
  { 3, &PPCDescs.OperandInfo[123] },  // Inst #248 = G_MEMCPY_INLINE
  { 4, &PPCDescs.OperandInfo[132] },  // Inst #247 = G_MEMCPY
  { 2, &PPCDescs.OperandInfo[130] },  // Inst #246 = G_WRITE_REGISTER
  { 2, &PPCDescs.OperandInfo[51] },  // Inst #245 = G_READ_REGISTER
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #244 = G_STRICT_FLDEXP
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #243 = G_STRICT_FSQRT
  { 4, &PPCDescs.OperandInfo[46] },  // Inst #242 = G_STRICT_FMA
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #241 = G_STRICT_FREM
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #240 = G_STRICT_FDIV
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #239 = G_STRICT_FMUL
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #238 = G_STRICT_FSUB
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #237 = G_STRICT_FADD
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #236 = G_STACKRESTORE
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #235 = G_STACKSAVE
  { 3, &PPCDescs.OperandInfo[64] },  // Inst #234 = G_DYN_STACKALLOC
  { 2, &PPCDescs.OperandInfo[51] },  // Inst #233 = G_JUMP_TABLE
  { 2, &PPCDescs.OperandInfo[51] },  // Inst #232 = G_BLOCK_ADDR
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #231 = G_ADDRSPACE_CAST
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #230 = G_FNEARBYINT
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #229 = G_FRINT
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #228 = G_FFLOOR
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #227 = G_FSQRT
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #226 = G_FSIN
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #225 = G_FCOS
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #224 = G_FCEIL
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #223 = G_BITREVERSE
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #222 = G_BSWAP
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #221 = G_CTPOP
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #220 = G_CTLZ_ZERO_UNDEF
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #219 = G_CTLZ
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #218 = G_CTTZ_ZERO_UNDEF
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #217 = G_CTTZ
  { 4, &PPCDescs.OperandInfo[126] },  // Inst #216 = G_SHUFFLE_VECTOR
  { 3, &PPCDescs.OperandInfo[123] },  // Inst #215 = G_EXTRACT_VECTOR_ELT
  { 4, &PPCDescs.OperandInfo[119] },  // Inst #214 = G_INSERT_VECTOR_ELT
  { 3, &PPCDescs.OperandInfo[116] },  // Inst #213 = G_BRJT
  { 1, &PPCDescs.OperandInfo[0] },  // Inst #212 = G_BR
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #211 = G_LLROUND
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #210 = G_LROUND
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #209 = G_ABS
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #208 = G_UMAX
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #207 = G_UMIN
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #206 = G_SMAX
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #205 = G_SMIN
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #204 = G_PTRMASK
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #203 = G_PTR_ADD
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #202 = G_RESET_FPMODE
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #201 = G_SET_FPMODE
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #200 = G_GET_FPMODE
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #199 = G_RESET_FPENV
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #198 = G_SET_FPENV
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #197 = G_GET_FPENV
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #196 = G_FMAXIMUM
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #195 = G_FMINIMUM
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #194 = G_FMAXNUM_IEEE
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #193 = G_FMINNUM_IEEE
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #192 = G_FMAXNUM
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #191 = G_FMINNUM
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #190 = G_FCANONICALIZE
  { 3, &PPCDescs.OperandInfo[93] },  // Inst #189 = G_IS_FPCLASS
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #188 = G_FCOPYSIGN
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #187 = G_FABS
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #186 = G_UITOFP
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #185 = G_SITOFP
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #184 = G_FPTOUI
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #183 = G_FPTOSI
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #182 = G_FPTRUNC
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #181 = G_FPEXT
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #180 = G_FNEG
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #179 = G_FFREXP
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #178 = G_FLDEXP
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #177 = G_FLOG10
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #176 = G_FLOG2
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #175 = G_FLOG
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #174 = G_FEXP10
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #173 = G_FEXP2
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #172 = G_FEXP
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #171 = G_FPOWI
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #170 = G_FPOW
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #169 = G_FREM
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #168 = G_FDIV
  { 4, &PPCDescs.OperandInfo[46] },  // Inst #167 = G_FMAD
  { 4, &PPCDescs.OperandInfo[46] },  // Inst #166 = G_FMA
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #165 = G_FMUL
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #164 = G_FSUB
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #163 = G_FADD
  { 4, &PPCDescs.OperandInfo[112] },  // Inst #162 = G_UDIVFIXSAT
  { 4, &PPCDescs.OperandInfo[112] },  // Inst #161 = G_SDIVFIXSAT
  { 4, &PPCDescs.OperandInfo[112] },  // Inst #160 = G_UDIVFIX
  { 4, &PPCDescs.OperandInfo[112] },  // Inst #159 = G_SDIVFIX
  { 4, &PPCDescs.OperandInfo[112] },  // Inst #158 = G_UMULFIXSAT
  { 4, &PPCDescs.OperandInfo[112] },  // Inst #157 = G_SMULFIXSAT
  { 4, &PPCDescs.OperandInfo[112] },  // Inst #156 = G_UMULFIX
  { 4, &PPCDescs.OperandInfo[112] },  // Inst #155 = G_SMULFIX
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #154 = G_SSHLSAT
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #153 = G_USHLSAT
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #152 = G_SSUBSAT
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #151 = G_USUBSAT
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #150 = G_SADDSAT
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #149 = G_UADDSAT
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #148 = G_SMULH
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #147 = G_UMULH
  { 4, &PPCDescs.OperandInfo[82] },  // Inst #146 = G_SMULO
  { 4, &PPCDescs.OperandInfo[82] },  // Inst #145 = G_UMULO
  { 5, &PPCDescs.OperandInfo[107] },  // Inst #144 = G_SSUBE
  { 4, &PPCDescs.OperandInfo[82] },  // Inst #143 = G_SSUBO
  { 5, &PPCDescs.OperandInfo[107] },  // Inst #142 = G_SADDE
  { 4, &PPCDescs.OperandInfo[82] },  // Inst #141 = G_SADDO
  { 5, &PPCDescs.OperandInfo[107] },  // Inst #140 = G_USUBE
  { 4, &PPCDescs.OperandInfo[82] },  // Inst #139 = G_USUBO
  { 5, &PPCDescs.OperandInfo[107] },  // Inst #138 = G_UADDE
  { 4, &PPCDescs.OperandInfo[82] },  // Inst #137 = G_UADDO
  { 4, &PPCDescs.OperandInfo[82] },  // Inst #136 = G_SELECT
  { 4, &PPCDescs.OperandInfo[103] },  // Inst #135 = G_FCMP
  { 4, &PPCDescs.OperandInfo[103] },  // Inst #134 = G_ICMP
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #133 = G_ROTL
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #132 = G_ROTR
  { 4, &PPCDescs.OperandInfo[99] },  // Inst #131 = G_FSHR
  { 4, &PPCDescs.OperandInfo[99] },  // Inst #130 = G_FSHL
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #129 = G_ASHR
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #128 = G_LSHR
  { 3, &PPCDescs.OperandInfo[96] },  // Inst #127 = G_SHL
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #126 = G_ZEXT
  { 3, &PPCDescs.OperandInfo[40] },  // Inst #125 = G_SEXT_INREG
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #124 = G_SEXT
  { 3, &PPCDescs.OperandInfo[93] },  // Inst #123 = G_VAARG
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #122 = G_VASTART
  { 2, &PPCDescs.OperandInfo[51] },  // Inst #121 = G_FCONSTANT
  { 2, &PPCDescs.OperandInfo[51] },  // Inst #120 = G_CONSTANT
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #119 = G_TRUNC
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #118 = G_ANYEXT
  { 1, &PPCDescs.OperandInfo[0] },  // Inst #117 = G_INTRINSIC_CONVERGENT_W_SIDE_EFFECTS
  { 1, &PPCDescs.OperandInfo[0] },  // Inst #116 = G_INTRINSIC_CONVERGENT
  { 1, &PPCDescs.OperandInfo[0] },  // Inst #115 = G_INTRINSIC_W_SIDE_EFFECTS
  { 1, &PPCDescs.OperandInfo[0] },  // Inst #114 = G_INTRINSIC
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #113 = G_INVOKE_REGION_START
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #112 = G_BRINDIRECT
  { 2, &PPCDescs.OperandInfo[51] },  // Inst #111 = G_BRCOND
  { 4, &PPCDescs.OperandInfo[89] },  // Inst #110 = G_PREFETCH
  { 2, &PPCDescs.OperandInfo[21] },  // Inst #109 = G_FENCE
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #108 = G_ATOMICRMW_UDEC_WRAP
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #107 = G_ATOMICRMW_UINC_WRAP
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #106 = G_ATOMICRMW_FMIN
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #105 = G_ATOMICRMW_FMAX
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #104 = G_ATOMICRMW_FSUB
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #103 = G_ATOMICRMW_FADD
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #102 = G_ATOMICRMW_UMIN
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #101 = G_ATOMICRMW_UMAX
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #100 = G_ATOMICRMW_MIN
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #99 = G_ATOMICRMW_MAX
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #98 = G_ATOMICRMW_XOR
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #97 = G_ATOMICRMW_OR
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #96 = G_ATOMICRMW_NAND
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #95 = G_ATOMICRMW_AND
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #94 = G_ATOMICRMW_SUB
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #93 = G_ATOMICRMW_ADD
  { 3, &PPCDescs.OperandInfo[86] },  // Inst #92 = G_ATOMICRMW_XCHG
  { 4, &PPCDescs.OperandInfo[82] },  // Inst #91 = G_ATOMIC_CMPXCHG
  { 5, &PPCDescs.OperandInfo[77] },  // Inst #90 = G_ATOMIC_CMPXCHG_WITH_SUCCESS
  { 5, &PPCDescs.OperandInfo[72] },  // Inst #89 = G_INDEXED_STORE
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #88 = G_STORE
  { 5, &PPCDescs.OperandInfo[67] },  // Inst #87 = G_INDEXED_ZEXTLOAD
  { 5, &PPCDescs.OperandInfo[67] },  // Inst #86 = G_INDEXED_SEXTLOAD
  { 5, &PPCDescs.OperandInfo[67] },  // Inst #85 = G_INDEXED_LOAD
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #84 = G_ZEXTLOAD
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #83 = G_SEXTLOAD
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #82 = G_LOAD
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #81 = G_READCYCLECOUNTER
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #80 = G_INTRINSIC_ROUNDEVEN
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #79 = G_INTRINSIC_LRINT
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #78 = G_INTRINSIC_ROUND
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #77 = G_INTRINSIC_TRUNC
  { 3, &PPCDescs.OperandInfo[64] },  // Inst #76 = G_INTRINSIC_FPTRUNC_ROUND
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #75 = G_CONSTANT_FOLD_BARRIER
  { 2, &PPCDescs.OperandInfo[62] },  // Inst #74 = G_FREEZE
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #73 = G_BITCAST
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #72 = G_INTTOPTR
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #71 = G_PTRTOINT
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #70 = G_CONCAT_VECTORS
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #69 = G_BUILD_VECTOR_TRUNC
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #68 = G_BUILD_VECTOR
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #67 = G_MERGE_VALUES
  { 4, &PPCDescs.OperandInfo[58] },  // Inst #66 = G_INSERT
  { 2, &PPCDescs.OperandInfo[56] },  // Inst #65 = G_UNMERGE_VALUES
  { 3, &PPCDescs.OperandInfo[53] },  // Inst #64 = G_EXTRACT
  { 2, &PPCDescs.OperandInfo[51] },  // Inst #63 = G_CONSTANT_POOL
  { 2, &PPCDescs.OperandInfo[51] },  // Inst #62 = G_GLOBAL_VALUE
  { 2, &PPCDescs.OperandInfo[51] },  // Inst #61 = G_FRAME_INDEX
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #60 = G_PHI
  { 1, &PPCDescs.OperandInfo[50] },  // Inst #59 = G_IMPLICIT_DEF
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #58 = G_XOR
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #57 = G_OR
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #56 = G_AND
  { 4, &PPCDescs.OperandInfo[46] },  // Inst #55 = G_UDIVREM
  { 4, &PPCDescs.OperandInfo[46] },  // Inst #54 = G_SDIVREM
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #53 = G_UREM
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #52 = G_SREM
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #51 = G_UDIV
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #50 = G_SDIV
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #49 = G_MUL
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #48 = G_SUB
  { 3, &PPCDescs.OperandInfo[43] },  // Inst #47 = G_ADD
  { 3, &PPCDescs.OperandInfo[40] },  // Inst #46 = G_ASSERT_ALIGN
  { 3, &PPCDescs.OperandInfo[40] },  // Inst #45 = G_ASSERT_ZEXT
  { 3, &PPCDescs.OperandInfo[40] },  // Inst #44 = G_ASSERT_SEXT
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #43 = JUMP_TABLE_DEBUG_INFO
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #42 = MEMBARRIER
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #41 = ICALL_BRANCH_FUNNEL
  { 3, &PPCDescs.OperandInfo[37] },  // Inst #40 = PATCHABLE_TYPED_EVENT_CALL
  { 2, &PPCDescs.OperandInfo[35] },  // Inst #39 = PATCHABLE_EVENT_CALL
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #38 = PATCHABLE_TAIL_CALL
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #37 = PATCHABLE_FUNCTION_EXIT
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #36 = PATCHABLE_RET
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #35 = PATCHABLE_FUNCTION_ENTER
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #34 = PATCHABLE_OP
  { 1, &PPCDescs.OperandInfo[0] },  // Inst #33 = FAULTING_OP
  { 2, &PPCDescs.OperandInfo[33] },  // Inst #32 = LOCAL_ESCAPE
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #31 = STATEPOINT
  { 3, &PPCDescs.OperandInfo[30] },  // Inst #30 = PREALLOCATED_ARG
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #29 = PREALLOCATED_SETUP
  { 1, &PPCDescs.OperandInfo[29] },  // Inst #28 = LOAD_STACK_GUARD
  { 6, &PPCDescs.OperandInfo[23] },  // Inst #27 = PATCHPOINT
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #26 = FENTRY_CALL
  { 2, &PPCDescs.OperandInfo[21] },  // Inst #25 = STACKMAP
  { 2, &PPCDescs.OperandInfo[19] },  // Inst #24 = ARITH_FENCE
  { 4, &PPCDescs.OperandInfo[15] },  // Inst #23 = PSEUDO_PROBE
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #22 = LIFETIME_END
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #21 = LIFETIME_START
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #20 = BUNDLE
  { 2, &PPCDescs.OperandInfo[13] },  // Inst #19 = COPY
  { 2, &PPCDescs.OperandInfo[13] },  // Inst #18 = REG_SEQUENCE
  { 1, &PPCDescs.OperandInfo[0] },  // Inst #17 = DBG_LABEL
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #16 = DBG_PHI
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #15 = DBG_INSTR_REF
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #14 = DBG_VALUE_LIST
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #13 = DBG_VALUE
  { 3, &PPCDescs.OperandInfo[2] },  // Inst #12 = COPY_TO_REGCLASS
  { 4, &PPCDescs.OperandInfo[9] },  // Inst #11 = SUBREG_TO_REG
  { 1, &PPCDescs.OperandInfo[0] },  // Inst #10 = IMPLICIT_DEF
  { 4, &PPCDescs.OperandInfo[5] },  // Inst #9 = INSERT_SUBREG
  { 3, &PPCDescs.OperandInfo[2] },  // Inst #8 = EXTRACT_SUBREG
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #7 = KILL
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #6 = ANNOTATION_LABEL
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #5 = GC_LABEL
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #4 = EH_LABEL
  { 1, &PPCDescs.OperandInfo[1] },  // Inst #3 = CFI_INSTRUCTION
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #2 = INLINEASM_BR
  { 0, &PPCDescs.OperandInfo[1] },  // Inst #1 = INLINEASM
  { 1, &PPCDescs.OperandInfo[0] },  // Inst #0 = PHI
  }, {
    /* 0 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1 */
    /* 1 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 2 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 5 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 9 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 13 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 15 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 19 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 21 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 23 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 29 */ { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 30 */ { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 33 */ { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 35 */ { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 37 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 40 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 43 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 46 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 50 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 51 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 53 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 56 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 58 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 62 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 64 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 67 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 72 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 77 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 82 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 86 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 89 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 93 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 96 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 99 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 103 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 107 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 112 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 116 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 119 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 },
    /* 123 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 },
    /* 126 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 130 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 132 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 136 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 140 */ { PPC_G8pRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_G8pRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 148 */ { PPC_G8pRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_G8pRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 154 */ { PPC_G8pRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 157 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_UACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 159 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 160 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 161 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 165 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 169 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 172 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 175 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 177 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 180 */ { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 183 */ { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 186 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 188 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 191 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 193 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 195 */ { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 198 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 201 */ { PPC_SPILLTOVSRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 204 */ { PPC_SPILLTOVSRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 207 */ { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 210 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 213 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 216 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 219 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 222 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 225 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 229 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 233 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 236 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 239 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 243 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 247 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 250 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 253 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 255 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 257 */ { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 259 */ { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 261 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 266 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 271 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 275 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 279 */ { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 280 */ { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 282 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 285 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 288 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 290 */ { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 291 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 295 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 298 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 301 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 303 */ { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 305 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 308 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 311 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 314 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 317 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 321 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 325 */ { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 328 */ { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 330 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 333 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 336 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 339 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 341 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 343 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 345 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 348 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 351 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 353 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 355 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 358 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 361 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 364 */ { PPC_DMRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_DMRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 366 */ { PPC_DMRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 367 */ { PPC_DMRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_DMRRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_DMRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 370 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_DMRROWpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 373 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 376 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_WACC_HIRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 379 */ { PPC_DMRROWpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 382 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 385 */ { PPC_WACC_HIRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 388 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 392 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 396 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 400 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 404 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 406 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 410 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 413 */ { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 416 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 419 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 422 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 425 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 428 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 431 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 434 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_FpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 437 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 441 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 445 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 448 */ { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 450 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 452 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 455 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 457 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 460 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 462 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 463 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 465 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 468 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 471 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 474 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 477 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 481 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 483 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 486 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 488 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 491 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 493 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 496 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 498 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 501 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 505 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 509 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 513 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 515 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 518 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 522 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 526 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 529 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 532 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) },
    /* 536 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) },
    /* 540 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 544 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 548 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 551 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 554 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 557 */ { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 560 */ { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 563 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 566 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 569 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 572 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) },
    /* 576 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 580 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 583 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 586 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) },
    /* 590 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 594 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 597 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 600 */ { PPC_G8pRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 603 */ { PPC_G8pRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 606 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 609 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 612 */ { PPC_VFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 615 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 618 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 621 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 623 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 626 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 629 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 632 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 635 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 639 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 643 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 645 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 646 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 649 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 650 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 652 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 654 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 656 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 658 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRSAVERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 660 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 662 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 663 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 665 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 667 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 669 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 671 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 675 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 678 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 680 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 682 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 684 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 686 */ { PPC_VRSAVERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 688 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 690 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 692 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 694 */ { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 696 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 699 */ { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 701 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 704 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 707 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 710 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 713 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 716 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 719 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 722 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 725 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 727 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 730 */ { PPC_VFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 733 */ { PPC_VFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 735 */ { PPC_VFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 738 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 741 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 744 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 746 */ { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 749 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 752 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 758 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 765 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 771 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 778 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 783 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 789 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 794 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 800 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 805 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 811 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 816 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 822 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 829 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 834 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 839 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(1) }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 844 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(1) }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 849 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 854 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 859 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 863 */ { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 867 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 871 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 874 */ { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 877 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 880 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 882 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 884 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 887 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 890 */ { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 892 */ { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 895 */ { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 898 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 902 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 906 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 910 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 912 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 916 */ { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 920 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 924 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 926 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 929 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 932 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 936 */ { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 939 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 943 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 945 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 949 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 953 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 956 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 959 */ { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 962 */ { PPC_G8pRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 965 */ { PPC_UACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 968 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 971 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 975 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 979 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 984 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 990 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 996 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1001 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1006 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1011 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1016 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1021 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1026 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1031 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1036 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1041 */ { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1046 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1051 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1056 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1061 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1066 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1071 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1075 */ { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1079 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1083 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRC_NOR0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1087 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RC_NOX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1091 */ { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QBRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1095 */ { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1099 */ { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_QSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1103 */ { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_SPERCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1107 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1111 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1115 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1117 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1119 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1121 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1123 */ { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1125 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1128 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8pRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1131 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 1135 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 1139 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 1143 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 1147 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 1151 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_F8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 1155 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_MEMORY, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 1159 */ { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { PPC_F4RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { 1, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(0) }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_MEMORY, 0 },
    /* 1163 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1166 */ { -1, 0, MCOI_OPERAND_PCREL, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1168 */ { PPC_CTRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1170 */ { PPC_CTRRC8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1172 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1175 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1178 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1182 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1185 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1188 */ { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1190 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1193 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1196 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1200 */ { PPC_VFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1202 */ { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1205 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1209 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1213 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1217 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1221 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1225 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1229 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 1233 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1237 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_VFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1240 */ { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1242 */ { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1245 */ { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1248 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1251 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1254 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1256 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1258 */ { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1260 */ { PPC_VFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1262 */ { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1264 */ { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1266 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_G8RCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1269 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1272 */ { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1276 */ { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1280 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1284 */ { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1288 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1290 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1293 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1296 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1298 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1301 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1304 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1308 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1311 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1315 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1318 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1322 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1325 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRpRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1329 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1333 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1336 */ { PPC_CRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1338 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1341 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1345 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1350 */ { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1353 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VRRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1356 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1360 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1361 */ { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1362 */ { PPC_VSSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1363 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 1365 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 1367 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1371 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1375 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSFRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1378 */ { PPC_ACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1379 */ { PPC_WACCRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 1380 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1384 */ { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { PPC_VSRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1387 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 1390 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1393 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 1397 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { PPC_CRBITRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
  }, {
    /* 0 */
    /* 0 */ PPC_CR0,
    /* 1 */ PPC_CR7,
    /* 2 */ PPC_XER,
    /* 3 */ PPC_XER, PPC_CR0,
    /* 5 */ PPC_CARRY,
    /* 6 */ PPC_CARRY, PPC_XER,
    /* 8 */ PPC_CARRY, PPC_XER, PPC_CR0,
    /* 11 */ PPC_CARRY, PPC_CR0,
    /* 13 */ PPC_CARRY, PPC_CARRY,
    /* 15 */ PPC_CARRY, PPC_CARRY, PPC_XER,
    /* 18 */ PPC_CARRY, PPC_CARRY, PPC_XER, PPC_CR0,
    /* 22 */ PPC_CARRY, PPC_CARRY, PPC_CR0,
    /* 25 */ PPC_X0, PPC_X3, PPC_X4, PPC_X5, PPC_X6, PPC_X7, PPC_X8, PPC_X9, PPC_X10, PPC_X11, PPC_X12, PPC_LR8, PPC_CTR8, PPC_CR0, PPC_CR1, PPC_CR5, PPC_CR6, PPC_CR7,
    /* 43 */ PPC_R0, PPC_R3, PPC_R4, PPC_R5, PPC_R6, PPC_R7, PPC_R8, PPC_R9, PPC_R10, PPC_R11, PPC_R12, PPC_LR, PPC_CTR, PPC_CR0, PPC_CR1, PPC_CR5, PPC_CR6, PPC_CR7,
    /* 61 */ PPC_R1, PPC_R1,
    /* 63 */ PPC_CTR,
    /* 64 */ PPC_CTR8,
    /* 65 */ PPC_CTR, PPC_RM, PPC_LR,
    /* 68 */ PPC_CTR8, PPC_RM, PPC_LR8,
    /* 71 */ PPC_RM, PPC_LR,
    /* 73 */ PPC_LR, PPC_RM,
    /* 75 */ PPC_LR, PPC_RM, PPC_LR,
    /* 78 */ PPC_CR6,
    /* 79 */ PPC_CTR8, PPC_RM, PPC_LR8, PPC_X2,
    /* 83 */ PPC_CTR8, PPC_RM, PPC_LR8, PPC_X2, PPC_RM,
    /* 88 */ PPC_CTR8, PPC_RM, PPC_LR8, PPC_RM,
    /* 92 */ PPC_CTR, PPC_RM, PPC_LR, PPC_R2,
    /* 96 */ PPC_CTR, PPC_RM, PPC_LR, PPC_R2, PPC_RM,
    /* 101 */ PPC_CTR, PPC_RM, PPC_LR, PPC_RM,
    /* 105 */ PPC_RM, PPC_LR8,
    /* 107 */ PPC_RM, PPC_LR8, PPC_RM,
    /* 110 */ PPC_RM, PPC_LR, PPC_RM,
    /* 113 */ PPC_LR8, PPC_RM,
    /* 115 */ PPC_CR1EQ,
    /* 116 */ PPC_CR1,
    /* 117 */ PPC_X1, PPC_X1,
    /* 119 */ PPC_CTR8, PPC_CTR8,
    /* 121 */ PPC_CTR, PPC_CTR,
    /* 123 */ PPC_RM,
    /* 124 */ PPC_RM, PPC_CR1,
    /* 126 */ PPC_X0, PPC_X4, PPC_X5, PPC_X6, PPC_X7, PPC_X8, PPC_X9, PPC_X10, PPC_X11, PPC_X12, PPC_LR8, PPC_CTR8, PPC_CR0, PPC_CR1, PPC_CR5, PPC_CR6, PPC_CR7,
    /* 143 */ PPC_R0, PPC_R4, PPC_R5, PPC_R6, PPC_R7, PPC_R8, PPC_R9, PPC_R10, PPC_R11, PPC_R12, PPC_LR, PPC_CTR, PPC_CR0, PPC_CR1, PPC_CR5, PPC_CR6, PPC_CR7,
    /* 160 */ PPC_R0, PPC_R4, PPC_R5, PPC_R11, PPC_LR, PPC_CR0,
    /* 166 */ PPC_X0, PPC_X4, PPC_X5, PPC_X11, PPC_LR8, PPC_CR0,
    /* 172 */ PPC_X0, PPC_X2, PPC_X4, PPC_X5, PPC_X6, PPC_X7, PPC_X8, PPC_X9, PPC_X10, PPC_X11, PPC_X12, PPC_LR8, PPC_CTR8, PPC_CR0, PPC_CR1, PPC_CR5, PPC_CR6, PPC_CR7,
    /* 190 */ PPC_R3, PPC_LR,
    /* 192 */ PPC_LR,
    /* 193 */ PPC_LR8,
    /* 194 */ PPC_RM, PPC_RM,
    /* 196 */ PPC_CTR, PPC_RM,
    /* 198 */ PPC_CTR8, PPC_RM,
    /* 200 */ PPC_RM, PPC_CR6,
    /* 202 */ PPC_CTR, PPC_RM, PPC_CTR,
    /* 205 */ PPC_CTR, PPC_LR, PPC_RM, PPC_CTR,
    /* 209 */ PPC_CTR, PPC_LR, PPC_RM, PPC_LR, PPC_CTR,
    /* 214 */ PPC_CTR, PPC_RM, PPC_LR, PPC_CTR,
  }
};

#endif // GET_INSTRINFO_MC_DESC

#ifdef GET_COMPUTE_FEATURES
#undef GET_COMPUTE_FEATURES

#endif // GET_COMPUTE_FEATURES

#ifdef GET_AVAILABLE_OPCODE_CHECKER
#undef GET_AVAILABLE_OPCODE_CHECKER

#endif // GET_AVAILABLE_OPCODE_CHECKER

#ifdef GET_INSTRMAP_INFO
#undef GET_INSTRMAP_INFO
enum IsVSXFMAAlt {
	IsVSXFMAAlt_1
};

enum RC {
	RC_0,
	RC_1
};

// getAltVSXFMAOpcode
int getAltVSXFMAOpcode(uint16_t Opcode) {
static const uint16_t getAltVSXFMAOpcodeTable[][2] = {
  { PPC_XSMADDADP, PPC_XSMADDMDP },
  { PPC_XSMADDASP, PPC_XSMADDMSP },
  { PPC_XSMSUBADP, PPC_XSMSUBMDP },
  { PPC_XSMSUBASP, PPC_XSMSUBMSP },
  { PPC_XSNMADDADP, PPC_XSNMADDMDP },
  { PPC_XSNMADDASP, PPC_XSNMADDMSP },
  { PPC_XSNMSUBADP, PPC_XSNMSUBMDP },
  { PPC_XSNMSUBASP, PPC_XSNMSUBMSP },
  { PPC_XVMADDADP, PPC_XVMADDMDP },
  { PPC_XVMADDASP, PPC_XVMADDMSP },
  { PPC_XVMSUBADP, PPC_XVMSUBMDP },
  { PPC_XVMSUBASP, PPC_XVMSUBMSP },
  { PPC_XVNMADDADP, PPC_XVNMADDMDP },
  { PPC_XVNMADDASP, PPC_XVNMADDMSP },
  { PPC_XVNMSUBADP, PPC_XVNMSUBMDP },
  { PPC_XVNMSUBASP, PPC_XVNMSUBMSP },
}; // End of getAltVSXFMAOpcodeTable

  unsigned mid;
  unsigned start = 0;
  unsigned end = 16;
  while (start < end) {
    mid = start + (end - start) / 2;
    if (Opcode == getAltVSXFMAOpcodeTable[mid][0]) {
      break;
    }
    if (Opcode < getAltVSXFMAOpcodeTable[mid][0])
      end = mid;
    else
      start = mid + 1;
  }
  if (start == end)
    return -1; // Instruction doesn't exist in this table.

  return getAltVSXFMAOpcodeTable[mid][1];
}

// getNonRecordFormOpcode
int getNonRecordFormOpcode(uint16_t Opcode) {
static const uint16_t getNonRecordFormOpcodeTable[][2] = {
  { PPC_ADD4O_rec, PPC_ADD4O },
  { PPC_ADD4_rec, PPC_ADD4 },
  { PPC_ADD8O_rec, PPC_ADD8O },
  { PPC_ADD8_rec, PPC_ADD8 },
  { PPC_ADDC8O_rec, PPC_ADDC8O },
  { PPC_ADDC8_rec, PPC_ADDC8 },
  { PPC_ADDCO_rec, PPC_ADDCO },
  { PPC_ADDC_rec, PPC_ADDC },
  { PPC_ADDE8O_rec, PPC_ADDE8O },
  { PPC_ADDE8_rec, PPC_ADDE8 },
  { PPC_ADDEO_rec, PPC_ADDEO },
  { PPC_ADDE_rec, PPC_ADDE },
  { PPC_ADDIC_rec, PPC_ADDIC },
  { PPC_ADDME8O_rec, PPC_ADDME8O },
  { PPC_ADDME8_rec, PPC_ADDME8 },
  { PPC_ADDMEO_rec, PPC_ADDMEO },
  { PPC_ADDME_rec, PPC_ADDME },
  { PPC_ADDZE8O_rec, PPC_ADDZE8O },
  { PPC_ADDZE8_rec, PPC_ADDZE8 },
  { PPC_ADDZEO_rec, PPC_ADDZEO },
  { PPC_ADDZE_rec, PPC_ADDZE },
  { PPC_AND8_rec, PPC_AND8 },
  { PPC_ANDC8_rec, PPC_ANDC8 },
  { PPC_ANDC_rec, PPC_ANDC },
  { PPC_AND_rec, PPC_AND },
  { PPC_CNTLZD_rec, PPC_CNTLZD },
  { PPC_CNTLZW8_rec, PPC_CNTLZW8 },
  { PPC_CNTLZW_rec, PPC_CNTLZW },
  { PPC_CNTTZD_rec, PPC_CNTTZD },
  { PPC_CNTTZW8_rec, PPC_CNTTZW8 },
  { PPC_CNTTZW_rec, PPC_CNTTZW },
  { PPC_DADDQ_rec, PPC_DADDQ },
  { PPC_DADD_rec, PPC_DADD },
  { PPC_DCFFIXQ_rec, PPC_DCFFIXQ },
  { PPC_DCFFIX_rec, PPC_DCFFIX },
  { PPC_DCTDP_rec, PPC_DCTDP },
  { PPC_DCTFIXQ_rec, PPC_DCTFIXQ },
  { PPC_DCTFIX_rec, PPC_DCTFIX },
  { PPC_DCTQPQ_rec, PPC_DCTQPQ },
  { PPC_DDEDPDQ_rec, PPC_DDEDPDQ },
  { PPC_DDEDPD_rec, PPC_DDEDPD },
  { PPC_DDIVQ_rec, PPC_DDIVQ },
  { PPC_DDIV_rec, PPC_DDIV },
  { PPC_DENBCDQ_rec, PPC_DENBCDQ },
  { PPC_DENBCD_rec, PPC_DENBCD },
  { PPC_DIEXQ_rec, PPC_DIEXQ },
  { PPC_DIEX_rec, PPC_DIEX },
  { PPC_DIVDEO_rec, PPC_DIVDEO },
  { PPC_DIVDEUO_rec, PPC_DIVDEUO },
  { PPC_DIVDEU_rec, PPC_DIVDEU },
  { PPC_DIVDE_rec, PPC_DIVDE },
  { PPC_DIVDO_rec, PPC_DIVDO },
  { PPC_DIVDUO_rec, PPC_DIVDUO },
  { PPC_DIVDU_rec, PPC_DIVDU },
  { PPC_DIVD_rec, PPC_DIVD },
  { PPC_DIVWEO_rec, PPC_DIVWEO },
  { PPC_DIVWEUO_rec, PPC_DIVWEUO },
  { PPC_DIVWEU_rec, PPC_DIVWEU },
  { PPC_DIVWE_rec, PPC_DIVWE },
  { PPC_DIVWO_rec, PPC_DIVWO },
  { PPC_DIVWUO_rec, PPC_DIVWUO },
  { PPC_DIVWU_rec, PPC_DIVWU },
  { PPC_DIVW_rec, PPC_DIVW },
  { PPC_DMULQ_rec, PPC_DMULQ },
  { PPC_DMUL_rec, PPC_DMUL },
  { PPC_DQUAIQ_rec, PPC_DQUAIQ },
  { PPC_DQUAI_rec, PPC_DQUAI },
  { PPC_DQUAQ_rec, PPC_DQUAQ },
  { PPC_DQUA_rec, PPC_DQUA },
  { PPC_DRDPQ_rec, PPC_DRDPQ },
  { PPC_DRINTNQ_rec, PPC_DRINTNQ },
  { PPC_DRINTN_rec, PPC_DRINTN },
  { PPC_DRINTXQ_rec, PPC_DRINTXQ },
  { PPC_DRINTX_rec, PPC_DRINTX },
  { PPC_DRRNDQ_rec, PPC_DRRNDQ },
  { PPC_DRRND_rec, PPC_DRRND },
  { PPC_DRSP_rec, PPC_DRSP },
  { PPC_DSCLIQ_rec, PPC_DSCLIQ },
  { PPC_DSCLI_rec, PPC_DSCLI },
  { PPC_DSCRIQ_rec, PPC_DSCRIQ },
  { PPC_DSCRI_rec, PPC_DSCRI },
  { PPC_DSUBQ_rec, PPC_DSUBQ },
  { PPC_DSUB_rec, PPC_DSUB },
  { PPC_DXEXQ_rec, PPC_DXEXQ },
  { PPC_DXEX_rec, PPC_DXEX },
  { PPC_EQV8_rec, PPC_EQV8 },
  { PPC_EQV_rec, PPC_EQV },
  { PPC_EXTSB8_rec, PPC_EXTSB8 },
  { PPC_EXTSB_rec, PPC_EXTSB },
  { PPC_EXTSH8_rec, PPC_EXTSH8 },
  { PPC_EXTSH_rec, PPC_EXTSH },
  { PPC_EXTSWSLI_32_64_rec, PPC_EXTSWSLI_32_64 },
  { PPC_EXTSWSLI_rec, PPC_EXTSWSLI },
  { PPC_EXTSW_32_64_rec, PPC_EXTSW_32_64 },
  { PPC_EXTSW_rec, PPC_EXTSW },
  { PPC_FABSD_rec, PPC_FABSD },
  { PPC_FABSS_rec, PPC_FABSS },
  { PPC_FADDS_rec, PPC_FADDS },
  { PPC_FADD_rec, PPC_FADD },
  { PPC_FCFIDS_rec, PPC_FCFIDS },
  { PPC_FCFIDUS_rec, PPC_FCFIDUS },
  { PPC_FCFIDU_rec, PPC_FCFIDU },
  { PPC_FCFID_rec, PPC_FCFID },
  { PPC_FCPSGND_rec, PPC_FCPSGND },
  { PPC_FCPSGNS_rec, PPC_FCPSGNS },
  { PPC_FCTIDUZ_rec, PPC_FCTIDUZ },
  { PPC_FCTIDU_rec, PPC_FCTIDU },
  { PPC_FCTIDZ_rec, PPC_FCTIDZ },
  { PPC_FCTID_rec, PPC_FCTID },
  { PPC_FCTIWUZ_rec, PPC_FCTIWUZ },
  { PPC_FCTIWU_rec, PPC_FCTIWU },
  { PPC_FCTIWZ_rec, PPC_FCTIWZ },
  { PPC_FCTIW_rec, PPC_FCTIW },
  { PPC_FDIVS_rec, PPC_FDIVS },
  { PPC_FDIV_rec, PPC_FDIV },
  { PPC_FMADDS_rec, PPC_FMADDS },
  { PPC_FMADD_rec, PPC_FMADD },
  { PPC_FMR_rec, PPC_FMR },
  { PPC_FMSUBS_rec, PPC_FMSUBS },
  { PPC_FMSUB_rec, PPC_FMSUB },
  { PPC_FMULS_rec, PPC_FMULS },
  { PPC_FMUL_rec, PPC_FMUL },
  { PPC_FNABSD_rec, PPC_FNABSD },
  { PPC_FNABSS_rec, PPC_FNABSS },
  { PPC_FNEGD_rec, PPC_FNEGD },
  { PPC_FNEGS_rec, PPC_FNEGS },
  { PPC_FNMADDS_rec, PPC_FNMADDS },
  { PPC_FNMADD_rec, PPC_FNMADD },
  { PPC_FNMSUBS_rec, PPC_FNMSUBS },
  { PPC_FNMSUB_rec, PPC_FNMSUB },
  { PPC_FRES_rec, PPC_FRES },
  { PPC_FRE_rec, PPC_FRE },
  { PPC_FRIMD_rec, PPC_FRIMD },
  { PPC_FRIMS_rec, PPC_FRIMS },
  { PPC_FRIND_rec, PPC_FRIND },
  { PPC_FRINS_rec, PPC_FRINS },
  { PPC_FRIPD_rec, PPC_FRIPD },
  { PPC_FRIPS_rec, PPC_FRIPS },
  { PPC_FRIZD_rec, PPC_FRIZD },
  { PPC_FRIZS_rec, PPC_FRIZS },
  { PPC_FRSP_rec, PPC_FRSP },
  { PPC_FRSQRTES_rec, PPC_FRSQRTES },
  { PPC_FRSQRTE_rec, PPC_FRSQRTE },
  { PPC_FSELD_rec, PPC_FSELD },
  { PPC_FSELS_rec, PPC_FSELS },
  { PPC_FSQRTS_rec, PPC_FSQRTS },
  { PPC_FSQRT_rec, PPC_FSQRT },
  { PPC_FSUBS_rec, PPC_FSUBS },
  { PPC_FSUB_rec, PPC_FSUB },
  { PPC_MULHDU_rec, PPC_MULHDU },
  { PPC_MULHD_rec, PPC_MULHD },
  { PPC_MULHWU_rec, PPC_MULHWU },
  { PPC_MULHW_rec, PPC_MULHW },
  { PPC_MULLDO_rec, PPC_MULLDO },
  { PPC_MULLD_rec, PPC_MULLD },
  { PPC_MULLWO_rec, PPC_MULLWO },
  { PPC_MULLW_rec, PPC_MULLW },
  { PPC_NAND8_rec, PPC_NAND8 },
  { PPC_NAND_rec, PPC_NAND },
  { PPC_NEG8O_rec, PPC_NEG8O },
  { PPC_NEG8_rec, PPC_NEG8 },
  { PPC_NEGO_rec, PPC_NEGO },
  { PPC_NEG_rec, PPC_NEG },
  { PPC_NOR8_rec, PPC_NOR8 },
  { PPC_NOR_rec, PPC_NOR },
  { PPC_OR8_rec, PPC_OR8 },
  { PPC_ORC8_rec, PPC_ORC8 },
  { PPC_ORC_rec, PPC_ORC },
  { PPC_OR_rec, PPC_OR },
  { PPC_RLDCL_rec, PPC_RLDCL },
  { PPC_RLDCR_rec, PPC_RLDCR },
  { PPC_RLDICL_32_rec, PPC_RLDICL_32 },
  { PPC_RLDICL_rec, PPC_RLDICL },
  { PPC_RLDICR_rec, PPC_RLDICR },
  { PPC_RLDIC_rec, PPC_RLDIC },
  { PPC_RLDIMI_rec, PPC_RLDIMI },
  { PPC_RLWIMI8_rec, PPC_RLWIMI8 },
  { PPC_RLWIMI_rec, PPC_RLWIMI },
  { PPC_RLWINM8_rec, PPC_RLWINM8 },
  { PPC_RLWINM_rec, PPC_RLWINM },
  { PPC_RLWNM8_rec, PPC_RLWNM8 },
  { PPC_RLWNM_rec, PPC_RLWNM },
  { PPC_SLD_rec, PPC_SLD },
  { PPC_SLW8_rec, PPC_SLW8 },
  { PPC_SLW_rec, PPC_SLW },
  { PPC_SRADI_rec, PPC_SRADI },
  { PPC_SRAD_rec, PPC_SRAD },
  { PPC_SRAWI_rec, PPC_SRAWI },
  { PPC_SRAW_rec, PPC_SRAW },
  { PPC_SRD_rec, PPC_SRD },
  { PPC_SRW8_rec, PPC_SRW8 },
  { PPC_SRW_rec, PPC_SRW },
  { PPC_SUBF8O_rec, PPC_SUBF8O },
  { PPC_SUBF8_rec, PPC_SUBF8 },
  { PPC_SUBFC8O_rec, PPC_SUBFC8O },
  { PPC_SUBFC8_rec, PPC_SUBFC8 },
  { PPC_SUBFCO_rec, PPC_SUBFCO },
  { PPC_SUBFC_rec, PPC_SUBFC },
  { PPC_SUBFE8O_rec, PPC_SUBFE8O },
  { PPC_SUBFE8_rec, PPC_SUBFE8 },
  { PPC_SUBFEO_rec, PPC_SUBFEO },
  { PPC_SUBFE_rec, PPC_SUBFE },
  { PPC_SUBFME8O_rec, PPC_SUBFME8O },
  { PPC_SUBFME8_rec, PPC_SUBFME8 },
  { PPC_SUBFMEO_rec, PPC_SUBFMEO },
  { PPC_SUBFME_rec, PPC_SUBFME },
  { PPC_SUBFO_rec, PPC_SUBFO },
  { PPC_SUBFUS_rec, PPC_SUBFUS },
  { PPC_SUBFZE8O_rec, PPC_SUBFZE8O },
  { PPC_SUBFZE8_rec, PPC_SUBFZE8 },
  { PPC_SUBFZEO_rec, PPC_SUBFZEO },
  { PPC_SUBFZE_rec, PPC_SUBFZE },
  { PPC_SUBF_rec, PPC_SUBF },
  { PPC_VSTRIBL_rec, PPC_VSTRIBL },
  { PPC_VSTRIBR_rec, PPC_VSTRIBR },
  { PPC_VSTRIHL_rec, PPC_VSTRIHL },
  { PPC_VSTRIHR_rec, PPC_VSTRIHR },
  { PPC_XOR8_rec, PPC_XOR8 },
  { PPC_XOR_rec, PPC_XOR },
}; // End of getNonRecordFormOpcodeTable

  unsigned mid;
  unsigned start = 0;
  unsigned end = 219;
  while (start < end) {
    mid = start + (end - start) / 2;
    if (Opcode == getNonRecordFormOpcodeTable[mid][0]) {
      break;
    }
    if (Opcode < getNonRecordFormOpcodeTable[mid][0])
      end = mid;
    else
      start = mid + 1;
  }
  if (start == end)
    return -1; // Instruction doesn't exist in this table.

  return getNonRecordFormOpcodeTable[mid][1];
}

// getRecordFormOpcode
int getRecordFormOpcode(uint16_t Opcode) {
static const uint16_t getRecordFormOpcodeTable[][2] = {
  { PPC_ADD4, PPC_ADD4_rec },
  { PPC_ADD4O, PPC_ADD4O_rec },
  { PPC_ADD8, PPC_ADD8_rec },
  { PPC_ADD8O, PPC_ADD8O_rec },
  { PPC_ADDC, PPC_ADDC_rec },
  { PPC_ADDC8, PPC_ADDC8_rec },
  { PPC_ADDC8O, PPC_ADDC8O_rec },
  { PPC_ADDCO, PPC_ADDCO_rec },
  { PPC_ADDE, PPC_ADDE_rec },
  { PPC_ADDE8, PPC_ADDE8_rec },
  { PPC_ADDE8O, PPC_ADDE8O_rec },
  { PPC_ADDEO, PPC_ADDEO_rec },
  { PPC_ADDIC, PPC_ADDIC_rec },
  { PPC_ADDME, PPC_ADDME_rec },
  { PPC_ADDME8, PPC_ADDME8_rec },
  { PPC_ADDME8O, PPC_ADDME8O_rec },
  { PPC_ADDMEO, PPC_ADDMEO_rec },
  { PPC_ADDZE, PPC_ADDZE_rec },
  { PPC_ADDZE8, PPC_ADDZE8_rec },
  { PPC_ADDZE8O, PPC_ADDZE8O_rec },
  { PPC_ADDZEO, PPC_ADDZEO_rec },
  { PPC_AND, PPC_AND_rec },
  { PPC_AND8, PPC_AND8_rec },
  { PPC_ANDC, PPC_ANDC_rec },
  { PPC_ANDC8, PPC_ANDC8_rec },
  { PPC_CNTLZD, PPC_CNTLZD_rec },
  { PPC_CNTLZW, PPC_CNTLZW_rec },
  { PPC_CNTLZW8, PPC_CNTLZW8_rec },
  { PPC_CNTTZD, PPC_CNTTZD_rec },
  { PPC_CNTTZW, PPC_CNTTZW_rec },
  { PPC_CNTTZW8, PPC_CNTTZW8_rec },
  { PPC_DADD, PPC_DADD_rec },
  { PPC_DADDQ, PPC_DADDQ_rec },
  { PPC_DCFFIX, PPC_DCFFIX_rec },
  { PPC_DCFFIXQ, PPC_DCFFIXQ_rec },
  { PPC_DCTDP, PPC_DCTDP_rec },
  { PPC_DCTFIX, PPC_DCTFIX_rec },
  { PPC_DCTFIXQ, PPC_DCTFIXQ_rec },
  { PPC_DCTQPQ, PPC_DCTQPQ_rec },
  { PPC_DDEDPD, PPC_DDEDPD_rec },
  { PPC_DDEDPDQ, PPC_DDEDPDQ_rec },
  { PPC_DDIV, PPC_DDIV_rec },
  { PPC_DDIVQ, PPC_DDIVQ_rec },
  { PPC_DENBCD, PPC_DENBCD_rec },
  { PPC_DENBCDQ, PPC_DENBCDQ_rec },
  { PPC_DIEX, PPC_DIEX_rec },
  { PPC_DIEXQ, PPC_DIEXQ_rec },
  { PPC_DIVD, PPC_DIVD_rec },
  { PPC_DIVDE, PPC_DIVDE_rec },
  { PPC_DIVDEO, PPC_DIVDEO_rec },
  { PPC_DIVDEU, PPC_DIVDEU_rec },
  { PPC_DIVDEUO, PPC_DIVDEUO_rec },
  { PPC_DIVDO, PPC_DIVDO_rec },
  { PPC_DIVDU, PPC_DIVDU_rec },
  { PPC_DIVDUO, PPC_DIVDUO_rec },
  { PPC_DIVW, PPC_DIVW_rec },
  { PPC_DIVWE, PPC_DIVWE_rec },
  { PPC_DIVWEO, PPC_DIVWEO_rec },
  { PPC_DIVWEU, PPC_DIVWEU_rec },
  { PPC_DIVWEUO, PPC_DIVWEUO_rec },
  { PPC_DIVWO, PPC_DIVWO_rec },
  { PPC_DIVWU, PPC_DIVWU_rec },
  { PPC_DIVWUO, PPC_DIVWUO_rec },
  { PPC_DMUL, PPC_DMUL_rec },
  { PPC_DMULQ, PPC_DMULQ_rec },
  { PPC_DQUA, PPC_DQUA_rec },
  { PPC_DQUAI, PPC_DQUAI_rec },
  { PPC_DQUAIQ, PPC_DQUAIQ_rec },
  { PPC_DQUAQ, PPC_DQUAQ_rec },
  { PPC_DRDPQ, PPC_DRDPQ_rec },
  { PPC_DRINTN, PPC_DRINTN_rec },
  { PPC_DRINTNQ, PPC_DRINTNQ_rec },
  { PPC_DRINTX, PPC_DRINTX_rec },
  { PPC_DRINTXQ, PPC_DRINTXQ_rec },
  { PPC_DRRND, PPC_DRRND_rec },
  { PPC_DRRNDQ, PPC_DRRNDQ_rec },
  { PPC_DRSP, PPC_DRSP_rec },
  { PPC_DSCLI, PPC_DSCLI_rec },
  { PPC_DSCLIQ, PPC_DSCLIQ_rec },
  { PPC_DSCRI, PPC_DSCRI_rec },
  { PPC_DSCRIQ, PPC_DSCRIQ_rec },
  { PPC_DSUB, PPC_DSUB_rec },
  { PPC_DSUBQ, PPC_DSUBQ_rec },
  { PPC_DXEX, PPC_DXEX_rec },
  { PPC_DXEXQ, PPC_DXEXQ_rec },
  { PPC_EQV, PPC_EQV_rec },
  { PPC_EQV8, PPC_EQV8_rec },
  { PPC_EXTSB, PPC_EXTSB_rec },
  { PPC_EXTSB8, PPC_EXTSB8_rec },
  { PPC_EXTSH, PPC_EXTSH_rec },
  { PPC_EXTSH8, PPC_EXTSH8_rec },
  { PPC_EXTSW, PPC_EXTSW_rec },
  { PPC_EXTSWSLI, PPC_EXTSWSLI_rec },
  { PPC_EXTSWSLI_32_64, PPC_EXTSWSLI_32_64_rec },
  { PPC_EXTSW_32_64, PPC_EXTSW_32_64_rec },
  { PPC_FABSD, PPC_FABSD_rec },
  { PPC_FABSS, PPC_FABSS_rec },
  { PPC_FADD, PPC_FADD_rec },
  { PPC_FADDS, PPC_FADDS_rec },
  { PPC_FCFID, PPC_FCFID_rec },
  { PPC_FCFIDS, PPC_FCFIDS_rec },
  { PPC_FCFIDU, PPC_FCFIDU_rec },
  { PPC_FCFIDUS, PPC_FCFIDUS_rec },
  { PPC_FCPSGND, PPC_FCPSGND_rec },
  { PPC_FCPSGNS, PPC_FCPSGNS_rec },
  { PPC_FCTID, PPC_FCTID_rec },
  { PPC_FCTIDU, PPC_FCTIDU_rec },
  { PPC_FCTIDUZ, PPC_FCTIDUZ_rec },
  { PPC_FCTIDZ, PPC_FCTIDZ_rec },
  { PPC_FCTIW, PPC_FCTIW_rec },
  { PPC_FCTIWU, PPC_FCTIWU_rec },
  { PPC_FCTIWUZ, PPC_FCTIWUZ_rec },
  { PPC_FCTIWZ, PPC_FCTIWZ_rec },
  { PPC_FDIV, PPC_FDIV_rec },
  { PPC_FDIVS, PPC_FDIVS_rec },
  { PPC_FMADD, PPC_FMADD_rec },
  { PPC_FMADDS, PPC_FMADDS_rec },
  { PPC_FMR, PPC_FMR_rec },
  { PPC_FMSUB, PPC_FMSUB_rec },
  { PPC_FMSUBS, PPC_FMSUBS_rec },
  { PPC_FMUL, PPC_FMUL_rec },
  { PPC_FMULS, PPC_FMULS_rec },
  { PPC_FNABSD, PPC_FNABSD_rec },
  { PPC_FNABSS, PPC_FNABSS_rec },
  { PPC_FNEGD, PPC_FNEGD_rec },
  { PPC_FNEGS, PPC_FNEGS_rec },
  { PPC_FNMADD, PPC_FNMADD_rec },
  { PPC_FNMADDS, PPC_FNMADDS_rec },
  { PPC_FNMSUB, PPC_FNMSUB_rec },
  { PPC_FNMSUBS, PPC_FNMSUBS_rec },
  { PPC_FRE, PPC_FRE_rec },
  { PPC_FRES, PPC_FRES_rec },
  { PPC_FRIMD, PPC_FRIMD_rec },
  { PPC_FRIMS, PPC_FRIMS_rec },
  { PPC_FRIND, PPC_FRIND_rec },
  { PPC_FRINS, PPC_FRINS_rec },
  { PPC_FRIPD, PPC_FRIPD_rec },
  { PPC_FRIPS, PPC_FRIPS_rec },
  { PPC_FRIZD, PPC_FRIZD_rec },
  { PPC_FRIZS, PPC_FRIZS_rec },
  { PPC_FRSP, PPC_FRSP_rec },
  { PPC_FRSQRTE, PPC_FRSQRTE_rec },
  { PPC_FRSQRTES, PPC_FRSQRTES_rec },
  { PPC_FSELD, PPC_FSELD_rec },
  { PPC_FSELS, PPC_FSELS_rec },
  { PPC_FSQRT, PPC_FSQRT_rec },
  { PPC_FSQRTS, PPC_FSQRTS_rec },
  { PPC_FSUB, PPC_FSUB_rec },
  { PPC_FSUBS, PPC_FSUBS_rec },
  { PPC_MULHD, PPC_MULHD_rec },
  { PPC_MULHDU, PPC_MULHDU_rec },
  { PPC_MULHW, PPC_MULHW_rec },
  { PPC_MULHWU, PPC_MULHWU_rec },
  { PPC_MULLD, PPC_MULLD_rec },
  { PPC_MULLDO, PPC_MULLDO_rec },
  { PPC_MULLW, PPC_MULLW_rec },
  { PPC_MULLWO, PPC_MULLWO_rec },
  { PPC_NAND, PPC_NAND_rec },
  { PPC_NAND8, PPC_NAND8_rec },
  { PPC_NEG, PPC_NEG_rec },
  { PPC_NEG8, PPC_NEG8_rec },
  { PPC_NEG8O, PPC_NEG8O_rec },
  { PPC_NEGO, PPC_NEGO_rec },
  { PPC_NOR, PPC_NOR_rec },
  { PPC_NOR8, PPC_NOR8_rec },
  { PPC_OR, PPC_OR_rec },
  { PPC_OR8, PPC_OR8_rec },
  { PPC_ORC, PPC_ORC_rec },
  { PPC_ORC8, PPC_ORC8_rec },
  { PPC_RLDCL, PPC_RLDCL_rec },
  { PPC_RLDCR, PPC_RLDCR_rec },
  { PPC_RLDIC, PPC_RLDIC_rec },
  { PPC_RLDICL, PPC_RLDICL_rec },
  { PPC_RLDICL_32, PPC_RLDICL_32_rec },
  { PPC_RLDICR, PPC_RLDICR_rec },
  { PPC_RLDIMI, PPC_RLDIMI_rec },
  { PPC_RLWIMI, PPC_RLWIMI_rec },
  { PPC_RLWIMI8, PPC_RLWIMI8_rec },
  { PPC_RLWINM, PPC_RLWINM_rec },
  { PPC_RLWINM8, PPC_RLWINM8_rec },
  { PPC_RLWNM, PPC_RLWNM_rec },
  { PPC_RLWNM8, PPC_RLWNM8_rec },
  { PPC_SLD, PPC_SLD_rec },
  { PPC_SLW, PPC_SLW_rec },
  { PPC_SLW8, PPC_SLW8_rec },
  { PPC_SRAD, PPC_SRAD_rec },
  { PPC_SRADI, PPC_SRADI_rec },
  { PPC_SRAW, PPC_SRAW_rec },
  { PPC_SRAWI, PPC_SRAWI_rec },
  { PPC_SRD, PPC_SRD_rec },
  { PPC_SRW, PPC_SRW_rec },
  { PPC_SRW8, PPC_SRW8_rec },
  { PPC_SUBF, PPC_SUBF_rec },
  { PPC_SUBF8, PPC_SUBF8_rec },
  { PPC_SUBF8O, PPC_SUBF8O_rec },
  { PPC_SUBFC, PPC_SUBFC_rec },
  { PPC_SUBFC8, PPC_SUBFC8_rec },
  { PPC_SUBFC8O, PPC_SUBFC8O_rec },
  { PPC_SUBFCO, PPC_SUBFCO_rec },
  { PPC_SUBFE, PPC_SUBFE_rec },
  { PPC_SUBFE8, PPC_SUBFE8_rec },
  { PPC_SUBFE8O, PPC_SUBFE8O_rec },
  { PPC_SUBFEO, PPC_SUBFEO_rec },
  { PPC_SUBFME, PPC_SUBFME_rec },
  { PPC_SUBFME8, PPC_SUBFME8_rec },
  { PPC_SUBFME8O, PPC_SUBFME8O_rec },
  { PPC_SUBFMEO, PPC_SUBFMEO_rec },
  { PPC_SUBFO, PPC_SUBFO_rec },
  { PPC_SUBFUS, PPC_SUBFUS_rec },
  { PPC_SUBFZE, PPC_SUBFZE_rec },
  { PPC_SUBFZE8, PPC_SUBFZE8_rec },
  { PPC_SUBFZE8O, PPC_SUBFZE8O_rec },
  { PPC_SUBFZEO, PPC_SUBFZEO_rec },
  { PPC_VSTRIBL, PPC_VSTRIBL_rec },
  { PPC_VSTRIBR, PPC_VSTRIBR_rec },
  { PPC_VSTRIHL, PPC_VSTRIHL_rec },
  { PPC_VSTRIHR, PPC_VSTRIHR_rec },
  { PPC_XOR, PPC_XOR_rec },
  { PPC_XOR8, PPC_XOR8_rec },
}; // End of getRecordFormOpcodeTable

  unsigned mid;
  unsigned start = 0;
  unsigned end = 219;
  while (start < end) {
    mid = start + (end - start) / 2;
    if (Opcode == getRecordFormOpcodeTable[mid][0]) {
      break;
    }
    if (Opcode < getRecordFormOpcodeTable[mid][0])
      end = mid;
    else
      start = mid + 1;
  }
  if (start == end)
    return -1; // Instruction doesn't exist in this table.

  return getRecordFormOpcodeTable[mid][1];
}

#endif // GET_INSTRMAP_INFO

