/* Capstone Disassembly Engine, https://www.capstone-engine.org */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2022, */
/*    Rot127 <unisono@quyllur.org> 2022-2024 */
/* Automatically generated file by Capstone's LLVM TableGen Disassembler Backend. */

/* LLVM-commit: <commit> */
/* LLVM-tag: <tag> */

/* Do not edit. */

/* Capstone's LLVM TableGen Backends: */
/* https://github.com/capstone-engine/llvm-capstone */

#ifdef GET_INSTRINFO_ENUM
#undef GET_INSTRINFO_ENUM

  enum {
    RISCV_PHI	= 0,
    RISCV_INLINEASM	= 1,
    RISCV_INLINEASM_BR	= 2,
    RISCV_CFI_INSTRUCTION	= 3,
    RISCV_EH_LABEL	= 4,
    RISCV_GC_LABEL	= 5,
    RISCV_ANNOTATION_LABEL	= 6,
    RISCV_KILL	= 7,
    RISCV_EXTRACT_SUBREG	= 8,
    RISCV_INSERT_SUBREG	= 9,
    RISCV_IMPLICIT_DEF	= 10,
    RISCV_SUBREG_TO_REG	= 11,
    RISCV_COPY_TO_REGCLASS	= 12,
    RISCV_DBG_VALUE	= 13,
    RISCV_DBG_VALUE_LIST	= 14,
    RISCV_DBG_INSTR_REF	= 15,
    RISCV_DBG_PHI	= 16,
    RISCV_DBG_LABEL	= 17,
    RISCV_REG_SEQUENCE	= 18,
    RISCV_COPY	= 19,
    RISCV_BUNDLE	= 20,
    RISCV_LIFETIME_START	= 21,
    RISCV_LIFETIME_END	= 22,
    RISCV_PSEUDO_PROBE	= 23,
    RISCV_ARITH_FENCE	= 24,
    RISCV_STACKMAP	= 25,
    RISCV_FENTRY_CALL	= 26,
    RISCV_PATCHPOINT	= 27,
    RISCV_LOAD_STACK_GUARD	= 28,
    RISCV_PREALLOCATED_SETUP	= 29,
    RISCV_PREALLOCATED_ARG	= 30,
    RISCV_STATEPOINT	= 31,
    RISCV_LOCAL_ESCAPE	= 32,
    RISCV_FAULTING_OP	= 33,
    RISCV_PATCHABLE_OP	= 34,
    RISCV_PATCHABLE_FUNCTION_ENTER	= 35,
    RISCV_PATCHABLE_RET	= 36,
    RISCV_PATCHABLE_FUNCTION_EXIT	= 37,
    RISCV_PATCHABLE_TAIL_CALL	= 38,
    RISCV_PATCHABLE_EVENT_CALL	= 39,
    RISCV_PATCHABLE_TYPED_EVENT_CALL	= 40,
    RISCV_ICALL_BRANCH_FUNNEL	= 41,
    RISCV_MEMBARRIER	= 42,
    RISCV_JUMP_TABLE_DEBUG_INFO	= 43,
    RISCV_G_ASSERT_SEXT	= 44,
    RISCV_G_ASSERT_ZEXT	= 45,
    RISCV_G_ASSERT_ALIGN	= 46,
    RISCV_G_ADD	= 47,
    RISCV_G_SUB	= 48,
    RISCV_G_MUL	= 49,
    RISCV_G_SDIV	= 50,
    RISCV_G_UDIV	= 51,
    RISCV_G_SREM	= 52,
    RISCV_G_UREM	= 53,
    RISCV_G_SDIVREM	= 54,
    RISCV_G_UDIVREM	= 55,
    RISCV_G_AND	= 56,
    RISCV_G_OR	= 57,
    RISCV_G_XOR	= 58,
    RISCV_G_IMPLICIT_DEF	= 59,
    RISCV_G_PHI	= 60,
    RISCV_G_FRAME_INDEX	= 61,
    RISCV_G_GLOBAL_VALUE	= 62,
    RISCV_G_CONSTANT_POOL	= 63,
    RISCV_G_EXTRACT	= 64,
    RISCV_G_UNMERGE_VALUES	= 65,
    RISCV_G_INSERT	= 66,
    RISCV_G_MERGE_VALUES	= 67,
    RISCV_G_BUILD_VECTOR	= 68,
    RISCV_G_BUILD_VECTOR_TRUNC	= 69,
    RISCV_G_CONCAT_VECTORS	= 70,
    RISCV_G_PTRTOINT	= 71,
    RISCV_G_INTTOPTR	= 72,
    RISCV_G_BITCAST	= 73,
    RISCV_G_FREEZE	= 74,
    RISCV_G_CONSTANT_FOLD_BARRIER	= 75,
    RISCV_G_INTRINSIC_FPTRUNC_ROUND	= 76,
    RISCV_G_INTRINSIC_TRUNC	= 77,
    RISCV_G_INTRINSIC_ROUND	= 78,
    RISCV_G_INTRINSIC_LRINT	= 79,
    RISCV_G_INTRINSIC_ROUNDEVEN	= 80,
    RISCV_G_READCYCLECOUNTER	= 81,
    RISCV_G_LOAD	= 82,
    RISCV_G_SEXTLOAD	= 83,
    RISCV_G_ZEXTLOAD	= 84,
    RISCV_G_INDEXED_LOAD	= 85,
    RISCV_G_INDEXED_SEXTLOAD	= 86,
    RISCV_G_INDEXED_ZEXTLOAD	= 87,
    RISCV_G_STORE	= 88,
    RISCV_G_INDEXED_STORE	= 89,
    RISCV_G_ATOMIC_CMPXCHG_WITH_SUCCESS	= 90,
    RISCV_G_ATOMIC_CMPXCHG	= 91,
    RISCV_G_ATOMICRMW_XCHG	= 92,
    RISCV_G_ATOMICRMW_ADD	= 93,
    RISCV_G_ATOMICRMW_SUB	= 94,
    RISCV_G_ATOMICRMW_AND	= 95,
    RISCV_G_ATOMICRMW_NAND	= 96,
    RISCV_G_ATOMICRMW_OR	= 97,
    RISCV_G_ATOMICRMW_XOR	= 98,
    RISCV_G_ATOMICRMW_MAX	= 99,
    RISCV_G_ATOMICRMW_MIN	= 100,
    RISCV_G_ATOMICRMW_UMAX	= 101,
    RISCV_G_ATOMICRMW_UMIN	= 102,
    RISCV_G_ATOMICRMW_FADD	= 103,
    RISCV_G_ATOMICRMW_FSUB	= 104,
    RISCV_G_ATOMICRMW_FMAX	= 105,
    RISCV_G_ATOMICRMW_FMIN	= 106,
    RISCV_G_ATOMICRMW_UINC_WRAP	= 107,
    RISCV_G_ATOMICRMW_UDEC_WRAP	= 108,
    RISCV_G_FENCE	= 109,
    RISCV_G_PREFETCH	= 110,
    RISCV_G_BRCOND	= 111,
    RISCV_G_BRINDIRECT	= 112,
    RISCV_G_INVOKE_REGION_START	= 113,
    RISCV_G_INTRINSIC	= 114,
    RISCV_G_INTRINSIC_W_SIDE_EFFECTS	= 115,
    RISCV_G_INTRINSIC_CONVERGENT	= 116,
    RISCV_G_INTRINSIC_CONVERGENT_W_SIDE_EFFECTS	= 117,
    RISCV_G_ANYEXT	= 118,
    RISCV_G_TRUNC	= 119,
    RISCV_G_CONSTANT	= 120,
    RISCV_G_FCONSTANT	= 121,
    RISCV_G_VASTART	= 122,
    RISCV_G_VAARG	= 123,
    RISCV_G_SEXT	= 124,
    RISCV_G_SEXT_INREG	= 125,
    RISCV_G_ZEXT	= 126,
    RISCV_G_SHL	= 127,
    RISCV_G_LSHR	= 128,
    RISCV_G_ASHR	= 129,
    RISCV_G_FSHL	= 130,
    RISCV_G_FSHR	= 131,
    RISCV_G_ROTR	= 132,
    RISCV_G_ROTL	= 133,
    RISCV_G_ICMP	= 134,
    RISCV_G_FCMP	= 135,
    RISCV_G_SELECT	= 136,
    RISCV_G_UADDO	= 137,
    RISCV_G_UADDE	= 138,
    RISCV_G_USUBO	= 139,
    RISCV_G_USUBE	= 140,
    RISCV_G_SADDO	= 141,
    RISCV_G_SADDE	= 142,
    RISCV_G_SSUBO	= 143,
    RISCV_G_SSUBE	= 144,
    RISCV_G_UMULO	= 145,
    RISCV_G_SMULO	= 146,
    RISCV_G_UMULH	= 147,
    RISCV_G_SMULH	= 148,
    RISCV_G_UADDSAT	= 149,
    RISCV_G_SADDSAT	= 150,
    RISCV_G_USUBSAT	= 151,
    RISCV_G_SSUBSAT	= 152,
    RISCV_G_USHLSAT	= 153,
    RISCV_G_SSHLSAT	= 154,
    RISCV_G_SMULFIX	= 155,
    RISCV_G_UMULFIX	= 156,
    RISCV_G_SMULFIXSAT	= 157,
    RISCV_G_UMULFIXSAT	= 158,
    RISCV_G_SDIVFIX	= 159,
    RISCV_G_UDIVFIX	= 160,
    RISCV_G_SDIVFIXSAT	= 161,
    RISCV_G_UDIVFIXSAT	= 162,
    RISCV_G_FADD	= 163,
    RISCV_G_FSUB	= 164,
    RISCV_G_FMUL	= 165,
    RISCV_G_FMA	= 166,
    RISCV_G_FMAD	= 167,
    RISCV_G_FDIV	= 168,
    RISCV_G_FREM	= 169,
    RISCV_G_FPOW	= 170,
    RISCV_G_FPOWI	= 171,
    RISCV_G_FEXP	= 172,
    RISCV_G_FEXP2	= 173,
    RISCV_G_FEXP10	= 174,
    RISCV_G_FLOG	= 175,
    RISCV_G_FLOG2	= 176,
    RISCV_G_FLOG10	= 177,
    RISCV_G_FLDEXP	= 178,
    RISCV_G_FFREXP	= 179,
    RISCV_G_FNEG	= 180,
    RISCV_G_FPEXT	= 181,
    RISCV_G_FPTRUNC	= 182,
    RISCV_G_FPTOSI	= 183,
    RISCV_G_FPTOUI	= 184,
    RISCV_G_SITOFP	= 185,
    RISCV_G_UITOFP	= 186,
    RISCV_G_FABS	= 187,
    RISCV_G_FCOPYSIGN	= 188,
    RISCV_G_IS_FPCLASS	= 189,
    RISCV_G_FCANONICALIZE	= 190,
    RISCV_G_FMINNUM	= 191,
    RISCV_G_FMAXNUM	= 192,
    RISCV_G_FMINNUM_IEEE	= 193,
    RISCV_G_FMAXNUM_IEEE	= 194,
    RISCV_G_FMINIMUM	= 195,
    RISCV_G_FMAXIMUM	= 196,
    RISCV_G_GET_FPENV	= 197,
    RISCV_G_SET_FPENV	= 198,
    RISCV_G_RESET_FPENV	= 199,
    RISCV_G_GET_FPMODE	= 200,
    RISCV_G_SET_FPMODE	= 201,
    RISCV_G_RESET_FPMODE	= 202,
    RISCV_G_PTR_ADD	= 203,
    RISCV_G_PTRMASK	= 204,
    RISCV_G_SMIN	= 205,
    RISCV_G_SMAX	= 206,
    RISCV_G_UMIN	= 207,
    RISCV_G_UMAX	= 208,
    RISCV_G_ABS	= 209,
    RISCV_G_LROUND	= 210,
    RISCV_G_LLROUND	= 211,
    RISCV_G_BR	= 212,
    RISCV_G_BRJT	= 213,
    RISCV_G_INSERT_VECTOR_ELT	= 214,
    RISCV_G_EXTRACT_VECTOR_ELT	= 215,
    RISCV_G_SHUFFLE_VECTOR	= 216,
    RISCV_G_CTTZ	= 217,
    RISCV_G_CTTZ_ZERO_UNDEF	= 218,
    RISCV_G_CTLZ	= 219,
    RISCV_G_CTLZ_ZERO_UNDEF	= 220,
    RISCV_G_CTPOP	= 221,
    RISCV_G_BSWAP	= 222,
    RISCV_G_BITREVERSE	= 223,
    RISCV_G_FCEIL	= 224,
    RISCV_G_FCOS	= 225,
    RISCV_G_FSIN	= 226,
    RISCV_G_FSQRT	= 227,
    RISCV_G_FFLOOR	= 228,
    RISCV_G_FRINT	= 229,
    RISCV_G_FNEARBYINT	= 230,
    RISCV_G_ADDRSPACE_CAST	= 231,
    RISCV_G_BLOCK_ADDR	= 232,
    RISCV_G_JUMP_TABLE	= 233,
    RISCV_G_DYN_STACKALLOC	= 234,
    RISCV_G_STACKSAVE	= 235,
    RISCV_G_STACKRESTORE	= 236,
    RISCV_G_STRICT_FADD	= 237,
    RISCV_G_STRICT_FSUB	= 238,
    RISCV_G_STRICT_FMUL	= 239,
    RISCV_G_STRICT_FDIV	= 240,
    RISCV_G_STRICT_FREM	= 241,
    RISCV_G_STRICT_FMA	= 242,
    RISCV_G_STRICT_FSQRT	= 243,
    RISCV_G_STRICT_FLDEXP	= 244,
    RISCV_G_READ_REGISTER	= 245,
    RISCV_G_WRITE_REGISTER	= 246,
    RISCV_G_MEMCPY	= 247,
    RISCV_G_MEMCPY_INLINE	= 248,
    RISCV_G_MEMMOVE	= 249,
    RISCV_G_MEMSET	= 250,
    RISCV_G_BZERO	= 251,
    RISCV_G_VECREDUCE_SEQ_FADD	= 252,
    RISCV_G_VECREDUCE_SEQ_FMUL	= 253,
    RISCV_G_VECREDUCE_FADD	= 254,
    RISCV_G_VECREDUCE_FMUL	= 255,
    RISCV_G_VECREDUCE_FMAX	= 256,
    RISCV_G_VECREDUCE_FMIN	= 257,
    RISCV_G_VECREDUCE_FMAXIMUM	= 258,
    RISCV_G_VECREDUCE_FMINIMUM	= 259,
    RISCV_G_VECREDUCE_ADD	= 260,
    RISCV_G_VECREDUCE_MUL	= 261,
    RISCV_G_VECREDUCE_AND	= 262,
    RISCV_G_VECREDUCE_OR	= 263,
    RISCV_G_VECREDUCE_XOR	= 264,
    RISCV_G_VECREDUCE_SMAX	= 265,
    RISCV_G_VECREDUCE_SMIN	= 266,
    RISCV_G_VECREDUCE_UMAX	= 267,
    RISCV_G_VECREDUCE_UMIN	= 268,
    RISCV_G_SBFX	= 269,
    RISCV_G_UBFX	= 270,
    RISCV_ADJCALLSTACKDOWN	= 271,
    RISCV_ADJCALLSTACKUP	= 272,
    RISCV_BuildPairF64Pseudo	= 273,
    RISCV_BuildPairF64Pseudo_INX	= 274,
    RISCV_G_FCLASS	= 275,
    RISCV_HWASAN_CHECK_MEMACCESS_SHORTGRANULES	= 276,
    RISCV_KCFI_CHECK	= 277,
    RISCV_PseudoAddTPRel	= 278,
    RISCV_PseudoAtomicLoadNand32	= 279,
    RISCV_PseudoAtomicLoadNand64	= 280,
    RISCV_PseudoBR	= 281,
    RISCV_PseudoBRIND	= 282,
    RISCV_PseudoCALL	= 283,
    RISCV_PseudoCALLIndirect	= 284,
    RISCV_PseudoCALLReg	= 285,
    RISCV_PseudoCCADD	= 286,
    RISCV_PseudoCCADDI	= 287,
    RISCV_PseudoCCADDIW	= 288,
    RISCV_PseudoCCADDW	= 289,
    RISCV_PseudoCCAND	= 290,
    RISCV_PseudoCCANDI	= 291,
    RISCV_PseudoCCANDN	= 292,
    RISCV_PseudoCCMOVGPR	= 293,
    RISCV_PseudoCCMOVGPRNoX0	= 294,
    RISCV_PseudoCCOR	= 295,
    RISCV_PseudoCCORI	= 296,
    RISCV_PseudoCCORN	= 297,
    RISCV_PseudoCCSLL	= 298,
    RISCV_PseudoCCSLLI	= 299,
    RISCV_PseudoCCSLLIW	= 300,
    RISCV_PseudoCCSLLW	= 301,
    RISCV_PseudoCCSRA	= 302,
    RISCV_PseudoCCSRAI	= 303,
    RISCV_PseudoCCSRAIW	= 304,
    RISCV_PseudoCCSRAW	= 305,
    RISCV_PseudoCCSRL	= 306,
    RISCV_PseudoCCSRLI	= 307,
    RISCV_PseudoCCSRLIW	= 308,
    RISCV_PseudoCCSRLW	= 309,
    RISCV_PseudoCCSUB	= 310,
    RISCV_PseudoCCSUBW	= 311,
    RISCV_PseudoCCXNOR	= 312,
    RISCV_PseudoCCXOR	= 313,
    RISCV_PseudoCCXORI	= 314,
    RISCV_PseudoCmpXchg32	= 315,
    RISCV_PseudoCmpXchg64	= 316,
    RISCV_PseudoFLD	= 317,
    RISCV_PseudoFLH	= 318,
    RISCV_PseudoFLW	= 319,
    RISCV_PseudoFROUND_D	= 320,
    RISCV_PseudoFROUND_D_IN32X	= 321,
    RISCV_PseudoFROUND_D_INX	= 322,
    RISCV_PseudoFROUND_H	= 323,
    RISCV_PseudoFROUND_H_INX	= 324,
    RISCV_PseudoFROUND_S	= 325,
    RISCV_PseudoFROUND_S_INX	= 326,
    RISCV_PseudoFSD	= 327,
    RISCV_PseudoFSH	= 328,
    RISCV_PseudoFSW	= 329,
    RISCV_PseudoJump	= 330,
    RISCV_PseudoLA	= 331,
    RISCV_PseudoLAImm	= 332,
    RISCV_PseudoLA_TLSDESC	= 333,
    RISCV_PseudoLA_TLS_GD	= 334,
    RISCV_PseudoLA_TLS_IE	= 335,
    RISCV_PseudoLB	= 336,
    RISCV_PseudoLBU	= 337,
    RISCV_PseudoLD	= 338,
    RISCV_PseudoLGA	= 339,
    RISCV_PseudoLH	= 340,
    RISCV_PseudoLHU	= 341,
    RISCV_PseudoLI	= 342,
    RISCV_PseudoLLA	= 343,
    RISCV_PseudoLLAImm	= 344,
    RISCV_PseudoLW	= 345,
    RISCV_PseudoLWU	= 346,
    RISCV_PseudoLongBEQ	= 347,
    RISCV_PseudoLongBGE	= 348,
    RISCV_PseudoLongBGEU	= 349,
    RISCV_PseudoLongBLT	= 350,
    RISCV_PseudoLongBLTU	= 351,
    RISCV_PseudoLongBNE	= 352,
    RISCV_PseudoMaskedAtomicLoadAdd32	= 353,
    RISCV_PseudoMaskedAtomicLoadMax32	= 354,
    RISCV_PseudoMaskedAtomicLoadMin32	= 355,
    RISCV_PseudoMaskedAtomicLoadNand32	= 356,
    RISCV_PseudoMaskedAtomicLoadSub32	= 357,
    RISCV_PseudoMaskedAtomicLoadUMax32	= 358,
    RISCV_PseudoMaskedAtomicLoadUMin32	= 359,
    RISCV_PseudoMaskedAtomicSwap32	= 360,
    RISCV_PseudoMaskedCmpXchg32	= 361,
    RISCV_PseudoMovImm	= 362,
    RISCV_PseudoQuietFLE_D	= 363,
    RISCV_PseudoQuietFLE_D_IN32X	= 364,
    RISCV_PseudoQuietFLE_D_INX	= 365,
    RISCV_PseudoQuietFLE_H	= 366,
    RISCV_PseudoQuietFLE_H_INX	= 367,
    RISCV_PseudoQuietFLE_S	= 368,
    RISCV_PseudoQuietFLE_S_INX	= 369,
    RISCV_PseudoQuietFLT_D	= 370,
    RISCV_PseudoQuietFLT_D_IN32X	= 371,
    RISCV_PseudoQuietFLT_D_INX	= 372,
    RISCV_PseudoQuietFLT_H	= 373,
    RISCV_PseudoQuietFLT_H_INX	= 374,
    RISCV_PseudoQuietFLT_S	= 375,
    RISCV_PseudoQuietFLT_S_INX	= 376,
    RISCV_PseudoRET	= 377,
    RISCV_PseudoRV32ZdinxLD	= 378,
    RISCV_PseudoRV32ZdinxSD	= 379,
    RISCV_PseudoRVVInitUndefM1	= 380,
    RISCV_PseudoRVVInitUndefM2	= 381,
    RISCV_PseudoRVVInitUndefM4	= 382,
    RISCV_PseudoRVVInitUndefM8	= 383,
    RISCV_PseudoReadVL	= 384,
    RISCV_PseudoReadVLENB	= 385,
    RISCV_PseudoSB	= 386,
    RISCV_PseudoSD	= 387,
    RISCV_PseudoSEXT_B	= 388,
    RISCV_PseudoSEXT_H	= 389,
    RISCV_PseudoSH	= 390,
    RISCV_PseudoSW	= 391,
    RISCV_PseudoTAIL	= 392,
    RISCV_PseudoTAILIndirect	= 393,
    RISCV_PseudoTHVdotVMAQASU_VV_M1	= 394,
    RISCV_PseudoTHVdotVMAQASU_VV_M1_MASK	= 395,
    RISCV_PseudoTHVdotVMAQASU_VV_M2	= 396,
    RISCV_PseudoTHVdotVMAQASU_VV_M2_MASK	= 397,
    RISCV_PseudoTHVdotVMAQASU_VV_M4	= 398,
    RISCV_PseudoTHVdotVMAQASU_VV_M4_MASK	= 399,
    RISCV_PseudoTHVdotVMAQASU_VV_M8	= 400,
    RISCV_PseudoTHVdotVMAQASU_VV_M8_MASK	= 401,
    RISCV_PseudoTHVdotVMAQASU_VV_MF2	= 402,
    RISCV_PseudoTHVdotVMAQASU_VV_MF2_MASK	= 403,
    RISCV_PseudoTHVdotVMAQASU_VX_M1	= 404,
    RISCV_PseudoTHVdotVMAQASU_VX_M1_MASK	= 405,
    RISCV_PseudoTHVdotVMAQASU_VX_M2	= 406,
    RISCV_PseudoTHVdotVMAQASU_VX_M2_MASK	= 407,
    RISCV_PseudoTHVdotVMAQASU_VX_M4	= 408,
    RISCV_PseudoTHVdotVMAQASU_VX_M4_MASK	= 409,
    RISCV_PseudoTHVdotVMAQASU_VX_M8	= 410,
    RISCV_PseudoTHVdotVMAQASU_VX_M8_MASK	= 411,
    RISCV_PseudoTHVdotVMAQASU_VX_MF2	= 412,
    RISCV_PseudoTHVdotVMAQASU_VX_MF2_MASK	= 413,
    RISCV_PseudoTHVdotVMAQAUS_VX_M1	= 414,
    RISCV_PseudoTHVdotVMAQAUS_VX_M1_MASK	= 415,
    RISCV_PseudoTHVdotVMAQAUS_VX_M2	= 416,
    RISCV_PseudoTHVdotVMAQAUS_VX_M2_MASK	= 417,
    RISCV_PseudoTHVdotVMAQAUS_VX_M4	= 418,
    RISCV_PseudoTHVdotVMAQAUS_VX_M4_MASK	= 419,
    RISCV_PseudoTHVdotVMAQAUS_VX_M8	= 420,
    RISCV_PseudoTHVdotVMAQAUS_VX_M8_MASK	= 421,
    RISCV_PseudoTHVdotVMAQAUS_VX_MF2	= 422,
    RISCV_PseudoTHVdotVMAQAUS_VX_MF2_MASK	= 423,
    RISCV_PseudoTHVdotVMAQAU_VV_M1	= 424,
    RISCV_PseudoTHVdotVMAQAU_VV_M1_MASK	= 425,
    RISCV_PseudoTHVdotVMAQAU_VV_M2	= 426,
    RISCV_PseudoTHVdotVMAQAU_VV_M2_MASK	= 427,
    RISCV_PseudoTHVdotVMAQAU_VV_M4	= 428,
    RISCV_PseudoTHVdotVMAQAU_VV_M4_MASK	= 429,
    RISCV_PseudoTHVdotVMAQAU_VV_M8	= 430,
    RISCV_PseudoTHVdotVMAQAU_VV_M8_MASK	= 431,
    RISCV_PseudoTHVdotVMAQAU_VV_MF2	= 432,
    RISCV_PseudoTHVdotVMAQAU_VV_MF2_MASK	= 433,
    RISCV_PseudoTHVdotVMAQAU_VX_M1	= 434,
    RISCV_PseudoTHVdotVMAQAU_VX_M1_MASK	= 435,
    RISCV_PseudoTHVdotVMAQAU_VX_M2	= 436,
    RISCV_PseudoTHVdotVMAQAU_VX_M2_MASK	= 437,
    RISCV_PseudoTHVdotVMAQAU_VX_M4	= 438,
    RISCV_PseudoTHVdotVMAQAU_VX_M4_MASK	= 439,
    RISCV_PseudoTHVdotVMAQAU_VX_M8	= 440,
    RISCV_PseudoTHVdotVMAQAU_VX_M8_MASK	= 441,
    RISCV_PseudoTHVdotVMAQAU_VX_MF2	= 442,
    RISCV_PseudoTHVdotVMAQAU_VX_MF2_MASK	= 443,
    RISCV_PseudoTHVdotVMAQA_VV_M1	= 444,
    RISCV_PseudoTHVdotVMAQA_VV_M1_MASK	= 445,
    RISCV_PseudoTHVdotVMAQA_VV_M2	= 446,
    RISCV_PseudoTHVdotVMAQA_VV_M2_MASK	= 447,
    RISCV_PseudoTHVdotVMAQA_VV_M4	= 448,
    RISCV_PseudoTHVdotVMAQA_VV_M4_MASK	= 449,
    RISCV_PseudoTHVdotVMAQA_VV_M8	= 450,
    RISCV_PseudoTHVdotVMAQA_VV_M8_MASK	= 451,
    RISCV_PseudoTHVdotVMAQA_VV_MF2	= 452,
    RISCV_PseudoTHVdotVMAQA_VV_MF2_MASK	= 453,
    RISCV_PseudoTHVdotVMAQA_VX_M1	= 454,
    RISCV_PseudoTHVdotVMAQA_VX_M1_MASK	= 455,
    RISCV_PseudoTHVdotVMAQA_VX_M2	= 456,
    RISCV_PseudoTHVdotVMAQA_VX_M2_MASK	= 457,
    RISCV_PseudoTHVdotVMAQA_VX_M4	= 458,
    RISCV_PseudoTHVdotVMAQA_VX_M4_MASK	= 459,
    RISCV_PseudoTHVdotVMAQA_VX_M8	= 460,
    RISCV_PseudoTHVdotVMAQA_VX_M8_MASK	= 461,
    RISCV_PseudoTHVdotVMAQA_VX_MF2	= 462,
    RISCV_PseudoTHVdotVMAQA_VX_MF2_MASK	= 463,
    RISCV_PseudoTLSDESCCall	= 464,
    RISCV_PseudoVAADDU_VV_M1	= 465,
    RISCV_PseudoVAADDU_VV_M1_MASK	= 466,
    RISCV_PseudoVAADDU_VV_M2	= 467,
    RISCV_PseudoVAADDU_VV_M2_MASK	= 468,
    RISCV_PseudoVAADDU_VV_M4	= 469,
    RISCV_PseudoVAADDU_VV_M4_MASK	= 470,
    RISCV_PseudoVAADDU_VV_M8	= 471,
    RISCV_PseudoVAADDU_VV_M8_MASK	= 472,
    RISCV_PseudoVAADDU_VV_MF2	= 473,
    RISCV_PseudoVAADDU_VV_MF2_MASK	= 474,
    RISCV_PseudoVAADDU_VV_MF4	= 475,
    RISCV_PseudoVAADDU_VV_MF4_MASK	= 476,
    RISCV_PseudoVAADDU_VV_MF8	= 477,
    RISCV_PseudoVAADDU_VV_MF8_MASK	= 478,
    RISCV_PseudoVAADDU_VX_M1	= 479,
    RISCV_PseudoVAADDU_VX_M1_MASK	= 480,
    RISCV_PseudoVAADDU_VX_M2	= 481,
    RISCV_PseudoVAADDU_VX_M2_MASK	= 482,
    RISCV_PseudoVAADDU_VX_M4	= 483,
    RISCV_PseudoVAADDU_VX_M4_MASK	= 484,
    RISCV_PseudoVAADDU_VX_M8	= 485,
    RISCV_PseudoVAADDU_VX_M8_MASK	= 486,
    RISCV_PseudoVAADDU_VX_MF2	= 487,
    RISCV_PseudoVAADDU_VX_MF2_MASK	= 488,
    RISCV_PseudoVAADDU_VX_MF4	= 489,
    RISCV_PseudoVAADDU_VX_MF4_MASK	= 490,
    RISCV_PseudoVAADDU_VX_MF8	= 491,
    RISCV_PseudoVAADDU_VX_MF8_MASK	= 492,
    RISCV_PseudoVAADD_VV_M1	= 493,
    RISCV_PseudoVAADD_VV_M1_MASK	= 494,
    RISCV_PseudoVAADD_VV_M2	= 495,
    RISCV_PseudoVAADD_VV_M2_MASK	= 496,
    RISCV_PseudoVAADD_VV_M4	= 497,
    RISCV_PseudoVAADD_VV_M4_MASK	= 498,
    RISCV_PseudoVAADD_VV_M8	= 499,
    RISCV_PseudoVAADD_VV_M8_MASK	= 500,
    RISCV_PseudoVAADD_VV_MF2	= 501,
    RISCV_PseudoVAADD_VV_MF2_MASK	= 502,
    RISCV_PseudoVAADD_VV_MF4	= 503,
    RISCV_PseudoVAADD_VV_MF4_MASK	= 504,
    RISCV_PseudoVAADD_VV_MF8	= 505,
    RISCV_PseudoVAADD_VV_MF8_MASK	= 506,
    RISCV_PseudoVAADD_VX_M1	= 507,
    RISCV_PseudoVAADD_VX_M1_MASK	= 508,
    RISCV_PseudoVAADD_VX_M2	= 509,
    RISCV_PseudoVAADD_VX_M2_MASK	= 510,
    RISCV_PseudoVAADD_VX_M4	= 511,
    RISCV_PseudoVAADD_VX_M4_MASK	= 512,
    RISCV_PseudoVAADD_VX_M8	= 513,
    RISCV_PseudoVAADD_VX_M8_MASK	= 514,
    RISCV_PseudoVAADD_VX_MF2	= 515,
    RISCV_PseudoVAADD_VX_MF2_MASK	= 516,
    RISCV_PseudoVAADD_VX_MF4	= 517,
    RISCV_PseudoVAADD_VX_MF4_MASK	= 518,
    RISCV_PseudoVAADD_VX_MF8	= 519,
    RISCV_PseudoVAADD_VX_MF8_MASK	= 520,
    RISCV_PseudoVADC_VIM_M1	= 521,
    RISCV_PseudoVADC_VIM_M2	= 522,
    RISCV_PseudoVADC_VIM_M4	= 523,
    RISCV_PseudoVADC_VIM_M8	= 524,
    RISCV_PseudoVADC_VIM_MF2	= 525,
    RISCV_PseudoVADC_VIM_MF4	= 526,
    RISCV_PseudoVADC_VIM_MF8	= 527,
    RISCV_PseudoVADC_VVM_M1	= 528,
    RISCV_PseudoVADC_VVM_M2	= 529,
    RISCV_PseudoVADC_VVM_M4	= 530,
    RISCV_PseudoVADC_VVM_M8	= 531,
    RISCV_PseudoVADC_VVM_MF2	= 532,
    RISCV_PseudoVADC_VVM_MF4	= 533,
    RISCV_PseudoVADC_VVM_MF8	= 534,
    RISCV_PseudoVADC_VXM_M1	= 535,
    RISCV_PseudoVADC_VXM_M2	= 536,
    RISCV_PseudoVADC_VXM_M4	= 537,
    RISCV_PseudoVADC_VXM_M8	= 538,
    RISCV_PseudoVADC_VXM_MF2	= 539,
    RISCV_PseudoVADC_VXM_MF4	= 540,
    RISCV_PseudoVADC_VXM_MF8	= 541,
    RISCV_PseudoVADD_VI_M1	= 542,
    RISCV_PseudoVADD_VI_M1_MASK	= 543,
    RISCV_PseudoVADD_VI_M2	= 544,
    RISCV_PseudoVADD_VI_M2_MASK	= 545,
    RISCV_PseudoVADD_VI_M4	= 546,
    RISCV_PseudoVADD_VI_M4_MASK	= 547,
    RISCV_PseudoVADD_VI_M8	= 548,
    RISCV_PseudoVADD_VI_M8_MASK	= 549,
    RISCV_PseudoVADD_VI_MF2	= 550,
    RISCV_PseudoVADD_VI_MF2_MASK	= 551,
    RISCV_PseudoVADD_VI_MF4	= 552,
    RISCV_PseudoVADD_VI_MF4_MASK	= 553,
    RISCV_PseudoVADD_VI_MF8	= 554,
    RISCV_PseudoVADD_VI_MF8_MASK	= 555,
    RISCV_PseudoVADD_VV_M1	= 556,
    RISCV_PseudoVADD_VV_M1_MASK	= 557,
    RISCV_PseudoVADD_VV_M2	= 558,
    RISCV_PseudoVADD_VV_M2_MASK	= 559,
    RISCV_PseudoVADD_VV_M4	= 560,
    RISCV_PseudoVADD_VV_M4_MASK	= 561,
    RISCV_PseudoVADD_VV_M8	= 562,
    RISCV_PseudoVADD_VV_M8_MASK	= 563,
    RISCV_PseudoVADD_VV_MF2	= 564,
    RISCV_PseudoVADD_VV_MF2_MASK	= 565,
    RISCV_PseudoVADD_VV_MF4	= 566,
    RISCV_PseudoVADD_VV_MF4_MASK	= 567,
    RISCV_PseudoVADD_VV_MF8	= 568,
    RISCV_PseudoVADD_VV_MF8_MASK	= 569,
    RISCV_PseudoVADD_VX_M1	= 570,
    RISCV_PseudoVADD_VX_M1_MASK	= 571,
    RISCV_PseudoVADD_VX_M2	= 572,
    RISCV_PseudoVADD_VX_M2_MASK	= 573,
    RISCV_PseudoVADD_VX_M4	= 574,
    RISCV_PseudoVADD_VX_M4_MASK	= 575,
    RISCV_PseudoVADD_VX_M8	= 576,
    RISCV_PseudoVADD_VX_M8_MASK	= 577,
    RISCV_PseudoVADD_VX_MF2	= 578,
    RISCV_PseudoVADD_VX_MF2_MASK	= 579,
    RISCV_PseudoVADD_VX_MF4	= 580,
    RISCV_PseudoVADD_VX_MF4_MASK	= 581,
    RISCV_PseudoVADD_VX_MF8	= 582,
    RISCV_PseudoVADD_VX_MF8_MASK	= 583,
    RISCV_PseudoVAESDF_VS_M1_M1	= 584,
    RISCV_PseudoVAESDF_VS_M1_MF2	= 585,
    RISCV_PseudoVAESDF_VS_M1_MF4	= 586,
    RISCV_PseudoVAESDF_VS_M1_MF8	= 587,
    RISCV_PseudoVAESDF_VS_M2_M1	= 588,
    RISCV_PseudoVAESDF_VS_M2_M2	= 589,
    RISCV_PseudoVAESDF_VS_M2_MF2	= 590,
    RISCV_PseudoVAESDF_VS_M2_MF4	= 591,
    RISCV_PseudoVAESDF_VS_M2_MF8	= 592,
    RISCV_PseudoVAESDF_VS_M4_M1	= 593,
    RISCV_PseudoVAESDF_VS_M4_M2	= 594,
    RISCV_PseudoVAESDF_VS_M4_M4	= 595,
    RISCV_PseudoVAESDF_VS_M4_MF2	= 596,
    RISCV_PseudoVAESDF_VS_M4_MF4	= 597,
    RISCV_PseudoVAESDF_VS_M4_MF8	= 598,
    RISCV_PseudoVAESDF_VS_M8_M1	= 599,
    RISCV_PseudoVAESDF_VS_M8_M2	= 600,
    RISCV_PseudoVAESDF_VS_M8_M4	= 601,
    RISCV_PseudoVAESDF_VS_M8_MF2	= 602,
    RISCV_PseudoVAESDF_VS_M8_MF4	= 603,
    RISCV_PseudoVAESDF_VS_M8_MF8	= 604,
    RISCV_PseudoVAESDF_VS_MF2_MF2	= 605,
    RISCV_PseudoVAESDF_VS_MF2_MF4	= 606,
    RISCV_PseudoVAESDF_VS_MF2_MF8	= 607,
    RISCV_PseudoVAESDF_VV_M1	= 608,
    RISCV_PseudoVAESDF_VV_M2	= 609,
    RISCV_PseudoVAESDF_VV_M4	= 610,
    RISCV_PseudoVAESDF_VV_M8	= 611,
    RISCV_PseudoVAESDF_VV_MF2	= 612,
    RISCV_PseudoVAESDM_VS_M1_M1	= 613,
    RISCV_PseudoVAESDM_VS_M1_MF2	= 614,
    RISCV_PseudoVAESDM_VS_M1_MF4	= 615,
    RISCV_PseudoVAESDM_VS_M1_MF8	= 616,
    RISCV_PseudoVAESDM_VS_M2_M1	= 617,
    RISCV_PseudoVAESDM_VS_M2_M2	= 618,
    RISCV_PseudoVAESDM_VS_M2_MF2	= 619,
    RISCV_PseudoVAESDM_VS_M2_MF4	= 620,
    RISCV_PseudoVAESDM_VS_M2_MF8	= 621,
    RISCV_PseudoVAESDM_VS_M4_M1	= 622,
    RISCV_PseudoVAESDM_VS_M4_M2	= 623,
    RISCV_PseudoVAESDM_VS_M4_M4	= 624,
    RISCV_PseudoVAESDM_VS_M4_MF2	= 625,
    RISCV_PseudoVAESDM_VS_M4_MF4	= 626,
    RISCV_PseudoVAESDM_VS_M4_MF8	= 627,
    RISCV_PseudoVAESDM_VS_M8_M1	= 628,
    RISCV_PseudoVAESDM_VS_M8_M2	= 629,
    RISCV_PseudoVAESDM_VS_M8_M4	= 630,
    RISCV_PseudoVAESDM_VS_M8_MF2	= 631,
    RISCV_PseudoVAESDM_VS_M8_MF4	= 632,
    RISCV_PseudoVAESDM_VS_M8_MF8	= 633,
    RISCV_PseudoVAESDM_VS_MF2_MF2	= 634,
    RISCV_PseudoVAESDM_VS_MF2_MF4	= 635,
    RISCV_PseudoVAESDM_VS_MF2_MF8	= 636,
    RISCV_PseudoVAESDM_VV_M1	= 637,
    RISCV_PseudoVAESDM_VV_M2	= 638,
    RISCV_PseudoVAESDM_VV_M4	= 639,
    RISCV_PseudoVAESDM_VV_M8	= 640,
    RISCV_PseudoVAESDM_VV_MF2	= 641,
    RISCV_PseudoVAESEF_VS_M1_M1	= 642,
    RISCV_PseudoVAESEF_VS_M1_MF2	= 643,
    RISCV_PseudoVAESEF_VS_M1_MF4	= 644,
    RISCV_PseudoVAESEF_VS_M1_MF8	= 645,
    RISCV_PseudoVAESEF_VS_M2_M1	= 646,
    RISCV_PseudoVAESEF_VS_M2_M2	= 647,
    RISCV_PseudoVAESEF_VS_M2_MF2	= 648,
    RISCV_PseudoVAESEF_VS_M2_MF4	= 649,
    RISCV_PseudoVAESEF_VS_M2_MF8	= 650,
    RISCV_PseudoVAESEF_VS_M4_M1	= 651,
    RISCV_PseudoVAESEF_VS_M4_M2	= 652,
    RISCV_PseudoVAESEF_VS_M4_M4	= 653,
    RISCV_PseudoVAESEF_VS_M4_MF2	= 654,
    RISCV_PseudoVAESEF_VS_M4_MF4	= 655,
    RISCV_PseudoVAESEF_VS_M4_MF8	= 656,
    RISCV_PseudoVAESEF_VS_M8_M1	= 657,
    RISCV_PseudoVAESEF_VS_M8_M2	= 658,
    RISCV_PseudoVAESEF_VS_M8_M4	= 659,
    RISCV_PseudoVAESEF_VS_M8_MF2	= 660,
    RISCV_PseudoVAESEF_VS_M8_MF4	= 661,
    RISCV_PseudoVAESEF_VS_M8_MF8	= 662,
    RISCV_PseudoVAESEF_VS_MF2_MF2	= 663,
    RISCV_PseudoVAESEF_VS_MF2_MF4	= 664,
    RISCV_PseudoVAESEF_VS_MF2_MF8	= 665,
    RISCV_PseudoVAESEF_VV_M1	= 666,
    RISCV_PseudoVAESEF_VV_M2	= 667,
    RISCV_PseudoVAESEF_VV_M4	= 668,
    RISCV_PseudoVAESEF_VV_M8	= 669,
    RISCV_PseudoVAESEF_VV_MF2	= 670,
    RISCV_PseudoVAESEM_VS_M1_M1	= 671,
    RISCV_PseudoVAESEM_VS_M1_MF2	= 672,
    RISCV_PseudoVAESEM_VS_M1_MF4	= 673,
    RISCV_PseudoVAESEM_VS_M1_MF8	= 674,
    RISCV_PseudoVAESEM_VS_M2_M1	= 675,
    RISCV_PseudoVAESEM_VS_M2_M2	= 676,
    RISCV_PseudoVAESEM_VS_M2_MF2	= 677,
    RISCV_PseudoVAESEM_VS_M2_MF4	= 678,
    RISCV_PseudoVAESEM_VS_M2_MF8	= 679,
    RISCV_PseudoVAESEM_VS_M4_M1	= 680,
    RISCV_PseudoVAESEM_VS_M4_M2	= 681,
    RISCV_PseudoVAESEM_VS_M4_M4	= 682,
    RISCV_PseudoVAESEM_VS_M4_MF2	= 683,
    RISCV_PseudoVAESEM_VS_M4_MF4	= 684,
    RISCV_PseudoVAESEM_VS_M4_MF8	= 685,
    RISCV_PseudoVAESEM_VS_M8_M1	= 686,
    RISCV_PseudoVAESEM_VS_M8_M2	= 687,
    RISCV_PseudoVAESEM_VS_M8_M4	= 688,
    RISCV_PseudoVAESEM_VS_M8_MF2	= 689,
    RISCV_PseudoVAESEM_VS_M8_MF4	= 690,
    RISCV_PseudoVAESEM_VS_M8_MF8	= 691,
    RISCV_PseudoVAESEM_VS_MF2_MF2	= 692,
    RISCV_PseudoVAESEM_VS_MF2_MF4	= 693,
    RISCV_PseudoVAESEM_VS_MF2_MF8	= 694,
    RISCV_PseudoVAESEM_VV_M1	= 695,
    RISCV_PseudoVAESEM_VV_M2	= 696,
    RISCV_PseudoVAESEM_VV_M4	= 697,
    RISCV_PseudoVAESEM_VV_M8	= 698,
    RISCV_PseudoVAESEM_VV_MF2	= 699,
    RISCV_PseudoVAESKF1_VI_M1	= 700,
    RISCV_PseudoVAESKF1_VI_M2	= 701,
    RISCV_PseudoVAESKF1_VI_M4	= 702,
    RISCV_PseudoVAESKF1_VI_M8	= 703,
    RISCV_PseudoVAESKF1_VI_MF2	= 704,
    RISCV_PseudoVAESKF2_VI_M1	= 705,
    RISCV_PseudoVAESKF2_VI_M2	= 706,
    RISCV_PseudoVAESKF2_VI_M4	= 707,
    RISCV_PseudoVAESKF2_VI_M8	= 708,
    RISCV_PseudoVAESKF2_VI_MF2	= 709,
    RISCV_PseudoVAESZ_VS_M1_M1	= 710,
    RISCV_PseudoVAESZ_VS_M1_MF2	= 711,
    RISCV_PseudoVAESZ_VS_M1_MF4	= 712,
    RISCV_PseudoVAESZ_VS_M1_MF8	= 713,
    RISCV_PseudoVAESZ_VS_M2_M1	= 714,
    RISCV_PseudoVAESZ_VS_M2_M2	= 715,
    RISCV_PseudoVAESZ_VS_M2_MF2	= 716,
    RISCV_PseudoVAESZ_VS_M2_MF4	= 717,
    RISCV_PseudoVAESZ_VS_M2_MF8	= 718,
    RISCV_PseudoVAESZ_VS_M4_M1	= 719,
    RISCV_PseudoVAESZ_VS_M4_M2	= 720,
    RISCV_PseudoVAESZ_VS_M4_M4	= 721,
    RISCV_PseudoVAESZ_VS_M4_MF2	= 722,
    RISCV_PseudoVAESZ_VS_M4_MF4	= 723,
    RISCV_PseudoVAESZ_VS_M4_MF8	= 724,
    RISCV_PseudoVAESZ_VS_M8_M1	= 725,
    RISCV_PseudoVAESZ_VS_M8_M2	= 726,
    RISCV_PseudoVAESZ_VS_M8_M4	= 727,
    RISCV_PseudoVAESZ_VS_M8_MF2	= 728,
    RISCV_PseudoVAESZ_VS_M8_MF4	= 729,
    RISCV_PseudoVAESZ_VS_M8_MF8	= 730,
    RISCV_PseudoVAESZ_VS_MF2_MF2	= 731,
    RISCV_PseudoVAESZ_VS_MF2_MF4	= 732,
    RISCV_PseudoVAESZ_VS_MF2_MF8	= 733,
    RISCV_PseudoVANDN_VV_M1	= 734,
    RISCV_PseudoVANDN_VV_M1_MASK	= 735,
    RISCV_PseudoVANDN_VV_M2	= 736,
    RISCV_PseudoVANDN_VV_M2_MASK	= 737,
    RISCV_PseudoVANDN_VV_M4	= 738,
    RISCV_PseudoVANDN_VV_M4_MASK	= 739,
    RISCV_PseudoVANDN_VV_M8	= 740,
    RISCV_PseudoVANDN_VV_M8_MASK	= 741,
    RISCV_PseudoVANDN_VV_MF2	= 742,
    RISCV_PseudoVANDN_VV_MF2_MASK	= 743,
    RISCV_PseudoVANDN_VV_MF4	= 744,
    RISCV_PseudoVANDN_VV_MF4_MASK	= 745,
    RISCV_PseudoVANDN_VV_MF8	= 746,
    RISCV_PseudoVANDN_VV_MF8_MASK	= 747,
    RISCV_PseudoVANDN_VX_M1	= 748,
    RISCV_PseudoVANDN_VX_M1_MASK	= 749,
    RISCV_PseudoVANDN_VX_M2	= 750,
    RISCV_PseudoVANDN_VX_M2_MASK	= 751,
    RISCV_PseudoVANDN_VX_M4	= 752,
    RISCV_PseudoVANDN_VX_M4_MASK	= 753,
    RISCV_PseudoVANDN_VX_M8	= 754,
    RISCV_PseudoVANDN_VX_M8_MASK	= 755,
    RISCV_PseudoVANDN_VX_MF2	= 756,
    RISCV_PseudoVANDN_VX_MF2_MASK	= 757,
    RISCV_PseudoVANDN_VX_MF4	= 758,
    RISCV_PseudoVANDN_VX_MF4_MASK	= 759,
    RISCV_PseudoVANDN_VX_MF8	= 760,
    RISCV_PseudoVANDN_VX_MF8_MASK	= 761,
    RISCV_PseudoVAND_VI_M1	= 762,
    RISCV_PseudoVAND_VI_M1_MASK	= 763,
    RISCV_PseudoVAND_VI_M2	= 764,
    RISCV_PseudoVAND_VI_M2_MASK	= 765,
    RISCV_PseudoVAND_VI_M4	= 766,
    RISCV_PseudoVAND_VI_M4_MASK	= 767,
    RISCV_PseudoVAND_VI_M8	= 768,
    RISCV_PseudoVAND_VI_M8_MASK	= 769,
    RISCV_PseudoVAND_VI_MF2	= 770,
    RISCV_PseudoVAND_VI_MF2_MASK	= 771,
    RISCV_PseudoVAND_VI_MF4	= 772,
    RISCV_PseudoVAND_VI_MF4_MASK	= 773,
    RISCV_PseudoVAND_VI_MF8	= 774,
    RISCV_PseudoVAND_VI_MF8_MASK	= 775,
    RISCV_PseudoVAND_VV_M1	= 776,
    RISCV_PseudoVAND_VV_M1_MASK	= 777,
    RISCV_PseudoVAND_VV_M2	= 778,
    RISCV_PseudoVAND_VV_M2_MASK	= 779,
    RISCV_PseudoVAND_VV_M4	= 780,
    RISCV_PseudoVAND_VV_M4_MASK	= 781,
    RISCV_PseudoVAND_VV_M8	= 782,
    RISCV_PseudoVAND_VV_M8_MASK	= 783,
    RISCV_PseudoVAND_VV_MF2	= 784,
    RISCV_PseudoVAND_VV_MF2_MASK	= 785,
    RISCV_PseudoVAND_VV_MF4	= 786,
    RISCV_PseudoVAND_VV_MF4_MASK	= 787,
    RISCV_PseudoVAND_VV_MF8	= 788,
    RISCV_PseudoVAND_VV_MF8_MASK	= 789,
    RISCV_PseudoVAND_VX_M1	= 790,
    RISCV_PseudoVAND_VX_M1_MASK	= 791,
    RISCV_PseudoVAND_VX_M2	= 792,
    RISCV_PseudoVAND_VX_M2_MASK	= 793,
    RISCV_PseudoVAND_VX_M4	= 794,
    RISCV_PseudoVAND_VX_M4_MASK	= 795,
    RISCV_PseudoVAND_VX_M8	= 796,
    RISCV_PseudoVAND_VX_M8_MASK	= 797,
    RISCV_PseudoVAND_VX_MF2	= 798,
    RISCV_PseudoVAND_VX_MF2_MASK	= 799,
    RISCV_PseudoVAND_VX_MF4	= 800,
    RISCV_PseudoVAND_VX_MF4_MASK	= 801,
    RISCV_PseudoVAND_VX_MF8	= 802,
    RISCV_PseudoVAND_VX_MF8_MASK	= 803,
    RISCV_PseudoVASUBU_VV_M1	= 804,
    RISCV_PseudoVASUBU_VV_M1_MASK	= 805,
    RISCV_PseudoVASUBU_VV_M2	= 806,
    RISCV_PseudoVASUBU_VV_M2_MASK	= 807,
    RISCV_PseudoVASUBU_VV_M4	= 808,
    RISCV_PseudoVASUBU_VV_M4_MASK	= 809,
    RISCV_PseudoVASUBU_VV_M8	= 810,
    RISCV_PseudoVASUBU_VV_M8_MASK	= 811,
    RISCV_PseudoVASUBU_VV_MF2	= 812,
    RISCV_PseudoVASUBU_VV_MF2_MASK	= 813,
    RISCV_PseudoVASUBU_VV_MF4	= 814,
    RISCV_PseudoVASUBU_VV_MF4_MASK	= 815,
    RISCV_PseudoVASUBU_VV_MF8	= 816,
    RISCV_PseudoVASUBU_VV_MF8_MASK	= 817,
    RISCV_PseudoVASUBU_VX_M1	= 818,
    RISCV_PseudoVASUBU_VX_M1_MASK	= 819,
    RISCV_PseudoVASUBU_VX_M2	= 820,
    RISCV_PseudoVASUBU_VX_M2_MASK	= 821,
    RISCV_PseudoVASUBU_VX_M4	= 822,
    RISCV_PseudoVASUBU_VX_M4_MASK	= 823,
    RISCV_PseudoVASUBU_VX_M8	= 824,
    RISCV_PseudoVASUBU_VX_M8_MASK	= 825,
    RISCV_PseudoVASUBU_VX_MF2	= 826,
    RISCV_PseudoVASUBU_VX_MF2_MASK	= 827,
    RISCV_PseudoVASUBU_VX_MF4	= 828,
    RISCV_PseudoVASUBU_VX_MF4_MASK	= 829,
    RISCV_PseudoVASUBU_VX_MF8	= 830,
    RISCV_PseudoVASUBU_VX_MF8_MASK	= 831,
    RISCV_PseudoVASUB_VV_M1	= 832,
    RISCV_PseudoVASUB_VV_M1_MASK	= 833,
    RISCV_PseudoVASUB_VV_M2	= 834,
    RISCV_PseudoVASUB_VV_M2_MASK	= 835,
    RISCV_PseudoVASUB_VV_M4	= 836,
    RISCV_PseudoVASUB_VV_M4_MASK	= 837,
    RISCV_PseudoVASUB_VV_M8	= 838,
    RISCV_PseudoVASUB_VV_M8_MASK	= 839,
    RISCV_PseudoVASUB_VV_MF2	= 840,
    RISCV_PseudoVASUB_VV_MF2_MASK	= 841,
    RISCV_PseudoVASUB_VV_MF4	= 842,
    RISCV_PseudoVASUB_VV_MF4_MASK	= 843,
    RISCV_PseudoVASUB_VV_MF8	= 844,
    RISCV_PseudoVASUB_VV_MF8_MASK	= 845,
    RISCV_PseudoVASUB_VX_M1	= 846,
    RISCV_PseudoVASUB_VX_M1_MASK	= 847,
    RISCV_PseudoVASUB_VX_M2	= 848,
    RISCV_PseudoVASUB_VX_M2_MASK	= 849,
    RISCV_PseudoVASUB_VX_M4	= 850,
    RISCV_PseudoVASUB_VX_M4_MASK	= 851,
    RISCV_PseudoVASUB_VX_M8	= 852,
    RISCV_PseudoVASUB_VX_M8_MASK	= 853,
    RISCV_PseudoVASUB_VX_MF2	= 854,
    RISCV_PseudoVASUB_VX_MF2_MASK	= 855,
    RISCV_PseudoVASUB_VX_MF4	= 856,
    RISCV_PseudoVASUB_VX_MF4_MASK	= 857,
    RISCV_PseudoVASUB_VX_MF8	= 858,
    RISCV_PseudoVASUB_VX_MF8_MASK	= 859,
    RISCV_PseudoVBREV8_V_M1	= 860,
    RISCV_PseudoVBREV8_V_M1_MASK	= 861,
    RISCV_PseudoVBREV8_V_M2	= 862,
    RISCV_PseudoVBREV8_V_M2_MASK	= 863,
    RISCV_PseudoVBREV8_V_M4	= 864,
    RISCV_PseudoVBREV8_V_M4_MASK	= 865,
    RISCV_PseudoVBREV8_V_M8	= 866,
    RISCV_PseudoVBREV8_V_M8_MASK	= 867,
    RISCV_PseudoVBREV8_V_MF2	= 868,
    RISCV_PseudoVBREV8_V_MF2_MASK	= 869,
    RISCV_PseudoVBREV8_V_MF4	= 870,
    RISCV_PseudoVBREV8_V_MF4_MASK	= 871,
    RISCV_PseudoVBREV8_V_MF8	= 872,
    RISCV_PseudoVBREV8_V_MF8_MASK	= 873,
    RISCV_PseudoVBREV_V_M1	= 874,
    RISCV_PseudoVBREV_V_M1_MASK	= 875,
    RISCV_PseudoVBREV_V_M2	= 876,
    RISCV_PseudoVBREV_V_M2_MASK	= 877,
    RISCV_PseudoVBREV_V_M4	= 878,
    RISCV_PseudoVBREV_V_M4_MASK	= 879,
    RISCV_PseudoVBREV_V_M8	= 880,
    RISCV_PseudoVBREV_V_M8_MASK	= 881,
    RISCV_PseudoVBREV_V_MF2	= 882,
    RISCV_PseudoVBREV_V_MF2_MASK	= 883,
    RISCV_PseudoVBREV_V_MF4	= 884,
    RISCV_PseudoVBREV_V_MF4_MASK	= 885,
    RISCV_PseudoVBREV_V_MF8	= 886,
    RISCV_PseudoVBREV_V_MF8_MASK	= 887,
    RISCV_PseudoVCLMULH_VV_M1	= 888,
    RISCV_PseudoVCLMULH_VV_M1_MASK	= 889,
    RISCV_PseudoVCLMULH_VV_M2	= 890,
    RISCV_PseudoVCLMULH_VV_M2_MASK	= 891,
    RISCV_PseudoVCLMULH_VV_M4	= 892,
    RISCV_PseudoVCLMULH_VV_M4_MASK	= 893,
    RISCV_PseudoVCLMULH_VV_M8	= 894,
    RISCV_PseudoVCLMULH_VV_M8_MASK	= 895,
    RISCV_PseudoVCLMULH_VV_MF2	= 896,
    RISCV_PseudoVCLMULH_VV_MF2_MASK	= 897,
    RISCV_PseudoVCLMULH_VV_MF4	= 898,
    RISCV_PseudoVCLMULH_VV_MF4_MASK	= 899,
    RISCV_PseudoVCLMULH_VV_MF8	= 900,
    RISCV_PseudoVCLMULH_VV_MF8_MASK	= 901,
    RISCV_PseudoVCLMULH_VX_M1	= 902,
    RISCV_PseudoVCLMULH_VX_M1_MASK	= 903,
    RISCV_PseudoVCLMULH_VX_M2	= 904,
    RISCV_PseudoVCLMULH_VX_M2_MASK	= 905,
    RISCV_PseudoVCLMULH_VX_M4	= 906,
    RISCV_PseudoVCLMULH_VX_M4_MASK	= 907,
    RISCV_PseudoVCLMULH_VX_M8	= 908,
    RISCV_PseudoVCLMULH_VX_M8_MASK	= 909,
    RISCV_PseudoVCLMULH_VX_MF2	= 910,
    RISCV_PseudoVCLMULH_VX_MF2_MASK	= 911,
    RISCV_PseudoVCLMULH_VX_MF4	= 912,
    RISCV_PseudoVCLMULH_VX_MF4_MASK	= 913,
    RISCV_PseudoVCLMULH_VX_MF8	= 914,
    RISCV_PseudoVCLMULH_VX_MF8_MASK	= 915,
    RISCV_PseudoVCLMUL_VV_M1	= 916,
    RISCV_PseudoVCLMUL_VV_M1_MASK	= 917,
    RISCV_PseudoVCLMUL_VV_M2	= 918,
    RISCV_PseudoVCLMUL_VV_M2_MASK	= 919,
    RISCV_PseudoVCLMUL_VV_M4	= 920,
    RISCV_PseudoVCLMUL_VV_M4_MASK	= 921,
    RISCV_PseudoVCLMUL_VV_M8	= 922,
    RISCV_PseudoVCLMUL_VV_M8_MASK	= 923,
    RISCV_PseudoVCLMUL_VV_MF2	= 924,
    RISCV_PseudoVCLMUL_VV_MF2_MASK	= 925,
    RISCV_PseudoVCLMUL_VV_MF4	= 926,
    RISCV_PseudoVCLMUL_VV_MF4_MASK	= 927,
    RISCV_PseudoVCLMUL_VV_MF8	= 928,
    RISCV_PseudoVCLMUL_VV_MF8_MASK	= 929,
    RISCV_PseudoVCLMUL_VX_M1	= 930,
    RISCV_PseudoVCLMUL_VX_M1_MASK	= 931,
    RISCV_PseudoVCLMUL_VX_M2	= 932,
    RISCV_PseudoVCLMUL_VX_M2_MASK	= 933,
    RISCV_PseudoVCLMUL_VX_M4	= 934,
    RISCV_PseudoVCLMUL_VX_M4_MASK	= 935,
    RISCV_PseudoVCLMUL_VX_M8	= 936,
    RISCV_PseudoVCLMUL_VX_M8_MASK	= 937,
    RISCV_PseudoVCLMUL_VX_MF2	= 938,
    RISCV_PseudoVCLMUL_VX_MF2_MASK	= 939,
    RISCV_PseudoVCLMUL_VX_MF4	= 940,
    RISCV_PseudoVCLMUL_VX_MF4_MASK	= 941,
    RISCV_PseudoVCLMUL_VX_MF8	= 942,
    RISCV_PseudoVCLMUL_VX_MF8_MASK	= 943,
    RISCV_PseudoVCLZ_V_M1	= 944,
    RISCV_PseudoVCLZ_V_M1_MASK	= 945,
    RISCV_PseudoVCLZ_V_M2	= 946,
    RISCV_PseudoVCLZ_V_M2_MASK	= 947,
    RISCV_PseudoVCLZ_V_M4	= 948,
    RISCV_PseudoVCLZ_V_M4_MASK	= 949,
    RISCV_PseudoVCLZ_V_M8	= 950,
    RISCV_PseudoVCLZ_V_M8_MASK	= 951,
    RISCV_PseudoVCLZ_V_MF2	= 952,
    RISCV_PseudoVCLZ_V_MF2_MASK	= 953,
    RISCV_PseudoVCLZ_V_MF4	= 954,
    RISCV_PseudoVCLZ_V_MF4_MASK	= 955,
    RISCV_PseudoVCLZ_V_MF8	= 956,
    RISCV_PseudoVCLZ_V_MF8_MASK	= 957,
    RISCV_PseudoVCOMPRESS_VM_M1_E16	= 958,
    RISCV_PseudoVCOMPRESS_VM_M1_E32	= 959,
    RISCV_PseudoVCOMPRESS_VM_M1_E64	= 960,
    RISCV_PseudoVCOMPRESS_VM_M1_E8	= 961,
    RISCV_PseudoVCOMPRESS_VM_M2_E16	= 962,
    RISCV_PseudoVCOMPRESS_VM_M2_E32	= 963,
    RISCV_PseudoVCOMPRESS_VM_M2_E64	= 964,
    RISCV_PseudoVCOMPRESS_VM_M2_E8	= 965,
    RISCV_PseudoVCOMPRESS_VM_M4_E16	= 966,
    RISCV_PseudoVCOMPRESS_VM_M4_E32	= 967,
    RISCV_PseudoVCOMPRESS_VM_M4_E64	= 968,
    RISCV_PseudoVCOMPRESS_VM_M4_E8	= 969,
    RISCV_PseudoVCOMPRESS_VM_M8_E16	= 970,
    RISCV_PseudoVCOMPRESS_VM_M8_E32	= 971,
    RISCV_PseudoVCOMPRESS_VM_M8_E64	= 972,
    RISCV_PseudoVCOMPRESS_VM_M8_E8	= 973,
    RISCV_PseudoVCOMPRESS_VM_MF2_E16	= 974,
    RISCV_PseudoVCOMPRESS_VM_MF2_E32	= 975,
    RISCV_PseudoVCOMPRESS_VM_MF2_E8	= 976,
    RISCV_PseudoVCOMPRESS_VM_MF4_E16	= 977,
    RISCV_PseudoVCOMPRESS_VM_MF4_E8	= 978,
    RISCV_PseudoVCOMPRESS_VM_MF8_E8	= 979,
    RISCV_PseudoVCPOP_M_B1	= 980,
    RISCV_PseudoVCPOP_M_B16	= 981,
    RISCV_PseudoVCPOP_M_B16_MASK	= 982,
    RISCV_PseudoVCPOP_M_B1_MASK	= 983,
    RISCV_PseudoVCPOP_M_B2	= 984,
    RISCV_PseudoVCPOP_M_B2_MASK	= 985,
    RISCV_PseudoVCPOP_M_B32	= 986,
    RISCV_PseudoVCPOP_M_B32_MASK	= 987,
    RISCV_PseudoVCPOP_M_B4	= 988,
    RISCV_PseudoVCPOP_M_B4_MASK	= 989,
    RISCV_PseudoVCPOP_M_B64	= 990,
    RISCV_PseudoVCPOP_M_B64_MASK	= 991,
    RISCV_PseudoVCPOP_M_B8	= 992,
    RISCV_PseudoVCPOP_M_B8_MASK	= 993,
    RISCV_PseudoVCPOP_V_M1	= 994,
    RISCV_PseudoVCPOP_V_M1_MASK	= 995,
    RISCV_PseudoVCPOP_V_M2	= 996,
    RISCV_PseudoVCPOP_V_M2_MASK	= 997,
    RISCV_PseudoVCPOP_V_M4	= 998,
    RISCV_PseudoVCPOP_V_M4_MASK	= 999,
    RISCV_PseudoVCPOP_V_M8	= 1000,
    RISCV_PseudoVCPOP_V_M8_MASK	= 1001,
    RISCV_PseudoVCPOP_V_MF2	= 1002,
    RISCV_PseudoVCPOP_V_MF2_MASK	= 1003,
    RISCV_PseudoVCPOP_V_MF4	= 1004,
    RISCV_PseudoVCPOP_V_MF4_MASK	= 1005,
    RISCV_PseudoVCPOP_V_MF8	= 1006,
    RISCV_PseudoVCPOP_V_MF8_MASK	= 1007,
    RISCV_PseudoVCTZ_V_M1	= 1008,
    RISCV_PseudoVCTZ_V_M1_MASK	= 1009,
    RISCV_PseudoVCTZ_V_M2	= 1010,
    RISCV_PseudoVCTZ_V_M2_MASK	= 1011,
    RISCV_PseudoVCTZ_V_M4	= 1012,
    RISCV_PseudoVCTZ_V_M4_MASK	= 1013,
    RISCV_PseudoVCTZ_V_M8	= 1014,
    RISCV_PseudoVCTZ_V_M8_MASK	= 1015,
    RISCV_PseudoVCTZ_V_MF2	= 1016,
    RISCV_PseudoVCTZ_V_MF2_MASK	= 1017,
    RISCV_PseudoVCTZ_V_MF4	= 1018,
    RISCV_PseudoVCTZ_V_MF4_MASK	= 1019,
    RISCV_PseudoVCTZ_V_MF8	= 1020,
    RISCV_PseudoVCTZ_V_MF8_MASK	= 1021,
    RISCV_PseudoVC_FPR16VV_SE_M1	= 1022,
    RISCV_PseudoVC_FPR16VV_SE_M2	= 1023,
    RISCV_PseudoVC_FPR16VV_SE_M4	= 1024,
    RISCV_PseudoVC_FPR16VV_SE_M8	= 1025,
    RISCV_PseudoVC_FPR16VV_SE_MF2	= 1026,
    RISCV_PseudoVC_FPR16VV_SE_MF4	= 1027,
    RISCV_PseudoVC_FPR16VW_SE_M1	= 1028,
    RISCV_PseudoVC_FPR16VW_SE_M2	= 1029,
    RISCV_PseudoVC_FPR16VW_SE_M4	= 1030,
    RISCV_PseudoVC_FPR16VW_SE_M8	= 1031,
    RISCV_PseudoVC_FPR16VW_SE_MF2	= 1032,
    RISCV_PseudoVC_FPR16VW_SE_MF4	= 1033,
    RISCV_PseudoVC_FPR16V_SE_M1	= 1034,
    RISCV_PseudoVC_FPR16V_SE_M2	= 1035,
    RISCV_PseudoVC_FPR16V_SE_M4	= 1036,
    RISCV_PseudoVC_FPR16V_SE_M8	= 1037,
    RISCV_PseudoVC_FPR16V_SE_MF2	= 1038,
    RISCV_PseudoVC_FPR16V_SE_MF4	= 1039,
    RISCV_PseudoVC_FPR32VV_SE_M1	= 1040,
    RISCV_PseudoVC_FPR32VV_SE_M2	= 1041,
    RISCV_PseudoVC_FPR32VV_SE_M4	= 1042,
    RISCV_PseudoVC_FPR32VV_SE_M8	= 1043,
    RISCV_PseudoVC_FPR32VV_SE_MF2	= 1044,
    RISCV_PseudoVC_FPR32VW_SE_M1	= 1045,
    RISCV_PseudoVC_FPR32VW_SE_M2	= 1046,
    RISCV_PseudoVC_FPR32VW_SE_M4	= 1047,
    RISCV_PseudoVC_FPR32VW_SE_M8	= 1048,
    RISCV_PseudoVC_FPR32VW_SE_MF2	= 1049,
    RISCV_PseudoVC_FPR32V_SE_M1	= 1050,
    RISCV_PseudoVC_FPR32V_SE_M2	= 1051,
    RISCV_PseudoVC_FPR32V_SE_M4	= 1052,
    RISCV_PseudoVC_FPR32V_SE_M8	= 1053,
    RISCV_PseudoVC_FPR32V_SE_MF2	= 1054,
    RISCV_PseudoVC_FPR64VV_SE_M1	= 1055,
    RISCV_PseudoVC_FPR64VV_SE_M2	= 1056,
    RISCV_PseudoVC_FPR64VV_SE_M4	= 1057,
    RISCV_PseudoVC_FPR64VV_SE_M8	= 1058,
    RISCV_PseudoVC_FPR64V_SE_M1	= 1059,
    RISCV_PseudoVC_FPR64V_SE_M2	= 1060,
    RISCV_PseudoVC_FPR64V_SE_M4	= 1061,
    RISCV_PseudoVC_FPR64V_SE_M8	= 1062,
    RISCV_PseudoVC_IVV_SE_M1	= 1063,
    RISCV_PseudoVC_IVV_SE_M2	= 1064,
    RISCV_PseudoVC_IVV_SE_M4	= 1065,
    RISCV_PseudoVC_IVV_SE_M8	= 1066,
    RISCV_PseudoVC_IVV_SE_MF2	= 1067,
    RISCV_PseudoVC_IVV_SE_MF4	= 1068,
    RISCV_PseudoVC_IVV_SE_MF8	= 1069,
    RISCV_PseudoVC_IVW_SE_M1	= 1070,
    RISCV_PseudoVC_IVW_SE_M2	= 1071,
    RISCV_PseudoVC_IVW_SE_M4	= 1072,
    RISCV_PseudoVC_IVW_SE_MF2	= 1073,
    RISCV_PseudoVC_IVW_SE_MF4	= 1074,
    RISCV_PseudoVC_IVW_SE_MF8	= 1075,
    RISCV_PseudoVC_IV_SE_M1	= 1076,
    RISCV_PseudoVC_IV_SE_M2	= 1077,
    RISCV_PseudoVC_IV_SE_M4	= 1078,
    RISCV_PseudoVC_IV_SE_M8	= 1079,
    RISCV_PseudoVC_IV_SE_MF2	= 1080,
    RISCV_PseudoVC_IV_SE_MF4	= 1081,
    RISCV_PseudoVC_IV_SE_MF8	= 1082,
    RISCV_PseudoVC_I_SE_M1	= 1083,
    RISCV_PseudoVC_I_SE_M2	= 1084,
    RISCV_PseudoVC_I_SE_M4	= 1085,
    RISCV_PseudoVC_I_SE_M8	= 1086,
    RISCV_PseudoVC_I_SE_MF2	= 1087,
    RISCV_PseudoVC_I_SE_MF4	= 1088,
    RISCV_PseudoVC_I_SE_MF8	= 1089,
    RISCV_PseudoVC_VVV_SE_M1	= 1090,
    RISCV_PseudoVC_VVV_SE_M2	= 1091,
    RISCV_PseudoVC_VVV_SE_M4	= 1092,
    RISCV_PseudoVC_VVV_SE_M8	= 1093,
    RISCV_PseudoVC_VVV_SE_MF2	= 1094,
    RISCV_PseudoVC_VVV_SE_MF4	= 1095,
    RISCV_PseudoVC_VVV_SE_MF8	= 1096,
    RISCV_PseudoVC_VVW_SE_M1	= 1097,
    RISCV_PseudoVC_VVW_SE_M2	= 1098,
    RISCV_PseudoVC_VVW_SE_M4	= 1099,
    RISCV_PseudoVC_VVW_SE_MF2	= 1100,
    RISCV_PseudoVC_VVW_SE_MF4	= 1101,
    RISCV_PseudoVC_VVW_SE_MF8	= 1102,
    RISCV_PseudoVC_VV_SE_M1	= 1103,
    RISCV_PseudoVC_VV_SE_M2	= 1104,
    RISCV_PseudoVC_VV_SE_M4	= 1105,
    RISCV_PseudoVC_VV_SE_M8	= 1106,
    RISCV_PseudoVC_VV_SE_MF2	= 1107,
    RISCV_PseudoVC_VV_SE_MF4	= 1108,
    RISCV_PseudoVC_VV_SE_MF8	= 1109,
    RISCV_PseudoVC_V_FPR16VV_M1	= 1110,
    RISCV_PseudoVC_V_FPR16VV_M2	= 1111,
    RISCV_PseudoVC_V_FPR16VV_M4	= 1112,
    RISCV_PseudoVC_V_FPR16VV_M8	= 1113,
    RISCV_PseudoVC_V_FPR16VV_MF2	= 1114,
    RISCV_PseudoVC_V_FPR16VV_MF4	= 1115,
    RISCV_PseudoVC_V_FPR16VV_SE_M1	= 1116,
    RISCV_PseudoVC_V_FPR16VV_SE_M2	= 1117,
    RISCV_PseudoVC_V_FPR16VV_SE_M4	= 1118,
    RISCV_PseudoVC_V_FPR16VV_SE_M8	= 1119,
    RISCV_PseudoVC_V_FPR16VV_SE_MF2	= 1120,
    RISCV_PseudoVC_V_FPR16VV_SE_MF4	= 1121,
    RISCV_PseudoVC_V_FPR16VW_M1	= 1122,
    RISCV_PseudoVC_V_FPR16VW_M2	= 1123,
    RISCV_PseudoVC_V_FPR16VW_M4	= 1124,
    RISCV_PseudoVC_V_FPR16VW_M8	= 1125,
    RISCV_PseudoVC_V_FPR16VW_MF2	= 1126,
    RISCV_PseudoVC_V_FPR16VW_MF4	= 1127,
    RISCV_PseudoVC_V_FPR16VW_SE_M1	= 1128,
    RISCV_PseudoVC_V_FPR16VW_SE_M2	= 1129,
    RISCV_PseudoVC_V_FPR16VW_SE_M4	= 1130,
    RISCV_PseudoVC_V_FPR16VW_SE_M8	= 1131,
    RISCV_PseudoVC_V_FPR16VW_SE_MF2	= 1132,
    RISCV_PseudoVC_V_FPR16VW_SE_MF4	= 1133,
    RISCV_PseudoVC_V_FPR16V_M1	= 1134,
    RISCV_PseudoVC_V_FPR16V_M2	= 1135,
    RISCV_PseudoVC_V_FPR16V_M4	= 1136,
    RISCV_PseudoVC_V_FPR16V_M8	= 1137,
    RISCV_PseudoVC_V_FPR16V_MF2	= 1138,
    RISCV_PseudoVC_V_FPR16V_MF4	= 1139,
    RISCV_PseudoVC_V_FPR16V_SE_M1	= 1140,
    RISCV_PseudoVC_V_FPR16V_SE_M2	= 1141,
    RISCV_PseudoVC_V_FPR16V_SE_M4	= 1142,
    RISCV_PseudoVC_V_FPR16V_SE_M8	= 1143,
    RISCV_PseudoVC_V_FPR16V_SE_MF2	= 1144,
    RISCV_PseudoVC_V_FPR16V_SE_MF4	= 1145,
    RISCV_PseudoVC_V_FPR32VV_M1	= 1146,
    RISCV_PseudoVC_V_FPR32VV_M2	= 1147,
    RISCV_PseudoVC_V_FPR32VV_M4	= 1148,
    RISCV_PseudoVC_V_FPR32VV_M8	= 1149,
    RISCV_PseudoVC_V_FPR32VV_MF2	= 1150,
    RISCV_PseudoVC_V_FPR32VV_SE_M1	= 1151,
    RISCV_PseudoVC_V_FPR32VV_SE_M2	= 1152,
    RISCV_PseudoVC_V_FPR32VV_SE_M4	= 1153,
    RISCV_PseudoVC_V_FPR32VV_SE_M8	= 1154,
    RISCV_PseudoVC_V_FPR32VV_SE_MF2	= 1155,
    RISCV_PseudoVC_V_FPR32VW_M1	= 1156,
    RISCV_PseudoVC_V_FPR32VW_M2	= 1157,
    RISCV_PseudoVC_V_FPR32VW_M4	= 1158,
    RISCV_PseudoVC_V_FPR32VW_M8	= 1159,
    RISCV_PseudoVC_V_FPR32VW_MF2	= 1160,
    RISCV_PseudoVC_V_FPR32VW_SE_M1	= 1161,
    RISCV_PseudoVC_V_FPR32VW_SE_M2	= 1162,
    RISCV_PseudoVC_V_FPR32VW_SE_M4	= 1163,
    RISCV_PseudoVC_V_FPR32VW_SE_M8	= 1164,
    RISCV_PseudoVC_V_FPR32VW_SE_MF2	= 1165,
    RISCV_PseudoVC_V_FPR32V_M1	= 1166,
    RISCV_PseudoVC_V_FPR32V_M2	= 1167,
    RISCV_PseudoVC_V_FPR32V_M4	= 1168,
    RISCV_PseudoVC_V_FPR32V_M8	= 1169,
    RISCV_PseudoVC_V_FPR32V_MF2	= 1170,
    RISCV_PseudoVC_V_FPR32V_SE_M1	= 1171,
    RISCV_PseudoVC_V_FPR32V_SE_M2	= 1172,
    RISCV_PseudoVC_V_FPR32V_SE_M4	= 1173,
    RISCV_PseudoVC_V_FPR32V_SE_M8	= 1174,
    RISCV_PseudoVC_V_FPR32V_SE_MF2	= 1175,
    RISCV_PseudoVC_V_FPR64VV_M1	= 1176,
    RISCV_PseudoVC_V_FPR64VV_M2	= 1177,
    RISCV_PseudoVC_V_FPR64VV_M4	= 1178,
    RISCV_PseudoVC_V_FPR64VV_M8	= 1179,
    RISCV_PseudoVC_V_FPR64VV_SE_M1	= 1180,
    RISCV_PseudoVC_V_FPR64VV_SE_M2	= 1181,
    RISCV_PseudoVC_V_FPR64VV_SE_M4	= 1182,
    RISCV_PseudoVC_V_FPR64VV_SE_M8	= 1183,
    RISCV_PseudoVC_V_FPR64V_M1	= 1184,
    RISCV_PseudoVC_V_FPR64V_M2	= 1185,
    RISCV_PseudoVC_V_FPR64V_M4	= 1186,
    RISCV_PseudoVC_V_FPR64V_M8	= 1187,
    RISCV_PseudoVC_V_FPR64V_SE_M1	= 1188,
    RISCV_PseudoVC_V_FPR64V_SE_M2	= 1189,
    RISCV_PseudoVC_V_FPR64V_SE_M4	= 1190,
    RISCV_PseudoVC_V_FPR64V_SE_M8	= 1191,
    RISCV_PseudoVC_V_IVV_M1	= 1192,
    RISCV_PseudoVC_V_IVV_M2	= 1193,
    RISCV_PseudoVC_V_IVV_M4	= 1194,
    RISCV_PseudoVC_V_IVV_M8	= 1195,
    RISCV_PseudoVC_V_IVV_MF2	= 1196,
    RISCV_PseudoVC_V_IVV_MF4	= 1197,
    RISCV_PseudoVC_V_IVV_MF8	= 1198,
    RISCV_PseudoVC_V_IVV_SE_M1	= 1199,
    RISCV_PseudoVC_V_IVV_SE_M2	= 1200,
    RISCV_PseudoVC_V_IVV_SE_M4	= 1201,
    RISCV_PseudoVC_V_IVV_SE_M8	= 1202,
    RISCV_PseudoVC_V_IVV_SE_MF2	= 1203,
    RISCV_PseudoVC_V_IVV_SE_MF4	= 1204,
    RISCV_PseudoVC_V_IVV_SE_MF8	= 1205,
    RISCV_PseudoVC_V_IVW_M1	= 1206,
    RISCV_PseudoVC_V_IVW_M2	= 1207,
    RISCV_PseudoVC_V_IVW_M4	= 1208,
    RISCV_PseudoVC_V_IVW_MF2	= 1209,
    RISCV_PseudoVC_V_IVW_MF4	= 1210,
    RISCV_PseudoVC_V_IVW_MF8	= 1211,
    RISCV_PseudoVC_V_IVW_SE_M1	= 1212,
    RISCV_PseudoVC_V_IVW_SE_M2	= 1213,
    RISCV_PseudoVC_V_IVW_SE_M4	= 1214,
    RISCV_PseudoVC_V_IVW_SE_MF2	= 1215,
    RISCV_PseudoVC_V_IVW_SE_MF4	= 1216,
    RISCV_PseudoVC_V_IVW_SE_MF8	= 1217,
    RISCV_PseudoVC_V_IV_M1	= 1218,
    RISCV_PseudoVC_V_IV_M2	= 1219,
    RISCV_PseudoVC_V_IV_M4	= 1220,
    RISCV_PseudoVC_V_IV_M8	= 1221,
    RISCV_PseudoVC_V_IV_MF2	= 1222,
    RISCV_PseudoVC_V_IV_MF4	= 1223,
    RISCV_PseudoVC_V_IV_MF8	= 1224,
    RISCV_PseudoVC_V_IV_SE_M1	= 1225,
    RISCV_PseudoVC_V_IV_SE_M2	= 1226,
    RISCV_PseudoVC_V_IV_SE_M4	= 1227,
    RISCV_PseudoVC_V_IV_SE_M8	= 1228,
    RISCV_PseudoVC_V_IV_SE_MF2	= 1229,
    RISCV_PseudoVC_V_IV_SE_MF4	= 1230,
    RISCV_PseudoVC_V_IV_SE_MF8	= 1231,
    RISCV_PseudoVC_V_I_M1	= 1232,
    RISCV_PseudoVC_V_I_M2	= 1233,
    RISCV_PseudoVC_V_I_M4	= 1234,
    RISCV_PseudoVC_V_I_M8	= 1235,
    RISCV_PseudoVC_V_I_MF2	= 1236,
    RISCV_PseudoVC_V_I_MF4	= 1237,
    RISCV_PseudoVC_V_I_MF8	= 1238,
    RISCV_PseudoVC_V_I_SE_M1	= 1239,
    RISCV_PseudoVC_V_I_SE_M2	= 1240,
    RISCV_PseudoVC_V_I_SE_M4	= 1241,
    RISCV_PseudoVC_V_I_SE_M8	= 1242,
    RISCV_PseudoVC_V_I_SE_MF2	= 1243,
    RISCV_PseudoVC_V_I_SE_MF4	= 1244,
    RISCV_PseudoVC_V_I_SE_MF8	= 1245,
    RISCV_PseudoVC_V_VVV_M1	= 1246,
    RISCV_PseudoVC_V_VVV_M2	= 1247,
    RISCV_PseudoVC_V_VVV_M4	= 1248,
    RISCV_PseudoVC_V_VVV_M8	= 1249,
    RISCV_PseudoVC_V_VVV_MF2	= 1250,
    RISCV_PseudoVC_V_VVV_MF4	= 1251,
    RISCV_PseudoVC_V_VVV_MF8	= 1252,
    RISCV_PseudoVC_V_VVV_SE_M1	= 1253,
    RISCV_PseudoVC_V_VVV_SE_M2	= 1254,
    RISCV_PseudoVC_V_VVV_SE_M4	= 1255,
    RISCV_PseudoVC_V_VVV_SE_M8	= 1256,
    RISCV_PseudoVC_V_VVV_SE_MF2	= 1257,
    RISCV_PseudoVC_V_VVV_SE_MF4	= 1258,
    RISCV_PseudoVC_V_VVV_SE_MF8	= 1259,
    RISCV_PseudoVC_V_VVW_M1	= 1260,
    RISCV_PseudoVC_V_VVW_M2	= 1261,
    RISCV_PseudoVC_V_VVW_M4	= 1262,
    RISCV_PseudoVC_V_VVW_MF2	= 1263,
    RISCV_PseudoVC_V_VVW_MF4	= 1264,
    RISCV_PseudoVC_V_VVW_MF8	= 1265,
    RISCV_PseudoVC_V_VVW_SE_M1	= 1266,
    RISCV_PseudoVC_V_VVW_SE_M2	= 1267,
    RISCV_PseudoVC_V_VVW_SE_M4	= 1268,
    RISCV_PseudoVC_V_VVW_SE_MF2	= 1269,
    RISCV_PseudoVC_V_VVW_SE_MF4	= 1270,
    RISCV_PseudoVC_V_VVW_SE_MF8	= 1271,
    RISCV_PseudoVC_V_VV_M1	= 1272,
    RISCV_PseudoVC_V_VV_M2	= 1273,
    RISCV_PseudoVC_V_VV_M4	= 1274,
    RISCV_PseudoVC_V_VV_M8	= 1275,
    RISCV_PseudoVC_V_VV_MF2	= 1276,
    RISCV_PseudoVC_V_VV_MF4	= 1277,
    RISCV_PseudoVC_V_VV_MF8	= 1278,
    RISCV_PseudoVC_V_VV_SE_M1	= 1279,
    RISCV_PseudoVC_V_VV_SE_M2	= 1280,
    RISCV_PseudoVC_V_VV_SE_M4	= 1281,
    RISCV_PseudoVC_V_VV_SE_M8	= 1282,
    RISCV_PseudoVC_V_VV_SE_MF2	= 1283,
    RISCV_PseudoVC_V_VV_SE_MF4	= 1284,
    RISCV_PseudoVC_V_VV_SE_MF8	= 1285,
    RISCV_PseudoVC_V_XVV_M1	= 1286,
    RISCV_PseudoVC_V_XVV_M2	= 1287,
    RISCV_PseudoVC_V_XVV_M4	= 1288,
    RISCV_PseudoVC_V_XVV_M8	= 1289,
    RISCV_PseudoVC_V_XVV_MF2	= 1290,
    RISCV_PseudoVC_V_XVV_MF4	= 1291,
    RISCV_PseudoVC_V_XVV_MF8	= 1292,
    RISCV_PseudoVC_V_XVV_SE_M1	= 1293,
    RISCV_PseudoVC_V_XVV_SE_M2	= 1294,
    RISCV_PseudoVC_V_XVV_SE_M4	= 1295,
    RISCV_PseudoVC_V_XVV_SE_M8	= 1296,
    RISCV_PseudoVC_V_XVV_SE_MF2	= 1297,
    RISCV_PseudoVC_V_XVV_SE_MF4	= 1298,
    RISCV_PseudoVC_V_XVV_SE_MF8	= 1299,
    RISCV_PseudoVC_V_XVW_M1	= 1300,
    RISCV_PseudoVC_V_XVW_M2	= 1301,
    RISCV_PseudoVC_V_XVW_M4	= 1302,
    RISCV_PseudoVC_V_XVW_MF2	= 1303,
    RISCV_PseudoVC_V_XVW_MF4	= 1304,
    RISCV_PseudoVC_V_XVW_MF8	= 1305,
    RISCV_PseudoVC_V_XVW_SE_M1	= 1306,
    RISCV_PseudoVC_V_XVW_SE_M2	= 1307,
    RISCV_PseudoVC_V_XVW_SE_M4	= 1308,
    RISCV_PseudoVC_V_XVW_SE_MF2	= 1309,
    RISCV_PseudoVC_V_XVW_SE_MF4	= 1310,
    RISCV_PseudoVC_V_XVW_SE_MF8	= 1311,
    RISCV_PseudoVC_V_XV_M1	= 1312,
    RISCV_PseudoVC_V_XV_M2	= 1313,
    RISCV_PseudoVC_V_XV_M4	= 1314,
    RISCV_PseudoVC_V_XV_M8	= 1315,
    RISCV_PseudoVC_V_XV_MF2	= 1316,
    RISCV_PseudoVC_V_XV_MF4	= 1317,
    RISCV_PseudoVC_V_XV_MF8	= 1318,
    RISCV_PseudoVC_V_XV_SE_M1	= 1319,
    RISCV_PseudoVC_V_XV_SE_M2	= 1320,
    RISCV_PseudoVC_V_XV_SE_M4	= 1321,
    RISCV_PseudoVC_V_XV_SE_M8	= 1322,
    RISCV_PseudoVC_V_XV_SE_MF2	= 1323,
    RISCV_PseudoVC_V_XV_SE_MF4	= 1324,
    RISCV_PseudoVC_V_XV_SE_MF8	= 1325,
    RISCV_PseudoVC_V_X_M1	= 1326,
    RISCV_PseudoVC_V_X_M2	= 1327,
    RISCV_PseudoVC_V_X_M4	= 1328,
    RISCV_PseudoVC_V_X_M8	= 1329,
    RISCV_PseudoVC_V_X_MF2	= 1330,
    RISCV_PseudoVC_V_X_MF4	= 1331,
    RISCV_PseudoVC_V_X_MF8	= 1332,
    RISCV_PseudoVC_V_X_SE_M1	= 1333,
    RISCV_PseudoVC_V_X_SE_M2	= 1334,
    RISCV_PseudoVC_V_X_SE_M4	= 1335,
    RISCV_PseudoVC_V_X_SE_M8	= 1336,
    RISCV_PseudoVC_V_X_SE_MF2	= 1337,
    RISCV_PseudoVC_V_X_SE_MF4	= 1338,
    RISCV_PseudoVC_V_X_SE_MF8	= 1339,
    RISCV_PseudoVC_XVV_SE_M1	= 1340,
    RISCV_PseudoVC_XVV_SE_M2	= 1341,
    RISCV_PseudoVC_XVV_SE_M4	= 1342,
    RISCV_PseudoVC_XVV_SE_M8	= 1343,
    RISCV_PseudoVC_XVV_SE_MF2	= 1344,
    RISCV_PseudoVC_XVV_SE_MF4	= 1345,
    RISCV_PseudoVC_XVV_SE_MF8	= 1346,
    RISCV_PseudoVC_XVW_SE_M1	= 1347,
    RISCV_PseudoVC_XVW_SE_M2	= 1348,
    RISCV_PseudoVC_XVW_SE_M4	= 1349,
    RISCV_PseudoVC_XVW_SE_MF2	= 1350,
    RISCV_PseudoVC_XVW_SE_MF4	= 1351,
    RISCV_PseudoVC_XVW_SE_MF8	= 1352,
    RISCV_PseudoVC_XV_SE_M1	= 1353,
    RISCV_PseudoVC_XV_SE_M2	= 1354,
    RISCV_PseudoVC_XV_SE_M4	= 1355,
    RISCV_PseudoVC_XV_SE_M8	= 1356,
    RISCV_PseudoVC_XV_SE_MF2	= 1357,
    RISCV_PseudoVC_XV_SE_MF4	= 1358,
    RISCV_PseudoVC_XV_SE_MF8	= 1359,
    RISCV_PseudoVC_X_SE_M1	= 1360,
    RISCV_PseudoVC_X_SE_M2	= 1361,
    RISCV_PseudoVC_X_SE_M4	= 1362,
    RISCV_PseudoVC_X_SE_M8	= 1363,
    RISCV_PseudoVC_X_SE_MF2	= 1364,
    RISCV_PseudoVC_X_SE_MF4	= 1365,
    RISCV_PseudoVC_X_SE_MF8	= 1366,
    RISCV_PseudoVDIVU_VV_M1_E16	= 1367,
    RISCV_PseudoVDIVU_VV_M1_E16_MASK	= 1368,
    RISCV_PseudoVDIVU_VV_M1_E32	= 1369,
    RISCV_PseudoVDIVU_VV_M1_E32_MASK	= 1370,
    RISCV_PseudoVDIVU_VV_M1_E64	= 1371,
    RISCV_PseudoVDIVU_VV_M1_E64_MASK	= 1372,
    RISCV_PseudoVDIVU_VV_M1_E8	= 1373,
    RISCV_PseudoVDIVU_VV_M1_E8_MASK	= 1374,
    RISCV_PseudoVDIVU_VV_M2_E16	= 1375,
    RISCV_PseudoVDIVU_VV_M2_E16_MASK	= 1376,
    RISCV_PseudoVDIVU_VV_M2_E32	= 1377,
    RISCV_PseudoVDIVU_VV_M2_E32_MASK	= 1378,
    RISCV_PseudoVDIVU_VV_M2_E64	= 1379,
    RISCV_PseudoVDIVU_VV_M2_E64_MASK	= 1380,
    RISCV_PseudoVDIVU_VV_M2_E8	= 1381,
    RISCV_PseudoVDIVU_VV_M2_E8_MASK	= 1382,
    RISCV_PseudoVDIVU_VV_M4_E16	= 1383,
    RISCV_PseudoVDIVU_VV_M4_E16_MASK	= 1384,
    RISCV_PseudoVDIVU_VV_M4_E32	= 1385,
    RISCV_PseudoVDIVU_VV_M4_E32_MASK	= 1386,
    RISCV_PseudoVDIVU_VV_M4_E64	= 1387,
    RISCV_PseudoVDIVU_VV_M4_E64_MASK	= 1388,
    RISCV_PseudoVDIVU_VV_M4_E8	= 1389,
    RISCV_PseudoVDIVU_VV_M4_E8_MASK	= 1390,
    RISCV_PseudoVDIVU_VV_M8_E16	= 1391,
    RISCV_PseudoVDIVU_VV_M8_E16_MASK	= 1392,
    RISCV_PseudoVDIVU_VV_M8_E32	= 1393,
    RISCV_PseudoVDIVU_VV_M8_E32_MASK	= 1394,
    RISCV_PseudoVDIVU_VV_M8_E64	= 1395,
    RISCV_PseudoVDIVU_VV_M8_E64_MASK	= 1396,
    RISCV_PseudoVDIVU_VV_M8_E8	= 1397,
    RISCV_PseudoVDIVU_VV_M8_E8_MASK	= 1398,
    RISCV_PseudoVDIVU_VV_MF2_E16	= 1399,
    RISCV_PseudoVDIVU_VV_MF2_E16_MASK	= 1400,
    RISCV_PseudoVDIVU_VV_MF2_E32	= 1401,
    RISCV_PseudoVDIVU_VV_MF2_E32_MASK	= 1402,
    RISCV_PseudoVDIVU_VV_MF2_E8	= 1403,
    RISCV_PseudoVDIVU_VV_MF2_E8_MASK	= 1404,
    RISCV_PseudoVDIVU_VV_MF4_E16	= 1405,
    RISCV_PseudoVDIVU_VV_MF4_E16_MASK	= 1406,
    RISCV_PseudoVDIVU_VV_MF4_E8	= 1407,
    RISCV_PseudoVDIVU_VV_MF4_E8_MASK	= 1408,
    RISCV_PseudoVDIVU_VV_MF8_E8	= 1409,
    RISCV_PseudoVDIVU_VV_MF8_E8_MASK	= 1410,
    RISCV_PseudoVDIVU_VX_M1_E16	= 1411,
    RISCV_PseudoVDIVU_VX_M1_E16_MASK	= 1412,
    RISCV_PseudoVDIVU_VX_M1_E32	= 1413,
    RISCV_PseudoVDIVU_VX_M1_E32_MASK	= 1414,
    RISCV_PseudoVDIVU_VX_M1_E64	= 1415,
    RISCV_PseudoVDIVU_VX_M1_E64_MASK	= 1416,
    RISCV_PseudoVDIVU_VX_M1_E8	= 1417,
    RISCV_PseudoVDIVU_VX_M1_E8_MASK	= 1418,
    RISCV_PseudoVDIVU_VX_M2_E16	= 1419,
    RISCV_PseudoVDIVU_VX_M2_E16_MASK	= 1420,
    RISCV_PseudoVDIVU_VX_M2_E32	= 1421,
    RISCV_PseudoVDIVU_VX_M2_E32_MASK	= 1422,
    RISCV_PseudoVDIVU_VX_M2_E64	= 1423,
    RISCV_PseudoVDIVU_VX_M2_E64_MASK	= 1424,
    RISCV_PseudoVDIVU_VX_M2_E8	= 1425,
    RISCV_PseudoVDIVU_VX_M2_E8_MASK	= 1426,
    RISCV_PseudoVDIVU_VX_M4_E16	= 1427,
    RISCV_PseudoVDIVU_VX_M4_E16_MASK	= 1428,
    RISCV_PseudoVDIVU_VX_M4_E32	= 1429,
    RISCV_PseudoVDIVU_VX_M4_E32_MASK	= 1430,
    RISCV_PseudoVDIVU_VX_M4_E64	= 1431,
    RISCV_PseudoVDIVU_VX_M4_E64_MASK	= 1432,
    RISCV_PseudoVDIVU_VX_M4_E8	= 1433,
    RISCV_PseudoVDIVU_VX_M4_E8_MASK	= 1434,
    RISCV_PseudoVDIVU_VX_M8_E16	= 1435,
    RISCV_PseudoVDIVU_VX_M8_E16_MASK	= 1436,
    RISCV_PseudoVDIVU_VX_M8_E32	= 1437,
    RISCV_PseudoVDIVU_VX_M8_E32_MASK	= 1438,
    RISCV_PseudoVDIVU_VX_M8_E64	= 1439,
    RISCV_PseudoVDIVU_VX_M8_E64_MASK	= 1440,
    RISCV_PseudoVDIVU_VX_M8_E8	= 1441,
    RISCV_PseudoVDIVU_VX_M8_E8_MASK	= 1442,
    RISCV_PseudoVDIVU_VX_MF2_E16	= 1443,
    RISCV_PseudoVDIVU_VX_MF2_E16_MASK	= 1444,
    RISCV_PseudoVDIVU_VX_MF2_E32	= 1445,
    RISCV_PseudoVDIVU_VX_MF2_E32_MASK	= 1446,
    RISCV_PseudoVDIVU_VX_MF2_E8	= 1447,
    RISCV_PseudoVDIVU_VX_MF2_E8_MASK	= 1448,
    RISCV_PseudoVDIVU_VX_MF4_E16	= 1449,
    RISCV_PseudoVDIVU_VX_MF4_E16_MASK	= 1450,
    RISCV_PseudoVDIVU_VX_MF4_E8	= 1451,
    RISCV_PseudoVDIVU_VX_MF4_E8_MASK	= 1452,
    RISCV_PseudoVDIVU_VX_MF8_E8	= 1453,
    RISCV_PseudoVDIVU_VX_MF8_E8_MASK	= 1454,
    RISCV_PseudoVDIV_VV_M1_E16	= 1455,
    RISCV_PseudoVDIV_VV_M1_E16_MASK	= 1456,
    RISCV_PseudoVDIV_VV_M1_E32	= 1457,
    RISCV_PseudoVDIV_VV_M1_E32_MASK	= 1458,
    RISCV_PseudoVDIV_VV_M1_E64	= 1459,
    RISCV_PseudoVDIV_VV_M1_E64_MASK	= 1460,
    RISCV_PseudoVDIV_VV_M1_E8	= 1461,
    RISCV_PseudoVDIV_VV_M1_E8_MASK	= 1462,
    RISCV_PseudoVDIV_VV_M2_E16	= 1463,
    RISCV_PseudoVDIV_VV_M2_E16_MASK	= 1464,
    RISCV_PseudoVDIV_VV_M2_E32	= 1465,
    RISCV_PseudoVDIV_VV_M2_E32_MASK	= 1466,
    RISCV_PseudoVDIV_VV_M2_E64	= 1467,
    RISCV_PseudoVDIV_VV_M2_E64_MASK	= 1468,
    RISCV_PseudoVDIV_VV_M2_E8	= 1469,
    RISCV_PseudoVDIV_VV_M2_E8_MASK	= 1470,
    RISCV_PseudoVDIV_VV_M4_E16	= 1471,
    RISCV_PseudoVDIV_VV_M4_E16_MASK	= 1472,
    RISCV_PseudoVDIV_VV_M4_E32	= 1473,
    RISCV_PseudoVDIV_VV_M4_E32_MASK	= 1474,
    RISCV_PseudoVDIV_VV_M4_E64	= 1475,
    RISCV_PseudoVDIV_VV_M4_E64_MASK	= 1476,
    RISCV_PseudoVDIV_VV_M4_E8	= 1477,
    RISCV_PseudoVDIV_VV_M4_E8_MASK	= 1478,
    RISCV_PseudoVDIV_VV_M8_E16	= 1479,
    RISCV_PseudoVDIV_VV_M8_E16_MASK	= 1480,
    RISCV_PseudoVDIV_VV_M8_E32	= 1481,
    RISCV_PseudoVDIV_VV_M8_E32_MASK	= 1482,
    RISCV_PseudoVDIV_VV_M8_E64	= 1483,
    RISCV_PseudoVDIV_VV_M8_E64_MASK	= 1484,
    RISCV_PseudoVDIV_VV_M8_E8	= 1485,
    RISCV_PseudoVDIV_VV_M8_E8_MASK	= 1486,
    RISCV_PseudoVDIV_VV_MF2_E16	= 1487,
    RISCV_PseudoVDIV_VV_MF2_E16_MASK	= 1488,
    RISCV_PseudoVDIV_VV_MF2_E32	= 1489,
    RISCV_PseudoVDIV_VV_MF2_E32_MASK	= 1490,
    RISCV_PseudoVDIV_VV_MF2_E8	= 1491,
    RISCV_PseudoVDIV_VV_MF2_E8_MASK	= 1492,
    RISCV_PseudoVDIV_VV_MF4_E16	= 1493,
    RISCV_PseudoVDIV_VV_MF4_E16_MASK	= 1494,
    RISCV_PseudoVDIV_VV_MF4_E8	= 1495,
    RISCV_PseudoVDIV_VV_MF4_E8_MASK	= 1496,
    RISCV_PseudoVDIV_VV_MF8_E8	= 1497,
    RISCV_PseudoVDIV_VV_MF8_E8_MASK	= 1498,
    RISCV_PseudoVDIV_VX_M1_E16	= 1499,
    RISCV_PseudoVDIV_VX_M1_E16_MASK	= 1500,
    RISCV_PseudoVDIV_VX_M1_E32	= 1501,
    RISCV_PseudoVDIV_VX_M1_E32_MASK	= 1502,
    RISCV_PseudoVDIV_VX_M1_E64	= 1503,
    RISCV_PseudoVDIV_VX_M1_E64_MASK	= 1504,
    RISCV_PseudoVDIV_VX_M1_E8	= 1505,
    RISCV_PseudoVDIV_VX_M1_E8_MASK	= 1506,
    RISCV_PseudoVDIV_VX_M2_E16	= 1507,
    RISCV_PseudoVDIV_VX_M2_E16_MASK	= 1508,
    RISCV_PseudoVDIV_VX_M2_E32	= 1509,
    RISCV_PseudoVDIV_VX_M2_E32_MASK	= 1510,
    RISCV_PseudoVDIV_VX_M2_E64	= 1511,
    RISCV_PseudoVDIV_VX_M2_E64_MASK	= 1512,
    RISCV_PseudoVDIV_VX_M2_E8	= 1513,
    RISCV_PseudoVDIV_VX_M2_E8_MASK	= 1514,
    RISCV_PseudoVDIV_VX_M4_E16	= 1515,
    RISCV_PseudoVDIV_VX_M4_E16_MASK	= 1516,
    RISCV_PseudoVDIV_VX_M4_E32	= 1517,
    RISCV_PseudoVDIV_VX_M4_E32_MASK	= 1518,
    RISCV_PseudoVDIV_VX_M4_E64	= 1519,
    RISCV_PseudoVDIV_VX_M4_E64_MASK	= 1520,
    RISCV_PseudoVDIV_VX_M4_E8	= 1521,
    RISCV_PseudoVDIV_VX_M4_E8_MASK	= 1522,
    RISCV_PseudoVDIV_VX_M8_E16	= 1523,
    RISCV_PseudoVDIV_VX_M8_E16_MASK	= 1524,
    RISCV_PseudoVDIV_VX_M8_E32	= 1525,
    RISCV_PseudoVDIV_VX_M8_E32_MASK	= 1526,
    RISCV_PseudoVDIV_VX_M8_E64	= 1527,
    RISCV_PseudoVDIV_VX_M8_E64_MASK	= 1528,
    RISCV_PseudoVDIV_VX_M8_E8	= 1529,
    RISCV_PseudoVDIV_VX_M8_E8_MASK	= 1530,
    RISCV_PseudoVDIV_VX_MF2_E16	= 1531,
    RISCV_PseudoVDIV_VX_MF2_E16_MASK	= 1532,
    RISCV_PseudoVDIV_VX_MF2_E32	= 1533,
    RISCV_PseudoVDIV_VX_MF2_E32_MASK	= 1534,
    RISCV_PseudoVDIV_VX_MF2_E8	= 1535,
    RISCV_PseudoVDIV_VX_MF2_E8_MASK	= 1536,
    RISCV_PseudoVDIV_VX_MF4_E16	= 1537,
    RISCV_PseudoVDIV_VX_MF4_E16_MASK	= 1538,
    RISCV_PseudoVDIV_VX_MF4_E8	= 1539,
    RISCV_PseudoVDIV_VX_MF4_E8_MASK	= 1540,
    RISCV_PseudoVDIV_VX_MF8_E8	= 1541,
    RISCV_PseudoVDIV_VX_MF8_E8_MASK	= 1542,
    RISCV_PseudoVFADD_VFPR16_M1	= 1543,
    RISCV_PseudoVFADD_VFPR16_M1_MASK	= 1544,
    RISCV_PseudoVFADD_VFPR16_M2	= 1545,
    RISCV_PseudoVFADD_VFPR16_M2_MASK	= 1546,
    RISCV_PseudoVFADD_VFPR16_M4	= 1547,
    RISCV_PseudoVFADD_VFPR16_M4_MASK	= 1548,
    RISCV_PseudoVFADD_VFPR16_M8	= 1549,
    RISCV_PseudoVFADD_VFPR16_M8_MASK	= 1550,
    RISCV_PseudoVFADD_VFPR16_MF2	= 1551,
    RISCV_PseudoVFADD_VFPR16_MF2_MASK	= 1552,
    RISCV_PseudoVFADD_VFPR16_MF4	= 1553,
    RISCV_PseudoVFADD_VFPR16_MF4_MASK	= 1554,
    RISCV_PseudoVFADD_VFPR32_M1	= 1555,
    RISCV_PseudoVFADD_VFPR32_M1_MASK	= 1556,
    RISCV_PseudoVFADD_VFPR32_M2	= 1557,
    RISCV_PseudoVFADD_VFPR32_M2_MASK	= 1558,
    RISCV_PseudoVFADD_VFPR32_M4	= 1559,
    RISCV_PseudoVFADD_VFPR32_M4_MASK	= 1560,
    RISCV_PseudoVFADD_VFPR32_M8	= 1561,
    RISCV_PseudoVFADD_VFPR32_M8_MASK	= 1562,
    RISCV_PseudoVFADD_VFPR32_MF2	= 1563,
    RISCV_PseudoVFADD_VFPR32_MF2_MASK	= 1564,
    RISCV_PseudoVFADD_VFPR64_M1	= 1565,
    RISCV_PseudoVFADD_VFPR64_M1_MASK	= 1566,
    RISCV_PseudoVFADD_VFPR64_M2	= 1567,
    RISCV_PseudoVFADD_VFPR64_M2_MASK	= 1568,
    RISCV_PseudoVFADD_VFPR64_M4	= 1569,
    RISCV_PseudoVFADD_VFPR64_M4_MASK	= 1570,
    RISCV_PseudoVFADD_VFPR64_M8	= 1571,
    RISCV_PseudoVFADD_VFPR64_M8_MASK	= 1572,
    RISCV_PseudoVFADD_VV_M1	= 1573,
    RISCV_PseudoVFADD_VV_M1_MASK	= 1574,
    RISCV_PseudoVFADD_VV_M2	= 1575,
    RISCV_PseudoVFADD_VV_M2_MASK	= 1576,
    RISCV_PseudoVFADD_VV_M4	= 1577,
    RISCV_PseudoVFADD_VV_M4_MASK	= 1578,
    RISCV_PseudoVFADD_VV_M8	= 1579,
    RISCV_PseudoVFADD_VV_M8_MASK	= 1580,
    RISCV_PseudoVFADD_VV_MF2	= 1581,
    RISCV_PseudoVFADD_VV_MF2_MASK	= 1582,
    RISCV_PseudoVFADD_VV_MF4	= 1583,
    RISCV_PseudoVFADD_VV_MF4_MASK	= 1584,
    RISCV_PseudoVFCLASS_V_M1	= 1585,
    RISCV_PseudoVFCLASS_V_M1_MASK	= 1586,
    RISCV_PseudoVFCLASS_V_M2	= 1587,
    RISCV_PseudoVFCLASS_V_M2_MASK	= 1588,
    RISCV_PseudoVFCLASS_V_M4	= 1589,
    RISCV_PseudoVFCLASS_V_M4_MASK	= 1590,
    RISCV_PseudoVFCLASS_V_M8	= 1591,
    RISCV_PseudoVFCLASS_V_M8_MASK	= 1592,
    RISCV_PseudoVFCLASS_V_MF2	= 1593,
    RISCV_PseudoVFCLASS_V_MF2_MASK	= 1594,
    RISCV_PseudoVFCLASS_V_MF4	= 1595,
    RISCV_PseudoVFCLASS_V_MF4_MASK	= 1596,
    RISCV_PseudoVFCVT_F_XU_V_M1	= 1597,
    RISCV_PseudoVFCVT_F_XU_V_M1_MASK	= 1598,
    RISCV_PseudoVFCVT_F_XU_V_M2	= 1599,
    RISCV_PseudoVFCVT_F_XU_V_M2_MASK	= 1600,
    RISCV_PseudoVFCVT_F_XU_V_M4	= 1601,
    RISCV_PseudoVFCVT_F_XU_V_M4_MASK	= 1602,
    RISCV_PseudoVFCVT_F_XU_V_M8	= 1603,
    RISCV_PseudoVFCVT_F_XU_V_M8_MASK	= 1604,
    RISCV_PseudoVFCVT_F_XU_V_MF2	= 1605,
    RISCV_PseudoVFCVT_F_XU_V_MF2_MASK	= 1606,
    RISCV_PseudoVFCVT_F_XU_V_MF4	= 1607,
    RISCV_PseudoVFCVT_F_XU_V_MF4_MASK	= 1608,
    RISCV_PseudoVFCVT_F_X_V_M1	= 1609,
    RISCV_PseudoVFCVT_F_X_V_M1_MASK	= 1610,
    RISCV_PseudoVFCVT_F_X_V_M2	= 1611,
    RISCV_PseudoVFCVT_F_X_V_M2_MASK	= 1612,
    RISCV_PseudoVFCVT_F_X_V_M4	= 1613,
    RISCV_PseudoVFCVT_F_X_V_M4_MASK	= 1614,
    RISCV_PseudoVFCVT_F_X_V_M8	= 1615,
    RISCV_PseudoVFCVT_F_X_V_M8_MASK	= 1616,
    RISCV_PseudoVFCVT_F_X_V_MF2	= 1617,
    RISCV_PseudoVFCVT_F_X_V_MF2_MASK	= 1618,
    RISCV_PseudoVFCVT_F_X_V_MF4	= 1619,
    RISCV_PseudoVFCVT_F_X_V_MF4_MASK	= 1620,
    RISCV_PseudoVFCVT_RM_F_XU_V_M1	= 1621,
    RISCV_PseudoVFCVT_RM_F_XU_V_M1_MASK	= 1622,
    RISCV_PseudoVFCVT_RM_F_XU_V_M2	= 1623,
    RISCV_PseudoVFCVT_RM_F_XU_V_M2_MASK	= 1624,
    RISCV_PseudoVFCVT_RM_F_XU_V_M4	= 1625,
    RISCV_PseudoVFCVT_RM_F_XU_V_M4_MASK	= 1626,
    RISCV_PseudoVFCVT_RM_F_XU_V_M8	= 1627,
    RISCV_PseudoVFCVT_RM_F_XU_V_M8_MASK	= 1628,
    RISCV_PseudoVFCVT_RM_F_XU_V_MF2	= 1629,
    RISCV_PseudoVFCVT_RM_F_XU_V_MF2_MASK	= 1630,
    RISCV_PseudoVFCVT_RM_F_XU_V_MF4	= 1631,
    RISCV_PseudoVFCVT_RM_F_XU_V_MF4_MASK	= 1632,
    RISCV_PseudoVFCVT_RM_F_X_V_M1	= 1633,
    RISCV_PseudoVFCVT_RM_F_X_V_M1_MASK	= 1634,
    RISCV_PseudoVFCVT_RM_F_X_V_M2	= 1635,
    RISCV_PseudoVFCVT_RM_F_X_V_M2_MASK	= 1636,
    RISCV_PseudoVFCVT_RM_F_X_V_M4	= 1637,
    RISCV_PseudoVFCVT_RM_F_X_V_M4_MASK	= 1638,
    RISCV_PseudoVFCVT_RM_F_X_V_M8	= 1639,
    RISCV_PseudoVFCVT_RM_F_X_V_M8_MASK	= 1640,
    RISCV_PseudoVFCVT_RM_F_X_V_MF2	= 1641,
    RISCV_PseudoVFCVT_RM_F_X_V_MF2_MASK	= 1642,
    RISCV_PseudoVFCVT_RM_F_X_V_MF4	= 1643,
    RISCV_PseudoVFCVT_RM_F_X_V_MF4_MASK	= 1644,
    RISCV_PseudoVFCVT_RM_XU_F_V_M1	= 1645,
    RISCV_PseudoVFCVT_RM_XU_F_V_M1_MASK	= 1646,
    RISCV_PseudoVFCVT_RM_XU_F_V_M2	= 1647,
    RISCV_PseudoVFCVT_RM_XU_F_V_M2_MASK	= 1648,
    RISCV_PseudoVFCVT_RM_XU_F_V_M4	= 1649,
    RISCV_PseudoVFCVT_RM_XU_F_V_M4_MASK	= 1650,
    RISCV_PseudoVFCVT_RM_XU_F_V_M8	= 1651,
    RISCV_PseudoVFCVT_RM_XU_F_V_M8_MASK	= 1652,
    RISCV_PseudoVFCVT_RM_XU_F_V_MF2	= 1653,
    RISCV_PseudoVFCVT_RM_XU_F_V_MF2_MASK	= 1654,
    RISCV_PseudoVFCVT_RM_XU_F_V_MF4	= 1655,
    RISCV_PseudoVFCVT_RM_XU_F_V_MF4_MASK	= 1656,
    RISCV_PseudoVFCVT_RM_X_F_V_M1	= 1657,
    RISCV_PseudoVFCVT_RM_X_F_V_M1_MASK	= 1658,
    RISCV_PseudoVFCVT_RM_X_F_V_M2	= 1659,
    RISCV_PseudoVFCVT_RM_X_F_V_M2_MASK	= 1660,
    RISCV_PseudoVFCVT_RM_X_F_V_M4	= 1661,
    RISCV_PseudoVFCVT_RM_X_F_V_M4_MASK	= 1662,
    RISCV_PseudoVFCVT_RM_X_F_V_M8	= 1663,
    RISCV_PseudoVFCVT_RM_X_F_V_M8_MASK	= 1664,
    RISCV_PseudoVFCVT_RM_X_F_V_MF2	= 1665,
    RISCV_PseudoVFCVT_RM_X_F_V_MF2_MASK	= 1666,
    RISCV_PseudoVFCVT_RM_X_F_V_MF4	= 1667,
    RISCV_PseudoVFCVT_RM_X_F_V_MF4_MASK	= 1668,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_M1	= 1669,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_M1_MASK	= 1670,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_M2	= 1671,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_M2_MASK	= 1672,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_M4	= 1673,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_M4_MASK	= 1674,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_M8	= 1675,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_M8_MASK	= 1676,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_MF2	= 1677,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_MF2_MASK	= 1678,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_MF4	= 1679,
    RISCV_PseudoVFCVT_RTZ_XU_F_V_MF4_MASK	= 1680,
    RISCV_PseudoVFCVT_RTZ_X_F_V_M1	= 1681,
    RISCV_PseudoVFCVT_RTZ_X_F_V_M1_MASK	= 1682,
    RISCV_PseudoVFCVT_RTZ_X_F_V_M2	= 1683,
    RISCV_PseudoVFCVT_RTZ_X_F_V_M2_MASK	= 1684,
    RISCV_PseudoVFCVT_RTZ_X_F_V_M4	= 1685,
    RISCV_PseudoVFCVT_RTZ_X_F_V_M4_MASK	= 1686,
    RISCV_PseudoVFCVT_RTZ_X_F_V_M8	= 1687,
    RISCV_PseudoVFCVT_RTZ_X_F_V_M8_MASK	= 1688,
    RISCV_PseudoVFCVT_RTZ_X_F_V_MF2	= 1689,
    RISCV_PseudoVFCVT_RTZ_X_F_V_MF2_MASK	= 1690,
    RISCV_PseudoVFCVT_RTZ_X_F_V_MF4	= 1691,
    RISCV_PseudoVFCVT_RTZ_X_F_V_MF4_MASK	= 1692,
    RISCV_PseudoVFCVT_XU_F_V_M1	= 1693,
    RISCV_PseudoVFCVT_XU_F_V_M1_MASK	= 1694,
    RISCV_PseudoVFCVT_XU_F_V_M2	= 1695,
    RISCV_PseudoVFCVT_XU_F_V_M2_MASK	= 1696,
    RISCV_PseudoVFCVT_XU_F_V_M4	= 1697,
    RISCV_PseudoVFCVT_XU_F_V_M4_MASK	= 1698,
    RISCV_PseudoVFCVT_XU_F_V_M8	= 1699,
    RISCV_PseudoVFCVT_XU_F_V_M8_MASK	= 1700,
    RISCV_PseudoVFCVT_XU_F_V_MF2	= 1701,
    RISCV_PseudoVFCVT_XU_F_V_MF2_MASK	= 1702,
    RISCV_PseudoVFCVT_XU_F_V_MF4	= 1703,
    RISCV_PseudoVFCVT_XU_F_V_MF4_MASK	= 1704,
    RISCV_PseudoVFCVT_X_F_V_M1	= 1705,
    RISCV_PseudoVFCVT_X_F_V_M1_MASK	= 1706,
    RISCV_PseudoVFCVT_X_F_V_M2	= 1707,
    RISCV_PseudoVFCVT_X_F_V_M2_MASK	= 1708,
    RISCV_PseudoVFCVT_X_F_V_M4	= 1709,
    RISCV_PseudoVFCVT_X_F_V_M4_MASK	= 1710,
    RISCV_PseudoVFCVT_X_F_V_M8	= 1711,
    RISCV_PseudoVFCVT_X_F_V_M8_MASK	= 1712,
    RISCV_PseudoVFCVT_X_F_V_MF2	= 1713,
    RISCV_PseudoVFCVT_X_F_V_MF2_MASK	= 1714,
    RISCV_PseudoVFCVT_X_F_V_MF4	= 1715,
    RISCV_PseudoVFCVT_X_F_V_MF4_MASK	= 1716,
    RISCV_PseudoVFDIV_VFPR16_M1_E16	= 1717,
    RISCV_PseudoVFDIV_VFPR16_M1_E16_MASK	= 1718,
    RISCV_PseudoVFDIV_VFPR16_M2_E16	= 1719,
    RISCV_PseudoVFDIV_VFPR16_M2_E16_MASK	= 1720,
    RISCV_PseudoVFDIV_VFPR16_M4_E16	= 1721,
    RISCV_PseudoVFDIV_VFPR16_M4_E16_MASK	= 1722,
    RISCV_PseudoVFDIV_VFPR16_M8_E16	= 1723,
    RISCV_PseudoVFDIV_VFPR16_M8_E16_MASK	= 1724,
    RISCV_PseudoVFDIV_VFPR16_MF2_E16	= 1725,
    RISCV_PseudoVFDIV_VFPR16_MF2_E16_MASK	= 1726,
    RISCV_PseudoVFDIV_VFPR16_MF4_E16	= 1727,
    RISCV_PseudoVFDIV_VFPR16_MF4_E16_MASK	= 1728,
    RISCV_PseudoVFDIV_VFPR32_M1_E32	= 1729,
    RISCV_PseudoVFDIV_VFPR32_M1_E32_MASK	= 1730,
    RISCV_PseudoVFDIV_VFPR32_M2_E32	= 1731,
    RISCV_PseudoVFDIV_VFPR32_M2_E32_MASK	= 1732,
    RISCV_PseudoVFDIV_VFPR32_M4_E32	= 1733,
    RISCV_PseudoVFDIV_VFPR32_M4_E32_MASK	= 1734,
    RISCV_PseudoVFDIV_VFPR32_M8_E32	= 1735,
    RISCV_PseudoVFDIV_VFPR32_M8_E32_MASK	= 1736,
    RISCV_PseudoVFDIV_VFPR32_MF2_E32	= 1737,
    RISCV_PseudoVFDIV_VFPR32_MF2_E32_MASK	= 1738,
    RISCV_PseudoVFDIV_VFPR64_M1_E64	= 1739,
    RISCV_PseudoVFDIV_VFPR64_M1_E64_MASK	= 1740,
    RISCV_PseudoVFDIV_VFPR64_M2_E64	= 1741,
    RISCV_PseudoVFDIV_VFPR64_M2_E64_MASK	= 1742,
    RISCV_PseudoVFDIV_VFPR64_M4_E64	= 1743,
    RISCV_PseudoVFDIV_VFPR64_M4_E64_MASK	= 1744,
    RISCV_PseudoVFDIV_VFPR64_M8_E64	= 1745,
    RISCV_PseudoVFDIV_VFPR64_M8_E64_MASK	= 1746,
    RISCV_PseudoVFDIV_VV_M1_E16	= 1747,
    RISCV_PseudoVFDIV_VV_M1_E16_MASK	= 1748,
    RISCV_PseudoVFDIV_VV_M1_E32	= 1749,
    RISCV_PseudoVFDIV_VV_M1_E32_MASK	= 1750,
    RISCV_PseudoVFDIV_VV_M1_E64	= 1751,
    RISCV_PseudoVFDIV_VV_M1_E64_MASK	= 1752,
    RISCV_PseudoVFDIV_VV_M2_E16	= 1753,
    RISCV_PseudoVFDIV_VV_M2_E16_MASK	= 1754,
    RISCV_PseudoVFDIV_VV_M2_E32	= 1755,
    RISCV_PseudoVFDIV_VV_M2_E32_MASK	= 1756,
    RISCV_PseudoVFDIV_VV_M2_E64	= 1757,
    RISCV_PseudoVFDIV_VV_M2_E64_MASK	= 1758,
    RISCV_PseudoVFDIV_VV_M4_E16	= 1759,
    RISCV_PseudoVFDIV_VV_M4_E16_MASK	= 1760,
    RISCV_PseudoVFDIV_VV_M4_E32	= 1761,
    RISCV_PseudoVFDIV_VV_M4_E32_MASK	= 1762,
    RISCV_PseudoVFDIV_VV_M4_E64	= 1763,
    RISCV_PseudoVFDIV_VV_M4_E64_MASK	= 1764,
    RISCV_PseudoVFDIV_VV_M8_E16	= 1765,
    RISCV_PseudoVFDIV_VV_M8_E16_MASK	= 1766,
    RISCV_PseudoVFDIV_VV_M8_E32	= 1767,
    RISCV_PseudoVFDIV_VV_M8_E32_MASK	= 1768,
    RISCV_PseudoVFDIV_VV_M8_E64	= 1769,
    RISCV_PseudoVFDIV_VV_M8_E64_MASK	= 1770,
    RISCV_PseudoVFDIV_VV_MF2_E16	= 1771,
    RISCV_PseudoVFDIV_VV_MF2_E16_MASK	= 1772,
    RISCV_PseudoVFDIV_VV_MF2_E32	= 1773,
    RISCV_PseudoVFDIV_VV_MF2_E32_MASK	= 1774,
    RISCV_PseudoVFDIV_VV_MF4_E16	= 1775,
    RISCV_PseudoVFDIV_VV_MF4_E16_MASK	= 1776,
    RISCV_PseudoVFIRST_M_B1	= 1777,
    RISCV_PseudoVFIRST_M_B16	= 1778,
    RISCV_PseudoVFIRST_M_B16_MASK	= 1779,
    RISCV_PseudoVFIRST_M_B1_MASK	= 1780,
    RISCV_PseudoVFIRST_M_B2	= 1781,
    RISCV_PseudoVFIRST_M_B2_MASK	= 1782,
    RISCV_PseudoVFIRST_M_B32	= 1783,
    RISCV_PseudoVFIRST_M_B32_MASK	= 1784,
    RISCV_PseudoVFIRST_M_B4	= 1785,
    RISCV_PseudoVFIRST_M_B4_MASK	= 1786,
    RISCV_PseudoVFIRST_M_B64	= 1787,
    RISCV_PseudoVFIRST_M_B64_MASK	= 1788,
    RISCV_PseudoVFIRST_M_B8	= 1789,
    RISCV_PseudoVFIRST_M_B8_MASK	= 1790,
    RISCV_PseudoVFMACC_VFPR16_M1	= 1791,
    RISCV_PseudoVFMACC_VFPR16_M1_MASK	= 1792,
    RISCV_PseudoVFMACC_VFPR16_M2	= 1793,
    RISCV_PseudoVFMACC_VFPR16_M2_MASK	= 1794,
    RISCV_PseudoVFMACC_VFPR16_M4	= 1795,
    RISCV_PseudoVFMACC_VFPR16_M4_MASK	= 1796,
    RISCV_PseudoVFMACC_VFPR16_M8	= 1797,
    RISCV_PseudoVFMACC_VFPR16_M8_MASK	= 1798,
    RISCV_PseudoVFMACC_VFPR16_MF2	= 1799,
    RISCV_PseudoVFMACC_VFPR16_MF2_MASK	= 1800,
    RISCV_PseudoVFMACC_VFPR16_MF4	= 1801,
    RISCV_PseudoVFMACC_VFPR16_MF4_MASK	= 1802,
    RISCV_PseudoVFMACC_VFPR32_M1	= 1803,
    RISCV_PseudoVFMACC_VFPR32_M1_MASK	= 1804,
    RISCV_PseudoVFMACC_VFPR32_M2	= 1805,
    RISCV_PseudoVFMACC_VFPR32_M2_MASK	= 1806,
    RISCV_PseudoVFMACC_VFPR32_M4	= 1807,
    RISCV_PseudoVFMACC_VFPR32_M4_MASK	= 1808,
    RISCV_PseudoVFMACC_VFPR32_M8	= 1809,
    RISCV_PseudoVFMACC_VFPR32_M8_MASK	= 1810,
    RISCV_PseudoVFMACC_VFPR32_MF2	= 1811,
    RISCV_PseudoVFMACC_VFPR32_MF2_MASK	= 1812,
    RISCV_PseudoVFMACC_VFPR64_M1	= 1813,
    RISCV_PseudoVFMACC_VFPR64_M1_MASK	= 1814,
    RISCV_PseudoVFMACC_VFPR64_M2	= 1815,
    RISCV_PseudoVFMACC_VFPR64_M2_MASK	= 1816,
    RISCV_PseudoVFMACC_VFPR64_M4	= 1817,
    RISCV_PseudoVFMACC_VFPR64_M4_MASK	= 1818,
    RISCV_PseudoVFMACC_VFPR64_M8	= 1819,
    RISCV_PseudoVFMACC_VFPR64_M8_MASK	= 1820,
    RISCV_PseudoVFMACC_VV_M1	= 1821,
    RISCV_PseudoVFMACC_VV_M1_MASK	= 1822,
    RISCV_PseudoVFMACC_VV_M2	= 1823,
    RISCV_PseudoVFMACC_VV_M2_MASK	= 1824,
    RISCV_PseudoVFMACC_VV_M4	= 1825,
    RISCV_PseudoVFMACC_VV_M4_MASK	= 1826,
    RISCV_PseudoVFMACC_VV_M8	= 1827,
    RISCV_PseudoVFMACC_VV_M8_MASK	= 1828,
    RISCV_PseudoVFMACC_VV_MF2	= 1829,
    RISCV_PseudoVFMACC_VV_MF2_MASK	= 1830,
    RISCV_PseudoVFMACC_VV_MF4	= 1831,
    RISCV_PseudoVFMACC_VV_MF4_MASK	= 1832,
    RISCV_PseudoVFMADD_VFPR16_M1	= 1833,
    RISCV_PseudoVFMADD_VFPR16_M1_MASK	= 1834,
    RISCV_PseudoVFMADD_VFPR16_M2	= 1835,
    RISCV_PseudoVFMADD_VFPR16_M2_MASK	= 1836,
    RISCV_PseudoVFMADD_VFPR16_M4	= 1837,
    RISCV_PseudoVFMADD_VFPR16_M4_MASK	= 1838,
    RISCV_PseudoVFMADD_VFPR16_M8	= 1839,
    RISCV_PseudoVFMADD_VFPR16_M8_MASK	= 1840,
    RISCV_PseudoVFMADD_VFPR16_MF2	= 1841,
    RISCV_PseudoVFMADD_VFPR16_MF2_MASK	= 1842,
    RISCV_PseudoVFMADD_VFPR16_MF4	= 1843,
    RISCV_PseudoVFMADD_VFPR16_MF4_MASK	= 1844,
    RISCV_PseudoVFMADD_VFPR32_M1	= 1845,
    RISCV_PseudoVFMADD_VFPR32_M1_MASK	= 1846,
    RISCV_PseudoVFMADD_VFPR32_M2	= 1847,
    RISCV_PseudoVFMADD_VFPR32_M2_MASK	= 1848,
    RISCV_PseudoVFMADD_VFPR32_M4	= 1849,
    RISCV_PseudoVFMADD_VFPR32_M4_MASK	= 1850,
    RISCV_PseudoVFMADD_VFPR32_M8	= 1851,
    RISCV_PseudoVFMADD_VFPR32_M8_MASK	= 1852,
    RISCV_PseudoVFMADD_VFPR32_MF2	= 1853,
    RISCV_PseudoVFMADD_VFPR32_MF2_MASK	= 1854,
    RISCV_PseudoVFMADD_VFPR64_M1	= 1855,
    RISCV_PseudoVFMADD_VFPR64_M1_MASK	= 1856,
    RISCV_PseudoVFMADD_VFPR64_M2	= 1857,
    RISCV_PseudoVFMADD_VFPR64_M2_MASK	= 1858,
    RISCV_PseudoVFMADD_VFPR64_M4	= 1859,
    RISCV_PseudoVFMADD_VFPR64_M4_MASK	= 1860,
    RISCV_PseudoVFMADD_VFPR64_M8	= 1861,
    RISCV_PseudoVFMADD_VFPR64_M8_MASK	= 1862,
    RISCV_PseudoVFMADD_VV_M1	= 1863,
    RISCV_PseudoVFMADD_VV_M1_MASK	= 1864,
    RISCV_PseudoVFMADD_VV_M2	= 1865,
    RISCV_PseudoVFMADD_VV_M2_MASK	= 1866,
    RISCV_PseudoVFMADD_VV_M4	= 1867,
    RISCV_PseudoVFMADD_VV_M4_MASK	= 1868,
    RISCV_PseudoVFMADD_VV_M8	= 1869,
    RISCV_PseudoVFMADD_VV_M8_MASK	= 1870,
    RISCV_PseudoVFMADD_VV_MF2	= 1871,
    RISCV_PseudoVFMADD_VV_MF2_MASK	= 1872,
    RISCV_PseudoVFMADD_VV_MF4	= 1873,
    RISCV_PseudoVFMADD_VV_MF4_MASK	= 1874,
    RISCV_PseudoVFMAX_VFPR16_M1	= 1875,
    RISCV_PseudoVFMAX_VFPR16_M1_MASK	= 1876,
    RISCV_PseudoVFMAX_VFPR16_M2	= 1877,
    RISCV_PseudoVFMAX_VFPR16_M2_MASK	= 1878,
    RISCV_PseudoVFMAX_VFPR16_M4	= 1879,
    RISCV_PseudoVFMAX_VFPR16_M4_MASK	= 1880,
    RISCV_PseudoVFMAX_VFPR16_M8	= 1881,
    RISCV_PseudoVFMAX_VFPR16_M8_MASK	= 1882,
    RISCV_PseudoVFMAX_VFPR16_MF2	= 1883,
    RISCV_PseudoVFMAX_VFPR16_MF2_MASK	= 1884,
    RISCV_PseudoVFMAX_VFPR16_MF4	= 1885,
    RISCV_PseudoVFMAX_VFPR16_MF4_MASK	= 1886,
    RISCV_PseudoVFMAX_VFPR32_M1	= 1887,
    RISCV_PseudoVFMAX_VFPR32_M1_MASK	= 1888,
    RISCV_PseudoVFMAX_VFPR32_M2	= 1889,
    RISCV_PseudoVFMAX_VFPR32_M2_MASK	= 1890,
    RISCV_PseudoVFMAX_VFPR32_M4	= 1891,
    RISCV_PseudoVFMAX_VFPR32_M4_MASK	= 1892,
    RISCV_PseudoVFMAX_VFPR32_M8	= 1893,
    RISCV_PseudoVFMAX_VFPR32_M8_MASK	= 1894,
    RISCV_PseudoVFMAX_VFPR32_MF2	= 1895,
    RISCV_PseudoVFMAX_VFPR32_MF2_MASK	= 1896,
    RISCV_PseudoVFMAX_VFPR64_M1	= 1897,
    RISCV_PseudoVFMAX_VFPR64_M1_MASK	= 1898,
    RISCV_PseudoVFMAX_VFPR64_M2	= 1899,
    RISCV_PseudoVFMAX_VFPR64_M2_MASK	= 1900,
    RISCV_PseudoVFMAX_VFPR64_M4	= 1901,
    RISCV_PseudoVFMAX_VFPR64_M4_MASK	= 1902,
    RISCV_PseudoVFMAX_VFPR64_M8	= 1903,
    RISCV_PseudoVFMAX_VFPR64_M8_MASK	= 1904,
    RISCV_PseudoVFMAX_VV_M1	= 1905,
    RISCV_PseudoVFMAX_VV_M1_MASK	= 1906,
    RISCV_PseudoVFMAX_VV_M2	= 1907,
    RISCV_PseudoVFMAX_VV_M2_MASK	= 1908,
    RISCV_PseudoVFMAX_VV_M4	= 1909,
    RISCV_PseudoVFMAX_VV_M4_MASK	= 1910,
    RISCV_PseudoVFMAX_VV_M8	= 1911,
    RISCV_PseudoVFMAX_VV_M8_MASK	= 1912,
    RISCV_PseudoVFMAX_VV_MF2	= 1913,
    RISCV_PseudoVFMAX_VV_MF2_MASK	= 1914,
    RISCV_PseudoVFMAX_VV_MF4	= 1915,
    RISCV_PseudoVFMAX_VV_MF4_MASK	= 1916,
    RISCV_PseudoVFMERGE_VFPR16M_M1	= 1917,
    RISCV_PseudoVFMERGE_VFPR16M_M2	= 1918,
    RISCV_PseudoVFMERGE_VFPR16M_M4	= 1919,
    RISCV_PseudoVFMERGE_VFPR16M_M8	= 1920,
    RISCV_PseudoVFMERGE_VFPR16M_MF2	= 1921,
    RISCV_PseudoVFMERGE_VFPR16M_MF4	= 1922,
    RISCV_PseudoVFMERGE_VFPR32M_M1	= 1923,
    RISCV_PseudoVFMERGE_VFPR32M_M2	= 1924,
    RISCV_PseudoVFMERGE_VFPR32M_M4	= 1925,
    RISCV_PseudoVFMERGE_VFPR32M_M8	= 1926,
    RISCV_PseudoVFMERGE_VFPR32M_MF2	= 1927,
    RISCV_PseudoVFMERGE_VFPR64M_M1	= 1928,
    RISCV_PseudoVFMERGE_VFPR64M_M2	= 1929,
    RISCV_PseudoVFMERGE_VFPR64M_M4	= 1930,
    RISCV_PseudoVFMERGE_VFPR64M_M8	= 1931,
    RISCV_PseudoVFMIN_VFPR16_M1	= 1932,
    RISCV_PseudoVFMIN_VFPR16_M1_MASK	= 1933,
    RISCV_PseudoVFMIN_VFPR16_M2	= 1934,
    RISCV_PseudoVFMIN_VFPR16_M2_MASK	= 1935,
    RISCV_PseudoVFMIN_VFPR16_M4	= 1936,
    RISCV_PseudoVFMIN_VFPR16_M4_MASK	= 1937,
    RISCV_PseudoVFMIN_VFPR16_M8	= 1938,
    RISCV_PseudoVFMIN_VFPR16_M8_MASK	= 1939,
    RISCV_PseudoVFMIN_VFPR16_MF2	= 1940,
    RISCV_PseudoVFMIN_VFPR16_MF2_MASK	= 1941,
    RISCV_PseudoVFMIN_VFPR16_MF4	= 1942,
    RISCV_PseudoVFMIN_VFPR16_MF4_MASK	= 1943,
    RISCV_PseudoVFMIN_VFPR32_M1	= 1944,
    RISCV_PseudoVFMIN_VFPR32_M1_MASK	= 1945,
    RISCV_PseudoVFMIN_VFPR32_M2	= 1946,
    RISCV_PseudoVFMIN_VFPR32_M2_MASK	= 1947,
    RISCV_PseudoVFMIN_VFPR32_M4	= 1948,
    RISCV_PseudoVFMIN_VFPR32_M4_MASK	= 1949,
    RISCV_PseudoVFMIN_VFPR32_M8	= 1950,
    RISCV_PseudoVFMIN_VFPR32_M8_MASK	= 1951,
    RISCV_PseudoVFMIN_VFPR32_MF2	= 1952,
    RISCV_PseudoVFMIN_VFPR32_MF2_MASK	= 1953,
    RISCV_PseudoVFMIN_VFPR64_M1	= 1954,
    RISCV_PseudoVFMIN_VFPR64_M1_MASK	= 1955,
    RISCV_PseudoVFMIN_VFPR64_M2	= 1956,
    RISCV_PseudoVFMIN_VFPR64_M2_MASK	= 1957,
    RISCV_PseudoVFMIN_VFPR64_M4	= 1958,
    RISCV_PseudoVFMIN_VFPR64_M4_MASK	= 1959,
    RISCV_PseudoVFMIN_VFPR64_M8	= 1960,
    RISCV_PseudoVFMIN_VFPR64_M8_MASK	= 1961,
    RISCV_PseudoVFMIN_VV_M1	= 1962,
    RISCV_PseudoVFMIN_VV_M1_MASK	= 1963,
    RISCV_PseudoVFMIN_VV_M2	= 1964,
    RISCV_PseudoVFMIN_VV_M2_MASK	= 1965,
    RISCV_PseudoVFMIN_VV_M4	= 1966,
    RISCV_PseudoVFMIN_VV_M4_MASK	= 1967,
    RISCV_PseudoVFMIN_VV_M8	= 1968,
    RISCV_PseudoVFMIN_VV_M8_MASK	= 1969,
    RISCV_PseudoVFMIN_VV_MF2	= 1970,
    RISCV_PseudoVFMIN_VV_MF2_MASK	= 1971,
    RISCV_PseudoVFMIN_VV_MF4	= 1972,
    RISCV_PseudoVFMIN_VV_MF4_MASK	= 1973,
    RISCV_PseudoVFMSAC_VFPR16_M1	= 1974,
    RISCV_PseudoVFMSAC_VFPR16_M1_MASK	= 1975,
    RISCV_PseudoVFMSAC_VFPR16_M2	= 1976,
    RISCV_PseudoVFMSAC_VFPR16_M2_MASK	= 1977,
    RISCV_PseudoVFMSAC_VFPR16_M4	= 1978,
    RISCV_PseudoVFMSAC_VFPR16_M4_MASK	= 1979,
    RISCV_PseudoVFMSAC_VFPR16_M8	= 1980,
    RISCV_PseudoVFMSAC_VFPR16_M8_MASK	= 1981,
    RISCV_PseudoVFMSAC_VFPR16_MF2	= 1982,
    RISCV_PseudoVFMSAC_VFPR16_MF2_MASK	= 1983,
    RISCV_PseudoVFMSAC_VFPR16_MF4	= 1984,
    RISCV_PseudoVFMSAC_VFPR16_MF4_MASK	= 1985,
    RISCV_PseudoVFMSAC_VFPR32_M1	= 1986,
    RISCV_PseudoVFMSAC_VFPR32_M1_MASK	= 1987,
    RISCV_PseudoVFMSAC_VFPR32_M2	= 1988,
    RISCV_PseudoVFMSAC_VFPR32_M2_MASK	= 1989,
    RISCV_PseudoVFMSAC_VFPR32_M4	= 1990,
    RISCV_PseudoVFMSAC_VFPR32_M4_MASK	= 1991,
    RISCV_PseudoVFMSAC_VFPR32_M8	= 1992,
    RISCV_PseudoVFMSAC_VFPR32_M8_MASK	= 1993,
    RISCV_PseudoVFMSAC_VFPR32_MF2	= 1994,
    RISCV_PseudoVFMSAC_VFPR32_MF2_MASK	= 1995,
    RISCV_PseudoVFMSAC_VFPR64_M1	= 1996,
    RISCV_PseudoVFMSAC_VFPR64_M1_MASK	= 1997,
    RISCV_PseudoVFMSAC_VFPR64_M2	= 1998,
    RISCV_PseudoVFMSAC_VFPR64_M2_MASK	= 1999,
    RISCV_PseudoVFMSAC_VFPR64_M4	= 2000,
    RISCV_PseudoVFMSAC_VFPR64_M4_MASK	= 2001,
    RISCV_PseudoVFMSAC_VFPR64_M8	= 2002,
    RISCV_PseudoVFMSAC_VFPR64_M8_MASK	= 2003,
    RISCV_PseudoVFMSAC_VV_M1	= 2004,
    RISCV_PseudoVFMSAC_VV_M1_MASK	= 2005,
    RISCV_PseudoVFMSAC_VV_M2	= 2006,
    RISCV_PseudoVFMSAC_VV_M2_MASK	= 2007,
    RISCV_PseudoVFMSAC_VV_M4	= 2008,
    RISCV_PseudoVFMSAC_VV_M4_MASK	= 2009,
    RISCV_PseudoVFMSAC_VV_M8	= 2010,
    RISCV_PseudoVFMSAC_VV_M8_MASK	= 2011,
    RISCV_PseudoVFMSAC_VV_MF2	= 2012,
    RISCV_PseudoVFMSAC_VV_MF2_MASK	= 2013,
    RISCV_PseudoVFMSAC_VV_MF4	= 2014,
    RISCV_PseudoVFMSAC_VV_MF4_MASK	= 2015,
    RISCV_PseudoVFMSUB_VFPR16_M1	= 2016,
    RISCV_PseudoVFMSUB_VFPR16_M1_MASK	= 2017,
    RISCV_PseudoVFMSUB_VFPR16_M2	= 2018,
    RISCV_PseudoVFMSUB_VFPR16_M2_MASK	= 2019,
    RISCV_PseudoVFMSUB_VFPR16_M4	= 2020,
    RISCV_PseudoVFMSUB_VFPR16_M4_MASK	= 2021,
    RISCV_PseudoVFMSUB_VFPR16_M8	= 2022,
    RISCV_PseudoVFMSUB_VFPR16_M8_MASK	= 2023,
    RISCV_PseudoVFMSUB_VFPR16_MF2	= 2024,
    RISCV_PseudoVFMSUB_VFPR16_MF2_MASK	= 2025,
    RISCV_PseudoVFMSUB_VFPR16_MF4	= 2026,
    RISCV_PseudoVFMSUB_VFPR16_MF4_MASK	= 2027,
    RISCV_PseudoVFMSUB_VFPR32_M1	= 2028,
    RISCV_PseudoVFMSUB_VFPR32_M1_MASK	= 2029,
    RISCV_PseudoVFMSUB_VFPR32_M2	= 2030,
    RISCV_PseudoVFMSUB_VFPR32_M2_MASK	= 2031,
    RISCV_PseudoVFMSUB_VFPR32_M4	= 2032,
    RISCV_PseudoVFMSUB_VFPR32_M4_MASK	= 2033,
    RISCV_PseudoVFMSUB_VFPR32_M8	= 2034,
    RISCV_PseudoVFMSUB_VFPR32_M8_MASK	= 2035,
    RISCV_PseudoVFMSUB_VFPR32_MF2	= 2036,
    RISCV_PseudoVFMSUB_VFPR32_MF2_MASK	= 2037,
    RISCV_PseudoVFMSUB_VFPR64_M1	= 2038,
    RISCV_PseudoVFMSUB_VFPR64_M1_MASK	= 2039,
    RISCV_PseudoVFMSUB_VFPR64_M2	= 2040,
    RISCV_PseudoVFMSUB_VFPR64_M2_MASK	= 2041,
    RISCV_PseudoVFMSUB_VFPR64_M4	= 2042,
    RISCV_PseudoVFMSUB_VFPR64_M4_MASK	= 2043,
    RISCV_PseudoVFMSUB_VFPR64_M8	= 2044,
    RISCV_PseudoVFMSUB_VFPR64_M8_MASK	= 2045,
    RISCV_PseudoVFMSUB_VV_M1	= 2046,
    RISCV_PseudoVFMSUB_VV_M1_MASK	= 2047,
    RISCV_PseudoVFMSUB_VV_M2	= 2048,
    RISCV_PseudoVFMSUB_VV_M2_MASK	= 2049,
    RISCV_PseudoVFMSUB_VV_M4	= 2050,
    RISCV_PseudoVFMSUB_VV_M4_MASK	= 2051,
    RISCV_PseudoVFMSUB_VV_M8	= 2052,
    RISCV_PseudoVFMSUB_VV_M8_MASK	= 2053,
    RISCV_PseudoVFMSUB_VV_MF2	= 2054,
    RISCV_PseudoVFMSUB_VV_MF2_MASK	= 2055,
    RISCV_PseudoVFMSUB_VV_MF4	= 2056,
    RISCV_PseudoVFMSUB_VV_MF4_MASK	= 2057,
    RISCV_PseudoVFMUL_VFPR16_M1	= 2058,
    RISCV_PseudoVFMUL_VFPR16_M1_MASK	= 2059,
    RISCV_PseudoVFMUL_VFPR16_M2	= 2060,
    RISCV_PseudoVFMUL_VFPR16_M2_MASK	= 2061,
    RISCV_PseudoVFMUL_VFPR16_M4	= 2062,
    RISCV_PseudoVFMUL_VFPR16_M4_MASK	= 2063,
    RISCV_PseudoVFMUL_VFPR16_M8	= 2064,
    RISCV_PseudoVFMUL_VFPR16_M8_MASK	= 2065,
    RISCV_PseudoVFMUL_VFPR16_MF2	= 2066,
    RISCV_PseudoVFMUL_VFPR16_MF2_MASK	= 2067,
    RISCV_PseudoVFMUL_VFPR16_MF4	= 2068,
    RISCV_PseudoVFMUL_VFPR16_MF4_MASK	= 2069,
    RISCV_PseudoVFMUL_VFPR32_M1	= 2070,
    RISCV_PseudoVFMUL_VFPR32_M1_MASK	= 2071,
    RISCV_PseudoVFMUL_VFPR32_M2	= 2072,
    RISCV_PseudoVFMUL_VFPR32_M2_MASK	= 2073,
    RISCV_PseudoVFMUL_VFPR32_M4	= 2074,
    RISCV_PseudoVFMUL_VFPR32_M4_MASK	= 2075,
    RISCV_PseudoVFMUL_VFPR32_M8	= 2076,
    RISCV_PseudoVFMUL_VFPR32_M8_MASK	= 2077,
    RISCV_PseudoVFMUL_VFPR32_MF2	= 2078,
    RISCV_PseudoVFMUL_VFPR32_MF2_MASK	= 2079,
    RISCV_PseudoVFMUL_VFPR64_M1	= 2080,
    RISCV_PseudoVFMUL_VFPR64_M1_MASK	= 2081,
    RISCV_PseudoVFMUL_VFPR64_M2	= 2082,
    RISCV_PseudoVFMUL_VFPR64_M2_MASK	= 2083,
    RISCV_PseudoVFMUL_VFPR64_M4	= 2084,
    RISCV_PseudoVFMUL_VFPR64_M4_MASK	= 2085,
    RISCV_PseudoVFMUL_VFPR64_M8	= 2086,
    RISCV_PseudoVFMUL_VFPR64_M8_MASK	= 2087,
    RISCV_PseudoVFMUL_VV_M1	= 2088,
    RISCV_PseudoVFMUL_VV_M1_MASK	= 2089,
    RISCV_PseudoVFMUL_VV_M2	= 2090,
    RISCV_PseudoVFMUL_VV_M2_MASK	= 2091,
    RISCV_PseudoVFMUL_VV_M4	= 2092,
    RISCV_PseudoVFMUL_VV_M4_MASK	= 2093,
    RISCV_PseudoVFMUL_VV_M8	= 2094,
    RISCV_PseudoVFMUL_VV_M8_MASK	= 2095,
    RISCV_PseudoVFMUL_VV_MF2	= 2096,
    RISCV_PseudoVFMUL_VV_MF2_MASK	= 2097,
    RISCV_PseudoVFMUL_VV_MF4	= 2098,
    RISCV_PseudoVFMUL_VV_MF4_MASK	= 2099,
    RISCV_PseudoVFMV_FPR16_S_M1	= 2100,
    RISCV_PseudoVFMV_FPR16_S_M2	= 2101,
    RISCV_PseudoVFMV_FPR16_S_M4	= 2102,
    RISCV_PseudoVFMV_FPR16_S_M8	= 2103,
    RISCV_PseudoVFMV_FPR16_S_MF2	= 2104,
    RISCV_PseudoVFMV_FPR16_S_MF4	= 2105,
    RISCV_PseudoVFMV_FPR32_S_M1	= 2106,
    RISCV_PseudoVFMV_FPR32_S_M2	= 2107,
    RISCV_PseudoVFMV_FPR32_S_M4	= 2108,
    RISCV_PseudoVFMV_FPR32_S_M8	= 2109,
    RISCV_PseudoVFMV_FPR32_S_MF2	= 2110,
    RISCV_PseudoVFMV_FPR64_S_M1	= 2111,
    RISCV_PseudoVFMV_FPR64_S_M2	= 2112,
    RISCV_PseudoVFMV_FPR64_S_M4	= 2113,
    RISCV_PseudoVFMV_FPR64_S_M8	= 2114,
    RISCV_PseudoVFMV_S_FPR16_M1	= 2115,
    RISCV_PseudoVFMV_S_FPR16_M2	= 2116,
    RISCV_PseudoVFMV_S_FPR16_M4	= 2117,
    RISCV_PseudoVFMV_S_FPR16_M8	= 2118,
    RISCV_PseudoVFMV_S_FPR16_MF2	= 2119,
    RISCV_PseudoVFMV_S_FPR16_MF4	= 2120,
    RISCV_PseudoVFMV_S_FPR32_M1	= 2121,
    RISCV_PseudoVFMV_S_FPR32_M2	= 2122,
    RISCV_PseudoVFMV_S_FPR32_M4	= 2123,
    RISCV_PseudoVFMV_S_FPR32_M8	= 2124,
    RISCV_PseudoVFMV_S_FPR32_MF2	= 2125,
    RISCV_PseudoVFMV_S_FPR64_M1	= 2126,
    RISCV_PseudoVFMV_S_FPR64_M2	= 2127,
    RISCV_PseudoVFMV_S_FPR64_M4	= 2128,
    RISCV_PseudoVFMV_S_FPR64_M8	= 2129,
    RISCV_PseudoVFMV_V_FPR16_M1	= 2130,
    RISCV_PseudoVFMV_V_FPR16_M2	= 2131,
    RISCV_PseudoVFMV_V_FPR16_M4	= 2132,
    RISCV_PseudoVFMV_V_FPR16_M8	= 2133,
    RISCV_PseudoVFMV_V_FPR16_MF2	= 2134,
    RISCV_PseudoVFMV_V_FPR16_MF4	= 2135,
    RISCV_PseudoVFMV_V_FPR32_M1	= 2136,
    RISCV_PseudoVFMV_V_FPR32_M2	= 2137,
    RISCV_PseudoVFMV_V_FPR32_M4	= 2138,
    RISCV_PseudoVFMV_V_FPR32_M8	= 2139,
    RISCV_PseudoVFMV_V_FPR32_MF2	= 2140,
    RISCV_PseudoVFMV_V_FPR64_M1	= 2141,
    RISCV_PseudoVFMV_V_FPR64_M2	= 2142,
    RISCV_PseudoVFMV_V_FPR64_M4	= 2143,
    RISCV_PseudoVFMV_V_FPR64_M8	= 2144,
    RISCV_PseudoVFNCVTBF16_F_F_W_M1	= 2145,
    RISCV_PseudoVFNCVTBF16_F_F_W_M1_MASK	= 2146,
    RISCV_PseudoVFNCVTBF16_F_F_W_M2	= 2147,
    RISCV_PseudoVFNCVTBF16_F_F_W_M2_MASK	= 2148,
    RISCV_PseudoVFNCVTBF16_F_F_W_M4	= 2149,
    RISCV_PseudoVFNCVTBF16_F_F_W_M4_MASK	= 2150,
    RISCV_PseudoVFNCVTBF16_F_F_W_MF2	= 2151,
    RISCV_PseudoVFNCVTBF16_F_F_W_MF2_MASK	= 2152,
    RISCV_PseudoVFNCVTBF16_F_F_W_MF4	= 2153,
    RISCV_PseudoVFNCVTBF16_F_F_W_MF4_MASK	= 2154,
    RISCV_PseudoVFNCVT_F_F_W_M1	= 2155,
    RISCV_PseudoVFNCVT_F_F_W_M1_MASK	= 2156,
    RISCV_PseudoVFNCVT_F_F_W_M2	= 2157,
    RISCV_PseudoVFNCVT_F_F_W_M2_MASK	= 2158,
    RISCV_PseudoVFNCVT_F_F_W_M4	= 2159,
    RISCV_PseudoVFNCVT_F_F_W_M4_MASK	= 2160,
    RISCV_PseudoVFNCVT_F_F_W_MF2	= 2161,
    RISCV_PseudoVFNCVT_F_F_W_MF2_MASK	= 2162,
    RISCV_PseudoVFNCVT_F_F_W_MF4	= 2163,
    RISCV_PseudoVFNCVT_F_F_W_MF4_MASK	= 2164,
    RISCV_PseudoVFNCVT_F_XU_W_M1	= 2165,
    RISCV_PseudoVFNCVT_F_XU_W_M1_MASK	= 2166,
    RISCV_PseudoVFNCVT_F_XU_W_M2	= 2167,
    RISCV_PseudoVFNCVT_F_XU_W_M2_MASK	= 2168,
    RISCV_PseudoVFNCVT_F_XU_W_M4	= 2169,
    RISCV_PseudoVFNCVT_F_XU_W_M4_MASK	= 2170,
    RISCV_PseudoVFNCVT_F_XU_W_MF2	= 2171,
    RISCV_PseudoVFNCVT_F_XU_W_MF2_MASK	= 2172,
    RISCV_PseudoVFNCVT_F_XU_W_MF4	= 2173,
    RISCV_PseudoVFNCVT_F_XU_W_MF4_MASK	= 2174,
    RISCV_PseudoVFNCVT_F_X_W_M1	= 2175,
    RISCV_PseudoVFNCVT_F_X_W_M1_MASK	= 2176,
    RISCV_PseudoVFNCVT_F_X_W_M2	= 2177,
    RISCV_PseudoVFNCVT_F_X_W_M2_MASK	= 2178,
    RISCV_PseudoVFNCVT_F_X_W_M4	= 2179,
    RISCV_PseudoVFNCVT_F_X_W_M4_MASK	= 2180,
    RISCV_PseudoVFNCVT_F_X_W_MF2	= 2181,
    RISCV_PseudoVFNCVT_F_X_W_MF2_MASK	= 2182,
    RISCV_PseudoVFNCVT_F_X_W_MF4	= 2183,
    RISCV_PseudoVFNCVT_F_X_W_MF4_MASK	= 2184,
    RISCV_PseudoVFNCVT_RM_F_XU_W_M1	= 2185,
    RISCV_PseudoVFNCVT_RM_F_XU_W_M1_MASK	= 2186,
    RISCV_PseudoVFNCVT_RM_F_XU_W_M2	= 2187,
    RISCV_PseudoVFNCVT_RM_F_XU_W_M2_MASK	= 2188,
    RISCV_PseudoVFNCVT_RM_F_XU_W_M4	= 2189,
    RISCV_PseudoVFNCVT_RM_F_XU_W_M4_MASK	= 2190,
    RISCV_PseudoVFNCVT_RM_F_XU_W_MF2	= 2191,
    RISCV_PseudoVFNCVT_RM_F_XU_W_MF2_MASK	= 2192,
    RISCV_PseudoVFNCVT_RM_F_XU_W_MF4	= 2193,
    RISCV_PseudoVFNCVT_RM_F_XU_W_MF4_MASK	= 2194,
    RISCV_PseudoVFNCVT_RM_F_X_W_M1	= 2195,
    RISCV_PseudoVFNCVT_RM_F_X_W_M1_MASK	= 2196,
    RISCV_PseudoVFNCVT_RM_F_X_W_M2	= 2197,
    RISCV_PseudoVFNCVT_RM_F_X_W_M2_MASK	= 2198,
    RISCV_PseudoVFNCVT_RM_F_X_W_M4	= 2199,
    RISCV_PseudoVFNCVT_RM_F_X_W_M4_MASK	= 2200,
    RISCV_PseudoVFNCVT_RM_F_X_W_MF2	= 2201,
    RISCV_PseudoVFNCVT_RM_F_X_W_MF2_MASK	= 2202,
    RISCV_PseudoVFNCVT_RM_F_X_W_MF4	= 2203,
    RISCV_PseudoVFNCVT_RM_F_X_W_MF4_MASK	= 2204,
    RISCV_PseudoVFNCVT_RM_XU_F_W_M1	= 2205,
    RISCV_PseudoVFNCVT_RM_XU_F_W_M1_MASK	= 2206,
    RISCV_PseudoVFNCVT_RM_XU_F_W_M2	= 2207,
    RISCV_PseudoVFNCVT_RM_XU_F_W_M2_MASK	= 2208,
    RISCV_PseudoVFNCVT_RM_XU_F_W_M4	= 2209,
    RISCV_PseudoVFNCVT_RM_XU_F_W_M4_MASK	= 2210,
    RISCV_PseudoVFNCVT_RM_XU_F_W_MF2	= 2211,
    RISCV_PseudoVFNCVT_RM_XU_F_W_MF2_MASK	= 2212,
    RISCV_PseudoVFNCVT_RM_XU_F_W_MF4	= 2213,
    RISCV_PseudoVFNCVT_RM_XU_F_W_MF4_MASK	= 2214,
    RISCV_PseudoVFNCVT_RM_XU_F_W_MF8	= 2215,
    RISCV_PseudoVFNCVT_RM_XU_F_W_MF8_MASK	= 2216,
    RISCV_PseudoVFNCVT_RM_X_F_W_M1	= 2217,
    RISCV_PseudoVFNCVT_RM_X_F_W_M1_MASK	= 2218,
    RISCV_PseudoVFNCVT_RM_X_F_W_M2	= 2219,
    RISCV_PseudoVFNCVT_RM_X_F_W_M2_MASK	= 2220,
    RISCV_PseudoVFNCVT_RM_X_F_W_M4	= 2221,
    RISCV_PseudoVFNCVT_RM_X_F_W_M4_MASK	= 2222,
    RISCV_PseudoVFNCVT_RM_X_F_W_MF2	= 2223,
    RISCV_PseudoVFNCVT_RM_X_F_W_MF2_MASK	= 2224,
    RISCV_PseudoVFNCVT_RM_X_F_W_MF4	= 2225,
    RISCV_PseudoVFNCVT_RM_X_F_W_MF4_MASK	= 2226,
    RISCV_PseudoVFNCVT_RM_X_F_W_MF8	= 2227,
    RISCV_PseudoVFNCVT_RM_X_F_W_MF8_MASK	= 2228,
    RISCV_PseudoVFNCVT_ROD_F_F_W_M1	= 2229,
    RISCV_PseudoVFNCVT_ROD_F_F_W_M1_MASK	= 2230,
    RISCV_PseudoVFNCVT_ROD_F_F_W_M2	= 2231,
    RISCV_PseudoVFNCVT_ROD_F_F_W_M2_MASK	= 2232,
    RISCV_PseudoVFNCVT_ROD_F_F_W_M4	= 2233,
    RISCV_PseudoVFNCVT_ROD_F_F_W_M4_MASK	= 2234,
    RISCV_PseudoVFNCVT_ROD_F_F_W_MF2	= 2235,
    RISCV_PseudoVFNCVT_ROD_F_F_W_MF2_MASK	= 2236,
    RISCV_PseudoVFNCVT_ROD_F_F_W_MF4	= 2237,
    RISCV_PseudoVFNCVT_ROD_F_F_W_MF4_MASK	= 2238,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_M1	= 2239,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_M1_MASK	= 2240,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_M2	= 2241,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_M2_MASK	= 2242,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_M4	= 2243,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_M4_MASK	= 2244,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_MF2	= 2245,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_MF2_MASK	= 2246,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_MF4	= 2247,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_MF4_MASK	= 2248,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_MF8	= 2249,
    RISCV_PseudoVFNCVT_RTZ_XU_F_W_MF8_MASK	= 2250,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_M1	= 2251,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_M1_MASK	= 2252,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_M2	= 2253,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_M2_MASK	= 2254,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_M4	= 2255,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_M4_MASK	= 2256,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_MF2	= 2257,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_MF2_MASK	= 2258,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_MF4	= 2259,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_MF4_MASK	= 2260,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_MF8	= 2261,
    RISCV_PseudoVFNCVT_RTZ_X_F_W_MF8_MASK	= 2262,
    RISCV_PseudoVFNCVT_XU_F_W_M1	= 2263,
    RISCV_PseudoVFNCVT_XU_F_W_M1_MASK	= 2264,
    RISCV_PseudoVFNCVT_XU_F_W_M2	= 2265,
    RISCV_PseudoVFNCVT_XU_F_W_M2_MASK	= 2266,
    RISCV_PseudoVFNCVT_XU_F_W_M4	= 2267,
    RISCV_PseudoVFNCVT_XU_F_W_M4_MASK	= 2268,
    RISCV_PseudoVFNCVT_XU_F_W_MF2	= 2269,
    RISCV_PseudoVFNCVT_XU_F_W_MF2_MASK	= 2270,
    RISCV_PseudoVFNCVT_XU_F_W_MF4	= 2271,
    RISCV_PseudoVFNCVT_XU_F_W_MF4_MASK	= 2272,
    RISCV_PseudoVFNCVT_XU_F_W_MF8	= 2273,
    RISCV_PseudoVFNCVT_XU_F_W_MF8_MASK	= 2274,
    RISCV_PseudoVFNCVT_X_F_W_M1	= 2275,
    RISCV_PseudoVFNCVT_X_F_W_M1_MASK	= 2276,
    RISCV_PseudoVFNCVT_X_F_W_M2	= 2277,
    RISCV_PseudoVFNCVT_X_F_W_M2_MASK	= 2278,
    RISCV_PseudoVFNCVT_X_F_W_M4	= 2279,
    RISCV_PseudoVFNCVT_X_F_W_M4_MASK	= 2280,
    RISCV_PseudoVFNCVT_X_F_W_MF2	= 2281,
    RISCV_PseudoVFNCVT_X_F_W_MF2_MASK	= 2282,
    RISCV_PseudoVFNCVT_X_F_W_MF4	= 2283,
    RISCV_PseudoVFNCVT_X_F_W_MF4_MASK	= 2284,
    RISCV_PseudoVFNCVT_X_F_W_MF8	= 2285,
    RISCV_PseudoVFNCVT_X_F_W_MF8_MASK	= 2286,
    RISCV_PseudoVFNMACC_VFPR16_M1	= 2287,
    RISCV_PseudoVFNMACC_VFPR16_M1_MASK	= 2288,
    RISCV_PseudoVFNMACC_VFPR16_M2	= 2289,
    RISCV_PseudoVFNMACC_VFPR16_M2_MASK	= 2290,
    RISCV_PseudoVFNMACC_VFPR16_M4	= 2291,
    RISCV_PseudoVFNMACC_VFPR16_M4_MASK	= 2292,
    RISCV_PseudoVFNMACC_VFPR16_M8	= 2293,
    RISCV_PseudoVFNMACC_VFPR16_M8_MASK	= 2294,
    RISCV_PseudoVFNMACC_VFPR16_MF2	= 2295,
    RISCV_PseudoVFNMACC_VFPR16_MF2_MASK	= 2296,
    RISCV_PseudoVFNMACC_VFPR16_MF4	= 2297,
    RISCV_PseudoVFNMACC_VFPR16_MF4_MASK	= 2298,
    RISCV_PseudoVFNMACC_VFPR32_M1	= 2299,
    RISCV_PseudoVFNMACC_VFPR32_M1_MASK	= 2300,
    RISCV_PseudoVFNMACC_VFPR32_M2	= 2301,
    RISCV_PseudoVFNMACC_VFPR32_M2_MASK	= 2302,
    RISCV_PseudoVFNMACC_VFPR32_M4	= 2303,
    RISCV_PseudoVFNMACC_VFPR32_M4_MASK	= 2304,
    RISCV_PseudoVFNMACC_VFPR32_M8	= 2305,
    RISCV_PseudoVFNMACC_VFPR32_M8_MASK	= 2306,
    RISCV_PseudoVFNMACC_VFPR32_MF2	= 2307,
    RISCV_PseudoVFNMACC_VFPR32_MF2_MASK	= 2308,
    RISCV_PseudoVFNMACC_VFPR64_M1	= 2309,
    RISCV_PseudoVFNMACC_VFPR64_M1_MASK	= 2310,
    RISCV_PseudoVFNMACC_VFPR64_M2	= 2311,
    RISCV_PseudoVFNMACC_VFPR64_M2_MASK	= 2312,
    RISCV_PseudoVFNMACC_VFPR64_M4	= 2313,
    RISCV_PseudoVFNMACC_VFPR64_M4_MASK	= 2314,
    RISCV_PseudoVFNMACC_VFPR64_M8	= 2315,
    RISCV_PseudoVFNMACC_VFPR64_M8_MASK	= 2316,
    RISCV_PseudoVFNMACC_VV_M1	= 2317,
    RISCV_PseudoVFNMACC_VV_M1_MASK	= 2318,
    RISCV_PseudoVFNMACC_VV_M2	= 2319,
    RISCV_PseudoVFNMACC_VV_M2_MASK	= 2320,
    RISCV_PseudoVFNMACC_VV_M4	= 2321,
    RISCV_PseudoVFNMACC_VV_M4_MASK	= 2322,
    RISCV_PseudoVFNMACC_VV_M8	= 2323,
    RISCV_PseudoVFNMACC_VV_M8_MASK	= 2324,
    RISCV_PseudoVFNMACC_VV_MF2	= 2325,
    RISCV_PseudoVFNMACC_VV_MF2_MASK	= 2326,
    RISCV_PseudoVFNMACC_VV_MF4	= 2327,
    RISCV_PseudoVFNMACC_VV_MF4_MASK	= 2328,
    RISCV_PseudoVFNMADD_VFPR16_M1	= 2329,
    RISCV_PseudoVFNMADD_VFPR16_M1_MASK	= 2330,
    RISCV_PseudoVFNMADD_VFPR16_M2	= 2331,
    RISCV_PseudoVFNMADD_VFPR16_M2_MASK	= 2332,
    RISCV_PseudoVFNMADD_VFPR16_M4	= 2333,
    RISCV_PseudoVFNMADD_VFPR16_M4_MASK	= 2334,
    RISCV_PseudoVFNMADD_VFPR16_M8	= 2335,
    RISCV_PseudoVFNMADD_VFPR16_M8_MASK	= 2336,
    RISCV_PseudoVFNMADD_VFPR16_MF2	= 2337,
    RISCV_PseudoVFNMADD_VFPR16_MF2_MASK	= 2338,
    RISCV_PseudoVFNMADD_VFPR16_MF4	= 2339,
    RISCV_PseudoVFNMADD_VFPR16_MF4_MASK	= 2340,
    RISCV_PseudoVFNMADD_VFPR32_M1	= 2341,
    RISCV_PseudoVFNMADD_VFPR32_M1_MASK	= 2342,
    RISCV_PseudoVFNMADD_VFPR32_M2	= 2343,
    RISCV_PseudoVFNMADD_VFPR32_M2_MASK	= 2344,
    RISCV_PseudoVFNMADD_VFPR32_M4	= 2345,
    RISCV_PseudoVFNMADD_VFPR32_M4_MASK	= 2346,
    RISCV_PseudoVFNMADD_VFPR32_M8	= 2347,
    RISCV_PseudoVFNMADD_VFPR32_M8_MASK	= 2348,
    RISCV_PseudoVFNMADD_VFPR32_MF2	= 2349,
    RISCV_PseudoVFNMADD_VFPR32_MF2_MASK	= 2350,
    RISCV_PseudoVFNMADD_VFPR64_M1	= 2351,
    RISCV_PseudoVFNMADD_VFPR64_M1_MASK	= 2352,
    RISCV_PseudoVFNMADD_VFPR64_M2	= 2353,
    RISCV_PseudoVFNMADD_VFPR64_M2_MASK	= 2354,
    RISCV_PseudoVFNMADD_VFPR64_M4	= 2355,
    RISCV_PseudoVFNMADD_VFPR64_M4_MASK	= 2356,
    RISCV_PseudoVFNMADD_VFPR64_M8	= 2357,
    RISCV_PseudoVFNMADD_VFPR64_M8_MASK	= 2358,
    RISCV_PseudoVFNMADD_VV_M1	= 2359,
    RISCV_PseudoVFNMADD_VV_M1_MASK	= 2360,
    RISCV_PseudoVFNMADD_VV_M2	= 2361,
    RISCV_PseudoVFNMADD_VV_M2_MASK	= 2362,
    RISCV_PseudoVFNMADD_VV_M4	= 2363,
    RISCV_PseudoVFNMADD_VV_M4_MASK	= 2364,
    RISCV_PseudoVFNMADD_VV_M8	= 2365,
    RISCV_PseudoVFNMADD_VV_M8_MASK	= 2366,
    RISCV_PseudoVFNMADD_VV_MF2	= 2367,
    RISCV_PseudoVFNMADD_VV_MF2_MASK	= 2368,
    RISCV_PseudoVFNMADD_VV_MF4	= 2369,
    RISCV_PseudoVFNMADD_VV_MF4_MASK	= 2370,
    RISCV_PseudoVFNMSAC_VFPR16_M1	= 2371,
    RISCV_PseudoVFNMSAC_VFPR16_M1_MASK	= 2372,
    RISCV_PseudoVFNMSAC_VFPR16_M2	= 2373,
    RISCV_PseudoVFNMSAC_VFPR16_M2_MASK	= 2374,
    RISCV_PseudoVFNMSAC_VFPR16_M4	= 2375,
    RISCV_PseudoVFNMSAC_VFPR16_M4_MASK	= 2376,
    RISCV_PseudoVFNMSAC_VFPR16_M8	= 2377,
    RISCV_PseudoVFNMSAC_VFPR16_M8_MASK	= 2378,
    RISCV_PseudoVFNMSAC_VFPR16_MF2	= 2379,
    RISCV_PseudoVFNMSAC_VFPR16_MF2_MASK	= 2380,
    RISCV_PseudoVFNMSAC_VFPR16_MF4	= 2381,
    RISCV_PseudoVFNMSAC_VFPR16_MF4_MASK	= 2382,
    RISCV_PseudoVFNMSAC_VFPR32_M1	= 2383,
    RISCV_PseudoVFNMSAC_VFPR32_M1_MASK	= 2384,
    RISCV_PseudoVFNMSAC_VFPR32_M2	= 2385,
    RISCV_PseudoVFNMSAC_VFPR32_M2_MASK	= 2386,
    RISCV_PseudoVFNMSAC_VFPR32_M4	= 2387,
    RISCV_PseudoVFNMSAC_VFPR32_M4_MASK	= 2388,
    RISCV_PseudoVFNMSAC_VFPR32_M8	= 2389,
    RISCV_PseudoVFNMSAC_VFPR32_M8_MASK	= 2390,
    RISCV_PseudoVFNMSAC_VFPR32_MF2	= 2391,
    RISCV_PseudoVFNMSAC_VFPR32_MF2_MASK	= 2392,
    RISCV_PseudoVFNMSAC_VFPR64_M1	= 2393,
    RISCV_PseudoVFNMSAC_VFPR64_M1_MASK	= 2394,
    RISCV_PseudoVFNMSAC_VFPR64_M2	= 2395,
    RISCV_PseudoVFNMSAC_VFPR64_M2_MASK	= 2396,
    RISCV_PseudoVFNMSAC_VFPR64_M4	= 2397,
    RISCV_PseudoVFNMSAC_VFPR64_M4_MASK	= 2398,
    RISCV_PseudoVFNMSAC_VFPR64_M8	= 2399,
    RISCV_PseudoVFNMSAC_VFPR64_M8_MASK	= 2400,
    RISCV_PseudoVFNMSAC_VV_M1	= 2401,
    RISCV_PseudoVFNMSAC_VV_M1_MASK	= 2402,
    RISCV_PseudoVFNMSAC_VV_M2	= 2403,
    RISCV_PseudoVFNMSAC_VV_M2_MASK	= 2404,
    RISCV_PseudoVFNMSAC_VV_M4	= 2405,
    RISCV_PseudoVFNMSAC_VV_M4_MASK	= 2406,
    RISCV_PseudoVFNMSAC_VV_M8	= 2407,
    RISCV_PseudoVFNMSAC_VV_M8_MASK	= 2408,
    RISCV_PseudoVFNMSAC_VV_MF2	= 2409,
    RISCV_PseudoVFNMSAC_VV_MF2_MASK	= 2410,
    RISCV_PseudoVFNMSAC_VV_MF4	= 2411,
    RISCV_PseudoVFNMSAC_VV_MF4_MASK	= 2412,
    RISCV_PseudoVFNMSUB_VFPR16_M1	= 2413,
    RISCV_PseudoVFNMSUB_VFPR16_M1_MASK	= 2414,
    RISCV_PseudoVFNMSUB_VFPR16_M2	= 2415,
    RISCV_PseudoVFNMSUB_VFPR16_M2_MASK	= 2416,
    RISCV_PseudoVFNMSUB_VFPR16_M4	= 2417,
    RISCV_PseudoVFNMSUB_VFPR16_M4_MASK	= 2418,
    RISCV_PseudoVFNMSUB_VFPR16_M8	= 2419,
    RISCV_PseudoVFNMSUB_VFPR16_M8_MASK	= 2420,
    RISCV_PseudoVFNMSUB_VFPR16_MF2	= 2421,
    RISCV_PseudoVFNMSUB_VFPR16_MF2_MASK	= 2422,
    RISCV_PseudoVFNMSUB_VFPR16_MF4	= 2423,
    RISCV_PseudoVFNMSUB_VFPR16_MF4_MASK	= 2424,
    RISCV_PseudoVFNMSUB_VFPR32_M1	= 2425,
    RISCV_PseudoVFNMSUB_VFPR32_M1_MASK	= 2426,
    RISCV_PseudoVFNMSUB_VFPR32_M2	= 2427,
    RISCV_PseudoVFNMSUB_VFPR32_M2_MASK	= 2428,
    RISCV_PseudoVFNMSUB_VFPR32_M4	= 2429,
    RISCV_PseudoVFNMSUB_VFPR32_M4_MASK	= 2430,
    RISCV_PseudoVFNMSUB_VFPR32_M8	= 2431,
    RISCV_PseudoVFNMSUB_VFPR32_M8_MASK	= 2432,
    RISCV_PseudoVFNMSUB_VFPR32_MF2	= 2433,
    RISCV_PseudoVFNMSUB_VFPR32_MF2_MASK	= 2434,
    RISCV_PseudoVFNMSUB_VFPR64_M1	= 2435,
    RISCV_PseudoVFNMSUB_VFPR64_M1_MASK	= 2436,
    RISCV_PseudoVFNMSUB_VFPR64_M2	= 2437,
    RISCV_PseudoVFNMSUB_VFPR64_M2_MASK	= 2438,
    RISCV_PseudoVFNMSUB_VFPR64_M4	= 2439,
    RISCV_PseudoVFNMSUB_VFPR64_M4_MASK	= 2440,
    RISCV_PseudoVFNMSUB_VFPR64_M8	= 2441,
    RISCV_PseudoVFNMSUB_VFPR64_M8_MASK	= 2442,
    RISCV_PseudoVFNMSUB_VV_M1	= 2443,
    RISCV_PseudoVFNMSUB_VV_M1_MASK	= 2444,
    RISCV_PseudoVFNMSUB_VV_M2	= 2445,
    RISCV_PseudoVFNMSUB_VV_M2_MASK	= 2446,
    RISCV_PseudoVFNMSUB_VV_M4	= 2447,
    RISCV_PseudoVFNMSUB_VV_M4_MASK	= 2448,
    RISCV_PseudoVFNMSUB_VV_M8	= 2449,
    RISCV_PseudoVFNMSUB_VV_M8_MASK	= 2450,
    RISCV_PseudoVFNMSUB_VV_MF2	= 2451,
    RISCV_PseudoVFNMSUB_VV_MF2_MASK	= 2452,
    RISCV_PseudoVFNMSUB_VV_MF4	= 2453,
    RISCV_PseudoVFNMSUB_VV_MF4_MASK	= 2454,
    RISCV_PseudoVFNRCLIP_XU_F_QF_M1	= 2455,
    RISCV_PseudoVFNRCLIP_XU_F_QF_M1_MASK	= 2456,
    RISCV_PseudoVFNRCLIP_XU_F_QF_M2	= 2457,
    RISCV_PseudoVFNRCLIP_XU_F_QF_M2_MASK	= 2458,
    RISCV_PseudoVFNRCLIP_XU_F_QF_MF2	= 2459,
    RISCV_PseudoVFNRCLIP_XU_F_QF_MF2_MASK	= 2460,
    RISCV_PseudoVFNRCLIP_XU_F_QF_MF4	= 2461,
    RISCV_PseudoVFNRCLIP_XU_F_QF_MF4_MASK	= 2462,
    RISCV_PseudoVFNRCLIP_XU_F_QF_MF8	= 2463,
    RISCV_PseudoVFNRCLIP_XU_F_QF_MF8_MASK	= 2464,
    RISCV_PseudoVFNRCLIP_X_F_QF_M1	= 2465,
    RISCV_PseudoVFNRCLIP_X_F_QF_M1_MASK	= 2466,
    RISCV_PseudoVFNRCLIP_X_F_QF_M2	= 2467,
    RISCV_PseudoVFNRCLIP_X_F_QF_M2_MASK	= 2468,
    RISCV_PseudoVFNRCLIP_X_F_QF_MF2	= 2469,
    RISCV_PseudoVFNRCLIP_X_F_QF_MF2_MASK	= 2470,
    RISCV_PseudoVFNRCLIP_X_F_QF_MF4	= 2471,
    RISCV_PseudoVFNRCLIP_X_F_QF_MF4_MASK	= 2472,
    RISCV_PseudoVFNRCLIP_X_F_QF_MF8	= 2473,
    RISCV_PseudoVFNRCLIP_X_F_QF_MF8_MASK	= 2474,
    RISCV_PseudoVFRDIV_VFPR16_M1_E16	= 2475,
    RISCV_PseudoVFRDIV_VFPR16_M1_E16_MASK	= 2476,
    RISCV_PseudoVFRDIV_VFPR16_M2_E16	= 2477,
    RISCV_PseudoVFRDIV_VFPR16_M2_E16_MASK	= 2478,
    RISCV_PseudoVFRDIV_VFPR16_M4_E16	= 2479,
    RISCV_PseudoVFRDIV_VFPR16_M4_E16_MASK	= 2480,
    RISCV_PseudoVFRDIV_VFPR16_M8_E16	= 2481,
    RISCV_PseudoVFRDIV_VFPR16_M8_E16_MASK	= 2482,
    RISCV_PseudoVFRDIV_VFPR16_MF2_E16	= 2483,
    RISCV_PseudoVFRDIV_VFPR16_MF2_E16_MASK	= 2484,
    RISCV_PseudoVFRDIV_VFPR16_MF4_E16	= 2485,
    RISCV_PseudoVFRDIV_VFPR16_MF4_E16_MASK	= 2486,
    RISCV_PseudoVFRDIV_VFPR32_M1_E32	= 2487,
    RISCV_PseudoVFRDIV_VFPR32_M1_E32_MASK	= 2488,
    RISCV_PseudoVFRDIV_VFPR32_M2_E32	= 2489,
    RISCV_PseudoVFRDIV_VFPR32_M2_E32_MASK	= 2490,
    RISCV_PseudoVFRDIV_VFPR32_M4_E32	= 2491,
    RISCV_PseudoVFRDIV_VFPR32_M4_E32_MASK	= 2492,
    RISCV_PseudoVFRDIV_VFPR32_M8_E32	= 2493,
    RISCV_PseudoVFRDIV_VFPR32_M8_E32_MASK	= 2494,
    RISCV_PseudoVFRDIV_VFPR32_MF2_E32	= 2495,
    RISCV_PseudoVFRDIV_VFPR32_MF2_E32_MASK	= 2496,
    RISCV_PseudoVFRDIV_VFPR64_M1_E64	= 2497,
    RISCV_PseudoVFRDIV_VFPR64_M1_E64_MASK	= 2498,
    RISCV_PseudoVFRDIV_VFPR64_M2_E64	= 2499,
    RISCV_PseudoVFRDIV_VFPR64_M2_E64_MASK	= 2500,
    RISCV_PseudoVFRDIV_VFPR64_M4_E64	= 2501,
    RISCV_PseudoVFRDIV_VFPR64_M4_E64_MASK	= 2502,
    RISCV_PseudoVFRDIV_VFPR64_M8_E64	= 2503,
    RISCV_PseudoVFRDIV_VFPR64_M8_E64_MASK	= 2504,
    RISCV_PseudoVFREC7_V_M1	= 2505,
    RISCV_PseudoVFREC7_V_M1_MASK	= 2506,
    RISCV_PseudoVFREC7_V_M2	= 2507,
    RISCV_PseudoVFREC7_V_M2_MASK	= 2508,
    RISCV_PseudoVFREC7_V_M4	= 2509,
    RISCV_PseudoVFREC7_V_M4_MASK	= 2510,
    RISCV_PseudoVFREC7_V_M8	= 2511,
    RISCV_PseudoVFREC7_V_M8_MASK	= 2512,
    RISCV_PseudoVFREC7_V_MF2	= 2513,
    RISCV_PseudoVFREC7_V_MF2_MASK	= 2514,
    RISCV_PseudoVFREC7_V_MF4	= 2515,
    RISCV_PseudoVFREC7_V_MF4_MASK	= 2516,
    RISCV_PseudoVFREDMAX_VS_M1_E16	= 2517,
    RISCV_PseudoVFREDMAX_VS_M1_E16_MASK	= 2518,
    RISCV_PseudoVFREDMAX_VS_M1_E32	= 2519,
    RISCV_PseudoVFREDMAX_VS_M1_E32_MASK	= 2520,
    RISCV_PseudoVFREDMAX_VS_M1_E64	= 2521,
    RISCV_PseudoVFREDMAX_VS_M1_E64_MASK	= 2522,
    RISCV_PseudoVFREDMAX_VS_M2_E16	= 2523,
    RISCV_PseudoVFREDMAX_VS_M2_E16_MASK	= 2524,
    RISCV_PseudoVFREDMAX_VS_M2_E32	= 2525,
    RISCV_PseudoVFREDMAX_VS_M2_E32_MASK	= 2526,
    RISCV_PseudoVFREDMAX_VS_M2_E64	= 2527,
    RISCV_PseudoVFREDMAX_VS_M2_E64_MASK	= 2528,
    RISCV_PseudoVFREDMAX_VS_M4_E16	= 2529,
    RISCV_PseudoVFREDMAX_VS_M4_E16_MASK	= 2530,
    RISCV_PseudoVFREDMAX_VS_M4_E32	= 2531,
    RISCV_PseudoVFREDMAX_VS_M4_E32_MASK	= 2532,
    RISCV_PseudoVFREDMAX_VS_M4_E64	= 2533,
    RISCV_PseudoVFREDMAX_VS_M4_E64_MASK	= 2534,
    RISCV_PseudoVFREDMAX_VS_M8_E16	= 2535,
    RISCV_PseudoVFREDMAX_VS_M8_E16_MASK	= 2536,
    RISCV_PseudoVFREDMAX_VS_M8_E32	= 2537,
    RISCV_PseudoVFREDMAX_VS_M8_E32_MASK	= 2538,
    RISCV_PseudoVFREDMAX_VS_M8_E64	= 2539,
    RISCV_PseudoVFREDMAX_VS_M8_E64_MASK	= 2540,
    RISCV_PseudoVFREDMAX_VS_MF2_E16	= 2541,
    RISCV_PseudoVFREDMAX_VS_MF2_E16_MASK	= 2542,
    RISCV_PseudoVFREDMAX_VS_MF2_E32	= 2543,
    RISCV_PseudoVFREDMAX_VS_MF2_E32_MASK	= 2544,
    RISCV_PseudoVFREDMAX_VS_MF4_E16	= 2545,
    RISCV_PseudoVFREDMAX_VS_MF4_E16_MASK	= 2546,
    RISCV_PseudoVFREDMIN_VS_M1_E16	= 2547,
    RISCV_PseudoVFREDMIN_VS_M1_E16_MASK	= 2548,
    RISCV_PseudoVFREDMIN_VS_M1_E32	= 2549,
    RISCV_PseudoVFREDMIN_VS_M1_E32_MASK	= 2550,
    RISCV_PseudoVFREDMIN_VS_M1_E64	= 2551,
    RISCV_PseudoVFREDMIN_VS_M1_E64_MASK	= 2552,
    RISCV_PseudoVFREDMIN_VS_M2_E16	= 2553,
    RISCV_PseudoVFREDMIN_VS_M2_E16_MASK	= 2554,
    RISCV_PseudoVFREDMIN_VS_M2_E32	= 2555,
    RISCV_PseudoVFREDMIN_VS_M2_E32_MASK	= 2556,
    RISCV_PseudoVFREDMIN_VS_M2_E64	= 2557,
    RISCV_PseudoVFREDMIN_VS_M2_E64_MASK	= 2558,
    RISCV_PseudoVFREDMIN_VS_M4_E16	= 2559,
    RISCV_PseudoVFREDMIN_VS_M4_E16_MASK	= 2560,
    RISCV_PseudoVFREDMIN_VS_M4_E32	= 2561,
    RISCV_PseudoVFREDMIN_VS_M4_E32_MASK	= 2562,
    RISCV_PseudoVFREDMIN_VS_M4_E64	= 2563,
    RISCV_PseudoVFREDMIN_VS_M4_E64_MASK	= 2564,
    RISCV_PseudoVFREDMIN_VS_M8_E16	= 2565,
    RISCV_PseudoVFREDMIN_VS_M8_E16_MASK	= 2566,
    RISCV_PseudoVFREDMIN_VS_M8_E32	= 2567,
    RISCV_PseudoVFREDMIN_VS_M8_E32_MASK	= 2568,
    RISCV_PseudoVFREDMIN_VS_M8_E64	= 2569,
    RISCV_PseudoVFREDMIN_VS_M8_E64_MASK	= 2570,
    RISCV_PseudoVFREDMIN_VS_MF2_E16	= 2571,
    RISCV_PseudoVFREDMIN_VS_MF2_E16_MASK	= 2572,
    RISCV_PseudoVFREDMIN_VS_MF2_E32	= 2573,
    RISCV_PseudoVFREDMIN_VS_MF2_E32_MASK	= 2574,
    RISCV_PseudoVFREDMIN_VS_MF4_E16	= 2575,
    RISCV_PseudoVFREDMIN_VS_MF4_E16_MASK	= 2576,
    RISCV_PseudoVFREDOSUM_VS_M1_E16	= 2577,
    RISCV_PseudoVFREDOSUM_VS_M1_E16_MASK	= 2578,
    RISCV_PseudoVFREDOSUM_VS_M1_E32	= 2579,
    RISCV_PseudoVFREDOSUM_VS_M1_E32_MASK	= 2580,
    RISCV_PseudoVFREDOSUM_VS_M1_E64	= 2581,
    RISCV_PseudoVFREDOSUM_VS_M1_E64_MASK	= 2582,
    RISCV_PseudoVFREDOSUM_VS_M2_E16	= 2583,
    RISCV_PseudoVFREDOSUM_VS_M2_E16_MASK	= 2584,
    RISCV_PseudoVFREDOSUM_VS_M2_E32	= 2585,
    RISCV_PseudoVFREDOSUM_VS_M2_E32_MASK	= 2586,
    RISCV_PseudoVFREDOSUM_VS_M2_E64	= 2587,
    RISCV_PseudoVFREDOSUM_VS_M2_E64_MASK	= 2588,
    RISCV_PseudoVFREDOSUM_VS_M4_E16	= 2589,
    RISCV_PseudoVFREDOSUM_VS_M4_E16_MASK	= 2590,
    RISCV_PseudoVFREDOSUM_VS_M4_E32	= 2591,
    RISCV_PseudoVFREDOSUM_VS_M4_E32_MASK	= 2592,
    RISCV_PseudoVFREDOSUM_VS_M4_E64	= 2593,
    RISCV_PseudoVFREDOSUM_VS_M4_E64_MASK	= 2594,
    RISCV_PseudoVFREDOSUM_VS_M8_E16	= 2595,
    RISCV_PseudoVFREDOSUM_VS_M8_E16_MASK	= 2596,
    RISCV_PseudoVFREDOSUM_VS_M8_E32	= 2597,
    RISCV_PseudoVFREDOSUM_VS_M8_E32_MASK	= 2598,
    RISCV_PseudoVFREDOSUM_VS_M8_E64	= 2599,
    RISCV_PseudoVFREDOSUM_VS_M8_E64_MASK	= 2600,
    RISCV_PseudoVFREDOSUM_VS_MF2_E16	= 2601,
    RISCV_PseudoVFREDOSUM_VS_MF2_E16_MASK	= 2602,
    RISCV_PseudoVFREDOSUM_VS_MF2_E32	= 2603,
    RISCV_PseudoVFREDOSUM_VS_MF2_E32_MASK	= 2604,
    RISCV_PseudoVFREDOSUM_VS_MF4_E16	= 2605,
    RISCV_PseudoVFREDOSUM_VS_MF4_E16_MASK	= 2606,
    RISCV_PseudoVFREDUSUM_VS_M1_E16	= 2607,
    RISCV_PseudoVFREDUSUM_VS_M1_E16_MASK	= 2608,
    RISCV_PseudoVFREDUSUM_VS_M1_E32	= 2609,
    RISCV_PseudoVFREDUSUM_VS_M1_E32_MASK	= 2610,
    RISCV_PseudoVFREDUSUM_VS_M1_E64	= 2611,
    RISCV_PseudoVFREDUSUM_VS_M1_E64_MASK	= 2612,
    RISCV_PseudoVFREDUSUM_VS_M2_E16	= 2613,
    RISCV_PseudoVFREDUSUM_VS_M2_E16_MASK	= 2614,
    RISCV_PseudoVFREDUSUM_VS_M2_E32	= 2615,
    RISCV_PseudoVFREDUSUM_VS_M2_E32_MASK	= 2616,
    RISCV_PseudoVFREDUSUM_VS_M2_E64	= 2617,
    RISCV_PseudoVFREDUSUM_VS_M2_E64_MASK	= 2618,
    RISCV_PseudoVFREDUSUM_VS_M4_E16	= 2619,
    RISCV_PseudoVFREDUSUM_VS_M4_E16_MASK	= 2620,
    RISCV_PseudoVFREDUSUM_VS_M4_E32	= 2621,
    RISCV_PseudoVFREDUSUM_VS_M4_E32_MASK	= 2622,
    RISCV_PseudoVFREDUSUM_VS_M4_E64	= 2623,
    RISCV_PseudoVFREDUSUM_VS_M4_E64_MASK	= 2624,
    RISCV_PseudoVFREDUSUM_VS_M8_E16	= 2625,
    RISCV_PseudoVFREDUSUM_VS_M8_E16_MASK	= 2626,
    RISCV_PseudoVFREDUSUM_VS_M8_E32	= 2627,
    RISCV_PseudoVFREDUSUM_VS_M8_E32_MASK	= 2628,
    RISCV_PseudoVFREDUSUM_VS_M8_E64	= 2629,
    RISCV_PseudoVFREDUSUM_VS_M8_E64_MASK	= 2630,
    RISCV_PseudoVFREDUSUM_VS_MF2_E16	= 2631,
    RISCV_PseudoVFREDUSUM_VS_MF2_E16_MASK	= 2632,
    RISCV_PseudoVFREDUSUM_VS_MF2_E32	= 2633,
    RISCV_PseudoVFREDUSUM_VS_MF2_E32_MASK	= 2634,
    RISCV_PseudoVFREDUSUM_VS_MF4_E16	= 2635,
    RISCV_PseudoVFREDUSUM_VS_MF4_E16_MASK	= 2636,
    RISCV_PseudoVFROUND_NOEXCEPT_V_M1_MASK	= 2637,
    RISCV_PseudoVFROUND_NOEXCEPT_V_M2_MASK	= 2638,
    RISCV_PseudoVFROUND_NOEXCEPT_V_M4_MASK	= 2639,
    RISCV_PseudoVFROUND_NOEXCEPT_V_M8_MASK	= 2640,
    RISCV_PseudoVFROUND_NOEXCEPT_V_MF2_MASK	= 2641,
    RISCV_PseudoVFROUND_NOEXCEPT_V_MF4_MASK	= 2642,
    RISCV_PseudoVFRSQRT7_V_M1	= 2643,
    RISCV_PseudoVFRSQRT7_V_M1_MASK	= 2644,
    RISCV_PseudoVFRSQRT7_V_M2	= 2645,
    RISCV_PseudoVFRSQRT7_V_M2_MASK	= 2646,
    RISCV_PseudoVFRSQRT7_V_M4	= 2647,
    RISCV_PseudoVFRSQRT7_V_M4_MASK	= 2648,
    RISCV_PseudoVFRSQRT7_V_M8	= 2649,
    RISCV_PseudoVFRSQRT7_V_M8_MASK	= 2650,
    RISCV_PseudoVFRSQRT7_V_MF2	= 2651,
    RISCV_PseudoVFRSQRT7_V_MF2_MASK	= 2652,
    RISCV_PseudoVFRSQRT7_V_MF4	= 2653,
    RISCV_PseudoVFRSQRT7_V_MF4_MASK	= 2654,
    RISCV_PseudoVFRSUB_VFPR16_M1	= 2655,
    RISCV_PseudoVFRSUB_VFPR16_M1_MASK	= 2656,
    RISCV_PseudoVFRSUB_VFPR16_M2	= 2657,
    RISCV_PseudoVFRSUB_VFPR16_M2_MASK	= 2658,
    RISCV_PseudoVFRSUB_VFPR16_M4	= 2659,
    RISCV_PseudoVFRSUB_VFPR16_M4_MASK	= 2660,
    RISCV_PseudoVFRSUB_VFPR16_M8	= 2661,
    RISCV_PseudoVFRSUB_VFPR16_M8_MASK	= 2662,
    RISCV_PseudoVFRSUB_VFPR16_MF2	= 2663,
    RISCV_PseudoVFRSUB_VFPR16_MF2_MASK	= 2664,
    RISCV_PseudoVFRSUB_VFPR16_MF4	= 2665,
    RISCV_PseudoVFRSUB_VFPR16_MF4_MASK	= 2666,
    RISCV_PseudoVFRSUB_VFPR32_M1	= 2667,
    RISCV_PseudoVFRSUB_VFPR32_M1_MASK	= 2668,
    RISCV_PseudoVFRSUB_VFPR32_M2	= 2669,
    RISCV_PseudoVFRSUB_VFPR32_M2_MASK	= 2670,
    RISCV_PseudoVFRSUB_VFPR32_M4	= 2671,
    RISCV_PseudoVFRSUB_VFPR32_M4_MASK	= 2672,
    RISCV_PseudoVFRSUB_VFPR32_M8	= 2673,
    RISCV_PseudoVFRSUB_VFPR32_M8_MASK	= 2674,
    RISCV_PseudoVFRSUB_VFPR32_MF2	= 2675,
    RISCV_PseudoVFRSUB_VFPR32_MF2_MASK	= 2676,
    RISCV_PseudoVFRSUB_VFPR64_M1	= 2677,
    RISCV_PseudoVFRSUB_VFPR64_M1_MASK	= 2678,
    RISCV_PseudoVFRSUB_VFPR64_M2	= 2679,
    RISCV_PseudoVFRSUB_VFPR64_M2_MASK	= 2680,
    RISCV_PseudoVFRSUB_VFPR64_M4	= 2681,
    RISCV_PseudoVFRSUB_VFPR64_M4_MASK	= 2682,
    RISCV_PseudoVFRSUB_VFPR64_M8	= 2683,
    RISCV_PseudoVFRSUB_VFPR64_M8_MASK	= 2684,
    RISCV_PseudoVFSGNJN_VFPR16_M1	= 2685,
    RISCV_PseudoVFSGNJN_VFPR16_M1_MASK	= 2686,
    RISCV_PseudoVFSGNJN_VFPR16_M2	= 2687,
    RISCV_PseudoVFSGNJN_VFPR16_M2_MASK	= 2688,
    RISCV_PseudoVFSGNJN_VFPR16_M4	= 2689,
    RISCV_PseudoVFSGNJN_VFPR16_M4_MASK	= 2690,
    RISCV_PseudoVFSGNJN_VFPR16_M8	= 2691,
    RISCV_PseudoVFSGNJN_VFPR16_M8_MASK	= 2692,
    RISCV_PseudoVFSGNJN_VFPR16_MF2	= 2693,
    RISCV_PseudoVFSGNJN_VFPR16_MF2_MASK	= 2694,
    RISCV_PseudoVFSGNJN_VFPR16_MF4	= 2695,
    RISCV_PseudoVFSGNJN_VFPR16_MF4_MASK	= 2696,
    RISCV_PseudoVFSGNJN_VFPR32_M1	= 2697,
    RISCV_PseudoVFSGNJN_VFPR32_M1_MASK	= 2698,
    RISCV_PseudoVFSGNJN_VFPR32_M2	= 2699,
    RISCV_PseudoVFSGNJN_VFPR32_M2_MASK	= 2700,
    RISCV_PseudoVFSGNJN_VFPR32_M4	= 2701,
    RISCV_PseudoVFSGNJN_VFPR32_M4_MASK	= 2702,
    RISCV_PseudoVFSGNJN_VFPR32_M8	= 2703,
    RISCV_PseudoVFSGNJN_VFPR32_M8_MASK	= 2704,
    RISCV_PseudoVFSGNJN_VFPR32_MF2	= 2705,
    RISCV_PseudoVFSGNJN_VFPR32_MF2_MASK	= 2706,
    RISCV_PseudoVFSGNJN_VFPR64_M1	= 2707,
    RISCV_PseudoVFSGNJN_VFPR64_M1_MASK	= 2708,
    RISCV_PseudoVFSGNJN_VFPR64_M2	= 2709,
    RISCV_PseudoVFSGNJN_VFPR64_M2_MASK	= 2710,
    RISCV_PseudoVFSGNJN_VFPR64_M4	= 2711,
    RISCV_PseudoVFSGNJN_VFPR64_M4_MASK	= 2712,
    RISCV_PseudoVFSGNJN_VFPR64_M8	= 2713,
    RISCV_PseudoVFSGNJN_VFPR64_M8_MASK	= 2714,
    RISCV_PseudoVFSGNJN_VV_M1	= 2715,
    RISCV_PseudoVFSGNJN_VV_M1_MASK	= 2716,
    RISCV_PseudoVFSGNJN_VV_M2	= 2717,
    RISCV_PseudoVFSGNJN_VV_M2_MASK	= 2718,
    RISCV_PseudoVFSGNJN_VV_M4	= 2719,
    RISCV_PseudoVFSGNJN_VV_M4_MASK	= 2720,
    RISCV_PseudoVFSGNJN_VV_M8	= 2721,
    RISCV_PseudoVFSGNJN_VV_M8_MASK	= 2722,
    RISCV_PseudoVFSGNJN_VV_MF2	= 2723,
    RISCV_PseudoVFSGNJN_VV_MF2_MASK	= 2724,
    RISCV_PseudoVFSGNJN_VV_MF4	= 2725,
    RISCV_PseudoVFSGNJN_VV_MF4_MASK	= 2726,
    RISCV_PseudoVFSGNJX_VFPR16_M1	= 2727,
    RISCV_PseudoVFSGNJX_VFPR16_M1_MASK	= 2728,
    RISCV_PseudoVFSGNJX_VFPR16_M2	= 2729,
    RISCV_PseudoVFSGNJX_VFPR16_M2_MASK	= 2730,
    RISCV_PseudoVFSGNJX_VFPR16_M4	= 2731,
    RISCV_PseudoVFSGNJX_VFPR16_M4_MASK	= 2732,
    RISCV_PseudoVFSGNJX_VFPR16_M8	= 2733,
    RISCV_PseudoVFSGNJX_VFPR16_M8_MASK	= 2734,
    RISCV_PseudoVFSGNJX_VFPR16_MF2	= 2735,
    RISCV_PseudoVFSGNJX_VFPR16_MF2_MASK	= 2736,
    RISCV_PseudoVFSGNJX_VFPR16_MF4	= 2737,
    RISCV_PseudoVFSGNJX_VFPR16_MF4_MASK	= 2738,
    RISCV_PseudoVFSGNJX_VFPR32_M1	= 2739,
    RISCV_PseudoVFSGNJX_VFPR32_M1_MASK	= 2740,
    RISCV_PseudoVFSGNJX_VFPR32_M2	= 2741,
    RISCV_PseudoVFSGNJX_VFPR32_M2_MASK	= 2742,
    RISCV_PseudoVFSGNJX_VFPR32_M4	= 2743,
    RISCV_PseudoVFSGNJX_VFPR32_M4_MASK	= 2744,
    RISCV_PseudoVFSGNJX_VFPR32_M8	= 2745,
    RISCV_PseudoVFSGNJX_VFPR32_M8_MASK	= 2746,
    RISCV_PseudoVFSGNJX_VFPR32_MF2	= 2747,
    RISCV_PseudoVFSGNJX_VFPR32_MF2_MASK	= 2748,
    RISCV_PseudoVFSGNJX_VFPR64_M1	= 2749,
    RISCV_PseudoVFSGNJX_VFPR64_M1_MASK	= 2750,
    RISCV_PseudoVFSGNJX_VFPR64_M2	= 2751,
    RISCV_PseudoVFSGNJX_VFPR64_M2_MASK	= 2752,
    RISCV_PseudoVFSGNJX_VFPR64_M4	= 2753,
    RISCV_PseudoVFSGNJX_VFPR64_M4_MASK	= 2754,
    RISCV_PseudoVFSGNJX_VFPR64_M8	= 2755,
    RISCV_PseudoVFSGNJX_VFPR64_M8_MASK	= 2756,
    RISCV_PseudoVFSGNJX_VV_M1	= 2757,
    RISCV_PseudoVFSGNJX_VV_M1_MASK	= 2758,
    RISCV_PseudoVFSGNJX_VV_M2	= 2759,
    RISCV_PseudoVFSGNJX_VV_M2_MASK	= 2760,
    RISCV_PseudoVFSGNJX_VV_M4	= 2761,
    RISCV_PseudoVFSGNJX_VV_M4_MASK	= 2762,
    RISCV_PseudoVFSGNJX_VV_M8	= 2763,
    RISCV_PseudoVFSGNJX_VV_M8_MASK	= 2764,
    RISCV_PseudoVFSGNJX_VV_MF2	= 2765,
    RISCV_PseudoVFSGNJX_VV_MF2_MASK	= 2766,
    RISCV_PseudoVFSGNJX_VV_MF4	= 2767,
    RISCV_PseudoVFSGNJX_VV_MF4_MASK	= 2768,
    RISCV_PseudoVFSGNJ_VFPR16_M1	= 2769,
    RISCV_PseudoVFSGNJ_VFPR16_M1_MASK	= 2770,
    RISCV_PseudoVFSGNJ_VFPR16_M2	= 2771,
    RISCV_PseudoVFSGNJ_VFPR16_M2_MASK	= 2772,
    RISCV_PseudoVFSGNJ_VFPR16_M4	= 2773,
    RISCV_PseudoVFSGNJ_VFPR16_M4_MASK	= 2774,
    RISCV_PseudoVFSGNJ_VFPR16_M8	= 2775,
    RISCV_PseudoVFSGNJ_VFPR16_M8_MASK	= 2776,
    RISCV_PseudoVFSGNJ_VFPR16_MF2	= 2777,
    RISCV_PseudoVFSGNJ_VFPR16_MF2_MASK	= 2778,
    RISCV_PseudoVFSGNJ_VFPR16_MF4	= 2779,
    RISCV_PseudoVFSGNJ_VFPR16_MF4_MASK	= 2780,
    RISCV_PseudoVFSGNJ_VFPR32_M1	= 2781,
    RISCV_PseudoVFSGNJ_VFPR32_M1_MASK	= 2782,
    RISCV_PseudoVFSGNJ_VFPR32_M2	= 2783,
    RISCV_PseudoVFSGNJ_VFPR32_M2_MASK	= 2784,
    RISCV_PseudoVFSGNJ_VFPR32_M4	= 2785,
    RISCV_PseudoVFSGNJ_VFPR32_M4_MASK	= 2786,
    RISCV_PseudoVFSGNJ_VFPR32_M8	= 2787,
    RISCV_PseudoVFSGNJ_VFPR32_M8_MASK	= 2788,
    RISCV_PseudoVFSGNJ_VFPR32_MF2	= 2789,
    RISCV_PseudoVFSGNJ_VFPR32_MF2_MASK	= 2790,
    RISCV_PseudoVFSGNJ_VFPR64_M1	= 2791,
    RISCV_PseudoVFSGNJ_VFPR64_M1_MASK	= 2792,
    RISCV_PseudoVFSGNJ_VFPR64_M2	= 2793,
    RISCV_PseudoVFSGNJ_VFPR64_M2_MASK	= 2794,
    RISCV_PseudoVFSGNJ_VFPR64_M4	= 2795,
    RISCV_PseudoVFSGNJ_VFPR64_M4_MASK	= 2796,
    RISCV_PseudoVFSGNJ_VFPR64_M8	= 2797,
    RISCV_PseudoVFSGNJ_VFPR64_M8_MASK	= 2798,
    RISCV_PseudoVFSGNJ_VV_M1	= 2799,
    RISCV_PseudoVFSGNJ_VV_M1_MASK	= 2800,
    RISCV_PseudoVFSGNJ_VV_M2	= 2801,
    RISCV_PseudoVFSGNJ_VV_M2_MASK	= 2802,
    RISCV_PseudoVFSGNJ_VV_M4	= 2803,
    RISCV_PseudoVFSGNJ_VV_M4_MASK	= 2804,
    RISCV_PseudoVFSGNJ_VV_M8	= 2805,
    RISCV_PseudoVFSGNJ_VV_M8_MASK	= 2806,
    RISCV_PseudoVFSGNJ_VV_MF2	= 2807,
    RISCV_PseudoVFSGNJ_VV_MF2_MASK	= 2808,
    RISCV_PseudoVFSGNJ_VV_MF4	= 2809,
    RISCV_PseudoVFSGNJ_VV_MF4_MASK	= 2810,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_M1	= 2811,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_M1_MASK	= 2812,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_M2	= 2813,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_M2_MASK	= 2814,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_M4	= 2815,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_M4_MASK	= 2816,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_M8	= 2817,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_M8_MASK	= 2818,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_MF2	= 2819,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_MF2_MASK	= 2820,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_MF4	= 2821,
    RISCV_PseudoVFSLIDE1DOWN_VFPR16_MF4_MASK	= 2822,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_M1	= 2823,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_M1_MASK	= 2824,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_M2	= 2825,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_M2_MASK	= 2826,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_M4	= 2827,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_M4_MASK	= 2828,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_M8	= 2829,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_M8_MASK	= 2830,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_MF2	= 2831,
    RISCV_PseudoVFSLIDE1DOWN_VFPR32_MF2_MASK	= 2832,
    RISCV_PseudoVFSLIDE1DOWN_VFPR64_M1	= 2833,
    RISCV_PseudoVFSLIDE1DOWN_VFPR64_M1_MASK	= 2834,
    RISCV_PseudoVFSLIDE1DOWN_VFPR64_M2	= 2835,
    RISCV_PseudoVFSLIDE1DOWN_VFPR64_M2_MASK	= 2836,
    RISCV_PseudoVFSLIDE1DOWN_VFPR64_M4	= 2837,
    RISCV_PseudoVFSLIDE1DOWN_VFPR64_M4_MASK	= 2838,
    RISCV_PseudoVFSLIDE1DOWN_VFPR64_M8	= 2839,
    RISCV_PseudoVFSLIDE1DOWN_VFPR64_M8_MASK	= 2840,
    RISCV_PseudoVFSLIDE1UP_VFPR16_M1	= 2841,
    RISCV_PseudoVFSLIDE1UP_VFPR16_M1_MASK	= 2842,
    RISCV_PseudoVFSLIDE1UP_VFPR16_M2	= 2843,
    RISCV_PseudoVFSLIDE1UP_VFPR16_M2_MASK	= 2844,
    RISCV_PseudoVFSLIDE1UP_VFPR16_M4	= 2845,
    RISCV_PseudoVFSLIDE1UP_VFPR16_M4_MASK	= 2846,
    RISCV_PseudoVFSLIDE1UP_VFPR16_M8	= 2847,
    RISCV_PseudoVFSLIDE1UP_VFPR16_M8_MASK	= 2848,
    RISCV_PseudoVFSLIDE1UP_VFPR16_MF2	= 2849,
    RISCV_PseudoVFSLIDE1UP_VFPR16_MF2_MASK	= 2850,
    RISCV_PseudoVFSLIDE1UP_VFPR16_MF4	= 2851,
    RISCV_PseudoVFSLIDE1UP_VFPR16_MF4_MASK	= 2852,
    RISCV_PseudoVFSLIDE1UP_VFPR32_M1	= 2853,
    RISCV_PseudoVFSLIDE1UP_VFPR32_M1_MASK	= 2854,
    RISCV_PseudoVFSLIDE1UP_VFPR32_M2	= 2855,
    RISCV_PseudoVFSLIDE1UP_VFPR32_M2_MASK	= 2856,
    RISCV_PseudoVFSLIDE1UP_VFPR32_M4	= 2857,
    RISCV_PseudoVFSLIDE1UP_VFPR32_M4_MASK	= 2858,
    RISCV_PseudoVFSLIDE1UP_VFPR32_M8	= 2859,
    RISCV_PseudoVFSLIDE1UP_VFPR32_M8_MASK	= 2860,
    RISCV_PseudoVFSLIDE1UP_VFPR32_MF2	= 2861,
    RISCV_PseudoVFSLIDE1UP_VFPR32_MF2_MASK	= 2862,
    RISCV_PseudoVFSLIDE1UP_VFPR64_M1	= 2863,
    RISCV_PseudoVFSLIDE1UP_VFPR64_M1_MASK	= 2864,
    RISCV_PseudoVFSLIDE1UP_VFPR64_M2	= 2865,
    RISCV_PseudoVFSLIDE1UP_VFPR64_M2_MASK	= 2866,
    RISCV_PseudoVFSLIDE1UP_VFPR64_M4	= 2867,
    RISCV_PseudoVFSLIDE1UP_VFPR64_M4_MASK	= 2868,
    RISCV_PseudoVFSLIDE1UP_VFPR64_M8	= 2869,
    RISCV_PseudoVFSLIDE1UP_VFPR64_M8_MASK	= 2870,
    RISCV_PseudoVFSQRT_V_M1_E16	= 2871,
    RISCV_PseudoVFSQRT_V_M1_E16_MASK	= 2872,
    RISCV_PseudoVFSQRT_V_M1_E32	= 2873,
    RISCV_PseudoVFSQRT_V_M1_E32_MASK	= 2874,
    RISCV_PseudoVFSQRT_V_M1_E64	= 2875,
    RISCV_PseudoVFSQRT_V_M1_E64_MASK	= 2876,
    RISCV_PseudoVFSQRT_V_M2_E16	= 2877,
    RISCV_PseudoVFSQRT_V_M2_E16_MASK	= 2878,
    RISCV_PseudoVFSQRT_V_M2_E32	= 2879,
    RISCV_PseudoVFSQRT_V_M2_E32_MASK	= 2880,
    RISCV_PseudoVFSQRT_V_M2_E64	= 2881,
    RISCV_PseudoVFSQRT_V_M2_E64_MASK	= 2882,
    RISCV_PseudoVFSQRT_V_M4_E16	= 2883,
    RISCV_PseudoVFSQRT_V_M4_E16_MASK	= 2884,
    RISCV_PseudoVFSQRT_V_M4_E32	= 2885,
    RISCV_PseudoVFSQRT_V_M4_E32_MASK	= 2886,
    RISCV_PseudoVFSQRT_V_M4_E64	= 2887,
    RISCV_PseudoVFSQRT_V_M4_E64_MASK	= 2888,
    RISCV_PseudoVFSQRT_V_M8_E16	= 2889,
    RISCV_PseudoVFSQRT_V_M8_E16_MASK	= 2890,
    RISCV_PseudoVFSQRT_V_M8_E32	= 2891,
    RISCV_PseudoVFSQRT_V_M8_E32_MASK	= 2892,
    RISCV_PseudoVFSQRT_V_M8_E64	= 2893,
    RISCV_PseudoVFSQRT_V_M8_E64_MASK	= 2894,
    RISCV_PseudoVFSQRT_V_MF2_E16	= 2895,
    RISCV_PseudoVFSQRT_V_MF2_E16_MASK	= 2896,
    RISCV_PseudoVFSQRT_V_MF2_E32	= 2897,
    RISCV_PseudoVFSQRT_V_MF2_E32_MASK	= 2898,
    RISCV_PseudoVFSQRT_V_MF4_E16	= 2899,
    RISCV_PseudoVFSQRT_V_MF4_E16_MASK	= 2900,
    RISCV_PseudoVFSUB_VFPR16_M1	= 2901,
    RISCV_PseudoVFSUB_VFPR16_M1_MASK	= 2902,
    RISCV_PseudoVFSUB_VFPR16_M2	= 2903,
    RISCV_PseudoVFSUB_VFPR16_M2_MASK	= 2904,
    RISCV_PseudoVFSUB_VFPR16_M4	= 2905,
    RISCV_PseudoVFSUB_VFPR16_M4_MASK	= 2906,
    RISCV_PseudoVFSUB_VFPR16_M8	= 2907,
    RISCV_PseudoVFSUB_VFPR16_M8_MASK	= 2908,
    RISCV_PseudoVFSUB_VFPR16_MF2	= 2909,
    RISCV_PseudoVFSUB_VFPR16_MF2_MASK	= 2910,
    RISCV_PseudoVFSUB_VFPR16_MF4	= 2911,
    RISCV_PseudoVFSUB_VFPR16_MF4_MASK	= 2912,
    RISCV_PseudoVFSUB_VFPR32_M1	= 2913,
    RISCV_PseudoVFSUB_VFPR32_M1_MASK	= 2914,
    RISCV_PseudoVFSUB_VFPR32_M2	= 2915,
    RISCV_PseudoVFSUB_VFPR32_M2_MASK	= 2916,
    RISCV_PseudoVFSUB_VFPR32_M4	= 2917,
    RISCV_PseudoVFSUB_VFPR32_M4_MASK	= 2918,
    RISCV_PseudoVFSUB_VFPR32_M8	= 2919,
    RISCV_PseudoVFSUB_VFPR32_M8_MASK	= 2920,
    RISCV_PseudoVFSUB_VFPR32_MF2	= 2921,
    RISCV_PseudoVFSUB_VFPR32_MF2_MASK	= 2922,
    RISCV_PseudoVFSUB_VFPR64_M1	= 2923,
    RISCV_PseudoVFSUB_VFPR64_M1_MASK	= 2924,
    RISCV_PseudoVFSUB_VFPR64_M2	= 2925,
    RISCV_PseudoVFSUB_VFPR64_M2_MASK	= 2926,
    RISCV_PseudoVFSUB_VFPR64_M4	= 2927,
    RISCV_PseudoVFSUB_VFPR64_M4_MASK	= 2928,
    RISCV_PseudoVFSUB_VFPR64_M8	= 2929,
    RISCV_PseudoVFSUB_VFPR64_M8_MASK	= 2930,
    RISCV_PseudoVFSUB_VV_M1	= 2931,
    RISCV_PseudoVFSUB_VV_M1_MASK	= 2932,
    RISCV_PseudoVFSUB_VV_M2	= 2933,
    RISCV_PseudoVFSUB_VV_M2_MASK	= 2934,
    RISCV_PseudoVFSUB_VV_M4	= 2935,
    RISCV_PseudoVFSUB_VV_M4_MASK	= 2936,
    RISCV_PseudoVFSUB_VV_M8	= 2937,
    RISCV_PseudoVFSUB_VV_M8_MASK	= 2938,
    RISCV_PseudoVFSUB_VV_MF2	= 2939,
    RISCV_PseudoVFSUB_VV_MF2_MASK	= 2940,
    RISCV_PseudoVFSUB_VV_MF4	= 2941,
    RISCV_PseudoVFSUB_VV_MF4_MASK	= 2942,
    RISCV_PseudoVFWADD_VFPR16_M1	= 2943,
    RISCV_PseudoVFWADD_VFPR16_M1_MASK	= 2944,
    RISCV_PseudoVFWADD_VFPR16_M2	= 2945,
    RISCV_PseudoVFWADD_VFPR16_M2_MASK	= 2946,
    RISCV_PseudoVFWADD_VFPR16_M4	= 2947,
    RISCV_PseudoVFWADD_VFPR16_M4_MASK	= 2948,
    RISCV_PseudoVFWADD_VFPR16_MF2	= 2949,
    RISCV_PseudoVFWADD_VFPR16_MF2_MASK	= 2950,
    RISCV_PseudoVFWADD_VFPR16_MF4	= 2951,
    RISCV_PseudoVFWADD_VFPR16_MF4_MASK	= 2952,
    RISCV_PseudoVFWADD_VFPR32_M1	= 2953,
    RISCV_PseudoVFWADD_VFPR32_M1_MASK	= 2954,
    RISCV_PseudoVFWADD_VFPR32_M2	= 2955,
    RISCV_PseudoVFWADD_VFPR32_M2_MASK	= 2956,
    RISCV_PseudoVFWADD_VFPR32_M4	= 2957,
    RISCV_PseudoVFWADD_VFPR32_M4_MASK	= 2958,
    RISCV_PseudoVFWADD_VFPR32_MF2	= 2959,
    RISCV_PseudoVFWADD_VFPR32_MF2_MASK	= 2960,
    RISCV_PseudoVFWADD_VV_M1	= 2961,
    RISCV_PseudoVFWADD_VV_M1_MASK	= 2962,
    RISCV_PseudoVFWADD_VV_M2	= 2963,
    RISCV_PseudoVFWADD_VV_M2_MASK	= 2964,
    RISCV_PseudoVFWADD_VV_M4	= 2965,
    RISCV_PseudoVFWADD_VV_M4_MASK	= 2966,
    RISCV_PseudoVFWADD_VV_MF2	= 2967,
    RISCV_PseudoVFWADD_VV_MF2_MASK	= 2968,
    RISCV_PseudoVFWADD_VV_MF4	= 2969,
    RISCV_PseudoVFWADD_VV_MF4_MASK	= 2970,
    RISCV_PseudoVFWADD_WFPR16_M1	= 2971,
    RISCV_PseudoVFWADD_WFPR16_M1_MASK	= 2972,
    RISCV_PseudoVFWADD_WFPR16_M2	= 2973,
    RISCV_PseudoVFWADD_WFPR16_M2_MASK	= 2974,
    RISCV_PseudoVFWADD_WFPR16_M4	= 2975,
    RISCV_PseudoVFWADD_WFPR16_M4_MASK	= 2976,
    RISCV_PseudoVFWADD_WFPR16_MF2	= 2977,
    RISCV_PseudoVFWADD_WFPR16_MF2_MASK	= 2978,
    RISCV_PseudoVFWADD_WFPR16_MF4	= 2979,
    RISCV_PseudoVFWADD_WFPR16_MF4_MASK	= 2980,
    RISCV_PseudoVFWADD_WFPR32_M1	= 2981,
    RISCV_PseudoVFWADD_WFPR32_M1_MASK	= 2982,
    RISCV_PseudoVFWADD_WFPR32_M2	= 2983,
    RISCV_PseudoVFWADD_WFPR32_M2_MASK	= 2984,
    RISCV_PseudoVFWADD_WFPR32_M4	= 2985,
    RISCV_PseudoVFWADD_WFPR32_M4_MASK	= 2986,
    RISCV_PseudoVFWADD_WFPR32_MF2	= 2987,
    RISCV_PseudoVFWADD_WFPR32_MF2_MASK	= 2988,
    RISCV_PseudoVFWADD_WV_M1	= 2989,
    RISCV_PseudoVFWADD_WV_M1_MASK	= 2990,
    RISCV_PseudoVFWADD_WV_M1_MASK_TIED	= 2991,
    RISCV_PseudoVFWADD_WV_M1_TIED	= 2992,
    RISCV_PseudoVFWADD_WV_M2	= 2993,
    RISCV_PseudoVFWADD_WV_M2_MASK	= 2994,
    RISCV_PseudoVFWADD_WV_M2_MASK_TIED	= 2995,
    RISCV_PseudoVFWADD_WV_M2_TIED	= 2996,
    RISCV_PseudoVFWADD_WV_M4	= 2997,
    RISCV_PseudoVFWADD_WV_M4_MASK	= 2998,
    RISCV_PseudoVFWADD_WV_M4_MASK_TIED	= 2999,
    RISCV_PseudoVFWADD_WV_M4_TIED	= 3000,
    RISCV_PseudoVFWADD_WV_MF2	= 3001,
    RISCV_PseudoVFWADD_WV_MF2_MASK	= 3002,
    RISCV_PseudoVFWADD_WV_MF2_MASK_TIED	= 3003,
    RISCV_PseudoVFWADD_WV_MF2_TIED	= 3004,
    RISCV_PseudoVFWADD_WV_MF4	= 3005,
    RISCV_PseudoVFWADD_WV_MF4_MASK	= 3006,
    RISCV_PseudoVFWADD_WV_MF4_MASK_TIED	= 3007,
    RISCV_PseudoVFWADD_WV_MF4_TIED	= 3008,
    RISCV_PseudoVFWCVTBF16_F_F_V_M1	= 3009,
    RISCV_PseudoVFWCVTBF16_F_F_V_M1_MASK	= 3010,
    RISCV_PseudoVFWCVTBF16_F_F_V_M2	= 3011,
    RISCV_PseudoVFWCVTBF16_F_F_V_M2_MASK	= 3012,
    RISCV_PseudoVFWCVTBF16_F_F_V_M4	= 3013,
    RISCV_PseudoVFWCVTBF16_F_F_V_M4_MASK	= 3014,
    RISCV_PseudoVFWCVTBF16_F_F_V_MF2	= 3015,
    RISCV_PseudoVFWCVTBF16_F_F_V_MF2_MASK	= 3016,
    RISCV_PseudoVFWCVTBF16_F_F_V_MF4	= 3017,
    RISCV_PseudoVFWCVTBF16_F_F_V_MF4_MASK	= 3018,
    RISCV_PseudoVFWCVT_F_F_V_M1	= 3019,
    RISCV_PseudoVFWCVT_F_F_V_M1_MASK	= 3020,
    RISCV_PseudoVFWCVT_F_F_V_M2	= 3021,
    RISCV_PseudoVFWCVT_F_F_V_M2_MASK	= 3022,
    RISCV_PseudoVFWCVT_F_F_V_M4	= 3023,
    RISCV_PseudoVFWCVT_F_F_V_M4_MASK	= 3024,
    RISCV_PseudoVFWCVT_F_F_V_MF2	= 3025,
    RISCV_PseudoVFWCVT_F_F_V_MF2_MASK	= 3026,
    RISCV_PseudoVFWCVT_F_F_V_MF4	= 3027,
    RISCV_PseudoVFWCVT_F_F_V_MF4_MASK	= 3028,
    RISCV_PseudoVFWCVT_F_XU_V_M1	= 3029,
    RISCV_PseudoVFWCVT_F_XU_V_M1_MASK	= 3030,
    RISCV_PseudoVFWCVT_F_XU_V_M2	= 3031,
    RISCV_PseudoVFWCVT_F_XU_V_M2_MASK	= 3032,
    RISCV_PseudoVFWCVT_F_XU_V_M4	= 3033,
    RISCV_PseudoVFWCVT_F_XU_V_M4_MASK	= 3034,
    RISCV_PseudoVFWCVT_F_XU_V_MF2	= 3035,
    RISCV_PseudoVFWCVT_F_XU_V_MF2_MASK	= 3036,
    RISCV_PseudoVFWCVT_F_XU_V_MF4	= 3037,
    RISCV_PseudoVFWCVT_F_XU_V_MF4_MASK	= 3038,
    RISCV_PseudoVFWCVT_F_XU_V_MF8	= 3039,
    RISCV_PseudoVFWCVT_F_XU_V_MF8_MASK	= 3040,
    RISCV_PseudoVFWCVT_F_X_V_M1	= 3041,
    RISCV_PseudoVFWCVT_F_X_V_M1_MASK	= 3042,
    RISCV_PseudoVFWCVT_F_X_V_M2	= 3043,
    RISCV_PseudoVFWCVT_F_X_V_M2_MASK	= 3044,
    RISCV_PseudoVFWCVT_F_X_V_M4	= 3045,
    RISCV_PseudoVFWCVT_F_X_V_M4_MASK	= 3046,
    RISCV_PseudoVFWCVT_F_X_V_MF2	= 3047,
    RISCV_PseudoVFWCVT_F_X_V_MF2_MASK	= 3048,
    RISCV_PseudoVFWCVT_F_X_V_MF4	= 3049,
    RISCV_PseudoVFWCVT_F_X_V_MF4_MASK	= 3050,
    RISCV_PseudoVFWCVT_F_X_V_MF8	= 3051,
    RISCV_PseudoVFWCVT_F_X_V_MF8_MASK	= 3052,
    RISCV_PseudoVFWCVT_RM_XU_F_V_M1	= 3053,
    RISCV_PseudoVFWCVT_RM_XU_F_V_M1_MASK	= 3054,
    RISCV_PseudoVFWCVT_RM_XU_F_V_M2	= 3055,
    RISCV_PseudoVFWCVT_RM_XU_F_V_M2_MASK	= 3056,
    RISCV_PseudoVFWCVT_RM_XU_F_V_M4	= 3057,
    RISCV_PseudoVFWCVT_RM_XU_F_V_M4_MASK	= 3058,
    RISCV_PseudoVFWCVT_RM_XU_F_V_MF2	= 3059,
    RISCV_PseudoVFWCVT_RM_XU_F_V_MF2_MASK	= 3060,
    RISCV_PseudoVFWCVT_RM_XU_F_V_MF4	= 3061,
    RISCV_PseudoVFWCVT_RM_XU_F_V_MF4_MASK	= 3062,
    RISCV_PseudoVFWCVT_RM_X_F_V_M1	= 3063,
    RISCV_PseudoVFWCVT_RM_X_F_V_M1_MASK	= 3064,
    RISCV_PseudoVFWCVT_RM_X_F_V_M2	= 3065,
    RISCV_PseudoVFWCVT_RM_X_F_V_M2_MASK	= 3066,
    RISCV_PseudoVFWCVT_RM_X_F_V_M4	= 3067,
    RISCV_PseudoVFWCVT_RM_X_F_V_M4_MASK	= 3068,
    RISCV_PseudoVFWCVT_RM_X_F_V_MF2	= 3069,
    RISCV_PseudoVFWCVT_RM_X_F_V_MF2_MASK	= 3070,
    RISCV_PseudoVFWCVT_RM_X_F_V_MF4	= 3071,
    RISCV_PseudoVFWCVT_RM_X_F_V_MF4_MASK	= 3072,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_M1	= 3073,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_M1_MASK	= 3074,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_M2	= 3075,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_M2_MASK	= 3076,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_M4	= 3077,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_M4_MASK	= 3078,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_MF2	= 3079,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_MF2_MASK	= 3080,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_MF4	= 3081,
    RISCV_PseudoVFWCVT_RTZ_XU_F_V_MF4_MASK	= 3082,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_M1	= 3083,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_M1_MASK	= 3084,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_M2	= 3085,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_M2_MASK	= 3086,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_M4	= 3087,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_M4_MASK	= 3088,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_MF2	= 3089,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_MF2_MASK	= 3090,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_MF4	= 3091,
    RISCV_PseudoVFWCVT_RTZ_X_F_V_MF4_MASK	= 3092,
    RISCV_PseudoVFWCVT_XU_F_V_M1	= 3093,
    RISCV_PseudoVFWCVT_XU_F_V_M1_MASK	= 3094,
    RISCV_PseudoVFWCVT_XU_F_V_M2	= 3095,
    RISCV_PseudoVFWCVT_XU_F_V_M2_MASK	= 3096,
    RISCV_PseudoVFWCVT_XU_F_V_M4	= 3097,
    RISCV_PseudoVFWCVT_XU_F_V_M4_MASK	= 3098,
    RISCV_PseudoVFWCVT_XU_F_V_MF2	= 3099,
    RISCV_PseudoVFWCVT_XU_F_V_MF2_MASK	= 3100,
    RISCV_PseudoVFWCVT_XU_F_V_MF4	= 3101,
    RISCV_PseudoVFWCVT_XU_F_V_MF4_MASK	= 3102,
    RISCV_PseudoVFWCVT_X_F_V_M1	= 3103,
    RISCV_PseudoVFWCVT_X_F_V_M1_MASK	= 3104,
    RISCV_PseudoVFWCVT_X_F_V_M2	= 3105,
    RISCV_PseudoVFWCVT_X_F_V_M2_MASK	= 3106,
    RISCV_PseudoVFWCVT_X_F_V_M4	= 3107,
    RISCV_PseudoVFWCVT_X_F_V_M4_MASK	= 3108,
    RISCV_PseudoVFWCVT_X_F_V_MF2	= 3109,
    RISCV_PseudoVFWCVT_X_F_V_MF2_MASK	= 3110,
    RISCV_PseudoVFWCVT_X_F_V_MF4	= 3111,
    RISCV_PseudoVFWCVT_X_F_V_MF4_MASK	= 3112,
    RISCV_PseudoVFWMACCBF16_VFPR16_M1	= 3113,
    RISCV_PseudoVFWMACCBF16_VFPR16_M1_MASK	= 3114,
    RISCV_PseudoVFWMACCBF16_VFPR16_M2	= 3115,
    RISCV_PseudoVFWMACCBF16_VFPR16_M2_MASK	= 3116,
    RISCV_PseudoVFWMACCBF16_VFPR16_M4	= 3117,
    RISCV_PseudoVFWMACCBF16_VFPR16_M4_MASK	= 3118,
    RISCV_PseudoVFWMACCBF16_VFPR16_MF2	= 3119,
    RISCV_PseudoVFWMACCBF16_VFPR16_MF2_MASK	= 3120,
    RISCV_PseudoVFWMACCBF16_VFPR16_MF4	= 3121,
    RISCV_PseudoVFWMACCBF16_VFPR16_MF4_MASK	= 3122,
    RISCV_PseudoVFWMACCBF16_VV_M1	= 3123,
    RISCV_PseudoVFWMACCBF16_VV_M1_MASK	= 3124,
    RISCV_PseudoVFWMACCBF16_VV_M2	= 3125,
    RISCV_PseudoVFWMACCBF16_VV_M2_MASK	= 3126,
    RISCV_PseudoVFWMACCBF16_VV_M4	= 3127,
    RISCV_PseudoVFWMACCBF16_VV_M4_MASK	= 3128,
    RISCV_PseudoVFWMACCBF16_VV_MF2	= 3129,
    RISCV_PseudoVFWMACCBF16_VV_MF2_MASK	= 3130,
    RISCV_PseudoVFWMACCBF16_VV_MF4	= 3131,
    RISCV_PseudoVFWMACCBF16_VV_MF4_MASK	= 3132,
    RISCV_PseudoVFWMACC_4x4x4_M1	= 3133,
    RISCV_PseudoVFWMACC_4x4x4_M2	= 3134,
    RISCV_PseudoVFWMACC_4x4x4_M4	= 3135,
    RISCV_PseudoVFWMACC_4x4x4_M8	= 3136,
    RISCV_PseudoVFWMACC_4x4x4_MF2	= 3137,
    RISCV_PseudoVFWMACC_4x4x4_MF4	= 3138,
    RISCV_PseudoVFWMACC_VFPR16_M1	= 3139,
    RISCV_PseudoVFWMACC_VFPR16_M1_MASK	= 3140,
    RISCV_PseudoVFWMACC_VFPR16_M2	= 3141,
    RISCV_PseudoVFWMACC_VFPR16_M2_MASK	= 3142,
    RISCV_PseudoVFWMACC_VFPR16_M4	= 3143,
    RISCV_PseudoVFWMACC_VFPR16_M4_MASK	= 3144,
    RISCV_PseudoVFWMACC_VFPR16_MF2	= 3145,
    RISCV_PseudoVFWMACC_VFPR16_MF2_MASK	= 3146,
    RISCV_PseudoVFWMACC_VFPR16_MF4	= 3147,
    RISCV_PseudoVFWMACC_VFPR16_MF4_MASK	= 3148,
    RISCV_PseudoVFWMACC_VFPR32_M1	= 3149,
    RISCV_PseudoVFWMACC_VFPR32_M1_MASK	= 3150,
    RISCV_PseudoVFWMACC_VFPR32_M2	= 3151,
    RISCV_PseudoVFWMACC_VFPR32_M2_MASK	= 3152,
    RISCV_PseudoVFWMACC_VFPR32_M4	= 3153,
    RISCV_PseudoVFWMACC_VFPR32_M4_MASK	= 3154,
    RISCV_PseudoVFWMACC_VFPR32_MF2	= 3155,
    RISCV_PseudoVFWMACC_VFPR32_MF2_MASK	= 3156,
    RISCV_PseudoVFWMACC_VV_M1	= 3157,
    RISCV_PseudoVFWMACC_VV_M1_MASK	= 3158,
    RISCV_PseudoVFWMACC_VV_M2	= 3159,
    RISCV_PseudoVFWMACC_VV_M2_MASK	= 3160,
    RISCV_PseudoVFWMACC_VV_M4	= 3161,
    RISCV_PseudoVFWMACC_VV_M4_MASK	= 3162,
    RISCV_PseudoVFWMACC_VV_MF2	= 3163,
    RISCV_PseudoVFWMACC_VV_MF2_MASK	= 3164,
    RISCV_PseudoVFWMACC_VV_MF4	= 3165,
    RISCV_PseudoVFWMACC_VV_MF4_MASK	= 3166,
    RISCV_PseudoVFWMSAC_VFPR16_M1	= 3167,
    RISCV_PseudoVFWMSAC_VFPR16_M1_MASK	= 3168,
    RISCV_PseudoVFWMSAC_VFPR16_M2	= 3169,
    RISCV_PseudoVFWMSAC_VFPR16_M2_MASK	= 3170,
    RISCV_PseudoVFWMSAC_VFPR16_M4	= 3171,
    RISCV_PseudoVFWMSAC_VFPR16_M4_MASK	= 3172,
    RISCV_PseudoVFWMSAC_VFPR16_MF2	= 3173,
    RISCV_PseudoVFWMSAC_VFPR16_MF2_MASK	= 3174,
    RISCV_PseudoVFWMSAC_VFPR16_MF4	= 3175,
    RISCV_PseudoVFWMSAC_VFPR16_MF4_MASK	= 3176,
    RISCV_PseudoVFWMSAC_VFPR32_M1	= 3177,
    RISCV_PseudoVFWMSAC_VFPR32_M1_MASK	= 3178,
    RISCV_PseudoVFWMSAC_VFPR32_M2	= 3179,
    RISCV_PseudoVFWMSAC_VFPR32_M2_MASK	= 3180,
    RISCV_PseudoVFWMSAC_VFPR32_M4	= 3181,
    RISCV_PseudoVFWMSAC_VFPR32_M4_MASK	= 3182,
    RISCV_PseudoVFWMSAC_VFPR32_MF2	= 3183,
    RISCV_PseudoVFWMSAC_VFPR32_MF2_MASK	= 3184,
    RISCV_PseudoVFWMSAC_VV_M1	= 3185,
    RISCV_PseudoVFWMSAC_VV_M1_MASK	= 3186,
    RISCV_PseudoVFWMSAC_VV_M2	= 3187,
    RISCV_PseudoVFWMSAC_VV_M2_MASK	= 3188,
    RISCV_PseudoVFWMSAC_VV_M4	= 3189,
    RISCV_PseudoVFWMSAC_VV_M4_MASK	= 3190,
    RISCV_PseudoVFWMSAC_VV_MF2	= 3191,
    RISCV_PseudoVFWMSAC_VV_MF2_MASK	= 3192,
    RISCV_PseudoVFWMSAC_VV_MF4	= 3193,
    RISCV_PseudoVFWMSAC_VV_MF4_MASK	= 3194,
    RISCV_PseudoVFWMUL_VFPR16_M1	= 3195,
    RISCV_PseudoVFWMUL_VFPR16_M1_MASK	= 3196,
    RISCV_PseudoVFWMUL_VFPR16_M2	= 3197,
    RISCV_PseudoVFWMUL_VFPR16_M2_MASK	= 3198,
    RISCV_PseudoVFWMUL_VFPR16_M4	= 3199,
    RISCV_PseudoVFWMUL_VFPR16_M4_MASK	= 3200,
    RISCV_PseudoVFWMUL_VFPR16_MF2	= 3201,
    RISCV_PseudoVFWMUL_VFPR16_MF2_MASK	= 3202,
    RISCV_PseudoVFWMUL_VFPR16_MF4	= 3203,
    RISCV_PseudoVFWMUL_VFPR16_MF4_MASK	= 3204,
    RISCV_PseudoVFWMUL_VFPR32_M1	= 3205,
    RISCV_PseudoVFWMUL_VFPR32_M1_MASK	= 3206,
    RISCV_PseudoVFWMUL_VFPR32_M2	= 3207,
    RISCV_PseudoVFWMUL_VFPR32_M2_MASK	= 3208,
    RISCV_PseudoVFWMUL_VFPR32_M4	= 3209,
    RISCV_PseudoVFWMUL_VFPR32_M4_MASK	= 3210,
    RISCV_PseudoVFWMUL_VFPR32_MF2	= 3211,
    RISCV_PseudoVFWMUL_VFPR32_MF2_MASK	= 3212,
    RISCV_PseudoVFWMUL_VV_M1	= 3213,
    RISCV_PseudoVFWMUL_VV_M1_MASK	= 3214,
    RISCV_PseudoVFWMUL_VV_M2	= 3215,
    RISCV_PseudoVFWMUL_VV_M2_MASK	= 3216,
    RISCV_PseudoVFWMUL_VV_M4	= 3217,
    RISCV_PseudoVFWMUL_VV_M4_MASK	= 3218,
    RISCV_PseudoVFWMUL_VV_MF2	= 3219,
    RISCV_PseudoVFWMUL_VV_MF2_MASK	= 3220,
    RISCV_PseudoVFWMUL_VV_MF4	= 3221,
    RISCV_PseudoVFWMUL_VV_MF4_MASK	= 3222,
    RISCV_PseudoVFWNMACC_VFPR16_M1	= 3223,
    RISCV_PseudoVFWNMACC_VFPR16_M1_MASK	= 3224,
    RISCV_PseudoVFWNMACC_VFPR16_M2	= 3225,
    RISCV_PseudoVFWNMACC_VFPR16_M2_MASK	= 3226,
    RISCV_PseudoVFWNMACC_VFPR16_M4	= 3227,
    RISCV_PseudoVFWNMACC_VFPR16_M4_MASK	= 3228,
    RISCV_PseudoVFWNMACC_VFPR16_MF2	= 3229,
    RISCV_PseudoVFWNMACC_VFPR16_MF2_MASK	= 3230,
    RISCV_PseudoVFWNMACC_VFPR16_MF4	= 3231,
    RISCV_PseudoVFWNMACC_VFPR16_MF4_MASK	= 3232,
    RISCV_PseudoVFWNMACC_VFPR32_M1	= 3233,
    RISCV_PseudoVFWNMACC_VFPR32_M1_MASK	= 3234,
    RISCV_PseudoVFWNMACC_VFPR32_M2	= 3235,
    RISCV_PseudoVFWNMACC_VFPR32_M2_MASK	= 3236,
    RISCV_PseudoVFWNMACC_VFPR32_M4	= 3237,
    RISCV_PseudoVFWNMACC_VFPR32_M4_MASK	= 3238,
    RISCV_PseudoVFWNMACC_VFPR32_MF2	= 3239,
    RISCV_PseudoVFWNMACC_VFPR32_MF2_MASK	= 3240,
    RISCV_PseudoVFWNMACC_VV_M1	= 3241,
    RISCV_PseudoVFWNMACC_VV_M1_MASK	= 3242,
    RISCV_PseudoVFWNMACC_VV_M2	= 3243,
    RISCV_PseudoVFWNMACC_VV_M2_MASK	= 3244,
    RISCV_PseudoVFWNMACC_VV_M4	= 3245,
    RISCV_PseudoVFWNMACC_VV_M4_MASK	= 3246,
    RISCV_PseudoVFWNMACC_VV_MF2	= 3247,
    RISCV_PseudoVFWNMACC_VV_MF2_MASK	= 3248,
    RISCV_PseudoVFWNMACC_VV_MF4	= 3249,
    RISCV_PseudoVFWNMACC_VV_MF4_MASK	= 3250,
    RISCV_PseudoVFWNMSAC_VFPR16_M1	= 3251,
    RISCV_PseudoVFWNMSAC_VFPR16_M1_MASK	= 3252,
    RISCV_PseudoVFWNMSAC_VFPR16_M2	= 3253,
    RISCV_PseudoVFWNMSAC_VFPR16_M2_MASK	= 3254,
    RISCV_PseudoVFWNMSAC_VFPR16_M4	= 3255,
    RISCV_PseudoVFWNMSAC_VFPR16_M4_MASK	= 3256,
    RISCV_PseudoVFWNMSAC_VFPR16_MF2	= 3257,
    RISCV_PseudoVFWNMSAC_VFPR16_MF2_MASK	= 3258,
    RISCV_PseudoVFWNMSAC_VFPR16_MF4	= 3259,
    RISCV_PseudoVFWNMSAC_VFPR16_MF4_MASK	= 3260,
    RISCV_PseudoVFWNMSAC_VFPR32_M1	= 3261,
    RISCV_PseudoVFWNMSAC_VFPR32_M1_MASK	= 3262,
    RISCV_PseudoVFWNMSAC_VFPR32_M2	= 3263,
    RISCV_PseudoVFWNMSAC_VFPR32_M2_MASK	= 3264,
    RISCV_PseudoVFWNMSAC_VFPR32_M4	= 3265,
    RISCV_PseudoVFWNMSAC_VFPR32_M4_MASK	= 3266,
    RISCV_PseudoVFWNMSAC_VFPR32_MF2	= 3267,
    RISCV_PseudoVFWNMSAC_VFPR32_MF2_MASK	= 3268,
    RISCV_PseudoVFWNMSAC_VV_M1	= 3269,
    RISCV_PseudoVFWNMSAC_VV_M1_MASK	= 3270,
    RISCV_PseudoVFWNMSAC_VV_M2	= 3271,
    RISCV_PseudoVFWNMSAC_VV_M2_MASK	= 3272,
    RISCV_PseudoVFWNMSAC_VV_M4	= 3273,
    RISCV_PseudoVFWNMSAC_VV_M4_MASK	= 3274,
    RISCV_PseudoVFWNMSAC_VV_MF2	= 3275,
    RISCV_PseudoVFWNMSAC_VV_MF2_MASK	= 3276,
    RISCV_PseudoVFWNMSAC_VV_MF4	= 3277,
    RISCV_PseudoVFWNMSAC_VV_MF4_MASK	= 3278,
    RISCV_PseudoVFWREDOSUM_VS_M1_E16	= 3279,
    RISCV_PseudoVFWREDOSUM_VS_M1_E16_MASK	= 3280,
    RISCV_PseudoVFWREDOSUM_VS_M1_E32	= 3281,
    RISCV_PseudoVFWREDOSUM_VS_M1_E32_MASK	= 3282,
    RISCV_PseudoVFWREDOSUM_VS_M2_E16	= 3283,
    RISCV_PseudoVFWREDOSUM_VS_M2_E16_MASK	= 3284,
    RISCV_PseudoVFWREDOSUM_VS_M2_E32	= 3285,
    RISCV_PseudoVFWREDOSUM_VS_M2_E32_MASK	= 3286,
    RISCV_PseudoVFWREDOSUM_VS_M4_E16	= 3287,
    RISCV_PseudoVFWREDOSUM_VS_M4_E16_MASK	= 3288,
    RISCV_PseudoVFWREDOSUM_VS_M4_E32	= 3289,
    RISCV_PseudoVFWREDOSUM_VS_M4_E32_MASK	= 3290,
    RISCV_PseudoVFWREDOSUM_VS_M8_E16	= 3291,
    RISCV_PseudoVFWREDOSUM_VS_M8_E16_MASK	= 3292,
    RISCV_PseudoVFWREDOSUM_VS_M8_E32	= 3293,
    RISCV_PseudoVFWREDOSUM_VS_M8_E32_MASK	= 3294,
    RISCV_PseudoVFWREDOSUM_VS_MF2_E16	= 3295,
    RISCV_PseudoVFWREDOSUM_VS_MF2_E16_MASK	= 3296,
    RISCV_PseudoVFWREDOSUM_VS_MF2_E32	= 3297,
    RISCV_PseudoVFWREDOSUM_VS_MF2_E32_MASK	= 3298,
    RISCV_PseudoVFWREDOSUM_VS_MF4_E16	= 3299,
    RISCV_PseudoVFWREDOSUM_VS_MF4_E16_MASK	= 3300,
    RISCV_PseudoVFWREDUSUM_VS_M1_E16	= 3301,
    RISCV_PseudoVFWREDUSUM_VS_M1_E16_MASK	= 3302,
    RISCV_PseudoVFWREDUSUM_VS_M1_E32	= 3303,
    RISCV_PseudoVFWREDUSUM_VS_M1_E32_MASK	= 3304,
    RISCV_PseudoVFWREDUSUM_VS_M2_E16	= 3305,
    RISCV_PseudoVFWREDUSUM_VS_M2_E16_MASK	= 3306,
    RISCV_PseudoVFWREDUSUM_VS_M2_E32	= 3307,
    RISCV_PseudoVFWREDUSUM_VS_M2_E32_MASK	= 3308,
    RISCV_PseudoVFWREDUSUM_VS_M4_E16	= 3309,
    RISCV_PseudoVFWREDUSUM_VS_M4_E16_MASK	= 3310,
    RISCV_PseudoVFWREDUSUM_VS_M4_E32	= 3311,
    RISCV_PseudoVFWREDUSUM_VS_M4_E32_MASK	= 3312,
    RISCV_PseudoVFWREDUSUM_VS_M8_E16	= 3313,
    RISCV_PseudoVFWREDUSUM_VS_M8_E16_MASK	= 3314,
    RISCV_PseudoVFWREDUSUM_VS_M8_E32	= 3315,
    RISCV_PseudoVFWREDUSUM_VS_M8_E32_MASK	= 3316,
    RISCV_PseudoVFWREDUSUM_VS_MF2_E16	= 3317,
    RISCV_PseudoVFWREDUSUM_VS_MF2_E16_MASK	= 3318,
    RISCV_PseudoVFWREDUSUM_VS_MF2_E32	= 3319,
    RISCV_PseudoVFWREDUSUM_VS_MF2_E32_MASK	= 3320,
    RISCV_PseudoVFWREDUSUM_VS_MF4_E16	= 3321,
    RISCV_PseudoVFWREDUSUM_VS_MF4_E16_MASK	= 3322,
    RISCV_PseudoVFWSUB_VFPR16_M1	= 3323,
    RISCV_PseudoVFWSUB_VFPR16_M1_MASK	= 3324,
    RISCV_PseudoVFWSUB_VFPR16_M2	= 3325,
    RISCV_PseudoVFWSUB_VFPR16_M2_MASK	= 3326,
    RISCV_PseudoVFWSUB_VFPR16_M4	= 3327,
    RISCV_PseudoVFWSUB_VFPR16_M4_MASK	= 3328,
    RISCV_PseudoVFWSUB_VFPR16_MF2	= 3329,
    RISCV_PseudoVFWSUB_VFPR16_MF2_MASK	= 3330,
    RISCV_PseudoVFWSUB_VFPR16_MF4	= 3331,
    RISCV_PseudoVFWSUB_VFPR16_MF4_MASK	= 3332,
    RISCV_PseudoVFWSUB_VFPR32_M1	= 3333,
    RISCV_PseudoVFWSUB_VFPR32_M1_MASK	= 3334,
    RISCV_PseudoVFWSUB_VFPR32_M2	= 3335,
    RISCV_PseudoVFWSUB_VFPR32_M2_MASK	= 3336,
    RISCV_PseudoVFWSUB_VFPR32_M4	= 3337,
    RISCV_PseudoVFWSUB_VFPR32_M4_MASK	= 3338,
    RISCV_PseudoVFWSUB_VFPR32_MF2	= 3339,
    RISCV_PseudoVFWSUB_VFPR32_MF2_MASK	= 3340,
    RISCV_PseudoVFWSUB_VV_M1	= 3341,
    RISCV_PseudoVFWSUB_VV_M1_MASK	= 3342,
    RISCV_PseudoVFWSUB_VV_M2	= 3343,
    RISCV_PseudoVFWSUB_VV_M2_MASK	= 3344,
    RISCV_PseudoVFWSUB_VV_M4	= 3345,
    RISCV_PseudoVFWSUB_VV_M4_MASK	= 3346,
    RISCV_PseudoVFWSUB_VV_MF2	= 3347,
    RISCV_PseudoVFWSUB_VV_MF2_MASK	= 3348,
    RISCV_PseudoVFWSUB_VV_MF4	= 3349,
    RISCV_PseudoVFWSUB_VV_MF4_MASK	= 3350,
    RISCV_PseudoVFWSUB_WFPR16_M1	= 3351,
    RISCV_PseudoVFWSUB_WFPR16_M1_MASK	= 3352,
    RISCV_PseudoVFWSUB_WFPR16_M2	= 3353,
    RISCV_PseudoVFWSUB_WFPR16_M2_MASK	= 3354,
    RISCV_PseudoVFWSUB_WFPR16_M4	= 3355,
    RISCV_PseudoVFWSUB_WFPR16_M4_MASK	= 3356,
    RISCV_PseudoVFWSUB_WFPR16_MF2	= 3357,
    RISCV_PseudoVFWSUB_WFPR16_MF2_MASK	= 3358,
    RISCV_PseudoVFWSUB_WFPR16_MF4	= 3359,
    RISCV_PseudoVFWSUB_WFPR16_MF4_MASK	= 3360,
    RISCV_PseudoVFWSUB_WFPR32_M1	= 3361,
    RISCV_PseudoVFWSUB_WFPR32_M1_MASK	= 3362,
    RISCV_PseudoVFWSUB_WFPR32_M2	= 3363,
    RISCV_PseudoVFWSUB_WFPR32_M2_MASK	= 3364,
    RISCV_PseudoVFWSUB_WFPR32_M4	= 3365,
    RISCV_PseudoVFWSUB_WFPR32_M4_MASK	= 3366,
    RISCV_PseudoVFWSUB_WFPR32_MF2	= 3367,
    RISCV_PseudoVFWSUB_WFPR32_MF2_MASK	= 3368,
    RISCV_PseudoVFWSUB_WV_M1	= 3369,
    RISCV_PseudoVFWSUB_WV_M1_MASK	= 3370,
    RISCV_PseudoVFWSUB_WV_M1_MASK_TIED	= 3371,
    RISCV_PseudoVFWSUB_WV_M1_TIED	= 3372,
    RISCV_PseudoVFWSUB_WV_M2	= 3373,
    RISCV_PseudoVFWSUB_WV_M2_MASK	= 3374,
    RISCV_PseudoVFWSUB_WV_M2_MASK_TIED	= 3375,
    RISCV_PseudoVFWSUB_WV_M2_TIED	= 3376,
    RISCV_PseudoVFWSUB_WV_M4	= 3377,
    RISCV_PseudoVFWSUB_WV_M4_MASK	= 3378,
    RISCV_PseudoVFWSUB_WV_M4_MASK_TIED	= 3379,
    RISCV_PseudoVFWSUB_WV_M4_TIED	= 3380,
    RISCV_PseudoVFWSUB_WV_MF2	= 3381,
    RISCV_PseudoVFWSUB_WV_MF2_MASK	= 3382,
    RISCV_PseudoVFWSUB_WV_MF2_MASK_TIED	= 3383,
    RISCV_PseudoVFWSUB_WV_MF2_TIED	= 3384,
    RISCV_PseudoVFWSUB_WV_MF4	= 3385,
    RISCV_PseudoVFWSUB_WV_MF4_MASK	= 3386,
    RISCV_PseudoVFWSUB_WV_MF4_MASK_TIED	= 3387,
    RISCV_PseudoVFWSUB_WV_MF4_TIED	= 3388,
    RISCV_PseudoVGHSH_VV_M1	= 3389,
    RISCV_PseudoVGHSH_VV_M2	= 3390,
    RISCV_PseudoVGHSH_VV_M4	= 3391,
    RISCV_PseudoVGHSH_VV_M8	= 3392,
    RISCV_PseudoVGHSH_VV_MF2	= 3393,
    RISCV_PseudoVGMUL_VV_M1	= 3394,
    RISCV_PseudoVGMUL_VV_M2	= 3395,
    RISCV_PseudoVGMUL_VV_M4	= 3396,
    RISCV_PseudoVGMUL_VV_M8	= 3397,
    RISCV_PseudoVGMUL_VV_MF2	= 3398,
    RISCV_PseudoVID_V_M1	= 3399,
    RISCV_PseudoVID_V_M1_MASK	= 3400,
    RISCV_PseudoVID_V_M2	= 3401,
    RISCV_PseudoVID_V_M2_MASK	= 3402,
    RISCV_PseudoVID_V_M4	= 3403,
    RISCV_PseudoVID_V_M4_MASK	= 3404,
    RISCV_PseudoVID_V_M8	= 3405,
    RISCV_PseudoVID_V_M8_MASK	= 3406,
    RISCV_PseudoVID_V_MF2	= 3407,
    RISCV_PseudoVID_V_MF2_MASK	= 3408,
    RISCV_PseudoVID_V_MF4	= 3409,
    RISCV_PseudoVID_V_MF4_MASK	= 3410,
    RISCV_PseudoVID_V_MF8	= 3411,
    RISCV_PseudoVID_V_MF8_MASK	= 3412,
    RISCV_PseudoVIOTA_M_M1	= 3413,
    RISCV_PseudoVIOTA_M_M1_MASK	= 3414,
    RISCV_PseudoVIOTA_M_M2	= 3415,
    RISCV_PseudoVIOTA_M_M2_MASK	= 3416,
    RISCV_PseudoVIOTA_M_M4	= 3417,
    RISCV_PseudoVIOTA_M_M4_MASK	= 3418,
    RISCV_PseudoVIOTA_M_M8	= 3419,
    RISCV_PseudoVIOTA_M_M8_MASK	= 3420,
    RISCV_PseudoVIOTA_M_MF2	= 3421,
    RISCV_PseudoVIOTA_M_MF2_MASK	= 3422,
    RISCV_PseudoVIOTA_M_MF4	= 3423,
    RISCV_PseudoVIOTA_M_MF4_MASK	= 3424,
    RISCV_PseudoVIOTA_M_MF8	= 3425,
    RISCV_PseudoVIOTA_M_MF8_MASK	= 3426,
    RISCV_PseudoVLE16FF_V_M1	= 3427,
    RISCV_PseudoVLE16FF_V_M1_MASK	= 3428,
    RISCV_PseudoVLE16FF_V_M2	= 3429,
    RISCV_PseudoVLE16FF_V_M2_MASK	= 3430,
    RISCV_PseudoVLE16FF_V_M4	= 3431,
    RISCV_PseudoVLE16FF_V_M4_MASK	= 3432,
    RISCV_PseudoVLE16FF_V_M8	= 3433,
    RISCV_PseudoVLE16FF_V_M8_MASK	= 3434,
    RISCV_PseudoVLE16FF_V_MF2	= 3435,
    RISCV_PseudoVLE16FF_V_MF2_MASK	= 3436,
    RISCV_PseudoVLE16FF_V_MF4	= 3437,
    RISCV_PseudoVLE16FF_V_MF4_MASK	= 3438,
    RISCV_PseudoVLE16_V_M1	= 3439,
    RISCV_PseudoVLE16_V_M1_MASK	= 3440,
    RISCV_PseudoVLE16_V_M2	= 3441,
    RISCV_PseudoVLE16_V_M2_MASK	= 3442,
    RISCV_PseudoVLE16_V_M4	= 3443,
    RISCV_PseudoVLE16_V_M4_MASK	= 3444,
    RISCV_PseudoVLE16_V_M8	= 3445,
    RISCV_PseudoVLE16_V_M8_MASK	= 3446,
    RISCV_PseudoVLE16_V_MF2	= 3447,
    RISCV_PseudoVLE16_V_MF2_MASK	= 3448,
    RISCV_PseudoVLE16_V_MF4	= 3449,
    RISCV_PseudoVLE16_V_MF4_MASK	= 3450,
    RISCV_PseudoVLE32FF_V_M1	= 3451,
    RISCV_PseudoVLE32FF_V_M1_MASK	= 3452,
    RISCV_PseudoVLE32FF_V_M2	= 3453,
    RISCV_PseudoVLE32FF_V_M2_MASK	= 3454,
    RISCV_PseudoVLE32FF_V_M4	= 3455,
    RISCV_PseudoVLE32FF_V_M4_MASK	= 3456,
    RISCV_PseudoVLE32FF_V_M8	= 3457,
    RISCV_PseudoVLE32FF_V_M8_MASK	= 3458,
    RISCV_PseudoVLE32FF_V_MF2	= 3459,
    RISCV_PseudoVLE32FF_V_MF2_MASK	= 3460,
    RISCV_PseudoVLE32_V_M1	= 3461,
    RISCV_PseudoVLE32_V_M1_MASK	= 3462,
    RISCV_PseudoVLE32_V_M2	= 3463,
    RISCV_PseudoVLE32_V_M2_MASK	= 3464,
    RISCV_PseudoVLE32_V_M4	= 3465,
    RISCV_PseudoVLE32_V_M4_MASK	= 3466,
    RISCV_PseudoVLE32_V_M8	= 3467,
    RISCV_PseudoVLE32_V_M8_MASK	= 3468,
    RISCV_PseudoVLE32_V_MF2	= 3469,
    RISCV_PseudoVLE32_V_MF2_MASK	= 3470,
    RISCV_PseudoVLE64FF_V_M1	= 3471,
    RISCV_PseudoVLE64FF_V_M1_MASK	= 3472,
    RISCV_PseudoVLE64FF_V_M2	= 3473,
    RISCV_PseudoVLE64FF_V_M2_MASK	= 3474,
    RISCV_PseudoVLE64FF_V_M4	= 3475,
    RISCV_PseudoVLE64FF_V_M4_MASK	= 3476,
    RISCV_PseudoVLE64FF_V_M8	= 3477,
    RISCV_PseudoVLE64FF_V_M8_MASK	= 3478,
    RISCV_PseudoVLE64_V_M1	= 3479,
    RISCV_PseudoVLE64_V_M1_MASK	= 3480,
    RISCV_PseudoVLE64_V_M2	= 3481,
    RISCV_PseudoVLE64_V_M2_MASK	= 3482,
    RISCV_PseudoVLE64_V_M4	= 3483,
    RISCV_PseudoVLE64_V_M4_MASK	= 3484,
    RISCV_PseudoVLE64_V_M8	= 3485,
    RISCV_PseudoVLE64_V_M8_MASK	= 3486,
    RISCV_PseudoVLE8FF_V_M1	= 3487,
    RISCV_PseudoVLE8FF_V_M1_MASK	= 3488,
    RISCV_PseudoVLE8FF_V_M2	= 3489,
    RISCV_PseudoVLE8FF_V_M2_MASK	= 3490,
    RISCV_PseudoVLE8FF_V_M4	= 3491,
    RISCV_PseudoVLE8FF_V_M4_MASK	= 3492,
    RISCV_PseudoVLE8FF_V_M8	= 3493,
    RISCV_PseudoVLE8FF_V_M8_MASK	= 3494,
    RISCV_PseudoVLE8FF_V_MF2	= 3495,
    RISCV_PseudoVLE8FF_V_MF2_MASK	= 3496,
    RISCV_PseudoVLE8FF_V_MF4	= 3497,
    RISCV_PseudoVLE8FF_V_MF4_MASK	= 3498,
    RISCV_PseudoVLE8FF_V_MF8	= 3499,
    RISCV_PseudoVLE8FF_V_MF8_MASK	= 3500,
    RISCV_PseudoVLE8_V_M1	= 3501,
    RISCV_PseudoVLE8_V_M1_MASK	= 3502,
    RISCV_PseudoVLE8_V_M2	= 3503,
    RISCV_PseudoVLE8_V_M2_MASK	= 3504,
    RISCV_PseudoVLE8_V_M4	= 3505,
    RISCV_PseudoVLE8_V_M4_MASK	= 3506,
    RISCV_PseudoVLE8_V_M8	= 3507,
    RISCV_PseudoVLE8_V_M8_MASK	= 3508,
    RISCV_PseudoVLE8_V_MF2	= 3509,
    RISCV_PseudoVLE8_V_MF2_MASK	= 3510,
    RISCV_PseudoVLE8_V_MF4	= 3511,
    RISCV_PseudoVLE8_V_MF4_MASK	= 3512,
    RISCV_PseudoVLE8_V_MF8	= 3513,
    RISCV_PseudoVLE8_V_MF8_MASK	= 3514,
    RISCV_PseudoVLM_V_B1	= 3515,
    RISCV_PseudoVLM_V_B16	= 3516,
    RISCV_PseudoVLM_V_B2	= 3517,
    RISCV_PseudoVLM_V_B32	= 3518,
    RISCV_PseudoVLM_V_B4	= 3519,
    RISCV_PseudoVLM_V_B64	= 3520,
    RISCV_PseudoVLM_V_B8	= 3521,
    RISCV_PseudoVLOXEI16_V_M1_M1	= 3522,
    RISCV_PseudoVLOXEI16_V_M1_M1_MASK	= 3523,
    RISCV_PseudoVLOXEI16_V_M1_M2	= 3524,
    RISCV_PseudoVLOXEI16_V_M1_M2_MASK	= 3525,
    RISCV_PseudoVLOXEI16_V_M1_M4	= 3526,
    RISCV_PseudoVLOXEI16_V_M1_M4_MASK	= 3527,
    RISCV_PseudoVLOXEI16_V_M1_MF2	= 3528,
    RISCV_PseudoVLOXEI16_V_M1_MF2_MASK	= 3529,
    RISCV_PseudoVLOXEI16_V_M2_M1	= 3530,
    RISCV_PseudoVLOXEI16_V_M2_M1_MASK	= 3531,
    RISCV_PseudoVLOXEI16_V_M2_M2	= 3532,
    RISCV_PseudoVLOXEI16_V_M2_M2_MASK	= 3533,
    RISCV_PseudoVLOXEI16_V_M2_M4	= 3534,
    RISCV_PseudoVLOXEI16_V_M2_M4_MASK	= 3535,
    RISCV_PseudoVLOXEI16_V_M2_M8	= 3536,
    RISCV_PseudoVLOXEI16_V_M2_M8_MASK	= 3537,
    RISCV_PseudoVLOXEI16_V_M4_M2	= 3538,
    RISCV_PseudoVLOXEI16_V_M4_M2_MASK	= 3539,
    RISCV_PseudoVLOXEI16_V_M4_M4	= 3540,
    RISCV_PseudoVLOXEI16_V_M4_M4_MASK	= 3541,
    RISCV_PseudoVLOXEI16_V_M4_M8	= 3542,
    RISCV_PseudoVLOXEI16_V_M4_M8_MASK	= 3543,
    RISCV_PseudoVLOXEI16_V_M8_M4	= 3544,
    RISCV_PseudoVLOXEI16_V_M8_M4_MASK	= 3545,
    RISCV_PseudoVLOXEI16_V_M8_M8	= 3546,
    RISCV_PseudoVLOXEI16_V_M8_M8_MASK	= 3547,
    RISCV_PseudoVLOXEI16_V_MF2_M1	= 3548,
    RISCV_PseudoVLOXEI16_V_MF2_M1_MASK	= 3549,
    RISCV_PseudoVLOXEI16_V_MF2_M2	= 3550,
    RISCV_PseudoVLOXEI16_V_MF2_M2_MASK	= 3551,
    RISCV_PseudoVLOXEI16_V_MF2_MF2	= 3552,
    RISCV_PseudoVLOXEI16_V_MF2_MF2_MASK	= 3553,
    RISCV_PseudoVLOXEI16_V_MF2_MF4	= 3554,
    RISCV_PseudoVLOXEI16_V_MF2_MF4_MASK	= 3555,
    RISCV_PseudoVLOXEI16_V_MF4_M1	= 3556,
    RISCV_PseudoVLOXEI16_V_MF4_M1_MASK	= 3557,
    RISCV_PseudoVLOXEI16_V_MF4_MF2	= 3558,
    RISCV_PseudoVLOXEI16_V_MF4_MF2_MASK	= 3559,
    RISCV_PseudoVLOXEI16_V_MF4_MF4	= 3560,
    RISCV_PseudoVLOXEI16_V_MF4_MF4_MASK	= 3561,
    RISCV_PseudoVLOXEI16_V_MF4_MF8	= 3562,
    RISCV_PseudoVLOXEI16_V_MF4_MF8_MASK	= 3563,
    RISCV_PseudoVLOXEI32_V_M1_M1	= 3564,
    RISCV_PseudoVLOXEI32_V_M1_M1_MASK	= 3565,
    RISCV_PseudoVLOXEI32_V_M1_M2	= 3566,
    RISCV_PseudoVLOXEI32_V_M1_M2_MASK	= 3567,
    RISCV_PseudoVLOXEI32_V_M1_MF2	= 3568,
    RISCV_PseudoVLOXEI32_V_M1_MF2_MASK	= 3569,
    RISCV_PseudoVLOXEI32_V_M1_MF4	= 3570,
    RISCV_PseudoVLOXEI32_V_M1_MF4_MASK	= 3571,
    RISCV_PseudoVLOXEI32_V_M2_M1	= 3572,
    RISCV_PseudoVLOXEI32_V_M2_M1_MASK	= 3573,
    RISCV_PseudoVLOXEI32_V_M2_M2	= 3574,
    RISCV_PseudoVLOXEI32_V_M2_M2_MASK	= 3575,
    RISCV_PseudoVLOXEI32_V_M2_M4	= 3576,
    RISCV_PseudoVLOXEI32_V_M2_M4_MASK	= 3577,
    RISCV_PseudoVLOXEI32_V_M2_MF2	= 3578,
    RISCV_PseudoVLOXEI32_V_M2_MF2_MASK	= 3579,
    RISCV_PseudoVLOXEI32_V_M4_M1	= 3580,
    RISCV_PseudoVLOXEI32_V_M4_M1_MASK	= 3581,
    RISCV_PseudoVLOXEI32_V_M4_M2	= 3582,
    RISCV_PseudoVLOXEI32_V_M4_M2_MASK	= 3583,
    RISCV_PseudoVLOXEI32_V_M4_M4	= 3584,
    RISCV_PseudoVLOXEI32_V_M4_M4_MASK	= 3585,
    RISCV_PseudoVLOXEI32_V_M4_M8	= 3586,
    RISCV_PseudoVLOXEI32_V_M4_M8_MASK	= 3587,
    RISCV_PseudoVLOXEI32_V_M8_M2	= 3588,
    RISCV_PseudoVLOXEI32_V_M8_M2_MASK	= 3589,
    RISCV_PseudoVLOXEI32_V_M8_M4	= 3590,
    RISCV_PseudoVLOXEI32_V_M8_M4_MASK	= 3591,
    RISCV_PseudoVLOXEI32_V_M8_M8	= 3592,
    RISCV_PseudoVLOXEI32_V_M8_M8_MASK	= 3593,
    RISCV_PseudoVLOXEI32_V_MF2_M1	= 3594,
    RISCV_PseudoVLOXEI32_V_MF2_M1_MASK	= 3595,
    RISCV_PseudoVLOXEI32_V_MF2_MF2	= 3596,
    RISCV_PseudoVLOXEI32_V_MF2_MF2_MASK	= 3597,
    RISCV_PseudoVLOXEI32_V_MF2_MF4	= 3598,
    RISCV_PseudoVLOXEI32_V_MF2_MF4_MASK	= 3599,
    RISCV_PseudoVLOXEI32_V_MF2_MF8	= 3600,
    RISCV_PseudoVLOXEI32_V_MF2_MF8_MASK	= 3601,
    RISCV_PseudoVLOXEI64_V_M1_M1	= 3602,
    RISCV_PseudoVLOXEI64_V_M1_M1_MASK	= 3603,
    RISCV_PseudoVLOXEI64_V_M1_MF2	= 3604,
    RISCV_PseudoVLOXEI64_V_M1_MF2_MASK	= 3605,
    RISCV_PseudoVLOXEI64_V_M1_MF4	= 3606,
    RISCV_PseudoVLOXEI64_V_M1_MF4_MASK	= 3607,
    RISCV_PseudoVLOXEI64_V_M1_MF8	= 3608,
    RISCV_PseudoVLOXEI64_V_M1_MF8_MASK	= 3609,
    RISCV_PseudoVLOXEI64_V_M2_M1	= 3610,
    RISCV_PseudoVLOXEI64_V_M2_M1_MASK	= 3611,
    RISCV_PseudoVLOXEI64_V_M2_M2	= 3612,
    RISCV_PseudoVLOXEI64_V_M2_M2_MASK	= 3613,
    RISCV_PseudoVLOXEI64_V_M2_MF2	= 3614,
    RISCV_PseudoVLOXEI64_V_M2_MF2_MASK	= 3615,
    RISCV_PseudoVLOXEI64_V_M2_MF4	= 3616,
    RISCV_PseudoVLOXEI64_V_M2_MF4_MASK	= 3617,
    RISCV_PseudoVLOXEI64_V_M4_M1	= 3618,
    RISCV_PseudoVLOXEI64_V_M4_M1_MASK	= 3619,
    RISCV_PseudoVLOXEI64_V_M4_M2	= 3620,
    RISCV_PseudoVLOXEI64_V_M4_M2_MASK	= 3621,
    RISCV_PseudoVLOXEI64_V_M4_M4	= 3622,
    RISCV_PseudoVLOXEI64_V_M4_M4_MASK	= 3623,
    RISCV_PseudoVLOXEI64_V_M4_MF2	= 3624,
    RISCV_PseudoVLOXEI64_V_M4_MF2_MASK	= 3625,
    RISCV_PseudoVLOXEI64_V_M8_M1	= 3626,
    RISCV_PseudoVLOXEI64_V_M8_M1_MASK	= 3627,
    RISCV_PseudoVLOXEI64_V_M8_M2	= 3628,
    RISCV_PseudoVLOXEI64_V_M8_M2_MASK	= 3629,
    RISCV_PseudoVLOXEI64_V_M8_M4	= 3630,
    RISCV_PseudoVLOXEI64_V_M8_M4_MASK	= 3631,
    RISCV_PseudoVLOXEI64_V_M8_M8	= 3632,
    RISCV_PseudoVLOXEI64_V_M8_M8_MASK	= 3633,
    RISCV_PseudoVLOXEI8_V_M1_M1	= 3634,
    RISCV_PseudoVLOXEI8_V_M1_M1_MASK	= 3635,
    RISCV_PseudoVLOXEI8_V_M1_M2	= 3636,
    RISCV_PseudoVLOXEI8_V_M1_M2_MASK	= 3637,
    RISCV_PseudoVLOXEI8_V_M1_M4	= 3638,
    RISCV_PseudoVLOXEI8_V_M1_M4_MASK	= 3639,
    RISCV_PseudoVLOXEI8_V_M1_M8	= 3640,
    RISCV_PseudoVLOXEI8_V_M1_M8_MASK	= 3641,
    RISCV_PseudoVLOXEI8_V_M2_M2	= 3642,
    RISCV_PseudoVLOXEI8_V_M2_M2_MASK	= 3643,
    RISCV_PseudoVLOXEI8_V_M2_M4	= 3644,
    RISCV_PseudoVLOXEI8_V_M2_M4_MASK	= 3645,
    RISCV_PseudoVLOXEI8_V_M2_M8	= 3646,
    RISCV_PseudoVLOXEI8_V_M2_M8_MASK	= 3647,
    RISCV_PseudoVLOXEI8_V_M4_M4	= 3648,
    RISCV_PseudoVLOXEI8_V_M4_M4_MASK	= 3649,
    RISCV_PseudoVLOXEI8_V_M4_M8	= 3650,
    RISCV_PseudoVLOXEI8_V_M4_M8_MASK	= 3651,
    RISCV_PseudoVLOXEI8_V_M8_M8	= 3652,
    RISCV_PseudoVLOXEI8_V_M8_M8_MASK	= 3653,
    RISCV_PseudoVLOXEI8_V_MF2_M1	= 3654,
    RISCV_PseudoVLOXEI8_V_MF2_M1_MASK	= 3655,
    RISCV_PseudoVLOXEI8_V_MF2_M2	= 3656,
    RISCV_PseudoVLOXEI8_V_MF2_M2_MASK	= 3657,
    RISCV_PseudoVLOXEI8_V_MF2_M4	= 3658,
    RISCV_PseudoVLOXEI8_V_MF2_M4_MASK	= 3659,
    RISCV_PseudoVLOXEI8_V_MF2_MF2	= 3660,
    RISCV_PseudoVLOXEI8_V_MF2_MF2_MASK	= 3661,
    RISCV_PseudoVLOXEI8_V_MF4_M1	= 3662,
    RISCV_PseudoVLOXEI8_V_MF4_M1_MASK	= 3663,
    RISCV_PseudoVLOXEI8_V_MF4_M2	= 3664,
    RISCV_PseudoVLOXEI8_V_MF4_M2_MASK	= 3665,
    RISCV_PseudoVLOXEI8_V_MF4_MF2	= 3666,
    RISCV_PseudoVLOXEI8_V_MF4_MF2_MASK	= 3667,
    RISCV_PseudoVLOXEI8_V_MF4_MF4	= 3668,
    RISCV_PseudoVLOXEI8_V_MF4_MF4_MASK	= 3669,
    RISCV_PseudoVLOXEI8_V_MF8_M1	= 3670,
    RISCV_PseudoVLOXEI8_V_MF8_M1_MASK	= 3671,
    RISCV_PseudoVLOXEI8_V_MF8_MF2	= 3672,
    RISCV_PseudoVLOXEI8_V_MF8_MF2_MASK	= 3673,
    RISCV_PseudoVLOXEI8_V_MF8_MF4	= 3674,
    RISCV_PseudoVLOXEI8_V_MF8_MF4_MASK	= 3675,
    RISCV_PseudoVLOXEI8_V_MF8_MF8	= 3676,
    RISCV_PseudoVLOXEI8_V_MF8_MF8_MASK	= 3677,
    RISCV_PseudoVLOXSEG2EI16_V_M1_M1	= 3678,
    RISCV_PseudoVLOXSEG2EI16_V_M1_M1_MASK	= 3679,
    RISCV_PseudoVLOXSEG2EI16_V_M1_M2	= 3680,
    RISCV_PseudoVLOXSEG2EI16_V_M1_M2_MASK	= 3681,
    RISCV_PseudoVLOXSEG2EI16_V_M1_M4	= 3682,
    RISCV_PseudoVLOXSEG2EI16_V_M1_M4_MASK	= 3683,
    RISCV_PseudoVLOXSEG2EI16_V_M1_MF2	= 3684,
    RISCV_PseudoVLOXSEG2EI16_V_M1_MF2_MASK	= 3685,
    RISCV_PseudoVLOXSEG2EI16_V_M2_M1	= 3686,
    RISCV_PseudoVLOXSEG2EI16_V_M2_M1_MASK	= 3687,
    RISCV_PseudoVLOXSEG2EI16_V_M2_M2	= 3688,
    RISCV_PseudoVLOXSEG2EI16_V_M2_M2_MASK	= 3689,
    RISCV_PseudoVLOXSEG2EI16_V_M2_M4	= 3690,
    RISCV_PseudoVLOXSEG2EI16_V_M2_M4_MASK	= 3691,
    RISCV_PseudoVLOXSEG2EI16_V_M4_M2	= 3692,
    RISCV_PseudoVLOXSEG2EI16_V_M4_M2_MASK	= 3693,
    RISCV_PseudoVLOXSEG2EI16_V_M4_M4	= 3694,
    RISCV_PseudoVLOXSEG2EI16_V_M4_M4_MASK	= 3695,
    RISCV_PseudoVLOXSEG2EI16_V_M8_M4	= 3696,
    RISCV_PseudoVLOXSEG2EI16_V_M8_M4_MASK	= 3697,
    RISCV_PseudoVLOXSEG2EI16_V_MF2_M1	= 3698,
    RISCV_PseudoVLOXSEG2EI16_V_MF2_M1_MASK	= 3699,
    RISCV_PseudoVLOXSEG2EI16_V_MF2_M2	= 3700,
    RISCV_PseudoVLOXSEG2EI16_V_MF2_M2_MASK	= 3701,
    RISCV_PseudoVLOXSEG2EI16_V_MF2_MF2	= 3702,
    RISCV_PseudoVLOXSEG2EI16_V_MF2_MF2_MASK	= 3703,
    RISCV_PseudoVLOXSEG2EI16_V_MF2_MF4	= 3704,
    RISCV_PseudoVLOXSEG2EI16_V_MF2_MF4_MASK	= 3705,
    RISCV_PseudoVLOXSEG2EI16_V_MF4_M1	= 3706,
    RISCV_PseudoVLOXSEG2EI16_V_MF4_M1_MASK	= 3707,
    RISCV_PseudoVLOXSEG2EI16_V_MF4_MF2	= 3708,
    RISCV_PseudoVLOXSEG2EI16_V_MF4_MF2_MASK	= 3709,
    RISCV_PseudoVLOXSEG2EI16_V_MF4_MF4	= 3710,
    RISCV_PseudoVLOXSEG2EI16_V_MF4_MF4_MASK	= 3711,
    RISCV_PseudoVLOXSEG2EI16_V_MF4_MF8	= 3712,
    RISCV_PseudoVLOXSEG2EI16_V_MF4_MF8_MASK	= 3713,
    RISCV_PseudoVLOXSEG2EI32_V_M1_M1	= 3714,
    RISCV_PseudoVLOXSEG2EI32_V_M1_M1_MASK	= 3715,
    RISCV_PseudoVLOXSEG2EI32_V_M1_M2	= 3716,
    RISCV_PseudoVLOXSEG2EI32_V_M1_M2_MASK	= 3717,
    RISCV_PseudoVLOXSEG2EI32_V_M1_MF2	= 3718,
    RISCV_PseudoVLOXSEG2EI32_V_M1_MF2_MASK	= 3719,
    RISCV_PseudoVLOXSEG2EI32_V_M1_MF4	= 3720,
    RISCV_PseudoVLOXSEG2EI32_V_M1_MF4_MASK	= 3721,
    RISCV_PseudoVLOXSEG2EI32_V_M2_M1	= 3722,
    RISCV_PseudoVLOXSEG2EI32_V_M2_M1_MASK	= 3723,
    RISCV_PseudoVLOXSEG2EI32_V_M2_M2	= 3724,
    RISCV_PseudoVLOXSEG2EI32_V_M2_M2_MASK	= 3725,
    RISCV_PseudoVLOXSEG2EI32_V_M2_M4	= 3726,
    RISCV_PseudoVLOXSEG2EI32_V_M2_M4_MASK	= 3727,
    RISCV_PseudoVLOXSEG2EI32_V_M2_MF2	= 3728,
    RISCV_PseudoVLOXSEG2EI32_V_M2_MF2_MASK	= 3729,
    RISCV_PseudoVLOXSEG2EI32_V_M4_M1	= 3730,
    RISCV_PseudoVLOXSEG2EI32_V_M4_M1_MASK	= 3731,
    RISCV_PseudoVLOXSEG2EI32_V_M4_M2	= 3732,
    RISCV_PseudoVLOXSEG2EI32_V_M4_M2_MASK	= 3733,
    RISCV_PseudoVLOXSEG2EI32_V_M4_M4	= 3734,
    RISCV_PseudoVLOXSEG2EI32_V_M4_M4_MASK	= 3735,
    RISCV_PseudoVLOXSEG2EI32_V_M8_M2	= 3736,
    RISCV_PseudoVLOXSEG2EI32_V_M8_M2_MASK	= 3737,
    RISCV_PseudoVLOXSEG2EI32_V_M8_M4	= 3738,
    RISCV_PseudoVLOXSEG2EI32_V_M8_M4_MASK	= 3739,
    RISCV_PseudoVLOXSEG2EI32_V_MF2_M1	= 3740,
    RISCV_PseudoVLOXSEG2EI32_V_MF2_M1_MASK	= 3741,
    RISCV_PseudoVLOXSEG2EI32_V_MF2_MF2	= 3742,
    RISCV_PseudoVLOXSEG2EI32_V_MF2_MF2_MASK	= 3743,
    RISCV_PseudoVLOXSEG2EI32_V_MF2_MF4	= 3744,
    RISCV_PseudoVLOXSEG2EI32_V_MF2_MF4_MASK	= 3745,
    RISCV_PseudoVLOXSEG2EI32_V_MF2_MF8	= 3746,
    RISCV_PseudoVLOXSEG2EI32_V_MF2_MF8_MASK	= 3747,
    RISCV_PseudoVLOXSEG2EI64_V_M1_M1	= 3748,
    RISCV_PseudoVLOXSEG2EI64_V_M1_M1_MASK	= 3749,
    RISCV_PseudoVLOXSEG2EI64_V_M1_MF2	= 3750,
    RISCV_PseudoVLOXSEG2EI64_V_M1_MF2_MASK	= 3751,
    RISCV_PseudoVLOXSEG2EI64_V_M1_MF4	= 3752,
    RISCV_PseudoVLOXSEG2EI64_V_M1_MF4_MASK	= 3753,
    RISCV_PseudoVLOXSEG2EI64_V_M1_MF8	= 3754,
    RISCV_PseudoVLOXSEG2EI64_V_M1_MF8_MASK	= 3755,
    RISCV_PseudoVLOXSEG2EI64_V_M2_M1	= 3756,
    RISCV_PseudoVLOXSEG2EI64_V_M2_M1_MASK	= 3757,
    RISCV_PseudoVLOXSEG2EI64_V_M2_M2	= 3758,
    RISCV_PseudoVLOXSEG2EI64_V_M2_M2_MASK	= 3759,
    RISCV_PseudoVLOXSEG2EI64_V_M2_MF2	= 3760,
    RISCV_PseudoVLOXSEG2EI64_V_M2_MF2_MASK	= 3761,
    RISCV_PseudoVLOXSEG2EI64_V_M2_MF4	= 3762,
    RISCV_PseudoVLOXSEG2EI64_V_M2_MF4_MASK	= 3763,
    RISCV_PseudoVLOXSEG2EI64_V_M4_M1	= 3764,
    RISCV_PseudoVLOXSEG2EI64_V_M4_M1_MASK	= 3765,
    RISCV_PseudoVLOXSEG2EI64_V_M4_M2	= 3766,
    RISCV_PseudoVLOXSEG2EI64_V_M4_M2_MASK	= 3767,
    RISCV_PseudoVLOXSEG2EI64_V_M4_M4	= 3768,
    RISCV_PseudoVLOXSEG2EI64_V_M4_M4_MASK	= 3769,
    RISCV_PseudoVLOXSEG2EI64_V_M4_MF2	= 3770,
    RISCV_PseudoVLOXSEG2EI64_V_M4_MF2_MASK	= 3771,
    RISCV_PseudoVLOXSEG2EI64_V_M8_M1	= 3772,
    RISCV_PseudoVLOXSEG2EI64_V_M8_M1_MASK	= 3773,
    RISCV_PseudoVLOXSEG2EI64_V_M8_M2	= 3774,
    RISCV_PseudoVLOXSEG2EI64_V_M8_M2_MASK	= 3775,
    RISCV_PseudoVLOXSEG2EI64_V_M8_M4	= 3776,
    RISCV_PseudoVLOXSEG2EI64_V_M8_M4_MASK	= 3777,
    RISCV_PseudoVLOXSEG2EI8_V_M1_M1	= 3778,
    RISCV_PseudoVLOXSEG2EI8_V_M1_M1_MASK	= 3779,
    RISCV_PseudoVLOXSEG2EI8_V_M1_M2	= 3780,
    RISCV_PseudoVLOXSEG2EI8_V_M1_M2_MASK	= 3781,
    RISCV_PseudoVLOXSEG2EI8_V_M1_M4	= 3782,
    RISCV_PseudoVLOXSEG2EI8_V_M1_M4_MASK	= 3783,
    RISCV_PseudoVLOXSEG2EI8_V_M2_M2	= 3784,
    RISCV_PseudoVLOXSEG2EI8_V_M2_M2_MASK	= 3785,
    RISCV_PseudoVLOXSEG2EI8_V_M2_M4	= 3786,
    RISCV_PseudoVLOXSEG2EI8_V_M2_M4_MASK	= 3787,
    RISCV_PseudoVLOXSEG2EI8_V_M4_M4	= 3788,
    RISCV_PseudoVLOXSEG2EI8_V_M4_M4_MASK	= 3789,
    RISCV_PseudoVLOXSEG2EI8_V_MF2_M1	= 3790,
    RISCV_PseudoVLOXSEG2EI8_V_MF2_M1_MASK	= 3791,
    RISCV_PseudoVLOXSEG2EI8_V_MF2_M2	= 3792,
    RISCV_PseudoVLOXSEG2EI8_V_MF2_M2_MASK	= 3793,
    RISCV_PseudoVLOXSEG2EI8_V_MF2_M4	= 3794,
    RISCV_PseudoVLOXSEG2EI8_V_MF2_M4_MASK	= 3795,
    RISCV_PseudoVLOXSEG2EI8_V_MF2_MF2	= 3796,
    RISCV_PseudoVLOXSEG2EI8_V_MF2_MF2_MASK	= 3797,
    RISCV_PseudoVLOXSEG2EI8_V_MF4_M1	= 3798,
    RISCV_PseudoVLOXSEG2EI8_V_MF4_M1_MASK	= 3799,
    RISCV_PseudoVLOXSEG2EI8_V_MF4_M2	= 3800,
    RISCV_PseudoVLOXSEG2EI8_V_MF4_M2_MASK	= 3801,
    RISCV_PseudoVLOXSEG2EI8_V_MF4_MF2	= 3802,
    RISCV_PseudoVLOXSEG2EI8_V_MF4_MF2_MASK	= 3803,
    RISCV_PseudoVLOXSEG2EI8_V_MF4_MF4	= 3804,
    RISCV_PseudoVLOXSEG2EI8_V_MF4_MF4_MASK	= 3805,
    RISCV_PseudoVLOXSEG2EI8_V_MF8_M1	= 3806,
    RISCV_PseudoVLOXSEG2EI8_V_MF8_M1_MASK	= 3807,
    RISCV_PseudoVLOXSEG2EI8_V_MF8_MF2	= 3808,
    RISCV_PseudoVLOXSEG2EI8_V_MF8_MF2_MASK	= 3809,
    RISCV_PseudoVLOXSEG2EI8_V_MF8_MF4	= 3810,
    RISCV_PseudoVLOXSEG2EI8_V_MF8_MF4_MASK	= 3811,
    RISCV_PseudoVLOXSEG2EI8_V_MF8_MF8	= 3812,
    RISCV_PseudoVLOXSEG2EI8_V_MF8_MF8_MASK	= 3813,
    RISCV_PseudoVLOXSEG3EI16_V_M1_M1	= 3814,
    RISCV_PseudoVLOXSEG3EI16_V_M1_M1_MASK	= 3815,
    RISCV_PseudoVLOXSEG3EI16_V_M1_M2	= 3816,
    RISCV_PseudoVLOXSEG3EI16_V_M1_M2_MASK	= 3817,
    RISCV_PseudoVLOXSEG3EI16_V_M1_MF2	= 3818,
    RISCV_PseudoVLOXSEG3EI16_V_M1_MF2_MASK	= 3819,
    RISCV_PseudoVLOXSEG3EI16_V_M2_M1	= 3820,
    RISCV_PseudoVLOXSEG3EI16_V_M2_M1_MASK	= 3821,
    RISCV_PseudoVLOXSEG3EI16_V_M2_M2	= 3822,
    RISCV_PseudoVLOXSEG3EI16_V_M2_M2_MASK	= 3823,
    RISCV_PseudoVLOXSEG3EI16_V_M4_M2	= 3824,
    RISCV_PseudoVLOXSEG3EI16_V_M4_M2_MASK	= 3825,
    RISCV_PseudoVLOXSEG3EI16_V_MF2_M1	= 3826,
    RISCV_PseudoVLOXSEG3EI16_V_MF2_M1_MASK	= 3827,
    RISCV_PseudoVLOXSEG3EI16_V_MF2_M2	= 3828,
    RISCV_PseudoVLOXSEG3EI16_V_MF2_M2_MASK	= 3829,
    RISCV_PseudoVLOXSEG3EI16_V_MF2_MF2	= 3830,
    RISCV_PseudoVLOXSEG3EI16_V_MF2_MF2_MASK	= 3831,
    RISCV_PseudoVLOXSEG3EI16_V_MF2_MF4	= 3832,
    RISCV_PseudoVLOXSEG3EI16_V_MF2_MF4_MASK	= 3833,
    RISCV_PseudoVLOXSEG3EI16_V_MF4_M1	= 3834,
    RISCV_PseudoVLOXSEG3EI16_V_MF4_M1_MASK	= 3835,
    RISCV_PseudoVLOXSEG3EI16_V_MF4_MF2	= 3836,
    RISCV_PseudoVLOXSEG3EI16_V_MF4_MF2_MASK	= 3837,
    RISCV_PseudoVLOXSEG3EI16_V_MF4_MF4	= 3838,
    RISCV_PseudoVLOXSEG3EI16_V_MF4_MF4_MASK	= 3839,
    RISCV_PseudoVLOXSEG3EI16_V_MF4_MF8	= 3840,
    RISCV_PseudoVLOXSEG3EI16_V_MF4_MF8_MASK	= 3841,
    RISCV_PseudoVLOXSEG3EI32_V_M1_M1	= 3842,
    RISCV_PseudoVLOXSEG3EI32_V_M1_M1_MASK	= 3843,
    RISCV_PseudoVLOXSEG3EI32_V_M1_M2	= 3844,
    RISCV_PseudoVLOXSEG3EI32_V_M1_M2_MASK	= 3845,
    RISCV_PseudoVLOXSEG3EI32_V_M1_MF2	= 3846,
    RISCV_PseudoVLOXSEG3EI32_V_M1_MF2_MASK	= 3847,
    RISCV_PseudoVLOXSEG3EI32_V_M1_MF4	= 3848,
    RISCV_PseudoVLOXSEG3EI32_V_M1_MF4_MASK	= 3849,
    RISCV_PseudoVLOXSEG3EI32_V_M2_M1	= 3850,
    RISCV_PseudoVLOXSEG3EI32_V_M2_M1_MASK	= 3851,
    RISCV_PseudoVLOXSEG3EI32_V_M2_M2	= 3852,
    RISCV_PseudoVLOXSEG3EI32_V_M2_M2_MASK	= 3853,
    RISCV_PseudoVLOXSEG3EI32_V_M2_MF2	= 3854,
    RISCV_PseudoVLOXSEG3EI32_V_M2_MF2_MASK	= 3855,
    RISCV_PseudoVLOXSEG3EI32_V_M4_M1	= 3856,
    RISCV_PseudoVLOXSEG3EI32_V_M4_M1_MASK	= 3857,
    RISCV_PseudoVLOXSEG3EI32_V_M4_M2	= 3858,
    RISCV_PseudoVLOXSEG3EI32_V_M4_M2_MASK	= 3859,
    RISCV_PseudoVLOXSEG3EI32_V_M8_M2	= 3860,
    RISCV_PseudoVLOXSEG3EI32_V_M8_M2_MASK	= 3861,
    RISCV_PseudoVLOXSEG3EI32_V_MF2_M1	= 3862,
    RISCV_PseudoVLOXSEG3EI32_V_MF2_M1_MASK	= 3863,
    RISCV_PseudoVLOXSEG3EI32_V_MF2_MF2	= 3864,
    RISCV_PseudoVLOXSEG3EI32_V_MF2_MF2_MASK	= 3865,
    RISCV_PseudoVLOXSEG3EI32_V_MF2_MF4	= 3866,
    RISCV_PseudoVLOXSEG3EI32_V_MF2_MF4_MASK	= 3867,
    RISCV_PseudoVLOXSEG3EI32_V_MF2_MF8	= 3868,
    RISCV_PseudoVLOXSEG3EI32_V_MF2_MF8_MASK	= 3869,
    RISCV_PseudoVLOXSEG3EI64_V_M1_M1	= 3870,
    RISCV_PseudoVLOXSEG3EI64_V_M1_M1_MASK	= 3871,
    RISCV_PseudoVLOXSEG3EI64_V_M1_MF2	= 3872,
    RISCV_PseudoVLOXSEG3EI64_V_M1_MF2_MASK	= 3873,
    RISCV_PseudoVLOXSEG3EI64_V_M1_MF4	= 3874,
    RISCV_PseudoVLOXSEG3EI64_V_M1_MF4_MASK	= 3875,
    RISCV_PseudoVLOXSEG3EI64_V_M1_MF8	= 3876,
    RISCV_PseudoVLOXSEG3EI64_V_M1_MF8_MASK	= 3877,
    RISCV_PseudoVLOXSEG3EI64_V_M2_M1	= 3878,
    RISCV_PseudoVLOXSEG3EI64_V_M2_M1_MASK	= 3879,
    RISCV_PseudoVLOXSEG3EI64_V_M2_M2	= 3880,
    RISCV_PseudoVLOXSEG3EI64_V_M2_M2_MASK	= 3881,
    RISCV_PseudoVLOXSEG3EI64_V_M2_MF2	= 3882,
    RISCV_PseudoVLOXSEG3EI64_V_M2_MF2_MASK	= 3883,
    RISCV_PseudoVLOXSEG3EI64_V_M2_MF4	= 3884,
    RISCV_PseudoVLOXSEG3EI64_V_M2_MF4_MASK	= 3885,
    RISCV_PseudoVLOXSEG3EI64_V_M4_M1	= 3886,
    RISCV_PseudoVLOXSEG3EI64_V_M4_M1_MASK	= 3887,
    RISCV_PseudoVLOXSEG3EI64_V_M4_M2	= 3888,
    RISCV_PseudoVLOXSEG3EI64_V_M4_M2_MASK	= 3889,
    RISCV_PseudoVLOXSEG3EI64_V_M4_MF2	= 3890,
    RISCV_PseudoVLOXSEG3EI64_V_M4_MF2_MASK	= 3891,
    RISCV_PseudoVLOXSEG3EI64_V_M8_M1	= 3892,
    RISCV_PseudoVLOXSEG3EI64_V_M8_M1_MASK	= 3893,
    RISCV_PseudoVLOXSEG3EI64_V_M8_M2	= 3894,
    RISCV_PseudoVLOXSEG3EI64_V_M8_M2_MASK	= 3895,
    RISCV_PseudoVLOXSEG3EI8_V_M1_M1	= 3896,
    RISCV_PseudoVLOXSEG3EI8_V_M1_M1_MASK	= 3897,
    RISCV_PseudoVLOXSEG3EI8_V_M1_M2	= 3898,
    RISCV_PseudoVLOXSEG3EI8_V_M1_M2_MASK	= 3899,
    RISCV_PseudoVLOXSEG3EI8_V_M2_M2	= 3900,
    RISCV_PseudoVLOXSEG3EI8_V_M2_M2_MASK	= 3901,
    RISCV_PseudoVLOXSEG3EI8_V_MF2_M1	= 3902,
    RISCV_PseudoVLOXSEG3EI8_V_MF2_M1_MASK	= 3903,
    RISCV_PseudoVLOXSEG3EI8_V_MF2_M2	= 3904,
    RISCV_PseudoVLOXSEG3EI8_V_MF2_M2_MASK	= 3905,
    RISCV_PseudoVLOXSEG3EI8_V_MF2_MF2	= 3906,
    RISCV_PseudoVLOXSEG3EI8_V_MF2_MF2_MASK	= 3907,
    RISCV_PseudoVLOXSEG3EI8_V_MF4_M1	= 3908,
    RISCV_PseudoVLOXSEG3EI8_V_MF4_M1_MASK	= 3909,
    RISCV_PseudoVLOXSEG3EI8_V_MF4_M2	= 3910,
    RISCV_PseudoVLOXSEG3EI8_V_MF4_M2_MASK	= 3911,
    RISCV_PseudoVLOXSEG3EI8_V_MF4_MF2	= 3912,
    RISCV_PseudoVLOXSEG3EI8_V_MF4_MF2_MASK	= 3913,
    RISCV_PseudoVLOXSEG3EI8_V_MF4_MF4	= 3914,
    RISCV_PseudoVLOXSEG3EI8_V_MF4_MF4_MASK	= 3915,
    RISCV_PseudoVLOXSEG3EI8_V_MF8_M1	= 3916,
    RISCV_PseudoVLOXSEG3EI8_V_MF8_M1_MASK	= 3917,
    RISCV_PseudoVLOXSEG3EI8_V_MF8_MF2	= 3918,
    RISCV_PseudoVLOXSEG3EI8_V_MF8_MF2_MASK	= 3919,
    RISCV_PseudoVLOXSEG3EI8_V_MF8_MF4	= 3920,
    RISCV_PseudoVLOXSEG3EI8_V_MF8_MF4_MASK	= 3921,
    RISCV_PseudoVLOXSEG3EI8_V_MF8_MF8	= 3922,
    RISCV_PseudoVLOXSEG3EI8_V_MF8_MF8_MASK	= 3923,
    RISCV_PseudoVLOXSEG4EI16_V_M1_M1	= 3924,
    RISCV_PseudoVLOXSEG4EI16_V_M1_M1_MASK	= 3925,
    RISCV_PseudoVLOXSEG4EI16_V_M1_M2	= 3926,
    RISCV_PseudoVLOXSEG4EI16_V_M1_M2_MASK	= 3927,
    RISCV_PseudoVLOXSEG4EI16_V_M1_MF2	= 3928,
    RISCV_PseudoVLOXSEG4EI16_V_M1_MF2_MASK	= 3929,
    RISCV_PseudoVLOXSEG4EI16_V_M2_M1	= 3930,
    RISCV_PseudoVLOXSEG4EI16_V_M2_M1_MASK	= 3931,
    RISCV_PseudoVLOXSEG4EI16_V_M2_M2	= 3932,
    RISCV_PseudoVLOXSEG4EI16_V_M2_M2_MASK	= 3933,
    RISCV_PseudoVLOXSEG4EI16_V_M4_M2	= 3934,
    RISCV_PseudoVLOXSEG4EI16_V_M4_M2_MASK	= 3935,
    RISCV_PseudoVLOXSEG4EI16_V_MF2_M1	= 3936,
    RISCV_PseudoVLOXSEG4EI16_V_MF2_M1_MASK	= 3937,
    RISCV_PseudoVLOXSEG4EI16_V_MF2_M2	= 3938,
    RISCV_PseudoVLOXSEG4EI16_V_MF2_M2_MASK	= 3939,
    RISCV_PseudoVLOXSEG4EI16_V_MF2_MF2	= 3940,
    RISCV_PseudoVLOXSEG4EI16_V_MF2_MF2_MASK	= 3941,
    RISCV_PseudoVLOXSEG4EI16_V_MF2_MF4	= 3942,
    RISCV_PseudoVLOXSEG4EI16_V_MF2_MF4_MASK	= 3943,
    RISCV_PseudoVLOXSEG4EI16_V_MF4_M1	= 3944,
    RISCV_PseudoVLOXSEG4EI16_V_MF4_M1_MASK	= 3945,
    RISCV_PseudoVLOXSEG4EI16_V_MF4_MF2	= 3946,
    RISCV_PseudoVLOXSEG4EI16_V_MF4_MF2_MASK	= 3947,
    RISCV_PseudoVLOXSEG4EI16_V_MF4_MF4	= 3948,
    RISCV_PseudoVLOXSEG4EI16_V_MF4_MF4_MASK	= 3949,
    RISCV_PseudoVLOXSEG4EI16_V_MF4_MF8	= 3950,
    RISCV_PseudoVLOXSEG4EI16_V_MF4_MF8_MASK	= 3951,
    RISCV_PseudoVLOXSEG4EI32_V_M1_M1	= 3952,
    RISCV_PseudoVLOXSEG4EI32_V_M1_M1_MASK	= 3953,
    RISCV_PseudoVLOXSEG4EI32_V_M1_M2	= 3954,
    RISCV_PseudoVLOXSEG4EI32_V_M1_M2_MASK	= 3955,
    RISCV_PseudoVLOXSEG4EI32_V_M1_MF2	= 3956,
    RISCV_PseudoVLOXSEG4EI32_V_M1_MF2_MASK	= 3957,
    RISCV_PseudoVLOXSEG4EI32_V_M1_MF4	= 3958,
    RISCV_PseudoVLOXSEG4EI32_V_M1_MF4_MASK	= 3959,
    RISCV_PseudoVLOXSEG4EI32_V_M2_M1	= 3960,
    RISCV_PseudoVLOXSEG4EI32_V_M2_M1_MASK	= 3961,
    RISCV_PseudoVLOXSEG4EI32_V_M2_M2	= 3962,
    RISCV_PseudoVLOXSEG4EI32_V_M2_M2_MASK	= 3963,
    RISCV_PseudoVLOXSEG4EI32_V_M2_MF2	= 3964,
    RISCV_PseudoVLOXSEG4EI32_V_M2_MF2_MASK	= 3965,
    RISCV_PseudoVLOXSEG4EI32_V_M4_M1	= 3966,
    RISCV_PseudoVLOXSEG4EI32_V_M4_M1_MASK	= 3967,
    RISCV_PseudoVLOXSEG4EI32_V_M4_M2	= 3968,
    RISCV_PseudoVLOXSEG4EI32_V_M4_M2_MASK	= 3969,
    RISCV_PseudoVLOXSEG4EI32_V_M8_M2	= 3970,
    RISCV_PseudoVLOXSEG4EI32_V_M8_M2_MASK	= 3971,
    RISCV_PseudoVLOXSEG4EI32_V_MF2_M1	= 3972,
    RISCV_PseudoVLOXSEG4EI32_V_MF2_M1_MASK	= 3973,
    RISCV_PseudoVLOXSEG4EI32_V_MF2_MF2	= 3974,
    RISCV_PseudoVLOXSEG4EI32_V_MF2_MF2_MASK	= 3975,
    RISCV_PseudoVLOXSEG4EI32_V_MF2_MF4	= 3976,
    RISCV_PseudoVLOXSEG4EI32_V_MF2_MF4_MASK	= 3977,
    RISCV_PseudoVLOXSEG4EI32_V_MF2_MF8	= 3978,
    RISCV_PseudoVLOXSEG4EI32_V_MF2_MF8_MASK	= 3979,
    RISCV_PseudoVLOXSEG4EI64_V_M1_M1	= 3980,
    RISCV_PseudoVLOXSEG4EI64_V_M1_M1_MASK	= 3981,
    RISCV_PseudoVLOXSEG4EI64_V_M1_MF2	= 3982,
    RISCV_PseudoVLOXSEG4EI64_V_M1_MF2_MASK	= 3983,
    RISCV_PseudoVLOXSEG4EI64_V_M1_MF4	= 3984,
    RISCV_PseudoVLOXSEG4EI64_V_M1_MF4_MASK	= 3985,
    RISCV_PseudoVLOXSEG4EI64_V_M1_MF8	= 3986,
    RISCV_PseudoVLOXSEG4EI64_V_M1_MF8_MASK	= 3987,
    RISCV_PseudoVLOXSEG4EI64_V_M2_M1	= 3988,
    RISCV_PseudoVLOXSEG4EI64_V_M2_M1_MASK	= 3989,
    RISCV_PseudoVLOXSEG4EI64_V_M2_M2	= 3990,
    RISCV_PseudoVLOXSEG4EI64_V_M2_M2_MASK	= 3991,
    RISCV_PseudoVLOXSEG4EI64_V_M2_MF2	= 3992,
    RISCV_PseudoVLOXSEG4EI64_V_M2_MF2_MASK	= 3993,
    RISCV_PseudoVLOXSEG4EI64_V_M2_MF4	= 3994,
    RISCV_PseudoVLOXSEG4EI64_V_M2_MF4_MASK	= 3995,
    RISCV_PseudoVLOXSEG4EI64_V_M4_M1	= 3996,
    RISCV_PseudoVLOXSEG4EI64_V_M4_M1_MASK	= 3997,
    RISCV_PseudoVLOXSEG4EI64_V_M4_M2	= 3998,
    RISCV_PseudoVLOXSEG4EI64_V_M4_M2_MASK	= 3999,
    RISCV_PseudoVLOXSEG4EI64_V_M4_MF2	= 4000,
    RISCV_PseudoVLOXSEG4EI64_V_M4_MF2_MASK	= 4001,
    RISCV_PseudoVLOXSEG4EI64_V_M8_M1	= 4002,
    RISCV_PseudoVLOXSEG4EI64_V_M8_M1_MASK	= 4003,
    RISCV_PseudoVLOXSEG4EI64_V_M8_M2	= 4004,
    RISCV_PseudoVLOXSEG4EI64_V_M8_M2_MASK	= 4005,
    RISCV_PseudoVLOXSEG4EI8_V_M1_M1	= 4006,
    RISCV_PseudoVLOXSEG4EI8_V_M1_M1_MASK	= 4007,
    RISCV_PseudoVLOXSEG4EI8_V_M1_M2	= 4008,
    RISCV_PseudoVLOXSEG4EI8_V_M1_M2_MASK	= 4009,
    RISCV_PseudoVLOXSEG4EI8_V_M2_M2	= 4010,
    RISCV_PseudoVLOXSEG4EI8_V_M2_M2_MASK	= 4011,
    RISCV_PseudoVLOXSEG4EI8_V_MF2_M1	= 4012,
    RISCV_PseudoVLOXSEG4EI8_V_MF2_M1_MASK	= 4013,
    RISCV_PseudoVLOXSEG4EI8_V_MF2_M2	= 4014,
    RISCV_PseudoVLOXSEG4EI8_V_MF2_M2_MASK	= 4015,
    RISCV_PseudoVLOXSEG4EI8_V_MF2_MF2	= 4016,
    RISCV_PseudoVLOXSEG4EI8_V_MF2_MF2_MASK	= 4017,
    RISCV_PseudoVLOXSEG4EI8_V_MF4_M1	= 4018,
    RISCV_PseudoVLOXSEG4EI8_V_MF4_M1_MASK	= 4019,
    RISCV_PseudoVLOXSEG4EI8_V_MF4_M2	= 4020,
    RISCV_PseudoVLOXSEG4EI8_V_MF4_M2_MASK	= 4021,
    RISCV_PseudoVLOXSEG4EI8_V_MF4_MF2	= 4022,
    RISCV_PseudoVLOXSEG4EI8_V_MF4_MF2_MASK	= 4023,
    RISCV_PseudoVLOXSEG4EI8_V_MF4_MF4	= 4024,
    RISCV_PseudoVLOXSEG4EI8_V_MF4_MF4_MASK	= 4025,
    RISCV_PseudoVLOXSEG4EI8_V_MF8_M1	= 4026,
    RISCV_PseudoVLOXSEG4EI8_V_MF8_M1_MASK	= 4027,
    RISCV_PseudoVLOXSEG4EI8_V_MF8_MF2	= 4028,
    RISCV_PseudoVLOXSEG4EI8_V_MF8_MF2_MASK	= 4029,
    RISCV_PseudoVLOXSEG4EI8_V_MF8_MF4	= 4030,
    RISCV_PseudoVLOXSEG4EI8_V_MF8_MF4_MASK	= 4031,
    RISCV_PseudoVLOXSEG4EI8_V_MF8_MF8	= 4032,
    RISCV_PseudoVLOXSEG4EI8_V_MF8_MF8_MASK	= 4033,
    RISCV_PseudoVLOXSEG5EI16_V_M1_M1	= 4034,
    RISCV_PseudoVLOXSEG5EI16_V_M1_M1_MASK	= 4035,
    RISCV_PseudoVLOXSEG5EI16_V_M1_MF2	= 4036,
    RISCV_PseudoVLOXSEG5EI16_V_M1_MF2_MASK	= 4037,
    RISCV_PseudoVLOXSEG5EI16_V_M2_M1	= 4038,
    RISCV_PseudoVLOXSEG5EI16_V_M2_M1_MASK	= 4039,
    RISCV_PseudoVLOXSEG5EI16_V_MF2_M1	= 4040,
    RISCV_PseudoVLOXSEG5EI16_V_MF2_M1_MASK	= 4041,
    RISCV_PseudoVLOXSEG5EI16_V_MF2_MF2	= 4042,
    RISCV_PseudoVLOXSEG5EI16_V_MF2_MF2_MASK	= 4043,
    RISCV_PseudoVLOXSEG5EI16_V_MF2_MF4	= 4044,
    RISCV_PseudoVLOXSEG5EI16_V_MF2_MF4_MASK	= 4045,
    RISCV_PseudoVLOXSEG5EI16_V_MF4_M1	= 4046,
    RISCV_PseudoVLOXSEG5EI16_V_MF4_M1_MASK	= 4047,
    RISCV_PseudoVLOXSEG5EI16_V_MF4_MF2	= 4048,
    RISCV_PseudoVLOXSEG5EI16_V_MF4_MF2_MASK	= 4049,
    RISCV_PseudoVLOXSEG5EI16_V_MF4_MF4	= 4050,
    RISCV_PseudoVLOXSEG5EI16_V_MF4_MF4_MASK	= 4051,
    RISCV_PseudoVLOXSEG5EI16_V_MF4_MF8	= 4052,
    RISCV_PseudoVLOXSEG5EI16_V_MF4_MF8_MASK	= 4053,
    RISCV_PseudoVLOXSEG5EI32_V_M1_M1	= 4054,
    RISCV_PseudoVLOXSEG5EI32_V_M1_M1_MASK	= 4055,
    RISCV_PseudoVLOXSEG5EI32_V_M1_MF2	= 4056,
    RISCV_PseudoVLOXSEG5EI32_V_M1_MF2_MASK	= 4057,
    RISCV_PseudoVLOXSEG5EI32_V_M1_MF4	= 4058,
    RISCV_PseudoVLOXSEG5EI32_V_M1_MF4_MASK	= 4059,
    RISCV_PseudoVLOXSEG5EI32_V_M2_M1	= 4060,
    RISCV_PseudoVLOXSEG5EI32_V_M2_M1_MASK	= 4061,
    RISCV_PseudoVLOXSEG5EI32_V_M2_MF2	= 4062,
    RISCV_PseudoVLOXSEG5EI32_V_M2_MF2_MASK	= 4063,
    RISCV_PseudoVLOXSEG5EI32_V_M4_M1	= 4064,
    RISCV_PseudoVLOXSEG5EI32_V_M4_M1_MASK	= 4065,
    RISCV_PseudoVLOXSEG5EI32_V_MF2_M1	= 4066,
    RISCV_PseudoVLOXSEG5EI32_V_MF2_M1_MASK	= 4067,
    RISCV_PseudoVLOXSEG5EI32_V_MF2_MF2	= 4068,
    RISCV_PseudoVLOXSEG5EI32_V_MF2_MF2_MASK	= 4069,
    RISCV_PseudoVLOXSEG5EI32_V_MF2_MF4	= 4070,
    RISCV_PseudoVLOXSEG5EI32_V_MF2_MF4_MASK	= 4071,
    RISCV_PseudoVLOXSEG5EI32_V_MF2_MF8	= 4072,
    RISCV_PseudoVLOXSEG5EI32_V_MF2_MF8_MASK	= 4073,
    RISCV_PseudoVLOXSEG5EI64_V_M1_M1	= 4074,
    RISCV_PseudoVLOXSEG5EI64_V_M1_M1_MASK	= 4075,
    RISCV_PseudoVLOXSEG5EI64_V_M1_MF2	= 4076,
    RISCV_PseudoVLOXSEG5EI64_V_M1_MF2_MASK	= 4077,
    RISCV_PseudoVLOXSEG5EI64_V_M1_MF4	= 4078,
    RISCV_PseudoVLOXSEG5EI64_V_M1_MF4_MASK	= 4079,
    RISCV_PseudoVLOXSEG5EI64_V_M1_MF8	= 4080,
    RISCV_PseudoVLOXSEG5EI64_V_M1_MF8_MASK	= 4081,
    RISCV_PseudoVLOXSEG5EI64_V_M2_M1	= 4082,
    RISCV_PseudoVLOXSEG5EI64_V_M2_M1_MASK	= 4083,
    RISCV_PseudoVLOXSEG5EI64_V_M2_MF2	= 4084,
    RISCV_PseudoVLOXSEG5EI64_V_M2_MF2_MASK	= 4085,
    RISCV_PseudoVLOXSEG5EI64_V_M2_MF4	= 4086,
    RISCV_PseudoVLOXSEG5EI64_V_M2_MF4_MASK	= 4087,
    RISCV_PseudoVLOXSEG5EI64_V_M4_M1	= 4088,
    RISCV_PseudoVLOXSEG5EI64_V_M4_M1_MASK	= 4089,
    RISCV_PseudoVLOXSEG5EI64_V_M4_MF2	= 4090,
    RISCV_PseudoVLOXSEG5EI64_V_M4_MF2_MASK	= 4091,
    RISCV_PseudoVLOXSEG5EI64_V_M8_M1	= 4092,
    RISCV_PseudoVLOXSEG5EI64_V_M8_M1_MASK	= 4093,
    RISCV_PseudoVLOXSEG5EI8_V_M1_M1	= 4094,
    RISCV_PseudoVLOXSEG5EI8_V_M1_M1_MASK	= 4095,
    RISCV_PseudoVLOXSEG5EI8_V_MF2_M1	= 4096,
    RISCV_PseudoVLOXSEG5EI8_V_MF2_M1_MASK	= 4097,
    RISCV_PseudoVLOXSEG5EI8_V_MF2_MF2	= 4098,
    RISCV_PseudoVLOXSEG5EI8_V_MF2_MF2_MASK	= 4099,
    RISCV_PseudoVLOXSEG5EI8_V_MF4_M1	= 4100,
    RISCV_PseudoVLOXSEG5EI8_V_MF4_M1_MASK	= 4101,
    RISCV_PseudoVLOXSEG5EI8_V_MF4_MF2	= 4102,
    RISCV_PseudoVLOXSEG5EI8_V_MF4_MF2_MASK	= 4103,
    RISCV_PseudoVLOXSEG5EI8_V_MF4_MF4	= 4104,
    RISCV_PseudoVLOXSEG5EI8_V_MF4_MF4_MASK	= 4105,
    RISCV_PseudoVLOXSEG5EI8_V_MF8_M1	= 4106,
    RISCV_PseudoVLOXSEG5EI8_V_MF8_M1_MASK	= 4107,
    RISCV_PseudoVLOXSEG5EI8_V_MF8_MF2	= 4108,
    RISCV_PseudoVLOXSEG5EI8_V_MF8_MF2_MASK	= 4109,
    RISCV_PseudoVLOXSEG5EI8_V_MF8_MF4	= 4110,
    RISCV_PseudoVLOXSEG5EI8_V_MF8_MF4_MASK	= 4111,
    RISCV_PseudoVLOXSEG5EI8_V_MF8_MF8	= 4112,
    RISCV_PseudoVLOXSEG5EI8_V_MF8_MF8_MASK	= 4113,
    RISCV_PseudoVLOXSEG6EI16_V_M1_M1	= 4114,
    RISCV_PseudoVLOXSEG6EI16_V_M1_M1_MASK	= 4115,
    RISCV_PseudoVLOXSEG6EI16_V_M1_MF2	= 4116,
    RISCV_PseudoVLOXSEG6EI16_V_M1_MF2_MASK	= 4117,
    RISCV_PseudoVLOXSEG6EI16_V_M2_M1	= 4118,
    RISCV_PseudoVLOXSEG6EI16_V_M2_M1_MASK	= 4119,
    RISCV_PseudoVLOXSEG6EI16_V_MF2_M1	= 4120,
    RISCV_PseudoVLOXSEG6EI16_V_MF2_M1_MASK	= 4121,
    RISCV_PseudoVLOXSEG6EI16_V_MF2_MF2	= 4122,
    RISCV_PseudoVLOXSEG6EI16_V_MF2_MF2_MASK	= 4123,
    RISCV_PseudoVLOXSEG6EI16_V_MF2_MF4	= 4124,
    RISCV_PseudoVLOXSEG6EI16_V_MF2_MF4_MASK	= 4125,
    RISCV_PseudoVLOXSEG6EI16_V_MF4_M1	= 4126,
    RISCV_PseudoVLOXSEG6EI16_V_MF4_M1_MASK	= 4127,
    RISCV_PseudoVLOXSEG6EI16_V_MF4_MF2	= 4128,
    RISCV_PseudoVLOXSEG6EI16_V_MF4_MF2_MASK	= 4129,
    RISCV_PseudoVLOXSEG6EI16_V_MF4_MF4	= 4130,
    RISCV_PseudoVLOXSEG6EI16_V_MF4_MF4_MASK	= 4131,
    RISCV_PseudoVLOXSEG6EI16_V_MF4_MF8	= 4132,
    RISCV_PseudoVLOXSEG6EI16_V_MF4_MF8_MASK	= 4133,
    RISCV_PseudoVLOXSEG6EI32_V_M1_M1	= 4134,
    RISCV_PseudoVLOXSEG6EI32_V_M1_M1_MASK	= 4135,
    RISCV_PseudoVLOXSEG6EI32_V_M1_MF2	= 4136,
    RISCV_PseudoVLOXSEG6EI32_V_M1_MF2_MASK	= 4137,
    RISCV_PseudoVLOXSEG6EI32_V_M1_MF4	= 4138,
    RISCV_PseudoVLOXSEG6EI32_V_M1_MF4_MASK	= 4139,
    RISCV_PseudoVLOXSEG6EI32_V_M2_M1	= 4140,
    RISCV_PseudoVLOXSEG6EI32_V_M2_M1_MASK	= 4141,
    RISCV_PseudoVLOXSEG6EI32_V_M2_MF2	= 4142,
    RISCV_PseudoVLOXSEG6EI32_V_M2_MF2_MASK	= 4143,
    RISCV_PseudoVLOXSEG6EI32_V_M4_M1	= 4144,
    RISCV_PseudoVLOXSEG6EI32_V_M4_M1_MASK	= 4145,
    RISCV_PseudoVLOXSEG6EI32_V_MF2_M1	= 4146,
    RISCV_PseudoVLOXSEG6EI32_V_MF2_M1_MASK	= 4147,
    RISCV_PseudoVLOXSEG6EI32_V_MF2_MF2	= 4148,
    RISCV_PseudoVLOXSEG6EI32_V_MF2_MF2_MASK	= 4149,
    RISCV_PseudoVLOXSEG6EI32_V_MF2_MF4	= 4150,
    RISCV_PseudoVLOXSEG6EI32_V_MF2_MF4_MASK	= 4151,
    RISCV_PseudoVLOXSEG6EI32_V_MF2_MF8	= 4152,
    RISCV_PseudoVLOXSEG6EI32_V_MF2_MF8_MASK	= 4153,
    RISCV_PseudoVLOXSEG6EI64_V_M1_M1	= 4154,
    RISCV_PseudoVLOXSEG6EI64_V_M1_M1_MASK	= 4155,
    RISCV_PseudoVLOXSEG6EI64_V_M1_MF2	= 4156,
    RISCV_PseudoVLOXSEG6EI64_V_M1_MF2_MASK	= 4157,
    RISCV_PseudoVLOXSEG6EI64_V_M1_MF4	= 4158,
    RISCV_PseudoVLOXSEG6EI64_V_M1_MF4_MASK	= 4159,
    RISCV_PseudoVLOXSEG6EI64_V_M1_MF8	= 4160,
    RISCV_PseudoVLOXSEG6EI64_V_M1_MF8_MASK	= 4161,
    RISCV_PseudoVLOXSEG6EI64_V_M2_M1	= 4162,
    RISCV_PseudoVLOXSEG6EI64_V_M2_M1_MASK	= 4163,
    RISCV_PseudoVLOXSEG6EI64_V_M2_MF2	= 4164,
    RISCV_PseudoVLOXSEG6EI64_V_M2_MF2_MASK	= 4165,
    RISCV_PseudoVLOXSEG6EI64_V_M2_MF4	= 4166,
    RISCV_PseudoVLOXSEG6EI64_V_M2_MF4_MASK	= 4167,
    RISCV_PseudoVLOXSEG6EI64_V_M4_M1	= 4168,
    RISCV_PseudoVLOXSEG6EI64_V_M4_M1_MASK	= 4169,
    RISCV_PseudoVLOXSEG6EI64_V_M4_MF2	= 4170,
    RISCV_PseudoVLOXSEG6EI64_V_M4_MF2_MASK	= 4171,
    RISCV_PseudoVLOXSEG6EI64_V_M8_M1	= 4172,
    RISCV_PseudoVLOXSEG6EI64_V_M8_M1_MASK	= 4173,
    RISCV_PseudoVLOXSEG6EI8_V_M1_M1	= 4174,
    RISCV_PseudoVLOXSEG6EI8_V_M1_M1_MASK	= 4175,
    RISCV_PseudoVLOXSEG6EI8_V_MF2_M1	= 4176,
    RISCV_PseudoVLOXSEG6EI8_V_MF2_M1_MASK	= 4177,
    RISCV_PseudoVLOXSEG6EI8_V_MF2_MF2	= 4178,
    RISCV_PseudoVLOXSEG6EI8_V_MF2_MF2_MASK	= 4179,
    RISCV_PseudoVLOXSEG6EI8_V_MF4_M1	= 4180,
    RISCV_PseudoVLOXSEG6EI8_V_MF4_M1_MASK	= 4181,
    RISCV_PseudoVLOXSEG6EI8_V_MF4_MF2	= 4182,
    RISCV_PseudoVLOXSEG6EI8_V_MF4_MF2_MASK	= 4183,
    RISCV_PseudoVLOXSEG6EI8_V_MF4_MF4	= 4184,
    RISCV_PseudoVLOXSEG6EI8_V_MF4_MF4_MASK	= 4185,
    RISCV_PseudoVLOXSEG6EI8_V_MF8_M1	= 4186,
    RISCV_PseudoVLOXSEG6EI8_V_MF8_M1_MASK	= 4187,
    RISCV_PseudoVLOXSEG6EI8_V_MF8_MF2	= 4188,
    RISCV_PseudoVLOXSEG6EI8_V_MF8_MF2_MASK	= 4189,
    RISCV_PseudoVLOXSEG6EI8_V_MF8_MF4	= 4190,
    RISCV_PseudoVLOXSEG6EI8_V_MF8_MF4_MASK	= 4191,
    RISCV_PseudoVLOXSEG6EI8_V_MF8_MF8	= 4192,
    RISCV_PseudoVLOXSEG6EI8_V_MF8_MF8_MASK	= 4193,
    RISCV_PseudoVLOXSEG7EI16_V_M1_M1	= 4194,
    RISCV_PseudoVLOXSEG7EI16_V_M1_M1_MASK	= 4195,
    RISCV_PseudoVLOXSEG7EI16_V_M1_MF2	= 4196,
    RISCV_PseudoVLOXSEG7EI16_V_M1_MF2_MASK	= 4197,
    RISCV_PseudoVLOXSEG7EI16_V_M2_M1	= 4198,
    RISCV_PseudoVLOXSEG7EI16_V_M2_M1_MASK	= 4199,
    RISCV_PseudoVLOXSEG7EI16_V_MF2_M1	= 4200,
    RISCV_PseudoVLOXSEG7EI16_V_MF2_M1_MASK	= 4201,
    RISCV_PseudoVLOXSEG7EI16_V_MF2_MF2	= 4202,
    RISCV_PseudoVLOXSEG7EI16_V_MF2_MF2_MASK	= 4203,
    RISCV_PseudoVLOXSEG7EI16_V_MF2_MF4	= 4204,
    RISCV_PseudoVLOXSEG7EI16_V_MF2_MF4_MASK	= 4205,
    RISCV_PseudoVLOXSEG7EI16_V_MF4_M1	= 4206,
    RISCV_PseudoVLOXSEG7EI16_V_MF4_M1_MASK	= 4207,
    RISCV_PseudoVLOXSEG7EI16_V_MF4_MF2	= 4208,
    RISCV_PseudoVLOXSEG7EI16_V_MF4_MF2_MASK	= 4209,
    RISCV_PseudoVLOXSEG7EI16_V_MF4_MF4	= 4210,
    RISCV_PseudoVLOXSEG7EI16_V_MF4_MF4_MASK	= 4211,
    RISCV_PseudoVLOXSEG7EI16_V_MF4_MF8	= 4212,
    RISCV_PseudoVLOXSEG7EI16_V_MF4_MF8_MASK	= 4213,
    RISCV_PseudoVLOXSEG7EI32_V_M1_M1	= 4214,
    RISCV_PseudoVLOXSEG7EI32_V_M1_M1_MASK	= 4215,
    RISCV_PseudoVLOXSEG7EI32_V_M1_MF2	= 4216,
    RISCV_PseudoVLOXSEG7EI32_V_M1_MF2_MASK	= 4217,
    RISCV_PseudoVLOXSEG7EI32_V_M1_MF4	= 4218,
    RISCV_PseudoVLOXSEG7EI32_V_M1_MF4_MASK	= 4219,
    RISCV_PseudoVLOXSEG7EI32_V_M2_M1	= 4220,
    RISCV_PseudoVLOXSEG7EI32_V_M2_M1_MASK	= 4221,
    RISCV_PseudoVLOXSEG7EI32_V_M2_MF2	= 4222,
    RISCV_PseudoVLOXSEG7EI32_V_M2_MF2_MASK	= 4223,
    RISCV_PseudoVLOXSEG7EI32_V_M4_M1	= 4224,
    RISCV_PseudoVLOXSEG7EI32_V_M4_M1_MASK	= 4225,
    RISCV_PseudoVLOXSEG7EI32_V_MF2_M1	= 4226,
    RISCV_PseudoVLOXSEG7EI32_V_MF2_M1_MASK	= 4227,
    RISCV_PseudoVLOXSEG7EI32_V_MF2_MF2	= 4228,
    RISCV_PseudoVLOXSEG7EI32_V_MF2_MF2_MASK	= 4229,
    RISCV_PseudoVLOXSEG7EI32_V_MF2_MF4	= 4230,
    RISCV_PseudoVLOXSEG7EI32_V_MF2_MF4_MASK	= 4231,
    RISCV_PseudoVLOXSEG7EI32_V_MF2_MF8	= 4232,
    RISCV_PseudoVLOXSEG7EI32_V_MF2_MF8_MASK	= 4233,
    RISCV_PseudoVLOXSEG7EI64_V_M1_M1	= 4234,
    RISCV_PseudoVLOXSEG7EI64_V_M1_M1_MASK	= 4235,
    RISCV_PseudoVLOXSEG7EI64_V_M1_MF2	= 4236,
    RISCV_PseudoVLOXSEG7EI64_V_M1_MF2_MASK	= 4237,
    RISCV_PseudoVLOXSEG7EI64_V_M1_MF4	= 4238,
    RISCV_PseudoVLOXSEG7EI64_V_M1_MF4_MASK	= 4239,
    RISCV_PseudoVLOXSEG7EI64_V_M1_MF8	= 4240,
    RISCV_PseudoVLOXSEG7EI64_V_M1_MF8_MASK	= 4241,
    RISCV_PseudoVLOXSEG7EI64_V_M2_M1	= 4242,
    RISCV_PseudoVLOXSEG7EI64_V_M2_M1_MASK	= 4243,
    RISCV_PseudoVLOXSEG7EI64_V_M2_MF2	= 4244,
    RISCV_PseudoVLOXSEG7EI64_V_M2_MF2_MASK	= 4245,
    RISCV_PseudoVLOXSEG7EI64_V_M2_MF4	= 4246,
    RISCV_PseudoVLOXSEG7EI64_V_M2_MF4_MASK	= 4247,
    RISCV_PseudoVLOXSEG7EI64_V_M4_M1	= 4248,
    RISCV_PseudoVLOXSEG7EI64_V_M4_M1_MASK	= 4249,
    RISCV_PseudoVLOXSEG7EI64_V_M4_MF2	= 4250,
    RISCV_PseudoVLOXSEG7EI64_V_M4_MF2_MASK	= 4251,
    RISCV_PseudoVLOXSEG7EI64_V_M8_M1	= 4252,
    RISCV_PseudoVLOXSEG7EI64_V_M8_M1_MASK	= 4253,
    RISCV_PseudoVLOXSEG7EI8_V_M1_M1	= 4254,
    RISCV_PseudoVLOXSEG7EI8_V_M1_M1_MASK	= 4255,
    RISCV_PseudoVLOXSEG7EI8_V_MF2_M1	= 4256,
    RISCV_PseudoVLOXSEG7EI8_V_MF2_M1_MASK	= 4257,
    RISCV_PseudoVLOXSEG7EI8_V_MF2_MF2	= 4258,
    RISCV_PseudoVLOXSEG7EI8_V_MF2_MF2_MASK	= 4259,
    RISCV_PseudoVLOXSEG7EI8_V_MF4_M1	= 4260,
    RISCV_PseudoVLOXSEG7EI8_V_MF4_M1_MASK	= 4261,
    RISCV_PseudoVLOXSEG7EI8_V_MF4_MF2	= 4262,
    RISCV_PseudoVLOXSEG7EI8_V_MF4_MF2_MASK	= 4263,
    RISCV_PseudoVLOXSEG7EI8_V_MF4_MF4	= 4264,
    RISCV_PseudoVLOXSEG7EI8_V_MF4_MF4_MASK	= 4265,
    RISCV_PseudoVLOXSEG7EI8_V_MF8_M1	= 4266,
    RISCV_PseudoVLOXSEG7EI8_V_MF8_M1_MASK	= 4267,
    RISCV_PseudoVLOXSEG7EI8_V_MF8_MF2	= 4268,
    RISCV_PseudoVLOXSEG7EI8_V_MF8_MF2_MASK	= 4269,
    RISCV_PseudoVLOXSEG7EI8_V_MF8_MF4	= 4270,
    RISCV_PseudoVLOXSEG7EI8_V_MF8_MF4_MASK	= 4271,
    RISCV_PseudoVLOXSEG7EI8_V_MF8_MF8	= 4272,
    RISCV_PseudoVLOXSEG7EI8_V_MF8_MF8_MASK	= 4273,
    RISCV_PseudoVLOXSEG8EI16_V_M1_M1	= 4274,
    RISCV_PseudoVLOXSEG8EI16_V_M1_M1_MASK	= 4275,
    RISCV_PseudoVLOXSEG8EI16_V_M1_MF2	= 4276,
    RISCV_PseudoVLOXSEG8EI16_V_M1_MF2_MASK	= 4277,
    RISCV_PseudoVLOXSEG8EI16_V_M2_M1	= 4278,
    RISCV_PseudoVLOXSEG8EI16_V_M2_M1_MASK	= 4279,
    RISCV_PseudoVLOXSEG8EI16_V_MF2_M1	= 4280,
    RISCV_PseudoVLOXSEG8EI16_V_MF2_M1_MASK	= 4281,
    RISCV_PseudoVLOXSEG8EI16_V_MF2_MF2	= 4282,
    RISCV_PseudoVLOXSEG8EI16_V_MF2_MF2_MASK	= 4283,
    RISCV_PseudoVLOXSEG8EI16_V_MF2_MF4	= 4284,
    RISCV_PseudoVLOXSEG8EI16_V_MF2_MF4_MASK	= 4285,
    RISCV_PseudoVLOXSEG8EI16_V_MF4_M1	= 4286,
    RISCV_PseudoVLOXSEG8EI16_V_MF4_M1_MASK	= 4287,
    RISCV_PseudoVLOXSEG8EI16_V_MF4_MF2	= 4288,
    RISCV_PseudoVLOXSEG8EI16_V_MF4_MF2_MASK	= 4289,
    RISCV_PseudoVLOXSEG8EI16_V_MF4_MF4	= 4290,
    RISCV_PseudoVLOXSEG8EI16_V_MF4_MF4_MASK	= 4291,
    RISCV_PseudoVLOXSEG8EI16_V_MF4_MF8	= 4292,
    RISCV_PseudoVLOXSEG8EI16_V_MF4_MF8_MASK	= 4293,
    RISCV_PseudoVLOXSEG8EI32_V_M1_M1	= 4294,
    RISCV_PseudoVLOXSEG8EI32_V_M1_M1_MASK	= 4295,
    RISCV_PseudoVLOXSEG8EI32_V_M1_MF2	= 4296,
    RISCV_PseudoVLOXSEG8EI32_V_M1_MF2_MASK	= 4297,
    RISCV_PseudoVLOXSEG8EI32_V_M1_MF4	= 4298,
    RISCV_PseudoVLOXSEG8EI32_V_M1_MF4_MASK	= 4299,
    RISCV_PseudoVLOXSEG8EI32_V_M2_M1	= 4300,
    RISCV_PseudoVLOXSEG8EI32_V_M2_M1_MASK	= 4301,
    RISCV_PseudoVLOXSEG8EI32_V_M2_MF2	= 4302,
    RISCV_PseudoVLOXSEG8EI32_V_M2_MF2_MASK	= 4303,
    RISCV_PseudoVLOXSEG8EI32_V_M4_M1	= 4304,
    RISCV_PseudoVLOXSEG8EI32_V_M4_M1_MASK	= 4305,
    RISCV_PseudoVLOXSEG8EI32_V_MF2_M1	= 4306,
    RISCV_PseudoVLOXSEG8EI32_V_MF2_M1_MASK	= 4307,
    RISCV_PseudoVLOXSEG8EI32_V_MF2_MF2	= 4308,
    RISCV_PseudoVLOXSEG8EI32_V_MF2_MF2_MASK	= 4309,
    RISCV_PseudoVLOXSEG8EI32_V_MF2_MF4	= 4310,
    RISCV_PseudoVLOXSEG8EI32_V_MF2_MF4_MASK	= 4311,
    RISCV_PseudoVLOXSEG8EI32_V_MF2_MF8	= 4312,
    RISCV_PseudoVLOXSEG8EI32_V_MF2_MF8_MASK	= 4313,
    RISCV_PseudoVLOXSEG8EI64_V_M1_M1	= 4314,
    RISCV_PseudoVLOXSEG8EI64_V_M1_M1_MASK	= 4315,
    RISCV_PseudoVLOXSEG8EI64_V_M1_MF2	= 4316,
    RISCV_PseudoVLOXSEG8EI64_V_M1_MF2_MASK	= 4317,
    RISCV_PseudoVLOXSEG8EI64_V_M1_MF4	= 4318,
    RISCV_PseudoVLOXSEG8EI64_V_M1_MF4_MASK	= 4319,
    RISCV_PseudoVLOXSEG8EI64_V_M1_MF8	= 4320,
    RISCV_PseudoVLOXSEG8EI64_V_M1_MF8_MASK	= 4321,
    RISCV_PseudoVLOXSEG8EI64_V_M2_M1	= 4322,
    RISCV_PseudoVLOXSEG8EI64_V_M2_M1_MASK	= 4323,
    RISCV_PseudoVLOXSEG8EI64_V_M2_MF2	= 4324,
    RISCV_PseudoVLOXSEG8EI64_V_M2_MF2_MASK	= 4325,
    RISCV_PseudoVLOXSEG8EI64_V_M2_MF4	= 4326,
    RISCV_PseudoVLOXSEG8EI64_V_M2_MF4_MASK	= 4327,
    RISCV_PseudoVLOXSEG8EI64_V_M4_M1	= 4328,
    RISCV_PseudoVLOXSEG8EI64_V_M4_M1_MASK	= 4329,
    RISCV_PseudoVLOXSEG8EI64_V_M4_MF2	= 4330,
    RISCV_PseudoVLOXSEG8EI64_V_M4_MF2_MASK	= 4331,
    RISCV_PseudoVLOXSEG8EI64_V_M8_M1	= 4332,
    RISCV_PseudoVLOXSEG8EI64_V_M8_M1_MASK	= 4333,
    RISCV_PseudoVLOXSEG8EI8_V_M1_M1	= 4334,
    RISCV_PseudoVLOXSEG8EI8_V_M1_M1_MASK	= 4335,
    RISCV_PseudoVLOXSEG8EI8_V_MF2_M1	= 4336,
    RISCV_PseudoVLOXSEG8EI8_V_MF2_M1_MASK	= 4337,
    RISCV_PseudoVLOXSEG8EI8_V_MF2_MF2	= 4338,
    RISCV_PseudoVLOXSEG8EI8_V_MF2_MF2_MASK	= 4339,
    RISCV_PseudoVLOXSEG8EI8_V_MF4_M1	= 4340,
    RISCV_PseudoVLOXSEG8EI8_V_MF4_M1_MASK	= 4341,
    RISCV_PseudoVLOXSEG8EI8_V_MF4_MF2	= 4342,
    RISCV_PseudoVLOXSEG8EI8_V_MF4_MF2_MASK	= 4343,
    RISCV_PseudoVLOXSEG8EI8_V_MF4_MF4	= 4344,
    RISCV_PseudoVLOXSEG8EI8_V_MF4_MF4_MASK	= 4345,
    RISCV_PseudoVLOXSEG8EI8_V_MF8_M1	= 4346,
    RISCV_PseudoVLOXSEG8EI8_V_MF8_M1_MASK	= 4347,
    RISCV_PseudoVLOXSEG8EI8_V_MF8_MF2	= 4348,
    RISCV_PseudoVLOXSEG8EI8_V_MF8_MF2_MASK	= 4349,
    RISCV_PseudoVLOXSEG8EI8_V_MF8_MF4	= 4350,
    RISCV_PseudoVLOXSEG8EI8_V_MF8_MF4_MASK	= 4351,
    RISCV_PseudoVLOXSEG8EI8_V_MF8_MF8	= 4352,
    RISCV_PseudoVLOXSEG8EI8_V_MF8_MF8_MASK	= 4353,
    RISCV_PseudoVLSE16_V_M1	= 4354,
    RISCV_PseudoVLSE16_V_M1_MASK	= 4355,
    RISCV_PseudoVLSE16_V_M2	= 4356,
    RISCV_PseudoVLSE16_V_M2_MASK	= 4357,
    RISCV_PseudoVLSE16_V_M4	= 4358,
    RISCV_PseudoVLSE16_V_M4_MASK	= 4359,
    RISCV_PseudoVLSE16_V_M8	= 4360,
    RISCV_PseudoVLSE16_V_M8_MASK	= 4361,
    RISCV_PseudoVLSE16_V_MF2	= 4362,
    RISCV_PseudoVLSE16_V_MF2_MASK	= 4363,
    RISCV_PseudoVLSE16_V_MF4	= 4364,
    RISCV_PseudoVLSE16_V_MF4_MASK	= 4365,
    RISCV_PseudoVLSE32_V_M1	= 4366,
    RISCV_PseudoVLSE32_V_M1_MASK	= 4367,
    RISCV_PseudoVLSE32_V_M2	= 4368,
    RISCV_PseudoVLSE32_V_M2_MASK	= 4369,
    RISCV_PseudoVLSE32_V_M4	= 4370,
    RISCV_PseudoVLSE32_V_M4_MASK	= 4371,
    RISCV_PseudoVLSE32_V_M8	= 4372,
    RISCV_PseudoVLSE32_V_M8_MASK	= 4373,
    RISCV_PseudoVLSE32_V_MF2	= 4374,
    RISCV_PseudoVLSE32_V_MF2_MASK	= 4375,
    RISCV_PseudoVLSE64_V_M1	= 4376,
    RISCV_PseudoVLSE64_V_M1_MASK	= 4377,
    RISCV_PseudoVLSE64_V_M2	= 4378,
    RISCV_PseudoVLSE64_V_M2_MASK	= 4379,
    RISCV_PseudoVLSE64_V_M4	= 4380,
    RISCV_PseudoVLSE64_V_M4_MASK	= 4381,
    RISCV_PseudoVLSE64_V_M8	= 4382,
    RISCV_PseudoVLSE64_V_M8_MASK	= 4383,
    RISCV_PseudoVLSE8_V_M1	= 4384,
    RISCV_PseudoVLSE8_V_M1_MASK	= 4385,
    RISCV_PseudoVLSE8_V_M2	= 4386,
    RISCV_PseudoVLSE8_V_M2_MASK	= 4387,
    RISCV_PseudoVLSE8_V_M4	= 4388,
    RISCV_PseudoVLSE8_V_M4_MASK	= 4389,
    RISCV_PseudoVLSE8_V_M8	= 4390,
    RISCV_PseudoVLSE8_V_M8_MASK	= 4391,
    RISCV_PseudoVLSE8_V_MF2	= 4392,
    RISCV_PseudoVLSE8_V_MF2_MASK	= 4393,
    RISCV_PseudoVLSE8_V_MF4	= 4394,
    RISCV_PseudoVLSE8_V_MF4_MASK	= 4395,
    RISCV_PseudoVLSE8_V_MF8	= 4396,
    RISCV_PseudoVLSE8_V_MF8_MASK	= 4397,
    RISCV_PseudoVLSEG2E16FF_V_M1	= 4398,
    RISCV_PseudoVLSEG2E16FF_V_M1_MASK	= 4399,
    RISCV_PseudoVLSEG2E16FF_V_M2	= 4400,
    RISCV_PseudoVLSEG2E16FF_V_M2_MASK	= 4401,
    RISCV_PseudoVLSEG2E16FF_V_M4	= 4402,
    RISCV_PseudoVLSEG2E16FF_V_M4_MASK	= 4403,
    RISCV_PseudoVLSEG2E16FF_V_MF2	= 4404,
    RISCV_PseudoVLSEG2E16FF_V_MF2_MASK	= 4405,
    RISCV_PseudoVLSEG2E16FF_V_MF4	= 4406,
    RISCV_PseudoVLSEG2E16FF_V_MF4_MASK	= 4407,
    RISCV_PseudoVLSEG2E16_V_M1	= 4408,
    RISCV_PseudoVLSEG2E16_V_M1_MASK	= 4409,
    RISCV_PseudoVLSEG2E16_V_M2	= 4410,
    RISCV_PseudoVLSEG2E16_V_M2_MASK	= 4411,
    RISCV_PseudoVLSEG2E16_V_M4	= 4412,
    RISCV_PseudoVLSEG2E16_V_M4_MASK	= 4413,
    RISCV_PseudoVLSEG2E16_V_MF2	= 4414,
    RISCV_PseudoVLSEG2E16_V_MF2_MASK	= 4415,
    RISCV_PseudoVLSEG2E16_V_MF4	= 4416,
    RISCV_PseudoVLSEG2E16_V_MF4_MASK	= 4417,
    RISCV_PseudoVLSEG2E32FF_V_M1	= 4418,
    RISCV_PseudoVLSEG2E32FF_V_M1_MASK	= 4419,
    RISCV_PseudoVLSEG2E32FF_V_M2	= 4420,
    RISCV_PseudoVLSEG2E32FF_V_M2_MASK	= 4421,
    RISCV_PseudoVLSEG2E32FF_V_M4	= 4422,
    RISCV_PseudoVLSEG2E32FF_V_M4_MASK	= 4423,
    RISCV_PseudoVLSEG2E32FF_V_MF2	= 4424,
    RISCV_PseudoVLSEG2E32FF_V_MF2_MASK	= 4425,
    RISCV_PseudoVLSEG2E32_V_M1	= 4426,
    RISCV_PseudoVLSEG2E32_V_M1_MASK	= 4427,
    RISCV_PseudoVLSEG2E32_V_M2	= 4428,
    RISCV_PseudoVLSEG2E32_V_M2_MASK	= 4429,
    RISCV_PseudoVLSEG2E32_V_M4	= 4430,
    RISCV_PseudoVLSEG2E32_V_M4_MASK	= 4431,
    RISCV_PseudoVLSEG2E32_V_MF2	= 4432,
    RISCV_PseudoVLSEG2E32_V_MF2_MASK	= 4433,
    RISCV_PseudoVLSEG2E64FF_V_M1	= 4434,
    RISCV_PseudoVLSEG2E64FF_V_M1_MASK	= 4435,
    RISCV_PseudoVLSEG2E64FF_V_M2	= 4436,
    RISCV_PseudoVLSEG2E64FF_V_M2_MASK	= 4437,
    RISCV_PseudoVLSEG2E64FF_V_M4	= 4438,
    RISCV_PseudoVLSEG2E64FF_V_M4_MASK	= 4439,
    RISCV_PseudoVLSEG2E64_V_M1	= 4440,
    RISCV_PseudoVLSEG2E64_V_M1_MASK	= 4441,
    RISCV_PseudoVLSEG2E64_V_M2	= 4442,
    RISCV_PseudoVLSEG2E64_V_M2_MASK	= 4443,
    RISCV_PseudoVLSEG2E64_V_M4	= 4444,
    RISCV_PseudoVLSEG2E64_V_M4_MASK	= 4445,
    RISCV_PseudoVLSEG2E8FF_V_M1	= 4446,
    RISCV_PseudoVLSEG2E8FF_V_M1_MASK	= 4447,
    RISCV_PseudoVLSEG2E8FF_V_M2	= 4448,
    RISCV_PseudoVLSEG2E8FF_V_M2_MASK	= 4449,
    RISCV_PseudoVLSEG2E8FF_V_M4	= 4450,
    RISCV_PseudoVLSEG2E8FF_V_M4_MASK	= 4451,
    RISCV_PseudoVLSEG2E8FF_V_MF2	= 4452,
    RISCV_PseudoVLSEG2E8FF_V_MF2_MASK	= 4453,
    RISCV_PseudoVLSEG2E8FF_V_MF4	= 4454,
    RISCV_PseudoVLSEG2E8FF_V_MF4_MASK	= 4455,
    RISCV_PseudoVLSEG2E8FF_V_MF8	= 4456,
    RISCV_PseudoVLSEG2E8FF_V_MF8_MASK	= 4457,
    RISCV_PseudoVLSEG2E8_V_M1	= 4458,
    RISCV_PseudoVLSEG2E8_V_M1_MASK	= 4459,
    RISCV_PseudoVLSEG2E8_V_M2	= 4460,
    RISCV_PseudoVLSEG2E8_V_M2_MASK	= 4461,
    RISCV_PseudoVLSEG2E8_V_M4	= 4462,
    RISCV_PseudoVLSEG2E8_V_M4_MASK	= 4463,
    RISCV_PseudoVLSEG2E8_V_MF2	= 4464,
    RISCV_PseudoVLSEG2E8_V_MF2_MASK	= 4465,
    RISCV_PseudoVLSEG2E8_V_MF4	= 4466,
    RISCV_PseudoVLSEG2E8_V_MF4_MASK	= 4467,
    RISCV_PseudoVLSEG2E8_V_MF8	= 4468,
    RISCV_PseudoVLSEG2E8_V_MF8_MASK	= 4469,
    RISCV_PseudoVLSEG3E16FF_V_M1	= 4470,
    RISCV_PseudoVLSEG3E16FF_V_M1_MASK	= 4471,
    RISCV_PseudoVLSEG3E16FF_V_M2	= 4472,
    RISCV_PseudoVLSEG3E16FF_V_M2_MASK	= 4473,
    RISCV_PseudoVLSEG3E16FF_V_MF2	= 4474,
    RISCV_PseudoVLSEG3E16FF_V_MF2_MASK	= 4475,
    RISCV_PseudoVLSEG3E16FF_V_MF4	= 4476,
    RISCV_PseudoVLSEG3E16FF_V_MF4_MASK	= 4477,
    RISCV_PseudoVLSEG3E16_V_M1	= 4478,
    RISCV_PseudoVLSEG3E16_V_M1_MASK	= 4479,
    RISCV_PseudoVLSEG3E16_V_M2	= 4480,
    RISCV_PseudoVLSEG3E16_V_M2_MASK	= 4481,
    RISCV_PseudoVLSEG3E16_V_MF2	= 4482,
    RISCV_PseudoVLSEG3E16_V_MF2_MASK	= 4483,
    RISCV_PseudoVLSEG3E16_V_MF4	= 4484,
    RISCV_PseudoVLSEG3E16_V_MF4_MASK	= 4485,
    RISCV_PseudoVLSEG3E32FF_V_M1	= 4486,
    RISCV_PseudoVLSEG3E32FF_V_M1_MASK	= 4487,
    RISCV_PseudoVLSEG3E32FF_V_M2	= 4488,
    RISCV_PseudoVLSEG3E32FF_V_M2_MASK	= 4489,
    RISCV_PseudoVLSEG3E32FF_V_MF2	= 4490,
    RISCV_PseudoVLSEG3E32FF_V_MF2_MASK	= 4491,
    RISCV_PseudoVLSEG3E32_V_M1	= 4492,
    RISCV_PseudoVLSEG3E32_V_M1_MASK	= 4493,
    RISCV_PseudoVLSEG3E32_V_M2	= 4494,
    RISCV_PseudoVLSEG3E32_V_M2_MASK	= 4495,
    RISCV_PseudoVLSEG3E32_V_MF2	= 4496,
    RISCV_PseudoVLSEG3E32_V_MF2_MASK	= 4497,
    RISCV_PseudoVLSEG3E64FF_V_M1	= 4498,
    RISCV_PseudoVLSEG3E64FF_V_M1_MASK	= 4499,
    RISCV_PseudoVLSEG3E64FF_V_M2	= 4500,
    RISCV_PseudoVLSEG3E64FF_V_M2_MASK	= 4501,
    RISCV_PseudoVLSEG3E64_V_M1	= 4502,
    RISCV_PseudoVLSEG3E64_V_M1_MASK	= 4503,
    RISCV_PseudoVLSEG3E64_V_M2	= 4504,
    RISCV_PseudoVLSEG3E64_V_M2_MASK	= 4505,
    RISCV_PseudoVLSEG3E8FF_V_M1	= 4506,
    RISCV_PseudoVLSEG3E8FF_V_M1_MASK	= 4507,
    RISCV_PseudoVLSEG3E8FF_V_M2	= 4508,
    RISCV_PseudoVLSEG3E8FF_V_M2_MASK	= 4509,
    RISCV_PseudoVLSEG3E8FF_V_MF2	= 4510,
    RISCV_PseudoVLSEG3E8FF_V_MF2_MASK	= 4511,
    RISCV_PseudoVLSEG3E8FF_V_MF4	= 4512,
    RISCV_PseudoVLSEG3E8FF_V_MF4_MASK	= 4513,
    RISCV_PseudoVLSEG3E8FF_V_MF8	= 4514,
    RISCV_PseudoVLSEG3E8FF_V_MF8_MASK	= 4515,
    RISCV_PseudoVLSEG3E8_V_M1	= 4516,
    RISCV_PseudoVLSEG3E8_V_M1_MASK	= 4517,
    RISCV_PseudoVLSEG3E8_V_M2	= 4518,
    RISCV_PseudoVLSEG3E8_V_M2_MASK	= 4519,
    RISCV_PseudoVLSEG3E8_V_MF2	= 4520,
    RISCV_PseudoVLSEG3E8_V_MF2_MASK	= 4521,
    RISCV_PseudoVLSEG3E8_V_MF4	= 4522,
    RISCV_PseudoVLSEG3E8_V_MF4_MASK	= 4523,
    RISCV_PseudoVLSEG3E8_V_MF8	= 4524,
    RISCV_PseudoVLSEG3E8_V_MF8_MASK	= 4525,
    RISCV_PseudoVLSEG4E16FF_V_M1	= 4526,
    RISCV_PseudoVLSEG4E16FF_V_M1_MASK	= 4527,
    RISCV_PseudoVLSEG4E16FF_V_M2	= 4528,
    RISCV_PseudoVLSEG4E16FF_V_M2_MASK	= 4529,
    RISCV_PseudoVLSEG4E16FF_V_MF2	= 4530,
    RISCV_PseudoVLSEG4E16FF_V_MF2_MASK	= 4531,
    RISCV_PseudoVLSEG4E16FF_V_MF4	= 4532,
    RISCV_PseudoVLSEG4E16FF_V_MF4_MASK	= 4533,
    RISCV_PseudoVLSEG4E16_V_M1	= 4534,
    RISCV_PseudoVLSEG4E16_V_M1_MASK	= 4535,
    RISCV_PseudoVLSEG4E16_V_M2	= 4536,
    RISCV_PseudoVLSEG4E16_V_M2_MASK	= 4537,
    RISCV_PseudoVLSEG4E16_V_MF2	= 4538,
    RISCV_PseudoVLSEG4E16_V_MF2_MASK	= 4539,
    RISCV_PseudoVLSEG4E16_V_MF4	= 4540,
    RISCV_PseudoVLSEG4E16_V_MF4_MASK	= 4541,
    RISCV_PseudoVLSEG4E32FF_V_M1	= 4542,
    RISCV_PseudoVLSEG4E32FF_V_M1_MASK	= 4543,
    RISCV_PseudoVLSEG4E32FF_V_M2	= 4544,
    RISCV_PseudoVLSEG4E32FF_V_M2_MASK	= 4545,
    RISCV_PseudoVLSEG4E32FF_V_MF2	= 4546,
    RISCV_PseudoVLSEG4E32FF_V_MF2_MASK	= 4547,
    RISCV_PseudoVLSEG4E32_V_M1	= 4548,
    RISCV_PseudoVLSEG4E32_V_M1_MASK	= 4549,
    RISCV_PseudoVLSEG4E32_V_M2	= 4550,
    RISCV_PseudoVLSEG4E32_V_M2_MASK	= 4551,
    RISCV_PseudoVLSEG4E32_V_MF2	= 4552,
    RISCV_PseudoVLSEG4E32_V_MF2_MASK	= 4553,
    RISCV_PseudoVLSEG4E64FF_V_M1	= 4554,
    RISCV_PseudoVLSEG4E64FF_V_M1_MASK	= 4555,
    RISCV_PseudoVLSEG4E64FF_V_M2	= 4556,
    RISCV_PseudoVLSEG4E64FF_V_M2_MASK	= 4557,
    RISCV_PseudoVLSEG4E64_V_M1	= 4558,
    RISCV_PseudoVLSEG4E64_V_M1_MASK	= 4559,
    RISCV_PseudoVLSEG4E64_V_M2	= 4560,
    RISCV_PseudoVLSEG4E64_V_M2_MASK	= 4561,
    RISCV_PseudoVLSEG4E8FF_V_M1	= 4562,
    RISCV_PseudoVLSEG4E8FF_V_M1_MASK	= 4563,
    RISCV_PseudoVLSEG4E8FF_V_M2	= 4564,
    RISCV_PseudoVLSEG4E8FF_V_M2_MASK	= 4565,
    RISCV_PseudoVLSEG4E8FF_V_MF2	= 4566,
    RISCV_PseudoVLSEG4E8FF_V_MF2_MASK	= 4567,
    RISCV_PseudoVLSEG4E8FF_V_MF4	= 4568,
    RISCV_PseudoVLSEG4E8FF_V_MF4_MASK	= 4569,
    RISCV_PseudoVLSEG4E8FF_V_MF8	= 4570,
    RISCV_PseudoVLSEG4E8FF_V_MF8_MASK	= 4571,
    RISCV_PseudoVLSEG4E8_V_M1	= 4572,
    RISCV_PseudoVLSEG4E8_V_M1_MASK	= 4573,
    RISCV_PseudoVLSEG4E8_V_M2	= 4574,
    RISCV_PseudoVLSEG4E8_V_M2_MASK	= 4575,
    RISCV_PseudoVLSEG4E8_V_MF2	= 4576,
    RISCV_PseudoVLSEG4E8_V_MF2_MASK	= 4577,
    RISCV_PseudoVLSEG4E8_V_MF4	= 4578,
    RISCV_PseudoVLSEG4E8_V_MF4_MASK	= 4579,
    RISCV_PseudoVLSEG4E8_V_MF8	= 4580,
    RISCV_PseudoVLSEG4E8_V_MF8_MASK	= 4581,
    RISCV_PseudoVLSEG5E16FF_V_M1	= 4582,
    RISCV_PseudoVLSEG5E16FF_V_M1_MASK	= 4583,
    RISCV_PseudoVLSEG5E16FF_V_MF2	= 4584,
    RISCV_PseudoVLSEG5E16FF_V_MF2_MASK	= 4585,
    RISCV_PseudoVLSEG5E16FF_V_MF4	= 4586,
    RISCV_PseudoVLSEG5E16FF_V_MF4_MASK	= 4587,
    RISCV_PseudoVLSEG5E16_V_M1	= 4588,
    RISCV_PseudoVLSEG5E16_V_M1_MASK	= 4589,
    RISCV_PseudoVLSEG5E16_V_MF2	= 4590,
    RISCV_PseudoVLSEG5E16_V_MF2_MASK	= 4591,
    RISCV_PseudoVLSEG5E16_V_MF4	= 4592,
    RISCV_PseudoVLSEG5E16_V_MF4_MASK	= 4593,
    RISCV_PseudoVLSEG5E32FF_V_M1	= 4594,
    RISCV_PseudoVLSEG5E32FF_V_M1_MASK	= 4595,
    RISCV_PseudoVLSEG5E32FF_V_MF2	= 4596,
    RISCV_PseudoVLSEG5E32FF_V_MF2_MASK	= 4597,
    RISCV_PseudoVLSEG5E32_V_M1	= 4598,
    RISCV_PseudoVLSEG5E32_V_M1_MASK	= 4599,
    RISCV_PseudoVLSEG5E32_V_MF2	= 4600,
    RISCV_PseudoVLSEG5E32_V_MF2_MASK	= 4601,
    RISCV_PseudoVLSEG5E64FF_V_M1	= 4602,
    RISCV_PseudoVLSEG5E64FF_V_M1_MASK	= 4603,
    RISCV_PseudoVLSEG5E64_V_M1	= 4604,
    RISCV_PseudoVLSEG5E64_V_M1_MASK	= 4605,
    RISCV_PseudoVLSEG5E8FF_V_M1	= 4606,
    RISCV_PseudoVLSEG5E8FF_V_M1_MASK	= 4607,
    RISCV_PseudoVLSEG5E8FF_V_MF2	= 4608,
    RISCV_PseudoVLSEG5E8FF_V_MF2_MASK	= 4609,
    RISCV_PseudoVLSEG5E8FF_V_MF4	= 4610,
    RISCV_PseudoVLSEG5E8FF_V_MF4_MASK	= 4611,
    RISCV_PseudoVLSEG5E8FF_V_MF8	= 4612,
    RISCV_PseudoVLSEG5E8FF_V_MF8_MASK	= 4613,
    RISCV_PseudoVLSEG5E8_V_M1	= 4614,
    RISCV_PseudoVLSEG5E8_V_M1_MASK	= 4615,
    RISCV_PseudoVLSEG5E8_V_MF2	= 4616,
    RISCV_PseudoVLSEG5E8_V_MF2_MASK	= 4617,
    RISCV_PseudoVLSEG5E8_V_MF4	= 4618,
    RISCV_PseudoVLSEG5E8_V_MF4_MASK	= 4619,
    RISCV_PseudoVLSEG5E8_V_MF8	= 4620,
    RISCV_PseudoVLSEG5E8_V_MF8_MASK	= 4621,
    RISCV_PseudoVLSEG6E16FF_V_M1	= 4622,
    RISCV_PseudoVLSEG6E16FF_V_M1_MASK	= 4623,
    RISCV_PseudoVLSEG6E16FF_V_MF2	= 4624,
    RISCV_PseudoVLSEG6E16FF_V_MF2_MASK	= 4625,
    RISCV_PseudoVLSEG6E16FF_V_MF4	= 4626,
    RISCV_PseudoVLSEG6E16FF_V_MF4_MASK	= 4627,
    RISCV_PseudoVLSEG6E16_V_M1	= 4628,
    RISCV_PseudoVLSEG6E16_V_M1_MASK	= 4629,
    RISCV_PseudoVLSEG6E16_V_MF2	= 4630,
    RISCV_PseudoVLSEG6E16_V_MF2_MASK	= 4631,
    RISCV_PseudoVLSEG6E16_V_MF4	= 4632,
    RISCV_PseudoVLSEG6E16_V_MF4_MASK	= 4633,
    RISCV_PseudoVLSEG6E32FF_V_M1	= 4634,
    RISCV_PseudoVLSEG6E32FF_V_M1_MASK	= 4635,
    RISCV_PseudoVLSEG6E32FF_V_MF2	= 4636,
    RISCV_PseudoVLSEG6E32FF_V_MF2_MASK	= 4637,
    RISCV_PseudoVLSEG6E32_V_M1	= 4638,
    RISCV_PseudoVLSEG6E32_V_M1_MASK	= 4639,
    RISCV_PseudoVLSEG6E32_V_MF2	= 4640,
    RISCV_PseudoVLSEG6E32_V_MF2_MASK	= 4641,
    RISCV_PseudoVLSEG6E64FF_V_M1	= 4642,
    RISCV_PseudoVLSEG6E64FF_V_M1_MASK	= 4643,
    RISCV_PseudoVLSEG6E64_V_M1	= 4644,
    RISCV_PseudoVLSEG6E64_V_M1_MASK	= 4645,
    RISCV_PseudoVLSEG6E8FF_V_M1	= 4646,
    RISCV_PseudoVLSEG6E8FF_V_M1_MASK	= 4647,
    RISCV_PseudoVLSEG6E8FF_V_MF2	= 4648,
    RISCV_PseudoVLSEG6E8FF_V_MF2_MASK	= 4649,
    RISCV_PseudoVLSEG6E8FF_V_MF4	= 4650,
    RISCV_PseudoVLSEG6E8FF_V_MF4_MASK	= 4651,
    RISCV_PseudoVLSEG6E8FF_V_MF8	= 4652,
    RISCV_PseudoVLSEG6E8FF_V_MF8_MASK	= 4653,
    RISCV_PseudoVLSEG6E8_V_M1	= 4654,
    RISCV_PseudoVLSEG6E8_V_M1_MASK	= 4655,
    RISCV_PseudoVLSEG6E8_V_MF2	= 4656,
    RISCV_PseudoVLSEG6E8_V_MF2_MASK	= 4657,
    RISCV_PseudoVLSEG6E8_V_MF4	= 4658,
    RISCV_PseudoVLSEG6E8_V_MF4_MASK	= 4659,
    RISCV_PseudoVLSEG6E8_V_MF8	= 4660,
    RISCV_PseudoVLSEG6E8_V_MF8_MASK	= 4661,
    RISCV_PseudoVLSEG7E16FF_V_M1	= 4662,
    RISCV_PseudoVLSEG7E16FF_V_M1_MASK	= 4663,
    RISCV_PseudoVLSEG7E16FF_V_MF2	= 4664,
    RISCV_PseudoVLSEG7E16FF_V_MF2_MASK	= 4665,
    RISCV_PseudoVLSEG7E16FF_V_MF4	= 4666,
    RISCV_PseudoVLSEG7E16FF_V_MF4_MASK	= 4667,
    RISCV_PseudoVLSEG7E16_V_M1	= 4668,
    RISCV_PseudoVLSEG7E16_V_M1_MASK	= 4669,
    RISCV_PseudoVLSEG7E16_V_MF2	= 4670,
    RISCV_PseudoVLSEG7E16_V_MF2_MASK	= 4671,
    RISCV_PseudoVLSEG7E16_V_MF4	= 4672,
    RISCV_PseudoVLSEG7E16_V_MF4_MASK	= 4673,
    RISCV_PseudoVLSEG7E32FF_V_M1	= 4674,
    RISCV_PseudoVLSEG7E32FF_V_M1_MASK	= 4675,
    RISCV_PseudoVLSEG7E32FF_V_MF2	= 4676,
    RISCV_PseudoVLSEG7E32FF_V_MF2_MASK	= 4677,
    RISCV_PseudoVLSEG7E32_V_M1	= 4678,
    RISCV_PseudoVLSEG7E32_V_M1_MASK	= 4679,
    RISCV_PseudoVLSEG7E32_V_MF2	= 4680,
    RISCV_PseudoVLSEG7E32_V_MF2_MASK	= 4681,
    RISCV_PseudoVLSEG7E64FF_V_M1	= 4682,
    RISCV_PseudoVLSEG7E64FF_V_M1_MASK	= 4683,
    RISCV_PseudoVLSEG7E64_V_M1	= 4684,
    RISCV_PseudoVLSEG7E64_V_M1_MASK	= 4685,
    RISCV_PseudoVLSEG7E8FF_V_M1	= 4686,
    RISCV_PseudoVLSEG7E8FF_V_M1_MASK	= 4687,
    RISCV_PseudoVLSEG7E8FF_V_MF2	= 4688,
    RISCV_PseudoVLSEG7E8FF_V_MF2_MASK	= 4689,
    RISCV_PseudoVLSEG7E8FF_V_MF4	= 4690,
    RISCV_PseudoVLSEG7E8FF_V_MF4_MASK	= 4691,
    RISCV_PseudoVLSEG7E8FF_V_MF8	= 4692,
    RISCV_PseudoVLSEG7E8FF_V_MF8_MASK	= 4693,
    RISCV_PseudoVLSEG7E8_V_M1	= 4694,
    RISCV_PseudoVLSEG7E8_V_M1_MASK	= 4695,
    RISCV_PseudoVLSEG7E8_V_MF2	= 4696,
    RISCV_PseudoVLSEG7E8_V_MF2_MASK	= 4697,
    RISCV_PseudoVLSEG7E8_V_MF4	= 4698,
    RISCV_PseudoVLSEG7E8_V_MF4_MASK	= 4699,
    RISCV_PseudoVLSEG7E8_V_MF8	= 4700,
    RISCV_PseudoVLSEG7E8_V_MF8_MASK	= 4701,
    RISCV_PseudoVLSEG8E16FF_V_M1	= 4702,
    RISCV_PseudoVLSEG8E16FF_V_M1_MASK	= 4703,
    RISCV_PseudoVLSEG8E16FF_V_MF2	= 4704,
    RISCV_PseudoVLSEG8E16FF_V_MF2_MASK	= 4705,
    RISCV_PseudoVLSEG8E16FF_V_MF4	= 4706,
    RISCV_PseudoVLSEG8E16FF_V_MF4_MASK	= 4707,
    RISCV_PseudoVLSEG8E16_V_M1	= 4708,
    RISCV_PseudoVLSEG8E16_V_M1_MASK	= 4709,
    RISCV_PseudoVLSEG8E16_V_MF2	= 4710,
    RISCV_PseudoVLSEG8E16_V_MF2_MASK	= 4711,
    RISCV_PseudoVLSEG8E16_V_MF4	= 4712,
    RISCV_PseudoVLSEG8E16_V_MF4_MASK	= 4713,
    RISCV_PseudoVLSEG8E32FF_V_M1	= 4714,
    RISCV_PseudoVLSEG8E32FF_V_M1_MASK	= 4715,
    RISCV_PseudoVLSEG8E32FF_V_MF2	= 4716,
    RISCV_PseudoVLSEG8E32FF_V_MF2_MASK	= 4717,
    RISCV_PseudoVLSEG8E32_V_M1	= 4718,
    RISCV_PseudoVLSEG8E32_V_M1_MASK	= 4719,
    RISCV_PseudoVLSEG8E32_V_MF2	= 4720,
    RISCV_PseudoVLSEG8E32_V_MF2_MASK	= 4721,
    RISCV_PseudoVLSEG8E64FF_V_M1	= 4722,
    RISCV_PseudoVLSEG8E64FF_V_M1_MASK	= 4723,
    RISCV_PseudoVLSEG8E64_V_M1	= 4724,
    RISCV_PseudoVLSEG8E64_V_M1_MASK	= 4725,
    RISCV_PseudoVLSEG8E8FF_V_M1	= 4726,
    RISCV_PseudoVLSEG8E8FF_V_M1_MASK	= 4727,
    RISCV_PseudoVLSEG8E8FF_V_MF2	= 4728,
    RISCV_PseudoVLSEG8E8FF_V_MF2_MASK	= 4729,
    RISCV_PseudoVLSEG8E8FF_V_MF4	= 4730,
    RISCV_PseudoVLSEG8E8FF_V_MF4_MASK	= 4731,
    RISCV_PseudoVLSEG8E8FF_V_MF8	= 4732,
    RISCV_PseudoVLSEG8E8FF_V_MF8_MASK	= 4733,
    RISCV_PseudoVLSEG8E8_V_M1	= 4734,
    RISCV_PseudoVLSEG8E8_V_M1_MASK	= 4735,
    RISCV_PseudoVLSEG8E8_V_MF2	= 4736,
    RISCV_PseudoVLSEG8E8_V_MF2_MASK	= 4737,
    RISCV_PseudoVLSEG8E8_V_MF4	= 4738,
    RISCV_PseudoVLSEG8E8_V_MF4_MASK	= 4739,
    RISCV_PseudoVLSEG8E8_V_MF8	= 4740,
    RISCV_PseudoVLSEG8E8_V_MF8_MASK	= 4741,
    RISCV_PseudoVLSSEG2E16_V_M1	= 4742,
    RISCV_PseudoVLSSEG2E16_V_M1_MASK	= 4743,
    RISCV_PseudoVLSSEG2E16_V_M2	= 4744,
    RISCV_PseudoVLSSEG2E16_V_M2_MASK	= 4745,
    RISCV_PseudoVLSSEG2E16_V_M4	= 4746,
    RISCV_PseudoVLSSEG2E16_V_M4_MASK	= 4747,
    RISCV_PseudoVLSSEG2E16_V_MF2	= 4748,
    RISCV_PseudoVLSSEG2E16_V_MF2_MASK	= 4749,
    RISCV_PseudoVLSSEG2E16_V_MF4	= 4750,
    RISCV_PseudoVLSSEG2E16_V_MF4_MASK	= 4751,
    RISCV_PseudoVLSSEG2E32_V_M1	= 4752,
    RISCV_PseudoVLSSEG2E32_V_M1_MASK	= 4753,
    RISCV_PseudoVLSSEG2E32_V_M2	= 4754,
    RISCV_PseudoVLSSEG2E32_V_M2_MASK	= 4755,
    RISCV_PseudoVLSSEG2E32_V_M4	= 4756,
    RISCV_PseudoVLSSEG2E32_V_M4_MASK	= 4757,
    RISCV_PseudoVLSSEG2E32_V_MF2	= 4758,
    RISCV_PseudoVLSSEG2E32_V_MF2_MASK	= 4759,
    RISCV_PseudoVLSSEG2E64_V_M1	= 4760,
    RISCV_PseudoVLSSEG2E64_V_M1_MASK	= 4761,
    RISCV_PseudoVLSSEG2E64_V_M2	= 4762,
    RISCV_PseudoVLSSEG2E64_V_M2_MASK	= 4763,
    RISCV_PseudoVLSSEG2E64_V_M4	= 4764,
    RISCV_PseudoVLSSEG2E64_V_M4_MASK	= 4765,
    RISCV_PseudoVLSSEG2E8_V_M1	= 4766,
    RISCV_PseudoVLSSEG2E8_V_M1_MASK	= 4767,
    RISCV_PseudoVLSSEG2E8_V_M2	= 4768,
    RISCV_PseudoVLSSEG2E8_V_M2_MASK	= 4769,
    RISCV_PseudoVLSSEG2E8_V_M4	= 4770,
    RISCV_PseudoVLSSEG2E8_V_M4_MASK	= 4771,
    RISCV_PseudoVLSSEG2E8_V_MF2	= 4772,
    RISCV_PseudoVLSSEG2E8_V_MF2_MASK	= 4773,
    RISCV_PseudoVLSSEG2E8_V_MF4	= 4774,
    RISCV_PseudoVLSSEG2E8_V_MF4_MASK	= 4775,
    RISCV_PseudoVLSSEG2E8_V_MF8	= 4776,
    RISCV_PseudoVLSSEG2E8_V_MF8_MASK	= 4777,
    RISCV_PseudoVLSSEG3E16_V_M1	= 4778,
    RISCV_PseudoVLSSEG3E16_V_M1_MASK	= 4779,
    RISCV_PseudoVLSSEG3E16_V_M2	= 4780,
    RISCV_PseudoVLSSEG3E16_V_M2_MASK	= 4781,
    RISCV_PseudoVLSSEG3E16_V_MF2	= 4782,
    RISCV_PseudoVLSSEG3E16_V_MF2_MASK	= 4783,
    RISCV_PseudoVLSSEG3E16_V_MF4	= 4784,
    RISCV_PseudoVLSSEG3E16_V_MF4_MASK	= 4785,
    RISCV_PseudoVLSSEG3E32_V_M1	= 4786,
    RISCV_PseudoVLSSEG3E32_V_M1_MASK	= 4787,
    RISCV_PseudoVLSSEG3E32_V_M2	= 4788,
    RISCV_PseudoVLSSEG3E32_V_M2_MASK	= 4789,
    RISCV_PseudoVLSSEG3E32_V_MF2	= 4790,
    RISCV_PseudoVLSSEG3E32_V_MF2_MASK	= 4791,
    RISCV_PseudoVLSSEG3E64_V_M1	= 4792,
    RISCV_PseudoVLSSEG3E64_V_M1_MASK	= 4793,
    RISCV_PseudoVLSSEG3E64_V_M2	= 4794,
    RISCV_PseudoVLSSEG3E64_V_M2_MASK	= 4795,
    RISCV_PseudoVLSSEG3E8_V_M1	= 4796,
    RISCV_PseudoVLSSEG3E8_V_M1_MASK	= 4797,
    RISCV_PseudoVLSSEG3E8_V_M2	= 4798,
    RISCV_PseudoVLSSEG3E8_V_M2_MASK	= 4799,
    RISCV_PseudoVLSSEG3E8_V_MF2	= 4800,
    RISCV_PseudoVLSSEG3E8_V_MF2_MASK	= 4801,
    RISCV_PseudoVLSSEG3E8_V_MF4	= 4802,
    RISCV_PseudoVLSSEG3E8_V_MF4_MASK	= 4803,
    RISCV_PseudoVLSSEG3E8_V_MF8	= 4804,
    RISCV_PseudoVLSSEG3E8_V_MF8_MASK	= 4805,
    RISCV_PseudoVLSSEG4E16_V_M1	= 4806,
    RISCV_PseudoVLSSEG4E16_V_M1_MASK	= 4807,
    RISCV_PseudoVLSSEG4E16_V_M2	= 4808,
    RISCV_PseudoVLSSEG4E16_V_M2_MASK	= 4809,
    RISCV_PseudoVLSSEG4E16_V_MF2	= 4810,
    RISCV_PseudoVLSSEG4E16_V_MF2_MASK	= 4811,
    RISCV_PseudoVLSSEG4E16_V_MF4	= 4812,
    RISCV_PseudoVLSSEG4E16_V_MF4_MASK	= 4813,
    RISCV_PseudoVLSSEG4E32_V_M1	= 4814,
    RISCV_PseudoVLSSEG4E32_V_M1_MASK	= 4815,
    RISCV_PseudoVLSSEG4E32_V_M2	= 4816,
    RISCV_PseudoVLSSEG4E32_V_M2_MASK	= 4817,
    RISCV_PseudoVLSSEG4E32_V_MF2	= 4818,
    RISCV_PseudoVLSSEG4E32_V_MF2_MASK	= 4819,
    RISCV_PseudoVLSSEG4E64_V_M1	= 4820,
    RISCV_PseudoVLSSEG4E64_V_M1_MASK	= 4821,
    RISCV_PseudoVLSSEG4E64_V_M2	= 4822,
    RISCV_PseudoVLSSEG4E64_V_M2_MASK	= 4823,
    RISCV_PseudoVLSSEG4E8_V_M1	= 4824,
    RISCV_PseudoVLSSEG4E8_V_M1_MASK	= 4825,
    RISCV_PseudoVLSSEG4E8_V_M2	= 4826,
    RISCV_PseudoVLSSEG4E8_V_M2_MASK	= 4827,
    RISCV_PseudoVLSSEG4E8_V_MF2	= 4828,
    RISCV_PseudoVLSSEG4E8_V_MF2_MASK	= 4829,
    RISCV_PseudoVLSSEG4E8_V_MF4	= 4830,
    RISCV_PseudoVLSSEG4E8_V_MF4_MASK	= 4831,
    RISCV_PseudoVLSSEG4E8_V_MF8	= 4832,
    RISCV_PseudoVLSSEG4E8_V_MF8_MASK	= 4833,
    RISCV_PseudoVLSSEG5E16_V_M1	= 4834,
    RISCV_PseudoVLSSEG5E16_V_M1_MASK	= 4835,
    RISCV_PseudoVLSSEG5E16_V_MF2	= 4836,
    RISCV_PseudoVLSSEG5E16_V_MF2_MASK	= 4837,
    RISCV_PseudoVLSSEG5E16_V_MF4	= 4838,
    RISCV_PseudoVLSSEG5E16_V_MF4_MASK	= 4839,
    RISCV_PseudoVLSSEG5E32_V_M1	= 4840,
    RISCV_PseudoVLSSEG5E32_V_M1_MASK	= 4841,
    RISCV_PseudoVLSSEG5E32_V_MF2	= 4842,
    RISCV_PseudoVLSSEG5E32_V_MF2_MASK	= 4843,
    RISCV_PseudoVLSSEG5E64_V_M1	= 4844,
    RISCV_PseudoVLSSEG5E64_V_M1_MASK	= 4845,
    RISCV_PseudoVLSSEG5E8_V_M1	= 4846,
    RISCV_PseudoVLSSEG5E8_V_M1_MASK	= 4847,
    RISCV_PseudoVLSSEG5E8_V_MF2	= 4848,
    RISCV_PseudoVLSSEG5E8_V_MF2_MASK	= 4849,
    RISCV_PseudoVLSSEG5E8_V_MF4	= 4850,
    RISCV_PseudoVLSSEG5E8_V_MF4_MASK	= 4851,
    RISCV_PseudoVLSSEG5E8_V_MF8	= 4852,
    RISCV_PseudoVLSSEG5E8_V_MF8_MASK	= 4853,
    RISCV_PseudoVLSSEG6E16_V_M1	= 4854,
    RISCV_PseudoVLSSEG6E16_V_M1_MASK	= 4855,
    RISCV_PseudoVLSSEG6E16_V_MF2	= 4856,
    RISCV_PseudoVLSSEG6E16_V_MF2_MASK	= 4857,
    RISCV_PseudoVLSSEG6E16_V_MF4	= 4858,
    RISCV_PseudoVLSSEG6E16_V_MF4_MASK	= 4859,
    RISCV_PseudoVLSSEG6E32_V_M1	= 4860,
    RISCV_PseudoVLSSEG6E32_V_M1_MASK	= 4861,
    RISCV_PseudoVLSSEG6E32_V_MF2	= 4862,
    RISCV_PseudoVLSSEG6E32_V_MF2_MASK	= 4863,
    RISCV_PseudoVLSSEG6E64_V_M1	= 4864,
    RISCV_PseudoVLSSEG6E64_V_M1_MASK	= 4865,
    RISCV_PseudoVLSSEG6E8_V_M1	= 4866,
    RISCV_PseudoVLSSEG6E8_V_M1_MASK	= 4867,
    RISCV_PseudoVLSSEG6E8_V_MF2	= 4868,
    RISCV_PseudoVLSSEG6E8_V_MF2_MASK	= 4869,
    RISCV_PseudoVLSSEG6E8_V_MF4	= 4870,
    RISCV_PseudoVLSSEG6E8_V_MF4_MASK	= 4871,
    RISCV_PseudoVLSSEG6E8_V_MF8	= 4872,
    RISCV_PseudoVLSSEG6E8_V_MF8_MASK	= 4873,
    RISCV_PseudoVLSSEG7E16_V_M1	= 4874,
    RISCV_PseudoVLSSEG7E16_V_M1_MASK	= 4875,
    RISCV_PseudoVLSSEG7E16_V_MF2	= 4876,
    RISCV_PseudoVLSSEG7E16_V_MF2_MASK	= 4877,
    RISCV_PseudoVLSSEG7E16_V_MF4	= 4878,
    RISCV_PseudoVLSSEG7E16_V_MF4_MASK	= 4879,
    RISCV_PseudoVLSSEG7E32_V_M1	= 4880,
    RISCV_PseudoVLSSEG7E32_V_M1_MASK	= 4881,
    RISCV_PseudoVLSSEG7E32_V_MF2	= 4882,
    RISCV_PseudoVLSSEG7E32_V_MF2_MASK	= 4883,
    RISCV_PseudoVLSSEG7E64_V_M1	= 4884,
    RISCV_PseudoVLSSEG7E64_V_M1_MASK	= 4885,
    RISCV_PseudoVLSSEG7E8_V_M1	= 4886,
    RISCV_PseudoVLSSEG7E8_V_M1_MASK	= 4887,
    RISCV_PseudoVLSSEG7E8_V_MF2	= 4888,
    RISCV_PseudoVLSSEG7E8_V_MF2_MASK	= 4889,
    RISCV_PseudoVLSSEG7E8_V_MF4	= 4890,
    RISCV_PseudoVLSSEG7E8_V_MF4_MASK	= 4891,
    RISCV_PseudoVLSSEG7E8_V_MF8	= 4892,
    RISCV_PseudoVLSSEG7E8_V_MF8_MASK	= 4893,
    RISCV_PseudoVLSSEG8E16_V_M1	= 4894,
    RISCV_PseudoVLSSEG8E16_V_M1_MASK	= 4895,
    RISCV_PseudoVLSSEG8E16_V_MF2	= 4896,
    RISCV_PseudoVLSSEG8E16_V_MF2_MASK	= 4897,
    RISCV_PseudoVLSSEG8E16_V_MF4	= 4898,
    RISCV_PseudoVLSSEG8E16_V_MF4_MASK	= 4899,
    RISCV_PseudoVLSSEG8E32_V_M1	= 4900,
    RISCV_PseudoVLSSEG8E32_V_M1_MASK	= 4901,
    RISCV_PseudoVLSSEG8E32_V_MF2	= 4902,
    RISCV_PseudoVLSSEG8E32_V_MF2_MASK	= 4903,
    RISCV_PseudoVLSSEG8E64_V_M1	= 4904,
    RISCV_PseudoVLSSEG8E64_V_M1_MASK	= 4905,
    RISCV_PseudoVLSSEG8E8_V_M1	= 4906,
    RISCV_PseudoVLSSEG8E8_V_M1_MASK	= 4907,
    RISCV_PseudoVLSSEG8E8_V_MF2	= 4908,
    RISCV_PseudoVLSSEG8E8_V_MF2_MASK	= 4909,
    RISCV_PseudoVLSSEG8E8_V_MF4	= 4910,
    RISCV_PseudoVLSSEG8E8_V_MF4_MASK	= 4911,
    RISCV_PseudoVLSSEG8E8_V_MF8	= 4912,
    RISCV_PseudoVLSSEG8E8_V_MF8_MASK	= 4913,
    RISCV_PseudoVLUXEI16_V_M1_M1	= 4914,
    RISCV_PseudoVLUXEI16_V_M1_M1_MASK	= 4915,
    RISCV_PseudoVLUXEI16_V_M1_M2	= 4916,
    RISCV_PseudoVLUXEI16_V_M1_M2_MASK	= 4917,
    RISCV_PseudoVLUXEI16_V_M1_M4	= 4918,
    RISCV_PseudoVLUXEI16_V_M1_M4_MASK	= 4919,
    RISCV_PseudoVLUXEI16_V_M1_MF2	= 4920,
    RISCV_PseudoVLUXEI16_V_M1_MF2_MASK	= 4921,
    RISCV_PseudoVLUXEI16_V_M2_M1	= 4922,
    RISCV_PseudoVLUXEI16_V_M2_M1_MASK	= 4923,
    RISCV_PseudoVLUXEI16_V_M2_M2	= 4924,
    RISCV_PseudoVLUXEI16_V_M2_M2_MASK	= 4925,
    RISCV_PseudoVLUXEI16_V_M2_M4	= 4926,
    RISCV_PseudoVLUXEI16_V_M2_M4_MASK	= 4927,
    RISCV_PseudoVLUXEI16_V_M2_M8	= 4928,
    RISCV_PseudoVLUXEI16_V_M2_M8_MASK	= 4929,
    RISCV_PseudoVLUXEI16_V_M4_M2	= 4930,
    RISCV_PseudoVLUXEI16_V_M4_M2_MASK	= 4931,
    RISCV_PseudoVLUXEI16_V_M4_M4	= 4932,
    RISCV_PseudoVLUXEI16_V_M4_M4_MASK	= 4933,
    RISCV_PseudoVLUXEI16_V_M4_M8	= 4934,
    RISCV_PseudoVLUXEI16_V_M4_M8_MASK	= 4935,
    RISCV_PseudoVLUXEI16_V_M8_M4	= 4936,
    RISCV_PseudoVLUXEI16_V_M8_M4_MASK	= 4937,
    RISCV_PseudoVLUXEI16_V_M8_M8	= 4938,
    RISCV_PseudoVLUXEI16_V_M8_M8_MASK	= 4939,
    RISCV_PseudoVLUXEI16_V_MF2_M1	= 4940,
    RISCV_PseudoVLUXEI16_V_MF2_M1_MASK	= 4941,
    RISCV_PseudoVLUXEI16_V_MF2_M2	= 4942,
    RISCV_PseudoVLUXEI16_V_MF2_M2_MASK	= 4943,
    RISCV_PseudoVLUXEI16_V_MF2_MF2	= 4944,
    RISCV_PseudoVLUXEI16_V_MF2_MF2_MASK	= 4945,
    RISCV_PseudoVLUXEI16_V_MF2_MF4	= 4946,
    RISCV_PseudoVLUXEI16_V_MF2_MF4_MASK	= 4947,
    RISCV_PseudoVLUXEI16_V_MF4_M1	= 4948,
    RISCV_PseudoVLUXEI16_V_MF4_M1_MASK	= 4949,
    RISCV_PseudoVLUXEI16_V_MF4_MF2	= 4950,
    RISCV_PseudoVLUXEI16_V_MF4_MF2_MASK	= 4951,
    RISCV_PseudoVLUXEI16_V_MF4_MF4	= 4952,
    RISCV_PseudoVLUXEI16_V_MF4_MF4_MASK	= 4953,
    RISCV_PseudoVLUXEI16_V_MF4_MF8	= 4954,
    RISCV_PseudoVLUXEI16_V_MF4_MF8_MASK	= 4955,
    RISCV_PseudoVLUXEI32_V_M1_M1	= 4956,
    RISCV_PseudoVLUXEI32_V_M1_M1_MASK	= 4957,
    RISCV_PseudoVLUXEI32_V_M1_M2	= 4958,
    RISCV_PseudoVLUXEI32_V_M1_M2_MASK	= 4959,
    RISCV_PseudoVLUXEI32_V_M1_MF2	= 4960,
    RISCV_PseudoVLUXEI32_V_M1_MF2_MASK	= 4961,
    RISCV_PseudoVLUXEI32_V_M1_MF4	= 4962,
    RISCV_PseudoVLUXEI32_V_M1_MF4_MASK	= 4963,
    RISCV_PseudoVLUXEI32_V_M2_M1	= 4964,
    RISCV_PseudoVLUXEI32_V_M2_M1_MASK	= 4965,
    RISCV_PseudoVLUXEI32_V_M2_M2	= 4966,
    RISCV_PseudoVLUXEI32_V_M2_M2_MASK	= 4967,
    RISCV_PseudoVLUXEI32_V_M2_M4	= 4968,
    RISCV_PseudoVLUXEI32_V_M2_M4_MASK	= 4969,
    RISCV_PseudoVLUXEI32_V_M2_MF2	= 4970,
    RISCV_PseudoVLUXEI32_V_M2_MF2_MASK	= 4971,
    RISCV_PseudoVLUXEI32_V_M4_M1	= 4972,
    RISCV_PseudoVLUXEI32_V_M4_M1_MASK	= 4973,
    RISCV_PseudoVLUXEI32_V_M4_M2	= 4974,
    RISCV_PseudoVLUXEI32_V_M4_M2_MASK	= 4975,
    RISCV_PseudoVLUXEI32_V_M4_M4	= 4976,
    RISCV_PseudoVLUXEI32_V_M4_M4_MASK	= 4977,
    RISCV_PseudoVLUXEI32_V_M4_M8	= 4978,
    RISCV_PseudoVLUXEI32_V_M4_M8_MASK	= 4979,
    RISCV_PseudoVLUXEI32_V_M8_M2	= 4980,
    RISCV_PseudoVLUXEI32_V_M8_M2_MASK	= 4981,
    RISCV_PseudoVLUXEI32_V_M8_M4	= 4982,
    RISCV_PseudoVLUXEI32_V_M8_M4_MASK	= 4983,
    RISCV_PseudoVLUXEI32_V_M8_M8	= 4984,
    RISCV_PseudoVLUXEI32_V_M8_M8_MASK	= 4985,
    RISCV_PseudoVLUXEI32_V_MF2_M1	= 4986,
    RISCV_PseudoVLUXEI32_V_MF2_M1_MASK	= 4987,
    RISCV_PseudoVLUXEI32_V_MF2_MF2	= 4988,
    RISCV_PseudoVLUXEI32_V_MF2_MF2_MASK	= 4989,
    RISCV_PseudoVLUXEI32_V_MF2_MF4	= 4990,
    RISCV_PseudoVLUXEI32_V_MF2_MF4_MASK	= 4991,
    RISCV_PseudoVLUXEI32_V_MF2_MF8	= 4992,
    RISCV_PseudoVLUXEI32_V_MF2_MF8_MASK	= 4993,
    RISCV_PseudoVLUXEI64_V_M1_M1	= 4994,
    RISCV_PseudoVLUXEI64_V_M1_M1_MASK	= 4995,
    RISCV_PseudoVLUXEI64_V_M1_MF2	= 4996,
    RISCV_PseudoVLUXEI64_V_M1_MF2_MASK	= 4997,
    RISCV_PseudoVLUXEI64_V_M1_MF4	= 4998,
    RISCV_PseudoVLUXEI64_V_M1_MF4_MASK	= 4999,
    RISCV_PseudoVLUXEI64_V_M1_MF8	= 5000,
    RISCV_PseudoVLUXEI64_V_M1_MF8_MASK	= 5001,
    RISCV_PseudoVLUXEI64_V_M2_M1	= 5002,
    RISCV_PseudoVLUXEI64_V_M2_M1_MASK	= 5003,
    RISCV_PseudoVLUXEI64_V_M2_M2	= 5004,
    RISCV_PseudoVLUXEI64_V_M2_M2_MASK	= 5005,
    RISCV_PseudoVLUXEI64_V_M2_MF2	= 5006,
    RISCV_PseudoVLUXEI64_V_M2_MF2_MASK	= 5007,
    RISCV_PseudoVLUXEI64_V_M2_MF4	= 5008,
    RISCV_PseudoVLUXEI64_V_M2_MF4_MASK	= 5009,
    RISCV_PseudoVLUXEI64_V_M4_M1	= 5010,
    RISCV_PseudoVLUXEI64_V_M4_M1_MASK	= 5011,
    RISCV_PseudoVLUXEI64_V_M4_M2	= 5012,
    RISCV_PseudoVLUXEI64_V_M4_M2_MASK	= 5013,
    RISCV_PseudoVLUXEI64_V_M4_M4	= 5014,
    RISCV_PseudoVLUXEI64_V_M4_M4_MASK	= 5015,
    RISCV_PseudoVLUXEI64_V_M4_MF2	= 5016,
    RISCV_PseudoVLUXEI64_V_M4_MF2_MASK	= 5017,
    RISCV_PseudoVLUXEI64_V_M8_M1	= 5018,
    RISCV_PseudoVLUXEI64_V_M8_M1_MASK	= 5019,
    RISCV_PseudoVLUXEI64_V_M8_M2	= 5020,
    RISCV_PseudoVLUXEI64_V_M8_M2_MASK	= 5021,
    RISCV_PseudoVLUXEI64_V_M8_M4	= 5022,
    RISCV_PseudoVLUXEI64_V_M8_M4_MASK	= 5023,
    RISCV_PseudoVLUXEI64_V_M8_M8	= 5024,
    RISCV_PseudoVLUXEI64_V_M8_M8_MASK	= 5025,
    RISCV_PseudoVLUXEI8_V_M1_M1	= 5026,
    RISCV_PseudoVLUXEI8_V_M1_M1_MASK	= 5027,
    RISCV_PseudoVLUXEI8_V_M1_M2	= 5028,
    RISCV_PseudoVLUXEI8_V_M1_M2_MASK	= 5029,
    RISCV_PseudoVLUXEI8_V_M1_M4	= 5030,
    RISCV_PseudoVLUXEI8_V_M1_M4_MASK	= 5031,
    RISCV_PseudoVLUXEI8_V_M1_M8	= 5032,
    RISCV_PseudoVLUXEI8_V_M1_M8_MASK	= 5033,
    RISCV_PseudoVLUXEI8_V_M2_M2	= 5034,
    RISCV_PseudoVLUXEI8_V_M2_M2_MASK	= 5035,
    RISCV_PseudoVLUXEI8_V_M2_M4	= 5036,
    RISCV_PseudoVLUXEI8_V_M2_M4_MASK	= 5037,
    RISCV_PseudoVLUXEI8_V_M2_M8	= 5038,
    RISCV_PseudoVLUXEI8_V_M2_M8_MASK	= 5039,
    RISCV_PseudoVLUXEI8_V_M4_M4	= 5040,
    RISCV_PseudoVLUXEI8_V_M4_M4_MASK	= 5041,
    RISCV_PseudoVLUXEI8_V_M4_M8	= 5042,
    RISCV_PseudoVLUXEI8_V_M4_M8_MASK	= 5043,
    RISCV_PseudoVLUXEI8_V_M8_M8	= 5044,
    RISCV_PseudoVLUXEI8_V_M8_M8_MASK	= 5045,
    RISCV_PseudoVLUXEI8_V_MF2_M1	= 5046,
    RISCV_PseudoVLUXEI8_V_MF2_M1_MASK	= 5047,
    RISCV_PseudoVLUXEI8_V_MF2_M2	= 5048,
    RISCV_PseudoVLUXEI8_V_MF2_M2_MASK	= 5049,
    RISCV_PseudoVLUXEI8_V_MF2_M4	= 5050,
    RISCV_PseudoVLUXEI8_V_MF2_M4_MASK	= 5051,
    RISCV_PseudoVLUXEI8_V_MF2_MF2	= 5052,
    RISCV_PseudoVLUXEI8_V_MF2_MF2_MASK	= 5053,
    RISCV_PseudoVLUXEI8_V_MF4_M1	= 5054,
    RISCV_PseudoVLUXEI8_V_MF4_M1_MASK	= 5055,
    RISCV_PseudoVLUXEI8_V_MF4_M2	= 5056,
    RISCV_PseudoVLUXEI8_V_MF4_M2_MASK	= 5057,
    RISCV_PseudoVLUXEI8_V_MF4_MF2	= 5058,
    RISCV_PseudoVLUXEI8_V_MF4_MF2_MASK	= 5059,
    RISCV_PseudoVLUXEI8_V_MF4_MF4	= 5060,
    RISCV_PseudoVLUXEI8_V_MF4_MF4_MASK	= 5061,
    RISCV_PseudoVLUXEI8_V_MF8_M1	= 5062,
    RISCV_PseudoVLUXEI8_V_MF8_M1_MASK	= 5063,
    RISCV_PseudoVLUXEI8_V_MF8_MF2	= 5064,
    RISCV_PseudoVLUXEI8_V_MF8_MF2_MASK	= 5065,
    RISCV_PseudoVLUXEI8_V_MF8_MF4	= 5066,
    RISCV_PseudoVLUXEI8_V_MF8_MF4_MASK	= 5067,
    RISCV_PseudoVLUXEI8_V_MF8_MF8	= 5068,
    RISCV_PseudoVLUXEI8_V_MF8_MF8_MASK	= 5069,
    RISCV_PseudoVLUXSEG2EI16_V_M1_M1	= 5070,
    RISCV_PseudoVLUXSEG2EI16_V_M1_M1_MASK	= 5071,
    RISCV_PseudoVLUXSEG2EI16_V_M1_M2	= 5072,
    RISCV_PseudoVLUXSEG2EI16_V_M1_M2_MASK	= 5073,
    RISCV_PseudoVLUXSEG2EI16_V_M1_M4	= 5074,
    RISCV_PseudoVLUXSEG2EI16_V_M1_M4_MASK	= 5075,
    RISCV_PseudoVLUXSEG2EI16_V_M1_MF2	= 5076,
    RISCV_PseudoVLUXSEG2EI16_V_M1_MF2_MASK	= 5077,
    RISCV_PseudoVLUXSEG2EI16_V_M2_M1	= 5078,
    RISCV_PseudoVLUXSEG2EI16_V_M2_M1_MASK	= 5079,
    RISCV_PseudoVLUXSEG2EI16_V_M2_M2	= 5080,
    RISCV_PseudoVLUXSEG2EI16_V_M2_M2_MASK	= 5081,
    RISCV_PseudoVLUXSEG2EI16_V_M2_M4	= 5082,
    RISCV_PseudoVLUXSEG2EI16_V_M2_M4_MASK	= 5083,
    RISCV_PseudoVLUXSEG2EI16_V_M4_M2	= 5084,
    RISCV_PseudoVLUXSEG2EI16_V_M4_M2_MASK	= 5085,
    RISCV_PseudoVLUXSEG2EI16_V_M4_M4	= 5086,
    RISCV_PseudoVLUXSEG2EI16_V_M4_M4_MASK	= 5087,
    RISCV_PseudoVLUXSEG2EI16_V_M8_M4	= 5088,
    RISCV_PseudoVLUXSEG2EI16_V_M8_M4_MASK	= 5089,
    RISCV_PseudoVLUXSEG2EI16_V_MF2_M1	= 5090,
    RISCV_PseudoVLUXSEG2EI16_V_MF2_M1_MASK	= 5091,
    RISCV_PseudoVLUXSEG2EI16_V_MF2_M2	= 5092,
    RISCV_PseudoVLUXSEG2EI16_V_MF2_M2_MASK	= 5093,
    RISCV_PseudoVLUXSEG2EI16_V_MF2_MF2	= 5094,
    RISCV_PseudoVLUXSEG2EI16_V_MF2_MF2_MASK	= 5095,
    RISCV_PseudoVLUXSEG2EI16_V_MF2_MF4	= 5096,
    RISCV_PseudoVLUXSEG2EI16_V_MF2_MF4_MASK	= 5097,
    RISCV_PseudoVLUXSEG2EI16_V_MF4_M1	= 5098,
    RISCV_PseudoVLUXSEG2EI16_V_MF4_M1_MASK	= 5099,
    RISCV_PseudoVLUXSEG2EI16_V_MF4_MF2	= 5100,
    RISCV_PseudoVLUXSEG2EI16_V_MF4_MF2_MASK	= 5101,
    RISCV_PseudoVLUXSEG2EI16_V_MF4_MF4	= 5102,
    RISCV_PseudoVLUXSEG2EI16_V_MF4_MF4_MASK	= 5103,
    RISCV_PseudoVLUXSEG2EI16_V_MF4_MF8	= 5104,
    RISCV_PseudoVLUXSEG2EI16_V_MF4_MF8_MASK	= 5105,
    RISCV_PseudoVLUXSEG2EI32_V_M1_M1	= 5106,
    RISCV_PseudoVLUXSEG2EI32_V_M1_M1_MASK	= 5107,
    RISCV_PseudoVLUXSEG2EI32_V_M1_M2	= 5108,
    RISCV_PseudoVLUXSEG2EI32_V_M1_M2_MASK	= 5109,
    RISCV_PseudoVLUXSEG2EI32_V_M1_MF2	= 5110,
    RISCV_PseudoVLUXSEG2EI32_V_M1_MF2_MASK	= 5111,
    RISCV_PseudoVLUXSEG2EI32_V_M1_MF4	= 5112,
    RISCV_PseudoVLUXSEG2EI32_V_M1_MF4_MASK	= 5113,
    RISCV_PseudoVLUXSEG2EI32_V_M2_M1	= 5114,
    RISCV_PseudoVLUXSEG2EI32_V_M2_M1_MASK	= 5115,
    RISCV_PseudoVLUXSEG2EI32_V_M2_M2	= 5116,
    RISCV_PseudoVLUXSEG2EI32_V_M2_M2_MASK	= 5117,
    RISCV_PseudoVLUXSEG2EI32_V_M2_M4	= 5118,
    RISCV_PseudoVLUXSEG2EI32_V_M2_M4_MASK	= 5119,
    RISCV_PseudoVLUXSEG2EI32_V_M2_MF2	= 5120,
    RISCV_PseudoVLUXSEG2EI32_V_M2_MF2_MASK	= 5121,
    RISCV_PseudoVLUXSEG2EI32_V_M4_M1	= 5122,
    RISCV_PseudoVLUXSEG2EI32_V_M4_M1_MASK	= 5123,
    RISCV_PseudoVLUXSEG2EI32_V_M4_M2	= 5124,
    RISCV_PseudoVLUXSEG2EI32_V_M4_M2_MASK	= 5125,
    RISCV_PseudoVLUXSEG2EI32_V_M4_M4	= 5126,
    RISCV_PseudoVLUXSEG2EI32_V_M4_M4_MASK	= 5127,
    RISCV_PseudoVLUXSEG2EI32_V_M8_M2	= 5128,
    RISCV_PseudoVLUXSEG2EI32_V_M8_M2_MASK	= 5129,
    RISCV_PseudoVLUXSEG2EI32_V_M8_M4	= 5130,
    RISCV_PseudoVLUXSEG2EI32_V_M8_M4_MASK	= 5131,
    RISCV_PseudoVLUXSEG2EI32_V_MF2_M1	= 5132,
    RISCV_PseudoVLUXSEG2EI32_V_MF2_M1_MASK	= 5133,
    RISCV_PseudoVLUXSEG2EI32_V_MF2_MF2	= 5134,
    RISCV_PseudoVLUXSEG2EI32_V_MF2_MF2_MASK	= 5135,
    RISCV_PseudoVLUXSEG2EI32_V_MF2_MF4	= 5136,
    RISCV_PseudoVLUXSEG2EI32_V_MF2_MF4_MASK	= 5137,
    RISCV_PseudoVLUXSEG2EI32_V_MF2_MF8	= 5138,
    RISCV_PseudoVLUXSEG2EI32_V_MF2_MF8_MASK	= 5139,
    RISCV_PseudoVLUXSEG2EI64_V_M1_M1	= 5140,
    RISCV_PseudoVLUXSEG2EI64_V_M1_M1_MASK	= 5141,
    RISCV_PseudoVLUXSEG2EI64_V_M1_MF2	= 5142,
    RISCV_PseudoVLUXSEG2EI64_V_M1_MF2_MASK	= 5143,
    RISCV_PseudoVLUXSEG2EI64_V_M1_MF4	= 5144,
    RISCV_PseudoVLUXSEG2EI64_V_M1_MF4_MASK	= 5145,
    RISCV_PseudoVLUXSEG2EI64_V_M1_MF8	= 5146,
    RISCV_PseudoVLUXSEG2EI64_V_M1_MF8_MASK	= 5147,
    RISCV_PseudoVLUXSEG2EI64_V_M2_M1	= 5148,
    RISCV_PseudoVLUXSEG2EI64_V_M2_M1_MASK	= 5149,
    RISCV_PseudoVLUXSEG2EI64_V_M2_M2	= 5150,
    RISCV_PseudoVLUXSEG2EI64_V_M2_M2_MASK	= 5151,
    RISCV_PseudoVLUXSEG2EI64_V_M2_MF2	= 5152,
    RISCV_PseudoVLUXSEG2EI64_V_M2_MF2_MASK	= 5153,
    RISCV_PseudoVLUXSEG2EI64_V_M2_MF4	= 5154,
    RISCV_PseudoVLUXSEG2EI64_V_M2_MF4_MASK	= 5155,
    RISCV_PseudoVLUXSEG2EI64_V_M4_M1	= 5156,
    RISCV_PseudoVLUXSEG2EI64_V_M4_M1_MASK	= 5157,
    RISCV_PseudoVLUXSEG2EI64_V_M4_M2	= 5158,
    RISCV_PseudoVLUXSEG2EI64_V_M4_M2_MASK	= 5159,
    RISCV_PseudoVLUXSEG2EI64_V_M4_M4	= 5160,
    RISCV_PseudoVLUXSEG2EI64_V_M4_M4_MASK	= 5161,
    RISCV_PseudoVLUXSEG2EI64_V_M4_MF2	= 5162,
    RISCV_PseudoVLUXSEG2EI64_V_M4_MF2_MASK	= 5163,
    RISCV_PseudoVLUXSEG2EI64_V_M8_M1	= 5164,
    RISCV_PseudoVLUXSEG2EI64_V_M8_M1_MASK	= 5165,
    RISCV_PseudoVLUXSEG2EI64_V_M8_M2	= 5166,
    RISCV_PseudoVLUXSEG2EI64_V_M8_M2_MASK	= 5167,
    RISCV_PseudoVLUXSEG2EI64_V_M8_M4	= 5168,
    RISCV_PseudoVLUXSEG2EI64_V_M8_M4_MASK	= 5169,
    RISCV_PseudoVLUXSEG2EI8_V_M1_M1	= 5170,
    RISCV_PseudoVLUXSEG2EI8_V_M1_M1_MASK	= 5171,
    RISCV_PseudoVLUXSEG2EI8_V_M1_M2	= 5172,
    RISCV_PseudoVLUXSEG2EI8_V_M1_M2_MASK	= 5173,
    RISCV_PseudoVLUXSEG2EI8_V_M1_M4	= 5174,
    RISCV_PseudoVLUXSEG2EI8_V_M1_M4_MASK	= 5175,
    RISCV_PseudoVLUXSEG2EI8_V_M2_M2	= 5176,
    RISCV_PseudoVLUXSEG2EI8_V_M2_M2_MASK	= 5177,
    RISCV_PseudoVLUXSEG2EI8_V_M2_M4	= 5178,
    RISCV_PseudoVLUXSEG2EI8_V_M2_M4_MASK	= 5179,
    RISCV_PseudoVLUXSEG2EI8_V_M4_M4	= 5180,
    RISCV_PseudoVLUXSEG2EI8_V_M4_M4_MASK	= 5181,
    RISCV_PseudoVLUXSEG2EI8_V_MF2_M1	= 5182,
    RISCV_PseudoVLUXSEG2EI8_V_MF2_M1_MASK	= 5183,
    RISCV_PseudoVLUXSEG2EI8_V_MF2_M2	= 5184,
    RISCV_PseudoVLUXSEG2EI8_V_MF2_M2_MASK	= 5185,
    RISCV_PseudoVLUXSEG2EI8_V_MF2_M4	= 5186,
    RISCV_PseudoVLUXSEG2EI8_V_MF2_M4_MASK	= 5187,
    RISCV_PseudoVLUXSEG2EI8_V_MF2_MF2	= 5188,
    RISCV_PseudoVLUXSEG2EI8_V_MF2_MF2_MASK	= 5189,
    RISCV_PseudoVLUXSEG2EI8_V_MF4_M1	= 5190,
    RISCV_PseudoVLUXSEG2EI8_V_MF4_M1_MASK	= 5191,
    RISCV_PseudoVLUXSEG2EI8_V_MF4_M2	= 5192,
    RISCV_PseudoVLUXSEG2EI8_V_MF4_M2_MASK	= 5193,
    RISCV_PseudoVLUXSEG2EI8_V_MF4_MF2	= 5194,
    RISCV_PseudoVLUXSEG2EI8_V_MF4_MF2_MASK	= 5195,
    RISCV_PseudoVLUXSEG2EI8_V_MF4_MF4	= 5196,
    RISCV_PseudoVLUXSEG2EI8_V_MF4_MF4_MASK	= 5197,
    RISCV_PseudoVLUXSEG2EI8_V_MF8_M1	= 5198,
    RISCV_PseudoVLUXSEG2EI8_V_MF8_M1_MASK	= 5199,
    RISCV_PseudoVLUXSEG2EI8_V_MF8_MF2	= 5200,
    RISCV_PseudoVLUXSEG2EI8_V_MF8_MF2_MASK	= 5201,
    RISCV_PseudoVLUXSEG2EI8_V_MF8_MF4	= 5202,
    RISCV_PseudoVLUXSEG2EI8_V_MF8_MF4_MASK	= 5203,
    RISCV_PseudoVLUXSEG2EI8_V_MF8_MF8	= 5204,
    RISCV_PseudoVLUXSEG2EI8_V_MF8_MF8_MASK	= 5205,
    RISCV_PseudoVLUXSEG3EI16_V_M1_M1	= 5206,
    RISCV_PseudoVLUXSEG3EI16_V_M1_M1_MASK	= 5207,
    RISCV_PseudoVLUXSEG3EI16_V_M1_M2	= 5208,
    RISCV_PseudoVLUXSEG3EI16_V_M1_M2_MASK	= 5209,
    RISCV_PseudoVLUXSEG3EI16_V_M1_MF2	= 5210,
    RISCV_PseudoVLUXSEG3EI16_V_M1_MF2_MASK	= 5211,
    RISCV_PseudoVLUXSEG3EI16_V_M2_M1	= 5212,
    RISCV_PseudoVLUXSEG3EI16_V_M2_M1_MASK	= 5213,
    RISCV_PseudoVLUXSEG3EI16_V_M2_M2	= 5214,
    RISCV_PseudoVLUXSEG3EI16_V_M2_M2_MASK	= 5215,
    RISCV_PseudoVLUXSEG3EI16_V_M4_M2	= 5216,
    RISCV_PseudoVLUXSEG3EI16_V_M4_M2_MASK	= 5217,
    RISCV_PseudoVLUXSEG3EI16_V_MF2_M1	= 5218,
    RISCV_PseudoVLUXSEG3EI16_V_MF2_M1_MASK	= 5219,
    RISCV_PseudoVLUXSEG3EI16_V_MF2_M2	= 5220,
    RISCV_PseudoVLUXSEG3EI16_V_MF2_M2_MASK	= 5221,
    RISCV_PseudoVLUXSEG3EI16_V_MF2_MF2	= 5222,
    RISCV_PseudoVLUXSEG3EI16_V_MF2_MF2_MASK	= 5223,
    RISCV_PseudoVLUXSEG3EI16_V_MF2_MF4	= 5224,
    RISCV_PseudoVLUXSEG3EI16_V_MF2_MF4_MASK	= 5225,
    RISCV_PseudoVLUXSEG3EI16_V_MF4_M1	= 5226,
    RISCV_PseudoVLUXSEG3EI16_V_MF4_M1_MASK	= 5227,
    RISCV_PseudoVLUXSEG3EI16_V_MF4_MF2	= 5228,
    RISCV_PseudoVLUXSEG3EI16_V_MF4_MF2_MASK	= 5229,
    RISCV_PseudoVLUXSEG3EI16_V_MF4_MF4	= 5230,
    RISCV_PseudoVLUXSEG3EI16_V_MF4_MF4_MASK	= 5231,
    RISCV_PseudoVLUXSEG3EI16_V_MF4_MF8	= 5232,
    RISCV_PseudoVLUXSEG3EI16_V_MF4_MF8_MASK	= 5233,
    RISCV_PseudoVLUXSEG3EI32_V_M1_M1	= 5234,
    RISCV_PseudoVLUXSEG3EI32_V_M1_M1_MASK	= 5235,
    RISCV_PseudoVLUXSEG3EI32_V_M1_M2	= 5236,
    RISCV_PseudoVLUXSEG3EI32_V_M1_M2_MASK	= 5237,
    RISCV_PseudoVLUXSEG3EI32_V_M1_MF2	= 5238,
    RISCV_PseudoVLUXSEG3EI32_V_M1_MF2_MASK	= 5239,
    RISCV_PseudoVLUXSEG3EI32_V_M1_MF4	= 5240,
    RISCV_PseudoVLUXSEG3EI32_V_M1_MF4_MASK	= 5241,
    RISCV_PseudoVLUXSEG3EI32_V_M2_M1	= 5242,
    RISCV_PseudoVLUXSEG3EI32_V_M2_M1_MASK	= 5243,
    RISCV_PseudoVLUXSEG3EI32_V_M2_M2	= 5244,
    RISCV_PseudoVLUXSEG3EI32_V_M2_M2_MASK	= 5245,
    RISCV_PseudoVLUXSEG3EI32_V_M2_MF2	= 5246,
    RISCV_PseudoVLUXSEG3EI32_V_M2_MF2_MASK	= 5247,
    RISCV_PseudoVLUXSEG3EI32_V_M4_M1	= 5248,
    RISCV_PseudoVLUXSEG3EI32_V_M4_M1_MASK	= 5249,
    RISCV_PseudoVLUXSEG3EI32_V_M4_M2	= 5250,
    RISCV_PseudoVLUXSEG3EI32_V_M4_M2_MASK	= 5251,
    RISCV_PseudoVLUXSEG3EI32_V_M8_M2	= 5252,
    RISCV_PseudoVLUXSEG3EI32_V_M8_M2_MASK	= 5253,
    RISCV_PseudoVLUXSEG3EI32_V_MF2_M1	= 5254,
    RISCV_PseudoVLUXSEG3EI32_V_MF2_M1_MASK	= 5255,
    RISCV_PseudoVLUXSEG3EI32_V_MF2_MF2	= 5256,
    RISCV_PseudoVLUXSEG3EI32_V_MF2_MF2_MASK	= 5257,
    RISCV_PseudoVLUXSEG3EI32_V_MF2_MF4	= 5258,
    RISCV_PseudoVLUXSEG3EI32_V_MF2_MF4_MASK	= 5259,
    RISCV_PseudoVLUXSEG3EI32_V_MF2_MF8	= 5260,
    RISCV_PseudoVLUXSEG3EI32_V_MF2_MF8_MASK	= 5261,
    RISCV_PseudoVLUXSEG3EI64_V_M1_M1	= 5262,
    RISCV_PseudoVLUXSEG3EI64_V_M1_M1_MASK	= 5263,
    RISCV_PseudoVLUXSEG3EI64_V_M1_MF2	= 5264,
    RISCV_PseudoVLUXSEG3EI64_V_M1_MF2_MASK	= 5265,
    RISCV_PseudoVLUXSEG3EI64_V_M1_MF4	= 5266,
    RISCV_PseudoVLUXSEG3EI64_V_M1_MF4_MASK	= 5267,
    RISCV_PseudoVLUXSEG3EI64_V_M1_MF8	= 5268,
    RISCV_PseudoVLUXSEG3EI64_V_M1_MF8_MASK	= 5269,
    RISCV_PseudoVLUXSEG3EI64_V_M2_M1	= 5270,
    RISCV_PseudoVLUXSEG3EI64_V_M2_M1_MASK	= 5271,
    RISCV_PseudoVLUXSEG3EI64_V_M2_M2	= 5272,
    RISCV_PseudoVLUXSEG3EI64_V_M2_M2_MASK	= 5273,
    RISCV_PseudoVLUXSEG3EI64_V_M2_MF2	= 5274,
    RISCV_PseudoVLUXSEG3EI64_V_M2_MF2_MASK	= 5275,
    RISCV_PseudoVLUXSEG3EI64_V_M2_MF4	= 5276,
    RISCV_PseudoVLUXSEG3EI64_V_M2_MF4_MASK	= 5277,
    RISCV_PseudoVLUXSEG3EI64_V_M4_M1	= 5278,
    RISCV_PseudoVLUXSEG3EI64_V_M4_M1_MASK	= 5279,
    RISCV_PseudoVLUXSEG3EI64_V_M4_M2	= 5280,
    RISCV_PseudoVLUXSEG3EI64_V_M4_M2_MASK	= 5281,
    RISCV_PseudoVLUXSEG3EI64_V_M4_MF2	= 5282,
    RISCV_PseudoVLUXSEG3EI64_V_M4_MF2_MASK	= 5283,
    RISCV_PseudoVLUXSEG3EI64_V_M8_M1	= 5284,
    RISCV_PseudoVLUXSEG3EI64_V_M8_M1_MASK	= 5285,
    RISCV_PseudoVLUXSEG3EI64_V_M8_M2	= 5286,
    RISCV_PseudoVLUXSEG3EI64_V_M8_M2_MASK	= 5287,
    RISCV_PseudoVLUXSEG3EI8_V_M1_M1	= 5288,
    RISCV_PseudoVLUXSEG3EI8_V_M1_M1_MASK	= 5289,
    RISCV_PseudoVLUXSEG3EI8_V_M1_M2	= 5290,
    RISCV_PseudoVLUXSEG3EI8_V_M1_M2_MASK	= 5291,
    RISCV_PseudoVLUXSEG3EI8_V_M2_M2	= 5292,
    RISCV_PseudoVLUXSEG3EI8_V_M2_M2_MASK	= 5293,
    RISCV_PseudoVLUXSEG3EI8_V_MF2_M1	= 5294,
    RISCV_PseudoVLUXSEG3EI8_V_MF2_M1_MASK	= 5295,
    RISCV_PseudoVLUXSEG3EI8_V_MF2_M2	= 5296,
    RISCV_PseudoVLUXSEG3EI8_V_MF2_M2_MASK	= 5297,
    RISCV_PseudoVLUXSEG3EI8_V_MF2_MF2	= 5298,
    RISCV_PseudoVLUXSEG3EI8_V_MF2_MF2_MASK	= 5299,
    RISCV_PseudoVLUXSEG3EI8_V_MF4_M1	= 5300,
    RISCV_PseudoVLUXSEG3EI8_V_MF4_M1_MASK	= 5301,
    RISCV_PseudoVLUXSEG3EI8_V_MF4_M2	= 5302,
    RISCV_PseudoVLUXSEG3EI8_V_MF4_M2_MASK	= 5303,
    RISCV_PseudoVLUXSEG3EI8_V_MF4_MF2	= 5304,
    RISCV_PseudoVLUXSEG3EI8_V_MF4_MF2_MASK	= 5305,
    RISCV_PseudoVLUXSEG3EI8_V_MF4_MF4	= 5306,
    RISCV_PseudoVLUXSEG3EI8_V_MF4_MF4_MASK	= 5307,
    RISCV_PseudoVLUXSEG3EI8_V_MF8_M1	= 5308,
    RISCV_PseudoVLUXSEG3EI8_V_MF8_M1_MASK	= 5309,
    RISCV_PseudoVLUXSEG3EI8_V_MF8_MF2	= 5310,
    RISCV_PseudoVLUXSEG3EI8_V_MF8_MF2_MASK	= 5311,
    RISCV_PseudoVLUXSEG3EI8_V_MF8_MF4	= 5312,
    RISCV_PseudoVLUXSEG3EI8_V_MF8_MF4_MASK	= 5313,
    RISCV_PseudoVLUXSEG3EI8_V_MF8_MF8	= 5314,
    RISCV_PseudoVLUXSEG3EI8_V_MF8_MF8_MASK	= 5315,
    RISCV_PseudoVLUXSEG4EI16_V_M1_M1	= 5316,
    RISCV_PseudoVLUXSEG4EI16_V_M1_M1_MASK	= 5317,
    RISCV_PseudoVLUXSEG4EI16_V_M1_M2	= 5318,
    RISCV_PseudoVLUXSEG4EI16_V_M1_M2_MASK	= 5319,
    RISCV_PseudoVLUXSEG4EI16_V_M1_MF2	= 5320,
    RISCV_PseudoVLUXSEG4EI16_V_M1_MF2_MASK	= 5321,
    RISCV_PseudoVLUXSEG4EI16_V_M2_M1	= 5322,
    RISCV_PseudoVLUXSEG4EI16_V_M2_M1_MASK	= 5323,
    RISCV_PseudoVLUXSEG4EI16_V_M2_M2	= 5324,
    RISCV_PseudoVLUXSEG4EI16_V_M2_M2_MASK	= 5325,
    RISCV_PseudoVLUXSEG4EI16_V_M4_M2	= 5326,
    RISCV_PseudoVLUXSEG4EI16_V_M4_M2_MASK	= 5327,
    RISCV_PseudoVLUXSEG4EI16_V_MF2_M1	= 5328,
    RISCV_PseudoVLUXSEG4EI16_V_MF2_M1_MASK	= 5329,
    RISCV_PseudoVLUXSEG4EI16_V_MF2_M2	= 5330,
    RISCV_PseudoVLUXSEG4EI16_V_MF2_M2_MASK	= 5331,
    RISCV_PseudoVLUXSEG4EI16_V_MF2_MF2	= 5332,
    RISCV_PseudoVLUXSEG4EI16_V_MF2_MF2_MASK	= 5333,
    RISCV_PseudoVLUXSEG4EI16_V_MF2_MF4	= 5334,
    RISCV_PseudoVLUXSEG4EI16_V_MF2_MF4_MASK	= 5335,
    RISCV_PseudoVLUXSEG4EI16_V_MF4_M1	= 5336,
    RISCV_PseudoVLUXSEG4EI16_V_MF4_M1_MASK	= 5337,
    RISCV_PseudoVLUXSEG4EI16_V_MF4_MF2	= 5338,
    RISCV_PseudoVLUXSEG4EI16_V_MF4_MF2_MASK	= 5339,
    RISCV_PseudoVLUXSEG4EI16_V_MF4_MF4	= 5340,
    RISCV_PseudoVLUXSEG4EI16_V_MF4_MF4_MASK	= 5341,
    RISCV_PseudoVLUXSEG4EI16_V_MF4_MF8	= 5342,
    RISCV_PseudoVLUXSEG4EI16_V_MF4_MF8_MASK	= 5343,
    RISCV_PseudoVLUXSEG4EI32_V_M1_M1	= 5344,
    RISCV_PseudoVLUXSEG4EI32_V_M1_M1_MASK	= 5345,
    RISCV_PseudoVLUXSEG4EI32_V_M1_M2	= 5346,
    RISCV_PseudoVLUXSEG4EI32_V_M1_M2_MASK	= 5347,
    RISCV_PseudoVLUXSEG4EI32_V_M1_MF2	= 5348,
    RISCV_PseudoVLUXSEG4EI32_V_M1_MF2_MASK	= 5349,
    RISCV_PseudoVLUXSEG4EI32_V_M1_MF4	= 5350,
    RISCV_PseudoVLUXSEG4EI32_V_M1_MF4_MASK	= 5351,
    RISCV_PseudoVLUXSEG4EI32_V_M2_M1	= 5352,
    RISCV_PseudoVLUXSEG4EI32_V_M2_M1_MASK	= 5353,
    RISCV_PseudoVLUXSEG4EI32_V_M2_M2	= 5354,
    RISCV_PseudoVLUXSEG4EI32_V_M2_M2_MASK	= 5355,
    RISCV_PseudoVLUXSEG4EI32_V_M2_MF2	= 5356,
    RISCV_PseudoVLUXSEG4EI32_V_M2_MF2_MASK	= 5357,
    RISCV_PseudoVLUXSEG4EI32_V_M4_M1	= 5358,
    RISCV_PseudoVLUXSEG4EI32_V_M4_M1_MASK	= 5359,
    RISCV_PseudoVLUXSEG4EI32_V_M4_M2	= 5360,
    RISCV_PseudoVLUXSEG4EI32_V_M4_M2_MASK	= 5361,
    RISCV_PseudoVLUXSEG4EI32_V_M8_M2	= 5362,
    RISCV_PseudoVLUXSEG4EI32_V_M8_M2_MASK	= 5363,
    RISCV_PseudoVLUXSEG4EI32_V_MF2_M1	= 5364,
    RISCV_PseudoVLUXSEG4EI32_V_MF2_M1_MASK	= 5365,
    RISCV_PseudoVLUXSEG4EI32_V_MF2_MF2	= 5366,
    RISCV_PseudoVLUXSEG4EI32_V_MF2_MF2_MASK	= 5367,
    RISCV_PseudoVLUXSEG4EI32_V_MF2_MF4	= 5368,
    RISCV_PseudoVLUXSEG4EI32_V_MF2_MF4_MASK	= 5369,
    RISCV_PseudoVLUXSEG4EI32_V_MF2_MF8	= 5370,
    RISCV_PseudoVLUXSEG4EI32_V_MF2_MF8_MASK	= 5371,
    RISCV_PseudoVLUXSEG4EI64_V_M1_M1	= 5372,
    RISCV_PseudoVLUXSEG4EI64_V_M1_M1_MASK	= 5373,
    RISCV_PseudoVLUXSEG4EI64_V_M1_MF2	= 5374,
    RISCV_PseudoVLUXSEG4EI64_V_M1_MF2_MASK	= 5375,
    RISCV_PseudoVLUXSEG4EI64_V_M1_MF4	= 5376,
    RISCV_PseudoVLUXSEG4EI64_V_M1_MF4_MASK	= 5377,
    RISCV_PseudoVLUXSEG4EI64_V_M1_MF8	= 5378,
    RISCV_PseudoVLUXSEG4EI64_V_M1_MF8_MASK	= 5379,
    RISCV_PseudoVLUXSEG4EI64_V_M2_M1	= 5380,
    RISCV_PseudoVLUXSEG4EI64_V_M2_M1_MASK	= 5381,
    RISCV_PseudoVLUXSEG4EI64_V_M2_M2	= 5382,
    RISCV_PseudoVLUXSEG4EI64_V_M2_M2_MASK	= 5383,
    RISCV_PseudoVLUXSEG4EI64_V_M2_MF2	= 5384,
    RISCV_PseudoVLUXSEG4EI64_V_M2_MF2_MASK	= 5385,
    RISCV_PseudoVLUXSEG4EI64_V_M2_MF4	= 5386,
    RISCV_PseudoVLUXSEG4EI64_V_M2_MF4_MASK	= 5387,
    RISCV_PseudoVLUXSEG4EI64_V_M4_M1	= 5388,
    RISCV_PseudoVLUXSEG4EI64_V_M4_M1_MASK	= 5389,
    RISCV_PseudoVLUXSEG4EI64_V_M4_M2	= 5390,
    RISCV_PseudoVLUXSEG4EI64_V_M4_M2_MASK	= 5391,
    RISCV_PseudoVLUXSEG4EI64_V_M4_MF2	= 5392,
    RISCV_PseudoVLUXSEG4EI64_V_M4_MF2_MASK	= 5393,
    RISCV_PseudoVLUXSEG4EI64_V_M8_M1	= 5394,
    RISCV_PseudoVLUXSEG4EI64_V_M8_M1_MASK	= 5395,
    RISCV_PseudoVLUXSEG4EI64_V_M8_M2	= 5396,
    RISCV_PseudoVLUXSEG4EI64_V_M8_M2_MASK	= 5397,
    RISCV_PseudoVLUXSEG4EI8_V_M1_M1	= 5398,
    RISCV_PseudoVLUXSEG4EI8_V_M1_M1_MASK	= 5399,
    RISCV_PseudoVLUXSEG4EI8_V_M1_M2	= 5400,
    RISCV_PseudoVLUXSEG4EI8_V_M1_M2_MASK	= 5401,
    RISCV_PseudoVLUXSEG4EI8_V_M2_M2	= 5402,
    RISCV_PseudoVLUXSEG4EI8_V_M2_M2_MASK	= 5403,
    RISCV_PseudoVLUXSEG4EI8_V_MF2_M1	= 5404,
    RISCV_PseudoVLUXSEG4EI8_V_MF2_M1_MASK	= 5405,
    RISCV_PseudoVLUXSEG4EI8_V_MF2_M2	= 5406,
    RISCV_PseudoVLUXSEG4EI8_V_MF2_M2_MASK	= 5407,
    RISCV_PseudoVLUXSEG4EI8_V_MF2_MF2	= 5408,
    RISCV_PseudoVLUXSEG4EI8_V_MF2_MF2_MASK	= 5409,
    RISCV_PseudoVLUXSEG4EI8_V_MF4_M1	= 5410,
    RISCV_PseudoVLUXSEG4EI8_V_MF4_M1_MASK	= 5411,
    RISCV_PseudoVLUXSEG4EI8_V_MF4_M2	= 5412,
    RISCV_PseudoVLUXSEG4EI8_V_MF4_M2_MASK	= 5413,
    RISCV_PseudoVLUXSEG4EI8_V_MF4_MF2	= 5414,
    RISCV_PseudoVLUXSEG4EI8_V_MF4_MF2_MASK	= 5415,
    RISCV_PseudoVLUXSEG4EI8_V_MF4_MF4	= 5416,
    RISCV_PseudoVLUXSEG4EI8_V_MF4_MF4_MASK	= 5417,
    RISCV_PseudoVLUXSEG4EI8_V_MF8_M1	= 5418,
    RISCV_PseudoVLUXSEG4EI8_V_MF8_M1_MASK	= 5419,
    RISCV_PseudoVLUXSEG4EI8_V_MF8_MF2	= 5420,
    RISCV_PseudoVLUXSEG4EI8_V_MF8_MF2_MASK	= 5421,
    RISCV_PseudoVLUXSEG4EI8_V_MF8_MF4	= 5422,
    RISCV_PseudoVLUXSEG4EI8_V_MF8_MF4_MASK	= 5423,
    RISCV_PseudoVLUXSEG4EI8_V_MF8_MF8	= 5424,
    RISCV_PseudoVLUXSEG4EI8_V_MF8_MF8_MASK	= 5425,
    RISCV_PseudoVLUXSEG5EI16_V_M1_M1	= 5426,
    RISCV_PseudoVLUXSEG5EI16_V_M1_M1_MASK	= 5427,
    RISCV_PseudoVLUXSEG5EI16_V_M1_MF2	= 5428,
    RISCV_PseudoVLUXSEG5EI16_V_M1_MF2_MASK	= 5429,
    RISCV_PseudoVLUXSEG5EI16_V_M2_M1	= 5430,
    RISCV_PseudoVLUXSEG5EI16_V_M2_M1_MASK	= 5431,
    RISCV_PseudoVLUXSEG5EI16_V_MF2_M1	= 5432,
    RISCV_PseudoVLUXSEG5EI16_V_MF2_M1_MASK	= 5433,
    RISCV_PseudoVLUXSEG5EI16_V_MF2_MF2	= 5434,
    RISCV_PseudoVLUXSEG5EI16_V_MF2_MF2_MASK	= 5435,
    RISCV_PseudoVLUXSEG5EI16_V_MF2_MF4	= 5436,
    RISCV_PseudoVLUXSEG5EI16_V_MF2_MF4_MASK	= 5437,
    RISCV_PseudoVLUXSEG5EI16_V_MF4_M1	= 5438,
    RISCV_PseudoVLUXSEG5EI16_V_MF4_M1_MASK	= 5439,
    RISCV_PseudoVLUXSEG5EI16_V_MF4_MF2	= 5440,
    RISCV_PseudoVLUXSEG5EI16_V_MF4_MF2_MASK	= 5441,
    RISCV_PseudoVLUXSEG5EI16_V_MF4_MF4	= 5442,
    RISCV_PseudoVLUXSEG5EI16_V_MF4_MF4_MASK	= 5443,
    RISCV_PseudoVLUXSEG5EI16_V_MF4_MF8	= 5444,
    RISCV_PseudoVLUXSEG5EI16_V_MF4_MF8_MASK	= 5445,
    RISCV_PseudoVLUXSEG5EI32_V_M1_M1	= 5446,
    RISCV_PseudoVLUXSEG5EI32_V_M1_M1_MASK	= 5447,
    RISCV_PseudoVLUXSEG5EI32_V_M1_MF2	= 5448,
    RISCV_PseudoVLUXSEG5EI32_V_M1_MF2_MASK	= 5449,
    RISCV_PseudoVLUXSEG5EI32_V_M1_MF4	= 5450,
    RISCV_PseudoVLUXSEG5EI32_V_M1_MF4_MASK	= 5451,
    RISCV_PseudoVLUXSEG5EI32_V_M2_M1	= 5452,
    RISCV_PseudoVLUXSEG5EI32_V_M2_M1_MASK	= 5453,
    RISCV_PseudoVLUXSEG5EI32_V_M2_MF2	= 5454,
    RISCV_PseudoVLUXSEG5EI32_V_M2_MF2_MASK	= 5455,
    RISCV_PseudoVLUXSEG5EI32_V_M4_M1	= 5456,
    RISCV_PseudoVLUXSEG5EI32_V_M4_M1_MASK	= 5457,
    RISCV_PseudoVLUXSEG5EI32_V_MF2_M1	= 5458,
    RISCV_PseudoVLUXSEG5EI32_V_MF2_M1_MASK	= 5459,
    RISCV_PseudoVLUXSEG5EI32_V_MF2_MF2	= 5460,
    RISCV_PseudoVLUXSEG5EI32_V_MF2_MF2_MASK	= 5461,
    RISCV_PseudoVLUXSEG5EI32_V_MF2_MF4	= 5462,
    RISCV_PseudoVLUXSEG5EI32_V_MF2_MF4_MASK	= 5463,
    RISCV_PseudoVLUXSEG5EI32_V_MF2_MF8	= 5464,
    RISCV_PseudoVLUXSEG5EI32_V_MF2_MF8_MASK	= 5465,
    RISCV_PseudoVLUXSEG5EI64_V_M1_M1	= 5466,
    RISCV_PseudoVLUXSEG5EI64_V_M1_M1_MASK	= 5467,
    RISCV_PseudoVLUXSEG5EI64_V_M1_MF2	= 5468,
    RISCV_PseudoVLUXSEG5EI64_V_M1_MF2_MASK	= 5469,
    RISCV_PseudoVLUXSEG5EI64_V_M1_MF4	= 5470,
    RISCV_PseudoVLUXSEG5EI64_V_M1_MF4_MASK	= 5471,
    RISCV_PseudoVLUXSEG5EI64_V_M1_MF8	= 5472,
    RISCV_PseudoVLUXSEG5EI64_V_M1_MF8_MASK	= 5473,
    RISCV_PseudoVLUXSEG5EI64_V_M2_M1	= 5474,
    RISCV_PseudoVLUXSEG5EI64_V_M2_M1_MASK	= 5475,
    RISCV_PseudoVLUXSEG5EI64_V_M2_MF2	= 5476,
    RISCV_PseudoVLUXSEG5EI64_V_M2_MF2_MASK	= 5477,
    RISCV_PseudoVLUXSEG5EI64_V_M2_MF4	= 5478,
    RISCV_PseudoVLUXSEG5EI64_V_M2_MF4_MASK	= 5479,
    RISCV_PseudoVLUXSEG5EI64_V_M4_M1	= 5480,
    RISCV_PseudoVLUXSEG5EI64_V_M4_M1_MASK	= 5481,
    RISCV_PseudoVLUXSEG5EI64_V_M4_MF2	= 5482,
    RISCV_PseudoVLUXSEG5EI64_V_M4_MF2_MASK	= 5483,
    RISCV_PseudoVLUXSEG5EI64_V_M8_M1	= 5484,
    RISCV_PseudoVLUXSEG5EI64_V_M8_M1_MASK	= 5485,
    RISCV_PseudoVLUXSEG5EI8_V_M1_M1	= 5486,
    RISCV_PseudoVLUXSEG5EI8_V_M1_M1_MASK	= 5487,
    RISCV_PseudoVLUXSEG5EI8_V_MF2_M1	= 5488,
    RISCV_PseudoVLUXSEG5EI8_V_MF2_M1_MASK	= 5489,
    RISCV_PseudoVLUXSEG5EI8_V_MF2_MF2	= 5490,
    RISCV_PseudoVLUXSEG5EI8_V_MF2_MF2_MASK	= 5491,
    RISCV_PseudoVLUXSEG5EI8_V_MF4_M1	= 5492,
    RISCV_PseudoVLUXSEG5EI8_V_MF4_M1_MASK	= 5493,
    RISCV_PseudoVLUXSEG5EI8_V_MF4_MF2	= 5494,
    RISCV_PseudoVLUXSEG5EI8_V_MF4_MF2_MASK	= 5495,
    RISCV_PseudoVLUXSEG5EI8_V_MF4_MF4	= 5496,
    RISCV_PseudoVLUXSEG5EI8_V_MF4_MF4_MASK	= 5497,
    RISCV_PseudoVLUXSEG5EI8_V_MF8_M1	= 5498,
    RISCV_PseudoVLUXSEG5EI8_V_MF8_M1_MASK	= 5499,
    RISCV_PseudoVLUXSEG5EI8_V_MF8_MF2	= 5500,
    RISCV_PseudoVLUXSEG5EI8_V_MF8_MF2_MASK	= 5501,
    RISCV_PseudoVLUXSEG5EI8_V_MF8_MF4	= 5502,
    RISCV_PseudoVLUXSEG5EI8_V_MF8_MF4_MASK	= 5503,
    RISCV_PseudoVLUXSEG5EI8_V_MF8_MF8	= 5504,
    RISCV_PseudoVLUXSEG5EI8_V_MF8_MF8_MASK	= 5505,
    RISCV_PseudoVLUXSEG6EI16_V_M1_M1	= 5506,
    RISCV_PseudoVLUXSEG6EI16_V_M1_M1_MASK	= 5507,
    RISCV_PseudoVLUXSEG6EI16_V_M1_MF2	= 5508,
    RISCV_PseudoVLUXSEG6EI16_V_M1_MF2_MASK	= 5509,
    RISCV_PseudoVLUXSEG6EI16_V_M2_M1	= 5510,
    RISCV_PseudoVLUXSEG6EI16_V_M2_M1_MASK	= 5511,
    RISCV_PseudoVLUXSEG6EI16_V_MF2_M1	= 5512,
    RISCV_PseudoVLUXSEG6EI16_V_MF2_M1_MASK	= 5513,
    RISCV_PseudoVLUXSEG6EI16_V_MF2_MF2	= 5514,
    RISCV_PseudoVLUXSEG6EI16_V_MF2_MF2_MASK	= 5515,
    RISCV_PseudoVLUXSEG6EI16_V_MF2_MF4	= 5516,
    RISCV_PseudoVLUXSEG6EI16_V_MF2_MF4_MASK	= 5517,
    RISCV_PseudoVLUXSEG6EI16_V_MF4_M1	= 5518,
    RISCV_PseudoVLUXSEG6EI16_V_MF4_M1_MASK	= 5519,
    RISCV_PseudoVLUXSEG6EI16_V_MF4_MF2	= 5520,
    RISCV_PseudoVLUXSEG6EI16_V_MF4_MF2_MASK	= 5521,
    RISCV_PseudoVLUXSEG6EI16_V_MF4_MF4	= 5522,
    RISCV_PseudoVLUXSEG6EI16_V_MF4_MF4_MASK	= 5523,
    RISCV_PseudoVLUXSEG6EI16_V_MF4_MF8	= 5524,
    RISCV_PseudoVLUXSEG6EI16_V_MF4_MF8_MASK	= 5525,
    RISCV_PseudoVLUXSEG6EI32_V_M1_M1	= 5526,
    RISCV_PseudoVLUXSEG6EI32_V_M1_M1_MASK	= 5527,
    RISCV_PseudoVLUXSEG6EI32_V_M1_MF2	= 5528,
    RISCV_PseudoVLUXSEG6EI32_V_M1_MF2_MASK	= 5529,
    RISCV_PseudoVLUXSEG6EI32_V_M1_MF4	= 5530,
    RISCV_PseudoVLUXSEG6EI32_V_M1_MF4_MASK	= 5531,
    RISCV_PseudoVLUXSEG6EI32_V_M2_M1	= 5532,
    RISCV_PseudoVLUXSEG6EI32_V_M2_M1_MASK	= 5533,
    RISCV_PseudoVLUXSEG6EI32_V_M2_MF2	= 5534,
    RISCV_PseudoVLUXSEG6EI32_V_M2_MF2_MASK	= 5535,
    RISCV_PseudoVLUXSEG6EI32_V_M4_M1	= 5536,
    RISCV_PseudoVLUXSEG6EI32_V_M4_M1_MASK	= 5537,
    RISCV_PseudoVLUXSEG6EI32_V_MF2_M1	= 5538,
    RISCV_PseudoVLUXSEG6EI32_V_MF2_M1_MASK	= 5539,
    RISCV_PseudoVLUXSEG6EI32_V_MF2_MF2	= 5540,
    RISCV_PseudoVLUXSEG6EI32_V_MF2_MF2_MASK	= 5541,
    RISCV_PseudoVLUXSEG6EI32_V_MF2_MF4	= 5542,
    RISCV_PseudoVLUXSEG6EI32_V_MF2_MF4_MASK	= 5543,
    RISCV_PseudoVLUXSEG6EI32_V_MF2_MF8	= 5544,
    RISCV_PseudoVLUXSEG6EI32_V_MF2_MF8_MASK	= 5545,
    RISCV_PseudoVLUXSEG6EI64_V_M1_M1	= 5546,
    RISCV_PseudoVLUXSEG6EI64_V_M1_M1_MASK	= 5547,
    RISCV_PseudoVLUXSEG6EI64_V_M1_MF2	= 5548,
    RISCV_PseudoVLUXSEG6EI64_V_M1_MF2_MASK	= 5549,
    RISCV_PseudoVLUXSEG6EI64_V_M1_MF4	= 5550,
    RISCV_PseudoVLUXSEG6EI64_V_M1_MF4_MASK	= 5551,
    RISCV_PseudoVLUXSEG6EI64_V_M1_MF8	= 5552,
    RISCV_PseudoVLUXSEG6EI64_V_M1_MF8_MASK	= 5553,
    RISCV_PseudoVLUXSEG6EI64_V_M2_M1	= 5554,
    RISCV_PseudoVLUXSEG6EI64_V_M2_M1_MASK	= 5555,
    RISCV_PseudoVLUXSEG6EI64_V_M2_MF2	= 5556,
    RISCV_PseudoVLUXSEG6EI64_V_M2_MF2_MASK	= 5557,
    RISCV_PseudoVLUXSEG6EI64_V_M2_MF4	= 5558,
    RISCV_PseudoVLUXSEG6EI64_V_M2_MF4_MASK	= 5559,
    RISCV_PseudoVLUXSEG6EI64_V_M4_M1	= 5560,
    RISCV_PseudoVLUXSEG6EI64_V_M4_M1_MASK	= 5561,
    RISCV_PseudoVLUXSEG6EI64_V_M4_MF2	= 5562,
    RISCV_PseudoVLUXSEG6EI64_V_M4_MF2_MASK	= 5563,
    RISCV_PseudoVLUXSEG6EI64_V_M8_M1	= 5564,
    RISCV_PseudoVLUXSEG6EI64_V_M8_M1_MASK	= 5565,
    RISCV_PseudoVLUXSEG6EI8_V_M1_M1	= 5566,
    RISCV_PseudoVLUXSEG6EI8_V_M1_M1_MASK	= 5567,
    RISCV_PseudoVLUXSEG6EI8_V_MF2_M1	= 5568,
    RISCV_PseudoVLUXSEG6EI8_V_MF2_M1_MASK	= 5569,
    RISCV_PseudoVLUXSEG6EI8_V_MF2_MF2	= 5570,
    RISCV_PseudoVLUXSEG6EI8_V_MF2_MF2_MASK	= 5571,
    RISCV_PseudoVLUXSEG6EI8_V_MF4_M1	= 5572,
    RISCV_PseudoVLUXSEG6EI8_V_MF4_M1_MASK	= 5573,
    RISCV_PseudoVLUXSEG6EI8_V_MF4_MF2	= 5574,
    RISCV_PseudoVLUXSEG6EI8_V_MF4_MF2_MASK	= 5575,
    RISCV_PseudoVLUXSEG6EI8_V_MF4_MF4	= 5576,
    RISCV_PseudoVLUXSEG6EI8_V_MF4_MF4_MASK	= 5577,
    RISCV_PseudoVLUXSEG6EI8_V_MF8_M1	= 5578,
    RISCV_PseudoVLUXSEG6EI8_V_MF8_M1_MASK	= 5579,
    RISCV_PseudoVLUXSEG6EI8_V_MF8_MF2	= 5580,
    RISCV_PseudoVLUXSEG6EI8_V_MF8_MF2_MASK	= 5581,
    RISCV_PseudoVLUXSEG6EI8_V_MF8_MF4	= 5582,
    RISCV_PseudoVLUXSEG6EI8_V_MF8_MF4_MASK	= 5583,
    RISCV_PseudoVLUXSEG6EI8_V_MF8_MF8	= 5584,
    RISCV_PseudoVLUXSEG6EI8_V_MF8_MF8_MASK	= 5585,
    RISCV_PseudoVLUXSEG7EI16_V_M1_M1	= 5586,
    RISCV_PseudoVLUXSEG7EI16_V_M1_M1_MASK	= 5587,
    RISCV_PseudoVLUXSEG7EI16_V_M1_MF2	= 5588,
    RISCV_PseudoVLUXSEG7EI16_V_M1_MF2_MASK	= 5589,
    RISCV_PseudoVLUXSEG7EI16_V_M2_M1	= 5590,
    RISCV_PseudoVLUXSEG7EI16_V_M2_M1_MASK	= 5591,
    RISCV_PseudoVLUXSEG7EI16_V_MF2_M1	= 5592,
    RISCV_PseudoVLUXSEG7EI16_V_MF2_M1_MASK	= 5593,
    RISCV_PseudoVLUXSEG7EI16_V_MF2_MF2	= 5594,
    RISCV_PseudoVLUXSEG7EI16_V_MF2_MF2_MASK	= 5595,
    RISCV_PseudoVLUXSEG7EI16_V_MF2_MF4	= 5596,
    RISCV_PseudoVLUXSEG7EI16_V_MF2_MF4_MASK	= 5597,
    RISCV_PseudoVLUXSEG7EI16_V_MF4_M1	= 5598,
    RISCV_PseudoVLUXSEG7EI16_V_MF4_M1_MASK	= 5599,
    RISCV_PseudoVLUXSEG7EI16_V_MF4_MF2	= 5600,
    RISCV_PseudoVLUXSEG7EI16_V_MF4_MF2_MASK	= 5601,
    RISCV_PseudoVLUXSEG7EI16_V_MF4_MF4	= 5602,
    RISCV_PseudoVLUXSEG7EI16_V_MF4_MF4_MASK	= 5603,
    RISCV_PseudoVLUXSEG7EI16_V_MF4_MF8	= 5604,
    RISCV_PseudoVLUXSEG7EI16_V_MF4_MF8_MASK	= 5605,
    RISCV_PseudoVLUXSEG7EI32_V_M1_M1	= 5606,
    RISCV_PseudoVLUXSEG7EI32_V_M1_M1_MASK	= 5607,
    RISCV_PseudoVLUXSEG7EI32_V_M1_MF2	= 5608,
    RISCV_PseudoVLUXSEG7EI32_V_M1_MF2_MASK	= 5609,
    RISCV_PseudoVLUXSEG7EI32_V_M1_MF4	= 5610,
    RISCV_PseudoVLUXSEG7EI32_V_M1_MF4_MASK	= 5611,
    RISCV_PseudoVLUXSEG7EI32_V_M2_M1	= 5612,
    RISCV_PseudoVLUXSEG7EI32_V_M2_M1_MASK	= 5613,
    RISCV_PseudoVLUXSEG7EI32_V_M2_MF2	= 5614,
    RISCV_PseudoVLUXSEG7EI32_V_M2_MF2_MASK	= 5615,
    RISCV_PseudoVLUXSEG7EI32_V_M4_M1	= 5616,
    RISCV_PseudoVLUXSEG7EI32_V_M4_M1_MASK	= 5617,
    RISCV_PseudoVLUXSEG7EI32_V_MF2_M1	= 5618,
    RISCV_PseudoVLUXSEG7EI32_V_MF2_M1_MASK	= 5619,
    RISCV_PseudoVLUXSEG7EI32_V_MF2_MF2	= 5620,
    RISCV_PseudoVLUXSEG7EI32_V_MF2_MF2_MASK	= 5621,
    RISCV_PseudoVLUXSEG7EI32_V_MF2_MF4	= 5622,
    RISCV_PseudoVLUXSEG7EI32_V_MF2_MF4_MASK	= 5623,
    RISCV_PseudoVLUXSEG7EI32_V_MF2_MF8	= 5624,
    RISCV_PseudoVLUXSEG7EI32_V_MF2_MF8_MASK	= 5625,
    RISCV_PseudoVLUXSEG7EI64_V_M1_M1	= 5626,
    RISCV_PseudoVLUXSEG7EI64_V_M1_M1_MASK	= 5627,
    RISCV_PseudoVLUXSEG7EI64_V_M1_MF2	= 5628,
    RISCV_PseudoVLUXSEG7EI64_V_M1_MF2_MASK	= 5629,
    RISCV_PseudoVLUXSEG7EI64_V_M1_MF4	= 5630,
    RISCV_PseudoVLUXSEG7EI64_V_M1_MF4_MASK	= 5631,
    RISCV_PseudoVLUXSEG7EI64_V_M1_MF8	= 5632,
    RISCV_PseudoVLUXSEG7EI64_V_M1_MF8_MASK	= 5633,
    RISCV_PseudoVLUXSEG7EI64_V_M2_M1	= 5634,
    RISCV_PseudoVLUXSEG7EI64_V_M2_M1_MASK	= 5635,
    RISCV_PseudoVLUXSEG7EI64_V_M2_MF2	= 5636,
    RISCV_PseudoVLUXSEG7EI64_V_M2_MF2_MASK	= 5637,
    RISCV_PseudoVLUXSEG7EI64_V_M2_MF4	= 5638,
    RISCV_PseudoVLUXSEG7EI64_V_M2_MF4_MASK	= 5639,
    RISCV_PseudoVLUXSEG7EI64_V_M4_M1	= 5640,
    RISCV_PseudoVLUXSEG7EI64_V_M4_M1_MASK	= 5641,
    RISCV_PseudoVLUXSEG7EI64_V_M4_MF2	= 5642,
    RISCV_PseudoVLUXSEG7EI64_V_M4_MF2_MASK	= 5643,
    RISCV_PseudoVLUXSEG7EI64_V_M8_M1	= 5644,
    RISCV_PseudoVLUXSEG7EI64_V_M8_M1_MASK	= 5645,
    RISCV_PseudoVLUXSEG7EI8_V_M1_M1	= 5646,
    RISCV_PseudoVLUXSEG7EI8_V_M1_M1_MASK	= 5647,
    RISCV_PseudoVLUXSEG7EI8_V_MF2_M1	= 5648,
    RISCV_PseudoVLUXSEG7EI8_V_MF2_M1_MASK	= 5649,
    RISCV_PseudoVLUXSEG7EI8_V_MF2_MF2	= 5650,
    RISCV_PseudoVLUXSEG7EI8_V_MF2_MF2_MASK	= 5651,
    RISCV_PseudoVLUXSEG7EI8_V_MF4_M1	= 5652,
    RISCV_PseudoVLUXSEG7EI8_V_MF4_M1_MASK	= 5653,
    RISCV_PseudoVLUXSEG7EI8_V_MF4_MF2	= 5654,
    RISCV_PseudoVLUXSEG7EI8_V_MF4_MF2_MASK	= 5655,
    RISCV_PseudoVLUXSEG7EI8_V_MF4_MF4	= 5656,
    RISCV_PseudoVLUXSEG7EI8_V_MF4_MF4_MASK	= 5657,
    RISCV_PseudoVLUXSEG7EI8_V_MF8_M1	= 5658,
    RISCV_PseudoVLUXSEG7EI8_V_MF8_M1_MASK	= 5659,
    RISCV_PseudoVLUXSEG7EI8_V_MF8_MF2	= 5660,
    RISCV_PseudoVLUXSEG7EI8_V_MF8_MF2_MASK	= 5661,
    RISCV_PseudoVLUXSEG7EI8_V_MF8_MF4	= 5662,
    RISCV_PseudoVLUXSEG7EI8_V_MF8_MF4_MASK	= 5663,
    RISCV_PseudoVLUXSEG7EI8_V_MF8_MF8	= 5664,
    RISCV_PseudoVLUXSEG7EI8_V_MF8_MF8_MASK	= 5665,
    RISCV_PseudoVLUXSEG8EI16_V_M1_M1	= 5666,
    RISCV_PseudoVLUXSEG8EI16_V_M1_M1_MASK	= 5667,
    RISCV_PseudoVLUXSEG8EI16_V_M1_MF2	= 5668,
    RISCV_PseudoVLUXSEG8EI16_V_M1_MF2_MASK	= 5669,
    RISCV_PseudoVLUXSEG8EI16_V_M2_M1	= 5670,
    RISCV_PseudoVLUXSEG8EI16_V_M2_M1_MASK	= 5671,
    RISCV_PseudoVLUXSEG8EI16_V_MF2_M1	= 5672,
    RISCV_PseudoVLUXSEG8EI16_V_MF2_M1_MASK	= 5673,
    RISCV_PseudoVLUXSEG8EI16_V_MF2_MF2	= 5674,
    RISCV_PseudoVLUXSEG8EI16_V_MF2_MF2_MASK	= 5675,
    RISCV_PseudoVLUXSEG8EI16_V_MF2_MF4	= 5676,
    RISCV_PseudoVLUXSEG8EI16_V_MF2_MF4_MASK	= 5677,
    RISCV_PseudoVLUXSEG8EI16_V_MF4_M1	= 5678,
    RISCV_PseudoVLUXSEG8EI16_V_MF4_M1_MASK	= 5679,
    RISCV_PseudoVLUXSEG8EI16_V_MF4_MF2	= 5680,
    RISCV_PseudoVLUXSEG8EI16_V_MF4_MF2_MASK	= 5681,
    RISCV_PseudoVLUXSEG8EI16_V_MF4_MF4	= 5682,
    RISCV_PseudoVLUXSEG8EI16_V_MF4_MF4_MASK	= 5683,
    RISCV_PseudoVLUXSEG8EI16_V_MF4_MF8	= 5684,
    RISCV_PseudoVLUXSEG8EI16_V_MF4_MF8_MASK	= 5685,
    RISCV_PseudoVLUXSEG8EI32_V_M1_M1	= 5686,
    RISCV_PseudoVLUXSEG8EI32_V_M1_M1_MASK	= 5687,
    RISCV_PseudoVLUXSEG8EI32_V_M1_MF2	= 5688,
    RISCV_PseudoVLUXSEG8EI32_V_M1_MF2_MASK	= 5689,
    RISCV_PseudoVLUXSEG8EI32_V_M1_MF4	= 5690,
    RISCV_PseudoVLUXSEG8EI32_V_M1_MF4_MASK	= 5691,
    RISCV_PseudoVLUXSEG8EI32_V_M2_M1	= 5692,
    RISCV_PseudoVLUXSEG8EI32_V_M2_M1_MASK	= 5693,
    RISCV_PseudoVLUXSEG8EI32_V_M2_MF2	= 5694,
    RISCV_PseudoVLUXSEG8EI32_V_M2_MF2_MASK	= 5695,
    RISCV_PseudoVLUXSEG8EI32_V_M4_M1	= 5696,
    RISCV_PseudoVLUXSEG8EI32_V_M4_M1_MASK	= 5697,
    RISCV_PseudoVLUXSEG8EI32_V_MF2_M1	= 5698,
    RISCV_PseudoVLUXSEG8EI32_V_MF2_M1_MASK	= 5699,
    RISCV_PseudoVLUXSEG8EI32_V_MF2_MF2	= 5700,
    RISCV_PseudoVLUXSEG8EI32_V_MF2_MF2_MASK	= 5701,
    RISCV_PseudoVLUXSEG8EI32_V_MF2_MF4	= 5702,
    RISCV_PseudoVLUXSEG8EI32_V_MF2_MF4_MASK	= 5703,
    RISCV_PseudoVLUXSEG8EI32_V_MF2_MF8	= 5704,
    RISCV_PseudoVLUXSEG8EI32_V_MF2_MF8_MASK	= 5705,
    RISCV_PseudoVLUXSEG8EI64_V_M1_M1	= 5706,
    RISCV_PseudoVLUXSEG8EI64_V_M1_M1_MASK	= 5707,
    RISCV_PseudoVLUXSEG8EI64_V_M1_MF2	= 5708,
    RISCV_PseudoVLUXSEG8EI64_V_M1_MF2_MASK	= 5709,
    RISCV_PseudoVLUXSEG8EI64_V_M1_MF4	= 5710,
    RISCV_PseudoVLUXSEG8EI64_V_M1_MF4_MASK	= 5711,
    RISCV_PseudoVLUXSEG8EI64_V_M1_MF8	= 5712,
    RISCV_PseudoVLUXSEG8EI64_V_M1_MF8_MASK	= 5713,
    RISCV_PseudoVLUXSEG8EI64_V_M2_M1	= 5714,
    RISCV_PseudoVLUXSEG8EI64_V_M2_M1_MASK	= 5715,
    RISCV_PseudoVLUXSEG8EI64_V_M2_MF2	= 5716,
    RISCV_PseudoVLUXSEG8EI64_V_M2_MF2_MASK	= 5717,
    RISCV_PseudoVLUXSEG8EI64_V_M2_MF4	= 5718,
    RISCV_PseudoVLUXSEG8EI64_V_M2_MF4_MASK	= 5719,
    RISCV_PseudoVLUXSEG8EI64_V_M4_M1	= 5720,
    RISCV_PseudoVLUXSEG8EI64_V_M4_M1_MASK	= 5721,
    RISCV_PseudoVLUXSEG8EI64_V_M4_MF2	= 5722,
    RISCV_PseudoVLUXSEG8EI64_V_M4_MF2_MASK	= 5723,
    RISCV_PseudoVLUXSEG8EI64_V_M8_M1	= 5724,
    RISCV_PseudoVLUXSEG8EI64_V_M8_M1_MASK	= 5725,
    RISCV_PseudoVLUXSEG8EI8_V_M1_M1	= 5726,
    RISCV_PseudoVLUXSEG8EI8_V_M1_M1_MASK	= 5727,
    RISCV_PseudoVLUXSEG8EI8_V_MF2_M1	= 5728,
    RISCV_PseudoVLUXSEG8EI8_V_MF2_M1_MASK	= 5729,
    RISCV_PseudoVLUXSEG8EI8_V_MF2_MF2	= 5730,
    RISCV_PseudoVLUXSEG8EI8_V_MF2_MF2_MASK	= 5731,
    RISCV_PseudoVLUXSEG8EI8_V_MF4_M1	= 5732,
    RISCV_PseudoVLUXSEG8EI8_V_MF4_M1_MASK	= 5733,
    RISCV_PseudoVLUXSEG8EI8_V_MF4_MF2	= 5734,
    RISCV_PseudoVLUXSEG8EI8_V_MF4_MF2_MASK	= 5735,
    RISCV_PseudoVLUXSEG8EI8_V_MF4_MF4	= 5736,
    RISCV_PseudoVLUXSEG8EI8_V_MF4_MF4_MASK	= 5737,
    RISCV_PseudoVLUXSEG8EI8_V_MF8_M1	= 5738,
    RISCV_PseudoVLUXSEG8EI8_V_MF8_M1_MASK	= 5739,
    RISCV_PseudoVLUXSEG8EI8_V_MF8_MF2	= 5740,
    RISCV_PseudoVLUXSEG8EI8_V_MF8_MF2_MASK	= 5741,
    RISCV_PseudoVLUXSEG8EI8_V_MF8_MF4	= 5742,
    RISCV_PseudoVLUXSEG8EI8_V_MF8_MF4_MASK	= 5743,
    RISCV_PseudoVLUXSEG8EI8_V_MF8_MF8	= 5744,
    RISCV_PseudoVLUXSEG8EI8_V_MF8_MF8_MASK	= 5745,
    RISCV_PseudoVMACC_VV_M1	= 5746,
    RISCV_PseudoVMACC_VV_M1_MASK	= 5747,
    RISCV_PseudoVMACC_VV_M2	= 5748,
    RISCV_PseudoVMACC_VV_M2_MASK	= 5749,
    RISCV_PseudoVMACC_VV_M4	= 5750,
    RISCV_PseudoVMACC_VV_M4_MASK	= 5751,
    RISCV_PseudoVMACC_VV_M8	= 5752,
    RISCV_PseudoVMACC_VV_M8_MASK	= 5753,
    RISCV_PseudoVMACC_VV_MF2	= 5754,
    RISCV_PseudoVMACC_VV_MF2_MASK	= 5755,
    RISCV_PseudoVMACC_VV_MF4	= 5756,
    RISCV_PseudoVMACC_VV_MF4_MASK	= 5757,
    RISCV_PseudoVMACC_VV_MF8	= 5758,
    RISCV_PseudoVMACC_VV_MF8_MASK	= 5759,
    RISCV_PseudoVMACC_VX_M1	= 5760,
    RISCV_PseudoVMACC_VX_M1_MASK	= 5761,
    RISCV_PseudoVMACC_VX_M2	= 5762,
    RISCV_PseudoVMACC_VX_M2_MASK	= 5763,
    RISCV_PseudoVMACC_VX_M4	= 5764,
    RISCV_PseudoVMACC_VX_M4_MASK	= 5765,
    RISCV_PseudoVMACC_VX_M8	= 5766,
    RISCV_PseudoVMACC_VX_M8_MASK	= 5767,
    RISCV_PseudoVMACC_VX_MF2	= 5768,
    RISCV_PseudoVMACC_VX_MF2_MASK	= 5769,
    RISCV_PseudoVMACC_VX_MF4	= 5770,
    RISCV_PseudoVMACC_VX_MF4_MASK	= 5771,
    RISCV_PseudoVMACC_VX_MF8	= 5772,
    RISCV_PseudoVMACC_VX_MF8_MASK	= 5773,
    RISCV_PseudoVMADC_VIM_M1	= 5774,
    RISCV_PseudoVMADC_VIM_M2	= 5775,
    RISCV_PseudoVMADC_VIM_M4	= 5776,
    RISCV_PseudoVMADC_VIM_M8	= 5777,
    RISCV_PseudoVMADC_VIM_MF2	= 5778,
    RISCV_PseudoVMADC_VIM_MF4	= 5779,
    RISCV_PseudoVMADC_VIM_MF8	= 5780,
    RISCV_PseudoVMADC_VI_M1	= 5781,
    RISCV_PseudoVMADC_VI_M2	= 5782,
    RISCV_PseudoVMADC_VI_M4	= 5783,
    RISCV_PseudoVMADC_VI_M8	= 5784,
    RISCV_PseudoVMADC_VI_MF2	= 5785,
    RISCV_PseudoVMADC_VI_MF4	= 5786,
    RISCV_PseudoVMADC_VI_MF8	= 5787,
    RISCV_PseudoVMADC_VVM_M1	= 5788,
    RISCV_PseudoVMADC_VVM_M2	= 5789,
    RISCV_PseudoVMADC_VVM_M4	= 5790,
    RISCV_PseudoVMADC_VVM_M8	= 5791,
    RISCV_PseudoVMADC_VVM_MF2	= 5792,
    RISCV_PseudoVMADC_VVM_MF4	= 5793,
    RISCV_PseudoVMADC_VVM_MF8	= 5794,
    RISCV_PseudoVMADC_VV_M1	= 5795,
    RISCV_PseudoVMADC_VV_M2	= 5796,
    RISCV_PseudoVMADC_VV_M4	= 5797,
    RISCV_PseudoVMADC_VV_M8	= 5798,
    RISCV_PseudoVMADC_VV_MF2	= 5799,
    RISCV_PseudoVMADC_VV_MF4	= 5800,
    RISCV_PseudoVMADC_VV_MF8	= 5801,
    RISCV_PseudoVMADC_VXM_M1	= 5802,
    RISCV_PseudoVMADC_VXM_M2	= 5803,
    RISCV_PseudoVMADC_VXM_M4	= 5804,
    RISCV_PseudoVMADC_VXM_M8	= 5805,
    RISCV_PseudoVMADC_VXM_MF2	= 5806,
    RISCV_PseudoVMADC_VXM_MF4	= 5807,
    RISCV_PseudoVMADC_VXM_MF8	= 5808,
    RISCV_PseudoVMADC_VX_M1	= 5809,
    RISCV_PseudoVMADC_VX_M2	= 5810,
    RISCV_PseudoVMADC_VX_M4	= 5811,
    RISCV_PseudoVMADC_VX_M8	= 5812,
    RISCV_PseudoVMADC_VX_MF2	= 5813,
    RISCV_PseudoVMADC_VX_MF4	= 5814,
    RISCV_PseudoVMADC_VX_MF8	= 5815,
    RISCV_PseudoVMADD_VV_M1	= 5816,
    RISCV_PseudoVMADD_VV_M1_MASK	= 5817,
    RISCV_PseudoVMADD_VV_M2	= 5818,
    RISCV_PseudoVMADD_VV_M2_MASK	= 5819,
    RISCV_PseudoVMADD_VV_M4	= 5820,
    RISCV_PseudoVMADD_VV_M4_MASK	= 5821,
    RISCV_PseudoVMADD_VV_M8	= 5822,
    RISCV_PseudoVMADD_VV_M8_MASK	= 5823,
    RISCV_PseudoVMADD_VV_MF2	= 5824,
    RISCV_PseudoVMADD_VV_MF2_MASK	= 5825,
    RISCV_PseudoVMADD_VV_MF4	= 5826,
    RISCV_PseudoVMADD_VV_MF4_MASK	= 5827,
    RISCV_PseudoVMADD_VV_MF8	= 5828,
    RISCV_PseudoVMADD_VV_MF8_MASK	= 5829,
    RISCV_PseudoVMADD_VX_M1	= 5830,
    RISCV_PseudoVMADD_VX_M1_MASK	= 5831,
    RISCV_PseudoVMADD_VX_M2	= 5832,
    RISCV_PseudoVMADD_VX_M2_MASK	= 5833,
    RISCV_PseudoVMADD_VX_M4	= 5834,
    RISCV_PseudoVMADD_VX_M4_MASK	= 5835,
    RISCV_PseudoVMADD_VX_M8	= 5836,
    RISCV_PseudoVMADD_VX_M8_MASK	= 5837,
    RISCV_PseudoVMADD_VX_MF2	= 5838,
    RISCV_PseudoVMADD_VX_MF2_MASK	= 5839,
    RISCV_PseudoVMADD_VX_MF4	= 5840,
    RISCV_PseudoVMADD_VX_MF4_MASK	= 5841,
    RISCV_PseudoVMADD_VX_MF8	= 5842,
    RISCV_PseudoVMADD_VX_MF8_MASK	= 5843,
    RISCV_PseudoVMANDN_MM_M1	= 5844,
    RISCV_PseudoVMANDN_MM_M2	= 5845,
    RISCV_PseudoVMANDN_MM_M4	= 5846,
    RISCV_PseudoVMANDN_MM_M8	= 5847,
    RISCV_PseudoVMANDN_MM_MF2	= 5848,
    RISCV_PseudoVMANDN_MM_MF4	= 5849,
    RISCV_PseudoVMANDN_MM_MF8	= 5850,
    RISCV_PseudoVMAND_MM_M1	= 5851,
    RISCV_PseudoVMAND_MM_M2	= 5852,
    RISCV_PseudoVMAND_MM_M4	= 5853,
    RISCV_PseudoVMAND_MM_M8	= 5854,
    RISCV_PseudoVMAND_MM_MF2	= 5855,
    RISCV_PseudoVMAND_MM_MF4	= 5856,
    RISCV_PseudoVMAND_MM_MF8	= 5857,
    RISCV_PseudoVMAXU_VV_M1	= 5858,
    RISCV_PseudoVMAXU_VV_M1_MASK	= 5859,
    RISCV_PseudoVMAXU_VV_M2	= 5860,
    RISCV_PseudoVMAXU_VV_M2_MASK	= 5861,
    RISCV_PseudoVMAXU_VV_M4	= 5862,
    RISCV_PseudoVMAXU_VV_M4_MASK	= 5863,
    RISCV_PseudoVMAXU_VV_M8	= 5864,
    RISCV_PseudoVMAXU_VV_M8_MASK	= 5865,
    RISCV_PseudoVMAXU_VV_MF2	= 5866,
    RISCV_PseudoVMAXU_VV_MF2_MASK	= 5867,
    RISCV_PseudoVMAXU_VV_MF4	= 5868,
    RISCV_PseudoVMAXU_VV_MF4_MASK	= 5869,
    RISCV_PseudoVMAXU_VV_MF8	= 5870,
    RISCV_PseudoVMAXU_VV_MF8_MASK	= 5871,
    RISCV_PseudoVMAXU_VX_M1	= 5872,
    RISCV_PseudoVMAXU_VX_M1_MASK	= 5873,
    RISCV_PseudoVMAXU_VX_M2	= 5874,
    RISCV_PseudoVMAXU_VX_M2_MASK	= 5875,
    RISCV_PseudoVMAXU_VX_M4	= 5876,
    RISCV_PseudoVMAXU_VX_M4_MASK	= 5877,
    RISCV_PseudoVMAXU_VX_M8	= 5878,
    RISCV_PseudoVMAXU_VX_M8_MASK	= 5879,
    RISCV_PseudoVMAXU_VX_MF2	= 5880,
    RISCV_PseudoVMAXU_VX_MF2_MASK	= 5881,
    RISCV_PseudoVMAXU_VX_MF4	= 5882,
    RISCV_PseudoVMAXU_VX_MF4_MASK	= 5883,
    RISCV_PseudoVMAXU_VX_MF8	= 5884,
    RISCV_PseudoVMAXU_VX_MF8_MASK	= 5885,
    RISCV_PseudoVMAX_VV_M1	= 5886,
    RISCV_PseudoVMAX_VV_M1_MASK	= 5887,
    RISCV_PseudoVMAX_VV_M2	= 5888,
    RISCV_PseudoVMAX_VV_M2_MASK	= 5889,
    RISCV_PseudoVMAX_VV_M4	= 5890,
    RISCV_PseudoVMAX_VV_M4_MASK	= 5891,
    RISCV_PseudoVMAX_VV_M8	= 5892,
    RISCV_PseudoVMAX_VV_M8_MASK	= 5893,
    RISCV_PseudoVMAX_VV_MF2	= 5894,
    RISCV_PseudoVMAX_VV_MF2_MASK	= 5895,
    RISCV_PseudoVMAX_VV_MF4	= 5896,
    RISCV_PseudoVMAX_VV_MF4_MASK	= 5897,
    RISCV_PseudoVMAX_VV_MF8	= 5898,
    RISCV_PseudoVMAX_VV_MF8_MASK	= 5899,
    RISCV_PseudoVMAX_VX_M1	= 5900,
    RISCV_PseudoVMAX_VX_M1_MASK	= 5901,
    RISCV_PseudoVMAX_VX_M2	= 5902,
    RISCV_PseudoVMAX_VX_M2_MASK	= 5903,
    RISCV_PseudoVMAX_VX_M4	= 5904,
    RISCV_PseudoVMAX_VX_M4_MASK	= 5905,
    RISCV_PseudoVMAX_VX_M8	= 5906,
    RISCV_PseudoVMAX_VX_M8_MASK	= 5907,
    RISCV_PseudoVMAX_VX_MF2	= 5908,
    RISCV_PseudoVMAX_VX_MF2_MASK	= 5909,
    RISCV_PseudoVMAX_VX_MF4	= 5910,
    RISCV_PseudoVMAX_VX_MF4_MASK	= 5911,
    RISCV_PseudoVMAX_VX_MF8	= 5912,
    RISCV_PseudoVMAX_VX_MF8_MASK	= 5913,
    RISCV_PseudoVMCLR_M_B1	= 5914,
    RISCV_PseudoVMCLR_M_B16	= 5915,
    RISCV_PseudoVMCLR_M_B2	= 5916,
    RISCV_PseudoVMCLR_M_B32	= 5917,
    RISCV_PseudoVMCLR_M_B4	= 5918,
    RISCV_PseudoVMCLR_M_B64	= 5919,
    RISCV_PseudoVMCLR_M_B8	= 5920,
    RISCV_PseudoVMERGE_VIM_M1	= 5921,
    RISCV_PseudoVMERGE_VIM_M2	= 5922,
    RISCV_PseudoVMERGE_VIM_M4	= 5923,
    RISCV_PseudoVMERGE_VIM_M8	= 5924,
    RISCV_PseudoVMERGE_VIM_MF2	= 5925,
    RISCV_PseudoVMERGE_VIM_MF4	= 5926,
    RISCV_PseudoVMERGE_VIM_MF8	= 5927,
    RISCV_PseudoVMERGE_VVM_M1	= 5928,
    RISCV_PseudoVMERGE_VVM_M2	= 5929,
    RISCV_PseudoVMERGE_VVM_M4	= 5930,
    RISCV_PseudoVMERGE_VVM_M8	= 5931,
    RISCV_PseudoVMERGE_VVM_MF2	= 5932,
    RISCV_PseudoVMERGE_VVM_MF4	= 5933,
    RISCV_PseudoVMERGE_VVM_MF8	= 5934,
    RISCV_PseudoVMERGE_VXM_M1	= 5935,
    RISCV_PseudoVMERGE_VXM_M2	= 5936,
    RISCV_PseudoVMERGE_VXM_M4	= 5937,
    RISCV_PseudoVMERGE_VXM_M8	= 5938,
    RISCV_PseudoVMERGE_VXM_MF2	= 5939,
    RISCV_PseudoVMERGE_VXM_MF4	= 5940,
    RISCV_PseudoVMERGE_VXM_MF8	= 5941,
    RISCV_PseudoVMFEQ_VFPR16_M1	= 5942,
    RISCV_PseudoVMFEQ_VFPR16_M1_MASK	= 5943,
    RISCV_PseudoVMFEQ_VFPR16_M2	= 5944,
    RISCV_PseudoVMFEQ_VFPR16_M2_MASK	= 5945,
    RISCV_PseudoVMFEQ_VFPR16_M4	= 5946,
    RISCV_PseudoVMFEQ_VFPR16_M4_MASK	= 5947,
    RISCV_PseudoVMFEQ_VFPR16_M8	= 5948,
    RISCV_PseudoVMFEQ_VFPR16_M8_MASK	= 5949,
    RISCV_PseudoVMFEQ_VFPR16_MF2	= 5950,
    RISCV_PseudoVMFEQ_VFPR16_MF2_MASK	= 5951,
    RISCV_PseudoVMFEQ_VFPR16_MF4	= 5952,
    RISCV_PseudoVMFEQ_VFPR16_MF4_MASK	= 5953,
    RISCV_PseudoVMFEQ_VFPR32_M1	= 5954,
    RISCV_PseudoVMFEQ_VFPR32_M1_MASK	= 5955,
    RISCV_PseudoVMFEQ_VFPR32_M2	= 5956,
    RISCV_PseudoVMFEQ_VFPR32_M2_MASK	= 5957,
    RISCV_PseudoVMFEQ_VFPR32_M4	= 5958,
    RISCV_PseudoVMFEQ_VFPR32_M4_MASK	= 5959,
    RISCV_PseudoVMFEQ_VFPR32_M8	= 5960,
    RISCV_PseudoVMFEQ_VFPR32_M8_MASK	= 5961,
    RISCV_PseudoVMFEQ_VFPR32_MF2	= 5962,
    RISCV_PseudoVMFEQ_VFPR32_MF2_MASK	= 5963,
    RISCV_PseudoVMFEQ_VFPR64_M1	= 5964,
    RISCV_PseudoVMFEQ_VFPR64_M1_MASK	= 5965,
    RISCV_PseudoVMFEQ_VFPR64_M2	= 5966,
    RISCV_PseudoVMFEQ_VFPR64_M2_MASK	= 5967,
    RISCV_PseudoVMFEQ_VFPR64_M4	= 5968,
    RISCV_PseudoVMFEQ_VFPR64_M4_MASK	= 5969,
    RISCV_PseudoVMFEQ_VFPR64_M8	= 5970,
    RISCV_PseudoVMFEQ_VFPR64_M8_MASK	= 5971,
    RISCV_PseudoVMFEQ_VV_M1	= 5972,
    RISCV_PseudoVMFEQ_VV_M1_MASK	= 5973,
    RISCV_PseudoVMFEQ_VV_M2	= 5974,
    RISCV_PseudoVMFEQ_VV_M2_MASK	= 5975,
    RISCV_PseudoVMFEQ_VV_M4	= 5976,
    RISCV_PseudoVMFEQ_VV_M4_MASK	= 5977,
    RISCV_PseudoVMFEQ_VV_M8	= 5978,
    RISCV_PseudoVMFEQ_VV_M8_MASK	= 5979,
    RISCV_PseudoVMFEQ_VV_MF2	= 5980,
    RISCV_PseudoVMFEQ_VV_MF2_MASK	= 5981,
    RISCV_PseudoVMFEQ_VV_MF4	= 5982,
    RISCV_PseudoVMFEQ_VV_MF4_MASK	= 5983,
    RISCV_PseudoVMFGE_VFPR16_M1	= 5984,
    RISCV_PseudoVMFGE_VFPR16_M1_MASK	= 5985,
    RISCV_PseudoVMFGE_VFPR16_M2	= 5986,
    RISCV_PseudoVMFGE_VFPR16_M2_MASK	= 5987,
    RISCV_PseudoVMFGE_VFPR16_M4	= 5988,
    RISCV_PseudoVMFGE_VFPR16_M4_MASK	= 5989,
    RISCV_PseudoVMFGE_VFPR16_M8	= 5990,
    RISCV_PseudoVMFGE_VFPR16_M8_MASK	= 5991,
    RISCV_PseudoVMFGE_VFPR16_MF2	= 5992,
    RISCV_PseudoVMFGE_VFPR16_MF2_MASK	= 5993,
    RISCV_PseudoVMFGE_VFPR16_MF4	= 5994,
    RISCV_PseudoVMFGE_VFPR16_MF4_MASK	= 5995,
    RISCV_PseudoVMFGE_VFPR32_M1	= 5996,
    RISCV_PseudoVMFGE_VFPR32_M1_MASK	= 5997,
    RISCV_PseudoVMFGE_VFPR32_M2	= 5998,
    RISCV_PseudoVMFGE_VFPR32_M2_MASK	= 5999,
    RISCV_PseudoVMFGE_VFPR32_M4	= 6000,
    RISCV_PseudoVMFGE_VFPR32_M4_MASK	= 6001,
    RISCV_PseudoVMFGE_VFPR32_M8	= 6002,
    RISCV_PseudoVMFGE_VFPR32_M8_MASK	= 6003,
    RISCV_PseudoVMFGE_VFPR32_MF2	= 6004,
    RISCV_PseudoVMFGE_VFPR32_MF2_MASK	= 6005,
    RISCV_PseudoVMFGE_VFPR64_M1	= 6006,
    RISCV_PseudoVMFGE_VFPR64_M1_MASK	= 6007,
    RISCV_PseudoVMFGE_VFPR64_M2	= 6008,
    RISCV_PseudoVMFGE_VFPR64_M2_MASK	= 6009,
    RISCV_PseudoVMFGE_VFPR64_M4	= 6010,
    RISCV_PseudoVMFGE_VFPR64_M4_MASK	= 6011,
    RISCV_PseudoVMFGE_VFPR64_M8	= 6012,
    RISCV_PseudoVMFGE_VFPR64_M8_MASK	= 6013,
    RISCV_PseudoVMFGT_VFPR16_M1	= 6014,
    RISCV_PseudoVMFGT_VFPR16_M1_MASK	= 6015,
    RISCV_PseudoVMFGT_VFPR16_M2	= 6016,
    RISCV_PseudoVMFGT_VFPR16_M2_MASK	= 6017,
    RISCV_PseudoVMFGT_VFPR16_M4	= 6018,
    RISCV_PseudoVMFGT_VFPR16_M4_MASK	= 6019,
    RISCV_PseudoVMFGT_VFPR16_M8	= 6020,
    RISCV_PseudoVMFGT_VFPR16_M8_MASK	= 6021,
    RISCV_PseudoVMFGT_VFPR16_MF2	= 6022,
    RISCV_PseudoVMFGT_VFPR16_MF2_MASK	= 6023,
    RISCV_PseudoVMFGT_VFPR16_MF4	= 6024,
    RISCV_PseudoVMFGT_VFPR16_MF4_MASK	= 6025,
    RISCV_PseudoVMFGT_VFPR32_M1	= 6026,
    RISCV_PseudoVMFGT_VFPR32_M1_MASK	= 6027,
    RISCV_PseudoVMFGT_VFPR32_M2	= 6028,
    RISCV_PseudoVMFGT_VFPR32_M2_MASK	= 6029,
    RISCV_PseudoVMFGT_VFPR32_M4	= 6030,
    RISCV_PseudoVMFGT_VFPR32_M4_MASK	= 6031,
    RISCV_PseudoVMFGT_VFPR32_M8	= 6032,
    RISCV_PseudoVMFGT_VFPR32_M8_MASK	= 6033,
    RISCV_PseudoVMFGT_VFPR32_MF2	= 6034,
    RISCV_PseudoVMFGT_VFPR32_MF2_MASK	= 6035,
    RISCV_PseudoVMFGT_VFPR64_M1	= 6036,
    RISCV_PseudoVMFGT_VFPR64_M1_MASK	= 6037,
    RISCV_PseudoVMFGT_VFPR64_M2	= 6038,
    RISCV_PseudoVMFGT_VFPR64_M2_MASK	= 6039,
    RISCV_PseudoVMFGT_VFPR64_M4	= 6040,
    RISCV_PseudoVMFGT_VFPR64_M4_MASK	= 6041,
    RISCV_PseudoVMFGT_VFPR64_M8	= 6042,
    RISCV_PseudoVMFGT_VFPR64_M8_MASK	= 6043,
    RISCV_PseudoVMFLE_VFPR16_M1	= 6044,
    RISCV_PseudoVMFLE_VFPR16_M1_MASK	= 6045,
    RISCV_PseudoVMFLE_VFPR16_M2	= 6046,
    RISCV_PseudoVMFLE_VFPR16_M2_MASK	= 6047,
    RISCV_PseudoVMFLE_VFPR16_M4	= 6048,
    RISCV_PseudoVMFLE_VFPR16_M4_MASK	= 6049,
    RISCV_PseudoVMFLE_VFPR16_M8	= 6050,
    RISCV_PseudoVMFLE_VFPR16_M8_MASK	= 6051,
    RISCV_PseudoVMFLE_VFPR16_MF2	= 6052,
    RISCV_PseudoVMFLE_VFPR16_MF2_MASK	= 6053,
    RISCV_PseudoVMFLE_VFPR16_MF4	= 6054,
    RISCV_PseudoVMFLE_VFPR16_MF4_MASK	= 6055,
    RISCV_PseudoVMFLE_VFPR32_M1	= 6056,
    RISCV_PseudoVMFLE_VFPR32_M1_MASK	= 6057,
    RISCV_PseudoVMFLE_VFPR32_M2	= 6058,
    RISCV_PseudoVMFLE_VFPR32_M2_MASK	= 6059,
    RISCV_PseudoVMFLE_VFPR32_M4	= 6060,
    RISCV_PseudoVMFLE_VFPR32_M4_MASK	= 6061,
    RISCV_PseudoVMFLE_VFPR32_M8	= 6062,
    RISCV_PseudoVMFLE_VFPR32_M8_MASK	= 6063,
    RISCV_PseudoVMFLE_VFPR32_MF2	= 6064,
    RISCV_PseudoVMFLE_VFPR32_MF2_MASK	= 6065,
    RISCV_PseudoVMFLE_VFPR64_M1	= 6066,
    RISCV_PseudoVMFLE_VFPR64_M1_MASK	= 6067,
    RISCV_PseudoVMFLE_VFPR64_M2	= 6068,
    RISCV_PseudoVMFLE_VFPR64_M2_MASK	= 6069,
    RISCV_PseudoVMFLE_VFPR64_M4	= 6070,
    RISCV_PseudoVMFLE_VFPR64_M4_MASK	= 6071,
    RISCV_PseudoVMFLE_VFPR64_M8	= 6072,
    RISCV_PseudoVMFLE_VFPR64_M8_MASK	= 6073,
    RISCV_PseudoVMFLE_VV_M1	= 6074,
    RISCV_PseudoVMFLE_VV_M1_MASK	= 6075,
    RISCV_PseudoVMFLE_VV_M2	= 6076,
    RISCV_PseudoVMFLE_VV_M2_MASK	= 6077,
    RISCV_PseudoVMFLE_VV_M4	= 6078,
    RISCV_PseudoVMFLE_VV_M4_MASK	= 6079,
    RISCV_PseudoVMFLE_VV_M8	= 6080,
    RISCV_PseudoVMFLE_VV_M8_MASK	= 6081,
    RISCV_PseudoVMFLE_VV_MF2	= 6082,
    RISCV_PseudoVMFLE_VV_MF2_MASK	= 6083,
    RISCV_PseudoVMFLE_VV_MF4	= 6084,
    RISCV_PseudoVMFLE_VV_MF4_MASK	= 6085,
    RISCV_PseudoVMFLT_VFPR16_M1	= 6086,
    RISCV_PseudoVMFLT_VFPR16_M1_MASK	= 6087,
    RISCV_PseudoVMFLT_VFPR16_M2	= 6088,
    RISCV_PseudoVMFLT_VFPR16_M2_MASK	= 6089,
    RISCV_PseudoVMFLT_VFPR16_M4	= 6090,
    RISCV_PseudoVMFLT_VFPR16_M4_MASK	= 6091,
    RISCV_PseudoVMFLT_VFPR16_M8	= 6092,
    RISCV_PseudoVMFLT_VFPR16_M8_MASK	= 6093,
    RISCV_PseudoVMFLT_VFPR16_MF2	= 6094,
    RISCV_PseudoVMFLT_VFPR16_MF2_MASK	= 6095,
    RISCV_PseudoVMFLT_VFPR16_MF4	= 6096,
    RISCV_PseudoVMFLT_VFPR16_MF4_MASK	= 6097,
    RISCV_PseudoVMFLT_VFPR32_M1	= 6098,
    RISCV_PseudoVMFLT_VFPR32_M1_MASK	= 6099,
    RISCV_PseudoVMFLT_VFPR32_M2	= 6100,
    RISCV_PseudoVMFLT_VFPR32_M2_MASK	= 6101,
    RISCV_PseudoVMFLT_VFPR32_M4	= 6102,
    RISCV_PseudoVMFLT_VFPR32_M4_MASK	= 6103,
    RISCV_PseudoVMFLT_VFPR32_M8	= 6104,
    RISCV_PseudoVMFLT_VFPR32_M8_MASK	= 6105,
    RISCV_PseudoVMFLT_VFPR32_MF2	= 6106,
    RISCV_PseudoVMFLT_VFPR32_MF2_MASK	= 6107,
    RISCV_PseudoVMFLT_VFPR64_M1	= 6108,
    RISCV_PseudoVMFLT_VFPR64_M1_MASK	= 6109,
    RISCV_PseudoVMFLT_VFPR64_M2	= 6110,
    RISCV_PseudoVMFLT_VFPR64_M2_MASK	= 6111,
    RISCV_PseudoVMFLT_VFPR64_M4	= 6112,
    RISCV_PseudoVMFLT_VFPR64_M4_MASK	= 6113,
    RISCV_PseudoVMFLT_VFPR64_M8	= 6114,
    RISCV_PseudoVMFLT_VFPR64_M8_MASK	= 6115,
    RISCV_PseudoVMFLT_VV_M1	= 6116,
    RISCV_PseudoVMFLT_VV_M1_MASK	= 6117,
    RISCV_PseudoVMFLT_VV_M2	= 6118,
    RISCV_PseudoVMFLT_VV_M2_MASK	= 6119,
    RISCV_PseudoVMFLT_VV_M4	= 6120,
    RISCV_PseudoVMFLT_VV_M4_MASK	= 6121,
    RISCV_PseudoVMFLT_VV_M8	= 6122,
    RISCV_PseudoVMFLT_VV_M8_MASK	= 6123,
    RISCV_PseudoVMFLT_VV_MF2	= 6124,
    RISCV_PseudoVMFLT_VV_MF2_MASK	= 6125,
    RISCV_PseudoVMFLT_VV_MF4	= 6126,
    RISCV_PseudoVMFLT_VV_MF4_MASK	= 6127,
    RISCV_PseudoVMFNE_VFPR16_M1	= 6128,
    RISCV_PseudoVMFNE_VFPR16_M1_MASK	= 6129,
    RISCV_PseudoVMFNE_VFPR16_M2	= 6130,
    RISCV_PseudoVMFNE_VFPR16_M2_MASK	= 6131,
    RISCV_PseudoVMFNE_VFPR16_M4	= 6132,
    RISCV_PseudoVMFNE_VFPR16_M4_MASK	= 6133,
    RISCV_PseudoVMFNE_VFPR16_M8	= 6134,
    RISCV_PseudoVMFNE_VFPR16_M8_MASK	= 6135,
    RISCV_PseudoVMFNE_VFPR16_MF2	= 6136,
    RISCV_PseudoVMFNE_VFPR16_MF2_MASK	= 6137,
    RISCV_PseudoVMFNE_VFPR16_MF4	= 6138,
    RISCV_PseudoVMFNE_VFPR16_MF4_MASK	= 6139,
    RISCV_PseudoVMFNE_VFPR32_M1	= 6140,
    RISCV_PseudoVMFNE_VFPR32_M1_MASK	= 6141,
    RISCV_PseudoVMFNE_VFPR32_M2	= 6142,
    RISCV_PseudoVMFNE_VFPR32_M2_MASK	= 6143,
    RISCV_PseudoVMFNE_VFPR32_M4	= 6144,
    RISCV_PseudoVMFNE_VFPR32_M4_MASK	= 6145,
    RISCV_PseudoVMFNE_VFPR32_M8	= 6146,
    RISCV_PseudoVMFNE_VFPR32_M8_MASK	= 6147,
    RISCV_PseudoVMFNE_VFPR32_MF2	= 6148,
    RISCV_PseudoVMFNE_VFPR32_MF2_MASK	= 6149,
    RISCV_PseudoVMFNE_VFPR64_M1	= 6150,
    RISCV_PseudoVMFNE_VFPR64_M1_MASK	= 6151,
    RISCV_PseudoVMFNE_VFPR64_M2	= 6152,
    RISCV_PseudoVMFNE_VFPR64_M2_MASK	= 6153,
    RISCV_PseudoVMFNE_VFPR64_M4	= 6154,
    RISCV_PseudoVMFNE_VFPR64_M4_MASK	= 6155,
    RISCV_PseudoVMFNE_VFPR64_M8	= 6156,
    RISCV_PseudoVMFNE_VFPR64_M8_MASK	= 6157,
    RISCV_PseudoVMFNE_VV_M1	= 6158,
    RISCV_PseudoVMFNE_VV_M1_MASK	= 6159,
    RISCV_PseudoVMFNE_VV_M2	= 6160,
    RISCV_PseudoVMFNE_VV_M2_MASK	= 6161,
    RISCV_PseudoVMFNE_VV_M4	= 6162,
    RISCV_PseudoVMFNE_VV_M4_MASK	= 6163,
    RISCV_PseudoVMFNE_VV_M8	= 6164,
    RISCV_PseudoVMFNE_VV_M8_MASK	= 6165,
    RISCV_PseudoVMFNE_VV_MF2	= 6166,
    RISCV_PseudoVMFNE_VV_MF2_MASK	= 6167,
    RISCV_PseudoVMFNE_VV_MF4	= 6168,
    RISCV_PseudoVMFNE_VV_MF4_MASK	= 6169,
    RISCV_PseudoVMINU_VV_M1	= 6170,
    RISCV_PseudoVMINU_VV_M1_MASK	= 6171,
    RISCV_PseudoVMINU_VV_M2	= 6172,
    RISCV_PseudoVMINU_VV_M2_MASK	= 6173,
    RISCV_PseudoVMINU_VV_M4	= 6174,
    RISCV_PseudoVMINU_VV_M4_MASK	= 6175,
    RISCV_PseudoVMINU_VV_M8	= 6176,
    RISCV_PseudoVMINU_VV_M8_MASK	= 6177,
    RISCV_PseudoVMINU_VV_MF2	= 6178,
    RISCV_PseudoVMINU_VV_MF2_MASK	= 6179,
    RISCV_PseudoVMINU_VV_MF4	= 6180,
    RISCV_PseudoVMINU_VV_MF4_MASK	= 6181,
    RISCV_PseudoVMINU_VV_MF8	= 6182,
    RISCV_PseudoVMINU_VV_MF8_MASK	= 6183,
    RISCV_PseudoVMINU_VX_M1	= 6184,
    RISCV_PseudoVMINU_VX_M1_MASK	= 6185,
    RISCV_PseudoVMINU_VX_M2	= 6186,
    RISCV_PseudoVMINU_VX_M2_MASK	= 6187,
    RISCV_PseudoVMINU_VX_M4	= 6188,
    RISCV_PseudoVMINU_VX_M4_MASK	= 6189,
    RISCV_PseudoVMINU_VX_M8	= 6190,
    RISCV_PseudoVMINU_VX_M8_MASK	= 6191,
    RISCV_PseudoVMINU_VX_MF2	= 6192,
    RISCV_PseudoVMINU_VX_MF2_MASK	= 6193,
    RISCV_PseudoVMINU_VX_MF4	= 6194,
    RISCV_PseudoVMINU_VX_MF4_MASK	= 6195,
    RISCV_PseudoVMINU_VX_MF8	= 6196,
    RISCV_PseudoVMINU_VX_MF8_MASK	= 6197,
    RISCV_PseudoVMIN_VV_M1	= 6198,
    RISCV_PseudoVMIN_VV_M1_MASK	= 6199,
    RISCV_PseudoVMIN_VV_M2	= 6200,
    RISCV_PseudoVMIN_VV_M2_MASK	= 6201,
    RISCV_PseudoVMIN_VV_M4	= 6202,
    RISCV_PseudoVMIN_VV_M4_MASK	= 6203,
    RISCV_PseudoVMIN_VV_M8	= 6204,
    RISCV_PseudoVMIN_VV_M8_MASK	= 6205,
    RISCV_PseudoVMIN_VV_MF2	= 6206,
    RISCV_PseudoVMIN_VV_MF2_MASK	= 6207,
    RISCV_PseudoVMIN_VV_MF4	= 6208,
    RISCV_PseudoVMIN_VV_MF4_MASK	= 6209,
    RISCV_PseudoVMIN_VV_MF8	= 6210,
    RISCV_PseudoVMIN_VV_MF8_MASK	= 6211,
    RISCV_PseudoVMIN_VX_M1	= 6212,
    RISCV_PseudoVMIN_VX_M1_MASK	= 6213,
    RISCV_PseudoVMIN_VX_M2	= 6214,
    RISCV_PseudoVMIN_VX_M2_MASK	= 6215,
    RISCV_PseudoVMIN_VX_M4	= 6216,
    RISCV_PseudoVMIN_VX_M4_MASK	= 6217,
    RISCV_PseudoVMIN_VX_M8	= 6218,
    RISCV_PseudoVMIN_VX_M8_MASK	= 6219,
    RISCV_PseudoVMIN_VX_MF2	= 6220,
    RISCV_PseudoVMIN_VX_MF2_MASK	= 6221,
    RISCV_PseudoVMIN_VX_MF4	= 6222,
    RISCV_PseudoVMIN_VX_MF4_MASK	= 6223,
    RISCV_PseudoVMIN_VX_MF8	= 6224,
    RISCV_PseudoVMIN_VX_MF8_MASK	= 6225,
    RISCV_PseudoVMNAND_MM_M1	= 6226,
    RISCV_PseudoVMNAND_MM_M2	= 6227,
    RISCV_PseudoVMNAND_MM_M4	= 6228,
    RISCV_PseudoVMNAND_MM_M8	= 6229,
    RISCV_PseudoVMNAND_MM_MF2	= 6230,
    RISCV_PseudoVMNAND_MM_MF4	= 6231,
    RISCV_PseudoVMNAND_MM_MF8	= 6232,
    RISCV_PseudoVMNOR_MM_M1	= 6233,
    RISCV_PseudoVMNOR_MM_M2	= 6234,
    RISCV_PseudoVMNOR_MM_M4	= 6235,
    RISCV_PseudoVMNOR_MM_M8	= 6236,
    RISCV_PseudoVMNOR_MM_MF2	= 6237,
    RISCV_PseudoVMNOR_MM_MF4	= 6238,
    RISCV_PseudoVMNOR_MM_MF8	= 6239,
    RISCV_PseudoVMORN_MM_M1	= 6240,
    RISCV_PseudoVMORN_MM_M2	= 6241,
    RISCV_PseudoVMORN_MM_M4	= 6242,
    RISCV_PseudoVMORN_MM_M8	= 6243,
    RISCV_PseudoVMORN_MM_MF2	= 6244,
    RISCV_PseudoVMORN_MM_MF4	= 6245,
    RISCV_PseudoVMORN_MM_MF8	= 6246,
    RISCV_PseudoVMOR_MM_M1	= 6247,
    RISCV_PseudoVMOR_MM_M2	= 6248,
    RISCV_PseudoVMOR_MM_M4	= 6249,
    RISCV_PseudoVMOR_MM_M8	= 6250,
    RISCV_PseudoVMOR_MM_MF2	= 6251,
    RISCV_PseudoVMOR_MM_MF4	= 6252,
    RISCV_PseudoVMOR_MM_MF8	= 6253,
    RISCV_PseudoVMSBC_VVM_M1	= 6254,
    RISCV_PseudoVMSBC_VVM_M2	= 6255,
    RISCV_PseudoVMSBC_VVM_M4	= 6256,
    RISCV_PseudoVMSBC_VVM_M8	= 6257,
    RISCV_PseudoVMSBC_VVM_MF2	= 6258,
    RISCV_PseudoVMSBC_VVM_MF4	= 6259,
    RISCV_PseudoVMSBC_VVM_MF8	= 6260,
    RISCV_PseudoVMSBC_VV_M1	= 6261,
    RISCV_PseudoVMSBC_VV_M2	= 6262,
    RISCV_PseudoVMSBC_VV_M4	= 6263,
    RISCV_PseudoVMSBC_VV_M8	= 6264,
    RISCV_PseudoVMSBC_VV_MF2	= 6265,
    RISCV_PseudoVMSBC_VV_MF4	= 6266,
    RISCV_PseudoVMSBC_VV_MF8	= 6267,
    RISCV_PseudoVMSBC_VXM_M1	= 6268,
    RISCV_PseudoVMSBC_VXM_M2	= 6269,
    RISCV_PseudoVMSBC_VXM_M4	= 6270,
    RISCV_PseudoVMSBC_VXM_M8	= 6271,
    RISCV_PseudoVMSBC_VXM_MF2	= 6272,
    RISCV_PseudoVMSBC_VXM_MF4	= 6273,
    RISCV_PseudoVMSBC_VXM_MF8	= 6274,
    RISCV_PseudoVMSBC_VX_M1	= 6275,
    RISCV_PseudoVMSBC_VX_M2	= 6276,
    RISCV_PseudoVMSBC_VX_M4	= 6277,
    RISCV_PseudoVMSBC_VX_M8	= 6278,
    RISCV_PseudoVMSBC_VX_MF2	= 6279,
    RISCV_PseudoVMSBC_VX_MF4	= 6280,
    RISCV_PseudoVMSBC_VX_MF8	= 6281,
    RISCV_PseudoVMSBF_M_B1	= 6282,
    RISCV_PseudoVMSBF_M_B16	= 6283,
    RISCV_PseudoVMSBF_M_B16_MASK	= 6284,
    RISCV_PseudoVMSBF_M_B1_MASK	= 6285,
    RISCV_PseudoVMSBF_M_B2	= 6286,
    RISCV_PseudoVMSBF_M_B2_MASK	= 6287,
    RISCV_PseudoVMSBF_M_B32	= 6288,
    RISCV_PseudoVMSBF_M_B32_MASK	= 6289,
    RISCV_PseudoVMSBF_M_B4	= 6290,
    RISCV_PseudoVMSBF_M_B4_MASK	= 6291,
    RISCV_PseudoVMSBF_M_B64	= 6292,
    RISCV_PseudoVMSBF_M_B64_MASK	= 6293,
    RISCV_PseudoVMSBF_M_B8	= 6294,
    RISCV_PseudoVMSBF_M_B8_MASK	= 6295,
    RISCV_PseudoVMSEQ_VI_M1	= 6296,
    RISCV_PseudoVMSEQ_VI_M1_MASK	= 6297,
    RISCV_PseudoVMSEQ_VI_M2	= 6298,
    RISCV_PseudoVMSEQ_VI_M2_MASK	= 6299,
    RISCV_PseudoVMSEQ_VI_M4	= 6300,
    RISCV_PseudoVMSEQ_VI_M4_MASK	= 6301,
    RISCV_PseudoVMSEQ_VI_M8	= 6302,
    RISCV_PseudoVMSEQ_VI_M8_MASK	= 6303,
    RISCV_PseudoVMSEQ_VI_MF2	= 6304,
    RISCV_PseudoVMSEQ_VI_MF2_MASK	= 6305,
    RISCV_PseudoVMSEQ_VI_MF4	= 6306,
    RISCV_PseudoVMSEQ_VI_MF4_MASK	= 6307,
    RISCV_PseudoVMSEQ_VI_MF8	= 6308,
    RISCV_PseudoVMSEQ_VI_MF8_MASK	= 6309,
    RISCV_PseudoVMSEQ_VV_M1	= 6310,
    RISCV_PseudoVMSEQ_VV_M1_MASK	= 6311,
    RISCV_PseudoVMSEQ_VV_M2	= 6312,
    RISCV_PseudoVMSEQ_VV_M2_MASK	= 6313,
    RISCV_PseudoVMSEQ_VV_M4	= 6314,
    RISCV_PseudoVMSEQ_VV_M4_MASK	= 6315,
    RISCV_PseudoVMSEQ_VV_M8	= 6316,
    RISCV_PseudoVMSEQ_VV_M8_MASK	= 6317,
    RISCV_PseudoVMSEQ_VV_MF2	= 6318,
    RISCV_PseudoVMSEQ_VV_MF2_MASK	= 6319,
    RISCV_PseudoVMSEQ_VV_MF4	= 6320,
    RISCV_PseudoVMSEQ_VV_MF4_MASK	= 6321,
    RISCV_PseudoVMSEQ_VV_MF8	= 6322,
    RISCV_PseudoVMSEQ_VV_MF8_MASK	= 6323,
    RISCV_PseudoVMSEQ_VX_M1	= 6324,
    RISCV_PseudoVMSEQ_VX_M1_MASK	= 6325,
    RISCV_PseudoVMSEQ_VX_M2	= 6326,
    RISCV_PseudoVMSEQ_VX_M2_MASK	= 6327,
    RISCV_PseudoVMSEQ_VX_M4	= 6328,
    RISCV_PseudoVMSEQ_VX_M4_MASK	= 6329,
    RISCV_PseudoVMSEQ_VX_M8	= 6330,
    RISCV_PseudoVMSEQ_VX_M8_MASK	= 6331,
    RISCV_PseudoVMSEQ_VX_MF2	= 6332,
    RISCV_PseudoVMSEQ_VX_MF2_MASK	= 6333,
    RISCV_PseudoVMSEQ_VX_MF4	= 6334,
    RISCV_PseudoVMSEQ_VX_MF4_MASK	= 6335,
    RISCV_PseudoVMSEQ_VX_MF8	= 6336,
    RISCV_PseudoVMSEQ_VX_MF8_MASK	= 6337,
    RISCV_PseudoVMSET_M_B1	= 6338,
    RISCV_PseudoVMSET_M_B16	= 6339,
    RISCV_PseudoVMSET_M_B2	= 6340,
    RISCV_PseudoVMSET_M_B32	= 6341,
    RISCV_PseudoVMSET_M_B4	= 6342,
    RISCV_PseudoVMSET_M_B64	= 6343,
    RISCV_PseudoVMSET_M_B8	= 6344,
    RISCV_PseudoVMSGEU_VI	= 6345,
    RISCV_PseudoVMSGEU_VX	= 6346,
    RISCV_PseudoVMSGEU_VX_M	= 6347,
    RISCV_PseudoVMSGEU_VX_M_T	= 6348,
    RISCV_PseudoVMSGE_VI	= 6349,
    RISCV_PseudoVMSGE_VX	= 6350,
    RISCV_PseudoVMSGE_VX_M	= 6351,
    RISCV_PseudoVMSGE_VX_M_T	= 6352,
    RISCV_PseudoVMSGTU_VI_M1	= 6353,
    RISCV_PseudoVMSGTU_VI_M1_MASK	= 6354,
    RISCV_PseudoVMSGTU_VI_M2	= 6355,
    RISCV_PseudoVMSGTU_VI_M2_MASK	= 6356,
    RISCV_PseudoVMSGTU_VI_M4	= 6357,
    RISCV_PseudoVMSGTU_VI_M4_MASK	= 6358,
    RISCV_PseudoVMSGTU_VI_M8	= 6359,
    RISCV_PseudoVMSGTU_VI_M8_MASK	= 6360,
    RISCV_PseudoVMSGTU_VI_MF2	= 6361,
    RISCV_PseudoVMSGTU_VI_MF2_MASK	= 6362,
    RISCV_PseudoVMSGTU_VI_MF4	= 6363,
    RISCV_PseudoVMSGTU_VI_MF4_MASK	= 6364,
    RISCV_PseudoVMSGTU_VI_MF8	= 6365,
    RISCV_PseudoVMSGTU_VI_MF8_MASK	= 6366,
    RISCV_PseudoVMSGTU_VX_M1	= 6367,
    RISCV_PseudoVMSGTU_VX_M1_MASK	= 6368,
    RISCV_PseudoVMSGTU_VX_M2	= 6369,
    RISCV_PseudoVMSGTU_VX_M2_MASK	= 6370,
    RISCV_PseudoVMSGTU_VX_M4	= 6371,
    RISCV_PseudoVMSGTU_VX_M4_MASK	= 6372,
    RISCV_PseudoVMSGTU_VX_M8	= 6373,
    RISCV_PseudoVMSGTU_VX_M8_MASK	= 6374,
    RISCV_PseudoVMSGTU_VX_MF2	= 6375,
    RISCV_PseudoVMSGTU_VX_MF2_MASK	= 6376,
    RISCV_PseudoVMSGTU_VX_MF4	= 6377,
    RISCV_PseudoVMSGTU_VX_MF4_MASK	= 6378,
    RISCV_PseudoVMSGTU_VX_MF8	= 6379,
    RISCV_PseudoVMSGTU_VX_MF8_MASK	= 6380,
    RISCV_PseudoVMSGT_VI_M1	= 6381,
    RISCV_PseudoVMSGT_VI_M1_MASK	= 6382,
    RISCV_PseudoVMSGT_VI_M2	= 6383,
    RISCV_PseudoVMSGT_VI_M2_MASK	= 6384,
    RISCV_PseudoVMSGT_VI_M4	= 6385,
    RISCV_PseudoVMSGT_VI_M4_MASK	= 6386,
    RISCV_PseudoVMSGT_VI_M8	= 6387,
    RISCV_PseudoVMSGT_VI_M8_MASK	= 6388,
    RISCV_PseudoVMSGT_VI_MF2	= 6389,
    RISCV_PseudoVMSGT_VI_MF2_MASK	= 6390,
    RISCV_PseudoVMSGT_VI_MF4	= 6391,
    RISCV_PseudoVMSGT_VI_MF4_MASK	= 6392,
    RISCV_PseudoVMSGT_VI_MF8	= 6393,
    RISCV_PseudoVMSGT_VI_MF8_MASK	= 6394,
    RISCV_PseudoVMSGT_VX_M1	= 6395,
    RISCV_PseudoVMSGT_VX_M1_MASK	= 6396,
    RISCV_PseudoVMSGT_VX_M2	= 6397,
    RISCV_PseudoVMSGT_VX_M2_MASK	= 6398,
    RISCV_PseudoVMSGT_VX_M4	= 6399,
    RISCV_PseudoVMSGT_VX_M4_MASK	= 6400,
    RISCV_PseudoVMSGT_VX_M8	= 6401,
    RISCV_PseudoVMSGT_VX_M8_MASK	= 6402,
    RISCV_PseudoVMSGT_VX_MF2	= 6403,
    RISCV_PseudoVMSGT_VX_MF2_MASK	= 6404,
    RISCV_PseudoVMSGT_VX_MF4	= 6405,
    RISCV_PseudoVMSGT_VX_MF4_MASK	= 6406,
    RISCV_PseudoVMSGT_VX_MF8	= 6407,
    RISCV_PseudoVMSGT_VX_MF8_MASK	= 6408,
    RISCV_PseudoVMSIF_M_B1	= 6409,
    RISCV_PseudoVMSIF_M_B16	= 6410,
    RISCV_PseudoVMSIF_M_B16_MASK	= 6411,
    RISCV_PseudoVMSIF_M_B1_MASK	= 6412,
    RISCV_PseudoVMSIF_M_B2	= 6413,
    RISCV_PseudoVMSIF_M_B2_MASK	= 6414,
    RISCV_PseudoVMSIF_M_B32	= 6415,
    RISCV_PseudoVMSIF_M_B32_MASK	= 6416,
    RISCV_PseudoVMSIF_M_B4	= 6417,
    RISCV_PseudoVMSIF_M_B4_MASK	= 6418,
    RISCV_PseudoVMSIF_M_B64	= 6419,
    RISCV_PseudoVMSIF_M_B64_MASK	= 6420,
    RISCV_PseudoVMSIF_M_B8	= 6421,
    RISCV_PseudoVMSIF_M_B8_MASK	= 6422,
    RISCV_PseudoVMSLEU_VI_M1	= 6423,
    RISCV_PseudoVMSLEU_VI_M1_MASK	= 6424,
    RISCV_PseudoVMSLEU_VI_M2	= 6425,
    RISCV_PseudoVMSLEU_VI_M2_MASK	= 6426,
    RISCV_PseudoVMSLEU_VI_M4	= 6427,
    RISCV_PseudoVMSLEU_VI_M4_MASK	= 6428,
    RISCV_PseudoVMSLEU_VI_M8	= 6429,
    RISCV_PseudoVMSLEU_VI_M8_MASK	= 6430,
    RISCV_PseudoVMSLEU_VI_MF2	= 6431,
    RISCV_PseudoVMSLEU_VI_MF2_MASK	= 6432,
    RISCV_PseudoVMSLEU_VI_MF4	= 6433,
    RISCV_PseudoVMSLEU_VI_MF4_MASK	= 6434,
    RISCV_PseudoVMSLEU_VI_MF8	= 6435,
    RISCV_PseudoVMSLEU_VI_MF8_MASK	= 6436,
    RISCV_PseudoVMSLEU_VV_M1	= 6437,
    RISCV_PseudoVMSLEU_VV_M1_MASK	= 6438,
    RISCV_PseudoVMSLEU_VV_M2	= 6439,
    RISCV_PseudoVMSLEU_VV_M2_MASK	= 6440,
    RISCV_PseudoVMSLEU_VV_M4	= 6441,
    RISCV_PseudoVMSLEU_VV_M4_MASK	= 6442,
    RISCV_PseudoVMSLEU_VV_M8	= 6443,
    RISCV_PseudoVMSLEU_VV_M8_MASK	= 6444,
    RISCV_PseudoVMSLEU_VV_MF2	= 6445,
    RISCV_PseudoVMSLEU_VV_MF2_MASK	= 6446,
    RISCV_PseudoVMSLEU_VV_MF4	= 6447,
    RISCV_PseudoVMSLEU_VV_MF4_MASK	= 6448,
    RISCV_PseudoVMSLEU_VV_MF8	= 6449,
    RISCV_PseudoVMSLEU_VV_MF8_MASK	= 6450,
    RISCV_PseudoVMSLEU_VX_M1	= 6451,
    RISCV_PseudoVMSLEU_VX_M1_MASK	= 6452,
    RISCV_PseudoVMSLEU_VX_M2	= 6453,
    RISCV_PseudoVMSLEU_VX_M2_MASK	= 6454,
    RISCV_PseudoVMSLEU_VX_M4	= 6455,
    RISCV_PseudoVMSLEU_VX_M4_MASK	= 6456,
    RISCV_PseudoVMSLEU_VX_M8	= 6457,
    RISCV_PseudoVMSLEU_VX_M8_MASK	= 6458,
    RISCV_PseudoVMSLEU_VX_MF2	= 6459,
    RISCV_PseudoVMSLEU_VX_MF2_MASK	= 6460,
    RISCV_PseudoVMSLEU_VX_MF4	= 6461,
    RISCV_PseudoVMSLEU_VX_MF4_MASK	= 6462,
    RISCV_PseudoVMSLEU_VX_MF8	= 6463,
    RISCV_PseudoVMSLEU_VX_MF8_MASK	= 6464,
    RISCV_PseudoVMSLE_VI_M1	= 6465,
    RISCV_PseudoVMSLE_VI_M1_MASK	= 6466,
    RISCV_PseudoVMSLE_VI_M2	= 6467,
    RISCV_PseudoVMSLE_VI_M2_MASK	= 6468,
    RISCV_PseudoVMSLE_VI_M4	= 6469,
    RISCV_PseudoVMSLE_VI_M4_MASK	= 6470,
    RISCV_PseudoVMSLE_VI_M8	= 6471,
    RISCV_PseudoVMSLE_VI_M8_MASK	= 6472,
    RISCV_PseudoVMSLE_VI_MF2	= 6473,
    RISCV_PseudoVMSLE_VI_MF2_MASK	= 6474,
    RISCV_PseudoVMSLE_VI_MF4	= 6475,
    RISCV_PseudoVMSLE_VI_MF4_MASK	= 6476,
    RISCV_PseudoVMSLE_VI_MF8	= 6477,
    RISCV_PseudoVMSLE_VI_MF8_MASK	= 6478,
    RISCV_PseudoVMSLE_VV_M1	= 6479,
    RISCV_PseudoVMSLE_VV_M1_MASK	= 6480,
    RISCV_PseudoVMSLE_VV_M2	= 6481,
    RISCV_PseudoVMSLE_VV_M2_MASK	= 6482,
    RISCV_PseudoVMSLE_VV_M4	= 6483,
    RISCV_PseudoVMSLE_VV_M4_MASK	= 6484,
    RISCV_PseudoVMSLE_VV_M8	= 6485,
    RISCV_PseudoVMSLE_VV_M8_MASK	= 6486,
    RISCV_PseudoVMSLE_VV_MF2	= 6487,
    RISCV_PseudoVMSLE_VV_MF2_MASK	= 6488,
    RISCV_PseudoVMSLE_VV_MF4	= 6489,
    RISCV_PseudoVMSLE_VV_MF4_MASK	= 6490,
    RISCV_PseudoVMSLE_VV_MF8	= 6491,
    RISCV_PseudoVMSLE_VV_MF8_MASK	= 6492,
    RISCV_PseudoVMSLE_VX_M1	= 6493,
    RISCV_PseudoVMSLE_VX_M1_MASK	= 6494,
    RISCV_PseudoVMSLE_VX_M2	= 6495,
    RISCV_PseudoVMSLE_VX_M2_MASK	= 6496,
    RISCV_PseudoVMSLE_VX_M4	= 6497,
    RISCV_PseudoVMSLE_VX_M4_MASK	= 6498,
    RISCV_PseudoVMSLE_VX_M8	= 6499,
    RISCV_PseudoVMSLE_VX_M8_MASK	= 6500,
    RISCV_PseudoVMSLE_VX_MF2	= 6501,
    RISCV_PseudoVMSLE_VX_MF2_MASK	= 6502,
    RISCV_PseudoVMSLE_VX_MF4	= 6503,
    RISCV_PseudoVMSLE_VX_MF4_MASK	= 6504,
    RISCV_PseudoVMSLE_VX_MF8	= 6505,
    RISCV_PseudoVMSLE_VX_MF8_MASK	= 6506,
    RISCV_PseudoVMSLTU_VI	= 6507,
    RISCV_PseudoVMSLTU_VV_M1	= 6508,
    RISCV_PseudoVMSLTU_VV_M1_MASK	= 6509,
    RISCV_PseudoVMSLTU_VV_M2	= 6510,
    RISCV_PseudoVMSLTU_VV_M2_MASK	= 6511,
    RISCV_PseudoVMSLTU_VV_M4	= 6512,
    RISCV_PseudoVMSLTU_VV_M4_MASK	= 6513,
    RISCV_PseudoVMSLTU_VV_M8	= 6514,
    RISCV_PseudoVMSLTU_VV_M8_MASK	= 6515,
    RISCV_PseudoVMSLTU_VV_MF2	= 6516,
    RISCV_PseudoVMSLTU_VV_MF2_MASK	= 6517,
    RISCV_PseudoVMSLTU_VV_MF4	= 6518,
    RISCV_PseudoVMSLTU_VV_MF4_MASK	= 6519,
    RISCV_PseudoVMSLTU_VV_MF8	= 6520,
    RISCV_PseudoVMSLTU_VV_MF8_MASK	= 6521,
    RISCV_PseudoVMSLTU_VX_M1	= 6522,
    RISCV_PseudoVMSLTU_VX_M1_MASK	= 6523,
    RISCV_PseudoVMSLTU_VX_M2	= 6524,
    RISCV_PseudoVMSLTU_VX_M2_MASK	= 6525,
    RISCV_PseudoVMSLTU_VX_M4	= 6526,
    RISCV_PseudoVMSLTU_VX_M4_MASK	= 6527,
    RISCV_PseudoVMSLTU_VX_M8	= 6528,
    RISCV_PseudoVMSLTU_VX_M8_MASK	= 6529,
    RISCV_PseudoVMSLTU_VX_MF2	= 6530,
    RISCV_PseudoVMSLTU_VX_MF2_MASK	= 6531,
    RISCV_PseudoVMSLTU_VX_MF4	= 6532,
    RISCV_PseudoVMSLTU_VX_MF4_MASK	= 6533,
    RISCV_PseudoVMSLTU_VX_MF8	= 6534,
    RISCV_PseudoVMSLTU_VX_MF8_MASK	= 6535,
    RISCV_PseudoVMSLT_VI	= 6536,
    RISCV_PseudoVMSLT_VV_M1	= 6537,
    RISCV_PseudoVMSLT_VV_M1_MASK	= 6538,
    RISCV_PseudoVMSLT_VV_M2	= 6539,
    RISCV_PseudoVMSLT_VV_M2_MASK	= 6540,
    RISCV_PseudoVMSLT_VV_M4	= 6541,
    RISCV_PseudoVMSLT_VV_M4_MASK	= 6542,
    RISCV_PseudoVMSLT_VV_M8	= 6543,
    RISCV_PseudoVMSLT_VV_M8_MASK	= 6544,
    RISCV_PseudoVMSLT_VV_MF2	= 6545,
    RISCV_PseudoVMSLT_VV_MF2_MASK	= 6546,
    RISCV_PseudoVMSLT_VV_MF4	= 6547,
    RISCV_PseudoVMSLT_VV_MF4_MASK	= 6548,
    RISCV_PseudoVMSLT_VV_MF8	= 6549,
    RISCV_PseudoVMSLT_VV_MF8_MASK	= 6550,
    RISCV_PseudoVMSLT_VX_M1	= 6551,
    RISCV_PseudoVMSLT_VX_M1_MASK	= 6552,
    RISCV_PseudoVMSLT_VX_M2	= 6553,
    RISCV_PseudoVMSLT_VX_M2_MASK	= 6554,
    RISCV_PseudoVMSLT_VX_M4	= 6555,
    RISCV_PseudoVMSLT_VX_M4_MASK	= 6556,
    RISCV_PseudoVMSLT_VX_M8	= 6557,
    RISCV_PseudoVMSLT_VX_M8_MASK	= 6558,
    RISCV_PseudoVMSLT_VX_MF2	= 6559,
    RISCV_PseudoVMSLT_VX_MF2_MASK	= 6560,
    RISCV_PseudoVMSLT_VX_MF4	= 6561,
    RISCV_PseudoVMSLT_VX_MF4_MASK	= 6562,
    RISCV_PseudoVMSLT_VX_MF8	= 6563,
    RISCV_PseudoVMSLT_VX_MF8_MASK	= 6564,
    RISCV_PseudoVMSNE_VI_M1	= 6565,
    RISCV_PseudoVMSNE_VI_M1_MASK	= 6566,
    RISCV_PseudoVMSNE_VI_M2	= 6567,
    RISCV_PseudoVMSNE_VI_M2_MASK	= 6568,
    RISCV_PseudoVMSNE_VI_M4	= 6569,
    RISCV_PseudoVMSNE_VI_M4_MASK	= 6570,
    RISCV_PseudoVMSNE_VI_M8	= 6571,
    RISCV_PseudoVMSNE_VI_M8_MASK	= 6572,
    RISCV_PseudoVMSNE_VI_MF2	= 6573,
    RISCV_PseudoVMSNE_VI_MF2_MASK	= 6574,
    RISCV_PseudoVMSNE_VI_MF4	= 6575,
    RISCV_PseudoVMSNE_VI_MF4_MASK	= 6576,
    RISCV_PseudoVMSNE_VI_MF8	= 6577,
    RISCV_PseudoVMSNE_VI_MF8_MASK	= 6578,
    RISCV_PseudoVMSNE_VV_M1	= 6579,
    RISCV_PseudoVMSNE_VV_M1_MASK	= 6580,
    RISCV_PseudoVMSNE_VV_M2	= 6581,
    RISCV_PseudoVMSNE_VV_M2_MASK	= 6582,
    RISCV_PseudoVMSNE_VV_M4	= 6583,
    RISCV_PseudoVMSNE_VV_M4_MASK	= 6584,
    RISCV_PseudoVMSNE_VV_M8	= 6585,
    RISCV_PseudoVMSNE_VV_M8_MASK	= 6586,
    RISCV_PseudoVMSNE_VV_MF2	= 6587,
    RISCV_PseudoVMSNE_VV_MF2_MASK	= 6588,
    RISCV_PseudoVMSNE_VV_MF4	= 6589,
    RISCV_PseudoVMSNE_VV_MF4_MASK	= 6590,
    RISCV_PseudoVMSNE_VV_MF8	= 6591,
    RISCV_PseudoVMSNE_VV_MF8_MASK	= 6592,
    RISCV_PseudoVMSNE_VX_M1	= 6593,
    RISCV_PseudoVMSNE_VX_M1_MASK	= 6594,
    RISCV_PseudoVMSNE_VX_M2	= 6595,
    RISCV_PseudoVMSNE_VX_M2_MASK	= 6596,
    RISCV_PseudoVMSNE_VX_M4	= 6597,
    RISCV_PseudoVMSNE_VX_M4_MASK	= 6598,
    RISCV_PseudoVMSNE_VX_M8	= 6599,
    RISCV_PseudoVMSNE_VX_M8_MASK	= 6600,
    RISCV_PseudoVMSNE_VX_MF2	= 6601,
    RISCV_PseudoVMSNE_VX_MF2_MASK	= 6602,
    RISCV_PseudoVMSNE_VX_MF4	= 6603,
    RISCV_PseudoVMSNE_VX_MF4_MASK	= 6604,
    RISCV_PseudoVMSNE_VX_MF8	= 6605,
    RISCV_PseudoVMSNE_VX_MF8_MASK	= 6606,
    RISCV_PseudoVMSOF_M_B1	= 6607,
    RISCV_PseudoVMSOF_M_B16	= 6608,
    RISCV_PseudoVMSOF_M_B16_MASK	= 6609,
    RISCV_PseudoVMSOF_M_B1_MASK	= 6610,
    RISCV_PseudoVMSOF_M_B2	= 6611,
    RISCV_PseudoVMSOF_M_B2_MASK	= 6612,
    RISCV_PseudoVMSOF_M_B32	= 6613,
    RISCV_PseudoVMSOF_M_B32_MASK	= 6614,
    RISCV_PseudoVMSOF_M_B4	= 6615,
    RISCV_PseudoVMSOF_M_B4_MASK	= 6616,
    RISCV_PseudoVMSOF_M_B64	= 6617,
    RISCV_PseudoVMSOF_M_B64_MASK	= 6618,
    RISCV_PseudoVMSOF_M_B8	= 6619,
    RISCV_PseudoVMSOF_M_B8_MASK	= 6620,
    RISCV_PseudoVMULHSU_VV_M1	= 6621,
    RISCV_PseudoVMULHSU_VV_M1_MASK	= 6622,
    RISCV_PseudoVMULHSU_VV_M2	= 6623,
    RISCV_PseudoVMULHSU_VV_M2_MASK	= 6624,
    RISCV_PseudoVMULHSU_VV_M4	= 6625,
    RISCV_PseudoVMULHSU_VV_M4_MASK	= 6626,
    RISCV_PseudoVMULHSU_VV_M8	= 6627,
    RISCV_PseudoVMULHSU_VV_M8_MASK	= 6628,
    RISCV_PseudoVMULHSU_VV_MF2	= 6629,
    RISCV_PseudoVMULHSU_VV_MF2_MASK	= 6630,
    RISCV_PseudoVMULHSU_VV_MF4	= 6631,
    RISCV_PseudoVMULHSU_VV_MF4_MASK	= 6632,
    RISCV_PseudoVMULHSU_VV_MF8	= 6633,
    RISCV_PseudoVMULHSU_VV_MF8_MASK	= 6634,
    RISCV_PseudoVMULHSU_VX_M1	= 6635,
    RISCV_PseudoVMULHSU_VX_M1_MASK	= 6636,
    RISCV_PseudoVMULHSU_VX_M2	= 6637,
    RISCV_PseudoVMULHSU_VX_M2_MASK	= 6638,
    RISCV_PseudoVMULHSU_VX_M4	= 6639,
    RISCV_PseudoVMULHSU_VX_M4_MASK	= 6640,
    RISCV_PseudoVMULHSU_VX_M8	= 6641,
    RISCV_PseudoVMULHSU_VX_M8_MASK	= 6642,
    RISCV_PseudoVMULHSU_VX_MF2	= 6643,
    RISCV_PseudoVMULHSU_VX_MF2_MASK	= 6644,
    RISCV_PseudoVMULHSU_VX_MF4	= 6645,
    RISCV_PseudoVMULHSU_VX_MF4_MASK	= 6646,
    RISCV_PseudoVMULHSU_VX_MF8	= 6647,
    RISCV_PseudoVMULHSU_VX_MF8_MASK	= 6648,
    RISCV_PseudoVMULHU_VV_M1	= 6649,
    RISCV_PseudoVMULHU_VV_M1_MASK	= 6650,
    RISCV_PseudoVMULHU_VV_M2	= 6651,
    RISCV_PseudoVMULHU_VV_M2_MASK	= 6652,
    RISCV_PseudoVMULHU_VV_M4	= 6653,
    RISCV_PseudoVMULHU_VV_M4_MASK	= 6654,
    RISCV_PseudoVMULHU_VV_M8	= 6655,
    RISCV_PseudoVMULHU_VV_M8_MASK	= 6656,
    RISCV_PseudoVMULHU_VV_MF2	= 6657,
    RISCV_PseudoVMULHU_VV_MF2_MASK	= 6658,
    RISCV_PseudoVMULHU_VV_MF4	= 6659,
    RISCV_PseudoVMULHU_VV_MF4_MASK	= 6660,
    RISCV_PseudoVMULHU_VV_MF8	= 6661,
    RISCV_PseudoVMULHU_VV_MF8_MASK	= 6662,
    RISCV_PseudoVMULHU_VX_M1	= 6663,
    RISCV_PseudoVMULHU_VX_M1_MASK	= 6664,
    RISCV_PseudoVMULHU_VX_M2	= 6665,
    RISCV_PseudoVMULHU_VX_M2_MASK	= 6666,
    RISCV_PseudoVMULHU_VX_M4	= 6667,
    RISCV_PseudoVMULHU_VX_M4_MASK	= 6668,
    RISCV_PseudoVMULHU_VX_M8	= 6669,
    RISCV_PseudoVMULHU_VX_M8_MASK	= 6670,
    RISCV_PseudoVMULHU_VX_MF2	= 6671,
    RISCV_PseudoVMULHU_VX_MF2_MASK	= 6672,
    RISCV_PseudoVMULHU_VX_MF4	= 6673,
    RISCV_PseudoVMULHU_VX_MF4_MASK	= 6674,
    RISCV_PseudoVMULHU_VX_MF8	= 6675,
    RISCV_PseudoVMULHU_VX_MF8_MASK	= 6676,
    RISCV_PseudoVMULH_VV_M1	= 6677,
    RISCV_PseudoVMULH_VV_M1_MASK	= 6678,
    RISCV_PseudoVMULH_VV_M2	= 6679,
    RISCV_PseudoVMULH_VV_M2_MASK	= 6680,
    RISCV_PseudoVMULH_VV_M4	= 6681,
    RISCV_PseudoVMULH_VV_M4_MASK	= 6682,
    RISCV_PseudoVMULH_VV_M8	= 6683,
    RISCV_PseudoVMULH_VV_M8_MASK	= 6684,
    RISCV_PseudoVMULH_VV_MF2	= 6685,
    RISCV_PseudoVMULH_VV_MF2_MASK	= 6686,
    RISCV_PseudoVMULH_VV_MF4	= 6687,
    RISCV_PseudoVMULH_VV_MF4_MASK	= 6688,
    RISCV_PseudoVMULH_VV_MF8	= 6689,
    RISCV_PseudoVMULH_VV_MF8_MASK	= 6690,
    RISCV_PseudoVMULH_VX_M1	= 6691,
    RISCV_PseudoVMULH_VX_M1_MASK	= 6692,
    RISCV_PseudoVMULH_VX_M2	= 6693,
    RISCV_PseudoVMULH_VX_M2_MASK	= 6694,
    RISCV_PseudoVMULH_VX_M4	= 6695,
    RISCV_PseudoVMULH_VX_M4_MASK	= 6696,
    RISCV_PseudoVMULH_VX_M8	= 6697,
    RISCV_PseudoVMULH_VX_M8_MASK	= 6698,
    RISCV_PseudoVMULH_VX_MF2	= 6699,
    RISCV_PseudoVMULH_VX_MF2_MASK	= 6700,
    RISCV_PseudoVMULH_VX_MF4	= 6701,
    RISCV_PseudoVMULH_VX_MF4_MASK	= 6702,
    RISCV_PseudoVMULH_VX_MF8	= 6703,
    RISCV_PseudoVMULH_VX_MF8_MASK	= 6704,
    RISCV_PseudoVMUL_VV_M1	= 6705,
    RISCV_PseudoVMUL_VV_M1_MASK	= 6706,
    RISCV_PseudoVMUL_VV_M2	= 6707,
    RISCV_PseudoVMUL_VV_M2_MASK	= 6708,
    RISCV_PseudoVMUL_VV_M4	= 6709,
    RISCV_PseudoVMUL_VV_M4_MASK	= 6710,
    RISCV_PseudoVMUL_VV_M8	= 6711,
    RISCV_PseudoVMUL_VV_M8_MASK	= 6712,
    RISCV_PseudoVMUL_VV_MF2	= 6713,
    RISCV_PseudoVMUL_VV_MF2_MASK	= 6714,
    RISCV_PseudoVMUL_VV_MF4	= 6715,
    RISCV_PseudoVMUL_VV_MF4_MASK	= 6716,
    RISCV_PseudoVMUL_VV_MF8	= 6717,
    RISCV_PseudoVMUL_VV_MF8_MASK	= 6718,
    RISCV_PseudoVMUL_VX_M1	= 6719,
    RISCV_PseudoVMUL_VX_M1_MASK	= 6720,
    RISCV_PseudoVMUL_VX_M2	= 6721,
    RISCV_PseudoVMUL_VX_M2_MASK	= 6722,
    RISCV_PseudoVMUL_VX_M4	= 6723,
    RISCV_PseudoVMUL_VX_M4_MASK	= 6724,
    RISCV_PseudoVMUL_VX_M8	= 6725,
    RISCV_PseudoVMUL_VX_M8_MASK	= 6726,
    RISCV_PseudoVMUL_VX_MF2	= 6727,
    RISCV_PseudoVMUL_VX_MF2_MASK	= 6728,
    RISCV_PseudoVMUL_VX_MF4	= 6729,
    RISCV_PseudoVMUL_VX_MF4_MASK	= 6730,
    RISCV_PseudoVMUL_VX_MF8	= 6731,
    RISCV_PseudoVMUL_VX_MF8_MASK	= 6732,
    RISCV_PseudoVMV_S_X	= 6733,
    RISCV_PseudoVMV_V_I_M1	= 6734,
    RISCV_PseudoVMV_V_I_M2	= 6735,
    RISCV_PseudoVMV_V_I_M4	= 6736,
    RISCV_PseudoVMV_V_I_M8	= 6737,
    RISCV_PseudoVMV_V_I_MF2	= 6738,
    RISCV_PseudoVMV_V_I_MF4	= 6739,
    RISCV_PseudoVMV_V_I_MF8	= 6740,
    RISCV_PseudoVMV_V_V_M1	= 6741,
    RISCV_PseudoVMV_V_V_M2	= 6742,
    RISCV_PseudoVMV_V_V_M4	= 6743,
    RISCV_PseudoVMV_V_V_M8	= 6744,
    RISCV_PseudoVMV_V_V_MF2	= 6745,
    RISCV_PseudoVMV_V_V_MF4	= 6746,
    RISCV_PseudoVMV_V_V_MF8	= 6747,
    RISCV_PseudoVMV_V_X_M1	= 6748,
    RISCV_PseudoVMV_V_X_M2	= 6749,
    RISCV_PseudoVMV_V_X_M4	= 6750,
    RISCV_PseudoVMV_V_X_M8	= 6751,
    RISCV_PseudoVMV_V_X_MF2	= 6752,
    RISCV_PseudoVMV_V_X_MF4	= 6753,
    RISCV_PseudoVMV_V_X_MF8	= 6754,
    RISCV_PseudoVMV_X_S	= 6755,
    RISCV_PseudoVMXNOR_MM_M1	= 6756,
    RISCV_PseudoVMXNOR_MM_M2	= 6757,
    RISCV_PseudoVMXNOR_MM_M4	= 6758,
    RISCV_PseudoVMXNOR_MM_M8	= 6759,
    RISCV_PseudoVMXNOR_MM_MF2	= 6760,
    RISCV_PseudoVMXNOR_MM_MF4	= 6761,
    RISCV_PseudoVMXNOR_MM_MF8	= 6762,
    RISCV_PseudoVMXOR_MM_M1	= 6763,
    RISCV_PseudoVMXOR_MM_M2	= 6764,
    RISCV_PseudoVMXOR_MM_M4	= 6765,
    RISCV_PseudoVMXOR_MM_M8	= 6766,
    RISCV_PseudoVMXOR_MM_MF2	= 6767,
    RISCV_PseudoVMXOR_MM_MF4	= 6768,
    RISCV_PseudoVMXOR_MM_MF8	= 6769,
    RISCV_PseudoVNCLIPU_WI_M1	= 6770,
    RISCV_PseudoVNCLIPU_WI_M1_MASK	= 6771,
    RISCV_PseudoVNCLIPU_WI_M2	= 6772,
    RISCV_PseudoVNCLIPU_WI_M2_MASK	= 6773,
    RISCV_PseudoVNCLIPU_WI_M4	= 6774,
    RISCV_PseudoVNCLIPU_WI_M4_MASK	= 6775,
    RISCV_PseudoVNCLIPU_WI_MF2	= 6776,
    RISCV_PseudoVNCLIPU_WI_MF2_MASK	= 6777,
    RISCV_PseudoVNCLIPU_WI_MF4	= 6778,
    RISCV_PseudoVNCLIPU_WI_MF4_MASK	= 6779,
    RISCV_PseudoVNCLIPU_WI_MF8	= 6780,
    RISCV_PseudoVNCLIPU_WI_MF8_MASK	= 6781,
    RISCV_PseudoVNCLIPU_WV_M1	= 6782,
    RISCV_PseudoVNCLIPU_WV_M1_MASK	= 6783,
    RISCV_PseudoVNCLIPU_WV_M2	= 6784,
    RISCV_PseudoVNCLIPU_WV_M2_MASK	= 6785,
    RISCV_PseudoVNCLIPU_WV_M4	= 6786,
    RISCV_PseudoVNCLIPU_WV_M4_MASK	= 6787,
    RISCV_PseudoVNCLIPU_WV_MF2	= 6788,
    RISCV_PseudoVNCLIPU_WV_MF2_MASK	= 6789,
    RISCV_PseudoVNCLIPU_WV_MF4	= 6790,
    RISCV_PseudoVNCLIPU_WV_MF4_MASK	= 6791,
    RISCV_PseudoVNCLIPU_WV_MF8	= 6792,
    RISCV_PseudoVNCLIPU_WV_MF8_MASK	= 6793,
    RISCV_PseudoVNCLIPU_WX_M1	= 6794,
    RISCV_PseudoVNCLIPU_WX_M1_MASK	= 6795,
    RISCV_PseudoVNCLIPU_WX_M2	= 6796,
    RISCV_PseudoVNCLIPU_WX_M2_MASK	= 6797,
    RISCV_PseudoVNCLIPU_WX_M4	= 6798,
    RISCV_PseudoVNCLIPU_WX_M4_MASK	= 6799,
    RISCV_PseudoVNCLIPU_WX_MF2	= 6800,
    RISCV_PseudoVNCLIPU_WX_MF2_MASK	= 6801,
    RISCV_PseudoVNCLIPU_WX_MF4	= 6802,
    RISCV_PseudoVNCLIPU_WX_MF4_MASK	= 6803,
    RISCV_PseudoVNCLIPU_WX_MF8	= 6804,
    RISCV_PseudoVNCLIPU_WX_MF8_MASK	= 6805,
    RISCV_PseudoVNCLIP_WI_M1	= 6806,
    RISCV_PseudoVNCLIP_WI_M1_MASK	= 6807,
    RISCV_PseudoVNCLIP_WI_M2	= 6808,
    RISCV_PseudoVNCLIP_WI_M2_MASK	= 6809,
    RISCV_PseudoVNCLIP_WI_M4	= 6810,
    RISCV_PseudoVNCLIP_WI_M4_MASK	= 6811,
    RISCV_PseudoVNCLIP_WI_MF2	= 6812,
    RISCV_PseudoVNCLIP_WI_MF2_MASK	= 6813,
    RISCV_PseudoVNCLIP_WI_MF4	= 6814,
    RISCV_PseudoVNCLIP_WI_MF4_MASK	= 6815,
    RISCV_PseudoVNCLIP_WI_MF8	= 6816,
    RISCV_PseudoVNCLIP_WI_MF8_MASK	= 6817,
    RISCV_PseudoVNCLIP_WV_M1	= 6818,
    RISCV_PseudoVNCLIP_WV_M1_MASK	= 6819,
    RISCV_PseudoVNCLIP_WV_M2	= 6820,
    RISCV_PseudoVNCLIP_WV_M2_MASK	= 6821,
    RISCV_PseudoVNCLIP_WV_M4	= 6822,
    RISCV_PseudoVNCLIP_WV_M4_MASK	= 6823,
    RISCV_PseudoVNCLIP_WV_MF2	= 6824,
    RISCV_PseudoVNCLIP_WV_MF2_MASK	= 6825,
    RISCV_PseudoVNCLIP_WV_MF4	= 6826,
    RISCV_PseudoVNCLIP_WV_MF4_MASK	= 6827,
    RISCV_PseudoVNCLIP_WV_MF8	= 6828,
    RISCV_PseudoVNCLIP_WV_MF8_MASK	= 6829,
    RISCV_PseudoVNCLIP_WX_M1	= 6830,
    RISCV_PseudoVNCLIP_WX_M1_MASK	= 6831,
    RISCV_PseudoVNCLIP_WX_M2	= 6832,
    RISCV_PseudoVNCLIP_WX_M2_MASK	= 6833,
    RISCV_PseudoVNCLIP_WX_M4	= 6834,
    RISCV_PseudoVNCLIP_WX_M4_MASK	= 6835,
    RISCV_PseudoVNCLIP_WX_MF2	= 6836,
    RISCV_PseudoVNCLIP_WX_MF2_MASK	= 6837,
    RISCV_PseudoVNCLIP_WX_MF4	= 6838,
    RISCV_PseudoVNCLIP_WX_MF4_MASK	= 6839,
    RISCV_PseudoVNCLIP_WX_MF8	= 6840,
    RISCV_PseudoVNCLIP_WX_MF8_MASK	= 6841,
    RISCV_PseudoVNMSAC_VV_M1	= 6842,
    RISCV_PseudoVNMSAC_VV_M1_MASK	= 6843,
    RISCV_PseudoVNMSAC_VV_M2	= 6844,
    RISCV_PseudoVNMSAC_VV_M2_MASK	= 6845,
    RISCV_PseudoVNMSAC_VV_M4	= 6846,
    RISCV_PseudoVNMSAC_VV_M4_MASK	= 6847,
    RISCV_PseudoVNMSAC_VV_M8	= 6848,
    RISCV_PseudoVNMSAC_VV_M8_MASK	= 6849,
    RISCV_PseudoVNMSAC_VV_MF2	= 6850,
    RISCV_PseudoVNMSAC_VV_MF2_MASK	= 6851,
    RISCV_PseudoVNMSAC_VV_MF4	= 6852,
    RISCV_PseudoVNMSAC_VV_MF4_MASK	= 6853,
    RISCV_PseudoVNMSAC_VV_MF8	= 6854,
    RISCV_PseudoVNMSAC_VV_MF8_MASK	= 6855,
    RISCV_PseudoVNMSAC_VX_M1	= 6856,
    RISCV_PseudoVNMSAC_VX_M1_MASK	= 6857,
    RISCV_PseudoVNMSAC_VX_M2	= 6858,
    RISCV_PseudoVNMSAC_VX_M2_MASK	= 6859,
    RISCV_PseudoVNMSAC_VX_M4	= 6860,
    RISCV_PseudoVNMSAC_VX_M4_MASK	= 6861,
    RISCV_PseudoVNMSAC_VX_M8	= 6862,
    RISCV_PseudoVNMSAC_VX_M8_MASK	= 6863,
    RISCV_PseudoVNMSAC_VX_MF2	= 6864,
    RISCV_PseudoVNMSAC_VX_MF2_MASK	= 6865,
    RISCV_PseudoVNMSAC_VX_MF4	= 6866,
    RISCV_PseudoVNMSAC_VX_MF4_MASK	= 6867,
    RISCV_PseudoVNMSAC_VX_MF8	= 6868,
    RISCV_PseudoVNMSAC_VX_MF8_MASK	= 6869,
    RISCV_PseudoVNMSUB_VV_M1	= 6870,
    RISCV_PseudoVNMSUB_VV_M1_MASK	= 6871,
    RISCV_PseudoVNMSUB_VV_M2	= 6872,
    RISCV_PseudoVNMSUB_VV_M2_MASK	= 6873,
    RISCV_PseudoVNMSUB_VV_M4	= 6874,
    RISCV_PseudoVNMSUB_VV_M4_MASK	= 6875,
    RISCV_PseudoVNMSUB_VV_M8	= 6876,
    RISCV_PseudoVNMSUB_VV_M8_MASK	= 6877,
    RISCV_PseudoVNMSUB_VV_MF2	= 6878,
    RISCV_PseudoVNMSUB_VV_MF2_MASK	= 6879,
    RISCV_PseudoVNMSUB_VV_MF4	= 6880,
    RISCV_PseudoVNMSUB_VV_MF4_MASK	= 6881,
    RISCV_PseudoVNMSUB_VV_MF8	= 6882,
    RISCV_PseudoVNMSUB_VV_MF8_MASK	= 6883,
    RISCV_PseudoVNMSUB_VX_M1	= 6884,
    RISCV_PseudoVNMSUB_VX_M1_MASK	= 6885,
    RISCV_PseudoVNMSUB_VX_M2	= 6886,
    RISCV_PseudoVNMSUB_VX_M2_MASK	= 6887,
    RISCV_PseudoVNMSUB_VX_M4	= 6888,
    RISCV_PseudoVNMSUB_VX_M4_MASK	= 6889,
    RISCV_PseudoVNMSUB_VX_M8	= 6890,
    RISCV_PseudoVNMSUB_VX_M8_MASK	= 6891,
    RISCV_PseudoVNMSUB_VX_MF2	= 6892,
    RISCV_PseudoVNMSUB_VX_MF2_MASK	= 6893,
    RISCV_PseudoVNMSUB_VX_MF4	= 6894,
    RISCV_PseudoVNMSUB_VX_MF4_MASK	= 6895,
    RISCV_PseudoVNMSUB_VX_MF8	= 6896,
    RISCV_PseudoVNMSUB_VX_MF8_MASK	= 6897,
    RISCV_PseudoVNSRA_WI_M1	= 6898,
    RISCV_PseudoVNSRA_WI_M1_MASK	= 6899,
    RISCV_PseudoVNSRA_WI_M2	= 6900,
    RISCV_PseudoVNSRA_WI_M2_MASK	= 6901,
    RISCV_PseudoVNSRA_WI_M4	= 6902,
    RISCV_PseudoVNSRA_WI_M4_MASK	= 6903,
    RISCV_PseudoVNSRA_WI_MF2	= 6904,
    RISCV_PseudoVNSRA_WI_MF2_MASK	= 6905,
    RISCV_PseudoVNSRA_WI_MF4	= 6906,
    RISCV_PseudoVNSRA_WI_MF4_MASK	= 6907,
    RISCV_PseudoVNSRA_WI_MF8	= 6908,
    RISCV_PseudoVNSRA_WI_MF8_MASK	= 6909,
    RISCV_PseudoVNSRA_WV_M1	= 6910,
    RISCV_PseudoVNSRA_WV_M1_MASK	= 6911,
    RISCV_PseudoVNSRA_WV_M2	= 6912,
    RISCV_PseudoVNSRA_WV_M2_MASK	= 6913,
    RISCV_PseudoVNSRA_WV_M4	= 6914,
    RISCV_PseudoVNSRA_WV_M4_MASK	= 6915,
    RISCV_PseudoVNSRA_WV_MF2	= 6916,
    RISCV_PseudoVNSRA_WV_MF2_MASK	= 6917,
    RISCV_PseudoVNSRA_WV_MF4	= 6918,
    RISCV_PseudoVNSRA_WV_MF4_MASK	= 6919,
    RISCV_PseudoVNSRA_WV_MF8	= 6920,
    RISCV_PseudoVNSRA_WV_MF8_MASK	= 6921,
    RISCV_PseudoVNSRA_WX_M1	= 6922,
    RISCV_PseudoVNSRA_WX_M1_MASK	= 6923,
    RISCV_PseudoVNSRA_WX_M2	= 6924,
    RISCV_PseudoVNSRA_WX_M2_MASK	= 6925,
    RISCV_PseudoVNSRA_WX_M4	= 6926,
    RISCV_PseudoVNSRA_WX_M4_MASK	= 6927,
    RISCV_PseudoVNSRA_WX_MF2	= 6928,
    RISCV_PseudoVNSRA_WX_MF2_MASK	= 6929,
    RISCV_PseudoVNSRA_WX_MF4	= 6930,
    RISCV_PseudoVNSRA_WX_MF4_MASK	= 6931,
    RISCV_PseudoVNSRA_WX_MF8	= 6932,
    RISCV_PseudoVNSRA_WX_MF8_MASK	= 6933,
    RISCV_PseudoVNSRL_WI_M1	= 6934,
    RISCV_PseudoVNSRL_WI_M1_MASK	= 6935,
    RISCV_PseudoVNSRL_WI_M2	= 6936,
    RISCV_PseudoVNSRL_WI_M2_MASK	= 6937,
    RISCV_PseudoVNSRL_WI_M4	= 6938,
    RISCV_PseudoVNSRL_WI_M4_MASK	= 6939,
    RISCV_PseudoVNSRL_WI_MF2	= 6940,
    RISCV_PseudoVNSRL_WI_MF2_MASK	= 6941,
    RISCV_PseudoVNSRL_WI_MF4	= 6942,
    RISCV_PseudoVNSRL_WI_MF4_MASK	= 6943,
    RISCV_PseudoVNSRL_WI_MF8	= 6944,
    RISCV_PseudoVNSRL_WI_MF8_MASK	= 6945,
    RISCV_PseudoVNSRL_WV_M1	= 6946,
    RISCV_PseudoVNSRL_WV_M1_MASK	= 6947,
    RISCV_PseudoVNSRL_WV_M2	= 6948,
    RISCV_PseudoVNSRL_WV_M2_MASK	= 6949,
    RISCV_PseudoVNSRL_WV_M4	= 6950,
    RISCV_PseudoVNSRL_WV_M4_MASK	= 6951,
    RISCV_PseudoVNSRL_WV_MF2	= 6952,
    RISCV_PseudoVNSRL_WV_MF2_MASK	= 6953,
    RISCV_PseudoVNSRL_WV_MF4	= 6954,
    RISCV_PseudoVNSRL_WV_MF4_MASK	= 6955,
    RISCV_PseudoVNSRL_WV_MF8	= 6956,
    RISCV_PseudoVNSRL_WV_MF8_MASK	= 6957,
    RISCV_PseudoVNSRL_WX_M1	= 6958,
    RISCV_PseudoVNSRL_WX_M1_MASK	= 6959,
    RISCV_PseudoVNSRL_WX_M2	= 6960,
    RISCV_PseudoVNSRL_WX_M2_MASK	= 6961,
    RISCV_PseudoVNSRL_WX_M4	= 6962,
    RISCV_PseudoVNSRL_WX_M4_MASK	= 6963,
    RISCV_PseudoVNSRL_WX_MF2	= 6964,
    RISCV_PseudoVNSRL_WX_MF2_MASK	= 6965,
    RISCV_PseudoVNSRL_WX_MF4	= 6966,
    RISCV_PseudoVNSRL_WX_MF4_MASK	= 6967,
    RISCV_PseudoVNSRL_WX_MF8	= 6968,
    RISCV_PseudoVNSRL_WX_MF8_MASK	= 6969,
    RISCV_PseudoVOR_VI_M1	= 6970,
    RISCV_PseudoVOR_VI_M1_MASK	= 6971,
    RISCV_PseudoVOR_VI_M2	= 6972,
    RISCV_PseudoVOR_VI_M2_MASK	= 6973,
    RISCV_PseudoVOR_VI_M4	= 6974,
    RISCV_PseudoVOR_VI_M4_MASK	= 6975,
    RISCV_PseudoVOR_VI_M8	= 6976,
    RISCV_PseudoVOR_VI_M8_MASK	= 6977,
    RISCV_PseudoVOR_VI_MF2	= 6978,
    RISCV_PseudoVOR_VI_MF2_MASK	= 6979,
    RISCV_PseudoVOR_VI_MF4	= 6980,
    RISCV_PseudoVOR_VI_MF4_MASK	= 6981,
    RISCV_PseudoVOR_VI_MF8	= 6982,
    RISCV_PseudoVOR_VI_MF8_MASK	= 6983,
    RISCV_PseudoVOR_VV_M1	= 6984,
    RISCV_PseudoVOR_VV_M1_MASK	= 6985,
    RISCV_PseudoVOR_VV_M2	= 6986,
    RISCV_PseudoVOR_VV_M2_MASK	= 6987,
    RISCV_PseudoVOR_VV_M4	= 6988,
    RISCV_PseudoVOR_VV_M4_MASK	= 6989,
    RISCV_PseudoVOR_VV_M8	= 6990,
    RISCV_PseudoVOR_VV_M8_MASK	= 6991,
    RISCV_PseudoVOR_VV_MF2	= 6992,
    RISCV_PseudoVOR_VV_MF2_MASK	= 6993,
    RISCV_PseudoVOR_VV_MF4	= 6994,
    RISCV_PseudoVOR_VV_MF4_MASK	= 6995,
    RISCV_PseudoVOR_VV_MF8	= 6996,
    RISCV_PseudoVOR_VV_MF8_MASK	= 6997,
    RISCV_PseudoVOR_VX_M1	= 6998,
    RISCV_PseudoVOR_VX_M1_MASK	= 6999,
    RISCV_PseudoVOR_VX_M2	= 7000,
    RISCV_PseudoVOR_VX_M2_MASK	= 7001,
    RISCV_PseudoVOR_VX_M4	= 7002,
    RISCV_PseudoVOR_VX_M4_MASK	= 7003,
    RISCV_PseudoVOR_VX_M8	= 7004,
    RISCV_PseudoVOR_VX_M8_MASK	= 7005,
    RISCV_PseudoVOR_VX_MF2	= 7006,
    RISCV_PseudoVOR_VX_MF2_MASK	= 7007,
    RISCV_PseudoVOR_VX_MF4	= 7008,
    RISCV_PseudoVOR_VX_MF4_MASK	= 7009,
    RISCV_PseudoVOR_VX_MF8	= 7010,
    RISCV_PseudoVOR_VX_MF8_MASK	= 7011,
    RISCV_PseudoVQMACCSU_2x8x2_M1	= 7012,
    RISCV_PseudoVQMACCSU_2x8x2_M2	= 7013,
    RISCV_PseudoVQMACCSU_2x8x2_M4	= 7014,
    RISCV_PseudoVQMACCSU_2x8x2_M8	= 7015,
    RISCV_PseudoVQMACCSU_4x8x4_M1	= 7016,
    RISCV_PseudoVQMACCSU_4x8x4_M2	= 7017,
    RISCV_PseudoVQMACCSU_4x8x4_M4	= 7018,
    RISCV_PseudoVQMACCSU_4x8x4_MF2	= 7019,
    RISCV_PseudoVQMACCUS_2x8x2_M1	= 7020,
    RISCV_PseudoVQMACCUS_2x8x2_M2	= 7021,
    RISCV_PseudoVQMACCUS_2x8x2_M4	= 7022,
    RISCV_PseudoVQMACCUS_2x8x2_M8	= 7023,
    RISCV_PseudoVQMACCUS_4x8x4_M1	= 7024,
    RISCV_PseudoVQMACCUS_4x8x4_M2	= 7025,
    RISCV_PseudoVQMACCUS_4x8x4_M4	= 7026,
    RISCV_PseudoVQMACCUS_4x8x4_MF2	= 7027,
    RISCV_PseudoVQMACCU_2x8x2_M1	= 7028,
    RISCV_PseudoVQMACCU_2x8x2_M2	= 7029,
    RISCV_PseudoVQMACCU_2x8x2_M4	= 7030,
    RISCV_PseudoVQMACCU_2x8x2_M8	= 7031,
    RISCV_PseudoVQMACCU_4x8x4_M1	= 7032,
    RISCV_PseudoVQMACCU_4x8x4_M2	= 7033,
    RISCV_PseudoVQMACCU_4x8x4_M4	= 7034,
    RISCV_PseudoVQMACCU_4x8x4_MF2	= 7035,
    RISCV_PseudoVQMACC_2x8x2_M1	= 7036,
    RISCV_PseudoVQMACC_2x8x2_M2	= 7037,
    RISCV_PseudoVQMACC_2x8x2_M4	= 7038,
    RISCV_PseudoVQMACC_2x8x2_M8	= 7039,
    RISCV_PseudoVQMACC_4x8x4_M1	= 7040,
    RISCV_PseudoVQMACC_4x8x4_M2	= 7041,
    RISCV_PseudoVQMACC_4x8x4_M4	= 7042,
    RISCV_PseudoVQMACC_4x8x4_MF2	= 7043,
    RISCV_PseudoVREDAND_VS_M1_E16	= 7044,
    RISCV_PseudoVREDAND_VS_M1_E16_MASK	= 7045,
    RISCV_PseudoVREDAND_VS_M1_E32	= 7046,
    RISCV_PseudoVREDAND_VS_M1_E32_MASK	= 7047,
    RISCV_PseudoVREDAND_VS_M1_E64	= 7048,
    RISCV_PseudoVREDAND_VS_M1_E64_MASK	= 7049,
    RISCV_PseudoVREDAND_VS_M1_E8	= 7050,
    RISCV_PseudoVREDAND_VS_M1_E8_MASK	= 7051,
    RISCV_PseudoVREDAND_VS_M2_E16	= 7052,
    RISCV_PseudoVREDAND_VS_M2_E16_MASK	= 7053,
    RISCV_PseudoVREDAND_VS_M2_E32	= 7054,
    RISCV_PseudoVREDAND_VS_M2_E32_MASK	= 7055,
    RISCV_PseudoVREDAND_VS_M2_E64	= 7056,
    RISCV_PseudoVREDAND_VS_M2_E64_MASK	= 7057,
    RISCV_PseudoVREDAND_VS_M2_E8	= 7058,
    RISCV_PseudoVREDAND_VS_M2_E8_MASK	= 7059,
    RISCV_PseudoVREDAND_VS_M4_E16	= 7060,
    RISCV_PseudoVREDAND_VS_M4_E16_MASK	= 7061,
    RISCV_PseudoVREDAND_VS_M4_E32	= 7062,
    RISCV_PseudoVREDAND_VS_M4_E32_MASK	= 7063,
    RISCV_PseudoVREDAND_VS_M4_E64	= 7064,
    RISCV_PseudoVREDAND_VS_M4_E64_MASK	= 7065,
    RISCV_PseudoVREDAND_VS_M4_E8	= 7066,
    RISCV_PseudoVREDAND_VS_M4_E8_MASK	= 7067,
    RISCV_PseudoVREDAND_VS_M8_E16	= 7068,
    RISCV_PseudoVREDAND_VS_M8_E16_MASK	= 7069,
    RISCV_PseudoVREDAND_VS_M8_E32	= 7070,
    RISCV_PseudoVREDAND_VS_M8_E32_MASK	= 7071,
    RISCV_PseudoVREDAND_VS_M8_E64	= 7072,
    RISCV_PseudoVREDAND_VS_M8_E64_MASK	= 7073,
    RISCV_PseudoVREDAND_VS_M8_E8	= 7074,
    RISCV_PseudoVREDAND_VS_M8_E8_MASK	= 7075,
    RISCV_PseudoVREDAND_VS_MF2_E16	= 7076,
    RISCV_PseudoVREDAND_VS_MF2_E16_MASK	= 7077,
    RISCV_PseudoVREDAND_VS_MF2_E32	= 7078,
    RISCV_PseudoVREDAND_VS_MF2_E32_MASK	= 7079,
    RISCV_PseudoVREDAND_VS_MF2_E8	= 7080,
    RISCV_PseudoVREDAND_VS_MF2_E8_MASK	= 7081,
    RISCV_PseudoVREDAND_VS_MF4_E16	= 7082,
    RISCV_PseudoVREDAND_VS_MF4_E16_MASK	= 7083,
    RISCV_PseudoVREDAND_VS_MF4_E8	= 7084,
    RISCV_PseudoVREDAND_VS_MF4_E8_MASK	= 7085,
    RISCV_PseudoVREDAND_VS_MF8_E8	= 7086,
    RISCV_PseudoVREDAND_VS_MF8_E8_MASK	= 7087,
    RISCV_PseudoVREDMAXU_VS_M1_E16	= 7088,
    RISCV_PseudoVREDMAXU_VS_M1_E16_MASK	= 7089,
    RISCV_PseudoVREDMAXU_VS_M1_E32	= 7090,
    RISCV_PseudoVREDMAXU_VS_M1_E32_MASK	= 7091,
    RISCV_PseudoVREDMAXU_VS_M1_E64	= 7092,
    RISCV_PseudoVREDMAXU_VS_M1_E64_MASK	= 7093,
    RISCV_PseudoVREDMAXU_VS_M1_E8	= 7094,
    RISCV_PseudoVREDMAXU_VS_M1_E8_MASK	= 7095,
    RISCV_PseudoVREDMAXU_VS_M2_E16	= 7096,
    RISCV_PseudoVREDMAXU_VS_M2_E16_MASK	= 7097,
    RISCV_PseudoVREDMAXU_VS_M2_E32	= 7098,
    RISCV_PseudoVREDMAXU_VS_M2_E32_MASK	= 7099,
    RISCV_PseudoVREDMAXU_VS_M2_E64	= 7100,
    RISCV_PseudoVREDMAXU_VS_M2_E64_MASK	= 7101,
    RISCV_PseudoVREDMAXU_VS_M2_E8	= 7102,
    RISCV_PseudoVREDMAXU_VS_M2_E8_MASK	= 7103,
    RISCV_PseudoVREDMAXU_VS_M4_E16	= 7104,
    RISCV_PseudoVREDMAXU_VS_M4_E16_MASK	= 7105,
    RISCV_PseudoVREDMAXU_VS_M4_E32	= 7106,
    RISCV_PseudoVREDMAXU_VS_M4_E32_MASK	= 7107,
    RISCV_PseudoVREDMAXU_VS_M4_E64	= 7108,
    RISCV_PseudoVREDMAXU_VS_M4_E64_MASK	= 7109,
    RISCV_PseudoVREDMAXU_VS_M4_E8	= 7110,
    RISCV_PseudoVREDMAXU_VS_M4_E8_MASK	= 7111,
    RISCV_PseudoVREDMAXU_VS_M8_E16	= 7112,
    RISCV_PseudoVREDMAXU_VS_M8_E16_MASK	= 7113,
    RISCV_PseudoVREDMAXU_VS_M8_E32	= 7114,
    RISCV_PseudoVREDMAXU_VS_M8_E32_MASK	= 7115,
    RISCV_PseudoVREDMAXU_VS_M8_E64	= 7116,
    RISCV_PseudoVREDMAXU_VS_M8_E64_MASK	= 7117,
    RISCV_PseudoVREDMAXU_VS_M8_E8	= 7118,
    RISCV_PseudoVREDMAXU_VS_M8_E8_MASK	= 7119,
    RISCV_PseudoVREDMAXU_VS_MF2_E16	= 7120,
    RISCV_PseudoVREDMAXU_VS_MF2_E16_MASK	= 7121,
    RISCV_PseudoVREDMAXU_VS_MF2_E32	= 7122,
    RISCV_PseudoVREDMAXU_VS_MF2_E32_MASK	= 7123,
    RISCV_PseudoVREDMAXU_VS_MF2_E8	= 7124,
    RISCV_PseudoVREDMAXU_VS_MF2_E8_MASK	= 7125,
    RISCV_PseudoVREDMAXU_VS_MF4_E16	= 7126,
    RISCV_PseudoVREDMAXU_VS_MF4_E16_MASK	= 7127,
    RISCV_PseudoVREDMAXU_VS_MF4_E8	= 7128,
    RISCV_PseudoVREDMAXU_VS_MF4_E8_MASK	= 7129,
    RISCV_PseudoVREDMAXU_VS_MF8_E8	= 7130,
    RISCV_PseudoVREDMAXU_VS_MF8_E8_MASK	= 7131,
    RISCV_PseudoVREDMAX_VS_M1_E16	= 7132,
    RISCV_PseudoVREDMAX_VS_M1_E16_MASK	= 7133,
    RISCV_PseudoVREDMAX_VS_M1_E32	= 7134,
    RISCV_PseudoVREDMAX_VS_M1_E32_MASK	= 7135,
    RISCV_PseudoVREDMAX_VS_M1_E64	= 7136,
    RISCV_PseudoVREDMAX_VS_M1_E64_MASK	= 7137,
    RISCV_PseudoVREDMAX_VS_M1_E8	= 7138,
    RISCV_PseudoVREDMAX_VS_M1_E8_MASK	= 7139,
    RISCV_PseudoVREDMAX_VS_M2_E16	= 7140,
    RISCV_PseudoVREDMAX_VS_M2_E16_MASK	= 7141,
    RISCV_PseudoVREDMAX_VS_M2_E32	= 7142,
    RISCV_PseudoVREDMAX_VS_M2_E32_MASK	= 7143,
    RISCV_PseudoVREDMAX_VS_M2_E64	= 7144,
    RISCV_PseudoVREDMAX_VS_M2_E64_MASK	= 7145,
    RISCV_PseudoVREDMAX_VS_M2_E8	= 7146,
    RISCV_PseudoVREDMAX_VS_M2_E8_MASK	= 7147,
    RISCV_PseudoVREDMAX_VS_M4_E16	= 7148,
    RISCV_PseudoVREDMAX_VS_M4_E16_MASK	= 7149,
    RISCV_PseudoVREDMAX_VS_M4_E32	= 7150,
    RISCV_PseudoVREDMAX_VS_M4_E32_MASK	= 7151,
    RISCV_PseudoVREDMAX_VS_M4_E64	= 7152,
    RISCV_PseudoVREDMAX_VS_M4_E64_MASK	= 7153,
    RISCV_PseudoVREDMAX_VS_M4_E8	= 7154,
    RISCV_PseudoVREDMAX_VS_M4_E8_MASK	= 7155,
    RISCV_PseudoVREDMAX_VS_M8_E16	= 7156,
    RISCV_PseudoVREDMAX_VS_M8_E16_MASK	= 7157,
    RISCV_PseudoVREDMAX_VS_M8_E32	= 7158,
    RISCV_PseudoVREDMAX_VS_M8_E32_MASK	= 7159,
    RISCV_PseudoVREDMAX_VS_M8_E64	= 7160,
    RISCV_PseudoVREDMAX_VS_M8_E64_MASK	= 7161,
    RISCV_PseudoVREDMAX_VS_M8_E8	= 7162,
    RISCV_PseudoVREDMAX_VS_M8_E8_MASK	= 7163,
    RISCV_PseudoVREDMAX_VS_MF2_E16	= 7164,
    RISCV_PseudoVREDMAX_VS_MF2_E16_MASK	= 7165,
    RISCV_PseudoVREDMAX_VS_MF2_E32	= 7166,
    RISCV_PseudoVREDMAX_VS_MF2_E32_MASK	= 7167,
    RISCV_PseudoVREDMAX_VS_MF2_E8	= 7168,
    RISCV_PseudoVREDMAX_VS_MF2_E8_MASK	= 7169,
    RISCV_PseudoVREDMAX_VS_MF4_E16	= 7170,
    RISCV_PseudoVREDMAX_VS_MF4_E16_MASK	= 7171,
    RISCV_PseudoVREDMAX_VS_MF4_E8	= 7172,
    RISCV_PseudoVREDMAX_VS_MF4_E8_MASK	= 7173,
    RISCV_PseudoVREDMAX_VS_MF8_E8	= 7174,
    RISCV_PseudoVREDMAX_VS_MF8_E8_MASK	= 7175,
    RISCV_PseudoVREDMINU_VS_M1_E16	= 7176,
    RISCV_PseudoVREDMINU_VS_M1_E16_MASK	= 7177,
    RISCV_PseudoVREDMINU_VS_M1_E32	= 7178,
    RISCV_PseudoVREDMINU_VS_M1_E32_MASK	= 7179,
    RISCV_PseudoVREDMINU_VS_M1_E64	= 7180,
    RISCV_PseudoVREDMINU_VS_M1_E64_MASK	= 7181,
    RISCV_PseudoVREDMINU_VS_M1_E8	= 7182,
    RISCV_PseudoVREDMINU_VS_M1_E8_MASK	= 7183,
    RISCV_PseudoVREDMINU_VS_M2_E16	= 7184,
    RISCV_PseudoVREDMINU_VS_M2_E16_MASK	= 7185,
    RISCV_PseudoVREDMINU_VS_M2_E32	= 7186,
    RISCV_PseudoVREDMINU_VS_M2_E32_MASK	= 7187,
    RISCV_PseudoVREDMINU_VS_M2_E64	= 7188,
    RISCV_PseudoVREDMINU_VS_M2_E64_MASK	= 7189,
    RISCV_PseudoVREDMINU_VS_M2_E8	= 7190,
    RISCV_PseudoVREDMINU_VS_M2_E8_MASK	= 7191,
    RISCV_PseudoVREDMINU_VS_M4_E16	= 7192,
    RISCV_PseudoVREDMINU_VS_M4_E16_MASK	= 7193,
    RISCV_PseudoVREDMINU_VS_M4_E32	= 7194,
    RISCV_PseudoVREDMINU_VS_M4_E32_MASK	= 7195,
    RISCV_PseudoVREDMINU_VS_M4_E64	= 7196,
    RISCV_PseudoVREDMINU_VS_M4_E64_MASK	= 7197,
    RISCV_PseudoVREDMINU_VS_M4_E8	= 7198,
    RISCV_PseudoVREDMINU_VS_M4_E8_MASK	= 7199,
    RISCV_PseudoVREDMINU_VS_M8_E16	= 7200,
    RISCV_PseudoVREDMINU_VS_M8_E16_MASK	= 7201,
    RISCV_PseudoVREDMINU_VS_M8_E32	= 7202,
    RISCV_PseudoVREDMINU_VS_M8_E32_MASK	= 7203,
    RISCV_PseudoVREDMINU_VS_M8_E64	= 7204,
    RISCV_PseudoVREDMINU_VS_M8_E64_MASK	= 7205,
    RISCV_PseudoVREDMINU_VS_M8_E8	= 7206,
    RISCV_PseudoVREDMINU_VS_M8_E8_MASK	= 7207,
    RISCV_PseudoVREDMINU_VS_MF2_E16	= 7208,
    RISCV_PseudoVREDMINU_VS_MF2_E16_MASK	= 7209,
    RISCV_PseudoVREDMINU_VS_MF2_E32	= 7210,
    RISCV_PseudoVREDMINU_VS_MF2_E32_MASK	= 7211,
    RISCV_PseudoVREDMINU_VS_MF2_E8	= 7212,
    RISCV_PseudoVREDMINU_VS_MF2_E8_MASK	= 7213,
    RISCV_PseudoVREDMINU_VS_MF4_E16	= 7214,
    RISCV_PseudoVREDMINU_VS_MF4_E16_MASK	= 7215,
    RISCV_PseudoVREDMINU_VS_MF4_E8	= 7216,
    RISCV_PseudoVREDMINU_VS_MF4_E8_MASK	= 7217,
    RISCV_PseudoVREDMINU_VS_MF8_E8	= 7218,
    RISCV_PseudoVREDMINU_VS_MF8_E8_MASK	= 7219,
    RISCV_PseudoVREDMIN_VS_M1_E16	= 7220,
    RISCV_PseudoVREDMIN_VS_M1_E16_MASK	= 7221,
    RISCV_PseudoVREDMIN_VS_M1_E32	= 7222,
    RISCV_PseudoVREDMIN_VS_M1_E32_MASK	= 7223,
    RISCV_PseudoVREDMIN_VS_M1_E64	= 7224,
    RISCV_PseudoVREDMIN_VS_M1_E64_MASK	= 7225,
    RISCV_PseudoVREDMIN_VS_M1_E8	= 7226,
    RISCV_PseudoVREDMIN_VS_M1_E8_MASK	= 7227,
    RISCV_PseudoVREDMIN_VS_M2_E16	= 7228,
    RISCV_PseudoVREDMIN_VS_M2_E16_MASK	= 7229,
    RISCV_PseudoVREDMIN_VS_M2_E32	= 7230,
    RISCV_PseudoVREDMIN_VS_M2_E32_MASK	= 7231,
    RISCV_PseudoVREDMIN_VS_M2_E64	= 7232,
    RISCV_PseudoVREDMIN_VS_M2_E64_MASK	= 7233,
    RISCV_PseudoVREDMIN_VS_M2_E8	= 7234,
    RISCV_PseudoVREDMIN_VS_M2_E8_MASK	= 7235,
    RISCV_PseudoVREDMIN_VS_M4_E16	= 7236,
    RISCV_PseudoVREDMIN_VS_M4_E16_MASK	= 7237,
    RISCV_PseudoVREDMIN_VS_M4_E32	= 7238,
    RISCV_PseudoVREDMIN_VS_M4_E32_MASK	= 7239,
    RISCV_PseudoVREDMIN_VS_M4_E64	= 7240,
    RISCV_PseudoVREDMIN_VS_M4_E64_MASK	= 7241,
    RISCV_PseudoVREDMIN_VS_M4_E8	= 7242,
    RISCV_PseudoVREDMIN_VS_M4_E8_MASK	= 7243,
    RISCV_PseudoVREDMIN_VS_M8_E16	= 7244,
    RISCV_PseudoVREDMIN_VS_M8_E16_MASK	= 7245,
    RISCV_PseudoVREDMIN_VS_M8_E32	= 7246,
    RISCV_PseudoVREDMIN_VS_M8_E32_MASK	= 7247,
    RISCV_PseudoVREDMIN_VS_M8_E64	= 7248,
    RISCV_PseudoVREDMIN_VS_M8_E64_MASK	= 7249,
    RISCV_PseudoVREDMIN_VS_M8_E8	= 7250,
    RISCV_PseudoVREDMIN_VS_M8_E8_MASK	= 7251,
    RISCV_PseudoVREDMIN_VS_MF2_E16	= 7252,
    RISCV_PseudoVREDMIN_VS_MF2_E16_MASK	= 7253,
    RISCV_PseudoVREDMIN_VS_MF2_E32	= 7254,
    RISCV_PseudoVREDMIN_VS_MF2_E32_MASK	= 7255,
    RISCV_PseudoVREDMIN_VS_MF2_E8	= 7256,
    RISCV_PseudoVREDMIN_VS_MF2_E8_MASK	= 7257,
    RISCV_PseudoVREDMIN_VS_MF4_E16	= 7258,
    RISCV_PseudoVREDMIN_VS_MF4_E16_MASK	= 7259,
    RISCV_PseudoVREDMIN_VS_MF4_E8	= 7260,
    RISCV_PseudoVREDMIN_VS_MF4_E8_MASK	= 7261,
    RISCV_PseudoVREDMIN_VS_MF8_E8	= 7262,
    RISCV_PseudoVREDMIN_VS_MF8_E8_MASK	= 7263,
    RISCV_PseudoVREDOR_VS_M1_E16	= 7264,
    RISCV_PseudoVREDOR_VS_M1_E16_MASK	= 7265,
    RISCV_PseudoVREDOR_VS_M1_E32	= 7266,
    RISCV_PseudoVREDOR_VS_M1_E32_MASK	= 7267,
    RISCV_PseudoVREDOR_VS_M1_E64	= 7268,
    RISCV_PseudoVREDOR_VS_M1_E64_MASK	= 7269,
    RISCV_PseudoVREDOR_VS_M1_E8	= 7270,
    RISCV_PseudoVREDOR_VS_M1_E8_MASK	= 7271,
    RISCV_PseudoVREDOR_VS_M2_E16	= 7272,
    RISCV_PseudoVREDOR_VS_M2_E16_MASK	= 7273,
    RISCV_PseudoVREDOR_VS_M2_E32	= 7274,
    RISCV_PseudoVREDOR_VS_M2_E32_MASK	= 7275,
    RISCV_PseudoVREDOR_VS_M2_E64	= 7276,
    RISCV_PseudoVREDOR_VS_M2_E64_MASK	= 7277,
    RISCV_PseudoVREDOR_VS_M2_E8	= 7278,
    RISCV_PseudoVREDOR_VS_M2_E8_MASK	= 7279,
    RISCV_PseudoVREDOR_VS_M4_E16	= 7280,
    RISCV_PseudoVREDOR_VS_M4_E16_MASK	= 7281,
    RISCV_PseudoVREDOR_VS_M4_E32	= 7282,
    RISCV_PseudoVREDOR_VS_M4_E32_MASK	= 7283,
    RISCV_PseudoVREDOR_VS_M4_E64	= 7284,
    RISCV_PseudoVREDOR_VS_M4_E64_MASK	= 7285,
    RISCV_PseudoVREDOR_VS_M4_E8	= 7286,
    RISCV_PseudoVREDOR_VS_M4_E8_MASK	= 7287,
    RISCV_PseudoVREDOR_VS_M8_E16	= 7288,
    RISCV_PseudoVREDOR_VS_M8_E16_MASK	= 7289,
    RISCV_PseudoVREDOR_VS_M8_E32	= 7290,
    RISCV_PseudoVREDOR_VS_M8_E32_MASK	= 7291,
    RISCV_PseudoVREDOR_VS_M8_E64	= 7292,
    RISCV_PseudoVREDOR_VS_M8_E64_MASK	= 7293,
    RISCV_PseudoVREDOR_VS_M8_E8	= 7294,
    RISCV_PseudoVREDOR_VS_M8_E8_MASK	= 7295,
    RISCV_PseudoVREDOR_VS_MF2_E16	= 7296,
    RISCV_PseudoVREDOR_VS_MF2_E16_MASK	= 7297,
    RISCV_PseudoVREDOR_VS_MF2_E32	= 7298,
    RISCV_PseudoVREDOR_VS_MF2_E32_MASK	= 7299,
    RISCV_PseudoVREDOR_VS_MF2_E8	= 7300,
    RISCV_PseudoVREDOR_VS_MF2_E8_MASK	= 7301,
    RISCV_PseudoVREDOR_VS_MF4_E16	= 7302,
    RISCV_PseudoVREDOR_VS_MF4_E16_MASK	= 7303,
    RISCV_PseudoVREDOR_VS_MF4_E8	= 7304,
    RISCV_PseudoVREDOR_VS_MF4_E8_MASK	= 7305,
    RISCV_PseudoVREDOR_VS_MF8_E8	= 7306,
    RISCV_PseudoVREDOR_VS_MF8_E8_MASK	= 7307,
    RISCV_PseudoVREDSUM_VS_M1_E16	= 7308,
    RISCV_PseudoVREDSUM_VS_M1_E16_MASK	= 7309,
    RISCV_PseudoVREDSUM_VS_M1_E32	= 7310,
    RISCV_PseudoVREDSUM_VS_M1_E32_MASK	= 7311,
    RISCV_PseudoVREDSUM_VS_M1_E64	= 7312,
    RISCV_PseudoVREDSUM_VS_M1_E64_MASK	= 7313,
    RISCV_PseudoVREDSUM_VS_M1_E8	= 7314,
    RISCV_PseudoVREDSUM_VS_M1_E8_MASK	= 7315,
    RISCV_PseudoVREDSUM_VS_M2_E16	= 7316,
    RISCV_PseudoVREDSUM_VS_M2_E16_MASK	= 7317,
    RISCV_PseudoVREDSUM_VS_M2_E32	= 7318,
    RISCV_PseudoVREDSUM_VS_M2_E32_MASK	= 7319,
    RISCV_PseudoVREDSUM_VS_M2_E64	= 7320,
    RISCV_PseudoVREDSUM_VS_M2_E64_MASK	= 7321,
    RISCV_PseudoVREDSUM_VS_M2_E8	= 7322,
    RISCV_PseudoVREDSUM_VS_M2_E8_MASK	= 7323,
    RISCV_PseudoVREDSUM_VS_M4_E16	= 7324,
    RISCV_PseudoVREDSUM_VS_M4_E16_MASK	= 7325,
    RISCV_PseudoVREDSUM_VS_M4_E32	= 7326,
    RISCV_PseudoVREDSUM_VS_M4_E32_MASK	= 7327,
    RISCV_PseudoVREDSUM_VS_M4_E64	= 7328,
    RISCV_PseudoVREDSUM_VS_M4_E64_MASK	= 7329,
    RISCV_PseudoVREDSUM_VS_M4_E8	= 7330,
    RISCV_PseudoVREDSUM_VS_M4_E8_MASK	= 7331,
    RISCV_PseudoVREDSUM_VS_M8_E16	= 7332,
    RISCV_PseudoVREDSUM_VS_M8_E16_MASK	= 7333,
    RISCV_PseudoVREDSUM_VS_M8_E32	= 7334,
    RISCV_PseudoVREDSUM_VS_M8_E32_MASK	= 7335,
    RISCV_PseudoVREDSUM_VS_M8_E64	= 7336,
    RISCV_PseudoVREDSUM_VS_M8_E64_MASK	= 7337,
    RISCV_PseudoVREDSUM_VS_M8_E8	= 7338,
    RISCV_PseudoVREDSUM_VS_M8_E8_MASK	= 7339,
    RISCV_PseudoVREDSUM_VS_MF2_E16	= 7340,
    RISCV_PseudoVREDSUM_VS_MF2_E16_MASK	= 7341,
    RISCV_PseudoVREDSUM_VS_MF2_E32	= 7342,
    RISCV_PseudoVREDSUM_VS_MF2_E32_MASK	= 7343,
    RISCV_PseudoVREDSUM_VS_MF2_E8	= 7344,
    RISCV_PseudoVREDSUM_VS_MF2_E8_MASK	= 7345,
    RISCV_PseudoVREDSUM_VS_MF4_E16	= 7346,
    RISCV_PseudoVREDSUM_VS_MF4_E16_MASK	= 7347,
    RISCV_PseudoVREDSUM_VS_MF4_E8	= 7348,
    RISCV_PseudoVREDSUM_VS_MF4_E8_MASK	= 7349,
    RISCV_PseudoVREDSUM_VS_MF8_E8	= 7350,
    RISCV_PseudoVREDSUM_VS_MF8_E8_MASK	= 7351,
    RISCV_PseudoVREDXOR_VS_M1_E16	= 7352,
    RISCV_PseudoVREDXOR_VS_M1_E16_MASK	= 7353,
    RISCV_PseudoVREDXOR_VS_M1_E32	= 7354,
    RISCV_PseudoVREDXOR_VS_M1_E32_MASK	= 7355,
    RISCV_PseudoVREDXOR_VS_M1_E64	= 7356,
    RISCV_PseudoVREDXOR_VS_M1_E64_MASK	= 7357,
    RISCV_PseudoVREDXOR_VS_M1_E8	= 7358,
    RISCV_PseudoVREDXOR_VS_M1_E8_MASK	= 7359,
    RISCV_PseudoVREDXOR_VS_M2_E16	= 7360,
    RISCV_PseudoVREDXOR_VS_M2_E16_MASK	= 7361,
    RISCV_PseudoVREDXOR_VS_M2_E32	= 7362,
    RISCV_PseudoVREDXOR_VS_M2_E32_MASK	= 7363,
    RISCV_PseudoVREDXOR_VS_M2_E64	= 7364,
    RISCV_PseudoVREDXOR_VS_M2_E64_MASK	= 7365,
    RISCV_PseudoVREDXOR_VS_M2_E8	= 7366,
    RISCV_PseudoVREDXOR_VS_M2_E8_MASK	= 7367,
    RISCV_PseudoVREDXOR_VS_M4_E16	= 7368,
    RISCV_PseudoVREDXOR_VS_M4_E16_MASK	= 7369,
    RISCV_PseudoVREDXOR_VS_M4_E32	= 7370,
    RISCV_PseudoVREDXOR_VS_M4_E32_MASK	= 7371,
    RISCV_PseudoVREDXOR_VS_M4_E64	= 7372,
    RISCV_PseudoVREDXOR_VS_M4_E64_MASK	= 7373,
    RISCV_PseudoVREDXOR_VS_M4_E8	= 7374,
    RISCV_PseudoVREDXOR_VS_M4_E8_MASK	= 7375,
    RISCV_PseudoVREDXOR_VS_M8_E16	= 7376,
    RISCV_PseudoVREDXOR_VS_M8_E16_MASK	= 7377,
    RISCV_PseudoVREDXOR_VS_M8_E32	= 7378,
    RISCV_PseudoVREDXOR_VS_M8_E32_MASK	= 7379,
    RISCV_PseudoVREDXOR_VS_M8_E64	= 7380,
    RISCV_PseudoVREDXOR_VS_M8_E64_MASK	= 7381,
    RISCV_PseudoVREDXOR_VS_M8_E8	= 7382,
    RISCV_PseudoVREDXOR_VS_M8_E8_MASK	= 7383,
    RISCV_PseudoVREDXOR_VS_MF2_E16	= 7384,
    RISCV_PseudoVREDXOR_VS_MF2_E16_MASK	= 7385,
    RISCV_PseudoVREDXOR_VS_MF2_E32	= 7386,
    RISCV_PseudoVREDXOR_VS_MF2_E32_MASK	= 7387,
    RISCV_PseudoVREDXOR_VS_MF2_E8	= 7388,
    RISCV_PseudoVREDXOR_VS_MF2_E8_MASK	= 7389,
    RISCV_PseudoVREDXOR_VS_MF4_E16	= 7390,
    RISCV_PseudoVREDXOR_VS_MF4_E16_MASK	= 7391,
    RISCV_PseudoVREDXOR_VS_MF4_E8	= 7392,
    RISCV_PseudoVREDXOR_VS_MF4_E8_MASK	= 7393,
    RISCV_PseudoVREDXOR_VS_MF8_E8	= 7394,
    RISCV_PseudoVREDXOR_VS_MF8_E8_MASK	= 7395,
    RISCV_PseudoVRELOAD2_M1	= 7396,
    RISCV_PseudoVRELOAD2_M2	= 7397,
    RISCV_PseudoVRELOAD2_M4	= 7398,
    RISCV_PseudoVRELOAD2_MF2	= 7399,
    RISCV_PseudoVRELOAD2_MF4	= 7400,
    RISCV_PseudoVRELOAD2_MF8	= 7401,
    RISCV_PseudoVRELOAD3_M1	= 7402,
    RISCV_PseudoVRELOAD3_M2	= 7403,
    RISCV_PseudoVRELOAD3_MF2	= 7404,
    RISCV_PseudoVRELOAD3_MF4	= 7405,
    RISCV_PseudoVRELOAD3_MF8	= 7406,
    RISCV_PseudoVRELOAD4_M1	= 7407,
    RISCV_PseudoVRELOAD4_M2	= 7408,
    RISCV_PseudoVRELOAD4_MF2	= 7409,
    RISCV_PseudoVRELOAD4_MF4	= 7410,
    RISCV_PseudoVRELOAD4_MF8	= 7411,
    RISCV_PseudoVRELOAD5_M1	= 7412,
    RISCV_PseudoVRELOAD5_MF2	= 7413,
    RISCV_PseudoVRELOAD5_MF4	= 7414,
    RISCV_PseudoVRELOAD5_MF8	= 7415,
    RISCV_PseudoVRELOAD6_M1	= 7416,
    RISCV_PseudoVRELOAD6_MF2	= 7417,
    RISCV_PseudoVRELOAD6_MF4	= 7418,
    RISCV_PseudoVRELOAD6_MF8	= 7419,
    RISCV_PseudoVRELOAD7_M1	= 7420,
    RISCV_PseudoVRELOAD7_MF2	= 7421,
    RISCV_PseudoVRELOAD7_MF4	= 7422,
    RISCV_PseudoVRELOAD7_MF8	= 7423,
    RISCV_PseudoVRELOAD8_M1	= 7424,
    RISCV_PseudoVRELOAD8_MF2	= 7425,
    RISCV_PseudoVRELOAD8_MF4	= 7426,
    RISCV_PseudoVRELOAD8_MF8	= 7427,
    RISCV_PseudoVREMU_VV_M1_E16	= 7428,
    RISCV_PseudoVREMU_VV_M1_E16_MASK	= 7429,
    RISCV_PseudoVREMU_VV_M1_E32	= 7430,
    RISCV_PseudoVREMU_VV_M1_E32_MASK	= 7431,
    RISCV_PseudoVREMU_VV_M1_E64	= 7432,
    RISCV_PseudoVREMU_VV_M1_E64_MASK	= 7433,
    RISCV_PseudoVREMU_VV_M1_E8	= 7434,
    RISCV_PseudoVREMU_VV_M1_E8_MASK	= 7435,
    RISCV_PseudoVREMU_VV_M2_E16	= 7436,
    RISCV_PseudoVREMU_VV_M2_E16_MASK	= 7437,
    RISCV_PseudoVREMU_VV_M2_E32	= 7438,
    RISCV_PseudoVREMU_VV_M2_E32_MASK	= 7439,
    RISCV_PseudoVREMU_VV_M2_E64	= 7440,
    RISCV_PseudoVREMU_VV_M2_E64_MASK	= 7441,
    RISCV_PseudoVREMU_VV_M2_E8	= 7442,
    RISCV_PseudoVREMU_VV_M2_E8_MASK	= 7443,
    RISCV_PseudoVREMU_VV_M4_E16	= 7444,
    RISCV_PseudoVREMU_VV_M4_E16_MASK	= 7445,
    RISCV_PseudoVREMU_VV_M4_E32	= 7446,
    RISCV_PseudoVREMU_VV_M4_E32_MASK	= 7447,
    RISCV_PseudoVREMU_VV_M4_E64	= 7448,
    RISCV_PseudoVREMU_VV_M4_E64_MASK	= 7449,
    RISCV_PseudoVREMU_VV_M4_E8	= 7450,
    RISCV_PseudoVREMU_VV_M4_E8_MASK	= 7451,
    RISCV_PseudoVREMU_VV_M8_E16	= 7452,
    RISCV_PseudoVREMU_VV_M8_E16_MASK	= 7453,
    RISCV_PseudoVREMU_VV_M8_E32	= 7454,
    RISCV_PseudoVREMU_VV_M8_E32_MASK	= 7455,
    RISCV_PseudoVREMU_VV_M8_E64	= 7456,
    RISCV_PseudoVREMU_VV_M8_E64_MASK	= 7457,
    RISCV_PseudoVREMU_VV_M8_E8	= 7458,
    RISCV_PseudoVREMU_VV_M8_E8_MASK	= 7459,
    RISCV_PseudoVREMU_VV_MF2_E16	= 7460,
    RISCV_PseudoVREMU_VV_MF2_E16_MASK	= 7461,
    RISCV_PseudoVREMU_VV_MF2_E32	= 7462,
    RISCV_PseudoVREMU_VV_MF2_E32_MASK	= 7463,
    RISCV_PseudoVREMU_VV_MF2_E8	= 7464,
    RISCV_PseudoVREMU_VV_MF2_E8_MASK	= 7465,
    RISCV_PseudoVREMU_VV_MF4_E16	= 7466,
    RISCV_PseudoVREMU_VV_MF4_E16_MASK	= 7467,
    RISCV_PseudoVREMU_VV_MF4_E8	= 7468,
    RISCV_PseudoVREMU_VV_MF4_E8_MASK	= 7469,
    RISCV_PseudoVREMU_VV_MF8_E8	= 7470,
    RISCV_PseudoVREMU_VV_MF8_E8_MASK	= 7471,
    RISCV_PseudoVREMU_VX_M1_E16	= 7472,
    RISCV_PseudoVREMU_VX_M1_E16_MASK	= 7473,
    RISCV_PseudoVREMU_VX_M1_E32	= 7474,
    RISCV_PseudoVREMU_VX_M1_E32_MASK	= 7475,
    RISCV_PseudoVREMU_VX_M1_E64	= 7476,
    RISCV_PseudoVREMU_VX_M1_E64_MASK	= 7477,
    RISCV_PseudoVREMU_VX_M1_E8	= 7478,
    RISCV_PseudoVREMU_VX_M1_E8_MASK	= 7479,
    RISCV_PseudoVREMU_VX_M2_E16	= 7480,
    RISCV_PseudoVREMU_VX_M2_E16_MASK	= 7481,
    RISCV_PseudoVREMU_VX_M2_E32	= 7482,
    RISCV_PseudoVREMU_VX_M2_E32_MASK	= 7483,
    RISCV_PseudoVREMU_VX_M2_E64	= 7484,
    RISCV_PseudoVREMU_VX_M2_E64_MASK	= 7485,
    RISCV_PseudoVREMU_VX_M2_E8	= 7486,
    RISCV_PseudoVREMU_VX_M2_E8_MASK	= 7487,
    RISCV_PseudoVREMU_VX_M4_E16	= 7488,
    RISCV_PseudoVREMU_VX_M4_E16_MASK	= 7489,
    RISCV_PseudoVREMU_VX_M4_E32	= 7490,
    RISCV_PseudoVREMU_VX_M4_E32_MASK	= 7491,
    RISCV_PseudoVREMU_VX_M4_E64	= 7492,
    RISCV_PseudoVREMU_VX_M4_E64_MASK	= 7493,
    RISCV_PseudoVREMU_VX_M4_E8	= 7494,
    RISCV_PseudoVREMU_VX_M4_E8_MASK	= 7495,
    RISCV_PseudoVREMU_VX_M8_E16	= 7496,
    RISCV_PseudoVREMU_VX_M8_E16_MASK	= 7497,
    RISCV_PseudoVREMU_VX_M8_E32	= 7498,
    RISCV_PseudoVREMU_VX_M8_E32_MASK	= 7499,
    RISCV_PseudoVREMU_VX_M8_E64	= 7500,
    RISCV_PseudoVREMU_VX_M8_E64_MASK	= 7501,
    RISCV_PseudoVREMU_VX_M8_E8	= 7502,
    RISCV_PseudoVREMU_VX_M8_E8_MASK	= 7503,
    RISCV_PseudoVREMU_VX_MF2_E16	= 7504,
    RISCV_PseudoVREMU_VX_MF2_E16_MASK	= 7505,
    RISCV_PseudoVREMU_VX_MF2_E32	= 7506,
    RISCV_PseudoVREMU_VX_MF2_E32_MASK	= 7507,
    RISCV_PseudoVREMU_VX_MF2_E8	= 7508,
    RISCV_PseudoVREMU_VX_MF2_E8_MASK	= 7509,
    RISCV_PseudoVREMU_VX_MF4_E16	= 7510,
    RISCV_PseudoVREMU_VX_MF4_E16_MASK	= 7511,
    RISCV_PseudoVREMU_VX_MF4_E8	= 7512,
    RISCV_PseudoVREMU_VX_MF4_E8_MASK	= 7513,
    RISCV_PseudoVREMU_VX_MF8_E8	= 7514,
    RISCV_PseudoVREMU_VX_MF8_E8_MASK	= 7515,
    RISCV_PseudoVREM_VV_M1_E16	= 7516,
    RISCV_PseudoVREM_VV_M1_E16_MASK	= 7517,
    RISCV_PseudoVREM_VV_M1_E32	= 7518,
    RISCV_PseudoVREM_VV_M1_E32_MASK	= 7519,
    RISCV_PseudoVREM_VV_M1_E64	= 7520,
    RISCV_PseudoVREM_VV_M1_E64_MASK	= 7521,
    RISCV_PseudoVREM_VV_M1_E8	= 7522,
    RISCV_PseudoVREM_VV_M1_E8_MASK	= 7523,
    RISCV_PseudoVREM_VV_M2_E16	= 7524,
    RISCV_PseudoVREM_VV_M2_E16_MASK	= 7525,
    RISCV_PseudoVREM_VV_M2_E32	= 7526,
    RISCV_PseudoVREM_VV_M2_E32_MASK	= 7527,
    RISCV_PseudoVREM_VV_M2_E64	= 7528,
    RISCV_PseudoVREM_VV_M2_E64_MASK	= 7529,
    RISCV_PseudoVREM_VV_M2_E8	= 7530,
    RISCV_PseudoVREM_VV_M2_E8_MASK	= 7531,
    RISCV_PseudoVREM_VV_M4_E16	= 7532,
    RISCV_PseudoVREM_VV_M4_E16_MASK	= 7533,
    RISCV_PseudoVREM_VV_M4_E32	= 7534,
    RISCV_PseudoVREM_VV_M4_E32_MASK	= 7535,
    RISCV_PseudoVREM_VV_M4_E64	= 7536,
    RISCV_PseudoVREM_VV_M4_E64_MASK	= 7537,
    RISCV_PseudoVREM_VV_M4_E8	= 7538,
    RISCV_PseudoVREM_VV_M4_E8_MASK	= 7539,
    RISCV_PseudoVREM_VV_M8_E16	= 7540,
    RISCV_PseudoVREM_VV_M8_E16_MASK	= 7541,
    RISCV_PseudoVREM_VV_M8_E32	= 7542,
    RISCV_PseudoVREM_VV_M8_E32_MASK	= 7543,
    RISCV_PseudoVREM_VV_M8_E64	= 7544,
    RISCV_PseudoVREM_VV_M8_E64_MASK	= 7545,
    RISCV_PseudoVREM_VV_M8_E8	= 7546,
    RISCV_PseudoVREM_VV_M8_E8_MASK	= 7547,
    RISCV_PseudoVREM_VV_MF2_E16	= 7548,
    RISCV_PseudoVREM_VV_MF2_E16_MASK	= 7549,
    RISCV_PseudoVREM_VV_MF2_E32	= 7550,
    RISCV_PseudoVREM_VV_MF2_E32_MASK	= 7551,
    RISCV_PseudoVREM_VV_MF2_E8	= 7552,
    RISCV_PseudoVREM_VV_MF2_E8_MASK	= 7553,
    RISCV_PseudoVREM_VV_MF4_E16	= 7554,
    RISCV_PseudoVREM_VV_MF4_E16_MASK	= 7555,
    RISCV_PseudoVREM_VV_MF4_E8	= 7556,
    RISCV_PseudoVREM_VV_MF4_E8_MASK	= 7557,
    RISCV_PseudoVREM_VV_MF8_E8	= 7558,
    RISCV_PseudoVREM_VV_MF8_E8_MASK	= 7559,
    RISCV_PseudoVREM_VX_M1_E16	= 7560,
    RISCV_PseudoVREM_VX_M1_E16_MASK	= 7561,
    RISCV_PseudoVREM_VX_M1_E32	= 7562,
    RISCV_PseudoVREM_VX_M1_E32_MASK	= 7563,
    RISCV_PseudoVREM_VX_M1_E64	= 7564,
    RISCV_PseudoVREM_VX_M1_E64_MASK	= 7565,
    RISCV_PseudoVREM_VX_M1_E8	= 7566,
    RISCV_PseudoVREM_VX_M1_E8_MASK	= 7567,
    RISCV_PseudoVREM_VX_M2_E16	= 7568,
    RISCV_PseudoVREM_VX_M2_E16_MASK	= 7569,
    RISCV_PseudoVREM_VX_M2_E32	= 7570,
    RISCV_PseudoVREM_VX_M2_E32_MASK	= 7571,
    RISCV_PseudoVREM_VX_M2_E64	= 7572,
    RISCV_PseudoVREM_VX_M2_E64_MASK	= 7573,
    RISCV_PseudoVREM_VX_M2_E8	= 7574,
    RISCV_PseudoVREM_VX_M2_E8_MASK	= 7575,
    RISCV_PseudoVREM_VX_M4_E16	= 7576,
    RISCV_PseudoVREM_VX_M4_E16_MASK	= 7577,
    RISCV_PseudoVREM_VX_M4_E32	= 7578,
    RISCV_PseudoVREM_VX_M4_E32_MASK	= 7579,
    RISCV_PseudoVREM_VX_M4_E64	= 7580,
    RISCV_PseudoVREM_VX_M4_E64_MASK	= 7581,
    RISCV_PseudoVREM_VX_M4_E8	= 7582,
    RISCV_PseudoVREM_VX_M4_E8_MASK	= 7583,
    RISCV_PseudoVREM_VX_M8_E16	= 7584,
    RISCV_PseudoVREM_VX_M8_E16_MASK	= 7585,
    RISCV_PseudoVREM_VX_M8_E32	= 7586,
    RISCV_PseudoVREM_VX_M8_E32_MASK	= 7587,
    RISCV_PseudoVREM_VX_M8_E64	= 7588,
    RISCV_PseudoVREM_VX_M8_E64_MASK	= 7589,
    RISCV_PseudoVREM_VX_M8_E8	= 7590,
    RISCV_PseudoVREM_VX_M8_E8_MASK	= 7591,
    RISCV_PseudoVREM_VX_MF2_E16	= 7592,
    RISCV_PseudoVREM_VX_MF2_E16_MASK	= 7593,
    RISCV_PseudoVREM_VX_MF2_E32	= 7594,
    RISCV_PseudoVREM_VX_MF2_E32_MASK	= 7595,
    RISCV_PseudoVREM_VX_MF2_E8	= 7596,
    RISCV_PseudoVREM_VX_MF2_E8_MASK	= 7597,
    RISCV_PseudoVREM_VX_MF4_E16	= 7598,
    RISCV_PseudoVREM_VX_MF4_E16_MASK	= 7599,
    RISCV_PseudoVREM_VX_MF4_E8	= 7600,
    RISCV_PseudoVREM_VX_MF4_E8_MASK	= 7601,
    RISCV_PseudoVREM_VX_MF8_E8	= 7602,
    RISCV_PseudoVREM_VX_MF8_E8_MASK	= 7603,
    RISCV_PseudoVREV8_V_M1	= 7604,
    RISCV_PseudoVREV8_V_M1_MASK	= 7605,
    RISCV_PseudoVREV8_V_M2	= 7606,
    RISCV_PseudoVREV8_V_M2_MASK	= 7607,
    RISCV_PseudoVREV8_V_M4	= 7608,
    RISCV_PseudoVREV8_V_M4_MASK	= 7609,
    RISCV_PseudoVREV8_V_M8	= 7610,
    RISCV_PseudoVREV8_V_M8_MASK	= 7611,
    RISCV_PseudoVREV8_V_MF2	= 7612,
    RISCV_PseudoVREV8_V_MF2_MASK	= 7613,
    RISCV_PseudoVREV8_V_MF4	= 7614,
    RISCV_PseudoVREV8_V_MF4_MASK	= 7615,
    RISCV_PseudoVREV8_V_MF8	= 7616,
    RISCV_PseudoVREV8_V_MF8_MASK	= 7617,
    RISCV_PseudoVRGATHEREI16_VV_M1_E16_M1	= 7618,
    RISCV_PseudoVRGATHEREI16_VV_M1_E16_M1_MASK	= 7619,
    RISCV_PseudoVRGATHEREI16_VV_M1_E16_M2	= 7620,
    RISCV_PseudoVRGATHEREI16_VV_M1_E16_M2_MASK	= 7621,
    RISCV_PseudoVRGATHEREI16_VV_M1_E16_MF2	= 7622,
    RISCV_PseudoVRGATHEREI16_VV_M1_E16_MF2_MASK	= 7623,
    RISCV_PseudoVRGATHEREI16_VV_M1_E16_MF4	= 7624,
    RISCV_PseudoVRGATHEREI16_VV_M1_E16_MF4_MASK	= 7625,
    RISCV_PseudoVRGATHEREI16_VV_M1_E32_M1	= 7626,
    RISCV_PseudoVRGATHEREI16_VV_M1_E32_M1_MASK	= 7627,
    RISCV_PseudoVRGATHEREI16_VV_M1_E32_M2	= 7628,
    RISCV_PseudoVRGATHEREI16_VV_M1_E32_M2_MASK	= 7629,
    RISCV_PseudoVRGATHEREI16_VV_M1_E32_MF2	= 7630,
    RISCV_PseudoVRGATHEREI16_VV_M1_E32_MF2_MASK	= 7631,
    RISCV_PseudoVRGATHEREI16_VV_M1_E32_MF4	= 7632,
    RISCV_PseudoVRGATHEREI16_VV_M1_E32_MF4_MASK	= 7633,
    RISCV_PseudoVRGATHEREI16_VV_M1_E64_M1	= 7634,
    RISCV_PseudoVRGATHEREI16_VV_M1_E64_M1_MASK	= 7635,
    RISCV_PseudoVRGATHEREI16_VV_M1_E64_M2	= 7636,
    RISCV_PseudoVRGATHEREI16_VV_M1_E64_M2_MASK	= 7637,
    RISCV_PseudoVRGATHEREI16_VV_M1_E64_MF2	= 7638,
    RISCV_PseudoVRGATHEREI16_VV_M1_E64_MF2_MASK	= 7639,
    RISCV_PseudoVRGATHEREI16_VV_M1_E64_MF4	= 7640,
    RISCV_PseudoVRGATHEREI16_VV_M1_E64_MF4_MASK	= 7641,
    RISCV_PseudoVRGATHEREI16_VV_M1_E8_M1	= 7642,
    RISCV_PseudoVRGATHEREI16_VV_M1_E8_M1_MASK	= 7643,
    RISCV_PseudoVRGATHEREI16_VV_M1_E8_M2	= 7644,
    RISCV_PseudoVRGATHEREI16_VV_M1_E8_M2_MASK	= 7645,
    RISCV_PseudoVRGATHEREI16_VV_M1_E8_MF2	= 7646,
    RISCV_PseudoVRGATHEREI16_VV_M1_E8_MF2_MASK	= 7647,
    RISCV_PseudoVRGATHEREI16_VV_M1_E8_MF4	= 7648,
    RISCV_PseudoVRGATHEREI16_VV_M1_E8_MF4_MASK	= 7649,
    RISCV_PseudoVRGATHEREI16_VV_M2_E16_M1	= 7650,
    RISCV_PseudoVRGATHEREI16_VV_M2_E16_M1_MASK	= 7651,
    RISCV_PseudoVRGATHEREI16_VV_M2_E16_M2	= 7652,
    RISCV_PseudoVRGATHEREI16_VV_M2_E16_M2_MASK	= 7653,
    RISCV_PseudoVRGATHEREI16_VV_M2_E16_M4	= 7654,
    RISCV_PseudoVRGATHEREI16_VV_M2_E16_M4_MASK	= 7655,
    RISCV_PseudoVRGATHEREI16_VV_M2_E16_MF2	= 7656,
    RISCV_PseudoVRGATHEREI16_VV_M2_E16_MF2_MASK	= 7657,
    RISCV_PseudoVRGATHEREI16_VV_M2_E32_M1	= 7658,
    RISCV_PseudoVRGATHEREI16_VV_M2_E32_M1_MASK	= 7659,
    RISCV_PseudoVRGATHEREI16_VV_M2_E32_M2	= 7660,
    RISCV_PseudoVRGATHEREI16_VV_M2_E32_M2_MASK	= 7661,
    RISCV_PseudoVRGATHEREI16_VV_M2_E32_M4	= 7662,
    RISCV_PseudoVRGATHEREI16_VV_M2_E32_M4_MASK	= 7663,
    RISCV_PseudoVRGATHEREI16_VV_M2_E32_MF2	= 7664,
    RISCV_PseudoVRGATHEREI16_VV_M2_E32_MF2_MASK	= 7665,
    RISCV_PseudoVRGATHEREI16_VV_M2_E64_M1	= 7666,
    RISCV_PseudoVRGATHEREI16_VV_M2_E64_M1_MASK	= 7667,
    RISCV_PseudoVRGATHEREI16_VV_M2_E64_M2	= 7668,
    RISCV_PseudoVRGATHEREI16_VV_M2_E64_M2_MASK	= 7669,
    RISCV_PseudoVRGATHEREI16_VV_M2_E64_M4	= 7670,
    RISCV_PseudoVRGATHEREI16_VV_M2_E64_M4_MASK	= 7671,
    RISCV_PseudoVRGATHEREI16_VV_M2_E64_MF2	= 7672,
    RISCV_PseudoVRGATHEREI16_VV_M2_E64_MF2_MASK	= 7673,
    RISCV_PseudoVRGATHEREI16_VV_M2_E8_M1	= 7674,
    RISCV_PseudoVRGATHEREI16_VV_M2_E8_M1_MASK	= 7675,
    RISCV_PseudoVRGATHEREI16_VV_M2_E8_M2	= 7676,
    RISCV_PseudoVRGATHEREI16_VV_M2_E8_M2_MASK	= 7677,
    RISCV_PseudoVRGATHEREI16_VV_M2_E8_M4	= 7678,
    RISCV_PseudoVRGATHEREI16_VV_M2_E8_M4_MASK	= 7679,
    RISCV_PseudoVRGATHEREI16_VV_M2_E8_MF2	= 7680,
    RISCV_PseudoVRGATHEREI16_VV_M2_E8_MF2_MASK	= 7681,
    RISCV_PseudoVRGATHEREI16_VV_M4_E16_M1	= 7682,
    RISCV_PseudoVRGATHEREI16_VV_M4_E16_M1_MASK	= 7683,
    RISCV_PseudoVRGATHEREI16_VV_M4_E16_M2	= 7684,
    RISCV_PseudoVRGATHEREI16_VV_M4_E16_M2_MASK	= 7685,
    RISCV_PseudoVRGATHEREI16_VV_M4_E16_M4	= 7686,
    RISCV_PseudoVRGATHEREI16_VV_M4_E16_M4_MASK	= 7687,
    RISCV_PseudoVRGATHEREI16_VV_M4_E16_M8	= 7688,
    RISCV_PseudoVRGATHEREI16_VV_M4_E16_M8_MASK	= 7689,
    RISCV_PseudoVRGATHEREI16_VV_M4_E32_M1	= 7690,
    RISCV_PseudoVRGATHEREI16_VV_M4_E32_M1_MASK	= 7691,
    RISCV_PseudoVRGATHEREI16_VV_M4_E32_M2	= 7692,
    RISCV_PseudoVRGATHEREI16_VV_M4_E32_M2_MASK	= 7693,
    RISCV_PseudoVRGATHEREI16_VV_M4_E32_M4	= 7694,
    RISCV_PseudoVRGATHEREI16_VV_M4_E32_M4_MASK	= 7695,
    RISCV_PseudoVRGATHEREI16_VV_M4_E32_M8	= 7696,
    RISCV_PseudoVRGATHEREI16_VV_M4_E32_M8_MASK	= 7697,
    RISCV_PseudoVRGATHEREI16_VV_M4_E64_M1	= 7698,
    RISCV_PseudoVRGATHEREI16_VV_M4_E64_M1_MASK	= 7699,
    RISCV_PseudoVRGATHEREI16_VV_M4_E64_M2	= 7700,
    RISCV_PseudoVRGATHEREI16_VV_M4_E64_M2_MASK	= 7701,
    RISCV_PseudoVRGATHEREI16_VV_M4_E64_M4	= 7702,
    RISCV_PseudoVRGATHEREI16_VV_M4_E64_M4_MASK	= 7703,
    RISCV_PseudoVRGATHEREI16_VV_M4_E64_M8	= 7704,
    RISCV_PseudoVRGATHEREI16_VV_M4_E64_M8_MASK	= 7705,
    RISCV_PseudoVRGATHEREI16_VV_M4_E8_M1	= 7706,
    RISCV_PseudoVRGATHEREI16_VV_M4_E8_M1_MASK	= 7707,
    RISCV_PseudoVRGATHEREI16_VV_M4_E8_M2	= 7708,
    RISCV_PseudoVRGATHEREI16_VV_M4_E8_M2_MASK	= 7709,
    RISCV_PseudoVRGATHEREI16_VV_M4_E8_M4	= 7710,
    RISCV_PseudoVRGATHEREI16_VV_M4_E8_M4_MASK	= 7711,
    RISCV_PseudoVRGATHEREI16_VV_M4_E8_M8	= 7712,
    RISCV_PseudoVRGATHEREI16_VV_M4_E8_M8_MASK	= 7713,
    RISCV_PseudoVRGATHEREI16_VV_M8_E16_M2	= 7714,
    RISCV_PseudoVRGATHEREI16_VV_M8_E16_M2_MASK	= 7715,
    RISCV_PseudoVRGATHEREI16_VV_M8_E16_M4	= 7716,
    RISCV_PseudoVRGATHEREI16_VV_M8_E16_M4_MASK	= 7717,
    RISCV_PseudoVRGATHEREI16_VV_M8_E16_M8	= 7718,
    RISCV_PseudoVRGATHEREI16_VV_M8_E16_M8_MASK	= 7719,
    RISCV_PseudoVRGATHEREI16_VV_M8_E32_M2	= 7720,
    RISCV_PseudoVRGATHEREI16_VV_M8_E32_M2_MASK	= 7721,
    RISCV_PseudoVRGATHEREI16_VV_M8_E32_M4	= 7722,
    RISCV_PseudoVRGATHEREI16_VV_M8_E32_M4_MASK	= 7723,
    RISCV_PseudoVRGATHEREI16_VV_M8_E32_M8	= 7724,
    RISCV_PseudoVRGATHEREI16_VV_M8_E32_M8_MASK	= 7725,
    RISCV_PseudoVRGATHEREI16_VV_M8_E64_M2	= 7726,
    RISCV_PseudoVRGATHEREI16_VV_M8_E64_M2_MASK	= 7727,
    RISCV_PseudoVRGATHEREI16_VV_M8_E64_M4	= 7728,
    RISCV_PseudoVRGATHEREI16_VV_M8_E64_M4_MASK	= 7729,
    RISCV_PseudoVRGATHEREI16_VV_M8_E64_M8	= 7730,
    RISCV_PseudoVRGATHEREI16_VV_M8_E64_M8_MASK	= 7731,
    RISCV_PseudoVRGATHEREI16_VV_M8_E8_M2	= 7732,
    RISCV_PseudoVRGATHEREI16_VV_M8_E8_M2_MASK	= 7733,
    RISCV_PseudoVRGATHEREI16_VV_M8_E8_M4	= 7734,
    RISCV_PseudoVRGATHEREI16_VV_M8_E8_M4_MASK	= 7735,
    RISCV_PseudoVRGATHEREI16_VV_M8_E8_M8	= 7736,
    RISCV_PseudoVRGATHEREI16_VV_M8_E8_M8_MASK	= 7737,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E16_M1	= 7738,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E16_M1_MASK	= 7739,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E16_MF2	= 7740,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E16_MF2_MASK	= 7741,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E16_MF4	= 7742,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E16_MF4_MASK	= 7743,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E16_MF8	= 7744,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E16_MF8_MASK	= 7745,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E32_M1	= 7746,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E32_M1_MASK	= 7747,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E32_MF2	= 7748,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E32_MF2_MASK	= 7749,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E32_MF4	= 7750,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E32_MF4_MASK	= 7751,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E32_MF8	= 7752,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E32_MF8_MASK	= 7753,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E8_M1	= 7754,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E8_M1_MASK	= 7755,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E8_MF2	= 7756,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E8_MF2_MASK	= 7757,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E8_MF4	= 7758,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E8_MF4_MASK	= 7759,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E8_MF8	= 7760,
    RISCV_PseudoVRGATHEREI16_VV_MF2_E8_MF8_MASK	= 7761,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E16_MF2	= 7762,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E16_MF2_MASK	= 7763,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E16_MF4	= 7764,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E16_MF4_MASK	= 7765,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E16_MF8	= 7766,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E16_MF8_MASK	= 7767,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E8_MF2	= 7768,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E8_MF2_MASK	= 7769,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E8_MF4	= 7770,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E8_MF4_MASK	= 7771,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E8_MF8	= 7772,
    RISCV_PseudoVRGATHEREI16_VV_MF4_E8_MF8_MASK	= 7773,
    RISCV_PseudoVRGATHEREI16_VV_MF8_E8_MF4	= 7774,
    RISCV_PseudoVRGATHEREI16_VV_MF8_E8_MF4_MASK	= 7775,
    RISCV_PseudoVRGATHEREI16_VV_MF8_E8_MF8	= 7776,
    RISCV_PseudoVRGATHEREI16_VV_MF8_E8_MF8_MASK	= 7777,
    RISCV_PseudoVRGATHER_VI_M1	= 7778,
    RISCV_PseudoVRGATHER_VI_M1_MASK	= 7779,
    RISCV_PseudoVRGATHER_VI_M2	= 7780,
    RISCV_PseudoVRGATHER_VI_M2_MASK	= 7781,
    RISCV_PseudoVRGATHER_VI_M4	= 7782,
    RISCV_PseudoVRGATHER_VI_M4_MASK	= 7783,
    RISCV_PseudoVRGATHER_VI_M8	= 7784,
    RISCV_PseudoVRGATHER_VI_M8_MASK	= 7785,
    RISCV_PseudoVRGATHER_VI_MF2	= 7786,
    RISCV_PseudoVRGATHER_VI_MF2_MASK	= 7787,
    RISCV_PseudoVRGATHER_VI_MF4	= 7788,
    RISCV_PseudoVRGATHER_VI_MF4_MASK	= 7789,
    RISCV_PseudoVRGATHER_VI_MF8	= 7790,
    RISCV_PseudoVRGATHER_VI_MF8_MASK	= 7791,
    RISCV_PseudoVRGATHER_VV_M1_E16	= 7792,
    RISCV_PseudoVRGATHER_VV_M1_E16_MASK	= 7793,
    RISCV_PseudoVRGATHER_VV_M1_E32	= 7794,
    RISCV_PseudoVRGATHER_VV_M1_E32_MASK	= 7795,
    RISCV_PseudoVRGATHER_VV_M1_E64	= 7796,
    RISCV_PseudoVRGATHER_VV_M1_E64_MASK	= 7797,
    RISCV_PseudoVRGATHER_VV_M1_E8	= 7798,
    RISCV_PseudoVRGATHER_VV_M1_E8_MASK	= 7799,
    RISCV_PseudoVRGATHER_VV_M2_E16	= 7800,
    RISCV_PseudoVRGATHER_VV_M2_E16_MASK	= 7801,
    RISCV_PseudoVRGATHER_VV_M2_E32	= 7802,
    RISCV_PseudoVRGATHER_VV_M2_E32_MASK	= 7803,
    RISCV_PseudoVRGATHER_VV_M2_E64	= 7804,
    RISCV_PseudoVRGATHER_VV_M2_E64_MASK	= 7805,
    RISCV_PseudoVRGATHER_VV_M2_E8	= 7806,
    RISCV_PseudoVRGATHER_VV_M2_E8_MASK	= 7807,
    RISCV_PseudoVRGATHER_VV_M4_E16	= 7808,
    RISCV_PseudoVRGATHER_VV_M4_E16_MASK	= 7809,
    RISCV_PseudoVRGATHER_VV_M4_E32	= 7810,
    RISCV_PseudoVRGATHER_VV_M4_E32_MASK	= 7811,
    RISCV_PseudoVRGATHER_VV_M4_E64	= 7812,
    RISCV_PseudoVRGATHER_VV_M4_E64_MASK	= 7813,
    RISCV_PseudoVRGATHER_VV_M4_E8	= 7814,
    RISCV_PseudoVRGATHER_VV_M4_E8_MASK	= 7815,
    RISCV_PseudoVRGATHER_VV_M8_E16	= 7816,
    RISCV_PseudoVRGATHER_VV_M8_E16_MASK	= 7817,
    RISCV_PseudoVRGATHER_VV_M8_E32	= 7818,
    RISCV_PseudoVRGATHER_VV_M8_E32_MASK	= 7819,
    RISCV_PseudoVRGATHER_VV_M8_E64	= 7820,
    RISCV_PseudoVRGATHER_VV_M8_E64_MASK	= 7821,
    RISCV_PseudoVRGATHER_VV_M8_E8	= 7822,
    RISCV_PseudoVRGATHER_VV_M8_E8_MASK	= 7823,
    RISCV_PseudoVRGATHER_VV_MF2_E16	= 7824,
    RISCV_PseudoVRGATHER_VV_MF2_E16_MASK	= 7825,
    RISCV_PseudoVRGATHER_VV_MF2_E32	= 7826,
    RISCV_PseudoVRGATHER_VV_MF2_E32_MASK	= 7827,
    RISCV_PseudoVRGATHER_VV_MF2_E8	= 7828,
    RISCV_PseudoVRGATHER_VV_MF2_E8_MASK	= 7829,
    RISCV_PseudoVRGATHER_VV_MF4_E16	= 7830,
    RISCV_PseudoVRGATHER_VV_MF4_E16_MASK	= 7831,
    RISCV_PseudoVRGATHER_VV_MF4_E8	= 7832,
    RISCV_PseudoVRGATHER_VV_MF4_E8_MASK	= 7833,
    RISCV_PseudoVRGATHER_VV_MF8_E8	= 7834,
    RISCV_PseudoVRGATHER_VV_MF8_E8_MASK	= 7835,
    RISCV_PseudoVRGATHER_VX_M1	= 7836,
    RISCV_PseudoVRGATHER_VX_M1_MASK	= 7837,
    RISCV_PseudoVRGATHER_VX_M2	= 7838,
    RISCV_PseudoVRGATHER_VX_M2_MASK	= 7839,
    RISCV_PseudoVRGATHER_VX_M4	= 7840,
    RISCV_PseudoVRGATHER_VX_M4_MASK	= 7841,
    RISCV_PseudoVRGATHER_VX_M8	= 7842,
    RISCV_PseudoVRGATHER_VX_M8_MASK	= 7843,
    RISCV_PseudoVRGATHER_VX_MF2	= 7844,
    RISCV_PseudoVRGATHER_VX_MF2_MASK	= 7845,
    RISCV_PseudoVRGATHER_VX_MF4	= 7846,
    RISCV_PseudoVRGATHER_VX_MF4_MASK	= 7847,
    RISCV_PseudoVRGATHER_VX_MF8	= 7848,
    RISCV_PseudoVRGATHER_VX_MF8_MASK	= 7849,
    RISCV_PseudoVROL_VV_M1	= 7850,
    RISCV_PseudoVROL_VV_M1_MASK	= 7851,
    RISCV_PseudoVROL_VV_M2	= 7852,
    RISCV_PseudoVROL_VV_M2_MASK	= 7853,
    RISCV_PseudoVROL_VV_M4	= 7854,
    RISCV_PseudoVROL_VV_M4_MASK	= 7855,
    RISCV_PseudoVROL_VV_M8	= 7856,
    RISCV_PseudoVROL_VV_M8_MASK	= 7857,
    RISCV_PseudoVROL_VV_MF2	= 7858,
    RISCV_PseudoVROL_VV_MF2_MASK	= 7859,
    RISCV_PseudoVROL_VV_MF4	= 7860,
    RISCV_PseudoVROL_VV_MF4_MASK	= 7861,
    RISCV_PseudoVROL_VV_MF8	= 7862,
    RISCV_PseudoVROL_VV_MF8_MASK	= 7863,
    RISCV_PseudoVROL_VX_M1	= 7864,
    RISCV_PseudoVROL_VX_M1_MASK	= 7865,
    RISCV_PseudoVROL_VX_M2	= 7866,
    RISCV_PseudoVROL_VX_M2_MASK	= 7867,
    RISCV_PseudoVROL_VX_M4	= 7868,
    RISCV_PseudoVROL_VX_M4_MASK	= 7869,
    RISCV_PseudoVROL_VX_M8	= 7870,
    RISCV_PseudoVROL_VX_M8_MASK	= 7871,
    RISCV_PseudoVROL_VX_MF2	= 7872,
    RISCV_PseudoVROL_VX_MF2_MASK	= 7873,
    RISCV_PseudoVROL_VX_MF4	= 7874,
    RISCV_PseudoVROL_VX_MF4_MASK	= 7875,
    RISCV_PseudoVROL_VX_MF8	= 7876,
    RISCV_PseudoVROL_VX_MF8_MASK	= 7877,
    RISCV_PseudoVROR_VI_M1	= 7878,
    RISCV_PseudoVROR_VI_M1_MASK	= 7879,
    RISCV_PseudoVROR_VI_M2	= 7880,
    RISCV_PseudoVROR_VI_M2_MASK	= 7881,
    RISCV_PseudoVROR_VI_M4	= 7882,
    RISCV_PseudoVROR_VI_M4_MASK	= 7883,
    RISCV_PseudoVROR_VI_M8	= 7884,
    RISCV_PseudoVROR_VI_M8_MASK	= 7885,
    RISCV_PseudoVROR_VI_MF2	= 7886,
    RISCV_PseudoVROR_VI_MF2_MASK	= 7887,
    RISCV_PseudoVROR_VI_MF4	= 7888,
    RISCV_PseudoVROR_VI_MF4_MASK	= 7889,
    RISCV_PseudoVROR_VI_MF8	= 7890,
    RISCV_PseudoVROR_VI_MF8_MASK	= 7891,
    RISCV_PseudoVROR_VV_M1	= 7892,
    RISCV_PseudoVROR_VV_M1_MASK	= 7893,
    RISCV_PseudoVROR_VV_M2	= 7894,
    RISCV_PseudoVROR_VV_M2_MASK	= 7895,
    RISCV_PseudoVROR_VV_M4	= 7896,
    RISCV_PseudoVROR_VV_M4_MASK	= 7897,
    RISCV_PseudoVROR_VV_M8	= 7898,
    RISCV_PseudoVROR_VV_M8_MASK	= 7899,
    RISCV_PseudoVROR_VV_MF2	= 7900,
    RISCV_PseudoVROR_VV_MF2_MASK	= 7901,
    RISCV_PseudoVROR_VV_MF4	= 7902,
    RISCV_PseudoVROR_VV_MF4_MASK	= 7903,
    RISCV_PseudoVROR_VV_MF8	= 7904,
    RISCV_PseudoVROR_VV_MF8_MASK	= 7905,
    RISCV_PseudoVROR_VX_M1	= 7906,
    RISCV_PseudoVROR_VX_M1_MASK	= 7907,
    RISCV_PseudoVROR_VX_M2	= 7908,
    RISCV_PseudoVROR_VX_M2_MASK	= 7909,
    RISCV_PseudoVROR_VX_M4	= 7910,
    RISCV_PseudoVROR_VX_M4_MASK	= 7911,
    RISCV_PseudoVROR_VX_M8	= 7912,
    RISCV_PseudoVROR_VX_M8_MASK	= 7913,
    RISCV_PseudoVROR_VX_MF2	= 7914,
    RISCV_PseudoVROR_VX_MF2_MASK	= 7915,
    RISCV_PseudoVROR_VX_MF4	= 7916,
    RISCV_PseudoVROR_VX_MF4_MASK	= 7917,
    RISCV_PseudoVROR_VX_MF8	= 7918,
    RISCV_PseudoVROR_VX_MF8_MASK	= 7919,
    RISCV_PseudoVRSUB_VI_M1	= 7920,
    RISCV_PseudoVRSUB_VI_M1_MASK	= 7921,
    RISCV_PseudoVRSUB_VI_M2	= 7922,
    RISCV_PseudoVRSUB_VI_M2_MASK	= 7923,
    RISCV_PseudoVRSUB_VI_M4	= 7924,
    RISCV_PseudoVRSUB_VI_M4_MASK	= 7925,
    RISCV_PseudoVRSUB_VI_M8	= 7926,
    RISCV_PseudoVRSUB_VI_M8_MASK	= 7927,
    RISCV_PseudoVRSUB_VI_MF2	= 7928,
    RISCV_PseudoVRSUB_VI_MF2_MASK	= 7929,
    RISCV_PseudoVRSUB_VI_MF4	= 7930,
    RISCV_PseudoVRSUB_VI_MF4_MASK	= 7931,
    RISCV_PseudoVRSUB_VI_MF8	= 7932,
    RISCV_PseudoVRSUB_VI_MF8_MASK	= 7933,
    RISCV_PseudoVRSUB_VX_M1	= 7934,
    RISCV_PseudoVRSUB_VX_M1_MASK	= 7935,
    RISCV_PseudoVRSUB_VX_M2	= 7936,
    RISCV_PseudoVRSUB_VX_M2_MASK	= 7937,
    RISCV_PseudoVRSUB_VX_M4	= 7938,
    RISCV_PseudoVRSUB_VX_M4_MASK	= 7939,
    RISCV_PseudoVRSUB_VX_M8	= 7940,
    RISCV_PseudoVRSUB_VX_M8_MASK	= 7941,
    RISCV_PseudoVRSUB_VX_MF2	= 7942,
    RISCV_PseudoVRSUB_VX_MF2_MASK	= 7943,
    RISCV_PseudoVRSUB_VX_MF4	= 7944,
    RISCV_PseudoVRSUB_VX_MF4_MASK	= 7945,
    RISCV_PseudoVRSUB_VX_MF8	= 7946,
    RISCV_PseudoVRSUB_VX_MF8_MASK	= 7947,
    RISCV_PseudoVSADDU_VI_M1	= 7948,
    RISCV_PseudoVSADDU_VI_M1_MASK	= 7949,
    RISCV_PseudoVSADDU_VI_M2	= 7950,
    RISCV_PseudoVSADDU_VI_M2_MASK	= 7951,
    RISCV_PseudoVSADDU_VI_M4	= 7952,
    RISCV_PseudoVSADDU_VI_M4_MASK	= 7953,
    RISCV_PseudoVSADDU_VI_M8	= 7954,
    RISCV_PseudoVSADDU_VI_M8_MASK	= 7955,
    RISCV_PseudoVSADDU_VI_MF2	= 7956,
    RISCV_PseudoVSADDU_VI_MF2_MASK	= 7957,
    RISCV_PseudoVSADDU_VI_MF4	= 7958,
    RISCV_PseudoVSADDU_VI_MF4_MASK	= 7959,
    RISCV_PseudoVSADDU_VI_MF8	= 7960,
    RISCV_PseudoVSADDU_VI_MF8_MASK	= 7961,
    RISCV_PseudoVSADDU_VV_M1	= 7962,
    RISCV_PseudoVSADDU_VV_M1_MASK	= 7963,
    RISCV_PseudoVSADDU_VV_M2	= 7964,
    RISCV_PseudoVSADDU_VV_M2_MASK	= 7965,
    RISCV_PseudoVSADDU_VV_M4	= 7966,
    RISCV_PseudoVSADDU_VV_M4_MASK	= 7967,
    RISCV_PseudoVSADDU_VV_M8	= 7968,
    RISCV_PseudoVSADDU_VV_M8_MASK	= 7969,
    RISCV_PseudoVSADDU_VV_MF2	= 7970,
    RISCV_PseudoVSADDU_VV_MF2_MASK	= 7971,
    RISCV_PseudoVSADDU_VV_MF4	= 7972,
    RISCV_PseudoVSADDU_VV_MF4_MASK	= 7973,
    RISCV_PseudoVSADDU_VV_MF8	= 7974,
    RISCV_PseudoVSADDU_VV_MF8_MASK	= 7975,
    RISCV_PseudoVSADDU_VX_M1	= 7976,
    RISCV_PseudoVSADDU_VX_M1_MASK	= 7977,
    RISCV_PseudoVSADDU_VX_M2	= 7978,
    RISCV_PseudoVSADDU_VX_M2_MASK	= 7979,
    RISCV_PseudoVSADDU_VX_M4	= 7980,
    RISCV_PseudoVSADDU_VX_M4_MASK	= 7981,
    RISCV_PseudoVSADDU_VX_M8	= 7982,
    RISCV_PseudoVSADDU_VX_M8_MASK	= 7983,
    RISCV_PseudoVSADDU_VX_MF2	= 7984,
    RISCV_PseudoVSADDU_VX_MF2_MASK	= 7985,
    RISCV_PseudoVSADDU_VX_MF4	= 7986,
    RISCV_PseudoVSADDU_VX_MF4_MASK	= 7987,
    RISCV_PseudoVSADDU_VX_MF8	= 7988,
    RISCV_PseudoVSADDU_VX_MF8_MASK	= 7989,
    RISCV_PseudoVSADD_VI_M1	= 7990,
    RISCV_PseudoVSADD_VI_M1_MASK	= 7991,
    RISCV_PseudoVSADD_VI_M2	= 7992,
    RISCV_PseudoVSADD_VI_M2_MASK	= 7993,
    RISCV_PseudoVSADD_VI_M4	= 7994,
    RISCV_PseudoVSADD_VI_M4_MASK	= 7995,
    RISCV_PseudoVSADD_VI_M8	= 7996,
    RISCV_PseudoVSADD_VI_M8_MASK	= 7997,
    RISCV_PseudoVSADD_VI_MF2	= 7998,
    RISCV_PseudoVSADD_VI_MF2_MASK	= 7999,
    RISCV_PseudoVSADD_VI_MF4	= 8000,
    RISCV_PseudoVSADD_VI_MF4_MASK	= 8001,
    RISCV_PseudoVSADD_VI_MF8	= 8002,
    RISCV_PseudoVSADD_VI_MF8_MASK	= 8003,
    RISCV_PseudoVSADD_VV_M1	= 8004,
    RISCV_PseudoVSADD_VV_M1_MASK	= 8005,
    RISCV_PseudoVSADD_VV_M2	= 8006,
    RISCV_PseudoVSADD_VV_M2_MASK	= 8007,
    RISCV_PseudoVSADD_VV_M4	= 8008,
    RISCV_PseudoVSADD_VV_M4_MASK	= 8009,
    RISCV_PseudoVSADD_VV_M8	= 8010,
    RISCV_PseudoVSADD_VV_M8_MASK	= 8011,
    RISCV_PseudoVSADD_VV_MF2	= 8012,
    RISCV_PseudoVSADD_VV_MF2_MASK	= 8013,
    RISCV_PseudoVSADD_VV_MF4	= 8014,
    RISCV_PseudoVSADD_VV_MF4_MASK	= 8015,
    RISCV_PseudoVSADD_VV_MF8	= 8016,
    RISCV_PseudoVSADD_VV_MF8_MASK	= 8017,
    RISCV_PseudoVSADD_VX_M1	= 8018,
    RISCV_PseudoVSADD_VX_M1_MASK	= 8019,
    RISCV_PseudoVSADD_VX_M2	= 8020,
    RISCV_PseudoVSADD_VX_M2_MASK	= 8021,
    RISCV_PseudoVSADD_VX_M4	= 8022,
    RISCV_PseudoVSADD_VX_M4_MASK	= 8023,
    RISCV_PseudoVSADD_VX_M8	= 8024,
    RISCV_PseudoVSADD_VX_M8_MASK	= 8025,
    RISCV_PseudoVSADD_VX_MF2	= 8026,
    RISCV_PseudoVSADD_VX_MF2_MASK	= 8027,
    RISCV_PseudoVSADD_VX_MF4	= 8028,
    RISCV_PseudoVSADD_VX_MF4_MASK	= 8029,
    RISCV_PseudoVSADD_VX_MF8	= 8030,
    RISCV_PseudoVSADD_VX_MF8_MASK	= 8031,
    RISCV_PseudoVSBC_VVM_M1	= 8032,
    RISCV_PseudoVSBC_VVM_M2	= 8033,
    RISCV_PseudoVSBC_VVM_M4	= 8034,
    RISCV_PseudoVSBC_VVM_M8	= 8035,
    RISCV_PseudoVSBC_VVM_MF2	= 8036,
    RISCV_PseudoVSBC_VVM_MF4	= 8037,
    RISCV_PseudoVSBC_VVM_MF8	= 8038,
    RISCV_PseudoVSBC_VXM_M1	= 8039,
    RISCV_PseudoVSBC_VXM_M2	= 8040,
    RISCV_PseudoVSBC_VXM_M4	= 8041,
    RISCV_PseudoVSBC_VXM_M8	= 8042,
    RISCV_PseudoVSBC_VXM_MF2	= 8043,
    RISCV_PseudoVSBC_VXM_MF4	= 8044,
    RISCV_PseudoVSBC_VXM_MF8	= 8045,
    RISCV_PseudoVSE16_V_M1	= 8046,
    RISCV_PseudoVSE16_V_M1_MASK	= 8047,
    RISCV_PseudoVSE16_V_M2	= 8048,
    RISCV_PseudoVSE16_V_M2_MASK	= 8049,
    RISCV_PseudoVSE16_V_M4	= 8050,
    RISCV_PseudoVSE16_V_M4_MASK	= 8051,
    RISCV_PseudoVSE16_V_M8	= 8052,
    RISCV_PseudoVSE16_V_M8_MASK	= 8053,
    RISCV_PseudoVSE16_V_MF2	= 8054,
    RISCV_PseudoVSE16_V_MF2_MASK	= 8055,
    RISCV_PseudoVSE16_V_MF4	= 8056,
    RISCV_PseudoVSE16_V_MF4_MASK	= 8057,
    RISCV_PseudoVSE32_V_M1	= 8058,
    RISCV_PseudoVSE32_V_M1_MASK	= 8059,
    RISCV_PseudoVSE32_V_M2	= 8060,
    RISCV_PseudoVSE32_V_M2_MASK	= 8061,
    RISCV_PseudoVSE32_V_M4	= 8062,
    RISCV_PseudoVSE32_V_M4_MASK	= 8063,
    RISCV_PseudoVSE32_V_M8	= 8064,
    RISCV_PseudoVSE32_V_M8_MASK	= 8065,
    RISCV_PseudoVSE32_V_MF2	= 8066,
    RISCV_PseudoVSE32_V_MF2_MASK	= 8067,
    RISCV_PseudoVSE64_V_M1	= 8068,
    RISCV_PseudoVSE64_V_M1_MASK	= 8069,
    RISCV_PseudoVSE64_V_M2	= 8070,
    RISCV_PseudoVSE64_V_M2_MASK	= 8071,
    RISCV_PseudoVSE64_V_M4	= 8072,
    RISCV_PseudoVSE64_V_M4_MASK	= 8073,
    RISCV_PseudoVSE64_V_M8	= 8074,
    RISCV_PseudoVSE64_V_M8_MASK	= 8075,
    RISCV_PseudoVSE8_V_M1	= 8076,
    RISCV_PseudoVSE8_V_M1_MASK	= 8077,
    RISCV_PseudoVSE8_V_M2	= 8078,
    RISCV_PseudoVSE8_V_M2_MASK	= 8079,
    RISCV_PseudoVSE8_V_M4	= 8080,
    RISCV_PseudoVSE8_V_M4_MASK	= 8081,
    RISCV_PseudoVSE8_V_M8	= 8082,
    RISCV_PseudoVSE8_V_M8_MASK	= 8083,
    RISCV_PseudoVSE8_V_MF2	= 8084,
    RISCV_PseudoVSE8_V_MF2_MASK	= 8085,
    RISCV_PseudoVSE8_V_MF4	= 8086,
    RISCV_PseudoVSE8_V_MF4_MASK	= 8087,
    RISCV_PseudoVSE8_V_MF8	= 8088,
    RISCV_PseudoVSE8_V_MF8_MASK	= 8089,
    RISCV_PseudoVSETIVLI	= 8090,
    RISCV_PseudoVSETVLI	= 8091,
    RISCV_PseudoVSETVLIX0	= 8092,
    RISCV_PseudoVSEXT_VF2_M1	= 8093,
    RISCV_PseudoVSEXT_VF2_M1_MASK	= 8094,
    RISCV_PseudoVSEXT_VF2_M2	= 8095,
    RISCV_PseudoVSEXT_VF2_M2_MASK	= 8096,
    RISCV_PseudoVSEXT_VF2_M4	= 8097,
    RISCV_PseudoVSEXT_VF2_M4_MASK	= 8098,
    RISCV_PseudoVSEXT_VF2_M8	= 8099,
    RISCV_PseudoVSEXT_VF2_M8_MASK	= 8100,
    RISCV_PseudoVSEXT_VF2_MF2	= 8101,
    RISCV_PseudoVSEXT_VF2_MF2_MASK	= 8102,
    RISCV_PseudoVSEXT_VF2_MF4	= 8103,
    RISCV_PseudoVSEXT_VF2_MF4_MASK	= 8104,
    RISCV_PseudoVSEXT_VF4_M1	= 8105,
    RISCV_PseudoVSEXT_VF4_M1_MASK	= 8106,
    RISCV_PseudoVSEXT_VF4_M2	= 8107,
    RISCV_PseudoVSEXT_VF4_M2_MASK	= 8108,
    RISCV_PseudoVSEXT_VF4_M4	= 8109,
    RISCV_PseudoVSEXT_VF4_M4_MASK	= 8110,
    RISCV_PseudoVSEXT_VF4_M8	= 8111,
    RISCV_PseudoVSEXT_VF4_M8_MASK	= 8112,
    RISCV_PseudoVSEXT_VF4_MF2	= 8113,
    RISCV_PseudoVSEXT_VF4_MF2_MASK	= 8114,
    RISCV_PseudoVSEXT_VF8_M1	= 8115,
    RISCV_PseudoVSEXT_VF8_M1_MASK	= 8116,
    RISCV_PseudoVSEXT_VF8_M2	= 8117,
    RISCV_PseudoVSEXT_VF8_M2_MASK	= 8118,
    RISCV_PseudoVSEXT_VF8_M4	= 8119,
    RISCV_PseudoVSEXT_VF8_M4_MASK	= 8120,
    RISCV_PseudoVSEXT_VF8_M8	= 8121,
    RISCV_PseudoVSEXT_VF8_M8_MASK	= 8122,
    RISCV_PseudoVSHA2CH_VV_M1	= 8123,
    RISCV_PseudoVSHA2CH_VV_M2	= 8124,
    RISCV_PseudoVSHA2CH_VV_M4	= 8125,
    RISCV_PseudoVSHA2CH_VV_M8	= 8126,
    RISCV_PseudoVSHA2CH_VV_MF2	= 8127,
    RISCV_PseudoVSHA2CL_VV_M1	= 8128,
    RISCV_PseudoVSHA2CL_VV_M2	= 8129,
    RISCV_PseudoVSHA2CL_VV_M4	= 8130,
    RISCV_PseudoVSHA2CL_VV_M8	= 8131,
    RISCV_PseudoVSHA2CL_VV_MF2	= 8132,
    RISCV_PseudoVSHA2MS_VV_M1	= 8133,
    RISCV_PseudoVSHA2MS_VV_M2	= 8134,
    RISCV_PseudoVSHA2MS_VV_M4	= 8135,
    RISCV_PseudoVSHA2MS_VV_M8	= 8136,
    RISCV_PseudoVSHA2MS_VV_MF2	= 8137,
    RISCV_PseudoVSLIDE1DOWN_VX_M1	= 8138,
    RISCV_PseudoVSLIDE1DOWN_VX_M1_MASK	= 8139,
    RISCV_PseudoVSLIDE1DOWN_VX_M2	= 8140,
    RISCV_PseudoVSLIDE1DOWN_VX_M2_MASK	= 8141,
    RISCV_PseudoVSLIDE1DOWN_VX_M4	= 8142,
    RISCV_PseudoVSLIDE1DOWN_VX_M4_MASK	= 8143,
    RISCV_PseudoVSLIDE1DOWN_VX_M8	= 8144,
    RISCV_PseudoVSLIDE1DOWN_VX_M8_MASK	= 8145,
    RISCV_PseudoVSLIDE1DOWN_VX_MF2	= 8146,
    RISCV_PseudoVSLIDE1DOWN_VX_MF2_MASK	= 8147,
    RISCV_PseudoVSLIDE1DOWN_VX_MF4	= 8148,
    RISCV_PseudoVSLIDE1DOWN_VX_MF4_MASK	= 8149,
    RISCV_PseudoVSLIDE1DOWN_VX_MF8	= 8150,
    RISCV_PseudoVSLIDE1DOWN_VX_MF8_MASK	= 8151,
    RISCV_PseudoVSLIDE1UP_VX_M1	= 8152,
    RISCV_PseudoVSLIDE1UP_VX_M1_MASK	= 8153,
    RISCV_PseudoVSLIDE1UP_VX_M2	= 8154,
    RISCV_PseudoVSLIDE1UP_VX_M2_MASK	= 8155,
    RISCV_PseudoVSLIDE1UP_VX_M4	= 8156,
    RISCV_PseudoVSLIDE1UP_VX_M4_MASK	= 8157,
    RISCV_PseudoVSLIDE1UP_VX_M8	= 8158,
    RISCV_PseudoVSLIDE1UP_VX_M8_MASK	= 8159,
    RISCV_PseudoVSLIDE1UP_VX_MF2	= 8160,
    RISCV_PseudoVSLIDE1UP_VX_MF2_MASK	= 8161,
    RISCV_PseudoVSLIDE1UP_VX_MF4	= 8162,
    RISCV_PseudoVSLIDE1UP_VX_MF4_MASK	= 8163,
    RISCV_PseudoVSLIDE1UP_VX_MF8	= 8164,
    RISCV_PseudoVSLIDE1UP_VX_MF8_MASK	= 8165,
    RISCV_PseudoVSLIDEDOWN_VI_M1	= 8166,
    RISCV_PseudoVSLIDEDOWN_VI_M1_MASK	= 8167,
    RISCV_PseudoVSLIDEDOWN_VI_M2	= 8168,
    RISCV_PseudoVSLIDEDOWN_VI_M2_MASK	= 8169,
    RISCV_PseudoVSLIDEDOWN_VI_M4	= 8170,
    RISCV_PseudoVSLIDEDOWN_VI_M4_MASK	= 8171,
    RISCV_PseudoVSLIDEDOWN_VI_M8	= 8172,
    RISCV_PseudoVSLIDEDOWN_VI_M8_MASK	= 8173,
    RISCV_PseudoVSLIDEDOWN_VI_MF2	= 8174,
    RISCV_PseudoVSLIDEDOWN_VI_MF2_MASK	= 8175,
    RISCV_PseudoVSLIDEDOWN_VI_MF4	= 8176,
    RISCV_PseudoVSLIDEDOWN_VI_MF4_MASK	= 8177,
    RISCV_PseudoVSLIDEDOWN_VI_MF8	= 8178,
    RISCV_PseudoVSLIDEDOWN_VI_MF8_MASK	= 8179,
    RISCV_PseudoVSLIDEDOWN_VX_M1	= 8180,
    RISCV_PseudoVSLIDEDOWN_VX_M1_MASK	= 8181,
    RISCV_PseudoVSLIDEDOWN_VX_M2	= 8182,
    RISCV_PseudoVSLIDEDOWN_VX_M2_MASK	= 8183,
    RISCV_PseudoVSLIDEDOWN_VX_M4	= 8184,
    RISCV_PseudoVSLIDEDOWN_VX_M4_MASK	= 8185,
    RISCV_PseudoVSLIDEDOWN_VX_M8	= 8186,
    RISCV_PseudoVSLIDEDOWN_VX_M8_MASK	= 8187,
    RISCV_PseudoVSLIDEDOWN_VX_MF2	= 8188,
    RISCV_PseudoVSLIDEDOWN_VX_MF2_MASK	= 8189,
    RISCV_PseudoVSLIDEDOWN_VX_MF4	= 8190,
    RISCV_PseudoVSLIDEDOWN_VX_MF4_MASK	= 8191,
    RISCV_PseudoVSLIDEDOWN_VX_MF8	= 8192,
    RISCV_PseudoVSLIDEDOWN_VX_MF8_MASK	= 8193,
    RISCV_PseudoVSLIDEUP_VI_M1	= 8194,
    RISCV_PseudoVSLIDEUP_VI_M1_MASK	= 8195,
    RISCV_PseudoVSLIDEUP_VI_M2	= 8196,
    RISCV_PseudoVSLIDEUP_VI_M2_MASK	= 8197,
    RISCV_PseudoVSLIDEUP_VI_M4	= 8198,
    RISCV_PseudoVSLIDEUP_VI_M4_MASK	= 8199,
    RISCV_PseudoVSLIDEUP_VI_M8	= 8200,
    RISCV_PseudoVSLIDEUP_VI_M8_MASK	= 8201,
    RISCV_PseudoVSLIDEUP_VI_MF2	= 8202,
    RISCV_PseudoVSLIDEUP_VI_MF2_MASK	= 8203,
    RISCV_PseudoVSLIDEUP_VI_MF4	= 8204,
    RISCV_PseudoVSLIDEUP_VI_MF4_MASK	= 8205,
    RISCV_PseudoVSLIDEUP_VI_MF8	= 8206,
    RISCV_PseudoVSLIDEUP_VI_MF8_MASK	= 8207,
    RISCV_PseudoVSLIDEUP_VX_M1	= 8208,
    RISCV_PseudoVSLIDEUP_VX_M1_MASK	= 8209,
    RISCV_PseudoVSLIDEUP_VX_M2	= 8210,
    RISCV_PseudoVSLIDEUP_VX_M2_MASK	= 8211,
    RISCV_PseudoVSLIDEUP_VX_M4	= 8212,
    RISCV_PseudoVSLIDEUP_VX_M4_MASK	= 8213,
    RISCV_PseudoVSLIDEUP_VX_M8	= 8214,
    RISCV_PseudoVSLIDEUP_VX_M8_MASK	= 8215,
    RISCV_PseudoVSLIDEUP_VX_MF2	= 8216,
    RISCV_PseudoVSLIDEUP_VX_MF2_MASK	= 8217,
    RISCV_PseudoVSLIDEUP_VX_MF4	= 8218,
    RISCV_PseudoVSLIDEUP_VX_MF4_MASK	= 8219,
    RISCV_PseudoVSLIDEUP_VX_MF8	= 8220,
    RISCV_PseudoVSLIDEUP_VX_MF8_MASK	= 8221,
    RISCV_PseudoVSLL_VI_M1	= 8222,
    RISCV_PseudoVSLL_VI_M1_MASK	= 8223,
    RISCV_PseudoVSLL_VI_M2	= 8224,
    RISCV_PseudoVSLL_VI_M2_MASK	= 8225,
    RISCV_PseudoVSLL_VI_M4	= 8226,
    RISCV_PseudoVSLL_VI_M4_MASK	= 8227,
    RISCV_PseudoVSLL_VI_M8	= 8228,
    RISCV_PseudoVSLL_VI_M8_MASK	= 8229,
    RISCV_PseudoVSLL_VI_MF2	= 8230,
    RISCV_PseudoVSLL_VI_MF2_MASK	= 8231,
    RISCV_PseudoVSLL_VI_MF4	= 8232,
    RISCV_PseudoVSLL_VI_MF4_MASK	= 8233,
    RISCV_PseudoVSLL_VI_MF8	= 8234,
    RISCV_PseudoVSLL_VI_MF8_MASK	= 8235,
    RISCV_PseudoVSLL_VV_M1	= 8236,
    RISCV_PseudoVSLL_VV_M1_MASK	= 8237,
    RISCV_PseudoVSLL_VV_M2	= 8238,
    RISCV_PseudoVSLL_VV_M2_MASK	= 8239,
    RISCV_PseudoVSLL_VV_M4	= 8240,
    RISCV_PseudoVSLL_VV_M4_MASK	= 8241,
    RISCV_PseudoVSLL_VV_M8	= 8242,
    RISCV_PseudoVSLL_VV_M8_MASK	= 8243,
    RISCV_PseudoVSLL_VV_MF2	= 8244,
    RISCV_PseudoVSLL_VV_MF2_MASK	= 8245,
    RISCV_PseudoVSLL_VV_MF4	= 8246,
    RISCV_PseudoVSLL_VV_MF4_MASK	= 8247,
    RISCV_PseudoVSLL_VV_MF8	= 8248,
    RISCV_PseudoVSLL_VV_MF8_MASK	= 8249,
    RISCV_PseudoVSLL_VX_M1	= 8250,
    RISCV_PseudoVSLL_VX_M1_MASK	= 8251,
    RISCV_PseudoVSLL_VX_M2	= 8252,
    RISCV_PseudoVSLL_VX_M2_MASK	= 8253,
    RISCV_PseudoVSLL_VX_M4	= 8254,
    RISCV_PseudoVSLL_VX_M4_MASK	= 8255,
    RISCV_PseudoVSLL_VX_M8	= 8256,
    RISCV_PseudoVSLL_VX_M8_MASK	= 8257,
    RISCV_PseudoVSLL_VX_MF2	= 8258,
    RISCV_PseudoVSLL_VX_MF2_MASK	= 8259,
    RISCV_PseudoVSLL_VX_MF4	= 8260,
    RISCV_PseudoVSLL_VX_MF4_MASK	= 8261,
    RISCV_PseudoVSLL_VX_MF8	= 8262,
    RISCV_PseudoVSLL_VX_MF8_MASK	= 8263,
    RISCV_PseudoVSM3C_VI_M1	= 8264,
    RISCV_PseudoVSM3C_VI_M2	= 8265,
    RISCV_PseudoVSM3C_VI_M4	= 8266,
    RISCV_PseudoVSM3C_VI_M8	= 8267,
    RISCV_PseudoVSM3C_VI_MF2	= 8268,
    RISCV_PseudoVSM3ME_VV_M1	= 8269,
    RISCV_PseudoVSM3ME_VV_M2	= 8270,
    RISCV_PseudoVSM3ME_VV_M4	= 8271,
    RISCV_PseudoVSM3ME_VV_M8	= 8272,
    RISCV_PseudoVSM3ME_VV_MF2	= 8273,
    RISCV_PseudoVSM4K_VI_M1	= 8274,
    RISCV_PseudoVSM4K_VI_M2	= 8275,
    RISCV_PseudoVSM4K_VI_M4	= 8276,
    RISCV_PseudoVSM4K_VI_M8	= 8277,
    RISCV_PseudoVSM4K_VI_MF2	= 8278,
    RISCV_PseudoVSM4R_VS_M1_M1	= 8279,
    RISCV_PseudoVSM4R_VS_M1_MF2	= 8280,
    RISCV_PseudoVSM4R_VS_M1_MF4	= 8281,
    RISCV_PseudoVSM4R_VS_M1_MF8	= 8282,
    RISCV_PseudoVSM4R_VS_M2_M1	= 8283,
    RISCV_PseudoVSM4R_VS_M2_M2	= 8284,
    RISCV_PseudoVSM4R_VS_M2_MF2	= 8285,
    RISCV_PseudoVSM4R_VS_M2_MF4	= 8286,
    RISCV_PseudoVSM4R_VS_M2_MF8	= 8287,
    RISCV_PseudoVSM4R_VS_M4_M1	= 8288,
    RISCV_PseudoVSM4R_VS_M4_M2	= 8289,
    RISCV_PseudoVSM4R_VS_M4_M4	= 8290,
    RISCV_PseudoVSM4R_VS_M4_MF2	= 8291,
    RISCV_PseudoVSM4R_VS_M4_MF4	= 8292,
    RISCV_PseudoVSM4R_VS_M4_MF8	= 8293,
    RISCV_PseudoVSM4R_VS_M8_M1	= 8294,
    RISCV_PseudoVSM4R_VS_M8_M2	= 8295,
    RISCV_PseudoVSM4R_VS_M8_M4	= 8296,
    RISCV_PseudoVSM4R_VS_M8_MF2	= 8297,
    RISCV_PseudoVSM4R_VS_M8_MF4	= 8298,
    RISCV_PseudoVSM4R_VS_M8_MF8	= 8299,
    RISCV_PseudoVSM4R_VS_MF2_MF2	= 8300,
    RISCV_PseudoVSM4R_VS_MF2_MF4	= 8301,
    RISCV_PseudoVSM4R_VS_MF2_MF8	= 8302,
    RISCV_PseudoVSM4R_VV_M1	= 8303,
    RISCV_PseudoVSM4R_VV_M2	= 8304,
    RISCV_PseudoVSM4R_VV_M4	= 8305,
    RISCV_PseudoVSM4R_VV_M8	= 8306,
    RISCV_PseudoVSM4R_VV_MF2	= 8307,
    RISCV_PseudoVSMUL_VV_M1	= 8308,
    RISCV_PseudoVSMUL_VV_M1_MASK	= 8309,
    RISCV_PseudoVSMUL_VV_M2	= 8310,
    RISCV_PseudoVSMUL_VV_M2_MASK	= 8311,
    RISCV_PseudoVSMUL_VV_M4	= 8312,
    RISCV_PseudoVSMUL_VV_M4_MASK	= 8313,
    RISCV_PseudoVSMUL_VV_M8	= 8314,
    RISCV_PseudoVSMUL_VV_M8_MASK	= 8315,
    RISCV_PseudoVSMUL_VV_MF2	= 8316,
    RISCV_PseudoVSMUL_VV_MF2_MASK	= 8317,
    RISCV_PseudoVSMUL_VV_MF4	= 8318,
    RISCV_PseudoVSMUL_VV_MF4_MASK	= 8319,
    RISCV_PseudoVSMUL_VV_MF8	= 8320,
    RISCV_PseudoVSMUL_VV_MF8_MASK	= 8321,
    RISCV_PseudoVSMUL_VX_M1	= 8322,
    RISCV_PseudoVSMUL_VX_M1_MASK	= 8323,
    RISCV_PseudoVSMUL_VX_M2	= 8324,
    RISCV_PseudoVSMUL_VX_M2_MASK	= 8325,
    RISCV_PseudoVSMUL_VX_M4	= 8326,
    RISCV_PseudoVSMUL_VX_M4_MASK	= 8327,
    RISCV_PseudoVSMUL_VX_M8	= 8328,
    RISCV_PseudoVSMUL_VX_M8_MASK	= 8329,
    RISCV_PseudoVSMUL_VX_MF2	= 8330,
    RISCV_PseudoVSMUL_VX_MF2_MASK	= 8331,
    RISCV_PseudoVSMUL_VX_MF4	= 8332,
    RISCV_PseudoVSMUL_VX_MF4_MASK	= 8333,
    RISCV_PseudoVSMUL_VX_MF8	= 8334,
    RISCV_PseudoVSMUL_VX_MF8_MASK	= 8335,
    RISCV_PseudoVSM_V_B1	= 8336,
    RISCV_PseudoVSM_V_B16	= 8337,
    RISCV_PseudoVSM_V_B2	= 8338,
    RISCV_PseudoVSM_V_B32	= 8339,
    RISCV_PseudoVSM_V_B4	= 8340,
    RISCV_PseudoVSM_V_B64	= 8341,
    RISCV_PseudoVSM_V_B8	= 8342,
    RISCV_PseudoVSOXEI16_V_M1_M1	= 8343,
    RISCV_PseudoVSOXEI16_V_M1_M1_MASK	= 8344,
    RISCV_PseudoVSOXEI16_V_M1_M2	= 8345,
    RISCV_PseudoVSOXEI16_V_M1_M2_MASK	= 8346,
    RISCV_PseudoVSOXEI16_V_M1_M4	= 8347,
    RISCV_PseudoVSOXEI16_V_M1_M4_MASK	= 8348,
    RISCV_PseudoVSOXEI16_V_M1_MF2	= 8349,
    RISCV_PseudoVSOXEI16_V_M1_MF2_MASK	= 8350,
    RISCV_PseudoVSOXEI16_V_M2_M1	= 8351,
    RISCV_PseudoVSOXEI16_V_M2_M1_MASK	= 8352,
    RISCV_PseudoVSOXEI16_V_M2_M2	= 8353,
    RISCV_PseudoVSOXEI16_V_M2_M2_MASK	= 8354,
    RISCV_PseudoVSOXEI16_V_M2_M4	= 8355,
    RISCV_PseudoVSOXEI16_V_M2_M4_MASK	= 8356,
    RISCV_PseudoVSOXEI16_V_M2_M8	= 8357,
    RISCV_PseudoVSOXEI16_V_M2_M8_MASK	= 8358,
    RISCV_PseudoVSOXEI16_V_M4_M2	= 8359,
    RISCV_PseudoVSOXEI16_V_M4_M2_MASK	= 8360,
    RISCV_PseudoVSOXEI16_V_M4_M4	= 8361,
    RISCV_PseudoVSOXEI16_V_M4_M4_MASK	= 8362,
    RISCV_PseudoVSOXEI16_V_M4_M8	= 8363,
    RISCV_PseudoVSOXEI16_V_M4_M8_MASK	= 8364,
    RISCV_PseudoVSOXEI16_V_M8_M4	= 8365,
    RISCV_PseudoVSOXEI16_V_M8_M4_MASK	= 8366,
    RISCV_PseudoVSOXEI16_V_M8_M8	= 8367,
    RISCV_PseudoVSOXEI16_V_M8_M8_MASK	= 8368,
    RISCV_PseudoVSOXEI16_V_MF2_M1	= 8369,
    RISCV_PseudoVSOXEI16_V_MF2_M1_MASK	= 8370,
    RISCV_PseudoVSOXEI16_V_MF2_M2	= 8371,
    RISCV_PseudoVSOXEI16_V_MF2_M2_MASK	= 8372,
    RISCV_PseudoVSOXEI16_V_MF2_MF2	= 8373,
    RISCV_PseudoVSOXEI16_V_MF2_MF2_MASK	= 8374,
    RISCV_PseudoVSOXEI16_V_MF2_MF4	= 8375,
    RISCV_PseudoVSOXEI16_V_MF2_MF4_MASK	= 8376,
    RISCV_PseudoVSOXEI16_V_MF4_M1	= 8377,
    RISCV_PseudoVSOXEI16_V_MF4_M1_MASK	= 8378,
    RISCV_PseudoVSOXEI16_V_MF4_MF2	= 8379,
    RISCV_PseudoVSOXEI16_V_MF4_MF2_MASK	= 8380,
    RISCV_PseudoVSOXEI16_V_MF4_MF4	= 8381,
    RISCV_PseudoVSOXEI16_V_MF4_MF4_MASK	= 8382,
    RISCV_PseudoVSOXEI16_V_MF4_MF8	= 8383,
    RISCV_PseudoVSOXEI16_V_MF4_MF8_MASK	= 8384,
    RISCV_PseudoVSOXEI32_V_M1_M1	= 8385,
    RISCV_PseudoVSOXEI32_V_M1_M1_MASK	= 8386,
    RISCV_PseudoVSOXEI32_V_M1_M2	= 8387,
    RISCV_PseudoVSOXEI32_V_M1_M2_MASK	= 8388,
    RISCV_PseudoVSOXEI32_V_M1_MF2	= 8389,
    RISCV_PseudoVSOXEI32_V_M1_MF2_MASK	= 8390,
    RISCV_PseudoVSOXEI32_V_M1_MF4	= 8391,
    RISCV_PseudoVSOXEI32_V_M1_MF4_MASK	= 8392,
    RISCV_PseudoVSOXEI32_V_M2_M1	= 8393,
    RISCV_PseudoVSOXEI32_V_M2_M1_MASK	= 8394,
    RISCV_PseudoVSOXEI32_V_M2_M2	= 8395,
    RISCV_PseudoVSOXEI32_V_M2_M2_MASK	= 8396,
    RISCV_PseudoVSOXEI32_V_M2_M4	= 8397,
    RISCV_PseudoVSOXEI32_V_M2_M4_MASK	= 8398,
    RISCV_PseudoVSOXEI32_V_M2_MF2	= 8399,
    RISCV_PseudoVSOXEI32_V_M2_MF2_MASK	= 8400,
    RISCV_PseudoVSOXEI32_V_M4_M1	= 8401,
    RISCV_PseudoVSOXEI32_V_M4_M1_MASK	= 8402,
    RISCV_PseudoVSOXEI32_V_M4_M2	= 8403,
    RISCV_PseudoVSOXEI32_V_M4_M2_MASK	= 8404,
    RISCV_PseudoVSOXEI32_V_M4_M4	= 8405,
    RISCV_PseudoVSOXEI32_V_M4_M4_MASK	= 8406,
    RISCV_PseudoVSOXEI32_V_M4_M8	= 8407,
    RISCV_PseudoVSOXEI32_V_M4_M8_MASK	= 8408,
    RISCV_PseudoVSOXEI32_V_M8_M2	= 8409,
    RISCV_PseudoVSOXEI32_V_M8_M2_MASK	= 8410,
    RISCV_PseudoVSOXEI32_V_M8_M4	= 8411,
    RISCV_PseudoVSOXEI32_V_M8_M4_MASK	= 8412,
    RISCV_PseudoVSOXEI32_V_M8_M8	= 8413,
    RISCV_PseudoVSOXEI32_V_M8_M8_MASK	= 8414,
    RISCV_PseudoVSOXEI32_V_MF2_M1	= 8415,
    RISCV_PseudoVSOXEI32_V_MF2_M1_MASK	= 8416,
    RISCV_PseudoVSOXEI32_V_MF2_MF2	= 8417,
    RISCV_PseudoVSOXEI32_V_MF2_MF2_MASK	= 8418,
    RISCV_PseudoVSOXEI32_V_MF2_MF4	= 8419,
    RISCV_PseudoVSOXEI32_V_MF2_MF4_MASK	= 8420,
    RISCV_PseudoVSOXEI32_V_MF2_MF8	= 8421,
    RISCV_PseudoVSOXEI32_V_MF2_MF8_MASK	= 8422,
    RISCV_PseudoVSOXEI64_V_M1_M1	= 8423,
    RISCV_PseudoVSOXEI64_V_M1_M1_MASK	= 8424,
    RISCV_PseudoVSOXEI64_V_M1_MF2	= 8425,
    RISCV_PseudoVSOXEI64_V_M1_MF2_MASK	= 8426,
    RISCV_PseudoVSOXEI64_V_M1_MF4	= 8427,
    RISCV_PseudoVSOXEI64_V_M1_MF4_MASK	= 8428,
    RISCV_PseudoVSOXEI64_V_M1_MF8	= 8429,
    RISCV_PseudoVSOXEI64_V_M1_MF8_MASK	= 8430,
    RISCV_PseudoVSOXEI64_V_M2_M1	= 8431,
    RISCV_PseudoVSOXEI64_V_M2_M1_MASK	= 8432,
    RISCV_PseudoVSOXEI64_V_M2_M2	= 8433,
    RISCV_PseudoVSOXEI64_V_M2_M2_MASK	= 8434,
    RISCV_PseudoVSOXEI64_V_M2_MF2	= 8435,
    RISCV_PseudoVSOXEI64_V_M2_MF2_MASK	= 8436,
    RISCV_PseudoVSOXEI64_V_M2_MF4	= 8437,
    RISCV_PseudoVSOXEI64_V_M2_MF4_MASK	= 8438,
    RISCV_PseudoVSOXEI64_V_M4_M1	= 8439,
    RISCV_PseudoVSOXEI64_V_M4_M1_MASK	= 8440,
    RISCV_PseudoVSOXEI64_V_M4_M2	= 8441,
    RISCV_PseudoVSOXEI64_V_M4_M2_MASK	= 8442,
    RISCV_PseudoVSOXEI64_V_M4_M4	= 8443,
    RISCV_PseudoVSOXEI64_V_M4_M4_MASK	= 8444,
    RISCV_PseudoVSOXEI64_V_M4_MF2	= 8445,
    RISCV_PseudoVSOXEI64_V_M4_MF2_MASK	= 8446,
    RISCV_PseudoVSOXEI64_V_M8_M1	= 8447,
    RISCV_PseudoVSOXEI64_V_M8_M1_MASK	= 8448,
    RISCV_PseudoVSOXEI64_V_M8_M2	= 8449,
    RISCV_PseudoVSOXEI64_V_M8_M2_MASK	= 8450,
    RISCV_PseudoVSOXEI64_V_M8_M4	= 8451,
    RISCV_PseudoVSOXEI64_V_M8_M4_MASK	= 8452,
    RISCV_PseudoVSOXEI64_V_M8_M8	= 8453,
    RISCV_PseudoVSOXEI64_V_M8_M8_MASK	= 8454,
    RISCV_PseudoVSOXEI8_V_M1_M1	= 8455,
    RISCV_PseudoVSOXEI8_V_M1_M1_MASK	= 8456,
    RISCV_PseudoVSOXEI8_V_M1_M2	= 8457,
    RISCV_PseudoVSOXEI8_V_M1_M2_MASK	= 8458,
    RISCV_PseudoVSOXEI8_V_M1_M4	= 8459,
    RISCV_PseudoVSOXEI8_V_M1_M4_MASK	= 8460,
    RISCV_PseudoVSOXEI8_V_M1_M8	= 8461,
    RISCV_PseudoVSOXEI8_V_M1_M8_MASK	= 8462,
    RISCV_PseudoVSOXEI8_V_M2_M2	= 8463,
    RISCV_PseudoVSOXEI8_V_M2_M2_MASK	= 8464,
    RISCV_PseudoVSOXEI8_V_M2_M4	= 8465,
    RISCV_PseudoVSOXEI8_V_M2_M4_MASK	= 8466,
    RISCV_PseudoVSOXEI8_V_M2_M8	= 8467,
    RISCV_PseudoVSOXEI8_V_M2_M8_MASK	= 8468,
    RISCV_PseudoVSOXEI8_V_M4_M4	= 8469,
    RISCV_PseudoVSOXEI8_V_M4_M4_MASK	= 8470,
    RISCV_PseudoVSOXEI8_V_M4_M8	= 8471,
    RISCV_PseudoVSOXEI8_V_M4_M8_MASK	= 8472,
    RISCV_PseudoVSOXEI8_V_M8_M8	= 8473,
    RISCV_PseudoVSOXEI8_V_M8_M8_MASK	= 8474,
    RISCV_PseudoVSOXEI8_V_MF2_M1	= 8475,
    RISCV_PseudoVSOXEI8_V_MF2_M1_MASK	= 8476,
    RISCV_PseudoVSOXEI8_V_MF2_M2	= 8477,
    RISCV_PseudoVSOXEI8_V_MF2_M2_MASK	= 8478,
    RISCV_PseudoVSOXEI8_V_MF2_M4	= 8479,
    RISCV_PseudoVSOXEI8_V_MF2_M4_MASK	= 8480,
    RISCV_PseudoVSOXEI8_V_MF2_MF2	= 8481,
    RISCV_PseudoVSOXEI8_V_MF2_MF2_MASK	= 8482,
    RISCV_PseudoVSOXEI8_V_MF4_M1	= 8483,
    RISCV_PseudoVSOXEI8_V_MF4_M1_MASK	= 8484,
    RISCV_PseudoVSOXEI8_V_MF4_M2	= 8485,
    RISCV_PseudoVSOXEI8_V_MF4_M2_MASK	= 8486,
    RISCV_PseudoVSOXEI8_V_MF4_MF2	= 8487,
    RISCV_PseudoVSOXEI8_V_MF4_MF2_MASK	= 8488,
    RISCV_PseudoVSOXEI8_V_MF4_MF4	= 8489,
    RISCV_PseudoVSOXEI8_V_MF4_MF4_MASK	= 8490,
    RISCV_PseudoVSOXEI8_V_MF8_M1	= 8491,
    RISCV_PseudoVSOXEI8_V_MF8_M1_MASK	= 8492,
    RISCV_PseudoVSOXEI8_V_MF8_MF2	= 8493,
    RISCV_PseudoVSOXEI8_V_MF8_MF2_MASK	= 8494,
    RISCV_PseudoVSOXEI8_V_MF8_MF4	= 8495,
    RISCV_PseudoVSOXEI8_V_MF8_MF4_MASK	= 8496,
    RISCV_PseudoVSOXEI8_V_MF8_MF8	= 8497,
    RISCV_PseudoVSOXEI8_V_MF8_MF8_MASK	= 8498,
    RISCV_PseudoVSOXSEG2EI16_V_M1_M1	= 8499,
    RISCV_PseudoVSOXSEG2EI16_V_M1_M1_MASK	= 8500,
    RISCV_PseudoVSOXSEG2EI16_V_M1_M2	= 8501,
    RISCV_PseudoVSOXSEG2EI16_V_M1_M2_MASK	= 8502,
    RISCV_PseudoVSOXSEG2EI16_V_M1_M4	= 8503,
    RISCV_PseudoVSOXSEG2EI16_V_M1_M4_MASK	= 8504,
    RISCV_PseudoVSOXSEG2EI16_V_M1_MF2	= 8505,
    RISCV_PseudoVSOXSEG2EI16_V_M1_MF2_MASK	= 8506,
    RISCV_PseudoVSOXSEG2EI16_V_M2_M1	= 8507,
    RISCV_PseudoVSOXSEG2EI16_V_M2_M1_MASK	= 8508,
    RISCV_PseudoVSOXSEG2EI16_V_M2_M2	= 8509,
    RISCV_PseudoVSOXSEG2EI16_V_M2_M2_MASK	= 8510,
    RISCV_PseudoVSOXSEG2EI16_V_M2_M4	= 8511,
    RISCV_PseudoVSOXSEG2EI16_V_M2_M4_MASK	= 8512,
    RISCV_PseudoVSOXSEG2EI16_V_M4_M2	= 8513,
    RISCV_PseudoVSOXSEG2EI16_V_M4_M2_MASK	= 8514,
    RISCV_PseudoVSOXSEG2EI16_V_M4_M4	= 8515,
    RISCV_PseudoVSOXSEG2EI16_V_M4_M4_MASK	= 8516,
    RISCV_PseudoVSOXSEG2EI16_V_M8_M4	= 8517,
    RISCV_PseudoVSOXSEG2EI16_V_M8_M4_MASK	= 8518,
    RISCV_PseudoVSOXSEG2EI16_V_MF2_M1	= 8519,
    RISCV_PseudoVSOXSEG2EI16_V_MF2_M1_MASK	= 8520,
    RISCV_PseudoVSOXSEG2EI16_V_MF2_M2	= 8521,
    RISCV_PseudoVSOXSEG2EI16_V_MF2_M2_MASK	= 8522,
    RISCV_PseudoVSOXSEG2EI16_V_MF2_MF2	= 8523,
    RISCV_PseudoVSOXSEG2EI16_V_MF2_MF2_MASK	= 8524,
    RISCV_PseudoVSOXSEG2EI16_V_MF2_MF4	= 8525,
    RISCV_PseudoVSOXSEG2EI16_V_MF2_MF4_MASK	= 8526,
    RISCV_PseudoVSOXSEG2EI16_V_MF4_M1	= 8527,
    RISCV_PseudoVSOXSEG2EI16_V_MF4_M1_MASK	= 8528,
    RISCV_PseudoVSOXSEG2EI16_V_MF4_MF2	= 8529,
    RISCV_PseudoVSOXSEG2EI16_V_MF4_MF2_MASK	= 8530,
    RISCV_PseudoVSOXSEG2EI16_V_MF4_MF4	= 8531,
    RISCV_PseudoVSOXSEG2EI16_V_MF4_MF4_MASK	= 8532,
    RISCV_PseudoVSOXSEG2EI16_V_MF4_MF8	= 8533,
    RISCV_PseudoVSOXSEG2EI16_V_MF4_MF8_MASK	= 8534,
    RISCV_PseudoVSOXSEG2EI32_V_M1_M1	= 8535,
    RISCV_PseudoVSOXSEG2EI32_V_M1_M1_MASK	= 8536,
    RISCV_PseudoVSOXSEG2EI32_V_M1_M2	= 8537,
    RISCV_PseudoVSOXSEG2EI32_V_M1_M2_MASK	= 8538,
    RISCV_PseudoVSOXSEG2EI32_V_M1_MF2	= 8539,
    RISCV_PseudoVSOXSEG2EI32_V_M1_MF2_MASK	= 8540,
    RISCV_PseudoVSOXSEG2EI32_V_M1_MF4	= 8541,
    RISCV_PseudoVSOXSEG2EI32_V_M1_MF4_MASK	= 8542,
    RISCV_PseudoVSOXSEG2EI32_V_M2_M1	= 8543,
    RISCV_PseudoVSOXSEG2EI32_V_M2_M1_MASK	= 8544,
    RISCV_PseudoVSOXSEG2EI32_V_M2_M2	= 8545,
    RISCV_PseudoVSOXSEG2EI32_V_M2_M2_MASK	= 8546,
    RISCV_PseudoVSOXSEG2EI32_V_M2_M4	= 8547,
    RISCV_PseudoVSOXSEG2EI32_V_M2_M4_MASK	= 8548,
    RISCV_PseudoVSOXSEG2EI32_V_M2_MF2	= 8549,
    RISCV_PseudoVSOXSEG2EI32_V_M2_MF2_MASK	= 8550,
    RISCV_PseudoVSOXSEG2EI32_V_M4_M1	= 8551,
    RISCV_PseudoVSOXSEG2EI32_V_M4_M1_MASK	= 8552,
    RISCV_PseudoVSOXSEG2EI32_V_M4_M2	= 8553,
    RISCV_PseudoVSOXSEG2EI32_V_M4_M2_MASK	= 8554,
    RISCV_PseudoVSOXSEG2EI32_V_M4_M4	= 8555,
    RISCV_PseudoVSOXSEG2EI32_V_M4_M4_MASK	= 8556,
    RISCV_PseudoVSOXSEG2EI32_V_M8_M2	= 8557,
    RISCV_PseudoVSOXSEG2EI32_V_M8_M2_MASK	= 8558,
    RISCV_PseudoVSOXSEG2EI32_V_M8_M4	= 8559,
    RISCV_PseudoVSOXSEG2EI32_V_M8_M4_MASK	= 8560,
    RISCV_PseudoVSOXSEG2EI32_V_MF2_M1	= 8561,
    RISCV_PseudoVSOXSEG2EI32_V_MF2_M1_MASK	= 8562,
    RISCV_PseudoVSOXSEG2EI32_V_MF2_MF2	= 8563,
    RISCV_PseudoVSOXSEG2EI32_V_MF2_MF2_MASK	= 8564,
    RISCV_PseudoVSOXSEG2EI32_V_MF2_MF4	= 8565,
    RISCV_PseudoVSOXSEG2EI32_V_MF2_MF4_MASK	= 8566,
    RISCV_PseudoVSOXSEG2EI32_V_MF2_MF8	= 8567,
    RISCV_PseudoVSOXSEG2EI32_V_MF2_MF8_MASK	= 8568,
    RISCV_PseudoVSOXSEG2EI64_V_M1_M1	= 8569,
    RISCV_PseudoVSOXSEG2EI64_V_M1_M1_MASK	= 8570,
    RISCV_PseudoVSOXSEG2EI64_V_M1_MF2	= 8571,
    RISCV_PseudoVSOXSEG2EI64_V_M1_MF2_MASK	= 8572,
    RISCV_PseudoVSOXSEG2EI64_V_M1_MF4	= 8573,
    RISCV_PseudoVSOXSEG2EI64_V_M1_MF4_MASK	= 8574,
    RISCV_PseudoVSOXSEG2EI64_V_M1_MF8	= 8575,
    RISCV_PseudoVSOXSEG2EI64_V_M1_MF8_MASK	= 8576,
    RISCV_PseudoVSOXSEG2EI64_V_M2_M1	= 8577,
    RISCV_PseudoVSOXSEG2EI64_V_M2_M1_MASK	= 8578,
    RISCV_PseudoVSOXSEG2EI64_V_M2_M2	= 8579,
    RISCV_PseudoVSOXSEG2EI64_V_M2_M2_MASK	= 8580,
    RISCV_PseudoVSOXSEG2EI64_V_M2_MF2	= 8581,
    RISCV_PseudoVSOXSEG2EI64_V_M2_MF2_MASK	= 8582,
    RISCV_PseudoVSOXSEG2EI64_V_M2_MF4	= 8583,
    RISCV_PseudoVSOXSEG2EI64_V_M2_MF4_MASK	= 8584,
    RISCV_PseudoVSOXSEG2EI64_V_M4_M1	= 8585,
    RISCV_PseudoVSOXSEG2EI64_V_M4_M1_MASK	= 8586,
    RISCV_PseudoVSOXSEG2EI64_V_M4_M2	= 8587,
    RISCV_PseudoVSOXSEG2EI64_V_M4_M2_MASK	= 8588,
    RISCV_PseudoVSOXSEG2EI64_V_M4_M4	= 8589,
    RISCV_PseudoVSOXSEG2EI64_V_M4_M4_MASK	= 8590,
    RISCV_PseudoVSOXSEG2EI64_V_M4_MF2	= 8591,
    RISCV_PseudoVSOXSEG2EI64_V_M4_MF2_MASK	= 8592,
    RISCV_PseudoVSOXSEG2EI64_V_M8_M1	= 8593,
    RISCV_PseudoVSOXSEG2EI64_V_M8_M1_MASK	= 8594,
    RISCV_PseudoVSOXSEG2EI64_V_M8_M2	= 8595,
    RISCV_PseudoVSOXSEG2EI64_V_M8_M2_MASK	= 8596,
    RISCV_PseudoVSOXSEG2EI64_V_M8_M4	= 8597,
    RISCV_PseudoVSOXSEG2EI64_V_M8_M4_MASK	= 8598,
    RISCV_PseudoVSOXSEG2EI8_V_M1_M1	= 8599,
    RISCV_PseudoVSOXSEG2EI8_V_M1_M1_MASK	= 8600,
    RISCV_PseudoVSOXSEG2EI8_V_M1_M2	= 8601,
    RISCV_PseudoVSOXSEG2EI8_V_M1_M2_MASK	= 8602,
    RISCV_PseudoVSOXSEG2EI8_V_M1_M4	= 8603,
    RISCV_PseudoVSOXSEG2EI8_V_M1_M4_MASK	= 8604,
    RISCV_PseudoVSOXSEG2EI8_V_M2_M2	= 8605,
    RISCV_PseudoVSOXSEG2EI8_V_M2_M2_MASK	= 8606,
    RISCV_PseudoVSOXSEG2EI8_V_M2_M4	= 8607,
    RISCV_PseudoVSOXSEG2EI8_V_M2_M4_MASK	= 8608,
    RISCV_PseudoVSOXSEG2EI8_V_M4_M4	= 8609,
    RISCV_PseudoVSOXSEG2EI8_V_M4_M4_MASK	= 8610,
    RISCV_PseudoVSOXSEG2EI8_V_MF2_M1	= 8611,
    RISCV_PseudoVSOXSEG2EI8_V_MF2_M1_MASK	= 8612,
    RISCV_PseudoVSOXSEG2EI8_V_MF2_M2	= 8613,
    RISCV_PseudoVSOXSEG2EI8_V_MF2_M2_MASK	= 8614,
    RISCV_PseudoVSOXSEG2EI8_V_MF2_M4	= 8615,
    RISCV_PseudoVSOXSEG2EI8_V_MF2_M4_MASK	= 8616,
    RISCV_PseudoVSOXSEG2EI8_V_MF2_MF2	= 8617,
    RISCV_PseudoVSOXSEG2EI8_V_MF2_MF2_MASK	= 8618,
    RISCV_PseudoVSOXSEG2EI8_V_MF4_M1	= 8619,
    RISCV_PseudoVSOXSEG2EI8_V_MF4_M1_MASK	= 8620,
    RISCV_PseudoVSOXSEG2EI8_V_MF4_M2	= 8621,
    RISCV_PseudoVSOXSEG2EI8_V_MF4_M2_MASK	= 8622,
    RISCV_PseudoVSOXSEG2EI8_V_MF4_MF2	= 8623,
    RISCV_PseudoVSOXSEG2EI8_V_MF4_MF2_MASK	= 8624,
    RISCV_PseudoVSOXSEG2EI8_V_MF4_MF4	= 8625,
    RISCV_PseudoVSOXSEG2EI8_V_MF4_MF4_MASK	= 8626,
    RISCV_PseudoVSOXSEG2EI8_V_MF8_M1	= 8627,
    RISCV_PseudoVSOXSEG2EI8_V_MF8_M1_MASK	= 8628,
    RISCV_PseudoVSOXSEG2EI8_V_MF8_MF2	= 8629,
    RISCV_PseudoVSOXSEG2EI8_V_MF8_MF2_MASK	= 8630,
    RISCV_PseudoVSOXSEG2EI8_V_MF8_MF4	= 8631,
    RISCV_PseudoVSOXSEG2EI8_V_MF8_MF4_MASK	= 8632,
    RISCV_PseudoVSOXSEG2EI8_V_MF8_MF8	= 8633,
    RISCV_PseudoVSOXSEG2EI8_V_MF8_MF8_MASK	= 8634,
    RISCV_PseudoVSOXSEG3EI16_V_M1_M1	= 8635,
    RISCV_PseudoVSOXSEG3EI16_V_M1_M1_MASK	= 8636,
    RISCV_PseudoVSOXSEG3EI16_V_M1_M2	= 8637,
    RISCV_PseudoVSOXSEG3EI16_V_M1_M2_MASK	= 8638,
    RISCV_PseudoVSOXSEG3EI16_V_M1_MF2	= 8639,
    RISCV_PseudoVSOXSEG3EI16_V_M1_MF2_MASK	= 8640,
    RISCV_PseudoVSOXSEG3EI16_V_M2_M1	= 8641,
    RISCV_PseudoVSOXSEG3EI16_V_M2_M1_MASK	= 8642,
    RISCV_PseudoVSOXSEG3EI16_V_M2_M2	= 8643,
    RISCV_PseudoVSOXSEG3EI16_V_M2_M2_MASK	= 8644,
    RISCV_PseudoVSOXSEG3EI16_V_M4_M2	= 8645,
    RISCV_PseudoVSOXSEG3EI16_V_M4_M2_MASK	= 8646,
    RISCV_PseudoVSOXSEG3EI16_V_MF2_M1	= 8647,
    RISCV_PseudoVSOXSEG3EI16_V_MF2_M1_MASK	= 8648,
    RISCV_PseudoVSOXSEG3EI16_V_MF2_M2	= 8649,
    RISCV_PseudoVSOXSEG3EI16_V_MF2_M2_MASK	= 8650,
    RISCV_PseudoVSOXSEG3EI16_V_MF2_MF2	= 8651,
    RISCV_PseudoVSOXSEG3EI16_V_MF2_MF2_MASK	= 8652,
    RISCV_PseudoVSOXSEG3EI16_V_MF2_MF4	= 8653,
    RISCV_PseudoVSOXSEG3EI16_V_MF2_MF4_MASK	= 8654,
    RISCV_PseudoVSOXSEG3EI16_V_MF4_M1	= 8655,
    RISCV_PseudoVSOXSEG3EI16_V_MF4_M1_MASK	= 8656,
    RISCV_PseudoVSOXSEG3EI16_V_MF4_MF2	= 8657,
    RISCV_PseudoVSOXSEG3EI16_V_MF4_MF2_MASK	= 8658,
    RISCV_PseudoVSOXSEG3EI16_V_MF4_MF4	= 8659,
    RISCV_PseudoVSOXSEG3EI16_V_MF4_MF4_MASK	= 8660,
    RISCV_PseudoVSOXSEG3EI16_V_MF4_MF8	= 8661,
    RISCV_PseudoVSOXSEG3EI16_V_MF4_MF8_MASK	= 8662,
    RISCV_PseudoVSOXSEG3EI32_V_M1_M1	= 8663,
    RISCV_PseudoVSOXSEG3EI32_V_M1_M1_MASK	= 8664,
    RISCV_PseudoVSOXSEG3EI32_V_M1_M2	= 8665,
    RISCV_PseudoVSOXSEG3EI32_V_M1_M2_MASK	= 8666,
    RISCV_PseudoVSOXSEG3EI32_V_M1_MF2	= 8667,
    RISCV_PseudoVSOXSEG3EI32_V_M1_MF2_MASK	= 8668,
    RISCV_PseudoVSOXSEG3EI32_V_M1_MF4	= 8669,
    RISCV_PseudoVSOXSEG3EI32_V_M1_MF4_MASK	= 8670,
    RISCV_PseudoVSOXSEG3EI32_V_M2_M1	= 8671,
    RISCV_PseudoVSOXSEG3EI32_V_M2_M1_MASK	= 8672,
    RISCV_PseudoVSOXSEG3EI32_V_M2_M2	= 8673,
    RISCV_PseudoVSOXSEG3EI32_V_M2_M2_MASK	= 8674,
    RISCV_PseudoVSOXSEG3EI32_V_M2_MF2	= 8675,
    RISCV_PseudoVSOXSEG3EI32_V_M2_MF2_MASK	= 8676,
    RISCV_PseudoVSOXSEG3EI32_V_M4_M1	= 8677,
    RISCV_PseudoVSOXSEG3EI32_V_M4_M1_MASK	= 8678,
    RISCV_PseudoVSOXSEG3EI32_V_M4_M2	= 8679,
    RISCV_PseudoVSOXSEG3EI32_V_M4_M2_MASK	= 8680,
    RISCV_PseudoVSOXSEG3EI32_V_M8_M2	= 8681,
    RISCV_PseudoVSOXSEG3EI32_V_M8_M2_MASK	= 8682,
    RISCV_PseudoVSOXSEG3EI32_V_MF2_M1	= 8683,
    RISCV_PseudoVSOXSEG3EI32_V_MF2_M1_MASK	= 8684,
    RISCV_PseudoVSOXSEG3EI32_V_MF2_MF2	= 8685,
    RISCV_PseudoVSOXSEG3EI32_V_MF2_MF2_MASK	= 8686,
    RISCV_PseudoVSOXSEG3EI32_V_MF2_MF4	= 8687,
    RISCV_PseudoVSOXSEG3EI32_V_MF2_MF4_MASK	= 8688,
    RISCV_PseudoVSOXSEG3EI32_V_MF2_MF8	= 8689,
    RISCV_PseudoVSOXSEG3EI32_V_MF2_MF8_MASK	= 8690,
    RISCV_PseudoVSOXSEG3EI64_V_M1_M1	= 8691,
    RISCV_PseudoVSOXSEG3EI64_V_M1_M1_MASK	= 8692,
    RISCV_PseudoVSOXSEG3EI64_V_M1_MF2	= 8693,
    RISCV_PseudoVSOXSEG3EI64_V_M1_MF2_MASK	= 8694,
    RISCV_PseudoVSOXSEG3EI64_V_M1_MF4	= 8695,
    RISCV_PseudoVSOXSEG3EI64_V_M1_MF4_MASK	= 8696,
    RISCV_PseudoVSOXSEG3EI64_V_M1_MF8	= 8697,
    RISCV_PseudoVSOXSEG3EI64_V_M1_MF8_MASK	= 8698,
    RISCV_PseudoVSOXSEG3EI64_V_M2_M1	= 8699,
    RISCV_PseudoVSOXSEG3EI64_V_M2_M1_MASK	= 8700,
    RISCV_PseudoVSOXSEG3EI64_V_M2_M2	= 8701,
    RISCV_PseudoVSOXSEG3EI64_V_M2_M2_MASK	= 8702,
    RISCV_PseudoVSOXSEG3EI64_V_M2_MF2	= 8703,
    RISCV_PseudoVSOXSEG3EI64_V_M2_MF2_MASK	= 8704,
    RISCV_PseudoVSOXSEG3EI64_V_M2_MF4	= 8705,
    RISCV_PseudoVSOXSEG3EI64_V_M2_MF4_MASK	= 8706,
    RISCV_PseudoVSOXSEG3EI64_V_M4_M1	= 8707,
    RISCV_PseudoVSOXSEG3EI64_V_M4_M1_MASK	= 8708,
    RISCV_PseudoVSOXSEG3EI64_V_M4_M2	= 8709,
    RISCV_PseudoVSOXSEG3EI64_V_M4_M2_MASK	= 8710,
    RISCV_PseudoVSOXSEG3EI64_V_M4_MF2	= 8711,
    RISCV_PseudoVSOXSEG3EI64_V_M4_MF2_MASK	= 8712,
    RISCV_PseudoVSOXSEG3EI64_V_M8_M1	= 8713,
    RISCV_PseudoVSOXSEG3EI64_V_M8_M1_MASK	= 8714,
    RISCV_PseudoVSOXSEG3EI64_V_M8_M2	= 8715,
    RISCV_PseudoVSOXSEG3EI64_V_M8_M2_MASK	= 8716,
    RISCV_PseudoVSOXSEG3EI8_V_M1_M1	= 8717,
    RISCV_PseudoVSOXSEG3EI8_V_M1_M1_MASK	= 8718,
    RISCV_PseudoVSOXSEG3EI8_V_M1_M2	= 8719,
    RISCV_PseudoVSOXSEG3EI8_V_M1_M2_MASK	= 8720,
    RISCV_PseudoVSOXSEG3EI8_V_M2_M2	= 8721,
    RISCV_PseudoVSOXSEG3EI8_V_M2_M2_MASK	= 8722,
    RISCV_PseudoVSOXSEG3EI8_V_MF2_M1	= 8723,
    RISCV_PseudoVSOXSEG3EI8_V_MF2_M1_MASK	= 8724,
    RISCV_PseudoVSOXSEG3EI8_V_MF2_M2	= 8725,
    RISCV_PseudoVSOXSEG3EI8_V_MF2_M2_MASK	= 8726,
    RISCV_PseudoVSOXSEG3EI8_V_MF2_MF2	= 8727,
    RISCV_PseudoVSOXSEG3EI8_V_MF2_MF2_MASK	= 8728,
    RISCV_PseudoVSOXSEG3EI8_V_MF4_M1	= 8729,
    RISCV_PseudoVSOXSEG3EI8_V_MF4_M1_MASK	= 8730,
    RISCV_PseudoVSOXSEG3EI8_V_MF4_M2	= 8731,
    RISCV_PseudoVSOXSEG3EI8_V_MF4_M2_MASK	= 8732,
    RISCV_PseudoVSOXSEG3EI8_V_MF4_MF2	= 8733,
    RISCV_PseudoVSOXSEG3EI8_V_MF4_MF2_MASK	= 8734,
    RISCV_PseudoVSOXSEG3EI8_V_MF4_MF4	= 8735,
    RISCV_PseudoVSOXSEG3EI8_V_MF4_MF4_MASK	= 8736,
    RISCV_PseudoVSOXSEG3EI8_V_MF8_M1	= 8737,
    RISCV_PseudoVSOXSEG3EI8_V_MF8_M1_MASK	= 8738,
    RISCV_PseudoVSOXSEG3EI8_V_MF8_MF2	= 8739,
    RISCV_PseudoVSOXSEG3EI8_V_MF8_MF2_MASK	= 8740,
    RISCV_PseudoVSOXSEG3EI8_V_MF8_MF4	= 8741,
    RISCV_PseudoVSOXSEG3EI8_V_MF8_MF4_MASK	= 8742,
    RISCV_PseudoVSOXSEG3EI8_V_MF8_MF8	= 8743,
    RISCV_PseudoVSOXSEG3EI8_V_MF8_MF8_MASK	= 8744,
    RISCV_PseudoVSOXSEG4EI16_V_M1_M1	= 8745,
    RISCV_PseudoVSOXSEG4EI16_V_M1_M1_MASK	= 8746,
    RISCV_PseudoVSOXSEG4EI16_V_M1_M2	= 8747,
    RISCV_PseudoVSOXSEG4EI16_V_M1_M2_MASK	= 8748,
    RISCV_PseudoVSOXSEG4EI16_V_M1_MF2	= 8749,
    RISCV_PseudoVSOXSEG4EI16_V_M1_MF2_MASK	= 8750,
    RISCV_PseudoVSOXSEG4EI16_V_M2_M1	= 8751,
    RISCV_PseudoVSOXSEG4EI16_V_M2_M1_MASK	= 8752,
    RISCV_PseudoVSOXSEG4EI16_V_M2_M2	= 8753,
    RISCV_PseudoVSOXSEG4EI16_V_M2_M2_MASK	= 8754,
    RISCV_PseudoVSOXSEG4EI16_V_M4_M2	= 8755,
    RISCV_PseudoVSOXSEG4EI16_V_M4_M2_MASK	= 8756,
    RISCV_PseudoVSOXSEG4EI16_V_MF2_M1	= 8757,
    RISCV_PseudoVSOXSEG4EI16_V_MF2_M1_MASK	= 8758,
    RISCV_PseudoVSOXSEG4EI16_V_MF2_M2	= 8759,
    RISCV_PseudoVSOXSEG4EI16_V_MF2_M2_MASK	= 8760,
    RISCV_PseudoVSOXSEG4EI16_V_MF2_MF2	= 8761,
    RISCV_PseudoVSOXSEG4EI16_V_MF2_MF2_MASK	= 8762,
    RISCV_PseudoVSOXSEG4EI16_V_MF2_MF4	= 8763,
    RISCV_PseudoVSOXSEG4EI16_V_MF2_MF4_MASK	= 8764,
    RISCV_PseudoVSOXSEG4EI16_V_MF4_M1	= 8765,
    RISCV_PseudoVSOXSEG4EI16_V_MF4_M1_MASK	= 8766,
    RISCV_PseudoVSOXSEG4EI16_V_MF4_MF2	= 8767,
    RISCV_PseudoVSOXSEG4EI16_V_MF4_MF2_MASK	= 8768,
    RISCV_PseudoVSOXSEG4EI16_V_MF4_MF4	= 8769,
    RISCV_PseudoVSOXSEG4EI16_V_MF4_MF4_MASK	= 8770,
    RISCV_PseudoVSOXSEG4EI16_V_MF4_MF8	= 8771,
    RISCV_PseudoVSOXSEG4EI16_V_MF4_MF8_MASK	= 8772,
    RISCV_PseudoVSOXSEG4EI32_V_M1_M1	= 8773,
    RISCV_PseudoVSOXSEG4EI32_V_M1_M1_MASK	= 8774,
    RISCV_PseudoVSOXSEG4EI32_V_M1_M2	= 8775,
    RISCV_PseudoVSOXSEG4EI32_V_M1_M2_MASK	= 8776,
    RISCV_PseudoVSOXSEG4EI32_V_M1_MF2	= 8777,
    RISCV_PseudoVSOXSEG4EI32_V_M1_MF2_MASK	= 8778,
    RISCV_PseudoVSOXSEG4EI32_V_M1_MF4	= 8779,
    RISCV_PseudoVSOXSEG4EI32_V_M1_MF4_MASK	= 8780,
    RISCV_PseudoVSOXSEG4EI32_V_M2_M1	= 8781,
    RISCV_PseudoVSOXSEG4EI32_V_M2_M1_MASK	= 8782,
    RISCV_PseudoVSOXSEG4EI32_V_M2_M2	= 8783,
    RISCV_PseudoVSOXSEG4EI32_V_M2_M2_MASK	= 8784,
    RISCV_PseudoVSOXSEG4EI32_V_M2_MF2	= 8785,
    RISCV_PseudoVSOXSEG4EI32_V_M2_MF2_MASK	= 8786,
    RISCV_PseudoVSOXSEG4EI32_V_M4_M1	= 8787,
    RISCV_PseudoVSOXSEG4EI32_V_M4_M1_MASK	= 8788,
    RISCV_PseudoVSOXSEG4EI32_V_M4_M2	= 8789,
    RISCV_PseudoVSOXSEG4EI32_V_M4_M2_MASK	= 8790,
    RISCV_PseudoVSOXSEG4EI32_V_M8_M2	= 8791,
    RISCV_PseudoVSOXSEG4EI32_V_M8_M2_MASK	= 8792,
    RISCV_PseudoVSOXSEG4EI32_V_MF2_M1	= 8793,
    RISCV_PseudoVSOXSEG4EI32_V_MF2_M1_MASK	= 8794,
    RISCV_PseudoVSOXSEG4EI32_V_MF2_MF2	= 8795,
    RISCV_PseudoVSOXSEG4EI32_V_MF2_MF2_MASK	= 8796,
    RISCV_PseudoVSOXSEG4EI32_V_MF2_MF4	= 8797,
    RISCV_PseudoVSOXSEG4EI32_V_MF2_MF4_MASK	= 8798,
    RISCV_PseudoVSOXSEG4EI32_V_MF2_MF8	= 8799,
    RISCV_PseudoVSOXSEG4EI32_V_MF2_MF8_MASK	= 8800,
    RISCV_PseudoVSOXSEG4EI64_V_M1_M1	= 8801,
    RISCV_PseudoVSOXSEG4EI64_V_M1_M1_MASK	= 8802,
    RISCV_PseudoVSOXSEG4EI64_V_M1_MF2	= 8803,
    RISCV_PseudoVSOXSEG4EI64_V_M1_MF2_MASK	= 8804,
    RISCV_PseudoVSOXSEG4EI64_V_M1_MF4	= 8805,
    RISCV_PseudoVSOXSEG4EI64_V_M1_MF4_MASK	= 8806,
    RISCV_PseudoVSOXSEG4EI64_V_M1_MF8	= 8807,
    RISCV_PseudoVSOXSEG4EI64_V_M1_MF8_MASK	= 8808,
    RISCV_PseudoVSOXSEG4EI64_V_M2_M1	= 8809,
    RISCV_PseudoVSOXSEG4EI64_V_M2_M1_MASK	= 8810,
    RISCV_PseudoVSOXSEG4EI64_V_M2_M2	= 8811,
    RISCV_PseudoVSOXSEG4EI64_V_M2_M2_MASK	= 8812,
    RISCV_PseudoVSOXSEG4EI64_V_M2_MF2	= 8813,
    RISCV_PseudoVSOXSEG4EI64_V_M2_MF2_MASK	= 8814,
    RISCV_PseudoVSOXSEG4EI64_V_M2_MF4	= 8815,
    RISCV_PseudoVSOXSEG4EI64_V_M2_MF4_MASK	= 8816,
    RISCV_PseudoVSOXSEG4EI64_V_M4_M1	= 8817,
    RISCV_PseudoVSOXSEG4EI64_V_M4_M1_MASK	= 8818,
    RISCV_PseudoVSOXSEG4EI64_V_M4_M2	= 8819,
    RISCV_PseudoVSOXSEG4EI64_V_M4_M2_MASK	= 8820,
    RISCV_PseudoVSOXSEG4EI64_V_M4_MF2	= 8821,
    RISCV_PseudoVSOXSEG4EI64_V_M4_MF2_MASK	= 8822,
    RISCV_PseudoVSOXSEG4EI64_V_M8_M1	= 8823,
    RISCV_PseudoVSOXSEG4EI64_V_M8_M1_MASK	= 8824,
    RISCV_PseudoVSOXSEG4EI64_V_M8_M2	= 8825,
    RISCV_PseudoVSOXSEG4EI64_V_M8_M2_MASK	= 8826,
    RISCV_PseudoVSOXSEG4EI8_V_M1_M1	= 8827,
    RISCV_PseudoVSOXSEG4EI8_V_M1_M1_MASK	= 8828,
    RISCV_PseudoVSOXSEG4EI8_V_M1_M2	= 8829,
    RISCV_PseudoVSOXSEG4EI8_V_M1_M2_MASK	= 8830,
    RISCV_PseudoVSOXSEG4EI8_V_M2_M2	= 8831,
    RISCV_PseudoVSOXSEG4EI8_V_M2_M2_MASK	= 8832,
    RISCV_PseudoVSOXSEG4EI8_V_MF2_M1	= 8833,
    RISCV_PseudoVSOXSEG4EI8_V_MF2_M1_MASK	= 8834,
    RISCV_PseudoVSOXSEG4EI8_V_MF2_M2	= 8835,
    RISCV_PseudoVSOXSEG4EI8_V_MF2_M2_MASK	= 8836,
    RISCV_PseudoVSOXSEG4EI8_V_MF2_MF2	= 8837,
    RISCV_PseudoVSOXSEG4EI8_V_MF2_MF2_MASK	= 8838,
    RISCV_PseudoVSOXSEG4EI8_V_MF4_M1	= 8839,
    RISCV_PseudoVSOXSEG4EI8_V_MF4_M1_MASK	= 8840,
    RISCV_PseudoVSOXSEG4EI8_V_MF4_M2	= 8841,
    RISCV_PseudoVSOXSEG4EI8_V_MF4_M2_MASK	= 8842,
    RISCV_PseudoVSOXSEG4EI8_V_MF4_MF2	= 8843,
    RISCV_PseudoVSOXSEG4EI8_V_MF4_MF2_MASK	= 8844,
    RISCV_PseudoVSOXSEG4EI8_V_MF4_MF4	= 8845,
    RISCV_PseudoVSOXSEG4EI8_V_MF4_MF4_MASK	= 8846,
    RISCV_PseudoVSOXSEG4EI8_V_MF8_M1	= 8847,
    RISCV_PseudoVSOXSEG4EI8_V_MF8_M1_MASK	= 8848,
    RISCV_PseudoVSOXSEG4EI8_V_MF8_MF2	= 8849,
    RISCV_PseudoVSOXSEG4EI8_V_MF8_MF2_MASK	= 8850,
    RISCV_PseudoVSOXSEG4EI8_V_MF8_MF4	= 8851,
    RISCV_PseudoVSOXSEG4EI8_V_MF8_MF4_MASK	= 8852,
    RISCV_PseudoVSOXSEG4EI8_V_MF8_MF8	= 8853,
    RISCV_PseudoVSOXSEG4EI8_V_MF8_MF8_MASK	= 8854,
    RISCV_PseudoVSOXSEG5EI16_V_M1_M1	= 8855,
    RISCV_PseudoVSOXSEG5EI16_V_M1_M1_MASK	= 8856,
    RISCV_PseudoVSOXSEG5EI16_V_M1_MF2	= 8857,
    RISCV_PseudoVSOXSEG5EI16_V_M1_MF2_MASK	= 8858,
    RISCV_PseudoVSOXSEG5EI16_V_M2_M1	= 8859,
    RISCV_PseudoVSOXSEG5EI16_V_M2_M1_MASK	= 8860,
    RISCV_PseudoVSOXSEG5EI16_V_MF2_M1	= 8861,
    RISCV_PseudoVSOXSEG5EI16_V_MF2_M1_MASK	= 8862,
    RISCV_PseudoVSOXSEG5EI16_V_MF2_MF2	= 8863,
    RISCV_PseudoVSOXSEG5EI16_V_MF2_MF2_MASK	= 8864,
    RISCV_PseudoVSOXSEG5EI16_V_MF2_MF4	= 8865,
    RISCV_PseudoVSOXSEG5EI16_V_MF2_MF4_MASK	= 8866,
    RISCV_PseudoVSOXSEG5EI16_V_MF4_M1	= 8867,
    RISCV_PseudoVSOXSEG5EI16_V_MF4_M1_MASK	= 8868,
    RISCV_PseudoVSOXSEG5EI16_V_MF4_MF2	= 8869,
    RISCV_PseudoVSOXSEG5EI16_V_MF4_MF2_MASK	= 8870,
    RISCV_PseudoVSOXSEG5EI16_V_MF4_MF4	= 8871,
    RISCV_PseudoVSOXSEG5EI16_V_MF4_MF4_MASK	= 8872,
    RISCV_PseudoVSOXSEG5EI16_V_MF4_MF8	= 8873,
    RISCV_PseudoVSOXSEG5EI16_V_MF4_MF8_MASK	= 8874,
    RISCV_PseudoVSOXSEG5EI32_V_M1_M1	= 8875,
    RISCV_PseudoVSOXSEG5EI32_V_M1_M1_MASK	= 8876,
    RISCV_PseudoVSOXSEG5EI32_V_M1_MF2	= 8877,
    RISCV_PseudoVSOXSEG5EI32_V_M1_MF2_MASK	= 8878,
    RISCV_PseudoVSOXSEG5EI32_V_M1_MF4	= 8879,
    RISCV_PseudoVSOXSEG5EI32_V_M1_MF4_MASK	= 8880,
    RISCV_PseudoVSOXSEG5EI32_V_M2_M1	= 8881,
    RISCV_PseudoVSOXSEG5EI32_V_M2_M1_MASK	= 8882,
    RISCV_PseudoVSOXSEG5EI32_V_M2_MF2	= 8883,
    RISCV_PseudoVSOXSEG5EI32_V_M2_MF2_MASK	= 8884,
    RISCV_PseudoVSOXSEG5EI32_V_M4_M1	= 8885,
    RISCV_PseudoVSOXSEG5EI32_V_M4_M1_MASK	= 8886,
    RISCV_PseudoVSOXSEG5EI32_V_MF2_M1	= 8887,
    RISCV_PseudoVSOXSEG5EI32_V_MF2_M1_MASK	= 8888,
    RISCV_PseudoVSOXSEG5EI32_V_MF2_MF2	= 8889,
    RISCV_PseudoVSOXSEG5EI32_V_MF2_MF2_MASK	= 8890,
    RISCV_PseudoVSOXSEG5EI32_V_MF2_MF4	= 8891,
    RISCV_PseudoVSOXSEG5EI32_V_MF2_MF4_MASK	= 8892,
    RISCV_PseudoVSOXSEG5EI32_V_MF2_MF8	= 8893,
    RISCV_PseudoVSOXSEG5EI32_V_MF2_MF8_MASK	= 8894,
    RISCV_PseudoVSOXSEG5EI64_V_M1_M1	= 8895,
    RISCV_PseudoVSOXSEG5EI64_V_M1_M1_MASK	= 8896,
    RISCV_PseudoVSOXSEG5EI64_V_M1_MF2	= 8897,
    RISCV_PseudoVSOXSEG5EI64_V_M1_MF2_MASK	= 8898,
    RISCV_PseudoVSOXSEG5EI64_V_M1_MF4	= 8899,
    RISCV_PseudoVSOXSEG5EI64_V_M1_MF4_MASK	= 8900,
    RISCV_PseudoVSOXSEG5EI64_V_M1_MF8	= 8901,
    RISCV_PseudoVSOXSEG5EI64_V_M1_MF8_MASK	= 8902,
    RISCV_PseudoVSOXSEG5EI64_V_M2_M1	= 8903,
    RISCV_PseudoVSOXSEG5EI64_V_M2_M1_MASK	= 8904,
    RISCV_PseudoVSOXSEG5EI64_V_M2_MF2	= 8905,
    RISCV_PseudoVSOXSEG5EI64_V_M2_MF2_MASK	= 8906,
    RISCV_PseudoVSOXSEG5EI64_V_M2_MF4	= 8907,
    RISCV_PseudoVSOXSEG5EI64_V_M2_MF4_MASK	= 8908,
    RISCV_PseudoVSOXSEG5EI64_V_M4_M1	= 8909,
    RISCV_PseudoVSOXSEG5EI64_V_M4_M1_MASK	= 8910,
    RISCV_PseudoVSOXSEG5EI64_V_M4_MF2	= 8911,
    RISCV_PseudoVSOXSEG5EI64_V_M4_MF2_MASK	= 8912,
    RISCV_PseudoVSOXSEG5EI64_V_M8_M1	= 8913,
    RISCV_PseudoVSOXSEG5EI64_V_M8_M1_MASK	= 8914,
    RISCV_PseudoVSOXSEG5EI8_V_M1_M1	= 8915,
    RISCV_PseudoVSOXSEG5EI8_V_M1_M1_MASK	= 8916,
    RISCV_PseudoVSOXSEG5EI8_V_MF2_M1	= 8917,
    RISCV_PseudoVSOXSEG5EI8_V_MF2_M1_MASK	= 8918,
    RISCV_PseudoVSOXSEG5EI8_V_MF2_MF2	= 8919,
    RISCV_PseudoVSOXSEG5EI8_V_MF2_MF2_MASK	= 8920,
    RISCV_PseudoVSOXSEG5EI8_V_MF4_M1	= 8921,
    RISCV_PseudoVSOXSEG5EI8_V_MF4_M1_MASK	= 8922,
    RISCV_PseudoVSOXSEG5EI8_V_MF4_MF2	= 8923,
    RISCV_PseudoVSOXSEG5EI8_V_MF4_MF2_MASK	= 8924,
    RISCV_PseudoVSOXSEG5EI8_V_MF4_MF4	= 8925,
    RISCV_PseudoVSOXSEG5EI8_V_MF4_MF4_MASK	= 8926,
    RISCV_PseudoVSOXSEG5EI8_V_MF8_M1	= 8927,
    RISCV_PseudoVSOXSEG5EI8_V_MF8_M1_MASK	= 8928,
    RISCV_PseudoVSOXSEG5EI8_V_MF8_MF2	= 8929,
    RISCV_PseudoVSOXSEG5EI8_V_MF8_MF2_MASK	= 8930,
    RISCV_PseudoVSOXSEG5EI8_V_MF8_MF4	= 8931,
    RISCV_PseudoVSOXSEG5EI8_V_MF8_MF4_MASK	= 8932,
    RISCV_PseudoVSOXSEG5EI8_V_MF8_MF8	= 8933,
    RISCV_PseudoVSOXSEG5EI8_V_MF8_MF8_MASK	= 8934,
    RISCV_PseudoVSOXSEG6EI16_V_M1_M1	= 8935,
    RISCV_PseudoVSOXSEG6EI16_V_M1_M1_MASK	= 8936,
    RISCV_PseudoVSOXSEG6EI16_V_M1_MF2	= 8937,
    RISCV_PseudoVSOXSEG6EI16_V_M1_MF2_MASK	= 8938,
    RISCV_PseudoVSOXSEG6EI16_V_M2_M1	= 8939,
    RISCV_PseudoVSOXSEG6EI16_V_M2_M1_MASK	= 8940,
    RISCV_PseudoVSOXSEG6EI16_V_MF2_M1	= 8941,
    RISCV_PseudoVSOXSEG6EI16_V_MF2_M1_MASK	= 8942,
    RISCV_PseudoVSOXSEG6EI16_V_MF2_MF2	= 8943,
    RISCV_PseudoVSOXSEG6EI16_V_MF2_MF2_MASK	= 8944,
    RISCV_PseudoVSOXSEG6EI16_V_MF2_MF4	= 8945,
    RISCV_PseudoVSOXSEG6EI16_V_MF2_MF4_MASK	= 8946,
    RISCV_PseudoVSOXSEG6EI16_V_MF4_M1	= 8947,
    RISCV_PseudoVSOXSEG6EI16_V_MF4_M1_MASK	= 8948,
    RISCV_PseudoVSOXSEG6EI16_V_MF4_MF2	= 8949,
    RISCV_PseudoVSOXSEG6EI16_V_MF4_MF2_MASK	= 8950,
    RISCV_PseudoVSOXSEG6EI16_V_MF4_MF4	= 8951,
    RISCV_PseudoVSOXSEG6EI16_V_MF4_MF4_MASK	= 8952,
    RISCV_PseudoVSOXSEG6EI16_V_MF4_MF8	= 8953,
    RISCV_PseudoVSOXSEG6EI16_V_MF4_MF8_MASK	= 8954,
    RISCV_PseudoVSOXSEG6EI32_V_M1_M1	= 8955,
    RISCV_PseudoVSOXSEG6EI32_V_M1_M1_MASK	= 8956,
    RISCV_PseudoVSOXSEG6EI32_V_M1_MF2	= 8957,
    RISCV_PseudoVSOXSEG6EI32_V_M1_MF2_MASK	= 8958,
    RISCV_PseudoVSOXSEG6EI32_V_M1_MF4	= 8959,
    RISCV_PseudoVSOXSEG6EI32_V_M1_MF4_MASK	= 8960,
    RISCV_PseudoVSOXSEG6EI32_V_M2_M1	= 8961,
    RISCV_PseudoVSOXSEG6EI32_V_M2_M1_MASK	= 8962,
    RISCV_PseudoVSOXSEG6EI32_V_M2_MF2	= 8963,
    RISCV_PseudoVSOXSEG6EI32_V_M2_MF2_MASK	= 8964,
    RISCV_PseudoVSOXSEG6EI32_V_M4_M1	= 8965,
    RISCV_PseudoVSOXSEG6EI32_V_M4_M1_MASK	= 8966,
    RISCV_PseudoVSOXSEG6EI32_V_MF2_M1	= 8967,
    RISCV_PseudoVSOXSEG6EI32_V_MF2_M1_MASK	= 8968,
    RISCV_PseudoVSOXSEG6EI32_V_MF2_MF2	= 8969,
    RISCV_PseudoVSOXSEG6EI32_V_MF2_MF2_MASK	= 8970,
    RISCV_PseudoVSOXSEG6EI32_V_MF2_MF4	= 8971,
    RISCV_PseudoVSOXSEG6EI32_V_MF2_MF4_MASK	= 8972,
    RISCV_PseudoVSOXSEG6EI32_V_MF2_MF8	= 8973,
    RISCV_PseudoVSOXSEG6EI32_V_MF2_MF8_MASK	= 8974,
    RISCV_PseudoVSOXSEG6EI64_V_M1_M1	= 8975,
    RISCV_PseudoVSOXSEG6EI64_V_M1_M1_MASK	= 8976,
    RISCV_PseudoVSOXSEG6EI64_V_M1_MF2	= 8977,
    RISCV_PseudoVSOXSEG6EI64_V_M1_MF2_MASK	= 8978,
    RISCV_PseudoVSOXSEG6EI64_V_M1_MF4	= 8979,
    RISCV_PseudoVSOXSEG6EI64_V_M1_MF4_MASK	= 8980,
    RISCV_PseudoVSOXSEG6EI64_V_M1_MF8	= 8981,
    RISCV_PseudoVSOXSEG6EI64_V_M1_MF8_MASK	= 8982,
    RISCV_PseudoVSOXSEG6EI64_V_M2_M1	= 8983,
    RISCV_PseudoVSOXSEG6EI64_V_M2_M1_MASK	= 8984,
    RISCV_PseudoVSOXSEG6EI64_V_M2_MF2	= 8985,
    RISCV_PseudoVSOXSEG6EI64_V_M2_MF2_MASK	= 8986,
    RISCV_PseudoVSOXSEG6EI64_V_M2_MF4	= 8987,
    RISCV_PseudoVSOXSEG6EI64_V_M2_MF4_MASK	= 8988,
    RISCV_PseudoVSOXSEG6EI64_V_M4_M1	= 8989,
    RISCV_PseudoVSOXSEG6EI64_V_M4_M1_MASK	= 8990,
    RISCV_PseudoVSOXSEG6EI64_V_M4_MF2	= 8991,
    RISCV_PseudoVSOXSEG6EI64_V_M4_MF2_MASK	= 8992,
    RISCV_PseudoVSOXSEG6EI64_V_M8_M1	= 8993,
    RISCV_PseudoVSOXSEG6EI64_V_M8_M1_MASK	= 8994,
    RISCV_PseudoVSOXSEG6EI8_V_M1_M1	= 8995,
    RISCV_PseudoVSOXSEG6EI8_V_M1_M1_MASK	= 8996,
    RISCV_PseudoVSOXSEG6EI8_V_MF2_M1	= 8997,
    RISCV_PseudoVSOXSEG6EI8_V_MF2_M1_MASK	= 8998,
    RISCV_PseudoVSOXSEG6EI8_V_MF2_MF2	= 8999,
    RISCV_PseudoVSOXSEG6EI8_V_MF2_MF2_MASK	= 9000,
    RISCV_PseudoVSOXSEG6EI8_V_MF4_M1	= 9001,
    RISCV_PseudoVSOXSEG6EI8_V_MF4_M1_MASK	= 9002,
    RISCV_PseudoVSOXSEG6EI8_V_MF4_MF2	= 9003,
    RISCV_PseudoVSOXSEG6EI8_V_MF4_MF2_MASK	= 9004,
    RISCV_PseudoVSOXSEG6EI8_V_MF4_MF4	= 9005,
    RISCV_PseudoVSOXSEG6EI8_V_MF4_MF4_MASK	= 9006,
    RISCV_PseudoVSOXSEG6EI8_V_MF8_M1	= 9007,
    RISCV_PseudoVSOXSEG6EI8_V_MF8_M1_MASK	= 9008,
    RISCV_PseudoVSOXSEG6EI8_V_MF8_MF2	= 9009,
    RISCV_PseudoVSOXSEG6EI8_V_MF8_MF2_MASK	= 9010,
    RISCV_PseudoVSOXSEG6EI8_V_MF8_MF4	= 9011,
    RISCV_PseudoVSOXSEG6EI8_V_MF8_MF4_MASK	= 9012,
    RISCV_PseudoVSOXSEG6EI8_V_MF8_MF8	= 9013,
    RISCV_PseudoVSOXSEG6EI8_V_MF8_MF8_MASK	= 9014,
    RISCV_PseudoVSOXSEG7EI16_V_M1_M1	= 9015,
    RISCV_PseudoVSOXSEG7EI16_V_M1_M1_MASK	= 9016,
    RISCV_PseudoVSOXSEG7EI16_V_M1_MF2	= 9017,
    RISCV_PseudoVSOXSEG7EI16_V_M1_MF2_MASK	= 9018,
    RISCV_PseudoVSOXSEG7EI16_V_M2_M1	= 9019,
    RISCV_PseudoVSOXSEG7EI16_V_M2_M1_MASK	= 9020,
    RISCV_PseudoVSOXSEG7EI16_V_MF2_M1	= 9021,
    RISCV_PseudoVSOXSEG7EI16_V_MF2_M1_MASK	= 9022,
    RISCV_PseudoVSOXSEG7EI16_V_MF2_MF2	= 9023,
    RISCV_PseudoVSOXSEG7EI16_V_MF2_MF2_MASK	= 9024,
    RISCV_PseudoVSOXSEG7EI16_V_MF2_MF4	= 9025,
    RISCV_PseudoVSOXSEG7EI16_V_MF2_MF4_MASK	= 9026,
    RISCV_PseudoVSOXSEG7EI16_V_MF4_M1	= 9027,
    RISCV_PseudoVSOXSEG7EI16_V_MF4_M1_MASK	= 9028,
    RISCV_PseudoVSOXSEG7EI16_V_MF4_MF2	= 9029,
    RISCV_PseudoVSOXSEG7EI16_V_MF4_MF2_MASK	= 9030,
    RISCV_PseudoVSOXSEG7EI16_V_MF4_MF4	= 9031,
    RISCV_PseudoVSOXSEG7EI16_V_MF4_MF4_MASK	= 9032,
    RISCV_PseudoVSOXSEG7EI16_V_MF4_MF8	= 9033,
    RISCV_PseudoVSOXSEG7EI16_V_MF4_MF8_MASK	= 9034,
    RISCV_PseudoVSOXSEG7EI32_V_M1_M1	= 9035,
    RISCV_PseudoVSOXSEG7EI32_V_M1_M1_MASK	= 9036,
    RISCV_PseudoVSOXSEG7EI32_V_M1_MF2	= 9037,
    RISCV_PseudoVSOXSEG7EI32_V_M1_MF2_MASK	= 9038,
    RISCV_PseudoVSOXSEG7EI32_V_M1_MF4	= 9039,
    RISCV_PseudoVSOXSEG7EI32_V_M1_MF4_MASK	= 9040,
    RISCV_PseudoVSOXSEG7EI32_V_M2_M1	= 9041,
    RISCV_PseudoVSOXSEG7EI32_V_M2_M1_MASK	= 9042,
    RISCV_PseudoVSOXSEG7EI32_V_M2_MF2	= 9043,
    RISCV_PseudoVSOXSEG7EI32_V_M2_MF2_MASK	= 9044,
    RISCV_PseudoVSOXSEG7EI32_V_M4_M1	= 9045,
    RISCV_PseudoVSOXSEG7EI32_V_M4_M1_MASK	= 9046,
    RISCV_PseudoVSOXSEG7EI32_V_MF2_M1	= 9047,
    RISCV_PseudoVSOXSEG7EI32_V_MF2_M1_MASK	= 9048,
    RISCV_PseudoVSOXSEG7EI32_V_MF2_MF2	= 9049,
    RISCV_PseudoVSOXSEG7EI32_V_MF2_MF2_MASK	= 9050,
    RISCV_PseudoVSOXSEG7EI32_V_MF2_MF4	= 9051,
    RISCV_PseudoVSOXSEG7EI32_V_MF2_MF4_MASK	= 9052,
    RISCV_PseudoVSOXSEG7EI32_V_MF2_MF8	= 9053,
    RISCV_PseudoVSOXSEG7EI32_V_MF2_MF8_MASK	= 9054,
    RISCV_PseudoVSOXSEG7EI64_V_M1_M1	= 9055,
    RISCV_PseudoVSOXSEG7EI64_V_M1_M1_MASK	= 9056,
    RISCV_PseudoVSOXSEG7EI64_V_M1_MF2	= 9057,
    RISCV_PseudoVSOXSEG7EI64_V_M1_MF2_MASK	= 9058,
    RISCV_PseudoVSOXSEG7EI64_V_M1_MF4	= 9059,
    RISCV_PseudoVSOXSEG7EI64_V_M1_MF4_MASK	= 9060,
    RISCV_PseudoVSOXSEG7EI64_V_M1_MF8	= 9061,
    RISCV_PseudoVSOXSEG7EI64_V_M1_MF8_MASK	= 9062,
    RISCV_PseudoVSOXSEG7EI64_V_M2_M1	= 9063,
    RISCV_PseudoVSOXSEG7EI64_V_M2_M1_MASK	= 9064,
    RISCV_PseudoVSOXSEG7EI64_V_M2_MF2	= 9065,
    RISCV_PseudoVSOXSEG7EI64_V_M2_MF2_MASK	= 9066,
    RISCV_PseudoVSOXSEG7EI64_V_M2_MF4	= 9067,
    RISCV_PseudoVSOXSEG7EI64_V_M2_MF4_MASK	= 9068,
    RISCV_PseudoVSOXSEG7EI64_V_M4_M1	= 9069,
    RISCV_PseudoVSOXSEG7EI64_V_M4_M1_MASK	= 9070,
    RISCV_PseudoVSOXSEG7EI64_V_M4_MF2	= 9071,
    RISCV_PseudoVSOXSEG7EI64_V_M4_MF2_MASK	= 9072,
    RISCV_PseudoVSOXSEG7EI64_V_M8_M1	= 9073,
    RISCV_PseudoVSOXSEG7EI64_V_M8_M1_MASK	= 9074,
    RISCV_PseudoVSOXSEG7EI8_V_M1_M1	= 9075,
    RISCV_PseudoVSOXSEG7EI8_V_M1_M1_MASK	= 9076,
    RISCV_PseudoVSOXSEG7EI8_V_MF2_M1	= 9077,
    RISCV_PseudoVSOXSEG7EI8_V_MF2_M1_MASK	= 9078,
    RISCV_PseudoVSOXSEG7EI8_V_MF2_MF2	= 9079,
    RISCV_PseudoVSOXSEG7EI8_V_MF2_MF2_MASK	= 9080,
    RISCV_PseudoVSOXSEG7EI8_V_MF4_M1	= 9081,
    RISCV_PseudoVSOXSEG7EI8_V_MF4_M1_MASK	= 9082,
    RISCV_PseudoVSOXSEG7EI8_V_MF4_MF2	= 9083,
    RISCV_PseudoVSOXSEG7EI8_V_MF4_MF2_MASK	= 9084,
    RISCV_PseudoVSOXSEG7EI8_V_MF4_MF4	= 9085,
    RISCV_PseudoVSOXSEG7EI8_V_MF4_MF4_MASK	= 9086,
    RISCV_PseudoVSOXSEG7EI8_V_MF8_M1	= 9087,
    RISCV_PseudoVSOXSEG7EI8_V_MF8_M1_MASK	= 9088,
    RISCV_PseudoVSOXSEG7EI8_V_MF8_MF2	= 9089,
    RISCV_PseudoVSOXSEG7EI8_V_MF8_MF2_MASK	= 9090,
    RISCV_PseudoVSOXSEG7EI8_V_MF8_MF4	= 9091,
    RISCV_PseudoVSOXSEG7EI8_V_MF8_MF4_MASK	= 9092,
    RISCV_PseudoVSOXSEG7EI8_V_MF8_MF8	= 9093,
    RISCV_PseudoVSOXSEG7EI8_V_MF8_MF8_MASK	= 9094,
    RISCV_PseudoVSOXSEG8EI16_V_M1_M1	= 9095,
    RISCV_PseudoVSOXSEG8EI16_V_M1_M1_MASK	= 9096,
    RISCV_PseudoVSOXSEG8EI16_V_M1_MF2	= 9097,
    RISCV_PseudoVSOXSEG8EI16_V_M1_MF2_MASK	= 9098,
    RISCV_PseudoVSOXSEG8EI16_V_M2_M1	= 9099,
    RISCV_PseudoVSOXSEG8EI16_V_M2_M1_MASK	= 9100,
    RISCV_PseudoVSOXSEG8EI16_V_MF2_M1	= 9101,
    RISCV_PseudoVSOXSEG8EI16_V_MF2_M1_MASK	= 9102,
    RISCV_PseudoVSOXSEG8EI16_V_MF2_MF2	= 9103,
    RISCV_PseudoVSOXSEG8EI16_V_MF2_MF2_MASK	= 9104,
    RISCV_PseudoVSOXSEG8EI16_V_MF2_MF4	= 9105,
    RISCV_PseudoVSOXSEG8EI16_V_MF2_MF4_MASK	= 9106,
    RISCV_PseudoVSOXSEG8EI16_V_MF4_M1	= 9107,
    RISCV_PseudoVSOXSEG8EI16_V_MF4_M1_MASK	= 9108,
    RISCV_PseudoVSOXSEG8EI16_V_MF4_MF2	= 9109,
    RISCV_PseudoVSOXSEG8EI16_V_MF4_MF2_MASK	= 9110,
    RISCV_PseudoVSOXSEG8EI16_V_MF4_MF4	= 9111,
    RISCV_PseudoVSOXSEG8EI16_V_MF4_MF4_MASK	= 9112,
    RISCV_PseudoVSOXSEG8EI16_V_MF4_MF8	= 9113,
    RISCV_PseudoVSOXSEG8EI16_V_MF4_MF8_MASK	= 9114,
    RISCV_PseudoVSOXSEG8EI32_V_M1_M1	= 9115,
    RISCV_PseudoVSOXSEG8EI32_V_M1_M1_MASK	= 9116,
    RISCV_PseudoVSOXSEG8EI32_V_M1_MF2	= 9117,
    RISCV_PseudoVSOXSEG8EI32_V_M1_MF2_MASK	= 9118,
    RISCV_PseudoVSOXSEG8EI32_V_M1_MF4	= 9119,
    RISCV_PseudoVSOXSEG8EI32_V_M1_MF4_MASK	= 9120,
    RISCV_PseudoVSOXSEG8EI32_V_M2_M1	= 9121,
    RISCV_PseudoVSOXSEG8EI32_V_M2_M1_MASK	= 9122,
    RISCV_PseudoVSOXSEG8EI32_V_M2_MF2	= 9123,
    RISCV_PseudoVSOXSEG8EI32_V_M2_MF2_MASK	= 9124,
    RISCV_PseudoVSOXSEG8EI32_V_M4_M1	= 9125,
    RISCV_PseudoVSOXSEG8EI32_V_M4_M1_MASK	= 9126,
    RISCV_PseudoVSOXSEG8EI32_V_MF2_M1	= 9127,
    RISCV_PseudoVSOXSEG8EI32_V_MF2_M1_MASK	= 9128,
    RISCV_PseudoVSOXSEG8EI32_V_MF2_MF2	= 9129,
    RISCV_PseudoVSOXSEG8EI32_V_MF2_MF2_MASK	= 9130,
    RISCV_PseudoVSOXSEG8EI32_V_MF2_MF4	= 9131,
    RISCV_PseudoVSOXSEG8EI32_V_MF2_MF4_MASK	= 9132,
    RISCV_PseudoVSOXSEG8EI32_V_MF2_MF8	= 9133,
    RISCV_PseudoVSOXSEG8EI32_V_MF2_MF8_MASK	= 9134,
    RISCV_PseudoVSOXSEG8EI64_V_M1_M1	= 9135,
    RISCV_PseudoVSOXSEG8EI64_V_M1_M1_MASK	= 9136,
    RISCV_PseudoVSOXSEG8EI64_V_M1_MF2	= 9137,
    RISCV_PseudoVSOXSEG8EI64_V_M1_MF2_MASK	= 9138,
    RISCV_PseudoVSOXSEG8EI64_V_M1_MF4	= 9139,
    RISCV_PseudoVSOXSEG8EI64_V_M1_MF4_MASK	= 9140,
    RISCV_PseudoVSOXSEG8EI64_V_M1_MF8	= 9141,
    RISCV_PseudoVSOXSEG8EI64_V_M1_MF8_MASK	= 9142,
    RISCV_PseudoVSOXSEG8EI64_V_M2_M1	= 9143,
    RISCV_PseudoVSOXSEG8EI64_V_M2_M1_MASK	= 9144,
    RISCV_PseudoVSOXSEG8EI64_V_M2_MF2	= 9145,
    RISCV_PseudoVSOXSEG8EI64_V_M2_MF2_MASK	= 9146,
    RISCV_PseudoVSOXSEG8EI64_V_M2_MF4	= 9147,
    RISCV_PseudoVSOXSEG8EI64_V_M2_MF4_MASK	= 9148,
    RISCV_PseudoVSOXSEG8EI64_V_M4_M1	= 9149,
    RISCV_PseudoVSOXSEG8EI64_V_M4_M1_MASK	= 9150,
    RISCV_PseudoVSOXSEG8EI64_V_M4_MF2	= 9151,
    RISCV_PseudoVSOXSEG8EI64_V_M4_MF2_MASK	= 9152,
    RISCV_PseudoVSOXSEG8EI64_V_M8_M1	= 9153,
    RISCV_PseudoVSOXSEG8EI64_V_M8_M1_MASK	= 9154,
    RISCV_PseudoVSOXSEG8EI8_V_M1_M1	= 9155,
    RISCV_PseudoVSOXSEG8EI8_V_M1_M1_MASK	= 9156,
    RISCV_PseudoVSOXSEG8EI8_V_MF2_M1	= 9157,
    RISCV_PseudoVSOXSEG8EI8_V_MF2_M1_MASK	= 9158,
    RISCV_PseudoVSOXSEG8EI8_V_MF2_MF2	= 9159,
    RISCV_PseudoVSOXSEG8EI8_V_MF2_MF2_MASK	= 9160,
    RISCV_PseudoVSOXSEG8EI8_V_MF4_M1	= 9161,
    RISCV_PseudoVSOXSEG8EI8_V_MF4_M1_MASK	= 9162,
    RISCV_PseudoVSOXSEG8EI8_V_MF4_MF2	= 9163,
    RISCV_PseudoVSOXSEG8EI8_V_MF4_MF2_MASK	= 9164,
    RISCV_PseudoVSOXSEG8EI8_V_MF4_MF4	= 9165,
    RISCV_PseudoVSOXSEG8EI8_V_MF4_MF4_MASK	= 9166,
    RISCV_PseudoVSOXSEG8EI8_V_MF8_M1	= 9167,
    RISCV_PseudoVSOXSEG8EI8_V_MF8_M1_MASK	= 9168,
    RISCV_PseudoVSOXSEG8EI8_V_MF8_MF2	= 9169,
    RISCV_PseudoVSOXSEG8EI8_V_MF8_MF2_MASK	= 9170,
    RISCV_PseudoVSOXSEG8EI8_V_MF8_MF4	= 9171,
    RISCV_PseudoVSOXSEG8EI8_V_MF8_MF4_MASK	= 9172,
    RISCV_PseudoVSOXSEG8EI8_V_MF8_MF8	= 9173,
    RISCV_PseudoVSOXSEG8EI8_V_MF8_MF8_MASK	= 9174,
    RISCV_PseudoVSPILL2_M1	= 9175,
    RISCV_PseudoVSPILL2_M2	= 9176,
    RISCV_PseudoVSPILL2_M4	= 9177,
    RISCV_PseudoVSPILL2_MF2	= 9178,
    RISCV_PseudoVSPILL2_MF4	= 9179,
    RISCV_PseudoVSPILL2_MF8	= 9180,
    RISCV_PseudoVSPILL3_M1	= 9181,
    RISCV_PseudoVSPILL3_M2	= 9182,
    RISCV_PseudoVSPILL3_MF2	= 9183,
    RISCV_PseudoVSPILL3_MF4	= 9184,
    RISCV_PseudoVSPILL3_MF8	= 9185,
    RISCV_PseudoVSPILL4_M1	= 9186,
    RISCV_PseudoVSPILL4_M2	= 9187,
    RISCV_PseudoVSPILL4_MF2	= 9188,
    RISCV_PseudoVSPILL4_MF4	= 9189,
    RISCV_PseudoVSPILL4_MF8	= 9190,
    RISCV_PseudoVSPILL5_M1	= 9191,
    RISCV_PseudoVSPILL5_MF2	= 9192,
    RISCV_PseudoVSPILL5_MF4	= 9193,
    RISCV_PseudoVSPILL5_MF8	= 9194,
    RISCV_PseudoVSPILL6_M1	= 9195,
    RISCV_PseudoVSPILL6_MF2	= 9196,
    RISCV_PseudoVSPILL6_MF4	= 9197,
    RISCV_PseudoVSPILL6_MF8	= 9198,
    RISCV_PseudoVSPILL7_M1	= 9199,
    RISCV_PseudoVSPILL7_MF2	= 9200,
    RISCV_PseudoVSPILL7_MF4	= 9201,
    RISCV_PseudoVSPILL7_MF8	= 9202,
    RISCV_PseudoVSPILL8_M1	= 9203,
    RISCV_PseudoVSPILL8_MF2	= 9204,
    RISCV_PseudoVSPILL8_MF4	= 9205,
    RISCV_PseudoVSPILL8_MF8	= 9206,
    RISCV_PseudoVSRA_VI_M1	= 9207,
    RISCV_PseudoVSRA_VI_M1_MASK	= 9208,
    RISCV_PseudoVSRA_VI_M2	= 9209,
    RISCV_PseudoVSRA_VI_M2_MASK	= 9210,
    RISCV_PseudoVSRA_VI_M4	= 9211,
    RISCV_PseudoVSRA_VI_M4_MASK	= 9212,
    RISCV_PseudoVSRA_VI_M8	= 9213,
    RISCV_PseudoVSRA_VI_M8_MASK	= 9214,
    RISCV_PseudoVSRA_VI_MF2	= 9215,
    RISCV_PseudoVSRA_VI_MF2_MASK	= 9216,
    RISCV_PseudoVSRA_VI_MF4	= 9217,
    RISCV_PseudoVSRA_VI_MF4_MASK	= 9218,
    RISCV_PseudoVSRA_VI_MF8	= 9219,
    RISCV_PseudoVSRA_VI_MF8_MASK	= 9220,
    RISCV_PseudoVSRA_VV_M1	= 9221,
    RISCV_PseudoVSRA_VV_M1_MASK	= 9222,
    RISCV_PseudoVSRA_VV_M2	= 9223,
    RISCV_PseudoVSRA_VV_M2_MASK	= 9224,
    RISCV_PseudoVSRA_VV_M4	= 9225,
    RISCV_PseudoVSRA_VV_M4_MASK	= 9226,
    RISCV_PseudoVSRA_VV_M8	= 9227,
    RISCV_PseudoVSRA_VV_M8_MASK	= 9228,
    RISCV_PseudoVSRA_VV_MF2	= 9229,
    RISCV_PseudoVSRA_VV_MF2_MASK	= 9230,
    RISCV_PseudoVSRA_VV_MF4	= 9231,
    RISCV_PseudoVSRA_VV_MF4_MASK	= 9232,
    RISCV_PseudoVSRA_VV_MF8	= 9233,
    RISCV_PseudoVSRA_VV_MF8_MASK	= 9234,
    RISCV_PseudoVSRA_VX_M1	= 9235,
    RISCV_PseudoVSRA_VX_M1_MASK	= 9236,
    RISCV_PseudoVSRA_VX_M2	= 9237,
    RISCV_PseudoVSRA_VX_M2_MASK	= 9238,
    RISCV_PseudoVSRA_VX_M4	= 9239,
    RISCV_PseudoVSRA_VX_M4_MASK	= 9240,
    RISCV_PseudoVSRA_VX_M8	= 9241,
    RISCV_PseudoVSRA_VX_M8_MASK	= 9242,
    RISCV_PseudoVSRA_VX_MF2	= 9243,
    RISCV_PseudoVSRA_VX_MF2_MASK	= 9244,
    RISCV_PseudoVSRA_VX_MF4	= 9245,
    RISCV_PseudoVSRA_VX_MF4_MASK	= 9246,
    RISCV_PseudoVSRA_VX_MF8	= 9247,
    RISCV_PseudoVSRA_VX_MF8_MASK	= 9248,
    RISCV_PseudoVSRL_VI_M1	= 9249,
    RISCV_PseudoVSRL_VI_M1_MASK	= 9250,
    RISCV_PseudoVSRL_VI_M2	= 9251,
    RISCV_PseudoVSRL_VI_M2_MASK	= 9252,
    RISCV_PseudoVSRL_VI_M4	= 9253,
    RISCV_PseudoVSRL_VI_M4_MASK	= 9254,
    RISCV_PseudoVSRL_VI_M8	= 9255,
    RISCV_PseudoVSRL_VI_M8_MASK	= 9256,
    RISCV_PseudoVSRL_VI_MF2	= 9257,
    RISCV_PseudoVSRL_VI_MF2_MASK	= 9258,
    RISCV_PseudoVSRL_VI_MF4	= 9259,
    RISCV_PseudoVSRL_VI_MF4_MASK	= 9260,
    RISCV_PseudoVSRL_VI_MF8	= 9261,
    RISCV_PseudoVSRL_VI_MF8_MASK	= 9262,
    RISCV_PseudoVSRL_VV_M1	= 9263,
    RISCV_PseudoVSRL_VV_M1_MASK	= 9264,
    RISCV_PseudoVSRL_VV_M2	= 9265,
    RISCV_PseudoVSRL_VV_M2_MASK	= 9266,
    RISCV_PseudoVSRL_VV_M4	= 9267,
    RISCV_PseudoVSRL_VV_M4_MASK	= 9268,
    RISCV_PseudoVSRL_VV_M8	= 9269,
    RISCV_PseudoVSRL_VV_M8_MASK	= 9270,
    RISCV_PseudoVSRL_VV_MF2	= 9271,
    RISCV_PseudoVSRL_VV_MF2_MASK	= 9272,
    RISCV_PseudoVSRL_VV_MF4	= 9273,
    RISCV_PseudoVSRL_VV_MF4_MASK	= 9274,
    RISCV_PseudoVSRL_VV_MF8	= 9275,
    RISCV_PseudoVSRL_VV_MF8_MASK	= 9276,
    RISCV_PseudoVSRL_VX_M1	= 9277,
    RISCV_PseudoVSRL_VX_M1_MASK	= 9278,
    RISCV_PseudoVSRL_VX_M2	= 9279,
    RISCV_PseudoVSRL_VX_M2_MASK	= 9280,
    RISCV_PseudoVSRL_VX_M4	= 9281,
    RISCV_PseudoVSRL_VX_M4_MASK	= 9282,
    RISCV_PseudoVSRL_VX_M8	= 9283,
    RISCV_PseudoVSRL_VX_M8_MASK	= 9284,
    RISCV_PseudoVSRL_VX_MF2	= 9285,
    RISCV_PseudoVSRL_VX_MF2_MASK	= 9286,
    RISCV_PseudoVSRL_VX_MF4	= 9287,
    RISCV_PseudoVSRL_VX_MF4_MASK	= 9288,
    RISCV_PseudoVSRL_VX_MF8	= 9289,
    RISCV_PseudoVSRL_VX_MF8_MASK	= 9290,
    RISCV_PseudoVSSE16_V_M1	= 9291,
    RISCV_PseudoVSSE16_V_M1_MASK	= 9292,
    RISCV_PseudoVSSE16_V_M2	= 9293,
    RISCV_PseudoVSSE16_V_M2_MASK	= 9294,
    RISCV_PseudoVSSE16_V_M4	= 9295,
    RISCV_PseudoVSSE16_V_M4_MASK	= 9296,
    RISCV_PseudoVSSE16_V_M8	= 9297,
    RISCV_PseudoVSSE16_V_M8_MASK	= 9298,
    RISCV_PseudoVSSE16_V_MF2	= 9299,
    RISCV_PseudoVSSE16_V_MF2_MASK	= 9300,
    RISCV_PseudoVSSE16_V_MF4	= 9301,
    RISCV_PseudoVSSE16_V_MF4_MASK	= 9302,
    RISCV_PseudoVSSE32_V_M1	= 9303,
    RISCV_PseudoVSSE32_V_M1_MASK	= 9304,
    RISCV_PseudoVSSE32_V_M2	= 9305,
    RISCV_PseudoVSSE32_V_M2_MASK	= 9306,
    RISCV_PseudoVSSE32_V_M4	= 9307,
    RISCV_PseudoVSSE32_V_M4_MASK	= 9308,
    RISCV_PseudoVSSE32_V_M8	= 9309,
    RISCV_PseudoVSSE32_V_M8_MASK	= 9310,
    RISCV_PseudoVSSE32_V_MF2	= 9311,
    RISCV_PseudoVSSE32_V_MF2_MASK	= 9312,
    RISCV_PseudoVSSE64_V_M1	= 9313,
    RISCV_PseudoVSSE64_V_M1_MASK	= 9314,
    RISCV_PseudoVSSE64_V_M2	= 9315,
    RISCV_PseudoVSSE64_V_M2_MASK	= 9316,
    RISCV_PseudoVSSE64_V_M4	= 9317,
    RISCV_PseudoVSSE64_V_M4_MASK	= 9318,
    RISCV_PseudoVSSE64_V_M8	= 9319,
    RISCV_PseudoVSSE64_V_M8_MASK	= 9320,
    RISCV_PseudoVSSE8_V_M1	= 9321,
    RISCV_PseudoVSSE8_V_M1_MASK	= 9322,
    RISCV_PseudoVSSE8_V_M2	= 9323,
    RISCV_PseudoVSSE8_V_M2_MASK	= 9324,
    RISCV_PseudoVSSE8_V_M4	= 9325,
    RISCV_PseudoVSSE8_V_M4_MASK	= 9326,
    RISCV_PseudoVSSE8_V_M8	= 9327,
    RISCV_PseudoVSSE8_V_M8_MASK	= 9328,
    RISCV_PseudoVSSE8_V_MF2	= 9329,
    RISCV_PseudoVSSE8_V_MF2_MASK	= 9330,
    RISCV_PseudoVSSE8_V_MF4	= 9331,
    RISCV_PseudoVSSE8_V_MF4_MASK	= 9332,
    RISCV_PseudoVSSE8_V_MF8	= 9333,
    RISCV_PseudoVSSE8_V_MF8_MASK	= 9334,
    RISCV_PseudoVSSEG2E16_V_M1	= 9335,
    RISCV_PseudoVSSEG2E16_V_M1_MASK	= 9336,
    RISCV_PseudoVSSEG2E16_V_M2	= 9337,
    RISCV_PseudoVSSEG2E16_V_M2_MASK	= 9338,
    RISCV_PseudoVSSEG2E16_V_M4	= 9339,
    RISCV_PseudoVSSEG2E16_V_M4_MASK	= 9340,
    RISCV_PseudoVSSEG2E16_V_MF2	= 9341,
    RISCV_PseudoVSSEG2E16_V_MF2_MASK	= 9342,
    RISCV_PseudoVSSEG2E16_V_MF4	= 9343,
    RISCV_PseudoVSSEG2E16_V_MF4_MASK	= 9344,
    RISCV_PseudoVSSEG2E32_V_M1	= 9345,
    RISCV_PseudoVSSEG2E32_V_M1_MASK	= 9346,
    RISCV_PseudoVSSEG2E32_V_M2	= 9347,
    RISCV_PseudoVSSEG2E32_V_M2_MASK	= 9348,
    RISCV_PseudoVSSEG2E32_V_M4	= 9349,
    RISCV_PseudoVSSEG2E32_V_M4_MASK	= 9350,
    RISCV_PseudoVSSEG2E32_V_MF2	= 9351,
    RISCV_PseudoVSSEG2E32_V_MF2_MASK	= 9352,
    RISCV_PseudoVSSEG2E64_V_M1	= 9353,
    RISCV_PseudoVSSEG2E64_V_M1_MASK	= 9354,
    RISCV_PseudoVSSEG2E64_V_M2	= 9355,
    RISCV_PseudoVSSEG2E64_V_M2_MASK	= 9356,
    RISCV_PseudoVSSEG2E64_V_M4	= 9357,
    RISCV_PseudoVSSEG2E64_V_M4_MASK	= 9358,
    RISCV_PseudoVSSEG2E8_V_M1	= 9359,
    RISCV_PseudoVSSEG2E8_V_M1_MASK	= 9360,
    RISCV_PseudoVSSEG2E8_V_M2	= 9361,
    RISCV_PseudoVSSEG2E8_V_M2_MASK	= 9362,
    RISCV_PseudoVSSEG2E8_V_M4	= 9363,
    RISCV_PseudoVSSEG2E8_V_M4_MASK	= 9364,
    RISCV_PseudoVSSEG2E8_V_MF2	= 9365,
    RISCV_PseudoVSSEG2E8_V_MF2_MASK	= 9366,
    RISCV_PseudoVSSEG2E8_V_MF4	= 9367,
    RISCV_PseudoVSSEG2E8_V_MF4_MASK	= 9368,
    RISCV_PseudoVSSEG2E8_V_MF8	= 9369,
    RISCV_PseudoVSSEG2E8_V_MF8_MASK	= 9370,
    RISCV_PseudoVSSEG3E16_V_M1	= 9371,
    RISCV_PseudoVSSEG3E16_V_M1_MASK	= 9372,
    RISCV_PseudoVSSEG3E16_V_M2	= 9373,
    RISCV_PseudoVSSEG3E16_V_M2_MASK	= 9374,
    RISCV_PseudoVSSEG3E16_V_MF2	= 9375,
    RISCV_PseudoVSSEG3E16_V_MF2_MASK	= 9376,
    RISCV_PseudoVSSEG3E16_V_MF4	= 9377,
    RISCV_PseudoVSSEG3E16_V_MF4_MASK	= 9378,
    RISCV_PseudoVSSEG3E32_V_M1	= 9379,
    RISCV_PseudoVSSEG3E32_V_M1_MASK	= 9380,
    RISCV_PseudoVSSEG3E32_V_M2	= 9381,
    RISCV_PseudoVSSEG3E32_V_M2_MASK	= 9382,
    RISCV_PseudoVSSEG3E32_V_MF2	= 9383,
    RISCV_PseudoVSSEG3E32_V_MF2_MASK	= 9384,
    RISCV_PseudoVSSEG3E64_V_M1	= 9385,
    RISCV_PseudoVSSEG3E64_V_M1_MASK	= 9386,
    RISCV_PseudoVSSEG3E64_V_M2	= 9387,
    RISCV_PseudoVSSEG3E64_V_M2_MASK	= 9388,
    RISCV_PseudoVSSEG3E8_V_M1	= 9389,
    RISCV_PseudoVSSEG3E8_V_M1_MASK	= 9390,
    RISCV_PseudoVSSEG3E8_V_M2	= 9391,
    RISCV_PseudoVSSEG3E8_V_M2_MASK	= 9392,
    RISCV_PseudoVSSEG3E8_V_MF2	= 9393,
    RISCV_PseudoVSSEG3E8_V_MF2_MASK	= 9394,
    RISCV_PseudoVSSEG3E8_V_MF4	= 9395,
    RISCV_PseudoVSSEG3E8_V_MF4_MASK	= 9396,
    RISCV_PseudoVSSEG3E8_V_MF8	= 9397,
    RISCV_PseudoVSSEG3E8_V_MF8_MASK	= 9398,
    RISCV_PseudoVSSEG4E16_V_M1	= 9399,
    RISCV_PseudoVSSEG4E16_V_M1_MASK	= 9400,
    RISCV_PseudoVSSEG4E16_V_M2	= 9401,
    RISCV_PseudoVSSEG4E16_V_M2_MASK	= 9402,
    RISCV_PseudoVSSEG4E16_V_MF2	= 9403,
    RISCV_PseudoVSSEG4E16_V_MF2_MASK	= 9404,
    RISCV_PseudoVSSEG4E16_V_MF4	= 9405,
    RISCV_PseudoVSSEG4E16_V_MF4_MASK	= 9406,
    RISCV_PseudoVSSEG4E32_V_M1	= 9407,
    RISCV_PseudoVSSEG4E32_V_M1_MASK	= 9408,
    RISCV_PseudoVSSEG4E32_V_M2	= 9409,
    RISCV_PseudoVSSEG4E32_V_M2_MASK	= 9410,
    RISCV_PseudoVSSEG4E32_V_MF2	= 9411,
    RISCV_PseudoVSSEG4E32_V_MF2_MASK	= 9412,
    RISCV_PseudoVSSEG4E64_V_M1	= 9413,
    RISCV_PseudoVSSEG4E64_V_M1_MASK	= 9414,
    RISCV_PseudoVSSEG4E64_V_M2	= 9415,
    RISCV_PseudoVSSEG4E64_V_M2_MASK	= 9416,
    RISCV_PseudoVSSEG4E8_V_M1	= 9417,
    RISCV_PseudoVSSEG4E8_V_M1_MASK	= 9418,
    RISCV_PseudoVSSEG4E8_V_M2	= 9419,
    RISCV_PseudoVSSEG4E8_V_M2_MASK	= 9420,
    RISCV_PseudoVSSEG4E8_V_MF2	= 9421,
    RISCV_PseudoVSSEG4E8_V_MF2_MASK	= 9422,
    RISCV_PseudoVSSEG4E8_V_MF4	= 9423,
    RISCV_PseudoVSSEG4E8_V_MF4_MASK	= 9424,
    RISCV_PseudoVSSEG4E8_V_MF8	= 9425,
    RISCV_PseudoVSSEG4E8_V_MF8_MASK	= 9426,
    RISCV_PseudoVSSEG5E16_V_M1	= 9427,
    RISCV_PseudoVSSEG5E16_V_M1_MASK	= 9428,
    RISCV_PseudoVSSEG5E16_V_MF2	= 9429,
    RISCV_PseudoVSSEG5E16_V_MF2_MASK	= 9430,
    RISCV_PseudoVSSEG5E16_V_MF4	= 9431,
    RISCV_PseudoVSSEG5E16_V_MF4_MASK	= 9432,
    RISCV_PseudoVSSEG5E32_V_M1	= 9433,
    RISCV_PseudoVSSEG5E32_V_M1_MASK	= 9434,
    RISCV_PseudoVSSEG5E32_V_MF2	= 9435,
    RISCV_PseudoVSSEG5E32_V_MF2_MASK	= 9436,
    RISCV_PseudoVSSEG5E64_V_M1	= 9437,
    RISCV_PseudoVSSEG5E64_V_M1_MASK	= 9438,
    RISCV_PseudoVSSEG5E8_V_M1	= 9439,
    RISCV_PseudoVSSEG5E8_V_M1_MASK	= 9440,
    RISCV_PseudoVSSEG5E8_V_MF2	= 9441,
    RISCV_PseudoVSSEG5E8_V_MF2_MASK	= 9442,
    RISCV_PseudoVSSEG5E8_V_MF4	= 9443,
    RISCV_PseudoVSSEG5E8_V_MF4_MASK	= 9444,
    RISCV_PseudoVSSEG5E8_V_MF8	= 9445,
    RISCV_PseudoVSSEG5E8_V_MF8_MASK	= 9446,
    RISCV_PseudoVSSEG6E16_V_M1	= 9447,
    RISCV_PseudoVSSEG6E16_V_M1_MASK	= 9448,
    RISCV_PseudoVSSEG6E16_V_MF2	= 9449,
    RISCV_PseudoVSSEG6E16_V_MF2_MASK	= 9450,
    RISCV_PseudoVSSEG6E16_V_MF4	= 9451,
    RISCV_PseudoVSSEG6E16_V_MF4_MASK	= 9452,
    RISCV_PseudoVSSEG6E32_V_M1	= 9453,
    RISCV_PseudoVSSEG6E32_V_M1_MASK	= 9454,
    RISCV_PseudoVSSEG6E32_V_MF2	= 9455,
    RISCV_PseudoVSSEG6E32_V_MF2_MASK	= 9456,
    RISCV_PseudoVSSEG6E64_V_M1	= 9457,
    RISCV_PseudoVSSEG6E64_V_M1_MASK	= 9458,
    RISCV_PseudoVSSEG6E8_V_M1	= 9459,
    RISCV_PseudoVSSEG6E8_V_M1_MASK	= 9460,
    RISCV_PseudoVSSEG6E8_V_MF2	= 9461,
    RISCV_PseudoVSSEG6E8_V_MF2_MASK	= 9462,
    RISCV_PseudoVSSEG6E8_V_MF4	= 9463,
    RISCV_PseudoVSSEG6E8_V_MF4_MASK	= 9464,
    RISCV_PseudoVSSEG6E8_V_MF8	= 9465,
    RISCV_PseudoVSSEG6E8_V_MF8_MASK	= 9466,
    RISCV_PseudoVSSEG7E16_V_M1	= 9467,
    RISCV_PseudoVSSEG7E16_V_M1_MASK	= 9468,
    RISCV_PseudoVSSEG7E16_V_MF2	= 9469,
    RISCV_PseudoVSSEG7E16_V_MF2_MASK	= 9470,
    RISCV_PseudoVSSEG7E16_V_MF4	= 9471,
    RISCV_PseudoVSSEG7E16_V_MF4_MASK	= 9472,
    RISCV_PseudoVSSEG7E32_V_M1	= 9473,
    RISCV_PseudoVSSEG7E32_V_M1_MASK	= 9474,
    RISCV_PseudoVSSEG7E32_V_MF2	= 9475,
    RISCV_PseudoVSSEG7E32_V_MF2_MASK	= 9476,
    RISCV_PseudoVSSEG7E64_V_M1	= 9477,
    RISCV_PseudoVSSEG7E64_V_M1_MASK	= 9478,
    RISCV_PseudoVSSEG7E8_V_M1	= 9479,
    RISCV_PseudoVSSEG7E8_V_M1_MASK	= 9480,
    RISCV_PseudoVSSEG7E8_V_MF2	= 9481,
    RISCV_PseudoVSSEG7E8_V_MF2_MASK	= 9482,
    RISCV_PseudoVSSEG7E8_V_MF4	= 9483,
    RISCV_PseudoVSSEG7E8_V_MF4_MASK	= 9484,
    RISCV_PseudoVSSEG7E8_V_MF8	= 9485,
    RISCV_PseudoVSSEG7E8_V_MF8_MASK	= 9486,
    RISCV_PseudoVSSEG8E16_V_M1	= 9487,
    RISCV_PseudoVSSEG8E16_V_M1_MASK	= 9488,
    RISCV_PseudoVSSEG8E16_V_MF2	= 9489,
    RISCV_PseudoVSSEG8E16_V_MF2_MASK	= 9490,
    RISCV_PseudoVSSEG8E16_V_MF4	= 9491,
    RISCV_PseudoVSSEG8E16_V_MF4_MASK	= 9492,
    RISCV_PseudoVSSEG8E32_V_M1	= 9493,
    RISCV_PseudoVSSEG8E32_V_M1_MASK	= 9494,
    RISCV_PseudoVSSEG8E32_V_MF2	= 9495,
    RISCV_PseudoVSSEG8E32_V_MF2_MASK	= 9496,
    RISCV_PseudoVSSEG8E64_V_M1	= 9497,
    RISCV_PseudoVSSEG8E64_V_M1_MASK	= 9498,
    RISCV_PseudoVSSEG8E8_V_M1	= 9499,
    RISCV_PseudoVSSEG8E8_V_M1_MASK	= 9500,
    RISCV_PseudoVSSEG8E8_V_MF2	= 9501,
    RISCV_PseudoVSSEG8E8_V_MF2_MASK	= 9502,
    RISCV_PseudoVSSEG8E8_V_MF4	= 9503,
    RISCV_PseudoVSSEG8E8_V_MF4_MASK	= 9504,
    RISCV_PseudoVSSEG8E8_V_MF8	= 9505,
    RISCV_PseudoVSSEG8E8_V_MF8_MASK	= 9506,
    RISCV_PseudoVSSRA_VI_M1	= 9507,
    RISCV_PseudoVSSRA_VI_M1_MASK	= 9508,
    RISCV_PseudoVSSRA_VI_M2	= 9509,
    RISCV_PseudoVSSRA_VI_M2_MASK	= 9510,
    RISCV_PseudoVSSRA_VI_M4	= 9511,
    RISCV_PseudoVSSRA_VI_M4_MASK	= 9512,
    RISCV_PseudoVSSRA_VI_M8	= 9513,
    RISCV_PseudoVSSRA_VI_M8_MASK	= 9514,
    RISCV_PseudoVSSRA_VI_MF2	= 9515,
    RISCV_PseudoVSSRA_VI_MF2_MASK	= 9516,
    RISCV_PseudoVSSRA_VI_MF4	= 9517,
    RISCV_PseudoVSSRA_VI_MF4_MASK	= 9518,
    RISCV_PseudoVSSRA_VI_MF8	= 9519,
    RISCV_PseudoVSSRA_VI_MF8_MASK	= 9520,
    RISCV_PseudoVSSRA_VV_M1	= 9521,
    RISCV_PseudoVSSRA_VV_M1_MASK	= 9522,
    RISCV_PseudoVSSRA_VV_M2	= 9523,
    RISCV_PseudoVSSRA_VV_M2_MASK	= 9524,
    RISCV_PseudoVSSRA_VV_M4	= 9525,
    RISCV_PseudoVSSRA_VV_M4_MASK	= 9526,
    RISCV_PseudoVSSRA_VV_M8	= 9527,
    RISCV_PseudoVSSRA_VV_M8_MASK	= 9528,
    RISCV_PseudoVSSRA_VV_MF2	= 9529,
    RISCV_PseudoVSSRA_VV_MF2_MASK	= 9530,
    RISCV_PseudoVSSRA_VV_MF4	= 9531,
    RISCV_PseudoVSSRA_VV_MF4_MASK	= 9532,
    RISCV_PseudoVSSRA_VV_MF8	= 9533,
    RISCV_PseudoVSSRA_VV_MF8_MASK	= 9534,
    RISCV_PseudoVSSRA_VX_M1	= 9535,
    RISCV_PseudoVSSRA_VX_M1_MASK	= 9536,
    RISCV_PseudoVSSRA_VX_M2	= 9537,
    RISCV_PseudoVSSRA_VX_M2_MASK	= 9538,
    RISCV_PseudoVSSRA_VX_M4	= 9539,
    RISCV_PseudoVSSRA_VX_M4_MASK	= 9540,
    RISCV_PseudoVSSRA_VX_M8	= 9541,
    RISCV_PseudoVSSRA_VX_M8_MASK	= 9542,
    RISCV_PseudoVSSRA_VX_MF2	= 9543,
    RISCV_PseudoVSSRA_VX_MF2_MASK	= 9544,
    RISCV_PseudoVSSRA_VX_MF4	= 9545,
    RISCV_PseudoVSSRA_VX_MF4_MASK	= 9546,
    RISCV_PseudoVSSRA_VX_MF8	= 9547,
    RISCV_PseudoVSSRA_VX_MF8_MASK	= 9548,
    RISCV_PseudoVSSRL_VI_M1	= 9549,
    RISCV_PseudoVSSRL_VI_M1_MASK	= 9550,
    RISCV_PseudoVSSRL_VI_M2	= 9551,
    RISCV_PseudoVSSRL_VI_M2_MASK	= 9552,
    RISCV_PseudoVSSRL_VI_M4	= 9553,
    RISCV_PseudoVSSRL_VI_M4_MASK	= 9554,
    RISCV_PseudoVSSRL_VI_M8	= 9555,
    RISCV_PseudoVSSRL_VI_M8_MASK	= 9556,
    RISCV_PseudoVSSRL_VI_MF2	= 9557,
    RISCV_PseudoVSSRL_VI_MF2_MASK	= 9558,
    RISCV_PseudoVSSRL_VI_MF4	= 9559,
    RISCV_PseudoVSSRL_VI_MF4_MASK	= 9560,
    RISCV_PseudoVSSRL_VI_MF8	= 9561,
    RISCV_PseudoVSSRL_VI_MF8_MASK	= 9562,
    RISCV_PseudoVSSRL_VV_M1	= 9563,
    RISCV_PseudoVSSRL_VV_M1_MASK	= 9564,
    RISCV_PseudoVSSRL_VV_M2	= 9565,
    RISCV_PseudoVSSRL_VV_M2_MASK	= 9566,
    RISCV_PseudoVSSRL_VV_M4	= 9567,
    RISCV_PseudoVSSRL_VV_M4_MASK	= 9568,
    RISCV_PseudoVSSRL_VV_M8	= 9569,
    RISCV_PseudoVSSRL_VV_M8_MASK	= 9570,
    RISCV_PseudoVSSRL_VV_MF2	= 9571,
    RISCV_PseudoVSSRL_VV_MF2_MASK	= 9572,
    RISCV_PseudoVSSRL_VV_MF4	= 9573,
    RISCV_PseudoVSSRL_VV_MF4_MASK	= 9574,
    RISCV_PseudoVSSRL_VV_MF8	= 9575,
    RISCV_PseudoVSSRL_VV_MF8_MASK	= 9576,
    RISCV_PseudoVSSRL_VX_M1	= 9577,
    RISCV_PseudoVSSRL_VX_M1_MASK	= 9578,
    RISCV_PseudoVSSRL_VX_M2	= 9579,
    RISCV_PseudoVSSRL_VX_M2_MASK	= 9580,
    RISCV_PseudoVSSRL_VX_M4	= 9581,
    RISCV_PseudoVSSRL_VX_M4_MASK	= 9582,
    RISCV_PseudoVSSRL_VX_M8	= 9583,
    RISCV_PseudoVSSRL_VX_M8_MASK	= 9584,
    RISCV_PseudoVSSRL_VX_MF2	= 9585,
    RISCV_PseudoVSSRL_VX_MF2_MASK	= 9586,
    RISCV_PseudoVSSRL_VX_MF4	= 9587,
    RISCV_PseudoVSSRL_VX_MF4_MASK	= 9588,
    RISCV_PseudoVSSRL_VX_MF8	= 9589,
    RISCV_PseudoVSSRL_VX_MF8_MASK	= 9590,
    RISCV_PseudoVSSSEG2E16_V_M1	= 9591,
    RISCV_PseudoVSSSEG2E16_V_M1_MASK	= 9592,
    RISCV_PseudoVSSSEG2E16_V_M2	= 9593,
    RISCV_PseudoVSSSEG2E16_V_M2_MASK	= 9594,
    RISCV_PseudoVSSSEG2E16_V_M4	= 9595,
    RISCV_PseudoVSSSEG2E16_V_M4_MASK	= 9596,
    RISCV_PseudoVSSSEG2E16_V_MF2	= 9597,
    RISCV_PseudoVSSSEG2E16_V_MF2_MASK	= 9598,
    RISCV_PseudoVSSSEG2E16_V_MF4	= 9599,
    RISCV_PseudoVSSSEG2E16_V_MF4_MASK	= 9600,
    RISCV_PseudoVSSSEG2E32_V_M1	= 9601,
    RISCV_PseudoVSSSEG2E32_V_M1_MASK	= 9602,
    RISCV_PseudoVSSSEG2E32_V_M2	= 9603,
    RISCV_PseudoVSSSEG2E32_V_M2_MASK	= 9604,
    RISCV_PseudoVSSSEG2E32_V_M4	= 9605,
    RISCV_PseudoVSSSEG2E32_V_M4_MASK	= 9606,
    RISCV_PseudoVSSSEG2E32_V_MF2	= 9607,
    RISCV_PseudoVSSSEG2E32_V_MF2_MASK	= 9608,
    RISCV_PseudoVSSSEG2E64_V_M1	= 9609,
    RISCV_PseudoVSSSEG2E64_V_M1_MASK	= 9610,
    RISCV_PseudoVSSSEG2E64_V_M2	= 9611,
    RISCV_PseudoVSSSEG2E64_V_M2_MASK	= 9612,
    RISCV_PseudoVSSSEG2E64_V_M4	= 9613,
    RISCV_PseudoVSSSEG2E64_V_M4_MASK	= 9614,
    RISCV_PseudoVSSSEG2E8_V_M1	= 9615,
    RISCV_PseudoVSSSEG2E8_V_M1_MASK	= 9616,
    RISCV_PseudoVSSSEG2E8_V_M2	= 9617,
    RISCV_PseudoVSSSEG2E8_V_M2_MASK	= 9618,
    RISCV_PseudoVSSSEG2E8_V_M4	= 9619,
    RISCV_PseudoVSSSEG2E8_V_M4_MASK	= 9620,
    RISCV_PseudoVSSSEG2E8_V_MF2	= 9621,
    RISCV_PseudoVSSSEG2E8_V_MF2_MASK	= 9622,
    RISCV_PseudoVSSSEG2E8_V_MF4	= 9623,
    RISCV_PseudoVSSSEG2E8_V_MF4_MASK	= 9624,
    RISCV_PseudoVSSSEG2E8_V_MF8	= 9625,
    RISCV_PseudoVSSSEG2E8_V_MF8_MASK	= 9626,
    RISCV_PseudoVSSSEG3E16_V_M1	= 9627,
    RISCV_PseudoVSSSEG3E16_V_M1_MASK	= 9628,
    RISCV_PseudoVSSSEG3E16_V_M2	= 9629,
    RISCV_PseudoVSSSEG3E16_V_M2_MASK	= 9630,
    RISCV_PseudoVSSSEG3E16_V_MF2	= 9631,
    RISCV_PseudoVSSSEG3E16_V_MF2_MASK	= 9632,
    RISCV_PseudoVSSSEG3E16_V_MF4	= 9633,
    RISCV_PseudoVSSSEG3E16_V_MF4_MASK	= 9634,
    RISCV_PseudoVSSSEG3E32_V_M1	= 9635,
    RISCV_PseudoVSSSEG3E32_V_M1_MASK	= 9636,
    RISCV_PseudoVSSSEG3E32_V_M2	= 9637,
    RISCV_PseudoVSSSEG3E32_V_M2_MASK	= 9638,
    RISCV_PseudoVSSSEG3E32_V_MF2	= 9639,
    RISCV_PseudoVSSSEG3E32_V_MF2_MASK	= 9640,
    RISCV_PseudoVSSSEG3E64_V_M1	= 9641,
    RISCV_PseudoVSSSEG3E64_V_M1_MASK	= 9642,
    RISCV_PseudoVSSSEG3E64_V_M2	= 9643,
    RISCV_PseudoVSSSEG3E64_V_M2_MASK	= 9644,
    RISCV_PseudoVSSSEG3E8_V_M1	= 9645,
    RISCV_PseudoVSSSEG3E8_V_M1_MASK	= 9646,
    RISCV_PseudoVSSSEG3E8_V_M2	= 9647,
    RISCV_PseudoVSSSEG3E8_V_M2_MASK	= 9648,
    RISCV_PseudoVSSSEG3E8_V_MF2	= 9649,
    RISCV_PseudoVSSSEG3E8_V_MF2_MASK	= 9650,
    RISCV_PseudoVSSSEG3E8_V_MF4	= 9651,
    RISCV_PseudoVSSSEG3E8_V_MF4_MASK	= 9652,
    RISCV_PseudoVSSSEG3E8_V_MF8	= 9653,
    RISCV_PseudoVSSSEG3E8_V_MF8_MASK	= 9654,
    RISCV_PseudoVSSSEG4E16_V_M1	= 9655,
    RISCV_PseudoVSSSEG4E16_V_M1_MASK	= 9656,
    RISCV_PseudoVSSSEG4E16_V_M2	= 9657,
    RISCV_PseudoVSSSEG4E16_V_M2_MASK	= 9658,
    RISCV_PseudoVSSSEG4E16_V_MF2	= 9659,
    RISCV_PseudoVSSSEG4E16_V_MF2_MASK	= 9660,
    RISCV_PseudoVSSSEG4E16_V_MF4	= 9661,
    RISCV_PseudoVSSSEG4E16_V_MF4_MASK	= 9662,
    RISCV_PseudoVSSSEG4E32_V_M1	= 9663,
    RISCV_PseudoVSSSEG4E32_V_M1_MASK	= 9664,
    RISCV_PseudoVSSSEG4E32_V_M2	= 9665,
    RISCV_PseudoVSSSEG4E32_V_M2_MASK	= 9666,
    RISCV_PseudoVSSSEG4E32_V_MF2	= 9667,
    RISCV_PseudoVSSSEG4E32_V_MF2_MASK	= 9668,
    RISCV_PseudoVSSSEG4E64_V_M1	= 9669,
    RISCV_PseudoVSSSEG4E64_V_M1_MASK	= 9670,
    RISCV_PseudoVSSSEG4E64_V_M2	= 9671,
    RISCV_PseudoVSSSEG4E64_V_M2_MASK	= 9672,
    RISCV_PseudoVSSSEG4E8_V_M1	= 9673,
    RISCV_PseudoVSSSEG4E8_V_M1_MASK	= 9674,
    RISCV_PseudoVSSSEG4E8_V_M2	= 9675,
    RISCV_PseudoVSSSEG4E8_V_M2_MASK	= 9676,
    RISCV_PseudoVSSSEG4E8_V_MF2	= 9677,
    RISCV_PseudoVSSSEG4E8_V_MF2_MASK	= 9678,
    RISCV_PseudoVSSSEG4E8_V_MF4	= 9679,
    RISCV_PseudoVSSSEG4E8_V_MF4_MASK	= 9680,
    RISCV_PseudoVSSSEG4E8_V_MF8	= 9681,
    RISCV_PseudoVSSSEG4E8_V_MF8_MASK	= 9682,
    RISCV_PseudoVSSSEG5E16_V_M1	= 9683,
    RISCV_PseudoVSSSEG5E16_V_M1_MASK	= 9684,
    RISCV_PseudoVSSSEG5E16_V_MF2	= 9685,
    RISCV_PseudoVSSSEG5E16_V_MF2_MASK	= 9686,
    RISCV_PseudoVSSSEG5E16_V_MF4	= 9687,
    RISCV_PseudoVSSSEG5E16_V_MF4_MASK	= 9688,
    RISCV_PseudoVSSSEG5E32_V_M1	= 9689,
    RISCV_PseudoVSSSEG5E32_V_M1_MASK	= 9690,
    RISCV_PseudoVSSSEG5E32_V_MF2	= 9691,
    RISCV_PseudoVSSSEG5E32_V_MF2_MASK	= 9692,
    RISCV_PseudoVSSSEG5E64_V_M1	= 9693,
    RISCV_PseudoVSSSEG5E64_V_M1_MASK	= 9694,
    RISCV_PseudoVSSSEG5E8_V_M1	= 9695,
    RISCV_PseudoVSSSEG5E8_V_M1_MASK	= 9696,
    RISCV_PseudoVSSSEG5E8_V_MF2	= 9697,
    RISCV_PseudoVSSSEG5E8_V_MF2_MASK	= 9698,
    RISCV_PseudoVSSSEG5E8_V_MF4	= 9699,
    RISCV_PseudoVSSSEG5E8_V_MF4_MASK	= 9700,
    RISCV_PseudoVSSSEG5E8_V_MF8	= 9701,
    RISCV_PseudoVSSSEG5E8_V_MF8_MASK	= 9702,
    RISCV_PseudoVSSSEG6E16_V_M1	= 9703,
    RISCV_PseudoVSSSEG6E16_V_M1_MASK	= 9704,
    RISCV_PseudoVSSSEG6E16_V_MF2	= 9705,
    RISCV_PseudoVSSSEG6E16_V_MF2_MASK	= 9706,
    RISCV_PseudoVSSSEG6E16_V_MF4	= 9707,
    RISCV_PseudoVSSSEG6E16_V_MF4_MASK	= 9708,
    RISCV_PseudoVSSSEG6E32_V_M1	= 9709,
    RISCV_PseudoVSSSEG6E32_V_M1_MASK	= 9710,
    RISCV_PseudoVSSSEG6E32_V_MF2	= 9711,
    RISCV_PseudoVSSSEG6E32_V_MF2_MASK	= 9712,
    RISCV_PseudoVSSSEG6E64_V_M1	= 9713,
    RISCV_PseudoVSSSEG6E64_V_M1_MASK	= 9714,
    RISCV_PseudoVSSSEG6E8_V_M1	= 9715,
    RISCV_PseudoVSSSEG6E8_V_M1_MASK	= 9716,
    RISCV_PseudoVSSSEG6E8_V_MF2	= 9717,
    RISCV_PseudoVSSSEG6E8_V_MF2_MASK	= 9718,
    RISCV_PseudoVSSSEG6E8_V_MF4	= 9719,
    RISCV_PseudoVSSSEG6E8_V_MF4_MASK	= 9720,
    RISCV_PseudoVSSSEG6E8_V_MF8	= 9721,
    RISCV_PseudoVSSSEG6E8_V_MF8_MASK	= 9722,
    RISCV_PseudoVSSSEG7E16_V_M1	= 9723,
    RISCV_PseudoVSSSEG7E16_V_M1_MASK	= 9724,
    RISCV_PseudoVSSSEG7E16_V_MF2	= 9725,
    RISCV_PseudoVSSSEG7E16_V_MF2_MASK	= 9726,
    RISCV_PseudoVSSSEG7E16_V_MF4	= 9727,
    RISCV_PseudoVSSSEG7E16_V_MF4_MASK	= 9728,
    RISCV_PseudoVSSSEG7E32_V_M1	= 9729,
    RISCV_PseudoVSSSEG7E32_V_M1_MASK	= 9730,
    RISCV_PseudoVSSSEG7E32_V_MF2	= 9731,
    RISCV_PseudoVSSSEG7E32_V_MF2_MASK	= 9732,
    RISCV_PseudoVSSSEG7E64_V_M1	= 9733,
    RISCV_PseudoVSSSEG7E64_V_M1_MASK	= 9734,
    RISCV_PseudoVSSSEG7E8_V_M1	= 9735,
    RISCV_PseudoVSSSEG7E8_V_M1_MASK	= 9736,
    RISCV_PseudoVSSSEG7E8_V_MF2	= 9737,
    RISCV_PseudoVSSSEG7E8_V_MF2_MASK	= 9738,
    RISCV_PseudoVSSSEG7E8_V_MF4	= 9739,
    RISCV_PseudoVSSSEG7E8_V_MF4_MASK	= 9740,
    RISCV_PseudoVSSSEG7E8_V_MF8	= 9741,
    RISCV_PseudoVSSSEG7E8_V_MF8_MASK	= 9742,
    RISCV_PseudoVSSSEG8E16_V_M1	= 9743,
    RISCV_PseudoVSSSEG8E16_V_M1_MASK	= 9744,
    RISCV_PseudoVSSSEG8E16_V_MF2	= 9745,
    RISCV_PseudoVSSSEG8E16_V_MF2_MASK	= 9746,
    RISCV_PseudoVSSSEG8E16_V_MF4	= 9747,
    RISCV_PseudoVSSSEG8E16_V_MF4_MASK	= 9748,
    RISCV_PseudoVSSSEG8E32_V_M1	= 9749,
    RISCV_PseudoVSSSEG8E32_V_M1_MASK	= 9750,
    RISCV_PseudoVSSSEG8E32_V_MF2	= 9751,
    RISCV_PseudoVSSSEG8E32_V_MF2_MASK	= 9752,
    RISCV_PseudoVSSSEG8E64_V_M1	= 9753,
    RISCV_PseudoVSSSEG8E64_V_M1_MASK	= 9754,
    RISCV_PseudoVSSSEG8E8_V_M1	= 9755,
    RISCV_PseudoVSSSEG8E8_V_M1_MASK	= 9756,
    RISCV_PseudoVSSSEG8E8_V_MF2	= 9757,
    RISCV_PseudoVSSSEG8E8_V_MF2_MASK	= 9758,
    RISCV_PseudoVSSSEG8E8_V_MF4	= 9759,
    RISCV_PseudoVSSSEG8E8_V_MF4_MASK	= 9760,
    RISCV_PseudoVSSSEG8E8_V_MF8	= 9761,
    RISCV_PseudoVSSSEG8E8_V_MF8_MASK	= 9762,
    RISCV_PseudoVSSUBU_VV_M1	= 9763,
    RISCV_PseudoVSSUBU_VV_M1_MASK	= 9764,
    RISCV_PseudoVSSUBU_VV_M2	= 9765,
    RISCV_PseudoVSSUBU_VV_M2_MASK	= 9766,
    RISCV_PseudoVSSUBU_VV_M4	= 9767,
    RISCV_PseudoVSSUBU_VV_M4_MASK	= 9768,
    RISCV_PseudoVSSUBU_VV_M8	= 9769,
    RISCV_PseudoVSSUBU_VV_M8_MASK	= 9770,
    RISCV_PseudoVSSUBU_VV_MF2	= 9771,
    RISCV_PseudoVSSUBU_VV_MF2_MASK	= 9772,
    RISCV_PseudoVSSUBU_VV_MF4	= 9773,
    RISCV_PseudoVSSUBU_VV_MF4_MASK	= 9774,
    RISCV_PseudoVSSUBU_VV_MF8	= 9775,
    RISCV_PseudoVSSUBU_VV_MF8_MASK	= 9776,
    RISCV_PseudoVSSUBU_VX_M1	= 9777,
    RISCV_PseudoVSSUBU_VX_M1_MASK	= 9778,
    RISCV_PseudoVSSUBU_VX_M2	= 9779,
    RISCV_PseudoVSSUBU_VX_M2_MASK	= 9780,
    RISCV_PseudoVSSUBU_VX_M4	= 9781,
    RISCV_PseudoVSSUBU_VX_M4_MASK	= 9782,
    RISCV_PseudoVSSUBU_VX_M8	= 9783,
    RISCV_PseudoVSSUBU_VX_M8_MASK	= 9784,
    RISCV_PseudoVSSUBU_VX_MF2	= 9785,
    RISCV_PseudoVSSUBU_VX_MF2_MASK	= 9786,
    RISCV_PseudoVSSUBU_VX_MF4	= 9787,
    RISCV_PseudoVSSUBU_VX_MF4_MASK	= 9788,
    RISCV_PseudoVSSUBU_VX_MF8	= 9789,
    RISCV_PseudoVSSUBU_VX_MF8_MASK	= 9790,
    RISCV_PseudoVSSUB_VV_M1	= 9791,
    RISCV_PseudoVSSUB_VV_M1_MASK	= 9792,
    RISCV_PseudoVSSUB_VV_M2	= 9793,
    RISCV_PseudoVSSUB_VV_M2_MASK	= 9794,
    RISCV_PseudoVSSUB_VV_M4	= 9795,
    RISCV_PseudoVSSUB_VV_M4_MASK	= 9796,
    RISCV_PseudoVSSUB_VV_M8	= 9797,
    RISCV_PseudoVSSUB_VV_M8_MASK	= 9798,
    RISCV_PseudoVSSUB_VV_MF2	= 9799,
    RISCV_PseudoVSSUB_VV_MF2_MASK	= 9800,
    RISCV_PseudoVSSUB_VV_MF4	= 9801,
    RISCV_PseudoVSSUB_VV_MF4_MASK	= 9802,
    RISCV_PseudoVSSUB_VV_MF8	= 9803,
    RISCV_PseudoVSSUB_VV_MF8_MASK	= 9804,
    RISCV_PseudoVSSUB_VX_M1	= 9805,
    RISCV_PseudoVSSUB_VX_M1_MASK	= 9806,
    RISCV_PseudoVSSUB_VX_M2	= 9807,
    RISCV_PseudoVSSUB_VX_M2_MASK	= 9808,
    RISCV_PseudoVSSUB_VX_M4	= 9809,
    RISCV_PseudoVSSUB_VX_M4_MASK	= 9810,
    RISCV_PseudoVSSUB_VX_M8	= 9811,
    RISCV_PseudoVSSUB_VX_M8_MASK	= 9812,
    RISCV_PseudoVSSUB_VX_MF2	= 9813,
    RISCV_PseudoVSSUB_VX_MF2_MASK	= 9814,
    RISCV_PseudoVSSUB_VX_MF4	= 9815,
    RISCV_PseudoVSSUB_VX_MF4_MASK	= 9816,
    RISCV_PseudoVSSUB_VX_MF8	= 9817,
    RISCV_PseudoVSSUB_VX_MF8_MASK	= 9818,
    RISCV_PseudoVSUB_VV_M1	= 9819,
    RISCV_PseudoVSUB_VV_M1_MASK	= 9820,
    RISCV_PseudoVSUB_VV_M2	= 9821,
    RISCV_PseudoVSUB_VV_M2_MASK	= 9822,
    RISCV_PseudoVSUB_VV_M4	= 9823,
    RISCV_PseudoVSUB_VV_M4_MASK	= 9824,
    RISCV_PseudoVSUB_VV_M8	= 9825,
    RISCV_PseudoVSUB_VV_M8_MASK	= 9826,
    RISCV_PseudoVSUB_VV_MF2	= 9827,
    RISCV_PseudoVSUB_VV_MF2_MASK	= 9828,
    RISCV_PseudoVSUB_VV_MF4	= 9829,
    RISCV_PseudoVSUB_VV_MF4_MASK	= 9830,
    RISCV_PseudoVSUB_VV_MF8	= 9831,
    RISCV_PseudoVSUB_VV_MF8_MASK	= 9832,
    RISCV_PseudoVSUB_VX_M1	= 9833,
    RISCV_PseudoVSUB_VX_M1_MASK	= 9834,
    RISCV_PseudoVSUB_VX_M2	= 9835,
    RISCV_PseudoVSUB_VX_M2_MASK	= 9836,
    RISCV_PseudoVSUB_VX_M4	= 9837,
    RISCV_PseudoVSUB_VX_M4_MASK	= 9838,
    RISCV_PseudoVSUB_VX_M8	= 9839,
    RISCV_PseudoVSUB_VX_M8_MASK	= 9840,
    RISCV_PseudoVSUB_VX_MF2	= 9841,
    RISCV_PseudoVSUB_VX_MF2_MASK	= 9842,
    RISCV_PseudoVSUB_VX_MF4	= 9843,
    RISCV_PseudoVSUB_VX_MF4_MASK	= 9844,
    RISCV_PseudoVSUB_VX_MF8	= 9845,
    RISCV_PseudoVSUB_VX_MF8_MASK	= 9846,
    RISCV_PseudoVSUXEI16_V_M1_M1	= 9847,
    RISCV_PseudoVSUXEI16_V_M1_M1_MASK	= 9848,
    RISCV_PseudoVSUXEI16_V_M1_M2	= 9849,
    RISCV_PseudoVSUXEI16_V_M1_M2_MASK	= 9850,
    RISCV_PseudoVSUXEI16_V_M1_M4	= 9851,
    RISCV_PseudoVSUXEI16_V_M1_M4_MASK	= 9852,
    RISCV_PseudoVSUXEI16_V_M1_MF2	= 9853,
    RISCV_PseudoVSUXEI16_V_M1_MF2_MASK	= 9854,
    RISCV_PseudoVSUXEI16_V_M2_M1	= 9855,
    RISCV_PseudoVSUXEI16_V_M2_M1_MASK	= 9856,
    RISCV_PseudoVSUXEI16_V_M2_M2	= 9857,
    RISCV_PseudoVSUXEI16_V_M2_M2_MASK	= 9858,
    RISCV_PseudoVSUXEI16_V_M2_M4	= 9859,
    RISCV_PseudoVSUXEI16_V_M2_M4_MASK	= 9860,
    RISCV_PseudoVSUXEI16_V_M2_M8	= 9861,
    RISCV_PseudoVSUXEI16_V_M2_M8_MASK	= 9862,
    RISCV_PseudoVSUXEI16_V_M4_M2	= 9863,
    RISCV_PseudoVSUXEI16_V_M4_M2_MASK	= 9864,
    RISCV_PseudoVSUXEI16_V_M4_M4	= 9865,
    RISCV_PseudoVSUXEI16_V_M4_M4_MASK	= 9866,
    RISCV_PseudoVSUXEI16_V_M4_M8	= 9867,
    RISCV_PseudoVSUXEI16_V_M4_M8_MASK	= 9868,
    RISCV_PseudoVSUXEI16_V_M8_M4	= 9869,
    RISCV_PseudoVSUXEI16_V_M8_M4_MASK	= 9870,
    RISCV_PseudoVSUXEI16_V_M8_M8	= 9871,
    RISCV_PseudoVSUXEI16_V_M8_M8_MASK	= 9872,
    RISCV_PseudoVSUXEI16_V_MF2_M1	= 9873,
    RISCV_PseudoVSUXEI16_V_MF2_M1_MASK	= 9874,
    RISCV_PseudoVSUXEI16_V_MF2_M2	= 9875,
    RISCV_PseudoVSUXEI16_V_MF2_M2_MASK	= 9876,
    RISCV_PseudoVSUXEI16_V_MF2_MF2	= 9877,
    RISCV_PseudoVSUXEI16_V_MF2_MF2_MASK	= 9878,
    RISCV_PseudoVSUXEI16_V_MF2_MF4	= 9879,
    RISCV_PseudoVSUXEI16_V_MF2_MF4_MASK	= 9880,
    RISCV_PseudoVSUXEI16_V_MF4_M1	= 9881,
    RISCV_PseudoVSUXEI16_V_MF4_M1_MASK	= 9882,
    RISCV_PseudoVSUXEI16_V_MF4_MF2	= 9883,
    RISCV_PseudoVSUXEI16_V_MF4_MF2_MASK	= 9884,
    RISCV_PseudoVSUXEI16_V_MF4_MF4	= 9885,
    RISCV_PseudoVSUXEI16_V_MF4_MF4_MASK	= 9886,
    RISCV_PseudoVSUXEI16_V_MF4_MF8	= 9887,
    RISCV_PseudoVSUXEI16_V_MF4_MF8_MASK	= 9888,
    RISCV_PseudoVSUXEI32_V_M1_M1	= 9889,
    RISCV_PseudoVSUXEI32_V_M1_M1_MASK	= 9890,
    RISCV_PseudoVSUXEI32_V_M1_M2	= 9891,
    RISCV_PseudoVSUXEI32_V_M1_M2_MASK	= 9892,
    RISCV_PseudoVSUXEI32_V_M1_MF2	= 9893,
    RISCV_PseudoVSUXEI32_V_M1_MF2_MASK	= 9894,
    RISCV_PseudoVSUXEI32_V_M1_MF4	= 9895,
    RISCV_PseudoVSUXEI32_V_M1_MF4_MASK	= 9896,
    RISCV_PseudoVSUXEI32_V_M2_M1	= 9897,
    RISCV_PseudoVSUXEI32_V_M2_M1_MASK	= 9898,
    RISCV_PseudoVSUXEI32_V_M2_M2	= 9899,
    RISCV_PseudoVSUXEI32_V_M2_M2_MASK	= 9900,
    RISCV_PseudoVSUXEI32_V_M2_M4	= 9901,
    RISCV_PseudoVSUXEI32_V_M2_M4_MASK	= 9902,
    RISCV_PseudoVSUXEI32_V_M2_MF2	= 9903,
    RISCV_PseudoVSUXEI32_V_M2_MF2_MASK	= 9904,
    RISCV_PseudoVSUXEI32_V_M4_M1	= 9905,
    RISCV_PseudoVSUXEI32_V_M4_M1_MASK	= 9906,
    RISCV_PseudoVSUXEI32_V_M4_M2	= 9907,
    RISCV_PseudoVSUXEI32_V_M4_M2_MASK	= 9908,
    RISCV_PseudoVSUXEI32_V_M4_M4	= 9909,
    RISCV_PseudoVSUXEI32_V_M4_M4_MASK	= 9910,
    RISCV_PseudoVSUXEI32_V_M4_M8	= 9911,
    RISCV_PseudoVSUXEI32_V_M4_M8_MASK	= 9912,
    RISCV_PseudoVSUXEI32_V_M8_M2	= 9913,
    RISCV_PseudoVSUXEI32_V_M8_M2_MASK	= 9914,
    RISCV_PseudoVSUXEI32_V_M8_M4	= 9915,
    RISCV_PseudoVSUXEI32_V_M8_M4_MASK	= 9916,
    RISCV_PseudoVSUXEI32_V_M8_M8	= 9917,
    RISCV_PseudoVSUXEI32_V_M8_M8_MASK	= 9918,
    RISCV_PseudoVSUXEI32_V_MF2_M1	= 9919,
    RISCV_PseudoVSUXEI32_V_MF2_M1_MASK	= 9920,
    RISCV_PseudoVSUXEI32_V_MF2_MF2	= 9921,
    RISCV_PseudoVSUXEI32_V_MF2_MF2_MASK	= 9922,
    RISCV_PseudoVSUXEI32_V_MF2_MF4	= 9923,
    RISCV_PseudoVSUXEI32_V_MF2_MF4_MASK	= 9924,
    RISCV_PseudoVSUXEI32_V_MF2_MF8	= 9925,
    RISCV_PseudoVSUXEI32_V_MF2_MF8_MASK	= 9926,
    RISCV_PseudoVSUXEI64_V_M1_M1	= 9927,
    RISCV_PseudoVSUXEI64_V_M1_M1_MASK	= 9928,
    RISCV_PseudoVSUXEI64_V_M1_MF2	= 9929,
    RISCV_PseudoVSUXEI64_V_M1_MF2_MASK	= 9930,
    RISCV_PseudoVSUXEI64_V_M1_MF4	= 9931,
    RISCV_PseudoVSUXEI64_V_M1_MF4_MASK	= 9932,
    RISCV_PseudoVSUXEI64_V_M1_MF8	= 9933,
    RISCV_PseudoVSUXEI64_V_M1_MF8_MASK	= 9934,
    RISCV_PseudoVSUXEI64_V_M2_M1	= 9935,
    RISCV_PseudoVSUXEI64_V_M2_M1_MASK	= 9936,
    RISCV_PseudoVSUXEI64_V_M2_M2	= 9937,
    RISCV_PseudoVSUXEI64_V_M2_M2_MASK	= 9938,
    RISCV_PseudoVSUXEI64_V_M2_MF2	= 9939,
    RISCV_PseudoVSUXEI64_V_M2_MF2_MASK	= 9940,
    RISCV_PseudoVSUXEI64_V_M2_MF4	= 9941,
    RISCV_PseudoVSUXEI64_V_M2_MF4_MASK	= 9942,
    RISCV_PseudoVSUXEI64_V_M4_M1	= 9943,
    RISCV_PseudoVSUXEI64_V_M4_M1_MASK	= 9944,
    RISCV_PseudoVSUXEI64_V_M4_M2	= 9945,
    RISCV_PseudoVSUXEI64_V_M4_M2_MASK	= 9946,
    RISCV_PseudoVSUXEI64_V_M4_M4	= 9947,
    RISCV_PseudoVSUXEI64_V_M4_M4_MASK	= 9948,
    RISCV_PseudoVSUXEI64_V_M4_MF2	= 9949,
    RISCV_PseudoVSUXEI64_V_M4_MF2_MASK	= 9950,
    RISCV_PseudoVSUXEI64_V_M8_M1	= 9951,
    RISCV_PseudoVSUXEI64_V_M8_M1_MASK	= 9952,
    RISCV_PseudoVSUXEI64_V_M8_M2	= 9953,
    RISCV_PseudoVSUXEI64_V_M8_M2_MASK	= 9954,
    RISCV_PseudoVSUXEI64_V_M8_M4	= 9955,
    RISCV_PseudoVSUXEI64_V_M8_M4_MASK	= 9956,
    RISCV_PseudoVSUXEI64_V_M8_M8	= 9957,
    RISCV_PseudoVSUXEI64_V_M8_M8_MASK	= 9958,
    RISCV_PseudoVSUXEI8_V_M1_M1	= 9959,
    RISCV_PseudoVSUXEI8_V_M1_M1_MASK	= 9960,
    RISCV_PseudoVSUXEI8_V_M1_M2	= 9961,
    RISCV_PseudoVSUXEI8_V_M1_M2_MASK	= 9962,
    RISCV_PseudoVSUXEI8_V_M1_M4	= 9963,
    RISCV_PseudoVSUXEI8_V_M1_M4_MASK	= 9964,
    RISCV_PseudoVSUXEI8_V_M1_M8	= 9965,
    RISCV_PseudoVSUXEI8_V_M1_M8_MASK	= 9966,
    RISCV_PseudoVSUXEI8_V_M2_M2	= 9967,
    RISCV_PseudoVSUXEI8_V_M2_M2_MASK	= 9968,
    RISCV_PseudoVSUXEI8_V_M2_M4	= 9969,
    RISCV_PseudoVSUXEI8_V_M2_M4_MASK	= 9970,
    RISCV_PseudoVSUXEI8_V_M2_M8	= 9971,
    RISCV_PseudoVSUXEI8_V_M2_M8_MASK	= 9972,
    RISCV_PseudoVSUXEI8_V_M4_M4	= 9973,
    RISCV_PseudoVSUXEI8_V_M4_M4_MASK	= 9974,
    RISCV_PseudoVSUXEI8_V_M4_M8	= 9975,
    RISCV_PseudoVSUXEI8_V_M4_M8_MASK	= 9976,
    RISCV_PseudoVSUXEI8_V_M8_M8	= 9977,
    RISCV_PseudoVSUXEI8_V_M8_M8_MASK	= 9978,
    RISCV_PseudoVSUXEI8_V_MF2_M1	= 9979,
    RISCV_PseudoVSUXEI8_V_MF2_M1_MASK	= 9980,
    RISCV_PseudoVSUXEI8_V_MF2_M2	= 9981,
    RISCV_PseudoVSUXEI8_V_MF2_M2_MASK	= 9982,
    RISCV_PseudoVSUXEI8_V_MF2_M4	= 9983,
    RISCV_PseudoVSUXEI8_V_MF2_M4_MASK	= 9984,
    RISCV_PseudoVSUXEI8_V_MF2_MF2	= 9985,
    RISCV_PseudoVSUXEI8_V_MF2_MF2_MASK	= 9986,
    RISCV_PseudoVSUXEI8_V_MF4_M1	= 9987,
    RISCV_PseudoVSUXEI8_V_MF4_M1_MASK	= 9988,
    RISCV_PseudoVSUXEI8_V_MF4_M2	= 9989,
    RISCV_PseudoVSUXEI8_V_MF4_M2_MASK	= 9990,
    RISCV_PseudoVSUXEI8_V_MF4_MF2	= 9991,
    RISCV_PseudoVSUXEI8_V_MF4_MF2_MASK	= 9992,
    RISCV_PseudoVSUXEI8_V_MF4_MF4	= 9993,
    RISCV_PseudoVSUXEI8_V_MF4_MF4_MASK	= 9994,
    RISCV_PseudoVSUXEI8_V_MF8_M1	= 9995,
    RISCV_PseudoVSUXEI8_V_MF8_M1_MASK	= 9996,
    RISCV_PseudoVSUXEI8_V_MF8_MF2	= 9997,
    RISCV_PseudoVSUXEI8_V_MF8_MF2_MASK	= 9998,
    RISCV_PseudoVSUXEI8_V_MF8_MF4	= 9999,
    RISCV_PseudoVSUXEI8_V_MF8_MF4_MASK	= 10000,
    RISCV_PseudoVSUXEI8_V_MF8_MF8	= 10001,
    RISCV_PseudoVSUXEI8_V_MF8_MF8_MASK	= 10002,
    RISCV_PseudoVSUXSEG2EI16_V_M1_M1	= 10003,
    RISCV_PseudoVSUXSEG2EI16_V_M1_M1_MASK	= 10004,
    RISCV_PseudoVSUXSEG2EI16_V_M1_M2	= 10005,
    RISCV_PseudoVSUXSEG2EI16_V_M1_M2_MASK	= 10006,
    RISCV_PseudoVSUXSEG2EI16_V_M1_M4	= 10007,
    RISCV_PseudoVSUXSEG2EI16_V_M1_M4_MASK	= 10008,
    RISCV_PseudoVSUXSEG2EI16_V_M1_MF2	= 10009,
    RISCV_PseudoVSUXSEG2EI16_V_M1_MF2_MASK	= 10010,
    RISCV_PseudoVSUXSEG2EI16_V_M2_M1	= 10011,
    RISCV_PseudoVSUXSEG2EI16_V_M2_M1_MASK	= 10012,
    RISCV_PseudoVSUXSEG2EI16_V_M2_M2	= 10013,
    RISCV_PseudoVSUXSEG2EI16_V_M2_M2_MASK	= 10014,
    RISCV_PseudoVSUXSEG2EI16_V_M2_M4	= 10015,
    RISCV_PseudoVSUXSEG2EI16_V_M2_M4_MASK	= 10016,
    RISCV_PseudoVSUXSEG2EI16_V_M4_M2	= 10017,
    RISCV_PseudoVSUXSEG2EI16_V_M4_M2_MASK	= 10018,
    RISCV_PseudoVSUXSEG2EI16_V_M4_M4	= 10019,
    RISCV_PseudoVSUXSEG2EI16_V_M4_M4_MASK	= 10020,
    RISCV_PseudoVSUXSEG2EI16_V_M8_M4	= 10021,
    RISCV_PseudoVSUXSEG2EI16_V_M8_M4_MASK	= 10022,
    RISCV_PseudoVSUXSEG2EI16_V_MF2_M1	= 10023,
    RISCV_PseudoVSUXSEG2EI16_V_MF2_M1_MASK	= 10024,
    RISCV_PseudoVSUXSEG2EI16_V_MF2_M2	= 10025,
    RISCV_PseudoVSUXSEG2EI16_V_MF2_M2_MASK	= 10026,
    RISCV_PseudoVSUXSEG2EI16_V_MF2_MF2	= 10027,
    RISCV_PseudoVSUXSEG2EI16_V_MF2_MF2_MASK	= 10028,
    RISCV_PseudoVSUXSEG2EI16_V_MF2_MF4	= 10029,
    RISCV_PseudoVSUXSEG2EI16_V_MF2_MF4_MASK	= 10030,
    RISCV_PseudoVSUXSEG2EI16_V_MF4_M1	= 10031,
    RISCV_PseudoVSUXSEG2EI16_V_MF4_M1_MASK	= 10032,
    RISCV_PseudoVSUXSEG2EI16_V_MF4_MF2	= 10033,
    RISCV_PseudoVSUXSEG2EI16_V_MF4_MF2_MASK	= 10034,
    RISCV_PseudoVSUXSEG2EI16_V_MF4_MF4	= 10035,
    RISCV_PseudoVSUXSEG2EI16_V_MF4_MF4_MASK	= 10036,
    RISCV_PseudoVSUXSEG2EI16_V_MF4_MF8	= 10037,
    RISCV_PseudoVSUXSEG2EI16_V_MF4_MF8_MASK	= 10038,
    RISCV_PseudoVSUXSEG2EI32_V_M1_M1	= 10039,
    RISCV_PseudoVSUXSEG2EI32_V_M1_M1_MASK	= 10040,
    RISCV_PseudoVSUXSEG2EI32_V_M1_M2	= 10041,
    RISCV_PseudoVSUXSEG2EI32_V_M1_M2_MASK	= 10042,
    RISCV_PseudoVSUXSEG2EI32_V_M1_MF2	= 10043,
    RISCV_PseudoVSUXSEG2EI32_V_M1_MF2_MASK	= 10044,
    RISCV_PseudoVSUXSEG2EI32_V_M1_MF4	= 10045,
    RISCV_PseudoVSUXSEG2EI32_V_M1_MF4_MASK	= 10046,
    RISCV_PseudoVSUXSEG2EI32_V_M2_M1	= 10047,
    RISCV_PseudoVSUXSEG2EI32_V_M2_M1_MASK	= 10048,
    RISCV_PseudoVSUXSEG2EI32_V_M2_M2	= 10049,
    RISCV_PseudoVSUXSEG2EI32_V_M2_M2_MASK	= 10050,
    RISCV_PseudoVSUXSEG2EI32_V_M2_M4	= 10051,
    RISCV_PseudoVSUXSEG2EI32_V_M2_M4_MASK	= 10052,
    RISCV_PseudoVSUXSEG2EI32_V_M2_MF2	= 10053,
    RISCV_PseudoVSUXSEG2EI32_V_M2_MF2_MASK	= 10054,
    RISCV_PseudoVSUXSEG2EI32_V_M4_M1	= 10055,
    RISCV_PseudoVSUXSEG2EI32_V_M4_M1_MASK	= 10056,
    RISCV_PseudoVSUXSEG2EI32_V_M4_M2	= 10057,
    RISCV_PseudoVSUXSEG2EI32_V_M4_M2_MASK	= 10058,
    RISCV_PseudoVSUXSEG2EI32_V_M4_M4	= 10059,
    RISCV_PseudoVSUXSEG2EI32_V_M4_M4_MASK	= 10060,
    RISCV_PseudoVSUXSEG2EI32_V_M8_M2	= 10061,
    RISCV_PseudoVSUXSEG2EI32_V_M8_M2_MASK	= 10062,
    RISCV_PseudoVSUXSEG2EI32_V_M8_M4	= 10063,
    RISCV_PseudoVSUXSEG2EI32_V_M8_M4_MASK	= 10064,
    RISCV_PseudoVSUXSEG2EI32_V_MF2_M1	= 10065,
    RISCV_PseudoVSUXSEG2EI32_V_MF2_M1_MASK	= 10066,
    RISCV_PseudoVSUXSEG2EI32_V_MF2_MF2	= 10067,
    RISCV_PseudoVSUXSEG2EI32_V_MF2_MF2_MASK	= 10068,
    RISCV_PseudoVSUXSEG2EI32_V_MF2_MF4	= 10069,
    RISCV_PseudoVSUXSEG2EI32_V_MF2_MF4_MASK	= 10070,
    RISCV_PseudoVSUXSEG2EI32_V_MF2_MF8	= 10071,
    RISCV_PseudoVSUXSEG2EI32_V_MF2_MF8_MASK	= 10072,
    RISCV_PseudoVSUXSEG2EI64_V_M1_M1	= 10073,
    RISCV_PseudoVSUXSEG2EI64_V_M1_M1_MASK	= 10074,
    RISCV_PseudoVSUXSEG2EI64_V_M1_MF2	= 10075,
    RISCV_PseudoVSUXSEG2EI64_V_M1_MF2_MASK	= 10076,
    RISCV_PseudoVSUXSEG2EI64_V_M1_MF4	= 10077,
    RISCV_PseudoVSUXSEG2EI64_V_M1_MF4_MASK	= 10078,
    RISCV_PseudoVSUXSEG2EI64_V_M1_MF8	= 10079,
    RISCV_PseudoVSUXSEG2EI64_V_M1_MF8_MASK	= 10080,
    RISCV_PseudoVSUXSEG2EI64_V_M2_M1	= 10081,
    RISCV_PseudoVSUXSEG2EI64_V_M2_M1_MASK	= 10082,
    RISCV_PseudoVSUXSEG2EI64_V_M2_M2	= 10083,
    RISCV_PseudoVSUXSEG2EI64_V_M2_M2_MASK	= 10084,
    RISCV_PseudoVSUXSEG2EI64_V_M2_MF2	= 10085,
    RISCV_PseudoVSUXSEG2EI64_V_M2_MF2_MASK	= 10086,
    RISCV_PseudoVSUXSEG2EI64_V_M2_MF4	= 10087,
    RISCV_PseudoVSUXSEG2EI64_V_M2_MF4_MASK	= 10088,
    RISCV_PseudoVSUXSEG2EI64_V_M4_M1	= 10089,
    RISCV_PseudoVSUXSEG2EI64_V_M4_M1_MASK	= 10090,
    RISCV_PseudoVSUXSEG2EI64_V_M4_M2	= 10091,
    RISCV_PseudoVSUXSEG2EI64_V_M4_M2_MASK	= 10092,
    RISCV_PseudoVSUXSEG2EI64_V_M4_M4	= 10093,
    RISCV_PseudoVSUXSEG2EI64_V_M4_M4_MASK	= 10094,
    RISCV_PseudoVSUXSEG2EI64_V_M4_MF2	= 10095,
    RISCV_PseudoVSUXSEG2EI64_V_M4_MF2_MASK	= 10096,
    RISCV_PseudoVSUXSEG2EI64_V_M8_M1	= 10097,
    RISCV_PseudoVSUXSEG2EI64_V_M8_M1_MASK	= 10098,
    RISCV_PseudoVSUXSEG2EI64_V_M8_M2	= 10099,
    RISCV_PseudoVSUXSEG2EI64_V_M8_M2_MASK	= 10100,
    RISCV_PseudoVSUXSEG2EI64_V_M8_M4	= 10101,
    RISCV_PseudoVSUXSEG2EI64_V_M8_M4_MASK	= 10102,
    RISCV_PseudoVSUXSEG2EI8_V_M1_M1	= 10103,
    RISCV_PseudoVSUXSEG2EI8_V_M1_M1_MASK	= 10104,
    RISCV_PseudoVSUXSEG2EI8_V_M1_M2	= 10105,
    RISCV_PseudoVSUXSEG2EI8_V_M1_M2_MASK	= 10106,
    RISCV_PseudoVSUXSEG2EI8_V_M1_M4	= 10107,
    RISCV_PseudoVSUXSEG2EI8_V_M1_M4_MASK	= 10108,
    RISCV_PseudoVSUXSEG2EI8_V_M2_M2	= 10109,
    RISCV_PseudoVSUXSEG2EI8_V_M2_M2_MASK	= 10110,
    RISCV_PseudoVSUXSEG2EI8_V_M2_M4	= 10111,
    RISCV_PseudoVSUXSEG2EI8_V_M2_M4_MASK	= 10112,
    RISCV_PseudoVSUXSEG2EI8_V_M4_M4	= 10113,
    RISCV_PseudoVSUXSEG2EI8_V_M4_M4_MASK	= 10114,
    RISCV_PseudoVSUXSEG2EI8_V_MF2_M1	= 10115,
    RISCV_PseudoVSUXSEG2EI8_V_MF2_M1_MASK	= 10116,
    RISCV_PseudoVSUXSEG2EI8_V_MF2_M2	= 10117,
    RISCV_PseudoVSUXSEG2EI8_V_MF2_M2_MASK	= 10118,
    RISCV_PseudoVSUXSEG2EI8_V_MF2_M4	= 10119,
    RISCV_PseudoVSUXSEG2EI8_V_MF2_M4_MASK	= 10120,
    RISCV_PseudoVSUXSEG2EI8_V_MF2_MF2	= 10121,
    RISCV_PseudoVSUXSEG2EI8_V_MF2_MF2_MASK	= 10122,
    RISCV_PseudoVSUXSEG2EI8_V_MF4_M1	= 10123,
    RISCV_PseudoVSUXSEG2EI8_V_MF4_M1_MASK	= 10124,
    RISCV_PseudoVSUXSEG2EI8_V_MF4_M2	= 10125,
    RISCV_PseudoVSUXSEG2EI8_V_MF4_M2_MASK	= 10126,
    RISCV_PseudoVSUXSEG2EI8_V_MF4_MF2	= 10127,
    RISCV_PseudoVSUXSEG2EI8_V_MF4_MF2_MASK	= 10128,
    RISCV_PseudoVSUXSEG2EI8_V_MF4_MF4	= 10129,
    RISCV_PseudoVSUXSEG2EI8_V_MF4_MF4_MASK	= 10130,
    RISCV_PseudoVSUXSEG2EI8_V_MF8_M1	= 10131,
    RISCV_PseudoVSUXSEG2EI8_V_MF8_M1_MASK	= 10132,
    RISCV_PseudoVSUXSEG2EI8_V_MF8_MF2	= 10133,
    RISCV_PseudoVSUXSEG2EI8_V_MF8_MF2_MASK	= 10134,
    RISCV_PseudoVSUXSEG2EI8_V_MF8_MF4	= 10135,
    RISCV_PseudoVSUXSEG2EI8_V_MF8_MF4_MASK	= 10136,
    RISCV_PseudoVSUXSEG2EI8_V_MF8_MF8	= 10137,
    RISCV_PseudoVSUXSEG2EI8_V_MF8_MF8_MASK	= 10138,
    RISCV_PseudoVSUXSEG3EI16_V_M1_M1	= 10139,
    RISCV_PseudoVSUXSEG3EI16_V_M1_M1_MASK	= 10140,
    RISCV_PseudoVSUXSEG3EI16_V_M1_M2	= 10141,
    RISCV_PseudoVSUXSEG3EI16_V_M1_M2_MASK	= 10142,
    RISCV_PseudoVSUXSEG3EI16_V_M1_MF2	= 10143,
    RISCV_PseudoVSUXSEG3EI16_V_M1_MF2_MASK	= 10144,
    RISCV_PseudoVSUXSEG3EI16_V_M2_M1	= 10145,
    RISCV_PseudoVSUXSEG3EI16_V_M2_M1_MASK	= 10146,
    RISCV_PseudoVSUXSEG3EI16_V_M2_M2	= 10147,
    RISCV_PseudoVSUXSEG3EI16_V_M2_M2_MASK	= 10148,
    RISCV_PseudoVSUXSEG3EI16_V_M4_M2	= 10149,
    RISCV_PseudoVSUXSEG3EI16_V_M4_M2_MASK	= 10150,
    RISCV_PseudoVSUXSEG3EI16_V_MF2_M1	= 10151,
    RISCV_PseudoVSUXSEG3EI16_V_MF2_M1_MASK	= 10152,
    RISCV_PseudoVSUXSEG3EI16_V_MF2_M2	= 10153,
    RISCV_PseudoVSUXSEG3EI16_V_MF2_M2_MASK	= 10154,
    RISCV_PseudoVSUXSEG3EI16_V_MF2_MF2	= 10155,
    RISCV_PseudoVSUXSEG3EI16_V_MF2_MF2_MASK	= 10156,
    RISCV_PseudoVSUXSEG3EI16_V_MF2_MF4	= 10157,
    RISCV_PseudoVSUXSEG3EI16_V_MF2_MF4_MASK	= 10158,
    RISCV_PseudoVSUXSEG3EI16_V_MF4_M1	= 10159,
    RISCV_PseudoVSUXSEG3EI16_V_MF4_M1_MASK	= 10160,
    RISCV_PseudoVSUXSEG3EI16_V_MF4_MF2	= 10161,
    RISCV_PseudoVSUXSEG3EI16_V_MF4_MF2_MASK	= 10162,
    RISCV_PseudoVSUXSEG3EI16_V_MF4_MF4	= 10163,
    RISCV_PseudoVSUXSEG3EI16_V_MF4_MF4_MASK	= 10164,
    RISCV_PseudoVSUXSEG3EI16_V_MF4_MF8	= 10165,
    RISCV_PseudoVSUXSEG3EI16_V_MF4_MF8_MASK	= 10166,
    RISCV_PseudoVSUXSEG3EI32_V_M1_M1	= 10167,
    RISCV_PseudoVSUXSEG3EI32_V_M1_M1_MASK	= 10168,
    RISCV_PseudoVSUXSEG3EI32_V_M1_M2	= 10169,
    RISCV_PseudoVSUXSEG3EI32_V_M1_M2_MASK	= 10170,
    RISCV_PseudoVSUXSEG3EI32_V_M1_MF2	= 10171,
    RISCV_PseudoVSUXSEG3EI32_V_M1_MF2_MASK	= 10172,
    RISCV_PseudoVSUXSEG3EI32_V_M1_MF4	= 10173,
    RISCV_PseudoVSUXSEG3EI32_V_M1_MF4_MASK	= 10174,
    RISCV_PseudoVSUXSEG3EI32_V_M2_M1	= 10175,
    RISCV_PseudoVSUXSEG3EI32_V_M2_M1_MASK	= 10176,
    RISCV_PseudoVSUXSEG3EI32_V_M2_M2	= 10177,
    RISCV_PseudoVSUXSEG3EI32_V_M2_M2_MASK	= 10178,
    RISCV_PseudoVSUXSEG3EI32_V_M2_MF2	= 10179,
    RISCV_PseudoVSUXSEG3EI32_V_M2_MF2_MASK	= 10180,
    RISCV_PseudoVSUXSEG3EI32_V_M4_M1	= 10181,
    RISCV_PseudoVSUXSEG3EI32_V_M4_M1_MASK	= 10182,
    RISCV_PseudoVSUXSEG3EI32_V_M4_M2	= 10183,
    RISCV_PseudoVSUXSEG3EI32_V_M4_M2_MASK	= 10184,
    RISCV_PseudoVSUXSEG3EI32_V_M8_M2	= 10185,
    RISCV_PseudoVSUXSEG3EI32_V_M8_M2_MASK	= 10186,
    RISCV_PseudoVSUXSEG3EI32_V_MF2_M1	= 10187,
    RISCV_PseudoVSUXSEG3EI32_V_MF2_M1_MASK	= 10188,
    RISCV_PseudoVSUXSEG3EI32_V_MF2_MF2	= 10189,
    RISCV_PseudoVSUXSEG3EI32_V_MF2_MF2_MASK	= 10190,
    RISCV_PseudoVSUXSEG3EI32_V_MF2_MF4	= 10191,
    RISCV_PseudoVSUXSEG3EI32_V_MF2_MF4_MASK	= 10192,
    RISCV_PseudoVSUXSEG3EI32_V_MF2_MF8	= 10193,
    RISCV_PseudoVSUXSEG3EI32_V_MF2_MF8_MASK	= 10194,
    RISCV_PseudoVSUXSEG3EI64_V_M1_M1	= 10195,
    RISCV_PseudoVSUXSEG3EI64_V_M1_M1_MASK	= 10196,
    RISCV_PseudoVSUXSEG3EI64_V_M1_MF2	= 10197,
    RISCV_PseudoVSUXSEG3EI64_V_M1_MF2_MASK	= 10198,
    RISCV_PseudoVSUXSEG3EI64_V_M1_MF4	= 10199,
    RISCV_PseudoVSUXSEG3EI64_V_M1_MF4_MASK	= 10200,
    RISCV_PseudoVSUXSEG3EI64_V_M1_MF8	= 10201,
    RISCV_PseudoVSUXSEG3EI64_V_M1_MF8_MASK	= 10202,
    RISCV_PseudoVSUXSEG3EI64_V_M2_M1	= 10203,
    RISCV_PseudoVSUXSEG3EI64_V_M2_M1_MASK	= 10204,
    RISCV_PseudoVSUXSEG3EI64_V_M2_M2	= 10205,
    RISCV_PseudoVSUXSEG3EI64_V_M2_M2_MASK	= 10206,
    RISCV_PseudoVSUXSEG3EI64_V_M2_MF2	= 10207,
    RISCV_PseudoVSUXSEG3EI64_V_M2_MF2_MASK	= 10208,
    RISCV_PseudoVSUXSEG3EI64_V_M2_MF4	= 10209,
    RISCV_PseudoVSUXSEG3EI64_V_M2_MF4_MASK	= 10210,
    RISCV_PseudoVSUXSEG3EI64_V_M4_M1	= 10211,
    RISCV_PseudoVSUXSEG3EI64_V_M4_M1_MASK	= 10212,
    RISCV_PseudoVSUXSEG3EI64_V_M4_M2	= 10213,
    RISCV_PseudoVSUXSEG3EI64_V_M4_M2_MASK	= 10214,
    RISCV_PseudoVSUXSEG3EI64_V_M4_MF2	= 10215,
    RISCV_PseudoVSUXSEG3EI64_V_M4_MF2_MASK	= 10216,
    RISCV_PseudoVSUXSEG3EI64_V_M8_M1	= 10217,
    RISCV_PseudoVSUXSEG3EI64_V_M8_M1_MASK	= 10218,
    RISCV_PseudoVSUXSEG3EI64_V_M8_M2	= 10219,
    RISCV_PseudoVSUXSEG3EI64_V_M8_M2_MASK	= 10220,
    RISCV_PseudoVSUXSEG3EI8_V_M1_M1	= 10221,
    RISCV_PseudoVSUXSEG3EI8_V_M1_M1_MASK	= 10222,
    RISCV_PseudoVSUXSEG3EI8_V_M1_M2	= 10223,
    RISCV_PseudoVSUXSEG3EI8_V_M1_M2_MASK	= 10224,
    RISCV_PseudoVSUXSEG3EI8_V_M2_M2	= 10225,
    RISCV_PseudoVSUXSEG3EI8_V_M2_M2_MASK	= 10226,
    RISCV_PseudoVSUXSEG3EI8_V_MF2_M1	= 10227,
    RISCV_PseudoVSUXSEG3EI8_V_MF2_M1_MASK	= 10228,
    RISCV_PseudoVSUXSEG3EI8_V_MF2_M2	= 10229,
    RISCV_PseudoVSUXSEG3EI8_V_MF2_M2_MASK	= 10230,
    RISCV_PseudoVSUXSEG3EI8_V_MF2_MF2	= 10231,
    RISCV_PseudoVSUXSEG3EI8_V_MF2_MF2_MASK	= 10232,
    RISCV_PseudoVSUXSEG3EI8_V_MF4_M1	= 10233,
    RISCV_PseudoVSUXSEG3EI8_V_MF4_M1_MASK	= 10234,
    RISCV_PseudoVSUXSEG3EI8_V_MF4_M2	= 10235,
    RISCV_PseudoVSUXSEG3EI8_V_MF4_M2_MASK	= 10236,
    RISCV_PseudoVSUXSEG3EI8_V_MF4_MF2	= 10237,
    RISCV_PseudoVSUXSEG3EI8_V_MF4_MF2_MASK	= 10238,
    RISCV_PseudoVSUXSEG3EI8_V_MF4_MF4	= 10239,
    RISCV_PseudoVSUXSEG3EI8_V_MF4_MF4_MASK	= 10240,
    RISCV_PseudoVSUXSEG3EI8_V_MF8_M1	= 10241,
    RISCV_PseudoVSUXSEG3EI8_V_MF8_M1_MASK	= 10242,
    RISCV_PseudoVSUXSEG3EI8_V_MF8_MF2	= 10243,
    RISCV_PseudoVSUXSEG3EI8_V_MF8_MF2_MASK	= 10244,
    RISCV_PseudoVSUXSEG3EI8_V_MF8_MF4	= 10245,
    RISCV_PseudoVSUXSEG3EI8_V_MF8_MF4_MASK	= 10246,
    RISCV_PseudoVSUXSEG3EI8_V_MF8_MF8	= 10247,
    RISCV_PseudoVSUXSEG3EI8_V_MF8_MF8_MASK	= 10248,
    RISCV_PseudoVSUXSEG4EI16_V_M1_M1	= 10249,
    RISCV_PseudoVSUXSEG4EI16_V_M1_M1_MASK	= 10250,
    RISCV_PseudoVSUXSEG4EI16_V_M1_M2	= 10251,
    RISCV_PseudoVSUXSEG4EI16_V_M1_M2_MASK	= 10252,
    RISCV_PseudoVSUXSEG4EI16_V_M1_MF2	= 10253,
    RISCV_PseudoVSUXSEG4EI16_V_M1_MF2_MASK	= 10254,
    RISCV_PseudoVSUXSEG4EI16_V_M2_M1	= 10255,
    RISCV_PseudoVSUXSEG4EI16_V_M2_M1_MASK	= 10256,
    RISCV_PseudoVSUXSEG4EI16_V_M2_M2	= 10257,
    RISCV_PseudoVSUXSEG4EI16_V_M2_M2_MASK	= 10258,
    RISCV_PseudoVSUXSEG4EI16_V_M4_M2	= 10259,
    RISCV_PseudoVSUXSEG4EI16_V_M4_M2_MASK	= 10260,
    RISCV_PseudoVSUXSEG4EI16_V_MF2_M1	= 10261,
    RISCV_PseudoVSUXSEG4EI16_V_MF2_M1_MASK	= 10262,
    RISCV_PseudoVSUXSEG4EI16_V_MF2_M2	= 10263,
    RISCV_PseudoVSUXSEG4EI16_V_MF2_M2_MASK	= 10264,
    RISCV_PseudoVSUXSEG4EI16_V_MF2_MF2	= 10265,
    RISCV_PseudoVSUXSEG4EI16_V_MF2_MF2_MASK	= 10266,
    RISCV_PseudoVSUXSEG4EI16_V_MF2_MF4	= 10267,
    RISCV_PseudoVSUXSEG4EI16_V_MF2_MF4_MASK	= 10268,
    RISCV_PseudoVSUXSEG4EI16_V_MF4_M1	= 10269,
    RISCV_PseudoVSUXSEG4EI16_V_MF4_M1_MASK	= 10270,
    RISCV_PseudoVSUXSEG4EI16_V_MF4_MF2	= 10271,
    RISCV_PseudoVSUXSEG4EI16_V_MF4_MF2_MASK	= 10272,
    RISCV_PseudoVSUXSEG4EI16_V_MF4_MF4	= 10273,
    RISCV_PseudoVSUXSEG4EI16_V_MF4_MF4_MASK	= 10274,
    RISCV_PseudoVSUXSEG4EI16_V_MF4_MF8	= 10275,
    RISCV_PseudoVSUXSEG4EI16_V_MF4_MF8_MASK	= 10276,
    RISCV_PseudoVSUXSEG4EI32_V_M1_M1	= 10277,
    RISCV_PseudoVSUXSEG4EI32_V_M1_M1_MASK	= 10278,
    RISCV_PseudoVSUXSEG4EI32_V_M1_M2	= 10279,
    RISCV_PseudoVSUXSEG4EI32_V_M1_M2_MASK	= 10280,
    RISCV_PseudoVSUXSEG4EI32_V_M1_MF2	= 10281,
    RISCV_PseudoVSUXSEG4EI32_V_M1_MF2_MASK	= 10282,
    RISCV_PseudoVSUXSEG4EI32_V_M1_MF4	= 10283,
    RISCV_PseudoVSUXSEG4EI32_V_M1_MF4_MASK	= 10284,
    RISCV_PseudoVSUXSEG4EI32_V_M2_M1	= 10285,
    RISCV_PseudoVSUXSEG4EI32_V_M2_M1_MASK	= 10286,
    RISCV_PseudoVSUXSEG4EI32_V_M2_M2	= 10287,
    RISCV_PseudoVSUXSEG4EI32_V_M2_M2_MASK	= 10288,
    RISCV_PseudoVSUXSEG4EI32_V_M2_MF2	= 10289,
    RISCV_PseudoVSUXSEG4EI32_V_M2_MF2_MASK	= 10290,
    RISCV_PseudoVSUXSEG4EI32_V_M4_M1	= 10291,
    RISCV_PseudoVSUXSEG4EI32_V_M4_M1_MASK	= 10292,
    RISCV_PseudoVSUXSEG4EI32_V_M4_M2	= 10293,
    RISCV_PseudoVSUXSEG4EI32_V_M4_M2_MASK	= 10294,
    RISCV_PseudoVSUXSEG4EI32_V_M8_M2	= 10295,
    RISCV_PseudoVSUXSEG4EI32_V_M8_M2_MASK	= 10296,
    RISCV_PseudoVSUXSEG4EI32_V_MF2_M1	= 10297,
    RISCV_PseudoVSUXSEG4EI32_V_MF2_M1_MASK	= 10298,
    RISCV_PseudoVSUXSEG4EI32_V_MF2_MF2	= 10299,
    RISCV_PseudoVSUXSEG4EI32_V_MF2_MF2_MASK	= 10300,
    RISCV_PseudoVSUXSEG4EI32_V_MF2_MF4	= 10301,
    RISCV_PseudoVSUXSEG4EI32_V_MF2_MF4_MASK	= 10302,
    RISCV_PseudoVSUXSEG4EI32_V_MF2_MF8	= 10303,
    RISCV_PseudoVSUXSEG4EI32_V_MF2_MF8_MASK	= 10304,
    RISCV_PseudoVSUXSEG4EI64_V_M1_M1	= 10305,
    RISCV_PseudoVSUXSEG4EI64_V_M1_M1_MASK	= 10306,
    RISCV_PseudoVSUXSEG4EI64_V_M1_MF2	= 10307,
    RISCV_PseudoVSUXSEG4EI64_V_M1_MF2_MASK	= 10308,
    RISCV_PseudoVSUXSEG4EI64_V_M1_MF4	= 10309,
    RISCV_PseudoVSUXSEG4EI64_V_M1_MF4_MASK	= 10310,
    RISCV_PseudoVSUXSEG4EI64_V_M1_MF8	= 10311,
    RISCV_PseudoVSUXSEG4EI64_V_M1_MF8_MASK	= 10312,
    RISCV_PseudoVSUXSEG4EI64_V_M2_M1	= 10313,
    RISCV_PseudoVSUXSEG4EI64_V_M2_M1_MASK	= 10314,
    RISCV_PseudoVSUXSEG4EI64_V_M2_M2	= 10315,
    RISCV_PseudoVSUXSEG4EI64_V_M2_M2_MASK	= 10316,
    RISCV_PseudoVSUXSEG4EI64_V_M2_MF2	= 10317,
    RISCV_PseudoVSUXSEG4EI64_V_M2_MF2_MASK	= 10318,
    RISCV_PseudoVSUXSEG4EI64_V_M2_MF4	= 10319,
    RISCV_PseudoVSUXSEG4EI64_V_M2_MF4_MASK	= 10320,
    RISCV_PseudoVSUXSEG4EI64_V_M4_M1	= 10321,
    RISCV_PseudoVSUXSEG4EI64_V_M4_M1_MASK	= 10322,
    RISCV_PseudoVSUXSEG4EI64_V_M4_M2	= 10323,
    RISCV_PseudoVSUXSEG4EI64_V_M4_M2_MASK	= 10324,
    RISCV_PseudoVSUXSEG4EI64_V_M4_MF2	= 10325,
    RISCV_PseudoVSUXSEG4EI64_V_M4_MF2_MASK	= 10326,
    RISCV_PseudoVSUXSEG4EI64_V_M8_M1	= 10327,
    RISCV_PseudoVSUXSEG4EI64_V_M8_M1_MASK	= 10328,
    RISCV_PseudoVSUXSEG4EI64_V_M8_M2	= 10329,
    RISCV_PseudoVSUXSEG4EI64_V_M8_M2_MASK	= 10330,
    RISCV_PseudoVSUXSEG4EI8_V_M1_M1	= 10331,
    RISCV_PseudoVSUXSEG4EI8_V_M1_M1_MASK	= 10332,
    RISCV_PseudoVSUXSEG4EI8_V_M1_M2	= 10333,
    RISCV_PseudoVSUXSEG4EI8_V_M1_M2_MASK	= 10334,
    RISCV_PseudoVSUXSEG4EI8_V_M2_M2	= 10335,
    RISCV_PseudoVSUXSEG4EI8_V_M2_M2_MASK	= 10336,
    RISCV_PseudoVSUXSEG4EI8_V_MF2_M1	= 10337,
    RISCV_PseudoVSUXSEG4EI8_V_MF2_M1_MASK	= 10338,
    RISCV_PseudoVSUXSEG4EI8_V_MF2_M2	= 10339,
    RISCV_PseudoVSUXSEG4EI8_V_MF2_M2_MASK	= 10340,
    RISCV_PseudoVSUXSEG4EI8_V_MF2_MF2	= 10341,
    RISCV_PseudoVSUXSEG4EI8_V_MF2_MF2_MASK	= 10342,
    RISCV_PseudoVSUXSEG4EI8_V_MF4_M1	= 10343,
    RISCV_PseudoVSUXSEG4EI8_V_MF4_M1_MASK	= 10344,
    RISCV_PseudoVSUXSEG4EI8_V_MF4_M2	= 10345,
    RISCV_PseudoVSUXSEG4EI8_V_MF4_M2_MASK	= 10346,
    RISCV_PseudoVSUXSEG4EI8_V_MF4_MF2	= 10347,
    RISCV_PseudoVSUXSEG4EI8_V_MF4_MF2_MASK	= 10348,
    RISCV_PseudoVSUXSEG4EI8_V_MF4_MF4	= 10349,
    RISCV_PseudoVSUXSEG4EI8_V_MF4_MF4_MASK	= 10350,
    RISCV_PseudoVSUXSEG4EI8_V_MF8_M1	= 10351,
    RISCV_PseudoVSUXSEG4EI8_V_MF8_M1_MASK	= 10352,
    RISCV_PseudoVSUXSEG4EI8_V_MF8_MF2	= 10353,
    RISCV_PseudoVSUXSEG4EI8_V_MF8_MF2_MASK	= 10354,
    RISCV_PseudoVSUXSEG4EI8_V_MF8_MF4	= 10355,
    RISCV_PseudoVSUXSEG4EI8_V_MF8_MF4_MASK	= 10356,
    RISCV_PseudoVSUXSEG4EI8_V_MF8_MF8	= 10357,
    RISCV_PseudoVSUXSEG4EI8_V_MF8_MF8_MASK	= 10358,
    RISCV_PseudoVSUXSEG5EI16_V_M1_M1	= 10359,
    RISCV_PseudoVSUXSEG5EI16_V_M1_M1_MASK	= 10360,
    RISCV_PseudoVSUXSEG5EI16_V_M1_MF2	= 10361,
    RISCV_PseudoVSUXSEG5EI16_V_M1_MF2_MASK	= 10362,
    RISCV_PseudoVSUXSEG5EI16_V_M2_M1	= 10363,
    RISCV_PseudoVSUXSEG5EI16_V_M2_M1_MASK	= 10364,
    RISCV_PseudoVSUXSEG5EI16_V_MF2_M1	= 10365,
    RISCV_PseudoVSUXSEG5EI16_V_MF2_M1_MASK	= 10366,
    RISCV_PseudoVSUXSEG5EI16_V_MF2_MF2	= 10367,
    RISCV_PseudoVSUXSEG5EI16_V_MF2_MF2_MASK	= 10368,
    RISCV_PseudoVSUXSEG5EI16_V_MF2_MF4	= 10369,
    RISCV_PseudoVSUXSEG5EI16_V_MF2_MF4_MASK	= 10370,
    RISCV_PseudoVSUXSEG5EI16_V_MF4_M1	= 10371,
    RISCV_PseudoVSUXSEG5EI16_V_MF4_M1_MASK	= 10372,
    RISCV_PseudoVSUXSEG5EI16_V_MF4_MF2	= 10373,
    RISCV_PseudoVSUXSEG5EI16_V_MF4_MF2_MASK	= 10374,
    RISCV_PseudoVSUXSEG5EI16_V_MF4_MF4	= 10375,
    RISCV_PseudoVSUXSEG5EI16_V_MF4_MF4_MASK	= 10376,
    RISCV_PseudoVSUXSEG5EI16_V_MF4_MF8	= 10377,
    RISCV_PseudoVSUXSEG5EI16_V_MF4_MF8_MASK	= 10378,
    RISCV_PseudoVSUXSEG5EI32_V_M1_M1	= 10379,
    RISCV_PseudoVSUXSEG5EI32_V_M1_M1_MASK	= 10380,
    RISCV_PseudoVSUXSEG5EI32_V_M1_MF2	= 10381,
    RISCV_PseudoVSUXSEG5EI32_V_M1_MF2_MASK	= 10382,
    RISCV_PseudoVSUXSEG5EI32_V_M1_MF4	= 10383,
    RISCV_PseudoVSUXSEG5EI32_V_M1_MF4_MASK	= 10384,
    RISCV_PseudoVSUXSEG5EI32_V_M2_M1	= 10385,
    RISCV_PseudoVSUXSEG5EI32_V_M2_M1_MASK	= 10386,
    RISCV_PseudoVSUXSEG5EI32_V_M2_MF2	= 10387,
    RISCV_PseudoVSUXSEG5EI32_V_M2_MF2_MASK	= 10388,
    RISCV_PseudoVSUXSEG5EI32_V_M4_M1	= 10389,
    RISCV_PseudoVSUXSEG5EI32_V_M4_M1_MASK	= 10390,
    RISCV_PseudoVSUXSEG5EI32_V_MF2_M1	= 10391,
    RISCV_PseudoVSUXSEG5EI32_V_MF2_M1_MASK	= 10392,
    RISCV_PseudoVSUXSEG5EI32_V_MF2_MF2	= 10393,
    RISCV_PseudoVSUXSEG5EI32_V_MF2_MF2_MASK	= 10394,
    RISCV_PseudoVSUXSEG5EI32_V_MF2_MF4	= 10395,
    RISCV_PseudoVSUXSEG5EI32_V_MF2_MF4_MASK	= 10396,
    RISCV_PseudoVSUXSEG5EI32_V_MF2_MF8	= 10397,
    RISCV_PseudoVSUXSEG5EI32_V_MF2_MF8_MASK	= 10398,
    RISCV_PseudoVSUXSEG5EI64_V_M1_M1	= 10399,
    RISCV_PseudoVSUXSEG5EI64_V_M1_M1_MASK	= 10400,
    RISCV_PseudoVSUXSEG5EI64_V_M1_MF2	= 10401,
    RISCV_PseudoVSUXSEG5EI64_V_M1_MF2_MASK	= 10402,
    RISCV_PseudoVSUXSEG5EI64_V_M1_MF4	= 10403,
    RISCV_PseudoVSUXSEG5EI64_V_M1_MF4_MASK	= 10404,
    RISCV_PseudoVSUXSEG5EI64_V_M1_MF8	= 10405,
    RISCV_PseudoVSUXSEG5EI64_V_M1_MF8_MASK	= 10406,
    RISCV_PseudoVSUXSEG5EI64_V_M2_M1	= 10407,
    RISCV_PseudoVSUXSEG5EI64_V_M2_M1_MASK	= 10408,
    RISCV_PseudoVSUXSEG5EI64_V_M2_MF2	= 10409,
    RISCV_PseudoVSUXSEG5EI64_V_M2_MF2_MASK	= 10410,
    RISCV_PseudoVSUXSEG5EI64_V_M2_MF4	= 10411,
    RISCV_PseudoVSUXSEG5EI64_V_M2_MF4_MASK	= 10412,
    RISCV_PseudoVSUXSEG5EI64_V_M4_M1	= 10413,
    RISCV_PseudoVSUXSEG5EI64_V_M4_M1_MASK	= 10414,
    RISCV_PseudoVSUXSEG5EI64_V_M4_MF2	= 10415,
    RISCV_PseudoVSUXSEG5EI64_V_M4_MF2_MASK	= 10416,
    RISCV_PseudoVSUXSEG5EI64_V_M8_M1	= 10417,
    RISCV_PseudoVSUXSEG5EI64_V_M8_M1_MASK	= 10418,
    RISCV_PseudoVSUXSEG5EI8_V_M1_M1	= 10419,
    RISCV_PseudoVSUXSEG5EI8_V_M1_M1_MASK	= 10420,
    RISCV_PseudoVSUXSEG5EI8_V_MF2_M1	= 10421,
    RISCV_PseudoVSUXSEG5EI8_V_MF2_M1_MASK	= 10422,
    RISCV_PseudoVSUXSEG5EI8_V_MF2_MF2	= 10423,
    RISCV_PseudoVSUXSEG5EI8_V_MF2_MF2_MASK	= 10424,
    RISCV_PseudoVSUXSEG5EI8_V_MF4_M1	= 10425,
    RISCV_PseudoVSUXSEG5EI8_V_MF4_M1_MASK	= 10426,
    RISCV_PseudoVSUXSEG5EI8_V_MF4_MF2	= 10427,
    RISCV_PseudoVSUXSEG5EI8_V_MF4_MF2_MASK	= 10428,
    RISCV_PseudoVSUXSEG5EI8_V_MF4_MF4	= 10429,
    RISCV_PseudoVSUXSEG5EI8_V_MF4_MF4_MASK	= 10430,
    RISCV_PseudoVSUXSEG5EI8_V_MF8_M1	= 10431,
    RISCV_PseudoVSUXSEG5EI8_V_MF8_M1_MASK	= 10432,
    RISCV_PseudoVSUXSEG5EI8_V_MF8_MF2	= 10433,
    RISCV_PseudoVSUXSEG5EI8_V_MF8_MF2_MASK	= 10434,
    RISCV_PseudoVSUXSEG5EI8_V_MF8_MF4	= 10435,
    RISCV_PseudoVSUXSEG5EI8_V_MF8_MF4_MASK	= 10436,
    RISCV_PseudoVSUXSEG5EI8_V_MF8_MF8	= 10437,
    RISCV_PseudoVSUXSEG5EI8_V_MF8_MF8_MASK	= 10438,
    RISCV_PseudoVSUXSEG6EI16_V_M1_M1	= 10439,
    RISCV_PseudoVSUXSEG6EI16_V_M1_M1_MASK	= 10440,
    RISCV_PseudoVSUXSEG6EI16_V_M1_MF2	= 10441,
    RISCV_PseudoVSUXSEG6EI16_V_M1_MF2_MASK	= 10442,
    RISCV_PseudoVSUXSEG6EI16_V_M2_M1	= 10443,
    RISCV_PseudoVSUXSEG6EI16_V_M2_M1_MASK	= 10444,
    RISCV_PseudoVSUXSEG6EI16_V_MF2_M1	= 10445,
    RISCV_PseudoVSUXSEG6EI16_V_MF2_M1_MASK	= 10446,
    RISCV_PseudoVSUXSEG6EI16_V_MF2_MF2	= 10447,
    RISCV_PseudoVSUXSEG6EI16_V_MF2_MF2_MASK	= 10448,
    RISCV_PseudoVSUXSEG6EI16_V_MF2_MF4	= 10449,
    RISCV_PseudoVSUXSEG6EI16_V_MF2_MF4_MASK	= 10450,
    RISCV_PseudoVSUXSEG6EI16_V_MF4_M1	= 10451,
    RISCV_PseudoVSUXSEG6EI16_V_MF4_M1_MASK	= 10452,
    RISCV_PseudoVSUXSEG6EI16_V_MF4_MF2	= 10453,
    RISCV_PseudoVSUXSEG6EI16_V_MF4_MF2_MASK	= 10454,
    RISCV_PseudoVSUXSEG6EI16_V_MF4_MF4	= 10455,
    RISCV_PseudoVSUXSEG6EI16_V_MF4_MF4_MASK	= 10456,
    RISCV_PseudoVSUXSEG6EI16_V_MF4_MF8	= 10457,
    RISCV_PseudoVSUXSEG6EI16_V_MF4_MF8_MASK	= 10458,
    RISCV_PseudoVSUXSEG6EI32_V_M1_M1	= 10459,
    RISCV_PseudoVSUXSEG6EI32_V_M1_M1_MASK	= 10460,
    RISCV_PseudoVSUXSEG6EI32_V_M1_MF2	= 10461,
    RISCV_PseudoVSUXSEG6EI32_V_M1_MF2_MASK	= 10462,
    RISCV_PseudoVSUXSEG6EI32_V_M1_MF4	= 10463,
    RISCV_PseudoVSUXSEG6EI32_V_M1_MF4_MASK	= 10464,
    RISCV_PseudoVSUXSEG6EI32_V_M2_M1	= 10465,
    RISCV_PseudoVSUXSEG6EI32_V_M2_M1_MASK	= 10466,
    RISCV_PseudoVSUXSEG6EI32_V_M2_MF2	= 10467,
    RISCV_PseudoVSUXSEG6EI32_V_M2_MF2_MASK	= 10468,
    RISCV_PseudoVSUXSEG6EI32_V_M4_M1	= 10469,
    RISCV_PseudoVSUXSEG6EI32_V_M4_M1_MASK	= 10470,
    RISCV_PseudoVSUXSEG6EI32_V_MF2_M1	= 10471,
    RISCV_PseudoVSUXSEG6EI32_V_MF2_M1_MASK	= 10472,
    RISCV_PseudoVSUXSEG6EI32_V_MF2_MF2	= 10473,
    RISCV_PseudoVSUXSEG6EI32_V_MF2_MF2_MASK	= 10474,
    RISCV_PseudoVSUXSEG6EI32_V_MF2_MF4	= 10475,
    RISCV_PseudoVSUXSEG6EI32_V_MF2_MF4_MASK	= 10476,
    RISCV_PseudoVSUXSEG6EI32_V_MF2_MF8	= 10477,
    RISCV_PseudoVSUXSEG6EI32_V_MF2_MF8_MASK	= 10478,
    RISCV_PseudoVSUXSEG6EI64_V_M1_M1	= 10479,
    RISCV_PseudoVSUXSEG6EI64_V_M1_M1_MASK	= 10480,
    RISCV_PseudoVSUXSEG6EI64_V_M1_MF2	= 10481,
    RISCV_PseudoVSUXSEG6EI64_V_M1_MF2_MASK	= 10482,
    RISCV_PseudoVSUXSEG6EI64_V_M1_MF4	= 10483,
    RISCV_PseudoVSUXSEG6EI64_V_M1_MF4_MASK	= 10484,
    RISCV_PseudoVSUXSEG6EI64_V_M1_MF8	= 10485,
    RISCV_PseudoVSUXSEG6EI64_V_M1_MF8_MASK	= 10486,
    RISCV_PseudoVSUXSEG6EI64_V_M2_M1	= 10487,
    RISCV_PseudoVSUXSEG6EI64_V_M2_M1_MASK	= 10488,
    RISCV_PseudoVSUXSEG6EI64_V_M2_MF2	= 10489,
    RISCV_PseudoVSUXSEG6EI64_V_M2_MF2_MASK	= 10490,
    RISCV_PseudoVSUXSEG6EI64_V_M2_MF4	= 10491,
    RISCV_PseudoVSUXSEG6EI64_V_M2_MF4_MASK	= 10492,
    RISCV_PseudoVSUXSEG6EI64_V_M4_M1	= 10493,
    RISCV_PseudoVSUXSEG6EI64_V_M4_M1_MASK	= 10494,
    RISCV_PseudoVSUXSEG6EI64_V_M4_MF2	= 10495,
    RISCV_PseudoVSUXSEG6EI64_V_M4_MF2_MASK	= 10496,
    RISCV_PseudoVSUXSEG6EI64_V_M8_M1	= 10497,
    RISCV_PseudoVSUXSEG6EI64_V_M8_M1_MASK	= 10498,
    RISCV_PseudoVSUXSEG6EI8_V_M1_M1	= 10499,
    RISCV_PseudoVSUXSEG6EI8_V_M1_M1_MASK	= 10500,
    RISCV_PseudoVSUXSEG6EI8_V_MF2_M1	= 10501,
    RISCV_PseudoVSUXSEG6EI8_V_MF2_M1_MASK	= 10502,
    RISCV_PseudoVSUXSEG6EI8_V_MF2_MF2	= 10503,
    RISCV_PseudoVSUXSEG6EI8_V_MF2_MF2_MASK	= 10504,
    RISCV_PseudoVSUXSEG6EI8_V_MF4_M1	= 10505,
    RISCV_PseudoVSUXSEG6EI8_V_MF4_M1_MASK	= 10506,
    RISCV_PseudoVSUXSEG6EI8_V_MF4_MF2	= 10507,
    RISCV_PseudoVSUXSEG6EI8_V_MF4_MF2_MASK	= 10508,
    RISCV_PseudoVSUXSEG6EI8_V_MF4_MF4	= 10509,
    RISCV_PseudoVSUXSEG6EI8_V_MF4_MF4_MASK	= 10510,
    RISCV_PseudoVSUXSEG6EI8_V_MF8_M1	= 10511,
    RISCV_PseudoVSUXSEG6EI8_V_MF8_M1_MASK	= 10512,
    RISCV_PseudoVSUXSEG6EI8_V_MF8_MF2	= 10513,
    RISCV_PseudoVSUXSEG6EI8_V_MF8_MF2_MASK	= 10514,
    RISCV_PseudoVSUXSEG6EI8_V_MF8_MF4	= 10515,
    RISCV_PseudoVSUXSEG6EI8_V_MF8_MF4_MASK	= 10516,
    RISCV_PseudoVSUXSEG6EI8_V_MF8_MF8	= 10517,
    RISCV_PseudoVSUXSEG6EI8_V_MF8_MF8_MASK	= 10518,
    RISCV_PseudoVSUXSEG7EI16_V_M1_M1	= 10519,
    RISCV_PseudoVSUXSEG7EI16_V_M1_M1_MASK	= 10520,
    RISCV_PseudoVSUXSEG7EI16_V_M1_MF2	= 10521,
    RISCV_PseudoVSUXSEG7EI16_V_M1_MF2_MASK	= 10522,
    RISCV_PseudoVSUXSEG7EI16_V_M2_M1	= 10523,
    RISCV_PseudoVSUXSEG7EI16_V_M2_M1_MASK	= 10524,
    RISCV_PseudoVSUXSEG7EI16_V_MF2_M1	= 10525,
    RISCV_PseudoVSUXSEG7EI16_V_MF2_M1_MASK	= 10526,
    RISCV_PseudoVSUXSEG7EI16_V_MF2_MF2	= 10527,
    RISCV_PseudoVSUXSEG7EI16_V_MF2_MF2_MASK	= 10528,
    RISCV_PseudoVSUXSEG7EI16_V_MF2_MF4	= 10529,
    RISCV_PseudoVSUXSEG7EI16_V_MF2_MF4_MASK	= 10530,
    RISCV_PseudoVSUXSEG7EI16_V_MF4_M1	= 10531,
    RISCV_PseudoVSUXSEG7EI16_V_MF4_M1_MASK	= 10532,
    RISCV_PseudoVSUXSEG7EI16_V_MF4_MF2	= 10533,
    RISCV_PseudoVSUXSEG7EI16_V_MF4_MF2_MASK	= 10534,
    RISCV_PseudoVSUXSEG7EI16_V_MF4_MF4	= 10535,
    RISCV_PseudoVSUXSEG7EI16_V_MF4_MF4_MASK	= 10536,
    RISCV_PseudoVSUXSEG7EI16_V_MF4_MF8	= 10537,
    RISCV_PseudoVSUXSEG7EI16_V_MF4_MF8_MASK	= 10538,
    RISCV_PseudoVSUXSEG7EI32_V_M1_M1	= 10539,
    RISCV_PseudoVSUXSEG7EI32_V_M1_M1_MASK	= 10540,
    RISCV_PseudoVSUXSEG7EI32_V_M1_MF2	= 10541,
    RISCV_PseudoVSUXSEG7EI32_V_M1_MF2_MASK	= 10542,
    RISCV_PseudoVSUXSEG7EI32_V_M1_MF4	= 10543,
    RISCV_PseudoVSUXSEG7EI32_V_M1_MF4_MASK	= 10544,
    RISCV_PseudoVSUXSEG7EI32_V_M2_M1	= 10545,
    RISCV_PseudoVSUXSEG7EI32_V_M2_M1_MASK	= 10546,
    RISCV_PseudoVSUXSEG7EI32_V_M2_MF2	= 10547,
    RISCV_PseudoVSUXSEG7EI32_V_M2_MF2_MASK	= 10548,
    RISCV_PseudoVSUXSEG7EI32_V_M4_M1	= 10549,
    RISCV_PseudoVSUXSEG7EI32_V_M4_M1_MASK	= 10550,
    RISCV_PseudoVSUXSEG7EI32_V_MF2_M1	= 10551,
    RISCV_PseudoVSUXSEG7EI32_V_MF2_M1_MASK	= 10552,
    RISCV_PseudoVSUXSEG7EI32_V_MF2_MF2	= 10553,
    RISCV_PseudoVSUXSEG7EI32_V_MF2_MF2_MASK	= 10554,
    RISCV_PseudoVSUXSEG7EI32_V_MF2_MF4	= 10555,
    RISCV_PseudoVSUXSEG7EI32_V_MF2_MF4_MASK	= 10556,
    RISCV_PseudoVSUXSEG7EI32_V_MF2_MF8	= 10557,
    RISCV_PseudoVSUXSEG7EI32_V_MF2_MF8_MASK	= 10558,
    RISCV_PseudoVSUXSEG7EI64_V_M1_M1	= 10559,
    RISCV_PseudoVSUXSEG7EI64_V_M1_M1_MASK	= 10560,
    RISCV_PseudoVSUXSEG7EI64_V_M1_MF2	= 10561,
    RISCV_PseudoVSUXSEG7EI64_V_M1_MF2_MASK	= 10562,
    RISCV_PseudoVSUXSEG7EI64_V_M1_MF4	= 10563,
    RISCV_PseudoVSUXSEG7EI64_V_M1_MF4_MASK	= 10564,
    RISCV_PseudoVSUXSEG7EI64_V_M1_MF8	= 10565,
    RISCV_PseudoVSUXSEG7EI64_V_M1_MF8_MASK	= 10566,
    RISCV_PseudoVSUXSEG7EI64_V_M2_M1	= 10567,
    RISCV_PseudoVSUXSEG7EI64_V_M2_M1_MASK	= 10568,
    RISCV_PseudoVSUXSEG7EI64_V_M2_MF2	= 10569,
    RISCV_PseudoVSUXSEG7EI64_V_M2_MF2_MASK	= 10570,
    RISCV_PseudoVSUXSEG7EI64_V_M2_MF4	= 10571,
    RISCV_PseudoVSUXSEG7EI64_V_M2_MF4_MASK	= 10572,
    RISCV_PseudoVSUXSEG7EI64_V_M4_M1	= 10573,
    RISCV_PseudoVSUXSEG7EI64_V_M4_M1_MASK	= 10574,
    RISCV_PseudoVSUXSEG7EI64_V_M4_MF2	= 10575,
    RISCV_PseudoVSUXSEG7EI64_V_M4_MF2_MASK	= 10576,
    RISCV_PseudoVSUXSEG7EI64_V_M8_M1	= 10577,
    RISCV_PseudoVSUXSEG7EI64_V_M8_M1_MASK	= 10578,
    RISCV_PseudoVSUXSEG7EI8_V_M1_M1	= 10579,
    RISCV_PseudoVSUXSEG7EI8_V_M1_M1_MASK	= 10580,
    RISCV_PseudoVSUXSEG7EI8_V_MF2_M1	= 10581,
    RISCV_PseudoVSUXSEG7EI8_V_MF2_M1_MASK	= 10582,
    RISCV_PseudoVSUXSEG7EI8_V_MF2_MF2	= 10583,
    RISCV_PseudoVSUXSEG7EI8_V_MF2_MF2_MASK	= 10584,
    RISCV_PseudoVSUXSEG7EI8_V_MF4_M1	= 10585,
    RISCV_PseudoVSUXSEG7EI8_V_MF4_M1_MASK	= 10586,
    RISCV_PseudoVSUXSEG7EI8_V_MF4_MF2	= 10587,
    RISCV_PseudoVSUXSEG7EI8_V_MF4_MF2_MASK	= 10588,
    RISCV_PseudoVSUXSEG7EI8_V_MF4_MF4	= 10589,
    RISCV_PseudoVSUXSEG7EI8_V_MF4_MF4_MASK	= 10590,
    RISCV_PseudoVSUXSEG7EI8_V_MF8_M1	= 10591,
    RISCV_PseudoVSUXSEG7EI8_V_MF8_M1_MASK	= 10592,
    RISCV_PseudoVSUXSEG7EI8_V_MF8_MF2	= 10593,
    RISCV_PseudoVSUXSEG7EI8_V_MF8_MF2_MASK	= 10594,
    RISCV_PseudoVSUXSEG7EI8_V_MF8_MF4	= 10595,
    RISCV_PseudoVSUXSEG7EI8_V_MF8_MF4_MASK	= 10596,
    RISCV_PseudoVSUXSEG7EI8_V_MF8_MF8	= 10597,
    RISCV_PseudoVSUXSEG7EI8_V_MF8_MF8_MASK	= 10598,
    RISCV_PseudoVSUXSEG8EI16_V_M1_M1	= 10599,
    RISCV_PseudoVSUXSEG8EI16_V_M1_M1_MASK	= 10600,
    RISCV_PseudoVSUXSEG8EI16_V_M1_MF2	= 10601,
    RISCV_PseudoVSUXSEG8EI16_V_M1_MF2_MASK	= 10602,
    RISCV_PseudoVSUXSEG8EI16_V_M2_M1	= 10603,
    RISCV_PseudoVSUXSEG8EI16_V_M2_M1_MASK	= 10604,
    RISCV_PseudoVSUXSEG8EI16_V_MF2_M1	= 10605,
    RISCV_PseudoVSUXSEG8EI16_V_MF2_M1_MASK	= 10606,
    RISCV_PseudoVSUXSEG8EI16_V_MF2_MF2	= 10607,
    RISCV_PseudoVSUXSEG8EI16_V_MF2_MF2_MASK	= 10608,
    RISCV_PseudoVSUXSEG8EI16_V_MF2_MF4	= 10609,
    RISCV_PseudoVSUXSEG8EI16_V_MF2_MF4_MASK	= 10610,
    RISCV_PseudoVSUXSEG8EI16_V_MF4_M1	= 10611,
    RISCV_PseudoVSUXSEG8EI16_V_MF4_M1_MASK	= 10612,
    RISCV_PseudoVSUXSEG8EI16_V_MF4_MF2	= 10613,
    RISCV_PseudoVSUXSEG8EI16_V_MF4_MF2_MASK	= 10614,
    RISCV_PseudoVSUXSEG8EI16_V_MF4_MF4	= 10615,
    RISCV_PseudoVSUXSEG8EI16_V_MF4_MF4_MASK	= 10616,
    RISCV_PseudoVSUXSEG8EI16_V_MF4_MF8	= 10617,
    RISCV_PseudoVSUXSEG8EI16_V_MF4_MF8_MASK	= 10618,
    RISCV_PseudoVSUXSEG8EI32_V_M1_M1	= 10619,
    RISCV_PseudoVSUXSEG8EI32_V_M1_M1_MASK	= 10620,
    RISCV_PseudoVSUXSEG8EI32_V_M1_MF2	= 10621,
    RISCV_PseudoVSUXSEG8EI32_V_M1_MF2_MASK	= 10622,
    RISCV_PseudoVSUXSEG8EI32_V_M1_MF4	= 10623,
    RISCV_PseudoVSUXSEG8EI32_V_M1_MF4_MASK	= 10624,
    RISCV_PseudoVSUXSEG8EI32_V_M2_M1	= 10625,
    RISCV_PseudoVSUXSEG8EI32_V_M2_M1_MASK	= 10626,
    RISCV_PseudoVSUXSEG8EI32_V_M2_MF2	= 10627,
    RISCV_PseudoVSUXSEG8EI32_V_M2_MF2_MASK	= 10628,
    RISCV_PseudoVSUXSEG8EI32_V_M4_M1	= 10629,
    RISCV_PseudoVSUXSEG8EI32_V_M4_M1_MASK	= 10630,
    RISCV_PseudoVSUXSEG8EI32_V_MF2_M1	= 10631,
    RISCV_PseudoVSUXSEG8EI32_V_MF2_M1_MASK	= 10632,
    RISCV_PseudoVSUXSEG8EI32_V_MF2_MF2	= 10633,
    RISCV_PseudoVSUXSEG8EI32_V_MF2_MF2_MASK	= 10634,
    RISCV_PseudoVSUXSEG8EI32_V_MF2_MF4	= 10635,
    RISCV_PseudoVSUXSEG8EI32_V_MF2_MF4_MASK	= 10636,
    RISCV_PseudoVSUXSEG8EI32_V_MF2_MF8	= 10637,
    RISCV_PseudoVSUXSEG8EI32_V_MF2_MF8_MASK	= 10638,
    RISCV_PseudoVSUXSEG8EI64_V_M1_M1	= 10639,
    RISCV_PseudoVSUXSEG8EI64_V_M1_M1_MASK	= 10640,
    RISCV_PseudoVSUXSEG8EI64_V_M1_MF2	= 10641,
    RISCV_PseudoVSUXSEG8EI64_V_M1_MF2_MASK	= 10642,
    RISCV_PseudoVSUXSEG8EI64_V_M1_MF4	= 10643,
    RISCV_PseudoVSUXSEG8EI64_V_M1_MF4_MASK	= 10644,
    RISCV_PseudoVSUXSEG8EI64_V_M1_MF8	= 10645,
    RISCV_PseudoVSUXSEG8EI64_V_M1_MF8_MASK	= 10646,
    RISCV_PseudoVSUXSEG8EI64_V_M2_M1	= 10647,
    RISCV_PseudoVSUXSEG8EI64_V_M2_M1_MASK	= 10648,
    RISCV_PseudoVSUXSEG8EI64_V_M2_MF2	= 10649,
    RISCV_PseudoVSUXSEG8EI64_V_M2_MF2_MASK	= 10650,
    RISCV_PseudoVSUXSEG8EI64_V_M2_MF4	= 10651,
    RISCV_PseudoVSUXSEG8EI64_V_M2_MF4_MASK	= 10652,
    RISCV_PseudoVSUXSEG8EI64_V_M4_M1	= 10653,
    RISCV_PseudoVSUXSEG8EI64_V_M4_M1_MASK	= 10654,
    RISCV_PseudoVSUXSEG8EI64_V_M4_MF2	= 10655,
    RISCV_PseudoVSUXSEG8EI64_V_M4_MF2_MASK	= 10656,
    RISCV_PseudoVSUXSEG8EI64_V_M8_M1	= 10657,
    RISCV_PseudoVSUXSEG8EI64_V_M8_M1_MASK	= 10658,
    RISCV_PseudoVSUXSEG8EI8_V_M1_M1	= 10659,
    RISCV_PseudoVSUXSEG8EI8_V_M1_M1_MASK	= 10660,
    RISCV_PseudoVSUXSEG8EI8_V_MF2_M1	= 10661,
    RISCV_PseudoVSUXSEG8EI8_V_MF2_M1_MASK	= 10662,
    RISCV_PseudoVSUXSEG8EI8_V_MF2_MF2	= 10663,
    RISCV_PseudoVSUXSEG8EI8_V_MF2_MF2_MASK	= 10664,
    RISCV_PseudoVSUXSEG8EI8_V_MF4_M1	= 10665,
    RISCV_PseudoVSUXSEG8EI8_V_MF4_M1_MASK	= 10666,
    RISCV_PseudoVSUXSEG8EI8_V_MF4_MF2	= 10667,
    RISCV_PseudoVSUXSEG8EI8_V_MF4_MF2_MASK	= 10668,
    RISCV_PseudoVSUXSEG8EI8_V_MF4_MF4	= 10669,
    RISCV_PseudoVSUXSEG8EI8_V_MF4_MF4_MASK	= 10670,
    RISCV_PseudoVSUXSEG8EI8_V_MF8_M1	= 10671,
    RISCV_PseudoVSUXSEG8EI8_V_MF8_M1_MASK	= 10672,
    RISCV_PseudoVSUXSEG8EI8_V_MF8_MF2	= 10673,
    RISCV_PseudoVSUXSEG8EI8_V_MF8_MF2_MASK	= 10674,
    RISCV_PseudoVSUXSEG8EI8_V_MF8_MF4	= 10675,
    RISCV_PseudoVSUXSEG8EI8_V_MF8_MF4_MASK	= 10676,
    RISCV_PseudoVSUXSEG8EI8_V_MF8_MF8	= 10677,
    RISCV_PseudoVSUXSEG8EI8_V_MF8_MF8_MASK	= 10678,
    RISCV_PseudoVWADDU_VV_M1	= 10679,
    RISCV_PseudoVWADDU_VV_M1_MASK	= 10680,
    RISCV_PseudoVWADDU_VV_M2	= 10681,
    RISCV_PseudoVWADDU_VV_M2_MASK	= 10682,
    RISCV_PseudoVWADDU_VV_M4	= 10683,
    RISCV_PseudoVWADDU_VV_M4_MASK	= 10684,
    RISCV_PseudoVWADDU_VV_MF2	= 10685,
    RISCV_PseudoVWADDU_VV_MF2_MASK	= 10686,
    RISCV_PseudoVWADDU_VV_MF4	= 10687,
    RISCV_PseudoVWADDU_VV_MF4_MASK	= 10688,
    RISCV_PseudoVWADDU_VV_MF8	= 10689,
    RISCV_PseudoVWADDU_VV_MF8_MASK	= 10690,
    RISCV_PseudoVWADDU_VX_M1	= 10691,
    RISCV_PseudoVWADDU_VX_M1_MASK	= 10692,
    RISCV_PseudoVWADDU_VX_M2	= 10693,
    RISCV_PseudoVWADDU_VX_M2_MASK	= 10694,
    RISCV_PseudoVWADDU_VX_M4	= 10695,
    RISCV_PseudoVWADDU_VX_M4_MASK	= 10696,
    RISCV_PseudoVWADDU_VX_MF2	= 10697,
    RISCV_PseudoVWADDU_VX_MF2_MASK	= 10698,
    RISCV_PseudoVWADDU_VX_MF4	= 10699,
    RISCV_PseudoVWADDU_VX_MF4_MASK	= 10700,
    RISCV_PseudoVWADDU_VX_MF8	= 10701,
    RISCV_PseudoVWADDU_VX_MF8_MASK	= 10702,
    RISCV_PseudoVWADDU_WV_M1	= 10703,
    RISCV_PseudoVWADDU_WV_M1_MASK	= 10704,
    RISCV_PseudoVWADDU_WV_M1_MASK_TIED	= 10705,
    RISCV_PseudoVWADDU_WV_M1_TIED	= 10706,
    RISCV_PseudoVWADDU_WV_M2	= 10707,
    RISCV_PseudoVWADDU_WV_M2_MASK	= 10708,
    RISCV_PseudoVWADDU_WV_M2_MASK_TIED	= 10709,
    RISCV_PseudoVWADDU_WV_M2_TIED	= 10710,
    RISCV_PseudoVWADDU_WV_M4	= 10711,
    RISCV_PseudoVWADDU_WV_M4_MASK	= 10712,
    RISCV_PseudoVWADDU_WV_M4_MASK_TIED	= 10713,
    RISCV_PseudoVWADDU_WV_M4_TIED	= 10714,
    RISCV_PseudoVWADDU_WV_MF2	= 10715,
    RISCV_PseudoVWADDU_WV_MF2_MASK	= 10716,
    RISCV_PseudoVWADDU_WV_MF2_MASK_TIED	= 10717,
    RISCV_PseudoVWADDU_WV_MF2_TIED	= 10718,
    RISCV_PseudoVWADDU_WV_MF4	= 10719,
    RISCV_PseudoVWADDU_WV_MF4_MASK	= 10720,
    RISCV_PseudoVWADDU_WV_MF4_MASK_TIED	= 10721,
    RISCV_PseudoVWADDU_WV_MF4_TIED	= 10722,
    RISCV_PseudoVWADDU_WV_MF8	= 10723,
    RISCV_PseudoVWADDU_WV_MF8_MASK	= 10724,
    RISCV_PseudoVWADDU_WV_MF8_MASK_TIED	= 10725,
    RISCV_PseudoVWADDU_WV_MF8_TIED	= 10726,
    RISCV_PseudoVWADDU_WX_M1	= 10727,
    RISCV_PseudoVWADDU_WX_M1_MASK	= 10728,
    RISCV_PseudoVWADDU_WX_M2	= 10729,
    RISCV_PseudoVWADDU_WX_M2_MASK	= 10730,
    RISCV_PseudoVWADDU_WX_M4	= 10731,
    RISCV_PseudoVWADDU_WX_M4_MASK	= 10732,
    RISCV_PseudoVWADDU_WX_MF2	= 10733,
    RISCV_PseudoVWADDU_WX_MF2_MASK	= 10734,
    RISCV_PseudoVWADDU_WX_MF4	= 10735,
    RISCV_PseudoVWADDU_WX_MF4_MASK	= 10736,
    RISCV_PseudoVWADDU_WX_MF8	= 10737,
    RISCV_PseudoVWADDU_WX_MF8_MASK	= 10738,
    RISCV_PseudoVWADD_VV_M1	= 10739,
    RISCV_PseudoVWADD_VV_M1_MASK	= 10740,
    RISCV_PseudoVWADD_VV_M2	= 10741,
    RISCV_PseudoVWADD_VV_M2_MASK	= 10742,
    RISCV_PseudoVWADD_VV_M4	= 10743,
    RISCV_PseudoVWADD_VV_M4_MASK	= 10744,
    RISCV_PseudoVWADD_VV_MF2	= 10745,
    RISCV_PseudoVWADD_VV_MF2_MASK	= 10746,
    RISCV_PseudoVWADD_VV_MF4	= 10747,
    RISCV_PseudoVWADD_VV_MF4_MASK	= 10748,
    RISCV_PseudoVWADD_VV_MF8	= 10749,
    RISCV_PseudoVWADD_VV_MF8_MASK	= 10750,
    RISCV_PseudoVWADD_VX_M1	= 10751,
    RISCV_PseudoVWADD_VX_M1_MASK	= 10752,
    RISCV_PseudoVWADD_VX_M2	= 10753,
    RISCV_PseudoVWADD_VX_M2_MASK	= 10754,
    RISCV_PseudoVWADD_VX_M4	= 10755,
    RISCV_PseudoVWADD_VX_M4_MASK	= 10756,
    RISCV_PseudoVWADD_VX_MF2	= 10757,
    RISCV_PseudoVWADD_VX_MF2_MASK	= 10758,
    RISCV_PseudoVWADD_VX_MF4	= 10759,
    RISCV_PseudoVWADD_VX_MF4_MASK	= 10760,
    RISCV_PseudoVWADD_VX_MF8	= 10761,
    RISCV_PseudoVWADD_VX_MF8_MASK	= 10762,
    RISCV_PseudoVWADD_WV_M1	= 10763,
    RISCV_PseudoVWADD_WV_M1_MASK	= 10764,
    RISCV_PseudoVWADD_WV_M1_MASK_TIED	= 10765,
    RISCV_PseudoVWADD_WV_M1_TIED	= 10766,
    RISCV_PseudoVWADD_WV_M2	= 10767,
    RISCV_PseudoVWADD_WV_M2_MASK	= 10768,
    RISCV_PseudoVWADD_WV_M2_MASK_TIED	= 10769,
    RISCV_PseudoVWADD_WV_M2_TIED	= 10770,
    RISCV_PseudoVWADD_WV_M4	= 10771,
    RISCV_PseudoVWADD_WV_M4_MASK	= 10772,
    RISCV_PseudoVWADD_WV_M4_MASK_TIED	= 10773,
    RISCV_PseudoVWADD_WV_M4_TIED	= 10774,
    RISCV_PseudoVWADD_WV_MF2	= 10775,
    RISCV_PseudoVWADD_WV_MF2_MASK	= 10776,
    RISCV_PseudoVWADD_WV_MF2_MASK_TIED	= 10777,
    RISCV_PseudoVWADD_WV_MF2_TIED	= 10778,
    RISCV_PseudoVWADD_WV_MF4	= 10779,
    RISCV_PseudoVWADD_WV_MF4_MASK	= 10780,
    RISCV_PseudoVWADD_WV_MF4_MASK_TIED	= 10781,
    RISCV_PseudoVWADD_WV_MF4_TIED	= 10782,
    RISCV_PseudoVWADD_WV_MF8	= 10783,
    RISCV_PseudoVWADD_WV_MF8_MASK	= 10784,
    RISCV_PseudoVWADD_WV_MF8_MASK_TIED	= 10785,
    RISCV_PseudoVWADD_WV_MF8_TIED	= 10786,
    RISCV_PseudoVWADD_WX_M1	= 10787,
    RISCV_PseudoVWADD_WX_M1_MASK	= 10788,
    RISCV_PseudoVWADD_WX_M2	= 10789,
    RISCV_PseudoVWADD_WX_M2_MASK	= 10790,
    RISCV_PseudoVWADD_WX_M4	= 10791,
    RISCV_PseudoVWADD_WX_M4_MASK	= 10792,
    RISCV_PseudoVWADD_WX_MF2	= 10793,
    RISCV_PseudoVWADD_WX_MF2_MASK	= 10794,
    RISCV_PseudoVWADD_WX_MF4	= 10795,
    RISCV_PseudoVWADD_WX_MF4_MASK	= 10796,
    RISCV_PseudoVWADD_WX_MF8	= 10797,
    RISCV_PseudoVWADD_WX_MF8_MASK	= 10798,
    RISCV_PseudoVWMACCSU_VV_M1	= 10799,
    RISCV_PseudoVWMACCSU_VV_M1_MASK	= 10800,
    RISCV_PseudoVWMACCSU_VV_M2	= 10801,
    RISCV_PseudoVWMACCSU_VV_M2_MASK	= 10802,
    RISCV_PseudoVWMACCSU_VV_M4	= 10803,
    RISCV_PseudoVWMACCSU_VV_M4_MASK	= 10804,
    RISCV_PseudoVWMACCSU_VV_MF2	= 10805,
    RISCV_PseudoVWMACCSU_VV_MF2_MASK	= 10806,
    RISCV_PseudoVWMACCSU_VV_MF4	= 10807,
    RISCV_PseudoVWMACCSU_VV_MF4_MASK	= 10808,
    RISCV_PseudoVWMACCSU_VV_MF8	= 10809,
    RISCV_PseudoVWMACCSU_VV_MF8_MASK	= 10810,
    RISCV_PseudoVWMACCSU_VX_M1	= 10811,
    RISCV_PseudoVWMACCSU_VX_M1_MASK	= 10812,
    RISCV_PseudoVWMACCSU_VX_M2	= 10813,
    RISCV_PseudoVWMACCSU_VX_M2_MASK	= 10814,
    RISCV_PseudoVWMACCSU_VX_M4	= 10815,
    RISCV_PseudoVWMACCSU_VX_M4_MASK	= 10816,
    RISCV_PseudoVWMACCSU_VX_MF2	= 10817,
    RISCV_PseudoVWMACCSU_VX_MF2_MASK	= 10818,
    RISCV_PseudoVWMACCSU_VX_MF4	= 10819,
    RISCV_PseudoVWMACCSU_VX_MF4_MASK	= 10820,
    RISCV_PseudoVWMACCSU_VX_MF8	= 10821,
    RISCV_PseudoVWMACCSU_VX_MF8_MASK	= 10822,
    RISCV_PseudoVWMACCUS_VX_M1	= 10823,
    RISCV_PseudoVWMACCUS_VX_M1_MASK	= 10824,
    RISCV_PseudoVWMACCUS_VX_M2	= 10825,
    RISCV_PseudoVWMACCUS_VX_M2_MASK	= 10826,
    RISCV_PseudoVWMACCUS_VX_M4	= 10827,
    RISCV_PseudoVWMACCUS_VX_M4_MASK	= 10828,
    RISCV_PseudoVWMACCUS_VX_MF2	= 10829,
    RISCV_PseudoVWMACCUS_VX_MF2_MASK	= 10830,
    RISCV_PseudoVWMACCUS_VX_MF4	= 10831,
    RISCV_PseudoVWMACCUS_VX_MF4_MASK	= 10832,
    RISCV_PseudoVWMACCUS_VX_MF8	= 10833,
    RISCV_PseudoVWMACCUS_VX_MF8_MASK	= 10834,
    RISCV_PseudoVWMACCU_VV_M1	= 10835,
    RISCV_PseudoVWMACCU_VV_M1_MASK	= 10836,
    RISCV_PseudoVWMACCU_VV_M2	= 10837,
    RISCV_PseudoVWMACCU_VV_M2_MASK	= 10838,
    RISCV_PseudoVWMACCU_VV_M4	= 10839,
    RISCV_PseudoVWMACCU_VV_M4_MASK	= 10840,
    RISCV_PseudoVWMACCU_VV_MF2	= 10841,
    RISCV_PseudoVWMACCU_VV_MF2_MASK	= 10842,
    RISCV_PseudoVWMACCU_VV_MF4	= 10843,
    RISCV_PseudoVWMACCU_VV_MF4_MASK	= 10844,
    RISCV_PseudoVWMACCU_VV_MF8	= 10845,
    RISCV_PseudoVWMACCU_VV_MF8_MASK	= 10846,
    RISCV_PseudoVWMACCU_VX_M1	= 10847,
    RISCV_PseudoVWMACCU_VX_M1_MASK	= 10848,
    RISCV_PseudoVWMACCU_VX_M2	= 10849,
    RISCV_PseudoVWMACCU_VX_M2_MASK	= 10850,
    RISCV_PseudoVWMACCU_VX_M4	= 10851,
    RISCV_PseudoVWMACCU_VX_M4_MASK	= 10852,
    RISCV_PseudoVWMACCU_VX_MF2	= 10853,
    RISCV_PseudoVWMACCU_VX_MF2_MASK	= 10854,
    RISCV_PseudoVWMACCU_VX_MF4	= 10855,
    RISCV_PseudoVWMACCU_VX_MF4_MASK	= 10856,
    RISCV_PseudoVWMACCU_VX_MF8	= 10857,
    RISCV_PseudoVWMACCU_VX_MF8_MASK	= 10858,
    RISCV_PseudoVWMACC_VV_M1	= 10859,
    RISCV_PseudoVWMACC_VV_M1_MASK	= 10860,
    RISCV_PseudoVWMACC_VV_M2	= 10861,
    RISCV_PseudoVWMACC_VV_M2_MASK	= 10862,
    RISCV_PseudoVWMACC_VV_M4	= 10863,
    RISCV_PseudoVWMACC_VV_M4_MASK	= 10864,
    RISCV_PseudoVWMACC_VV_MF2	= 10865,
    RISCV_PseudoVWMACC_VV_MF2_MASK	= 10866,
    RISCV_PseudoVWMACC_VV_MF4	= 10867,
    RISCV_PseudoVWMACC_VV_MF4_MASK	= 10868,
    RISCV_PseudoVWMACC_VV_MF8	= 10869,
    RISCV_PseudoVWMACC_VV_MF8_MASK	= 10870,
    RISCV_PseudoVWMACC_VX_M1	= 10871,
    RISCV_PseudoVWMACC_VX_M1_MASK	= 10872,
    RISCV_PseudoVWMACC_VX_M2	= 10873,
    RISCV_PseudoVWMACC_VX_M2_MASK	= 10874,
    RISCV_PseudoVWMACC_VX_M4	= 10875,
    RISCV_PseudoVWMACC_VX_M4_MASK	= 10876,
    RISCV_PseudoVWMACC_VX_MF2	= 10877,
    RISCV_PseudoVWMACC_VX_MF2_MASK	= 10878,
    RISCV_PseudoVWMACC_VX_MF4	= 10879,
    RISCV_PseudoVWMACC_VX_MF4_MASK	= 10880,
    RISCV_PseudoVWMACC_VX_MF8	= 10881,
    RISCV_PseudoVWMACC_VX_MF8_MASK	= 10882,
    RISCV_PseudoVWMULSU_VV_M1	= 10883,
    RISCV_PseudoVWMULSU_VV_M1_MASK	= 10884,
    RISCV_PseudoVWMULSU_VV_M2	= 10885,
    RISCV_PseudoVWMULSU_VV_M2_MASK	= 10886,
    RISCV_PseudoVWMULSU_VV_M4	= 10887,
    RISCV_PseudoVWMULSU_VV_M4_MASK	= 10888,
    RISCV_PseudoVWMULSU_VV_MF2	= 10889,
    RISCV_PseudoVWMULSU_VV_MF2_MASK	= 10890,
    RISCV_PseudoVWMULSU_VV_MF4	= 10891,
    RISCV_PseudoVWMULSU_VV_MF4_MASK	= 10892,
    RISCV_PseudoVWMULSU_VV_MF8	= 10893,
    RISCV_PseudoVWMULSU_VV_MF8_MASK	= 10894,
    RISCV_PseudoVWMULSU_VX_M1	= 10895,
    RISCV_PseudoVWMULSU_VX_M1_MASK	= 10896,
    RISCV_PseudoVWMULSU_VX_M2	= 10897,
    RISCV_PseudoVWMULSU_VX_M2_MASK	= 10898,
    RISCV_PseudoVWMULSU_VX_M4	= 10899,
    RISCV_PseudoVWMULSU_VX_M4_MASK	= 10900,
    RISCV_PseudoVWMULSU_VX_MF2	= 10901,
    RISCV_PseudoVWMULSU_VX_MF2_MASK	= 10902,
    RISCV_PseudoVWMULSU_VX_MF4	= 10903,
    RISCV_PseudoVWMULSU_VX_MF4_MASK	= 10904,
    RISCV_PseudoVWMULSU_VX_MF8	= 10905,
    RISCV_PseudoVWMULSU_VX_MF8_MASK	= 10906,
    RISCV_PseudoVWMULU_VV_M1	= 10907,
    RISCV_PseudoVWMULU_VV_M1_MASK	= 10908,
    RISCV_PseudoVWMULU_VV_M2	= 10909,
    RISCV_PseudoVWMULU_VV_M2_MASK	= 10910,
    RISCV_PseudoVWMULU_VV_M4	= 10911,
    RISCV_PseudoVWMULU_VV_M4_MASK	= 10912,
    RISCV_PseudoVWMULU_VV_MF2	= 10913,
    RISCV_PseudoVWMULU_VV_MF2_MASK	= 10914,
    RISCV_PseudoVWMULU_VV_MF4	= 10915,
    RISCV_PseudoVWMULU_VV_MF4_MASK	= 10916,
    RISCV_PseudoVWMULU_VV_MF8	= 10917,
    RISCV_PseudoVWMULU_VV_MF8_MASK	= 10918,
    RISCV_PseudoVWMULU_VX_M1	= 10919,
    RISCV_PseudoVWMULU_VX_M1_MASK	= 10920,
    RISCV_PseudoVWMULU_VX_M2	= 10921,
    RISCV_PseudoVWMULU_VX_M2_MASK	= 10922,
    RISCV_PseudoVWMULU_VX_M4	= 10923,
    RISCV_PseudoVWMULU_VX_M4_MASK	= 10924,
    RISCV_PseudoVWMULU_VX_MF2	= 10925,
    RISCV_PseudoVWMULU_VX_MF2_MASK	= 10926,
    RISCV_PseudoVWMULU_VX_MF4	= 10927,
    RISCV_PseudoVWMULU_VX_MF4_MASK	= 10928,
    RISCV_PseudoVWMULU_VX_MF8	= 10929,
    RISCV_PseudoVWMULU_VX_MF8_MASK	= 10930,
    RISCV_PseudoVWMUL_VV_M1	= 10931,
    RISCV_PseudoVWMUL_VV_M1_MASK	= 10932,
    RISCV_PseudoVWMUL_VV_M2	= 10933,
    RISCV_PseudoVWMUL_VV_M2_MASK	= 10934,
    RISCV_PseudoVWMUL_VV_M4	= 10935,
    RISCV_PseudoVWMUL_VV_M4_MASK	= 10936,
    RISCV_PseudoVWMUL_VV_MF2	= 10937,
    RISCV_PseudoVWMUL_VV_MF2_MASK	= 10938,
    RISCV_PseudoVWMUL_VV_MF4	= 10939,
    RISCV_PseudoVWMUL_VV_MF4_MASK	= 10940,
    RISCV_PseudoVWMUL_VV_MF8	= 10941,
    RISCV_PseudoVWMUL_VV_MF8_MASK	= 10942,
    RISCV_PseudoVWMUL_VX_M1	= 10943,
    RISCV_PseudoVWMUL_VX_M1_MASK	= 10944,
    RISCV_PseudoVWMUL_VX_M2	= 10945,
    RISCV_PseudoVWMUL_VX_M2_MASK	= 10946,
    RISCV_PseudoVWMUL_VX_M4	= 10947,
    RISCV_PseudoVWMUL_VX_M4_MASK	= 10948,
    RISCV_PseudoVWMUL_VX_MF2	= 10949,
    RISCV_PseudoVWMUL_VX_MF2_MASK	= 10950,
    RISCV_PseudoVWMUL_VX_MF4	= 10951,
    RISCV_PseudoVWMUL_VX_MF4_MASK	= 10952,
    RISCV_PseudoVWMUL_VX_MF8	= 10953,
    RISCV_PseudoVWMUL_VX_MF8_MASK	= 10954,
    RISCV_PseudoVWREDSUMU_VS_M1_E16	= 10955,
    RISCV_PseudoVWREDSUMU_VS_M1_E16_MASK	= 10956,
    RISCV_PseudoVWREDSUMU_VS_M1_E32	= 10957,
    RISCV_PseudoVWREDSUMU_VS_M1_E32_MASK	= 10958,
    RISCV_PseudoVWREDSUMU_VS_M1_E8	= 10959,
    RISCV_PseudoVWREDSUMU_VS_M1_E8_MASK	= 10960,
    RISCV_PseudoVWREDSUMU_VS_M2_E16	= 10961,
    RISCV_PseudoVWREDSUMU_VS_M2_E16_MASK	= 10962,
    RISCV_PseudoVWREDSUMU_VS_M2_E32	= 10963,
    RISCV_PseudoVWREDSUMU_VS_M2_E32_MASK	= 10964,
    RISCV_PseudoVWREDSUMU_VS_M2_E8	= 10965,
    RISCV_PseudoVWREDSUMU_VS_M2_E8_MASK	= 10966,
    RISCV_PseudoVWREDSUMU_VS_M4_E16	= 10967,
    RISCV_PseudoVWREDSUMU_VS_M4_E16_MASK	= 10968,
    RISCV_PseudoVWREDSUMU_VS_M4_E32	= 10969,
    RISCV_PseudoVWREDSUMU_VS_M4_E32_MASK	= 10970,
    RISCV_PseudoVWREDSUMU_VS_M4_E8	= 10971,
    RISCV_PseudoVWREDSUMU_VS_M4_E8_MASK	= 10972,
    RISCV_PseudoVWREDSUMU_VS_M8_E16	= 10973,
    RISCV_PseudoVWREDSUMU_VS_M8_E16_MASK	= 10974,
    RISCV_PseudoVWREDSUMU_VS_M8_E32	= 10975,
    RISCV_PseudoVWREDSUMU_VS_M8_E32_MASK	= 10976,
    RISCV_PseudoVWREDSUMU_VS_M8_E8	= 10977,
    RISCV_PseudoVWREDSUMU_VS_M8_E8_MASK	= 10978,
    RISCV_PseudoVWREDSUMU_VS_MF2_E16	= 10979,
    RISCV_PseudoVWREDSUMU_VS_MF2_E16_MASK	= 10980,
    RISCV_PseudoVWREDSUMU_VS_MF2_E32	= 10981,
    RISCV_PseudoVWREDSUMU_VS_MF2_E32_MASK	= 10982,
    RISCV_PseudoVWREDSUMU_VS_MF2_E8	= 10983,
    RISCV_PseudoVWREDSUMU_VS_MF2_E8_MASK	= 10984,
    RISCV_PseudoVWREDSUMU_VS_MF4_E16	= 10985,
    RISCV_PseudoVWREDSUMU_VS_MF4_E16_MASK	= 10986,
    RISCV_PseudoVWREDSUMU_VS_MF4_E8	= 10987,
    RISCV_PseudoVWREDSUMU_VS_MF4_E8_MASK	= 10988,
    RISCV_PseudoVWREDSUMU_VS_MF8_E8	= 10989,
    RISCV_PseudoVWREDSUMU_VS_MF8_E8_MASK	= 10990,
    RISCV_PseudoVWREDSUM_VS_M1_E16	= 10991,
    RISCV_PseudoVWREDSUM_VS_M1_E16_MASK	= 10992,
    RISCV_PseudoVWREDSUM_VS_M1_E32	= 10993,
    RISCV_PseudoVWREDSUM_VS_M1_E32_MASK	= 10994,
    RISCV_PseudoVWREDSUM_VS_M1_E8	= 10995,
    RISCV_PseudoVWREDSUM_VS_M1_E8_MASK	= 10996,
    RISCV_PseudoVWREDSUM_VS_M2_E16	= 10997,
    RISCV_PseudoVWREDSUM_VS_M2_E16_MASK	= 10998,
    RISCV_PseudoVWREDSUM_VS_M2_E32	= 10999,
    RISCV_PseudoVWREDSUM_VS_M2_E32_MASK	= 11000,
    RISCV_PseudoVWREDSUM_VS_M2_E8	= 11001,
    RISCV_PseudoVWREDSUM_VS_M2_E8_MASK	= 11002,
    RISCV_PseudoVWREDSUM_VS_M4_E16	= 11003,
    RISCV_PseudoVWREDSUM_VS_M4_E16_MASK	= 11004,
    RISCV_PseudoVWREDSUM_VS_M4_E32	= 11005,
    RISCV_PseudoVWREDSUM_VS_M4_E32_MASK	= 11006,
    RISCV_PseudoVWREDSUM_VS_M4_E8	= 11007,
    RISCV_PseudoVWREDSUM_VS_M4_E8_MASK	= 11008,
    RISCV_PseudoVWREDSUM_VS_M8_E16	= 11009,
    RISCV_PseudoVWREDSUM_VS_M8_E16_MASK	= 11010,
    RISCV_PseudoVWREDSUM_VS_M8_E32	= 11011,
    RISCV_PseudoVWREDSUM_VS_M8_E32_MASK	= 11012,
    RISCV_PseudoVWREDSUM_VS_M8_E8	= 11013,
    RISCV_PseudoVWREDSUM_VS_M8_E8_MASK	= 11014,
    RISCV_PseudoVWREDSUM_VS_MF2_E16	= 11015,
    RISCV_PseudoVWREDSUM_VS_MF2_E16_MASK	= 11016,
    RISCV_PseudoVWREDSUM_VS_MF2_E32	= 11017,
    RISCV_PseudoVWREDSUM_VS_MF2_E32_MASK	= 11018,
    RISCV_PseudoVWREDSUM_VS_MF2_E8	= 11019,
    RISCV_PseudoVWREDSUM_VS_MF2_E8_MASK	= 11020,
    RISCV_PseudoVWREDSUM_VS_MF4_E16	= 11021,
    RISCV_PseudoVWREDSUM_VS_MF4_E16_MASK	= 11022,
    RISCV_PseudoVWREDSUM_VS_MF4_E8	= 11023,
    RISCV_PseudoVWREDSUM_VS_MF4_E8_MASK	= 11024,
    RISCV_PseudoVWREDSUM_VS_MF8_E8	= 11025,
    RISCV_PseudoVWREDSUM_VS_MF8_E8_MASK	= 11026,
    RISCV_PseudoVWSLL_VI_M1	= 11027,
    RISCV_PseudoVWSLL_VI_M1_MASK	= 11028,
    RISCV_PseudoVWSLL_VI_M2	= 11029,
    RISCV_PseudoVWSLL_VI_M2_MASK	= 11030,
    RISCV_PseudoVWSLL_VI_M4	= 11031,
    RISCV_PseudoVWSLL_VI_M4_MASK	= 11032,
    RISCV_PseudoVWSLL_VI_MF2	= 11033,
    RISCV_PseudoVWSLL_VI_MF2_MASK	= 11034,
    RISCV_PseudoVWSLL_VI_MF4	= 11035,
    RISCV_PseudoVWSLL_VI_MF4_MASK	= 11036,
    RISCV_PseudoVWSLL_VI_MF8	= 11037,
    RISCV_PseudoVWSLL_VI_MF8_MASK	= 11038,
    RISCV_PseudoVWSLL_VV_M1	= 11039,
    RISCV_PseudoVWSLL_VV_M1_MASK	= 11040,
    RISCV_PseudoVWSLL_VV_M2	= 11041,
    RISCV_PseudoVWSLL_VV_M2_MASK	= 11042,
    RISCV_PseudoVWSLL_VV_M4	= 11043,
    RISCV_PseudoVWSLL_VV_M4_MASK	= 11044,
    RISCV_PseudoVWSLL_VV_MF2	= 11045,
    RISCV_PseudoVWSLL_VV_MF2_MASK	= 11046,
    RISCV_PseudoVWSLL_VV_MF4	= 11047,
    RISCV_PseudoVWSLL_VV_MF4_MASK	= 11048,
    RISCV_PseudoVWSLL_VV_MF8	= 11049,
    RISCV_PseudoVWSLL_VV_MF8_MASK	= 11050,
    RISCV_PseudoVWSLL_VX_M1	= 11051,
    RISCV_PseudoVWSLL_VX_M1_MASK	= 11052,
    RISCV_PseudoVWSLL_VX_M2	= 11053,
    RISCV_PseudoVWSLL_VX_M2_MASK	= 11054,
    RISCV_PseudoVWSLL_VX_M4	= 11055,
    RISCV_PseudoVWSLL_VX_M4_MASK	= 11056,
    RISCV_PseudoVWSLL_VX_MF2	= 11057,
    RISCV_PseudoVWSLL_VX_MF2_MASK	= 11058,
    RISCV_PseudoVWSLL_VX_MF4	= 11059,
    RISCV_PseudoVWSLL_VX_MF4_MASK	= 11060,
    RISCV_PseudoVWSLL_VX_MF8	= 11061,
    RISCV_PseudoVWSLL_VX_MF8_MASK	= 11062,
    RISCV_PseudoVWSUBU_VV_M1	= 11063,
    RISCV_PseudoVWSUBU_VV_M1_MASK	= 11064,
    RISCV_PseudoVWSUBU_VV_M2	= 11065,
    RISCV_PseudoVWSUBU_VV_M2_MASK	= 11066,
    RISCV_PseudoVWSUBU_VV_M4	= 11067,
    RISCV_PseudoVWSUBU_VV_M4_MASK	= 11068,
    RISCV_PseudoVWSUBU_VV_MF2	= 11069,
    RISCV_PseudoVWSUBU_VV_MF2_MASK	= 11070,
    RISCV_PseudoVWSUBU_VV_MF4	= 11071,
    RISCV_PseudoVWSUBU_VV_MF4_MASK	= 11072,
    RISCV_PseudoVWSUBU_VV_MF8	= 11073,
    RISCV_PseudoVWSUBU_VV_MF8_MASK	= 11074,
    RISCV_PseudoVWSUBU_VX_M1	= 11075,
    RISCV_PseudoVWSUBU_VX_M1_MASK	= 11076,
    RISCV_PseudoVWSUBU_VX_M2	= 11077,
    RISCV_PseudoVWSUBU_VX_M2_MASK	= 11078,
    RISCV_PseudoVWSUBU_VX_M4	= 11079,
    RISCV_PseudoVWSUBU_VX_M4_MASK	= 11080,
    RISCV_PseudoVWSUBU_VX_MF2	= 11081,
    RISCV_PseudoVWSUBU_VX_MF2_MASK	= 11082,
    RISCV_PseudoVWSUBU_VX_MF4	= 11083,
    RISCV_PseudoVWSUBU_VX_MF4_MASK	= 11084,
    RISCV_PseudoVWSUBU_VX_MF8	= 11085,
    RISCV_PseudoVWSUBU_VX_MF8_MASK	= 11086,
    RISCV_PseudoVWSUBU_WV_M1	= 11087,
    RISCV_PseudoVWSUBU_WV_M1_MASK	= 11088,
    RISCV_PseudoVWSUBU_WV_M1_MASK_TIED	= 11089,
    RISCV_PseudoVWSUBU_WV_M1_TIED	= 11090,
    RISCV_PseudoVWSUBU_WV_M2	= 11091,
    RISCV_PseudoVWSUBU_WV_M2_MASK	= 11092,
    RISCV_PseudoVWSUBU_WV_M2_MASK_TIED	= 11093,
    RISCV_PseudoVWSUBU_WV_M2_TIED	= 11094,
    RISCV_PseudoVWSUBU_WV_M4	= 11095,
    RISCV_PseudoVWSUBU_WV_M4_MASK	= 11096,
    RISCV_PseudoVWSUBU_WV_M4_MASK_TIED	= 11097,
    RISCV_PseudoVWSUBU_WV_M4_TIED	= 11098,
    RISCV_PseudoVWSUBU_WV_MF2	= 11099,
    RISCV_PseudoVWSUBU_WV_MF2_MASK	= 11100,
    RISCV_PseudoVWSUBU_WV_MF2_MASK_TIED	= 11101,
    RISCV_PseudoVWSUBU_WV_MF2_TIED	= 11102,
    RISCV_PseudoVWSUBU_WV_MF4	= 11103,
    RISCV_PseudoVWSUBU_WV_MF4_MASK	= 11104,
    RISCV_PseudoVWSUBU_WV_MF4_MASK_TIED	= 11105,
    RISCV_PseudoVWSUBU_WV_MF4_TIED	= 11106,
    RISCV_PseudoVWSUBU_WV_MF8	= 11107,
    RISCV_PseudoVWSUBU_WV_MF8_MASK	= 11108,
    RISCV_PseudoVWSUBU_WV_MF8_MASK_TIED	= 11109,
    RISCV_PseudoVWSUBU_WV_MF8_TIED	= 11110,
    RISCV_PseudoVWSUBU_WX_M1	= 11111,
    RISCV_PseudoVWSUBU_WX_M1_MASK	= 11112,
    RISCV_PseudoVWSUBU_WX_M2	= 11113,
    RISCV_PseudoVWSUBU_WX_M2_MASK	= 11114,
    RISCV_PseudoVWSUBU_WX_M4	= 11115,
    RISCV_PseudoVWSUBU_WX_M4_MASK	= 11116,
    RISCV_PseudoVWSUBU_WX_MF2	= 11117,
    RISCV_PseudoVWSUBU_WX_MF2_MASK	= 11118,
    RISCV_PseudoVWSUBU_WX_MF4	= 11119,
    RISCV_PseudoVWSUBU_WX_MF4_MASK	= 11120,
    RISCV_PseudoVWSUBU_WX_MF8	= 11121,
    RISCV_PseudoVWSUBU_WX_MF8_MASK	= 11122,
    RISCV_PseudoVWSUB_VV_M1	= 11123,
    RISCV_PseudoVWSUB_VV_M1_MASK	= 11124,
    RISCV_PseudoVWSUB_VV_M2	= 11125,
    RISCV_PseudoVWSUB_VV_M2_MASK	= 11126,
    RISCV_PseudoVWSUB_VV_M4	= 11127,
    RISCV_PseudoVWSUB_VV_M4_MASK	= 11128,
    RISCV_PseudoVWSUB_VV_MF2	= 11129,
    RISCV_PseudoVWSUB_VV_MF2_MASK	= 11130,
    RISCV_PseudoVWSUB_VV_MF4	= 11131,
    RISCV_PseudoVWSUB_VV_MF4_MASK	= 11132,
    RISCV_PseudoVWSUB_VV_MF8	= 11133,
    RISCV_PseudoVWSUB_VV_MF8_MASK	= 11134,
    RISCV_PseudoVWSUB_VX_M1	= 11135,
    RISCV_PseudoVWSUB_VX_M1_MASK	= 11136,
    RISCV_PseudoVWSUB_VX_M2	= 11137,
    RISCV_PseudoVWSUB_VX_M2_MASK	= 11138,
    RISCV_PseudoVWSUB_VX_M4	= 11139,
    RISCV_PseudoVWSUB_VX_M4_MASK	= 11140,
    RISCV_PseudoVWSUB_VX_MF2	= 11141,
    RISCV_PseudoVWSUB_VX_MF2_MASK	= 11142,
    RISCV_PseudoVWSUB_VX_MF4	= 11143,
    RISCV_PseudoVWSUB_VX_MF4_MASK	= 11144,
    RISCV_PseudoVWSUB_VX_MF8	= 11145,
    RISCV_PseudoVWSUB_VX_MF8_MASK	= 11146,
    RISCV_PseudoVWSUB_WV_M1	= 11147,
    RISCV_PseudoVWSUB_WV_M1_MASK	= 11148,
    RISCV_PseudoVWSUB_WV_M1_MASK_TIED	= 11149,
    RISCV_PseudoVWSUB_WV_M1_TIED	= 11150,
    RISCV_PseudoVWSUB_WV_M2	= 11151,
    RISCV_PseudoVWSUB_WV_M2_MASK	= 11152,
    RISCV_PseudoVWSUB_WV_M2_MASK_TIED	= 11153,
    RISCV_PseudoVWSUB_WV_M2_TIED	= 11154,
    RISCV_PseudoVWSUB_WV_M4	= 11155,
    RISCV_PseudoVWSUB_WV_M4_MASK	= 11156,
    RISCV_PseudoVWSUB_WV_M4_MASK_TIED	= 11157,
    RISCV_PseudoVWSUB_WV_M4_TIED	= 11158,
    RISCV_PseudoVWSUB_WV_MF2	= 11159,
    RISCV_PseudoVWSUB_WV_MF2_MASK	= 11160,
    RISCV_PseudoVWSUB_WV_MF2_MASK_TIED	= 11161,
    RISCV_PseudoVWSUB_WV_MF2_TIED	= 11162,
    RISCV_PseudoVWSUB_WV_MF4	= 11163,
    RISCV_PseudoVWSUB_WV_MF4_MASK	= 11164,
    RISCV_PseudoVWSUB_WV_MF4_MASK_TIED	= 11165,
    RISCV_PseudoVWSUB_WV_MF4_TIED	= 11166,
    RISCV_PseudoVWSUB_WV_MF8	= 11167,
    RISCV_PseudoVWSUB_WV_MF8_MASK	= 11168,
    RISCV_PseudoVWSUB_WV_MF8_MASK_TIED	= 11169,
    RISCV_PseudoVWSUB_WV_MF8_TIED	= 11170,
    RISCV_PseudoVWSUB_WX_M1	= 11171,
    RISCV_PseudoVWSUB_WX_M1_MASK	= 11172,
    RISCV_PseudoVWSUB_WX_M2	= 11173,
    RISCV_PseudoVWSUB_WX_M2_MASK	= 11174,
    RISCV_PseudoVWSUB_WX_M4	= 11175,
    RISCV_PseudoVWSUB_WX_M4_MASK	= 11176,
    RISCV_PseudoVWSUB_WX_MF2	= 11177,
    RISCV_PseudoVWSUB_WX_MF2_MASK	= 11178,
    RISCV_PseudoVWSUB_WX_MF4	= 11179,
    RISCV_PseudoVWSUB_WX_MF4_MASK	= 11180,
    RISCV_PseudoVWSUB_WX_MF8	= 11181,
    RISCV_PseudoVWSUB_WX_MF8_MASK	= 11182,
    RISCV_PseudoVXOR_VI_M1	= 11183,
    RISCV_PseudoVXOR_VI_M1_MASK	= 11184,
    RISCV_PseudoVXOR_VI_M2	= 11185,
    RISCV_PseudoVXOR_VI_M2_MASK	= 11186,
    RISCV_PseudoVXOR_VI_M4	= 11187,
    RISCV_PseudoVXOR_VI_M4_MASK	= 11188,
    RISCV_PseudoVXOR_VI_M8	= 11189,
    RISCV_PseudoVXOR_VI_M8_MASK	= 11190,
    RISCV_PseudoVXOR_VI_MF2	= 11191,
    RISCV_PseudoVXOR_VI_MF2_MASK	= 11192,
    RISCV_PseudoVXOR_VI_MF4	= 11193,
    RISCV_PseudoVXOR_VI_MF4_MASK	= 11194,
    RISCV_PseudoVXOR_VI_MF8	= 11195,
    RISCV_PseudoVXOR_VI_MF8_MASK	= 11196,
    RISCV_PseudoVXOR_VV_M1	= 11197,
    RISCV_PseudoVXOR_VV_M1_MASK	= 11198,
    RISCV_PseudoVXOR_VV_M2	= 11199,
    RISCV_PseudoVXOR_VV_M2_MASK	= 11200,
    RISCV_PseudoVXOR_VV_M4	= 11201,
    RISCV_PseudoVXOR_VV_M4_MASK	= 11202,
    RISCV_PseudoVXOR_VV_M8	= 11203,
    RISCV_PseudoVXOR_VV_M8_MASK	= 11204,
    RISCV_PseudoVXOR_VV_MF2	= 11205,
    RISCV_PseudoVXOR_VV_MF2_MASK	= 11206,
    RISCV_PseudoVXOR_VV_MF4	= 11207,
    RISCV_PseudoVXOR_VV_MF4_MASK	= 11208,
    RISCV_PseudoVXOR_VV_MF8	= 11209,
    RISCV_PseudoVXOR_VV_MF8_MASK	= 11210,
    RISCV_PseudoVXOR_VX_M1	= 11211,
    RISCV_PseudoVXOR_VX_M1_MASK	= 11212,
    RISCV_PseudoVXOR_VX_M2	= 11213,
    RISCV_PseudoVXOR_VX_M2_MASK	= 11214,
    RISCV_PseudoVXOR_VX_M4	= 11215,
    RISCV_PseudoVXOR_VX_M4_MASK	= 11216,
    RISCV_PseudoVXOR_VX_M8	= 11217,
    RISCV_PseudoVXOR_VX_M8_MASK	= 11218,
    RISCV_PseudoVXOR_VX_MF2	= 11219,
    RISCV_PseudoVXOR_VX_MF2_MASK	= 11220,
    RISCV_PseudoVXOR_VX_MF4	= 11221,
    RISCV_PseudoVXOR_VX_MF4_MASK	= 11222,
    RISCV_PseudoVXOR_VX_MF8	= 11223,
    RISCV_PseudoVXOR_VX_MF8_MASK	= 11224,
    RISCV_PseudoVZEXT_VF2_M1	= 11225,
    RISCV_PseudoVZEXT_VF2_M1_MASK	= 11226,
    RISCV_PseudoVZEXT_VF2_M2	= 11227,
    RISCV_PseudoVZEXT_VF2_M2_MASK	= 11228,
    RISCV_PseudoVZEXT_VF2_M4	= 11229,
    RISCV_PseudoVZEXT_VF2_M4_MASK	= 11230,
    RISCV_PseudoVZEXT_VF2_M8	= 11231,
    RISCV_PseudoVZEXT_VF2_M8_MASK	= 11232,
    RISCV_PseudoVZEXT_VF2_MF2	= 11233,
    RISCV_PseudoVZEXT_VF2_MF2_MASK	= 11234,
    RISCV_PseudoVZEXT_VF2_MF4	= 11235,
    RISCV_PseudoVZEXT_VF2_MF4_MASK	= 11236,
    RISCV_PseudoVZEXT_VF4_M1	= 11237,
    RISCV_PseudoVZEXT_VF4_M1_MASK	= 11238,
    RISCV_PseudoVZEXT_VF4_M2	= 11239,
    RISCV_PseudoVZEXT_VF4_M2_MASK	= 11240,
    RISCV_PseudoVZEXT_VF4_M4	= 11241,
    RISCV_PseudoVZEXT_VF4_M4_MASK	= 11242,
    RISCV_PseudoVZEXT_VF4_M8	= 11243,
    RISCV_PseudoVZEXT_VF4_M8_MASK	= 11244,
    RISCV_PseudoVZEXT_VF4_MF2	= 11245,
    RISCV_PseudoVZEXT_VF4_MF2_MASK	= 11246,
    RISCV_PseudoVZEXT_VF8_M1	= 11247,
    RISCV_PseudoVZEXT_VF8_M1_MASK	= 11248,
    RISCV_PseudoVZEXT_VF8_M2	= 11249,
    RISCV_PseudoVZEXT_VF8_M2_MASK	= 11250,
    RISCV_PseudoVZEXT_VF8_M4	= 11251,
    RISCV_PseudoVZEXT_VF8_M4_MASK	= 11252,
    RISCV_PseudoVZEXT_VF8_M8	= 11253,
    RISCV_PseudoVZEXT_VF8_M8_MASK	= 11254,
    RISCV_PseudoZEXT_H	= 11255,
    RISCV_PseudoZEXT_W	= 11256,
    RISCV_ReadCycleWide	= 11257,
    RISCV_ReadFFLAGS	= 11258,
    RISCV_ReadFRM	= 11259,
    RISCV_Select_FPR16INX_Using_CC_GPR	= 11260,
    RISCV_Select_FPR16_Using_CC_GPR	= 11261,
    RISCV_Select_FPR32INX_Using_CC_GPR	= 11262,
    RISCV_Select_FPR32_Using_CC_GPR	= 11263,
    RISCV_Select_FPR64IN32X_Using_CC_GPR	= 11264,
    RISCV_Select_FPR64INX_Using_CC_GPR	= 11265,
    RISCV_Select_FPR64_Using_CC_GPR	= 11266,
    RISCV_Select_GPR_Using_CC_GPR	= 11267,
    RISCV_SplitF64Pseudo	= 11268,
    RISCV_SplitF64Pseudo_INX	= 11269,
    RISCV_SwapFRMImm	= 11270,
    RISCV_WriteFFLAGS	= 11271,
    RISCV_WriteFRM	= 11272,
    RISCV_WriteFRMImm	= 11273,
    RISCV_WriteVXRMImm	= 11274,
    RISCV_ADD	= 11275,
    RISCV_ADDI	= 11276,
    RISCV_ADDIW	= 11277,
    RISCV_ADDW	= 11278,
    RISCV_ADD_UW	= 11279,
    RISCV_AES32DSI	= 11280,
    RISCV_AES32DSMI	= 11281,
    RISCV_AES32ESI	= 11282,
    RISCV_AES32ESMI	= 11283,
    RISCV_AES64DS	= 11284,
    RISCV_AES64DSM	= 11285,
    RISCV_AES64ES	= 11286,
    RISCV_AES64ESM	= 11287,
    RISCV_AES64IM	= 11288,
    RISCV_AES64KS1I	= 11289,
    RISCV_AES64KS2	= 11290,
    RISCV_AMOADD_D	= 11291,
    RISCV_AMOADD_D_AQ	= 11292,
    RISCV_AMOADD_D_AQ_RL	= 11293,
    RISCV_AMOADD_D_RL	= 11294,
    RISCV_AMOADD_W	= 11295,
    RISCV_AMOADD_W_AQ	= 11296,
    RISCV_AMOADD_W_AQ_RL	= 11297,
    RISCV_AMOADD_W_RL	= 11298,
    RISCV_AMOAND_D	= 11299,
    RISCV_AMOAND_D_AQ	= 11300,
    RISCV_AMOAND_D_AQ_RL	= 11301,
    RISCV_AMOAND_D_RL	= 11302,
    RISCV_AMOAND_W	= 11303,
    RISCV_AMOAND_W_AQ	= 11304,
    RISCV_AMOAND_W_AQ_RL	= 11305,
    RISCV_AMOAND_W_RL	= 11306,
    RISCV_AMOCAS_D_RV32	= 11307,
    RISCV_AMOCAS_D_RV32_AQ	= 11308,
    RISCV_AMOCAS_D_RV32_AQ_RL	= 11309,
    RISCV_AMOCAS_D_RV32_RL	= 11310,
    RISCV_AMOCAS_D_RV64	= 11311,
    RISCV_AMOCAS_D_RV64_AQ	= 11312,
    RISCV_AMOCAS_D_RV64_AQ_RL	= 11313,
    RISCV_AMOCAS_D_RV64_RL	= 11314,
    RISCV_AMOCAS_Q	= 11315,
    RISCV_AMOCAS_Q_AQ	= 11316,
    RISCV_AMOCAS_Q_AQ_RL	= 11317,
    RISCV_AMOCAS_Q_RL	= 11318,
    RISCV_AMOCAS_W	= 11319,
    RISCV_AMOCAS_W_AQ	= 11320,
    RISCV_AMOCAS_W_AQ_RL	= 11321,
    RISCV_AMOCAS_W_RL	= 11322,
    RISCV_AMOMAXU_D	= 11323,
    RISCV_AMOMAXU_D_AQ	= 11324,
    RISCV_AMOMAXU_D_AQ_RL	= 11325,
    RISCV_AMOMAXU_D_RL	= 11326,
    RISCV_AMOMAXU_W	= 11327,
    RISCV_AMOMAXU_W_AQ	= 11328,
    RISCV_AMOMAXU_W_AQ_RL	= 11329,
    RISCV_AMOMAXU_W_RL	= 11330,
    RISCV_AMOMAX_D	= 11331,
    RISCV_AMOMAX_D_AQ	= 11332,
    RISCV_AMOMAX_D_AQ_RL	= 11333,
    RISCV_AMOMAX_D_RL	= 11334,
    RISCV_AMOMAX_W	= 11335,
    RISCV_AMOMAX_W_AQ	= 11336,
    RISCV_AMOMAX_W_AQ_RL	= 11337,
    RISCV_AMOMAX_W_RL	= 11338,
    RISCV_AMOMINU_D	= 11339,
    RISCV_AMOMINU_D_AQ	= 11340,
    RISCV_AMOMINU_D_AQ_RL	= 11341,
    RISCV_AMOMINU_D_RL	= 11342,
    RISCV_AMOMINU_W	= 11343,
    RISCV_AMOMINU_W_AQ	= 11344,
    RISCV_AMOMINU_W_AQ_RL	= 11345,
    RISCV_AMOMINU_W_RL	= 11346,
    RISCV_AMOMIN_D	= 11347,
    RISCV_AMOMIN_D_AQ	= 11348,
    RISCV_AMOMIN_D_AQ_RL	= 11349,
    RISCV_AMOMIN_D_RL	= 11350,
    RISCV_AMOMIN_W	= 11351,
    RISCV_AMOMIN_W_AQ	= 11352,
    RISCV_AMOMIN_W_AQ_RL	= 11353,
    RISCV_AMOMIN_W_RL	= 11354,
    RISCV_AMOOR_D	= 11355,
    RISCV_AMOOR_D_AQ	= 11356,
    RISCV_AMOOR_D_AQ_RL	= 11357,
    RISCV_AMOOR_D_RL	= 11358,
    RISCV_AMOOR_W	= 11359,
    RISCV_AMOOR_W_AQ	= 11360,
    RISCV_AMOOR_W_AQ_RL	= 11361,
    RISCV_AMOOR_W_RL	= 11362,
    RISCV_AMOSWAP_D	= 11363,
    RISCV_AMOSWAP_D_AQ	= 11364,
    RISCV_AMOSWAP_D_AQ_RL	= 11365,
    RISCV_AMOSWAP_D_RL	= 11366,
    RISCV_AMOSWAP_W	= 11367,
    RISCV_AMOSWAP_W_AQ	= 11368,
    RISCV_AMOSWAP_W_AQ_RL	= 11369,
    RISCV_AMOSWAP_W_RL	= 11370,
    RISCV_AMOXOR_D	= 11371,
    RISCV_AMOXOR_D_AQ	= 11372,
    RISCV_AMOXOR_D_AQ_RL	= 11373,
    RISCV_AMOXOR_D_RL	= 11374,
    RISCV_AMOXOR_W	= 11375,
    RISCV_AMOXOR_W_AQ	= 11376,
    RISCV_AMOXOR_W_AQ_RL	= 11377,
    RISCV_AMOXOR_W_RL	= 11378,
    RISCV_AND	= 11379,
    RISCV_ANDI	= 11380,
    RISCV_ANDN	= 11381,
    RISCV_AUIPC	= 11382,
    RISCV_BCLR	= 11383,
    RISCV_BCLRI	= 11384,
    RISCV_BEQ	= 11385,
    RISCV_BEXT	= 11386,
    RISCV_BEXTI	= 11387,
    RISCV_BGE	= 11388,
    RISCV_BGEU	= 11389,
    RISCV_BINV	= 11390,
    RISCV_BINVI	= 11391,
    RISCV_BLT	= 11392,
    RISCV_BLTU	= 11393,
    RISCV_BNE	= 11394,
    RISCV_BREV8	= 11395,
    RISCV_BSET	= 11396,
    RISCV_BSETI	= 11397,
    RISCV_CBO_CLEAN	= 11398,
    RISCV_CBO_FLUSH	= 11399,
    RISCV_CBO_INVAL	= 11400,
    RISCV_CBO_ZERO	= 11401,
    RISCV_CLMUL	= 11402,
    RISCV_CLMULH	= 11403,
    RISCV_CLMULR	= 11404,
    RISCV_CLZ	= 11405,
    RISCV_CLZW	= 11406,
    RISCV_CMOP1	= 11407,
    RISCV_CMOP11	= 11408,
    RISCV_CMOP13	= 11409,
    RISCV_CMOP15	= 11410,
    RISCV_CMOP3	= 11411,
    RISCV_CMOP5	= 11412,
    RISCV_CMOP7	= 11413,
    RISCV_CMOP9	= 11414,
    RISCV_CM_JALT	= 11415,
    RISCV_CM_JT	= 11416,
    RISCV_CM_MVA01S	= 11417,
    RISCV_CM_MVSA01	= 11418,
    RISCV_CM_POP	= 11419,
    RISCV_CM_POPRET	= 11420,
    RISCV_CM_POPRETZ	= 11421,
    RISCV_CM_PUSH	= 11422,
    RISCV_CPOP	= 11423,
    RISCV_CPOPW	= 11424,
    RISCV_CSRRC	= 11425,
    RISCV_CSRRCI	= 11426,
    RISCV_CSRRS	= 11427,
    RISCV_CSRRSI	= 11428,
    RISCV_CSRRW	= 11429,
    RISCV_CSRRWI	= 11430,
    RISCV_CTZ	= 11431,
    RISCV_CTZW	= 11432,
    RISCV_CV_ABS	= 11433,
    RISCV_CV_ABS_B	= 11434,
    RISCV_CV_ABS_H	= 11435,
    RISCV_CV_ADDN	= 11436,
    RISCV_CV_ADDNR	= 11437,
    RISCV_CV_ADDRN	= 11438,
    RISCV_CV_ADDRNR	= 11439,
    RISCV_CV_ADDUN	= 11440,
    RISCV_CV_ADDUNR	= 11441,
    RISCV_CV_ADDURN	= 11442,
    RISCV_CV_ADDURNR	= 11443,
    RISCV_CV_ADD_B	= 11444,
    RISCV_CV_ADD_DIV2	= 11445,
    RISCV_CV_ADD_DIV4	= 11446,
    RISCV_CV_ADD_DIV8	= 11447,
    RISCV_CV_ADD_H	= 11448,
    RISCV_CV_ADD_SCI_B	= 11449,
    RISCV_CV_ADD_SCI_H	= 11450,
    RISCV_CV_ADD_SC_B	= 11451,
    RISCV_CV_ADD_SC_H	= 11452,
    RISCV_CV_AND_B	= 11453,
    RISCV_CV_AND_H	= 11454,
    RISCV_CV_AND_SCI_B	= 11455,
    RISCV_CV_AND_SCI_H	= 11456,
    RISCV_CV_AND_SC_B	= 11457,
    RISCV_CV_AND_SC_H	= 11458,
    RISCV_CV_AVGU_B	= 11459,
    RISCV_CV_AVGU_H	= 11460,
    RISCV_CV_AVGU_SCI_B	= 11461,
    RISCV_CV_AVGU_SCI_H	= 11462,
    RISCV_CV_AVGU_SC_B	= 11463,
    RISCV_CV_AVGU_SC_H	= 11464,
    RISCV_CV_AVG_B	= 11465,
    RISCV_CV_AVG_H	= 11466,
    RISCV_CV_AVG_SCI_B	= 11467,
    RISCV_CV_AVG_SCI_H	= 11468,
    RISCV_CV_AVG_SC_B	= 11469,
    RISCV_CV_AVG_SC_H	= 11470,
    RISCV_CV_BCLR	= 11471,
    RISCV_CV_BCLRR	= 11472,
    RISCV_CV_BEQIMM	= 11473,
    RISCV_CV_BITREV	= 11474,
    RISCV_CV_BNEIMM	= 11475,
    RISCV_CV_BSET	= 11476,
    RISCV_CV_BSETR	= 11477,
    RISCV_CV_CLB	= 11478,
    RISCV_CV_CLIP	= 11479,
    RISCV_CV_CLIPR	= 11480,
    RISCV_CV_CLIPU	= 11481,
    RISCV_CV_CLIPUR	= 11482,
    RISCV_CV_CMPEQ_B	= 11483,
    RISCV_CV_CMPEQ_H	= 11484,
    RISCV_CV_CMPEQ_SCI_B	= 11485,
    RISCV_CV_CMPEQ_SCI_H	= 11486,
    RISCV_CV_CMPEQ_SC_B	= 11487,
    RISCV_CV_CMPEQ_SC_H	= 11488,
    RISCV_CV_CMPGEU_B	= 11489,
    RISCV_CV_CMPGEU_H	= 11490,
    RISCV_CV_CMPGEU_SCI_B	= 11491,
    RISCV_CV_CMPGEU_SCI_H	= 11492,
    RISCV_CV_CMPGEU_SC_B	= 11493,
    RISCV_CV_CMPGEU_SC_H	= 11494,
    RISCV_CV_CMPGE_B	= 11495,
    RISCV_CV_CMPGE_H	= 11496,
    RISCV_CV_CMPGE_SCI_B	= 11497,
    RISCV_CV_CMPGE_SCI_H	= 11498,
    RISCV_CV_CMPGE_SC_B	= 11499,
    RISCV_CV_CMPGE_SC_H	= 11500,
    RISCV_CV_CMPGTU_B	= 11501,
    RISCV_CV_CMPGTU_H	= 11502,
    RISCV_CV_CMPGTU_SCI_B	= 11503,
    RISCV_CV_CMPGTU_SCI_H	= 11504,
    RISCV_CV_CMPGTU_SC_B	= 11505,
    RISCV_CV_CMPGTU_SC_H	= 11506,
    RISCV_CV_CMPGT_B	= 11507,
    RISCV_CV_CMPGT_H	= 11508,
    RISCV_CV_CMPGT_SCI_B	= 11509,
    RISCV_CV_CMPGT_SCI_H	= 11510,
    RISCV_CV_CMPGT_SC_B	= 11511,
    RISCV_CV_CMPGT_SC_H	= 11512,
    RISCV_CV_CMPLEU_B	= 11513,
    RISCV_CV_CMPLEU_H	= 11514,
    RISCV_CV_CMPLEU_SCI_B	= 11515,
    RISCV_CV_CMPLEU_SCI_H	= 11516,
    RISCV_CV_CMPLEU_SC_B	= 11517,
    RISCV_CV_CMPLEU_SC_H	= 11518,
    RISCV_CV_CMPLE_B	= 11519,
    RISCV_CV_CMPLE_H	= 11520,
    RISCV_CV_CMPLE_SCI_B	= 11521,
    RISCV_CV_CMPLE_SCI_H	= 11522,
    RISCV_CV_CMPLE_SC_B	= 11523,
    RISCV_CV_CMPLE_SC_H	= 11524,
    RISCV_CV_CMPLTU_B	= 11525,
    RISCV_CV_CMPLTU_H	= 11526,
    RISCV_CV_CMPLTU_SCI_B	= 11527,
    RISCV_CV_CMPLTU_SCI_H	= 11528,
    RISCV_CV_CMPLTU_SC_B	= 11529,
    RISCV_CV_CMPLTU_SC_H	= 11530,
    RISCV_CV_CMPLT_B	= 11531,
    RISCV_CV_CMPLT_H	= 11532,
    RISCV_CV_CMPLT_SCI_B	= 11533,
    RISCV_CV_CMPLT_SCI_H	= 11534,
    RISCV_CV_CMPLT_SC_B	= 11535,
    RISCV_CV_CMPLT_SC_H	= 11536,
    RISCV_CV_CMPNE_B	= 11537,
    RISCV_CV_CMPNE_H	= 11538,
    RISCV_CV_CMPNE_SCI_B	= 11539,
    RISCV_CV_CMPNE_SCI_H	= 11540,
    RISCV_CV_CMPNE_SC_B	= 11541,
    RISCV_CV_CMPNE_SC_H	= 11542,
    RISCV_CV_CNT	= 11543,
    RISCV_CV_CPLXCONJ	= 11544,
    RISCV_CV_CPLXMUL_I	= 11545,
    RISCV_CV_CPLXMUL_I_DIV2	= 11546,
    RISCV_CV_CPLXMUL_I_DIV4	= 11547,
    RISCV_CV_CPLXMUL_I_DIV8	= 11548,
    RISCV_CV_CPLXMUL_R	= 11549,
    RISCV_CV_CPLXMUL_R_DIV2	= 11550,
    RISCV_CV_CPLXMUL_R_DIV4	= 11551,
    RISCV_CV_CPLXMUL_R_DIV8	= 11552,
    RISCV_CV_DOTSP_B	= 11553,
    RISCV_CV_DOTSP_H	= 11554,
    RISCV_CV_DOTSP_SCI_B	= 11555,
    RISCV_CV_DOTSP_SCI_H	= 11556,
    RISCV_CV_DOTSP_SC_B	= 11557,
    RISCV_CV_DOTSP_SC_H	= 11558,
    RISCV_CV_DOTUP_B	= 11559,
    RISCV_CV_DOTUP_H	= 11560,
    RISCV_CV_DOTUP_SCI_B	= 11561,
    RISCV_CV_DOTUP_SCI_H	= 11562,
    RISCV_CV_DOTUP_SC_B	= 11563,
    RISCV_CV_DOTUP_SC_H	= 11564,
    RISCV_CV_DOTUSP_B	= 11565,
    RISCV_CV_DOTUSP_H	= 11566,
    RISCV_CV_DOTUSP_SCI_B	= 11567,
    RISCV_CV_DOTUSP_SCI_H	= 11568,
    RISCV_CV_DOTUSP_SC_B	= 11569,
    RISCV_CV_DOTUSP_SC_H	= 11570,
    RISCV_CV_ELW	= 11571,
    RISCV_CV_EXTBS	= 11572,
    RISCV_CV_EXTBZ	= 11573,
    RISCV_CV_EXTHS	= 11574,
    RISCV_CV_EXTHZ	= 11575,
    RISCV_CV_EXTRACT	= 11576,
    RISCV_CV_EXTRACTR	= 11577,
    RISCV_CV_EXTRACTU	= 11578,
    RISCV_CV_EXTRACTUR	= 11579,
    RISCV_CV_EXTRACTU_B	= 11580,
    RISCV_CV_EXTRACTU_H	= 11581,
    RISCV_CV_EXTRACT_B	= 11582,
    RISCV_CV_EXTRACT_H	= 11583,
    RISCV_CV_FF1	= 11584,
    RISCV_CV_FL1	= 11585,
    RISCV_CV_INSERT	= 11586,
    RISCV_CV_INSERTR	= 11587,
    RISCV_CV_INSERT_B	= 11588,
    RISCV_CV_INSERT_H	= 11589,
    RISCV_CV_LBU_ri_inc	= 11590,
    RISCV_CV_LBU_rr	= 11591,
    RISCV_CV_LBU_rr_inc	= 11592,
    RISCV_CV_LB_ri_inc	= 11593,
    RISCV_CV_LB_rr	= 11594,
    RISCV_CV_LB_rr_inc	= 11595,
    RISCV_CV_LHU_ri_inc	= 11596,
    RISCV_CV_LHU_rr	= 11597,
    RISCV_CV_LHU_rr_inc	= 11598,
    RISCV_CV_LH_ri_inc	= 11599,
    RISCV_CV_LH_rr	= 11600,
    RISCV_CV_LH_rr_inc	= 11601,
    RISCV_CV_LW_ri_inc	= 11602,
    RISCV_CV_LW_rr	= 11603,
    RISCV_CV_LW_rr_inc	= 11604,
    RISCV_CV_MAC	= 11605,
    RISCV_CV_MACHHSN	= 11606,
    RISCV_CV_MACHHSRN	= 11607,
    RISCV_CV_MACHHUN	= 11608,
    RISCV_CV_MACHHURN	= 11609,
    RISCV_CV_MACSN	= 11610,
    RISCV_CV_MACSRN	= 11611,
    RISCV_CV_MACUN	= 11612,
    RISCV_CV_MACURN	= 11613,
    RISCV_CV_MAX	= 11614,
    RISCV_CV_MAXU	= 11615,
    RISCV_CV_MAXU_B	= 11616,
    RISCV_CV_MAXU_H	= 11617,
    RISCV_CV_MAXU_SCI_B	= 11618,
    RISCV_CV_MAXU_SCI_H	= 11619,
    RISCV_CV_MAXU_SC_B	= 11620,
    RISCV_CV_MAXU_SC_H	= 11621,
    RISCV_CV_MAX_B	= 11622,
    RISCV_CV_MAX_H	= 11623,
    RISCV_CV_MAX_SCI_B	= 11624,
    RISCV_CV_MAX_SCI_H	= 11625,
    RISCV_CV_MAX_SC_B	= 11626,
    RISCV_CV_MAX_SC_H	= 11627,
    RISCV_CV_MIN	= 11628,
    RISCV_CV_MINU	= 11629,
    RISCV_CV_MINU_B	= 11630,
    RISCV_CV_MINU_H	= 11631,
    RISCV_CV_MINU_SCI_B	= 11632,
    RISCV_CV_MINU_SCI_H	= 11633,
    RISCV_CV_MINU_SC_B	= 11634,
    RISCV_CV_MINU_SC_H	= 11635,
    RISCV_CV_MIN_B	= 11636,
    RISCV_CV_MIN_H	= 11637,
    RISCV_CV_MIN_SCI_B	= 11638,
    RISCV_CV_MIN_SCI_H	= 11639,
    RISCV_CV_MIN_SC_B	= 11640,
    RISCV_CV_MIN_SC_H	= 11641,
    RISCV_CV_MSU	= 11642,
    RISCV_CV_MULHHSN	= 11643,
    RISCV_CV_MULHHSRN	= 11644,
    RISCV_CV_MULHHUN	= 11645,
    RISCV_CV_MULHHURN	= 11646,
    RISCV_CV_MULSN	= 11647,
    RISCV_CV_MULSRN	= 11648,
    RISCV_CV_MULUN	= 11649,
    RISCV_CV_MULURN	= 11650,
    RISCV_CV_OR_B	= 11651,
    RISCV_CV_OR_H	= 11652,
    RISCV_CV_OR_SCI_B	= 11653,
    RISCV_CV_OR_SCI_H	= 11654,
    RISCV_CV_OR_SC_B	= 11655,
    RISCV_CV_OR_SC_H	= 11656,
    RISCV_CV_PACK	= 11657,
    RISCV_CV_PACKHI_B	= 11658,
    RISCV_CV_PACKLO_B	= 11659,
    RISCV_CV_PACK_H	= 11660,
    RISCV_CV_ROR	= 11661,
    RISCV_CV_SB_ri_inc	= 11662,
    RISCV_CV_SB_rr	= 11663,
    RISCV_CV_SB_rr_inc	= 11664,
    RISCV_CV_SDOTSP_B	= 11665,
    RISCV_CV_SDOTSP_H	= 11666,
    RISCV_CV_SDOTSP_SCI_B	= 11667,
    RISCV_CV_SDOTSP_SCI_H	= 11668,
    RISCV_CV_SDOTSP_SC_B	= 11669,
    RISCV_CV_SDOTSP_SC_H	= 11670,
    RISCV_CV_SDOTUP_B	= 11671,
    RISCV_CV_SDOTUP_H	= 11672,
    RISCV_CV_SDOTUP_SCI_B	= 11673,
    RISCV_CV_SDOTUP_SCI_H	= 11674,
    RISCV_CV_SDOTUP_SC_B	= 11675,
    RISCV_CV_SDOTUP_SC_H	= 11676,
    RISCV_CV_SDOTUSP_B	= 11677,
    RISCV_CV_SDOTUSP_H	= 11678,
    RISCV_CV_SDOTUSP_SCI_B	= 11679,
    RISCV_CV_SDOTUSP_SCI_H	= 11680,
    RISCV_CV_SDOTUSP_SC_B	= 11681,
    RISCV_CV_SDOTUSP_SC_H	= 11682,
    RISCV_CV_SHUFFLE2_B	= 11683,
    RISCV_CV_SHUFFLE2_H	= 11684,
    RISCV_CV_SHUFFLEI0_SCI_B	= 11685,
    RISCV_CV_SHUFFLEI1_SCI_B	= 11686,
    RISCV_CV_SHUFFLEI2_SCI_B	= 11687,
    RISCV_CV_SHUFFLEI3_SCI_B	= 11688,
    RISCV_CV_SHUFFLE_B	= 11689,
    RISCV_CV_SHUFFLE_H	= 11690,
    RISCV_CV_SHUFFLE_SCI_H	= 11691,
    RISCV_CV_SH_ri_inc	= 11692,
    RISCV_CV_SH_rr	= 11693,
    RISCV_CV_SH_rr_inc	= 11694,
    RISCV_CV_SLET	= 11695,
    RISCV_CV_SLETU	= 11696,
    RISCV_CV_SLL_B	= 11697,
    RISCV_CV_SLL_H	= 11698,
    RISCV_CV_SLL_SCI_B	= 11699,
    RISCV_CV_SLL_SCI_H	= 11700,
    RISCV_CV_SLL_SC_B	= 11701,
    RISCV_CV_SLL_SC_H	= 11702,
    RISCV_CV_SRA_B	= 11703,
    RISCV_CV_SRA_H	= 11704,
    RISCV_CV_SRA_SCI_B	= 11705,
    RISCV_CV_SRA_SCI_H	= 11706,
    RISCV_CV_SRA_SC_B	= 11707,
    RISCV_CV_SRA_SC_H	= 11708,
    RISCV_CV_SRL_B	= 11709,
    RISCV_CV_SRL_H	= 11710,
    RISCV_CV_SRL_SCI_B	= 11711,
    RISCV_CV_SRL_SCI_H	= 11712,
    RISCV_CV_SRL_SC_B	= 11713,
    RISCV_CV_SRL_SC_H	= 11714,
    RISCV_CV_SUBN	= 11715,
    RISCV_CV_SUBNR	= 11716,
    RISCV_CV_SUBRN	= 11717,
    RISCV_CV_SUBRNR	= 11718,
    RISCV_CV_SUBROTMJ	= 11719,
    RISCV_CV_SUBROTMJ_DIV2	= 11720,
    RISCV_CV_SUBROTMJ_DIV4	= 11721,
    RISCV_CV_SUBROTMJ_DIV8	= 11722,
    RISCV_CV_SUBUN	= 11723,
    RISCV_CV_SUBUNR	= 11724,
    RISCV_CV_SUBURN	= 11725,
    RISCV_CV_SUBURNR	= 11726,
    RISCV_CV_SUB_B	= 11727,
    RISCV_CV_SUB_DIV2	= 11728,
    RISCV_CV_SUB_DIV4	= 11729,
    RISCV_CV_SUB_DIV8	= 11730,
    RISCV_CV_SUB_H	= 11731,
    RISCV_CV_SUB_SCI_B	= 11732,
    RISCV_CV_SUB_SCI_H	= 11733,
    RISCV_CV_SUB_SC_B	= 11734,
    RISCV_CV_SUB_SC_H	= 11735,
    RISCV_CV_SW_ri_inc	= 11736,
    RISCV_CV_SW_rr	= 11737,
    RISCV_CV_SW_rr_inc	= 11738,
    RISCV_CV_XOR_B	= 11739,
    RISCV_CV_XOR_H	= 11740,
    RISCV_CV_XOR_SCI_B	= 11741,
    RISCV_CV_XOR_SCI_H	= 11742,
    RISCV_CV_XOR_SC_B	= 11743,
    RISCV_CV_XOR_SC_H	= 11744,
    RISCV_CZERO_EQZ	= 11745,
    RISCV_CZERO_NEZ	= 11746,
    RISCV_C_ADD	= 11747,
    RISCV_C_ADDI	= 11748,
    RISCV_C_ADDI16SP	= 11749,
    RISCV_C_ADDI4SPN	= 11750,
    RISCV_C_ADDIW	= 11751,
    RISCV_C_ADDI_HINT_IMM_ZERO	= 11752,
    RISCV_C_ADDI_NOP	= 11753,
    RISCV_C_ADDW	= 11754,
    RISCV_C_ADD_HINT	= 11755,
    RISCV_C_AND	= 11756,
    RISCV_C_ANDI	= 11757,
    RISCV_C_BEQZ	= 11758,
    RISCV_C_BNEZ	= 11759,
    RISCV_C_EBREAK	= 11760,
    RISCV_C_FLD	= 11761,
    RISCV_C_FLDSP	= 11762,
    RISCV_C_FLW	= 11763,
    RISCV_C_FLWSP	= 11764,
    RISCV_C_FSD	= 11765,
    RISCV_C_FSDSP	= 11766,
    RISCV_C_FSW	= 11767,
    RISCV_C_FSWSP	= 11768,
    RISCV_C_J	= 11769,
    RISCV_C_JAL	= 11770,
    RISCV_C_JALR	= 11771,
    RISCV_C_JR	= 11772,
    RISCV_C_LBU	= 11773,
    RISCV_C_LD	= 11774,
    RISCV_C_LDSP	= 11775,
    RISCV_C_LH	= 11776,
    RISCV_C_LHU	= 11777,
    RISCV_C_LI	= 11778,
    RISCV_C_LI_HINT	= 11779,
    RISCV_C_LUI	= 11780,
    RISCV_C_LUI_HINT	= 11781,
    RISCV_C_LW	= 11782,
    RISCV_C_LWSP	= 11783,
    RISCV_C_MUL	= 11784,
    RISCV_C_MV	= 11785,
    RISCV_C_MV_HINT	= 11786,
    RISCV_C_NOP	= 11787,
    RISCV_C_NOP_HINT	= 11788,
    RISCV_C_NOT	= 11789,
    RISCV_C_OR	= 11790,
    RISCV_C_SB	= 11791,
    RISCV_C_SD	= 11792,
    RISCV_C_SDSP	= 11793,
    RISCV_C_SEXT_B	= 11794,
    RISCV_C_SEXT_H	= 11795,
    RISCV_C_SH	= 11796,
    RISCV_C_SLLI	= 11797,
    RISCV_C_SLLI64_HINT	= 11798,
    RISCV_C_SLLI_HINT	= 11799,
    RISCV_C_SRAI	= 11800,
    RISCV_C_SRAI64_HINT	= 11801,
    RISCV_C_SRLI	= 11802,
    RISCV_C_SRLI64_HINT	= 11803,
    RISCV_C_SSPOPCHK	= 11804,
    RISCV_C_SSPUSH	= 11805,
    RISCV_C_SUB	= 11806,
    RISCV_C_SUBW	= 11807,
    RISCV_C_SW	= 11808,
    RISCV_C_SWSP	= 11809,
    RISCV_C_UNIMP	= 11810,
    RISCV_C_XOR	= 11811,
    RISCV_C_ZEXT_B	= 11812,
    RISCV_C_ZEXT_H	= 11813,
    RISCV_C_ZEXT_W	= 11814,
    RISCV_DIV	= 11815,
    RISCV_DIVU	= 11816,
    RISCV_DIVUW	= 11817,
    RISCV_DIVW	= 11818,
    RISCV_DRET	= 11819,
    RISCV_EBREAK	= 11820,
    RISCV_ECALL	= 11821,
    RISCV_FADD_D	= 11822,
    RISCV_FADD_D_IN32X	= 11823,
    RISCV_FADD_D_INX	= 11824,
    RISCV_FADD_H	= 11825,
    RISCV_FADD_H_INX	= 11826,
    RISCV_FADD_S	= 11827,
    RISCV_FADD_S_INX	= 11828,
    RISCV_FCLASS_D	= 11829,
    RISCV_FCLASS_D_IN32X	= 11830,
    RISCV_FCLASS_D_INX	= 11831,
    RISCV_FCLASS_H	= 11832,
    RISCV_FCLASS_H_INX	= 11833,
    RISCV_FCLASS_S	= 11834,
    RISCV_FCLASS_S_INX	= 11835,
    RISCV_FCVTMOD_W_D	= 11836,
    RISCV_FCVT_BF16_S	= 11837,
    RISCV_FCVT_D_H	= 11838,
    RISCV_FCVT_D_H_IN32X	= 11839,
    RISCV_FCVT_D_H_INX	= 11840,
    RISCV_FCVT_D_L	= 11841,
    RISCV_FCVT_D_LU	= 11842,
    RISCV_FCVT_D_LU_INX	= 11843,
    RISCV_FCVT_D_L_INX	= 11844,
    RISCV_FCVT_D_S	= 11845,
    RISCV_FCVT_D_S_IN32X	= 11846,
    RISCV_FCVT_D_S_INX	= 11847,
    RISCV_FCVT_D_W	= 11848,
    RISCV_FCVT_D_WU	= 11849,
    RISCV_FCVT_D_WU_IN32X	= 11850,
    RISCV_FCVT_D_WU_INX	= 11851,
    RISCV_FCVT_D_W_IN32X	= 11852,
    RISCV_FCVT_D_W_INX	= 11853,
    RISCV_FCVT_H_D	= 11854,
    RISCV_FCVT_H_D_IN32X	= 11855,
    RISCV_FCVT_H_D_INX	= 11856,
    RISCV_FCVT_H_L	= 11857,
    RISCV_FCVT_H_LU	= 11858,
    RISCV_FCVT_H_LU_INX	= 11859,
    RISCV_FCVT_H_L_INX	= 11860,
    RISCV_FCVT_H_S	= 11861,
    RISCV_FCVT_H_S_INX	= 11862,
    RISCV_FCVT_H_W	= 11863,
    RISCV_FCVT_H_WU	= 11864,
    RISCV_FCVT_H_WU_INX	= 11865,
    RISCV_FCVT_H_W_INX	= 11866,
    RISCV_FCVT_LU_D	= 11867,
    RISCV_FCVT_LU_D_INX	= 11868,
    RISCV_FCVT_LU_H	= 11869,
    RISCV_FCVT_LU_H_INX	= 11870,
    RISCV_FCVT_LU_S	= 11871,
    RISCV_FCVT_LU_S_INX	= 11872,
    RISCV_FCVT_L_D	= 11873,
    RISCV_FCVT_L_D_INX	= 11874,
    RISCV_FCVT_L_H	= 11875,
    RISCV_FCVT_L_H_INX	= 11876,
    RISCV_FCVT_L_S	= 11877,
    RISCV_FCVT_L_S_INX	= 11878,
    RISCV_FCVT_S_BF16	= 11879,
    RISCV_FCVT_S_D	= 11880,
    RISCV_FCVT_S_D_IN32X	= 11881,
    RISCV_FCVT_S_D_INX	= 11882,
    RISCV_FCVT_S_H	= 11883,
    RISCV_FCVT_S_H_INX	= 11884,
    RISCV_FCVT_S_L	= 11885,
    RISCV_FCVT_S_LU	= 11886,
    RISCV_FCVT_S_LU_INX	= 11887,
    RISCV_FCVT_S_L_INX	= 11888,
    RISCV_FCVT_S_W	= 11889,
    RISCV_FCVT_S_WU	= 11890,
    RISCV_FCVT_S_WU_INX	= 11891,
    RISCV_FCVT_S_W_INX	= 11892,
    RISCV_FCVT_WU_D	= 11893,
    RISCV_FCVT_WU_D_IN32X	= 11894,
    RISCV_FCVT_WU_D_INX	= 11895,
    RISCV_FCVT_WU_H	= 11896,
    RISCV_FCVT_WU_H_INX	= 11897,
    RISCV_FCVT_WU_S	= 11898,
    RISCV_FCVT_WU_S_INX	= 11899,
    RISCV_FCVT_W_D	= 11900,
    RISCV_FCVT_W_D_IN32X	= 11901,
    RISCV_FCVT_W_D_INX	= 11902,
    RISCV_FCVT_W_H	= 11903,
    RISCV_FCVT_W_H_INX	= 11904,
    RISCV_FCVT_W_S	= 11905,
    RISCV_FCVT_W_S_INX	= 11906,
    RISCV_FDIV_D	= 11907,
    RISCV_FDIV_D_IN32X	= 11908,
    RISCV_FDIV_D_INX	= 11909,
    RISCV_FDIV_H	= 11910,
    RISCV_FDIV_H_INX	= 11911,
    RISCV_FDIV_S	= 11912,
    RISCV_FDIV_S_INX	= 11913,
    RISCV_FENCE	= 11914,
    RISCV_FENCE_I	= 11915,
    RISCV_FENCE_TSO	= 11916,
    RISCV_FEQ_D	= 11917,
    RISCV_FEQ_D_IN32X	= 11918,
    RISCV_FEQ_D_INX	= 11919,
    RISCV_FEQ_H	= 11920,
    RISCV_FEQ_H_INX	= 11921,
    RISCV_FEQ_S	= 11922,
    RISCV_FEQ_S_INX	= 11923,
    RISCV_FLD	= 11924,
    RISCV_FLEQ_D	= 11925,
    RISCV_FLEQ_H	= 11926,
    RISCV_FLEQ_S	= 11927,
    RISCV_FLE_D	= 11928,
    RISCV_FLE_D_IN32X	= 11929,
    RISCV_FLE_D_INX	= 11930,
    RISCV_FLE_H	= 11931,
    RISCV_FLE_H_INX	= 11932,
    RISCV_FLE_S	= 11933,
    RISCV_FLE_S_INX	= 11934,
    RISCV_FLH	= 11935,
    RISCV_FLI_D	= 11936,
    RISCV_FLI_H	= 11937,
    RISCV_FLI_S	= 11938,
    RISCV_FLTQ_D	= 11939,
    RISCV_FLTQ_H	= 11940,
    RISCV_FLTQ_S	= 11941,
    RISCV_FLT_D	= 11942,
    RISCV_FLT_D_IN32X	= 11943,
    RISCV_FLT_D_INX	= 11944,
    RISCV_FLT_H	= 11945,
    RISCV_FLT_H_INX	= 11946,
    RISCV_FLT_S	= 11947,
    RISCV_FLT_S_INX	= 11948,
    RISCV_FLW	= 11949,
    RISCV_FMADD_D	= 11950,
    RISCV_FMADD_D_IN32X	= 11951,
    RISCV_FMADD_D_INX	= 11952,
    RISCV_FMADD_H	= 11953,
    RISCV_FMADD_H_INX	= 11954,
    RISCV_FMADD_S	= 11955,
    RISCV_FMADD_S_INX	= 11956,
    RISCV_FMAXM_D	= 11957,
    RISCV_FMAXM_H	= 11958,
    RISCV_FMAXM_S	= 11959,
    RISCV_FMAX_D	= 11960,
    RISCV_FMAX_D_IN32X	= 11961,
    RISCV_FMAX_D_INX	= 11962,
    RISCV_FMAX_H	= 11963,
    RISCV_FMAX_H_INX	= 11964,
    RISCV_FMAX_S	= 11965,
    RISCV_FMAX_S_INX	= 11966,
    RISCV_FMINM_D	= 11967,
    RISCV_FMINM_H	= 11968,
    RISCV_FMINM_S	= 11969,
    RISCV_FMIN_D	= 11970,
    RISCV_FMIN_D_IN32X	= 11971,
    RISCV_FMIN_D_INX	= 11972,
    RISCV_FMIN_H	= 11973,
    RISCV_FMIN_H_INX	= 11974,
    RISCV_FMIN_S	= 11975,
    RISCV_FMIN_S_INX	= 11976,
    RISCV_FMSUB_D	= 11977,
    RISCV_FMSUB_D_IN32X	= 11978,
    RISCV_FMSUB_D_INX	= 11979,
    RISCV_FMSUB_H	= 11980,
    RISCV_FMSUB_H_INX	= 11981,
    RISCV_FMSUB_S	= 11982,
    RISCV_FMSUB_S_INX	= 11983,
    RISCV_FMUL_D	= 11984,
    RISCV_FMUL_D_IN32X	= 11985,
    RISCV_FMUL_D_INX	= 11986,
    RISCV_FMUL_H	= 11987,
    RISCV_FMUL_H_INX	= 11988,
    RISCV_FMUL_S	= 11989,
    RISCV_FMUL_S_INX	= 11990,
    RISCV_FMVH_X_D	= 11991,
    RISCV_FMVP_D_X	= 11992,
    RISCV_FMV_D_X	= 11993,
    RISCV_FMV_H_X	= 11994,
    RISCV_FMV_W_X	= 11995,
    RISCV_FMV_X_D	= 11996,
    RISCV_FMV_X_H	= 11997,
    RISCV_FMV_X_W	= 11998,
    RISCV_FMV_X_W_FPR64	= 11999,
    RISCV_FNMADD_D	= 12000,
    RISCV_FNMADD_D_IN32X	= 12001,
    RISCV_FNMADD_D_INX	= 12002,
    RISCV_FNMADD_H	= 12003,
    RISCV_FNMADD_H_INX	= 12004,
    RISCV_FNMADD_S	= 12005,
    RISCV_FNMADD_S_INX	= 12006,
    RISCV_FNMSUB_D	= 12007,
    RISCV_FNMSUB_D_IN32X	= 12008,
    RISCV_FNMSUB_D_INX	= 12009,
    RISCV_FNMSUB_H	= 12010,
    RISCV_FNMSUB_H_INX	= 12011,
    RISCV_FNMSUB_S	= 12012,
    RISCV_FNMSUB_S_INX	= 12013,
    RISCV_FROUNDNX_D	= 12014,
    RISCV_FROUNDNX_H	= 12015,
    RISCV_FROUNDNX_S	= 12016,
    RISCV_FROUND_D	= 12017,
    RISCV_FROUND_H	= 12018,
    RISCV_FROUND_S	= 12019,
    RISCV_FSD	= 12020,
    RISCV_FSGNJN_D	= 12021,
    RISCV_FSGNJN_D_IN32X	= 12022,
    RISCV_FSGNJN_D_INX	= 12023,
    RISCV_FSGNJN_H	= 12024,
    RISCV_FSGNJN_H_INX	= 12025,
    RISCV_FSGNJN_S	= 12026,
    RISCV_FSGNJN_S_INX	= 12027,
    RISCV_FSGNJX_D	= 12028,
    RISCV_FSGNJX_D_IN32X	= 12029,
    RISCV_FSGNJX_D_INX	= 12030,
    RISCV_FSGNJX_H	= 12031,
    RISCV_FSGNJX_H_INX	= 12032,
    RISCV_FSGNJX_S	= 12033,
    RISCV_FSGNJX_S_INX	= 12034,
    RISCV_FSGNJ_D	= 12035,
    RISCV_FSGNJ_D_IN32X	= 12036,
    RISCV_FSGNJ_D_INX	= 12037,
    RISCV_FSGNJ_H	= 12038,
    RISCV_FSGNJ_H_INX	= 12039,
    RISCV_FSGNJ_S	= 12040,
    RISCV_FSGNJ_S_INX	= 12041,
    RISCV_FSH	= 12042,
    RISCV_FSQRT_D	= 12043,
    RISCV_FSQRT_D_IN32X	= 12044,
    RISCV_FSQRT_D_INX	= 12045,
    RISCV_FSQRT_H	= 12046,
    RISCV_FSQRT_H_INX	= 12047,
    RISCV_FSQRT_S	= 12048,
    RISCV_FSQRT_S_INX	= 12049,
    RISCV_FSUB_D	= 12050,
    RISCV_FSUB_D_IN32X	= 12051,
    RISCV_FSUB_D_INX	= 12052,
    RISCV_FSUB_H	= 12053,
    RISCV_FSUB_H_INX	= 12054,
    RISCV_FSUB_S	= 12055,
    RISCV_FSUB_S_INX	= 12056,
    RISCV_FSW	= 12057,
    RISCV_HFENCE_GVMA	= 12058,
    RISCV_HFENCE_VVMA	= 12059,
    RISCV_HINVAL_GVMA	= 12060,
    RISCV_HINVAL_VVMA	= 12061,
    RISCV_HLVX_HU	= 12062,
    RISCV_HLVX_WU	= 12063,
    RISCV_HLV_B	= 12064,
    RISCV_HLV_BU	= 12065,
    RISCV_HLV_D	= 12066,
    RISCV_HLV_H	= 12067,
    RISCV_HLV_HU	= 12068,
    RISCV_HLV_W	= 12069,
    RISCV_HLV_WU	= 12070,
    RISCV_HSV_B	= 12071,
    RISCV_HSV_D	= 12072,
    RISCV_HSV_H	= 12073,
    RISCV_HSV_W	= 12074,
    RISCV_InsnB	= 12075,
    RISCV_InsnCA	= 12076,
    RISCV_InsnCB	= 12077,
    RISCV_InsnCI	= 12078,
    RISCV_InsnCIW	= 12079,
    RISCV_InsnCJ	= 12080,
    RISCV_InsnCL	= 12081,
    RISCV_InsnCR	= 12082,
    RISCV_InsnCS	= 12083,
    RISCV_InsnCSS	= 12084,
    RISCV_InsnI	= 12085,
    RISCV_InsnI_Mem	= 12086,
    RISCV_InsnJ	= 12087,
    RISCV_InsnR	= 12088,
    RISCV_InsnR4	= 12089,
    RISCV_InsnS	= 12090,
    RISCV_InsnU	= 12091,
    RISCV_JAL	= 12092,
    RISCV_JALR	= 12093,
    RISCV_LB	= 12094,
    RISCV_LBU	= 12095,
    RISCV_LD	= 12096,
    RISCV_LH	= 12097,
    RISCV_LHU	= 12098,
    RISCV_LR_D	= 12099,
    RISCV_LR_D_AQ	= 12100,
    RISCV_LR_D_AQ_RL	= 12101,
    RISCV_LR_D_RL	= 12102,
    RISCV_LR_W	= 12103,
    RISCV_LR_W_AQ	= 12104,
    RISCV_LR_W_AQ_RL	= 12105,
    RISCV_LR_W_RL	= 12106,
    RISCV_LUI	= 12107,
    RISCV_LW	= 12108,
    RISCV_LWU	= 12109,
    RISCV_MAX	= 12110,
    RISCV_MAXU	= 12111,
    RISCV_MIN	= 12112,
    RISCV_MINU	= 12113,
    RISCV_MOPR0	= 12114,
    RISCV_MOPR1	= 12115,
    RISCV_MOPR10	= 12116,
    RISCV_MOPR11	= 12117,
    RISCV_MOPR12	= 12118,
    RISCV_MOPR13	= 12119,
    RISCV_MOPR14	= 12120,
    RISCV_MOPR15	= 12121,
    RISCV_MOPR16	= 12122,
    RISCV_MOPR17	= 12123,
    RISCV_MOPR18	= 12124,
    RISCV_MOPR19	= 12125,
    RISCV_MOPR2	= 12126,
    RISCV_MOPR20	= 12127,
    RISCV_MOPR21	= 12128,
    RISCV_MOPR22	= 12129,
    RISCV_MOPR23	= 12130,
    RISCV_MOPR24	= 12131,
    RISCV_MOPR25	= 12132,
    RISCV_MOPR26	= 12133,
    RISCV_MOPR27	= 12134,
    RISCV_MOPR28	= 12135,
    RISCV_MOPR29	= 12136,
    RISCV_MOPR3	= 12137,
    RISCV_MOPR30	= 12138,
    RISCV_MOPR31	= 12139,
    RISCV_MOPR4	= 12140,
    RISCV_MOPR5	= 12141,
    RISCV_MOPR6	= 12142,
    RISCV_MOPR7	= 12143,
    RISCV_MOPR8	= 12144,
    RISCV_MOPR9	= 12145,
    RISCV_MOPRR0	= 12146,
    RISCV_MOPRR1	= 12147,
    RISCV_MOPRR2	= 12148,
    RISCV_MOPRR3	= 12149,
    RISCV_MOPRR4	= 12150,
    RISCV_MOPRR5	= 12151,
    RISCV_MOPRR6	= 12152,
    RISCV_MOPRR7	= 12153,
    RISCV_MRET	= 12154,
    RISCV_MUL	= 12155,
    RISCV_MULH	= 12156,
    RISCV_MULHSU	= 12157,
    RISCV_MULHU	= 12158,
    RISCV_MULW	= 12159,
    RISCV_OR	= 12160,
    RISCV_ORC_B	= 12161,
    RISCV_ORI	= 12162,
    RISCV_ORN	= 12163,
    RISCV_PACK	= 12164,
    RISCV_PACKH	= 12165,
    RISCV_PACKW	= 12166,
    RISCV_PREFETCH_I	= 12167,
    RISCV_PREFETCH_R	= 12168,
    RISCV_PREFETCH_W	= 12169,
    RISCV_REM	= 12170,
    RISCV_REMU	= 12171,
    RISCV_REMUW	= 12172,
    RISCV_REMW	= 12173,
    RISCV_REV8_RV32	= 12174,
    RISCV_REV8_RV64	= 12175,
    RISCV_ROL	= 12176,
    RISCV_ROLW	= 12177,
    RISCV_ROR	= 12178,
    RISCV_RORI	= 12179,
    RISCV_RORIW	= 12180,
    RISCV_RORW	= 12181,
    RISCV_SB	= 12182,
    RISCV_SC_D	= 12183,
    RISCV_SC_D_AQ	= 12184,
    RISCV_SC_D_AQ_RL	= 12185,
    RISCV_SC_D_RL	= 12186,
    RISCV_SC_W	= 12187,
    RISCV_SC_W_AQ	= 12188,
    RISCV_SC_W_AQ_RL	= 12189,
    RISCV_SC_W_RL	= 12190,
    RISCV_SD	= 12191,
    RISCV_SEXT_B	= 12192,
    RISCV_SEXT_H	= 12193,
    RISCV_SFENCE_INVAL_IR	= 12194,
    RISCV_SFENCE_VMA	= 12195,
    RISCV_SFENCE_W_INVAL	= 12196,
    RISCV_SH	= 12197,
    RISCV_SH1ADD	= 12198,
    RISCV_SH1ADD_UW	= 12199,
    RISCV_SH2ADD	= 12200,
    RISCV_SH2ADD_UW	= 12201,
    RISCV_SH3ADD	= 12202,
    RISCV_SH3ADD_UW	= 12203,
    RISCV_SHA256SIG0	= 12204,
    RISCV_SHA256SIG1	= 12205,
    RISCV_SHA256SUM0	= 12206,
    RISCV_SHA256SUM1	= 12207,
    RISCV_SHA512SIG0	= 12208,
    RISCV_SHA512SIG0H	= 12209,
    RISCV_SHA512SIG0L	= 12210,
    RISCV_SHA512SIG1	= 12211,
    RISCV_SHA512SIG1H	= 12212,
    RISCV_SHA512SIG1L	= 12213,
    RISCV_SHA512SUM0	= 12214,
    RISCV_SHA512SUM0R	= 12215,
    RISCV_SHA512SUM1	= 12216,
    RISCV_SHA512SUM1R	= 12217,
    RISCV_SINVAL_VMA	= 12218,
    RISCV_SLL	= 12219,
    RISCV_SLLI	= 12220,
    RISCV_SLLIW	= 12221,
    RISCV_SLLI_UW	= 12222,
    RISCV_SLLW	= 12223,
    RISCV_SLT	= 12224,
    RISCV_SLTI	= 12225,
    RISCV_SLTIU	= 12226,
    RISCV_SLTU	= 12227,
    RISCV_SM3P0	= 12228,
    RISCV_SM3P1	= 12229,
    RISCV_SM4ED	= 12230,
    RISCV_SM4KS	= 12231,
    RISCV_SRA	= 12232,
    RISCV_SRAI	= 12233,
    RISCV_SRAIW	= 12234,
    RISCV_SRAW	= 12235,
    RISCV_SRET	= 12236,
    RISCV_SRL	= 12237,
    RISCV_SRLI	= 12238,
    RISCV_SRLIW	= 12239,
    RISCV_SRLW	= 12240,
    RISCV_SSAMOSWAP_D	= 12241,
    RISCV_SSAMOSWAP_D_AQ	= 12242,
    RISCV_SSAMOSWAP_D_AQ_RL	= 12243,
    RISCV_SSAMOSWAP_D_RL	= 12244,
    RISCV_SSAMOSWAP_W	= 12245,
    RISCV_SSAMOSWAP_W_AQ	= 12246,
    RISCV_SSAMOSWAP_W_AQ_RL	= 12247,
    RISCV_SSAMOSWAP_W_RL	= 12248,
    RISCV_SSPOPCHK	= 12249,
    RISCV_SSPUSH	= 12250,
    RISCV_SSRDP	= 12251,
    RISCV_SUB	= 12252,
    RISCV_SUBW	= 12253,
    RISCV_SW	= 12254,
    RISCV_THVdotVMAQASU_VV	= 12255,
    RISCV_THVdotVMAQASU_VX	= 12256,
    RISCV_THVdotVMAQAUS_VX	= 12257,
    RISCV_THVdotVMAQAU_VV	= 12258,
    RISCV_THVdotVMAQAU_VX	= 12259,
    RISCV_THVdotVMAQA_VV	= 12260,
    RISCV_THVdotVMAQA_VX	= 12261,
    RISCV_TH_ADDSL	= 12262,
    RISCV_TH_DCACHE_CALL	= 12263,
    RISCV_TH_DCACHE_CIALL	= 12264,
    RISCV_TH_DCACHE_CIPA	= 12265,
    RISCV_TH_DCACHE_CISW	= 12266,
    RISCV_TH_DCACHE_CIVA	= 12267,
    RISCV_TH_DCACHE_CPA	= 12268,
    RISCV_TH_DCACHE_CPAL1	= 12269,
    RISCV_TH_DCACHE_CSW	= 12270,
    RISCV_TH_DCACHE_CVA	= 12271,
    RISCV_TH_DCACHE_CVAL1	= 12272,
    RISCV_TH_DCACHE_IALL	= 12273,
    RISCV_TH_DCACHE_IPA	= 12274,
    RISCV_TH_DCACHE_ISW	= 12275,
    RISCV_TH_DCACHE_IVA	= 12276,
    RISCV_TH_EXT	= 12277,
    RISCV_TH_EXTU	= 12278,
    RISCV_TH_FF0	= 12279,
    RISCV_TH_FF1	= 12280,
    RISCV_TH_FLRD	= 12281,
    RISCV_TH_FLRW	= 12282,
    RISCV_TH_FLURD	= 12283,
    RISCV_TH_FLURW	= 12284,
    RISCV_TH_FSRD	= 12285,
    RISCV_TH_FSRW	= 12286,
    RISCV_TH_FSURD	= 12287,
    RISCV_TH_FSURW	= 12288,
    RISCV_TH_ICACHE_IALL	= 12289,
    RISCV_TH_ICACHE_IALLS	= 12290,
    RISCV_TH_ICACHE_IPA	= 12291,
    RISCV_TH_ICACHE_IVA	= 12292,
    RISCV_TH_L2CACHE_CALL	= 12293,
    RISCV_TH_L2CACHE_CIALL	= 12294,
    RISCV_TH_L2CACHE_IALL	= 12295,
    RISCV_TH_LBIA	= 12296,
    RISCV_TH_LBIB	= 12297,
    RISCV_TH_LBUIA	= 12298,
    RISCV_TH_LBUIB	= 12299,
    RISCV_TH_LDD	= 12300,
    RISCV_TH_LDIA	= 12301,
    RISCV_TH_LDIB	= 12302,
    RISCV_TH_LHIA	= 12303,
    RISCV_TH_LHIB	= 12304,
    RISCV_TH_LHUIA	= 12305,
    RISCV_TH_LHUIB	= 12306,
    RISCV_TH_LRB	= 12307,
    RISCV_TH_LRBU	= 12308,
    RISCV_TH_LRD	= 12309,
    RISCV_TH_LRH	= 12310,
    RISCV_TH_LRHU	= 12311,
    RISCV_TH_LRW	= 12312,
    RISCV_TH_LRWU	= 12313,
    RISCV_TH_LURB	= 12314,
    RISCV_TH_LURBU	= 12315,
    RISCV_TH_LURD	= 12316,
    RISCV_TH_LURH	= 12317,
    RISCV_TH_LURHU	= 12318,
    RISCV_TH_LURW	= 12319,
    RISCV_TH_LURWU	= 12320,
    RISCV_TH_LWD	= 12321,
    RISCV_TH_LWIA	= 12322,
    RISCV_TH_LWIB	= 12323,
    RISCV_TH_LWUD	= 12324,
    RISCV_TH_LWUIA	= 12325,
    RISCV_TH_LWUIB	= 12326,
    RISCV_TH_MULA	= 12327,
    RISCV_TH_MULAH	= 12328,
    RISCV_TH_MULAW	= 12329,
    RISCV_TH_MULS	= 12330,
    RISCV_TH_MULSH	= 12331,
    RISCV_TH_MULSW	= 12332,
    RISCV_TH_MVEQZ	= 12333,
    RISCV_TH_MVNEZ	= 12334,
    RISCV_TH_REV	= 12335,
    RISCV_TH_REVW	= 12336,
    RISCV_TH_SBIA	= 12337,
    RISCV_TH_SBIB	= 12338,
    RISCV_TH_SDD	= 12339,
    RISCV_TH_SDIA	= 12340,
    RISCV_TH_SDIB	= 12341,
    RISCV_TH_SFENCE_VMAS	= 12342,
    RISCV_TH_SHIA	= 12343,
    RISCV_TH_SHIB	= 12344,
    RISCV_TH_SRB	= 12345,
    RISCV_TH_SRD	= 12346,
    RISCV_TH_SRH	= 12347,
    RISCV_TH_SRRI	= 12348,
    RISCV_TH_SRRIW	= 12349,
    RISCV_TH_SRW	= 12350,
    RISCV_TH_SURB	= 12351,
    RISCV_TH_SURD	= 12352,
    RISCV_TH_SURH	= 12353,
    RISCV_TH_SURW	= 12354,
    RISCV_TH_SWD	= 12355,
    RISCV_TH_SWIA	= 12356,
    RISCV_TH_SWIB	= 12357,
    RISCV_TH_SYNC	= 12358,
    RISCV_TH_SYNC_I	= 12359,
    RISCV_TH_SYNC_IS	= 12360,
    RISCV_TH_SYNC_S	= 12361,
    RISCV_TH_TST	= 12362,
    RISCV_TH_TSTNBZ	= 12363,
    RISCV_UNIMP	= 12364,
    RISCV_UNZIP_RV32	= 12365,
    RISCV_VAADDU_VV	= 12366,
    RISCV_VAADDU_VX	= 12367,
    RISCV_VAADD_VV	= 12368,
    RISCV_VAADD_VX	= 12369,
    RISCV_VADC_VIM	= 12370,
    RISCV_VADC_VVM	= 12371,
    RISCV_VADC_VXM	= 12372,
    RISCV_VADD_VI	= 12373,
    RISCV_VADD_VV	= 12374,
    RISCV_VADD_VX	= 12375,
    RISCV_VAESDF_VS	= 12376,
    RISCV_VAESDF_VV	= 12377,
    RISCV_VAESDM_VS	= 12378,
    RISCV_VAESDM_VV	= 12379,
    RISCV_VAESEF_VS	= 12380,
    RISCV_VAESEF_VV	= 12381,
    RISCV_VAESEM_VS	= 12382,
    RISCV_VAESEM_VV	= 12383,
    RISCV_VAESKF1_VI	= 12384,
    RISCV_VAESKF2_VI	= 12385,
    RISCV_VAESZ_VS	= 12386,
    RISCV_VANDN_VV	= 12387,
    RISCV_VANDN_VX	= 12388,
    RISCV_VAND_VI	= 12389,
    RISCV_VAND_VV	= 12390,
    RISCV_VAND_VX	= 12391,
    RISCV_VASUBU_VV	= 12392,
    RISCV_VASUBU_VX	= 12393,
    RISCV_VASUB_VV	= 12394,
    RISCV_VASUB_VX	= 12395,
    RISCV_VBREV8_V	= 12396,
    RISCV_VBREV_V	= 12397,
    RISCV_VCLMULH_VV	= 12398,
    RISCV_VCLMULH_VX	= 12399,
    RISCV_VCLMUL_VV	= 12400,
    RISCV_VCLMUL_VX	= 12401,
    RISCV_VCLZ_V	= 12402,
    RISCV_VCOMPRESS_VM	= 12403,
    RISCV_VCPOP_M	= 12404,
    RISCV_VCPOP_V	= 12405,
    RISCV_VCTZ_V	= 12406,
    RISCV_VC_FV	= 12407,
    RISCV_VC_FVV	= 12408,
    RISCV_VC_FVW	= 12409,
    RISCV_VC_I	= 12410,
    RISCV_VC_IV	= 12411,
    RISCV_VC_IVV	= 12412,
    RISCV_VC_IVW	= 12413,
    RISCV_VC_VV	= 12414,
    RISCV_VC_VVV	= 12415,
    RISCV_VC_VVW	= 12416,
    RISCV_VC_V_FV	= 12417,
    RISCV_VC_V_FVV	= 12418,
    RISCV_VC_V_FVW	= 12419,
    RISCV_VC_V_I	= 12420,
    RISCV_VC_V_IV	= 12421,
    RISCV_VC_V_IVV	= 12422,
    RISCV_VC_V_IVW	= 12423,
    RISCV_VC_V_VV	= 12424,
    RISCV_VC_V_VVV	= 12425,
    RISCV_VC_V_VVW	= 12426,
    RISCV_VC_V_X	= 12427,
    RISCV_VC_V_XV	= 12428,
    RISCV_VC_V_XVV	= 12429,
    RISCV_VC_V_XVW	= 12430,
    RISCV_VC_X	= 12431,
    RISCV_VC_XV	= 12432,
    RISCV_VC_XVV	= 12433,
    RISCV_VC_XVW	= 12434,
    RISCV_VDIVU_VV	= 12435,
    RISCV_VDIVU_VX	= 12436,
    RISCV_VDIV_VV	= 12437,
    RISCV_VDIV_VX	= 12438,
    RISCV_VFADD_VF	= 12439,
    RISCV_VFADD_VV	= 12440,
    RISCV_VFCLASS_V	= 12441,
    RISCV_VFCVT_F_XU_V	= 12442,
    RISCV_VFCVT_F_X_V	= 12443,
    RISCV_VFCVT_RTZ_XU_F_V	= 12444,
    RISCV_VFCVT_RTZ_X_F_V	= 12445,
    RISCV_VFCVT_XU_F_V	= 12446,
    RISCV_VFCVT_X_F_V	= 12447,
    RISCV_VFDIV_VF	= 12448,
    RISCV_VFDIV_VV	= 12449,
    RISCV_VFIRST_M	= 12450,
    RISCV_VFMACC_VF	= 12451,
    RISCV_VFMACC_VV	= 12452,
    RISCV_VFMADD_VF	= 12453,
    RISCV_VFMADD_VV	= 12454,
    RISCV_VFMAX_VF	= 12455,
    RISCV_VFMAX_VV	= 12456,
    RISCV_VFMERGE_VFM	= 12457,
    RISCV_VFMIN_VF	= 12458,
    RISCV_VFMIN_VV	= 12459,
    RISCV_VFMSAC_VF	= 12460,
    RISCV_VFMSAC_VV	= 12461,
    RISCV_VFMSUB_VF	= 12462,
    RISCV_VFMSUB_VV	= 12463,
    RISCV_VFMUL_VF	= 12464,
    RISCV_VFMUL_VV	= 12465,
    RISCV_VFMV_F_S	= 12466,
    RISCV_VFMV_S_F	= 12467,
    RISCV_VFMV_V_F	= 12468,
    RISCV_VFNCVTBF16_F_F_W	= 12469,
    RISCV_VFNCVT_F_F_W	= 12470,
    RISCV_VFNCVT_F_XU_W	= 12471,
    RISCV_VFNCVT_F_X_W	= 12472,
    RISCV_VFNCVT_ROD_F_F_W	= 12473,
    RISCV_VFNCVT_RTZ_XU_F_W	= 12474,
    RISCV_VFNCVT_RTZ_X_F_W	= 12475,
    RISCV_VFNCVT_XU_F_W	= 12476,
    RISCV_VFNCVT_X_F_W	= 12477,
    RISCV_VFNMACC_VF	= 12478,
    RISCV_VFNMACC_VV	= 12479,
    RISCV_VFNMADD_VF	= 12480,
    RISCV_VFNMADD_VV	= 12481,
    RISCV_VFNMSAC_VF	= 12482,
    RISCV_VFNMSAC_VV	= 12483,
    RISCV_VFNMSUB_VF	= 12484,
    RISCV_VFNMSUB_VV	= 12485,
    RISCV_VFNRCLIP_XU_F_QF	= 12486,
    RISCV_VFNRCLIP_X_F_QF	= 12487,
    RISCV_VFRDIV_VF	= 12488,
    RISCV_VFREC7_V	= 12489,
    RISCV_VFREDMAX_VS	= 12490,
    RISCV_VFREDMIN_VS	= 12491,
    RISCV_VFREDOSUM_VS	= 12492,
    RISCV_VFREDUSUM_VS	= 12493,
    RISCV_VFRSQRT7_V	= 12494,
    RISCV_VFRSUB_VF	= 12495,
    RISCV_VFSGNJN_VF	= 12496,
    RISCV_VFSGNJN_VV	= 12497,
    RISCV_VFSGNJX_VF	= 12498,
    RISCV_VFSGNJX_VV	= 12499,
    RISCV_VFSGNJ_VF	= 12500,
    RISCV_VFSGNJ_VV	= 12501,
    RISCV_VFSLIDE1DOWN_VF	= 12502,
    RISCV_VFSLIDE1UP_VF	= 12503,
    RISCV_VFSQRT_V	= 12504,
    RISCV_VFSUB_VF	= 12505,
    RISCV_VFSUB_VV	= 12506,
    RISCV_VFWADD_VF	= 12507,
    RISCV_VFWADD_VV	= 12508,
    RISCV_VFWADD_WF	= 12509,
    RISCV_VFWADD_WV	= 12510,
    RISCV_VFWCVTBF16_F_F_V	= 12511,
    RISCV_VFWCVT_F_F_V	= 12512,
    RISCV_VFWCVT_F_XU_V	= 12513,
    RISCV_VFWCVT_F_X_V	= 12514,
    RISCV_VFWCVT_RTZ_XU_F_V	= 12515,
    RISCV_VFWCVT_RTZ_X_F_V	= 12516,
    RISCV_VFWCVT_XU_F_V	= 12517,
    RISCV_VFWCVT_X_F_V	= 12518,
    RISCV_VFWMACCBF16_VF	= 12519,
    RISCV_VFWMACCBF16_VV	= 12520,
    RISCV_VFWMACC_4x4x4	= 12521,
    RISCV_VFWMACC_VF	= 12522,
    RISCV_VFWMACC_VV	= 12523,
    RISCV_VFWMSAC_VF	= 12524,
    RISCV_VFWMSAC_VV	= 12525,
    RISCV_VFWMUL_VF	= 12526,
    RISCV_VFWMUL_VV	= 12527,
    RISCV_VFWNMACC_VF	= 12528,
    RISCV_VFWNMACC_VV	= 12529,
    RISCV_VFWNMSAC_VF	= 12530,
    RISCV_VFWNMSAC_VV	= 12531,
    RISCV_VFWREDOSUM_VS	= 12532,
    RISCV_VFWREDUSUM_VS	= 12533,
    RISCV_VFWSUB_VF	= 12534,
    RISCV_VFWSUB_VV	= 12535,
    RISCV_VFWSUB_WF	= 12536,
    RISCV_VFWSUB_WV	= 12537,
    RISCV_VGHSH_VV	= 12538,
    RISCV_VGMUL_VV	= 12539,
    RISCV_VID_V	= 12540,
    RISCV_VIOTA_M	= 12541,
    RISCV_VL1RE16_V	= 12542,
    RISCV_VL1RE32_V	= 12543,
    RISCV_VL1RE64_V	= 12544,
    RISCV_VL1RE8_V	= 12545,
    RISCV_VL2RE16_V	= 12546,
    RISCV_VL2RE32_V	= 12547,
    RISCV_VL2RE64_V	= 12548,
    RISCV_VL2RE8_V	= 12549,
    RISCV_VL4RE16_V	= 12550,
    RISCV_VL4RE32_V	= 12551,
    RISCV_VL4RE64_V	= 12552,
    RISCV_VL4RE8_V	= 12553,
    RISCV_VL8RE16_V	= 12554,
    RISCV_VL8RE32_V	= 12555,
    RISCV_VL8RE64_V	= 12556,
    RISCV_VL8RE8_V	= 12557,
    RISCV_VLE16FF_V	= 12558,
    RISCV_VLE16_V	= 12559,
    RISCV_VLE32FF_V	= 12560,
    RISCV_VLE32_V	= 12561,
    RISCV_VLE64FF_V	= 12562,
    RISCV_VLE64_V	= 12563,
    RISCV_VLE8FF_V	= 12564,
    RISCV_VLE8_V	= 12565,
    RISCV_VLM_V	= 12566,
    RISCV_VLOXEI16_V	= 12567,
    RISCV_VLOXEI32_V	= 12568,
    RISCV_VLOXEI64_V	= 12569,
    RISCV_VLOXEI8_V	= 12570,
    RISCV_VLOXSEG2EI16_V	= 12571,
    RISCV_VLOXSEG2EI32_V	= 12572,
    RISCV_VLOXSEG2EI64_V	= 12573,
    RISCV_VLOXSEG2EI8_V	= 12574,
    RISCV_VLOXSEG3EI16_V	= 12575,
    RISCV_VLOXSEG3EI32_V	= 12576,
    RISCV_VLOXSEG3EI64_V	= 12577,
    RISCV_VLOXSEG3EI8_V	= 12578,
    RISCV_VLOXSEG4EI16_V	= 12579,
    RISCV_VLOXSEG4EI32_V	= 12580,
    RISCV_VLOXSEG4EI64_V	= 12581,
    RISCV_VLOXSEG4EI8_V	= 12582,
    RISCV_VLOXSEG5EI16_V	= 12583,
    RISCV_VLOXSEG5EI32_V	= 12584,
    RISCV_VLOXSEG5EI64_V	= 12585,
    RISCV_VLOXSEG5EI8_V	= 12586,
    RISCV_VLOXSEG6EI16_V	= 12587,
    RISCV_VLOXSEG6EI32_V	= 12588,
    RISCV_VLOXSEG6EI64_V	= 12589,
    RISCV_VLOXSEG6EI8_V	= 12590,
    RISCV_VLOXSEG7EI16_V	= 12591,
    RISCV_VLOXSEG7EI32_V	= 12592,
    RISCV_VLOXSEG7EI64_V	= 12593,
    RISCV_VLOXSEG7EI8_V	= 12594,
    RISCV_VLOXSEG8EI16_V	= 12595,
    RISCV_VLOXSEG8EI32_V	= 12596,
    RISCV_VLOXSEG8EI64_V	= 12597,
    RISCV_VLOXSEG8EI8_V	= 12598,
    RISCV_VLSE16_V	= 12599,
    RISCV_VLSE32_V	= 12600,
    RISCV_VLSE64_V	= 12601,
    RISCV_VLSE8_V	= 12602,
    RISCV_VLSEG2E16FF_V	= 12603,
    RISCV_VLSEG2E16_V	= 12604,
    RISCV_VLSEG2E32FF_V	= 12605,
    RISCV_VLSEG2E32_V	= 12606,
    RISCV_VLSEG2E64FF_V	= 12607,
    RISCV_VLSEG2E64_V	= 12608,
    RISCV_VLSEG2E8FF_V	= 12609,
    RISCV_VLSEG2E8_V	= 12610,
    RISCV_VLSEG3E16FF_V	= 12611,
    RISCV_VLSEG3E16_V	= 12612,
    RISCV_VLSEG3E32FF_V	= 12613,
    RISCV_VLSEG3E32_V	= 12614,
    RISCV_VLSEG3E64FF_V	= 12615,
    RISCV_VLSEG3E64_V	= 12616,
    RISCV_VLSEG3E8FF_V	= 12617,
    RISCV_VLSEG3E8_V	= 12618,
    RISCV_VLSEG4E16FF_V	= 12619,
    RISCV_VLSEG4E16_V	= 12620,
    RISCV_VLSEG4E32FF_V	= 12621,
    RISCV_VLSEG4E32_V	= 12622,
    RISCV_VLSEG4E64FF_V	= 12623,
    RISCV_VLSEG4E64_V	= 12624,
    RISCV_VLSEG4E8FF_V	= 12625,
    RISCV_VLSEG4E8_V	= 12626,
    RISCV_VLSEG5E16FF_V	= 12627,
    RISCV_VLSEG5E16_V	= 12628,
    RISCV_VLSEG5E32FF_V	= 12629,
    RISCV_VLSEG5E32_V	= 12630,
    RISCV_VLSEG5E64FF_V	= 12631,
    RISCV_VLSEG5E64_V	= 12632,
    RISCV_VLSEG5E8FF_V	= 12633,
    RISCV_VLSEG5E8_V	= 12634,
    RISCV_VLSEG6E16FF_V	= 12635,
    RISCV_VLSEG6E16_V	= 12636,
    RISCV_VLSEG6E32FF_V	= 12637,
    RISCV_VLSEG6E32_V	= 12638,
    RISCV_VLSEG6E64FF_V	= 12639,
    RISCV_VLSEG6E64_V	= 12640,
    RISCV_VLSEG6E8FF_V	= 12641,
    RISCV_VLSEG6E8_V	= 12642,
    RISCV_VLSEG7E16FF_V	= 12643,
    RISCV_VLSEG7E16_V	= 12644,
    RISCV_VLSEG7E32FF_V	= 12645,
    RISCV_VLSEG7E32_V	= 12646,
    RISCV_VLSEG7E64FF_V	= 12647,
    RISCV_VLSEG7E64_V	= 12648,
    RISCV_VLSEG7E8FF_V	= 12649,
    RISCV_VLSEG7E8_V	= 12650,
    RISCV_VLSEG8E16FF_V	= 12651,
    RISCV_VLSEG8E16_V	= 12652,
    RISCV_VLSEG8E32FF_V	= 12653,
    RISCV_VLSEG8E32_V	= 12654,
    RISCV_VLSEG8E64FF_V	= 12655,
    RISCV_VLSEG8E64_V	= 12656,
    RISCV_VLSEG8E8FF_V	= 12657,
    RISCV_VLSEG8E8_V	= 12658,
    RISCV_VLSSEG2E16_V	= 12659,
    RISCV_VLSSEG2E32_V	= 12660,
    RISCV_VLSSEG2E64_V	= 12661,
    RISCV_VLSSEG2E8_V	= 12662,
    RISCV_VLSSEG3E16_V	= 12663,
    RISCV_VLSSEG3E32_V	= 12664,
    RISCV_VLSSEG3E64_V	= 12665,
    RISCV_VLSSEG3E8_V	= 12666,
    RISCV_VLSSEG4E16_V	= 12667,
    RISCV_VLSSEG4E32_V	= 12668,
    RISCV_VLSSEG4E64_V	= 12669,
    RISCV_VLSSEG4E8_V	= 12670,
    RISCV_VLSSEG5E16_V	= 12671,
    RISCV_VLSSEG5E32_V	= 12672,
    RISCV_VLSSEG5E64_V	= 12673,
    RISCV_VLSSEG5E8_V	= 12674,
    RISCV_VLSSEG6E16_V	= 12675,
    RISCV_VLSSEG6E32_V	= 12676,
    RISCV_VLSSEG6E64_V	= 12677,
    RISCV_VLSSEG6E8_V	= 12678,
    RISCV_VLSSEG7E16_V	= 12679,
    RISCV_VLSSEG7E32_V	= 12680,
    RISCV_VLSSEG7E64_V	= 12681,
    RISCV_VLSSEG7E8_V	= 12682,
    RISCV_VLSSEG8E16_V	= 12683,
    RISCV_VLSSEG8E32_V	= 12684,
    RISCV_VLSSEG8E64_V	= 12685,
    RISCV_VLSSEG8E8_V	= 12686,
    RISCV_VLUXEI16_V	= 12687,
    RISCV_VLUXEI32_V	= 12688,
    RISCV_VLUXEI64_V	= 12689,
    RISCV_VLUXEI8_V	= 12690,
    RISCV_VLUXSEG2EI16_V	= 12691,
    RISCV_VLUXSEG2EI32_V	= 12692,
    RISCV_VLUXSEG2EI64_V	= 12693,
    RISCV_VLUXSEG2EI8_V	= 12694,
    RISCV_VLUXSEG3EI16_V	= 12695,
    RISCV_VLUXSEG3EI32_V	= 12696,
    RISCV_VLUXSEG3EI64_V	= 12697,
    RISCV_VLUXSEG3EI8_V	= 12698,
    RISCV_VLUXSEG4EI16_V	= 12699,
    RISCV_VLUXSEG4EI32_V	= 12700,
    RISCV_VLUXSEG4EI64_V	= 12701,
    RISCV_VLUXSEG4EI8_V	= 12702,
    RISCV_VLUXSEG5EI16_V	= 12703,
    RISCV_VLUXSEG5EI32_V	= 12704,
    RISCV_VLUXSEG5EI64_V	= 12705,
    RISCV_VLUXSEG5EI8_V	= 12706,
    RISCV_VLUXSEG6EI16_V	= 12707,
    RISCV_VLUXSEG6EI32_V	= 12708,
    RISCV_VLUXSEG6EI64_V	= 12709,
    RISCV_VLUXSEG6EI8_V	= 12710,
    RISCV_VLUXSEG7EI16_V	= 12711,
    RISCV_VLUXSEG7EI32_V	= 12712,
    RISCV_VLUXSEG7EI64_V	= 12713,
    RISCV_VLUXSEG7EI8_V	= 12714,
    RISCV_VLUXSEG8EI16_V	= 12715,
    RISCV_VLUXSEG8EI32_V	= 12716,
    RISCV_VLUXSEG8EI64_V	= 12717,
    RISCV_VLUXSEG8EI8_V	= 12718,
    RISCV_VMACC_VV	= 12719,
    RISCV_VMACC_VX	= 12720,
    RISCV_VMADC_VI	= 12721,
    RISCV_VMADC_VIM	= 12722,
    RISCV_VMADC_VV	= 12723,
    RISCV_VMADC_VVM	= 12724,
    RISCV_VMADC_VX	= 12725,
    RISCV_VMADC_VXM	= 12726,
    RISCV_VMADD_VV	= 12727,
    RISCV_VMADD_VX	= 12728,
    RISCV_VMANDN_MM	= 12729,
    RISCV_VMAND_MM	= 12730,
    RISCV_VMAXU_VV	= 12731,
    RISCV_VMAXU_VX	= 12732,
    RISCV_VMAX_VV	= 12733,
    RISCV_VMAX_VX	= 12734,
    RISCV_VMERGE_VIM	= 12735,
    RISCV_VMERGE_VVM	= 12736,
    RISCV_VMERGE_VXM	= 12737,
    RISCV_VMFEQ_VF	= 12738,
    RISCV_VMFEQ_VV	= 12739,
    RISCV_VMFGE_VF	= 12740,
    RISCV_VMFGT_VF	= 12741,
    RISCV_VMFLE_VF	= 12742,
    RISCV_VMFLE_VV	= 12743,
    RISCV_VMFLT_VF	= 12744,
    RISCV_VMFLT_VV	= 12745,
    RISCV_VMFNE_VF	= 12746,
    RISCV_VMFNE_VV	= 12747,
    RISCV_VMINU_VV	= 12748,
    RISCV_VMINU_VX	= 12749,
    RISCV_VMIN_VV	= 12750,
    RISCV_VMIN_VX	= 12751,
    RISCV_VMNAND_MM	= 12752,
    RISCV_VMNOR_MM	= 12753,
    RISCV_VMORN_MM	= 12754,
    RISCV_VMOR_MM	= 12755,
    RISCV_VMSBC_VV	= 12756,
    RISCV_VMSBC_VVM	= 12757,
    RISCV_VMSBC_VX	= 12758,
    RISCV_VMSBC_VXM	= 12759,
    RISCV_VMSBF_M	= 12760,
    RISCV_VMSEQ_VI	= 12761,
    RISCV_VMSEQ_VV	= 12762,
    RISCV_VMSEQ_VX	= 12763,
    RISCV_VMSGTU_VI	= 12764,
    RISCV_VMSGTU_VX	= 12765,
    RISCV_VMSGT_VI	= 12766,
    RISCV_VMSGT_VX	= 12767,
    RISCV_VMSIF_M	= 12768,
    RISCV_VMSLEU_VI	= 12769,
    RISCV_VMSLEU_VV	= 12770,
    RISCV_VMSLEU_VX	= 12771,
    RISCV_VMSLE_VI	= 12772,
    RISCV_VMSLE_VV	= 12773,
    RISCV_VMSLE_VX	= 12774,
    RISCV_VMSLTU_VV	= 12775,
    RISCV_VMSLTU_VX	= 12776,
    RISCV_VMSLT_VV	= 12777,
    RISCV_VMSLT_VX	= 12778,
    RISCV_VMSNE_VI	= 12779,
    RISCV_VMSNE_VV	= 12780,
    RISCV_VMSNE_VX	= 12781,
    RISCV_VMSOF_M	= 12782,
    RISCV_VMULHSU_VV	= 12783,
    RISCV_VMULHSU_VX	= 12784,
    RISCV_VMULHU_VV	= 12785,
    RISCV_VMULHU_VX	= 12786,
    RISCV_VMULH_VV	= 12787,
    RISCV_VMULH_VX	= 12788,
    RISCV_VMUL_VV	= 12789,
    RISCV_VMUL_VX	= 12790,
    RISCV_VMV1R_V	= 12791,
    RISCV_VMV2R_V	= 12792,
    RISCV_VMV4R_V	= 12793,
    RISCV_VMV8R_V	= 12794,
    RISCV_VMV_S_X	= 12795,
    RISCV_VMV_V_I	= 12796,
    RISCV_VMV_V_V	= 12797,
    RISCV_VMV_V_X	= 12798,
    RISCV_VMV_X_S	= 12799,
    RISCV_VMXNOR_MM	= 12800,
    RISCV_VMXOR_MM	= 12801,
    RISCV_VNCLIPU_WI	= 12802,
    RISCV_VNCLIPU_WV	= 12803,
    RISCV_VNCLIPU_WX	= 12804,
    RISCV_VNCLIP_WI	= 12805,
    RISCV_VNCLIP_WV	= 12806,
    RISCV_VNCLIP_WX	= 12807,
    RISCV_VNMSAC_VV	= 12808,
    RISCV_VNMSAC_VX	= 12809,
    RISCV_VNMSUB_VV	= 12810,
    RISCV_VNMSUB_VX	= 12811,
    RISCV_VNSRA_WI	= 12812,
    RISCV_VNSRA_WV	= 12813,
    RISCV_VNSRA_WX	= 12814,
    RISCV_VNSRL_WI	= 12815,
    RISCV_VNSRL_WV	= 12816,
    RISCV_VNSRL_WX	= 12817,
    RISCV_VOR_VI	= 12818,
    RISCV_VOR_VV	= 12819,
    RISCV_VOR_VX	= 12820,
    RISCV_VQMACCSU_2x8x2	= 12821,
    RISCV_VQMACCSU_4x8x4	= 12822,
    RISCV_VQMACCUS_2x8x2	= 12823,
    RISCV_VQMACCUS_4x8x4	= 12824,
    RISCV_VQMACCU_2x8x2	= 12825,
    RISCV_VQMACCU_4x8x4	= 12826,
    RISCV_VQMACC_2x8x2	= 12827,
    RISCV_VQMACC_4x8x4	= 12828,
    RISCV_VREDAND_VS	= 12829,
    RISCV_VREDMAXU_VS	= 12830,
    RISCV_VREDMAX_VS	= 12831,
    RISCV_VREDMINU_VS	= 12832,
    RISCV_VREDMIN_VS	= 12833,
    RISCV_VREDOR_VS	= 12834,
    RISCV_VREDSUM_VS	= 12835,
    RISCV_VREDXOR_VS	= 12836,
    RISCV_VREMU_VV	= 12837,
    RISCV_VREMU_VX	= 12838,
    RISCV_VREM_VV	= 12839,
    RISCV_VREM_VX	= 12840,
    RISCV_VREV8_V	= 12841,
    RISCV_VRGATHEREI16_VV	= 12842,
    RISCV_VRGATHER_VI	= 12843,
    RISCV_VRGATHER_VV	= 12844,
    RISCV_VRGATHER_VX	= 12845,
    RISCV_VROL_VV	= 12846,
    RISCV_VROL_VX	= 12847,
    RISCV_VROR_VI	= 12848,
    RISCV_VROR_VV	= 12849,
    RISCV_VROR_VX	= 12850,
    RISCV_VRSUB_VI	= 12851,
    RISCV_VRSUB_VX	= 12852,
    RISCV_VS1R_V	= 12853,
    RISCV_VS2R_V	= 12854,
    RISCV_VS4R_V	= 12855,
    RISCV_VS8R_V	= 12856,
    RISCV_VSADDU_VI	= 12857,
    RISCV_VSADDU_VV	= 12858,
    RISCV_VSADDU_VX	= 12859,
    RISCV_VSADD_VI	= 12860,
    RISCV_VSADD_VV	= 12861,
    RISCV_VSADD_VX	= 12862,
    RISCV_VSBC_VVM	= 12863,
    RISCV_VSBC_VXM	= 12864,
    RISCV_VSE16_V	= 12865,
    RISCV_VSE32_V	= 12866,
    RISCV_VSE64_V	= 12867,
    RISCV_VSE8_V	= 12868,
    RISCV_VSETIVLI	= 12869,
    RISCV_VSETVL	= 12870,
    RISCV_VSETVLI	= 12871,
    RISCV_VSEXT_VF2	= 12872,
    RISCV_VSEXT_VF4	= 12873,
    RISCV_VSEXT_VF8	= 12874,
    RISCV_VSHA2CH_VV	= 12875,
    RISCV_VSHA2CL_VV	= 12876,
    RISCV_VSHA2MS_VV	= 12877,
    RISCV_VSLIDE1DOWN_VX	= 12878,
    RISCV_VSLIDE1UP_VX	= 12879,
    RISCV_VSLIDEDOWN_VI	= 12880,
    RISCV_VSLIDEDOWN_VX	= 12881,
    RISCV_VSLIDEUP_VI	= 12882,
    RISCV_VSLIDEUP_VX	= 12883,
    RISCV_VSLL_VI	= 12884,
    RISCV_VSLL_VV	= 12885,
    RISCV_VSLL_VX	= 12886,
    RISCV_VSM3C_VI	= 12887,
    RISCV_VSM3ME_VV	= 12888,
    RISCV_VSM4K_VI	= 12889,
    RISCV_VSM4R_VS	= 12890,
    RISCV_VSM4R_VV	= 12891,
    RISCV_VSMUL_VV	= 12892,
    RISCV_VSMUL_VX	= 12893,
    RISCV_VSM_V	= 12894,
    RISCV_VSOXEI16_V	= 12895,
    RISCV_VSOXEI32_V	= 12896,
    RISCV_VSOXEI64_V	= 12897,
    RISCV_VSOXEI8_V	= 12898,
    RISCV_VSOXSEG2EI16_V	= 12899,
    RISCV_VSOXSEG2EI32_V	= 12900,
    RISCV_VSOXSEG2EI64_V	= 12901,
    RISCV_VSOXSEG2EI8_V	= 12902,
    RISCV_VSOXSEG3EI16_V	= 12903,
    RISCV_VSOXSEG3EI32_V	= 12904,
    RISCV_VSOXSEG3EI64_V	= 12905,
    RISCV_VSOXSEG3EI8_V	= 12906,
    RISCV_VSOXSEG4EI16_V	= 12907,
    RISCV_VSOXSEG4EI32_V	= 12908,
    RISCV_VSOXSEG4EI64_V	= 12909,
    RISCV_VSOXSEG4EI8_V	= 12910,
    RISCV_VSOXSEG5EI16_V	= 12911,
    RISCV_VSOXSEG5EI32_V	= 12912,
    RISCV_VSOXSEG5EI64_V	= 12913,
    RISCV_VSOXSEG5EI8_V	= 12914,
    RISCV_VSOXSEG6EI16_V	= 12915,
    RISCV_VSOXSEG6EI32_V	= 12916,
    RISCV_VSOXSEG6EI64_V	= 12917,
    RISCV_VSOXSEG6EI8_V	= 12918,
    RISCV_VSOXSEG7EI16_V	= 12919,
    RISCV_VSOXSEG7EI32_V	= 12920,
    RISCV_VSOXSEG7EI64_V	= 12921,
    RISCV_VSOXSEG7EI8_V	= 12922,
    RISCV_VSOXSEG8EI16_V	= 12923,
    RISCV_VSOXSEG8EI32_V	= 12924,
    RISCV_VSOXSEG8EI64_V	= 12925,
    RISCV_VSOXSEG8EI8_V	= 12926,
    RISCV_VSRA_VI	= 12927,
    RISCV_VSRA_VV	= 12928,
    RISCV_VSRA_VX	= 12929,
    RISCV_VSRL_VI	= 12930,
    RISCV_VSRL_VV	= 12931,
    RISCV_VSRL_VX	= 12932,
    RISCV_VSSE16_V	= 12933,
    RISCV_VSSE32_V	= 12934,
    RISCV_VSSE64_V	= 12935,
    RISCV_VSSE8_V	= 12936,
    RISCV_VSSEG2E16_V	= 12937,
    RISCV_VSSEG2E32_V	= 12938,
    RISCV_VSSEG2E64_V	= 12939,
    RISCV_VSSEG2E8_V	= 12940,
    RISCV_VSSEG3E16_V	= 12941,
    RISCV_VSSEG3E32_V	= 12942,
    RISCV_VSSEG3E64_V	= 12943,
    RISCV_VSSEG3E8_V	= 12944,
    RISCV_VSSEG4E16_V	= 12945,
    RISCV_VSSEG4E32_V	= 12946,
    RISCV_VSSEG4E64_V	= 12947,
    RISCV_VSSEG4E8_V	= 12948,
    RISCV_VSSEG5E16_V	= 12949,
    RISCV_VSSEG5E32_V	= 12950,
    RISCV_VSSEG5E64_V	= 12951,
    RISCV_VSSEG5E8_V	= 12952,
    RISCV_VSSEG6E16_V	= 12953,
    RISCV_VSSEG6E32_V	= 12954,
    RISCV_VSSEG6E64_V	= 12955,
    RISCV_VSSEG6E8_V	= 12956,
    RISCV_VSSEG7E16_V	= 12957,
    RISCV_VSSEG7E32_V	= 12958,
    RISCV_VSSEG7E64_V	= 12959,
    RISCV_VSSEG7E8_V	= 12960,
    RISCV_VSSEG8E16_V	= 12961,
    RISCV_VSSEG8E32_V	= 12962,
    RISCV_VSSEG8E64_V	= 12963,
    RISCV_VSSEG8E8_V	= 12964,
    RISCV_VSSRA_VI	= 12965,
    RISCV_VSSRA_VV	= 12966,
    RISCV_VSSRA_VX	= 12967,
    RISCV_VSSRL_VI	= 12968,
    RISCV_VSSRL_VV	= 12969,
    RISCV_VSSRL_VX	= 12970,
    RISCV_VSSSEG2E16_V	= 12971,
    RISCV_VSSSEG2E32_V	= 12972,
    RISCV_VSSSEG2E64_V	= 12973,
    RISCV_VSSSEG2E8_V	= 12974,
    RISCV_VSSSEG3E16_V	= 12975,
    RISCV_VSSSEG3E32_V	= 12976,
    RISCV_VSSSEG3E64_V	= 12977,
    RISCV_VSSSEG3E8_V	= 12978,
    RISCV_VSSSEG4E16_V	= 12979,
    RISCV_VSSSEG4E32_V	= 12980,
    RISCV_VSSSEG4E64_V	= 12981,
    RISCV_VSSSEG4E8_V	= 12982,
    RISCV_VSSSEG5E16_V	= 12983,
    RISCV_VSSSEG5E32_V	= 12984,
    RISCV_VSSSEG5E64_V	= 12985,
    RISCV_VSSSEG5E8_V	= 12986,
    RISCV_VSSSEG6E16_V	= 12987,
    RISCV_VSSSEG6E32_V	= 12988,
    RISCV_VSSSEG6E64_V	= 12989,
    RISCV_VSSSEG6E8_V	= 12990,
    RISCV_VSSSEG7E16_V	= 12991,
    RISCV_VSSSEG7E32_V	= 12992,
    RISCV_VSSSEG7E64_V	= 12993,
    RISCV_VSSSEG7E8_V	= 12994,
    RISCV_VSSSEG8E16_V	= 12995,
    RISCV_VSSSEG8E32_V	= 12996,
    RISCV_VSSSEG8E64_V	= 12997,
    RISCV_VSSSEG8E8_V	= 12998,
    RISCV_VSSUBU_VV	= 12999,
    RISCV_VSSUBU_VX	= 13000,
    RISCV_VSSUB_VV	= 13001,
    RISCV_VSSUB_VX	= 13002,
    RISCV_VSUB_VV	= 13003,
    RISCV_VSUB_VX	= 13004,
    RISCV_VSUXEI16_V	= 13005,
    RISCV_VSUXEI32_V	= 13006,
    RISCV_VSUXEI64_V	= 13007,
    RISCV_VSUXEI8_V	= 13008,
    RISCV_VSUXSEG2EI16_V	= 13009,
    RISCV_VSUXSEG2EI32_V	= 13010,
    RISCV_VSUXSEG2EI64_V	= 13011,
    RISCV_VSUXSEG2EI8_V	= 13012,
    RISCV_VSUXSEG3EI16_V	= 13013,
    RISCV_VSUXSEG3EI32_V	= 13014,
    RISCV_VSUXSEG3EI64_V	= 13015,
    RISCV_VSUXSEG3EI8_V	= 13016,
    RISCV_VSUXSEG4EI16_V	= 13017,
    RISCV_VSUXSEG4EI32_V	= 13018,
    RISCV_VSUXSEG4EI64_V	= 13019,
    RISCV_VSUXSEG4EI8_V	= 13020,
    RISCV_VSUXSEG5EI16_V	= 13021,
    RISCV_VSUXSEG5EI32_V	= 13022,
    RISCV_VSUXSEG5EI64_V	= 13023,
    RISCV_VSUXSEG5EI8_V	= 13024,
    RISCV_VSUXSEG6EI16_V	= 13025,
    RISCV_VSUXSEG6EI32_V	= 13026,
    RISCV_VSUXSEG6EI64_V	= 13027,
    RISCV_VSUXSEG6EI8_V	= 13028,
    RISCV_VSUXSEG7EI16_V	= 13029,
    RISCV_VSUXSEG7EI32_V	= 13030,
    RISCV_VSUXSEG7EI64_V	= 13031,
    RISCV_VSUXSEG7EI8_V	= 13032,
    RISCV_VSUXSEG8EI16_V	= 13033,
    RISCV_VSUXSEG8EI32_V	= 13034,
    RISCV_VSUXSEG8EI64_V	= 13035,
    RISCV_VSUXSEG8EI8_V	= 13036,
    RISCV_VT_MASKC	= 13037,
    RISCV_VT_MASKCN	= 13038,
    RISCV_VWADDU_VV	= 13039,
    RISCV_VWADDU_VX	= 13040,
    RISCV_VWADDU_WV	= 13041,
    RISCV_VWADDU_WX	= 13042,
    RISCV_VWADD_VV	= 13043,
    RISCV_VWADD_VX	= 13044,
    RISCV_VWADD_WV	= 13045,
    RISCV_VWADD_WX	= 13046,
    RISCV_VWMACCSU_VV	= 13047,
    RISCV_VWMACCSU_VX	= 13048,
    RISCV_VWMACCUS_VX	= 13049,
    RISCV_VWMACCU_VV	= 13050,
    RISCV_VWMACCU_VX	= 13051,
    RISCV_VWMACC_VV	= 13052,
    RISCV_VWMACC_VX	= 13053,
    RISCV_VWMULSU_VV	= 13054,
    RISCV_VWMULSU_VX	= 13055,
    RISCV_VWMULU_VV	= 13056,
    RISCV_VWMULU_VX	= 13057,
    RISCV_VWMUL_VV	= 13058,
    RISCV_VWMUL_VX	= 13059,
    RISCV_VWREDSUMU_VS	= 13060,
    RISCV_VWREDSUM_VS	= 13061,
    RISCV_VWSLL_VI	= 13062,
    RISCV_VWSLL_VV	= 13063,
    RISCV_VWSLL_VX	= 13064,
    RISCV_VWSUBU_VV	= 13065,
    RISCV_VWSUBU_VX	= 13066,
    RISCV_VWSUBU_WV	= 13067,
    RISCV_VWSUBU_WX	= 13068,
    RISCV_VWSUB_VV	= 13069,
    RISCV_VWSUB_VX	= 13070,
    RISCV_VWSUB_WV	= 13071,
    RISCV_VWSUB_WX	= 13072,
    RISCV_VXOR_VI	= 13073,
    RISCV_VXOR_VV	= 13074,
    RISCV_VXOR_VX	= 13075,
    RISCV_VZEXT_VF2	= 13076,
    RISCV_VZEXT_VF4	= 13077,
    RISCV_VZEXT_VF8	= 13078,
    RISCV_WFI	= 13079,
    RISCV_WRS_NTO	= 13080,
    RISCV_WRS_STO	= 13081,
    RISCV_XNOR	= 13082,
    RISCV_XOR	= 13083,
    RISCV_XORI	= 13084,
    RISCV_XPERM4	= 13085,
    RISCV_XPERM8	= 13086,
    RISCV_ZEXT_H_RV32	= 13087,
    RISCV_ZEXT_H_RV64	= 13088,
    RISCV_ZIP_RV32	= 13089,
    INSTRUCTION_LIST_END = 13090
  };

#endif // GET_INSTRINFO_ENUM

#if defined(GET_INSTRINFO_MC_DESC) || defined(GET_INSTRINFO_CTOR_DTOR)
typedef struct RISCVInstrTable {
  MCInstrDesc Insts[13090];
  MCOperandInfo OperandInfo[8522];
  MCPhysReg ImplicitOps[36];
} RISCVInstrTable;

#endif // defined(GET_INSTRINFO_MC_DESC) || defined(GET_INSTRINFO_CTOR_DTOR)

#ifdef GET_INSTRINFO_MC_DESC
#undef GET_INSTRINFO_MC_DESC

static const unsigned RISCVImpOpBase = sizeof(MCOperandInfo) / (sizeof(MCPhysReg));

static const RISCVInstrTable RISCVDescs = {
  {
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #13089 = ZIP_RV32
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #13088 = ZEXT_H_RV64
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #13087 = ZEXT_H_RV32
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #13086 = XPERM8
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #13085 = XPERM4
  { 3, &RISCVDescs.OperandInfo[7808] },  // Inst #13084 = XORI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #13083 = XOR
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #13082 = XNOR
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #13081 = WRS_STO
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #13080 = WRS_NTO
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #13079 = WFI
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #13078 = VZEXT_VF8
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #13077 = VZEXT_VF4
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #13076 = VZEXT_VF2
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #13075 = VXOR_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #13074 = VXOR_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #13073 = VXOR_VI
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13072 = VWSUB_WX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13071 = VWSUB_WV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13070 = VWSUB_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13069 = VWSUB_VV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13068 = VWSUBU_WX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13067 = VWSUBU_WV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13066 = VWSUBU_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13065 = VWSUBU_VV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13064 = VWSLL_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13063 = VWSLL_VV
  { 4, &RISCVDescs.OperandInfo[8507] },  // Inst #13062 = VWSLL_VI
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13061 = VWREDSUM_VS
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13060 = VWREDSUMU_VS
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13059 = VWMUL_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13058 = VWMUL_VV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13057 = VWMULU_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13056 = VWMULU_VV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13055 = VWMULSU_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13054 = VWMULSU_VV
  { 5, &RISCVDescs.OperandInfo[8473] },  // Inst #13053 = VWMACC_VX
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #13052 = VWMACC_VV
  { 5, &RISCVDescs.OperandInfo[8473] },  // Inst #13051 = VWMACCU_VX
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #13050 = VWMACCU_VV
  { 5, &RISCVDescs.OperandInfo[8473] },  // Inst #13049 = VWMACCUS_VX
  { 5, &RISCVDescs.OperandInfo[8473] },  // Inst #13048 = VWMACCSU_VX
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #13047 = VWMACCSU_VV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13046 = VWADD_WX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13045 = VWADD_WV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13044 = VWADD_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13043 = VWADD_VV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13042 = VWADDU_WX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13041 = VWADDU_WV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #13040 = VWADDU_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #13039 = VWADDU_VV
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #13038 = VT_MASKCN
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #13037 = VT_MASKC
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13036 = VSUXSEG8EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13035 = VSUXSEG8EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13034 = VSUXSEG8EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13033 = VSUXSEG8EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13032 = VSUXSEG7EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13031 = VSUXSEG7EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13030 = VSUXSEG7EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13029 = VSUXSEG7EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13028 = VSUXSEG6EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13027 = VSUXSEG6EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13026 = VSUXSEG6EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13025 = VSUXSEG6EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13024 = VSUXSEG5EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13023 = VSUXSEG5EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13022 = VSUXSEG5EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13021 = VSUXSEG5EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13020 = VSUXSEG4EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13019 = VSUXSEG4EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13018 = VSUXSEG4EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13017 = VSUXSEG4EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13016 = VSUXSEG3EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13015 = VSUXSEG3EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13014 = VSUXSEG3EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13013 = VSUXSEG3EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13012 = VSUXSEG2EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13011 = VSUXSEG2EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13010 = VSUXSEG2EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13009 = VSUXSEG2EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13008 = VSUXEI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13007 = VSUXEI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13006 = VSUXEI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #13005 = VSUXEI16_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #13004 = VSUB_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #13003 = VSUB_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #13002 = VSSUB_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #13001 = VSSUB_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #13000 = VSSUBU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12999 = VSSUBU_VV
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12998 = VSSSEG8E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12997 = VSSSEG8E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12996 = VSSSEG8E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12995 = VSSSEG8E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12994 = VSSSEG7E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12993 = VSSSEG7E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12992 = VSSSEG7E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12991 = VSSSEG7E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12990 = VSSSEG6E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12989 = VSSSEG6E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12988 = VSSSEG6E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12987 = VSSSEG6E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12986 = VSSSEG5E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12985 = VSSSEG5E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12984 = VSSSEG5E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12983 = VSSSEG5E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12982 = VSSSEG4E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12981 = VSSSEG4E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12980 = VSSSEG4E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12979 = VSSSEG4E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12978 = VSSSEG3E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12977 = VSSSEG3E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12976 = VSSSEG3E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12975 = VSSSEG3E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12974 = VSSSEG2E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12973 = VSSSEG2E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12972 = VSSSEG2E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12971 = VSSSEG2E16_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12970 = VSSRL_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12969 = VSSRL_VV
  { 4, &RISCVDescs.OperandInfo[8518] },  // Inst #12968 = VSSRL_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12967 = VSSRA_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12966 = VSSRA_VV
  { 4, &RISCVDescs.OperandInfo[8518] },  // Inst #12965 = VSSRA_VI
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12964 = VSSEG8E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12963 = VSSEG8E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12962 = VSSEG8E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12961 = VSSEG8E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12960 = VSSEG7E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12959 = VSSEG7E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12958 = VSSEG7E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12957 = VSSEG7E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12956 = VSSEG6E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12955 = VSSEG6E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12954 = VSSEG6E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12953 = VSSEG6E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12952 = VSSEG5E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12951 = VSSEG5E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12950 = VSSEG5E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12949 = VSSEG5E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12948 = VSSEG4E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12947 = VSSEG4E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12946 = VSSEG4E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12945 = VSSEG4E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12944 = VSSEG3E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12943 = VSSEG3E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12942 = VSSEG3E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12941 = VSSEG3E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12940 = VSSEG2E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12939 = VSSEG2E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12938 = VSSEG2E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12937 = VSSEG2E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12936 = VSSE8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12935 = VSSE64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12934 = VSSE32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12933 = VSSE16_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12932 = VSRL_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12931 = VSRL_VV
  { 4, &RISCVDescs.OperandInfo[8518] },  // Inst #12930 = VSRL_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12929 = VSRA_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12928 = VSRA_VV
  { 4, &RISCVDescs.OperandInfo[8518] },  // Inst #12927 = VSRA_VI
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12926 = VSOXSEG8EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12925 = VSOXSEG8EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12924 = VSOXSEG8EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12923 = VSOXSEG8EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12922 = VSOXSEG7EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12921 = VSOXSEG7EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12920 = VSOXSEG7EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12919 = VSOXSEG7EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12918 = VSOXSEG6EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12917 = VSOXSEG6EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12916 = VSOXSEG6EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12915 = VSOXSEG6EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12914 = VSOXSEG5EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12913 = VSOXSEG5EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12912 = VSOXSEG5EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12911 = VSOXSEG5EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12910 = VSOXSEG4EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12909 = VSOXSEG4EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12908 = VSOXSEG4EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12907 = VSOXSEG4EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12906 = VSOXSEG3EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12905 = VSOXSEG3EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12904 = VSOXSEG3EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12903 = VSOXSEG3EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12902 = VSOXSEG2EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12901 = VSOXSEG2EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12900 = VSOXSEG2EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12899 = VSOXSEG2EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12898 = VSOXEI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12897 = VSOXEI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12896 = VSOXEI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12895 = VSOXEI16_V
  { 2, &RISCVDescs.OperandInfo[8454] },  // Inst #12894 = VSM_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12893 = VSMUL_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12892 = VSMUL_VV
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12891 = VSM4R_VV
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12890 = VSM4R_VS
  { 3, &RISCVDescs.OperandInfo[8316] },  // Inst #12889 = VSM4K_VI
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12888 = VSM3ME_VV
  { 3, &RISCVDescs.OperandInfo[8316] },  // Inst #12887 = VSM3C_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12886 = VSLL_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12885 = VSLL_VV
  { 4, &RISCVDescs.OperandInfo[8518] },  // Inst #12884 = VSLL_VI
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #12883 = VSLIDEUP_VX
  { 4, &RISCVDescs.OperandInfo[8507] },  // Inst #12882 = VSLIDEUP_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12881 = VSLIDEDOWN_VX
  { 4, &RISCVDescs.OperandInfo[8518] },  // Inst #12880 = VSLIDEDOWN_VI
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #12879 = VSLIDE1UP_VX
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12878 = VSLIDE1DOWN_VX
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12877 = VSHA2MS_VV
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12876 = VSHA2CL_VV
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12875 = VSHA2CH_VV
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12874 = VSEXT_VF8
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12873 = VSEXT_VF4
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12872 = VSEXT_VF2
  { 3, &RISCVDescs.OperandInfo[8515] },  // Inst #12871 = VSETVLI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12870 = VSETVL
  { 3, &RISCVDescs.OperandInfo[6556] },  // Inst #12869 = VSETIVLI
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12868 = VSE8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12867 = VSE64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12866 = VSE32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12865 = VSE16_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12864 = VSBC_VXM
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12863 = VSBC_VVM
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12862 = VSADD_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12861 = VSADD_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12860 = VSADD_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12859 = VSADDU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12858 = VSADDU_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12857 = VSADDU_VI
  { 2, &RISCVDescs.OperandInfo[8460] },  // Inst #12856 = VS8R_V
  { 2, &RISCVDescs.OperandInfo[8458] },  // Inst #12855 = VS4R_V
  { 2, &RISCVDescs.OperandInfo[8456] },  // Inst #12854 = VS2R_V
  { 2, &RISCVDescs.OperandInfo[8454] },  // Inst #12853 = VS1R_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12852 = VRSUB_VX
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12851 = VRSUB_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12850 = VROR_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12849 = VROR_VV
  { 4, &RISCVDescs.OperandInfo[8511] },  // Inst #12848 = VROR_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12847 = VROL_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12846 = VROL_VV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #12845 = VRGATHER_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12844 = VRGATHER_VV
  { 4, &RISCVDescs.OperandInfo[8507] },  // Inst #12843 = VRGATHER_VI
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12842 = VRGATHEREI16_VV
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12841 = VREV8_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12840 = VREM_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12839 = VREM_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12838 = VREMU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12837 = VREMU_VV
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12836 = VREDXOR_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12835 = VREDSUM_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12834 = VREDOR_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12833 = VREDMIN_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12832 = VREDMINU_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12831 = VREDMAX_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12830 = VREDMAXU_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12829 = VREDAND_VS
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12828 = VQMACC_4x8x4
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12827 = VQMACC_2x8x2
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12826 = VQMACCU_4x8x4
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12825 = VQMACCU_2x8x2
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12824 = VQMACCUS_4x8x4
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12823 = VQMACCUS_2x8x2
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12822 = VQMACCSU_4x8x4
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12821 = VQMACCSU_2x8x2
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12820 = VOR_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12819 = VOR_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12818 = VOR_VI
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #12817 = VNSRL_WX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12816 = VNSRL_WV
  { 4, &RISCVDescs.OperandInfo[8507] },  // Inst #12815 = VNSRL_WI
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #12814 = VNSRA_WX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12813 = VNSRA_WV
  { 4, &RISCVDescs.OperandInfo[8507] },  // Inst #12812 = VNSRA_WI
  { 5, &RISCVDescs.OperandInfo[8473] },  // Inst #12811 = VNMSUB_VX
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12810 = VNMSUB_VV
  { 5, &RISCVDescs.OperandInfo[8473] },  // Inst #12809 = VNMSAC_VX
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12808 = VNMSAC_VV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #12807 = VNCLIP_WX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12806 = VNCLIP_WV
  { 4, &RISCVDescs.OperandInfo[8507] },  // Inst #12805 = VNCLIP_WI
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #12804 = VNCLIPU_WX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12803 = VNCLIPU_WV
  { 4, &RISCVDescs.OperandInfo[8507] },  // Inst #12802 = VNCLIPU_WI
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12801 = VMXOR_MM
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12800 = VMXNOR_MM
  { 2, &RISCVDescs.OperandInfo[8505] },  // Inst #12799 = VMV_X_S
  { 2, &RISCVDescs.OperandInfo[8503] },  // Inst #12798 = VMV_V_X
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12797 = VMV_V_V
  { 2, &RISCVDescs.OperandInfo[8501] },  // Inst #12796 = VMV_V_I
  { 3, &RISCVDescs.OperandInfo[8498] },  // Inst #12795 = VMV_S_X
  { 2, &RISCVDescs.OperandInfo[8496] },  // Inst #12794 = VMV8R_V
  { 2, &RISCVDescs.OperandInfo[8494] },  // Inst #12793 = VMV4R_V
  { 2, &RISCVDescs.OperandInfo[8492] },  // Inst #12792 = VMV2R_V
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12791 = VMV1R_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12790 = VMUL_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12789 = VMUL_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12788 = VMULH_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12787 = VMULH_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12786 = VMULHU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12785 = VMULHU_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12784 = VMULHSU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12783 = VMULHSU_VV
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12782 = VMSOF_M
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12781 = VMSNE_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12780 = VMSNE_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12779 = VMSNE_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12778 = VMSLT_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12777 = VMSLT_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12776 = VMSLTU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12775 = VMSLTU_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12774 = VMSLE_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12773 = VMSLE_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12772 = VMSLE_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12771 = VMSLEU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12770 = VMSLEU_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12769 = VMSLEU_VI
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12768 = VMSIF_M
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12767 = VMSGT_VX
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12766 = VMSGT_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12765 = VMSGTU_VX
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12764 = VMSGTU_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12763 = VMSEQ_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12762 = VMSEQ_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12761 = VMSEQ_VI
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12760 = VMSBF_M
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #12759 = VMSBC_VXM
  { 3, &RISCVDescs.OperandInfo[8485] },  // Inst #12758 = VMSBC_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12757 = VMSBC_VVM
  { 3, &RISCVDescs.OperandInfo[8322] },  // Inst #12756 = VMSBC_VV
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12755 = VMOR_MM
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12754 = VMORN_MM
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12753 = VMNOR_MM
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12752 = VMNAND_MM
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12751 = VMIN_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12750 = VMIN_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12749 = VMINU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12748 = VMINU_VV
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12747 = VMFNE_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12746 = VMFNE_VF
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12745 = VMFLT_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12744 = VMFLT_VF
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12743 = VMFLE_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12742 = VMFLE_VF
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12741 = VMFGT_VF
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12740 = VMFGE_VF
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12739 = VMFEQ_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12738 = VMFEQ_VF
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12737 = VMERGE_VXM
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12736 = VMERGE_VVM
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12735 = VMERGE_VIM
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12734 = VMAX_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12733 = VMAX_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12732 = VMAXU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12731 = VMAXU_VV
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12730 = VMAND_MM
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12729 = VMANDN_MM
  { 5, &RISCVDescs.OperandInfo[8473] },  // Inst #12728 = VMADD_VX
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12727 = VMADD_VV
  { 4, &RISCVDescs.OperandInfo[8488] },  // Inst #12726 = VMADC_VXM
  { 3, &RISCVDescs.OperandInfo[8485] },  // Inst #12725 = VMADC_VX
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12724 = VMADC_VVM
  { 3, &RISCVDescs.OperandInfo[8322] },  // Inst #12723 = VMADC_VV
  { 4, &RISCVDescs.OperandInfo[8481] },  // Inst #12722 = VMADC_VIM
  { 3, &RISCVDescs.OperandInfo[8478] },  // Inst #12721 = VMADC_VI
  { 5, &RISCVDescs.OperandInfo[8473] },  // Inst #12720 = VMACC_VX
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12719 = VMACC_VV
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12718 = VLUXSEG8EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12717 = VLUXSEG8EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12716 = VLUXSEG8EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12715 = VLUXSEG8EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12714 = VLUXSEG7EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12713 = VLUXSEG7EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12712 = VLUXSEG7EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12711 = VLUXSEG7EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12710 = VLUXSEG6EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12709 = VLUXSEG6EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12708 = VLUXSEG6EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12707 = VLUXSEG6EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12706 = VLUXSEG5EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12705 = VLUXSEG5EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12704 = VLUXSEG5EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12703 = VLUXSEG5EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12702 = VLUXSEG4EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12701 = VLUXSEG4EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12700 = VLUXSEG4EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12699 = VLUXSEG4EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12698 = VLUXSEG3EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12697 = VLUXSEG3EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12696 = VLUXSEG3EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12695 = VLUXSEG3EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12694 = VLUXSEG2EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12693 = VLUXSEG2EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12692 = VLUXSEG2EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12691 = VLUXSEG2EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12690 = VLUXEI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12689 = VLUXEI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12688 = VLUXEI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12687 = VLUXEI16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12686 = VLSSEG8E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12685 = VLSSEG8E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12684 = VLSSEG8E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12683 = VLSSEG8E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12682 = VLSSEG7E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12681 = VLSSEG7E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12680 = VLSSEG7E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12679 = VLSSEG7E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12678 = VLSSEG6E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12677 = VLSSEG6E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12676 = VLSSEG6E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12675 = VLSSEG6E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12674 = VLSSEG5E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12673 = VLSSEG5E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12672 = VLSSEG5E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12671 = VLSSEG5E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12670 = VLSSEG4E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12669 = VLSSEG4E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12668 = VLSSEG4E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12667 = VLSSEG4E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12666 = VLSSEG3E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12665 = VLSSEG3E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12664 = VLSSEG3E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12663 = VLSSEG3E16_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12662 = VLSSEG2E8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12661 = VLSSEG2E64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12660 = VLSSEG2E32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12659 = VLSSEG2E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12658 = VLSEG8E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12657 = VLSEG8E8FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12656 = VLSEG8E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12655 = VLSEG8E64FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12654 = VLSEG8E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12653 = VLSEG8E32FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12652 = VLSEG8E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12651 = VLSEG8E16FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12650 = VLSEG7E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12649 = VLSEG7E8FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12648 = VLSEG7E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12647 = VLSEG7E64FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12646 = VLSEG7E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12645 = VLSEG7E32FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12644 = VLSEG7E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12643 = VLSEG7E16FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12642 = VLSEG6E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12641 = VLSEG6E8FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12640 = VLSEG6E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12639 = VLSEG6E64FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12638 = VLSEG6E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12637 = VLSEG6E32FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12636 = VLSEG6E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12635 = VLSEG6E16FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12634 = VLSEG5E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12633 = VLSEG5E8FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12632 = VLSEG5E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12631 = VLSEG5E64FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12630 = VLSEG5E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12629 = VLSEG5E32FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12628 = VLSEG5E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12627 = VLSEG5E16FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12626 = VLSEG4E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12625 = VLSEG4E8FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12624 = VLSEG4E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12623 = VLSEG4E64FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12622 = VLSEG4E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12621 = VLSEG4E32FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12620 = VLSEG4E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12619 = VLSEG4E16FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12618 = VLSEG3E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12617 = VLSEG3E8FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12616 = VLSEG3E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12615 = VLSEG3E64FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12614 = VLSEG3E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12613 = VLSEG3E32FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12612 = VLSEG3E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12611 = VLSEG3E16FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12610 = VLSEG2E8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12609 = VLSEG2E8FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12608 = VLSEG2E64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12607 = VLSEG2E64FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12606 = VLSEG2E32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12605 = VLSEG2E32FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12604 = VLSEG2E16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12603 = VLSEG2E16FF_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12602 = VLSE8_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12601 = VLSE64_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12600 = VLSE32_V
  { 4, &RISCVDescs.OperandInfo[8469] },  // Inst #12599 = VLSE16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12598 = VLOXSEG8EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12597 = VLOXSEG8EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12596 = VLOXSEG8EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12595 = VLOXSEG8EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12594 = VLOXSEG7EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12593 = VLOXSEG7EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12592 = VLOXSEG7EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12591 = VLOXSEG7EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12590 = VLOXSEG6EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12589 = VLOXSEG6EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12588 = VLOXSEG6EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12587 = VLOXSEG6EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12586 = VLOXSEG5EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12585 = VLOXSEG5EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12584 = VLOXSEG5EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12583 = VLOXSEG5EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12582 = VLOXSEG4EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12581 = VLOXSEG4EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12580 = VLOXSEG4EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12579 = VLOXSEG4EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12578 = VLOXSEG3EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12577 = VLOXSEG3EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12576 = VLOXSEG3EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12575 = VLOXSEG3EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12574 = VLOXSEG2EI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12573 = VLOXSEG2EI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12572 = VLOXSEG2EI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12571 = VLOXSEG2EI16_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12570 = VLOXEI8_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12569 = VLOXEI64_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12568 = VLOXEI32_V
  { 4, &RISCVDescs.OperandInfo[8465] },  // Inst #12567 = VLOXEI16_V
  { 2, &RISCVDescs.OperandInfo[8454] },  // Inst #12566 = VLM_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12565 = VLE8_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12564 = VLE8FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12563 = VLE64_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12562 = VLE64FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12561 = VLE32_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12560 = VLE32FF_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12559 = VLE16_V
  { 3, &RISCVDescs.OperandInfo[8462] },  // Inst #12558 = VLE16FF_V
  { 2, &RISCVDescs.OperandInfo[8460] },  // Inst #12557 = VL8RE8_V
  { 2, &RISCVDescs.OperandInfo[8460] },  // Inst #12556 = VL8RE64_V
  { 2, &RISCVDescs.OperandInfo[8460] },  // Inst #12555 = VL8RE32_V
  { 2, &RISCVDescs.OperandInfo[8460] },  // Inst #12554 = VL8RE16_V
  { 2, &RISCVDescs.OperandInfo[8458] },  // Inst #12553 = VL4RE8_V
  { 2, &RISCVDescs.OperandInfo[8458] },  // Inst #12552 = VL4RE64_V
  { 2, &RISCVDescs.OperandInfo[8458] },  // Inst #12551 = VL4RE32_V
  { 2, &RISCVDescs.OperandInfo[8458] },  // Inst #12550 = VL4RE16_V
  { 2, &RISCVDescs.OperandInfo[8456] },  // Inst #12549 = VL2RE8_V
  { 2, &RISCVDescs.OperandInfo[8456] },  // Inst #12548 = VL2RE64_V
  { 2, &RISCVDescs.OperandInfo[8456] },  // Inst #12547 = VL2RE32_V
  { 2, &RISCVDescs.OperandInfo[8456] },  // Inst #12546 = VL2RE16_V
  { 2, &RISCVDescs.OperandInfo[8454] },  // Inst #12545 = VL1RE8_V
  { 2, &RISCVDescs.OperandInfo[8454] },  // Inst #12544 = VL1RE64_V
  { 2, &RISCVDescs.OperandInfo[8454] },  // Inst #12543 = VL1RE32_V
  { 2, &RISCVDescs.OperandInfo[8454] },  // Inst #12542 = VL1RE16_V
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12541 = VIOTA_M
  { 2, &RISCVDescs.OperandInfo[8452] },  // Inst #12540 = VID_V
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12539 = VGMUL_VV
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12538 = VGHSH_VV
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12537 = VFWSUB_WV
  { 4, &RISCVDescs.OperandInfo[8436] },  // Inst #12536 = VFWSUB_WF
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12535 = VFWSUB_VV
  { 4, &RISCVDescs.OperandInfo[8436] },  // Inst #12534 = VFWSUB_VF
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12533 = VFWREDUSUM_VS
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12532 = VFWREDOSUM_VS
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #12531 = VFWNMSAC_VV
  { 5, &RISCVDescs.OperandInfo[8444] },  // Inst #12530 = VFWNMSAC_VF
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #12529 = VFWNMACC_VV
  { 5, &RISCVDescs.OperandInfo[8444] },  // Inst #12528 = VFWNMACC_VF
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12527 = VFWMUL_VV
  { 4, &RISCVDescs.OperandInfo[8436] },  // Inst #12526 = VFWMUL_VF
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #12525 = VFWMSAC_VV
  { 5, &RISCVDescs.OperandInfo[8444] },  // Inst #12524 = VFWMSAC_VF
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #12523 = VFWMACC_VV
  { 5, &RISCVDescs.OperandInfo[8444] },  // Inst #12522 = VFWMACC_VF
  { 3, &RISCVDescs.OperandInfo[8449] },  // Inst #12521 = VFWMACC_4x4x4
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #12520 = VFWMACCBF16_VV
  { 5, &RISCVDescs.OperandInfo[8444] },  // Inst #12519 = VFWMACCBF16_VF
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12518 = VFWCVT_X_F_V
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12517 = VFWCVT_XU_F_V
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12516 = VFWCVT_RTZ_X_F_V
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12515 = VFWCVT_RTZ_XU_F_V
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12514 = VFWCVT_F_X_V
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12513 = VFWCVT_F_XU_V
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12512 = VFWCVT_F_F_V
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12511 = VFWCVTBF16_F_F_V
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12510 = VFWADD_WV
  { 4, &RISCVDescs.OperandInfo[8436] },  // Inst #12509 = VFWADD_WF
  { 4, &RISCVDescs.OperandInfo[8440] },  // Inst #12508 = VFWADD_VV
  { 4, &RISCVDescs.OperandInfo[8436] },  // Inst #12507 = VFWADD_VF
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12506 = VFSUB_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12505 = VFSUB_VF
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12504 = VFSQRT_V
  { 4, &RISCVDescs.OperandInfo[8436] },  // Inst #12503 = VFSLIDE1UP_VF
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12502 = VFSLIDE1DOWN_VF
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12501 = VFSGNJ_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12500 = VFSGNJ_VF
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12499 = VFSGNJX_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12498 = VFSGNJX_VF
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12497 = VFSGNJN_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12496 = VFSGNJN_VF
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12495 = VFRSUB_VF
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12494 = VFRSQRT7_V
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12493 = VFREDUSUM_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12492 = VFREDOSUM_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12491 = VFREDMIN_VS
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12490 = VFREDMAX_VS
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12489 = VFREC7_V
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12488 = VFRDIV_VF
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12487 = VFNRCLIP_X_F_QF
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12486 = VFNRCLIP_XU_F_QF
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12485 = VFNMSUB_VV
  { 5, &RISCVDescs.OperandInfo[8416] },  // Inst #12484 = VFNMSUB_VF
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12483 = VFNMSAC_VV
  { 5, &RISCVDescs.OperandInfo[8416] },  // Inst #12482 = VFNMSAC_VF
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12481 = VFNMADD_VV
  { 5, &RISCVDescs.OperandInfo[8416] },  // Inst #12480 = VFNMADD_VF
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12479 = VFNMACC_VV
  { 5, &RISCVDescs.OperandInfo[8416] },  // Inst #12478 = VFNMACC_VF
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12477 = VFNCVT_X_F_W
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12476 = VFNCVT_XU_F_W
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12475 = VFNCVT_RTZ_X_F_W
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12474 = VFNCVT_RTZ_XU_F_W
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12473 = VFNCVT_ROD_F_F_W
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12472 = VFNCVT_F_X_W
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12471 = VFNCVT_F_XU_W
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12470 = VFNCVT_F_F_W
  { 3, &RISCVDescs.OperandInfo[8433] },  // Inst #12469 = VFNCVTBF16_F_F_W
  { 2, &RISCVDescs.OperandInfo[8431] },  // Inst #12468 = VFMV_V_F
  { 3, &RISCVDescs.OperandInfo[8428] },  // Inst #12467 = VFMV_S_F
  { 2, &RISCVDescs.OperandInfo[8426] },  // Inst #12466 = VFMV_F_S
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12465 = VFMUL_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12464 = VFMUL_VF
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12463 = VFMSUB_VV
  { 5, &RISCVDescs.OperandInfo[8416] },  // Inst #12462 = VFMSUB_VF
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12461 = VFMSAC_VV
  { 5, &RISCVDescs.OperandInfo[8416] },  // Inst #12460 = VFMSAC_VF
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12459 = VFMIN_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12458 = VFMIN_VF
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12457 = VFMERGE_VFM
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12456 = VFMAX_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12455 = VFMAX_VF
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12454 = VFMADD_VV
  { 5, &RISCVDescs.OperandInfo[8416] },  // Inst #12453 = VFMADD_VF
  { 5, &RISCVDescs.OperandInfo[8421] },  // Inst #12452 = VFMACC_VV
  { 5, &RISCVDescs.OperandInfo[8416] },  // Inst #12451 = VFMACC_VF
  { 3, &RISCVDescs.OperandInfo[8325] },  // Inst #12450 = VFIRST_M
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12449 = VFDIV_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12448 = VFDIV_VF
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12447 = VFCVT_X_F_V
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12446 = VFCVT_XU_F_V
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12445 = VFCVT_RTZ_X_F_V
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12444 = VFCVT_RTZ_XU_F_V
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12443 = VFCVT_F_X_V
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12442 = VFCVT_F_XU_V
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12441 = VFCLASS_V
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12440 = VFADD_VV
  { 4, &RISCVDescs.OperandInfo[8412] },  // Inst #12439 = VFADD_VF
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12438 = VDIV_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12437 = VDIV_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12436 = VDIVU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12435 = VDIVU_VV
  { 4, &RISCVDescs.OperandInfo[8408] },  // Inst #12434 = VC_XVW
  { 4, &RISCVDescs.OperandInfo[8408] },  // Inst #12433 = VC_XVV
  { 4, &RISCVDescs.OperandInfo[8404] },  // Inst #12432 = VC_XV
  { 4, &RISCVDescs.OperandInfo[8400] },  // Inst #12431 = VC_X
  { 5, &RISCVDescs.OperandInfo[8395] },  // Inst #12430 = VC_V_XVW
  { 5, &RISCVDescs.OperandInfo[8395] },  // Inst #12429 = VC_V_XVV
  { 4, &RISCVDescs.OperandInfo[8391] },  // Inst #12428 = VC_V_XV
  { 4, &RISCVDescs.OperandInfo[8387] },  // Inst #12427 = VC_V_X
  { 5, &RISCVDescs.OperandInfo[8382] },  // Inst #12426 = VC_V_VVW
  { 5, &RISCVDescs.OperandInfo[8382] },  // Inst #12425 = VC_V_VVV
  { 4, &RISCVDescs.OperandInfo[8378] },  // Inst #12424 = VC_V_VV
  { 5, &RISCVDescs.OperandInfo[8373] },  // Inst #12423 = VC_V_IVW
  { 5, &RISCVDescs.OperandInfo[8373] },  // Inst #12422 = VC_V_IVV
  { 4, &RISCVDescs.OperandInfo[8369] },  // Inst #12421 = VC_V_IV
  { 4, &RISCVDescs.OperandInfo[8365] },  // Inst #12420 = VC_V_I
  { 5, &RISCVDescs.OperandInfo[8360] },  // Inst #12419 = VC_V_FVW
  { 5, &RISCVDescs.OperandInfo[8360] },  // Inst #12418 = VC_V_FVV
  { 4, &RISCVDescs.OperandInfo[8356] },  // Inst #12417 = VC_V_FV
  { 4, &RISCVDescs.OperandInfo[8352] },  // Inst #12416 = VC_VVW
  { 4, &RISCVDescs.OperandInfo[8352] },  // Inst #12415 = VC_VVV
  { 4, &RISCVDescs.OperandInfo[8348] },  // Inst #12414 = VC_VV
  { 4, &RISCVDescs.OperandInfo[8344] },  // Inst #12413 = VC_IVW
  { 4, &RISCVDescs.OperandInfo[8344] },  // Inst #12412 = VC_IVV
  { 4, &RISCVDescs.OperandInfo[8340] },  // Inst #12411 = VC_IV
  { 4, &RISCVDescs.OperandInfo[8336] },  // Inst #12410 = VC_I
  { 4, &RISCVDescs.OperandInfo[8332] },  // Inst #12409 = VC_FVW
  { 4, &RISCVDescs.OperandInfo[8332] },  // Inst #12408 = VC_FVV
  { 4, &RISCVDescs.OperandInfo[8328] },  // Inst #12407 = VC_FV
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12406 = VCTZ_V
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12405 = VCPOP_V
  { 3, &RISCVDescs.OperandInfo[8325] },  // Inst #12404 = VCPOP_M
  { 3, &RISCVDescs.OperandInfo[8322] },  // Inst #12403 = VCOMPRESS_VM
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12402 = VCLZ_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12401 = VCLMUL_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12400 = VCLMUL_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12399 = VCLMULH_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12398 = VCLMULH_VV
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12397 = VBREV_V
  { 3, &RISCVDescs.OperandInfo[8319] },  // Inst #12396 = VBREV8_V
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12395 = VASUB_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12394 = VASUB_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12393 = VASUBU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12392 = VASUBU_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12391 = VAND_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12390 = VAND_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12389 = VAND_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12388 = VANDN_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12387 = VANDN_VV
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12386 = VAESZ_VS
  { 3, &RISCVDescs.OperandInfo[8316] },  // Inst #12385 = VAESKF2_VI
  { 3, &RISCVDescs.OperandInfo[8316] },  // Inst #12384 = VAESKF1_VI
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12383 = VAESEM_VV
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12382 = VAESEM_VS
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12381 = VAESEF_VV
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12380 = VAESEF_VS
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12379 = VAESDM_VV
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12378 = VAESDM_VS
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12377 = VAESDF_VV
  { 2, &RISCVDescs.OperandInfo[8314] },  // Inst #12376 = VAESDF_VS
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12375 = VADD_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12374 = VADD_VV
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12373 = VADD_VI
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12372 = VADC_VXM
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12371 = VADC_VVM
  { 4, &RISCVDescs.OperandInfo[8310] },  // Inst #12370 = VADC_VIM
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12369 = VAADD_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12368 = VAADD_VV
  { 4, &RISCVDescs.OperandInfo[8306] },  // Inst #12367 = VAADDU_VX
  { 4, &RISCVDescs.OperandInfo[8302] },  // Inst #12366 = VAADDU_VV
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12365 = UNZIP_RV32
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12364 = UNIMP
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12363 = TH_TSTNBZ
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #12362 = TH_TST
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12361 = TH_SYNC_S
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12360 = TH_SYNC_IS
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12359 = TH_SYNC_I
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12358 = TH_SYNC
  { 5, &RISCVDescs.OperandInfo[8292] },  // Inst #12357 = TH_SWIB
  { 5, &RISCVDescs.OperandInfo[8292] },  // Inst #12356 = TH_SWIA
  { 5, &RISCVDescs.OperandInfo[8297] },  // Inst #12355 = TH_SWD
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12354 = TH_SURW
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12353 = TH_SURH
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12352 = TH_SURD
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12351 = TH_SURB
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12350 = TH_SRW
  { 3, &RISCVDescs.OperandInfo[7869] },  // Inst #12349 = TH_SRRIW
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #12348 = TH_SRRI
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12347 = TH_SRH
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12346 = TH_SRD
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12345 = TH_SRB
  { 5, &RISCVDescs.OperandInfo[8292] },  // Inst #12344 = TH_SHIB
  { 5, &RISCVDescs.OperandInfo[8292] },  // Inst #12343 = TH_SHIA
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12342 = TH_SFENCE_VMAS
  { 5, &RISCVDescs.OperandInfo[8292] },  // Inst #12341 = TH_SDIB
  { 5, &RISCVDescs.OperandInfo[8292] },  // Inst #12340 = TH_SDIA
  { 5, &RISCVDescs.OperandInfo[8297] },  // Inst #12339 = TH_SDD
  { 5, &RISCVDescs.OperandInfo[8292] },  // Inst #12338 = TH_SBIB
  { 5, &RISCVDescs.OperandInfo[8292] },  // Inst #12337 = TH_SBIA
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12336 = TH_REVW
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12335 = TH_REV
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #12334 = TH_MVNEZ
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #12333 = TH_MVEQZ
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #12332 = TH_MULSW
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #12331 = TH_MULSH
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #12330 = TH_MULS
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #12329 = TH_MULAW
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #12328 = TH_MULAH
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #12327 = TH_MULA
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12326 = TH_LWUIB
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12325 = TH_LWUIA
  { 5, &RISCVDescs.OperandInfo[8287] },  // Inst #12324 = TH_LWUD
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12323 = TH_LWIB
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12322 = TH_LWIA
  { 5, &RISCVDescs.OperandInfo[8287] },  // Inst #12321 = TH_LWD
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12320 = TH_LURWU
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12319 = TH_LURW
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12318 = TH_LURHU
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12317 = TH_LURH
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12316 = TH_LURD
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12315 = TH_LURBU
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12314 = TH_LURB
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12313 = TH_LRWU
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12312 = TH_LRW
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12311 = TH_LRHU
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12310 = TH_LRH
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12309 = TH_LRD
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12308 = TH_LRBU
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12307 = TH_LRB
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12306 = TH_LHUIB
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12305 = TH_LHUIA
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12304 = TH_LHIB
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12303 = TH_LHIA
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12302 = TH_LDIB
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12301 = TH_LDIA
  { 5, &RISCVDescs.OperandInfo[8287] },  // Inst #12300 = TH_LDD
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12299 = TH_LBUIB
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12298 = TH_LBUIA
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12297 = TH_LBIB
  { 5, &RISCVDescs.OperandInfo[8282] },  // Inst #12296 = TH_LBIA
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12295 = TH_L2CACHE_IALL
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12294 = TH_L2CACHE_CIALL
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12293 = TH_L2CACHE_CALL
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12292 = TH_ICACHE_IVA
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12291 = TH_ICACHE_IPA
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12290 = TH_ICACHE_IALLS
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12289 = TH_ICACHE_IALL
  { 4, &RISCVDescs.OperandInfo[8278] },  // Inst #12288 = TH_FSURW
  { 4, &RISCVDescs.OperandInfo[8274] },  // Inst #12287 = TH_FSURD
  { 4, &RISCVDescs.OperandInfo[8278] },  // Inst #12286 = TH_FSRW
  { 4, &RISCVDescs.OperandInfo[8274] },  // Inst #12285 = TH_FSRD
  { 4, &RISCVDescs.OperandInfo[8278] },  // Inst #12284 = TH_FLURW
  { 4, &RISCVDescs.OperandInfo[8274] },  // Inst #12283 = TH_FLURD
  { 4, &RISCVDescs.OperandInfo[8278] },  // Inst #12282 = TH_FLRW
  { 4, &RISCVDescs.OperandInfo[8274] },  // Inst #12281 = TH_FLRD
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12280 = TH_FF1
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12279 = TH_FF0
  { 4, &RISCVDescs.OperandInfo[8270] },  // Inst #12278 = TH_EXTU
  { 4, &RISCVDescs.OperandInfo[8270] },  // Inst #12277 = TH_EXT
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12276 = TH_DCACHE_IVA
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12275 = TH_DCACHE_ISW
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12274 = TH_DCACHE_IPA
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12273 = TH_DCACHE_IALL
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12272 = TH_DCACHE_CVAL1
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12271 = TH_DCACHE_CVA
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12270 = TH_DCACHE_CSW
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12269 = TH_DCACHE_CPAL1
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12268 = TH_DCACHE_CPA
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12267 = TH_DCACHE_CIVA
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12266 = TH_DCACHE_CISW
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #12265 = TH_DCACHE_CIPA
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12264 = TH_DCACHE_CIALL
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #12263 = TH_DCACHE_CALL
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12262 = TH_ADDSL
  { 5, &RISCVDescs.OperandInfo[8265] },  // Inst #12261 = THVdotVMAQA_VX
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #12260 = THVdotVMAQA_VV
  { 5, &RISCVDescs.OperandInfo[8265] },  // Inst #12259 = THVdotVMAQAU_VX
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #12258 = THVdotVMAQAU_VV
  { 5, &RISCVDescs.OperandInfo[8265] },  // Inst #12257 = THVdotVMAQAUS_VX
  { 5, &RISCVDescs.OperandInfo[8265] },  // Inst #12256 = THVdotVMAQASU_VX
  { 5, &RISCVDescs.OperandInfo[8260] },  // Inst #12255 = THVdotVMAQASU_VV
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12254 = SW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12253 = SUBW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12252 = SUB
  { 1, &RISCVDescs.OperandInfo[7959] },  // Inst #12251 = SSRDP
  { 1, &RISCVDescs.OperandInfo[8259] },  // Inst #12250 = SSPUSH
  { 1, &RISCVDescs.OperandInfo[8259] },  // Inst #12249 = SSPOPCHK
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12248 = SSAMOSWAP_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12247 = SSAMOSWAP_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12246 = SSAMOSWAP_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12245 = SSAMOSWAP_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12244 = SSAMOSWAP_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12243 = SSAMOSWAP_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12242 = SSAMOSWAP_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12241 = SSAMOSWAP_D
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12240 = SRLW
  { 3, &RISCVDescs.OperandInfo[7869] },  // Inst #12239 = SRLIW
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #12238 = SRLI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12237 = SRL
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12236 = SRET
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12235 = SRAW
  { 3, &RISCVDescs.OperandInfo[7869] },  // Inst #12234 = SRAIW
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #12233 = SRAI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12232 = SRA
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12231 = SM4KS
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #12230 = SM4ED
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12229 = SM3P1
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12228 = SM3P0
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12227 = SLTU
  { 3, &RISCVDescs.OperandInfo[7808] },  // Inst #12226 = SLTIU
  { 3, &RISCVDescs.OperandInfo[7808] },  // Inst #12225 = SLTI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12224 = SLT
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12223 = SLLW
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #12222 = SLLI_UW
  { 3, &RISCVDescs.OperandInfo[7869] },  // Inst #12221 = SLLIW
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #12220 = SLLI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12219 = SLL
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12218 = SINVAL_VMA
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12217 = SHA512SUM1R
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12216 = SHA512SUM1
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12215 = SHA512SUM0R
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12214 = SHA512SUM0
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12213 = SHA512SIG1L
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12212 = SHA512SIG1H
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12211 = SHA512SIG1
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12210 = SHA512SIG0L
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12209 = SHA512SIG0H
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12208 = SHA512SIG0
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12207 = SHA256SUM1
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12206 = SHA256SUM0
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12205 = SHA256SIG1
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12204 = SHA256SIG0
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12203 = SH3ADD_UW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12202 = SH3ADD
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12201 = SH2ADD_UW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12200 = SH2ADD
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12199 = SH1ADD_UW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12198 = SH1ADD
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12197 = SH
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12196 = SFENCE_W_INVAL
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12195 = SFENCE_VMA
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12194 = SFENCE_INVAL_IR
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12193 = SEXT_H
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12192 = SEXT_B
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12191 = SD
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12190 = SC_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12189 = SC_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12188 = SC_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12187 = SC_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12186 = SC_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12185 = SC_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12184 = SC_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #12183 = SC_D
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12182 = SB
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12181 = RORW
  { 3, &RISCVDescs.OperandInfo[7869] },  // Inst #12180 = RORIW
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #12179 = RORI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12178 = ROR
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12177 = ROLW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12176 = ROL
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12175 = REV8_RV64
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12174 = REV8_RV32
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12173 = REMW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12172 = REMUW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12171 = REMU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12170 = REM
  { 2, &RISCVDescs.OperandInfo[8257] },  // Inst #12169 = PREFETCH_W
  { 2, &RISCVDescs.OperandInfo[8257] },  // Inst #12168 = PREFETCH_R
  { 2, &RISCVDescs.OperandInfo[8257] },  // Inst #12167 = PREFETCH_I
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12166 = PACKW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12165 = PACKH
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12164 = PACK
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12163 = ORN
  { 3, &RISCVDescs.OperandInfo[7808] },  // Inst #12162 = ORI
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12161 = ORC_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12160 = OR
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12159 = MULW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12158 = MULHU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12157 = MULHSU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12156 = MULH
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12155 = MUL
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12154 = MRET
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12153 = MOPRR7
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12152 = MOPRR6
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12151 = MOPRR5
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12150 = MOPRR4
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12149 = MOPRR3
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12148 = MOPRR2
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12147 = MOPRR1
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12146 = MOPRR0
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12145 = MOPR9
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12144 = MOPR8
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12143 = MOPR7
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12142 = MOPR6
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12141 = MOPR5
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12140 = MOPR4
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12139 = MOPR31
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12138 = MOPR30
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12137 = MOPR3
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12136 = MOPR29
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12135 = MOPR28
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12134 = MOPR27
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12133 = MOPR26
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12132 = MOPR25
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12131 = MOPR24
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12130 = MOPR23
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12129 = MOPR22
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12128 = MOPR21
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12127 = MOPR20
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12126 = MOPR2
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12125 = MOPR19
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12124 = MOPR18
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12123 = MOPR17
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12122 = MOPR16
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12121 = MOPR15
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12120 = MOPR14
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12119 = MOPR13
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12118 = MOPR12
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12117 = MOPR11
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12116 = MOPR10
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12115 = MOPR1
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12114 = MOPR0
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12113 = MINU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12112 = MIN
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12111 = MAXU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12110 = MAX
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12109 = LWU
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12108 = LW
  { 2, &RISCVDescs.OperandInfo[7829] },  // Inst #12107 = LUI
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12106 = LR_W_RL
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12105 = LR_W_AQ_RL
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12104 = LR_W_AQ
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12103 = LR_W
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12102 = LR_D_RL
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12101 = LR_D_AQ_RL
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12100 = LR_D_AQ
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12099 = LR_D
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12098 = LHU
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12097 = LH
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12096 = LD
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12095 = LBU
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #12094 = LB
  { 3, &RISCVDescs.OperandInfo[7808] },  // Inst #12093 = JALR
  { 2, &RISCVDescs.OperandInfo[8255] },  // Inst #12092 = JAL
  { 3, &RISCVDescs.OperandInfo[8252] },  // Inst #12091 = InsnU
  { 5, &RISCVDescs.OperandInfo[8247] },  // Inst #12090 = InsnS
  { 7, &RISCVDescs.OperandInfo[8240] },  // Inst #12089 = InsnR4
  { 6, &RISCVDescs.OperandInfo[8234] },  // Inst #12088 = InsnR
  { 3, &RISCVDescs.OperandInfo[8231] },  // Inst #12087 = InsnJ
  { 5, &RISCVDescs.OperandInfo[8226] },  // Inst #12086 = InsnI_Mem
  { 5, &RISCVDescs.OperandInfo[8226] },  // Inst #12085 = InsnI
  { 4, &RISCVDescs.OperandInfo[8222] },  // Inst #12084 = InsnCSS
  { 5, &RISCVDescs.OperandInfo[8217] },  // Inst #12083 = InsnCS
  { 4, &RISCVDescs.OperandInfo[8213] },  // Inst #12082 = InsnCR
  { 5, &RISCVDescs.OperandInfo[8208] },  // Inst #12081 = InsnCL
  { 3, &RISCVDescs.OperandInfo[8205] },  // Inst #12080 = InsnCJ
  { 4, &RISCVDescs.OperandInfo[8201] },  // Inst #12079 = InsnCIW
  { 4, &RISCVDescs.OperandInfo[8197] },  // Inst #12078 = InsnCI
  { 4, &RISCVDescs.OperandInfo[8193] },  // Inst #12077 = InsnCB
  { 5, &RISCVDescs.OperandInfo[8188] },  // Inst #12076 = InsnCA
  { 5, &RISCVDescs.OperandInfo[8183] },  // Inst #12075 = InsnB
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12074 = HSV_W
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12073 = HSV_H
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12072 = HSV_D
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12071 = HSV_B
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12070 = HLV_WU
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12069 = HLV_W
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12068 = HLV_HU
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12067 = HLV_H
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12066 = HLV_D
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12065 = HLV_BU
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12064 = HLV_B
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12063 = HLVX_WU
  { 2, &RISCVDescs.OperandInfo[8181] },  // Inst #12062 = HLVX_HU
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12061 = HINVAL_VVMA
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12060 = HINVAL_GVMA
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12059 = HFENCE_VVMA
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #12058 = HFENCE_GVMA
  { 3, &RISCVDescs.OperandInfo[8101] },  // Inst #12057 = FSW
  { 4, &RISCVDescs.OperandInfo[224] },  // Inst #12056 = FSUB_S_INX
  { 4, &RISCVDescs.OperandInfo[220] },  // Inst #12055 = FSUB_S
  { 4, &RISCVDescs.OperandInfo[216] },  // Inst #12054 = FSUB_H_INX
  { 4, &RISCVDescs.OperandInfo[212] },  // Inst #12053 = FSUB_H
  { 4, &RISCVDescs.OperandInfo[148] },  // Inst #12052 = FSUB_D_INX
  { 4, &RISCVDescs.OperandInfo[208] },  // Inst #12051 = FSUB_D_IN32X
  { 4, &RISCVDescs.OperandInfo[204] },  // Inst #12050 = FSUB_D
  { 3, &RISCVDescs.OperandInfo[8178] },  // Inst #12049 = FSQRT_S_INX
  { 3, &RISCVDescs.OperandInfo[8169] },  // Inst #12048 = FSQRT_S
  { 3, &RISCVDescs.OperandInfo[8175] },  // Inst #12047 = FSQRT_H_INX
  { 3, &RISCVDescs.OperandInfo[8166] },  // Inst #12046 = FSQRT_H
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #12045 = FSQRT_D_INX
  { 3, &RISCVDescs.OperandInfo[8172] },  // Inst #12044 = FSQRT_D_IN32X
  { 3, &RISCVDescs.OperandInfo[8163] },  // Inst #12043 = FSQRT_D
  { 3, &RISCVDescs.OperandInfo[8092] },  // Inst #12042 = FSH
  { 3, &RISCVDescs.OperandInfo[8154] },  // Inst #12041 = FSGNJ_S_INX
  { 3, &RISCVDescs.OperandInfo[8145] },  // Inst #12040 = FSGNJ_S
  { 3, &RISCVDescs.OperandInfo[8151] },  // Inst #12039 = FSGNJ_H_INX
  { 3, &RISCVDescs.OperandInfo[8142] },  // Inst #12038 = FSGNJ_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12037 = FSGNJ_D_INX
  { 3, &RISCVDescs.OperandInfo[8148] },  // Inst #12036 = FSGNJ_D_IN32X
  { 3, &RISCVDescs.OperandInfo[8139] },  // Inst #12035 = FSGNJ_D
  { 3, &RISCVDescs.OperandInfo[8154] },  // Inst #12034 = FSGNJX_S_INX
  { 3, &RISCVDescs.OperandInfo[8145] },  // Inst #12033 = FSGNJX_S
  { 3, &RISCVDescs.OperandInfo[8151] },  // Inst #12032 = FSGNJX_H_INX
  { 3, &RISCVDescs.OperandInfo[8142] },  // Inst #12031 = FSGNJX_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12030 = FSGNJX_D_INX
  { 3, &RISCVDescs.OperandInfo[8148] },  // Inst #12029 = FSGNJX_D_IN32X
  { 3, &RISCVDescs.OperandInfo[8139] },  // Inst #12028 = FSGNJX_D
  { 3, &RISCVDescs.OperandInfo[8154] },  // Inst #12027 = FSGNJN_S_INX
  { 3, &RISCVDescs.OperandInfo[8145] },  // Inst #12026 = FSGNJN_S
  { 3, &RISCVDescs.OperandInfo[8151] },  // Inst #12025 = FSGNJN_H_INX
  { 3, &RISCVDescs.OperandInfo[8142] },  // Inst #12024 = FSGNJN_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #12023 = FSGNJN_D_INX
  { 3, &RISCVDescs.OperandInfo[8148] },  // Inst #12022 = FSGNJN_D_IN32X
  { 3, &RISCVDescs.OperandInfo[8139] },  // Inst #12021 = FSGNJN_D
  { 3, &RISCVDescs.OperandInfo[8089] },  // Inst #12020 = FSD
  { 3, &RISCVDescs.OperandInfo[8169] },  // Inst #12019 = FROUND_S
  { 3, &RISCVDescs.OperandInfo[8166] },  // Inst #12018 = FROUND_H
  { 3, &RISCVDescs.OperandInfo[8163] },  // Inst #12017 = FROUND_D
  { 3, &RISCVDescs.OperandInfo[8169] },  // Inst #12016 = FROUNDNX_S
  { 3, &RISCVDescs.OperandInfo[8166] },  // Inst #12015 = FROUNDNX_H
  { 3, &RISCVDescs.OperandInfo[8163] },  // Inst #12014 = FROUNDNX_D
  { 5, &RISCVDescs.OperandInfo[8134] },  // Inst #12013 = FNMSUB_S_INX
  { 5, &RISCVDescs.OperandInfo[8129] },  // Inst #12012 = FNMSUB_S
  { 5, &RISCVDescs.OperandInfo[8124] },  // Inst #12011 = FNMSUB_H_INX
  { 5, &RISCVDescs.OperandInfo[8119] },  // Inst #12010 = FNMSUB_H
  { 5, &RISCVDescs.OperandInfo[8114] },  // Inst #12009 = FNMSUB_D_INX
  { 5, &RISCVDescs.OperandInfo[8109] },  // Inst #12008 = FNMSUB_D_IN32X
  { 5, &RISCVDescs.OperandInfo[8104] },  // Inst #12007 = FNMSUB_D
  { 5, &RISCVDescs.OperandInfo[8134] },  // Inst #12006 = FNMADD_S_INX
  { 5, &RISCVDescs.OperandInfo[8129] },  // Inst #12005 = FNMADD_S
  { 5, &RISCVDescs.OperandInfo[8124] },  // Inst #12004 = FNMADD_H_INX
  { 5, &RISCVDescs.OperandInfo[8119] },  // Inst #12003 = FNMADD_H
  { 5, &RISCVDescs.OperandInfo[8114] },  // Inst #12002 = FNMADD_D_INX
  { 5, &RISCVDescs.OperandInfo[8109] },  // Inst #12001 = FNMADD_D_IN32X
  { 5, &RISCVDescs.OperandInfo[8104] },  // Inst #12000 = FNMADD_D
  { 2, &RISCVDescs.OperandInfo[8012] },  // Inst #11999 = FMV_X_W_FPR64
  { 2, &RISCVDescs.OperandInfo[8020] },  // Inst #11998 = FMV_X_W
  { 2, &RISCVDescs.OperandInfo[8016] },  // Inst #11997 = FMV_X_H
  { 2, &RISCVDescs.OperandInfo[8012] },  // Inst #11996 = FMV_X_D
  { 2, &RISCVDescs.OperandInfo[8161] },  // Inst #11995 = FMV_W_X
  { 2, &RISCVDescs.OperandInfo[8159] },  // Inst #11994 = FMV_H_X
  { 2, &RISCVDescs.OperandInfo[8157] },  // Inst #11993 = FMV_D_X
  { 3, &RISCVDescs.OperandInfo[140] },  // Inst #11992 = FMVP_D_X
  { 2, &RISCVDescs.OperandInfo[8012] },  // Inst #11991 = FMVH_X_D
  { 4, &RISCVDescs.OperandInfo[224] },  // Inst #11990 = FMUL_S_INX
  { 4, &RISCVDescs.OperandInfo[220] },  // Inst #11989 = FMUL_S
  { 4, &RISCVDescs.OperandInfo[216] },  // Inst #11988 = FMUL_H_INX
  { 4, &RISCVDescs.OperandInfo[212] },  // Inst #11987 = FMUL_H
  { 4, &RISCVDescs.OperandInfo[148] },  // Inst #11986 = FMUL_D_INX
  { 4, &RISCVDescs.OperandInfo[208] },  // Inst #11985 = FMUL_D_IN32X
  { 4, &RISCVDescs.OperandInfo[204] },  // Inst #11984 = FMUL_D
  { 5, &RISCVDescs.OperandInfo[8134] },  // Inst #11983 = FMSUB_S_INX
  { 5, &RISCVDescs.OperandInfo[8129] },  // Inst #11982 = FMSUB_S
  { 5, &RISCVDescs.OperandInfo[8124] },  // Inst #11981 = FMSUB_H_INX
  { 5, &RISCVDescs.OperandInfo[8119] },  // Inst #11980 = FMSUB_H
  { 5, &RISCVDescs.OperandInfo[8114] },  // Inst #11979 = FMSUB_D_INX
  { 5, &RISCVDescs.OperandInfo[8109] },  // Inst #11978 = FMSUB_D_IN32X
  { 5, &RISCVDescs.OperandInfo[8104] },  // Inst #11977 = FMSUB_D
  { 3, &RISCVDescs.OperandInfo[8154] },  // Inst #11976 = FMIN_S_INX
  { 3, &RISCVDescs.OperandInfo[8145] },  // Inst #11975 = FMIN_S
  { 3, &RISCVDescs.OperandInfo[8151] },  // Inst #11974 = FMIN_H_INX
  { 3, &RISCVDescs.OperandInfo[8142] },  // Inst #11973 = FMIN_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11972 = FMIN_D_INX
  { 3, &RISCVDescs.OperandInfo[8148] },  // Inst #11971 = FMIN_D_IN32X
  { 3, &RISCVDescs.OperandInfo[8139] },  // Inst #11970 = FMIN_D
  { 3, &RISCVDescs.OperandInfo[8145] },  // Inst #11969 = FMINM_S
  { 3, &RISCVDescs.OperandInfo[8142] },  // Inst #11968 = FMINM_H
  { 3, &RISCVDescs.OperandInfo[8139] },  // Inst #11967 = FMINM_D
  { 3, &RISCVDescs.OperandInfo[8154] },  // Inst #11966 = FMAX_S_INX
  { 3, &RISCVDescs.OperandInfo[8145] },  // Inst #11965 = FMAX_S
  { 3, &RISCVDescs.OperandInfo[8151] },  // Inst #11964 = FMAX_H_INX
  { 3, &RISCVDescs.OperandInfo[8142] },  // Inst #11963 = FMAX_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11962 = FMAX_D_INX
  { 3, &RISCVDescs.OperandInfo[8148] },  // Inst #11961 = FMAX_D_IN32X
  { 3, &RISCVDescs.OperandInfo[8139] },  // Inst #11960 = FMAX_D
  { 3, &RISCVDescs.OperandInfo[8145] },  // Inst #11959 = FMAXM_S
  { 3, &RISCVDescs.OperandInfo[8142] },  // Inst #11958 = FMAXM_H
  { 3, &RISCVDescs.OperandInfo[8139] },  // Inst #11957 = FMAXM_D
  { 5, &RISCVDescs.OperandInfo[8134] },  // Inst #11956 = FMADD_S_INX
  { 5, &RISCVDescs.OperandInfo[8129] },  // Inst #11955 = FMADD_S
  { 5, &RISCVDescs.OperandInfo[8124] },  // Inst #11954 = FMADD_H_INX
  { 5, &RISCVDescs.OperandInfo[8119] },  // Inst #11953 = FMADD_H
  { 5, &RISCVDescs.OperandInfo[8114] },  // Inst #11952 = FMADD_D_INX
  { 5, &RISCVDescs.OperandInfo[8109] },  // Inst #11951 = FMADD_D_IN32X
  { 5, &RISCVDescs.OperandInfo[8104] },  // Inst #11950 = FMADD_D
  { 3, &RISCVDescs.OperandInfo[8101] },  // Inst #11949 = FLW
  { 3, &RISCVDescs.OperandInfo[273] },  // Inst #11948 = FLT_S_INX
  { 3, &RISCVDescs.OperandInfo[270] },  // Inst #11947 = FLT_S
  { 3, &RISCVDescs.OperandInfo[267] },  // Inst #11946 = FLT_H_INX
  { 3, &RISCVDescs.OperandInfo[264] },  // Inst #11945 = FLT_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11944 = FLT_D_INX
  { 3, &RISCVDescs.OperandInfo[258] },  // Inst #11943 = FLT_D_IN32X
  { 3, &RISCVDescs.OperandInfo[255] },  // Inst #11942 = FLT_D
  { 3, &RISCVDescs.OperandInfo[270] },  // Inst #11941 = FLTQ_S
  { 3, &RISCVDescs.OperandInfo[264] },  // Inst #11940 = FLTQ_H
  { 3, &RISCVDescs.OperandInfo[255] },  // Inst #11939 = FLTQ_D
  { 2, &RISCVDescs.OperandInfo[8099] },  // Inst #11938 = FLI_S
  { 2, &RISCVDescs.OperandInfo[8097] },  // Inst #11937 = FLI_H
  { 2, &RISCVDescs.OperandInfo[8095] },  // Inst #11936 = FLI_D
  { 3, &RISCVDescs.OperandInfo[8092] },  // Inst #11935 = FLH
  { 3, &RISCVDescs.OperandInfo[273] },  // Inst #11934 = FLE_S_INX
  { 3, &RISCVDescs.OperandInfo[270] },  // Inst #11933 = FLE_S
  { 3, &RISCVDescs.OperandInfo[267] },  // Inst #11932 = FLE_H_INX
  { 3, &RISCVDescs.OperandInfo[264] },  // Inst #11931 = FLE_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11930 = FLE_D_INX
  { 3, &RISCVDescs.OperandInfo[258] },  // Inst #11929 = FLE_D_IN32X
  { 3, &RISCVDescs.OperandInfo[255] },  // Inst #11928 = FLE_D
  { 3, &RISCVDescs.OperandInfo[270] },  // Inst #11927 = FLEQ_S
  { 3, &RISCVDescs.OperandInfo[264] },  // Inst #11926 = FLEQ_H
  { 3, &RISCVDescs.OperandInfo[255] },  // Inst #11925 = FLEQ_D
  { 3, &RISCVDescs.OperandInfo[8089] },  // Inst #11924 = FLD
  { 3, &RISCVDescs.OperandInfo[273] },  // Inst #11923 = FEQ_S_INX
  { 3, &RISCVDescs.OperandInfo[270] },  // Inst #11922 = FEQ_S
  { 3, &RISCVDescs.OperandInfo[267] },  // Inst #11921 = FEQ_H_INX
  { 3, &RISCVDescs.OperandInfo[264] },  // Inst #11920 = FEQ_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11919 = FEQ_D_INX
  { 3, &RISCVDescs.OperandInfo[258] },  // Inst #11918 = FEQ_D_IN32X
  { 3, &RISCVDescs.OperandInfo[255] },  // Inst #11917 = FEQ_D
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11916 = FENCE_TSO
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11915 = FENCE_I
  { 2, &RISCVDescs.OperandInfo[8087] },  // Inst #11914 = FENCE
  { 4, &RISCVDescs.OperandInfo[224] },  // Inst #11913 = FDIV_S_INX
  { 4, &RISCVDescs.OperandInfo[220] },  // Inst #11912 = FDIV_S
  { 4, &RISCVDescs.OperandInfo[216] },  // Inst #11911 = FDIV_H_INX
  { 4, &RISCVDescs.OperandInfo[212] },  // Inst #11910 = FDIV_H
  { 4, &RISCVDescs.OperandInfo[148] },  // Inst #11909 = FDIV_D_INX
  { 4, &RISCVDescs.OperandInfo[208] },  // Inst #11908 = FDIV_D_IN32X
  { 4, &RISCVDescs.OperandInfo[204] },  // Inst #11907 = FDIV_D
  { 3, &RISCVDescs.OperandInfo[8045] },  // Inst #11906 = FCVT_W_S_INX
  { 3, &RISCVDescs.OperandInfo[201] },  // Inst #11905 = FCVT_W_S
  { 3, &RISCVDescs.OperandInfo[8033] },  // Inst #11904 = FCVT_W_H_INX
  { 3, &RISCVDescs.OperandInfo[198] },  // Inst #11903 = FCVT_W_H
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #11902 = FCVT_W_D_INX
  { 3, &RISCVDescs.OperandInfo[8084] },  // Inst #11901 = FCVT_W_D_IN32X
  { 3, &RISCVDescs.OperandInfo[195] },  // Inst #11900 = FCVT_W_D
  { 3, &RISCVDescs.OperandInfo[8045] },  // Inst #11899 = FCVT_WU_S_INX
  { 3, &RISCVDescs.OperandInfo[201] },  // Inst #11898 = FCVT_WU_S
  { 3, &RISCVDescs.OperandInfo[8033] },  // Inst #11897 = FCVT_WU_H_INX
  { 3, &RISCVDescs.OperandInfo[198] },  // Inst #11896 = FCVT_WU_H
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #11895 = FCVT_WU_D_INX
  { 3, &RISCVDescs.OperandInfo[8084] },  // Inst #11894 = FCVT_WU_D_IN32X
  { 3, &RISCVDescs.OperandInfo[195] },  // Inst #11893 = FCVT_WU_D
  { 3, &RISCVDescs.OperandInfo[8075] },  // Inst #11892 = FCVT_S_W_INX
  { 3, &RISCVDescs.OperandInfo[8075] },  // Inst #11891 = FCVT_S_WU_INX
  { 3, &RISCVDescs.OperandInfo[8081] },  // Inst #11890 = FCVT_S_WU
  { 3, &RISCVDescs.OperandInfo[8081] },  // Inst #11889 = FCVT_S_W
  { 3, &RISCVDescs.OperandInfo[8075] },  // Inst #11888 = FCVT_S_L_INX
  { 3, &RISCVDescs.OperandInfo[8075] },  // Inst #11887 = FCVT_S_LU_INX
  { 3, &RISCVDescs.OperandInfo[8081] },  // Inst #11886 = FCVT_S_LU
  { 3, &RISCVDescs.OperandInfo[8081] },  // Inst #11885 = FCVT_S_L
  { 3, &RISCVDescs.OperandInfo[8078] },  // Inst #11884 = FCVT_S_H_INX
  { 3, &RISCVDescs.OperandInfo[8066] },  // Inst #11883 = FCVT_S_H
  { 3, &RISCVDescs.OperandInfo[8075] },  // Inst #11882 = FCVT_S_D_INX
  { 3, &RISCVDescs.OperandInfo[8072] },  // Inst #11881 = FCVT_S_D_IN32X
  { 3, &RISCVDescs.OperandInfo[8069] },  // Inst #11880 = FCVT_S_D
  { 3, &RISCVDescs.OperandInfo[8066] },  // Inst #11879 = FCVT_S_BF16
  { 3, &RISCVDescs.OperandInfo[8045] },  // Inst #11878 = FCVT_L_S_INX
  { 3, &RISCVDescs.OperandInfo[201] },  // Inst #11877 = FCVT_L_S
  { 3, &RISCVDescs.OperandInfo[8033] },  // Inst #11876 = FCVT_L_H_INX
  { 3, &RISCVDescs.OperandInfo[198] },  // Inst #11875 = FCVT_L_H
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #11874 = FCVT_L_D_INX
  { 3, &RISCVDescs.OperandInfo[195] },  // Inst #11873 = FCVT_L_D
  { 3, &RISCVDescs.OperandInfo[8045] },  // Inst #11872 = FCVT_LU_S_INX
  { 3, &RISCVDescs.OperandInfo[201] },  // Inst #11871 = FCVT_LU_S
  { 3, &RISCVDescs.OperandInfo[8033] },  // Inst #11870 = FCVT_LU_H_INX
  { 3, &RISCVDescs.OperandInfo[198] },  // Inst #11869 = FCVT_LU_H
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #11868 = FCVT_LU_D_INX
  { 3, &RISCVDescs.OperandInfo[195] },  // Inst #11867 = FCVT_LU_D
  { 3, &RISCVDescs.OperandInfo[8057] },  // Inst #11866 = FCVT_H_W_INX
  { 3, &RISCVDescs.OperandInfo[8057] },  // Inst #11865 = FCVT_H_WU_INX
  { 3, &RISCVDescs.OperandInfo[8060] },  // Inst #11864 = FCVT_H_WU
  { 3, &RISCVDescs.OperandInfo[8060] },  // Inst #11863 = FCVT_H_W
  { 3, &RISCVDescs.OperandInfo[8063] },  // Inst #11862 = FCVT_H_S_INX
  { 3, &RISCVDescs.OperandInfo[8024] },  // Inst #11861 = FCVT_H_S
  { 3, &RISCVDescs.OperandInfo[8057] },  // Inst #11860 = FCVT_H_L_INX
  { 3, &RISCVDescs.OperandInfo[8057] },  // Inst #11859 = FCVT_H_LU_INX
  { 3, &RISCVDescs.OperandInfo[8060] },  // Inst #11858 = FCVT_H_LU
  { 3, &RISCVDescs.OperandInfo[8060] },  // Inst #11857 = FCVT_H_L
  { 3, &RISCVDescs.OperandInfo[8057] },  // Inst #11856 = FCVT_H_D_INX
  { 3, &RISCVDescs.OperandInfo[8054] },  // Inst #11855 = FCVT_H_D_IN32X
  { 3, &RISCVDescs.OperandInfo[8051] },  // Inst #11854 = FCVT_H_D
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #11853 = FCVT_D_W_INX
  { 3, &RISCVDescs.OperandInfo[8048] },  // Inst #11852 = FCVT_D_W_IN32X
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #11851 = FCVT_D_WU_INX
  { 3, &RISCVDescs.OperandInfo[8048] },  // Inst #11850 = FCVT_D_WU_IN32X
  { 3, &RISCVDescs.OperandInfo[8036] },  // Inst #11849 = FCVT_D_WU
  { 3, &RISCVDescs.OperandInfo[8036] },  // Inst #11848 = FCVT_D_W
  { 3, &RISCVDescs.OperandInfo[8045] },  // Inst #11847 = FCVT_D_S_INX
  { 3, &RISCVDescs.OperandInfo[8042] },  // Inst #11846 = FCVT_D_S_IN32X
  { 3, &RISCVDescs.OperandInfo[8039] },  // Inst #11845 = FCVT_D_S
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #11844 = FCVT_D_L_INX
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #11843 = FCVT_D_LU_INX
  { 3, &RISCVDescs.OperandInfo[8036] },  // Inst #11842 = FCVT_D_LU
  { 3, &RISCVDescs.OperandInfo[8036] },  // Inst #11841 = FCVT_D_L
  { 3, &RISCVDescs.OperandInfo[8033] },  // Inst #11840 = FCVT_D_H_INX
  { 3, &RISCVDescs.OperandInfo[8030] },  // Inst #11839 = FCVT_D_H_IN32X
  { 3, &RISCVDescs.OperandInfo[8027] },  // Inst #11838 = FCVT_D_H
  { 3, &RISCVDescs.OperandInfo[8024] },  // Inst #11837 = FCVT_BF16_S
  { 3, &RISCVDescs.OperandInfo[195] },  // Inst #11836 = FCVTMOD_W_D
  { 2, &RISCVDescs.OperandInfo[8022] },  // Inst #11835 = FCLASS_S_INX
  { 2, &RISCVDescs.OperandInfo[8020] },  // Inst #11834 = FCLASS_S
  { 2, &RISCVDescs.OperandInfo[8018] },  // Inst #11833 = FCLASS_H_INX
  { 2, &RISCVDescs.OperandInfo[8016] },  // Inst #11832 = FCLASS_H
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11831 = FCLASS_D_INX
  { 2, &RISCVDescs.OperandInfo[8014] },  // Inst #11830 = FCLASS_D_IN32X
  { 2, &RISCVDescs.OperandInfo[8012] },  // Inst #11829 = FCLASS_D
  { 4, &RISCVDescs.OperandInfo[224] },  // Inst #11828 = FADD_S_INX
  { 4, &RISCVDescs.OperandInfo[220] },  // Inst #11827 = FADD_S
  { 4, &RISCVDescs.OperandInfo[216] },  // Inst #11826 = FADD_H_INX
  { 4, &RISCVDescs.OperandInfo[212] },  // Inst #11825 = FADD_H
  { 4, &RISCVDescs.OperandInfo[148] },  // Inst #11824 = FADD_D_INX
  { 4, &RISCVDescs.OperandInfo[208] },  // Inst #11823 = FADD_D_IN32X
  { 4, &RISCVDescs.OperandInfo[204] },  // Inst #11822 = FADD_D
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11821 = ECALL
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11820 = EBREAK
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11819 = DRET
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11818 = DIVW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11817 = DIVUW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11816 = DIVU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11815 = DIV
  { 2, &RISCVDescs.OperandInfo[7991] },  // Inst #11814 = C_ZEXT_W
  { 2, &RISCVDescs.OperandInfo[7991] },  // Inst #11813 = C_ZEXT_H
  { 2, &RISCVDescs.OperandInfo[7991] },  // Inst #11812 = C_ZEXT_B
  { 3, &RISCVDescs.OperandInfo[7936] },  // Inst #11811 = C_XOR
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11810 = C_UNIMP
  { 3, &RISCVDescs.OperandInfo[8009] },  // Inst #11809 = C_SWSP
  { 3, &RISCVDescs.OperandInfo[7980] },  // Inst #11808 = C_SW
  { 3, &RISCVDescs.OperandInfo[7936] },  // Inst #11807 = C_SUBW
  { 3, &RISCVDescs.OperandInfo[7936] },  // Inst #11806 = C_SUB
  { 1, &RISCVDescs.OperandInfo[8008] },  // Inst #11805 = C_SSPUSH
  { 1, &RISCVDescs.OperandInfo[8007] },  // Inst #11804 = C_SSPOPCHK
  { 2, &RISCVDescs.OperandInfo[7991] },  // Inst #11803 = C_SRLI64_HINT
  { 3, &RISCVDescs.OperandInfo[8004] },  // Inst #11802 = C_SRLI
  { 2, &RISCVDescs.OperandInfo[7991] },  // Inst #11801 = C_SRAI64_HINT
  { 3, &RISCVDescs.OperandInfo[8004] },  // Inst #11800 = C_SRAI
  { 3, &RISCVDescs.OperandInfo[8001] },  // Inst #11799 = C_SLLI_HINT
  { 2, &RISCVDescs.OperandInfo[7999] },  // Inst #11798 = C_SLLI64_HINT
  { 3, &RISCVDescs.OperandInfo[7996] },  // Inst #11797 = C_SLLI
  { 3, &RISCVDescs.OperandInfo[7969] },  // Inst #11796 = C_SH
  { 2, &RISCVDescs.OperandInfo[7991] },  // Inst #11795 = C_SEXT_H
  { 2, &RISCVDescs.OperandInfo[7991] },  // Inst #11794 = C_SEXT_B
  { 3, &RISCVDescs.OperandInfo[7993] },  // Inst #11793 = C_SDSP
  { 3, &RISCVDescs.OperandInfo[7963] },  // Inst #11792 = C_SD
  { 3, &RISCVDescs.OperandInfo[7960] },  // Inst #11791 = C_SB
  { 3, &RISCVDescs.OperandInfo[7936] },  // Inst #11790 = C_OR
  { 2, &RISCVDescs.OperandInfo[7991] },  // Inst #11789 = C_NOT
  { 1, &RISCVDescs.OperandInfo[7990] },  // Inst #11788 = C_NOP_HINT
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11787 = C_NOP
  { 2, &RISCVDescs.OperandInfo[7988] },  // Inst #11786 = C_MV_HINT
  { 2, &RISCVDescs.OperandInfo[7986] },  // Inst #11785 = C_MV
  { 3, &RISCVDescs.OperandInfo[7936] },  // Inst #11784 = C_MUL
  { 3, &RISCVDescs.OperandInfo[7983] },  // Inst #11783 = C_LWSP
  { 3, &RISCVDescs.OperandInfo[7980] },  // Inst #11782 = C_LW
  { 2, &RISCVDescs.OperandInfo[7978] },  // Inst #11781 = C_LUI_HINT
  { 2, &RISCVDescs.OperandInfo[7976] },  // Inst #11780 = C_LUI
  { 2, &RISCVDescs.OperandInfo[7974] },  // Inst #11779 = C_LI_HINT
  { 2, &RISCVDescs.OperandInfo[7972] },  // Inst #11778 = C_LI
  { 3, &RISCVDescs.OperandInfo[7969] },  // Inst #11777 = C_LHU
  { 3, &RISCVDescs.OperandInfo[7969] },  // Inst #11776 = C_LH
  { 3, &RISCVDescs.OperandInfo[7966] },  // Inst #11775 = C_LDSP
  { 3, &RISCVDescs.OperandInfo[7963] },  // Inst #11774 = C_LD
  { 3, &RISCVDescs.OperandInfo[7960] },  // Inst #11773 = C_LBU
  { 1, &RISCVDescs.OperandInfo[7959] },  // Inst #11772 = C_JR
  { 1, &RISCVDescs.OperandInfo[7959] },  // Inst #11771 = C_JALR
  { 1, &RISCVDescs.OperandInfo[157] },  // Inst #11770 = C_JAL
  { 1, &RISCVDescs.OperandInfo[157] },  // Inst #11769 = C_J
  { 3, &RISCVDescs.OperandInfo[7956] },  // Inst #11768 = C_FSWSP
  { 3, &RISCVDescs.OperandInfo[7953] },  // Inst #11767 = C_FSW
  { 3, &RISCVDescs.OperandInfo[7950] },  // Inst #11766 = C_FSDSP
  { 3, &RISCVDescs.OperandInfo[7947] },  // Inst #11765 = C_FSD
  { 3, &RISCVDescs.OperandInfo[7956] },  // Inst #11764 = C_FLWSP
  { 3, &RISCVDescs.OperandInfo[7953] },  // Inst #11763 = C_FLW
  { 3, &RISCVDescs.OperandInfo[7950] },  // Inst #11762 = C_FLDSP
  { 3, &RISCVDescs.OperandInfo[7947] },  // Inst #11761 = C_FLD
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11760 = C_EBREAK
  { 2, &RISCVDescs.OperandInfo[7945] },  // Inst #11759 = C_BNEZ
  { 2, &RISCVDescs.OperandInfo[7945] },  // Inst #11758 = C_BEQZ
  { 3, &RISCVDescs.OperandInfo[7942] },  // Inst #11757 = C_ANDI
  { 3, &RISCVDescs.OperandInfo[7936] },  // Inst #11756 = C_AND
  { 3, &RISCVDescs.OperandInfo[7939] },  // Inst #11755 = C_ADD_HINT
  { 3, &RISCVDescs.OperandInfo[7936] },  // Inst #11754 = C_ADDW
  { 3, &RISCVDescs.OperandInfo[7933] },  // Inst #11753 = C_ADDI_NOP
  { 3, &RISCVDescs.OperandInfo[7930] },  // Inst #11752 = C_ADDI_HINT_IMM_ZERO
  { 3, &RISCVDescs.OperandInfo[7927] },  // Inst #11751 = C_ADDIW
  { 3, &RISCVDescs.OperandInfo[7924] },  // Inst #11750 = C_ADDI4SPN
  { 3, &RISCVDescs.OperandInfo[7921] },  // Inst #11749 = C_ADDI16SP
  { 3, &RISCVDescs.OperandInfo[7918] },  // Inst #11748 = C_ADDI
  { 3, &RISCVDescs.OperandInfo[7915] },  // Inst #11747 = C_ADD
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11746 = CZERO_NEZ
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11745 = CZERO_EQZ
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11744 = CV_XOR_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11743 = CV_XOR_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11742 = CV_XOR_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11741 = CV_XOR_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11740 = CV_XOR_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11739 = CV_XOR_B
  { 4, &RISCVDescs.OperandInfo[7901] },  // Inst #11738 = CV_SW_rr_inc
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #11737 = CV_SW_rr
  { 4, &RISCVDescs.OperandInfo[7897] },  // Inst #11736 = CV_SW_ri_inc
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11735 = CV_SUB_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11734 = CV_SUB_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11733 = CV_SUB_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11732 = CV_SUB_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11731 = CV_SUB_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11730 = CV_SUB_DIV8
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11729 = CV_SUB_DIV4
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11728 = CV_SUB_DIV2
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11727 = CV_SUB_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11726 = CV_SUBURNR
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11725 = CV_SUBURN
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11724 = CV_SUBUNR
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11723 = CV_SUBUN
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11722 = CV_SUBROTMJ_DIV8
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11721 = CV_SUBROTMJ_DIV4
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11720 = CV_SUBROTMJ_DIV2
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11719 = CV_SUBROTMJ
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11718 = CV_SUBRNR
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11717 = CV_SUBRN
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11716 = CV_SUBNR
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11715 = CV_SUBN
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11714 = CV_SRL_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11713 = CV_SRL_SC_B
  { 3, &RISCVDescs.OperandInfo[7912] },  // Inst #11712 = CV_SRL_SCI_H
  { 3, &RISCVDescs.OperandInfo[7909] },  // Inst #11711 = CV_SRL_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11710 = CV_SRL_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11709 = CV_SRL_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11708 = CV_SRA_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11707 = CV_SRA_SC_B
  { 3, &RISCVDescs.OperandInfo[7912] },  // Inst #11706 = CV_SRA_SCI_H
  { 3, &RISCVDescs.OperandInfo[7909] },  // Inst #11705 = CV_SRA_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11704 = CV_SRA_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11703 = CV_SRA_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11702 = CV_SLL_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11701 = CV_SLL_SC_B
  { 3, &RISCVDescs.OperandInfo[7912] },  // Inst #11700 = CV_SLL_SCI_H
  { 3, &RISCVDescs.OperandInfo[7909] },  // Inst #11699 = CV_SLL_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11698 = CV_SLL_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11697 = CV_SLL_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11696 = CV_SLETU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11695 = CV_SLET
  { 4, &RISCVDescs.OperandInfo[7901] },  // Inst #11694 = CV_SH_rr_inc
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #11693 = CV_SH_rr
  { 4, &RISCVDescs.OperandInfo[7897] },  // Inst #11692 = CV_SH_ri_inc
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11691 = CV_SHUFFLE_SCI_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11690 = CV_SHUFFLE_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11689 = CV_SHUFFLE_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11688 = CV_SHUFFLEI3_SCI_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11687 = CV_SHUFFLEI2_SCI_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11686 = CV_SHUFFLEI1_SCI_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11685 = CV_SHUFFLEI0_SCI_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11684 = CV_SHUFFLE2_H
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11683 = CV_SHUFFLE2_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11682 = CV_SDOTUSP_SC_H
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11681 = CV_SDOTUSP_SC_B
  { 4, &RISCVDescs.OperandInfo[7905] },  // Inst #11680 = CV_SDOTUSP_SCI_H
  { 4, &RISCVDescs.OperandInfo[7905] },  // Inst #11679 = CV_SDOTUSP_SCI_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11678 = CV_SDOTUSP_H
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11677 = CV_SDOTUSP_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11676 = CV_SDOTUP_SC_H
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11675 = CV_SDOTUP_SC_B
  { 4, &RISCVDescs.OperandInfo[7880] },  // Inst #11674 = CV_SDOTUP_SCI_H
  { 4, &RISCVDescs.OperandInfo[7880] },  // Inst #11673 = CV_SDOTUP_SCI_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11672 = CV_SDOTUP_H
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11671 = CV_SDOTUP_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11670 = CV_SDOTSP_SC_H
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11669 = CV_SDOTSP_SC_B
  { 4, &RISCVDescs.OperandInfo[7905] },  // Inst #11668 = CV_SDOTSP_SCI_H
  { 4, &RISCVDescs.OperandInfo[7905] },  // Inst #11667 = CV_SDOTSP_SCI_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11666 = CV_SDOTSP_H
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11665 = CV_SDOTSP_B
  { 4, &RISCVDescs.OperandInfo[7901] },  // Inst #11664 = CV_SB_rr_inc
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #11663 = CV_SB_rr
  { 4, &RISCVDescs.OperandInfo[7897] },  // Inst #11662 = CV_SB_ri_inc
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11661 = CV_ROR
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11660 = CV_PACK_H
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11659 = CV_PACKLO_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11658 = CV_PACKHI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11657 = CV_PACK
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11656 = CV_OR_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11655 = CV_OR_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11654 = CV_OR_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11653 = CV_OR_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11652 = CV_OR_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11651 = CV_OR_B
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11650 = CV_MULURN
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11649 = CV_MULUN
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11648 = CV_MULSRN
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11647 = CV_MULSN
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11646 = CV_MULHHURN
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11645 = CV_MULHHUN
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11644 = CV_MULHHSRN
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11643 = CV_MULHHSN
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11642 = CV_MSU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11641 = CV_MIN_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11640 = CV_MIN_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11639 = CV_MIN_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11638 = CV_MIN_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11637 = CV_MIN_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11636 = CV_MIN_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11635 = CV_MINU_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11634 = CV_MINU_SC_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11633 = CV_MINU_SCI_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11632 = CV_MINU_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11631 = CV_MINU_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11630 = CV_MINU_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11629 = CV_MINU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11628 = CV_MIN
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11627 = CV_MAX_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11626 = CV_MAX_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11625 = CV_MAX_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11624 = CV_MAX_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11623 = CV_MAX_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11622 = CV_MAX_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11621 = CV_MAXU_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11620 = CV_MAXU_SC_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11619 = CV_MAXU_SCI_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11618 = CV_MAXU_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11617 = CV_MAXU_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11616 = CV_MAXU_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11615 = CV_MAXU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11614 = CV_MAX
  { 5, &RISCVDescs.OperandInfo[7892] },  // Inst #11613 = CV_MACURN
  { 5, &RISCVDescs.OperandInfo[7892] },  // Inst #11612 = CV_MACUN
  { 5, &RISCVDescs.OperandInfo[7892] },  // Inst #11611 = CV_MACSRN
  { 5, &RISCVDescs.OperandInfo[7892] },  // Inst #11610 = CV_MACSN
  { 5, &RISCVDescs.OperandInfo[7892] },  // Inst #11609 = CV_MACHHURN
  { 5, &RISCVDescs.OperandInfo[7892] },  // Inst #11608 = CV_MACHHUN
  { 5, &RISCVDescs.OperandInfo[7892] },  // Inst #11607 = CV_MACHHSRN
  { 5, &RISCVDescs.OperandInfo[7892] },  // Inst #11606 = CV_MACHHSN
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11605 = CV_MAC
  { 4, &RISCVDescs.OperandInfo[7888] },  // Inst #11604 = CV_LW_rr_inc
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #11603 = CV_LW_rr
  { 4, &RISCVDescs.OperandInfo[7884] },  // Inst #11602 = CV_LW_ri_inc
  { 4, &RISCVDescs.OperandInfo[7888] },  // Inst #11601 = CV_LH_rr_inc
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #11600 = CV_LH_rr
  { 4, &RISCVDescs.OperandInfo[7884] },  // Inst #11599 = CV_LH_ri_inc
  { 4, &RISCVDescs.OperandInfo[7888] },  // Inst #11598 = CV_LHU_rr_inc
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #11597 = CV_LHU_rr
  { 4, &RISCVDescs.OperandInfo[7884] },  // Inst #11596 = CV_LHU_ri_inc
  { 4, &RISCVDescs.OperandInfo[7888] },  // Inst #11595 = CV_LB_rr_inc
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #11594 = CV_LB_rr
  { 4, &RISCVDescs.OperandInfo[7884] },  // Inst #11593 = CV_LB_ri_inc
  { 4, &RISCVDescs.OperandInfo[7888] },  // Inst #11592 = CV_LBU_rr_inc
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #11591 = CV_LBU_rr
  { 4, &RISCVDescs.OperandInfo[7884] },  // Inst #11590 = CV_LBU_ri_inc
  { 4, &RISCVDescs.OperandInfo[7880] },  // Inst #11589 = CV_INSERT_H
  { 4, &RISCVDescs.OperandInfo[7880] },  // Inst #11588 = CV_INSERT_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11587 = CV_INSERTR
  { 5, &RISCVDescs.OperandInfo[7875] },  // Inst #11586 = CV_INSERT
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11585 = CV_FL1
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11584 = CV_FF1
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11583 = CV_EXTRACT_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11582 = CV_EXTRACT_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11581 = CV_EXTRACTU_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11580 = CV_EXTRACTU_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11579 = CV_EXTRACTUR
  { 4, &RISCVDescs.OperandInfo[7858] },  // Inst #11578 = CV_EXTRACTU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11577 = CV_EXTRACTR
  { 4, &RISCVDescs.OperandInfo[7858] },  // Inst #11576 = CV_EXTRACT
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11575 = CV_EXTHZ
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11574 = CV_EXTHS
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11573 = CV_EXTBZ
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11572 = CV_EXTBS
  { 3, &RISCVDescs.OperandInfo[7872] },  // Inst #11571 = CV_ELW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11570 = CV_DOTUSP_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11569 = CV_DOTUSP_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11568 = CV_DOTUSP_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11567 = CV_DOTUSP_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11566 = CV_DOTUSP_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11565 = CV_DOTUSP_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11564 = CV_DOTUP_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11563 = CV_DOTUP_SC_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11562 = CV_DOTUP_SCI_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11561 = CV_DOTUP_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11560 = CV_DOTUP_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11559 = CV_DOTUP_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11558 = CV_DOTSP_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11557 = CV_DOTSP_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11556 = CV_DOTSP_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11555 = CV_DOTSP_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11554 = CV_DOTSP_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11553 = CV_DOTSP_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11552 = CV_CPLXMUL_R_DIV8
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11551 = CV_CPLXMUL_R_DIV4
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11550 = CV_CPLXMUL_R_DIV2
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11549 = CV_CPLXMUL_R
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11548 = CV_CPLXMUL_I_DIV8
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11547 = CV_CPLXMUL_I_DIV4
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11546 = CV_CPLXMUL_I_DIV2
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11545 = CV_CPLXMUL_I
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11544 = CV_CPLXCONJ
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11543 = CV_CNT
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11542 = CV_CMPNE_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11541 = CV_CMPNE_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11540 = CV_CMPNE_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11539 = CV_CMPNE_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11538 = CV_CMPNE_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11537 = CV_CMPNE_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11536 = CV_CMPLT_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11535 = CV_CMPLT_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11534 = CV_CMPLT_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11533 = CV_CMPLT_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11532 = CV_CMPLT_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11531 = CV_CMPLT_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11530 = CV_CMPLTU_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11529 = CV_CMPLTU_SC_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11528 = CV_CMPLTU_SCI_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11527 = CV_CMPLTU_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11526 = CV_CMPLTU_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11525 = CV_CMPLTU_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11524 = CV_CMPLE_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11523 = CV_CMPLE_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11522 = CV_CMPLE_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11521 = CV_CMPLE_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11520 = CV_CMPLE_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11519 = CV_CMPLE_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11518 = CV_CMPLEU_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11517 = CV_CMPLEU_SC_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11516 = CV_CMPLEU_SCI_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11515 = CV_CMPLEU_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11514 = CV_CMPLEU_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11513 = CV_CMPLEU_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11512 = CV_CMPGT_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11511 = CV_CMPGT_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11510 = CV_CMPGT_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11509 = CV_CMPGT_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11508 = CV_CMPGT_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11507 = CV_CMPGT_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11506 = CV_CMPGTU_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11505 = CV_CMPGTU_SC_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11504 = CV_CMPGTU_SCI_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11503 = CV_CMPGTU_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11502 = CV_CMPGTU_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11501 = CV_CMPGTU_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11500 = CV_CMPGE_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11499 = CV_CMPGE_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11498 = CV_CMPGE_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11497 = CV_CMPGE_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11496 = CV_CMPGE_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11495 = CV_CMPGE_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11494 = CV_CMPGEU_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11493 = CV_CMPGEU_SC_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11492 = CV_CMPGEU_SCI_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11491 = CV_CMPGEU_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11490 = CV_CMPGEU_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11489 = CV_CMPGEU_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11488 = CV_CMPEQ_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11487 = CV_CMPEQ_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11486 = CV_CMPEQ_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11485 = CV_CMPEQ_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11484 = CV_CMPEQ_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11483 = CV_CMPEQ_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11482 = CV_CLIPUR
  { 3, &RISCVDescs.OperandInfo[7869] },  // Inst #11481 = CV_CLIPU
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11480 = CV_CLIPR
  { 3, &RISCVDescs.OperandInfo[7869] },  // Inst #11479 = CV_CLIP
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11478 = CV_CLB
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11477 = CV_BSETR
  { 4, &RISCVDescs.OperandInfo[7858] },  // Inst #11476 = CV_BSET
  { 3, &RISCVDescs.OperandInfo[7862] },  // Inst #11475 = CV_BNEIMM
  { 4, &RISCVDescs.OperandInfo[7865] },  // Inst #11474 = CV_BITREV
  { 3, &RISCVDescs.OperandInfo[7862] },  // Inst #11473 = CV_BEQIMM
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11472 = CV_BCLRR
  { 4, &RISCVDescs.OperandInfo[7858] },  // Inst #11471 = CV_BCLR
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11470 = CV_AVG_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11469 = CV_AVG_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11468 = CV_AVG_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11467 = CV_AVG_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11466 = CV_AVG_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11465 = CV_AVG_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11464 = CV_AVGU_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11463 = CV_AVGU_SC_B
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11462 = CV_AVGU_SCI_H
  { 3, &RISCVDescs.OperandInfo[7855] },  // Inst #11461 = CV_AVGU_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11460 = CV_AVGU_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11459 = CV_AVGU_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11458 = CV_AND_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11457 = CV_AND_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11456 = CV_AND_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11455 = CV_AND_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11454 = CV_AND_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11453 = CV_AND_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11452 = CV_ADD_SC_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11451 = CV_ADD_SC_B
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11450 = CV_ADD_SCI_H
  { 3, &RISCVDescs.OperandInfo[7852] },  // Inst #11449 = CV_ADD_SCI_B
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11448 = CV_ADD_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11447 = CV_ADD_DIV8
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11446 = CV_ADD_DIV4
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11445 = CV_ADD_DIV2
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11444 = CV_ADD_B
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11443 = CV_ADDURNR
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11442 = CV_ADDURN
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11441 = CV_ADDUNR
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11440 = CV_ADDUN
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11439 = CV_ADDRNR
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11438 = CV_ADDRN
  { 4, &RISCVDescs.OperandInfo[7848] },  // Inst #11437 = CV_ADDNR
  { 4, &RISCVDescs.OperandInfo[7844] },  // Inst #11436 = CV_ADDN
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11435 = CV_ABS_H
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11434 = CV_ABS_B
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11433 = CV_ABS
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11432 = CTZW
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11431 = CTZ
  { 3, &RISCVDescs.OperandInfo[7841] },  // Inst #11430 = CSRRWI
  { 3, &RISCVDescs.OperandInfo[7838] },  // Inst #11429 = CSRRW
  { 3, &RISCVDescs.OperandInfo[7841] },  // Inst #11428 = CSRRSI
  { 3, &RISCVDescs.OperandInfo[7838] },  // Inst #11427 = CSRRS
  { 3, &RISCVDescs.OperandInfo[7841] },  // Inst #11426 = CSRRCI
  { 3, &RISCVDescs.OperandInfo[7838] },  // Inst #11425 = CSRRC
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11424 = CPOPW
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11423 = CPOP
  { 2, &RISCVDescs.OperandInfo[13] },  // Inst #11422 = CM_PUSH
  { 2, &RISCVDescs.OperandInfo[13] },  // Inst #11421 = CM_POPRETZ
  { 2, &RISCVDescs.OperandInfo[13] },  // Inst #11420 = CM_POPRET
  { 2, &RISCVDescs.OperandInfo[13] },  // Inst #11419 = CM_POP
  { 2, &RISCVDescs.OperandInfo[7836] },  // Inst #11418 = CM_MVSA01
  { 2, &RISCVDescs.OperandInfo[7836] },  // Inst #11417 = CM_MVA01S
  { 1, &RISCVDescs.OperandInfo[7807] },  // Inst #11416 = CM_JT
  { 1, &RISCVDescs.OperandInfo[7835] },  // Inst #11415 = CM_JALT
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11414 = CMOP9
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11413 = CMOP7
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11412 = CMOP5
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11411 = CMOP3
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11410 = CMOP15
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11409 = CMOP13
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11408 = CMOP11
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #11407 = CMOP1
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11406 = CLZW
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11405 = CLZ
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11404 = CLMULR
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11403 = CLMULH
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11402 = CLMUL
  { 1, &RISCVDescs.OperandInfo[7834] },  // Inst #11401 = CBO_ZERO
  { 1, &RISCVDescs.OperandInfo[7834] },  // Inst #11400 = CBO_INVAL
  { 1, &RISCVDescs.OperandInfo[7834] },  // Inst #11399 = CBO_FLUSH
  { 1, &RISCVDescs.OperandInfo[7834] },  // Inst #11398 = CBO_CLEAN
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #11397 = BSETI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11396 = BSET
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11395 = BREV8
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #11394 = BNE
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #11393 = BLTU
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #11392 = BLT
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #11391 = BINVI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11390 = BINV
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #11389 = BGEU
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #11388 = BGE
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #11387 = BEXTI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11386 = BEXT
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #11385 = BEQ
  { 3, &RISCVDescs.OperandInfo[7831] },  // Inst #11384 = BCLRI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11383 = BCLR
  { 2, &RISCVDescs.OperandInfo[7829] },  // Inst #11382 = AUIPC
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11381 = ANDN
  { 3, &RISCVDescs.OperandInfo[7808] },  // Inst #11380 = ANDI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11379 = AND
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11378 = AMOXOR_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11377 = AMOXOR_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11376 = AMOXOR_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11375 = AMOXOR_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11374 = AMOXOR_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11373 = AMOXOR_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11372 = AMOXOR_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11371 = AMOXOR_D
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11370 = AMOSWAP_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11369 = AMOSWAP_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11368 = AMOSWAP_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11367 = AMOSWAP_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11366 = AMOSWAP_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11365 = AMOSWAP_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11364 = AMOSWAP_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11363 = AMOSWAP_D
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11362 = AMOOR_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11361 = AMOOR_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11360 = AMOOR_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11359 = AMOOR_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11358 = AMOOR_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11357 = AMOOR_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11356 = AMOOR_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11355 = AMOOR_D
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11354 = AMOMIN_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11353 = AMOMIN_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11352 = AMOMIN_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11351 = AMOMIN_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11350 = AMOMIN_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11349 = AMOMIN_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11348 = AMOMIN_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11347 = AMOMIN_D
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11346 = AMOMINU_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11345 = AMOMINU_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11344 = AMOMINU_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11343 = AMOMINU_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11342 = AMOMINU_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11341 = AMOMINU_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11340 = AMOMINU_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11339 = AMOMINU_D
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11338 = AMOMAX_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11337 = AMOMAX_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11336 = AMOMAX_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11335 = AMOMAX_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11334 = AMOMAX_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11333 = AMOMAX_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11332 = AMOMAX_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11331 = AMOMAX_D
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11330 = AMOMAXU_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11329 = AMOMAXU_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11328 = AMOMAXU_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11327 = AMOMAXU_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11326 = AMOMAXU_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11325 = AMOMAXU_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11324 = AMOMAXU_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11323 = AMOMAXU_D
  { 4, &RISCVDescs.OperandInfo[7825] },  // Inst #11322 = AMOCAS_W_RL
  { 4, &RISCVDescs.OperandInfo[7825] },  // Inst #11321 = AMOCAS_W_AQ_RL
  { 4, &RISCVDescs.OperandInfo[7825] },  // Inst #11320 = AMOCAS_W_AQ
  { 4, &RISCVDescs.OperandInfo[7825] },  // Inst #11319 = AMOCAS_W
  { 4, &RISCVDescs.OperandInfo[7821] },  // Inst #11318 = AMOCAS_Q_RL
  { 4, &RISCVDescs.OperandInfo[7821] },  // Inst #11317 = AMOCAS_Q_AQ_RL
  { 4, &RISCVDescs.OperandInfo[7821] },  // Inst #11316 = AMOCAS_Q_AQ
  { 4, &RISCVDescs.OperandInfo[7821] },  // Inst #11315 = AMOCAS_Q
  { 4, &RISCVDescs.OperandInfo[7825] },  // Inst #11314 = AMOCAS_D_RV64_RL
  { 4, &RISCVDescs.OperandInfo[7825] },  // Inst #11313 = AMOCAS_D_RV64_AQ_RL
  { 4, &RISCVDescs.OperandInfo[7825] },  // Inst #11312 = AMOCAS_D_RV64_AQ
  { 4, &RISCVDescs.OperandInfo[7825] },  // Inst #11311 = AMOCAS_D_RV64
  { 4, &RISCVDescs.OperandInfo[7821] },  // Inst #11310 = AMOCAS_D_RV32_RL
  { 4, &RISCVDescs.OperandInfo[7821] },  // Inst #11309 = AMOCAS_D_RV32_AQ_RL
  { 4, &RISCVDescs.OperandInfo[7821] },  // Inst #11308 = AMOCAS_D_RV32_AQ
  { 4, &RISCVDescs.OperandInfo[7821] },  // Inst #11307 = AMOCAS_D_RV32
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11306 = AMOAND_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11305 = AMOAND_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11304 = AMOAND_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11303 = AMOAND_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11302 = AMOAND_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11301 = AMOAND_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11300 = AMOAND_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11299 = AMOAND_D
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11298 = AMOADD_W_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11297 = AMOADD_W_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11296 = AMOADD_W_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11295 = AMOADD_W
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11294 = AMOADD_D_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11293 = AMOADD_D_AQ_RL
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11292 = AMOADD_D_AQ
  { 3, &RISCVDescs.OperandInfo[7818] },  // Inst #11291 = AMOADD_D
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11290 = AES64KS2
  { 3, &RISCVDescs.OperandInfo[7815] },  // Inst #11289 = AES64KS1I
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11288 = AES64IM
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11287 = AES64ESM
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11286 = AES64ES
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11285 = AES64DSM
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11284 = AES64DS
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #11283 = AES32ESMI
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #11282 = AES32ESI
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #11281 = AES32DSMI
  { 4, &RISCVDescs.OperandInfo[7811] },  // Inst #11280 = AES32DSI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11279 = ADD_UW
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11278 = ADDW
  { 3, &RISCVDescs.OperandInfo[7808] },  // Inst #11277 = ADDIW
  { 3, &RISCVDescs.OperandInfo[7808] },  // Inst #11276 = ADDI
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #11275 = ADD
  { 1, &RISCVDescs.OperandInfo[7807] },  // Inst #11274 = WriteVXRMImm
  { 1, &RISCVDescs.OperandInfo[7807] },  // Inst #11273 = WriteFRMImm
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #11272 = WriteFRM
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #11271 = WriteFFLAGS
  { 2, &RISCVDescs.OperandInfo[7805] },  // Inst #11270 = SwapFRMImm
  { 3, &RISCVDescs.OperandInfo[7802] },  // Inst #11269 = SplitF64Pseudo_INX
  { 3, &RISCVDescs.OperandInfo[7799] },  // Inst #11268 = SplitF64Pseudo
  { 6, &RISCVDescs.OperandInfo[7787] },  // Inst #11267 = Select_GPR_Using_CC_GPR
  { 6, &RISCVDescs.OperandInfo[7793] },  // Inst #11266 = Select_FPR64_Using_CC_GPR
  { 6, &RISCVDescs.OperandInfo[7787] },  // Inst #11265 = Select_FPR64INX_Using_CC_GPR
  { 6, &RISCVDescs.OperandInfo[7781] },  // Inst #11264 = Select_FPR64IN32X_Using_CC_GPR
  { 6, &RISCVDescs.OperandInfo[7775] },  // Inst #11263 = Select_FPR32_Using_CC_GPR
  { 6, &RISCVDescs.OperandInfo[7769] },  // Inst #11262 = Select_FPR32INX_Using_CC_GPR
  { 6, &RISCVDescs.OperandInfo[7763] },  // Inst #11261 = Select_FPR16_Using_CC_GPR
  { 6, &RISCVDescs.OperandInfo[7757] },  // Inst #11260 = Select_FPR16INX_Using_CC_GPR
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #11259 = ReadFRM
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #11258 = ReadFFLAGS
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11257 = ReadCycleWide
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11256 = PseudoZEXT_W
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #11255 = PseudoZEXT_H
  { 7, &RISCVDescs.OperandInfo[3787] },  // Inst #11254 = PseudoVZEXT_VF8_M8_MASK
  { 6, &RISCVDescs.OperandInfo[3781] },  // Inst #11253 = PseudoVZEXT_VF8_M8
  { 7, &RISCVDescs.OperandInfo[3774] },  // Inst #11252 = PseudoVZEXT_VF8_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3768] },  // Inst #11251 = PseudoVZEXT_VF8_M4
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #11250 = PseudoVZEXT_VF8_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #11249 = PseudoVZEXT_VF8_M2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11248 = PseudoVZEXT_VF8_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11247 = PseudoVZEXT_VF8_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11246 = PseudoVZEXT_VF4_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11245 = PseudoVZEXT_VF4_MF2
  { 7, &RISCVDescs.OperandInfo[6571] },  // Inst #11244 = PseudoVZEXT_VF4_M8_MASK
  { 6, &RISCVDescs.OperandInfo[6565] },  // Inst #11243 = PseudoVZEXT_VF4_M8
  { 7, &RISCVDescs.OperandInfo[3774] },  // Inst #11242 = PseudoVZEXT_VF4_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3768] },  // Inst #11241 = PseudoVZEXT_VF4_M4
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #11240 = PseudoVZEXT_VF4_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #11239 = PseudoVZEXT_VF4_M2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11238 = PseudoVZEXT_VF4_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11237 = PseudoVZEXT_VF4_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11236 = PseudoVZEXT_VF2_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11235 = PseudoVZEXT_VF2_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11234 = PseudoVZEXT_VF2_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11233 = PseudoVZEXT_VF2_MF2
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #11232 = PseudoVZEXT_VF2_M8_MASK
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #11231 = PseudoVZEXT_VF2_M8
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #11230 = PseudoVZEXT_VF2_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #11229 = PseudoVZEXT_VF2_M4
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #11228 = PseudoVZEXT_VF2_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #11227 = PseudoVZEXT_VF2_M2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11226 = PseudoVZEXT_VF2_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11225 = PseudoVZEXT_VF2_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11224 = PseudoVXOR_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11223 = PseudoVXOR_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11222 = PseudoVXOR_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11221 = PseudoVXOR_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11220 = PseudoVXOR_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11219 = PseudoVXOR_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #11218 = PseudoVXOR_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #11217 = PseudoVXOR_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #11216 = PseudoVXOR_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #11215 = PseudoVXOR_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #11214 = PseudoVXOR_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #11213 = PseudoVXOR_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11212 = PseudoVXOR_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11211 = PseudoVXOR_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11210 = PseudoVXOR_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11209 = PseudoVXOR_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11208 = PseudoVXOR_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11207 = PseudoVXOR_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11206 = PseudoVXOR_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11205 = PseudoVXOR_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #11204 = PseudoVXOR_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #11203 = PseudoVXOR_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #11202 = PseudoVXOR_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #11201 = PseudoVXOR_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #11200 = PseudoVXOR_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #11199 = PseudoVXOR_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11198 = PseudoVXOR_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11197 = PseudoVXOR_VV_M1
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #11196 = PseudoVXOR_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #11195 = PseudoVXOR_VI_MF8
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #11194 = PseudoVXOR_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #11193 = PseudoVXOR_VI_MF4
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #11192 = PseudoVXOR_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #11191 = PseudoVXOR_VI_MF2
  { 8, &RISCVDescs.OperandInfo[689] },  // Inst #11190 = PseudoVXOR_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[682] },  // Inst #11189 = PseudoVXOR_VI_M8
  { 8, &RISCVDescs.OperandInfo[674] },  // Inst #11188 = PseudoVXOR_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[667] },  // Inst #11187 = PseudoVXOR_VI_M4
  { 8, &RISCVDescs.OperandInfo[659] },  // Inst #11186 = PseudoVXOR_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[652] },  // Inst #11185 = PseudoVXOR_VI_M2
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #11184 = PseudoVXOR_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #11183 = PseudoVXOR_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11182 = PseudoVWSUB_WX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11181 = PseudoVWSUB_WX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11180 = PseudoVWSUB_WX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11179 = PseudoVWSUB_WX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11178 = PseudoVWSUB_WX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11177 = PseudoVWSUB_WX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #11176 = PseudoVWSUB_WX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #11175 = PseudoVWSUB_WX_M4
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #11174 = PseudoVWSUB_WX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #11173 = PseudoVWSUB_WX_M2
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #11172 = PseudoVWSUB_WX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #11171 = PseudoVWSUB_WX_M1
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11170 = PseudoVWSUB_WV_MF8_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11169 = PseudoVWSUB_WV_MF8_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11168 = PseudoVWSUB_WV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11167 = PseudoVWSUB_WV_MF8
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11166 = PseudoVWSUB_WV_MF4_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11165 = PseudoVWSUB_WV_MF4_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11164 = PseudoVWSUB_WV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11163 = PseudoVWSUB_WV_MF4
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11162 = PseudoVWSUB_WV_MF2_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11161 = PseudoVWSUB_WV_MF2_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11160 = PseudoVWSUB_WV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11159 = PseudoVWSUB_WV_MF2
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #11158 = PseudoVWSUB_WV_M4_TIED
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #11157 = PseudoVWSUB_WV_M4_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6332] },  // Inst #11156 = PseudoVWSUB_WV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6325] },  // Inst #11155 = PseudoVWSUB_WV_M4
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #11154 = PseudoVWSUB_WV_M2_TIED
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #11153 = PseudoVWSUB_WV_M2_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6287] },  // Inst #11152 = PseudoVWSUB_WV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6280] },  // Inst #11151 = PseudoVWSUB_WV_M2
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #11150 = PseudoVWSUB_WV_M1_TIED
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #11149 = PseudoVWSUB_WV_M1_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #11148 = PseudoVWSUB_WV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #11147 = PseudoVWSUB_WV_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #11146 = PseudoVWSUB_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #11145 = PseudoVWSUB_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #11144 = PseudoVWSUB_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #11143 = PseudoVWSUB_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #11142 = PseudoVWSUB_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #11141 = PseudoVWSUB_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7659] },  // Inst #11140 = PseudoVWSUB_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7652] },  // Inst #11139 = PseudoVWSUB_VX_M4
  { 8, &RISCVDescs.OperandInfo[7644] },  // Inst #11138 = PseudoVWSUB_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7637] },  // Inst #11137 = PseudoVWSUB_VX_M2
  { 8, &RISCVDescs.OperandInfo[7629] },  // Inst #11136 = PseudoVWSUB_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7622] },  // Inst #11135 = PseudoVWSUB_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11134 = PseudoVWSUB_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11133 = PseudoVWSUB_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11132 = PseudoVWSUB_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11131 = PseudoVWSUB_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11130 = PseudoVWSUB_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11129 = PseudoVWSUB_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #11128 = PseudoVWSUB_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #11127 = PseudoVWSUB_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #11126 = PseudoVWSUB_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #11125 = PseudoVWSUB_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #11124 = PseudoVWSUB_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #11123 = PseudoVWSUB_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11122 = PseudoVWSUBU_WX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11121 = PseudoVWSUBU_WX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11120 = PseudoVWSUBU_WX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11119 = PseudoVWSUBU_WX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #11118 = PseudoVWSUBU_WX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #11117 = PseudoVWSUBU_WX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #11116 = PseudoVWSUBU_WX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #11115 = PseudoVWSUBU_WX_M4
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #11114 = PseudoVWSUBU_WX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #11113 = PseudoVWSUBU_WX_M2
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #11112 = PseudoVWSUBU_WX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #11111 = PseudoVWSUBU_WX_M1
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11110 = PseudoVWSUBU_WV_MF8_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11109 = PseudoVWSUBU_WV_MF8_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11108 = PseudoVWSUBU_WV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11107 = PseudoVWSUBU_WV_MF8
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11106 = PseudoVWSUBU_WV_MF4_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11105 = PseudoVWSUBU_WV_MF4_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11104 = PseudoVWSUBU_WV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11103 = PseudoVWSUBU_WV_MF4
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #11102 = PseudoVWSUBU_WV_MF2_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #11101 = PseudoVWSUBU_WV_MF2_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11100 = PseudoVWSUBU_WV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11099 = PseudoVWSUBU_WV_MF2
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #11098 = PseudoVWSUBU_WV_M4_TIED
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #11097 = PseudoVWSUBU_WV_M4_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6332] },  // Inst #11096 = PseudoVWSUBU_WV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6325] },  // Inst #11095 = PseudoVWSUBU_WV_M4
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #11094 = PseudoVWSUBU_WV_M2_TIED
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #11093 = PseudoVWSUBU_WV_M2_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6287] },  // Inst #11092 = PseudoVWSUBU_WV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6280] },  // Inst #11091 = PseudoVWSUBU_WV_M2
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #11090 = PseudoVWSUBU_WV_M1_TIED
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #11089 = PseudoVWSUBU_WV_M1_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #11088 = PseudoVWSUBU_WV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #11087 = PseudoVWSUBU_WV_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #11086 = PseudoVWSUBU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #11085 = PseudoVWSUBU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #11084 = PseudoVWSUBU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #11083 = PseudoVWSUBU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #11082 = PseudoVWSUBU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #11081 = PseudoVWSUBU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7659] },  // Inst #11080 = PseudoVWSUBU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7652] },  // Inst #11079 = PseudoVWSUBU_VX_M4
  { 8, &RISCVDescs.OperandInfo[7644] },  // Inst #11078 = PseudoVWSUBU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7637] },  // Inst #11077 = PseudoVWSUBU_VX_M2
  { 8, &RISCVDescs.OperandInfo[7629] },  // Inst #11076 = PseudoVWSUBU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7622] },  // Inst #11075 = PseudoVWSUBU_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11074 = PseudoVWSUBU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11073 = PseudoVWSUBU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11072 = PseudoVWSUBU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11071 = PseudoVWSUBU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11070 = PseudoVWSUBU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11069 = PseudoVWSUBU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #11068 = PseudoVWSUBU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #11067 = PseudoVWSUBU_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #11066 = PseudoVWSUBU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #11065 = PseudoVWSUBU_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #11064 = PseudoVWSUBU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #11063 = PseudoVWSUBU_VV_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #11062 = PseudoVWSLL_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #11061 = PseudoVWSLL_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #11060 = PseudoVWSLL_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #11059 = PseudoVWSLL_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #11058 = PseudoVWSLL_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #11057 = PseudoVWSLL_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7659] },  // Inst #11056 = PseudoVWSLL_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7652] },  // Inst #11055 = PseudoVWSLL_VX_M4
  { 8, &RISCVDescs.OperandInfo[7644] },  // Inst #11054 = PseudoVWSLL_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7637] },  // Inst #11053 = PseudoVWSLL_VX_M2
  { 8, &RISCVDescs.OperandInfo[7629] },  // Inst #11052 = PseudoVWSLL_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7622] },  // Inst #11051 = PseudoVWSLL_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11050 = PseudoVWSLL_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11049 = PseudoVWSLL_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11048 = PseudoVWSLL_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11047 = PseudoVWSLL_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #11046 = PseudoVWSLL_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #11045 = PseudoVWSLL_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #11044 = PseudoVWSLL_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #11043 = PseudoVWSLL_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #11042 = PseudoVWSLL_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #11041 = PseudoVWSLL_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #11040 = PseudoVWSLL_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #11039 = PseudoVWSLL_VV_M1
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #11038 = PseudoVWSLL_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #11037 = PseudoVWSLL_VI_MF8
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #11036 = PseudoVWSLL_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #11035 = PseudoVWSLL_VI_MF4
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #11034 = PseudoVWSLL_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #11033 = PseudoVWSLL_VI_MF2
  { 8, &RISCVDescs.OperandInfo[7749] },  // Inst #11032 = PseudoVWSLL_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7742] },  // Inst #11031 = PseudoVWSLL_VI_M4
  { 8, &RISCVDescs.OperandInfo[7734] },  // Inst #11030 = PseudoVWSLL_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7727] },  // Inst #11029 = PseudoVWSLL_VI_M2
  { 8, &RISCVDescs.OperandInfo[7719] },  // Inst #11028 = PseudoVWSLL_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7712] },  // Inst #11027 = PseudoVWSLL_VI_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11026 = PseudoVWREDSUM_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11025 = PseudoVWREDSUM_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11024 = PseudoVWREDSUM_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11023 = PseudoVWREDSUM_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11022 = PseudoVWREDSUM_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11021 = PseudoVWREDSUM_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11020 = PseudoVWREDSUM_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11019 = PseudoVWREDSUM_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11018 = PseudoVWREDSUM_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11017 = PseudoVWREDSUM_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #11016 = PseudoVWREDSUM_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #11015 = PseudoVWREDSUM_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #11014 = PseudoVWREDSUM_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #11013 = PseudoVWREDSUM_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #11012 = PseudoVWREDSUM_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #11011 = PseudoVWREDSUM_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #11010 = PseudoVWREDSUM_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #11009 = PseudoVWREDSUM_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #11008 = PseudoVWREDSUM_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #11007 = PseudoVWREDSUM_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #11006 = PseudoVWREDSUM_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #11005 = PseudoVWREDSUM_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #11004 = PseudoVWREDSUM_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #11003 = PseudoVWREDSUM_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #11002 = PseudoVWREDSUM_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #11001 = PseudoVWREDSUM_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #11000 = PseudoVWREDSUM_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #10999 = PseudoVWREDSUM_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #10998 = PseudoVWREDSUM_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #10997 = PseudoVWREDSUM_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10996 = PseudoVWREDSUM_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10995 = PseudoVWREDSUM_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10994 = PseudoVWREDSUM_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10993 = PseudoVWREDSUM_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10992 = PseudoVWREDSUM_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10991 = PseudoVWREDSUM_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10990 = PseudoVWREDSUMU_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10989 = PseudoVWREDSUMU_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10988 = PseudoVWREDSUMU_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10987 = PseudoVWREDSUMU_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10986 = PseudoVWREDSUMU_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10985 = PseudoVWREDSUMU_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10984 = PseudoVWREDSUMU_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10983 = PseudoVWREDSUMU_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10982 = PseudoVWREDSUMU_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10981 = PseudoVWREDSUMU_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10980 = PseudoVWREDSUMU_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10979 = PseudoVWREDSUMU_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #10978 = PseudoVWREDSUMU_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #10977 = PseudoVWREDSUMU_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #10976 = PseudoVWREDSUMU_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #10975 = PseudoVWREDSUMU_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #10974 = PseudoVWREDSUMU_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #10973 = PseudoVWREDSUMU_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #10972 = PseudoVWREDSUMU_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #10971 = PseudoVWREDSUMU_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #10970 = PseudoVWREDSUMU_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #10969 = PseudoVWREDSUMU_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #10968 = PseudoVWREDSUMU_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #10967 = PseudoVWREDSUMU_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #10966 = PseudoVWREDSUMU_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #10965 = PseudoVWREDSUMU_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #10964 = PseudoVWREDSUMU_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #10963 = PseudoVWREDSUMU_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #10962 = PseudoVWREDSUMU_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #10961 = PseudoVWREDSUMU_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10960 = PseudoVWREDSUMU_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10959 = PseudoVWREDSUMU_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10958 = PseudoVWREDSUMU_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10957 = PseudoVWREDSUMU_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #10956 = PseudoVWREDSUMU_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #10955 = PseudoVWREDSUMU_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10954 = PseudoVWMUL_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10953 = PseudoVWMUL_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10952 = PseudoVWMUL_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10951 = PseudoVWMUL_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10950 = PseudoVWMUL_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10949 = PseudoVWMUL_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7659] },  // Inst #10948 = PseudoVWMUL_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7652] },  // Inst #10947 = PseudoVWMUL_VX_M4
  { 8, &RISCVDescs.OperandInfo[7644] },  // Inst #10946 = PseudoVWMUL_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7637] },  // Inst #10945 = PseudoVWMUL_VX_M2
  { 8, &RISCVDescs.OperandInfo[7629] },  // Inst #10944 = PseudoVWMUL_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7622] },  // Inst #10943 = PseudoVWMUL_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10942 = PseudoVWMUL_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10941 = PseudoVWMUL_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10940 = PseudoVWMUL_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10939 = PseudoVWMUL_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10938 = PseudoVWMUL_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10937 = PseudoVWMUL_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #10936 = PseudoVWMUL_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #10935 = PseudoVWMUL_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #10934 = PseudoVWMUL_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #10933 = PseudoVWMUL_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #10932 = PseudoVWMUL_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #10931 = PseudoVWMUL_VV_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10930 = PseudoVWMULU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10929 = PseudoVWMULU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10928 = PseudoVWMULU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10927 = PseudoVWMULU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10926 = PseudoVWMULU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10925 = PseudoVWMULU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7659] },  // Inst #10924 = PseudoVWMULU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7652] },  // Inst #10923 = PseudoVWMULU_VX_M4
  { 8, &RISCVDescs.OperandInfo[7644] },  // Inst #10922 = PseudoVWMULU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7637] },  // Inst #10921 = PseudoVWMULU_VX_M2
  { 8, &RISCVDescs.OperandInfo[7629] },  // Inst #10920 = PseudoVWMULU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7622] },  // Inst #10919 = PseudoVWMULU_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10918 = PseudoVWMULU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10917 = PseudoVWMULU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10916 = PseudoVWMULU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10915 = PseudoVWMULU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10914 = PseudoVWMULU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10913 = PseudoVWMULU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #10912 = PseudoVWMULU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #10911 = PseudoVWMULU_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #10910 = PseudoVWMULU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #10909 = PseudoVWMULU_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #10908 = PseudoVWMULU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #10907 = PseudoVWMULU_VV_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10906 = PseudoVWMULSU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10905 = PseudoVWMULSU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10904 = PseudoVWMULSU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10903 = PseudoVWMULSU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10902 = PseudoVWMULSU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10901 = PseudoVWMULSU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7659] },  // Inst #10900 = PseudoVWMULSU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7652] },  // Inst #10899 = PseudoVWMULSU_VX_M4
  { 8, &RISCVDescs.OperandInfo[7644] },  // Inst #10898 = PseudoVWMULSU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7637] },  // Inst #10897 = PseudoVWMULSU_VX_M2
  { 8, &RISCVDescs.OperandInfo[7629] },  // Inst #10896 = PseudoVWMULSU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7622] },  // Inst #10895 = PseudoVWMULSU_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10894 = PseudoVWMULSU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10893 = PseudoVWMULSU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10892 = PseudoVWMULSU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10891 = PseudoVWMULSU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10890 = PseudoVWMULSU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10889 = PseudoVWMULSU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #10888 = PseudoVWMULSU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #10887 = PseudoVWMULSU_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #10886 = PseudoVWMULSU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #10885 = PseudoVWMULSU_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #10884 = PseudoVWMULSU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #10883 = PseudoVWMULSU_VV_M1
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10882 = PseudoVWMACC_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10881 = PseudoVWMACC_VX_MF8
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10880 = PseudoVWMACC_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10879 = PseudoVWMACC_VX_MF4
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10878 = PseudoVWMACC_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10877 = PseudoVWMACC_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7704] },  // Inst #10876 = PseudoVWMACC_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7697] },  // Inst #10875 = PseudoVWMACC_VX_M4
  { 8, &RISCVDescs.OperandInfo[7689] },  // Inst #10874 = PseudoVWMACC_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7682] },  // Inst #10873 = PseudoVWMACC_VX_M2
  { 8, &RISCVDescs.OperandInfo[7674] },  // Inst #10872 = PseudoVWMACC_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7667] },  // Inst #10871 = PseudoVWMACC_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10870 = PseudoVWMACC_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10869 = PseudoVWMACC_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10868 = PseudoVWMACC_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10867 = PseudoVWMACC_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10866 = PseudoVWMACC_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10865 = PseudoVWMACC_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #10864 = PseudoVWMACC_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #10863 = PseudoVWMACC_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #10862 = PseudoVWMACC_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #10861 = PseudoVWMACC_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #10860 = PseudoVWMACC_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #10859 = PseudoVWMACC_VV_M1
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10858 = PseudoVWMACCU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10857 = PseudoVWMACCU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10856 = PseudoVWMACCU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10855 = PseudoVWMACCU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10854 = PseudoVWMACCU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10853 = PseudoVWMACCU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7704] },  // Inst #10852 = PseudoVWMACCU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7697] },  // Inst #10851 = PseudoVWMACCU_VX_M4
  { 8, &RISCVDescs.OperandInfo[7689] },  // Inst #10850 = PseudoVWMACCU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7682] },  // Inst #10849 = PseudoVWMACCU_VX_M2
  { 8, &RISCVDescs.OperandInfo[7674] },  // Inst #10848 = PseudoVWMACCU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7667] },  // Inst #10847 = PseudoVWMACCU_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10846 = PseudoVWMACCU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10845 = PseudoVWMACCU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10844 = PseudoVWMACCU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10843 = PseudoVWMACCU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10842 = PseudoVWMACCU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10841 = PseudoVWMACCU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #10840 = PseudoVWMACCU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #10839 = PseudoVWMACCU_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #10838 = PseudoVWMACCU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #10837 = PseudoVWMACCU_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #10836 = PseudoVWMACCU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #10835 = PseudoVWMACCU_VV_M1
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10834 = PseudoVWMACCUS_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10833 = PseudoVWMACCUS_VX_MF8
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10832 = PseudoVWMACCUS_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10831 = PseudoVWMACCUS_VX_MF4
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10830 = PseudoVWMACCUS_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10829 = PseudoVWMACCUS_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7704] },  // Inst #10828 = PseudoVWMACCUS_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7697] },  // Inst #10827 = PseudoVWMACCUS_VX_M4
  { 8, &RISCVDescs.OperandInfo[7689] },  // Inst #10826 = PseudoVWMACCUS_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7682] },  // Inst #10825 = PseudoVWMACCUS_VX_M2
  { 8, &RISCVDescs.OperandInfo[7674] },  // Inst #10824 = PseudoVWMACCUS_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7667] },  // Inst #10823 = PseudoVWMACCUS_VX_M1
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10822 = PseudoVWMACCSU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10821 = PseudoVWMACCSU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10820 = PseudoVWMACCSU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10819 = PseudoVWMACCSU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #10818 = PseudoVWMACCSU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #10817 = PseudoVWMACCSU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7704] },  // Inst #10816 = PseudoVWMACCSU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7697] },  // Inst #10815 = PseudoVWMACCSU_VX_M4
  { 8, &RISCVDescs.OperandInfo[7689] },  // Inst #10814 = PseudoVWMACCSU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7682] },  // Inst #10813 = PseudoVWMACCSU_VX_M2
  { 8, &RISCVDescs.OperandInfo[7674] },  // Inst #10812 = PseudoVWMACCSU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7667] },  // Inst #10811 = PseudoVWMACCSU_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10810 = PseudoVWMACCSU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10809 = PseudoVWMACCSU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10808 = PseudoVWMACCSU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10807 = PseudoVWMACCSU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10806 = PseudoVWMACCSU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10805 = PseudoVWMACCSU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #10804 = PseudoVWMACCSU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #10803 = PseudoVWMACCSU_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #10802 = PseudoVWMACCSU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #10801 = PseudoVWMACCSU_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #10800 = PseudoVWMACCSU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #10799 = PseudoVWMACCSU_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #10798 = PseudoVWADD_WX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #10797 = PseudoVWADD_WX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #10796 = PseudoVWADD_WX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #10795 = PseudoVWADD_WX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #10794 = PseudoVWADD_WX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #10793 = PseudoVWADD_WX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #10792 = PseudoVWADD_WX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #10791 = PseudoVWADD_WX_M4
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #10790 = PseudoVWADD_WX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #10789 = PseudoVWADD_WX_M2
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #10788 = PseudoVWADD_WX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #10787 = PseudoVWADD_WX_M1
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #10786 = PseudoVWADD_WV_MF8_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #10785 = PseudoVWADD_WV_MF8_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10784 = PseudoVWADD_WV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10783 = PseudoVWADD_WV_MF8
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #10782 = PseudoVWADD_WV_MF4_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #10781 = PseudoVWADD_WV_MF4_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10780 = PseudoVWADD_WV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10779 = PseudoVWADD_WV_MF4
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #10778 = PseudoVWADD_WV_MF2_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #10777 = PseudoVWADD_WV_MF2_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10776 = PseudoVWADD_WV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10775 = PseudoVWADD_WV_MF2
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #10774 = PseudoVWADD_WV_M4_TIED
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #10773 = PseudoVWADD_WV_M4_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6332] },  // Inst #10772 = PseudoVWADD_WV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6325] },  // Inst #10771 = PseudoVWADD_WV_M4
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #10770 = PseudoVWADD_WV_M2_TIED
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #10769 = PseudoVWADD_WV_M2_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6287] },  // Inst #10768 = PseudoVWADD_WV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6280] },  // Inst #10767 = PseudoVWADD_WV_M2
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #10766 = PseudoVWADD_WV_M1_TIED
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #10765 = PseudoVWADD_WV_M1_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #10764 = PseudoVWADD_WV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #10763 = PseudoVWADD_WV_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10762 = PseudoVWADD_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10761 = PseudoVWADD_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10760 = PseudoVWADD_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10759 = PseudoVWADD_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10758 = PseudoVWADD_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10757 = PseudoVWADD_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7659] },  // Inst #10756 = PseudoVWADD_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7652] },  // Inst #10755 = PseudoVWADD_VX_M4
  { 8, &RISCVDescs.OperandInfo[7644] },  // Inst #10754 = PseudoVWADD_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7637] },  // Inst #10753 = PseudoVWADD_VX_M2
  { 8, &RISCVDescs.OperandInfo[7629] },  // Inst #10752 = PseudoVWADD_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7622] },  // Inst #10751 = PseudoVWADD_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10750 = PseudoVWADD_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10749 = PseudoVWADD_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10748 = PseudoVWADD_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10747 = PseudoVWADD_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10746 = PseudoVWADD_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10745 = PseudoVWADD_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #10744 = PseudoVWADD_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #10743 = PseudoVWADD_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #10742 = PseudoVWADD_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #10741 = PseudoVWADD_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #10740 = PseudoVWADD_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #10739 = PseudoVWADD_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #10738 = PseudoVWADDU_WX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #10737 = PseudoVWADDU_WX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #10736 = PseudoVWADDU_WX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #10735 = PseudoVWADDU_WX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #10734 = PseudoVWADDU_WX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #10733 = PseudoVWADDU_WX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #10732 = PseudoVWADDU_WX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #10731 = PseudoVWADDU_WX_M4
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #10730 = PseudoVWADDU_WX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #10729 = PseudoVWADDU_WX_M2
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #10728 = PseudoVWADDU_WX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #10727 = PseudoVWADDU_WX_M1
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #10726 = PseudoVWADDU_WV_MF8_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #10725 = PseudoVWADDU_WV_MF8_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10724 = PseudoVWADDU_WV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10723 = PseudoVWADDU_WV_MF8
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #10722 = PseudoVWADDU_WV_MF4_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #10721 = PseudoVWADDU_WV_MF4_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10720 = PseudoVWADDU_WV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10719 = PseudoVWADDU_WV_MF4
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #10718 = PseudoVWADDU_WV_MF2_TIED
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #10717 = PseudoVWADDU_WV_MF2_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10716 = PseudoVWADDU_WV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10715 = PseudoVWADDU_WV_MF2
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #10714 = PseudoVWADDU_WV_M4_TIED
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #10713 = PseudoVWADDU_WV_M4_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6332] },  // Inst #10712 = PseudoVWADDU_WV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6325] },  // Inst #10711 = PseudoVWADDU_WV_M4
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #10710 = PseudoVWADDU_WV_M2_TIED
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #10709 = PseudoVWADDU_WV_M2_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6287] },  // Inst #10708 = PseudoVWADDU_WV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6280] },  // Inst #10707 = PseudoVWADDU_WV_M2
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #10706 = PseudoVWADDU_WV_M1_TIED
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #10705 = PseudoVWADDU_WV_M1_MASK_TIED
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #10704 = PseudoVWADDU_WV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #10703 = PseudoVWADDU_WV_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10702 = PseudoVWADDU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10701 = PseudoVWADDU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10700 = PseudoVWADDU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10699 = PseudoVWADDU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #10698 = PseudoVWADDU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #10697 = PseudoVWADDU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[7659] },  // Inst #10696 = PseudoVWADDU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7652] },  // Inst #10695 = PseudoVWADDU_VX_M4
  { 8, &RISCVDescs.OperandInfo[7644] },  // Inst #10694 = PseudoVWADDU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7637] },  // Inst #10693 = PseudoVWADDU_VX_M2
  { 8, &RISCVDescs.OperandInfo[7629] },  // Inst #10692 = PseudoVWADDU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7622] },  // Inst #10691 = PseudoVWADDU_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10690 = PseudoVWADDU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10689 = PseudoVWADDU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10688 = PseudoVWADDU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10687 = PseudoVWADDU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #10686 = PseudoVWADDU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #10685 = PseudoVWADDU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[7614] },  // Inst #10684 = PseudoVWADDU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[7607] },  // Inst #10683 = PseudoVWADDU_VV_M4
  { 8, &RISCVDescs.OperandInfo[7599] },  // Inst #10682 = PseudoVWADDU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[7592] },  // Inst #10681 = PseudoVWADDU_VV_M2
  { 8, &RISCVDescs.OperandInfo[7584] },  // Inst #10680 = PseudoVWADDU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[7577] },  // Inst #10679 = PseudoVWADDU_VV_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10678 = PseudoVSUXSEG8EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10677 = PseudoVSUXSEG8EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10676 = PseudoVSUXSEG8EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10675 = PseudoVSUXSEG8EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10674 = PseudoVSUXSEG8EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10673 = PseudoVSUXSEG8EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10672 = PseudoVSUXSEG8EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10671 = PseudoVSUXSEG8EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10670 = PseudoVSUXSEG8EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10669 = PseudoVSUXSEG8EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10668 = PseudoVSUXSEG8EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10667 = PseudoVSUXSEG8EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10666 = PseudoVSUXSEG8EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10665 = PseudoVSUXSEG8EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10664 = PseudoVSUXSEG8EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10663 = PseudoVSUXSEG8EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10662 = PseudoVSUXSEG8EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10661 = PseudoVSUXSEG8EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10660 = PseudoVSUXSEG8EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10659 = PseudoVSUXSEG8EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7256] },  // Inst #10658 = PseudoVSUXSEG8EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7251] },  // Inst #10657 = PseudoVSUXSEG8EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7245] },  // Inst #10656 = PseudoVSUXSEG8EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7240] },  // Inst #10655 = PseudoVSUXSEG8EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7245] },  // Inst #10654 = PseudoVSUXSEG8EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7240] },  // Inst #10653 = PseudoVSUXSEG8EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #10652 = PseudoVSUXSEG8EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #10651 = PseudoVSUXSEG8EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #10650 = PseudoVSUXSEG8EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #10649 = PseudoVSUXSEG8EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #10648 = PseudoVSUXSEG8EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #10647 = PseudoVSUXSEG8EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10646 = PseudoVSUXSEG8EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10645 = PseudoVSUXSEG8EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10644 = PseudoVSUXSEG8EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10643 = PseudoVSUXSEG8EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10642 = PseudoVSUXSEG8EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10641 = PseudoVSUXSEG8EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10640 = PseudoVSUXSEG8EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10639 = PseudoVSUXSEG8EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10638 = PseudoVSUXSEG8EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10637 = PseudoVSUXSEG8EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10636 = PseudoVSUXSEG8EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10635 = PseudoVSUXSEG8EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10634 = PseudoVSUXSEG8EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10633 = PseudoVSUXSEG8EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10632 = PseudoVSUXSEG8EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10631 = PseudoVSUXSEG8EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7245] },  // Inst #10630 = PseudoVSUXSEG8EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7240] },  // Inst #10629 = PseudoVSUXSEG8EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #10628 = PseudoVSUXSEG8EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #10627 = PseudoVSUXSEG8EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #10626 = PseudoVSUXSEG8EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #10625 = PseudoVSUXSEG8EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10624 = PseudoVSUXSEG8EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10623 = PseudoVSUXSEG8EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10622 = PseudoVSUXSEG8EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10621 = PseudoVSUXSEG8EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10620 = PseudoVSUXSEG8EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10619 = PseudoVSUXSEG8EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10618 = PseudoVSUXSEG8EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10617 = PseudoVSUXSEG8EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10616 = PseudoVSUXSEG8EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10615 = PseudoVSUXSEG8EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10614 = PseudoVSUXSEG8EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10613 = PseudoVSUXSEG8EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10612 = PseudoVSUXSEG8EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10611 = PseudoVSUXSEG8EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10610 = PseudoVSUXSEG8EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10609 = PseudoVSUXSEG8EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10608 = PseudoVSUXSEG8EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10607 = PseudoVSUXSEG8EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10606 = PseudoVSUXSEG8EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10605 = PseudoVSUXSEG8EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #10604 = PseudoVSUXSEG8EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #10603 = PseudoVSUXSEG8EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10602 = PseudoVSUXSEG8EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10601 = PseudoVSUXSEG8EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #10600 = PseudoVSUXSEG8EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #10599 = PseudoVSUXSEG8EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10598 = PseudoVSUXSEG7EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10597 = PseudoVSUXSEG7EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10596 = PseudoVSUXSEG7EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10595 = PseudoVSUXSEG7EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10594 = PseudoVSUXSEG7EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10593 = PseudoVSUXSEG7EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10592 = PseudoVSUXSEG7EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10591 = PseudoVSUXSEG7EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10590 = PseudoVSUXSEG7EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10589 = PseudoVSUXSEG7EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10588 = PseudoVSUXSEG7EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10587 = PseudoVSUXSEG7EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10586 = PseudoVSUXSEG7EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10585 = PseudoVSUXSEG7EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10584 = PseudoVSUXSEG7EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10583 = PseudoVSUXSEG7EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10582 = PseudoVSUXSEG7EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10581 = PseudoVSUXSEG7EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10580 = PseudoVSUXSEG7EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10579 = PseudoVSUXSEG7EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7212] },  // Inst #10578 = PseudoVSUXSEG7EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7207] },  // Inst #10577 = PseudoVSUXSEG7EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7201] },  // Inst #10576 = PseudoVSUXSEG7EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7196] },  // Inst #10575 = PseudoVSUXSEG7EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7201] },  // Inst #10574 = PseudoVSUXSEG7EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7196] },  // Inst #10573 = PseudoVSUXSEG7EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #10572 = PseudoVSUXSEG7EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #10571 = PseudoVSUXSEG7EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #10570 = PseudoVSUXSEG7EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #10569 = PseudoVSUXSEG7EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #10568 = PseudoVSUXSEG7EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #10567 = PseudoVSUXSEG7EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10566 = PseudoVSUXSEG7EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10565 = PseudoVSUXSEG7EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10564 = PseudoVSUXSEG7EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10563 = PseudoVSUXSEG7EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10562 = PseudoVSUXSEG7EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10561 = PseudoVSUXSEG7EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10560 = PseudoVSUXSEG7EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10559 = PseudoVSUXSEG7EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10558 = PseudoVSUXSEG7EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10557 = PseudoVSUXSEG7EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10556 = PseudoVSUXSEG7EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10555 = PseudoVSUXSEG7EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10554 = PseudoVSUXSEG7EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10553 = PseudoVSUXSEG7EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10552 = PseudoVSUXSEG7EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10551 = PseudoVSUXSEG7EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7201] },  // Inst #10550 = PseudoVSUXSEG7EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7196] },  // Inst #10549 = PseudoVSUXSEG7EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #10548 = PseudoVSUXSEG7EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #10547 = PseudoVSUXSEG7EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #10546 = PseudoVSUXSEG7EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #10545 = PseudoVSUXSEG7EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10544 = PseudoVSUXSEG7EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10543 = PseudoVSUXSEG7EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10542 = PseudoVSUXSEG7EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10541 = PseudoVSUXSEG7EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10540 = PseudoVSUXSEG7EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10539 = PseudoVSUXSEG7EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10538 = PseudoVSUXSEG7EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10537 = PseudoVSUXSEG7EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10536 = PseudoVSUXSEG7EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10535 = PseudoVSUXSEG7EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10534 = PseudoVSUXSEG7EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10533 = PseudoVSUXSEG7EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10532 = PseudoVSUXSEG7EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10531 = PseudoVSUXSEG7EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10530 = PseudoVSUXSEG7EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10529 = PseudoVSUXSEG7EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10528 = PseudoVSUXSEG7EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10527 = PseudoVSUXSEG7EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10526 = PseudoVSUXSEG7EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10525 = PseudoVSUXSEG7EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #10524 = PseudoVSUXSEG7EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #10523 = PseudoVSUXSEG7EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10522 = PseudoVSUXSEG7EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10521 = PseudoVSUXSEG7EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #10520 = PseudoVSUXSEG7EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #10519 = PseudoVSUXSEG7EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10518 = PseudoVSUXSEG6EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10517 = PseudoVSUXSEG6EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10516 = PseudoVSUXSEG6EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10515 = PseudoVSUXSEG6EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10514 = PseudoVSUXSEG6EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10513 = PseudoVSUXSEG6EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10512 = PseudoVSUXSEG6EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10511 = PseudoVSUXSEG6EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10510 = PseudoVSUXSEG6EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10509 = PseudoVSUXSEG6EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10508 = PseudoVSUXSEG6EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10507 = PseudoVSUXSEG6EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10506 = PseudoVSUXSEG6EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10505 = PseudoVSUXSEG6EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10504 = PseudoVSUXSEG6EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10503 = PseudoVSUXSEG6EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10502 = PseudoVSUXSEG6EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10501 = PseudoVSUXSEG6EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10500 = PseudoVSUXSEG6EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10499 = PseudoVSUXSEG6EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7168] },  // Inst #10498 = PseudoVSUXSEG6EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7163] },  // Inst #10497 = PseudoVSUXSEG6EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7157] },  // Inst #10496 = PseudoVSUXSEG6EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7152] },  // Inst #10495 = PseudoVSUXSEG6EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7157] },  // Inst #10494 = PseudoVSUXSEG6EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7152] },  // Inst #10493 = PseudoVSUXSEG6EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #10492 = PseudoVSUXSEG6EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #10491 = PseudoVSUXSEG6EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #10490 = PseudoVSUXSEG6EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #10489 = PseudoVSUXSEG6EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #10488 = PseudoVSUXSEG6EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #10487 = PseudoVSUXSEG6EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10486 = PseudoVSUXSEG6EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10485 = PseudoVSUXSEG6EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10484 = PseudoVSUXSEG6EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10483 = PseudoVSUXSEG6EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10482 = PseudoVSUXSEG6EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10481 = PseudoVSUXSEG6EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10480 = PseudoVSUXSEG6EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10479 = PseudoVSUXSEG6EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10478 = PseudoVSUXSEG6EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10477 = PseudoVSUXSEG6EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10476 = PseudoVSUXSEG6EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10475 = PseudoVSUXSEG6EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10474 = PseudoVSUXSEG6EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10473 = PseudoVSUXSEG6EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10472 = PseudoVSUXSEG6EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10471 = PseudoVSUXSEG6EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7157] },  // Inst #10470 = PseudoVSUXSEG6EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7152] },  // Inst #10469 = PseudoVSUXSEG6EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #10468 = PseudoVSUXSEG6EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #10467 = PseudoVSUXSEG6EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #10466 = PseudoVSUXSEG6EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #10465 = PseudoVSUXSEG6EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10464 = PseudoVSUXSEG6EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10463 = PseudoVSUXSEG6EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10462 = PseudoVSUXSEG6EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10461 = PseudoVSUXSEG6EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10460 = PseudoVSUXSEG6EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10459 = PseudoVSUXSEG6EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10458 = PseudoVSUXSEG6EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10457 = PseudoVSUXSEG6EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10456 = PseudoVSUXSEG6EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10455 = PseudoVSUXSEG6EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10454 = PseudoVSUXSEG6EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10453 = PseudoVSUXSEG6EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10452 = PseudoVSUXSEG6EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10451 = PseudoVSUXSEG6EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10450 = PseudoVSUXSEG6EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10449 = PseudoVSUXSEG6EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10448 = PseudoVSUXSEG6EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10447 = PseudoVSUXSEG6EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10446 = PseudoVSUXSEG6EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10445 = PseudoVSUXSEG6EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #10444 = PseudoVSUXSEG6EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #10443 = PseudoVSUXSEG6EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10442 = PseudoVSUXSEG6EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10441 = PseudoVSUXSEG6EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #10440 = PseudoVSUXSEG6EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #10439 = PseudoVSUXSEG6EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10438 = PseudoVSUXSEG5EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10437 = PseudoVSUXSEG5EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10436 = PseudoVSUXSEG5EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10435 = PseudoVSUXSEG5EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10434 = PseudoVSUXSEG5EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10433 = PseudoVSUXSEG5EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10432 = PseudoVSUXSEG5EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10431 = PseudoVSUXSEG5EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10430 = PseudoVSUXSEG5EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10429 = PseudoVSUXSEG5EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10428 = PseudoVSUXSEG5EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10427 = PseudoVSUXSEG5EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10426 = PseudoVSUXSEG5EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10425 = PseudoVSUXSEG5EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10424 = PseudoVSUXSEG5EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10423 = PseudoVSUXSEG5EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10422 = PseudoVSUXSEG5EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10421 = PseudoVSUXSEG5EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10420 = PseudoVSUXSEG5EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10419 = PseudoVSUXSEG5EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7124] },  // Inst #10418 = PseudoVSUXSEG5EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7119] },  // Inst #10417 = PseudoVSUXSEG5EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7113] },  // Inst #10416 = PseudoVSUXSEG5EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7108] },  // Inst #10415 = PseudoVSUXSEG5EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7113] },  // Inst #10414 = PseudoVSUXSEG5EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7108] },  // Inst #10413 = PseudoVSUXSEG5EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #10412 = PseudoVSUXSEG5EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #10411 = PseudoVSUXSEG5EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #10410 = PseudoVSUXSEG5EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #10409 = PseudoVSUXSEG5EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #10408 = PseudoVSUXSEG5EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #10407 = PseudoVSUXSEG5EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10406 = PseudoVSUXSEG5EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10405 = PseudoVSUXSEG5EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10404 = PseudoVSUXSEG5EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10403 = PseudoVSUXSEG5EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10402 = PseudoVSUXSEG5EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10401 = PseudoVSUXSEG5EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10400 = PseudoVSUXSEG5EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10399 = PseudoVSUXSEG5EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10398 = PseudoVSUXSEG5EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10397 = PseudoVSUXSEG5EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10396 = PseudoVSUXSEG5EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10395 = PseudoVSUXSEG5EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10394 = PseudoVSUXSEG5EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10393 = PseudoVSUXSEG5EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10392 = PseudoVSUXSEG5EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10391 = PseudoVSUXSEG5EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7113] },  // Inst #10390 = PseudoVSUXSEG5EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7108] },  // Inst #10389 = PseudoVSUXSEG5EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #10388 = PseudoVSUXSEG5EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #10387 = PseudoVSUXSEG5EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #10386 = PseudoVSUXSEG5EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #10385 = PseudoVSUXSEG5EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10384 = PseudoVSUXSEG5EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10383 = PseudoVSUXSEG5EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10382 = PseudoVSUXSEG5EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10381 = PseudoVSUXSEG5EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10380 = PseudoVSUXSEG5EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10379 = PseudoVSUXSEG5EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10378 = PseudoVSUXSEG5EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10377 = PseudoVSUXSEG5EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10376 = PseudoVSUXSEG5EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10375 = PseudoVSUXSEG5EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10374 = PseudoVSUXSEG5EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10373 = PseudoVSUXSEG5EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10372 = PseudoVSUXSEG5EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10371 = PseudoVSUXSEG5EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10370 = PseudoVSUXSEG5EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10369 = PseudoVSUXSEG5EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10368 = PseudoVSUXSEG5EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10367 = PseudoVSUXSEG5EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10366 = PseudoVSUXSEG5EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10365 = PseudoVSUXSEG5EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #10364 = PseudoVSUXSEG5EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #10363 = PseudoVSUXSEG5EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10362 = PseudoVSUXSEG5EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10361 = PseudoVSUXSEG5EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #10360 = PseudoVSUXSEG5EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #10359 = PseudoVSUXSEG5EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10358 = PseudoVSUXSEG4EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10357 = PseudoVSUXSEG4EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10356 = PseudoVSUXSEG4EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10355 = PseudoVSUXSEG4EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10354 = PseudoVSUXSEG4EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10353 = PseudoVSUXSEG4EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10352 = PseudoVSUXSEG4EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10351 = PseudoVSUXSEG4EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10350 = PseudoVSUXSEG4EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10349 = PseudoVSUXSEG4EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10348 = PseudoVSUXSEG4EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10347 = PseudoVSUXSEG4EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #10346 = PseudoVSUXSEG4EI8_V_MF4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #10345 = PseudoVSUXSEG4EI8_V_MF4_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10344 = PseudoVSUXSEG4EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10343 = PseudoVSUXSEG4EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10342 = PseudoVSUXSEG4EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10341 = PseudoVSUXSEG4EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #10340 = PseudoVSUXSEG4EI8_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #10339 = PseudoVSUXSEG4EI8_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10338 = PseudoVSUXSEG4EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10337 = PseudoVSUXSEG4EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7036] },  // Inst #10336 = PseudoVSUXSEG4EI8_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7031] },  // Inst #10335 = PseudoVSUXSEG4EI8_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #10334 = PseudoVSUXSEG4EI8_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #10333 = PseudoVSUXSEG4EI8_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10332 = PseudoVSUXSEG4EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10331 = PseudoVSUXSEG4EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7069] },  // Inst #10330 = PseudoVSUXSEG4EI64_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7064] },  // Inst #10329 = PseudoVSUXSEG4EI64_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[7080] },  // Inst #10328 = PseudoVSUXSEG4EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7075] },  // Inst #10327 = PseudoVSUXSEG4EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7058] },  // Inst #10326 = PseudoVSUXSEG4EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7053] },  // Inst #10325 = PseudoVSUXSEG4EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7047] },  // Inst #10324 = PseudoVSUXSEG4EI64_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7042] },  // Inst #10323 = PseudoVSUXSEG4EI64_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[7058] },  // Inst #10322 = PseudoVSUXSEG4EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7053] },  // Inst #10321 = PseudoVSUXSEG4EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #10320 = PseudoVSUXSEG4EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #10319 = PseudoVSUXSEG4EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #10318 = PseudoVSUXSEG4EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #10317 = PseudoVSUXSEG4EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7036] },  // Inst #10316 = PseudoVSUXSEG4EI64_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7031] },  // Inst #10315 = PseudoVSUXSEG4EI64_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #10314 = PseudoVSUXSEG4EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #10313 = PseudoVSUXSEG4EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10312 = PseudoVSUXSEG4EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10311 = PseudoVSUXSEG4EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10310 = PseudoVSUXSEG4EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10309 = PseudoVSUXSEG4EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10308 = PseudoVSUXSEG4EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10307 = PseudoVSUXSEG4EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10306 = PseudoVSUXSEG4EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10305 = PseudoVSUXSEG4EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10304 = PseudoVSUXSEG4EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10303 = PseudoVSUXSEG4EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10302 = PseudoVSUXSEG4EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10301 = PseudoVSUXSEG4EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10300 = PseudoVSUXSEG4EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10299 = PseudoVSUXSEG4EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10298 = PseudoVSUXSEG4EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10297 = PseudoVSUXSEG4EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7069] },  // Inst #10296 = PseudoVSUXSEG4EI32_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7064] },  // Inst #10295 = PseudoVSUXSEG4EI32_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[7047] },  // Inst #10294 = PseudoVSUXSEG4EI32_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7042] },  // Inst #10293 = PseudoVSUXSEG4EI32_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[7058] },  // Inst #10292 = PseudoVSUXSEG4EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7053] },  // Inst #10291 = PseudoVSUXSEG4EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #10290 = PseudoVSUXSEG4EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #10289 = PseudoVSUXSEG4EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7036] },  // Inst #10288 = PseudoVSUXSEG4EI32_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7031] },  // Inst #10287 = PseudoVSUXSEG4EI32_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #10286 = PseudoVSUXSEG4EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #10285 = PseudoVSUXSEG4EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10284 = PseudoVSUXSEG4EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10283 = PseudoVSUXSEG4EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10282 = PseudoVSUXSEG4EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10281 = PseudoVSUXSEG4EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #10280 = PseudoVSUXSEG4EI32_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #10279 = PseudoVSUXSEG4EI32_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10278 = PseudoVSUXSEG4EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10277 = PseudoVSUXSEG4EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10276 = PseudoVSUXSEG4EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10275 = PseudoVSUXSEG4EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10274 = PseudoVSUXSEG4EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10273 = PseudoVSUXSEG4EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10272 = PseudoVSUXSEG4EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10271 = PseudoVSUXSEG4EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10270 = PseudoVSUXSEG4EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10269 = PseudoVSUXSEG4EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10268 = PseudoVSUXSEG4EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10267 = PseudoVSUXSEG4EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10266 = PseudoVSUXSEG4EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10265 = PseudoVSUXSEG4EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #10264 = PseudoVSUXSEG4EI16_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #10263 = PseudoVSUXSEG4EI16_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10262 = PseudoVSUXSEG4EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10261 = PseudoVSUXSEG4EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7047] },  // Inst #10260 = PseudoVSUXSEG4EI16_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7042] },  // Inst #10259 = PseudoVSUXSEG4EI16_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[7036] },  // Inst #10258 = PseudoVSUXSEG4EI16_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7031] },  // Inst #10257 = PseudoVSUXSEG4EI16_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #10256 = PseudoVSUXSEG4EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #10255 = PseudoVSUXSEG4EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10254 = PseudoVSUXSEG4EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10253 = PseudoVSUXSEG4EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #10252 = PseudoVSUXSEG4EI16_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #10251 = PseudoVSUXSEG4EI16_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #10250 = PseudoVSUXSEG4EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #10249 = PseudoVSUXSEG4EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10248 = PseudoVSUXSEG3EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10247 = PseudoVSUXSEG3EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10246 = PseudoVSUXSEG3EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10245 = PseudoVSUXSEG3EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10244 = PseudoVSUXSEG3EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10243 = PseudoVSUXSEG3EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10242 = PseudoVSUXSEG3EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10241 = PseudoVSUXSEG3EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10240 = PseudoVSUXSEG3EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10239 = PseudoVSUXSEG3EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10238 = PseudoVSUXSEG3EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10237 = PseudoVSUXSEG3EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #10236 = PseudoVSUXSEG3EI8_V_MF4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #10235 = PseudoVSUXSEG3EI8_V_MF4_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10234 = PseudoVSUXSEG3EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10233 = PseudoVSUXSEG3EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10232 = PseudoVSUXSEG3EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10231 = PseudoVSUXSEG3EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #10230 = PseudoVSUXSEG3EI8_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #10229 = PseudoVSUXSEG3EI8_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10228 = PseudoVSUXSEG3EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10227 = PseudoVSUXSEG3EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6948] },  // Inst #10226 = PseudoVSUXSEG3EI8_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6943] },  // Inst #10225 = PseudoVSUXSEG3EI8_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #10224 = PseudoVSUXSEG3EI8_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #10223 = PseudoVSUXSEG3EI8_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10222 = PseudoVSUXSEG3EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10221 = PseudoVSUXSEG3EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6981] },  // Inst #10220 = PseudoVSUXSEG3EI64_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6976] },  // Inst #10219 = PseudoVSUXSEG3EI64_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6992] },  // Inst #10218 = PseudoVSUXSEG3EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6987] },  // Inst #10217 = PseudoVSUXSEG3EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[6970] },  // Inst #10216 = PseudoVSUXSEG3EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6965] },  // Inst #10215 = PseudoVSUXSEG3EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[6959] },  // Inst #10214 = PseudoVSUXSEG3EI64_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6954] },  // Inst #10213 = PseudoVSUXSEG3EI64_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6970] },  // Inst #10212 = PseudoVSUXSEG3EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6965] },  // Inst #10211 = PseudoVSUXSEG3EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #10210 = PseudoVSUXSEG3EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #10209 = PseudoVSUXSEG3EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #10208 = PseudoVSUXSEG3EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #10207 = PseudoVSUXSEG3EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6948] },  // Inst #10206 = PseudoVSUXSEG3EI64_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6943] },  // Inst #10205 = PseudoVSUXSEG3EI64_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #10204 = PseudoVSUXSEG3EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #10203 = PseudoVSUXSEG3EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10202 = PseudoVSUXSEG3EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10201 = PseudoVSUXSEG3EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10200 = PseudoVSUXSEG3EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10199 = PseudoVSUXSEG3EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10198 = PseudoVSUXSEG3EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10197 = PseudoVSUXSEG3EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10196 = PseudoVSUXSEG3EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10195 = PseudoVSUXSEG3EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10194 = PseudoVSUXSEG3EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10193 = PseudoVSUXSEG3EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10192 = PseudoVSUXSEG3EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10191 = PseudoVSUXSEG3EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10190 = PseudoVSUXSEG3EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10189 = PseudoVSUXSEG3EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10188 = PseudoVSUXSEG3EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10187 = PseudoVSUXSEG3EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6981] },  // Inst #10186 = PseudoVSUXSEG3EI32_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6976] },  // Inst #10185 = PseudoVSUXSEG3EI32_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6959] },  // Inst #10184 = PseudoVSUXSEG3EI32_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6954] },  // Inst #10183 = PseudoVSUXSEG3EI32_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6970] },  // Inst #10182 = PseudoVSUXSEG3EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6965] },  // Inst #10181 = PseudoVSUXSEG3EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #10180 = PseudoVSUXSEG3EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #10179 = PseudoVSUXSEG3EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6948] },  // Inst #10178 = PseudoVSUXSEG3EI32_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6943] },  // Inst #10177 = PseudoVSUXSEG3EI32_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #10176 = PseudoVSUXSEG3EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #10175 = PseudoVSUXSEG3EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10174 = PseudoVSUXSEG3EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10173 = PseudoVSUXSEG3EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10172 = PseudoVSUXSEG3EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10171 = PseudoVSUXSEG3EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #10170 = PseudoVSUXSEG3EI32_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #10169 = PseudoVSUXSEG3EI32_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10168 = PseudoVSUXSEG3EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10167 = PseudoVSUXSEG3EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10166 = PseudoVSUXSEG3EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10165 = PseudoVSUXSEG3EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10164 = PseudoVSUXSEG3EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10163 = PseudoVSUXSEG3EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10162 = PseudoVSUXSEG3EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10161 = PseudoVSUXSEG3EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10160 = PseudoVSUXSEG3EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10159 = PseudoVSUXSEG3EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10158 = PseudoVSUXSEG3EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10157 = PseudoVSUXSEG3EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10156 = PseudoVSUXSEG3EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10155 = PseudoVSUXSEG3EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #10154 = PseudoVSUXSEG3EI16_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #10153 = PseudoVSUXSEG3EI16_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10152 = PseudoVSUXSEG3EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10151 = PseudoVSUXSEG3EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6959] },  // Inst #10150 = PseudoVSUXSEG3EI16_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6954] },  // Inst #10149 = PseudoVSUXSEG3EI16_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6948] },  // Inst #10148 = PseudoVSUXSEG3EI16_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6943] },  // Inst #10147 = PseudoVSUXSEG3EI16_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #10146 = PseudoVSUXSEG3EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #10145 = PseudoVSUXSEG3EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10144 = PseudoVSUXSEG3EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10143 = PseudoVSUXSEG3EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #10142 = PseudoVSUXSEG3EI16_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #10141 = PseudoVSUXSEG3EI16_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #10140 = PseudoVSUXSEG3EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #10139 = PseudoVSUXSEG3EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10138 = PseudoVSUXSEG2EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10137 = PseudoVSUXSEG2EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10136 = PseudoVSUXSEG2EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10135 = PseudoVSUXSEG2EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10134 = PseudoVSUXSEG2EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10133 = PseudoVSUXSEG2EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10132 = PseudoVSUXSEG2EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10131 = PseudoVSUXSEG2EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10130 = PseudoVSUXSEG2EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10129 = PseudoVSUXSEG2EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10128 = PseudoVSUXSEG2EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10127 = PseudoVSUXSEG2EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #10126 = PseudoVSUXSEG2EI8_V_MF4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #10125 = PseudoVSUXSEG2EI8_V_MF4_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10124 = PseudoVSUXSEG2EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10123 = PseudoVSUXSEG2EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10122 = PseudoVSUXSEG2EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10121 = PseudoVSUXSEG2EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6805] },  // Inst #10120 = PseudoVSUXSEG2EI8_V_MF2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6800] },  // Inst #10119 = PseudoVSUXSEG2EI8_V_MF2_M4
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #10118 = PseudoVSUXSEG2EI8_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #10117 = PseudoVSUXSEG2EI8_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10116 = PseudoVSUXSEG2EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10115 = PseudoVSUXSEG2EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6860] },  // Inst #10114 = PseudoVSUXSEG2EI8_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6855] },  // Inst #10113 = PseudoVSUXSEG2EI8_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6838] },  // Inst #10112 = PseudoVSUXSEG2EI8_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6833] },  // Inst #10111 = PseudoVSUXSEG2EI8_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6827] },  // Inst #10110 = PseudoVSUXSEG2EI8_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6822] },  // Inst #10109 = PseudoVSUXSEG2EI8_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6805] },  // Inst #10108 = PseudoVSUXSEG2EI8_V_M1_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6800] },  // Inst #10107 = PseudoVSUXSEG2EI8_V_M1_M4
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #10106 = PseudoVSUXSEG2EI8_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #10105 = PseudoVSUXSEG2EI8_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10104 = PseudoVSUXSEG2EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10103 = PseudoVSUXSEG2EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6871] },  // Inst #10102 = PseudoVSUXSEG2EI64_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6866] },  // Inst #10101 = PseudoVSUXSEG2EI64_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6893] },  // Inst #10100 = PseudoVSUXSEG2EI64_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6888] },  // Inst #10099 = PseudoVSUXSEG2EI64_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6904] },  // Inst #10098 = PseudoVSUXSEG2EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6899] },  // Inst #10097 = PseudoVSUXSEG2EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[6882] },  // Inst #10096 = PseudoVSUXSEG2EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6877] },  // Inst #10095 = PseudoVSUXSEG2EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[6860] },  // Inst #10094 = PseudoVSUXSEG2EI64_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6855] },  // Inst #10093 = PseudoVSUXSEG2EI64_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6849] },  // Inst #10092 = PseudoVSUXSEG2EI64_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6844] },  // Inst #10091 = PseudoVSUXSEG2EI64_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6882] },  // Inst #10090 = PseudoVSUXSEG2EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6877] },  // Inst #10089 = PseudoVSUXSEG2EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #10088 = PseudoVSUXSEG2EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #10087 = PseudoVSUXSEG2EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #10086 = PseudoVSUXSEG2EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #10085 = PseudoVSUXSEG2EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6827] },  // Inst #10084 = PseudoVSUXSEG2EI64_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6822] },  // Inst #10083 = PseudoVSUXSEG2EI64_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #10082 = PseudoVSUXSEG2EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #10081 = PseudoVSUXSEG2EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10080 = PseudoVSUXSEG2EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10079 = PseudoVSUXSEG2EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10078 = PseudoVSUXSEG2EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10077 = PseudoVSUXSEG2EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10076 = PseudoVSUXSEG2EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10075 = PseudoVSUXSEG2EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10074 = PseudoVSUXSEG2EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10073 = PseudoVSUXSEG2EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10072 = PseudoVSUXSEG2EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10071 = PseudoVSUXSEG2EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10070 = PseudoVSUXSEG2EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10069 = PseudoVSUXSEG2EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10068 = PseudoVSUXSEG2EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10067 = PseudoVSUXSEG2EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10066 = PseudoVSUXSEG2EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10065 = PseudoVSUXSEG2EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6871] },  // Inst #10064 = PseudoVSUXSEG2EI32_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6866] },  // Inst #10063 = PseudoVSUXSEG2EI32_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6893] },  // Inst #10062 = PseudoVSUXSEG2EI32_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6888] },  // Inst #10061 = PseudoVSUXSEG2EI32_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6860] },  // Inst #10060 = PseudoVSUXSEG2EI32_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6855] },  // Inst #10059 = PseudoVSUXSEG2EI32_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6849] },  // Inst #10058 = PseudoVSUXSEG2EI32_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6844] },  // Inst #10057 = PseudoVSUXSEG2EI32_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6882] },  // Inst #10056 = PseudoVSUXSEG2EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6877] },  // Inst #10055 = PseudoVSUXSEG2EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #10054 = PseudoVSUXSEG2EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #10053 = PseudoVSUXSEG2EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6838] },  // Inst #10052 = PseudoVSUXSEG2EI32_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6833] },  // Inst #10051 = PseudoVSUXSEG2EI32_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6827] },  // Inst #10050 = PseudoVSUXSEG2EI32_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6822] },  // Inst #10049 = PseudoVSUXSEG2EI32_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #10048 = PseudoVSUXSEG2EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #10047 = PseudoVSUXSEG2EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10046 = PseudoVSUXSEG2EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10045 = PseudoVSUXSEG2EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10044 = PseudoVSUXSEG2EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10043 = PseudoVSUXSEG2EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #10042 = PseudoVSUXSEG2EI32_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #10041 = PseudoVSUXSEG2EI32_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10040 = PseudoVSUXSEG2EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10039 = PseudoVSUXSEG2EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10038 = PseudoVSUXSEG2EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10037 = PseudoVSUXSEG2EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10036 = PseudoVSUXSEG2EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10035 = PseudoVSUXSEG2EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10034 = PseudoVSUXSEG2EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10033 = PseudoVSUXSEG2EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10032 = PseudoVSUXSEG2EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10031 = PseudoVSUXSEG2EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10030 = PseudoVSUXSEG2EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10029 = PseudoVSUXSEG2EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10028 = PseudoVSUXSEG2EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10027 = PseudoVSUXSEG2EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #10026 = PseudoVSUXSEG2EI16_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #10025 = PseudoVSUXSEG2EI16_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10024 = PseudoVSUXSEG2EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10023 = PseudoVSUXSEG2EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6871] },  // Inst #10022 = PseudoVSUXSEG2EI16_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6866] },  // Inst #10021 = PseudoVSUXSEG2EI16_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6860] },  // Inst #10020 = PseudoVSUXSEG2EI16_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6855] },  // Inst #10019 = PseudoVSUXSEG2EI16_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6849] },  // Inst #10018 = PseudoVSUXSEG2EI16_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6844] },  // Inst #10017 = PseudoVSUXSEG2EI16_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6838] },  // Inst #10016 = PseudoVSUXSEG2EI16_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6833] },  // Inst #10015 = PseudoVSUXSEG2EI16_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6827] },  // Inst #10014 = PseudoVSUXSEG2EI16_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6822] },  // Inst #10013 = PseudoVSUXSEG2EI16_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #10012 = PseudoVSUXSEG2EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #10011 = PseudoVSUXSEG2EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10010 = PseudoVSUXSEG2EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10009 = PseudoVSUXSEG2EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6805] },  // Inst #10008 = PseudoVSUXSEG2EI16_V_M1_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6800] },  // Inst #10007 = PseudoVSUXSEG2EI16_V_M1_M4
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #10006 = PseudoVSUXSEG2EI16_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #10005 = PseudoVSUXSEG2EI16_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #10004 = PseudoVSUXSEG2EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #10003 = PseudoVSUXSEG2EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #10002 = PseudoVSUXEI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #10001 = PseudoVSUXEI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #10000 = PseudoVSUXEI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9999 = PseudoVSUXEI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9998 = PseudoVSUXEI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9997 = PseudoVSUXEI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9996 = PseudoVSUXEI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9995 = PseudoVSUXEI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9994 = PseudoVSUXEI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9993 = PseudoVSUXEI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9992 = PseudoVSUXEI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9991 = PseudoVSUXEI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #9990 = PseudoVSUXEI8_V_MF4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #9989 = PseudoVSUXEI8_V_MF4_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9988 = PseudoVSUXEI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9987 = PseudoVSUXEI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9986 = PseudoVSUXEI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9985 = PseudoVSUXEI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6629] },  // Inst #9984 = PseudoVSUXEI8_V_MF2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6624] },  // Inst #9983 = PseudoVSUXEI8_V_MF2_M4
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #9982 = PseudoVSUXEI8_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #9981 = PseudoVSUXEI8_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9980 = PseudoVSUXEI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9979 = PseudoVSUXEI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6728] },  // Inst #9978 = PseudoVSUXEI8_V_M8_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6723] },  // Inst #9977 = PseudoVSUXEI8_V_M8_M8
  { 6, &RISCVDescs.OperandInfo[6706] },  // Inst #9976 = PseudoVSUXEI8_V_M4_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6701] },  // Inst #9975 = PseudoVSUXEI8_V_M4_M8
  { 6, &RISCVDescs.OperandInfo[6695] },  // Inst #9974 = PseudoVSUXEI8_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6690] },  // Inst #9973 = PseudoVSUXEI8_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6673] },  // Inst #9972 = PseudoVSUXEI8_V_M2_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6668] },  // Inst #9971 = PseudoVSUXEI8_V_M2_M8
  { 6, &RISCVDescs.OperandInfo[6662] },  // Inst #9970 = PseudoVSUXEI8_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6657] },  // Inst #9969 = PseudoVSUXEI8_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6651] },  // Inst #9968 = PseudoVSUXEI8_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6646] },  // Inst #9967 = PseudoVSUXEI8_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6772] },  // Inst #9966 = PseudoVSUXEI8_V_M1_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6767] },  // Inst #9965 = PseudoVSUXEI8_V_M1_M8
  { 6, &RISCVDescs.OperandInfo[6629] },  // Inst #9964 = PseudoVSUXEI8_V_M1_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6624] },  // Inst #9963 = PseudoVSUXEI8_V_M1_M4
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #9962 = PseudoVSUXEI8_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #9961 = PseudoVSUXEI8_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9960 = PseudoVSUXEI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9959 = PseudoVSUXEI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6728] },  // Inst #9958 = PseudoVSUXEI64_V_M8_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6723] },  // Inst #9957 = PseudoVSUXEI64_V_M8_M8
  { 6, &RISCVDescs.OperandInfo[6717] },  // Inst #9956 = PseudoVSUXEI64_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6712] },  // Inst #9955 = PseudoVSUXEI64_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6750] },  // Inst #9954 = PseudoVSUXEI64_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6745] },  // Inst #9953 = PseudoVSUXEI64_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6761] },  // Inst #9952 = PseudoVSUXEI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6756] },  // Inst #9951 = PseudoVSUXEI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[6739] },  // Inst #9950 = PseudoVSUXEI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6734] },  // Inst #9949 = PseudoVSUXEI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[6695] },  // Inst #9948 = PseudoVSUXEI64_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6690] },  // Inst #9947 = PseudoVSUXEI64_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6684] },  // Inst #9946 = PseudoVSUXEI64_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6679] },  // Inst #9945 = PseudoVSUXEI64_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6739] },  // Inst #9944 = PseudoVSUXEI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6734] },  // Inst #9943 = PseudoVSUXEI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #9942 = PseudoVSUXEI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #9941 = PseudoVSUXEI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #9940 = PseudoVSUXEI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #9939 = PseudoVSUXEI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6651] },  // Inst #9938 = PseudoVSUXEI64_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6646] },  // Inst #9937 = PseudoVSUXEI64_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #9936 = PseudoVSUXEI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #9935 = PseudoVSUXEI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9934 = PseudoVSUXEI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9933 = PseudoVSUXEI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9932 = PseudoVSUXEI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9931 = PseudoVSUXEI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9930 = PseudoVSUXEI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9929 = PseudoVSUXEI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9928 = PseudoVSUXEI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9927 = PseudoVSUXEI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9926 = PseudoVSUXEI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9925 = PseudoVSUXEI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9924 = PseudoVSUXEI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9923 = PseudoVSUXEI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9922 = PseudoVSUXEI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9921 = PseudoVSUXEI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9920 = PseudoVSUXEI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9919 = PseudoVSUXEI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6728] },  // Inst #9918 = PseudoVSUXEI32_V_M8_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6723] },  // Inst #9917 = PseudoVSUXEI32_V_M8_M8
  { 6, &RISCVDescs.OperandInfo[6717] },  // Inst #9916 = PseudoVSUXEI32_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6712] },  // Inst #9915 = PseudoVSUXEI32_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6750] },  // Inst #9914 = PseudoVSUXEI32_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6745] },  // Inst #9913 = PseudoVSUXEI32_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6706] },  // Inst #9912 = PseudoVSUXEI32_V_M4_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6701] },  // Inst #9911 = PseudoVSUXEI32_V_M4_M8
  { 6, &RISCVDescs.OperandInfo[6695] },  // Inst #9910 = PseudoVSUXEI32_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6690] },  // Inst #9909 = PseudoVSUXEI32_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6684] },  // Inst #9908 = PseudoVSUXEI32_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6679] },  // Inst #9907 = PseudoVSUXEI32_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6739] },  // Inst #9906 = PseudoVSUXEI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6734] },  // Inst #9905 = PseudoVSUXEI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #9904 = PseudoVSUXEI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #9903 = PseudoVSUXEI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6662] },  // Inst #9902 = PseudoVSUXEI32_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6657] },  // Inst #9901 = PseudoVSUXEI32_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6651] },  // Inst #9900 = PseudoVSUXEI32_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6646] },  // Inst #9899 = PseudoVSUXEI32_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #9898 = PseudoVSUXEI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #9897 = PseudoVSUXEI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9896 = PseudoVSUXEI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9895 = PseudoVSUXEI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9894 = PseudoVSUXEI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9893 = PseudoVSUXEI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #9892 = PseudoVSUXEI32_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #9891 = PseudoVSUXEI32_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9890 = PseudoVSUXEI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9889 = PseudoVSUXEI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9888 = PseudoVSUXEI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9887 = PseudoVSUXEI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9886 = PseudoVSUXEI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9885 = PseudoVSUXEI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9884 = PseudoVSUXEI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9883 = PseudoVSUXEI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9882 = PseudoVSUXEI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9881 = PseudoVSUXEI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9880 = PseudoVSUXEI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9879 = PseudoVSUXEI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9878 = PseudoVSUXEI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9877 = PseudoVSUXEI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #9876 = PseudoVSUXEI16_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #9875 = PseudoVSUXEI16_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9874 = PseudoVSUXEI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9873 = PseudoVSUXEI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6728] },  // Inst #9872 = PseudoVSUXEI16_V_M8_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6723] },  // Inst #9871 = PseudoVSUXEI16_V_M8_M8
  { 6, &RISCVDescs.OperandInfo[6717] },  // Inst #9870 = PseudoVSUXEI16_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6712] },  // Inst #9869 = PseudoVSUXEI16_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6706] },  // Inst #9868 = PseudoVSUXEI16_V_M4_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6701] },  // Inst #9867 = PseudoVSUXEI16_V_M4_M8
  { 6, &RISCVDescs.OperandInfo[6695] },  // Inst #9866 = PseudoVSUXEI16_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6690] },  // Inst #9865 = PseudoVSUXEI16_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6684] },  // Inst #9864 = PseudoVSUXEI16_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6679] },  // Inst #9863 = PseudoVSUXEI16_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6673] },  // Inst #9862 = PseudoVSUXEI16_V_M2_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6668] },  // Inst #9861 = PseudoVSUXEI16_V_M2_M8
  { 6, &RISCVDescs.OperandInfo[6662] },  // Inst #9860 = PseudoVSUXEI16_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6657] },  // Inst #9859 = PseudoVSUXEI16_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6651] },  // Inst #9858 = PseudoVSUXEI16_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6646] },  // Inst #9857 = PseudoVSUXEI16_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #9856 = PseudoVSUXEI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #9855 = PseudoVSUXEI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9854 = PseudoVSUXEI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9853 = PseudoVSUXEI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6629] },  // Inst #9852 = PseudoVSUXEI16_V_M1_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6624] },  // Inst #9851 = PseudoVSUXEI16_V_M1_M4
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #9850 = PseudoVSUXEI16_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #9849 = PseudoVSUXEI16_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #9848 = PseudoVSUXEI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #9847 = PseudoVSUXEI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9846 = PseudoVSUB_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9845 = PseudoVSUB_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9844 = PseudoVSUB_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9843 = PseudoVSUB_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9842 = PseudoVSUB_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9841 = PseudoVSUB_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #9840 = PseudoVSUB_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #9839 = PseudoVSUB_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #9838 = PseudoVSUB_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #9837 = PseudoVSUB_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #9836 = PseudoVSUB_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #9835 = PseudoVSUB_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9834 = PseudoVSUB_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9833 = PseudoVSUB_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9832 = PseudoVSUB_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9831 = PseudoVSUB_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9830 = PseudoVSUB_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9829 = PseudoVSUB_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9828 = PseudoVSUB_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9827 = PseudoVSUB_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #9826 = PseudoVSUB_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #9825 = PseudoVSUB_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #9824 = PseudoVSUB_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #9823 = PseudoVSUB_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #9822 = PseudoVSUB_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #9821 = PseudoVSUB_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9820 = PseudoVSUB_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9819 = PseudoVSUB_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9818 = PseudoVSSUB_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9817 = PseudoVSSUB_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9816 = PseudoVSSUB_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9815 = PseudoVSSUB_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9814 = PseudoVSSUB_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9813 = PseudoVSSUB_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #9812 = PseudoVSSUB_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #9811 = PseudoVSSUB_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #9810 = PseudoVSSUB_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #9809 = PseudoVSSUB_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #9808 = PseudoVSSUB_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #9807 = PseudoVSSUB_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9806 = PseudoVSSUB_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9805 = PseudoVSSUB_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9804 = PseudoVSSUB_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9803 = PseudoVSSUB_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9802 = PseudoVSSUB_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9801 = PseudoVSSUB_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9800 = PseudoVSSUB_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9799 = PseudoVSSUB_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #9798 = PseudoVSSUB_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #9797 = PseudoVSSUB_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #9796 = PseudoVSSUB_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #9795 = PseudoVSSUB_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #9794 = PseudoVSSUB_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #9793 = PseudoVSSUB_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9792 = PseudoVSSUB_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9791 = PseudoVSSUB_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9790 = PseudoVSSUBU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9789 = PseudoVSSUBU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9788 = PseudoVSSUBU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9787 = PseudoVSSUBU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9786 = PseudoVSSUBU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9785 = PseudoVSSUBU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #9784 = PseudoVSSUBU_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #9783 = PseudoVSSUBU_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #9782 = PseudoVSSUBU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #9781 = PseudoVSSUBU_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #9780 = PseudoVSSUBU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #9779 = PseudoVSSUBU_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9778 = PseudoVSSUBU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9777 = PseudoVSSUBU_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9776 = PseudoVSSUBU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9775 = PseudoVSSUBU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9774 = PseudoVSSUBU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9773 = PseudoVSSUBU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9772 = PseudoVSSUBU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9771 = PseudoVSSUBU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #9770 = PseudoVSSUBU_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #9769 = PseudoVSSUBU_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #9768 = PseudoVSSUBU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #9767 = PseudoVSSUBU_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #9766 = PseudoVSSUBU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #9765 = PseudoVSSUBU_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9764 = PseudoVSSUBU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9763 = PseudoVSSUBU_VV_M1
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9762 = PseudoVSSSEG8E8_V_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9761 = PseudoVSSSEG8E8_V_MF8
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9760 = PseudoVSSSEG8E8_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9759 = PseudoVSSSEG8E8_V_MF4
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9758 = PseudoVSSSEG8E8_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9757 = PseudoVSSSEG8E8_V_MF2
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9756 = PseudoVSSSEG8E8_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9755 = PseudoVSSSEG8E8_V_M1
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9754 = PseudoVSSSEG8E64_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9753 = PseudoVSSSEG8E64_V_M1
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9752 = PseudoVSSSEG8E32_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9751 = PseudoVSSSEG8E32_V_MF2
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9750 = PseudoVSSSEG8E32_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9749 = PseudoVSSSEG8E32_V_M1
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9748 = PseudoVSSSEG8E16_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9747 = PseudoVSSSEG8E16_V_MF4
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9746 = PseudoVSSSEG8E16_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9745 = PseudoVSSSEG8E16_V_MF2
  { 6, &RISCVDescs.OperandInfo[7571] },  // Inst #9744 = PseudoVSSSEG8E16_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7566] },  // Inst #9743 = PseudoVSSSEG8E16_V_M1
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9742 = PseudoVSSSEG7E8_V_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9741 = PseudoVSSSEG7E8_V_MF8
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9740 = PseudoVSSSEG7E8_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9739 = PseudoVSSSEG7E8_V_MF4
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9738 = PseudoVSSSEG7E8_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9737 = PseudoVSSSEG7E8_V_MF2
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9736 = PseudoVSSSEG7E8_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9735 = PseudoVSSSEG7E8_V_M1
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9734 = PseudoVSSSEG7E64_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9733 = PseudoVSSSEG7E64_V_M1
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9732 = PseudoVSSSEG7E32_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9731 = PseudoVSSSEG7E32_V_MF2
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9730 = PseudoVSSSEG7E32_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9729 = PseudoVSSSEG7E32_V_M1
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9728 = PseudoVSSSEG7E16_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9727 = PseudoVSSSEG7E16_V_MF4
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9726 = PseudoVSSSEG7E16_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9725 = PseudoVSSSEG7E16_V_MF2
  { 6, &RISCVDescs.OperandInfo[7560] },  // Inst #9724 = PseudoVSSSEG7E16_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7555] },  // Inst #9723 = PseudoVSSSEG7E16_V_M1
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9722 = PseudoVSSSEG6E8_V_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9721 = PseudoVSSSEG6E8_V_MF8
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9720 = PseudoVSSSEG6E8_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9719 = PseudoVSSSEG6E8_V_MF4
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9718 = PseudoVSSSEG6E8_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9717 = PseudoVSSSEG6E8_V_MF2
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9716 = PseudoVSSSEG6E8_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9715 = PseudoVSSSEG6E8_V_M1
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9714 = PseudoVSSSEG6E64_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9713 = PseudoVSSSEG6E64_V_M1
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9712 = PseudoVSSSEG6E32_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9711 = PseudoVSSSEG6E32_V_MF2
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9710 = PseudoVSSSEG6E32_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9709 = PseudoVSSSEG6E32_V_M1
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9708 = PseudoVSSSEG6E16_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9707 = PseudoVSSSEG6E16_V_MF4
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9706 = PseudoVSSSEG6E16_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9705 = PseudoVSSSEG6E16_V_MF2
  { 6, &RISCVDescs.OperandInfo[7549] },  // Inst #9704 = PseudoVSSSEG6E16_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7544] },  // Inst #9703 = PseudoVSSSEG6E16_V_M1
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9702 = PseudoVSSSEG5E8_V_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9701 = PseudoVSSSEG5E8_V_MF8
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9700 = PseudoVSSSEG5E8_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9699 = PseudoVSSSEG5E8_V_MF4
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9698 = PseudoVSSSEG5E8_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9697 = PseudoVSSSEG5E8_V_MF2
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9696 = PseudoVSSSEG5E8_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9695 = PseudoVSSSEG5E8_V_M1
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9694 = PseudoVSSSEG5E64_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9693 = PseudoVSSSEG5E64_V_M1
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9692 = PseudoVSSSEG5E32_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9691 = PseudoVSSSEG5E32_V_MF2
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9690 = PseudoVSSSEG5E32_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9689 = PseudoVSSSEG5E32_V_M1
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9688 = PseudoVSSSEG5E16_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9687 = PseudoVSSSEG5E16_V_MF4
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9686 = PseudoVSSSEG5E16_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9685 = PseudoVSSSEG5E16_V_MF2
  { 6, &RISCVDescs.OperandInfo[7538] },  // Inst #9684 = PseudoVSSSEG5E16_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7533] },  // Inst #9683 = PseudoVSSSEG5E16_V_M1
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9682 = PseudoVSSSEG4E8_V_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9681 = PseudoVSSSEG4E8_V_MF8
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9680 = PseudoVSSSEG4E8_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9679 = PseudoVSSSEG4E8_V_MF4
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9678 = PseudoVSSSEG4E8_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9677 = PseudoVSSSEG4E8_V_MF2
  { 6, &RISCVDescs.OperandInfo[7527] },  // Inst #9676 = PseudoVSSSEG4E8_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7522] },  // Inst #9675 = PseudoVSSSEG4E8_V_M2
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9674 = PseudoVSSSEG4E8_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9673 = PseudoVSSSEG4E8_V_M1
  { 6, &RISCVDescs.OperandInfo[7527] },  // Inst #9672 = PseudoVSSSEG4E64_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7522] },  // Inst #9671 = PseudoVSSSEG4E64_V_M2
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9670 = PseudoVSSSEG4E64_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9669 = PseudoVSSSEG4E64_V_M1
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9668 = PseudoVSSSEG4E32_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9667 = PseudoVSSSEG4E32_V_MF2
  { 6, &RISCVDescs.OperandInfo[7527] },  // Inst #9666 = PseudoVSSSEG4E32_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7522] },  // Inst #9665 = PseudoVSSSEG4E32_V_M2
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9664 = PseudoVSSSEG4E32_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9663 = PseudoVSSSEG4E32_V_M1
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9662 = PseudoVSSSEG4E16_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9661 = PseudoVSSSEG4E16_V_MF4
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9660 = PseudoVSSSEG4E16_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9659 = PseudoVSSSEG4E16_V_MF2
  { 6, &RISCVDescs.OperandInfo[7527] },  // Inst #9658 = PseudoVSSSEG4E16_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7522] },  // Inst #9657 = PseudoVSSSEG4E16_V_M2
  { 6, &RISCVDescs.OperandInfo[7516] },  // Inst #9656 = PseudoVSSSEG4E16_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7511] },  // Inst #9655 = PseudoVSSSEG4E16_V_M1
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9654 = PseudoVSSSEG3E8_V_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9653 = PseudoVSSSEG3E8_V_MF8
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9652 = PseudoVSSSEG3E8_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9651 = PseudoVSSSEG3E8_V_MF4
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9650 = PseudoVSSSEG3E8_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9649 = PseudoVSSSEG3E8_V_MF2
  { 6, &RISCVDescs.OperandInfo[7505] },  // Inst #9648 = PseudoVSSSEG3E8_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7500] },  // Inst #9647 = PseudoVSSSEG3E8_V_M2
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9646 = PseudoVSSSEG3E8_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9645 = PseudoVSSSEG3E8_V_M1
  { 6, &RISCVDescs.OperandInfo[7505] },  // Inst #9644 = PseudoVSSSEG3E64_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7500] },  // Inst #9643 = PseudoVSSSEG3E64_V_M2
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9642 = PseudoVSSSEG3E64_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9641 = PseudoVSSSEG3E64_V_M1
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9640 = PseudoVSSSEG3E32_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9639 = PseudoVSSSEG3E32_V_MF2
  { 6, &RISCVDescs.OperandInfo[7505] },  // Inst #9638 = PseudoVSSSEG3E32_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7500] },  // Inst #9637 = PseudoVSSSEG3E32_V_M2
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9636 = PseudoVSSSEG3E32_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9635 = PseudoVSSSEG3E32_V_M1
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9634 = PseudoVSSSEG3E16_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9633 = PseudoVSSSEG3E16_V_MF4
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9632 = PseudoVSSSEG3E16_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9631 = PseudoVSSSEG3E16_V_MF2
  { 6, &RISCVDescs.OperandInfo[7505] },  // Inst #9630 = PseudoVSSSEG3E16_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7500] },  // Inst #9629 = PseudoVSSSEG3E16_V_M2
  { 6, &RISCVDescs.OperandInfo[7494] },  // Inst #9628 = PseudoVSSSEG3E16_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7489] },  // Inst #9627 = PseudoVSSSEG3E16_V_M1
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9626 = PseudoVSSSEG2E8_V_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9625 = PseudoVSSSEG2E8_V_MF8
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9624 = PseudoVSSSEG2E8_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9623 = PseudoVSSSEG2E8_V_MF4
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9622 = PseudoVSSSEG2E8_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9621 = PseudoVSSSEG2E8_V_MF2
  { 6, &RISCVDescs.OperandInfo[7483] },  // Inst #9620 = PseudoVSSSEG2E8_V_M4_MASK
  { 5, &RISCVDescs.OperandInfo[7478] },  // Inst #9619 = PseudoVSSSEG2E8_V_M4
  { 6, &RISCVDescs.OperandInfo[7472] },  // Inst #9618 = PseudoVSSSEG2E8_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7467] },  // Inst #9617 = PseudoVSSSEG2E8_V_M2
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9616 = PseudoVSSSEG2E8_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9615 = PseudoVSSSEG2E8_V_M1
  { 6, &RISCVDescs.OperandInfo[7483] },  // Inst #9614 = PseudoVSSSEG2E64_V_M4_MASK
  { 5, &RISCVDescs.OperandInfo[7478] },  // Inst #9613 = PseudoVSSSEG2E64_V_M4
  { 6, &RISCVDescs.OperandInfo[7472] },  // Inst #9612 = PseudoVSSSEG2E64_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7467] },  // Inst #9611 = PseudoVSSSEG2E64_V_M2
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9610 = PseudoVSSSEG2E64_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9609 = PseudoVSSSEG2E64_V_M1
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9608 = PseudoVSSSEG2E32_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9607 = PseudoVSSSEG2E32_V_MF2
  { 6, &RISCVDescs.OperandInfo[7483] },  // Inst #9606 = PseudoVSSSEG2E32_V_M4_MASK
  { 5, &RISCVDescs.OperandInfo[7478] },  // Inst #9605 = PseudoVSSSEG2E32_V_M4
  { 6, &RISCVDescs.OperandInfo[7472] },  // Inst #9604 = PseudoVSSSEG2E32_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7467] },  // Inst #9603 = PseudoVSSSEG2E32_V_M2
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9602 = PseudoVSSSEG2E32_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9601 = PseudoVSSSEG2E32_V_M1
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9600 = PseudoVSSSEG2E16_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9599 = PseudoVSSSEG2E16_V_MF4
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9598 = PseudoVSSSEG2E16_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9597 = PseudoVSSSEG2E16_V_MF2
  { 6, &RISCVDescs.OperandInfo[7483] },  // Inst #9596 = PseudoVSSSEG2E16_V_M4_MASK
  { 5, &RISCVDescs.OperandInfo[7478] },  // Inst #9595 = PseudoVSSSEG2E16_V_M4
  { 6, &RISCVDescs.OperandInfo[7472] },  // Inst #9594 = PseudoVSSSEG2E16_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7467] },  // Inst #9593 = PseudoVSSSEG2E16_V_M2
  { 6, &RISCVDescs.OperandInfo[7461] },  // Inst #9592 = PseudoVSSSEG2E16_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7456] },  // Inst #9591 = PseudoVSSSEG2E16_V_M1
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #9590 = PseudoVSSRL_VX_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #9589 = PseudoVSSRL_VX_MF8
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #9588 = PseudoVSSRL_VX_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #9587 = PseudoVSSRL_VX_MF4
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #9586 = PseudoVSSRL_VX_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #9585 = PseudoVSSRL_VX_MF2
  { 9, &RISCVDescs.OperandInfo[544] },  // Inst #9584 = PseudoVSSRL_VX_M8_MASK
  { 8, &RISCVDescs.OperandInfo[536] },  // Inst #9583 = PseudoVSSRL_VX_M8
  { 9, &RISCVDescs.OperandInfo[527] },  // Inst #9582 = PseudoVSSRL_VX_M4_MASK
  { 8, &RISCVDescs.OperandInfo[519] },  // Inst #9581 = PseudoVSSRL_VX_M4
  { 9, &RISCVDescs.OperandInfo[510] },  // Inst #9580 = PseudoVSSRL_VX_M2_MASK
  { 8, &RISCVDescs.OperandInfo[502] },  // Inst #9579 = PseudoVSSRL_VX_M2
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #9578 = PseudoVSSRL_VX_M1_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #9577 = PseudoVSSRL_VX_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #9576 = PseudoVSSRL_VV_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #9575 = PseudoVSSRL_VV_MF8
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #9574 = PseudoVSSRL_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #9573 = PseudoVSSRL_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #9572 = PseudoVSSRL_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #9571 = PseudoVSSRL_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #9570 = PseudoVSSRL_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #9569 = PseudoVSSRL_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #9568 = PseudoVSSRL_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #9567 = PseudoVSSRL_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #9566 = PseudoVSSRL_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #9565 = PseudoVSSRL_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #9564 = PseudoVSSRL_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #9563 = PseudoVSSRL_VV_M1
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #9562 = PseudoVSSRL_VI_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #9561 = PseudoVSSRL_VI_MF8
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #9560 = PseudoVSSRL_VI_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #9559 = PseudoVSSRL_VI_MF4
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #9558 = PseudoVSSRL_VI_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #9557 = PseudoVSSRL_VI_MF2
  { 9, &RISCVDescs.OperandInfo[7447] },  // Inst #9556 = PseudoVSSRL_VI_M8_MASK
  { 8, &RISCVDescs.OperandInfo[7439] },  // Inst #9555 = PseudoVSSRL_VI_M8
  { 9, &RISCVDescs.OperandInfo[7430] },  // Inst #9554 = PseudoVSSRL_VI_M4_MASK
  { 8, &RISCVDescs.OperandInfo[7422] },  // Inst #9553 = PseudoVSSRL_VI_M4
  { 9, &RISCVDescs.OperandInfo[7413] },  // Inst #9552 = PseudoVSSRL_VI_M2_MASK
  { 8, &RISCVDescs.OperandInfo[7405] },  // Inst #9551 = PseudoVSSRL_VI_M2
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #9550 = PseudoVSSRL_VI_M1_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #9549 = PseudoVSSRL_VI_M1
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #9548 = PseudoVSSRA_VX_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #9547 = PseudoVSSRA_VX_MF8
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #9546 = PseudoVSSRA_VX_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #9545 = PseudoVSSRA_VX_MF4
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #9544 = PseudoVSSRA_VX_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #9543 = PseudoVSSRA_VX_MF2
  { 9, &RISCVDescs.OperandInfo[544] },  // Inst #9542 = PseudoVSSRA_VX_M8_MASK
  { 8, &RISCVDescs.OperandInfo[536] },  // Inst #9541 = PseudoVSSRA_VX_M8
  { 9, &RISCVDescs.OperandInfo[527] },  // Inst #9540 = PseudoVSSRA_VX_M4_MASK
  { 8, &RISCVDescs.OperandInfo[519] },  // Inst #9539 = PseudoVSSRA_VX_M4
  { 9, &RISCVDescs.OperandInfo[510] },  // Inst #9538 = PseudoVSSRA_VX_M2_MASK
  { 8, &RISCVDescs.OperandInfo[502] },  // Inst #9537 = PseudoVSSRA_VX_M2
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #9536 = PseudoVSSRA_VX_M1_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #9535 = PseudoVSSRA_VX_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #9534 = PseudoVSSRA_VV_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #9533 = PseudoVSSRA_VV_MF8
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #9532 = PseudoVSSRA_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #9531 = PseudoVSSRA_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #9530 = PseudoVSSRA_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #9529 = PseudoVSSRA_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #9528 = PseudoVSSRA_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #9527 = PseudoVSSRA_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #9526 = PseudoVSSRA_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #9525 = PseudoVSSRA_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #9524 = PseudoVSSRA_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #9523 = PseudoVSSRA_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #9522 = PseudoVSSRA_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #9521 = PseudoVSSRA_VV_M1
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #9520 = PseudoVSSRA_VI_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #9519 = PseudoVSSRA_VI_MF8
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #9518 = PseudoVSSRA_VI_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #9517 = PseudoVSSRA_VI_MF4
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #9516 = PseudoVSSRA_VI_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #9515 = PseudoVSSRA_VI_MF2
  { 9, &RISCVDescs.OperandInfo[7447] },  // Inst #9514 = PseudoVSSRA_VI_M8_MASK
  { 8, &RISCVDescs.OperandInfo[7439] },  // Inst #9513 = PseudoVSSRA_VI_M8
  { 9, &RISCVDescs.OperandInfo[7430] },  // Inst #9512 = PseudoVSSRA_VI_M4_MASK
  { 8, &RISCVDescs.OperandInfo[7422] },  // Inst #9511 = PseudoVSSRA_VI_M4
  { 9, &RISCVDescs.OperandInfo[7413] },  // Inst #9510 = PseudoVSSRA_VI_M2_MASK
  { 8, &RISCVDescs.OperandInfo[7405] },  // Inst #9509 = PseudoVSSRA_VI_M2
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #9508 = PseudoVSSRA_VI_M1_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #9507 = PseudoVSSRA_VI_M1
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9506 = PseudoVSSEG8E8_V_MF8_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9505 = PseudoVSSEG8E8_V_MF8
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9504 = PseudoVSSEG8E8_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9503 = PseudoVSSEG8E8_V_MF4
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9502 = PseudoVSSEG8E8_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9501 = PseudoVSSEG8E8_V_MF2
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9500 = PseudoVSSEG8E8_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9499 = PseudoVSSEG8E8_V_M1
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9498 = PseudoVSSEG8E64_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9497 = PseudoVSSEG8E64_V_M1
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9496 = PseudoVSSEG8E32_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9495 = PseudoVSSEG8E32_V_MF2
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9494 = PseudoVSSEG8E32_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9493 = PseudoVSSEG8E32_V_M1
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9492 = PseudoVSSEG8E16_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9491 = PseudoVSSEG8E16_V_MF4
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9490 = PseudoVSSEG8E16_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9489 = PseudoVSSEG8E16_V_MF2
  { 5, &RISCVDescs.OperandInfo[7400] },  // Inst #9488 = PseudoVSSEG8E16_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7396] },  // Inst #9487 = PseudoVSSEG8E16_V_M1
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9486 = PseudoVSSEG7E8_V_MF8_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9485 = PseudoVSSEG7E8_V_MF8
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9484 = PseudoVSSEG7E8_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9483 = PseudoVSSEG7E8_V_MF4
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9482 = PseudoVSSEG7E8_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9481 = PseudoVSSEG7E8_V_MF2
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9480 = PseudoVSSEG7E8_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9479 = PseudoVSSEG7E8_V_M1
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9478 = PseudoVSSEG7E64_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9477 = PseudoVSSEG7E64_V_M1
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9476 = PseudoVSSEG7E32_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9475 = PseudoVSSEG7E32_V_MF2
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9474 = PseudoVSSEG7E32_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9473 = PseudoVSSEG7E32_V_M1
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9472 = PseudoVSSEG7E16_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9471 = PseudoVSSEG7E16_V_MF4
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9470 = PseudoVSSEG7E16_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9469 = PseudoVSSEG7E16_V_MF2
  { 5, &RISCVDescs.OperandInfo[7391] },  // Inst #9468 = PseudoVSSEG7E16_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7387] },  // Inst #9467 = PseudoVSSEG7E16_V_M1
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9466 = PseudoVSSEG6E8_V_MF8_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9465 = PseudoVSSEG6E8_V_MF8
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9464 = PseudoVSSEG6E8_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9463 = PseudoVSSEG6E8_V_MF4
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9462 = PseudoVSSEG6E8_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9461 = PseudoVSSEG6E8_V_MF2
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9460 = PseudoVSSEG6E8_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9459 = PseudoVSSEG6E8_V_M1
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9458 = PseudoVSSEG6E64_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9457 = PseudoVSSEG6E64_V_M1
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9456 = PseudoVSSEG6E32_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9455 = PseudoVSSEG6E32_V_MF2
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9454 = PseudoVSSEG6E32_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9453 = PseudoVSSEG6E32_V_M1
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9452 = PseudoVSSEG6E16_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9451 = PseudoVSSEG6E16_V_MF4
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9450 = PseudoVSSEG6E16_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9449 = PseudoVSSEG6E16_V_MF2
  { 5, &RISCVDescs.OperandInfo[7382] },  // Inst #9448 = PseudoVSSEG6E16_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7378] },  // Inst #9447 = PseudoVSSEG6E16_V_M1
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9446 = PseudoVSSEG5E8_V_MF8_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9445 = PseudoVSSEG5E8_V_MF8
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9444 = PseudoVSSEG5E8_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9443 = PseudoVSSEG5E8_V_MF4
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9442 = PseudoVSSEG5E8_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9441 = PseudoVSSEG5E8_V_MF2
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9440 = PseudoVSSEG5E8_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9439 = PseudoVSSEG5E8_V_M1
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9438 = PseudoVSSEG5E64_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9437 = PseudoVSSEG5E64_V_M1
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9436 = PseudoVSSEG5E32_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9435 = PseudoVSSEG5E32_V_MF2
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9434 = PseudoVSSEG5E32_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9433 = PseudoVSSEG5E32_V_M1
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9432 = PseudoVSSEG5E16_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9431 = PseudoVSSEG5E16_V_MF4
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9430 = PseudoVSSEG5E16_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9429 = PseudoVSSEG5E16_V_MF2
  { 5, &RISCVDescs.OperandInfo[7373] },  // Inst #9428 = PseudoVSSEG5E16_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7369] },  // Inst #9427 = PseudoVSSEG5E16_V_M1
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9426 = PseudoVSSEG4E8_V_MF8_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9425 = PseudoVSSEG4E8_V_MF8
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9424 = PseudoVSSEG4E8_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9423 = PseudoVSSEG4E8_V_MF4
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9422 = PseudoVSSEG4E8_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9421 = PseudoVSSEG4E8_V_MF2
  { 5, &RISCVDescs.OperandInfo[7364] },  // Inst #9420 = PseudoVSSEG4E8_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7360] },  // Inst #9419 = PseudoVSSEG4E8_V_M2
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9418 = PseudoVSSEG4E8_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9417 = PseudoVSSEG4E8_V_M1
  { 5, &RISCVDescs.OperandInfo[7364] },  // Inst #9416 = PseudoVSSEG4E64_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7360] },  // Inst #9415 = PseudoVSSEG4E64_V_M2
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9414 = PseudoVSSEG4E64_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9413 = PseudoVSSEG4E64_V_M1
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9412 = PseudoVSSEG4E32_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9411 = PseudoVSSEG4E32_V_MF2
  { 5, &RISCVDescs.OperandInfo[7364] },  // Inst #9410 = PseudoVSSEG4E32_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7360] },  // Inst #9409 = PseudoVSSEG4E32_V_M2
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9408 = PseudoVSSEG4E32_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9407 = PseudoVSSEG4E32_V_M1
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9406 = PseudoVSSEG4E16_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9405 = PseudoVSSEG4E16_V_MF4
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9404 = PseudoVSSEG4E16_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9403 = PseudoVSSEG4E16_V_MF2
  { 5, &RISCVDescs.OperandInfo[7364] },  // Inst #9402 = PseudoVSSEG4E16_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7360] },  // Inst #9401 = PseudoVSSEG4E16_V_M2
  { 5, &RISCVDescs.OperandInfo[7355] },  // Inst #9400 = PseudoVSSEG4E16_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7351] },  // Inst #9399 = PseudoVSSEG4E16_V_M1
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9398 = PseudoVSSEG3E8_V_MF8_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9397 = PseudoVSSEG3E8_V_MF8
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9396 = PseudoVSSEG3E8_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9395 = PseudoVSSEG3E8_V_MF4
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9394 = PseudoVSSEG3E8_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9393 = PseudoVSSEG3E8_V_MF2
  { 5, &RISCVDescs.OperandInfo[7346] },  // Inst #9392 = PseudoVSSEG3E8_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7342] },  // Inst #9391 = PseudoVSSEG3E8_V_M2
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9390 = PseudoVSSEG3E8_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9389 = PseudoVSSEG3E8_V_M1
  { 5, &RISCVDescs.OperandInfo[7346] },  // Inst #9388 = PseudoVSSEG3E64_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7342] },  // Inst #9387 = PseudoVSSEG3E64_V_M2
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9386 = PseudoVSSEG3E64_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9385 = PseudoVSSEG3E64_V_M1
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9384 = PseudoVSSEG3E32_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9383 = PseudoVSSEG3E32_V_MF2
  { 5, &RISCVDescs.OperandInfo[7346] },  // Inst #9382 = PseudoVSSEG3E32_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7342] },  // Inst #9381 = PseudoVSSEG3E32_V_M2
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9380 = PseudoVSSEG3E32_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9379 = PseudoVSSEG3E32_V_M1
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9378 = PseudoVSSEG3E16_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9377 = PseudoVSSEG3E16_V_MF4
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9376 = PseudoVSSEG3E16_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9375 = PseudoVSSEG3E16_V_MF2
  { 5, &RISCVDescs.OperandInfo[7346] },  // Inst #9374 = PseudoVSSEG3E16_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7342] },  // Inst #9373 = PseudoVSSEG3E16_V_M2
  { 5, &RISCVDescs.OperandInfo[7337] },  // Inst #9372 = PseudoVSSEG3E16_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7333] },  // Inst #9371 = PseudoVSSEG3E16_V_M1
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9370 = PseudoVSSEG2E8_V_MF8_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9369 = PseudoVSSEG2E8_V_MF8
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9368 = PseudoVSSEG2E8_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9367 = PseudoVSSEG2E8_V_MF4
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9366 = PseudoVSSEG2E8_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9365 = PseudoVSSEG2E8_V_MF2
  { 5, &RISCVDescs.OperandInfo[7328] },  // Inst #9364 = PseudoVSSEG2E8_V_M4_MASK
  { 4, &RISCVDescs.OperandInfo[7324] },  // Inst #9363 = PseudoVSSEG2E8_V_M4
  { 5, &RISCVDescs.OperandInfo[7319] },  // Inst #9362 = PseudoVSSEG2E8_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7315] },  // Inst #9361 = PseudoVSSEG2E8_V_M2
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9360 = PseudoVSSEG2E8_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9359 = PseudoVSSEG2E8_V_M1
  { 5, &RISCVDescs.OperandInfo[7328] },  // Inst #9358 = PseudoVSSEG2E64_V_M4_MASK
  { 4, &RISCVDescs.OperandInfo[7324] },  // Inst #9357 = PseudoVSSEG2E64_V_M4
  { 5, &RISCVDescs.OperandInfo[7319] },  // Inst #9356 = PseudoVSSEG2E64_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7315] },  // Inst #9355 = PseudoVSSEG2E64_V_M2
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9354 = PseudoVSSEG2E64_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9353 = PseudoVSSEG2E64_V_M1
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9352 = PseudoVSSEG2E32_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9351 = PseudoVSSEG2E32_V_MF2
  { 5, &RISCVDescs.OperandInfo[7328] },  // Inst #9350 = PseudoVSSEG2E32_V_M4_MASK
  { 4, &RISCVDescs.OperandInfo[7324] },  // Inst #9349 = PseudoVSSEG2E32_V_M4
  { 5, &RISCVDescs.OperandInfo[7319] },  // Inst #9348 = PseudoVSSEG2E32_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7315] },  // Inst #9347 = PseudoVSSEG2E32_V_M2
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9346 = PseudoVSSEG2E32_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9345 = PseudoVSSEG2E32_V_M1
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9344 = PseudoVSSEG2E16_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9343 = PseudoVSSEG2E16_V_MF4
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9342 = PseudoVSSEG2E16_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9341 = PseudoVSSEG2E16_V_MF2
  { 5, &RISCVDescs.OperandInfo[7328] },  // Inst #9340 = PseudoVSSEG2E16_V_M4_MASK
  { 4, &RISCVDescs.OperandInfo[7324] },  // Inst #9339 = PseudoVSSEG2E16_V_M4
  { 5, &RISCVDescs.OperandInfo[7319] },  // Inst #9338 = PseudoVSSEG2E16_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[7315] },  // Inst #9337 = PseudoVSSEG2E16_V_M2
  { 5, &RISCVDescs.OperandInfo[7310] },  // Inst #9336 = PseudoVSSEG2E16_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[7306] },  // Inst #9335 = PseudoVSSEG2E16_V_M1
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9334 = PseudoVSSE8_V_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9333 = PseudoVSSE8_V_MF8
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9332 = PseudoVSSE8_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9331 = PseudoVSSE8_V_MF4
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9330 = PseudoVSSE8_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9329 = PseudoVSSE8_V_MF2
  { 6, &RISCVDescs.OperandInfo[7300] },  // Inst #9328 = PseudoVSSE8_V_M8_MASK
  { 5, &RISCVDescs.OperandInfo[7295] },  // Inst #9327 = PseudoVSSE8_V_M8
  { 6, &RISCVDescs.OperandInfo[7289] },  // Inst #9326 = PseudoVSSE8_V_M4_MASK
  { 5, &RISCVDescs.OperandInfo[7284] },  // Inst #9325 = PseudoVSSE8_V_M4
  { 6, &RISCVDescs.OperandInfo[7278] },  // Inst #9324 = PseudoVSSE8_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7273] },  // Inst #9323 = PseudoVSSE8_V_M2
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9322 = PseudoVSSE8_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9321 = PseudoVSSE8_V_M1
  { 6, &RISCVDescs.OperandInfo[7300] },  // Inst #9320 = PseudoVSSE64_V_M8_MASK
  { 5, &RISCVDescs.OperandInfo[7295] },  // Inst #9319 = PseudoVSSE64_V_M8
  { 6, &RISCVDescs.OperandInfo[7289] },  // Inst #9318 = PseudoVSSE64_V_M4_MASK
  { 5, &RISCVDescs.OperandInfo[7284] },  // Inst #9317 = PseudoVSSE64_V_M4
  { 6, &RISCVDescs.OperandInfo[7278] },  // Inst #9316 = PseudoVSSE64_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7273] },  // Inst #9315 = PseudoVSSE64_V_M2
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9314 = PseudoVSSE64_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9313 = PseudoVSSE64_V_M1
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9312 = PseudoVSSE32_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9311 = PseudoVSSE32_V_MF2
  { 6, &RISCVDescs.OperandInfo[7300] },  // Inst #9310 = PseudoVSSE32_V_M8_MASK
  { 5, &RISCVDescs.OperandInfo[7295] },  // Inst #9309 = PseudoVSSE32_V_M8
  { 6, &RISCVDescs.OperandInfo[7289] },  // Inst #9308 = PseudoVSSE32_V_M4_MASK
  { 5, &RISCVDescs.OperandInfo[7284] },  // Inst #9307 = PseudoVSSE32_V_M4
  { 6, &RISCVDescs.OperandInfo[7278] },  // Inst #9306 = PseudoVSSE32_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7273] },  // Inst #9305 = PseudoVSSE32_V_M2
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9304 = PseudoVSSE32_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9303 = PseudoVSSE32_V_M1
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9302 = PseudoVSSE16_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9301 = PseudoVSSE16_V_MF4
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9300 = PseudoVSSE16_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9299 = PseudoVSSE16_V_MF2
  { 6, &RISCVDescs.OperandInfo[7300] },  // Inst #9298 = PseudoVSSE16_V_M8_MASK
  { 5, &RISCVDescs.OperandInfo[7295] },  // Inst #9297 = PseudoVSSE16_V_M8
  { 6, &RISCVDescs.OperandInfo[7289] },  // Inst #9296 = PseudoVSSE16_V_M4_MASK
  { 5, &RISCVDescs.OperandInfo[7284] },  // Inst #9295 = PseudoVSSE16_V_M4
  { 6, &RISCVDescs.OperandInfo[7278] },  // Inst #9294 = PseudoVSSE16_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7273] },  // Inst #9293 = PseudoVSSE16_V_M2
  { 6, &RISCVDescs.OperandInfo[7267] },  // Inst #9292 = PseudoVSSE16_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7262] },  // Inst #9291 = PseudoVSSE16_V_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9290 = PseudoVSRL_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9289 = PseudoVSRL_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9288 = PseudoVSRL_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9287 = PseudoVSRL_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9286 = PseudoVSRL_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9285 = PseudoVSRL_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #9284 = PseudoVSRL_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #9283 = PseudoVSRL_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #9282 = PseudoVSRL_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #9281 = PseudoVSRL_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #9280 = PseudoVSRL_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #9279 = PseudoVSRL_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9278 = PseudoVSRL_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9277 = PseudoVSRL_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9276 = PseudoVSRL_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9275 = PseudoVSRL_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9274 = PseudoVSRL_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9273 = PseudoVSRL_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9272 = PseudoVSRL_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9271 = PseudoVSRL_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #9270 = PseudoVSRL_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #9269 = PseudoVSRL_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #9268 = PseudoVSRL_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #9267 = PseudoVSRL_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #9266 = PseudoVSRL_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #9265 = PseudoVSRL_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9264 = PseudoVSRL_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9263 = PseudoVSRL_VV_M1
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #9262 = PseudoVSRL_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #9261 = PseudoVSRL_VI_MF8
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #9260 = PseudoVSRL_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #9259 = PseudoVSRL_VI_MF4
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #9258 = PseudoVSRL_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #9257 = PseudoVSRL_VI_MF2
  { 8, &RISCVDescs.OperandInfo[6594] },  // Inst #9256 = PseudoVSRL_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[898] },  // Inst #9255 = PseudoVSRL_VI_M8
  { 8, &RISCVDescs.OperandInfo[6586] },  // Inst #9254 = PseudoVSRL_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[891] },  // Inst #9253 = PseudoVSRL_VI_M4
  { 8, &RISCVDescs.OperandInfo[6578] },  // Inst #9252 = PseudoVSRL_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[884] },  // Inst #9251 = PseudoVSRL_VI_M2
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #9250 = PseudoVSRL_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #9249 = PseudoVSRL_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9248 = PseudoVSRA_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9247 = PseudoVSRA_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9246 = PseudoVSRA_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9245 = PseudoVSRA_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9244 = PseudoVSRA_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9243 = PseudoVSRA_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #9242 = PseudoVSRA_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #9241 = PseudoVSRA_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #9240 = PseudoVSRA_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #9239 = PseudoVSRA_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #9238 = PseudoVSRA_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #9237 = PseudoVSRA_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #9236 = PseudoVSRA_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #9235 = PseudoVSRA_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9234 = PseudoVSRA_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9233 = PseudoVSRA_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9232 = PseudoVSRA_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9231 = PseudoVSRA_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9230 = PseudoVSRA_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9229 = PseudoVSRA_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #9228 = PseudoVSRA_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #9227 = PseudoVSRA_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #9226 = PseudoVSRA_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #9225 = PseudoVSRA_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #9224 = PseudoVSRA_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #9223 = PseudoVSRA_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #9222 = PseudoVSRA_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #9221 = PseudoVSRA_VV_M1
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #9220 = PseudoVSRA_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #9219 = PseudoVSRA_VI_MF8
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #9218 = PseudoVSRA_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #9217 = PseudoVSRA_VI_MF4
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #9216 = PseudoVSRA_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #9215 = PseudoVSRA_VI_MF2
  { 8, &RISCVDescs.OperandInfo[6594] },  // Inst #9214 = PseudoVSRA_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[898] },  // Inst #9213 = PseudoVSRA_VI_M8
  { 8, &RISCVDescs.OperandInfo[6586] },  // Inst #9212 = PseudoVSRA_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[891] },  // Inst #9211 = PseudoVSRA_VI_M4
  { 8, &RISCVDescs.OperandInfo[6578] },  // Inst #9210 = PseudoVSRA_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[884] },  // Inst #9209 = PseudoVSRA_VI_M2
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #9208 = PseudoVSRA_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #9207 = PseudoVSRA_VI_M1
  { 2, &RISCVDescs.OperandInfo[6218] },  // Inst #9206 = PseudoVSPILL8_MF8
  { 2, &RISCVDescs.OperandInfo[6218] },  // Inst #9205 = PseudoVSPILL8_MF4
  { 2, &RISCVDescs.OperandInfo[6218] },  // Inst #9204 = PseudoVSPILL8_MF2
  { 2, &RISCVDescs.OperandInfo[6218] },  // Inst #9203 = PseudoVSPILL8_M1
  { 2, &RISCVDescs.OperandInfo[6216] },  // Inst #9202 = PseudoVSPILL7_MF8
  { 2, &RISCVDescs.OperandInfo[6216] },  // Inst #9201 = PseudoVSPILL7_MF4
  { 2, &RISCVDescs.OperandInfo[6216] },  // Inst #9200 = PseudoVSPILL7_MF2
  { 2, &RISCVDescs.OperandInfo[6216] },  // Inst #9199 = PseudoVSPILL7_M1
  { 2, &RISCVDescs.OperandInfo[6214] },  // Inst #9198 = PseudoVSPILL6_MF8
  { 2, &RISCVDescs.OperandInfo[6214] },  // Inst #9197 = PseudoVSPILL6_MF4
  { 2, &RISCVDescs.OperandInfo[6214] },  // Inst #9196 = PseudoVSPILL6_MF2
  { 2, &RISCVDescs.OperandInfo[6214] },  // Inst #9195 = PseudoVSPILL6_M1
  { 2, &RISCVDescs.OperandInfo[6212] },  // Inst #9194 = PseudoVSPILL5_MF8
  { 2, &RISCVDescs.OperandInfo[6212] },  // Inst #9193 = PseudoVSPILL5_MF4
  { 2, &RISCVDescs.OperandInfo[6212] },  // Inst #9192 = PseudoVSPILL5_MF2
  { 2, &RISCVDescs.OperandInfo[6212] },  // Inst #9191 = PseudoVSPILL5_M1
  { 2, &RISCVDescs.OperandInfo[6208] },  // Inst #9190 = PseudoVSPILL4_MF8
  { 2, &RISCVDescs.OperandInfo[6208] },  // Inst #9189 = PseudoVSPILL4_MF4
  { 2, &RISCVDescs.OperandInfo[6208] },  // Inst #9188 = PseudoVSPILL4_MF2
  { 2, &RISCVDescs.OperandInfo[6210] },  // Inst #9187 = PseudoVSPILL4_M2
  { 2, &RISCVDescs.OperandInfo[6208] },  // Inst #9186 = PseudoVSPILL4_M1
  { 2, &RISCVDescs.OperandInfo[6204] },  // Inst #9185 = PseudoVSPILL3_MF8
  { 2, &RISCVDescs.OperandInfo[6204] },  // Inst #9184 = PseudoVSPILL3_MF4
  { 2, &RISCVDescs.OperandInfo[6204] },  // Inst #9183 = PseudoVSPILL3_MF2
  { 2, &RISCVDescs.OperandInfo[6206] },  // Inst #9182 = PseudoVSPILL3_M2
  { 2, &RISCVDescs.OperandInfo[6204] },  // Inst #9181 = PseudoVSPILL3_M1
  { 2, &RISCVDescs.OperandInfo[6198] },  // Inst #9180 = PseudoVSPILL2_MF8
  { 2, &RISCVDescs.OperandInfo[6198] },  // Inst #9179 = PseudoVSPILL2_MF4
  { 2, &RISCVDescs.OperandInfo[6198] },  // Inst #9178 = PseudoVSPILL2_MF2
  { 2, &RISCVDescs.OperandInfo[6202] },  // Inst #9177 = PseudoVSPILL2_M4
  { 2, &RISCVDescs.OperandInfo[6200] },  // Inst #9176 = PseudoVSPILL2_M2
  { 2, &RISCVDescs.OperandInfo[6198] },  // Inst #9175 = PseudoVSPILL2_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9174 = PseudoVSOXSEG8EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9173 = PseudoVSOXSEG8EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9172 = PseudoVSOXSEG8EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9171 = PseudoVSOXSEG8EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9170 = PseudoVSOXSEG8EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9169 = PseudoVSOXSEG8EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9168 = PseudoVSOXSEG8EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9167 = PseudoVSOXSEG8EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9166 = PseudoVSOXSEG8EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9165 = PseudoVSOXSEG8EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9164 = PseudoVSOXSEG8EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9163 = PseudoVSOXSEG8EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9162 = PseudoVSOXSEG8EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9161 = PseudoVSOXSEG8EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9160 = PseudoVSOXSEG8EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9159 = PseudoVSOXSEG8EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9158 = PseudoVSOXSEG8EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9157 = PseudoVSOXSEG8EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9156 = PseudoVSOXSEG8EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9155 = PseudoVSOXSEG8EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7256] },  // Inst #9154 = PseudoVSOXSEG8EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7251] },  // Inst #9153 = PseudoVSOXSEG8EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7245] },  // Inst #9152 = PseudoVSOXSEG8EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7240] },  // Inst #9151 = PseudoVSOXSEG8EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7245] },  // Inst #9150 = PseudoVSOXSEG8EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7240] },  // Inst #9149 = PseudoVSOXSEG8EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #9148 = PseudoVSOXSEG8EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #9147 = PseudoVSOXSEG8EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #9146 = PseudoVSOXSEG8EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #9145 = PseudoVSOXSEG8EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #9144 = PseudoVSOXSEG8EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #9143 = PseudoVSOXSEG8EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9142 = PseudoVSOXSEG8EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9141 = PseudoVSOXSEG8EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9140 = PseudoVSOXSEG8EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9139 = PseudoVSOXSEG8EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9138 = PseudoVSOXSEG8EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9137 = PseudoVSOXSEG8EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9136 = PseudoVSOXSEG8EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9135 = PseudoVSOXSEG8EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9134 = PseudoVSOXSEG8EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9133 = PseudoVSOXSEG8EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9132 = PseudoVSOXSEG8EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9131 = PseudoVSOXSEG8EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9130 = PseudoVSOXSEG8EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9129 = PseudoVSOXSEG8EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9128 = PseudoVSOXSEG8EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9127 = PseudoVSOXSEG8EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7245] },  // Inst #9126 = PseudoVSOXSEG8EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7240] },  // Inst #9125 = PseudoVSOXSEG8EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #9124 = PseudoVSOXSEG8EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #9123 = PseudoVSOXSEG8EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #9122 = PseudoVSOXSEG8EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #9121 = PseudoVSOXSEG8EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9120 = PseudoVSOXSEG8EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9119 = PseudoVSOXSEG8EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9118 = PseudoVSOXSEG8EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9117 = PseudoVSOXSEG8EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9116 = PseudoVSOXSEG8EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9115 = PseudoVSOXSEG8EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9114 = PseudoVSOXSEG8EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9113 = PseudoVSOXSEG8EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9112 = PseudoVSOXSEG8EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9111 = PseudoVSOXSEG8EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9110 = PseudoVSOXSEG8EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9109 = PseudoVSOXSEG8EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9108 = PseudoVSOXSEG8EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9107 = PseudoVSOXSEG8EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9106 = PseudoVSOXSEG8EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9105 = PseudoVSOXSEG8EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9104 = PseudoVSOXSEG8EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9103 = PseudoVSOXSEG8EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9102 = PseudoVSOXSEG8EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9101 = PseudoVSOXSEG8EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7234] },  // Inst #9100 = PseudoVSOXSEG8EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7229] },  // Inst #9099 = PseudoVSOXSEG8EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9098 = PseudoVSOXSEG8EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9097 = PseudoVSOXSEG8EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7223] },  // Inst #9096 = PseudoVSOXSEG8EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7218] },  // Inst #9095 = PseudoVSOXSEG8EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9094 = PseudoVSOXSEG7EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9093 = PseudoVSOXSEG7EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9092 = PseudoVSOXSEG7EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9091 = PseudoVSOXSEG7EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9090 = PseudoVSOXSEG7EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9089 = PseudoVSOXSEG7EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9088 = PseudoVSOXSEG7EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9087 = PseudoVSOXSEG7EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9086 = PseudoVSOXSEG7EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9085 = PseudoVSOXSEG7EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9084 = PseudoVSOXSEG7EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9083 = PseudoVSOXSEG7EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9082 = PseudoVSOXSEG7EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9081 = PseudoVSOXSEG7EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9080 = PseudoVSOXSEG7EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9079 = PseudoVSOXSEG7EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9078 = PseudoVSOXSEG7EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9077 = PseudoVSOXSEG7EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9076 = PseudoVSOXSEG7EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9075 = PseudoVSOXSEG7EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7212] },  // Inst #9074 = PseudoVSOXSEG7EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7207] },  // Inst #9073 = PseudoVSOXSEG7EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7201] },  // Inst #9072 = PseudoVSOXSEG7EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7196] },  // Inst #9071 = PseudoVSOXSEG7EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7201] },  // Inst #9070 = PseudoVSOXSEG7EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7196] },  // Inst #9069 = PseudoVSOXSEG7EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #9068 = PseudoVSOXSEG7EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #9067 = PseudoVSOXSEG7EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #9066 = PseudoVSOXSEG7EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #9065 = PseudoVSOXSEG7EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #9064 = PseudoVSOXSEG7EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #9063 = PseudoVSOXSEG7EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9062 = PseudoVSOXSEG7EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9061 = PseudoVSOXSEG7EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9060 = PseudoVSOXSEG7EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9059 = PseudoVSOXSEG7EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9058 = PseudoVSOXSEG7EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9057 = PseudoVSOXSEG7EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9056 = PseudoVSOXSEG7EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9055 = PseudoVSOXSEG7EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9054 = PseudoVSOXSEG7EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9053 = PseudoVSOXSEG7EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9052 = PseudoVSOXSEG7EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9051 = PseudoVSOXSEG7EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9050 = PseudoVSOXSEG7EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9049 = PseudoVSOXSEG7EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9048 = PseudoVSOXSEG7EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9047 = PseudoVSOXSEG7EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7201] },  // Inst #9046 = PseudoVSOXSEG7EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7196] },  // Inst #9045 = PseudoVSOXSEG7EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #9044 = PseudoVSOXSEG7EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #9043 = PseudoVSOXSEG7EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #9042 = PseudoVSOXSEG7EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #9041 = PseudoVSOXSEG7EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9040 = PseudoVSOXSEG7EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9039 = PseudoVSOXSEG7EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9038 = PseudoVSOXSEG7EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9037 = PseudoVSOXSEG7EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9036 = PseudoVSOXSEG7EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9035 = PseudoVSOXSEG7EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9034 = PseudoVSOXSEG7EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9033 = PseudoVSOXSEG7EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9032 = PseudoVSOXSEG7EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9031 = PseudoVSOXSEG7EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9030 = PseudoVSOXSEG7EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9029 = PseudoVSOXSEG7EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9028 = PseudoVSOXSEG7EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9027 = PseudoVSOXSEG7EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9026 = PseudoVSOXSEG7EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9025 = PseudoVSOXSEG7EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9024 = PseudoVSOXSEG7EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9023 = PseudoVSOXSEG7EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9022 = PseudoVSOXSEG7EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9021 = PseudoVSOXSEG7EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7190] },  // Inst #9020 = PseudoVSOXSEG7EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7185] },  // Inst #9019 = PseudoVSOXSEG7EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9018 = PseudoVSOXSEG7EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9017 = PseudoVSOXSEG7EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7179] },  // Inst #9016 = PseudoVSOXSEG7EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7174] },  // Inst #9015 = PseudoVSOXSEG7EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #9014 = PseudoVSOXSEG6EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #9013 = PseudoVSOXSEG6EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #9012 = PseudoVSOXSEG6EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #9011 = PseudoVSOXSEG6EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #9010 = PseudoVSOXSEG6EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #9009 = PseudoVSOXSEG6EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #9008 = PseudoVSOXSEG6EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #9007 = PseudoVSOXSEG6EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #9006 = PseudoVSOXSEG6EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #9005 = PseudoVSOXSEG6EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #9004 = PseudoVSOXSEG6EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #9003 = PseudoVSOXSEG6EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #9002 = PseudoVSOXSEG6EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #9001 = PseudoVSOXSEG6EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #9000 = PseudoVSOXSEG6EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8999 = PseudoVSOXSEG6EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8998 = PseudoVSOXSEG6EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8997 = PseudoVSOXSEG6EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8996 = PseudoVSOXSEG6EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8995 = PseudoVSOXSEG6EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7168] },  // Inst #8994 = PseudoVSOXSEG6EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7163] },  // Inst #8993 = PseudoVSOXSEG6EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7157] },  // Inst #8992 = PseudoVSOXSEG6EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7152] },  // Inst #8991 = PseudoVSOXSEG6EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7157] },  // Inst #8990 = PseudoVSOXSEG6EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7152] },  // Inst #8989 = PseudoVSOXSEG6EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #8988 = PseudoVSOXSEG6EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #8987 = PseudoVSOXSEG6EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #8986 = PseudoVSOXSEG6EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #8985 = PseudoVSOXSEG6EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #8984 = PseudoVSOXSEG6EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #8983 = PseudoVSOXSEG6EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8982 = PseudoVSOXSEG6EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8981 = PseudoVSOXSEG6EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8980 = PseudoVSOXSEG6EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8979 = PseudoVSOXSEG6EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8978 = PseudoVSOXSEG6EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8977 = PseudoVSOXSEG6EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8976 = PseudoVSOXSEG6EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8975 = PseudoVSOXSEG6EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8974 = PseudoVSOXSEG6EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8973 = PseudoVSOXSEG6EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8972 = PseudoVSOXSEG6EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8971 = PseudoVSOXSEG6EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8970 = PseudoVSOXSEG6EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8969 = PseudoVSOXSEG6EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8968 = PseudoVSOXSEG6EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8967 = PseudoVSOXSEG6EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7157] },  // Inst #8966 = PseudoVSOXSEG6EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7152] },  // Inst #8965 = PseudoVSOXSEG6EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #8964 = PseudoVSOXSEG6EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #8963 = PseudoVSOXSEG6EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #8962 = PseudoVSOXSEG6EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #8961 = PseudoVSOXSEG6EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8960 = PseudoVSOXSEG6EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8959 = PseudoVSOXSEG6EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8958 = PseudoVSOXSEG6EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8957 = PseudoVSOXSEG6EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8956 = PseudoVSOXSEG6EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8955 = PseudoVSOXSEG6EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8954 = PseudoVSOXSEG6EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8953 = PseudoVSOXSEG6EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8952 = PseudoVSOXSEG6EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8951 = PseudoVSOXSEG6EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8950 = PseudoVSOXSEG6EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8949 = PseudoVSOXSEG6EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8948 = PseudoVSOXSEG6EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8947 = PseudoVSOXSEG6EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8946 = PseudoVSOXSEG6EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8945 = PseudoVSOXSEG6EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8944 = PseudoVSOXSEG6EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8943 = PseudoVSOXSEG6EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8942 = PseudoVSOXSEG6EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8941 = PseudoVSOXSEG6EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7146] },  // Inst #8940 = PseudoVSOXSEG6EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7141] },  // Inst #8939 = PseudoVSOXSEG6EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8938 = PseudoVSOXSEG6EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8937 = PseudoVSOXSEG6EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7135] },  // Inst #8936 = PseudoVSOXSEG6EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7130] },  // Inst #8935 = PseudoVSOXSEG6EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8934 = PseudoVSOXSEG5EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8933 = PseudoVSOXSEG5EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8932 = PseudoVSOXSEG5EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8931 = PseudoVSOXSEG5EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8930 = PseudoVSOXSEG5EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8929 = PseudoVSOXSEG5EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8928 = PseudoVSOXSEG5EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8927 = PseudoVSOXSEG5EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8926 = PseudoVSOXSEG5EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8925 = PseudoVSOXSEG5EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8924 = PseudoVSOXSEG5EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8923 = PseudoVSOXSEG5EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8922 = PseudoVSOXSEG5EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8921 = PseudoVSOXSEG5EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8920 = PseudoVSOXSEG5EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8919 = PseudoVSOXSEG5EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8918 = PseudoVSOXSEG5EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8917 = PseudoVSOXSEG5EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8916 = PseudoVSOXSEG5EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8915 = PseudoVSOXSEG5EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7124] },  // Inst #8914 = PseudoVSOXSEG5EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7119] },  // Inst #8913 = PseudoVSOXSEG5EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7113] },  // Inst #8912 = PseudoVSOXSEG5EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7108] },  // Inst #8911 = PseudoVSOXSEG5EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7113] },  // Inst #8910 = PseudoVSOXSEG5EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7108] },  // Inst #8909 = PseudoVSOXSEG5EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #8908 = PseudoVSOXSEG5EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #8907 = PseudoVSOXSEG5EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #8906 = PseudoVSOXSEG5EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #8905 = PseudoVSOXSEG5EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #8904 = PseudoVSOXSEG5EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #8903 = PseudoVSOXSEG5EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8902 = PseudoVSOXSEG5EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8901 = PseudoVSOXSEG5EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8900 = PseudoVSOXSEG5EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8899 = PseudoVSOXSEG5EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8898 = PseudoVSOXSEG5EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8897 = PseudoVSOXSEG5EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8896 = PseudoVSOXSEG5EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8895 = PseudoVSOXSEG5EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8894 = PseudoVSOXSEG5EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8893 = PseudoVSOXSEG5EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8892 = PseudoVSOXSEG5EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8891 = PseudoVSOXSEG5EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8890 = PseudoVSOXSEG5EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8889 = PseudoVSOXSEG5EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8888 = PseudoVSOXSEG5EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8887 = PseudoVSOXSEG5EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7113] },  // Inst #8886 = PseudoVSOXSEG5EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7108] },  // Inst #8885 = PseudoVSOXSEG5EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #8884 = PseudoVSOXSEG5EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #8883 = PseudoVSOXSEG5EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #8882 = PseudoVSOXSEG5EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #8881 = PseudoVSOXSEG5EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8880 = PseudoVSOXSEG5EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8879 = PseudoVSOXSEG5EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8878 = PseudoVSOXSEG5EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8877 = PseudoVSOXSEG5EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8876 = PseudoVSOXSEG5EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8875 = PseudoVSOXSEG5EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8874 = PseudoVSOXSEG5EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8873 = PseudoVSOXSEG5EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8872 = PseudoVSOXSEG5EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8871 = PseudoVSOXSEG5EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8870 = PseudoVSOXSEG5EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8869 = PseudoVSOXSEG5EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8868 = PseudoVSOXSEG5EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8867 = PseudoVSOXSEG5EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8866 = PseudoVSOXSEG5EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8865 = PseudoVSOXSEG5EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8864 = PseudoVSOXSEG5EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8863 = PseudoVSOXSEG5EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8862 = PseudoVSOXSEG5EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8861 = PseudoVSOXSEG5EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7102] },  // Inst #8860 = PseudoVSOXSEG5EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7097] },  // Inst #8859 = PseudoVSOXSEG5EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8858 = PseudoVSOXSEG5EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8857 = PseudoVSOXSEG5EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7091] },  // Inst #8856 = PseudoVSOXSEG5EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7086] },  // Inst #8855 = PseudoVSOXSEG5EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8854 = PseudoVSOXSEG4EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8853 = PseudoVSOXSEG4EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8852 = PseudoVSOXSEG4EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8851 = PseudoVSOXSEG4EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8850 = PseudoVSOXSEG4EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8849 = PseudoVSOXSEG4EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8848 = PseudoVSOXSEG4EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8847 = PseudoVSOXSEG4EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8846 = PseudoVSOXSEG4EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8845 = PseudoVSOXSEG4EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8844 = PseudoVSOXSEG4EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8843 = PseudoVSOXSEG4EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #8842 = PseudoVSOXSEG4EI8_V_MF4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #8841 = PseudoVSOXSEG4EI8_V_MF4_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8840 = PseudoVSOXSEG4EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8839 = PseudoVSOXSEG4EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8838 = PseudoVSOXSEG4EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8837 = PseudoVSOXSEG4EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #8836 = PseudoVSOXSEG4EI8_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #8835 = PseudoVSOXSEG4EI8_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8834 = PseudoVSOXSEG4EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8833 = PseudoVSOXSEG4EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7036] },  // Inst #8832 = PseudoVSOXSEG4EI8_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7031] },  // Inst #8831 = PseudoVSOXSEG4EI8_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #8830 = PseudoVSOXSEG4EI8_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #8829 = PseudoVSOXSEG4EI8_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8828 = PseudoVSOXSEG4EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8827 = PseudoVSOXSEG4EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7069] },  // Inst #8826 = PseudoVSOXSEG4EI64_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7064] },  // Inst #8825 = PseudoVSOXSEG4EI64_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[7080] },  // Inst #8824 = PseudoVSOXSEG4EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7075] },  // Inst #8823 = PseudoVSOXSEG4EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[7058] },  // Inst #8822 = PseudoVSOXSEG4EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7053] },  // Inst #8821 = PseudoVSOXSEG4EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[7047] },  // Inst #8820 = PseudoVSOXSEG4EI64_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7042] },  // Inst #8819 = PseudoVSOXSEG4EI64_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[7058] },  // Inst #8818 = PseudoVSOXSEG4EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7053] },  // Inst #8817 = PseudoVSOXSEG4EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #8816 = PseudoVSOXSEG4EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #8815 = PseudoVSOXSEG4EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #8814 = PseudoVSOXSEG4EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #8813 = PseudoVSOXSEG4EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7036] },  // Inst #8812 = PseudoVSOXSEG4EI64_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7031] },  // Inst #8811 = PseudoVSOXSEG4EI64_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #8810 = PseudoVSOXSEG4EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #8809 = PseudoVSOXSEG4EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8808 = PseudoVSOXSEG4EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8807 = PseudoVSOXSEG4EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8806 = PseudoVSOXSEG4EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8805 = PseudoVSOXSEG4EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8804 = PseudoVSOXSEG4EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8803 = PseudoVSOXSEG4EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8802 = PseudoVSOXSEG4EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8801 = PseudoVSOXSEG4EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8800 = PseudoVSOXSEG4EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8799 = PseudoVSOXSEG4EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8798 = PseudoVSOXSEG4EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8797 = PseudoVSOXSEG4EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8796 = PseudoVSOXSEG4EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8795 = PseudoVSOXSEG4EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8794 = PseudoVSOXSEG4EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8793 = PseudoVSOXSEG4EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7069] },  // Inst #8792 = PseudoVSOXSEG4EI32_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7064] },  // Inst #8791 = PseudoVSOXSEG4EI32_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[7047] },  // Inst #8790 = PseudoVSOXSEG4EI32_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7042] },  // Inst #8789 = PseudoVSOXSEG4EI32_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[7058] },  // Inst #8788 = PseudoVSOXSEG4EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7053] },  // Inst #8787 = PseudoVSOXSEG4EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #8786 = PseudoVSOXSEG4EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #8785 = PseudoVSOXSEG4EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[7036] },  // Inst #8784 = PseudoVSOXSEG4EI32_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7031] },  // Inst #8783 = PseudoVSOXSEG4EI32_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #8782 = PseudoVSOXSEG4EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #8781 = PseudoVSOXSEG4EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8780 = PseudoVSOXSEG4EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8779 = PseudoVSOXSEG4EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8778 = PseudoVSOXSEG4EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8777 = PseudoVSOXSEG4EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #8776 = PseudoVSOXSEG4EI32_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #8775 = PseudoVSOXSEG4EI32_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8774 = PseudoVSOXSEG4EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8773 = PseudoVSOXSEG4EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8772 = PseudoVSOXSEG4EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8771 = PseudoVSOXSEG4EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8770 = PseudoVSOXSEG4EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8769 = PseudoVSOXSEG4EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8768 = PseudoVSOXSEG4EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8767 = PseudoVSOXSEG4EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8766 = PseudoVSOXSEG4EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8765 = PseudoVSOXSEG4EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8764 = PseudoVSOXSEG4EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8763 = PseudoVSOXSEG4EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8762 = PseudoVSOXSEG4EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8761 = PseudoVSOXSEG4EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #8760 = PseudoVSOXSEG4EI16_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #8759 = PseudoVSOXSEG4EI16_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8758 = PseudoVSOXSEG4EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8757 = PseudoVSOXSEG4EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[7047] },  // Inst #8756 = PseudoVSOXSEG4EI16_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7042] },  // Inst #8755 = PseudoVSOXSEG4EI16_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[7036] },  // Inst #8754 = PseudoVSOXSEG4EI16_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7031] },  // Inst #8753 = PseudoVSOXSEG4EI16_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[7025] },  // Inst #8752 = PseudoVSOXSEG4EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[7020] },  // Inst #8751 = PseudoVSOXSEG4EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8750 = PseudoVSOXSEG4EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8749 = PseudoVSOXSEG4EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[7014] },  // Inst #8748 = PseudoVSOXSEG4EI16_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[7009] },  // Inst #8747 = PseudoVSOXSEG4EI16_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[7003] },  // Inst #8746 = PseudoVSOXSEG4EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6998] },  // Inst #8745 = PseudoVSOXSEG4EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8744 = PseudoVSOXSEG3EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8743 = PseudoVSOXSEG3EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8742 = PseudoVSOXSEG3EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8741 = PseudoVSOXSEG3EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8740 = PseudoVSOXSEG3EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8739 = PseudoVSOXSEG3EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8738 = PseudoVSOXSEG3EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8737 = PseudoVSOXSEG3EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8736 = PseudoVSOXSEG3EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8735 = PseudoVSOXSEG3EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8734 = PseudoVSOXSEG3EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8733 = PseudoVSOXSEG3EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #8732 = PseudoVSOXSEG3EI8_V_MF4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #8731 = PseudoVSOXSEG3EI8_V_MF4_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8730 = PseudoVSOXSEG3EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8729 = PseudoVSOXSEG3EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8728 = PseudoVSOXSEG3EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8727 = PseudoVSOXSEG3EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #8726 = PseudoVSOXSEG3EI8_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #8725 = PseudoVSOXSEG3EI8_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8724 = PseudoVSOXSEG3EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8723 = PseudoVSOXSEG3EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6948] },  // Inst #8722 = PseudoVSOXSEG3EI8_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6943] },  // Inst #8721 = PseudoVSOXSEG3EI8_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #8720 = PseudoVSOXSEG3EI8_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #8719 = PseudoVSOXSEG3EI8_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8718 = PseudoVSOXSEG3EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8717 = PseudoVSOXSEG3EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6981] },  // Inst #8716 = PseudoVSOXSEG3EI64_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6976] },  // Inst #8715 = PseudoVSOXSEG3EI64_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6992] },  // Inst #8714 = PseudoVSOXSEG3EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6987] },  // Inst #8713 = PseudoVSOXSEG3EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[6970] },  // Inst #8712 = PseudoVSOXSEG3EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6965] },  // Inst #8711 = PseudoVSOXSEG3EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[6959] },  // Inst #8710 = PseudoVSOXSEG3EI64_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6954] },  // Inst #8709 = PseudoVSOXSEG3EI64_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6970] },  // Inst #8708 = PseudoVSOXSEG3EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6965] },  // Inst #8707 = PseudoVSOXSEG3EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #8706 = PseudoVSOXSEG3EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #8705 = PseudoVSOXSEG3EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #8704 = PseudoVSOXSEG3EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #8703 = PseudoVSOXSEG3EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6948] },  // Inst #8702 = PseudoVSOXSEG3EI64_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6943] },  // Inst #8701 = PseudoVSOXSEG3EI64_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #8700 = PseudoVSOXSEG3EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #8699 = PseudoVSOXSEG3EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8698 = PseudoVSOXSEG3EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8697 = PseudoVSOXSEG3EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8696 = PseudoVSOXSEG3EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8695 = PseudoVSOXSEG3EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8694 = PseudoVSOXSEG3EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8693 = PseudoVSOXSEG3EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8692 = PseudoVSOXSEG3EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8691 = PseudoVSOXSEG3EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8690 = PseudoVSOXSEG3EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8689 = PseudoVSOXSEG3EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8688 = PseudoVSOXSEG3EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8687 = PseudoVSOXSEG3EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8686 = PseudoVSOXSEG3EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8685 = PseudoVSOXSEG3EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8684 = PseudoVSOXSEG3EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8683 = PseudoVSOXSEG3EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6981] },  // Inst #8682 = PseudoVSOXSEG3EI32_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6976] },  // Inst #8681 = PseudoVSOXSEG3EI32_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6959] },  // Inst #8680 = PseudoVSOXSEG3EI32_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6954] },  // Inst #8679 = PseudoVSOXSEG3EI32_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6970] },  // Inst #8678 = PseudoVSOXSEG3EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6965] },  // Inst #8677 = PseudoVSOXSEG3EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #8676 = PseudoVSOXSEG3EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #8675 = PseudoVSOXSEG3EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6948] },  // Inst #8674 = PseudoVSOXSEG3EI32_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6943] },  // Inst #8673 = PseudoVSOXSEG3EI32_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #8672 = PseudoVSOXSEG3EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #8671 = PseudoVSOXSEG3EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8670 = PseudoVSOXSEG3EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8669 = PseudoVSOXSEG3EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8668 = PseudoVSOXSEG3EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8667 = PseudoVSOXSEG3EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #8666 = PseudoVSOXSEG3EI32_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #8665 = PseudoVSOXSEG3EI32_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8664 = PseudoVSOXSEG3EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8663 = PseudoVSOXSEG3EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8662 = PseudoVSOXSEG3EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8661 = PseudoVSOXSEG3EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8660 = PseudoVSOXSEG3EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8659 = PseudoVSOXSEG3EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8658 = PseudoVSOXSEG3EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8657 = PseudoVSOXSEG3EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8656 = PseudoVSOXSEG3EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8655 = PseudoVSOXSEG3EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8654 = PseudoVSOXSEG3EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8653 = PseudoVSOXSEG3EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8652 = PseudoVSOXSEG3EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8651 = PseudoVSOXSEG3EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #8650 = PseudoVSOXSEG3EI16_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #8649 = PseudoVSOXSEG3EI16_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8648 = PseudoVSOXSEG3EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8647 = PseudoVSOXSEG3EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6959] },  // Inst #8646 = PseudoVSOXSEG3EI16_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6954] },  // Inst #8645 = PseudoVSOXSEG3EI16_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6948] },  // Inst #8644 = PseudoVSOXSEG3EI16_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6943] },  // Inst #8643 = PseudoVSOXSEG3EI16_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6937] },  // Inst #8642 = PseudoVSOXSEG3EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6932] },  // Inst #8641 = PseudoVSOXSEG3EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8640 = PseudoVSOXSEG3EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8639 = PseudoVSOXSEG3EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6926] },  // Inst #8638 = PseudoVSOXSEG3EI16_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6921] },  // Inst #8637 = PseudoVSOXSEG3EI16_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6915] },  // Inst #8636 = PseudoVSOXSEG3EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6910] },  // Inst #8635 = PseudoVSOXSEG3EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8634 = PseudoVSOXSEG2EI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8633 = PseudoVSOXSEG2EI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8632 = PseudoVSOXSEG2EI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8631 = PseudoVSOXSEG2EI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8630 = PseudoVSOXSEG2EI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8629 = PseudoVSOXSEG2EI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8628 = PseudoVSOXSEG2EI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8627 = PseudoVSOXSEG2EI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8626 = PseudoVSOXSEG2EI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8625 = PseudoVSOXSEG2EI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8624 = PseudoVSOXSEG2EI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8623 = PseudoVSOXSEG2EI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #8622 = PseudoVSOXSEG2EI8_V_MF4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #8621 = PseudoVSOXSEG2EI8_V_MF4_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8620 = PseudoVSOXSEG2EI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8619 = PseudoVSOXSEG2EI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8618 = PseudoVSOXSEG2EI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8617 = PseudoVSOXSEG2EI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6805] },  // Inst #8616 = PseudoVSOXSEG2EI8_V_MF2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6800] },  // Inst #8615 = PseudoVSOXSEG2EI8_V_MF2_M4
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #8614 = PseudoVSOXSEG2EI8_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #8613 = PseudoVSOXSEG2EI8_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8612 = PseudoVSOXSEG2EI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8611 = PseudoVSOXSEG2EI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6860] },  // Inst #8610 = PseudoVSOXSEG2EI8_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6855] },  // Inst #8609 = PseudoVSOXSEG2EI8_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6838] },  // Inst #8608 = PseudoVSOXSEG2EI8_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6833] },  // Inst #8607 = PseudoVSOXSEG2EI8_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6827] },  // Inst #8606 = PseudoVSOXSEG2EI8_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6822] },  // Inst #8605 = PseudoVSOXSEG2EI8_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6805] },  // Inst #8604 = PseudoVSOXSEG2EI8_V_M1_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6800] },  // Inst #8603 = PseudoVSOXSEG2EI8_V_M1_M4
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #8602 = PseudoVSOXSEG2EI8_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #8601 = PseudoVSOXSEG2EI8_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8600 = PseudoVSOXSEG2EI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8599 = PseudoVSOXSEG2EI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6871] },  // Inst #8598 = PseudoVSOXSEG2EI64_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6866] },  // Inst #8597 = PseudoVSOXSEG2EI64_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6893] },  // Inst #8596 = PseudoVSOXSEG2EI64_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6888] },  // Inst #8595 = PseudoVSOXSEG2EI64_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6904] },  // Inst #8594 = PseudoVSOXSEG2EI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6899] },  // Inst #8593 = PseudoVSOXSEG2EI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[6882] },  // Inst #8592 = PseudoVSOXSEG2EI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6877] },  // Inst #8591 = PseudoVSOXSEG2EI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[6860] },  // Inst #8590 = PseudoVSOXSEG2EI64_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6855] },  // Inst #8589 = PseudoVSOXSEG2EI64_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6849] },  // Inst #8588 = PseudoVSOXSEG2EI64_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6844] },  // Inst #8587 = PseudoVSOXSEG2EI64_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6882] },  // Inst #8586 = PseudoVSOXSEG2EI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6877] },  // Inst #8585 = PseudoVSOXSEG2EI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #8584 = PseudoVSOXSEG2EI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #8583 = PseudoVSOXSEG2EI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #8582 = PseudoVSOXSEG2EI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #8581 = PseudoVSOXSEG2EI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6827] },  // Inst #8580 = PseudoVSOXSEG2EI64_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6822] },  // Inst #8579 = PseudoVSOXSEG2EI64_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #8578 = PseudoVSOXSEG2EI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #8577 = PseudoVSOXSEG2EI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8576 = PseudoVSOXSEG2EI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8575 = PseudoVSOXSEG2EI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8574 = PseudoVSOXSEG2EI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8573 = PseudoVSOXSEG2EI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8572 = PseudoVSOXSEG2EI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8571 = PseudoVSOXSEG2EI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8570 = PseudoVSOXSEG2EI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8569 = PseudoVSOXSEG2EI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8568 = PseudoVSOXSEG2EI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8567 = PseudoVSOXSEG2EI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8566 = PseudoVSOXSEG2EI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8565 = PseudoVSOXSEG2EI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8564 = PseudoVSOXSEG2EI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8563 = PseudoVSOXSEG2EI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8562 = PseudoVSOXSEG2EI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8561 = PseudoVSOXSEG2EI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6871] },  // Inst #8560 = PseudoVSOXSEG2EI32_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6866] },  // Inst #8559 = PseudoVSOXSEG2EI32_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6893] },  // Inst #8558 = PseudoVSOXSEG2EI32_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6888] },  // Inst #8557 = PseudoVSOXSEG2EI32_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6860] },  // Inst #8556 = PseudoVSOXSEG2EI32_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6855] },  // Inst #8555 = PseudoVSOXSEG2EI32_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6849] },  // Inst #8554 = PseudoVSOXSEG2EI32_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6844] },  // Inst #8553 = PseudoVSOXSEG2EI32_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6882] },  // Inst #8552 = PseudoVSOXSEG2EI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6877] },  // Inst #8551 = PseudoVSOXSEG2EI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #8550 = PseudoVSOXSEG2EI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #8549 = PseudoVSOXSEG2EI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6838] },  // Inst #8548 = PseudoVSOXSEG2EI32_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6833] },  // Inst #8547 = PseudoVSOXSEG2EI32_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6827] },  // Inst #8546 = PseudoVSOXSEG2EI32_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6822] },  // Inst #8545 = PseudoVSOXSEG2EI32_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #8544 = PseudoVSOXSEG2EI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #8543 = PseudoVSOXSEG2EI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8542 = PseudoVSOXSEG2EI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8541 = PseudoVSOXSEG2EI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8540 = PseudoVSOXSEG2EI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8539 = PseudoVSOXSEG2EI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #8538 = PseudoVSOXSEG2EI32_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #8537 = PseudoVSOXSEG2EI32_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8536 = PseudoVSOXSEG2EI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8535 = PseudoVSOXSEG2EI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8534 = PseudoVSOXSEG2EI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8533 = PseudoVSOXSEG2EI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8532 = PseudoVSOXSEG2EI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8531 = PseudoVSOXSEG2EI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8530 = PseudoVSOXSEG2EI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8529 = PseudoVSOXSEG2EI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8528 = PseudoVSOXSEG2EI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8527 = PseudoVSOXSEG2EI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8526 = PseudoVSOXSEG2EI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8525 = PseudoVSOXSEG2EI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8524 = PseudoVSOXSEG2EI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8523 = PseudoVSOXSEG2EI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #8522 = PseudoVSOXSEG2EI16_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #8521 = PseudoVSOXSEG2EI16_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8520 = PseudoVSOXSEG2EI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8519 = PseudoVSOXSEG2EI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6871] },  // Inst #8518 = PseudoVSOXSEG2EI16_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6866] },  // Inst #8517 = PseudoVSOXSEG2EI16_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6860] },  // Inst #8516 = PseudoVSOXSEG2EI16_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6855] },  // Inst #8515 = PseudoVSOXSEG2EI16_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6849] },  // Inst #8514 = PseudoVSOXSEG2EI16_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6844] },  // Inst #8513 = PseudoVSOXSEG2EI16_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6838] },  // Inst #8512 = PseudoVSOXSEG2EI16_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6833] },  // Inst #8511 = PseudoVSOXSEG2EI16_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6827] },  // Inst #8510 = PseudoVSOXSEG2EI16_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6822] },  // Inst #8509 = PseudoVSOXSEG2EI16_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6816] },  // Inst #8508 = PseudoVSOXSEG2EI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6811] },  // Inst #8507 = PseudoVSOXSEG2EI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8506 = PseudoVSOXSEG2EI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8505 = PseudoVSOXSEG2EI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6805] },  // Inst #8504 = PseudoVSOXSEG2EI16_V_M1_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6800] },  // Inst #8503 = PseudoVSOXSEG2EI16_V_M1_M4
  { 6, &RISCVDescs.OperandInfo[6794] },  // Inst #8502 = PseudoVSOXSEG2EI16_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6789] },  // Inst #8501 = PseudoVSOXSEG2EI16_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6783] },  // Inst #8500 = PseudoVSOXSEG2EI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6778] },  // Inst #8499 = PseudoVSOXSEG2EI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8498 = PseudoVSOXEI8_V_MF8_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8497 = PseudoVSOXEI8_V_MF8_MF8
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8496 = PseudoVSOXEI8_V_MF8_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8495 = PseudoVSOXEI8_V_MF8_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8494 = PseudoVSOXEI8_V_MF8_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8493 = PseudoVSOXEI8_V_MF8_MF2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8492 = PseudoVSOXEI8_V_MF8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8491 = PseudoVSOXEI8_V_MF8_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8490 = PseudoVSOXEI8_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8489 = PseudoVSOXEI8_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8488 = PseudoVSOXEI8_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8487 = PseudoVSOXEI8_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #8486 = PseudoVSOXEI8_V_MF4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #8485 = PseudoVSOXEI8_V_MF4_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8484 = PseudoVSOXEI8_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8483 = PseudoVSOXEI8_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8482 = PseudoVSOXEI8_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8481 = PseudoVSOXEI8_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6629] },  // Inst #8480 = PseudoVSOXEI8_V_MF2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6624] },  // Inst #8479 = PseudoVSOXEI8_V_MF2_M4
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #8478 = PseudoVSOXEI8_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #8477 = PseudoVSOXEI8_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8476 = PseudoVSOXEI8_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8475 = PseudoVSOXEI8_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6728] },  // Inst #8474 = PseudoVSOXEI8_V_M8_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6723] },  // Inst #8473 = PseudoVSOXEI8_V_M8_M8
  { 6, &RISCVDescs.OperandInfo[6706] },  // Inst #8472 = PseudoVSOXEI8_V_M4_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6701] },  // Inst #8471 = PseudoVSOXEI8_V_M4_M8
  { 6, &RISCVDescs.OperandInfo[6695] },  // Inst #8470 = PseudoVSOXEI8_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6690] },  // Inst #8469 = PseudoVSOXEI8_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6673] },  // Inst #8468 = PseudoVSOXEI8_V_M2_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6668] },  // Inst #8467 = PseudoVSOXEI8_V_M2_M8
  { 6, &RISCVDescs.OperandInfo[6662] },  // Inst #8466 = PseudoVSOXEI8_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6657] },  // Inst #8465 = PseudoVSOXEI8_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6651] },  // Inst #8464 = PseudoVSOXEI8_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6646] },  // Inst #8463 = PseudoVSOXEI8_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6772] },  // Inst #8462 = PseudoVSOXEI8_V_M1_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6767] },  // Inst #8461 = PseudoVSOXEI8_V_M1_M8
  { 6, &RISCVDescs.OperandInfo[6629] },  // Inst #8460 = PseudoVSOXEI8_V_M1_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6624] },  // Inst #8459 = PseudoVSOXEI8_V_M1_M4
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #8458 = PseudoVSOXEI8_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #8457 = PseudoVSOXEI8_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8456 = PseudoVSOXEI8_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8455 = PseudoVSOXEI8_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6728] },  // Inst #8454 = PseudoVSOXEI64_V_M8_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6723] },  // Inst #8453 = PseudoVSOXEI64_V_M8_M8
  { 6, &RISCVDescs.OperandInfo[6717] },  // Inst #8452 = PseudoVSOXEI64_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6712] },  // Inst #8451 = PseudoVSOXEI64_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6750] },  // Inst #8450 = PseudoVSOXEI64_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6745] },  // Inst #8449 = PseudoVSOXEI64_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6761] },  // Inst #8448 = PseudoVSOXEI64_V_M8_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6756] },  // Inst #8447 = PseudoVSOXEI64_V_M8_M1
  { 6, &RISCVDescs.OperandInfo[6739] },  // Inst #8446 = PseudoVSOXEI64_V_M4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6734] },  // Inst #8445 = PseudoVSOXEI64_V_M4_MF2
  { 6, &RISCVDescs.OperandInfo[6695] },  // Inst #8444 = PseudoVSOXEI64_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6690] },  // Inst #8443 = PseudoVSOXEI64_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6684] },  // Inst #8442 = PseudoVSOXEI64_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6679] },  // Inst #8441 = PseudoVSOXEI64_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6739] },  // Inst #8440 = PseudoVSOXEI64_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6734] },  // Inst #8439 = PseudoVSOXEI64_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #8438 = PseudoVSOXEI64_V_M2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #8437 = PseudoVSOXEI64_V_M2_MF4
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #8436 = PseudoVSOXEI64_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #8435 = PseudoVSOXEI64_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6651] },  // Inst #8434 = PseudoVSOXEI64_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6646] },  // Inst #8433 = PseudoVSOXEI64_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #8432 = PseudoVSOXEI64_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #8431 = PseudoVSOXEI64_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8430 = PseudoVSOXEI64_V_M1_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8429 = PseudoVSOXEI64_V_M1_MF8
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8428 = PseudoVSOXEI64_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8427 = PseudoVSOXEI64_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8426 = PseudoVSOXEI64_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8425 = PseudoVSOXEI64_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8424 = PseudoVSOXEI64_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8423 = PseudoVSOXEI64_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8422 = PseudoVSOXEI32_V_MF2_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8421 = PseudoVSOXEI32_V_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8420 = PseudoVSOXEI32_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8419 = PseudoVSOXEI32_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8418 = PseudoVSOXEI32_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8417 = PseudoVSOXEI32_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8416 = PseudoVSOXEI32_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8415 = PseudoVSOXEI32_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6728] },  // Inst #8414 = PseudoVSOXEI32_V_M8_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6723] },  // Inst #8413 = PseudoVSOXEI32_V_M8_M8
  { 6, &RISCVDescs.OperandInfo[6717] },  // Inst #8412 = PseudoVSOXEI32_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6712] },  // Inst #8411 = PseudoVSOXEI32_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6750] },  // Inst #8410 = PseudoVSOXEI32_V_M8_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6745] },  // Inst #8409 = PseudoVSOXEI32_V_M8_M2
  { 6, &RISCVDescs.OperandInfo[6706] },  // Inst #8408 = PseudoVSOXEI32_V_M4_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6701] },  // Inst #8407 = PseudoVSOXEI32_V_M4_M8
  { 6, &RISCVDescs.OperandInfo[6695] },  // Inst #8406 = PseudoVSOXEI32_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6690] },  // Inst #8405 = PseudoVSOXEI32_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6684] },  // Inst #8404 = PseudoVSOXEI32_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6679] },  // Inst #8403 = PseudoVSOXEI32_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6739] },  // Inst #8402 = PseudoVSOXEI32_V_M4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6734] },  // Inst #8401 = PseudoVSOXEI32_V_M4_M1
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #8400 = PseudoVSOXEI32_V_M2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #8399 = PseudoVSOXEI32_V_M2_MF2
  { 6, &RISCVDescs.OperandInfo[6662] },  // Inst #8398 = PseudoVSOXEI32_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6657] },  // Inst #8397 = PseudoVSOXEI32_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6651] },  // Inst #8396 = PseudoVSOXEI32_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6646] },  // Inst #8395 = PseudoVSOXEI32_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #8394 = PseudoVSOXEI32_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #8393 = PseudoVSOXEI32_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8392 = PseudoVSOXEI32_V_M1_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8391 = PseudoVSOXEI32_V_M1_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8390 = PseudoVSOXEI32_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8389 = PseudoVSOXEI32_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #8388 = PseudoVSOXEI32_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #8387 = PseudoVSOXEI32_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8386 = PseudoVSOXEI32_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8385 = PseudoVSOXEI32_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8384 = PseudoVSOXEI16_V_MF4_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8383 = PseudoVSOXEI16_V_MF4_MF8
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8382 = PseudoVSOXEI16_V_MF4_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8381 = PseudoVSOXEI16_V_MF4_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8380 = PseudoVSOXEI16_V_MF4_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8379 = PseudoVSOXEI16_V_MF4_MF2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8378 = PseudoVSOXEI16_V_MF4_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8377 = PseudoVSOXEI16_V_MF4_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8376 = PseudoVSOXEI16_V_MF2_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8375 = PseudoVSOXEI16_V_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8374 = PseudoVSOXEI16_V_MF2_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8373 = PseudoVSOXEI16_V_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #8372 = PseudoVSOXEI16_V_MF2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #8371 = PseudoVSOXEI16_V_MF2_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8370 = PseudoVSOXEI16_V_MF2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8369 = PseudoVSOXEI16_V_MF2_M1
  { 6, &RISCVDescs.OperandInfo[6728] },  // Inst #8368 = PseudoVSOXEI16_V_M8_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6723] },  // Inst #8367 = PseudoVSOXEI16_V_M8_M8
  { 6, &RISCVDescs.OperandInfo[6717] },  // Inst #8366 = PseudoVSOXEI16_V_M8_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6712] },  // Inst #8365 = PseudoVSOXEI16_V_M8_M4
  { 6, &RISCVDescs.OperandInfo[6706] },  // Inst #8364 = PseudoVSOXEI16_V_M4_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6701] },  // Inst #8363 = PseudoVSOXEI16_V_M4_M8
  { 6, &RISCVDescs.OperandInfo[6695] },  // Inst #8362 = PseudoVSOXEI16_V_M4_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6690] },  // Inst #8361 = PseudoVSOXEI16_V_M4_M4
  { 6, &RISCVDescs.OperandInfo[6684] },  // Inst #8360 = PseudoVSOXEI16_V_M4_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6679] },  // Inst #8359 = PseudoVSOXEI16_V_M4_M2
  { 6, &RISCVDescs.OperandInfo[6673] },  // Inst #8358 = PseudoVSOXEI16_V_M2_M8_MASK
  { 5, &RISCVDescs.OperandInfo[6668] },  // Inst #8357 = PseudoVSOXEI16_V_M2_M8
  { 6, &RISCVDescs.OperandInfo[6662] },  // Inst #8356 = PseudoVSOXEI16_V_M2_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6657] },  // Inst #8355 = PseudoVSOXEI16_V_M2_M4
  { 6, &RISCVDescs.OperandInfo[6651] },  // Inst #8354 = PseudoVSOXEI16_V_M2_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6646] },  // Inst #8353 = PseudoVSOXEI16_V_M2_M2
  { 6, &RISCVDescs.OperandInfo[6640] },  // Inst #8352 = PseudoVSOXEI16_V_M2_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6635] },  // Inst #8351 = PseudoVSOXEI16_V_M2_M1
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8350 = PseudoVSOXEI16_V_M1_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8349 = PseudoVSOXEI16_V_M1_MF2
  { 6, &RISCVDescs.OperandInfo[6629] },  // Inst #8348 = PseudoVSOXEI16_V_M1_M4_MASK
  { 5, &RISCVDescs.OperandInfo[6624] },  // Inst #8347 = PseudoVSOXEI16_V_M1_M4
  { 6, &RISCVDescs.OperandInfo[6618] },  // Inst #8346 = PseudoVSOXEI16_V_M1_M2_MASK
  { 5, &RISCVDescs.OperandInfo[6613] },  // Inst #8345 = PseudoVSOXEI16_V_M1_M2
  { 6, &RISCVDescs.OperandInfo[6607] },  // Inst #8344 = PseudoVSOXEI16_V_M1_M1_MASK
  { 5, &RISCVDescs.OperandInfo[6602] },  // Inst #8343 = PseudoVSOXEI16_V_M1_M1
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8342 = PseudoVSM_V_B8
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8341 = PseudoVSM_V_B64
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8340 = PseudoVSM_V_B4
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8339 = PseudoVSM_V_B32
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8338 = PseudoVSM_V_B2
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8337 = PseudoVSM_V_B16
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8336 = PseudoVSM_V_B1
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #8335 = PseudoVSMUL_VX_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #8334 = PseudoVSMUL_VX_MF8
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #8333 = PseudoVSMUL_VX_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #8332 = PseudoVSMUL_VX_MF4
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #8331 = PseudoVSMUL_VX_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #8330 = PseudoVSMUL_VX_MF2
  { 9, &RISCVDescs.OperandInfo[544] },  // Inst #8329 = PseudoVSMUL_VX_M8_MASK
  { 8, &RISCVDescs.OperandInfo[536] },  // Inst #8328 = PseudoVSMUL_VX_M8
  { 9, &RISCVDescs.OperandInfo[527] },  // Inst #8327 = PseudoVSMUL_VX_M4_MASK
  { 8, &RISCVDescs.OperandInfo[519] },  // Inst #8326 = PseudoVSMUL_VX_M4
  { 9, &RISCVDescs.OperandInfo[510] },  // Inst #8325 = PseudoVSMUL_VX_M2_MASK
  { 8, &RISCVDescs.OperandInfo[502] },  // Inst #8324 = PseudoVSMUL_VX_M2
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #8323 = PseudoVSMUL_VX_M1_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #8322 = PseudoVSMUL_VX_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #8321 = PseudoVSMUL_VV_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #8320 = PseudoVSMUL_VV_MF8
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #8319 = PseudoVSMUL_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #8318 = PseudoVSMUL_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #8317 = PseudoVSMUL_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #8316 = PseudoVSMUL_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #8315 = PseudoVSMUL_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #8314 = PseudoVSMUL_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #8313 = PseudoVSMUL_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #8312 = PseudoVSMUL_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #8311 = PseudoVSMUL_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #8310 = PseudoVSMUL_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #8309 = PseudoVSMUL_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #8308 = PseudoVSMUL_VV_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #8307 = PseudoVSM4R_VV_MF2
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #8306 = PseudoVSM4R_VV_M8
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #8305 = PseudoVSM4R_VV_M4
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #8304 = PseudoVSM4R_VV_M2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #8303 = PseudoVSM4R_VV_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #8302 = PseudoVSM4R_VS_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #8301 = PseudoVSM4R_VS_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #8300 = PseudoVSM4R_VS_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #8299 = PseudoVSM4R_VS_M8_MF8
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #8298 = PseudoVSM4R_VS_M8_MF4
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #8297 = PseudoVSM4R_VS_M8_MF2
  { 6, &RISCVDescs.OperandInfo[865] },  // Inst #8296 = PseudoVSM4R_VS_M8_M4
  { 6, &RISCVDescs.OperandInfo[859] },  // Inst #8295 = PseudoVSM4R_VS_M8_M2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #8294 = PseudoVSM4R_VS_M8_M1
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #8293 = PseudoVSM4R_VS_M4_MF8
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #8292 = PseudoVSM4R_VS_M4_MF4
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #8291 = PseudoVSM4R_VS_M4_MF2
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #8290 = PseudoVSM4R_VS_M4_M4
  { 6, &RISCVDescs.OperandInfo[841] },  // Inst #8289 = PseudoVSM4R_VS_M4_M2
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #8288 = PseudoVSM4R_VS_M4_M1
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #8287 = PseudoVSM4R_VS_M2_MF8
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #8286 = PseudoVSM4R_VS_M2_MF4
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #8285 = PseudoVSM4R_VS_M2_MF2
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #8284 = PseudoVSM4R_VS_M2_M2
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #8283 = PseudoVSM4R_VS_M2_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #8282 = PseudoVSM4R_VS_M1_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #8281 = PseudoVSM4R_VS_M1_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #8280 = PseudoVSM4R_VS_M1_MF2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #8279 = PseudoVSM4R_VS_M1_M1
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8278 = PseudoVSM4K_VI_MF2
  { 7, &RISCVDescs.OperandInfo[898] },  // Inst #8277 = PseudoVSM4K_VI_M8
  { 7, &RISCVDescs.OperandInfo[891] },  // Inst #8276 = PseudoVSM4K_VI_M4
  { 7, &RISCVDescs.OperandInfo[884] },  // Inst #8275 = PseudoVSM4K_VI_M2
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8274 = PseudoVSM4K_VI_M1
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8273 = PseudoVSM3ME_VV_MF2
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #8272 = PseudoVSM3ME_VV_M8
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #8271 = PseudoVSM3ME_VV_M4
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #8270 = PseudoVSM3ME_VV_M2
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8269 = PseudoVSM3ME_VV_M1
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8268 = PseudoVSM3C_VI_MF2
  { 7, &RISCVDescs.OperandInfo[898] },  // Inst #8267 = PseudoVSM3C_VI_M8
  { 7, &RISCVDescs.OperandInfo[891] },  // Inst #8266 = PseudoVSM3C_VI_M4
  { 7, &RISCVDescs.OperandInfo[884] },  // Inst #8265 = PseudoVSM3C_VI_M2
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8264 = PseudoVSM3C_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8263 = PseudoVSLL_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8262 = PseudoVSLL_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8261 = PseudoVSLL_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8260 = PseudoVSLL_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8259 = PseudoVSLL_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8258 = PseudoVSLL_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #8257 = PseudoVSLL_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #8256 = PseudoVSLL_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #8255 = PseudoVSLL_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #8254 = PseudoVSLL_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #8253 = PseudoVSLL_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #8252 = PseudoVSLL_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8251 = PseudoVSLL_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8250 = PseudoVSLL_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #8249 = PseudoVSLL_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8248 = PseudoVSLL_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #8247 = PseudoVSLL_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8246 = PseudoVSLL_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #8245 = PseudoVSLL_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8244 = PseudoVSLL_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #8243 = PseudoVSLL_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #8242 = PseudoVSLL_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #8241 = PseudoVSLL_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #8240 = PseudoVSLL_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #8239 = PseudoVSLL_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #8238 = PseudoVSLL_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #8237 = PseudoVSLL_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8236 = PseudoVSLL_VV_M1
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #8235 = PseudoVSLL_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8234 = PseudoVSLL_VI_MF8
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #8233 = PseudoVSLL_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8232 = PseudoVSLL_VI_MF4
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #8231 = PseudoVSLL_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8230 = PseudoVSLL_VI_MF2
  { 8, &RISCVDescs.OperandInfo[6594] },  // Inst #8229 = PseudoVSLL_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[898] },  // Inst #8228 = PseudoVSLL_VI_M8
  { 8, &RISCVDescs.OperandInfo[6586] },  // Inst #8227 = PseudoVSLL_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[891] },  // Inst #8226 = PseudoVSLL_VI_M4
  { 8, &RISCVDescs.OperandInfo[6578] },  // Inst #8225 = PseudoVSLL_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[884] },  // Inst #8224 = PseudoVSLL_VI_M2
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #8223 = PseudoVSLL_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8222 = PseudoVSLL_VI_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #8221 = PseudoVSLIDEUP_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #8220 = PseudoVSLIDEUP_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #8219 = PseudoVSLIDEUP_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #8218 = PseudoVSLIDEUP_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #8217 = PseudoVSLIDEUP_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #8216 = PseudoVSLIDEUP_VX_MF2
  { 8, &RISCVDescs.OperandInfo[6452] },  // Inst #8215 = PseudoVSLIDEUP_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6445] },  // Inst #8214 = PseudoVSLIDEUP_VX_M8
  { 8, &RISCVDescs.OperandInfo[6437] },  // Inst #8213 = PseudoVSLIDEUP_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6430] },  // Inst #8212 = PseudoVSLIDEUP_VX_M4
  { 8, &RISCVDescs.OperandInfo[6422] },  // Inst #8211 = PseudoVSLIDEUP_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6415] },  // Inst #8210 = PseudoVSLIDEUP_VX_M2
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #8209 = PseudoVSLIDEUP_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #8208 = PseudoVSLIDEUP_VX_M1
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #8207 = PseudoVSLIDEUP_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #8206 = PseudoVSLIDEUP_VI_MF8
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #8205 = PseudoVSLIDEUP_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #8204 = PseudoVSLIDEUP_VI_MF4
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #8203 = PseudoVSLIDEUP_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #8202 = PseudoVSLIDEUP_VI_MF2
  { 8, &RISCVDescs.OperandInfo[6392] },  // Inst #8201 = PseudoVSLIDEUP_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6385] },  // Inst #8200 = PseudoVSLIDEUP_VI_M8
  { 8, &RISCVDescs.OperandInfo[6377] },  // Inst #8199 = PseudoVSLIDEUP_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6370] },  // Inst #8198 = PseudoVSLIDEUP_VI_M4
  { 8, &RISCVDescs.OperandInfo[6362] },  // Inst #8197 = PseudoVSLIDEUP_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6355] },  // Inst #8196 = PseudoVSLIDEUP_VI_M2
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #8195 = PseudoVSLIDEUP_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #8194 = PseudoVSLIDEUP_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8193 = PseudoVSLIDEDOWN_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8192 = PseudoVSLIDEDOWN_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8191 = PseudoVSLIDEDOWN_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8190 = PseudoVSLIDEDOWN_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8189 = PseudoVSLIDEDOWN_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8188 = PseudoVSLIDEDOWN_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #8187 = PseudoVSLIDEDOWN_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #8186 = PseudoVSLIDEDOWN_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #8185 = PseudoVSLIDEDOWN_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #8184 = PseudoVSLIDEDOWN_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #8183 = PseudoVSLIDEDOWN_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #8182 = PseudoVSLIDEDOWN_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8181 = PseudoVSLIDEDOWN_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8180 = PseudoVSLIDEDOWN_VX_M1
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #8179 = PseudoVSLIDEDOWN_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8178 = PseudoVSLIDEDOWN_VI_MF8
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #8177 = PseudoVSLIDEDOWN_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8176 = PseudoVSLIDEDOWN_VI_MF4
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #8175 = PseudoVSLIDEDOWN_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8174 = PseudoVSLIDEDOWN_VI_MF2
  { 8, &RISCVDescs.OperandInfo[6594] },  // Inst #8173 = PseudoVSLIDEDOWN_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[898] },  // Inst #8172 = PseudoVSLIDEDOWN_VI_M8
  { 8, &RISCVDescs.OperandInfo[6586] },  // Inst #8171 = PseudoVSLIDEDOWN_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[891] },  // Inst #8170 = PseudoVSLIDEDOWN_VI_M4
  { 8, &RISCVDescs.OperandInfo[6578] },  // Inst #8169 = PseudoVSLIDEDOWN_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[884] },  // Inst #8168 = PseudoVSLIDEDOWN_VI_M2
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #8167 = PseudoVSLIDEDOWN_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #8166 = PseudoVSLIDEDOWN_VI_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #8165 = PseudoVSLIDE1UP_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #8164 = PseudoVSLIDE1UP_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #8163 = PseudoVSLIDE1UP_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #8162 = PseudoVSLIDE1UP_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #8161 = PseudoVSLIDE1UP_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #8160 = PseudoVSLIDE1UP_VX_MF2
  { 8, &RISCVDescs.OperandInfo[6452] },  // Inst #8159 = PseudoVSLIDE1UP_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6445] },  // Inst #8158 = PseudoVSLIDE1UP_VX_M8
  { 8, &RISCVDescs.OperandInfo[6437] },  // Inst #8157 = PseudoVSLIDE1UP_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6430] },  // Inst #8156 = PseudoVSLIDE1UP_VX_M4
  { 8, &RISCVDescs.OperandInfo[6422] },  // Inst #8155 = PseudoVSLIDE1UP_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6415] },  // Inst #8154 = PseudoVSLIDE1UP_VX_M2
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #8153 = PseudoVSLIDE1UP_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #8152 = PseudoVSLIDE1UP_VX_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8151 = PseudoVSLIDE1DOWN_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8150 = PseudoVSLIDE1DOWN_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8149 = PseudoVSLIDE1DOWN_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8148 = PseudoVSLIDE1DOWN_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8147 = PseudoVSLIDE1DOWN_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8146 = PseudoVSLIDE1DOWN_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #8145 = PseudoVSLIDE1DOWN_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #8144 = PseudoVSLIDE1DOWN_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #8143 = PseudoVSLIDE1DOWN_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #8142 = PseudoVSLIDE1DOWN_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #8141 = PseudoVSLIDE1DOWN_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #8140 = PseudoVSLIDE1DOWN_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8139 = PseudoVSLIDE1DOWN_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8138 = PseudoVSLIDE1DOWN_VX_M1
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8137 = PseudoVSHA2MS_VV_MF2
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #8136 = PseudoVSHA2MS_VV_M8
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #8135 = PseudoVSHA2MS_VV_M4
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #8134 = PseudoVSHA2MS_VV_M2
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8133 = PseudoVSHA2MS_VV_M1
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8132 = PseudoVSHA2CL_VV_MF2
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #8131 = PseudoVSHA2CL_VV_M8
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #8130 = PseudoVSHA2CL_VV_M4
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #8129 = PseudoVSHA2CL_VV_M2
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8128 = PseudoVSHA2CL_VV_M1
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8127 = PseudoVSHA2CH_VV_MF2
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #8126 = PseudoVSHA2CH_VV_M8
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #8125 = PseudoVSHA2CH_VV_M4
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #8124 = PseudoVSHA2CH_VV_M2
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8123 = PseudoVSHA2CH_VV_M1
  { 7, &RISCVDescs.OperandInfo[3787] },  // Inst #8122 = PseudoVSEXT_VF8_M8_MASK
  { 6, &RISCVDescs.OperandInfo[3781] },  // Inst #8121 = PseudoVSEXT_VF8_M8
  { 7, &RISCVDescs.OperandInfo[3774] },  // Inst #8120 = PseudoVSEXT_VF8_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3768] },  // Inst #8119 = PseudoVSEXT_VF8_M4
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #8118 = PseudoVSEXT_VF8_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #8117 = PseudoVSEXT_VF8_M2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #8116 = PseudoVSEXT_VF8_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #8115 = PseudoVSEXT_VF8_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #8114 = PseudoVSEXT_VF4_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #8113 = PseudoVSEXT_VF4_MF2
  { 7, &RISCVDescs.OperandInfo[6571] },  // Inst #8112 = PseudoVSEXT_VF4_M8_MASK
  { 6, &RISCVDescs.OperandInfo[6565] },  // Inst #8111 = PseudoVSEXT_VF4_M8
  { 7, &RISCVDescs.OperandInfo[3774] },  // Inst #8110 = PseudoVSEXT_VF4_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3768] },  // Inst #8109 = PseudoVSEXT_VF4_M4
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #8108 = PseudoVSEXT_VF4_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #8107 = PseudoVSEXT_VF4_M2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #8106 = PseudoVSEXT_VF4_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #8105 = PseudoVSEXT_VF4_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #8104 = PseudoVSEXT_VF2_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #8103 = PseudoVSEXT_VF2_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #8102 = PseudoVSEXT_VF2_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #8101 = PseudoVSEXT_VF2_MF2
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #8100 = PseudoVSEXT_VF2_M8_MASK
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #8099 = PseudoVSEXT_VF2_M8
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #8098 = PseudoVSEXT_VF2_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #8097 = PseudoVSEXT_VF2_M4
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #8096 = PseudoVSEXT_VF2_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #8095 = PseudoVSEXT_VF2_M2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #8094 = PseudoVSEXT_VF2_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #8093 = PseudoVSEXT_VF2_M1
  { 3, &RISCVDescs.OperandInfo[6562] },  // Inst #8092 = PseudoVSETVLIX0
  { 3, &RISCVDescs.OperandInfo[6559] },  // Inst #8091 = PseudoVSETVLI
  { 3, &RISCVDescs.OperandInfo[6556] },  // Inst #8090 = PseudoVSETIVLI
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8089 = PseudoVSE8_V_MF8_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8088 = PseudoVSE8_V_MF8
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8087 = PseudoVSE8_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8086 = PseudoVSE8_V_MF4
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8085 = PseudoVSE8_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8084 = PseudoVSE8_V_MF2
  { 5, &RISCVDescs.OperandInfo[6551] },  // Inst #8083 = PseudoVSE8_V_M8_MASK
  { 4, &RISCVDescs.OperandInfo[6547] },  // Inst #8082 = PseudoVSE8_V_M8
  { 5, &RISCVDescs.OperandInfo[6542] },  // Inst #8081 = PseudoVSE8_V_M4_MASK
  { 4, &RISCVDescs.OperandInfo[6538] },  // Inst #8080 = PseudoVSE8_V_M4
  { 5, &RISCVDescs.OperandInfo[6533] },  // Inst #8079 = PseudoVSE8_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[6529] },  // Inst #8078 = PseudoVSE8_V_M2
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8077 = PseudoVSE8_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8076 = PseudoVSE8_V_M1
  { 5, &RISCVDescs.OperandInfo[6551] },  // Inst #8075 = PseudoVSE64_V_M8_MASK
  { 4, &RISCVDescs.OperandInfo[6547] },  // Inst #8074 = PseudoVSE64_V_M8
  { 5, &RISCVDescs.OperandInfo[6542] },  // Inst #8073 = PseudoVSE64_V_M4_MASK
  { 4, &RISCVDescs.OperandInfo[6538] },  // Inst #8072 = PseudoVSE64_V_M4
  { 5, &RISCVDescs.OperandInfo[6533] },  // Inst #8071 = PseudoVSE64_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[6529] },  // Inst #8070 = PseudoVSE64_V_M2
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8069 = PseudoVSE64_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8068 = PseudoVSE64_V_M1
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8067 = PseudoVSE32_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8066 = PseudoVSE32_V_MF2
  { 5, &RISCVDescs.OperandInfo[6551] },  // Inst #8065 = PseudoVSE32_V_M8_MASK
  { 4, &RISCVDescs.OperandInfo[6547] },  // Inst #8064 = PseudoVSE32_V_M8
  { 5, &RISCVDescs.OperandInfo[6542] },  // Inst #8063 = PseudoVSE32_V_M4_MASK
  { 4, &RISCVDescs.OperandInfo[6538] },  // Inst #8062 = PseudoVSE32_V_M4
  { 5, &RISCVDescs.OperandInfo[6533] },  // Inst #8061 = PseudoVSE32_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[6529] },  // Inst #8060 = PseudoVSE32_V_M2
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8059 = PseudoVSE32_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8058 = PseudoVSE32_V_M1
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8057 = PseudoVSE16_V_MF4_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8056 = PseudoVSE16_V_MF4
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8055 = PseudoVSE16_V_MF2_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8054 = PseudoVSE16_V_MF2
  { 5, &RISCVDescs.OperandInfo[6551] },  // Inst #8053 = PseudoVSE16_V_M8_MASK
  { 4, &RISCVDescs.OperandInfo[6547] },  // Inst #8052 = PseudoVSE16_V_M8
  { 5, &RISCVDescs.OperandInfo[6542] },  // Inst #8051 = PseudoVSE16_V_M4_MASK
  { 4, &RISCVDescs.OperandInfo[6538] },  // Inst #8050 = PseudoVSE16_V_M4
  { 5, &RISCVDescs.OperandInfo[6533] },  // Inst #8049 = PseudoVSE16_V_M2_MASK
  { 4, &RISCVDescs.OperandInfo[6529] },  // Inst #8048 = PseudoVSE16_V_M2
  { 5, &RISCVDescs.OperandInfo[6524] },  // Inst #8047 = PseudoVSE16_V_M1_MASK
  { 4, &RISCVDescs.OperandInfo[6520] },  // Inst #8046 = PseudoVSE16_V_M1
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #8045 = PseudoVSBC_VXM_MF8
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #8044 = PseudoVSBC_VXM_MF4
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #8043 = PseudoVSBC_VXM_MF2
  { 7, &RISCVDescs.OperandInfo[630] },  // Inst #8042 = PseudoVSBC_VXM_M8
  { 7, &RISCVDescs.OperandInfo[623] },  // Inst #8041 = PseudoVSBC_VXM_M4
  { 7, &RISCVDescs.OperandInfo[616] },  // Inst #8040 = PseudoVSBC_VXM_M2
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #8039 = PseudoVSBC_VXM_M1
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #8038 = PseudoVSBC_VVM_MF8
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #8037 = PseudoVSBC_VVM_MF4
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #8036 = PseudoVSBC_VVM_MF2
  { 7, &RISCVDescs.OperandInfo[602] },  // Inst #8035 = PseudoVSBC_VVM_M8
  { 7, &RISCVDescs.OperandInfo[595] },  // Inst #8034 = PseudoVSBC_VVM_M4
  { 7, &RISCVDescs.OperandInfo[588] },  // Inst #8033 = PseudoVSBC_VVM_M2
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #8032 = PseudoVSBC_VVM_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8031 = PseudoVSADD_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8030 = PseudoVSADD_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8029 = PseudoVSADD_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8028 = PseudoVSADD_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8027 = PseudoVSADD_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8026 = PseudoVSADD_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #8025 = PseudoVSADD_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #8024 = PseudoVSADD_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #8023 = PseudoVSADD_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #8022 = PseudoVSADD_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #8021 = PseudoVSADD_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #8020 = PseudoVSADD_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #8019 = PseudoVSADD_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #8018 = PseudoVSADD_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #8017 = PseudoVSADD_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8016 = PseudoVSADD_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #8015 = PseudoVSADD_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8014 = PseudoVSADD_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #8013 = PseudoVSADD_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8012 = PseudoVSADD_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #8011 = PseudoVSADD_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #8010 = PseudoVSADD_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #8009 = PseudoVSADD_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #8008 = PseudoVSADD_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #8007 = PseudoVSADD_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #8006 = PseudoVSADD_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #8005 = PseudoVSADD_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #8004 = PseudoVSADD_VV_M1
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #8003 = PseudoVSADD_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #8002 = PseudoVSADD_VI_MF8
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #8001 = PseudoVSADD_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #8000 = PseudoVSADD_VI_MF4
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7999 = PseudoVSADD_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7998 = PseudoVSADD_VI_MF2
  { 8, &RISCVDescs.OperandInfo[689] },  // Inst #7997 = PseudoVSADD_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[682] },  // Inst #7996 = PseudoVSADD_VI_M8
  { 8, &RISCVDescs.OperandInfo[674] },  // Inst #7995 = PseudoVSADD_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[667] },  // Inst #7994 = PseudoVSADD_VI_M4
  { 8, &RISCVDescs.OperandInfo[659] },  // Inst #7993 = PseudoVSADD_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[652] },  // Inst #7992 = PseudoVSADD_VI_M2
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7991 = PseudoVSADD_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7990 = PseudoVSADD_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7989 = PseudoVSADDU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7988 = PseudoVSADDU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7987 = PseudoVSADDU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7986 = PseudoVSADDU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7985 = PseudoVSADDU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7984 = PseudoVSADDU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7983 = PseudoVSADDU_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7982 = PseudoVSADDU_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7981 = PseudoVSADDU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7980 = PseudoVSADDU_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7979 = PseudoVSADDU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7978 = PseudoVSADDU_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7977 = PseudoVSADDU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7976 = PseudoVSADDU_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7975 = PseudoVSADDU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7974 = PseudoVSADDU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7973 = PseudoVSADDU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7972 = PseudoVSADDU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7971 = PseudoVSADDU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7970 = PseudoVSADDU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7969 = PseudoVSADDU_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7968 = PseudoVSADDU_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7967 = PseudoVSADDU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7966 = PseudoVSADDU_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7965 = PseudoVSADDU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7964 = PseudoVSADDU_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7963 = PseudoVSADDU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7962 = PseudoVSADDU_VV_M1
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7961 = PseudoVSADDU_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7960 = PseudoVSADDU_VI_MF8
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7959 = PseudoVSADDU_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7958 = PseudoVSADDU_VI_MF4
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7957 = PseudoVSADDU_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7956 = PseudoVSADDU_VI_MF2
  { 8, &RISCVDescs.OperandInfo[689] },  // Inst #7955 = PseudoVSADDU_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[682] },  // Inst #7954 = PseudoVSADDU_VI_M8
  { 8, &RISCVDescs.OperandInfo[674] },  // Inst #7953 = PseudoVSADDU_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[667] },  // Inst #7952 = PseudoVSADDU_VI_M4
  { 8, &RISCVDescs.OperandInfo[659] },  // Inst #7951 = PseudoVSADDU_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[652] },  // Inst #7950 = PseudoVSADDU_VI_M2
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7949 = PseudoVSADDU_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7948 = PseudoVSADDU_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7947 = PseudoVRSUB_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7946 = PseudoVRSUB_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7945 = PseudoVRSUB_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7944 = PseudoVRSUB_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7943 = PseudoVRSUB_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7942 = PseudoVRSUB_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7941 = PseudoVRSUB_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7940 = PseudoVRSUB_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7939 = PseudoVRSUB_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7938 = PseudoVRSUB_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7937 = PseudoVRSUB_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7936 = PseudoVRSUB_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7935 = PseudoVRSUB_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7934 = PseudoVRSUB_VX_M1
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7933 = PseudoVRSUB_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7932 = PseudoVRSUB_VI_MF8
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7931 = PseudoVRSUB_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7930 = PseudoVRSUB_VI_MF4
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7929 = PseudoVRSUB_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7928 = PseudoVRSUB_VI_MF2
  { 8, &RISCVDescs.OperandInfo[689] },  // Inst #7927 = PseudoVRSUB_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[682] },  // Inst #7926 = PseudoVRSUB_VI_M8
  { 8, &RISCVDescs.OperandInfo[674] },  // Inst #7925 = PseudoVRSUB_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[667] },  // Inst #7924 = PseudoVRSUB_VI_M4
  { 8, &RISCVDescs.OperandInfo[659] },  // Inst #7923 = PseudoVRSUB_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[652] },  // Inst #7922 = PseudoVRSUB_VI_M2
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #7921 = PseudoVRSUB_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #7920 = PseudoVRSUB_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7919 = PseudoVROR_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7918 = PseudoVROR_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7917 = PseudoVROR_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7916 = PseudoVROR_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7915 = PseudoVROR_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7914 = PseudoVROR_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7913 = PseudoVROR_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7912 = PseudoVROR_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7911 = PseudoVROR_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7910 = PseudoVROR_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7909 = PseudoVROR_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7908 = PseudoVROR_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7907 = PseudoVROR_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7906 = PseudoVROR_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7905 = PseudoVROR_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7904 = PseudoVROR_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7903 = PseudoVROR_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7902 = PseudoVROR_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7901 = PseudoVROR_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7900 = PseudoVROR_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7899 = PseudoVROR_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7898 = PseudoVROR_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7897 = PseudoVROR_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7896 = PseudoVROR_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7895 = PseudoVROR_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7894 = PseudoVROR_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7893 = PseudoVROR_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7892 = PseudoVROR_VV_M1
  { 8, &RISCVDescs.OperandInfo[6467] },  // Inst #7891 = PseudoVROR_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6460] },  // Inst #7890 = PseudoVROR_VI_MF8
  { 8, &RISCVDescs.OperandInfo[6467] },  // Inst #7889 = PseudoVROR_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6460] },  // Inst #7888 = PseudoVROR_VI_MF4
  { 8, &RISCVDescs.OperandInfo[6467] },  // Inst #7887 = PseudoVROR_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6460] },  // Inst #7886 = PseudoVROR_VI_MF2
  { 8, &RISCVDescs.OperandInfo[6512] },  // Inst #7885 = PseudoVROR_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6505] },  // Inst #7884 = PseudoVROR_VI_M8
  { 8, &RISCVDescs.OperandInfo[6497] },  // Inst #7883 = PseudoVROR_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6490] },  // Inst #7882 = PseudoVROR_VI_M4
  { 8, &RISCVDescs.OperandInfo[6482] },  // Inst #7881 = PseudoVROR_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6475] },  // Inst #7880 = PseudoVROR_VI_M2
  { 8, &RISCVDescs.OperandInfo[6467] },  // Inst #7879 = PseudoVROR_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6460] },  // Inst #7878 = PseudoVROR_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7877 = PseudoVROL_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7876 = PseudoVROL_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7875 = PseudoVROL_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7874 = PseudoVROL_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7873 = PseudoVROL_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7872 = PseudoVROL_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7871 = PseudoVROL_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7870 = PseudoVROL_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7869 = PseudoVROL_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7868 = PseudoVROL_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7867 = PseudoVROL_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7866 = PseudoVROL_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7865 = PseudoVROL_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7864 = PseudoVROL_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7863 = PseudoVROL_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7862 = PseudoVROL_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7861 = PseudoVROL_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7860 = PseudoVROL_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7859 = PseudoVROL_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7858 = PseudoVROL_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7857 = PseudoVROL_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7856 = PseudoVROL_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7855 = PseudoVROL_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7854 = PseudoVROL_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7853 = PseudoVROL_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7852 = PseudoVROL_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7851 = PseudoVROL_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7850 = PseudoVROL_VV_M1
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #7849 = PseudoVRGATHER_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #7848 = PseudoVRGATHER_VX_MF8
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #7847 = PseudoVRGATHER_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #7846 = PseudoVRGATHER_VX_MF4
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #7845 = PseudoVRGATHER_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #7844 = PseudoVRGATHER_VX_MF2
  { 8, &RISCVDescs.OperandInfo[6452] },  // Inst #7843 = PseudoVRGATHER_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6445] },  // Inst #7842 = PseudoVRGATHER_VX_M8
  { 8, &RISCVDescs.OperandInfo[6437] },  // Inst #7841 = PseudoVRGATHER_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6430] },  // Inst #7840 = PseudoVRGATHER_VX_M4
  { 8, &RISCVDescs.OperandInfo[6422] },  // Inst #7839 = PseudoVRGATHER_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6415] },  // Inst #7838 = PseudoVRGATHER_VX_M2
  { 8, &RISCVDescs.OperandInfo[6407] },  // Inst #7837 = PseudoVRGATHER_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6400] },  // Inst #7836 = PseudoVRGATHER_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7835 = PseudoVRGATHER_VV_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7834 = PseudoVRGATHER_VV_MF8_E8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7833 = PseudoVRGATHER_VV_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7832 = PseudoVRGATHER_VV_MF4_E8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7831 = PseudoVRGATHER_VV_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7830 = PseudoVRGATHER_VV_MF4_E16
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7829 = PseudoVRGATHER_VV_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7828 = PseudoVRGATHER_VV_MF2_E8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7827 = PseudoVRGATHER_VV_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7826 = PseudoVRGATHER_VV_MF2_E32
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7825 = PseudoVRGATHER_VV_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7824 = PseudoVRGATHER_VV_MF2_E16
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #7823 = PseudoVRGATHER_VV_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #7822 = PseudoVRGATHER_VV_M8_E8
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #7821 = PseudoVRGATHER_VV_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #7820 = PseudoVRGATHER_VV_M8_E64
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #7819 = PseudoVRGATHER_VV_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #7818 = PseudoVRGATHER_VV_M8_E32
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #7817 = PseudoVRGATHER_VV_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #7816 = PseudoVRGATHER_VV_M8_E16
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #7815 = PseudoVRGATHER_VV_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #7814 = PseudoVRGATHER_VV_M4_E8
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #7813 = PseudoVRGATHER_VV_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #7812 = PseudoVRGATHER_VV_M4_E64
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #7811 = PseudoVRGATHER_VV_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #7810 = PseudoVRGATHER_VV_M4_E32
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #7809 = PseudoVRGATHER_VV_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #7808 = PseudoVRGATHER_VV_M4_E16
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #7807 = PseudoVRGATHER_VV_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #7806 = PseudoVRGATHER_VV_M2_E8
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #7805 = PseudoVRGATHER_VV_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #7804 = PseudoVRGATHER_VV_M2_E64
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #7803 = PseudoVRGATHER_VV_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #7802 = PseudoVRGATHER_VV_M2_E32
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #7801 = PseudoVRGATHER_VV_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #7800 = PseudoVRGATHER_VV_M2_E16
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7799 = PseudoVRGATHER_VV_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7798 = PseudoVRGATHER_VV_M1_E8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7797 = PseudoVRGATHER_VV_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7796 = PseudoVRGATHER_VV_M1_E64
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7795 = PseudoVRGATHER_VV_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7794 = PseudoVRGATHER_VV_M1_E32
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7793 = PseudoVRGATHER_VV_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7792 = PseudoVRGATHER_VV_M1_E16
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #7791 = PseudoVRGATHER_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #7790 = PseudoVRGATHER_VI_MF8
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #7789 = PseudoVRGATHER_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #7788 = PseudoVRGATHER_VI_MF4
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #7787 = PseudoVRGATHER_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #7786 = PseudoVRGATHER_VI_MF2
  { 8, &RISCVDescs.OperandInfo[6392] },  // Inst #7785 = PseudoVRGATHER_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6385] },  // Inst #7784 = PseudoVRGATHER_VI_M8
  { 8, &RISCVDescs.OperandInfo[6377] },  // Inst #7783 = PseudoVRGATHER_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6370] },  // Inst #7782 = PseudoVRGATHER_VI_M4
  { 8, &RISCVDescs.OperandInfo[6362] },  // Inst #7781 = PseudoVRGATHER_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6355] },  // Inst #7780 = PseudoVRGATHER_VI_M2
  { 8, &RISCVDescs.OperandInfo[6347] },  // Inst #7779 = PseudoVRGATHER_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6340] },  // Inst #7778 = PseudoVRGATHER_VI_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7777 = PseudoVRGATHEREI16_VV_MF8_E8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7776 = PseudoVRGATHEREI16_VV_MF8_E8_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7775 = PseudoVRGATHEREI16_VV_MF8_E8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7774 = PseudoVRGATHEREI16_VV_MF8_E8_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7773 = PseudoVRGATHEREI16_VV_MF4_E8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7772 = PseudoVRGATHEREI16_VV_MF4_E8_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7771 = PseudoVRGATHEREI16_VV_MF4_E8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7770 = PseudoVRGATHEREI16_VV_MF4_E8_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7769 = PseudoVRGATHEREI16_VV_MF4_E8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7768 = PseudoVRGATHEREI16_VV_MF4_E8_MF2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7767 = PseudoVRGATHEREI16_VV_MF4_E16_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7766 = PseudoVRGATHEREI16_VV_MF4_E16_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7765 = PseudoVRGATHEREI16_VV_MF4_E16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7764 = PseudoVRGATHEREI16_VV_MF4_E16_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7763 = PseudoVRGATHEREI16_VV_MF4_E16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7762 = PseudoVRGATHEREI16_VV_MF4_E16_MF2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7761 = PseudoVRGATHEREI16_VV_MF2_E8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7760 = PseudoVRGATHEREI16_VV_MF2_E8_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7759 = PseudoVRGATHEREI16_VV_MF2_E8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7758 = PseudoVRGATHEREI16_VV_MF2_E8_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7757 = PseudoVRGATHEREI16_VV_MF2_E8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7756 = PseudoVRGATHEREI16_VV_MF2_E8_MF2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7755 = PseudoVRGATHEREI16_VV_MF2_E8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7754 = PseudoVRGATHEREI16_VV_MF2_E8_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7753 = PseudoVRGATHEREI16_VV_MF2_E32_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7752 = PseudoVRGATHEREI16_VV_MF2_E32_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7751 = PseudoVRGATHEREI16_VV_MF2_E32_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7750 = PseudoVRGATHEREI16_VV_MF2_E32_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7749 = PseudoVRGATHEREI16_VV_MF2_E32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7748 = PseudoVRGATHEREI16_VV_MF2_E32_MF2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7747 = PseudoVRGATHEREI16_VV_MF2_E32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7746 = PseudoVRGATHEREI16_VV_MF2_E32_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7745 = PseudoVRGATHEREI16_VV_MF2_E16_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7744 = PseudoVRGATHEREI16_VV_MF2_E16_MF8
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7743 = PseudoVRGATHEREI16_VV_MF2_E16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7742 = PseudoVRGATHEREI16_VV_MF2_E16_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7741 = PseudoVRGATHEREI16_VV_MF2_E16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7740 = PseudoVRGATHEREI16_VV_MF2_E16_MF2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7739 = PseudoVRGATHEREI16_VV_MF2_E16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7738 = PseudoVRGATHEREI16_VV_MF2_E16_M1
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #7737 = PseudoVRGATHEREI16_VV_M8_E8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #7736 = PseudoVRGATHEREI16_VV_M8_E8_M8
  { 8, &RISCVDescs.OperandInfo[6332] },  // Inst #7735 = PseudoVRGATHEREI16_VV_M8_E8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6325] },  // Inst #7734 = PseudoVRGATHEREI16_VV_M8_E8_M4
  { 8, &RISCVDescs.OperandInfo[6317] },  // Inst #7733 = PseudoVRGATHEREI16_VV_M8_E8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6310] },  // Inst #7732 = PseudoVRGATHEREI16_VV_M8_E8_M2
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #7731 = PseudoVRGATHEREI16_VV_M8_E64_M8_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #7730 = PseudoVRGATHEREI16_VV_M8_E64_M8
  { 8, &RISCVDescs.OperandInfo[6332] },  // Inst #7729 = PseudoVRGATHEREI16_VV_M8_E64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6325] },  // Inst #7728 = PseudoVRGATHEREI16_VV_M8_E64_M4
  { 8, &RISCVDescs.OperandInfo[6317] },  // Inst #7727 = PseudoVRGATHEREI16_VV_M8_E64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6310] },  // Inst #7726 = PseudoVRGATHEREI16_VV_M8_E64_M2
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #7725 = PseudoVRGATHEREI16_VV_M8_E32_M8_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #7724 = PseudoVRGATHEREI16_VV_M8_E32_M8
  { 8, &RISCVDescs.OperandInfo[6332] },  // Inst #7723 = PseudoVRGATHEREI16_VV_M8_E32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6325] },  // Inst #7722 = PseudoVRGATHEREI16_VV_M8_E32_M4
  { 8, &RISCVDescs.OperandInfo[6317] },  // Inst #7721 = PseudoVRGATHEREI16_VV_M8_E32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6310] },  // Inst #7720 = PseudoVRGATHEREI16_VV_M8_E32_M2
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #7719 = PseudoVRGATHEREI16_VV_M8_E16_M8_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #7718 = PseudoVRGATHEREI16_VV_M8_E16_M8
  { 8, &RISCVDescs.OperandInfo[6332] },  // Inst #7717 = PseudoVRGATHEREI16_VV_M8_E16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6325] },  // Inst #7716 = PseudoVRGATHEREI16_VV_M8_E16_M4
  { 8, &RISCVDescs.OperandInfo[6317] },  // Inst #7715 = PseudoVRGATHEREI16_VV_M8_E16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6310] },  // Inst #7714 = PseudoVRGATHEREI16_VV_M8_E16_M2
  { 8, &RISCVDescs.OperandInfo[6302] },  // Inst #7713 = PseudoVRGATHEREI16_VV_M4_E8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6295] },  // Inst #7712 = PseudoVRGATHEREI16_VV_M4_E8_M8
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #7711 = PseudoVRGATHEREI16_VV_M4_E8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #7710 = PseudoVRGATHEREI16_VV_M4_E8_M4
  { 8, &RISCVDescs.OperandInfo[6287] },  // Inst #7709 = PseudoVRGATHEREI16_VV_M4_E8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6280] },  // Inst #7708 = PseudoVRGATHEREI16_VV_M4_E8_M2
  { 8, &RISCVDescs.OperandInfo[6272] },  // Inst #7707 = PseudoVRGATHEREI16_VV_M4_E8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6265] },  // Inst #7706 = PseudoVRGATHEREI16_VV_M4_E8_M1
  { 8, &RISCVDescs.OperandInfo[6302] },  // Inst #7705 = PseudoVRGATHEREI16_VV_M4_E64_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6295] },  // Inst #7704 = PseudoVRGATHEREI16_VV_M4_E64_M8
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #7703 = PseudoVRGATHEREI16_VV_M4_E64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #7702 = PseudoVRGATHEREI16_VV_M4_E64_M4
  { 8, &RISCVDescs.OperandInfo[6287] },  // Inst #7701 = PseudoVRGATHEREI16_VV_M4_E64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6280] },  // Inst #7700 = PseudoVRGATHEREI16_VV_M4_E64_M2
  { 8, &RISCVDescs.OperandInfo[6272] },  // Inst #7699 = PseudoVRGATHEREI16_VV_M4_E64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6265] },  // Inst #7698 = PseudoVRGATHEREI16_VV_M4_E64_M1
  { 8, &RISCVDescs.OperandInfo[6302] },  // Inst #7697 = PseudoVRGATHEREI16_VV_M4_E32_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6295] },  // Inst #7696 = PseudoVRGATHEREI16_VV_M4_E32_M8
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #7695 = PseudoVRGATHEREI16_VV_M4_E32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #7694 = PseudoVRGATHEREI16_VV_M4_E32_M4
  { 8, &RISCVDescs.OperandInfo[6287] },  // Inst #7693 = PseudoVRGATHEREI16_VV_M4_E32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6280] },  // Inst #7692 = PseudoVRGATHEREI16_VV_M4_E32_M2
  { 8, &RISCVDescs.OperandInfo[6272] },  // Inst #7691 = PseudoVRGATHEREI16_VV_M4_E32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6265] },  // Inst #7690 = PseudoVRGATHEREI16_VV_M4_E32_M1
  { 8, &RISCVDescs.OperandInfo[6302] },  // Inst #7689 = PseudoVRGATHEREI16_VV_M4_E16_M8_MASK
  { 7, &RISCVDescs.OperandInfo[6295] },  // Inst #7688 = PseudoVRGATHEREI16_VV_M4_E16_M8
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #7687 = PseudoVRGATHEREI16_VV_M4_E16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #7686 = PseudoVRGATHEREI16_VV_M4_E16_M4
  { 8, &RISCVDescs.OperandInfo[6287] },  // Inst #7685 = PseudoVRGATHEREI16_VV_M4_E16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6280] },  // Inst #7684 = PseudoVRGATHEREI16_VV_M4_E16_M2
  { 8, &RISCVDescs.OperandInfo[6272] },  // Inst #7683 = PseudoVRGATHEREI16_VV_M4_E16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6265] },  // Inst #7682 = PseudoVRGATHEREI16_VV_M4_E16_M1
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #7681 = PseudoVRGATHEREI16_VV_M2_E8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #7680 = PseudoVRGATHEREI16_VV_M2_E8_MF2
  { 8, &RISCVDescs.OperandInfo[6257] },  // Inst #7679 = PseudoVRGATHEREI16_VV_M2_E8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6250] },  // Inst #7678 = PseudoVRGATHEREI16_VV_M2_E8_M4
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #7677 = PseudoVRGATHEREI16_VV_M2_E8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #7676 = PseudoVRGATHEREI16_VV_M2_E8_M2
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #7675 = PseudoVRGATHEREI16_VV_M2_E8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #7674 = PseudoVRGATHEREI16_VV_M2_E8_M1
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #7673 = PseudoVRGATHEREI16_VV_M2_E64_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #7672 = PseudoVRGATHEREI16_VV_M2_E64_MF2
  { 8, &RISCVDescs.OperandInfo[6257] },  // Inst #7671 = PseudoVRGATHEREI16_VV_M2_E64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6250] },  // Inst #7670 = PseudoVRGATHEREI16_VV_M2_E64_M4
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #7669 = PseudoVRGATHEREI16_VV_M2_E64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #7668 = PseudoVRGATHEREI16_VV_M2_E64_M2
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #7667 = PseudoVRGATHEREI16_VV_M2_E64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #7666 = PseudoVRGATHEREI16_VV_M2_E64_M1
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #7665 = PseudoVRGATHEREI16_VV_M2_E32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #7664 = PseudoVRGATHEREI16_VV_M2_E32_MF2
  { 8, &RISCVDescs.OperandInfo[6257] },  // Inst #7663 = PseudoVRGATHEREI16_VV_M2_E32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6250] },  // Inst #7662 = PseudoVRGATHEREI16_VV_M2_E32_M4
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #7661 = PseudoVRGATHEREI16_VV_M2_E32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #7660 = PseudoVRGATHEREI16_VV_M2_E32_M2
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #7659 = PseudoVRGATHEREI16_VV_M2_E32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #7658 = PseudoVRGATHEREI16_VV_M2_E32_M1
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #7657 = PseudoVRGATHEREI16_VV_M2_E16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #7656 = PseudoVRGATHEREI16_VV_M2_E16_MF2
  { 8, &RISCVDescs.OperandInfo[6257] },  // Inst #7655 = PseudoVRGATHEREI16_VV_M2_E16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6250] },  // Inst #7654 = PseudoVRGATHEREI16_VV_M2_E16_M4
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #7653 = PseudoVRGATHEREI16_VV_M2_E16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #7652 = PseudoVRGATHEREI16_VV_M2_E16_M2
  { 8, &RISCVDescs.OperandInfo[6242] },  // Inst #7651 = PseudoVRGATHEREI16_VV_M2_E16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6235] },  // Inst #7650 = PseudoVRGATHEREI16_VV_M2_E16_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7649 = PseudoVRGATHEREI16_VV_M1_E8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7648 = PseudoVRGATHEREI16_VV_M1_E8_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7647 = PseudoVRGATHEREI16_VV_M1_E8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7646 = PseudoVRGATHEREI16_VV_M1_E8_MF2
  { 8, &RISCVDescs.OperandInfo[6227] },  // Inst #7645 = PseudoVRGATHEREI16_VV_M1_E8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6220] },  // Inst #7644 = PseudoVRGATHEREI16_VV_M1_E8_M2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7643 = PseudoVRGATHEREI16_VV_M1_E8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7642 = PseudoVRGATHEREI16_VV_M1_E8_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7641 = PseudoVRGATHEREI16_VV_M1_E64_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7640 = PseudoVRGATHEREI16_VV_M1_E64_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7639 = PseudoVRGATHEREI16_VV_M1_E64_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7638 = PseudoVRGATHEREI16_VV_M1_E64_MF2
  { 8, &RISCVDescs.OperandInfo[6227] },  // Inst #7637 = PseudoVRGATHEREI16_VV_M1_E64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6220] },  // Inst #7636 = PseudoVRGATHEREI16_VV_M1_E64_M2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7635 = PseudoVRGATHEREI16_VV_M1_E64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7634 = PseudoVRGATHEREI16_VV_M1_E64_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7633 = PseudoVRGATHEREI16_VV_M1_E32_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7632 = PseudoVRGATHEREI16_VV_M1_E32_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7631 = PseudoVRGATHEREI16_VV_M1_E32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7630 = PseudoVRGATHEREI16_VV_M1_E32_MF2
  { 8, &RISCVDescs.OperandInfo[6227] },  // Inst #7629 = PseudoVRGATHEREI16_VV_M1_E32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6220] },  // Inst #7628 = PseudoVRGATHEREI16_VV_M1_E32_M2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7627 = PseudoVRGATHEREI16_VV_M1_E32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7626 = PseudoVRGATHEREI16_VV_M1_E32_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7625 = PseudoVRGATHEREI16_VV_M1_E16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7624 = PseudoVRGATHEREI16_VV_M1_E16_MF4
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7623 = PseudoVRGATHEREI16_VV_M1_E16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7622 = PseudoVRGATHEREI16_VV_M1_E16_MF2
  { 8, &RISCVDescs.OperandInfo[6227] },  // Inst #7621 = PseudoVRGATHEREI16_VV_M1_E16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6220] },  // Inst #7620 = PseudoVRGATHEREI16_VV_M1_E16_M2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #7619 = PseudoVRGATHEREI16_VV_M1_E16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #7618 = PseudoVRGATHEREI16_VV_M1_E16_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #7617 = PseudoVREV8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #7616 = PseudoVREV8_V_MF8
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #7615 = PseudoVREV8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #7614 = PseudoVREV8_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #7613 = PseudoVREV8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #7612 = PseudoVREV8_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #7611 = PseudoVREV8_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #7610 = PseudoVREV8_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #7609 = PseudoVREV8_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #7608 = PseudoVREV8_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #7607 = PseudoVREV8_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #7606 = PseudoVREV8_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #7605 = PseudoVREV8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #7604 = PseudoVREV8_V_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7603 = PseudoVREM_VX_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7602 = PseudoVREM_VX_MF8_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7601 = PseudoVREM_VX_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7600 = PseudoVREM_VX_MF4_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7599 = PseudoVREM_VX_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7598 = PseudoVREM_VX_MF4_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7597 = PseudoVREM_VX_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7596 = PseudoVREM_VX_MF2_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7595 = PseudoVREM_VX_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7594 = PseudoVREM_VX_MF2_E32
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7593 = PseudoVREM_VX_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7592 = PseudoVREM_VX_MF2_E16
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7591 = PseudoVREM_VX_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7590 = PseudoVREM_VX_M8_E8
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7589 = PseudoVREM_VX_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7588 = PseudoVREM_VX_M8_E64
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7587 = PseudoVREM_VX_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7586 = PseudoVREM_VX_M8_E32
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7585 = PseudoVREM_VX_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7584 = PseudoVREM_VX_M8_E16
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7583 = PseudoVREM_VX_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7582 = PseudoVREM_VX_M4_E8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7581 = PseudoVREM_VX_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7580 = PseudoVREM_VX_M4_E64
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7579 = PseudoVREM_VX_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7578 = PseudoVREM_VX_M4_E32
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7577 = PseudoVREM_VX_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7576 = PseudoVREM_VX_M4_E16
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7575 = PseudoVREM_VX_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7574 = PseudoVREM_VX_M2_E8
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7573 = PseudoVREM_VX_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7572 = PseudoVREM_VX_M2_E64
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7571 = PseudoVREM_VX_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7570 = PseudoVREM_VX_M2_E32
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7569 = PseudoVREM_VX_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7568 = PseudoVREM_VX_M2_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7567 = PseudoVREM_VX_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7566 = PseudoVREM_VX_M1_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7565 = PseudoVREM_VX_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7564 = PseudoVREM_VX_M1_E64
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7563 = PseudoVREM_VX_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7562 = PseudoVREM_VX_M1_E32
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7561 = PseudoVREM_VX_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7560 = PseudoVREM_VX_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7559 = PseudoVREM_VV_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7558 = PseudoVREM_VV_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7557 = PseudoVREM_VV_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7556 = PseudoVREM_VV_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7555 = PseudoVREM_VV_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7554 = PseudoVREM_VV_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7553 = PseudoVREM_VV_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7552 = PseudoVREM_VV_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7551 = PseudoVREM_VV_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7550 = PseudoVREM_VV_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7549 = PseudoVREM_VV_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7548 = PseudoVREM_VV_MF2_E16
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7547 = PseudoVREM_VV_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7546 = PseudoVREM_VV_M8_E8
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7545 = PseudoVREM_VV_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7544 = PseudoVREM_VV_M8_E64
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7543 = PseudoVREM_VV_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7542 = PseudoVREM_VV_M8_E32
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7541 = PseudoVREM_VV_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7540 = PseudoVREM_VV_M8_E16
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7539 = PseudoVREM_VV_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7538 = PseudoVREM_VV_M4_E8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7537 = PseudoVREM_VV_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7536 = PseudoVREM_VV_M4_E64
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7535 = PseudoVREM_VV_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7534 = PseudoVREM_VV_M4_E32
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7533 = PseudoVREM_VV_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7532 = PseudoVREM_VV_M4_E16
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7531 = PseudoVREM_VV_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7530 = PseudoVREM_VV_M2_E8
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7529 = PseudoVREM_VV_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7528 = PseudoVREM_VV_M2_E64
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7527 = PseudoVREM_VV_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7526 = PseudoVREM_VV_M2_E32
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7525 = PseudoVREM_VV_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7524 = PseudoVREM_VV_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7523 = PseudoVREM_VV_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7522 = PseudoVREM_VV_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7521 = PseudoVREM_VV_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7520 = PseudoVREM_VV_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7519 = PseudoVREM_VV_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7518 = PseudoVREM_VV_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7517 = PseudoVREM_VV_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7516 = PseudoVREM_VV_M1_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7515 = PseudoVREMU_VX_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7514 = PseudoVREMU_VX_MF8_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7513 = PseudoVREMU_VX_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7512 = PseudoVREMU_VX_MF4_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7511 = PseudoVREMU_VX_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7510 = PseudoVREMU_VX_MF4_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7509 = PseudoVREMU_VX_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7508 = PseudoVREMU_VX_MF2_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7507 = PseudoVREMU_VX_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7506 = PseudoVREMU_VX_MF2_E32
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7505 = PseudoVREMU_VX_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7504 = PseudoVREMU_VX_MF2_E16
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7503 = PseudoVREMU_VX_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7502 = PseudoVREMU_VX_M8_E8
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7501 = PseudoVREMU_VX_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7500 = PseudoVREMU_VX_M8_E64
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7499 = PseudoVREMU_VX_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7498 = PseudoVREMU_VX_M8_E32
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7497 = PseudoVREMU_VX_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7496 = PseudoVREMU_VX_M8_E16
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7495 = PseudoVREMU_VX_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7494 = PseudoVREMU_VX_M4_E8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7493 = PseudoVREMU_VX_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7492 = PseudoVREMU_VX_M4_E64
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7491 = PseudoVREMU_VX_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7490 = PseudoVREMU_VX_M4_E32
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7489 = PseudoVREMU_VX_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7488 = PseudoVREMU_VX_M4_E16
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7487 = PseudoVREMU_VX_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7486 = PseudoVREMU_VX_M2_E8
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7485 = PseudoVREMU_VX_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7484 = PseudoVREMU_VX_M2_E64
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7483 = PseudoVREMU_VX_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7482 = PseudoVREMU_VX_M2_E32
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7481 = PseudoVREMU_VX_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7480 = PseudoVREMU_VX_M2_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7479 = PseudoVREMU_VX_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7478 = PseudoVREMU_VX_M1_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7477 = PseudoVREMU_VX_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7476 = PseudoVREMU_VX_M1_E64
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7475 = PseudoVREMU_VX_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7474 = PseudoVREMU_VX_M1_E32
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7473 = PseudoVREMU_VX_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7472 = PseudoVREMU_VX_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7471 = PseudoVREMU_VV_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7470 = PseudoVREMU_VV_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7469 = PseudoVREMU_VV_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7468 = PseudoVREMU_VV_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7467 = PseudoVREMU_VV_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7466 = PseudoVREMU_VV_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7465 = PseudoVREMU_VV_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7464 = PseudoVREMU_VV_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7463 = PseudoVREMU_VV_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7462 = PseudoVREMU_VV_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7461 = PseudoVREMU_VV_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7460 = PseudoVREMU_VV_MF2_E16
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7459 = PseudoVREMU_VV_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7458 = PseudoVREMU_VV_M8_E8
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7457 = PseudoVREMU_VV_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7456 = PseudoVREMU_VV_M8_E64
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7455 = PseudoVREMU_VV_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7454 = PseudoVREMU_VV_M8_E32
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #7453 = PseudoVREMU_VV_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #7452 = PseudoVREMU_VV_M8_E16
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7451 = PseudoVREMU_VV_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7450 = PseudoVREMU_VV_M4_E8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7449 = PseudoVREMU_VV_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7448 = PseudoVREMU_VV_M4_E64
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7447 = PseudoVREMU_VV_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7446 = PseudoVREMU_VV_M4_E32
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #7445 = PseudoVREMU_VV_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #7444 = PseudoVREMU_VV_M4_E16
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7443 = PseudoVREMU_VV_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7442 = PseudoVREMU_VV_M2_E8
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7441 = PseudoVREMU_VV_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7440 = PseudoVREMU_VV_M2_E64
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7439 = PseudoVREMU_VV_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7438 = PseudoVREMU_VV_M2_E32
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #7437 = PseudoVREMU_VV_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #7436 = PseudoVREMU_VV_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7435 = PseudoVREMU_VV_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7434 = PseudoVREMU_VV_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7433 = PseudoVREMU_VV_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7432 = PseudoVREMU_VV_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7431 = PseudoVREMU_VV_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7430 = PseudoVREMU_VV_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7429 = PseudoVREMU_VV_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7428 = PseudoVREMU_VV_M1_E16
  { 2, &RISCVDescs.OperandInfo[6218] },  // Inst #7427 = PseudoVRELOAD8_MF8
  { 2, &RISCVDescs.OperandInfo[6218] },  // Inst #7426 = PseudoVRELOAD8_MF4
  { 2, &RISCVDescs.OperandInfo[6218] },  // Inst #7425 = PseudoVRELOAD8_MF2
  { 2, &RISCVDescs.OperandInfo[6218] },  // Inst #7424 = PseudoVRELOAD8_M1
  { 2, &RISCVDescs.OperandInfo[6216] },  // Inst #7423 = PseudoVRELOAD7_MF8
  { 2, &RISCVDescs.OperandInfo[6216] },  // Inst #7422 = PseudoVRELOAD7_MF4
  { 2, &RISCVDescs.OperandInfo[6216] },  // Inst #7421 = PseudoVRELOAD7_MF2
  { 2, &RISCVDescs.OperandInfo[6216] },  // Inst #7420 = PseudoVRELOAD7_M1
  { 2, &RISCVDescs.OperandInfo[6214] },  // Inst #7419 = PseudoVRELOAD6_MF8
  { 2, &RISCVDescs.OperandInfo[6214] },  // Inst #7418 = PseudoVRELOAD6_MF4
  { 2, &RISCVDescs.OperandInfo[6214] },  // Inst #7417 = PseudoVRELOAD6_MF2
  { 2, &RISCVDescs.OperandInfo[6214] },  // Inst #7416 = PseudoVRELOAD6_M1
  { 2, &RISCVDescs.OperandInfo[6212] },  // Inst #7415 = PseudoVRELOAD5_MF8
  { 2, &RISCVDescs.OperandInfo[6212] },  // Inst #7414 = PseudoVRELOAD5_MF4
  { 2, &RISCVDescs.OperandInfo[6212] },  // Inst #7413 = PseudoVRELOAD5_MF2
  { 2, &RISCVDescs.OperandInfo[6212] },  // Inst #7412 = PseudoVRELOAD5_M1
  { 2, &RISCVDescs.OperandInfo[6208] },  // Inst #7411 = PseudoVRELOAD4_MF8
  { 2, &RISCVDescs.OperandInfo[6208] },  // Inst #7410 = PseudoVRELOAD4_MF4
  { 2, &RISCVDescs.OperandInfo[6208] },  // Inst #7409 = PseudoVRELOAD4_MF2
  { 2, &RISCVDescs.OperandInfo[6210] },  // Inst #7408 = PseudoVRELOAD4_M2
  { 2, &RISCVDescs.OperandInfo[6208] },  // Inst #7407 = PseudoVRELOAD4_M1
  { 2, &RISCVDescs.OperandInfo[6204] },  // Inst #7406 = PseudoVRELOAD3_MF8
  { 2, &RISCVDescs.OperandInfo[6204] },  // Inst #7405 = PseudoVRELOAD3_MF4
  { 2, &RISCVDescs.OperandInfo[6204] },  // Inst #7404 = PseudoVRELOAD3_MF2
  { 2, &RISCVDescs.OperandInfo[6206] },  // Inst #7403 = PseudoVRELOAD3_M2
  { 2, &RISCVDescs.OperandInfo[6204] },  // Inst #7402 = PseudoVRELOAD3_M1
  { 2, &RISCVDescs.OperandInfo[6198] },  // Inst #7401 = PseudoVRELOAD2_MF8
  { 2, &RISCVDescs.OperandInfo[6198] },  // Inst #7400 = PseudoVRELOAD2_MF4
  { 2, &RISCVDescs.OperandInfo[6198] },  // Inst #7399 = PseudoVRELOAD2_MF2
  { 2, &RISCVDescs.OperandInfo[6202] },  // Inst #7398 = PseudoVRELOAD2_M4
  { 2, &RISCVDescs.OperandInfo[6200] },  // Inst #7397 = PseudoVRELOAD2_M2
  { 2, &RISCVDescs.OperandInfo[6198] },  // Inst #7396 = PseudoVRELOAD2_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7395 = PseudoVREDXOR_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7394 = PseudoVREDXOR_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7393 = PseudoVREDXOR_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7392 = PseudoVREDXOR_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7391 = PseudoVREDXOR_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7390 = PseudoVREDXOR_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7389 = PseudoVREDXOR_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7388 = PseudoVREDXOR_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7387 = PseudoVREDXOR_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7386 = PseudoVREDXOR_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7385 = PseudoVREDXOR_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7384 = PseudoVREDXOR_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7383 = PseudoVREDXOR_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7382 = PseudoVREDXOR_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7381 = PseudoVREDXOR_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7380 = PseudoVREDXOR_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7379 = PseudoVREDXOR_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7378 = PseudoVREDXOR_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7377 = PseudoVREDXOR_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7376 = PseudoVREDXOR_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7375 = PseudoVREDXOR_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7374 = PseudoVREDXOR_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7373 = PseudoVREDXOR_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7372 = PseudoVREDXOR_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7371 = PseudoVREDXOR_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7370 = PseudoVREDXOR_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7369 = PseudoVREDXOR_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7368 = PseudoVREDXOR_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7367 = PseudoVREDXOR_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7366 = PseudoVREDXOR_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7365 = PseudoVREDXOR_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7364 = PseudoVREDXOR_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7363 = PseudoVREDXOR_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7362 = PseudoVREDXOR_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7361 = PseudoVREDXOR_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7360 = PseudoVREDXOR_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7359 = PseudoVREDXOR_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7358 = PseudoVREDXOR_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7357 = PseudoVREDXOR_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7356 = PseudoVREDXOR_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7355 = PseudoVREDXOR_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7354 = PseudoVREDXOR_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7353 = PseudoVREDXOR_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7352 = PseudoVREDXOR_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7351 = PseudoVREDSUM_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7350 = PseudoVREDSUM_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7349 = PseudoVREDSUM_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7348 = PseudoVREDSUM_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7347 = PseudoVREDSUM_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7346 = PseudoVREDSUM_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7345 = PseudoVREDSUM_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7344 = PseudoVREDSUM_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7343 = PseudoVREDSUM_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7342 = PseudoVREDSUM_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7341 = PseudoVREDSUM_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7340 = PseudoVREDSUM_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7339 = PseudoVREDSUM_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7338 = PseudoVREDSUM_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7337 = PseudoVREDSUM_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7336 = PseudoVREDSUM_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7335 = PseudoVREDSUM_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7334 = PseudoVREDSUM_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7333 = PseudoVREDSUM_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7332 = PseudoVREDSUM_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7331 = PseudoVREDSUM_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7330 = PseudoVREDSUM_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7329 = PseudoVREDSUM_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7328 = PseudoVREDSUM_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7327 = PseudoVREDSUM_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7326 = PseudoVREDSUM_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7325 = PseudoVREDSUM_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7324 = PseudoVREDSUM_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7323 = PseudoVREDSUM_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7322 = PseudoVREDSUM_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7321 = PseudoVREDSUM_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7320 = PseudoVREDSUM_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7319 = PseudoVREDSUM_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7318 = PseudoVREDSUM_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7317 = PseudoVREDSUM_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7316 = PseudoVREDSUM_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7315 = PseudoVREDSUM_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7314 = PseudoVREDSUM_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7313 = PseudoVREDSUM_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7312 = PseudoVREDSUM_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7311 = PseudoVREDSUM_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7310 = PseudoVREDSUM_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7309 = PseudoVREDSUM_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7308 = PseudoVREDSUM_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7307 = PseudoVREDOR_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7306 = PseudoVREDOR_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7305 = PseudoVREDOR_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7304 = PseudoVREDOR_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7303 = PseudoVREDOR_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7302 = PseudoVREDOR_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7301 = PseudoVREDOR_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7300 = PseudoVREDOR_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7299 = PseudoVREDOR_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7298 = PseudoVREDOR_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7297 = PseudoVREDOR_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7296 = PseudoVREDOR_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7295 = PseudoVREDOR_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7294 = PseudoVREDOR_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7293 = PseudoVREDOR_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7292 = PseudoVREDOR_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7291 = PseudoVREDOR_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7290 = PseudoVREDOR_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7289 = PseudoVREDOR_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7288 = PseudoVREDOR_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7287 = PseudoVREDOR_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7286 = PseudoVREDOR_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7285 = PseudoVREDOR_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7284 = PseudoVREDOR_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7283 = PseudoVREDOR_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7282 = PseudoVREDOR_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7281 = PseudoVREDOR_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7280 = PseudoVREDOR_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7279 = PseudoVREDOR_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7278 = PseudoVREDOR_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7277 = PseudoVREDOR_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7276 = PseudoVREDOR_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7275 = PseudoVREDOR_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7274 = PseudoVREDOR_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7273 = PseudoVREDOR_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7272 = PseudoVREDOR_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7271 = PseudoVREDOR_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7270 = PseudoVREDOR_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7269 = PseudoVREDOR_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7268 = PseudoVREDOR_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7267 = PseudoVREDOR_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7266 = PseudoVREDOR_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7265 = PseudoVREDOR_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7264 = PseudoVREDOR_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7263 = PseudoVREDMIN_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7262 = PseudoVREDMIN_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7261 = PseudoVREDMIN_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7260 = PseudoVREDMIN_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7259 = PseudoVREDMIN_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7258 = PseudoVREDMIN_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7257 = PseudoVREDMIN_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7256 = PseudoVREDMIN_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7255 = PseudoVREDMIN_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7254 = PseudoVREDMIN_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7253 = PseudoVREDMIN_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7252 = PseudoVREDMIN_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7251 = PseudoVREDMIN_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7250 = PseudoVREDMIN_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7249 = PseudoVREDMIN_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7248 = PseudoVREDMIN_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7247 = PseudoVREDMIN_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7246 = PseudoVREDMIN_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7245 = PseudoVREDMIN_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7244 = PseudoVREDMIN_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7243 = PseudoVREDMIN_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7242 = PseudoVREDMIN_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7241 = PseudoVREDMIN_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7240 = PseudoVREDMIN_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7239 = PseudoVREDMIN_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7238 = PseudoVREDMIN_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7237 = PseudoVREDMIN_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7236 = PseudoVREDMIN_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7235 = PseudoVREDMIN_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7234 = PseudoVREDMIN_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7233 = PseudoVREDMIN_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7232 = PseudoVREDMIN_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7231 = PseudoVREDMIN_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7230 = PseudoVREDMIN_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7229 = PseudoVREDMIN_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7228 = PseudoVREDMIN_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7227 = PseudoVREDMIN_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7226 = PseudoVREDMIN_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7225 = PseudoVREDMIN_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7224 = PseudoVREDMIN_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7223 = PseudoVREDMIN_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7222 = PseudoVREDMIN_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7221 = PseudoVREDMIN_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7220 = PseudoVREDMIN_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7219 = PseudoVREDMINU_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7218 = PseudoVREDMINU_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7217 = PseudoVREDMINU_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7216 = PseudoVREDMINU_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7215 = PseudoVREDMINU_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7214 = PseudoVREDMINU_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7213 = PseudoVREDMINU_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7212 = PseudoVREDMINU_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7211 = PseudoVREDMINU_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7210 = PseudoVREDMINU_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7209 = PseudoVREDMINU_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7208 = PseudoVREDMINU_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7207 = PseudoVREDMINU_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7206 = PseudoVREDMINU_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7205 = PseudoVREDMINU_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7204 = PseudoVREDMINU_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7203 = PseudoVREDMINU_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7202 = PseudoVREDMINU_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7201 = PseudoVREDMINU_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7200 = PseudoVREDMINU_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7199 = PseudoVREDMINU_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7198 = PseudoVREDMINU_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7197 = PseudoVREDMINU_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7196 = PseudoVREDMINU_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7195 = PseudoVREDMINU_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7194 = PseudoVREDMINU_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7193 = PseudoVREDMINU_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7192 = PseudoVREDMINU_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7191 = PseudoVREDMINU_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7190 = PseudoVREDMINU_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7189 = PseudoVREDMINU_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7188 = PseudoVREDMINU_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7187 = PseudoVREDMINU_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7186 = PseudoVREDMINU_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7185 = PseudoVREDMINU_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7184 = PseudoVREDMINU_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7183 = PseudoVREDMINU_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7182 = PseudoVREDMINU_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7181 = PseudoVREDMINU_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7180 = PseudoVREDMINU_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7179 = PseudoVREDMINU_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7178 = PseudoVREDMINU_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7177 = PseudoVREDMINU_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7176 = PseudoVREDMINU_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7175 = PseudoVREDMAX_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7174 = PseudoVREDMAX_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7173 = PseudoVREDMAX_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7172 = PseudoVREDMAX_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7171 = PseudoVREDMAX_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7170 = PseudoVREDMAX_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7169 = PseudoVREDMAX_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7168 = PseudoVREDMAX_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7167 = PseudoVREDMAX_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7166 = PseudoVREDMAX_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7165 = PseudoVREDMAX_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7164 = PseudoVREDMAX_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7163 = PseudoVREDMAX_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7162 = PseudoVREDMAX_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7161 = PseudoVREDMAX_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7160 = PseudoVREDMAX_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7159 = PseudoVREDMAX_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7158 = PseudoVREDMAX_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7157 = PseudoVREDMAX_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7156 = PseudoVREDMAX_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7155 = PseudoVREDMAX_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7154 = PseudoVREDMAX_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7153 = PseudoVREDMAX_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7152 = PseudoVREDMAX_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7151 = PseudoVREDMAX_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7150 = PseudoVREDMAX_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7149 = PseudoVREDMAX_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7148 = PseudoVREDMAX_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7147 = PseudoVREDMAX_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7146 = PseudoVREDMAX_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7145 = PseudoVREDMAX_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7144 = PseudoVREDMAX_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7143 = PseudoVREDMAX_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7142 = PseudoVREDMAX_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7141 = PseudoVREDMAX_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7140 = PseudoVREDMAX_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7139 = PseudoVREDMAX_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7138 = PseudoVREDMAX_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7137 = PseudoVREDMAX_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7136 = PseudoVREDMAX_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7135 = PseudoVREDMAX_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7134 = PseudoVREDMAX_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7133 = PseudoVREDMAX_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7132 = PseudoVREDMAX_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7131 = PseudoVREDMAXU_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7130 = PseudoVREDMAXU_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7129 = PseudoVREDMAXU_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7128 = PseudoVREDMAXU_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7127 = PseudoVREDMAXU_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7126 = PseudoVREDMAXU_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7125 = PseudoVREDMAXU_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7124 = PseudoVREDMAXU_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7123 = PseudoVREDMAXU_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7122 = PseudoVREDMAXU_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7121 = PseudoVREDMAXU_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7120 = PseudoVREDMAXU_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7119 = PseudoVREDMAXU_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7118 = PseudoVREDMAXU_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7117 = PseudoVREDMAXU_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7116 = PseudoVREDMAXU_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7115 = PseudoVREDMAXU_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7114 = PseudoVREDMAXU_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7113 = PseudoVREDMAXU_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7112 = PseudoVREDMAXU_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7111 = PseudoVREDMAXU_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7110 = PseudoVREDMAXU_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7109 = PseudoVREDMAXU_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7108 = PseudoVREDMAXU_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7107 = PseudoVREDMAXU_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7106 = PseudoVREDMAXU_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7105 = PseudoVREDMAXU_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7104 = PseudoVREDMAXU_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7103 = PseudoVREDMAXU_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7102 = PseudoVREDMAXU_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7101 = PseudoVREDMAXU_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7100 = PseudoVREDMAXU_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7099 = PseudoVREDMAXU_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7098 = PseudoVREDMAXU_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7097 = PseudoVREDMAXU_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7096 = PseudoVREDMAXU_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7095 = PseudoVREDMAXU_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7094 = PseudoVREDMAXU_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7093 = PseudoVREDMAXU_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7092 = PseudoVREDMAXU_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7091 = PseudoVREDMAXU_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7090 = PseudoVREDMAXU_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7089 = PseudoVREDMAXU_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7088 = PseudoVREDMAXU_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7087 = PseudoVREDAND_VS_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7086 = PseudoVREDAND_VS_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7085 = PseudoVREDAND_VS_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7084 = PseudoVREDAND_VS_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7083 = PseudoVREDAND_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7082 = PseudoVREDAND_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7081 = PseudoVREDAND_VS_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7080 = PseudoVREDAND_VS_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7079 = PseudoVREDAND_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7078 = PseudoVREDAND_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7077 = PseudoVREDAND_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7076 = PseudoVREDAND_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7075 = PseudoVREDAND_VS_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7074 = PseudoVREDAND_VS_M8_E8
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7073 = PseudoVREDAND_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7072 = PseudoVREDAND_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7071 = PseudoVREDAND_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7070 = PseudoVREDAND_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #7069 = PseudoVREDAND_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #7068 = PseudoVREDAND_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7067 = PseudoVREDAND_VS_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7066 = PseudoVREDAND_VS_M4_E8
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7065 = PseudoVREDAND_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7064 = PseudoVREDAND_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7063 = PseudoVREDAND_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7062 = PseudoVREDAND_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #7061 = PseudoVREDAND_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #7060 = PseudoVREDAND_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7059 = PseudoVREDAND_VS_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7058 = PseudoVREDAND_VS_M2_E8
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7057 = PseudoVREDAND_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7056 = PseudoVREDAND_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7055 = PseudoVREDAND_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7054 = PseudoVREDAND_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #7053 = PseudoVREDAND_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #7052 = PseudoVREDAND_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7051 = PseudoVREDAND_VS_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7050 = PseudoVREDAND_VS_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7049 = PseudoVREDAND_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7048 = PseudoVREDAND_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7047 = PseudoVREDAND_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7046 = PseudoVREDAND_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #7045 = PseudoVREDAND_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7044 = PseudoVREDAND_VS_M1_E16
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7043 = PseudoVQMACC_4x8x4_MF2
  { 7, &RISCVDescs.OperandInfo[3642] },  // Inst #7042 = PseudoVQMACC_4x8x4_M4
  { 7, &RISCVDescs.OperandInfo[3635] },  // Inst #7041 = PseudoVQMACC_4x8x4_M2
  { 7, &RISCVDescs.OperandInfo[3628] },  // Inst #7040 = PseudoVQMACC_4x8x4_M1
  { 7, &RISCVDescs.OperandInfo[6191] },  // Inst #7039 = PseudoVQMACC_2x8x2_M8
  { 7, &RISCVDescs.OperandInfo[6184] },  // Inst #7038 = PseudoVQMACC_2x8x2_M4
  { 7, &RISCVDescs.OperandInfo[6177] },  // Inst #7037 = PseudoVQMACC_2x8x2_M2
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7036 = PseudoVQMACC_2x8x2_M1
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7035 = PseudoVQMACCU_4x8x4_MF2
  { 7, &RISCVDescs.OperandInfo[3642] },  // Inst #7034 = PseudoVQMACCU_4x8x4_M4
  { 7, &RISCVDescs.OperandInfo[3635] },  // Inst #7033 = PseudoVQMACCU_4x8x4_M2
  { 7, &RISCVDescs.OperandInfo[3628] },  // Inst #7032 = PseudoVQMACCU_4x8x4_M1
  { 7, &RISCVDescs.OperandInfo[6191] },  // Inst #7031 = PseudoVQMACCU_2x8x2_M8
  { 7, &RISCVDescs.OperandInfo[6184] },  // Inst #7030 = PseudoVQMACCU_2x8x2_M4
  { 7, &RISCVDescs.OperandInfo[6177] },  // Inst #7029 = PseudoVQMACCU_2x8x2_M2
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7028 = PseudoVQMACCU_2x8x2_M1
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7027 = PseudoVQMACCUS_4x8x4_MF2
  { 7, &RISCVDescs.OperandInfo[3642] },  // Inst #7026 = PseudoVQMACCUS_4x8x4_M4
  { 7, &RISCVDescs.OperandInfo[3635] },  // Inst #7025 = PseudoVQMACCUS_4x8x4_M2
  { 7, &RISCVDescs.OperandInfo[3628] },  // Inst #7024 = PseudoVQMACCUS_4x8x4_M1
  { 7, &RISCVDescs.OperandInfo[6191] },  // Inst #7023 = PseudoVQMACCUS_2x8x2_M8
  { 7, &RISCVDescs.OperandInfo[6184] },  // Inst #7022 = PseudoVQMACCUS_2x8x2_M4
  { 7, &RISCVDescs.OperandInfo[6177] },  // Inst #7021 = PseudoVQMACCUS_2x8x2_M2
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7020 = PseudoVQMACCUS_2x8x2_M1
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7019 = PseudoVQMACCSU_4x8x4_MF2
  { 7, &RISCVDescs.OperandInfo[3642] },  // Inst #7018 = PseudoVQMACCSU_4x8x4_M4
  { 7, &RISCVDescs.OperandInfo[3635] },  // Inst #7017 = PseudoVQMACCSU_4x8x4_M2
  { 7, &RISCVDescs.OperandInfo[3628] },  // Inst #7016 = PseudoVQMACCSU_4x8x4_M1
  { 7, &RISCVDescs.OperandInfo[6191] },  // Inst #7015 = PseudoVQMACCSU_2x8x2_M8
  { 7, &RISCVDescs.OperandInfo[6184] },  // Inst #7014 = PseudoVQMACCSU_2x8x2_M4
  { 7, &RISCVDescs.OperandInfo[6177] },  // Inst #7013 = PseudoVQMACCSU_2x8x2_M2
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #7012 = PseudoVQMACCSU_2x8x2_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7011 = PseudoVOR_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7010 = PseudoVOR_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7009 = PseudoVOR_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7008 = PseudoVOR_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #7007 = PseudoVOR_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #7006 = PseudoVOR_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #7005 = PseudoVOR_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #7004 = PseudoVOR_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #7003 = PseudoVOR_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #7002 = PseudoVOR_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #7001 = PseudoVOR_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #7000 = PseudoVOR_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6999 = PseudoVOR_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6998 = PseudoVOR_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6997 = PseudoVOR_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6996 = PseudoVOR_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6995 = PseudoVOR_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6994 = PseudoVOR_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6993 = PseudoVOR_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6992 = PseudoVOR_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #6991 = PseudoVOR_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #6990 = PseudoVOR_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #6989 = PseudoVOR_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #6988 = PseudoVOR_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #6987 = PseudoVOR_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #6986 = PseudoVOR_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6985 = PseudoVOR_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6984 = PseudoVOR_VV_M1
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #6983 = PseudoVOR_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #6982 = PseudoVOR_VI_MF8
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #6981 = PseudoVOR_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #6980 = PseudoVOR_VI_MF4
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #6979 = PseudoVOR_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #6978 = PseudoVOR_VI_MF2
  { 8, &RISCVDescs.OperandInfo[689] },  // Inst #6977 = PseudoVOR_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[682] },  // Inst #6976 = PseudoVOR_VI_M8
  { 8, &RISCVDescs.OperandInfo[674] },  // Inst #6975 = PseudoVOR_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[667] },  // Inst #6974 = PseudoVOR_VI_M4
  { 8, &RISCVDescs.OperandInfo[659] },  // Inst #6973 = PseudoVOR_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[652] },  // Inst #6972 = PseudoVOR_VI_M2
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #6971 = PseudoVOR_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #6970 = PseudoVOR_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6969 = PseudoVNSRL_WX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6968 = PseudoVNSRL_WX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6967 = PseudoVNSRL_WX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6966 = PseudoVNSRL_WX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6965 = PseudoVNSRL_WX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6964 = PseudoVNSRL_WX_MF2
  { 8, &RISCVDescs.OperandInfo[6169] },  // Inst #6963 = PseudoVNSRL_WX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6162] },  // Inst #6962 = PseudoVNSRL_WX_M4
  { 8, &RISCVDescs.OperandInfo[6154] },  // Inst #6961 = PseudoVNSRL_WX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6147] },  // Inst #6960 = PseudoVNSRL_WX_M2
  { 8, &RISCVDescs.OperandInfo[6139] },  // Inst #6959 = PseudoVNSRL_WX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6132] },  // Inst #6958 = PseudoVNSRL_WX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6957 = PseudoVNSRL_WV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6956 = PseudoVNSRL_WV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6955 = PseudoVNSRL_WV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6954 = PseudoVNSRL_WV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6953 = PseudoVNSRL_WV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6952 = PseudoVNSRL_WV_MF2
  { 8, &RISCVDescs.OperandInfo[6124] },  // Inst #6951 = PseudoVNSRL_WV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6117] },  // Inst #6950 = PseudoVNSRL_WV_M4
  { 8, &RISCVDescs.OperandInfo[6109] },  // Inst #6949 = PseudoVNSRL_WV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6102] },  // Inst #6948 = PseudoVNSRL_WV_M2
  { 8, &RISCVDescs.OperandInfo[6094] },  // Inst #6947 = PseudoVNSRL_WV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6087] },  // Inst #6946 = PseudoVNSRL_WV_M1
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #6945 = PseudoVNSRL_WI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #6944 = PseudoVNSRL_WI_MF8
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #6943 = PseudoVNSRL_WI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #6942 = PseudoVNSRL_WI_MF4
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #6941 = PseudoVNSRL_WI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #6940 = PseudoVNSRL_WI_MF2
  { 8, &RISCVDescs.OperandInfo[6071] },  // Inst #6939 = PseudoVNSRL_WI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6064] },  // Inst #6938 = PseudoVNSRL_WI_M4
  { 8, &RISCVDescs.OperandInfo[6056] },  // Inst #6937 = PseudoVNSRL_WI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6049] },  // Inst #6936 = PseudoVNSRL_WI_M2
  { 8, &RISCVDescs.OperandInfo[6041] },  // Inst #6935 = PseudoVNSRL_WI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6034] },  // Inst #6934 = PseudoVNSRL_WI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6933 = PseudoVNSRA_WX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6932 = PseudoVNSRA_WX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6931 = PseudoVNSRA_WX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6930 = PseudoVNSRA_WX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6929 = PseudoVNSRA_WX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6928 = PseudoVNSRA_WX_MF2
  { 8, &RISCVDescs.OperandInfo[6169] },  // Inst #6927 = PseudoVNSRA_WX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6162] },  // Inst #6926 = PseudoVNSRA_WX_M4
  { 8, &RISCVDescs.OperandInfo[6154] },  // Inst #6925 = PseudoVNSRA_WX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6147] },  // Inst #6924 = PseudoVNSRA_WX_M2
  { 8, &RISCVDescs.OperandInfo[6139] },  // Inst #6923 = PseudoVNSRA_WX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6132] },  // Inst #6922 = PseudoVNSRA_WX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6921 = PseudoVNSRA_WV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6920 = PseudoVNSRA_WV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6919 = PseudoVNSRA_WV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6918 = PseudoVNSRA_WV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6917 = PseudoVNSRA_WV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6916 = PseudoVNSRA_WV_MF2
  { 8, &RISCVDescs.OperandInfo[6124] },  // Inst #6915 = PseudoVNSRA_WV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6117] },  // Inst #6914 = PseudoVNSRA_WV_M4
  { 8, &RISCVDescs.OperandInfo[6109] },  // Inst #6913 = PseudoVNSRA_WV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6102] },  // Inst #6912 = PseudoVNSRA_WV_M2
  { 8, &RISCVDescs.OperandInfo[6094] },  // Inst #6911 = PseudoVNSRA_WV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6087] },  // Inst #6910 = PseudoVNSRA_WV_M1
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #6909 = PseudoVNSRA_WI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #6908 = PseudoVNSRA_WI_MF8
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #6907 = PseudoVNSRA_WI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #6906 = PseudoVNSRA_WI_MF4
  { 8, &RISCVDescs.OperandInfo[6079] },  // Inst #6905 = PseudoVNSRA_WI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #6904 = PseudoVNSRA_WI_MF2
  { 8, &RISCVDescs.OperandInfo[6071] },  // Inst #6903 = PseudoVNSRA_WI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[6064] },  // Inst #6902 = PseudoVNSRA_WI_M4
  { 8, &RISCVDescs.OperandInfo[6056] },  // Inst #6901 = PseudoVNSRA_WI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[6049] },  // Inst #6900 = PseudoVNSRA_WI_M2
  { 8, &RISCVDescs.OperandInfo[6041] },  // Inst #6899 = PseudoVNSRA_WI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[6034] },  // Inst #6898 = PseudoVNSRA_WI_M1
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #6897 = PseudoVNMSUB_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #6896 = PseudoVNMSUB_VX_MF8
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #6895 = PseudoVNMSUB_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #6894 = PseudoVNMSUB_VX_MF4
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #6893 = PseudoVNMSUB_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #6892 = PseudoVNMSUB_VX_MF2
  { 8, &RISCVDescs.OperandInfo[5406] },  // Inst #6891 = PseudoVNMSUB_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[5399] },  // Inst #6890 = PseudoVNMSUB_VX_M8
  { 8, &RISCVDescs.OperandInfo[5391] },  // Inst #6889 = PseudoVNMSUB_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[5384] },  // Inst #6888 = PseudoVNMSUB_VX_M4
  { 8, &RISCVDescs.OperandInfo[5376] },  // Inst #6887 = PseudoVNMSUB_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5369] },  // Inst #6886 = PseudoVNMSUB_VX_M2
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #6885 = PseudoVNMSUB_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #6884 = PseudoVNMSUB_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6883 = PseudoVNMSUB_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6882 = PseudoVNMSUB_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6881 = PseudoVNMSUB_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6880 = PseudoVNMSUB_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6879 = PseudoVNMSUB_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6878 = PseudoVNMSUB_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #6877 = PseudoVNMSUB_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #6876 = PseudoVNMSUB_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #6875 = PseudoVNMSUB_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #6874 = PseudoVNMSUB_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #6873 = PseudoVNMSUB_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #6872 = PseudoVNMSUB_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6871 = PseudoVNMSUB_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6870 = PseudoVNMSUB_VV_M1
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #6869 = PseudoVNMSAC_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #6868 = PseudoVNMSAC_VX_MF8
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #6867 = PseudoVNMSAC_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #6866 = PseudoVNMSAC_VX_MF4
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #6865 = PseudoVNMSAC_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #6864 = PseudoVNMSAC_VX_MF2
  { 8, &RISCVDescs.OperandInfo[5406] },  // Inst #6863 = PseudoVNMSAC_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[5399] },  // Inst #6862 = PseudoVNMSAC_VX_M8
  { 8, &RISCVDescs.OperandInfo[5391] },  // Inst #6861 = PseudoVNMSAC_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[5384] },  // Inst #6860 = PseudoVNMSAC_VX_M4
  { 8, &RISCVDescs.OperandInfo[5376] },  // Inst #6859 = PseudoVNMSAC_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5369] },  // Inst #6858 = PseudoVNMSAC_VX_M2
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #6857 = PseudoVNMSAC_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #6856 = PseudoVNMSAC_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6855 = PseudoVNMSAC_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6854 = PseudoVNMSAC_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6853 = PseudoVNMSAC_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6852 = PseudoVNMSAC_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6851 = PseudoVNMSAC_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6850 = PseudoVNMSAC_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #6849 = PseudoVNMSAC_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #6848 = PseudoVNMSAC_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #6847 = PseudoVNMSAC_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #6846 = PseudoVNMSAC_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #6845 = PseudoVNMSAC_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #6844 = PseudoVNMSAC_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6843 = PseudoVNMSAC_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6842 = PseudoVNMSAC_VV_M1
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #6841 = PseudoVNCLIP_WX_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #6840 = PseudoVNCLIP_WX_MF8
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #6839 = PseudoVNCLIP_WX_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #6838 = PseudoVNCLIP_WX_MF4
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #6837 = PseudoVNCLIP_WX_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #6836 = PseudoVNCLIP_WX_MF2
  { 9, &RISCVDescs.OperandInfo[6025] },  // Inst #6835 = PseudoVNCLIP_WX_M4_MASK
  { 8, &RISCVDescs.OperandInfo[6017] },  // Inst #6834 = PseudoVNCLIP_WX_M4
  { 9, &RISCVDescs.OperandInfo[6008] },  // Inst #6833 = PseudoVNCLIP_WX_M2_MASK
  { 8, &RISCVDescs.OperandInfo[6000] },  // Inst #6832 = PseudoVNCLIP_WX_M2
  { 9, &RISCVDescs.OperandInfo[5991] },  // Inst #6831 = PseudoVNCLIP_WX_M1_MASK
  { 8, &RISCVDescs.OperandInfo[5983] },  // Inst #6830 = PseudoVNCLIP_WX_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #6829 = PseudoVNCLIP_WV_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #6828 = PseudoVNCLIP_WV_MF8
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #6827 = PseudoVNCLIP_WV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #6826 = PseudoVNCLIP_WV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #6825 = PseudoVNCLIP_WV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #6824 = PseudoVNCLIP_WV_MF2
  { 9, &RISCVDescs.OperandInfo[5974] },  // Inst #6823 = PseudoVNCLIP_WV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[5966] },  // Inst #6822 = PseudoVNCLIP_WV_M4
  { 9, &RISCVDescs.OperandInfo[5957] },  // Inst #6821 = PseudoVNCLIP_WV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[5949] },  // Inst #6820 = PseudoVNCLIP_WV_M2
  { 9, &RISCVDescs.OperandInfo[5940] },  // Inst #6819 = PseudoVNCLIP_WV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[5932] },  // Inst #6818 = PseudoVNCLIP_WV_M1
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #6817 = PseudoVNCLIP_WI_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #6816 = PseudoVNCLIP_WI_MF8
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #6815 = PseudoVNCLIP_WI_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #6814 = PseudoVNCLIP_WI_MF4
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #6813 = PseudoVNCLIP_WI_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #6812 = PseudoVNCLIP_WI_MF2
  { 9, &RISCVDescs.OperandInfo[5906] },  // Inst #6811 = PseudoVNCLIP_WI_M4_MASK
  { 8, &RISCVDescs.OperandInfo[5898] },  // Inst #6810 = PseudoVNCLIP_WI_M4
  { 9, &RISCVDescs.OperandInfo[5889] },  // Inst #6809 = PseudoVNCLIP_WI_M2_MASK
  { 8, &RISCVDescs.OperandInfo[5881] },  // Inst #6808 = PseudoVNCLIP_WI_M2
  { 9, &RISCVDescs.OperandInfo[5872] },  // Inst #6807 = PseudoVNCLIP_WI_M1_MASK
  { 8, &RISCVDescs.OperandInfo[5864] },  // Inst #6806 = PseudoVNCLIP_WI_M1
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #6805 = PseudoVNCLIPU_WX_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #6804 = PseudoVNCLIPU_WX_MF8
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #6803 = PseudoVNCLIPU_WX_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #6802 = PseudoVNCLIPU_WX_MF4
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #6801 = PseudoVNCLIPU_WX_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #6800 = PseudoVNCLIPU_WX_MF2
  { 9, &RISCVDescs.OperandInfo[6025] },  // Inst #6799 = PseudoVNCLIPU_WX_M4_MASK
  { 8, &RISCVDescs.OperandInfo[6017] },  // Inst #6798 = PseudoVNCLIPU_WX_M4
  { 9, &RISCVDescs.OperandInfo[6008] },  // Inst #6797 = PseudoVNCLIPU_WX_M2_MASK
  { 8, &RISCVDescs.OperandInfo[6000] },  // Inst #6796 = PseudoVNCLIPU_WX_M2
  { 9, &RISCVDescs.OperandInfo[5991] },  // Inst #6795 = PseudoVNCLIPU_WX_M1_MASK
  { 8, &RISCVDescs.OperandInfo[5983] },  // Inst #6794 = PseudoVNCLIPU_WX_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #6793 = PseudoVNCLIPU_WV_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #6792 = PseudoVNCLIPU_WV_MF8
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #6791 = PseudoVNCLIPU_WV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #6790 = PseudoVNCLIPU_WV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #6789 = PseudoVNCLIPU_WV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #6788 = PseudoVNCLIPU_WV_MF2
  { 9, &RISCVDescs.OperandInfo[5974] },  // Inst #6787 = PseudoVNCLIPU_WV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[5966] },  // Inst #6786 = PseudoVNCLIPU_WV_M4
  { 9, &RISCVDescs.OperandInfo[5957] },  // Inst #6785 = PseudoVNCLIPU_WV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[5949] },  // Inst #6784 = PseudoVNCLIPU_WV_M2
  { 9, &RISCVDescs.OperandInfo[5940] },  // Inst #6783 = PseudoVNCLIPU_WV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[5932] },  // Inst #6782 = PseudoVNCLIPU_WV_M1
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #6781 = PseudoVNCLIPU_WI_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #6780 = PseudoVNCLIPU_WI_MF8
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #6779 = PseudoVNCLIPU_WI_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #6778 = PseudoVNCLIPU_WI_MF4
  { 9, &RISCVDescs.OperandInfo[5923] },  // Inst #6777 = PseudoVNCLIPU_WI_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[5915] },  // Inst #6776 = PseudoVNCLIPU_WI_MF2
  { 9, &RISCVDescs.OperandInfo[5906] },  // Inst #6775 = PseudoVNCLIPU_WI_M4_MASK
  { 8, &RISCVDescs.OperandInfo[5898] },  // Inst #6774 = PseudoVNCLIPU_WI_M4
  { 9, &RISCVDescs.OperandInfo[5889] },  // Inst #6773 = PseudoVNCLIPU_WI_M2_MASK
  { 8, &RISCVDescs.OperandInfo[5881] },  // Inst #6772 = PseudoVNCLIPU_WI_M2
  { 9, &RISCVDescs.OperandInfo[5872] },  // Inst #6771 = PseudoVNCLIPU_WI_M1_MASK
  { 8, &RISCVDescs.OperandInfo[5864] },  // Inst #6770 = PseudoVNCLIPU_WI_M1
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6769 = PseudoVMXOR_MM_MF8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6768 = PseudoVMXOR_MM_MF4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6767 = PseudoVMXOR_MM_MF2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6766 = PseudoVMXOR_MM_M8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6765 = PseudoVMXOR_MM_M4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6764 = PseudoVMXOR_MM_M2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6763 = PseudoVMXOR_MM_M1
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6762 = PseudoVMXNOR_MM_MF8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6761 = PseudoVMXNOR_MM_MF4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6760 = PseudoVMXNOR_MM_MF2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6759 = PseudoVMXNOR_MM_M8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6758 = PseudoVMXNOR_MM_M4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6757 = PseudoVMXNOR_MM_M2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6756 = PseudoVMXNOR_MM_M1
  { 3, &RISCVDescs.OperandInfo[5861] },  // Inst #6755 = PseudoVMV_X_S
  { 6, &RISCVDescs.OperandInfo[5837] },  // Inst #6754 = PseudoVMV_V_X_MF8
  { 6, &RISCVDescs.OperandInfo[5837] },  // Inst #6753 = PseudoVMV_V_X_MF4
  { 6, &RISCVDescs.OperandInfo[5837] },  // Inst #6752 = PseudoVMV_V_X_MF2
  { 6, &RISCVDescs.OperandInfo[5855] },  // Inst #6751 = PseudoVMV_V_X_M8
  { 6, &RISCVDescs.OperandInfo[5849] },  // Inst #6750 = PseudoVMV_V_X_M4
  { 6, &RISCVDescs.OperandInfo[5843] },  // Inst #6749 = PseudoVMV_V_X_M2
  { 6, &RISCVDescs.OperandInfo[5837] },  // Inst #6748 = PseudoVMV_V_X_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #6747 = PseudoVMV_V_V_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #6746 = PseudoVMV_V_V_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #6745 = PseudoVMV_V_V_MF2
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #6744 = PseudoVMV_V_V_M8
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #6743 = PseudoVMV_V_V_M4
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #6742 = PseudoVMV_V_V_M2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #6741 = PseudoVMV_V_V_M1
  { 6, &RISCVDescs.OperandInfo[5813] },  // Inst #6740 = PseudoVMV_V_I_MF8
  { 6, &RISCVDescs.OperandInfo[5813] },  // Inst #6739 = PseudoVMV_V_I_MF4
  { 6, &RISCVDescs.OperandInfo[5813] },  // Inst #6738 = PseudoVMV_V_I_MF2
  { 6, &RISCVDescs.OperandInfo[5831] },  // Inst #6737 = PseudoVMV_V_I_M8
  { 6, &RISCVDescs.OperandInfo[5825] },  // Inst #6736 = PseudoVMV_V_I_M4
  { 6, &RISCVDescs.OperandInfo[5819] },  // Inst #6735 = PseudoVMV_V_I_M2
  { 6, &RISCVDescs.OperandInfo[5813] },  // Inst #6734 = PseudoVMV_V_I_M1
  { 5, &RISCVDescs.OperandInfo[5808] },  // Inst #6733 = PseudoVMV_S_X
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6732 = PseudoVMUL_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6731 = PseudoVMUL_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6730 = PseudoVMUL_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6729 = PseudoVMUL_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6728 = PseudoVMUL_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6727 = PseudoVMUL_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #6726 = PseudoVMUL_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #6725 = PseudoVMUL_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #6724 = PseudoVMUL_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #6723 = PseudoVMUL_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #6722 = PseudoVMUL_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #6721 = PseudoVMUL_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6720 = PseudoVMUL_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6719 = PseudoVMUL_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6718 = PseudoVMUL_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6717 = PseudoVMUL_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6716 = PseudoVMUL_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6715 = PseudoVMUL_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6714 = PseudoVMUL_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6713 = PseudoVMUL_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #6712 = PseudoVMUL_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #6711 = PseudoVMUL_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #6710 = PseudoVMUL_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #6709 = PseudoVMUL_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #6708 = PseudoVMUL_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #6707 = PseudoVMUL_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6706 = PseudoVMUL_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6705 = PseudoVMUL_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6704 = PseudoVMULH_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6703 = PseudoVMULH_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6702 = PseudoVMULH_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6701 = PseudoVMULH_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6700 = PseudoVMULH_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6699 = PseudoVMULH_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #6698 = PseudoVMULH_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #6697 = PseudoVMULH_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #6696 = PseudoVMULH_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #6695 = PseudoVMULH_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #6694 = PseudoVMULH_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #6693 = PseudoVMULH_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6692 = PseudoVMULH_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6691 = PseudoVMULH_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6690 = PseudoVMULH_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6689 = PseudoVMULH_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6688 = PseudoVMULH_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6687 = PseudoVMULH_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6686 = PseudoVMULH_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6685 = PseudoVMULH_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #6684 = PseudoVMULH_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #6683 = PseudoVMULH_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #6682 = PseudoVMULH_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #6681 = PseudoVMULH_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #6680 = PseudoVMULH_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #6679 = PseudoVMULH_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6678 = PseudoVMULH_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6677 = PseudoVMULH_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6676 = PseudoVMULHU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6675 = PseudoVMULHU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6674 = PseudoVMULHU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6673 = PseudoVMULHU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6672 = PseudoVMULHU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6671 = PseudoVMULHU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #6670 = PseudoVMULHU_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #6669 = PseudoVMULHU_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #6668 = PseudoVMULHU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #6667 = PseudoVMULHU_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #6666 = PseudoVMULHU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #6665 = PseudoVMULHU_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6664 = PseudoVMULHU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6663 = PseudoVMULHU_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6662 = PseudoVMULHU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6661 = PseudoVMULHU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6660 = PseudoVMULHU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6659 = PseudoVMULHU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6658 = PseudoVMULHU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6657 = PseudoVMULHU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #6656 = PseudoVMULHU_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #6655 = PseudoVMULHU_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #6654 = PseudoVMULHU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #6653 = PseudoVMULHU_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #6652 = PseudoVMULHU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #6651 = PseudoVMULHU_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6650 = PseudoVMULHU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6649 = PseudoVMULHU_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6648 = PseudoVMULHSU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6647 = PseudoVMULHSU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6646 = PseudoVMULHSU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6645 = PseudoVMULHSU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6644 = PseudoVMULHSU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6643 = PseudoVMULHSU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #6642 = PseudoVMULHSU_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #6641 = PseudoVMULHSU_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #6640 = PseudoVMULHSU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #6639 = PseudoVMULHSU_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #6638 = PseudoVMULHSU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #6637 = PseudoVMULHSU_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6636 = PseudoVMULHSU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6635 = PseudoVMULHSU_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6634 = PseudoVMULHSU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6633 = PseudoVMULHSU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6632 = PseudoVMULHSU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6631 = PseudoVMULHSU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6630 = PseudoVMULHSU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6629 = PseudoVMULHSU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #6628 = PseudoVMULHSU_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #6627 = PseudoVMULHSU_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #6626 = PseudoVMULHSU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #6625 = PseudoVMULHSU_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #6624 = PseudoVMULHSU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #6623 = PseudoVMULHSU_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6622 = PseudoVMULHSU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6621 = PseudoVMULHSU_VV_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6620 = PseudoVMSOF_M_B8_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6619 = PseudoVMSOF_M_B8
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6618 = PseudoVMSOF_M_B64_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6617 = PseudoVMSOF_M_B64
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6616 = PseudoVMSOF_M_B4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6615 = PseudoVMSOF_M_B4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6614 = PseudoVMSOF_M_B32_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6613 = PseudoVMSOF_M_B32
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6612 = PseudoVMSOF_M_B2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6611 = PseudoVMSOF_M_B2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6610 = PseudoVMSOF_M_B1_MASK
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6609 = PseudoVMSOF_M_B16_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6608 = PseudoVMSOF_M_B16
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6607 = PseudoVMSOF_M_B1
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6606 = PseudoVMSNE_VX_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6605 = PseudoVMSNE_VX_MF8
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6604 = PseudoVMSNE_VX_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6603 = PseudoVMSNE_VX_MF4
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6602 = PseudoVMSNE_VX_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6601 = PseudoVMSNE_VX_MF2
  { 7, &RISCVDescs.OperandInfo[5785] },  // Inst #6600 = PseudoVMSNE_VX_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #6599 = PseudoVMSNE_VX_M8
  { 7, &RISCVDescs.OperandInfo[5778] },  // Inst #6598 = PseudoVMSNE_VX_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #6597 = PseudoVMSNE_VX_M4
  { 7, &RISCVDescs.OperandInfo[5771] },  // Inst #6596 = PseudoVMSNE_VX_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #6595 = PseudoVMSNE_VX_M2
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6594 = PseudoVMSNE_VX_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6593 = PseudoVMSNE_VX_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6592 = PseudoVMSNE_VV_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6591 = PseudoVMSNE_VV_MF8
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6590 = PseudoVMSNE_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6589 = PseudoVMSNE_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6588 = PseudoVMSNE_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6587 = PseudoVMSNE_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #6586 = PseudoVMSNE_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6585 = PseudoVMSNE_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #6584 = PseudoVMSNE_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6583 = PseudoVMSNE_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #6582 = PseudoVMSNE_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6581 = PseudoVMSNE_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6580 = PseudoVMSNE_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6579 = PseudoVMSNE_VV_M1
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6578 = PseudoVMSNE_VI_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6577 = PseudoVMSNE_VI_MF8
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6576 = PseudoVMSNE_VI_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6575 = PseudoVMSNE_VI_MF4
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6574 = PseudoVMSNE_VI_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6573 = PseudoVMSNE_VI_MF2
  { 7, &RISCVDescs.OperandInfo[5752] },  // Inst #6572 = PseudoVMSNE_VI_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5453] },  // Inst #6571 = PseudoVMSNE_VI_M8
  { 7, &RISCVDescs.OperandInfo[5745] },  // Inst #6570 = PseudoVMSNE_VI_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5448] },  // Inst #6569 = PseudoVMSNE_VI_M4
  { 7, &RISCVDescs.OperandInfo[5738] },  // Inst #6568 = PseudoVMSNE_VI_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5443] },  // Inst #6567 = PseudoVMSNE_VI_M2
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6566 = PseudoVMSNE_VI_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6565 = PseudoVMSNE_VI_M1
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6564 = PseudoVMSLT_VX_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6563 = PseudoVMSLT_VX_MF8
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6562 = PseudoVMSLT_VX_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6561 = PseudoVMSLT_VX_MF4
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6560 = PseudoVMSLT_VX_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6559 = PseudoVMSLT_VX_MF2
  { 7, &RISCVDescs.OperandInfo[5785] },  // Inst #6558 = PseudoVMSLT_VX_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #6557 = PseudoVMSLT_VX_M8
  { 7, &RISCVDescs.OperandInfo[5778] },  // Inst #6556 = PseudoVMSLT_VX_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #6555 = PseudoVMSLT_VX_M4
  { 7, &RISCVDescs.OperandInfo[5771] },  // Inst #6554 = PseudoVMSLT_VX_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #6553 = PseudoVMSLT_VX_M2
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6552 = PseudoVMSLT_VX_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6551 = PseudoVMSLT_VX_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6550 = PseudoVMSLT_VV_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6549 = PseudoVMSLT_VV_MF8
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6548 = PseudoVMSLT_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6547 = PseudoVMSLT_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6546 = PseudoVMSLT_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6545 = PseudoVMSLT_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #6544 = PseudoVMSLT_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6543 = PseudoVMSLT_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #6542 = PseudoVMSLT_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6541 = PseudoVMSLT_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #6540 = PseudoVMSLT_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6539 = PseudoVMSLT_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6538 = PseudoVMSLT_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6537 = PseudoVMSLT_VV_M1
  { 4, &RISCVDescs.OperandInfo[5792] },  // Inst #6536 = PseudoVMSLT_VI
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6535 = PseudoVMSLTU_VX_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6534 = PseudoVMSLTU_VX_MF8
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6533 = PseudoVMSLTU_VX_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6532 = PseudoVMSLTU_VX_MF4
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6531 = PseudoVMSLTU_VX_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6530 = PseudoVMSLTU_VX_MF2
  { 7, &RISCVDescs.OperandInfo[5785] },  // Inst #6529 = PseudoVMSLTU_VX_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #6528 = PseudoVMSLTU_VX_M8
  { 7, &RISCVDescs.OperandInfo[5778] },  // Inst #6527 = PseudoVMSLTU_VX_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #6526 = PseudoVMSLTU_VX_M4
  { 7, &RISCVDescs.OperandInfo[5771] },  // Inst #6525 = PseudoVMSLTU_VX_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #6524 = PseudoVMSLTU_VX_M2
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6523 = PseudoVMSLTU_VX_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6522 = PseudoVMSLTU_VX_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6521 = PseudoVMSLTU_VV_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6520 = PseudoVMSLTU_VV_MF8
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6519 = PseudoVMSLTU_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6518 = PseudoVMSLTU_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6517 = PseudoVMSLTU_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6516 = PseudoVMSLTU_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #6515 = PseudoVMSLTU_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6514 = PseudoVMSLTU_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #6513 = PseudoVMSLTU_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6512 = PseudoVMSLTU_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #6511 = PseudoVMSLTU_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6510 = PseudoVMSLTU_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6509 = PseudoVMSLTU_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6508 = PseudoVMSLTU_VV_M1
  { 4, &RISCVDescs.OperandInfo[5792] },  // Inst #6507 = PseudoVMSLTU_VI
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6506 = PseudoVMSLE_VX_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6505 = PseudoVMSLE_VX_MF8
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6504 = PseudoVMSLE_VX_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6503 = PseudoVMSLE_VX_MF4
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6502 = PseudoVMSLE_VX_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6501 = PseudoVMSLE_VX_MF2
  { 7, &RISCVDescs.OperandInfo[5785] },  // Inst #6500 = PseudoVMSLE_VX_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #6499 = PseudoVMSLE_VX_M8
  { 7, &RISCVDescs.OperandInfo[5778] },  // Inst #6498 = PseudoVMSLE_VX_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #6497 = PseudoVMSLE_VX_M4
  { 7, &RISCVDescs.OperandInfo[5771] },  // Inst #6496 = PseudoVMSLE_VX_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #6495 = PseudoVMSLE_VX_M2
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6494 = PseudoVMSLE_VX_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6493 = PseudoVMSLE_VX_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6492 = PseudoVMSLE_VV_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6491 = PseudoVMSLE_VV_MF8
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6490 = PseudoVMSLE_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6489 = PseudoVMSLE_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6488 = PseudoVMSLE_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6487 = PseudoVMSLE_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #6486 = PseudoVMSLE_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6485 = PseudoVMSLE_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #6484 = PseudoVMSLE_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6483 = PseudoVMSLE_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #6482 = PseudoVMSLE_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6481 = PseudoVMSLE_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6480 = PseudoVMSLE_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6479 = PseudoVMSLE_VV_M1
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6478 = PseudoVMSLE_VI_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6477 = PseudoVMSLE_VI_MF8
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6476 = PseudoVMSLE_VI_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6475 = PseudoVMSLE_VI_MF4
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6474 = PseudoVMSLE_VI_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6473 = PseudoVMSLE_VI_MF2
  { 7, &RISCVDescs.OperandInfo[5752] },  // Inst #6472 = PseudoVMSLE_VI_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5453] },  // Inst #6471 = PseudoVMSLE_VI_M8
  { 7, &RISCVDescs.OperandInfo[5745] },  // Inst #6470 = PseudoVMSLE_VI_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5448] },  // Inst #6469 = PseudoVMSLE_VI_M4
  { 7, &RISCVDescs.OperandInfo[5738] },  // Inst #6468 = PseudoVMSLE_VI_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5443] },  // Inst #6467 = PseudoVMSLE_VI_M2
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6466 = PseudoVMSLE_VI_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6465 = PseudoVMSLE_VI_M1
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6464 = PseudoVMSLEU_VX_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6463 = PseudoVMSLEU_VX_MF8
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6462 = PseudoVMSLEU_VX_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6461 = PseudoVMSLEU_VX_MF4
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6460 = PseudoVMSLEU_VX_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6459 = PseudoVMSLEU_VX_MF2
  { 7, &RISCVDescs.OperandInfo[5785] },  // Inst #6458 = PseudoVMSLEU_VX_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #6457 = PseudoVMSLEU_VX_M8
  { 7, &RISCVDescs.OperandInfo[5778] },  // Inst #6456 = PseudoVMSLEU_VX_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #6455 = PseudoVMSLEU_VX_M4
  { 7, &RISCVDescs.OperandInfo[5771] },  // Inst #6454 = PseudoVMSLEU_VX_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #6453 = PseudoVMSLEU_VX_M2
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6452 = PseudoVMSLEU_VX_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6451 = PseudoVMSLEU_VX_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6450 = PseudoVMSLEU_VV_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6449 = PseudoVMSLEU_VV_MF8
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6448 = PseudoVMSLEU_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6447 = PseudoVMSLEU_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6446 = PseudoVMSLEU_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6445 = PseudoVMSLEU_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #6444 = PseudoVMSLEU_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6443 = PseudoVMSLEU_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #6442 = PseudoVMSLEU_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6441 = PseudoVMSLEU_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #6440 = PseudoVMSLEU_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6439 = PseudoVMSLEU_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6438 = PseudoVMSLEU_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6437 = PseudoVMSLEU_VV_M1
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6436 = PseudoVMSLEU_VI_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6435 = PseudoVMSLEU_VI_MF8
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6434 = PseudoVMSLEU_VI_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6433 = PseudoVMSLEU_VI_MF4
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6432 = PseudoVMSLEU_VI_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6431 = PseudoVMSLEU_VI_MF2
  { 7, &RISCVDescs.OperandInfo[5752] },  // Inst #6430 = PseudoVMSLEU_VI_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5453] },  // Inst #6429 = PseudoVMSLEU_VI_M8
  { 7, &RISCVDescs.OperandInfo[5745] },  // Inst #6428 = PseudoVMSLEU_VI_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5448] },  // Inst #6427 = PseudoVMSLEU_VI_M4
  { 7, &RISCVDescs.OperandInfo[5738] },  // Inst #6426 = PseudoVMSLEU_VI_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5443] },  // Inst #6425 = PseudoVMSLEU_VI_M2
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6424 = PseudoVMSLEU_VI_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6423 = PseudoVMSLEU_VI_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6422 = PseudoVMSIF_M_B8_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6421 = PseudoVMSIF_M_B8
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6420 = PseudoVMSIF_M_B64_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6419 = PseudoVMSIF_M_B64
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6418 = PseudoVMSIF_M_B4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6417 = PseudoVMSIF_M_B4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6416 = PseudoVMSIF_M_B32_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6415 = PseudoVMSIF_M_B32
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6414 = PseudoVMSIF_M_B2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6413 = PseudoVMSIF_M_B2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6412 = PseudoVMSIF_M_B1_MASK
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6411 = PseudoVMSIF_M_B16_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6410 = PseudoVMSIF_M_B16
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6409 = PseudoVMSIF_M_B1
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6408 = PseudoVMSGT_VX_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6407 = PseudoVMSGT_VX_MF8
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6406 = PseudoVMSGT_VX_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6405 = PseudoVMSGT_VX_MF4
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6404 = PseudoVMSGT_VX_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6403 = PseudoVMSGT_VX_MF2
  { 7, &RISCVDescs.OperandInfo[5785] },  // Inst #6402 = PseudoVMSGT_VX_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #6401 = PseudoVMSGT_VX_M8
  { 7, &RISCVDescs.OperandInfo[5778] },  // Inst #6400 = PseudoVMSGT_VX_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #6399 = PseudoVMSGT_VX_M4
  { 7, &RISCVDescs.OperandInfo[5771] },  // Inst #6398 = PseudoVMSGT_VX_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #6397 = PseudoVMSGT_VX_M2
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6396 = PseudoVMSGT_VX_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6395 = PseudoVMSGT_VX_M1
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6394 = PseudoVMSGT_VI_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6393 = PseudoVMSGT_VI_MF8
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6392 = PseudoVMSGT_VI_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6391 = PseudoVMSGT_VI_MF4
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6390 = PseudoVMSGT_VI_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6389 = PseudoVMSGT_VI_MF2
  { 7, &RISCVDescs.OperandInfo[5752] },  // Inst #6388 = PseudoVMSGT_VI_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5453] },  // Inst #6387 = PseudoVMSGT_VI_M8
  { 7, &RISCVDescs.OperandInfo[5745] },  // Inst #6386 = PseudoVMSGT_VI_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5448] },  // Inst #6385 = PseudoVMSGT_VI_M4
  { 7, &RISCVDescs.OperandInfo[5738] },  // Inst #6384 = PseudoVMSGT_VI_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5443] },  // Inst #6383 = PseudoVMSGT_VI_M2
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6382 = PseudoVMSGT_VI_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6381 = PseudoVMSGT_VI_M1
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6380 = PseudoVMSGTU_VX_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6379 = PseudoVMSGTU_VX_MF8
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6378 = PseudoVMSGTU_VX_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6377 = PseudoVMSGTU_VX_MF4
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6376 = PseudoVMSGTU_VX_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6375 = PseudoVMSGTU_VX_MF2
  { 7, &RISCVDescs.OperandInfo[5785] },  // Inst #6374 = PseudoVMSGTU_VX_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #6373 = PseudoVMSGTU_VX_M8
  { 7, &RISCVDescs.OperandInfo[5778] },  // Inst #6372 = PseudoVMSGTU_VX_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #6371 = PseudoVMSGTU_VX_M4
  { 7, &RISCVDescs.OperandInfo[5771] },  // Inst #6370 = PseudoVMSGTU_VX_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #6369 = PseudoVMSGTU_VX_M2
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6368 = PseudoVMSGTU_VX_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6367 = PseudoVMSGTU_VX_M1
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6366 = PseudoVMSGTU_VI_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6365 = PseudoVMSGTU_VI_MF8
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6364 = PseudoVMSGTU_VI_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6363 = PseudoVMSGTU_VI_MF4
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6362 = PseudoVMSGTU_VI_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6361 = PseudoVMSGTU_VI_MF2
  { 7, &RISCVDescs.OperandInfo[5752] },  // Inst #6360 = PseudoVMSGTU_VI_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5453] },  // Inst #6359 = PseudoVMSGTU_VI_M8
  { 7, &RISCVDescs.OperandInfo[5745] },  // Inst #6358 = PseudoVMSGTU_VI_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5448] },  // Inst #6357 = PseudoVMSGTU_VI_M4
  { 7, &RISCVDescs.OperandInfo[5738] },  // Inst #6356 = PseudoVMSGTU_VI_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5443] },  // Inst #6355 = PseudoVMSGTU_VI_M2
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6354 = PseudoVMSGTU_VI_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6353 = PseudoVMSGTU_VI_M1
  { 5, &RISCVDescs.OperandInfo[5803] },  // Inst #6352 = PseudoVMSGE_VX_M_T
  { 4, &RISCVDescs.OperandInfo[5799] },  // Inst #6351 = PseudoVMSGE_VX_M
  { 3, &RISCVDescs.OperandInfo[5796] },  // Inst #6350 = PseudoVMSGE_VX
  { 4, &RISCVDescs.OperandInfo[5792] },  // Inst #6349 = PseudoVMSGE_VI
  { 5, &RISCVDescs.OperandInfo[5803] },  // Inst #6348 = PseudoVMSGEU_VX_M_T
  { 4, &RISCVDescs.OperandInfo[5799] },  // Inst #6347 = PseudoVMSGEU_VX_M
  { 3, &RISCVDescs.OperandInfo[5796] },  // Inst #6346 = PseudoVMSGEU_VX
  { 4, &RISCVDescs.OperandInfo[5792] },  // Inst #6345 = PseudoVMSGEU_VI
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #6344 = PseudoVMSET_M_B8
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #6343 = PseudoVMSET_M_B64
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #6342 = PseudoVMSET_M_B4
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #6341 = PseudoVMSET_M_B32
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #6340 = PseudoVMSET_M_B2
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #6339 = PseudoVMSET_M_B16
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #6338 = PseudoVMSET_M_B1
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6337 = PseudoVMSEQ_VX_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6336 = PseudoVMSEQ_VX_MF8
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6335 = PseudoVMSEQ_VX_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6334 = PseudoVMSEQ_VX_MF4
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6333 = PseudoVMSEQ_VX_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6332 = PseudoVMSEQ_VX_MF2
  { 7, &RISCVDescs.OperandInfo[5785] },  // Inst #6331 = PseudoVMSEQ_VX_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #6330 = PseudoVMSEQ_VX_M8
  { 7, &RISCVDescs.OperandInfo[5778] },  // Inst #6329 = PseudoVMSEQ_VX_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #6328 = PseudoVMSEQ_VX_M4
  { 7, &RISCVDescs.OperandInfo[5771] },  // Inst #6327 = PseudoVMSEQ_VX_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #6326 = PseudoVMSEQ_VX_M2
  { 7, &RISCVDescs.OperandInfo[5764] },  // Inst #6325 = PseudoVMSEQ_VX_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5759] },  // Inst #6324 = PseudoVMSEQ_VX_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6323 = PseudoVMSEQ_VV_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6322 = PseudoVMSEQ_VV_MF8
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6321 = PseudoVMSEQ_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6320 = PseudoVMSEQ_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6319 = PseudoVMSEQ_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6318 = PseudoVMSEQ_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #6317 = PseudoVMSEQ_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6316 = PseudoVMSEQ_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #6315 = PseudoVMSEQ_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6314 = PseudoVMSEQ_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #6313 = PseudoVMSEQ_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6312 = PseudoVMSEQ_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6311 = PseudoVMSEQ_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6310 = PseudoVMSEQ_VV_M1
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6309 = PseudoVMSEQ_VI_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6308 = PseudoVMSEQ_VI_MF8
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6307 = PseudoVMSEQ_VI_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6306 = PseudoVMSEQ_VI_MF4
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6305 = PseudoVMSEQ_VI_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6304 = PseudoVMSEQ_VI_MF2
  { 7, &RISCVDescs.OperandInfo[5752] },  // Inst #6303 = PseudoVMSEQ_VI_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5453] },  // Inst #6302 = PseudoVMSEQ_VI_M8
  { 7, &RISCVDescs.OperandInfo[5745] },  // Inst #6301 = PseudoVMSEQ_VI_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5448] },  // Inst #6300 = PseudoVMSEQ_VI_M4
  { 7, &RISCVDescs.OperandInfo[5738] },  // Inst #6299 = PseudoVMSEQ_VI_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5443] },  // Inst #6298 = PseudoVMSEQ_VI_M2
  { 7, &RISCVDescs.OperandInfo[5731] },  // Inst #6297 = PseudoVMSEQ_VI_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5726] },  // Inst #6296 = PseudoVMSEQ_VI_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6295 = PseudoVMSBF_M_B8_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6294 = PseudoVMSBF_M_B8
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6293 = PseudoVMSBF_M_B64_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6292 = PseudoVMSBF_M_B64
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6291 = PseudoVMSBF_M_B4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6290 = PseudoVMSBF_M_B4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6289 = PseudoVMSBF_M_B32_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6288 = PseudoVMSBF_M_B32
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6287 = PseudoVMSBF_M_B2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6286 = PseudoVMSBF_M_B2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6285 = PseudoVMSBF_M_B1_MASK
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #6284 = PseudoVMSBF_M_B16_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6283 = PseudoVMSBF_M_B16
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #6282 = PseudoVMSBF_M_B1
  { 5, &RISCVDescs.OperandInfo[5526] },  // Inst #6281 = PseudoVMSBC_VX_MF8
  { 5, &RISCVDescs.OperandInfo[5526] },  // Inst #6280 = PseudoVMSBC_VX_MF4
  { 5, &RISCVDescs.OperandInfo[5526] },  // Inst #6279 = PseudoVMSBC_VX_MF2
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #6278 = PseudoVMSBC_VX_M8
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #6277 = PseudoVMSBC_VX_M4
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #6276 = PseudoVMSBC_VX_M2
  { 5, &RISCVDescs.OperandInfo[5526] },  // Inst #6275 = PseudoVMSBC_VX_M1
  { 6, &RISCVDescs.OperandInfo[5502] },  // Inst #6274 = PseudoVMSBC_VXM_MF8
  { 6, &RISCVDescs.OperandInfo[5502] },  // Inst #6273 = PseudoVMSBC_VXM_MF4
  { 6, &RISCVDescs.OperandInfo[5502] },  // Inst #6272 = PseudoVMSBC_VXM_MF2
  { 6, &RISCVDescs.OperandInfo[5520] },  // Inst #6271 = PseudoVMSBC_VXM_M8
  { 6, &RISCVDescs.OperandInfo[5514] },  // Inst #6270 = PseudoVMSBC_VXM_M4
  { 6, &RISCVDescs.OperandInfo[5508] },  // Inst #6269 = PseudoVMSBC_VXM_M2
  { 6, &RISCVDescs.OperandInfo[5502] },  // Inst #6268 = PseudoVMSBC_VXM_M1
  { 5, &RISCVDescs.OperandInfo[5482] },  // Inst #6267 = PseudoVMSBC_VV_MF8
  { 5, &RISCVDescs.OperandInfo[5482] },  // Inst #6266 = PseudoVMSBC_VV_MF4
  { 5, &RISCVDescs.OperandInfo[5482] },  // Inst #6265 = PseudoVMSBC_VV_MF2
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6264 = PseudoVMSBC_VV_M8
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6263 = PseudoVMSBC_VV_M4
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6262 = PseudoVMSBC_VV_M2
  { 5, &RISCVDescs.OperandInfo[5482] },  // Inst #6261 = PseudoVMSBC_VV_M1
  { 6, &RISCVDescs.OperandInfo[5458] },  // Inst #6260 = PseudoVMSBC_VVM_MF8
  { 6, &RISCVDescs.OperandInfo[5458] },  // Inst #6259 = PseudoVMSBC_VVM_MF4
  { 6, &RISCVDescs.OperandInfo[5458] },  // Inst #6258 = PseudoVMSBC_VVM_MF2
  { 6, &RISCVDescs.OperandInfo[5476] },  // Inst #6257 = PseudoVMSBC_VVM_M8
  { 6, &RISCVDescs.OperandInfo[5470] },  // Inst #6256 = PseudoVMSBC_VVM_M4
  { 6, &RISCVDescs.OperandInfo[5464] },  // Inst #6255 = PseudoVMSBC_VVM_M2
  { 6, &RISCVDescs.OperandInfo[5458] },  // Inst #6254 = PseudoVMSBC_VVM_M1
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6253 = PseudoVMOR_MM_MF8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6252 = PseudoVMOR_MM_MF4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6251 = PseudoVMOR_MM_MF2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6250 = PseudoVMOR_MM_M8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6249 = PseudoVMOR_MM_M4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6248 = PseudoVMOR_MM_M2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6247 = PseudoVMOR_MM_M1
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6246 = PseudoVMORN_MM_MF8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6245 = PseudoVMORN_MM_MF4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6244 = PseudoVMORN_MM_MF2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6243 = PseudoVMORN_MM_M8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6242 = PseudoVMORN_MM_M4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6241 = PseudoVMORN_MM_M2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6240 = PseudoVMORN_MM_M1
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6239 = PseudoVMNOR_MM_MF8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6238 = PseudoVMNOR_MM_MF4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6237 = PseudoVMNOR_MM_MF2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6236 = PseudoVMNOR_MM_M8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6235 = PseudoVMNOR_MM_M4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6234 = PseudoVMNOR_MM_M2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6233 = PseudoVMNOR_MM_M1
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6232 = PseudoVMNAND_MM_MF8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6231 = PseudoVMNAND_MM_MF4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6230 = PseudoVMNAND_MM_MF2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6229 = PseudoVMNAND_MM_M8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6228 = PseudoVMNAND_MM_M4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6227 = PseudoVMNAND_MM_M2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6226 = PseudoVMNAND_MM_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6225 = PseudoVMIN_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6224 = PseudoVMIN_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6223 = PseudoVMIN_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6222 = PseudoVMIN_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6221 = PseudoVMIN_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6220 = PseudoVMIN_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #6219 = PseudoVMIN_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #6218 = PseudoVMIN_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #6217 = PseudoVMIN_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #6216 = PseudoVMIN_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #6215 = PseudoVMIN_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #6214 = PseudoVMIN_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6213 = PseudoVMIN_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6212 = PseudoVMIN_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6211 = PseudoVMIN_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6210 = PseudoVMIN_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6209 = PseudoVMIN_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6208 = PseudoVMIN_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6207 = PseudoVMIN_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6206 = PseudoVMIN_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #6205 = PseudoVMIN_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #6204 = PseudoVMIN_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #6203 = PseudoVMIN_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #6202 = PseudoVMIN_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #6201 = PseudoVMIN_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #6200 = PseudoVMIN_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6199 = PseudoVMIN_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6198 = PseudoVMIN_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6197 = PseudoVMINU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6196 = PseudoVMINU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6195 = PseudoVMINU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6194 = PseudoVMINU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6193 = PseudoVMINU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6192 = PseudoVMINU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #6191 = PseudoVMINU_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #6190 = PseudoVMINU_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #6189 = PseudoVMINU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #6188 = PseudoVMINU_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #6187 = PseudoVMINU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #6186 = PseudoVMINU_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #6185 = PseudoVMINU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #6184 = PseudoVMINU_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6183 = PseudoVMINU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6182 = PseudoVMINU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6181 = PseudoVMINU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6180 = PseudoVMINU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6179 = PseudoVMINU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6178 = PseudoVMINU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #6177 = PseudoVMINU_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #6176 = PseudoVMINU_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #6175 = PseudoVMINU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #6174 = PseudoVMINU_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #6173 = PseudoVMINU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #6172 = PseudoVMINU_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #6171 = PseudoVMINU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #6170 = PseudoVMINU_VV_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6169 = PseudoVMFNE_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6168 = PseudoVMFNE_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6167 = PseudoVMFNE_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6166 = PseudoVMFNE_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #6165 = PseudoVMFNE_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6164 = PseudoVMFNE_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #6163 = PseudoVMFNE_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6162 = PseudoVMFNE_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #6161 = PseudoVMFNE_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6160 = PseudoVMFNE_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6159 = PseudoVMFNE_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6158 = PseudoVMFNE_VV_M1
  { 7, &RISCVDescs.OperandInfo[5691] },  // Inst #6157 = PseudoVMFNE_VFPR64_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5686] },  // Inst #6156 = PseudoVMFNE_VFPR64_M8
  { 7, &RISCVDescs.OperandInfo[5679] },  // Inst #6155 = PseudoVMFNE_VFPR64_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5674] },  // Inst #6154 = PseudoVMFNE_VFPR64_M4
  { 7, &RISCVDescs.OperandInfo[5667] },  // Inst #6153 = PseudoVMFNE_VFPR64_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5662] },  // Inst #6152 = PseudoVMFNE_VFPR64_M2
  { 7, &RISCVDescs.OperandInfo[5655] },  // Inst #6151 = PseudoVMFNE_VFPR64_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5650] },  // Inst #6150 = PseudoVMFNE_VFPR64_M1
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #6149 = PseudoVMFNE_VFPR32_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #6148 = PseudoVMFNE_VFPR32_MF2
  { 7, &RISCVDescs.OperandInfo[5643] },  // Inst #6147 = PseudoVMFNE_VFPR32_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5638] },  // Inst #6146 = PseudoVMFNE_VFPR32_M8
  { 7, &RISCVDescs.OperandInfo[5631] },  // Inst #6145 = PseudoVMFNE_VFPR32_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5626] },  // Inst #6144 = PseudoVMFNE_VFPR32_M4
  { 7, &RISCVDescs.OperandInfo[5619] },  // Inst #6143 = PseudoVMFNE_VFPR32_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5614] },  // Inst #6142 = PseudoVMFNE_VFPR32_M2
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #6141 = PseudoVMFNE_VFPR32_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #6140 = PseudoVMFNE_VFPR32_M1
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6139 = PseudoVMFNE_VFPR16_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6138 = PseudoVMFNE_VFPR16_MF4
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6137 = PseudoVMFNE_VFPR16_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6136 = PseudoVMFNE_VFPR16_MF2
  { 7, &RISCVDescs.OperandInfo[5595] },  // Inst #6135 = PseudoVMFNE_VFPR16_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5590] },  // Inst #6134 = PseudoVMFNE_VFPR16_M8
  { 7, &RISCVDescs.OperandInfo[5583] },  // Inst #6133 = PseudoVMFNE_VFPR16_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5578] },  // Inst #6132 = PseudoVMFNE_VFPR16_M4
  { 7, &RISCVDescs.OperandInfo[5571] },  // Inst #6131 = PseudoVMFNE_VFPR16_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5566] },  // Inst #6130 = PseudoVMFNE_VFPR16_M2
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6129 = PseudoVMFNE_VFPR16_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6128 = PseudoVMFNE_VFPR16_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6127 = PseudoVMFLT_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6126 = PseudoVMFLT_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6125 = PseudoVMFLT_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6124 = PseudoVMFLT_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #6123 = PseudoVMFLT_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6122 = PseudoVMFLT_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #6121 = PseudoVMFLT_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6120 = PseudoVMFLT_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #6119 = PseudoVMFLT_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6118 = PseudoVMFLT_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6117 = PseudoVMFLT_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6116 = PseudoVMFLT_VV_M1
  { 7, &RISCVDescs.OperandInfo[5691] },  // Inst #6115 = PseudoVMFLT_VFPR64_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5686] },  // Inst #6114 = PseudoVMFLT_VFPR64_M8
  { 7, &RISCVDescs.OperandInfo[5679] },  // Inst #6113 = PseudoVMFLT_VFPR64_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5674] },  // Inst #6112 = PseudoVMFLT_VFPR64_M4
  { 7, &RISCVDescs.OperandInfo[5667] },  // Inst #6111 = PseudoVMFLT_VFPR64_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5662] },  // Inst #6110 = PseudoVMFLT_VFPR64_M2
  { 7, &RISCVDescs.OperandInfo[5655] },  // Inst #6109 = PseudoVMFLT_VFPR64_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5650] },  // Inst #6108 = PseudoVMFLT_VFPR64_M1
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #6107 = PseudoVMFLT_VFPR32_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #6106 = PseudoVMFLT_VFPR32_MF2
  { 7, &RISCVDescs.OperandInfo[5643] },  // Inst #6105 = PseudoVMFLT_VFPR32_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5638] },  // Inst #6104 = PseudoVMFLT_VFPR32_M8
  { 7, &RISCVDescs.OperandInfo[5631] },  // Inst #6103 = PseudoVMFLT_VFPR32_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5626] },  // Inst #6102 = PseudoVMFLT_VFPR32_M4
  { 7, &RISCVDescs.OperandInfo[5619] },  // Inst #6101 = PseudoVMFLT_VFPR32_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5614] },  // Inst #6100 = PseudoVMFLT_VFPR32_M2
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #6099 = PseudoVMFLT_VFPR32_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #6098 = PseudoVMFLT_VFPR32_M1
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6097 = PseudoVMFLT_VFPR16_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6096 = PseudoVMFLT_VFPR16_MF4
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6095 = PseudoVMFLT_VFPR16_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6094 = PseudoVMFLT_VFPR16_MF2
  { 7, &RISCVDescs.OperandInfo[5595] },  // Inst #6093 = PseudoVMFLT_VFPR16_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5590] },  // Inst #6092 = PseudoVMFLT_VFPR16_M8
  { 7, &RISCVDescs.OperandInfo[5583] },  // Inst #6091 = PseudoVMFLT_VFPR16_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5578] },  // Inst #6090 = PseudoVMFLT_VFPR16_M4
  { 7, &RISCVDescs.OperandInfo[5571] },  // Inst #6089 = PseudoVMFLT_VFPR16_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5566] },  // Inst #6088 = PseudoVMFLT_VFPR16_M2
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6087 = PseudoVMFLT_VFPR16_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6086 = PseudoVMFLT_VFPR16_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6085 = PseudoVMFLE_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6084 = PseudoVMFLE_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6083 = PseudoVMFLE_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6082 = PseudoVMFLE_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #6081 = PseudoVMFLE_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #6080 = PseudoVMFLE_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #6079 = PseudoVMFLE_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #6078 = PseudoVMFLE_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #6077 = PseudoVMFLE_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #6076 = PseudoVMFLE_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #6075 = PseudoVMFLE_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #6074 = PseudoVMFLE_VV_M1
  { 7, &RISCVDescs.OperandInfo[5691] },  // Inst #6073 = PseudoVMFLE_VFPR64_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5686] },  // Inst #6072 = PseudoVMFLE_VFPR64_M8
  { 7, &RISCVDescs.OperandInfo[5679] },  // Inst #6071 = PseudoVMFLE_VFPR64_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5674] },  // Inst #6070 = PseudoVMFLE_VFPR64_M4
  { 7, &RISCVDescs.OperandInfo[5667] },  // Inst #6069 = PseudoVMFLE_VFPR64_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5662] },  // Inst #6068 = PseudoVMFLE_VFPR64_M2
  { 7, &RISCVDescs.OperandInfo[5655] },  // Inst #6067 = PseudoVMFLE_VFPR64_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5650] },  // Inst #6066 = PseudoVMFLE_VFPR64_M1
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #6065 = PseudoVMFLE_VFPR32_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #6064 = PseudoVMFLE_VFPR32_MF2
  { 7, &RISCVDescs.OperandInfo[5643] },  // Inst #6063 = PseudoVMFLE_VFPR32_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5638] },  // Inst #6062 = PseudoVMFLE_VFPR32_M8
  { 7, &RISCVDescs.OperandInfo[5631] },  // Inst #6061 = PseudoVMFLE_VFPR32_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5626] },  // Inst #6060 = PseudoVMFLE_VFPR32_M4
  { 7, &RISCVDescs.OperandInfo[5619] },  // Inst #6059 = PseudoVMFLE_VFPR32_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5614] },  // Inst #6058 = PseudoVMFLE_VFPR32_M2
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #6057 = PseudoVMFLE_VFPR32_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #6056 = PseudoVMFLE_VFPR32_M1
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6055 = PseudoVMFLE_VFPR16_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6054 = PseudoVMFLE_VFPR16_MF4
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6053 = PseudoVMFLE_VFPR16_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6052 = PseudoVMFLE_VFPR16_MF2
  { 7, &RISCVDescs.OperandInfo[5595] },  // Inst #6051 = PseudoVMFLE_VFPR16_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5590] },  // Inst #6050 = PseudoVMFLE_VFPR16_M8
  { 7, &RISCVDescs.OperandInfo[5583] },  // Inst #6049 = PseudoVMFLE_VFPR16_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5578] },  // Inst #6048 = PseudoVMFLE_VFPR16_M4
  { 7, &RISCVDescs.OperandInfo[5571] },  // Inst #6047 = PseudoVMFLE_VFPR16_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5566] },  // Inst #6046 = PseudoVMFLE_VFPR16_M2
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6045 = PseudoVMFLE_VFPR16_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6044 = PseudoVMFLE_VFPR16_M1
  { 7, &RISCVDescs.OperandInfo[5691] },  // Inst #6043 = PseudoVMFGT_VFPR64_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5686] },  // Inst #6042 = PseudoVMFGT_VFPR64_M8
  { 7, &RISCVDescs.OperandInfo[5679] },  // Inst #6041 = PseudoVMFGT_VFPR64_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5674] },  // Inst #6040 = PseudoVMFGT_VFPR64_M4
  { 7, &RISCVDescs.OperandInfo[5667] },  // Inst #6039 = PseudoVMFGT_VFPR64_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5662] },  // Inst #6038 = PseudoVMFGT_VFPR64_M2
  { 7, &RISCVDescs.OperandInfo[5655] },  // Inst #6037 = PseudoVMFGT_VFPR64_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5650] },  // Inst #6036 = PseudoVMFGT_VFPR64_M1
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #6035 = PseudoVMFGT_VFPR32_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #6034 = PseudoVMFGT_VFPR32_MF2
  { 7, &RISCVDescs.OperandInfo[5643] },  // Inst #6033 = PseudoVMFGT_VFPR32_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5638] },  // Inst #6032 = PseudoVMFGT_VFPR32_M8
  { 7, &RISCVDescs.OperandInfo[5631] },  // Inst #6031 = PseudoVMFGT_VFPR32_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5626] },  // Inst #6030 = PseudoVMFGT_VFPR32_M4
  { 7, &RISCVDescs.OperandInfo[5619] },  // Inst #6029 = PseudoVMFGT_VFPR32_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5614] },  // Inst #6028 = PseudoVMFGT_VFPR32_M2
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #6027 = PseudoVMFGT_VFPR32_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #6026 = PseudoVMFGT_VFPR32_M1
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6025 = PseudoVMFGT_VFPR16_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6024 = PseudoVMFGT_VFPR16_MF4
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6023 = PseudoVMFGT_VFPR16_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6022 = PseudoVMFGT_VFPR16_MF2
  { 7, &RISCVDescs.OperandInfo[5595] },  // Inst #6021 = PseudoVMFGT_VFPR16_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5590] },  // Inst #6020 = PseudoVMFGT_VFPR16_M8
  { 7, &RISCVDescs.OperandInfo[5583] },  // Inst #6019 = PseudoVMFGT_VFPR16_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5578] },  // Inst #6018 = PseudoVMFGT_VFPR16_M4
  { 7, &RISCVDescs.OperandInfo[5571] },  // Inst #6017 = PseudoVMFGT_VFPR16_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5566] },  // Inst #6016 = PseudoVMFGT_VFPR16_M2
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #6015 = PseudoVMFGT_VFPR16_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #6014 = PseudoVMFGT_VFPR16_M1
  { 7, &RISCVDescs.OperandInfo[5691] },  // Inst #6013 = PseudoVMFGE_VFPR64_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5686] },  // Inst #6012 = PseudoVMFGE_VFPR64_M8
  { 7, &RISCVDescs.OperandInfo[5679] },  // Inst #6011 = PseudoVMFGE_VFPR64_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5674] },  // Inst #6010 = PseudoVMFGE_VFPR64_M4
  { 7, &RISCVDescs.OperandInfo[5667] },  // Inst #6009 = PseudoVMFGE_VFPR64_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5662] },  // Inst #6008 = PseudoVMFGE_VFPR64_M2
  { 7, &RISCVDescs.OperandInfo[5655] },  // Inst #6007 = PseudoVMFGE_VFPR64_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5650] },  // Inst #6006 = PseudoVMFGE_VFPR64_M1
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #6005 = PseudoVMFGE_VFPR32_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #6004 = PseudoVMFGE_VFPR32_MF2
  { 7, &RISCVDescs.OperandInfo[5643] },  // Inst #6003 = PseudoVMFGE_VFPR32_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5638] },  // Inst #6002 = PseudoVMFGE_VFPR32_M8
  { 7, &RISCVDescs.OperandInfo[5631] },  // Inst #6001 = PseudoVMFGE_VFPR32_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5626] },  // Inst #6000 = PseudoVMFGE_VFPR32_M4
  { 7, &RISCVDescs.OperandInfo[5619] },  // Inst #5999 = PseudoVMFGE_VFPR32_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5614] },  // Inst #5998 = PseudoVMFGE_VFPR32_M2
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #5997 = PseudoVMFGE_VFPR32_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #5996 = PseudoVMFGE_VFPR32_M1
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #5995 = PseudoVMFGE_VFPR16_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #5994 = PseudoVMFGE_VFPR16_MF4
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #5993 = PseudoVMFGE_VFPR16_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #5992 = PseudoVMFGE_VFPR16_MF2
  { 7, &RISCVDescs.OperandInfo[5595] },  // Inst #5991 = PseudoVMFGE_VFPR16_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5590] },  // Inst #5990 = PseudoVMFGE_VFPR16_M8
  { 7, &RISCVDescs.OperandInfo[5583] },  // Inst #5989 = PseudoVMFGE_VFPR16_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5578] },  // Inst #5988 = PseudoVMFGE_VFPR16_M4
  { 7, &RISCVDescs.OperandInfo[5571] },  // Inst #5987 = PseudoVMFGE_VFPR16_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5566] },  // Inst #5986 = PseudoVMFGE_VFPR16_M2
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #5985 = PseudoVMFGE_VFPR16_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #5984 = PseudoVMFGE_VFPR16_M1
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #5983 = PseudoVMFEQ_VV_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5982 = PseudoVMFEQ_VV_MF4
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #5981 = PseudoVMFEQ_VV_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5980 = PseudoVMFEQ_VV_MF2
  { 7, &RISCVDescs.OperandInfo[5719] },  // Inst #5979 = PseudoVMFEQ_VV_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #5978 = PseudoVMFEQ_VV_M8
  { 7, &RISCVDescs.OperandInfo[5712] },  // Inst #5977 = PseudoVMFEQ_VV_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #5976 = PseudoVMFEQ_VV_M4
  { 7, &RISCVDescs.OperandInfo[5705] },  // Inst #5975 = PseudoVMFEQ_VV_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #5974 = PseudoVMFEQ_VV_M2
  { 7, &RISCVDescs.OperandInfo[5698] },  // Inst #5973 = PseudoVMFEQ_VV_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5972 = PseudoVMFEQ_VV_M1
  { 7, &RISCVDescs.OperandInfo[5691] },  // Inst #5971 = PseudoVMFEQ_VFPR64_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5686] },  // Inst #5970 = PseudoVMFEQ_VFPR64_M8
  { 7, &RISCVDescs.OperandInfo[5679] },  // Inst #5969 = PseudoVMFEQ_VFPR64_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5674] },  // Inst #5968 = PseudoVMFEQ_VFPR64_M4
  { 7, &RISCVDescs.OperandInfo[5667] },  // Inst #5967 = PseudoVMFEQ_VFPR64_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5662] },  // Inst #5966 = PseudoVMFEQ_VFPR64_M2
  { 7, &RISCVDescs.OperandInfo[5655] },  // Inst #5965 = PseudoVMFEQ_VFPR64_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5650] },  // Inst #5964 = PseudoVMFEQ_VFPR64_M1
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #5963 = PseudoVMFEQ_VFPR32_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #5962 = PseudoVMFEQ_VFPR32_MF2
  { 7, &RISCVDescs.OperandInfo[5643] },  // Inst #5961 = PseudoVMFEQ_VFPR32_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5638] },  // Inst #5960 = PseudoVMFEQ_VFPR32_M8
  { 7, &RISCVDescs.OperandInfo[5631] },  // Inst #5959 = PseudoVMFEQ_VFPR32_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5626] },  // Inst #5958 = PseudoVMFEQ_VFPR32_M4
  { 7, &RISCVDescs.OperandInfo[5619] },  // Inst #5957 = PseudoVMFEQ_VFPR32_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5614] },  // Inst #5956 = PseudoVMFEQ_VFPR32_M2
  { 7, &RISCVDescs.OperandInfo[5607] },  // Inst #5955 = PseudoVMFEQ_VFPR32_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5602] },  // Inst #5954 = PseudoVMFEQ_VFPR32_M1
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #5953 = PseudoVMFEQ_VFPR16_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #5952 = PseudoVMFEQ_VFPR16_MF4
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #5951 = PseudoVMFEQ_VFPR16_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #5950 = PseudoVMFEQ_VFPR16_MF2
  { 7, &RISCVDescs.OperandInfo[5595] },  // Inst #5949 = PseudoVMFEQ_VFPR16_M8_MASK
  { 5, &RISCVDescs.OperandInfo[5590] },  // Inst #5948 = PseudoVMFEQ_VFPR16_M8
  { 7, &RISCVDescs.OperandInfo[5583] },  // Inst #5947 = PseudoVMFEQ_VFPR16_M4_MASK
  { 5, &RISCVDescs.OperandInfo[5578] },  // Inst #5946 = PseudoVMFEQ_VFPR16_M4
  { 7, &RISCVDescs.OperandInfo[5571] },  // Inst #5945 = PseudoVMFEQ_VFPR16_M2_MASK
  { 5, &RISCVDescs.OperandInfo[5566] },  // Inst #5944 = PseudoVMFEQ_VFPR16_M2
  { 7, &RISCVDescs.OperandInfo[5559] },  // Inst #5943 = PseudoVMFEQ_VFPR16_M1_MASK
  { 5, &RISCVDescs.OperandInfo[5554] },  // Inst #5942 = PseudoVMFEQ_VFPR16_M1
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #5941 = PseudoVMERGE_VXM_MF8
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #5940 = PseudoVMERGE_VXM_MF4
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #5939 = PseudoVMERGE_VXM_MF2
  { 7, &RISCVDescs.OperandInfo[630] },  // Inst #5938 = PseudoVMERGE_VXM_M8
  { 7, &RISCVDescs.OperandInfo[623] },  // Inst #5937 = PseudoVMERGE_VXM_M4
  { 7, &RISCVDescs.OperandInfo[616] },  // Inst #5936 = PseudoVMERGE_VXM_M2
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #5935 = PseudoVMERGE_VXM_M1
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #5934 = PseudoVMERGE_VVM_MF8
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #5933 = PseudoVMERGE_VVM_MF4
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #5932 = PseudoVMERGE_VVM_MF2
  { 7, &RISCVDescs.OperandInfo[602] },  // Inst #5931 = PseudoVMERGE_VVM_M8
  { 7, &RISCVDescs.OperandInfo[595] },  // Inst #5930 = PseudoVMERGE_VVM_M4
  { 7, &RISCVDescs.OperandInfo[588] },  // Inst #5929 = PseudoVMERGE_VVM_M2
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #5928 = PseudoVMERGE_VVM_M1
  { 7, &RISCVDescs.OperandInfo[553] },  // Inst #5927 = PseudoVMERGE_VIM_MF8
  { 7, &RISCVDescs.OperandInfo[553] },  // Inst #5926 = PseudoVMERGE_VIM_MF4
  { 7, &RISCVDescs.OperandInfo[553] },  // Inst #5925 = PseudoVMERGE_VIM_MF2
  { 7, &RISCVDescs.OperandInfo[574] },  // Inst #5924 = PseudoVMERGE_VIM_M8
  { 7, &RISCVDescs.OperandInfo[567] },  // Inst #5923 = PseudoVMERGE_VIM_M4
  { 7, &RISCVDescs.OperandInfo[560] },  // Inst #5922 = PseudoVMERGE_VIM_M2
  { 7, &RISCVDescs.OperandInfo[553] },  // Inst #5921 = PseudoVMERGE_VIM_M1
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #5920 = PseudoVMCLR_M_B8
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #5919 = PseudoVMCLR_M_B64
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #5918 = PseudoVMCLR_M_B4
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #5917 = PseudoVMCLR_M_B32
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #5916 = PseudoVMCLR_M_B2
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #5915 = PseudoVMCLR_M_B16
  { 3, &RISCVDescs.OperandInfo[5551] },  // Inst #5914 = PseudoVMCLR_M_B1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #5913 = PseudoVMAX_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #5912 = PseudoVMAX_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #5911 = PseudoVMAX_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #5910 = PseudoVMAX_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #5909 = PseudoVMAX_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #5908 = PseudoVMAX_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #5907 = PseudoVMAX_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #5906 = PseudoVMAX_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #5905 = PseudoVMAX_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #5904 = PseudoVMAX_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #5903 = PseudoVMAX_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #5902 = PseudoVMAX_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #5901 = PseudoVMAX_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #5900 = PseudoVMAX_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5899 = PseudoVMAX_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5898 = PseudoVMAX_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5897 = PseudoVMAX_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5896 = PseudoVMAX_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5895 = PseudoVMAX_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5894 = PseudoVMAX_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #5893 = PseudoVMAX_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #5892 = PseudoVMAX_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #5891 = PseudoVMAX_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #5890 = PseudoVMAX_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #5889 = PseudoVMAX_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #5888 = PseudoVMAX_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5887 = PseudoVMAX_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5886 = PseudoVMAX_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #5885 = PseudoVMAXU_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #5884 = PseudoVMAXU_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #5883 = PseudoVMAXU_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #5882 = PseudoVMAXU_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #5881 = PseudoVMAXU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #5880 = PseudoVMAXU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #5879 = PseudoVMAXU_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #5878 = PseudoVMAXU_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #5877 = PseudoVMAXU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #5876 = PseudoVMAXU_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #5875 = PseudoVMAXU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #5874 = PseudoVMAXU_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #5873 = PseudoVMAXU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #5872 = PseudoVMAXU_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5871 = PseudoVMAXU_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5870 = PseudoVMAXU_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5869 = PseudoVMAXU_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5868 = PseudoVMAXU_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5867 = PseudoVMAXU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5866 = PseudoVMAXU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #5865 = PseudoVMAXU_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #5864 = PseudoVMAXU_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #5863 = PseudoVMAXU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #5862 = PseudoVMAXU_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #5861 = PseudoVMAXU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #5860 = PseudoVMAXU_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5859 = PseudoVMAXU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5858 = PseudoVMAXU_VV_M1
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5857 = PseudoVMAND_MM_MF8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5856 = PseudoVMAND_MM_MF4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5855 = PseudoVMAND_MM_MF2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5854 = PseudoVMAND_MM_M8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5853 = PseudoVMAND_MM_M4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5852 = PseudoVMAND_MM_M2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5851 = PseudoVMAND_MM_M1
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5850 = PseudoVMANDN_MM_MF8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5849 = PseudoVMANDN_MM_MF4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5848 = PseudoVMANDN_MM_MF2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5847 = PseudoVMANDN_MM_M8
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5846 = PseudoVMANDN_MM_M4
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5845 = PseudoVMANDN_MM_M2
  { 5, &RISCVDescs.OperandInfo[5546] },  // Inst #5844 = PseudoVMANDN_MM_M1
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #5843 = PseudoVMADD_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #5842 = PseudoVMADD_VX_MF8
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #5841 = PseudoVMADD_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #5840 = PseudoVMADD_VX_MF4
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #5839 = PseudoVMADD_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #5838 = PseudoVMADD_VX_MF2
  { 8, &RISCVDescs.OperandInfo[5406] },  // Inst #5837 = PseudoVMADD_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[5399] },  // Inst #5836 = PseudoVMADD_VX_M8
  { 8, &RISCVDescs.OperandInfo[5391] },  // Inst #5835 = PseudoVMADD_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[5384] },  // Inst #5834 = PseudoVMADD_VX_M4
  { 8, &RISCVDescs.OperandInfo[5376] },  // Inst #5833 = PseudoVMADD_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5369] },  // Inst #5832 = PseudoVMADD_VX_M2
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #5831 = PseudoVMADD_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #5830 = PseudoVMADD_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5829 = PseudoVMADD_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5828 = PseudoVMADD_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5827 = PseudoVMADD_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5826 = PseudoVMADD_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5825 = PseudoVMADD_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5824 = PseudoVMADD_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #5823 = PseudoVMADD_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #5822 = PseudoVMADD_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #5821 = PseudoVMADD_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #5820 = PseudoVMADD_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #5819 = PseudoVMADD_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #5818 = PseudoVMADD_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5817 = PseudoVMADD_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5816 = PseudoVMADD_VV_M1
  { 5, &RISCVDescs.OperandInfo[5526] },  // Inst #5815 = PseudoVMADC_VX_MF8
  { 5, &RISCVDescs.OperandInfo[5526] },  // Inst #5814 = PseudoVMADC_VX_MF4
  { 5, &RISCVDescs.OperandInfo[5526] },  // Inst #5813 = PseudoVMADC_VX_MF2
  { 5, &RISCVDescs.OperandInfo[5541] },  // Inst #5812 = PseudoVMADC_VX_M8
  { 5, &RISCVDescs.OperandInfo[5536] },  // Inst #5811 = PseudoVMADC_VX_M4
  { 5, &RISCVDescs.OperandInfo[5531] },  // Inst #5810 = PseudoVMADC_VX_M2
  { 5, &RISCVDescs.OperandInfo[5526] },  // Inst #5809 = PseudoVMADC_VX_M1
  { 6, &RISCVDescs.OperandInfo[5502] },  // Inst #5808 = PseudoVMADC_VXM_MF8
  { 6, &RISCVDescs.OperandInfo[5502] },  // Inst #5807 = PseudoVMADC_VXM_MF4
  { 6, &RISCVDescs.OperandInfo[5502] },  // Inst #5806 = PseudoVMADC_VXM_MF2
  { 6, &RISCVDescs.OperandInfo[5520] },  // Inst #5805 = PseudoVMADC_VXM_M8
  { 6, &RISCVDescs.OperandInfo[5514] },  // Inst #5804 = PseudoVMADC_VXM_M4
  { 6, &RISCVDescs.OperandInfo[5508] },  // Inst #5803 = PseudoVMADC_VXM_M2
  { 6, &RISCVDescs.OperandInfo[5502] },  // Inst #5802 = PseudoVMADC_VXM_M1
  { 5, &RISCVDescs.OperandInfo[5482] },  // Inst #5801 = PseudoVMADC_VV_MF8
  { 5, &RISCVDescs.OperandInfo[5482] },  // Inst #5800 = PseudoVMADC_VV_MF4
  { 5, &RISCVDescs.OperandInfo[5482] },  // Inst #5799 = PseudoVMADC_VV_MF2
  { 5, &RISCVDescs.OperandInfo[5497] },  // Inst #5798 = PseudoVMADC_VV_M8
  { 5, &RISCVDescs.OperandInfo[5492] },  // Inst #5797 = PseudoVMADC_VV_M4
  { 5, &RISCVDescs.OperandInfo[5487] },  // Inst #5796 = PseudoVMADC_VV_M2
  { 5, &RISCVDescs.OperandInfo[5482] },  // Inst #5795 = PseudoVMADC_VV_M1
  { 6, &RISCVDescs.OperandInfo[5458] },  // Inst #5794 = PseudoVMADC_VVM_MF8
  { 6, &RISCVDescs.OperandInfo[5458] },  // Inst #5793 = PseudoVMADC_VVM_MF4
  { 6, &RISCVDescs.OperandInfo[5458] },  // Inst #5792 = PseudoVMADC_VVM_MF2
  { 6, &RISCVDescs.OperandInfo[5476] },  // Inst #5791 = PseudoVMADC_VVM_M8
  { 6, &RISCVDescs.OperandInfo[5470] },  // Inst #5790 = PseudoVMADC_VVM_M4
  { 6, &RISCVDescs.OperandInfo[5464] },  // Inst #5789 = PseudoVMADC_VVM_M2
  { 6, &RISCVDescs.OperandInfo[5458] },  // Inst #5788 = PseudoVMADC_VVM_M1
  { 5, &RISCVDescs.OperandInfo[5438] },  // Inst #5787 = PseudoVMADC_VI_MF8
  { 5, &RISCVDescs.OperandInfo[5438] },  // Inst #5786 = PseudoVMADC_VI_MF4
  { 5, &RISCVDescs.OperandInfo[5438] },  // Inst #5785 = PseudoVMADC_VI_MF2
  { 5, &RISCVDescs.OperandInfo[5453] },  // Inst #5784 = PseudoVMADC_VI_M8
  { 5, &RISCVDescs.OperandInfo[5448] },  // Inst #5783 = PseudoVMADC_VI_M4
  { 5, &RISCVDescs.OperandInfo[5443] },  // Inst #5782 = PseudoVMADC_VI_M2
  { 5, &RISCVDescs.OperandInfo[5438] },  // Inst #5781 = PseudoVMADC_VI_M1
  { 6, &RISCVDescs.OperandInfo[5414] },  // Inst #5780 = PseudoVMADC_VIM_MF8
  { 6, &RISCVDescs.OperandInfo[5414] },  // Inst #5779 = PseudoVMADC_VIM_MF4
  { 6, &RISCVDescs.OperandInfo[5414] },  // Inst #5778 = PseudoVMADC_VIM_MF2
  { 6, &RISCVDescs.OperandInfo[5432] },  // Inst #5777 = PseudoVMADC_VIM_M8
  { 6, &RISCVDescs.OperandInfo[5426] },  // Inst #5776 = PseudoVMADC_VIM_M4
  { 6, &RISCVDescs.OperandInfo[5420] },  // Inst #5775 = PseudoVMADC_VIM_M2
  { 6, &RISCVDescs.OperandInfo[5414] },  // Inst #5774 = PseudoVMADC_VIM_M1
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #5773 = PseudoVMACC_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #5772 = PseudoVMACC_VX_MF8
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #5771 = PseudoVMACC_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #5770 = PseudoVMACC_VX_MF4
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #5769 = PseudoVMACC_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #5768 = PseudoVMACC_VX_MF2
  { 8, &RISCVDescs.OperandInfo[5406] },  // Inst #5767 = PseudoVMACC_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[5399] },  // Inst #5766 = PseudoVMACC_VX_M8
  { 8, &RISCVDescs.OperandInfo[5391] },  // Inst #5765 = PseudoVMACC_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[5384] },  // Inst #5764 = PseudoVMACC_VX_M4
  { 8, &RISCVDescs.OperandInfo[5376] },  // Inst #5763 = PseudoVMACC_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5369] },  // Inst #5762 = PseudoVMACC_VX_M2
  { 8, &RISCVDescs.OperandInfo[5361] },  // Inst #5761 = PseudoVMACC_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5354] },  // Inst #5760 = PseudoVMACC_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5759 = PseudoVMACC_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5758 = PseudoVMACC_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5757 = PseudoVMACC_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5756 = PseudoVMACC_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5755 = PseudoVMACC_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5754 = PseudoVMACC_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #5753 = PseudoVMACC_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #5752 = PseudoVMACC_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #5751 = PseudoVMACC_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #5750 = PseudoVMACC_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #5749 = PseudoVMACC_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #5748 = PseudoVMACC_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #5747 = PseudoVMACC_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #5746 = PseudoVMACC_VV_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5745 = PseudoVLUXSEG8EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5744 = PseudoVLUXSEG8EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5743 = PseudoVLUXSEG8EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5742 = PseudoVLUXSEG8EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5741 = PseudoVLUXSEG8EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5740 = PseudoVLUXSEG8EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5739 = PseudoVLUXSEG8EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5738 = PseudoVLUXSEG8EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5737 = PseudoVLUXSEG8EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5736 = PseudoVLUXSEG8EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5735 = PseudoVLUXSEG8EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5734 = PseudoVLUXSEG8EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5733 = PseudoVLUXSEG8EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5732 = PseudoVLUXSEG8EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5731 = PseudoVLUXSEG8EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5730 = PseudoVLUXSEG8EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5729 = PseudoVLUXSEG8EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5728 = PseudoVLUXSEG8EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5727 = PseudoVLUXSEG8EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5726 = PseudoVLUXSEG8EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4813] },  // Inst #5725 = PseudoVLUXSEG8EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4806] },  // Inst #5724 = PseudoVLUXSEG8EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4798] },  // Inst #5723 = PseudoVLUXSEG8EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4791] },  // Inst #5722 = PseudoVLUXSEG8EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4798] },  // Inst #5721 = PseudoVLUXSEG8EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4791] },  // Inst #5720 = PseudoVLUXSEG8EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #5719 = PseudoVLUXSEG8EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #5718 = PseudoVLUXSEG8EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #5717 = PseudoVLUXSEG8EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #5716 = PseudoVLUXSEG8EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #5715 = PseudoVLUXSEG8EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #5714 = PseudoVLUXSEG8EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5713 = PseudoVLUXSEG8EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5712 = PseudoVLUXSEG8EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5711 = PseudoVLUXSEG8EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5710 = PseudoVLUXSEG8EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5709 = PseudoVLUXSEG8EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5708 = PseudoVLUXSEG8EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5707 = PseudoVLUXSEG8EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5706 = PseudoVLUXSEG8EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5705 = PseudoVLUXSEG8EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5704 = PseudoVLUXSEG8EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5703 = PseudoVLUXSEG8EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5702 = PseudoVLUXSEG8EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5701 = PseudoVLUXSEG8EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5700 = PseudoVLUXSEG8EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5699 = PseudoVLUXSEG8EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5698 = PseudoVLUXSEG8EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4798] },  // Inst #5697 = PseudoVLUXSEG8EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4791] },  // Inst #5696 = PseudoVLUXSEG8EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #5695 = PseudoVLUXSEG8EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #5694 = PseudoVLUXSEG8EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #5693 = PseudoVLUXSEG8EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #5692 = PseudoVLUXSEG8EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5691 = PseudoVLUXSEG8EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5690 = PseudoVLUXSEG8EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5689 = PseudoVLUXSEG8EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5688 = PseudoVLUXSEG8EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5687 = PseudoVLUXSEG8EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5686 = PseudoVLUXSEG8EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5685 = PseudoVLUXSEG8EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5684 = PseudoVLUXSEG8EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5683 = PseudoVLUXSEG8EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5682 = PseudoVLUXSEG8EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5681 = PseudoVLUXSEG8EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5680 = PseudoVLUXSEG8EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5679 = PseudoVLUXSEG8EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5678 = PseudoVLUXSEG8EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5677 = PseudoVLUXSEG8EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5676 = PseudoVLUXSEG8EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5675 = PseudoVLUXSEG8EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5674 = PseudoVLUXSEG8EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5673 = PseudoVLUXSEG8EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5672 = PseudoVLUXSEG8EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #5671 = PseudoVLUXSEG8EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #5670 = PseudoVLUXSEG8EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5669 = PseudoVLUXSEG8EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5668 = PseudoVLUXSEG8EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #5667 = PseudoVLUXSEG8EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #5666 = PseudoVLUXSEG8EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5665 = PseudoVLUXSEG7EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5664 = PseudoVLUXSEG7EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5663 = PseudoVLUXSEG7EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5662 = PseudoVLUXSEG7EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5661 = PseudoVLUXSEG7EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5660 = PseudoVLUXSEG7EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5659 = PseudoVLUXSEG7EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5658 = PseudoVLUXSEG7EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5657 = PseudoVLUXSEG7EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5656 = PseudoVLUXSEG7EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5655 = PseudoVLUXSEG7EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5654 = PseudoVLUXSEG7EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5653 = PseudoVLUXSEG7EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5652 = PseudoVLUXSEG7EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5651 = PseudoVLUXSEG7EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5650 = PseudoVLUXSEG7EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5649 = PseudoVLUXSEG7EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5648 = PseudoVLUXSEG7EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5647 = PseudoVLUXSEG7EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5646 = PseudoVLUXSEG7EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4753] },  // Inst #5645 = PseudoVLUXSEG7EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4746] },  // Inst #5644 = PseudoVLUXSEG7EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4738] },  // Inst #5643 = PseudoVLUXSEG7EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4731] },  // Inst #5642 = PseudoVLUXSEG7EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4738] },  // Inst #5641 = PseudoVLUXSEG7EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4731] },  // Inst #5640 = PseudoVLUXSEG7EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #5639 = PseudoVLUXSEG7EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #5638 = PseudoVLUXSEG7EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #5637 = PseudoVLUXSEG7EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #5636 = PseudoVLUXSEG7EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #5635 = PseudoVLUXSEG7EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #5634 = PseudoVLUXSEG7EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5633 = PseudoVLUXSEG7EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5632 = PseudoVLUXSEG7EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5631 = PseudoVLUXSEG7EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5630 = PseudoVLUXSEG7EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5629 = PseudoVLUXSEG7EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5628 = PseudoVLUXSEG7EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5627 = PseudoVLUXSEG7EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5626 = PseudoVLUXSEG7EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5625 = PseudoVLUXSEG7EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5624 = PseudoVLUXSEG7EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5623 = PseudoVLUXSEG7EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5622 = PseudoVLUXSEG7EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5621 = PseudoVLUXSEG7EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5620 = PseudoVLUXSEG7EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5619 = PseudoVLUXSEG7EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5618 = PseudoVLUXSEG7EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4738] },  // Inst #5617 = PseudoVLUXSEG7EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4731] },  // Inst #5616 = PseudoVLUXSEG7EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #5615 = PseudoVLUXSEG7EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #5614 = PseudoVLUXSEG7EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #5613 = PseudoVLUXSEG7EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #5612 = PseudoVLUXSEG7EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5611 = PseudoVLUXSEG7EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5610 = PseudoVLUXSEG7EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5609 = PseudoVLUXSEG7EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5608 = PseudoVLUXSEG7EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5607 = PseudoVLUXSEG7EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5606 = PseudoVLUXSEG7EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5605 = PseudoVLUXSEG7EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5604 = PseudoVLUXSEG7EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5603 = PseudoVLUXSEG7EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5602 = PseudoVLUXSEG7EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5601 = PseudoVLUXSEG7EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5600 = PseudoVLUXSEG7EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5599 = PseudoVLUXSEG7EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5598 = PseudoVLUXSEG7EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5597 = PseudoVLUXSEG7EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5596 = PseudoVLUXSEG7EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5595 = PseudoVLUXSEG7EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5594 = PseudoVLUXSEG7EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5593 = PseudoVLUXSEG7EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5592 = PseudoVLUXSEG7EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #5591 = PseudoVLUXSEG7EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #5590 = PseudoVLUXSEG7EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5589 = PseudoVLUXSEG7EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5588 = PseudoVLUXSEG7EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #5587 = PseudoVLUXSEG7EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #5586 = PseudoVLUXSEG7EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5585 = PseudoVLUXSEG6EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5584 = PseudoVLUXSEG6EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5583 = PseudoVLUXSEG6EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5582 = PseudoVLUXSEG6EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5581 = PseudoVLUXSEG6EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5580 = PseudoVLUXSEG6EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5579 = PseudoVLUXSEG6EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5578 = PseudoVLUXSEG6EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5577 = PseudoVLUXSEG6EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5576 = PseudoVLUXSEG6EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5575 = PseudoVLUXSEG6EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5574 = PseudoVLUXSEG6EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5573 = PseudoVLUXSEG6EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5572 = PseudoVLUXSEG6EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5571 = PseudoVLUXSEG6EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5570 = PseudoVLUXSEG6EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5569 = PseudoVLUXSEG6EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5568 = PseudoVLUXSEG6EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5567 = PseudoVLUXSEG6EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5566 = PseudoVLUXSEG6EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4693] },  // Inst #5565 = PseudoVLUXSEG6EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4686] },  // Inst #5564 = PseudoVLUXSEG6EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4678] },  // Inst #5563 = PseudoVLUXSEG6EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4671] },  // Inst #5562 = PseudoVLUXSEG6EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4678] },  // Inst #5561 = PseudoVLUXSEG6EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4671] },  // Inst #5560 = PseudoVLUXSEG6EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #5559 = PseudoVLUXSEG6EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #5558 = PseudoVLUXSEG6EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #5557 = PseudoVLUXSEG6EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #5556 = PseudoVLUXSEG6EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #5555 = PseudoVLUXSEG6EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #5554 = PseudoVLUXSEG6EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5553 = PseudoVLUXSEG6EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5552 = PseudoVLUXSEG6EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5551 = PseudoVLUXSEG6EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5550 = PseudoVLUXSEG6EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5549 = PseudoVLUXSEG6EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5548 = PseudoVLUXSEG6EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5547 = PseudoVLUXSEG6EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5546 = PseudoVLUXSEG6EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5545 = PseudoVLUXSEG6EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5544 = PseudoVLUXSEG6EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5543 = PseudoVLUXSEG6EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5542 = PseudoVLUXSEG6EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5541 = PseudoVLUXSEG6EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5540 = PseudoVLUXSEG6EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5539 = PseudoVLUXSEG6EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5538 = PseudoVLUXSEG6EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4678] },  // Inst #5537 = PseudoVLUXSEG6EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4671] },  // Inst #5536 = PseudoVLUXSEG6EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #5535 = PseudoVLUXSEG6EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #5534 = PseudoVLUXSEG6EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #5533 = PseudoVLUXSEG6EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #5532 = PseudoVLUXSEG6EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5531 = PseudoVLUXSEG6EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5530 = PseudoVLUXSEG6EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5529 = PseudoVLUXSEG6EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5528 = PseudoVLUXSEG6EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5527 = PseudoVLUXSEG6EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5526 = PseudoVLUXSEG6EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5525 = PseudoVLUXSEG6EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5524 = PseudoVLUXSEG6EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5523 = PseudoVLUXSEG6EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5522 = PseudoVLUXSEG6EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5521 = PseudoVLUXSEG6EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5520 = PseudoVLUXSEG6EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5519 = PseudoVLUXSEG6EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5518 = PseudoVLUXSEG6EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5517 = PseudoVLUXSEG6EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5516 = PseudoVLUXSEG6EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5515 = PseudoVLUXSEG6EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5514 = PseudoVLUXSEG6EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5513 = PseudoVLUXSEG6EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5512 = PseudoVLUXSEG6EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #5511 = PseudoVLUXSEG6EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #5510 = PseudoVLUXSEG6EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5509 = PseudoVLUXSEG6EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5508 = PseudoVLUXSEG6EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #5507 = PseudoVLUXSEG6EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #5506 = PseudoVLUXSEG6EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5505 = PseudoVLUXSEG5EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5504 = PseudoVLUXSEG5EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5503 = PseudoVLUXSEG5EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5502 = PseudoVLUXSEG5EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5501 = PseudoVLUXSEG5EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5500 = PseudoVLUXSEG5EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5499 = PseudoVLUXSEG5EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5498 = PseudoVLUXSEG5EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5497 = PseudoVLUXSEG5EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5496 = PseudoVLUXSEG5EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5495 = PseudoVLUXSEG5EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5494 = PseudoVLUXSEG5EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5493 = PseudoVLUXSEG5EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5492 = PseudoVLUXSEG5EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5491 = PseudoVLUXSEG5EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5490 = PseudoVLUXSEG5EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5489 = PseudoVLUXSEG5EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5488 = PseudoVLUXSEG5EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5487 = PseudoVLUXSEG5EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5486 = PseudoVLUXSEG5EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4633] },  // Inst #5485 = PseudoVLUXSEG5EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4626] },  // Inst #5484 = PseudoVLUXSEG5EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4618] },  // Inst #5483 = PseudoVLUXSEG5EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4611] },  // Inst #5482 = PseudoVLUXSEG5EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4618] },  // Inst #5481 = PseudoVLUXSEG5EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4611] },  // Inst #5480 = PseudoVLUXSEG5EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #5479 = PseudoVLUXSEG5EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #5478 = PseudoVLUXSEG5EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #5477 = PseudoVLUXSEG5EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #5476 = PseudoVLUXSEG5EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #5475 = PseudoVLUXSEG5EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #5474 = PseudoVLUXSEG5EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5473 = PseudoVLUXSEG5EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5472 = PseudoVLUXSEG5EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5471 = PseudoVLUXSEG5EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5470 = PseudoVLUXSEG5EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5469 = PseudoVLUXSEG5EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5468 = PseudoVLUXSEG5EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5467 = PseudoVLUXSEG5EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5466 = PseudoVLUXSEG5EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5465 = PseudoVLUXSEG5EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5464 = PseudoVLUXSEG5EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5463 = PseudoVLUXSEG5EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5462 = PseudoVLUXSEG5EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5461 = PseudoVLUXSEG5EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5460 = PseudoVLUXSEG5EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5459 = PseudoVLUXSEG5EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5458 = PseudoVLUXSEG5EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4618] },  // Inst #5457 = PseudoVLUXSEG5EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4611] },  // Inst #5456 = PseudoVLUXSEG5EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #5455 = PseudoVLUXSEG5EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #5454 = PseudoVLUXSEG5EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #5453 = PseudoVLUXSEG5EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #5452 = PseudoVLUXSEG5EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5451 = PseudoVLUXSEG5EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5450 = PseudoVLUXSEG5EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5449 = PseudoVLUXSEG5EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5448 = PseudoVLUXSEG5EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5447 = PseudoVLUXSEG5EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5446 = PseudoVLUXSEG5EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5445 = PseudoVLUXSEG5EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5444 = PseudoVLUXSEG5EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5443 = PseudoVLUXSEG5EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5442 = PseudoVLUXSEG5EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5441 = PseudoVLUXSEG5EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5440 = PseudoVLUXSEG5EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5439 = PseudoVLUXSEG5EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5438 = PseudoVLUXSEG5EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5437 = PseudoVLUXSEG5EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5436 = PseudoVLUXSEG5EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5435 = PseudoVLUXSEG5EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5434 = PseudoVLUXSEG5EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5433 = PseudoVLUXSEG5EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5432 = PseudoVLUXSEG5EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #5431 = PseudoVLUXSEG5EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #5430 = PseudoVLUXSEG5EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5429 = PseudoVLUXSEG5EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5428 = PseudoVLUXSEG5EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #5427 = PseudoVLUXSEG5EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #5426 = PseudoVLUXSEG5EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5425 = PseudoVLUXSEG4EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5424 = PseudoVLUXSEG4EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5423 = PseudoVLUXSEG4EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5422 = PseudoVLUXSEG4EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5421 = PseudoVLUXSEG4EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5420 = PseudoVLUXSEG4EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5419 = PseudoVLUXSEG4EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5418 = PseudoVLUXSEG4EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5417 = PseudoVLUXSEG4EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5416 = PseudoVLUXSEG4EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5415 = PseudoVLUXSEG4EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5414 = PseudoVLUXSEG4EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #5413 = PseudoVLUXSEG4EI8_V_MF4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #5412 = PseudoVLUXSEG4EI8_V_MF4_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5411 = PseudoVLUXSEG4EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5410 = PseudoVLUXSEG4EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5409 = PseudoVLUXSEG4EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5408 = PseudoVLUXSEG4EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #5407 = PseudoVLUXSEG4EI8_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #5406 = PseudoVLUXSEG4EI8_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5405 = PseudoVLUXSEG4EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5404 = PseudoVLUXSEG4EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4513] },  // Inst #5403 = PseudoVLUXSEG4EI8_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4506] },  // Inst #5402 = PseudoVLUXSEG4EI8_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #5401 = PseudoVLUXSEG4EI8_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #5400 = PseudoVLUXSEG4EI8_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5399 = PseudoVLUXSEG4EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5398 = PseudoVLUXSEG4EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4558] },  // Inst #5397 = PseudoVLUXSEG4EI64_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4551] },  // Inst #5396 = PseudoVLUXSEG4EI64_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4573] },  // Inst #5395 = PseudoVLUXSEG4EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4566] },  // Inst #5394 = PseudoVLUXSEG4EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4543] },  // Inst #5393 = PseudoVLUXSEG4EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4536] },  // Inst #5392 = PseudoVLUXSEG4EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4528] },  // Inst #5391 = PseudoVLUXSEG4EI64_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4521] },  // Inst #5390 = PseudoVLUXSEG4EI64_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4543] },  // Inst #5389 = PseudoVLUXSEG4EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4536] },  // Inst #5388 = PseudoVLUXSEG4EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #5387 = PseudoVLUXSEG4EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #5386 = PseudoVLUXSEG4EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #5385 = PseudoVLUXSEG4EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #5384 = PseudoVLUXSEG4EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4513] },  // Inst #5383 = PseudoVLUXSEG4EI64_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4506] },  // Inst #5382 = PseudoVLUXSEG4EI64_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #5381 = PseudoVLUXSEG4EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #5380 = PseudoVLUXSEG4EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5379 = PseudoVLUXSEG4EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5378 = PseudoVLUXSEG4EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5377 = PseudoVLUXSEG4EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5376 = PseudoVLUXSEG4EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5375 = PseudoVLUXSEG4EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5374 = PseudoVLUXSEG4EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5373 = PseudoVLUXSEG4EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5372 = PseudoVLUXSEG4EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5371 = PseudoVLUXSEG4EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5370 = PseudoVLUXSEG4EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5369 = PseudoVLUXSEG4EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5368 = PseudoVLUXSEG4EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5367 = PseudoVLUXSEG4EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5366 = PseudoVLUXSEG4EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5365 = PseudoVLUXSEG4EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5364 = PseudoVLUXSEG4EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4558] },  // Inst #5363 = PseudoVLUXSEG4EI32_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4551] },  // Inst #5362 = PseudoVLUXSEG4EI32_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4528] },  // Inst #5361 = PseudoVLUXSEG4EI32_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4521] },  // Inst #5360 = PseudoVLUXSEG4EI32_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4543] },  // Inst #5359 = PseudoVLUXSEG4EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4536] },  // Inst #5358 = PseudoVLUXSEG4EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #5357 = PseudoVLUXSEG4EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #5356 = PseudoVLUXSEG4EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4513] },  // Inst #5355 = PseudoVLUXSEG4EI32_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4506] },  // Inst #5354 = PseudoVLUXSEG4EI32_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #5353 = PseudoVLUXSEG4EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #5352 = PseudoVLUXSEG4EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5351 = PseudoVLUXSEG4EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5350 = PseudoVLUXSEG4EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5349 = PseudoVLUXSEG4EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5348 = PseudoVLUXSEG4EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #5347 = PseudoVLUXSEG4EI32_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #5346 = PseudoVLUXSEG4EI32_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5345 = PseudoVLUXSEG4EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5344 = PseudoVLUXSEG4EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5343 = PseudoVLUXSEG4EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5342 = PseudoVLUXSEG4EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5341 = PseudoVLUXSEG4EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5340 = PseudoVLUXSEG4EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5339 = PseudoVLUXSEG4EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5338 = PseudoVLUXSEG4EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5337 = PseudoVLUXSEG4EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5336 = PseudoVLUXSEG4EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5335 = PseudoVLUXSEG4EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5334 = PseudoVLUXSEG4EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5333 = PseudoVLUXSEG4EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5332 = PseudoVLUXSEG4EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #5331 = PseudoVLUXSEG4EI16_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #5330 = PseudoVLUXSEG4EI16_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5329 = PseudoVLUXSEG4EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5328 = PseudoVLUXSEG4EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4528] },  // Inst #5327 = PseudoVLUXSEG4EI16_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4521] },  // Inst #5326 = PseudoVLUXSEG4EI16_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4513] },  // Inst #5325 = PseudoVLUXSEG4EI16_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4506] },  // Inst #5324 = PseudoVLUXSEG4EI16_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #5323 = PseudoVLUXSEG4EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #5322 = PseudoVLUXSEG4EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5321 = PseudoVLUXSEG4EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5320 = PseudoVLUXSEG4EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #5319 = PseudoVLUXSEG4EI16_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #5318 = PseudoVLUXSEG4EI16_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #5317 = PseudoVLUXSEG4EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #5316 = PseudoVLUXSEG4EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5315 = PseudoVLUXSEG3EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5314 = PseudoVLUXSEG3EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5313 = PseudoVLUXSEG3EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5312 = PseudoVLUXSEG3EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5311 = PseudoVLUXSEG3EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5310 = PseudoVLUXSEG3EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5309 = PseudoVLUXSEG3EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5308 = PseudoVLUXSEG3EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5307 = PseudoVLUXSEG3EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5306 = PseudoVLUXSEG3EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5305 = PseudoVLUXSEG3EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5304 = PseudoVLUXSEG3EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #5303 = PseudoVLUXSEG3EI8_V_MF4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #5302 = PseudoVLUXSEG3EI8_V_MF4_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5301 = PseudoVLUXSEG3EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5300 = PseudoVLUXSEG3EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5299 = PseudoVLUXSEG3EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5298 = PseudoVLUXSEG3EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #5297 = PseudoVLUXSEG3EI8_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #5296 = PseudoVLUXSEG3EI8_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5295 = PseudoVLUXSEG3EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5294 = PseudoVLUXSEG3EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4393] },  // Inst #5293 = PseudoVLUXSEG3EI8_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4386] },  // Inst #5292 = PseudoVLUXSEG3EI8_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #5291 = PseudoVLUXSEG3EI8_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #5290 = PseudoVLUXSEG3EI8_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5289 = PseudoVLUXSEG3EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5288 = PseudoVLUXSEG3EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4438] },  // Inst #5287 = PseudoVLUXSEG3EI64_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4431] },  // Inst #5286 = PseudoVLUXSEG3EI64_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4453] },  // Inst #5285 = PseudoVLUXSEG3EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4446] },  // Inst #5284 = PseudoVLUXSEG3EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4423] },  // Inst #5283 = PseudoVLUXSEG3EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4416] },  // Inst #5282 = PseudoVLUXSEG3EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4408] },  // Inst #5281 = PseudoVLUXSEG3EI64_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4401] },  // Inst #5280 = PseudoVLUXSEG3EI64_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4423] },  // Inst #5279 = PseudoVLUXSEG3EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4416] },  // Inst #5278 = PseudoVLUXSEG3EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #5277 = PseudoVLUXSEG3EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #5276 = PseudoVLUXSEG3EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #5275 = PseudoVLUXSEG3EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #5274 = PseudoVLUXSEG3EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4393] },  // Inst #5273 = PseudoVLUXSEG3EI64_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4386] },  // Inst #5272 = PseudoVLUXSEG3EI64_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #5271 = PseudoVLUXSEG3EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #5270 = PseudoVLUXSEG3EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5269 = PseudoVLUXSEG3EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5268 = PseudoVLUXSEG3EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5267 = PseudoVLUXSEG3EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5266 = PseudoVLUXSEG3EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5265 = PseudoVLUXSEG3EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5264 = PseudoVLUXSEG3EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5263 = PseudoVLUXSEG3EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5262 = PseudoVLUXSEG3EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5261 = PseudoVLUXSEG3EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5260 = PseudoVLUXSEG3EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5259 = PseudoVLUXSEG3EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5258 = PseudoVLUXSEG3EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5257 = PseudoVLUXSEG3EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5256 = PseudoVLUXSEG3EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5255 = PseudoVLUXSEG3EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5254 = PseudoVLUXSEG3EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4438] },  // Inst #5253 = PseudoVLUXSEG3EI32_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4431] },  // Inst #5252 = PseudoVLUXSEG3EI32_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4408] },  // Inst #5251 = PseudoVLUXSEG3EI32_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4401] },  // Inst #5250 = PseudoVLUXSEG3EI32_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4423] },  // Inst #5249 = PseudoVLUXSEG3EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4416] },  // Inst #5248 = PseudoVLUXSEG3EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #5247 = PseudoVLUXSEG3EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #5246 = PseudoVLUXSEG3EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4393] },  // Inst #5245 = PseudoVLUXSEG3EI32_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4386] },  // Inst #5244 = PseudoVLUXSEG3EI32_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #5243 = PseudoVLUXSEG3EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #5242 = PseudoVLUXSEG3EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5241 = PseudoVLUXSEG3EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5240 = PseudoVLUXSEG3EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5239 = PseudoVLUXSEG3EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5238 = PseudoVLUXSEG3EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #5237 = PseudoVLUXSEG3EI32_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #5236 = PseudoVLUXSEG3EI32_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5235 = PseudoVLUXSEG3EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5234 = PseudoVLUXSEG3EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5233 = PseudoVLUXSEG3EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5232 = PseudoVLUXSEG3EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5231 = PseudoVLUXSEG3EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5230 = PseudoVLUXSEG3EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5229 = PseudoVLUXSEG3EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5228 = PseudoVLUXSEG3EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5227 = PseudoVLUXSEG3EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5226 = PseudoVLUXSEG3EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5225 = PseudoVLUXSEG3EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5224 = PseudoVLUXSEG3EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5223 = PseudoVLUXSEG3EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5222 = PseudoVLUXSEG3EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #5221 = PseudoVLUXSEG3EI16_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #5220 = PseudoVLUXSEG3EI16_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5219 = PseudoVLUXSEG3EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5218 = PseudoVLUXSEG3EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4408] },  // Inst #5217 = PseudoVLUXSEG3EI16_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4401] },  // Inst #5216 = PseudoVLUXSEG3EI16_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4393] },  // Inst #5215 = PseudoVLUXSEG3EI16_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4386] },  // Inst #5214 = PseudoVLUXSEG3EI16_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #5213 = PseudoVLUXSEG3EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #5212 = PseudoVLUXSEG3EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5211 = PseudoVLUXSEG3EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5210 = PseudoVLUXSEG3EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #5209 = PseudoVLUXSEG3EI16_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #5208 = PseudoVLUXSEG3EI16_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #5207 = PseudoVLUXSEG3EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #5206 = PseudoVLUXSEG3EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5205 = PseudoVLUXSEG2EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5204 = PseudoVLUXSEG2EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5203 = PseudoVLUXSEG2EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5202 = PseudoVLUXSEG2EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5201 = PseudoVLUXSEG2EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5200 = PseudoVLUXSEG2EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5199 = PseudoVLUXSEG2EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5198 = PseudoVLUXSEG2EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5197 = PseudoVLUXSEG2EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5196 = PseudoVLUXSEG2EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5195 = PseudoVLUXSEG2EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5194 = PseudoVLUXSEG2EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #5193 = PseudoVLUXSEG2EI8_V_MF4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #5192 = PseudoVLUXSEG2EI8_V_MF4_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5191 = PseudoVLUXSEG2EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5190 = PseudoVLUXSEG2EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5189 = PseudoVLUXSEG2EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5188 = PseudoVLUXSEG2EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4198] },  // Inst #5187 = PseudoVLUXSEG2EI8_V_MF2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4191] },  // Inst #5186 = PseudoVLUXSEG2EI8_V_MF2_M4
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #5185 = PseudoVLUXSEG2EI8_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #5184 = PseudoVLUXSEG2EI8_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5183 = PseudoVLUXSEG2EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5182 = PseudoVLUXSEG2EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4273] },  // Inst #5181 = PseudoVLUXSEG2EI8_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4266] },  // Inst #5180 = PseudoVLUXSEG2EI8_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4243] },  // Inst #5179 = PseudoVLUXSEG2EI8_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4236] },  // Inst #5178 = PseudoVLUXSEG2EI8_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[4228] },  // Inst #5177 = PseudoVLUXSEG2EI8_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4221] },  // Inst #5176 = PseudoVLUXSEG2EI8_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4198] },  // Inst #5175 = PseudoVLUXSEG2EI8_V_M1_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4191] },  // Inst #5174 = PseudoVLUXSEG2EI8_V_M1_M4
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #5173 = PseudoVLUXSEG2EI8_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #5172 = PseudoVLUXSEG2EI8_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5171 = PseudoVLUXSEG2EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5170 = PseudoVLUXSEG2EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4288] },  // Inst #5169 = PseudoVLUXSEG2EI64_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4281] },  // Inst #5168 = PseudoVLUXSEG2EI64_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4318] },  // Inst #5167 = PseudoVLUXSEG2EI64_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4311] },  // Inst #5166 = PseudoVLUXSEG2EI64_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4333] },  // Inst #5165 = PseudoVLUXSEG2EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4326] },  // Inst #5164 = PseudoVLUXSEG2EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4303] },  // Inst #5163 = PseudoVLUXSEG2EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4296] },  // Inst #5162 = PseudoVLUXSEG2EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4273] },  // Inst #5161 = PseudoVLUXSEG2EI64_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4266] },  // Inst #5160 = PseudoVLUXSEG2EI64_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4258] },  // Inst #5159 = PseudoVLUXSEG2EI64_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4251] },  // Inst #5158 = PseudoVLUXSEG2EI64_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4303] },  // Inst #5157 = PseudoVLUXSEG2EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4296] },  // Inst #5156 = PseudoVLUXSEG2EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #5155 = PseudoVLUXSEG2EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #5154 = PseudoVLUXSEG2EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #5153 = PseudoVLUXSEG2EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #5152 = PseudoVLUXSEG2EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4228] },  // Inst #5151 = PseudoVLUXSEG2EI64_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4221] },  // Inst #5150 = PseudoVLUXSEG2EI64_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #5149 = PseudoVLUXSEG2EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #5148 = PseudoVLUXSEG2EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5147 = PseudoVLUXSEG2EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5146 = PseudoVLUXSEG2EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5145 = PseudoVLUXSEG2EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5144 = PseudoVLUXSEG2EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5143 = PseudoVLUXSEG2EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5142 = PseudoVLUXSEG2EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5141 = PseudoVLUXSEG2EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5140 = PseudoVLUXSEG2EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5139 = PseudoVLUXSEG2EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5138 = PseudoVLUXSEG2EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5137 = PseudoVLUXSEG2EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5136 = PseudoVLUXSEG2EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5135 = PseudoVLUXSEG2EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5134 = PseudoVLUXSEG2EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5133 = PseudoVLUXSEG2EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5132 = PseudoVLUXSEG2EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4288] },  // Inst #5131 = PseudoVLUXSEG2EI32_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4281] },  // Inst #5130 = PseudoVLUXSEG2EI32_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4318] },  // Inst #5129 = PseudoVLUXSEG2EI32_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4311] },  // Inst #5128 = PseudoVLUXSEG2EI32_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4273] },  // Inst #5127 = PseudoVLUXSEG2EI32_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4266] },  // Inst #5126 = PseudoVLUXSEG2EI32_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4258] },  // Inst #5125 = PseudoVLUXSEG2EI32_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4251] },  // Inst #5124 = PseudoVLUXSEG2EI32_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4303] },  // Inst #5123 = PseudoVLUXSEG2EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4296] },  // Inst #5122 = PseudoVLUXSEG2EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #5121 = PseudoVLUXSEG2EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #5120 = PseudoVLUXSEG2EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4243] },  // Inst #5119 = PseudoVLUXSEG2EI32_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4236] },  // Inst #5118 = PseudoVLUXSEG2EI32_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[4228] },  // Inst #5117 = PseudoVLUXSEG2EI32_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4221] },  // Inst #5116 = PseudoVLUXSEG2EI32_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #5115 = PseudoVLUXSEG2EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #5114 = PseudoVLUXSEG2EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5113 = PseudoVLUXSEG2EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5112 = PseudoVLUXSEG2EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5111 = PseudoVLUXSEG2EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5110 = PseudoVLUXSEG2EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #5109 = PseudoVLUXSEG2EI32_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #5108 = PseudoVLUXSEG2EI32_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5107 = PseudoVLUXSEG2EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5106 = PseudoVLUXSEG2EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5105 = PseudoVLUXSEG2EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5104 = PseudoVLUXSEG2EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5103 = PseudoVLUXSEG2EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5102 = PseudoVLUXSEG2EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5101 = PseudoVLUXSEG2EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5100 = PseudoVLUXSEG2EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5099 = PseudoVLUXSEG2EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5098 = PseudoVLUXSEG2EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5097 = PseudoVLUXSEG2EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5096 = PseudoVLUXSEG2EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5095 = PseudoVLUXSEG2EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5094 = PseudoVLUXSEG2EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #5093 = PseudoVLUXSEG2EI16_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #5092 = PseudoVLUXSEG2EI16_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5091 = PseudoVLUXSEG2EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5090 = PseudoVLUXSEG2EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4288] },  // Inst #5089 = PseudoVLUXSEG2EI16_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4281] },  // Inst #5088 = PseudoVLUXSEG2EI16_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4273] },  // Inst #5087 = PseudoVLUXSEG2EI16_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4266] },  // Inst #5086 = PseudoVLUXSEG2EI16_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4258] },  // Inst #5085 = PseudoVLUXSEG2EI16_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4251] },  // Inst #5084 = PseudoVLUXSEG2EI16_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4243] },  // Inst #5083 = PseudoVLUXSEG2EI16_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4236] },  // Inst #5082 = PseudoVLUXSEG2EI16_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[4228] },  // Inst #5081 = PseudoVLUXSEG2EI16_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4221] },  // Inst #5080 = PseudoVLUXSEG2EI16_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #5079 = PseudoVLUXSEG2EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #5078 = PseudoVLUXSEG2EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5077 = PseudoVLUXSEG2EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5076 = PseudoVLUXSEG2EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4198] },  // Inst #5075 = PseudoVLUXSEG2EI16_V_M1_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4191] },  // Inst #5074 = PseudoVLUXSEG2EI16_V_M1_M4
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #5073 = PseudoVLUXSEG2EI16_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #5072 = PseudoVLUXSEG2EI16_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #5071 = PseudoVLUXSEG2EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #5070 = PseudoVLUXSEG2EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #5069 = PseudoVLUXEI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #5068 = PseudoVLUXEI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #5067 = PseudoVLUXEI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #5066 = PseudoVLUXEI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #5065 = PseudoVLUXEI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #5064 = PseudoVLUXEI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #5063 = PseudoVLUXEI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #5062 = PseudoVLUXEI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #5061 = PseudoVLUXEI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #5060 = PseudoVLUXEI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #5059 = PseudoVLUXEI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #5058 = PseudoVLUXEI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #5057 = PseudoVLUXEI8_V_MF4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #5056 = PseudoVLUXEI8_V_MF4_M2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #5055 = PseudoVLUXEI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #5054 = PseudoVLUXEI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #5053 = PseudoVLUXEI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #5052 = PseudoVLUXEI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[3943] },  // Inst #5051 = PseudoVLUXEI8_V_MF2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3936] },  // Inst #5050 = PseudoVLUXEI8_V_MF2_M4
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #5049 = PseudoVLUXEI8_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #5048 = PseudoVLUXEI8_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #5047 = PseudoVLUXEI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #5046 = PseudoVLUXEI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4093] },  // Inst #5045 = PseudoVLUXEI8_V_M8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4086] },  // Inst #5044 = PseudoVLUXEI8_V_M8_M8
  { 8, &RISCVDescs.OperandInfo[4063] },  // Inst #5043 = PseudoVLUXEI8_V_M4_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4056] },  // Inst #5042 = PseudoVLUXEI8_V_M4_M8
  { 8, &RISCVDescs.OperandInfo[4048] },  // Inst #5041 = PseudoVLUXEI8_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4041] },  // Inst #5040 = PseudoVLUXEI8_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4018] },  // Inst #5039 = PseudoVLUXEI8_V_M2_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4011] },  // Inst #5038 = PseudoVLUXEI8_V_M2_M8
  { 8, &RISCVDescs.OperandInfo[4003] },  // Inst #5037 = PseudoVLUXEI8_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3996] },  // Inst #5036 = PseudoVLUXEI8_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[3988] },  // Inst #5035 = PseudoVLUXEI8_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3981] },  // Inst #5034 = PseudoVLUXEI8_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4153] },  // Inst #5033 = PseudoVLUXEI8_V_M1_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4146] },  // Inst #5032 = PseudoVLUXEI8_V_M1_M8
  { 8, &RISCVDescs.OperandInfo[3943] },  // Inst #5031 = PseudoVLUXEI8_V_M1_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3936] },  // Inst #5030 = PseudoVLUXEI8_V_M1_M4
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #5029 = PseudoVLUXEI8_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #5028 = PseudoVLUXEI8_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #5027 = PseudoVLUXEI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #5026 = PseudoVLUXEI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4093] },  // Inst #5025 = PseudoVLUXEI64_V_M8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4086] },  // Inst #5024 = PseudoVLUXEI64_V_M8_M8
  { 8, &RISCVDescs.OperandInfo[4078] },  // Inst #5023 = PseudoVLUXEI64_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4071] },  // Inst #5022 = PseudoVLUXEI64_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4123] },  // Inst #5021 = PseudoVLUXEI64_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4116] },  // Inst #5020 = PseudoVLUXEI64_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4138] },  // Inst #5019 = PseudoVLUXEI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4131] },  // Inst #5018 = PseudoVLUXEI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4108] },  // Inst #5017 = PseudoVLUXEI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4101] },  // Inst #5016 = PseudoVLUXEI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4048] },  // Inst #5015 = PseudoVLUXEI64_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4041] },  // Inst #5014 = PseudoVLUXEI64_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4033] },  // Inst #5013 = PseudoVLUXEI64_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4026] },  // Inst #5012 = PseudoVLUXEI64_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4108] },  // Inst #5011 = PseudoVLUXEI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4101] },  // Inst #5010 = PseudoVLUXEI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #5009 = PseudoVLUXEI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #5008 = PseudoVLUXEI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #5007 = PseudoVLUXEI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #5006 = PseudoVLUXEI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[3988] },  // Inst #5005 = PseudoVLUXEI64_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3981] },  // Inst #5004 = PseudoVLUXEI64_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #5003 = PseudoVLUXEI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #5002 = PseudoVLUXEI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #5001 = PseudoVLUXEI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #5000 = PseudoVLUXEI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4999 = PseudoVLUXEI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4998 = PseudoVLUXEI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4997 = PseudoVLUXEI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4996 = PseudoVLUXEI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #4995 = PseudoVLUXEI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #4994 = PseudoVLUXEI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4993 = PseudoVLUXEI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4992 = PseudoVLUXEI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4991 = PseudoVLUXEI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4990 = PseudoVLUXEI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #4989 = PseudoVLUXEI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #4988 = PseudoVLUXEI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4987 = PseudoVLUXEI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4986 = PseudoVLUXEI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4093] },  // Inst #4985 = PseudoVLUXEI32_V_M8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4086] },  // Inst #4984 = PseudoVLUXEI32_V_M8_M8
  { 8, &RISCVDescs.OperandInfo[4078] },  // Inst #4983 = PseudoVLUXEI32_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4071] },  // Inst #4982 = PseudoVLUXEI32_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4123] },  // Inst #4981 = PseudoVLUXEI32_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4116] },  // Inst #4980 = PseudoVLUXEI32_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4063] },  // Inst #4979 = PseudoVLUXEI32_V_M4_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4056] },  // Inst #4978 = PseudoVLUXEI32_V_M4_M8
  { 8, &RISCVDescs.OperandInfo[4048] },  // Inst #4977 = PseudoVLUXEI32_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4041] },  // Inst #4976 = PseudoVLUXEI32_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4033] },  // Inst #4975 = PseudoVLUXEI32_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4026] },  // Inst #4974 = PseudoVLUXEI32_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4108] },  // Inst #4973 = PseudoVLUXEI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4101] },  // Inst #4972 = PseudoVLUXEI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #4971 = PseudoVLUXEI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #4970 = PseudoVLUXEI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4003] },  // Inst #4969 = PseudoVLUXEI32_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3996] },  // Inst #4968 = PseudoVLUXEI32_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[3988] },  // Inst #4967 = PseudoVLUXEI32_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3981] },  // Inst #4966 = PseudoVLUXEI32_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #4965 = PseudoVLUXEI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #4964 = PseudoVLUXEI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4963 = PseudoVLUXEI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4962 = PseudoVLUXEI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4961 = PseudoVLUXEI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4960 = PseudoVLUXEI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #4959 = PseudoVLUXEI32_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #4958 = PseudoVLUXEI32_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #4957 = PseudoVLUXEI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #4956 = PseudoVLUXEI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4955 = PseudoVLUXEI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4954 = PseudoVLUXEI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #4953 = PseudoVLUXEI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #4952 = PseudoVLUXEI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4951 = PseudoVLUXEI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4950 = PseudoVLUXEI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4949 = PseudoVLUXEI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4948 = PseudoVLUXEI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4947 = PseudoVLUXEI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4946 = PseudoVLUXEI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #4945 = PseudoVLUXEI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #4944 = PseudoVLUXEI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #4943 = PseudoVLUXEI16_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #4942 = PseudoVLUXEI16_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4941 = PseudoVLUXEI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4940 = PseudoVLUXEI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4093] },  // Inst #4939 = PseudoVLUXEI16_V_M8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4086] },  // Inst #4938 = PseudoVLUXEI16_V_M8_M8
  { 8, &RISCVDescs.OperandInfo[4078] },  // Inst #4937 = PseudoVLUXEI16_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4071] },  // Inst #4936 = PseudoVLUXEI16_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4063] },  // Inst #4935 = PseudoVLUXEI16_V_M4_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4056] },  // Inst #4934 = PseudoVLUXEI16_V_M4_M8
  { 8, &RISCVDescs.OperandInfo[4048] },  // Inst #4933 = PseudoVLUXEI16_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4041] },  // Inst #4932 = PseudoVLUXEI16_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4033] },  // Inst #4931 = PseudoVLUXEI16_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4026] },  // Inst #4930 = PseudoVLUXEI16_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4018] },  // Inst #4929 = PseudoVLUXEI16_V_M2_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4011] },  // Inst #4928 = PseudoVLUXEI16_V_M2_M8
  { 8, &RISCVDescs.OperandInfo[4003] },  // Inst #4927 = PseudoVLUXEI16_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3996] },  // Inst #4926 = PseudoVLUXEI16_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[3988] },  // Inst #4925 = PseudoVLUXEI16_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3981] },  // Inst #4924 = PseudoVLUXEI16_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #4923 = PseudoVLUXEI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #4922 = PseudoVLUXEI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #4921 = PseudoVLUXEI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #4920 = PseudoVLUXEI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[3943] },  // Inst #4919 = PseudoVLUXEI16_V_M1_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3936] },  // Inst #4918 = PseudoVLUXEI16_V_M1_M4
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #4917 = PseudoVLUXEI16_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #4916 = PseudoVLUXEI16_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #4915 = PseudoVLUXEI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #4914 = PseudoVLUXEI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4913 = PseudoVLSSEG8E8_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4912 = PseudoVLSSEG8E8_V_MF8
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4911 = PseudoVLSSEG8E8_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4910 = PseudoVLSSEG8E8_V_MF4
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4909 = PseudoVLSSEG8E8_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4908 = PseudoVLSSEG8E8_V_MF2
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4907 = PseudoVLSSEG8E8_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4906 = PseudoVLSSEG8E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4905 = PseudoVLSSEG8E64_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4904 = PseudoVLSSEG8E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4903 = PseudoVLSSEG8E32_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4902 = PseudoVLSSEG8E32_V_MF2
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4901 = PseudoVLSSEG8E32_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4900 = PseudoVLSSEG8E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4899 = PseudoVLSSEG8E16_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4898 = PseudoVLSSEG8E16_V_MF4
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4897 = PseudoVLSSEG8E16_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4896 = PseudoVLSSEG8E16_V_MF2
  { 8, &RISCVDescs.OperandInfo[5346] },  // Inst #4895 = PseudoVLSSEG8E16_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5339] },  // Inst #4894 = PseudoVLSSEG8E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4893 = PseudoVLSSEG7E8_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4892 = PseudoVLSSEG7E8_V_MF8
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4891 = PseudoVLSSEG7E8_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4890 = PseudoVLSSEG7E8_V_MF4
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4889 = PseudoVLSSEG7E8_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4888 = PseudoVLSSEG7E8_V_MF2
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4887 = PseudoVLSSEG7E8_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4886 = PseudoVLSSEG7E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4885 = PseudoVLSSEG7E64_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4884 = PseudoVLSSEG7E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4883 = PseudoVLSSEG7E32_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4882 = PseudoVLSSEG7E32_V_MF2
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4881 = PseudoVLSSEG7E32_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4880 = PseudoVLSSEG7E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4879 = PseudoVLSSEG7E16_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4878 = PseudoVLSSEG7E16_V_MF4
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4877 = PseudoVLSSEG7E16_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4876 = PseudoVLSSEG7E16_V_MF2
  { 8, &RISCVDescs.OperandInfo[5331] },  // Inst #4875 = PseudoVLSSEG7E16_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5324] },  // Inst #4874 = PseudoVLSSEG7E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4873 = PseudoVLSSEG6E8_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4872 = PseudoVLSSEG6E8_V_MF8
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4871 = PseudoVLSSEG6E8_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4870 = PseudoVLSSEG6E8_V_MF4
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4869 = PseudoVLSSEG6E8_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4868 = PseudoVLSSEG6E8_V_MF2
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4867 = PseudoVLSSEG6E8_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4866 = PseudoVLSSEG6E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4865 = PseudoVLSSEG6E64_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4864 = PseudoVLSSEG6E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4863 = PseudoVLSSEG6E32_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4862 = PseudoVLSSEG6E32_V_MF2
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4861 = PseudoVLSSEG6E32_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4860 = PseudoVLSSEG6E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4859 = PseudoVLSSEG6E16_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4858 = PseudoVLSSEG6E16_V_MF4
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4857 = PseudoVLSSEG6E16_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4856 = PseudoVLSSEG6E16_V_MF2
  { 8, &RISCVDescs.OperandInfo[5316] },  // Inst #4855 = PseudoVLSSEG6E16_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5309] },  // Inst #4854 = PseudoVLSSEG6E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4853 = PseudoVLSSEG5E8_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4852 = PseudoVLSSEG5E8_V_MF8
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4851 = PseudoVLSSEG5E8_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4850 = PseudoVLSSEG5E8_V_MF4
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4849 = PseudoVLSSEG5E8_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4848 = PseudoVLSSEG5E8_V_MF2
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4847 = PseudoVLSSEG5E8_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4846 = PseudoVLSSEG5E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4845 = PseudoVLSSEG5E64_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4844 = PseudoVLSSEG5E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4843 = PseudoVLSSEG5E32_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4842 = PseudoVLSSEG5E32_V_MF2
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4841 = PseudoVLSSEG5E32_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4840 = PseudoVLSSEG5E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4839 = PseudoVLSSEG5E16_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4838 = PseudoVLSSEG5E16_V_MF4
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4837 = PseudoVLSSEG5E16_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4836 = PseudoVLSSEG5E16_V_MF2
  { 8, &RISCVDescs.OperandInfo[5301] },  // Inst #4835 = PseudoVLSSEG5E16_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5294] },  // Inst #4834 = PseudoVLSSEG5E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4833 = PseudoVLSSEG4E8_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4832 = PseudoVLSSEG4E8_V_MF8
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4831 = PseudoVLSSEG4E8_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4830 = PseudoVLSSEG4E8_V_MF4
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4829 = PseudoVLSSEG4E8_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4828 = PseudoVLSSEG4E8_V_MF2
  { 8, &RISCVDescs.OperandInfo[5286] },  // Inst #4827 = PseudoVLSSEG4E8_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5279] },  // Inst #4826 = PseudoVLSSEG4E8_V_M2
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4825 = PseudoVLSSEG4E8_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4824 = PseudoVLSSEG4E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5286] },  // Inst #4823 = PseudoVLSSEG4E64_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5279] },  // Inst #4822 = PseudoVLSSEG4E64_V_M2
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4821 = PseudoVLSSEG4E64_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4820 = PseudoVLSSEG4E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4819 = PseudoVLSSEG4E32_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4818 = PseudoVLSSEG4E32_V_MF2
  { 8, &RISCVDescs.OperandInfo[5286] },  // Inst #4817 = PseudoVLSSEG4E32_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5279] },  // Inst #4816 = PseudoVLSSEG4E32_V_M2
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4815 = PseudoVLSSEG4E32_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4814 = PseudoVLSSEG4E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4813 = PseudoVLSSEG4E16_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4812 = PseudoVLSSEG4E16_V_MF4
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4811 = PseudoVLSSEG4E16_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4810 = PseudoVLSSEG4E16_V_MF2
  { 8, &RISCVDescs.OperandInfo[5286] },  // Inst #4809 = PseudoVLSSEG4E16_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5279] },  // Inst #4808 = PseudoVLSSEG4E16_V_M2
  { 8, &RISCVDescs.OperandInfo[5271] },  // Inst #4807 = PseudoVLSSEG4E16_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5264] },  // Inst #4806 = PseudoVLSSEG4E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4805 = PseudoVLSSEG3E8_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4804 = PseudoVLSSEG3E8_V_MF8
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4803 = PseudoVLSSEG3E8_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4802 = PseudoVLSSEG3E8_V_MF4
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4801 = PseudoVLSSEG3E8_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4800 = PseudoVLSSEG3E8_V_MF2
  { 8, &RISCVDescs.OperandInfo[5256] },  // Inst #4799 = PseudoVLSSEG3E8_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5249] },  // Inst #4798 = PseudoVLSSEG3E8_V_M2
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4797 = PseudoVLSSEG3E8_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4796 = PseudoVLSSEG3E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5256] },  // Inst #4795 = PseudoVLSSEG3E64_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5249] },  // Inst #4794 = PseudoVLSSEG3E64_V_M2
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4793 = PseudoVLSSEG3E64_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4792 = PseudoVLSSEG3E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4791 = PseudoVLSSEG3E32_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4790 = PseudoVLSSEG3E32_V_MF2
  { 8, &RISCVDescs.OperandInfo[5256] },  // Inst #4789 = PseudoVLSSEG3E32_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5249] },  // Inst #4788 = PseudoVLSSEG3E32_V_M2
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4787 = PseudoVLSSEG3E32_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4786 = PseudoVLSSEG3E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4785 = PseudoVLSSEG3E16_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4784 = PseudoVLSSEG3E16_V_MF4
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4783 = PseudoVLSSEG3E16_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4782 = PseudoVLSSEG3E16_V_MF2
  { 8, &RISCVDescs.OperandInfo[5256] },  // Inst #4781 = PseudoVLSSEG3E16_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5249] },  // Inst #4780 = PseudoVLSSEG3E16_V_M2
  { 8, &RISCVDescs.OperandInfo[5241] },  // Inst #4779 = PseudoVLSSEG3E16_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5234] },  // Inst #4778 = PseudoVLSSEG3E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4777 = PseudoVLSSEG2E8_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4776 = PseudoVLSSEG2E8_V_MF8
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4775 = PseudoVLSSEG2E8_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4774 = PseudoVLSSEG2E8_V_MF4
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4773 = PseudoVLSSEG2E8_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4772 = PseudoVLSSEG2E8_V_MF2
  { 8, &RISCVDescs.OperandInfo[5226] },  // Inst #4771 = PseudoVLSSEG2E8_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[5219] },  // Inst #4770 = PseudoVLSSEG2E8_V_M4
  { 8, &RISCVDescs.OperandInfo[5211] },  // Inst #4769 = PseudoVLSSEG2E8_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5204] },  // Inst #4768 = PseudoVLSSEG2E8_V_M2
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4767 = PseudoVLSSEG2E8_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4766 = PseudoVLSSEG2E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5226] },  // Inst #4765 = PseudoVLSSEG2E64_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[5219] },  // Inst #4764 = PseudoVLSSEG2E64_V_M4
  { 8, &RISCVDescs.OperandInfo[5211] },  // Inst #4763 = PseudoVLSSEG2E64_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5204] },  // Inst #4762 = PseudoVLSSEG2E64_V_M2
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4761 = PseudoVLSSEG2E64_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4760 = PseudoVLSSEG2E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4759 = PseudoVLSSEG2E32_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4758 = PseudoVLSSEG2E32_V_MF2
  { 8, &RISCVDescs.OperandInfo[5226] },  // Inst #4757 = PseudoVLSSEG2E32_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[5219] },  // Inst #4756 = PseudoVLSSEG2E32_V_M4
  { 8, &RISCVDescs.OperandInfo[5211] },  // Inst #4755 = PseudoVLSSEG2E32_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5204] },  // Inst #4754 = PseudoVLSSEG2E32_V_M2
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4753 = PseudoVLSSEG2E32_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4752 = PseudoVLSSEG2E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4751 = PseudoVLSSEG2E16_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4750 = PseudoVLSSEG2E16_V_MF4
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4749 = PseudoVLSSEG2E16_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4748 = PseudoVLSSEG2E16_V_MF2
  { 8, &RISCVDescs.OperandInfo[5226] },  // Inst #4747 = PseudoVLSSEG2E16_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[5219] },  // Inst #4746 = PseudoVLSSEG2E16_V_M4
  { 8, &RISCVDescs.OperandInfo[5211] },  // Inst #4745 = PseudoVLSSEG2E16_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5204] },  // Inst #4744 = PseudoVLSSEG2E16_V_M2
  { 8, &RISCVDescs.OperandInfo[5196] },  // Inst #4743 = PseudoVLSSEG2E16_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5189] },  // Inst #4742 = PseudoVLSSEG2E16_V_M1
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4741 = PseudoVLSEG8E8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4740 = PseudoVLSEG8E8_V_MF8
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4739 = PseudoVLSEG8E8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4738 = PseudoVLSEG8E8_V_MF4
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4737 = PseudoVLSEG8E8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4736 = PseudoVLSEG8E8_V_MF2
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4735 = PseudoVLSEG8E8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4734 = PseudoVLSEG8E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4733 = PseudoVLSEG8E8FF_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4732 = PseudoVLSEG8E8FF_V_MF8
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4731 = PseudoVLSEG8E8FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4730 = PseudoVLSEG8E8FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4729 = PseudoVLSEG8E8FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4728 = PseudoVLSEG8E8FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4727 = PseudoVLSEG8E8FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4726 = PseudoVLSEG8E8FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4725 = PseudoVLSEG8E64_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4724 = PseudoVLSEG8E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4723 = PseudoVLSEG8E64FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4722 = PseudoVLSEG8E64FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4721 = PseudoVLSEG8E32_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4720 = PseudoVLSEG8E32_V_MF2
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4719 = PseudoVLSEG8E32_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4718 = PseudoVLSEG8E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4717 = PseudoVLSEG8E32FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4716 = PseudoVLSEG8E32FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4715 = PseudoVLSEG8E32FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4714 = PseudoVLSEG8E32FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4713 = PseudoVLSEG8E16_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4712 = PseudoVLSEG8E16_V_MF4
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4711 = PseudoVLSEG8E16_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4710 = PseudoVLSEG8E16_V_MF2
  { 7, &RISCVDescs.OperandInfo[5182] },  // Inst #4709 = PseudoVLSEG8E16_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5176] },  // Inst #4708 = PseudoVLSEG8E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4707 = PseudoVLSEG8E16FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4706 = PseudoVLSEG8E16FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4705 = PseudoVLSEG8E16FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4704 = PseudoVLSEG8E16FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5168] },  // Inst #4703 = PseudoVLSEG8E16FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5161] },  // Inst #4702 = PseudoVLSEG8E16FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4701 = PseudoVLSEG7E8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4700 = PseudoVLSEG7E8_V_MF8
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4699 = PseudoVLSEG7E8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4698 = PseudoVLSEG7E8_V_MF4
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4697 = PseudoVLSEG7E8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4696 = PseudoVLSEG7E8_V_MF2
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4695 = PseudoVLSEG7E8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4694 = PseudoVLSEG7E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4693 = PseudoVLSEG7E8FF_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4692 = PseudoVLSEG7E8FF_V_MF8
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4691 = PseudoVLSEG7E8FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4690 = PseudoVLSEG7E8FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4689 = PseudoVLSEG7E8FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4688 = PseudoVLSEG7E8FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4687 = PseudoVLSEG7E8FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4686 = PseudoVLSEG7E8FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4685 = PseudoVLSEG7E64_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4684 = PseudoVLSEG7E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4683 = PseudoVLSEG7E64FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4682 = PseudoVLSEG7E64FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4681 = PseudoVLSEG7E32_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4680 = PseudoVLSEG7E32_V_MF2
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4679 = PseudoVLSEG7E32_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4678 = PseudoVLSEG7E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4677 = PseudoVLSEG7E32FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4676 = PseudoVLSEG7E32FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4675 = PseudoVLSEG7E32FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4674 = PseudoVLSEG7E32FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4673 = PseudoVLSEG7E16_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4672 = PseudoVLSEG7E16_V_MF4
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4671 = PseudoVLSEG7E16_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4670 = PseudoVLSEG7E16_V_MF2
  { 7, &RISCVDescs.OperandInfo[5154] },  // Inst #4669 = PseudoVLSEG7E16_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5148] },  // Inst #4668 = PseudoVLSEG7E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4667 = PseudoVLSEG7E16FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4666 = PseudoVLSEG7E16FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4665 = PseudoVLSEG7E16FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4664 = PseudoVLSEG7E16FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5140] },  // Inst #4663 = PseudoVLSEG7E16FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5133] },  // Inst #4662 = PseudoVLSEG7E16FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4661 = PseudoVLSEG6E8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4660 = PseudoVLSEG6E8_V_MF8
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4659 = PseudoVLSEG6E8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4658 = PseudoVLSEG6E8_V_MF4
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4657 = PseudoVLSEG6E8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4656 = PseudoVLSEG6E8_V_MF2
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4655 = PseudoVLSEG6E8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4654 = PseudoVLSEG6E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4653 = PseudoVLSEG6E8FF_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4652 = PseudoVLSEG6E8FF_V_MF8
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4651 = PseudoVLSEG6E8FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4650 = PseudoVLSEG6E8FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4649 = PseudoVLSEG6E8FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4648 = PseudoVLSEG6E8FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4647 = PseudoVLSEG6E8FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4646 = PseudoVLSEG6E8FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4645 = PseudoVLSEG6E64_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4644 = PseudoVLSEG6E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4643 = PseudoVLSEG6E64FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4642 = PseudoVLSEG6E64FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4641 = PseudoVLSEG6E32_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4640 = PseudoVLSEG6E32_V_MF2
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4639 = PseudoVLSEG6E32_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4638 = PseudoVLSEG6E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4637 = PseudoVLSEG6E32FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4636 = PseudoVLSEG6E32FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4635 = PseudoVLSEG6E32FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4634 = PseudoVLSEG6E32FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4633 = PseudoVLSEG6E16_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4632 = PseudoVLSEG6E16_V_MF4
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4631 = PseudoVLSEG6E16_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4630 = PseudoVLSEG6E16_V_MF2
  { 7, &RISCVDescs.OperandInfo[5126] },  // Inst #4629 = PseudoVLSEG6E16_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5120] },  // Inst #4628 = PseudoVLSEG6E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4627 = PseudoVLSEG6E16FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4626 = PseudoVLSEG6E16FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4625 = PseudoVLSEG6E16FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4624 = PseudoVLSEG6E16FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5112] },  // Inst #4623 = PseudoVLSEG6E16FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5105] },  // Inst #4622 = PseudoVLSEG6E16FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4621 = PseudoVLSEG5E8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4620 = PseudoVLSEG5E8_V_MF8
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4619 = PseudoVLSEG5E8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4618 = PseudoVLSEG5E8_V_MF4
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4617 = PseudoVLSEG5E8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4616 = PseudoVLSEG5E8_V_MF2
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4615 = PseudoVLSEG5E8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4614 = PseudoVLSEG5E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4613 = PseudoVLSEG5E8FF_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4612 = PseudoVLSEG5E8FF_V_MF8
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4611 = PseudoVLSEG5E8FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4610 = PseudoVLSEG5E8FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4609 = PseudoVLSEG5E8FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4608 = PseudoVLSEG5E8FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4607 = PseudoVLSEG5E8FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4606 = PseudoVLSEG5E8FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4605 = PseudoVLSEG5E64_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4604 = PseudoVLSEG5E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4603 = PseudoVLSEG5E64FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4602 = PseudoVLSEG5E64FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4601 = PseudoVLSEG5E32_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4600 = PseudoVLSEG5E32_V_MF2
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4599 = PseudoVLSEG5E32_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4598 = PseudoVLSEG5E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4597 = PseudoVLSEG5E32FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4596 = PseudoVLSEG5E32FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4595 = PseudoVLSEG5E32FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4594 = PseudoVLSEG5E32FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4593 = PseudoVLSEG5E16_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4592 = PseudoVLSEG5E16_V_MF4
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4591 = PseudoVLSEG5E16_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4590 = PseudoVLSEG5E16_V_MF2
  { 7, &RISCVDescs.OperandInfo[5098] },  // Inst #4589 = PseudoVLSEG5E16_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5092] },  // Inst #4588 = PseudoVLSEG5E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4587 = PseudoVLSEG5E16FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4586 = PseudoVLSEG5E16FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4585 = PseudoVLSEG5E16FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4584 = PseudoVLSEG5E16FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5084] },  // Inst #4583 = PseudoVLSEG5E16FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5077] },  // Inst #4582 = PseudoVLSEG5E16FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4581 = PseudoVLSEG4E8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4580 = PseudoVLSEG4E8_V_MF8
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4579 = PseudoVLSEG4E8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4578 = PseudoVLSEG4E8_V_MF4
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4577 = PseudoVLSEG4E8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4576 = PseudoVLSEG4E8_V_MF2
  { 7, &RISCVDescs.OperandInfo[5070] },  // Inst #4575 = PseudoVLSEG4E8_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[5064] },  // Inst #4574 = PseudoVLSEG4E8_V_M2
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4573 = PseudoVLSEG4E8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4572 = PseudoVLSEG4E8_V_M1
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4571 = PseudoVLSEG4E8FF_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4570 = PseudoVLSEG4E8FF_V_MF8
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4569 = PseudoVLSEG4E8FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4568 = PseudoVLSEG4E8FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4567 = PseudoVLSEG4E8FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4566 = PseudoVLSEG4E8FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5043] },  // Inst #4565 = PseudoVLSEG4E8FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5036] },  // Inst #4564 = PseudoVLSEG4E8FF_V_M2
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4563 = PseudoVLSEG4E8FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4562 = PseudoVLSEG4E8FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5070] },  // Inst #4561 = PseudoVLSEG4E64_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[5064] },  // Inst #4560 = PseudoVLSEG4E64_V_M2
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4559 = PseudoVLSEG4E64_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4558 = PseudoVLSEG4E64_V_M1
  { 8, &RISCVDescs.OperandInfo[5043] },  // Inst #4557 = PseudoVLSEG4E64FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5036] },  // Inst #4556 = PseudoVLSEG4E64FF_V_M2
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4555 = PseudoVLSEG4E64FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4554 = PseudoVLSEG4E64FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4553 = PseudoVLSEG4E32_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4552 = PseudoVLSEG4E32_V_MF2
  { 7, &RISCVDescs.OperandInfo[5070] },  // Inst #4551 = PseudoVLSEG4E32_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[5064] },  // Inst #4550 = PseudoVLSEG4E32_V_M2
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4549 = PseudoVLSEG4E32_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4548 = PseudoVLSEG4E32_V_M1
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4547 = PseudoVLSEG4E32FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4546 = PseudoVLSEG4E32FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5043] },  // Inst #4545 = PseudoVLSEG4E32FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5036] },  // Inst #4544 = PseudoVLSEG4E32FF_V_M2
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4543 = PseudoVLSEG4E32FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4542 = PseudoVLSEG4E32FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4541 = PseudoVLSEG4E16_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4540 = PseudoVLSEG4E16_V_MF4
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4539 = PseudoVLSEG4E16_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4538 = PseudoVLSEG4E16_V_MF2
  { 7, &RISCVDescs.OperandInfo[5070] },  // Inst #4537 = PseudoVLSEG4E16_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[5064] },  // Inst #4536 = PseudoVLSEG4E16_V_M2
  { 7, &RISCVDescs.OperandInfo[5057] },  // Inst #4535 = PseudoVLSEG4E16_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[5051] },  // Inst #4534 = PseudoVLSEG4E16_V_M1
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4533 = PseudoVLSEG4E16FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4532 = PseudoVLSEG4E16FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4531 = PseudoVLSEG4E16FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4530 = PseudoVLSEG4E16FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[5043] },  // Inst #4529 = PseudoVLSEG4E16FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[5036] },  // Inst #4528 = PseudoVLSEG4E16FF_V_M2
  { 8, &RISCVDescs.OperandInfo[5028] },  // Inst #4527 = PseudoVLSEG4E16FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[5021] },  // Inst #4526 = PseudoVLSEG4E16FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4525 = PseudoVLSEG3E8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4524 = PseudoVLSEG3E8_V_MF8
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4523 = PseudoVLSEG3E8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4522 = PseudoVLSEG3E8_V_MF4
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4521 = PseudoVLSEG3E8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4520 = PseudoVLSEG3E8_V_MF2
  { 7, &RISCVDescs.OperandInfo[5014] },  // Inst #4519 = PseudoVLSEG3E8_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[5008] },  // Inst #4518 = PseudoVLSEG3E8_V_M2
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4517 = PseudoVLSEG3E8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4516 = PseudoVLSEG3E8_V_M1
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4515 = PseudoVLSEG3E8FF_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4514 = PseudoVLSEG3E8FF_V_MF8
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4513 = PseudoVLSEG3E8FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4512 = PseudoVLSEG3E8FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4511 = PseudoVLSEG3E8FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4510 = PseudoVLSEG3E8FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[4987] },  // Inst #4509 = PseudoVLSEG3E8FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4980] },  // Inst #4508 = PseudoVLSEG3E8FF_V_M2
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4507 = PseudoVLSEG3E8FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4506 = PseudoVLSEG3E8FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5014] },  // Inst #4505 = PseudoVLSEG3E64_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[5008] },  // Inst #4504 = PseudoVLSEG3E64_V_M2
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4503 = PseudoVLSEG3E64_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4502 = PseudoVLSEG3E64_V_M1
  { 8, &RISCVDescs.OperandInfo[4987] },  // Inst #4501 = PseudoVLSEG3E64FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4980] },  // Inst #4500 = PseudoVLSEG3E64FF_V_M2
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4499 = PseudoVLSEG3E64FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4498 = PseudoVLSEG3E64FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4497 = PseudoVLSEG3E32_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4496 = PseudoVLSEG3E32_V_MF2
  { 7, &RISCVDescs.OperandInfo[5014] },  // Inst #4495 = PseudoVLSEG3E32_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[5008] },  // Inst #4494 = PseudoVLSEG3E32_V_M2
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4493 = PseudoVLSEG3E32_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4492 = PseudoVLSEG3E32_V_M1
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4491 = PseudoVLSEG3E32FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4490 = PseudoVLSEG3E32FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[4987] },  // Inst #4489 = PseudoVLSEG3E32FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4980] },  // Inst #4488 = PseudoVLSEG3E32FF_V_M2
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4487 = PseudoVLSEG3E32FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4486 = PseudoVLSEG3E32FF_V_M1
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4485 = PseudoVLSEG3E16_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4484 = PseudoVLSEG3E16_V_MF4
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4483 = PseudoVLSEG3E16_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4482 = PseudoVLSEG3E16_V_MF2
  { 7, &RISCVDescs.OperandInfo[5014] },  // Inst #4481 = PseudoVLSEG3E16_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[5008] },  // Inst #4480 = PseudoVLSEG3E16_V_M2
  { 7, &RISCVDescs.OperandInfo[5001] },  // Inst #4479 = PseudoVLSEG3E16_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[4995] },  // Inst #4478 = PseudoVLSEG3E16_V_M1
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4477 = PseudoVLSEG3E16FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4476 = PseudoVLSEG3E16FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4475 = PseudoVLSEG3E16FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4474 = PseudoVLSEG3E16FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[4987] },  // Inst #4473 = PseudoVLSEG3E16FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4980] },  // Inst #4472 = PseudoVLSEG3E16FF_V_M2
  { 8, &RISCVDescs.OperandInfo[4972] },  // Inst #4471 = PseudoVLSEG3E16FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4965] },  // Inst #4470 = PseudoVLSEG3E16FF_V_M1
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4469 = PseudoVLSEG2E8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4468 = PseudoVLSEG2E8_V_MF8
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4467 = PseudoVLSEG2E8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4466 = PseudoVLSEG2E8_V_MF4
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4465 = PseudoVLSEG2E8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4464 = PseudoVLSEG2E8_V_MF2
  { 7, &RISCVDescs.OperandInfo[4958] },  // Inst #4463 = PseudoVLSEG2E8_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[4952] },  // Inst #4462 = PseudoVLSEG2E8_V_M4
  { 7, &RISCVDescs.OperandInfo[4945] },  // Inst #4461 = PseudoVLSEG2E8_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[4939] },  // Inst #4460 = PseudoVLSEG2E8_V_M2
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4459 = PseudoVLSEG2E8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4458 = PseudoVLSEG2E8_V_M1
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4457 = PseudoVLSEG2E8FF_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4456 = PseudoVLSEG2E8FF_V_MF8
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4455 = PseudoVLSEG2E8FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4454 = PseudoVLSEG2E8FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4453 = PseudoVLSEG2E8FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4452 = PseudoVLSEG2E8FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[4918] },  // Inst #4451 = PseudoVLSEG2E8FF_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4911] },  // Inst #4450 = PseudoVLSEG2E8FF_V_M4
  { 8, &RISCVDescs.OperandInfo[4903] },  // Inst #4449 = PseudoVLSEG2E8FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4896] },  // Inst #4448 = PseudoVLSEG2E8FF_V_M2
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4447 = PseudoVLSEG2E8FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4446 = PseudoVLSEG2E8FF_V_M1
  { 7, &RISCVDescs.OperandInfo[4958] },  // Inst #4445 = PseudoVLSEG2E64_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[4952] },  // Inst #4444 = PseudoVLSEG2E64_V_M4
  { 7, &RISCVDescs.OperandInfo[4945] },  // Inst #4443 = PseudoVLSEG2E64_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[4939] },  // Inst #4442 = PseudoVLSEG2E64_V_M2
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4441 = PseudoVLSEG2E64_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4440 = PseudoVLSEG2E64_V_M1
  { 8, &RISCVDescs.OperandInfo[4918] },  // Inst #4439 = PseudoVLSEG2E64FF_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4911] },  // Inst #4438 = PseudoVLSEG2E64FF_V_M4
  { 8, &RISCVDescs.OperandInfo[4903] },  // Inst #4437 = PseudoVLSEG2E64FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4896] },  // Inst #4436 = PseudoVLSEG2E64FF_V_M2
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4435 = PseudoVLSEG2E64FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4434 = PseudoVLSEG2E64FF_V_M1
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4433 = PseudoVLSEG2E32_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4432 = PseudoVLSEG2E32_V_MF2
  { 7, &RISCVDescs.OperandInfo[4958] },  // Inst #4431 = PseudoVLSEG2E32_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[4952] },  // Inst #4430 = PseudoVLSEG2E32_V_M4
  { 7, &RISCVDescs.OperandInfo[4945] },  // Inst #4429 = PseudoVLSEG2E32_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[4939] },  // Inst #4428 = PseudoVLSEG2E32_V_M2
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4427 = PseudoVLSEG2E32_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4426 = PseudoVLSEG2E32_V_M1
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4425 = PseudoVLSEG2E32FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4424 = PseudoVLSEG2E32FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[4918] },  // Inst #4423 = PseudoVLSEG2E32FF_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4911] },  // Inst #4422 = PseudoVLSEG2E32FF_V_M4
  { 8, &RISCVDescs.OperandInfo[4903] },  // Inst #4421 = PseudoVLSEG2E32FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4896] },  // Inst #4420 = PseudoVLSEG2E32FF_V_M2
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4419 = PseudoVLSEG2E32FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4418 = PseudoVLSEG2E32FF_V_M1
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4417 = PseudoVLSEG2E16_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4416 = PseudoVLSEG2E16_V_MF4
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4415 = PseudoVLSEG2E16_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4414 = PseudoVLSEG2E16_V_MF2
  { 7, &RISCVDescs.OperandInfo[4958] },  // Inst #4413 = PseudoVLSEG2E16_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[4952] },  // Inst #4412 = PseudoVLSEG2E16_V_M4
  { 7, &RISCVDescs.OperandInfo[4945] },  // Inst #4411 = PseudoVLSEG2E16_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[4939] },  // Inst #4410 = PseudoVLSEG2E16_V_M2
  { 7, &RISCVDescs.OperandInfo[4932] },  // Inst #4409 = PseudoVLSEG2E16_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[4926] },  // Inst #4408 = PseudoVLSEG2E16_V_M1
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4407 = PseudoVLSEG2E16FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4406 = PseudoVLSEG2E16FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4405 = PseudoVLSEG2E16FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4404 = PseudoVLSEG2E16FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[4918] },  // Inst #4403 = PseudoVLSEG2E16FF_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4911] },  // Inst #4402 = PseudoVLSEG2E16FF_V_M4
  { 8, &RISCVDescs.OperandInfo[4903] },  // Inst #4401 = PseudoVLSEG2E16FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4896] },  // Inst #4400 = PseudoVLSEG2E16FF_V_M2
  { 8, &RISCVDescs.OperandInfo[4888] },  // Inst #4399 = PseudoVLSEG2E16FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4881] },  // Inst #4398 = PseudoVLSEG2E16FF_V_M1
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4397 = PseudoVLSE8_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4396 = PseudoVLSE8_V_MF8
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4395 = PseudoVLSE8_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4394 = PseudoVLSE8_V_MF4
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4393 = PseudoVLSE8_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4392 = PseudoVLSE8_V_MF2
  { 8, &RISCVDescs.OperandInfo[4873] },  // Inst #4391 = PseudoVLSE8_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4866] },  // Inst #4390 = PseudoVLSE8_V_M8
  { 8, &RISCVDescs.OperandInfo[4858] },  // Inst #4389 = PseudoVLSE8_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4851] },  // Inst #4388 = PseudoVLSE8_V_M4
  { 8, &RISCVDescs.OperandInfo[4843] },  // Inst #4387 = PseudoVLSE8_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4836] },  // Inst #4386 = PseudoVLSE8_V_M2
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4385 = PseudoVLSE8_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4384 = PseudoVLSE8_V_M1
  { 8, &RISCVDescs.OperandInfo[4873] },  // Inst #4383 = PseudoVLSE64_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4866] },  // Inst #4382 = PseudoVLSE64_V_M8
  { 8, &RISCVDescs.OperandInfo[4858] },  // Inst #4381 = PseudoVLSE64_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4851] },  // Inst #4380 = PseudoVLSE64_V_M4
  { 8, &RISCVDescs.OperandInfo[4843] },  // Inst #4379 = PseudoVLSE64_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4836] },  // Inst #4378 = PseudoVLSE64_V_M2
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4377 = PseudoVLSE64_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4376 = PseudoVLSE64_V_M1
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4375 = PseudoVLSE32_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4374 = PseudoVLSE32_V_MF2
  { 8, &RISCVDescs.OperandInfo[4873] },  // Inst #4373 = PseudoVLSE32_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4866] },  // Inst #4372 = PseudoVLSE32_V_M8
  { 8, &RISCVDescs.OperandInfo[4858] },  // Inst #4371 = PseudoVLSE32_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4851] },  // Inst #4370 = PseudoVLSE32_V_M4
  { 8, &RISCVDescs.OperandInfo[4843] },  // Inst #4369 = PseudoVLSE32_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4836] },  // Inst #4368 = PseudoVLSE32_V_M2
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4367 = PseudoVLSE32_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4366 = PseudoVLSE32_V_M1
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4365 = PseudoVLSE16_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4364 = PseudoVLSE16_V_MF4
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4363 = PseudoVLSE16_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4362 = PseudoVLSE16_V_MF2
  { 8, &RISCVDescs.OperandInfo[4873] },  // Inst #4361 = PseudoVLSE16_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4866] },  // Inst #4360 = PseudoVLSE16_V_M8
  { 8, &RISCVDescs.OperandInfo[4858] },  // Inst #4359 = PseudoVLSE16_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4851] },  // Inst #4358 = PseudoVLSE16_V_M4
  { 8, &RISCVDescs.OperandInfo[4843] },  // Inst #4357 = PseudoVLSE16_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4836] },  // Inst #4356 = PseudoVLSE16_V_M2
  { 8, &RISCVDescs.OperandInfo[4828] },  // Inst #4355 = PseudoVLSE16_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4821] },  // Inst #4354 = PseudoVLSE16_V_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4353 = PseudoVLOXSEG8EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4352 = PseudoVLOXSEG8EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4351 = PseudoVLOXSEG8EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4350 = PseudoVLOXSEG8EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4349 = PseudoVLOXSEG8EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4348 = PseudoVLOXSEG8EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4347 = PseudoVLOXSEG8EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4346 = PseudoVLOXSEG8EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4345 = PseudoVLOXSEG8EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4344 = PseudoVLOXSEG8EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4343 = PseudoVLOXSEG8EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4342 = PseudoVLOXSEG8EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4341 = PseudoVLOXSEG8EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4340 = PseudoVLOXSEG8EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4339 = PseudoVLOXSEG8EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4338 = PseudoVLOXSEG8EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4337 = PseudoVLOXSEG8EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4336 = PseudoVLOXSEG8EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4335 = PseudoVLOXSEG8EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4334 = PseudoVLOXSEG8EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4813] },  // Inst #4333 = PseudoVLOXSEG8EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4806] },  // Inst #4332 = PseudoVLOXSEG8EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4798] },  // Inst #4331 = PseudoVLOXSEG8EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4791] },  // Inst #4330 = PseudoVLOXSEG8EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4798] },  // Inst #4329 = PseudoVLOXSEG8EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4791] },  // Inst #4328 = PseudoVLOXSEG8EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #4327 = PseudoVLOXSEG8EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #4326 = PseudoVLOXSEG8EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #4325 = PseudoVLOXSEG8EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #4324 = PseudoVLOXSEG8EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #4323 = PseudoVLOXSEG8EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #4322 = PseudoVLOXSEG8EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4321 = PseudoVLOXSEG8EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4320 = PseudoVLOXSEG8EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4319 = PseudoVLOXSEG8EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4318 = PseudoVLOXSEG8EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4317 = PseudoVLOXSEG8EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4316 = PseudoVLOXSEG8EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4315 = PseudoVLOXSEG8EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4314 = PseudoVLOXSEG8EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4313 = PseudoVLOXSEG8EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4312 = PseudoVLOXSEG8EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4311 = PseudoVLOXSEG8EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4310 = PseudoVLOXSEG8EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4309 = PseudoVLOXSEG8EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4308 = PseudoVLOXSEG8EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4307 = PseudoVLOXSEG8EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4306 = PseudoVLOXSEG8EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4798] },  // Inst #4305 = PseudoVLOXSEG8EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4791] },  // Inst #4304 = PseudoVLOXSEG8EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #4303 = PseudoVLOXSEG8EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #4302 = PseudoVLOXSEG8EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #4301 = PseudoVLOXSEG8EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #4300 = PseudoVLOXSEG8EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4299 = PseudoVLOXSEG8EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4298 = PseudoVLOXSEG8EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4297 = PseudoVLOXSEG8EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4296 = PseudoVLOXSEG8EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4295 = PseudoVLOXSEG8EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4294 = PseudoVLOXSEG8EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4293 = PseudoVLOXSEG8EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4292 = PseudoVLOXSEG8EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4291 = PseudoVLOXSEG8EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4290 = PseudoVLOXSEG8EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4289 = PseudoVLOXSEG8EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4288 = PseudoVLOXSEG8EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4287 = PseudoVLOXSEG8EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4286 = PseudoVLOXSEG8EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4285 = PseudoVLOXSEG8EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4284 = PseudoVLOXSEG8EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4283 = PseudoVLOXSEG8EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4282 = PseudoVLOXSEG8EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4281 = PseudoVLOXSEG8EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4280 = PseudoVLOXSEG8EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4783] },  // Inst #4279 = PseudoVLOXSEG8EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4776] },  // Inst #4278 = PseudoVLOXSEG8EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4277 = PseudoVLOXSEG8EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4276 = PseudoVLOXSEG8EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4768] },  // Inst #4275 = PseudoVLOXSEG8EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4761] },  // Inst #4274 = PseudoVLOXSEG8EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4273 = PseudoVLOXSEG7EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4272 = PseudoVLOXSEG7EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4271 = PseudoVLOXSEG7EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4270 = PseudoVLOXSEG7EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4269 = PseudoVLOXSEG7EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4268 = PseudoVLOXSEG7EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4267 = PseudoVLOXSEG7EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4266 = PseudoVLOXSEG7EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4265 = PseudoVLOXSEG7EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4264 = PseudoVLOXSEG7EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4263 = PseudoVLOXSEG7EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4262 = PseudoVLOXSEG7EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4261 = PseudoVLOXSEG7EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4260 = PseudoVLOXSEG7EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4259 = PseudoVLOXSEG7EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4258 = PseudoVLOXSEG7EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4257 = PseudoVLOXSEG7EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4256 = PseudoVLOXSEG7EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4255 = PseudoVLOXSEG7EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4254 = PseudoVLOXSEG7EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4753] },  // Inst #4253 = PseudoVLOXSEG7EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4746] },  // Inst #4252 = PseudoVLOXSEG7EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4738] },  // Inst #4251 = PseudoVLOXSEG7EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4731] },  // Inst #4250 = PseudoVLOXSEG7EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4738] },  // Inst #4249 = PseudoVLOXSEG7EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4731] },  // Inst #4248 = PseudoVLOXSEG7EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #4247 = PseudoVLOXSEG7EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #4246 = PseudoVLOXSEG7EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #4245 = PseudoVLOXSEG7EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #4244 = PseudoVLOXSEG7EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #4243 = PseudoVLOXSEG7EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #4242 = PseudoVLOXSEG7EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4241 = PseudoVLOXSEG7EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4240 = PseudoVLOXSEG7EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4239 = PseudoVLOXSEG7EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4238 = PseudoVLOXSEG7EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4237 = PseudoVLOXSEG7EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4236 = PseudoVLOXSEG7EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4235 = PseudoVLOXSEG7EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4234 = PseudoVLOXSEG7EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4233 = PseudoVLOXSEG7EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4232 = PseudoVLOXSEG7EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4231 = PseudoVLOXSEG7EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4230 = PseudoVLOXSEG7EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4229 = PseudoVLOXSEG7EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4228 = PseudoVLOXSEG7EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4227 = PseudoVLOXSEG7EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4226 = PseudoVLOXSEG7EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4738] },  // Inst #4225 = PseudoVLOXSEG7EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4731] },  // Inst #4224 = PseudoVLOXSEG7EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #4223 = PseudoVLOXSEG7EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #4222 = PseudoVLOXSEG7EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #4221 = PseudoVLOXSEG7EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #4220 = PseudoVLOXSEG7EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4219 = PseudoVLOXSEG7EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4218 = PseudoVLOXSEG7EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4217 = PseudoVLOXSEG7EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4216 = PseudoVLOXSEG7EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4215 = PseudoVLOXSEG7EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4214 = PseudoVLOXSEG7EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4213 = PseudoVLOXSEG7EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4212 = PseudoVLOXSEG7EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4211 = PseudoVLOXSEG7EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4210 = PseudoVLOXSEG7EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4209 = PseudoVLOXSEG7EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4208 = PseudoVLOXSEG7EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4207 = PseudoVLOXSEG7EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4206 = PseudoVLOXSEG7EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4205 = PseudoVLOXSEG7EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4204 = PseudoVLOXSEG7EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4203 = PseudoVLOXSEG7EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4202 = PseudoVLOXSEG7EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4201 = PseudoVLOXSEG7EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4200 = PseudoVLOXSEG7EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4723] },  // Inst #4199 = PseudoVLOXSEG7EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4716] },  // Inst #4198 = PseudoVLOXSEG7EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4197 = PseudoVLOXSEG7EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4196 = PseudoVLOXSEG7EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4708] },  // Inst #4195 = PseudoVLOXSEG7EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4701] },  // Inst #4194 = PseudoVLOXSEG7EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4193 = PseudoVLOXSEG6EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4192 = PseudoVLOXSEG6EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4191 = PseudoVLOXSEG6EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4190 = PseudoVLOXSEG6EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4189 = PseudoVLOXSEG6EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4188 = PseudoVLOXSEG6EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4187 = PseudoVLOXSEG6EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4186 = PseudoVLOXSEG6EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4185 = PseudoVLOXSEG6EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4184 = PseudoVLOXSEG6EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4183 = PseudoVLOXSEG6EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4182 = PseudoVLOXSEG6EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4181 = PseudoVLOXSEG6EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4180 = PseudoVLOXSEG6EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4179 = PseudoVLOXSEG6EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4178 = PseudoVLOXSEG6EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4177 = PseudoVLOXSEG6EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4176 = PseudoVLOXSEG6EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4175 = PseudoVLOXSEG6EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4174 = PseudoVLOXSEG6EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4693] },  // Inst #4173 = PseudoVLOXSEG6EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4686] },  // Inst #4172 = PseudoVLOXSEG6EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4678] },  // Inst #4171 = PseudoVLOXSEG6EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4671] },  // Inst #4170 = PseudoVLOXSEG6EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4678] },  // Inst #4169 = PseudoVLOXSEG6EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4671] },  // Inst #4168 = PseudoVLOXSEG6EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #4167 = PseudoVLOXSEG6EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #4166 = PseudoVLOXSEG6EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #4165 = PseudoVLOXSEG6EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #4164 = PseudoVLOXSEG6EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #4163 = PseudoVLOXSEG6EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #4162 = PseudoVLOXSEG6EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4161 = PseudoVLOXSEG6EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4160 = PseudoVLOXSEG6EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4159 = PseudoVLOXSEG6EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4158 = PseudoVLOXSEG6EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4157 = PseudoVLOXSEG6EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4156 = PseudoVLOXSEG6EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4155 = PseudoVLOXSEG6EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4154 = PseudoVLOXSEG6EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4153 = PseudoVLOXSEG6EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4152 = PseudoVLOXSEG6EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4151 = PseudoVLOXSEG6EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4150 = PseudoVLOXSEG6EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4149 = PseudoVLOXSEG6EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4148 = PseudoVLOXSEG6EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4147 = PseudoVLOXSEG6EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4146 = PseudoVLOXSEG6EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4678] },  // Inst #4145 = PseudoVLOXSEG6EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4671] },  // Inst #4144 = PseudoVLOXSEG6EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #4143 = PseudoVLOXSEG6EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #4142 = PseudoVLOXSEG6EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #4141 = PseudoVLOXSEG6EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #4140 = PseudoVLOXSEG6EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4139 = PseudoVLOXSEG6EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4138 = PseudoVLOXSEG6EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4137 = PseudoVLOXSEG6EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4136 = PseudoVLOXSEG6EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4135 = PseudoVLOXSEG6EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4134 = PseudoVLOXSEG6EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4133 = PseudoVLOXSEG6EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4132 = PseudoVLOXSEG6EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4131 = PseudoVLOXSEG6EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4130 = PseudoVLOXSEG6EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4129 = PseudoVLOXSEG6EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4128 = PseudoVLOXSEG6EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4127 = PseudoVLOXSEG6EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4126 = PseudoVLOXSEG6EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4125 = PseudoVLOXSEG6EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4124 = PseudoVLOXSEG6EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4123 = PseudoVLOXSEG6EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4122 = PseudoVLOXSEG6EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4121 = PseudoVLOXSEG6EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4120 = PseudoVLOXSEG6EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4663] },  // Inst #4119 = PseudoVLOXSEG6EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4656] },  // Inst #4118 = PseudoVLOXSEG6EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4117 = PseudoVLOXSEG6EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4116 = PseudoVLOXSEG6EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4648] },  // Inst #4115 = PseudoVLOXSEG6EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4641] },  // Inst #4114 = PseudoVLOXSEG6EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4113 = PseudoVLOXSEG5EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4112 = PseudoVLOXSEG5EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4111 = PseudoVLOXSEG5EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4110 = PseudoVLOXSEG5EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4109 = PseudoVLOXSEG5EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4108 = PseudoVLOXSEG5EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4107 = PseudoVLOXSEG5EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4106 = PseudoVLOXSEG5EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4105 = PseudoVLOXSEG5EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4104 = PseudoVLOXSEG5EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4103 = PseudoVLOXSEG5EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4102 = PseudoVLOXSEG5EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4101 = PseudoVLOXSEG5EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4100 = PseudoVLOXSEG5EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4099 = PseudoVLOXSEG5EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4098 = PseudoVLOXSEG5EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4097 = PseudoVLOXSEG5EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4096 = PseudoVLOXSEG5EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4095 = PseudoVLOXSEG5EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4094 = PseudoVLOXSEG5EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4633] },  // Inst #4093 = PseudoVLOXSEG5EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4626] },  // Inst #4092 = PseudoVLOXSEG5EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4618] },  // Inst #4091 = PseudoVLOXSEG5EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4611] },  // Inst #4090 = PseudoVLOXSEG5EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4618] },  // Inst #4089 = PseudoVLOXSEG5EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4611] },  // Inst #4088 = PseudoVLOXSEG5EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #4087 = PseudoVLOXSEG5EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #4086 = PseudoVLOXSEG5EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #4085 = PseudoVLOXSEG5EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #4084 = PseudoVLOXSEG5EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #4083 = PseudoVLOXSEG5EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #4082 = PseudoVLOXSEG5EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4081 = PseudoVLOXSEG5EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4080 = PseudoVLOXSEG5EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4079 = PseudoVLOXSEG5EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4078 = PseudoVLOXSEG5EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4077 = PseudoVLOXSEG5EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4076 = PseudoVLOXSEG5EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4075 = PseudoVLOXSEG5EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4074 = PseudoVLOXSEG5EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4073 = PseudoVLOXSEG5EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4072 = PseudoVLOXSEG5EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4071 = PseudoVLOXSEG5EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4070 = PseudoVLOXSEG5EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4069 = PseudoVLOXSEG5EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4068 = PseudoVLOXSEG5EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4067 = PseudoVLOXSEG5EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4066 = PseudoVLOXSEG5EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4618] },  // Inst #4065 = PseudoVLOXSEG5EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4611] },  // Inst #4064 = PseudoVLOXSEG5EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #4063 = PseudoVLOXSEG5EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #4062 = PseudoVLOXSEG5EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #4061 = PseudoVLOXSEG5EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #4060 = PseudoVLOXSEG5EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4059 = PseudoVLOXSEG5EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4058 = PseudoVLOXSEG5EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4057 = PseudoVLOXSEG5EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4056 = PseudoVLOXSEG5EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4055 = PseudoVLOXSEG5EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4054 = PseudoVLOXSEG5EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4053 = PseudoVLOXSEG5EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4052 = PseudoVLOXSEG5EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4051 = PseudoVLOXSEG5EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4050 = PseudoVLOXSEG5EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4049 = PseudoVLOXSEG5EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4048 = PseudoVLOXSEG5EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4047 = PseudoVLOXSEG5EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4046 = PseudoVLOXSEG5EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4045 = PseudoVLOXSEG5EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4044 = PseudoVLOXSEG5EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4043 = PseudoVLOXSEG5EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4042 = PseudoVLOXSEG5EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4041 = PseudoVLOXSEG5EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4040 = PseudoVLOXSEG5EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4603] },  // Inst #4039 = PseudoVLOXSEG5EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4596] },  // Inst #4038 = PseudoVLOXSEG5EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4037 = PseudoVLOXSEG5EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4036 = PseudoVLOXSEG5EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4588] },  // Inst #4035 = PseudoVLOXSEG5EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4581] },  // Inst #4034 = PseudoVLOXSEG5EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4033 = PseudoVLOXSEG4EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4032 = PseudoVLOXSEG4EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4031 = PseudoVLOXSEG4EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4030 = PseudoVLOXSEG4EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4029 = PseudoVLOXSEG4EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4028 = PseudoVLOXSEG4EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4027 = PseudoVLOXSEG4EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4026 = PseudoVLOXSEG4EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4025 = PseudoVLOXSEG4EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4024 = PseudoVLOXSEG4EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4023 = PseudoVLOXSEG4EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4022 = PseudoVLOXSEG4EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #4021 = PseudoVLOXSEG4EI8_V_MF4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #4020 = PseudoVLOXSEG4EI8_V_MF4_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4019 = PseudoVLOXSEG4EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4018 = PseudoVLOXSEG4EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4017 = PseudoVLOXSEG4EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4016 = PseudoVLOXSEG4EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #4015 = PseudoVLOXSEG4EI8_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #4014 = PseudoVLOXSEG4EI8_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4013 = PseudoVLOXSEG4EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4012 = PseudoVLOXSEG4EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4513] },  // Inst #4011 = PseudoVLOXSEG4EI8_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4506] },  // Inst #4010 = PseudoVLOXSEG4EI8_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #4009 = PseudoVLOXSEG4EI8_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #4008 = PseudoVLOXSEG4EI8_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #4007 = PseudoVLOXSEG4EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #4006 = PseudoVLOXSEG4EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4558] },  // Inst #4005 = PseudoVLOXSEG4EI64_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4551] },  // Inst #4004 = PseudoVLOXSEG4EI64_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4573] },  // Inst #4003 = PseudoVLOXSEG4EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4566] },  // Inst #4002 = PseudoVLOXSEG4EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4543] },  // Inst #4001 = PseudoVLOXSEG4EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4536] },  // Inst #4000 = PseudoVLOXSEG4EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4528] },  // Inst #3999 = PseudoVLOXSEG4EI64_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4521] },  // Inst #3998 = PseudoVLOXSEG4EI64_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4543] },  // Inst #3997 = PseudoVLOXSEG4EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4536] },  // Inst #3996 = PseudoVLOXSEG4EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #3995 = PseudoVLOXSEG4EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #3994 = PseudoVLOXSEG4EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #3993 = PseudoVLOXSEG4EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #3992 = PseudoVLOXSEG4EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4513] },  // Inst #3991 = PseudoVLOXSEG4EI64_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4506] },  // Inst #3990 = PseudoVLOXSEG4EI64_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #3989 = PseudoVLOXSEG4EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #3988 = PseudoVLOXSEG4EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3987 = PseudoVLOXSEG4EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3986 = PseudoVLOXSEG4EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3985 = PseudoVLOXSEG4EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3984 = PseudoVLOXSEG4EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3983 = PseudoVLOXSEG4EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3982 = PseudoVLOXSEG4EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3981 = PseudoVLOXSEG4EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3980 = PseudoVLOXSEG4EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3979 = PseudoVLOXSEG4EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3978 = PseudoVLOXSEG4EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3977 = PseudoVLOXSEG4EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3976 = PseudoVLOXSEG4EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3975 = PseudoVLOXSEG4EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3974 = PseudoVLOXSEG4EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3973 = PseudoVLOXSEG4EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3972 = PseudoVLOXSEG4EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4558] },  // Inst #3971 = PseudoVLOXSEG4EI32_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4551] },  // Inst #3970 = PseudoVLOXSEG4EI32_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4528] },  // Inst #3969 = PseudoVLOXSEG4EI32_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4521] },  // Inst #3968 = PseudoVLOXSEG4EI32_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4543] },  // Inst #3967 = PseudoVLOXSEG4EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4536] },  // Inst #3966 = PseudoVLOXSEG4EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #3965 = PseudoVLOXSEG4EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #3964 = PseudoVLOXSEG4EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4513] },  // Inst #3963 = PseudoVLOXSEG4EI32_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4506] },  // Inst #3962 = PseudoVLOXSEG4EI32_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #3961 = PseudoVLOXSEG4EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #3960 = PseudoVLOXSEG4EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3959 = PseudoVLOXSEG4EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3958 = PseudoVLOXSEG4EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3957 = PseudoVLOXSEG4EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3956 = PseudoVLOXSEG4EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #3955 = PseudoVLOXSEG4EI32_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #3954 = PseudoVLOXSEG4EI32_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3953 = PseudoVLOXSEG4EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3952 = PseudoVLOXSEG4EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3951 = PseudoVLOXSEG4EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3950 = PseudoVLOXSEG4EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3949 = PseudoVLOXSEG4EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3948 = PseudoVLOXSEG4EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3947 = PseudoVLOXSEG4EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3946 = PseudoVLOXSEG4EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3945 = PseudoVLOXSEG4EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3944 = PseudoVLOXSEG4EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3943 = PseudoVLOXSEG4EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3942 = PseudoVLOXSEG4EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3941 = PseudoVLOXSEG4EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3940 = PseudoVLOXSEG4EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #3939 = PseudoVLOXSEG4EI16_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #3938 = PseudoVLOXSEG4EI16_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3937 = PseudoVLOXSEG4EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3936 = PseudoVLOXSEG4EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4528] },  // Inst #3935 = PseudoVLOXSEG4EI16_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4521] },  // Inst #3934 = PseudoVLOXSEG4EI16_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4513] },  // Inst #3933 = PseudoVLOXSEG4EI16_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4506] },  // Inst #3932 = PseudoVLOXSEG4EI16_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4498] },  // Inst #3931 = PseudoVLOXSEG4EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4491] },  // Inst #3930 = PseudoVLOXSEG4EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3929 = PseudoVLOXSEG4EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3928 = PseudoVLOXSEG4EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4483] },  // Inst #3927 = PseudoVLOXSEG4EI16_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4476] },  // Inst #3926 = PseudoVLOXSEG4EI16_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4468] },  // Inst #3925 = PseudoVLOXSEG4EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4461] },  // Inst #3924 = PseudoVLOXSEG4EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3923 = PseudoVLOXSEG3EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3922 = PseudoVLOXSEG3EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3921 = PseudoVLOXSEG3EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3920 = PseudoVLOXSEG3EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3919 = PseudoVLOXSEG3EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3918 = PseudoVLOXSEG3EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3917 = PseudoVLOXSEG3EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3916 = PseudoVLOXSEG3EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3915 = PseudoVLOXSEG3EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3914 = PseudoVLOXSEG3EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3913 = PseudoVLOXSEG3EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3912 = PseudoVLOXSEG3EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #3911 = PseudoVLOXSEG3EI8_V_MF4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #3910 = PseudoVLOXSEG3EI8_V_MF4_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3909 = PseudoVLOXSEG3EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3908 = PseudoVLOXSEG3EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3907 = PseudoVLOXSEG3EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3906 = PseudoVLOXSEG3EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #3905 = PseudoVLOXSEG3EI8_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #3904 = PseudoVLOXSEG3EI8_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3903 = PseudoVLOXSEG3EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3902 = PseudoVLOXSEG3EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4393] },  // Inst #3901 = PseudoVLOXSEG3EI8_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4386] },  // Inst #3900 = PseudoVLOXSEG3EI8_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #3899 = PseudoVLOXSEG3EI8_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #3898 = PseudoVLOXSEG3EI8_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3897 = PseudoVLOXSEG3EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3896 = PseudoVLOXSEG3EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4438] },  // Inst #3895 = PseudoVLOXSEG3EI64_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4431] },  // Inst #3894 = PseudoVLOXSEG3EI64_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4453] },  // Inst #3893 = PseudoVLOXSEG3EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4446] },  // Inst #3892 = PseudoVLOXSEG3EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4423] },  // Inst #3891 = PseudoVLOXSEG3EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4416] },  // Inst #3890 = PseudoVLOXSEG3EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4408] },  // Inst #3889 = PseudoVLOXSEG3EI64_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4401] },  // Inst #3888 = PseudoVLOXSEG3EI64_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4423] },  // Inst #3887 = PseudoVLOXSEG3EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4416] },  // Inst #3886 = PseudoVLOXSEG3EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #3885 = PseudoVLOXSEG3EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #3884 = PseudoVLOXSEG3EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #3883 = PseudoVLOXSEG3EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #3882 = PseudoVLOXSEG3EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4393] },  // Inst #3881 = PseudoVLOXSEG3EI64_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4386] },  // Inst #3880 = PseudoVLOXSEG3EI64_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #3879 = PseudoVLOXSEG3EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #3878 = PseudoVLOXSEG3EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3877 = PseudoVLOXSEG3EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3876 = PseudoVLOXSEG3EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3875 = PseudoVLOXSEG3EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3874 = PseudoVLOXSEG3EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3873 = PseudoVLOXSEG3EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3872 = PseudoVLOXSEG3EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3871 = PseudoVLOXSEG3EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3870 = PseudoVLOXSEG3EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3869 = PseudoVLOXSEG3EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3868 = PseudoVLOXSEG3EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3867 = PseudoVLOXSEG3EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3866 = PseudoVLOXSEG3EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3865 = PseudoVLOXSEG3EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3864 = PseudoVLOXSEG3EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3863 = PseudoVLOXSEG3EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3862 = PseudoVLOXSEG3EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4438] },  // Inst #3861 = PseudoVLOXSEG3EI32_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4431] },  // Inst #3860 = PseudoVLOXSEG3EI32_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4408] },  // Inst #3859 = PseudoVLOXSEG3EI32_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4401] },  // Inst #3858 = PseudoVLOXSEG3EI32_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4423] },  // Inst #3857 = PseudoVLOXSEG3EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4416] },  // Inst #3856 = PseudoVLOXSEG3EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #3855 = PseudoVLOXSEG3EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #3854 = PseudoVLOXSEG3EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4393] },  // Inst #3853 = PseudoVLOXSEG3EI32_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4386] },  // Inst #3852 = PseudoVLOXSEG3EI32_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #3851 = PseudoVLOXSEG3EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #3850 = PseudoVLOXSEG3EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3849 = PseudoVLOXSEG3EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3848 = PseudoVLOXSEG3EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3847 = PseudoVLOXSEG3EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3846 = PseudoVLOXSEG3EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #3845 = PseudoVLOXSEG3EI32_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #3844 = PseudoVLOXSEG3EI32_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3843 = PseudoVLOXSEG3EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3842 = PseudoVLOXSEG3EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3841 = PseudoVLOXSEG3EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3840 = PseudoVLOXSEG3EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3839 = PseudoVLOXSEG3EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3838 = PseudoVLOXSEG3EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3837 = PseudoVLOXSEG3EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3836 = PseudoVLOXSEG3EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3835 = PseudoVLOXSEG3EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3834 = PseudoVLOXSEG3EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3833 = PseudoVLOXSEG3EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3832 = PseudoVLOXSEG3EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3831 = PseudoVLOXSEG3EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3830 = PseudoVLOXSEG3EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #3829 = PseudoVLOXSEG3EI16_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #3828 = PseudoVLOXSEG3EI16_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3827 = PseudoVLOXSEG3EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3826 = PseudoVLOXSEG3EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4408] },  // Inst #3825 = PseudoVLOXSEG3EI16_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4401] },  // Inst #3824 = PseudoVLOXSEG3EI16_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4393] },  // Inst #3823 = PseudoVLOXSEG3EI16_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4386] },  // Inst #3822 = PseudoVLOXSEG3EI16_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4378] },  // Inst #3821 = PseudoVLOXSEG3EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4371] },  // Inst #3820 = PseudoVLOXSEG3EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3819 = PseudoVLOXSEG3EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3818 = PseudoVLOXSEG3EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4363] },  // Inst #3817 = PseudoVLOXSEG3EI16_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4356] },  // Inst #3816 = PseudoVLOXSEG3EI16_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4348] },  // Inst #3815 = PseudoVLOXSEG3EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4341] },  // Inst #3814 = PseudoVLOXSEG3EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3813 = PseudoVLOXSEG2EI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3812 = PseudoVLOXSEG2EI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3811 = PseudoVLOXSEG2EI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3810 = PseudoVLOXSEG2EI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3809 = PseudoVLOXSEG2EI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3808 = PseudoVLOXSEG2EI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3807 = PseudoVLOXSEG2EI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3806 = PseudoVLOXSEG2EI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3805 = PseudoVLOXSEG2EI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3804 = PseudoVLOXSEG2EI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3803 = PseudoVLOXSEG2EI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3802 = PseudoVLOXSEG2EI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #3801 = PseudoVLOXSEG2EI8_V_MF4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #3800 = PseudoVLOXSEG2EI8_V_MF4_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3799 = PseudoVLOXSEG2EI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3798 = PseudoVLOXSEG2EI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3797 = PseudoVLOXSEG2EI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3796 = PseudoVLOXSEG2EI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4198] },  // Inst #3795 = PseudoVLOXSEG2EI8_V_MF2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4191] },  // Inst #3794 = PseudoVLOXSEG2EI8_V_MF2_M4
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #3793 = PseudoVLOXSEG2EI8_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #3792 = PseudoVLOXSEG2EI8_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3791 = PseudoVLOXSEG2EI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3790 = PseudoVLOXSEG2EI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4273] },  // Inst #3789 = PseudoVLOXSEG2EI8_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4266] },  // Inst #3788 = PseudoVLOXSEG2EI8_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4243] },  // Inst #3787 = PseudoVLOXSEG2EI8_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4236] },  // Inst #3786 = PseudoVLOXSEG2EI8_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[4228] },  // Inst #3785 = PseudoVLOXSEG2EI8_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4221] },  // Inst #3784 = PseudoVLOXSEG2EI8_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4198] },  // Inst #3783 = PseudoVLOXSEG2EI8_V_M1_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4191] },  // Inst #3782 = PseudoVLOXSEG2EI8_V_M1_M4
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #3781 = PseudoVLOXSEG2EI8_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #3780 = PseudoVLOXSEG2EI8_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3779 = PseudoVLOXSEG2EI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3778 = PseudoVLOXSEG2EI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4288] },  // Inst #3777 = PseudoVLOXSEG2EI64_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4281] },  // Inst #3776 = PseudoVLOXSEG2EI64_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4318] },  // Inst #3775 = PseudoVLOXSEG2EI64_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4311] },  // Inst #3774 = PseudoVLOXSEG2EI64_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4333] },  // Inst #3773 = PseudoVLOXSEG2EI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4326] },  // Inst #3772 = PseudoVLOXSEG2EI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4303] },  // Inst #3771 = PseudoVLOXSEG2EI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4296] },  // Inst #3770 = PseudoVLOXSEG2EI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4273] },  // Inst #3769 = PseudoVLOXSEG2EI64_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4266] },  // Inst #3768 = PseudoVLOXSEG2EI64_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4258] },  // Inst #3767 = PseudoVLOXSEG2EI64_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4251] },  // Inst #3766 = PseudoVLOXSEG2EI64_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4303] },  // Inst #3765 = PseudoVLOXSEG2EI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4296] },  // Inst #3764 = PseudoVLOXSEG2EI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #3763 = PseudoVLOXSEG2EI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #3762 = PseudoVLOXSEG2EI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #3761 = PseudoVLOXSEG2EI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #3760 = PseudoVLOXSEG2EI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4228] },  // Inst #3759 = PseudoVLOXSEG2EI64_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4221] },  // Inst #3758 = PseudoVLOXSEG2EI64_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #3757 = PseudoVLOXSEG2EI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #3756 = PseudoVLOXSEG2EI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3755 = PseudoVLOXSEG2EI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3754 = PseudoVLOXSEG2EI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3753 = PseudoVLOXSEG2EI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3752 = PseudoVLOXSEG2EI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3751 = PseudoVLOXSEG2EI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3750 = PseudoVLOXSEG2EI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3749 = PseudoVLOXSEG2EI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3748 = PseudoVLOXSEG2EI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3747 = PseudoVLOXSEG2EI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3746 = PseudoVLOXSEG2EI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3745 = PseudoVLOXSEG2EI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3744 = PseudoVLOXSEG2EI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3743 = PseudoVLOXSEG2EI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3742 = PseudoVLOXSEG2EI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3741 = PseudoVLOXSEG2EI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3740 = PseudoVLOXSEG2EI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4288] },  // Inst #3739 = PseudoVLOXSEG2EI32_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4281] },  // Inst #3738 = PseudoVLOXSEG2EI32_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4318] },  // Inst #3737 = PseudoVLOXSEG2EI32_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4311] },  // Inst #3736 = PseudoVLOXSEG2EI32_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4273] },  // Inst #3735 = PseudoVLOXSEG2EI32_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4266] },  // Inst #3734 = PseudoVLOXSEG2EI32_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4258] },  // Inst #3733 = PseudoVLOXSEG2EI32_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4251] },  // Inst #3732 = PseudoVLOXSEG2EI32_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4303] },  // Inst #3731 = PseudoVLOXSEG2EI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4296] },  // Inst #3730 = PseudoVLOXSEG2EI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #3729 = PseudoVLOXSEG2EI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #3728 = PseudoVLOXSEG2EI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4243] },  // Inst #3727 = PseudoVLOXSEG2EI32_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4236] },  // Inst #3726 = PseudoVLOXSEG2EI32_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[4228] },  // Inst #3725 = PseudoVLOXSEG2EI32_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4221] },  // Inst #3724 = PseudoVLOXSEG2EI32_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #3723 = PseudoVLOXSEG2EI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #3722 = PseudoVLOXSEG2EI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3721 = PseudoVLOXSEG2EI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3720 = PseudoVLOXSEG2EI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3719 = PseudoVLOXSEG2EI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3718 = PseudoVLOXSEG2EI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #3717 = PseudoVLOXSEG2EI32_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #3716 = PseudoVLOXSEG2EI32_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3715 = PseudoVLOXSEG2EI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3714 = PseudoVLOXSEG2EI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3713 = PseudoVLOXSEG2EI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3712 = PseudoVLOXSEG2EI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3711 = PseudoVLOXSEG2EI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3710 = PseudoVLOXSEG2EI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3709 = PseudoVLOXSEG2EI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3708 = PseudoVLOXSEG2EI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3707 = PseudoVLOXSEG2EI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3706 = PseudoVLOXSEG2EI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3705 = PseudoVLOXSEG2EI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3704 = PseudoVLOXSEG2EI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3703 = PseudoVLOXSEG2EI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3702 = PseudoVLOXSEG2EI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #3701 = PseudoVLOXSEG2EI16_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #3700 = PseudoVLOXSEG2EI16_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3699 = PseudoVLOXSEG2EI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3698 = PseudoVLOXSEG2EI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4288] },  // Inst #3697 = PseudoVLOXSEG2EI16_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4281] },  // Inst #3696 = PseudoVLOXSEG2EI16_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4273] },  // Inst #3695 = PseudoVLOXSEG2EI16_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4266] },  // Inst #3694 = PseudoVLOXSEG2EI16_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4258] },  // Inst #3693 = PseudoVLOXSEG2EI16_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4251] },  // Inst #3692 = PseudoVLOXSEG2EI16_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4243] },  // Inst #3691 = PseudoVLOXSEG2EI16_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4236] },  // Inst #3690 = PseudoVLOXSEG2EI16_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[4228] },  // Inst #3689 = PseudoVLOXSEG2EI16_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4221] },  // Inst #3688 = PseudoVLOXSEG2EI16_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4213] },  // Inst #3687 = PseudoVLOXSEG2EI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4206] },  // Inst #3686 = PseudoVLOXSEG2EI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3685 = PseudoVLOXSEG2EI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3684 = PseudoVLOXSEG2EI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[4198] },  // Inst #3683 = PseudoVLOXSEG2EI16_V_M1_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4191] },  // Inst #3682 = PseudoVLOXSEG2EI16_V_M1_M4
  { 8, &RISCVDescs.OperandInfo[4183] },  // Inst #3681 = PseudoVLOXSEG2EI16_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4176] },  // Inst #3680 = PseudoVLOXSEG2EI16_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[4168] },  // Inst #3679 = PseudoVLOXSEG2EI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4161] },  // Inst #3678 = PseudoVLOXSEG2EI16_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3677 = PseudoVLOXEI8_V_MF8_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3676 = PseudoVLOXEI8_V_MF8_MF8
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3675 = PseudoVLOXEI8_V_MF8_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3674 = PseudoVLOXEI8_V_MF8_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3673 = PseudoVLOXEI8_V_MF8_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3672 = PseudoVLOXEI8_V_MF8_MF2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3671 = PseudoVLOXEI8_V_MF8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3670 = PseudoVLOXEI8_V_MF8_M1
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3669 = PseudoVLOXEI8_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3668 = PseudoVLOXEI8_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3667 = PseudoVLOXEI8_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3666 = PseudoVLOXEI8_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #3665 = PseudoVLOXEI8_V_MF4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #3664 = PseudoVLOXEI8_V_MF4_M2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3663 = PseudoVLOXEI8_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3662 = PseudoVLOXEI8_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3661 = PseudoVLOXEI8_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3660 = PseudoVLOXEI8_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[3943] },  // Inst #3659 = PseudoVLOXEI8_V_MF2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3936] },  // Inst #3658 = PseudoVLOXEI8_V_MF2_M4
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #3657 = PseudoVLOXEI8_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #3656 = PseudoVLOXEI8_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3655 = PseudoVLOXEI8_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3654 = PseudoVLOXEI8_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4093] },  // Inst #3653 = PseudoVLOXEI8_V_M8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4086] },  // Inst #3652 = PseudoVLOXEI8_V_M8_M8
  { 8, &RISCVDescs.OperandInfo[4063] },  // Inst #3651 = PseudoVLOXEI8_V_M4_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4056] },  // Inst #3650 = PseudoVLOXEI8_V_M4_M8
  { 8, &RISCVDescs.OperandInfo[4048] },  // Inst #3649 = PseudoVLOXEI8_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4041] },  // Inst #3648 = PseudoVLOXEI8_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4018] },  // Inst #3647 = PseudoVLOXEI8_V_M2_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4011] },  // Inst #3646 = PseudoVLOXEI8_V_M2_M8
  { 8, &RISCVDescs.OperandInfo[4003] },  // Inst #3645 = PseudoVLOXEI8_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3996] },  // Inst #3644 = PseudoVLOXEI8_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[3988] },  // Inst #3643 = PseudoVLOXEI8_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3981] },  // Inst #3642 = PseudoVLOXEI8_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[4153] },  // Inst #3641 = PseudoVLOXEI8_V_M1_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4146] },  // Inst #3640 = PseudoVLOXEI8_V_M1_M8
  { 8, &RISCVDescs.OperandInfo[3943] },  // Inst #3639 = PseudoVLOXEI8_V_M1_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3936] },  // Inst #3638 = PseudoVLOXEI8_V_M1_M4
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #3637 = PseudoVLOXEI8_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #3636 = PseudoVLOXEI8_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3635 = PseudoVLOXEI8_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3634 = PseudoVLOXEI8_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[4093] },  // Inst #3633 = PseudoVLOXEI64_V_M8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4086] },  // Inst #3632 = PseudoVLOXEI64_V_M8_M8
  { 8, &RISCVDescs.OperandInfo[4078] },  // Inst #3631 = PseudoVLOXEI64_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4071] },  // Inst #3630 = PseudoVLOXEI64_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4123] },  // Inst #3629 = PseudoVLOXEI64_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4116] },  // Inst #3628 = PseudoVLOXEI64_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4138] },  // Inst #3627 = PseudoVLOXEI64_V_M8_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4131] },  // Inst #3626 = PseudoVLOXEI64_V_M8_M1
  { 8, &RISCVDescs.OperandInfo[4108] },  // Inst #3625 = PseudoVLOXEI64_V_M4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[4101] },  // Inst #3624 = PseudoVLOXEI64_V_M4_MF2
  { 8, &RISCVDescs.OperandInfo[4048] },  // Inst #3623 = PseudoVLOXEI64_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4041] },  // Inst #3622 = PseudoVLOXEI64_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4033] },  // Inst #3621 = PseudoVLOXEI64_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4026] },  // Inst #3620 = PseudoVLOXEI64_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4108] },  // Inst #3619 = PseudoVLOXEI64_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4101] },  // Inst #3618 = PseudoVLOXEI64_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #3617 = PseudoVLOXEI64_V_M2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #3616 = PseudoVLOXEI64_V_M2_MF4
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #3615 = PseudoVLOXEI64_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #3614 = PseudoVLOXEI64_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[3988] },  // Inst #3613 = PseudoVLOXEI64_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3981] },  // Inst #3612 = PseudoVLOXEI64_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #3611 = PseudoVLOXEI64_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #3610 = PseudoVLOXEI64_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3609 = PseudoVLOXEI64_V_M1_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3608 = PseudoVLOXEI64_V_M1_MF8
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3607 = PseudoVLOXEI64_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3606 = PseudoVLOXEI64_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3605 = PseudoVLOXEI64_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3604 = PseudoVLOXEI64_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3603 = PseudoVLOXEI64_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3602 = PseudoVLOXEI64_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3601 = PseudoVLOXEI32_V_MF2_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3600 = PseudoVLOXEI32_V_MF2_MF8
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3599 = PseudoVLOXEI32_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3598 = PseudoVLOXEI32_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3597 = PseudoVLOXEI32_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3596 = PseudoVLOXEI32_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3595 = PseudoVLOXEI32_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3594 = PseudoVLOXEI32_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4093] },  // Inst #3593 = PseudoVLOXEI32_V_M8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4086] },  // Inst #3592 = PseudoVLOXEI32_V_M8_M8
  { 8, &RISCVDescs.OperandInfo[4078] },  // Inst #3591 = PseudoVLOXEI32_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4071] },  // Inst #3590 = PseudoVLOXEI32_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4123] },  // Inst #3589 = PseudoVLOXEI32_V_M8_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4116] },  // Inst #3588 = PseudoVLOXEI32_V_M8_M2
  { 8, &RISCVDescs.OperandInfo[4063] },  // Inst #3587 = PseudoVLOXEI32_V_M4_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4056] },  // Inst #3586 = PseudoVLOXEI32_V_M4_M8
  { 8, &RISCVDescs.OperandInfo[4048] },  // Inst #3585 = PseudoVLOXEI32_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4041] },  // Inst #3584 = PseudoVLOXEI32_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4033] },  // Inst #3583 = PseudoVLOXEI32_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4026] },  // Inst #3582 = PseudoVLOXEI32_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4108] },  // Inst #3581 = PseudoVLOXEI32_V_M4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[4101] },  // Inst #3580 = PseudoVLOXEI32_V_M4_M1
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #3579 = PseudoVLOXEI32_V_M2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #3578 = PseudoVLOXEI32_V_M2_MF2
  { 8, &RISCVDescs.OperandInfo[4003] },  // Inst #3577 = PseudoVLOXEI32_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3996] },  // Inst #3576 = PseudoVLOXEI32_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[3988] },  // Inst #3575 = PseudoVLOXEI32_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3981] },  // Inst #3574 = PseudoVLOXEI32_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #3573 = PseudoVLOXEI32_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #3572 = PseudoVLOXEI32_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3571 = PseudoVLOXEI32_V_M1_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3570 = PseudoVLOXEI32_V_M1_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3569 = PseudoVLOXEI32_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3568 = PseudoVLOXEI32_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #3567 = PseudoVLOXEI32_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #3566 = PseudoVLOXEI32_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3565 = PseudoVLOXEI32_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3564 = PseudoVLOXEI32_V_M1_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3563 = PseudoVLOXEI16_V_MF4_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3562 = PseudoVLOXEI16_V_MF4_MF8
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3561 = PseudoVLOXEI16_V_MF4_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3560 = PseudoVLOXEI16_V_MF4_MF4
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3559 = PseudoVLOXEI16_V_MF4_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3558 = PseudoVLOXEI16_V_MF4_MF2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3557 = PseudoVLOXEI16_V_MF4_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3556 = PseudoVLOXEI16_V_MF4_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3555 = PseudoVLOXEI16_V_MF2_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3554 = PseudoVLOXEI16_V_MF2_MF4
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3553 = PseudoVLOXEI16_V_MF2_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3552 = PseudoVLOXEI16_V_MF2_MF2
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #3551 = PseudoVLOXEI16_V_MF2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #3550 = PseudoVLOXEI16_V_MF2_M2
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3549 = PseudoVLOXEI16_V_MF2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3548 = PseudoVLOXEI16_V_MF2_M1
  { 8, &RISCVDescs.OperandInfo[4093] },  // Inst #3547 = PseudoVLOXEI16_V_M8_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4086] },  // Inst #3546 = PseudoVLOXEI16_V_M8_M8
  { 8, &RISCVDescs.OperandInfo[4078] },  // Inst #3545 = PseudoVLOXEI16_V_M8_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4071] },  // Inst #3544 = PseudoVLOXEI16_V_M8_M4
  { 8, &RISCVDescs.OperandInfo[4063] },  // Inst #3543 = PseudoVLOXEI16_V_M4_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4056] },  // Inst #3542 = PseudoVLOXEI16_V_M4_M8
  { 8, &RISCVDescs.OperandInfo[4048] },  // Inst #3541 = PseudoVLOXEI16_V_M4_M4_MASK
  { 7, &RISCVDescs.OperandInfo[4041] },  // Inst #3540 = PseudoVLOXEI16_V_M4_M4
  { 8, &RISCVDescs.OperandInfo[4033] },  // Inst #3539 = PseudoVLOXEI16_V_M4_M2_MASK
  { 7, &RISCVDescs.OperandInfo[4026] },  // Inst #3538 = PseudoVLOXEI16_V_M4_M2
  { 8, &RISCVDescs.OperandInfo[4018] },  // Inst #3537 = PseudoVLOXEI16_V_M2_M8_MASK
  { 7, &RISCVDescs.OperandInfo[4011] },  // Inst #3536 = PseudoVLOXEI16_V_M2_M8
  { 8, &RISCVDescs.OperandInfo[4003] },  // Inst #3535 = PseudoVLOXEI16_V_M2_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3996] },  // Inst #3534 = PseudoVLOXEI16_V_M2_M4
  { 8, &RISCVDescs.OperandInfo[3988] },  // Inst #3533 = PseudoVLOXEI16_V_M2_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3981] },  // Inst #3532 = PseudoVLOXEI16_V_M2_M2
  { 8, &RISCVDescs.OperandInfo[3973] },  // Inst #3531 = PseudoVLOXEI16_V_M2_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3966] },  // Inst #3530 = PseudoVLOXEI16_V_M2_M1
  { 8, &RISCVDescs.OperandInfo[3958] },  // Inst #3529 = PseudoVLOXEI16_V_M1_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3951] },  // Inst #3528 = PseudoVLOXEI16_V_M1_MF2
  { 8, &RISCVDescs.OperandInfo[3943] },  // Inst #3527 = PseudoVLOXEI16_V_M1_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3936] },  // Inst #3526 = PseudoVLOXEI16_V_M1_M4
  { 8, &RISCVDescs.OperandInfo[3928] },  // Inst #3525 = PseudoVLOXEI16_V_M1_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3921] },  // Inst #3524 = PseudoVLOXEI16_V_M1_M2
  { 8, &RISCVDescs.OperandInfo[3913] },  // Inst #3523 = PseudoVLOXEI16_V_M1_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3906] },  // Inst #3522 = PseudoVLOXEI16_V_M1_M1
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3521 = PseudoVLM_V_B8
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3520 = PseudoVLM_V_B64
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3519 = PseudoVLM_V_B4
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3518 = PseudoVLM_V_B32
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3517 = PseudoVLM_V_B2
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3516 = PseudoVLM_V_B16
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3515 = PseudoVLM_V_B1
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3514 = PseudoVLE8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3513 = PseudoVLE8_V_MF8
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3512 = PseudoVLE8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3511 = PseudoVLE8_V_MF4
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3510 = PseudoVLE8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3509 = PseudoVLE8_V_MF2
  { 7, &RISCVDescs.OperandInfo[3899] },  // Inst #3508 = PseudoVLE8_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[3893] },  // Inst #3507 = PseudoVLE8_V_M8
  { 7, &RISCVDescs.OperandInfo[3886] },  // Inst #3506 = PseudoVLE8_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3880] },  // Inst #3505 = PseudoVLE8_V_M4
  { 7, &RISCVDescs.OperandInfo[3873] },  // Inst #3504 = PseudoVLE8_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3867] },  // Inst #3503 = PseudoVLE8_V_M2
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3502 = PseudoVLE8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3501 = PseudoVLE8_V_M1
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3500 = PseudoVLE8FF_V_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3499 = PseudoVLE8FF_V_MF8
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3498 = PseudoVLE8FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3497 = PseudoVLE8FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3496 = PseudoVLE8FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3495 = PseudoVLE8FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[3846] },  // Inst #3494 = PseudoVLE8FF_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[3839] },  // Inst #3493 = PseudoVLE8FF_V_M8
  { 8, &RISCVDescs.OperandInfo[3831] },  // Inst #3492 = PseudoVLE8FF_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3824] },  // Inst #3491 = PseudoVLE8FF_V_M4
  { 8, &RISCVDescs.OperandInfo[3816] },  // Inst #3490 = PseudoVLE8FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3809] },  // Inst #3489 = PseudoVLE8FF_V_M2
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3488 = PseudoVLE8FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3487 = PseudoVLE8FF_V_M1
  { 7, &RISCVDescs.OperandInfo[3899] },  // Inst #3486 = PseudoVLE64_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[3893] },  // Inst #3485 = PseudoVLE64_V_M8
  { 7, &RISCVDescs.OperandInfo[3886] },  // Inst #3484 = PseudoVLE64_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3880] },  // Inst #3483 = PseudoVLE64_V_M4
  { 7, &RISCVDescs.OperandInfo[3873] },  // Inst #3482 = PseudoVLE64_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3867] },  // Inst #3481 = PseudoVLE64_V_M2
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3480 = PseudoVLE64_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3479 = PseudoVLE64_V_M1
  { 8, &RISCVDescs.OperandInfo[3846] },  // Inst #3478 = PseudoVLE64FF_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[3839] },  // Inst #3477 = PseudoVLE64FF_V_M8
  { 8, &RISCVDescs.OperandInfo[3831] },  // Inst #3476 = PseudoVLE64FF_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3824] },  // Inst #3475 = PseudoVLE64FF_V_M4
  { 8, &RISCVDescs.OperandInfo[3816] },  // Inst #3474 = PseudoVLE64FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3809] },  // Inst #3473 = PseudoVLE64FF_V_M2
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3472 = PseudoVLE64FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3471 = PseudoVLE64FF_V_M1
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3470 = PseudoVLE32_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3469 = PseudoVLE32_V_MF2
  { 7, &RISCVDescs.OperandInfo[3899] },  // Inst #3468 = PseudoVLE32_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[3893] },  // Inst #3467 = PseudoVLE32_V_M8
  { 7, &RISCVDescs.OperandInfo[3886] },  // Inst #3466 = PseudoVLE32_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3880] },  // Inst #3465 = PseudoVLE32_V_M4
  { 7, &RISCVDescs.OperandInfo[3873] },  // Inst #3464 = PseudoVLE32_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3867] },  // Inst #3463 = PseudoVLE32_V_M2
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3462 = PseudoVLE32_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3461 = PseudoVLE32_V_M1
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3460 = PseudoVLE32FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3459 = PseudoVLE32FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[3846] },  // Inst #3458 = PseudoVLE32FF_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[3839] },  // Inst #3457 = PseudoVLE32FF_V_M8
  { 8, &RISCVDescs.OperandInfo[3831] },  // Inst #3456 = PseudoVLE32FF_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3824] },  // Inst #3455 = PseudoVLE32FF_V_M4
  { 8, &RISCVDescs.OperandInfo[3816] },  // Inst #3454 = PseudoVLE32FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3809] },  // Inst #3453 = PseudoVLE32FF_V_M2
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3452 = PseudoVLE32FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3451 = PseudoVLE32FF_V_M1
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3450 = PseudoVLE16_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3449 = PseudoVLE16_V_MF4
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3448 = PseudoVLE16_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3447 = PseudoVLE16_V_MF2
  { 7, &RISCVDescs.OperandInfo[3899] },  // Inst #3446 = PseudoVLE16_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[3893] },  // Inst #3445 = PseudoVLE16_V_M8
  { 7, &RISCVDescs.OperandInfo[3886] },  // Inst #3444 = PseudoVLE16_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3880] },  // Inst #3443 = PseudoVLE16_V_M4
  { 7, &RISCVDescs.OperandInfo[3873] },  // Inst #3442 = PseudoVLE16_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3867] },  // Inst #3441 = PseudoVLE16_V_M2
  { 7, &RISCVDescs.OperandInfo[3860] },  // Inst #3440 = PseudoVLE16_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3854] },  // Inst #3439 = PseudoVLE16_V_M1
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3438 = PseudoVLE16FF_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3437 = PseudoVLE16FF_V_MF4
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3436 = PseudoVLE16FF_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3435 = PseudoVLE16FF_V_MF2
  { 8, &RISCVDescs.OperandInfo[3846] },  // Inst #3434 = PseudoVLE16FF_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[3839] },  // Inst #3433 = PseudoVLE16FF_V_M8
  { 8, &RISCVDescs.OperandInfo[3831] },  // Inst #3432 = PseudoVLE16FF_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3824] },  // Inst #3431 = PseudoVLE16FF_V_M4
  { 8, &RISCVDescs.OperandInfo[3816] },  // Inst #3430 = PseudoVLE16FF_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3809] },  // Inst #3429 = PseudoVLE16FF_V_M2
  { 8, &RISCVDescs.OperandInfo[3801] },  // Inst #3428 = PseudoVLE16FF_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3794] },  // Inst #3427 = PseudoVLE16FF_V_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3426 = PseudoVIOTA_M_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3425 = PseudoVIOTA_M_MF8
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3424 = PseudoVIOTA_M_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3423 = PseudoVIOTA_M_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3422 = PseudoVIOTA_M_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3421 = PseudoVIOTA_M_MF2
  { 7, &RISCVDescs.OperandInfo[3787] },  // Inst #3420 = PseudoVIOTA_M_M8_MASK
  { 6, &RISCVDescs.OperandInfo[3781] },  // Inst #3419 = PseudoVIOTA_M_M8
  { 7, &RISCVDescs.OperandInfo[3774] },  // Inst #3418 = PseudoVIOTA_M_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3768] },  // Inst #3417 = PseudoVIOTA_M_M4
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #3416 = PseudoVIOTA_M_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #3415 = PseudoVIOTA_M_M2
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3414 = PseudoVIOTA_M_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3413 = PseudoVIOTA_M_M1
  { 6, &RISCVDescs.OperandInfo[3729] },  // Inst #3412 = PseudoVID_V_MF8_MASK
  { 5, &RISCVDescs.OperandInfo[3724] },  // Inst #3411 = PseudoVID_V_MF8
  { 6, &RISCVDescs.OperandInfo[3729] },  // Inst #3410 = PseudoVID_V_MF4_MASK
  { 5, &RISCVDescs.OperandInfo[3724] },  // Inst #3409 = PseudoVID_V_MF4
  { 6, &RISCVDescs.OperandInfo[3729] },  // Inst #3408 = PseudoVID_V_MF2_MASK
  { 5, &RISCVDescs.OperandInfo[3724] },  // Inst #3407 = PseudoVID_V_MF2
  { 6, &RISCVDescs.OperandInfo[3762] },  // Inst #3406 = PseudoVID_V_M8_MASK
  { 5, &RISCVDescs.OperandInfo[3757] },  // Inst #3405 = PseudoVID_V_M8
  { 6, &RISCVDescs.OperandInfo[3751] },  // Inst #3404 = PseudoVID_V_M4_MASK
  { 5, &RISCVDescs.OperandInfo[3746] },  // Inst #3403 = PseudoVID_V_M4
  { 6, &RISCVDescs.OperandInfo[3740] },  // Inst #3402 = PseudoVID_V_M2_MASK
  { 5, &RISCVDescs.OperandInfo[3735] },  // Inst #3401 = PseudoVID_V_M2
  { 6, &RISCVDescs.OperandInfo[3729] },  // Inst #3400 = PseudoVID_V_M1_MASK
  { 5, &RISCVDescs.OperandInfo[3724] },  // Inst #3399 = PseudoVID_V_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #3398 = PseudoVGMUL_VV_MF2
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #3397 = PseudoVGMUL_VV_M8
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #3396 = PseudoVGMUL_VV_M4
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #3395 = PseudoVGMUL_VV_M2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #3394 = PseudoVGMUL_VV_M1
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #3393 = PseudoVGHSH_VV_MF2
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #3392 = PseudoVGHSH_VV_M8
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #3391 = PseudoVGHSH_VV_M4
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #3390 = PseudoVGHSH_VV_M2
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #3389 = PseudoVGHSH_VV_M1
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3388 = PseudoVFWSUB_WV_MF4_TIED
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3387 = PseudoVFWSUB_WV_MF4_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3386 = PseudoVFWSUB_WV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3385 = PseudoVFWSUB_WV_MF4
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3384 = PseudoVFWSUB_WV_MF2_TIED
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3383 = PseudoVFWSUB_WV_MF2_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3382 = PseudoVFWSUB_WV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3381 = PseudoVFWSUB_WV_MF2
  { 7, &RISCVDescs.OperandInfo[3514] },  // Inst #3380 = PseudoVFWSUB_WV_M4_TIED
  { 8, &RISCVDescs.OperandInfo[3506] },  // Inst #3379 = PseudoVFWSUB_WV_M4_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3497] },  // Inst #3378 = PseudoVFWSUB_WV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3489] },  // Inst #3377 = PseudoVFWSUB_WV_M4
  { 7, &RISCVDescs.OperandInfo[3482] },  // Inst #3376 = PseudoVFWSUB_WV_M2_TIED
  { 8, &RISCVDescs.OperandInfo[3474] },  // Inst #3375 = PseudoVFWSUB_WV_M2_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3465] },  // Inst #3374 = PseudoVFWSUB_WV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3457] },  // Inst #3373 = PseudoVFWSUB_WV_M2
  { 7, &RISCVDescs.OperandInfo[3450] },  // Inst #3372 = PseudoVFWSUB_WV_M1_TIED
  { 8, &RISCVDescs.OperandInfo[3442] },  // Inst #3371 = PseudoVFWSUB_WV_M1_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3433] },  // Inst #3370 = PseudoVFWSUB_WV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3425] },  // Inst #3369 = PseudoVFWSUB_WV_M1
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #3368 = PseudoVFWSUB_WFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #3367 = PseudoVFWSUB_WFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2009] },  // Inst #3366 = PseudoVFWSUB_WFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2001] },  // Inst #3365 = PseudoVFWSUB_WFPR32_M4
  { 9, &RISCVDescs.OperandInfo[1992] },  // Inst #3364 = PseudoVFWSUB_WFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1984] },  // Inst #3363 = PseudoVFWSUB_WFPR32_M2
  { 9, &RISCVDescs.OperandInfo[1975] },  // Inst #3362 = PseudoVFWSUB_WFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1967] },  // Inst #3361 = PseudoVFWSUB_WFPR32_M1
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #3360 = PseudoVFWSUB_WFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #3359 = PseudoVFWSUB_WFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #3358 = PseudoVFWSUB_WFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #3357 = PseudoVFWSUB_WFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[1941] },  // Inst #3356 = PseudoVFWSUB_WFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1933] },  // Inst #3355 = PseudoVFWSUB_WFPR16_M4
  { 9, &RISCVDescs.OperandInfo[1924] },  // Inst #3354 = PseudoVFWSUB_WFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1916] },  // Inst #3353 = PseudoVFWSUB_WFPR16_M2
  { 9, &RISCVDescs.OperandInfo[1907] },  // Inst #3352 = PseudoVFWSUB_WFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1899] },  // Inst #3351 = PseudoVFWSUB_WFPR16_M1
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3350 = PseudoVFWSUB_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3349 = PseudoVFWSUB_VV_MF4
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3348 = PseudoVFWSUB_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3347 = PseudoVFWSUB_VV_MF2
  { 9, &RISCVDescs.OperandInfo[3399] },  // Inst #3346 = PseudoVFWSUB_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3391] },  // Inst #3345 = PseudoVFWSUB_VV_M4
  { 9, &RISCVDescs.OperandInfo[3382] },  // Inst #3344 = PseudoVFWSUB_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3374] },  // Inst #3343 = PseudoVFWSUB_VV_M2
  { 9, &RISCVDescs.OperandInfo[3365] },  // Inst #3342 = PseudoVFWSUB_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3357] },  // Inst #3341 = PseudoVFWSUB_VV_M1
  { 9, &RISCVDescs.OperandInfo[2953] },  // Inst #3340 = PseudoVFWSUB_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2945] },  // Inst #3339 = PseudoVFWSUB_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[3348] },  // Inst #3338 = PseudoVFWSUB_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3340] },  // Inst #3337 = PseudoVFWSUB_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[3331] },  // Inst #3336 = PseudoVFWSUB_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3323] },  // Inst #3335 = PseudoVFWSUB_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[3314] },  // Inst #3334 = PseudoVFWSUB_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3306] },  // Inst #3333 = PseudoVFWSUB_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[3297] },  // Inst #3332 = PseudoVFWSUB_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3289] },  // Inst #3331 = PseudoVFWSUB_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[3297] },  // Inst #3330 = PseudoVFWSUB_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3289] },  // Inst #3329 = PseudoVFWSUB_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[3280] },  // Inst #3328 = PseudoVFWSUB_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3272] },  // Inst #3327 = PseudoVFWSUB_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[3263] },  // Inst #3326 = PseudoVFWSUB_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3255] },  // Inst #3325 = PseudoVFWSUB_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[3246] },  // Inst #3324 = PseudoVFWSUB_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3238] },  // Inst #3323 = PseudoVFWSUB_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3322 = PseudoVFWREDUSUM_VS_MF4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3321 = PseudoVFWREDUSUM_VS_MF4_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3320 = PseudoVFWREDUSUM_VS_MF2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3319 = PseudoVFWREDUSUM_VS_MF2_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3318 = PseudoVFWREDUSUM_VS_MF2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3317 = PseudoVFWREDUSUM_VS_MF2_E16
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #3316 = PseudoVFWREDUSUM_VS_M8_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #3315 = PseudoVFWREDUSUM_VS_M8_E32
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #3314 = PseudoVFWREDUSUM_VS_M8_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #3313 = PseudoVFWREDUSUM_VS_M8_E16
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #3312 = PseudoVFWREDUSUM_VS_M4_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #3311 = PseudoVFWREDUSUM_VS_M4_E32
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #3310 = PseudoVFWREDUSUM_VS_M4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #3309 = PseudoVFWREDUSUM_VS_M4_E16
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #3308 = PseudoVFWREDUSUM_VS_M2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #3307 = PseudoVFWREDUSUM_VS_M2_E32
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #3306 = PseudoVFWREDUSUM_VS_M2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #3305 = PseudoVFWREDUSUM_VS_M2_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3304 = PseudoVFWREDUSUM_VS_M1_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3303 = PseudoVFWREDUSUM_VS_M1_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3302 = PseudoVFWREDUSUM_VS_M1_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3301 = PseudoVFWREDUSUM_VS_M1_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3300 = PseudoVFWREDOSUM_VS_MF4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3299 = PseudoVFWREDOSUM_VS_MF4_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3298 = PseudoVFWREDOSUM_VS_MF2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3297 = PseudoVFWREDOSUM_VS_MF2_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3296 = PseudoVFWREDOSUM_VS_MF2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3295 = PseudoVFWREDOSUM_VS_MF2_E16
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #3294 = PseudoVFWREDOSUM_VS_M8_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #3293 = PseudoVFWREDOSUM_VS_M8_E32
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #3292 = PseudoVFWREDOSUM_VS_M8_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #3291 = PseudoVFWREDOSUM_VS_M8_E16
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #3290 = PseudoVFWREDOSUM_VS_M4_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #3289 = PseudoVFWREDOSUM_VS_M4_E32
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #3288 = PseudoVFWREDOSUM_VS_M4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #3287 = PseudoVFWREDOSUM_VS_M4_E16
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #3286 = PseudoVFWREDOSUM_VS_M2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #3285 = PseudoVFWREDOSUM_VS_M2_E32
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #3284 = PseudoVFWREDOSUM_VS_M2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #3283 = PseudoVFWREDOSUM_VS_M2_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3282 = PseudoVFWREDOSUM_VS_M1_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3281 = PseudoVFWREDOSUM_VS_M1_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #3280 = PseudoVFWREDOSUM_VS_M1_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #3279 = PseudoVFWREDOSUM_VS_M1_E16
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3278 = PseudoVFWNMSAC_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3277 = PseudoVFWNMSAC_VV_MF4
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3276 = PseudoVFWNMSAC_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3275 = PseudoVFWNMSAC_VV_MF2
  { 9, &RISCVDescs.OperandInfo[3399] },  // Inst #3274 = PseudoVFWNMSAC_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3391] },  // Inst #3273 = PseudoVFWNMSAC_VV_M4
  { 9, &RISCVDescs.OperandInfo[3382] },  // Inst #3272 = PseudoVFWNMSAC_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3374] },  // Inst #3271 = PseudoVFWNMSAC_VV_M2
  { 9, &RISCVDescs.OperandInfo[3365] },  // Inst #3270 = PseudoVFWNMSAC_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3357] },  // Inst #3269 = PseudoVFWNMSAC_VV_M1
  { 9, &RISCVDescs.OperandInfo[3715] },  // Inst #3268 = PseudoVFWNMSAC_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3707] },  // Inst #3267 = PseudoVFWNMSAC_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[3698] },  // Inst #3266 = PseudoVFWNMSAC_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3690] },  // Inst #3265 = PseudoVFWNMSAC_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[3681] },  // Inst #3264 = PseudoVFWNMSAC_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3673] },  // Inst #3263 = PseudoVFWNMSAC_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[3664] },  // Inst #3262 = PseudoVFWNMSAC_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3656] },  // Inst #3261 = PseudoVFWNMSAC_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3260 = PseudoVFWNMSAC_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3259 = PseudoVFWNMSAC_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3258 = PseudoVFWNMSAC_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3257 = PseudoVFWNMSAC_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[3602] },  // Inst #3256 = PseudoVFWNMSAC_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3594] },  // Inst #3255 = PseudoVFWNMSAC_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[3585] },  // Inst #3254 = PseudoVFWNMSAC_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3577] },  // Inst #3253 = PseudoVFWNMSAC_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[3568] },  // Inst #3252 = PseudoVFWNMSAC_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3560] },  // Inst #3251 = PseudoVFWNMSAC_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3250 = PseudoVFWNMACC_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3249 = PseudoVFWNMACC_VV_MF4
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3248 = PseudoVFWNMACC_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3247 = PseudoVFWNMACC_VV_MF2
  { 9, &RISCVDescs.OperandInfo[3399] },  // Inst #3246 = PseudoVFWNMACC_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3391] },  // Inst #3245 = PseudoVFWNMACC_VV_M4
  { 9, &RISCVDescs.OperandInfo[3382] },  // Inst #3244 = PseudoVFWNMACC_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3374] },  // Inst #3243 = PseudoVFWNMACC_VV_M2
  { 9, &RISCVDescs.OperandInfo[3365] },  // Inst #3242 = PseudoVFWNMACC_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3357] },  // Inst #3241 = PseudoVFWNMACC_VV_M1
  { 9, &RISCVDescs.OperandInfo[3715] },  // Inst #3240 = PseudoVFWNMACC_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3707] },  // Inst #3239 = PseudoVFWNMACC_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[3698] },  // Inst #3238 = PseudoVFWNMACC_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3690] },  // Inst #3237 = PseudoVFWNMACC_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[3681] },  // Inst #3236 = PseudoVFWNMACC_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3673] },  // Inst #3235 = PseudoVFWNMACC_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[3664] },  // Inst #3234 = PseudoVFWNMACC_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3656] },  // Inst #3233 = PseudoVFWNMACC_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3232 = PseudoVFWNMACC_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3231 = PseudoVFWNMACC_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3230 = PseudoVFWNMACC_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3229 = PseudoVFWNMACC_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[3602] },  // Inst #3228 = PseudoVFWNMACC_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3594] },  // Inst #3227 = PseudoVFWNMACC_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[3585] },  // Inst #3226 = PseudoVFWNMACC_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3577] },  // Inst #3225 = PseudoVFWNMACC_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[3568] },  // Inst #3224 = PseudoVFWNMACC_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3560] },  // Inst #3223 = PseudoVFWNMACC_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3222 = PseudoVFWMUL_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3221 = PseudoVFWMUL_VV_MF4
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3220 = PseudoVFWMUL_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3219 = PseudoVFWMUL_VV_MF2
  { 9, &RISCVDescs.OperandInfo[3399] },  // Inst #3218 = PseudoVFWMUL_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3391] },  // Inst #3217 = PseudoVFWMUL_VV_M4
  { 9, &RISCVDescs.OperandInfo[3382] },  // Inst #3216 = PseudoVFWMUL_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3374] },  // Inst #3215 = PseudoVFWMUL_VV_M2
  { 9, &RISCVDescs.OperandInfo[3365] },  // Inst #3214 = PseudoVFWMUL_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3357] },  // Inst #3213 = PseudoVFWMUL_VV_M1
  { 9, &RISCVDescs.OperandInfo[2953] },  // Inst #3212 = PseudoVFWMUL_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2945] },  // Inst #3211 = PseudoVFWMUL_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[3348] },  // Inst #3210 = PseudoVFWMUL_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3340] },  // Inst #3209 = PseudoVFWMUL_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[3331] },  // Inst #3208 = PseudoVFWMUL_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3323] },  // Inst #3207 = PseudoVFWMUL_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[3314] },  // Inst #3206 = PseudoVFWMUL_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3306] },  // Inst #3205 = PseudoVFWMUL_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[3297] },  // Inst #3204 = PseudoVFWMUL_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3289] },  // Inst #3203 = PseudoVFWMUL_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[3297] },  // Inst #3202 = PseudoVFWMUL_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3289] },  // Inst #3201 = PseudoVFWMUL_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[3280] },  // Inst #3200 = PseudoVFWMUL_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3272] },  // Inst #3199 = PseudoVFWMUL_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[3263] },  // Inst #3198 = PseudoVFWMUL_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3255] },  // Inst #3197 = PseudoVFWMUL_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[3246] },  // Inst #3196 = PseudoVFWMUL_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3238] },  // Inst #3195 = PseudoVFWMUL_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3194 = PseudoVFWMSAC_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3193 = PseudoVFWMSAC_VV_MF4
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3192 = PseudoVFWMSAC_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3191 = PseudoVFWMSAC_VV_MF2
  { 9, &RISCVDescs.OperandInfo[3399] },  // Inst #3190 = PseudoVFWMSAC_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3391] },  // Inst #3189 = PseudoVFWMSAC_VV_M4
  { 9, &RISCVDescs.OperandInfo[3382] },  // Inst #3188 = PseudoVFWMSAC_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3374] },  // Inst #3187 = PseudoVFWMSAC_VV_M2
  { 9, &RISCVDescs.OperandInfo[3365] },  // Inst #3186 = PseudoVFWMSAC_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3357] },  // Inst #3185 = PseudoVFWMSAC_VV_M1
  { 9, &RISCVDescs.OperandInfo[3715] },  // Inst #3184 = PseudoVFWMSAC_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3707] },  // Inst #3183 = PseudoVFWMSAC_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[3698] },  // Inst #3182 = PseudoVFWMSAC_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3690] },  // Inst #3181 = PseudoVFWMSAC_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[3681] },  // Inst #3180 = PseudoVFWMSAC_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3673] },  // Inst #3179 = PseudoVFWMSAC_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[3664] },  // Inst #3178 = PseudoVFWMSAC_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3656] },  // Inst #3177 = PseudoVFWMSAC_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3176 = PseudoVFWMSAC_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3175 = PseudoVFWMSAC_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3174 = PseudoVFWMSAC_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3173 = PseudoVFWMSAC_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[3602] },  // Inst #3172 = PseudoVFWMSAC_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3594] },  // Inst #3171 = PseudoVFWMSAC_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[3585] },  // Inst #3170 = PseudoVFWMSAC_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3577] },  // Inst #3169 = PseudoVFWMSAC_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[3568] },  // Inst #3168 = PseudoVFWMSAC_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3560] },  // Inst #3167 = PseudoVFWMSAC_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3166 = PseudoVFWMACC_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3165 = PseudoVFWMACC_VV_MF4
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3164 = PseudoVFWMACC_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3163 = PseudoVFWMACC_VV_MF2
  { 9, &RISCVDescs.OperandInfo[3399] },  // Inst #3162 = PseudoVFWMACC_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3391] },  // Inst #3161 = PseudoVFWMACC_VV_M4
  { 9, &RISCVDescs.OperandInfo[3382] },  // Inst #3160 = PseudoVFWMACC_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3374] },  // Inst #3159 = PseudoVFWMACC_VV_M2
  { 9, &RISCVDescs.OperandInfo[3365] },  // Inst #3158 = PseudoVFWMACC_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3357] },  // Inst #3157 = PseudoVFWMACC_VV_M1
  { 9, &RISCVDescs.OperandInfo[3715] },  // Inst #3156 = PseudoVFWMACC_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3707] },  // Inst #3155 = PseudoVFWMACC_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[3698] },  // Inst #3154 = PseudoVFWMACC_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3690] },  // Inst #3153 = PseudoVFWMACC_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[3681] },  // Inst #3152 = PseudoVFWMACC_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3673] },  // Inst #3151 = PseudoVFWMACC_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[3664] },  // Inst #3150 = PseudoVFWMACC_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3656] },  // Inst #3149 = PseudoVFWMACC_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3148 = PseudoVFWMACC_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3147 = PseudoVFWMACC_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3146 = PseudoVFWMACC_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3145 = PseudoVFWMACC_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[3602] },  // Inst #3144 = PseudoVFWMACC_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3594] },  // Inst #3143 = PseudoVFWMACC_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[3585] },  // Inst #3142 = PseudoVFWMACC_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3577] },  // Inst #3141 = PseudoVFWMACC_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[3568] },  // Inst #3140 = PseudoVFWMACC_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3560] },  // Inst #3139 = PseudoVFWMACC_VFPR16_M1
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #3138 = PseudoVFWMACC_4x4x4_MF4
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #3137 = PseudoVFWMACC_4x4x4_MF2
  { 7, &RISCVDescs.OperandInfo[3649] },  // Inst #3136 = PseudoVFWMACC_4x4x4_M8
  { 7, &RISCVDescs.OperandInfo[3642] },  // Inst #3135 = PseudoVFWMACC_4x4x4_M4
  { 7, &RISCVDescs.OperandInfo[3635] },  // Inst #3134 = PseudoVFWMACC_4x4x4_M2
  { 7, &RISCVDescs.OperandInfo[3628] },  // Inst #3133 = PseudoVFWMACC_4x4x4_M1
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3132 = PseudoVFWMACCBF16_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3131 = PseudoVFWMACCBF16_VV_MF4
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3130 = PseudoVFWMACCBF16_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3129 = PseudoVFWMACCBF16_VV_MF2
  { 9, &RISCVDescs.OperandInfo[3399] },  // Inst #3128 = PseudoVFWMACCBF16_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3391] },  // Inst #3127 = PseudoVFWMACCBF16_VV_M4
  { 9, &RISCVDescs.OperandInfo[3382] },  // Inst #3126 = PseudoVFWMACCBF16_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3374] },  // Inst #3125 = PseudoVFWMACCBF16_VV_M2
  { 9, &RISCVDescs.OperandInfo[3365] },  // Inst #3124 = PseudoVFWMACCBF16_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3357] },  // Inst #3123 = PseudoVFWMACCBF16_VV_M1
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3122 = PseudoVFWMACCBF16_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3121 = PseudoVFWMACCBF16_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[3619] },  // Inst #3120 = PseudoVFWMACCBF16_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3611] },  // Inst #3119 = PseudoVFWMACCBF16_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[3602] },  // Inst #3118 = PseudoVFWMACCBF16_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3594] },  // Inst #3117 = PseudoVFWMACCBF16_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[3585] },  // Inst #3116 = PseudoVFWMACCBF16_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3577] },  // Inst #3115 = PseudoVFWMACCBF16_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[3568] },  // Inst #3114 = PseudoVFWMACCBF16_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3560] },  // Inst #3113 = PseudoVFWMACCBF16_VFPR16_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3112 = PseudoVFWCVT_X_F_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3111 = PseudoVFWCVT_X_F_V_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3110 = PseudoVFWCVT_X_F_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3109 = PseudoVFWCVT_X_F_V_MF2
  { 8, &RISCVDescs.OperandInfo[3506] },  // Inst #3108 = PseudoVFWCVT_X_F_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3514] },  // Inst #3107 = PseudoVFWCVT_X_F_V_M4
  { 8, &RISCVDescs.OperandInfo[3474] },  // Inst #3106 = PseudoVFWCVT_X_F_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3482] },  // Inst #3105 = PseudoVFWCVT_X_F_V_M2
  { 8, &RISCVDescs.OperandInfo[3442] },  // Inst #3104 = PseudoVFWCVT_X_F_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3450] },  // Inst #3103 = PseudoVFWCVT_X_F_V_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3102 = PseudoVFWCVT_XU_F_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3101 = PseudoVFWCVT_XU_F_V_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3100 = PseudoVFWCVT_XU_F_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3099 = PseudoVFWCVT_XU_F_V_MF2
  { 8, &RISCVDescs.OperandInfo[3506] },  // Inst #3098 = PseudoVFWCVT_XU_F_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3514] },  // Inst #3097 = PseudoVFWCVT_XU_F_V_M4
  { 8, &RISCVDescs.OperandInfo[3474] },  // Inst #3096 = PseudoVFWCVT_XU_F_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3482] },  // Inst #3095 = PseudoVFWCVT_XU_F_V_M2
  { 8, &RISCVDescs.OperandInfo[3442] },  // Inst #3094 = PseudoVFWCVT_XU_F_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3450] },  // Inst #3093 = PseudoVFWCVT_XU_F_V_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3092 = PseudoVFWCVT_RTZ_X_F_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3091 = PseudoVFWCVT_RTZ_X_F_V_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3090 = PseudoVFWCVT_RTZ_X_F_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3089 = PseudoVFWCVT_RTZ_X_F_V_MF2
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #3088 = PseudoVFWCVT_RTZ_X_F_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #3087 = PseudoVFWCVT_RTZ_X_F_V_M4
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #3086 = PseudoVFWCVT_RTZ_X_F_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #3085 = PseudoVFWCVT_RTZ_X_F_V_M2
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #3084 = PseudoVFWCVT_RTZ_X_F_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #3083 = PseudoVFWCVT_RTZ_X_F_V_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3082 = PseudoVFWCVT_RTZ_XU_F_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3081 = PseudoVFWCVT_RTZ_XU_F_V_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3080 = PseudoVFWCVT_RTZ_XU_F_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3079 = PseudoVFWCVT_RTZ_XU_F_V_MF2
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #3078 = PseudoVFWCVT_RTZ_XU_F_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #3077 = PseudoVFWCVT_RTZ_XU_F_V_M4
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #3076 = PseudoVFWCVT_RTZ_XU_F_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #3075 = PseudoVFWCVT_RTZ_XU_F_V_M2
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #3074 = PseudoVFWCVT_RTZ_XU_F_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #3073 = PseudoVFWCVT_RTZ_XU_F_V_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3072 = PseudoVFWCVT_RM_X_F_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3071 = PseudoVFWCVT_RM_X_F_V_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3070 = PseudoVFWCVT_RM_X_F_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3069 = PseudoVFWCVT_RM_X_F_V_MF2
  { 8, &RISCVDescs.OperandInfo[3506] },  // Inst #3068 = PseudoVFWCVT_RM_X_F_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3514] },  // Inst #3067 = PseudoVFWCVT_RM_X_F_V_M4
  { 8, &RISCVDescs.OperandInfo[3474] },  // Inst #3066 = PseudoVFWCVT_RM_X_F_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3482] },  // Inst #3065 = PseudoVFWCVT_RM_X_F_V_M2
  { 8, &RISCVDescs.OperandInfo[3442] },  // Inst #3064 = PseudoVFWCVT_RM_X_F_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3450] },  // Inst #3063 = PseudoVFWCVT_RM_X_F_V_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3062 = PseudoVFWCVT_RM_XU_F_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3061 = PseudoVFWCVT_RM_XU_F_V_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3060 = PseudoVFWCVT_RM_XU_F_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3059 = PseudoVFWCVT_RM_XU_F_V_MF2
  { 8, &RISCVDescs.OperandInfo[3506] },  // Inst #3058 = PseudoVFWCVT_RM_XU_F_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3514] },  // Inst #3057 = PseudoVFWCVT_RM_XU_F_V_M4
  { 8, &RISCVDescs.OperandInfo[3474] },  // Inst #3056 = PseudoVFWCVT_RM_XU_F_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3482] },  // Inst #3055 = PseudoVFWCVT_RM_XU_F_V_M2
  { 8, &RISCVDescs.OperandInfo[3442] },  // Inst #3054 = PseudoVFWCVT_RM_XU_F_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3450] },  // Inst #3053 = PseudoVFWCVT_RM_XU_F_V_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3052 = PseudoVFWCVT_F_X_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3051 = PseudoVFWCVT_F_X_V_MF8
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3050 = PseudoVFWCVT_F_X_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3049 = PseudoVFWCVT_F_X_V_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3048 = PseudoVFWCVT_F_X_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3047 = PseudoVFWCVT_F_X_V_MF2
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #3046 = PseudoVFWCVT_F_X_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #3045 = PseudoVFWCVT_F_X_V_M4
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #3044 = PseudoVFWCVT_F_X_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #3043 = PseudoVFWCVT_F_X_V_M2
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #3042 = PseudoVFWCVT_F_X_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #3041 = PseudoVFWCVT_F_X_V_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3040 = PseudoVFWCVT_F_XU_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3039 = PseudoVFWCVT_F_XU_V_MF8
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3038 = PseudoVFWCVT_F_XU_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3037 = PseudoVFWCVT_F_XU_V_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3036 = PseudoVFWCVT_F_XU_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3035 = PseudoVFWCVT_F_XU_V_MF2
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #3034 = PseudoVFWCVT_F_XU_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #3033 = PseudoVFWCVT_F_XU_V_M4
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #3032 = PseudoVFWCVT_F_XU_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #3031 = PseudoVFWCVT_F_XU_V_M2
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #3030 = PseudoVFWCVT_F_XU_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #3029 = PseudoVFWCVT_F_XU_V_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3028 = PseudoVFWCVT_F_F_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3027 = PseudoVFWCVT_F_F_V_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3026 = PseudoVFWCVT_F_F_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3025 = PseudoVFWCVT_F_F_V_MF2
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #3024 = PseudoVFWCVT_F_F_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #3023 = PseudoVFWCVT_F_F_V_M4
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #3022 = PseudoVFWCVT_F_F_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #3021 = PseudoVFWCVT_F_F_V_M2
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #3020 = PseudoVFWCVT_F_F_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #3019 = PseudoVFWCVT_F_F_V_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3018 = PseudoVFWCVTBF16_F_F_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3017 = PseudoVFWCVTBF16_F_F_V_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #3016 = PseudoVFWCVTBF16_F_F_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #3015 = PseudoVFWCVTBF16_F_F_V_MF2
  { 7, &RISCVDescs.OperandInfo[3553] },  // Inst #3014 = PseudoVFWCVTBF16_F_F_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[3547] },  // Inst #3013 = PseudoVFWCVTBF16_F_F_V_M4
  { 7, &RISCVDescs.OperandInfo[3540] },  // Inst #3012 = PseudoVFWCVTBF16_F_F_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[3534] },  // Inst #3011 = PseudoVFWCVTBF16_F_F_V_M2
  { 7, &RISCVDescs.OperandInfo[3527] },  // Inst #3010 = PseudoVFWCVTBF16_F_F_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[3521] },  // Inst #3009 = PseudoVFWCVTBF16_F_F_V_M1
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3008 = PseudoVFWADD_WV_MF4_TIED
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3007 = PseudoVFWADD_WV_MF4_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3006 = PseudoVFWADD_WV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3005 = PseudoVFWADD_WV_MF4
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #3004 = PseudoVFWADD_WV_MF2_TIED
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #3003 = PseudoVFWADD_WV_MF2_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #3002 = PseudoVFWADD_WV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #3001 = PseudoVFWADD_WV_MF2
  { 7, &RISCVDescs.OperandInfo[3514] },  // Inst #3000 = PseudoVFWADD_WV_M4_TIED
  { 8, &RISCVDescs.OperandInfo[3506] },  // Inst #2999 = PseudoVFWADD_WV_M4_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3497] },  // Inst #2998 = PseudoVFWADD_WV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3489] },  // Inst #2997 = PseudoVFWADD_WV_M4
  { 7, &RISCVDescs.OperandInfo[3482] },  // Inst #2996 = PseudoVFWADD_WV_M2_TIED
  { 8, &RISCVDescs.OperandInfo[3474] },  // Inst #2995 = PseudoVFWADD_WV_M2_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3465] },  // Inst #2994 = PseudoVFWADD_WV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3457] },  // Inst #2993 = PseudoVFWADD_WV_M2
  { 7, &RISCVDescs.OperandInfo[3450] },  // Inst #2992 = PseudoVFWADD_WV_M1_TIED
  { 8, &RISCVDescs.OperandInfo[3442] },  // Inst #2991 = PseudoVFWADD_WV_M1_MASK_TIED
  { 9, &RISCVDescs.OperandInfo[3433] },  // Inst #2990 = PseudoVFWADD_WV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3425] },  // Inst #2989 = PseudoVFWADD_WV_M1
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #2988 = PseudoVFWADD_WFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #2987 = PseudoVFWADD_WFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2009] },  // Inst #2986 = PseudoVFWADD_WFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2001] },  // Inst #2985 = PseudoVFWADD_WFPR32_M4
  { 9, &RISCVDescs.OperandInfo[1992] },  // Inst #2984 = PseudoVFWADD_WFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1984] },  // Inst #2983 = PseudoVFWADD_WFPR32_M2
  { 9, &RISCVDescs.OperandInfo[1975] },  // Inst #2982 = PseudoVFWADD_WFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1967] },  // Inst #2981 = PseudoVFWADD_WFPR32_M1
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2980 = PseudoVFWADD_WFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2979 = PseudoVFWADD_WFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2978 = PseudoVFWADD_WFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2977 = PseudoVFWADD_WFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[1941] },  // Inst #2976 = PseudoVFWADD_WFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1933] },  // Inst #2975 = PseudoVFWADD_WFPR16_M4
  { 9, &RISCVDescs.OperandInfo[1924] },  // Inst #2974 = PseudoVFWADD_WFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1916] },  // Inst #2973 = PseudoVFWADD_WFPR16_M2
  { 9, &RISCVDescs.OperandInfo[1907] },  // Inst #2972 = PseudoVFWADD_WFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1899] },  // Inst #2971 = PseudoVFWADD_WFPR16_M1
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #2970 = PseudoVFWADD_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #2969 = PseudoVFWADD_VV_MF4
  { 9, &RISCVDescs.OperandInfo[3416] },  // Inst #2968 = PseudoVFWADD_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3408] },  // Inst #2967 = PseudoVFWADD_VV_MF2
  { 9, &RISCVDescs.OperandInfo[3399] },  // Inst #2966 = PseudoVFWADD_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3391] },  // Inst #2965 = PseudoVFWADD_VV_M4
  { 9, &RISCVDescs.OperandInfo[3382] },  // Inst #2964 = PseudoVFWADD_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3374] },  // Inst #2963 = PseudoVFWADD_VV_M2
  { 9, &RISCVDescs.OperandInfo[3365] },  // Inst #2962 = PseudoVFWADD_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3357] },  // Inst #2961 = PseudoVFWADD_VV_M1
  { 9, &RISCVDescs.OperandInfo[2953] },  // Inst #2960 = PseudoVFWADD_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2945] },  // Inst #2959 = PseudoVFWADD_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[3348] },  // Inst #2958 = PseudoVFWADD_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3340] },  // Inst #2957 = PseudoVFWADD_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[3331] },  // Inst #2956 = PseudoVFWADD_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3323] },  // Inst #2955 = PseudoVFWADD_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[3314] },  // Inst #2954 = PseudoVFWADD_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3306] },  // Inst #2953 = PseudoVFWADD_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[3297] },  // Inst #2952 = PseudoVFWADD_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[3289] },  // Inst #2951 = PseudoVFWADD_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[3297] },  // Inst #2950 = PseudoVFWADD_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[3289] },  // Inst #2949 = PseudoVFWADD_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[3280] },  // Inst #2948 = PseudoVFWADD_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[3272] },  // Inst #2947 = PseudoVFWADD_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[3263] },  // Inst #2946 = PseudoVFWADD_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[3255] },  // Inst #2945 = PseudoVFWADD_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[3246] },  // Inst #2944 = PseudoVFWADD_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[3238] },  // Inst #2943 = PseudoVFWADD_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2942 = PseudoVFSUB_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2941 = PseudoVFSUB_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2940 = PseudoVFSUB_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2939 = PseudoVFSUB_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #2938 = PseudoVFSUB_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #2937 = PseudoVFSUB_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #2936 = PseudoVFSUB_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #2935 = PseudoVFSUB_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #2934 = PseudoVFSUB_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #2933 = PseudoVFSUB_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2932 = PseudoVFSUB_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2931 = PseudoVFSUB_VV_M1
  { 9, &RISCVDescs.OperandInfo[2077] },  // Inst #2930 = PseudoVFSUB_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2069] },  // Inst #2929 = PseudoVFSUB_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2060] },  // Inst #2928 = PseudoVFSUB_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2052] },  // Inst #2927 = PseudoVFSUB_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2043] },  // Inst #2926 = PseudoVFSUB_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2035] },  // Inst #2925 = PseudoVFSUB_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2026] },  // Inst #2924 = PseudoVFSUB_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2018] },  // Inst #2923 = PseudoVFSUB_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #2922 = PseudoVFSUB_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #2921 = PseudoVFSUB_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2009] },  // Inst #2920 = PseudoVFSUB_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2001] },  // Inst #2919 = PseudoVFSUB_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[1992] },  // Inst #2918 = PseudoVFSUB_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1984] },  // Inst #2917 = PseudoVFSUB_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[1975] },  // Inst #2916 = PseudoVFSUB_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1967] },  // Inst #2915 = PseudoVFSUB_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #2914 = PseudoVFSUB_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #2913 = PseudoVFSUB_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2912 = PseudoVFSUB_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2911 = PseudoVFSUB_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2910 = PseudoVFSUB_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2909 = PseudoVFSUB_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[1941] },  // Inst #2908 = PseudoVFSUB_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[1933] },  // Inst #2907 = PseudoVFSUB_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[1924] },  // Inst #2906 = PseudoVFSUB_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1916] },  // Inst #2905 = PseudoVFSUB_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[1907] },  // Inst #2904 = PseudoVFSUB_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1899] },  // Inst #2903 = PseudoVFSUB_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2902 = PseudoVFSUB_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2901 = PseudoVFSUB_VFPR16_M1
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #2900 = PseudoVFSQRT_V_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #2899 = PseudoVFSQRT_V_MF4_E16
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #2898 = PseudoVFSQRT_V_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #2897 = PseudoVFSQRT_V_MF2_E32
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #2896 = PseudoVFSQRT_V_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #2895 = PseudoVFSQRT_V_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #2894 = PseudoVFSQRT_V_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #2893 = PseudoVFSQRT_V_M8_E64
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #2892 = PseudoVFSQRT_V_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #2891 = PseudoVFSQRT_V_M8_E32
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #2890 = PseudoVFSQRT_V_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #2889 = PseudoVFSQRT_V_M8_E16
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #2888 = PseudoVFSQRT_V_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #2887 = PseudoVFSQRT_V_M4_E64
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #2886 = PseudoVFSQRT_V_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #2885 = PseudoVFSQRT_V_M4_E32
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #2884 = PseudoVFSQRT_V_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #2883 = PseudoVFSQRT_V_M4_E16
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #2882 = PseudoVFSQRT_V_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #2881 = PseudoVFSQRT_V_M2_E64
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #2880 = PseudoVFSQRT_V_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #2879 = PseudoVFSQRT_V_M2_E32
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #2878 = PseudoVFSQRT_V_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #2877 = PseudoVFSQRT_V_M2_E16
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #2876 = PseudoVFSQRT_V_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #2875 = PseudoVFSQRT_V_M1_E64
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #2874 = PseudoVFSQRT_V_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #2873 = PseudoVFSQRT_V_M1_E32
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #2872 = PseudoVFSQRT_V_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #2871 = PseudoVFSQRT_V_M1_E16
  { 8, &RISCVDescs.OperandInfo[3230] },  // Inst #2870 = PseudoVFSLIDE1UP_VFPR64_M8_MASK
  { 7, &RISCVDescs.OperandInfo[3223] },  // Inst #2869 = PseudoVFSLIDE1UP_VFPR64_M8
  { 8, &RISCVDescs.OperandInfo[3215] },  // Inst #2868 = PseudoVFSLIDE1UP_VFPR64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3208] },  // Inst #2867 = PseudoVFSLIDE1UP_VFPR64_M4
  { 8, &RISCVDescs.OperandInfo[3200] },  // Inst #2866 = PseudoVFSLIDE1UP_VFPR64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3193] },  // Inst #2865 = PseudoVFSLIDE1UP_VFPR64_M2
  { 8, &RISCVDescs.OperandInfo[3185] },  // Inst #2864 = PseudoVFSLIDE1UP_VFPR64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3178] },  // Inst #2863 = PseudoVFSLIDE1UP_VFPR64_M1
  { 8, &RISCVDescs.OperandInfo[3125] },  // Inst #2862 = PseudoVFSLIDE1UP_VFPR32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3118] },  // Inst #2861 = PseudoVFSLIDE1UP_VFPR32_MF2
  { 8, &RISCVDescs.OperandInfo[3170] },  // Inst #2860 = PseudoVFSLIDE1UP_VFPR32_M8_MASK
  { 7, &RISCVDescs.OperandInfo[3163] },  // Inst #2859 = PseudoVFSLIDE1UP_VFPR32_M8
  { 8, &RISCVDescs.OperandInfo[3155] },  // Inst #2858 = PseudoVFSLIDE1UP_VFPR32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3148] },  // Inst #2857 = PseudoVFSLIDE1UP_VFPR32_M4
  { 8, &RISCVDescs.OperandInfo[3140] },  // Inst #2856 = PseudoVFSLIDE1UP_VFPR32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3133] },  // Inst #2855 = PseudoVFSLIDE1UP_VFPR32_M2
  { 8, &RISCVDescs.OperandInfo[3125] },  // Inst #2854 = PseudoVFSLIDE1UP_VFPR32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3118] },  // Inst #2853 = PseudoVFSLIDE1UP_VFPR32_M1
  { 8, &RISCVDescs.OperandInfo[3065] },  // Inst #2852 = PseudoVFSLIDE1UP_VFPR16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[3058] },  // Inst #2851 = PseudoVFSLIDE1UP_VFPR16_MF4
  { 8, &RISCVDescs.OperandInfo[3065] },  // Inst #2850 = PseudoVFSLIDE1UP_VFPR16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[3058] },  // Inst #2849 = PseudoVFSLIDE1UP_VFPR16_MF2
  { 8, &RISCVDescs.OperandInfo[3110] },  // Inst #2848 = PseudoVFSLIDE1UP_VFPR16_M8_MASK
  { 7, &RISCVDescs.OperandInfo[3103] },  // Inst #2847 = PseudoVFSLIDE1UP_VFPR16_M8
  { 8, &RISCVDescs.OperandInfo[3095] },  // Inst #2846 = PseudoVFSLIDE1UP_VFPR16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[3088] },  // Inst #2845 = PseudoVFSLIDE1UP_VFPR16_M4
  { 8, &RISCVDescs.OperandInfo[3080] },  // Inst #2844 = PseudoVFSLIDE1UP_VFPR16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[3073] },  // Inst #2843 = PseudoVFSLIDE1UP_VFPR16_M2
  { 8, &RISCVDescs.OperandInfo[3065] },  // Inst #2842 = PseudoVFSLIDE1UP_VFPR16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[3058] },  // Inst #2841 = PseudoVFSLIDE1UP_VFPR16_M1
  { 8, &RISCVDescs.OperandInfo[2522] },  // Inst #2840 = PseudoVFSLIDE1DOWN_VFPR64_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2515] },  // Inst #2839 = PseudoVFSLIDE1DOWN_VFPR64_M8
  { 8, &RISCVDescs.OperandInfo[2507] },  // Inst #2838 = PseudoVFSLIDE1DOWN_VFPR64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2500] },  // Inst #2837 = PseudoVFSLIDE1DOWN_VFPR64_M4
  { 8, &RISCVDescs.OperandInfo[2492] },  // Inst #2836 = PseudoVFSLIDE1DOWN_VFPR64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2485] },  // Inst #2835 = PseudoVFSLIDE1DOWN_VFPR64_M2
  { 8, &RISCVDescs.OperandInfo[2477] },  // Inst #2834 = PseudoVFSLIDE1DOWN_VFPR64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2470] },  // Inst #2833 = PseudoVFSLIDE1DOWN_VFPR64_M1
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #2832 = PseudoVFSLIDE1DOWN_VFPR32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #2831 = PseudoVFSLIDE1DOWN_VFPR32_MF2
  { 8, &RISCVDescs.OperandInfo[2462] },  // Inst #2830 = PseudoVFSLIDE1DOWN_VFPR32_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2455] },  // Inst #2829 = PseudoVFSLIDE1DOWN_VFPR32_M8
  { 8, &RISCVDescs.OperandInfo[2447] },  // Inst #2828 = PseudoVFSLIDE1DOWN_VFPR32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2440] },  // Inst #2827 = PseudoVFSLIDE1DOWN_VFPR32_M4
  { 8, &RISCVDescs.OperandInfo[2432] },  // Inst #2826 = PseudoVFSLIDE1DOWN_VFPR32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2425] },  // Inst #2825 = PseudoVFSLIDE1DOWN_VFPR32_M2
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #2824 = PseudoVFSLIDE1DOWN_VFPR32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #2823 = PseudoVFSLIDE1DOWN_VFPR32_M1
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2822 = PseudoVFSLIDE1DOWN_VFPR16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2821 = PseudoVFSLIDE1DOWN_VFPR16_MF4
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2820 = PseudoVFSLIDE1DOWN_VFPR16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2819 = PseudoVFSLIDE1DOWN_VFPR16_MF2
  { 8, &RISCVDescs.OperandInfo[2402] },  // Inst #2818 = PseudoVFSLIDE1DOWN_VFPR16_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2395] },  // Inst #2817 = PseudoVFSLIDE1DOWN_VFPR16_M8
  { 8, &RISCVDescs.OperandInfo[2387] },  // Inst #2816 = PseudoVFSLIDE1DOWN_VFPR16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2380] },  // Inst #2815 = PseudoVFSLIDE1DOWN_VFPR16_M4
  { 8, &RISCVDescs.OperandInfo[2372] },  // Inst #2814 = PseudoVFSLIDE1DOWN_VFPR16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2365] },  // Inst #2813 = PseudoVFSLIDE1DOWN_VFPR16_M2
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2812 = PseudoVFSLIDE1DOWN_VFPR16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2811 = PseudoVFSLIDE1DOWN_VFPR16_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2810 = PseudoVFSGNJ_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2809 = PseudoVFSGNJ_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2808 = PseudoVFSGNJ_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2807 = PseudoVFSGNJ_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #2806 = PseudoVFSGNJ_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #2805 = PseudoVFSGNJ_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #2804 = PseudoVFSGNJ_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #2803 = PseudoVFSGNJ_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #2802 = PseudoVFSGNJ_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #2801 = PseudoVFSGNJ_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2800 = PseudoVFSGNJ_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2799 = PseudoVFSGNJ_VV_M1
  { 8, &RISCVDescs.OperandInfo[2522] },  // Inst #2798 = PseudoVFSGNJ_VFPR64_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2515] },  // Inst #2797 = PseudoVFSGNJ_VFPR64_M8
  { 8, &RISCVDescs.OperandInfo[2507] },  // Inst #2796 = PseudoVFSGNJ_VFPR64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2500] },  // Inst #2795 = PseudoVFSGNJ_VFPR64_M4
  { 8, &RISCVDescs.OperandInfo[2492] },  // Inst #2794 = PseudoVFSGNJ_VFPR64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2485] },  // Inst #2793 = PseudoVFSGNJ_VFPR64_M2
  { 8, &RISCVDescs.OperandInfo[2477] },  // Inst #2792 = PseudoVFSGNJ_VFPR64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2470] },  // Inst #2791 = PseudoVFSGNJ_VFPR64_M1
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #2790 = PseudoVFSGNJ_VFPR32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #2789 = PseudoVFSGNJ_VFPR32_MF2
  { 8, &RISCVDescs.OperandInfo[2462] },  // Inst #2788 = PseudoVFSGNJ_VFPR32_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2455] },  // Inst #2787 = PseudoVFSGNJ_VFPR32_M8
  { 8, &RISCVDescs.OperandInfo[2447] },  // Inst #2786 = PseudoVFSGNJ_VFPR32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2440] },  // Inst #2785 = PseudoVFSGNJ_VFPR32_M4
  { 8, &RISCVDescs.OperandInfo[2432] },  // Inst #2784 = PseudoVFSGNJ_VFPR32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2425] },  // Inst #2783 = PseudoVFSGNJ_VFPR32_M2
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #2782 = PseudoVFSGNJ_VFPR32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #2781 = PseudoVFSGNJ_VFPR32_M1
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2780 = PseudoVFSGNJ_VFPR16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2779 = PseudoVFSGNJ_VFPR16_MF4
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2778 = PseudoVFSGNJ_VFPR16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2777 = PseudoVFSGNJ_VFPR16_MF2
  { 8, &RISCVDescs.OperandInfo[2402] },  // Inst #2776 = PseudoVFSGNJ_VFPR16_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2395] },  // Inst #2775 = PseudoVFSGNJ_VFPR16_M8
  { 8, &RISCVDescs.OperandInfo[2387] },  // Inst #2774 = PseudoVFSGNJ_VFPR16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2380] },  // Inst #2773 = PseudoVFSGNJ_VFPR16_M4
  { 8, &RISCVDescs.OperandInfo[2372] },  // Inst #2772 = PseudoVFSGNJ_VFPR16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2365] },  // Inst #2771 = PseudoVFSGNJ_VFPR16_M2
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2770 = PseudoVFSGNJ_VFPR16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2769 = PseudoVFSGNJ_VFPR16_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2768 = PseudoVFSGNJX_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2767 = PseudoVFSGNJX_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2766 = PseudoVFSGNJX_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2765 = PseudoVFSGNJX_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #2764 = PseudoVFSGNJX_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #2763 = PseudoVFSGNJX_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #2762 = PseudoVFSGNJX_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #2761 = PseudoVFSGNJX_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #2760 = PseudoVFSGNJX_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #2759 = PseudoVFSGNJX_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2758 = PseudoVFSGNJX_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2757 = PseudoVFSGNJX_VV_M1
  { 8, &RISCVDescs.OperandInfo[2522] },  // Inst #2756 = PseudoVFSGNJX_VFPR64_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2515] },  // Inst #2755 = PseudoVFSGNJX_VFPR64_M8
  { 8, &RISCVDescs.OperandInfo[2507] },  // Inst #2754 = PseudoVFSGNJX_VFPR64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2500] },  // Inst #2753 = PseudoVFSGNJX_VFPR64_M4
  { 8, &RISCVDescs.OperandInfo[2492] },  // Inst #2752 = PseudoVFSGNJX_VFPR64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2485] },  // Inst #2751 = PseudoVFSGNJX_VFPR64_M2
  { 8, &RISCVDescs.OperandInfo[2477] },  // Inst #2750 = PseudoVFSGNJX_VFPR64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2470] },  // Inst #2749 = PseudoVFSGNJX_VFPR64_M1
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #2748 = PseudoVFSGNJX_VFPR32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #2747 = PseudoVFSGNJX_VFPR32_MF2
  { 8, &RISCVDescs.OperandInfo[2462] },  // Inst #2746 = PseudoVFSGNJX_VFPR32_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2455] },  // Inst #2745 = PseudoVFSGNJX_VFPR32_M8
  { 8, &RISCVDescs.OperandInfo[2447] },  // Inst #2744 = PseudoVFSGNJX_VFPR32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2440] },  // Inst #2743 = PseudoVFSGNJX_VFPR32_M4
  { 8, &RISCVDescs.OperandInfo[2432] },  // Inst #2742 = PseudoVFSGNJX_VFPR32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2425] },  // Inst #2741 = PseudoVFSGNJX_VFPR32_M2
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #2740 = PseudoVFSGNJX_VFPR32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #2739 = PseudoVFSGNJX_VFPR32_M1
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2738 = PseudoVFSGNJX_VFPR16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2737 = PseudoVFSGNJX_VFPR16_MF4
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2736 = PseudoVFSGNJX_VFPR16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2735 = PseudoVFSGNJX_VFPR16_MF2
  { 8, &RISCVDescs.OperandInfo[2402] },  // Inst #2734 = PseudoVFSGNJX_VFPR16_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2395] },  // Inst #2733 = PseudoVFSGNJX_VFPR16_M8
  { 8, &RISCVDescs.OperandInfo[2387] },  // Inst #2732 = PseudoVFSGNJX_VFPR16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2380] },  // Inst #2731 = PseudoVFSGNJX_VFPR16_M4
  { 8, &RISCVDescs.OperandInfo[2372] },  // Inst #2730 = PseudoVFSGNJX_VFPR16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2365] },  // Inst #2729 = PseudoVFSGNJX_VFPR16_M2
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2728 = PseudoVFSGNJX_VFPR16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2727 = PseudoVFSGNJX_VFPR16_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2726 = PseudoVFSGNJN_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2725 = PseudoVFSGNJN_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2724 = PseudoVFSGNJN_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2723 = PseudoVFSGNJN_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #2722 = PseudoVFSGNJN_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #2721 = PseudoVFSGNJN_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #2720 = PseudoVFSGNJN_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #2719 = PseudoVFSGNJN_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #2718 = PseudoVFSGNJN_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #2717 = PseudoVFSGNJN_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2716 = PseudoVFSGNJN_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2715 = PseudoVFSGNJN_VV_M1
  { 8, &RISCVDescs.OperandInfo[2522] },  // Inst #2714 = PseudoVFSGNJN_VFPR64_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2515] },  // Inst #2713 = PseudoVFSGNJN_VFPR64_M8
  { 8, &RISCVDescs.OperandInfo[2507] },  // Inst #2712 = PseudoVFSGNJN_VFPR64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2500] },  // Inst #2711 = PseudoVFSGNJN_VFPR64_M4
  { 8, &RISCVDescs.OperandInfo[2492] },  // Inst #2710 = PseudoVFSGNJN_VFPR64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2485] },  // Inst #2709 = PseudoVFSGNJN_VFPR64_M2
  { 8, &RISCVDescs.OperandInfo[2477] },  // Inst #2708 = PseudoVFSGNJN_VFPR64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2470] },  // Inst #2707 = PseudoVFSGNJN_VFPR64_M1
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #2706 = PseudoVFSGNJN_VFPR32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #2705 = PseudoVFSGNJN_VFPR32_MF2
  { 8, &RISCVDescs.OperandInfo[2462] },  // Inst #2704 = PseudoVFSGNJN_VFPR32_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2455] },  // Inst #2703 = PseudoVFSGNJN_VFPR32_M8
  { 8, &RISCVDescs.OperandInfo[2447] },  // Inst #2702 = PseudoVFSGNJN_VFPR32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2440] },  // Inst #2701 = PseudoVFSGNJN_VFPR32_M4
  { 8, &RISCVDescs.OperandInfo[2432] },  // Inst #2700 = PseudoVFSGNJN_VFPR32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2425] },  // Inst #2699 = PseudoVFSGNJN_VFPR32_M2
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #2698 = PseudoVFSGNJN_VFPR32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #2697 = PseudoVFSGNJN_VFPR32_M1
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2696 = PseudoVFSGNJN_VFPR16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2695 = PseudoVFSGNJN_VFPR16_MF4
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2694 = PseudoVFSGNJN_VFPR16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2693 = PseudoVFSGNJN_VFPR16_MF2
  { 8, &RISCVDescs.OperandInfo[2402] },  // Inst #2692 = PseudoVFSGNJN_VFPR16_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2395] },  // Inst #2691 = PseudoVFSGNJN_VFPR16_M8
  { 8, &RISCVDescs.OperandInfo[2387] },  // Inst #2690 = PseudoVFSGNJN_VFPR16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2380] },  // Inst #2689 = PseudoVFSGNJN_VFPR16_M4
  { 8, &RISCVDescs.OperandInfo[2372] },  // Inst #2688 = PseudoVFSGNJN_VFPR16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2365] },  // Inst #2687 = PseudoVFSGNJN_VFPR16_M2
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #2686 = PseudoVFSGNJN_VFPR16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #2685 = PseudoVFSGNJN_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[2077] },  // Inst #2684 = PseudoVFRSUB_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2069] },  // Inst #2683 = PseudoVFRSUB_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2060] },  // Inst #2682 = PseudoVFRSUB_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2052] },  // Inst #2681 = PseudoVFRSUB_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2043] },  // Inst #2680 = PseudoVFRSUB_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2035] },  // Inst #2679 = PseudoVFRSUB_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2026] },  // Inst #2678 = PseudoVFRSUB_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2018] },  // Inst #2677 = PseudoVFRSUB_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #2676 = PseudoVFRSUB_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #2675 = PseudoVFRSUB_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2009] },  // Inst #2674 = PseudoVFRSUB_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2001] },  // Inst #2673 = PseudoVFRSUB_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[1992] },  // Inst #2672 = PseudoVFRSUB_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1984] },  // Inst #2671 = PseudoVFRSUB_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[1975] },  // Inst #2670 = PseudoVFRSUB_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1967] },  // Inst #2669 = PseudoVFRSUB_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #2668 = PseudoVFRSUB_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #2667 = PseudoVFRSUB_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2666 = PseudoVFRSUB_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2665 = PseudoVFRSUB_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2664 = PseudoVFRSUB_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2663 = PseudoVFRSUB_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[1941] },  // Inst #2662 = PseudoVFRSUB_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[1933] },  // Inst #2661 = PseudoVFRSUB_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[1924] },  // Inst #2660 = PseudoVFRSUB_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1916] },  // Inst #2659 = PseudoVFRSUB_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[1907] },  // Inst #2658 = PseudoVFRSUB_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1899] },  // Inst #2657 = PseudoVFRSUB_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2656 = PseudoVFRSUB_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2655 = PseudoVFRSUB_VFPR16_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #2654 = PseudoVFRSQRT7_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #2653 = PseudoVFRSQRT7_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #2652 = PseudoVFRSQRT7_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #2651 = PseudoVFRSQRT7_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #2650 = PseudoVFRSQRT7_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #2649 = PseudoVFRSQRT7_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #2648 = PseudoVFRSQRT7_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #2647 = PseudoVFRSQRT7_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #2646 = PseudoVFRSQRT7_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #2645 = PseudoVFRSQRT7_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #2644 = PseudoVFRSQRT7_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #2643 = PseudoVFRSQRT7_V_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #2642 = PseudoVFROUND_NOEXCEPT_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #2641 = PseudoVFROUND_NOEXCEPT_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #2640 = PseudoVFROUND_NOEXCEPT_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #2639 = PseudoVFROUND_NOEXCEPT_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #2638 = PseudoVFROUND_NOEXCEPT_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #2637 = PseudoVFROUND_NOEXCEPT_V_M1_MASK
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2636 = PseudoVFREDUSUM_VS_MF4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2635 = PseudoVFREDUSUM_VS_MF4_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2634 = PseudoVFREDUSUM_VS_MF2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2633 = PseudoVFREDUSUM_VS_MF2_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2632 = PseudoVFREDUSUM_VS_MF2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2631 = PseudoVFREDUSUM_VS_MF2_E16
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #2630 = PseudoVFREDUSUM_VS_M8_E64_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #2629 = PseudoVFREDUSUM_VS_M8_E64
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #2628 = PseudoVFREDUSUM_VS_M8_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #2627 = PseudoVFREDUSUM_VS_M8_E32
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #2626 = PseudoVFREDUSUM_VS_M8_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #2625 = PseudoVFREDUSUM_VS_M8_E16
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #2624 = PseudoVFREDUSUM_VS_M4_E64_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #2623 = PseudoVFREDUSUM_VS_M4_E64
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #2622 = PseudoVFREDUSUM_VS_M4_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #2621 = PseudoVFREDUSUM_VS_M4_E32
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #2620 = PseudoVFREDUSUM_VS_M4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #2619 = PseudoVFREDUSUM_VS_M4_E16
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #2618 = PseudoVFREDUSUM_VS_M2_E64_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #2617 = PseudoVFREDUSUM_VS_M2_E64
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #2616 = PseudoVFREDUSUM_VS_M2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #2615 = PseudoVFREDUSUM_VS_M2_E32
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #2614 = PseudoVFREDUSUM_VS_M2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #2613 = PseudoVFREDUSUM_VS_M2_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2612 = PseudoVFREDUSUM_VS_M1_E64_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2611 = PseudoVFREDUSUM_VS_M1_E64
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2610 = PseudoVFREDUSUM_VS_M1_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2609 = PseudoVFREDUSUM_VS_M1_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2608 = PseudoVFREDUSUM_VS_M1_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2607 = PseudoVFREDUSUM_VS_M1_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2606 = PseudoVFREDOSUM_VS_MF4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2605 = PseudoVFREDOSUM_VS_MF4_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2604 = PseudoVFREDOSUM_VS_MF2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2603 = PseudoVFREDOSUM_VS_MF2_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2602 = PseudoVFREDOSUM_VS_MF2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2601 = PseudoVFREDOSUM_VS_MF2_E16
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #2600 = PseudoVFREDOSUM_VS_M8_E64_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #2599 = PseudoVFREDOSUM_VS_M8_E64
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #2598 = PseudoVFREDOSUM_VS_M8_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #2597 = PseudoVFREDOSUM_VS_M8_E32
  { 9, &RISCVDescs.OperandInfo[3049] },  // Inst #2596 = PseudoVFREDOSUM_VS_M8_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3041] },  // Inst #2595 = PseudoVFREDOSUM_VS_M8_E16
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #2594 = PseudoVFREDOSUM_VS_M4_E64_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #2593 = PseudoVFREDOSUM_VS_M4_E64
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #2592 = PseudoVFREDOSUM_VS_M4_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #2591 = PseudoVFREDOSUM_VS_M4_E32
  { 9, &RISCVDescs.OperandInfo[3032] },  // Inst #2590 = PseudoVFREDOSUM_VS_M4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3024] },  // Inst #2589 = PseudoVFREDOSUM_VS_M4_E16
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #2588 = PseudoVFREDOSUM_VS_M2_E64_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #2587 = PseudoVFREDOSUM_VS_M2_E64
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #2586 = PseudoVFREDOSUM_VS_M2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #2585 = PseudoVFREDOSUM_VS_M2_E32
  { 9, &RISCVDescs.OperandInfo[3015] },  // Inst #2584 = PseudoVFREDOSUM_VS_M2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[3007] },  // Inst #2583 = PseudoVFREDOSUM_VS_M2_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2582 = PseudoVFREDOSUM_VS_M1_E64_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2581 = PseudoVFREDOSUM_VS_M1_E64
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2580 = PseudoVFREDOSUM_VS_M1_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2579 = PseudoVFREDOSUM_VS_M1_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2578 = PseudoVFREDOSUM_VS_M1_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2577 = PseudoVFREDOSUM_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2576 = PseudoVFREDMIN_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2575 = PseudoVFREDMIN_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2574 = PseudoVFREDMIN_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2573 = PseudoVFREDMIN_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2572 = PseudoVFREDMIN_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2571 = PseudoVFREDMIN_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #2570 = PseudoVFREDMIN_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #2569 = PseudoVFREDMIN_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #2568 = PseudoVFREDMIN_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #2567 = PseudoVFREDMIN_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #2566 = PseudoVFREDMIN_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #2565 = PseudoVFREDMIN_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #2564 = PseudoVFREDMIN_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #2563 = PseudoVFREDMIN_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #2562 = PseudoVFREDMIN_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #2561 = PseudoVFREDMIN_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #2560 = PseudoVFREDMIN_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #2559 = PseudoVFREDMIN_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #2558 = PseudoVFREDMIN_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #2557 = PseudoVFREDMIN_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #2556 = PseudoVFREDMIN_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #2555 = PseudoVFREDMIN_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #2554 = PseudoVFREDMIN_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #2553 = PseudoVFREDMIN_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2552 = PseudoVFREDMIN_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2551 = PseudoVFREDMIN_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2550 = PseudoVFREDMIN_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2549 = PseudoVFREDMIN_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2548 = PseudoVFREDMIN_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2547 = PseudoVFREDMIN_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2546 = PseudoVFREDMAX_VS_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2545 = PseudoVFREDMAX_VS_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2544 = PseudoVFREDMAX_VS_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2543 = PseudoVFREDMAX_VS_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2542 = PseudoVFREDMAX_VS_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2541 = PseudoVFREDMAX_VS_MF2_E16
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #2540 = PseudoVFREDMAX_VS_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #2539 = PseudoVFREDMAX_VS_M8_E64
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #2538 = PseudoVFREDMAX_VS_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #2537 = PseudoVFREDMAX_VS_M8_E32
  { 8, &RISCVDescs.OperandInfo[2999] },  // Inst #2536 = PseudoVFREDMAX_VS_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2992] },  // Inst #2535 = PseudoVFREDMAX_VS_M8_E16
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #2534 = PseudoVFREDMAX_VS_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #2533 = PseudoVFREDMAX_VS_M4_E64
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #2532 = PseudoVFREDMAX_VS_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #2531 = PseudoVFREDMAX_VS_M4_E32
  { 8, &RISCVDescs.OperandInfo[2984] },  // Inst #2530 = PseudoVFREDMAX_VS_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2977] },  // Inst #2529 = PseudoVFREDMAX_VS_M4_E16
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #2528 = PseudoVFREDMAX_VS_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #2527 = PseudoVFREDMAX_VS_M2_E64
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #2526 = PseudoVFREDMAX_VS_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #2525 = PseudoVFREDMAX_VS_M2_E32
  { 8, &RISCVDescs.OperandInfo[2969] },  // Inst #2524 = PseudoVFREDMAX_VS_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[2962] },  // Inst #2523 = PseudoVFREDMAX_VS_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2522 = PseudoVFREDMAX_VS_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2521 = PseudoVFREDMAX_VS_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2520 = PseudoVFREDMAX_VS_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2519 = PseudoVFREDMAX_VS_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #2518 = PseudoVFREDMAX_VS_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #2517 = PseudoVFREDMAX_VS_M1_E16
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #2516 = PseudoVFREC7_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #2515 = PseudoVFREC7_V_MF4
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #2514 = PseudoVFREC7_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #2513 = PseudoVFREC7_V_MF2
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #2512 = PseudoVFREC7_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #2511 = PseudoVFREC7_V_M8
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #2510 = PseudoVFREC7_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #2509 = PseudoVFREC7_V_M4
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #2508 = PseudoVFREC7_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #2507 = PseudoVFREC7_V_M2
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #2506 = PseudoVFREC7_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #2505 = PseudoVFREC7_V_M1
  { 9, &RISCVDescs.OperandInfo[2077] },  // Inst #2504 = PseudoVFRDIV_VFPR64_M8_E64_MASK
  { 8, &RISCVDescs.OperandInfo[2069] },  // Inst #2503 = PseudoVFRDIV_VFPR64_M8_E64
  { 9, &RISCVDescs.OperandInfo[2060] },  // Inst #2502 = PseudoVFRDIV_VFPR64_M4_E64_MASK
  { 8, &RISCVDescs.OperandInfo[2052] },  // Inst #2501 = PseudoVFRDIV_VFPR64_M4_E64
  { 9, &RISCVDescs.OperandInfo[2043] },  // Inst #2500 = PseudoVFRDIV_VFPR64_M2_E64_MASK
  { 8, &RISCVDescs.OperandInfo[2035] },  // Inst #2499 = PseudoVFRDIV_VFPR64_M2_E64
  { 9, &RISCVDescs.OperandInfo[2026] },  // Inst #2498 = PseudoVFRDIV_VFPR64_M1_E64_MASK
  { 8, &RISCVDescs.OperandInfo[2018] },  // Inst #2497 = PseudoVFRDIV_VFPR64_M1_E64
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #2496 = PseudoVFRDIV_VFPR32_MF2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #2495 = PseudoVFRDIV_VFPR32_MF2_E32
  { 9, &RISCVDescs.OperandInfo[2009] },  // Inst #2494 = PseudoVFRDIV_VFPR32_M8_E32_MASK
  { 8, &RISCVDescs.OperandInfo[2001] },  // Inst #2493 = PseudoVFRDIV_VFPR32_M8_E32
  { 9, &RISCVDescs.OperandInfo[1992] },  // Inst #2492 = PseudoVFRDIV_VFPR32_M4_E32_MASK
  { 8, &RISCVDescs.OperandInfo[1984] },  // Inst #2491 = PseudoVFRDIV_VFPR32_M4_E32
  { 9, &RISCVDescs.OperandInfo[1975] },  // Inst #2490 = PseudoVFRDIV_VFPR32_M2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[1967] },  // Inst #2489 = PseudoVFRDIV_VFPR32_M2_E32
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #2488 = PseudoVFRDIV_VFPR32_M1_E32_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #2487 = PseudoVFRDIV_VFPR32_M1_E32
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2486 = PseudoVFRDIV_VFPR16_MF4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2485 = PseudoVFRDIV_VFPR16_MF4_E16
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2484 = PseudoVFRDIV_VFPR16_MF2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2483 = PseudoVFRDIV_VFPR16_MF2_E16
  { 9, &RISCVDescs.OperandInfo[1941] },  // Inst #2482 = PseudoVFRDIV_VFPR16_M8_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1933] },  // Inst #2481 = PseudoVFRDIV_VFPR16_M8_E16
  { 9, &RISCVDescs.OperandInfo[1924] },  // Inst #2480 = PseudoVFRDIV_VFPR16_M4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1916] },  // Inst #2479 = PseudoVFRDIV_VFPR16_M4_E16
  { 9, &RISCVDescs.OperandInfo[1907] },  // Inst #2478 = PseudoVFRDIV_VFPR16_M2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1899] },  // Inst #2477 = PseudoVFRDIV_VFPR16_M2_E16
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2476 = PseudoVFRDIV_VFPR16_M1_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2475 = PseudoVFRDIV_VFPR16_M1_E16
  { 9, &RISCVDescs.OperandInfo[2953] },  // Inst #2474 = PseudoVFNRCLIP_X_F_QF_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[2945] },  // Inst #2473 = PseudoVFNRCLIP_X_F_QF_MF8
  { 9, &RISCVDescs.OperandInfo[2953] },  // Inst #2472 = PseudoVFNRCLIP_X_F_QF_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2945] },  // Inst #2471 = PseudoVFNRCLIP_X_F_QF_MF4
  { 9, &RISCVDescs.OperandInfo[2936] },  // Inst #2470 = PseudoVFNRCLIP_X_F_QF_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2928] },  // Inst #2469 = PseudoVFNRCLIP_X_F_QF_MF2
  { 9, &RISCVDescs.OperandInfo[2919] },  // Inst #2468 = PseudoVFNRCLIP_X_F_QF_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2911] },  // Inst #2467 = PseudoVFNRCLIP_X_F_QF_M2
  { 9, &RISCVDescs.OperandInfo[2902] },  // Inst #2466 = PseudoVFNRCLIP_X_F_QF_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2894] },  // Inst #2465 = PseudoVFNRCLIP_X_F_QF_M1
  { 9, &RISCVDescs.OperandInfo[2953] },  // Inst #2464 = PseudoVFNRCLIP_XU_F_QF_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[2945] },  // Inst #2463 = PseudoVFNRCLIP_XU_F_QF_MF8
  { 9, &RISCVDescs.OperandInfo[2953] },  // Inst #2462 = PseudoVFNRCLIP_XU_F_QF_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2945] },  // Inst #2461 = PseudoVFNRCLIP_XU_F_QF_MF4
  { 9, &RISCVDescs.OperandInfo[2936] },  // Inst #2460 = PseudoVFNRCLIP_XU_F_QF_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2928] },  // Inst #2459 = PseudoVFNRCLIP_XU_F_QF_MF2
  { 9, &RISCVDescs.OperandInfo[2919] },  // Inst #2458 = PseudoVFNRCLIP_XU_F_QF_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2911] },  // Inst #2457 = PseudoVFNRCLIP_XU_F_QF_M2
  { 9, &RISCVDescs.OperandInfo[2902] },  // Inst #2456 = PseudoVFNRCLIP_XU_F_QF_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2894] },  // Inst #2455 = PseudoVFNRCLIP_XU_F_QF_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2454 = PseudoVFNMSUB_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2453 = PseudoVFNMSUB_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2452 = PseudoVFNMSUB_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2451 = PseudoVFNMSUB_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #2450 = PseudoVFNMSUB_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #2449 = PseudoVFNMSUB_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #2448 = PseudoVFNMSUB_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #2447 = PseudoVFNMSUB_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #2446 = PseudoVFNMSUB_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #2445 = PseudoVFNMSUB_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2444 = PseudoVFNMSUB_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2443 = PseudoVFNMSUB_VV_M1
  { 9, &RISCVDescs.OperandInfo[2341] },  // Inst #2442 = PseudoVFNMSUB_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2333] },  // Inst #2441 = PseudoVFNMSUB_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2324] },  // Inst #2440 = PseudoVFNMSUB_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2316] },  // Inst #2439 = PseudoVFNMSUB_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2307] },  // Inst #2438 = PseudoVFNMSUB_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2299] },  // Inst #2437 = PseudoVFNMSUB_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2290] },  // Inst #2436 = PseudoVFNMSUB_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2282] },  // Inst #2435 = PseudoVFNMSUB_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2434 = PseudoVFNMSUB_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2433 = PseudoVFNMSUB_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2273] },  // Inst #2432 = PseudoVFNMSUB_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2265] },  // Inst #2431 = PseudoVFNMSUB_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[2256] },  // Inst #2430 = PseudoVFNMSUB_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2248] },  // Inst #2429 = PseudoVFNMSUB_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[2239] },  // Inst #2428 = PseudoVFNMSUB_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2231] },  // Inst #2427 = PseudoVFNMSUB_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2426 = PseudoVFNMSUB_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2425 = PseudoVFNMSUB_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2424 = PseudoVFNMSUB_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2423 = PseudoVFNMSUB_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2422 = PseudoVFNMSUB_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2421 = PseudoVFNMSUB_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[2205] },  // Inst #2420 = PseudoVFNMSUB_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2197] },  // Inst #2419 = PseudoVFNMSUB_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[2188] },  // Inst #2418 = PseudoVFNMSUB_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2180] },  // Inst #2417 = PseudoVFNMSUB_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[2171] },  // Inst #2416 = PseudoVFNMSUB_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2163] },  // Inst #2415 = PseudoVFNMSUB_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2414 = PseudoVFNMSUB_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2413 = PseudoVFNMSUB_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2412 = PseudoVFNMSAC_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2411 = PseudoVFNMSAC_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2410 = PseudoVFNMSAC_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2409 = PseudoVFNMSAC_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #2408 = PseudoVFNMSAC_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #2407 = PseudoVFNMSAC_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #2406 = PseudoVFNMSAC_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #2405 = PseudoVFNMSAC_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #2404 = PseudoVFNMSAC_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #2403 = PseudoVFNMSAC_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2402 = PseudoVFNMSAC_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2401 = PseudoVFNMSAC_VV_M1
  { 9, &RISCVDescs.OperandInfo[2341] },  // Inst #2400 = PseudoVFNMSAC_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2333] },  // Inst #2399 = PseudoVFNMSAC_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2324] },  // Inst #2398 = PseudoVFNMSAC_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2316] },  // Inst #2397 = PseudoVFNMSAC_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2307] },  // Inst #2396 = PseudoVFNMSAC_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2299] },  // Inst #2395 = PseudoVFNMSAC_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2290] },  // Inst #2394 = PseudoVFNMSAC_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2282] },  // Inst #2393 = PseudoVFNMSAC_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2392 = PseudoVFNMSAC_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2391 = PseudoVFNMSAC_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2273] },  // Inst #2390 = PseudoVFNMSAC_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2265] },  // Inst #2389 = PseudoVFNMSAC_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[2256] },  // Inst #2388 = PseudoVFNMSAC_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2248] },  // Inst #2387 = PseudoVFNMSAC_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[2239] },  // Inst #2386 = PseudoVFNMSAC_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2231] },  // Inst #2385 = PseudoVFNMSAC_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2384 = PseudoVFNMSAC_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2383 = PseudoVFNMSAC_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2382 = PseudoVFNMSAC_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2381 = PseudoVFNMSAC_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2380 = PseudoVFNMSAC_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2379 = PseudoVFNMSAC_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[2205] },  // Inst #2378 = PseudoVFNMSAC_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2197] },  // Inst #2377 = PseudoVFNMSAC_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[2188] },  // Inst #2376 = PseudoVFNMSAC_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2180] },  // Inst #2375 = PseudoVFNMSAC_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[2171] },  // Inst #2374 = PseudoVFNMSAC_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2163] },  // Inst #2373 = PseudoVFNMSAC_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2372 = PseudoVFNMSAC_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2371 = PseudoVFNMSAC_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2370 = PseudoVFNMADD_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2369 = PseudoVFNMADD_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2368 = PseudoVFNMADD_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2367 = PseudoVFNMADD_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #2366 = PseudoVFNMADD_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #2365 = PseudoVFNMADD_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #2364 = PseudoVFNMADD_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #2363 = PseudoVFNMADD_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #2362 = PseudoVFNMADD_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #2361 = PseudoVFNMADD_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2360 = PseudoVFNMADD_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2359 = PseudoVFNMADD_VV_M1
  { 9, &RISCVDescs.OperandInfo[2341] },  // Inst #2358 = PseudoVFNMADD_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2333] },  // Inst #2357 = PseudoVFNMADD_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2324] },  // Inst #2356 = PseudoVFNMADD_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2316] },  // Inst #2355 = PseudoVFNMADD_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2307] },  // Inst #2354 = PseudoVFNMADD_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2299] },  // Inst #2353 = PseudoVFNMADD_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2290] },  // Inst #2352 = PseudoVFNMADD_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2282] },  // Inst #2351 = PseudoVFNMADD_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2350 = PseudoVFNMADD_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2349 = PseudoVFNMADD_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2273] },  // Inst #2348 = PseudoVFNMADD_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2265] },  // Inst #2347 = PseudoVFNMADD_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[2256] },  // Inst #2346 = PseudoVFNMADD_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2248] },  // Inst #2345 = PseudoVFNMADD_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[2239] },  // Inst #2344 = PseudoVFNMADD_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2231] },  // Inst #2343 = PseudoVFNMADD_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2342 = PseudoVFNMADD_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2341 = PseudoVFNMADD_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2340 = PseudoVFNMADD_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2339 = PseudoVFNMADD_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2338 = PseudoVFNMADD_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2337 = PseudoVFNMADD_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[2205] },  // Inst #2336 = PseudoVFNMADD_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2197] },  // Inst #2335 = PseudoVFNMADD_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[2188] },  // Inst #2334 = PseudoVFNMADD_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2180] },  // Inst #2333 = PseudoVFNMADD_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[2171] },  // Inst #2332 = PseudoVFNMADD_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2163] },  // Inst #2331 = PseudoVFNMADD_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2330 = PseudoVFNMADD_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2329 = PseudoVFNMADD_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2328 = PseudoVFNMACC_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2327 = PseudoVFNMACC_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2326 = PseudoVFNMACC_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2325 = PseudoVFNMACC_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #2324 = PseudoVFNMACC_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #2323 = PseudoVFNMACC_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #2322 = PseudoVFNMACC_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #2321 = PseudoVFNMACC_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #2320 = PseudoVFNMACC_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #2319 = PseudoVFNMACC_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2318 = PseudoVFNMACC_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2317 = PseudoVFNMACC_VV_M1
  { 9, &RISCVDescs.OperandInfo[2341] },  // Inst #2316 = PseudoVFNMACC_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2333] },  // Inst #2315 = PseudoVFNMACC_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2324] },  // Inst #2314 = PseudoVFNMACC_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2316] },  // Inst #2313 = PseudoVFNMACC_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2307] },  // Inst #2312 = PseudoVFNMACC_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2299] },  // Inst #2311 = PseudoVFNMACC_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2290] },  // Inst #2310 = PseudoVFNMACC_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2282] },  // Inst #2309 = PseudoVFNMACC_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2308 = PseudoVFNMACC_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2307 = PseudoVFNMACC_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2273] },  // Inst #2306 = PseudoVFNMACC_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2265] },  // Inst #2305 = PseudoVFNMACC_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[2256] },  // Inst #2304 = PseudoVFNMACC_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2248] },  // Inst #2303 = PseudoVFNMACC_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[2239] },  // Inst #2302 = PseudoVFNMACC_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2231] },  // Inst #2301 = PseudoVFNMACC_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2300 = PseudoVFNMACC_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2299 = PseudoVFNMACC_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2298 = PseudoVFNMACC_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2297 = PseudoVFNMACC_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2296 = PseudoVFNMACC_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2295 = PseudoVFNMACC_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[2205] },  // Inst #2294 = PseudoVFNMACC_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2197] },  // Inst #2293 = PseudoVFNMACC_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[2188] },  // Inst #2292 = PseudoVFNMACC_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2180] },  // Inst #2291 = PseudoVFNMACC_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[2171] },  // Inst #2290 = PseudoVFNMACC_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2163] },  // Inst #2289 = PseudoVFNMACC_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2288 = PseudoVFNMACC_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2287 = PseudoVFNMACC_VFPR16_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2286 = PseudoVFNCVT_X_F_W_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2285 = PseudoVFNCVT_X_F_W_MF8
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2284 = PseudoVFNCVT_X_F_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2283 = PseudoVFNCVT_X_F_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2282 = PseudoVFNCVT_X_F_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2281 = PseudoVFNCVT_X_F_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2280 = PseudoVFNCVT_X_F_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2279 = PseudoVFNCVT_X_F_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2278 = PseudoVFNCVT_X_F_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2277 = PseudoVFNCVT_X_F_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2276 = PseudoVFNCVT_X_F_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2275 = PseudoVFNCVT_X_F_W_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2274 = PseudoVFNCVT_XU_F_W_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2273 = PseudoVFNCVT_XU_F_W_MF8
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2272 = PseudoVFNCVT_XU_F_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2271 = PseudoVFNCVT_XU_F_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2270 = PseudoVFNCVT_XU_F_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2269 = PseudoVFNCVT_XU_F_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2268 = PseudoVFNCVT_XU_F_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2267 = PseudoVFNCVT_XU_F_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2266 = PseudoVFNCVT_XU_F_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2265 = PseudoVFNCVT_XU_F_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2264 = PseudoVFNCVT_XU_F_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2263 = PseudoVFNCVT_XU_F_W_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #2262 = PseudoVFNCVT_RTZ_X_F_W_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #2261 = PseudoVFNCVT_RTZ_X_F_W_MF8
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #2260 = PseudoVFNCVT_RTZ_X_F_W_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #2259 = PseudoVFNCVT_RTZ_X_F_W_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #2258 = PseudoVFNCVT_RTZ_X_F_W_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #2257 = PseudoVFNCVT_RTZ_X_F_W_MF2
  { 7, &RISCVDescs.OperandInfo[2874] },  // Inst #2256 = PseudoVFNCVT_RTZ_X_F_W_M4_MASK
  { 6, &RISCVDescs.OperandInfo[2868] },  // Inst #2255 = PseudoVFNCVT_RTZ_X_F_W_M4
  { 7, &RISCVDescs.OperandInfo[2861] },  // Inst #2254 = PseudoVFNCVT_RTZ_X_F_W_M2_MASK
  { 6, &RISCVDescs.OperandInfo[2855] },  // Inst #2253 = PseudoVFNCVT_RTZ_X_F_W_M2
  { 7, &RISCVDescs.OperandInfo[2848] },  // Inst #2252 = PseudoVFNCVT_RTZ_X_F_W_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2842] },  // Inst #2251 = PseudoVFNCVT_RTZ_X_F_W_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #2250 = PseudoVFNCVT_RTZ_XU_F_W_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #2249 = PseudoVFNCVT_RTZ_XU_F_W_MF8
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #2248 = PseudoVFNCVT_RTZ_XU_F_W_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #2247 = PseudoVFNCVT_RTZ_XU_F_W_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #2246 = PseudoVFNCVT_RTZ_XU_F_W_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #2245 = PseudoVFNCVT_RTZ_XU_F_W_MF2
  { 7, &RISCVDescs.OperandInfo[2874] },  // Inst #2244 = PseudoVFNCVT_RTZ_XU_F_W_M4_MASK
  { 6, &RISCVDescs.OperandInfo[2868] },  // Inst #2243 = PseudoVFNCVT_RTZ_XU_F_W_M4
  { 7, &RISCVDescs.OperandInfo[2861] },  // Inst #2242 = PseudoVFNCVT_RTZ_XU_F_W_M2_MASK
  { 6, &RISCVDescs.OperandInfo[2855] },  // Inst #2241 = PseudoVFNCVT_RTZ_XU_F_W_M2
  { 7, &RISCVDescs.OperandInfo[2848] },  // Inst #2240 = PseudoVFNCVT_RTZ_XU_F_W_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2842] },  // Inst #2239 = PseudoVFNCVT_RTZ_XU_F_W_M1
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #2238 = PseudoVFNCVT_ROD_F_F_W_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #2237 = PseudoVFNCVT_ROD_F_F_W_MF4
  { 7, &RISCVDescs.OperandInfo[2887] },  // Inst #2236 = PseudoVFNCVT_ROD_F_F_W_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[2881] },  // Inst #2235 = PseudoVFNCVT_ROD_F_F_W_MF2
  { 7, &RISCVDescs.OperandInfo[2874] },  // Inst #2234 = PseudoVFNCVT_ROD_F_F_W_M4_MASK
  { 6, &RISCVDescs.OperandInfo[2868] },  // Inst #2233 = PseudoVFNCVT_ROD_F_F_W_M4
  { 7, &RISCVDescs.OperandInfo[2861] },  // Inst #2232 = PseudoVFNCVT_ROD_F_F_W_M2_MASK
  { 6, &RISCVDescs.OperandInfo[2855] },  // Inst #2231 = PseudoVFNCVT_ROD_F_F_W_M2
  { 7, &RISCVDescs.OperandInfo[2848] },  // Inst #2230 = PseudoVFNCVT_ROD_F_F_W_M1_MASK
  { 6, &RISCVDescs.OperandInfo[2842] },  // Inst #2229 = PseudoVFNCVT_ROD_F_F_W_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2228 = PseudoVFNCVT_RM_X_F_W_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2227 = PseudoVFNCVT_RM_X_F_W_MF8
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2226 = PseudoVFNCVT_RM_X_F_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2225 = PseudoVFNCVT_RM_X_F_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2224 = PseudoVFNCVT_RM_X_F_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2223 = PseudoVFNCVT_RM_X_F_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2222 = PseudoVFNCVT_RM_X_F_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2221 = PseudoVFNCVT_RM_X_F_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2220 = PseudoVFNCVT_RM_X_F_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2219 = PseudoVFNCVT_RM_X_F_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2218 = PseudoVFNCVT_RM_X_F_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2217 = PseudoVFNCVT_RM_X_F_W_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2216 = PseudoVFNCVT_RM_XU_F_W_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2215 = PseudoVFNCVT_RM_XU_F_W_MF8
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2214 = PseudoVFNCVT_RM_XU_F_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2213 = PseudoVFNCVT_RM_XU_F_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2212 = PseudoVFNCVT_RM_XU_F_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2211 = PseudoVFNCVT_RM_XU_F_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2210 = PseudoVFNCVT_RM_XU_F_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2209 = PseudoVFNCVT_RM_XU_F_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2208 = PseudoVFNCVT_RM_XU_F_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2207 = PseudoVFNCVT_RM_XU_F_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2206 = PseudoVFNCVT_RM_XU_F_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2205 = PseudoVFNCVT_RM_XU_F_W_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2204 = PseudoVFNCVT_RM_F_X_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2203 = PseudoVFNCVT_RM_F_X_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2202 = PseudoVFNCVT_RM_F_X_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2201 = PseudoVFNCVT_RM_F_X_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2200 = PseudoVFNCVT_RM_F_X_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2199 = PseudoVFNCVT_RM_F_X_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2198 = PseudoVFNCVT_RM_F_X_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2197 = PseudoVFNCVT_RM_F_X_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2196 = PseudoVFNCVT_RM_F_X_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2195 = PseudoVFNCVT_RM_F_X_W_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2194 = PseudoVFNCVT_RM_F_XU_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2193 = PseudoVFNCVT_RM_F_XU_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2192 = PseudoVFNCVT_RM_F_XU_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2191 = PseudoVFNCVT_RM_F_XU_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2190 = PseudoVFNCVT_RM_F_XU_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2189 = PseudoVFNCVT_RM_F_XU_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2188 = PseudoVFNCVT_RM_F_XU_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2187 = PseudoVFNCVT_RM_F_XU_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2186 = PseudoVFNCVT_RM_F_XU_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2185 = PseudoVFNCVT_RM_F_XU_W_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2184 = PseudoVFNCVT_F_X_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2183 = PseudoVFNCVT_F_X_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2182 = PseudoVFNCVT_F_X_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2181 = PseudoVFNCVT_F_X_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2180 = PseudoVFNCVT_F_X_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2179 = PseudoVFNCVT_F_X_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2178 = PseudoVFNCVT_F_X_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2177 = PseudoVFNCVT_F_X_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2176 = PseudoVFNCVT_F_X_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2175 = PseudoVFNCVT_F_X_W_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2174 = PseudoVFNCVT_F_XU_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2173 = PseudoVFNCVT_F_XU_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2172 = PseudoVFNCVT_F_XU_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2171 = PseudoVFNCVT_F_XU_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2170 = PseudoVFNCVT_F_XU_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2169 = PseudoVFNCVT_F_XU_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2168 = PseudoVFNCVT_F_XU_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2167 = PseudoVFNCVT_F_XU_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2166 = PseudoVFNCVT_F_XU_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2165 = PseudoVFNCVT_F_XU_W_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2164 = PseudoVFNCVT_F_F_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2163 = PseudoVFNCVT_F_F_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2162 = PseudoVFNCVT_F_F_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2161 = PseudoVFNCVT_F_F_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2160 = PseudoVFNCVT_F_F_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2159 = PseudoVFNCVT_F_F_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2158 = PseudoVFNCVT_F_F_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2157 = PseudoVFNCVT_F_F_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2156 = PseudoVFNCVT_F_F_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2155 = PseudoVFNCVT_F_F_W_M1
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2154 = PseudoVFNCVTBF16_F_F_W_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2153 = PseudoVFNCVTBF16_F_F_W_MF4
  { 8, &RISCVDescs.OperandInfo[2834] },  // Inst #2152 = PseudoVFNCVTBF16_F_F_W_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2827] },  // Inst #2151 = PseudoVFNCVTBF16_F_F_W_MF2
  { 8, &RISCVDescs.OperandInfo[2819] },  // Inst #2150 = PseudoVFNCVTBF16_F_F_W_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2812] },  // Inst #2149 = PseudoVFNCVTBF16_F_F_W_M4
  { 8, &RISCVDescs.OperandInfo[2804] },  // Inst #2148 = PseudoVFNCVTBF16_F_F_W_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2797] },  // Inst #2147 = PseudoVFNCVTBF16_F_F_W_M2
  { 8, &RISCVDescs.OperandInfo[2789] },  // Inst #2146 = PseudoVFNCVTBF16_F_F_W_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2782] },  // Inst #2145 = PseudoVFNCVTBF16_F_F_W_M1
  { 6, &RISCVDescs.OperandInfo[2776] },  // Inst #2144 = PseudoVFMV_V_FPR64_M8
  { 6, &RISCVDescs.OperandInfo[2770] },  // Inst #2143 = PseudoVFMV_V_FPR64_M4
  { 6, &RISCVDescs.OperandInfo[2764] },  // Inst #2142 = PseudoVFMV_V_FPR64_M2
  { 6, &RISCVDescs.OperandInfo[2758] },  // Inst #2141 = PseudoVFMV_V_FPR64_M1
  { 6, &RISCVDescs.OperandInfo[2734] },  // Inst #2140 = PseudoVFMV_V_FPR32_MF2
  { 6, &RISCVDescs.OperandInfo[2752] },  // Inst #2139 = PseudoVFMV_V_FPR32_M8
  { 6, &RISCVDescs.OperandInfo[2746] },  // Inst #2138 = PseudoVFMV_V_FPR32_M4
  { 6, &RISCVDescs.OperandInfo[2740] },  // Inst #2137 = PseudoVFMV_V_FPR32_M2
  { 6, &RISCVDescs.OperandInfo[2734] },  // Inst #2136 = PseudoVFMV_V_FPR32_M1
  { 6, &RISCVDescs.OperandInfo[2710] },  // Inst #2135 = PseudoVFMV_V_FPR16_MF4
  { 6, &RISCVDescs.OperandInfo[2710] },  // Inst #2134 = PseudoVFMV_V_FPR16_MF2
  { 6, &RISCVDescs.OperandInfo[2728] },  // Inst #2133 = PseudoVFMV_V_FPR16_M8
  { 6, &RISCVDescs.OperandInfo[2722] },  // Inst #2132 = PseudoVFMV_V_FPR16_M4
  { 6, &RISCVDescs.OperandInfo[2716] },  // Inst #2131 = PseudoVFMV_V_FPR16_M2
  { 6, &RISCVDescs.OperandInfo[2710] },  // Inst #2130 = PseudoVFMV_V_FPR16_M1
  { 5, &RISCVDescs.OperandInfo[2705] },  // Inst #2129 = PseudoVFMV_S_FPR64_M8
  { 5, &RISCVDescs.OperandInfo[2700] },  // Inst #2128 = PseudoVFMV_S_FPR64_M4
  { 5, &RISCVDescs.OperandInfo[2695] },  // Inst #2127 = PseudoVFMV_S_FPR64_M2
  { 5, &RISCVDescs.OperandInfo[2690] },  // Inst #2126 = PseudoVFMV_S_FPR64_M1
  { 5, &RISCVDescs.OperandInfo[2670] },  // Inst #2125 = PseudoVFMV_S_FPR32_MF2
  { 5, &RISCVDescs.OperandInfo[2685] },  // Inst #2124 = PseudoVFMV_S_FPR32_M8
  { 5, &RISCVDescs.OperandInfo[2680] },  // Inst #2123 = PseudoVFMV_S_FPR32_M4
  { 5, &RISCVDescs.OperandInfo[2675] },  // Inst #2122 = PseudoVFMV_S_FPR32_M2
  { 5, &RISCVDescs.OperandInfo[2670] },  // Inst #2121 = PseudoVFMV_S_FPR32_M1
  { 5, &RISCVDescs.OperandInfo[2650] },  // Inst #2120 = PseudoVFMV_S_FPR16_MF4
  { 5, &RISCVDescs.OperandInfo[2650] },  // Inst #2119 = PseudoVFMV_S_FPR16_MF2
  { 5, &RISCVDescs.OperandInfo[2665] },  // Inst #2118 = PseudoVFMV_S_FPR16_M8
  { 5, &RISCVDescs.OperandInfo[2660] },  // Inst #2117 = PseudoVFMV_S_FPR16_M4
  { 5, &RISCVDescs.OperandInfo[2655] },  // Inst #2116 = PseudoVFMV_S_FPR16_M2
  { 5, &RISCVDescs.OperandInfo[2650] },  // Inst #2115 = PseudoVFMV_S_FPR16_M1
  { 3, &RISCVDescs.OperandInfo[2647] },  // Inst #2114 = PseudoVFMV_FPR64_S_M8
  { 3, &RISCVDescs.OperandInfo[2644] },  // Inst #2113 = PseudoVFMV_FPR64_S_M4
  { 3, &RISCVDescs.OperandInfo[2641] },  // Inst #2112 = PseudoVFMV_FPR64_S_M2
  { 3, &RISCVDescs.OperandInfo[2638] },  // Inst #2111 = PseudoVFMV_FPR64_S_M1
  { 3, &RISCVDescs.OperandInfo[2626] },  // Inst #2110 = PseudoVFMV_FPR32_S_MF2
  { 3, &RISCVDescs.OperandInfo[2635] },  // Inst #2109 = PseudoVFMV_FPR32_S_M8
  { 3, &RISCVDescs.OperandInfo[2632] },  // Inst #2108 = PseudoVFMV_FPR32_S_M4
  { 3, &RISCVDescs.OperandInfo[2629] },  // Inst #2107 = PseudoVFMV_FPR32_S_M2
  { 3, &RISCVDescs.OperandInfo[2626] },  // Inst #2106 = PseudoVFMV_FPR32_S_M1
  { 3, &RISCVDescs.OperandInfo[2614] },  // Inst #2105 = PseudoVFMV_FPR16_S_MF4
  { 3, &RISCVDescs.OperandInfo[2614] },  // Inst #2104 = PseudoVFMV_FPR16_S_MF2
  { 3, &RISCVDescs.OperandInfo[2623] },  // Inst #2103 = PseudoVFMV_FPR16_S_M8
  { 3, &RISCVDescs.OperandInfo[2620] },  // Inst #2102 = PseudoVFMV_FPR16_S_M4
  { 3, &RISCVDescs.OperandInfo[2617] },  // Inst #2101 = PseudoVFMV_FPR16_S_M2
  { 3, &RISCVDescs.OperandInfo[2614] },  // Inst #2100 = PseudoVFMV_FPR16_S_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2099 = PseudoVFMUL_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2098 = PseudoVFMUL_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2097 = PseudoVFMUL_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2096 = PseudoVFMUL_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #2095 = PseudoVFMUL_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #2094 = PseudoVFMUL_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #2093 = PseudoVFMUL_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #2092 = PseudoVFMUL_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #2091 = PseudoVFMUL_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #2090 = PseudoVFMUL_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2089 = PseudoVFMUL_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2088 = PseudoVFMUL_VV_M1
  { 9, &RISCVDescs.OperandInfo[2077] },  // Inst #2087 = PseudoVFMUL_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2069] },  // Inst #2086 = PseudoVFMUL_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2060] },  // Inst #2085 = PseudoVFMUL_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2052] },  // Inst #2084 = PseudoVFMUL_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2043] },  // Inst #2083 = PseudoVFMUL_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2035] },  // Inst #2082 = PseudoVFMUL_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2026] },  // Inst #2081 = PseudoVFMUL_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2018] },  // Inst #2080 = PseudoVFMUL_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #2079 = PseudoVFMUL_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #2078 = PseudoVFMUL_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2009] },  // Inst #2077 = PseudoVFMUL_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2001] },  // Inst #2076 = PseudoVFMUL_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[1992] },  // Inst #2075 = PseudoVFMUL_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1984] },  // Inst #2074 = PseudoVFMUL_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[1975] },  // Inst #2073 = PseudoVFMUL_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1967] },  // Inst #2072 = PseudoVFMUL_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #2071 = PseudoVFMUL_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #2070 = PseudoVFMUL_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2069 = PseudoVFMUL_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2068 = PseudoVFMUL_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2067 = PseudoVFMUL_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2066 = PseudoVFMUL_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[1941] },  // Inst #2065 = PseudoVFMUL_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[1933] },  // Inst #2064 = PseudoVFMUL_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[1924] },  // Inst #2063 = PseudoVFMUL_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1916] },  // Inst #2062 = PseudoVFMUL_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[1907] },  // Inst #2061 = PseudoVFMUL_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1899] },  // Inst #2060 = PseudoVFMUL_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #2059 = PseudoVFMUL_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #2058 = PseudoVFMUL_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2057 = PseudoVFMSUB_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2056 = PseudoVFMSUB_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2055 = PseudoVFMSUB_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2054 = PseudoVFMSUB_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #2053 = PseudoVFMSUB_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #2052 = PseudoVFMSUB_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #2051 = PseudoVFMSUB_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #2050 = PseudoVFMSUB_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #2049 = PseudoVFMSUB_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #2048 = PseudoVFMSUB_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2047 = PseudoVFMSUB_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2046 = PseudoVFMSUB_VV_M1
  { 9, &RISCVDescs.OperandInfo[2341] },  // Inst #2045 = PseudoVFMSUB_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2333] },  // Inst #2044 = PseudoVFMSUB_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2324] },  // Inst #2043 = PseudoVFMSUB_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2316] },  // Inst #2042 = PseudoVFMSUB_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2307] },  // Inst #2041 = PseudoVFMSUB_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2299] },  // Inst #2040 = PseudoVFMSUB_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2290] },  // Inst #2039 = PseudoVFMSUB_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2282] },  // Inst #2038 = PseudoVFMSUB_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2037 = PseudoVFMSUB_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2036 = PseudoVFMSUB_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2273] },  // Inst #2035 = PseudoVFMSUB_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2265] },  // Inst #2034 = PseudoVFMSUB_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[2256] },  // Inst #2033 = PseudoVFMSUB_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2248] },  // Inst #2032 = PseudoVFMSUB_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[2239] },  // Inst #2031 = PseudoVFMSUB_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2231] },  // Inst #2030 = PseudoVFMSUB_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #2029 = PseudoVFMSUB_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #2028 = PseudoVFMSUB_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2027 = PseudoVFMSUB_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2026 = PseudoVFMSUB_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2025 = PseudoVFMSUB_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2024 = PseudoVFMSUB_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[2205] },  // Inst #2023 = PseudoVFMSUB_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2197] },  // Inst #2022 = PseudoVFMSUB_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[2188] },  // Inst #2021 = PseudoVFMSUB_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2180] },  // Inst #2020 = PseudoVFMSUB_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[2171] },  // Inst #2019 = PseudoVFMSUB_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2163] },  // Inst #2018 = PseudoVFMSUB_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #2017 = PseudoVFMSUB_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #2016 = PseudoVFMSUB_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2015 = PseudoVFMSAC_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2014 = PseudoVFMSAC_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2013 = PseudoVFMSAC_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2012 = PseudoVFMSAC_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #2011 = PseudoVFMSAC_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #2010 = PseudoVFMSAC_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #2009 = PseudoVFMSAC_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #2008 = PseudoVFMSAC_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #2007 = PseudoVFMSAC_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #2006 = PseudoVFMSAC_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #2005 = PseudoVFMSAC_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #2004 = PseudoVFMSAC_VV_M1
  { 9, &RISCVDescs.OperandInfo[2341] },  // Inst #2003 = PseudoVFMSAC_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2333] },  // Inst #2002 = PseudoVFMSAC_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2324] },  // Inst #2001 = PseudoVFMSAC_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2316] },  // Inst #2000 = PseudoVFMSAC_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2307] },  // Inst #1999 = PseudoVFMSAC_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2299] },  // Inst #1998 = PseudoVFMSAC_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2290] },  // Inst #1997 = PseudoVFMSAC_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2282] },  // Inst #1996 = PseudoVFMSAC_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #1995 = PseudoVFMSAC_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #1994 = PseudoVFMSAC_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2273] },  // Inst #1993 = PseudoVFMSAC_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2265] },  // Inst #1992 = PseudoVFMSAC_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[2256] },  // Inst #1991 = PseudoVFMSAC_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2248] },  // Inst #1990 = PseudoVFMSAC_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[2239] },  // Inst #1989 = PseudoVFMSAC_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2231] },  // Inst #1988 = PseudoVFMSAC_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #1987 = PseudoVFMSAC_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #1986 = PseudoVFMSAC_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #1985 = PseudoVFMSAC_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #1984 = PseudoVFMSAC_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #1983 = PseudoVFMSAC_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #1982 = PseudoVFMSAC_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[2205] },  // Inst #1981 = PseudoVFMSAC_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2197] },  // Inst #1980 = PseudoVFMSAC_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[2188] },  // Inst #1979 = PseudoVFMSAC_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2180] },  // Inst #1978 = PseudoVFMSAC_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[2171] },  // Inst #1977 = PseudoVFMSAC_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2163] },  // Inst #1976 = PseudoVFMSAC_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #1975 = PseudoVFMSAC_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #1974 = PseudoVFMSAC_VFPR16_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1973 = PseudoVFMIN_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1972 = PseudoVFMIN_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1971 = PseudoVFMIN_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1970 = PseudoVFMIN_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1969 = PseudoVFMIN_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1968 = PseudoVFMIN_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1967 = PseudoVFMIN_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1966 = PseudoVFMIN_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1965 = PseudoVFMIN_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1964 = PseudoVFMIN_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1963 = PseudoVFMIN_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1962 = PseudoVFMIN_VV_M1
  { 8, &RISCVDescs.OperandInfo[2522] },  // Inst #1961 = PseudoVFMIN_VFPR64_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2515] },  // Inst #1960 = PseudoVFMIN_VFPR64_M8
  { 8, &RISCVDescs.OperandInfo[2507] },  // Inst #1959 = PseudoVFMIN_VFPR64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2500] },  // Inst #1958 = PseudoVFMIN_VFPR64_M4
  { 8, &RISCVDescs.OperandInfo[2492] },  // Inst #1957 = PseudoVFMIN_VFPR64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2485] },  // Inst #1956 = PseudoVFMIN_VFPR64_M2
  { 8, &RISCVDescs.OperandInfo[2477] },  // Inst #1955 = PseudoVFMIN_VFPR64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2470] },  // Inst #1954 = PseudoVFMIN_VFPR64_M1
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #1953 = PseudoVFMIN_VFPR32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #1952 = PseudoVFMIN_VFPR32_MF2
  { 8, &RISCVDescs.OperandInfo[2462] },  // Inst #1951 = PseudoVFMIN_VFPR32_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2455] },  // Inst #1950 = PseudoVFMIN_VFPR32_M8
  { 8, &RISCVDescs.OperandInfo[2447] },  // Inst #1949 = PseudoVFMIN_VFPR32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2440] },  // Inst #1948 = PseudoVFMIN_VFPR32_M4
  { 8, &RISCVDescs.OperandInfo[2432] },  // Inst #1947 = PseudoVFMIN_VFPR32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2425] },  // Inst #1946 = PseudoVFMIN_VFPR32_M2
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #1945 = PseudoVFMIN_VFPR32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #1944 = PseudoVFMIN_VFPR32_M1
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #1943 = PseudoVFMIN_VFPR16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #1942 = PseudoVFMIN_VFPR16_MF4
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #1941 = PseudoVFMIN_VFPR16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #1940 = PseudoVFMIN_VFPR16_MF2
  { 8, &RISCVDescs.OperandInfo[2402] },  // Inst #1939 = PseudoVFMIN_VFPR16_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2395] },  // Inst #1938 = PseudoVFMIN_VFPR16_M8
  { 8, &RISCVDescs.OperandInfo[2387] },  // Inst #1937 = PseudoVFMIN_VFPR16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2380] },  // Inst #1936 = PseudoVFMIN_VFPR16_M4
  { 8, &RISCVDescs.OperandInfo[2372] },  // Inst #1935 = PseudoVFMIN_VFPR16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2365] },  // Inst #1934 = PseudoVFMIN_VFPR16_M2
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #1933 = PseudoVFMIN_VFPR16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #1932 = PseudoVFMIN_VFPR16_M1
  { 7, &RISCVDescs.OperandInfo[2607] },  // Inst #1931 = PseudoVFMERGE_VFPR64M_M8
  { 7, &RISCVDescs.OperandInfo[2600] },  // Inst #1930 = PseudoVFMERGE_VFPR64M_M4
  { 7, &RISCVDescs.OperandInfo[2593] },  // Inst #1929 = PseudoVFMERGE_VFPR64M_M2
  { 7, &RISCVDescs.OperandInfo[2586] },  // Inst #1928 = PseudoVFMERGE_VFPR64M_M1
  { 7, &RISCVDescs.OperandInfo[2558] },  // Inst #1927 = PseudoVFMERGE_VFPR32M_MF2
  { 7, &RISCVDescs.OperandInfo[2579] },  // Inst #1926 = PseudoVFMERGE_VFPR32M_M8
  { 7, &RISCVDescs.OperandInfo[2572] },  // Inst #1925 = PseudoVFMERGE_VFPR32M_M4
  { 7, &RISCVDescs.OperandInfo[2565] },  // Inst #1924 = PseudoVFMERGE_VFPR32M_M2
  { 7, &RISCVDescs.OperandInfo[2558] },  // Inst #1923 = PseudoVFMERGE_VFPR32M_M1
  { 7, &RISCVDescs.OperandInfo[2530] },  // Inst #1922 = PseudoVFMERGE_VFPR16M_MF4
  { 7, &RISCVDescs.OperandInfo[2530] },  // Inst #1921 = PseudoVFMERGE_VFPR16M_MF2
  { 7, &RISCVDescs.OperandInfo[2551] },  // Inst #1920 = PseudoVFMERGE_VFPR16M_M8
  { 7, &RISCVDescs.OperandInfo[2544] },  // Inst #1919 = PseudoVFMERGE_VFPR16M_M4
  { 7, &RISCVDescs.OperandInfo[2537] },  // Inst #1918 = PseudoVFMERGE_VFPR16M_M2
  { 7, &RISCVDescs.OperandInfo[2530] },  // Inst #1917 = PseudoVFMERGE_VFPR16M_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1916 = PseudoVFMAX_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1915 = PseudoVFMAX_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1914 = PseudoVFMAX_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1913 = PseudoVFMAX_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1912 = PseudoVFMAX_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1911 = PseudoVFMAX_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1910 = PseudoVFMAX_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1909 = PseudoVFMAX_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1908 = PseudoVFMAX_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1907 = PseudoVFMAX_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1906 = PseudoVFMAX_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1905 = PseudoVFMAX_VV_M1
  { 8, &RISCVDescs.OperandInfo[2522] },  // Inst #1904 = PseudoVFMAX_VFPR64_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2515] },  // Inst #1903 = PseudoVFMAX_VFPR64_M8
  { 8, &RISCVDescs.OperandInfo[2507] },  // Inst #1902 = PseudoVFMAX_VFPR64_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2500] },  // Inst #1901 = PseudoVFMAX_VFPR64_M4
  { 8, &RISCVDescs.OperandInfo[2492] },  // Inst #1900 = PseudoVFMAX_VFPR64_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2485] },  // Inst #1899 = PseudoVFMAX_VFPR64_M2
  { 8, &RISCVDescs.OperandInfo[2477] },  // Inst #1898 = PseudoVFMAX_VFPR64_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2470] },  // Inst #1897 = PseudoVFMAX_VFPR64_M1
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #1896 = PseudoVFMAX_VFPR32_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #1895 = PseudoVFMAX_VFPR32_MF2
  { 8, &RISCVDescs.OperandInfo[2462] },  // Inst #1894 = PseudoVFMAX_VFPR32_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2455] },  // Inst #1893 = PseudoVFMAX_VFPR32_M8
  { 8, &RISCVDescs.OperandInfo[2447] },  // Inst #1892 = PseudoVFMAX_VFPR32_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2440] },  // Inst #1891 = PseudoVFMAX_VFPR32_M4
  { 8, &RISCVDescs.OperandInfo[2432] },  // Inst #1890 = PseudoVFMAX_VFPR32_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2425] },  // Inst #1889 = PseudoVFMAX_VFPR32_M2
  { 8, &RISCVDescs.OperandInfo[2417] },  // Inst #1888 = PseudoVFMAX_VFPR32_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2410] },  // Inst #1887 = PseudoVFMAX_VFPR32_M1
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #1886 = PseudoVFMAX_VFPR16_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #1885 = PseudoVFMAX_VFPR16_MF4
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #1884 = PseudoVFMAX_VFPR16_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #1883 = PseudoVFMAX_VFPR16_MF2
  { 8, &RISCVDescs.OperandInfo[2402] },  // Inst #1882 = PseudoVFMAX_VFPR16_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2395] },  // Inst #1881 = PseudoVFMAX_VFPR16_M8
  { 8, &RISCVDescs.OperandInfo[2387] },  // Inst #1880 = PseudoVFMAX_VFPR16_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2380] },  // Inst #1879 = PseudoVFMAX_VFPR16_M4
  { 8, &RISCVDescs.OperandInfo[2372] },  // Inst #1878 = PseudoVFMAX_VFPR16_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2365] },  // Inst #1877 = PseudoVFMAX_VFPR16_M2
  { 8, &RISCVDescs.OperandInfo[2357] },  // Inst #1876 = PseudoVFMAX_VFPR16_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2350] },  // Inst #1875 = PseudoVFMAX_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1874 = PseudoVFMADD_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1873 = PseudoVFMADD_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1872 = PseudoVFMADD_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1871 = PseudoVFMADD_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #1870 = PseudoVFMADD_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #1869 = PseudoVFMADD_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #1868 = PseudoVFMADD_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #1867 = PseudoVFMADD_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #1866 = PseudoVFMADD_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #1865 = PseudoVFMADD_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1864 = PseudoVFMADD_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1863 = PseudoVFMADD_VV_M1
  { 9, &RISCVDescs.OperandInfo[2341] },  // Inst #1862 = PseudoVFMADD_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2333] },  // Inst #1861 = PseudoVFMADD_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2324] },  // Inst #1860 = PseudoVFMADD_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2316] },  // Inst #1859 = PseudoVFMADD_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2307] },  // Inst #1858 = PseudoVFMADD_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2299] },  // Inst #1857 = PseudoVFMADD_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2290] },  // Inst #1856 = PseudoVFMADD_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2282] },  // Inst #1855 = PseudoVFMADD_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #1854 = PseudoVFMADD_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #1853 = PseudoVFMADD_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2273] },  // Inst #1852 = PseudoVFMADD_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2265] },  // Inst #1851 = PseudoVFMADD_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[2256] },  // Inst #1850 = PseudoVFMADD_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2248] },  // Inst #1849 = PseudoVFMADD_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[2239] },  // Inst #1848 = PseudoVFMADD_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2231] },  // Inst #1847 = PseudoVFMADD_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #1846 = PseudoVFMADD_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #1845 = PseudoVFMADD_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #1844 = PseudoVFMADD_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #1843 = PseudoVFMADD_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #1842 = PseudoVFMADD_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #1841 = PseudoVFMADD_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[2205] },  // Inst #1840 = PseudoVFMADD_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2197] },  // Inst #1839 = PseudoVFMADD_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[2188] },  // Inst #1838 = PseudoVFMADD_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2180] },  // Inst #1837 = PseudoVFMADD_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[2171] },  // Inst #1836 = PseudoVFMADD_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2163] },  // Inst #1835 = PseudoVFMADD_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #1834 = PseudoVFMADD_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #1833 = PseudoVFMADD_VFPR16_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1832 = PseudoVFMACC_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1831 = PseudoVFMACC_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1830 = PseudoVFMACC_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1829 = PseudoVFMACC_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #1828 = PseudoVFMACC_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #1827 = PseudoVFMACC_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #1826 = PseudoVFMACC_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #1825 = PseudoVFMACC_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #1824 = PseudoVFMACC_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #1823 = PseudoVFMACC_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1822 = PseudoVFMACC_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1821 = PseudoVFMACC_VV_M1
  { 9, &RISCVDescs.OperandInfo[2341] },  // Inst #1820 = PseudoVFMACC_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2333] },  // Inst #1819 = PseudoVFMACC_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2324] },  // Inst #1818 = PseudoVFMACC_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2316] },  // Inst #1817 = PseudoVFMACC_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2307] },  // Inst #1816 = PseudoVFMACC_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2299] },  // Inst #1815 = PseudoVFMACC_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2290] },  // Inst #1814 = PseudoVFMACC_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2282] },  // Inst #1813 = PseudoVFMACC_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #1812 = PseudoVFMACC_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #1811 = PseudoVFMACC_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2273] },  // Inst #1810 = PseudoVFMACC_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2265] },  // Inst #1809 = PseudoVFMACC_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[2256] },  // Inst #1808 = PseudoVFMACC_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2248] },  // Inst #1807 = PseudoVFMACC_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[2239] },  // Inst #1806 = PseudoVFMACC_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2231] },  // Inst #1805 = PseudoVFMACC_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[2222] },  // Inst #1804 = PseudoVFMACC_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2214] },  // Inst #1803 = PseudoVFMACC_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #1802 = PseudoVFMACC_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #1801 = PseudoVFMACC_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #1800 = PseudoVFMACC_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #1799 = PseudoVFMACC_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[2205] },  // Inst #1798 = PseudoVFMACC_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2197] },  // Inst #1797 = PseudoVFMACC_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[2188] },  // Inst #1796 = PseudoVFMACC_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2180] },  // Inst #1795 = PseudoVFMACC_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[2171] },  // Inst #1794 = PseudoVFMACC_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2163] },  // Inst #1793 = PseudoVFMACC_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[2154] },  // Inst #1792 = PseudoVFMACC_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2146] },  // Inst #1791 = PseudoVFMACC_VFPR16_M1
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #1790 = PseudoVFIRST_M_B8_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #1789 = PseudoVFIRST_M_B8
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #1788 = PseudoVFIRST_M_B64_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #1787 = PseudoVFIRST_M_B64
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #1786 = PseudoVFIRST_M_B4_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #1785 = PseudoVFIRST_M_B4
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #1784 = PseudoVFIRST_M_B32_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #1783 = PseudoVFIRST_M_B32
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #1782 = PseudoVFIRST_M_B2_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #1781 = PseudoVFIRST_M_B2
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #1780 = PseudoVFIRST_M_B1_MASK
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #1779 = PseudoVFIRST_M_B16_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #1778 = PseudoVFIRST_M_B16
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #1777 = PseudoVFIRST_M_B1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1776 = PseudoVFDIV_VV_MF4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1775 = PseudoVFDIV_VV_MF4_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1774 = PseudoVFDIV_VV_MF2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1773 = PseudoVFDIV_VV_MF2_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1772 = PseudoVFDIV_VV_MF2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1771 = PseudoVFDIV_VV_MF2_E16
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #1770 = PseudoVFDIV_VV_M8_E64_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #1769 = PseudoVFDIV_VV_M8_E64
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #1768 = PseudoVFDIV_VV_M8_E32_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #1767 = PseudoVFDIV_VV_M8_E32
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #1766 = PseudoVFDIV_VV_M8_E16_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #1765 = PseudoVFDIV_VV_M8_E16
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #1764 = PseudoVFDIV_VV_M4_E64_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #1763 = PseudoVFDIV_VV_M4_E64
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #1762 = PseudoVFDIV_VV_M4_E32_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #1761 = PseudoVFDIV_VV_M4_E32
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #1760 = PseudoVFDIV_VV_M4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #1759 = PseudoVFDIV_VV_M4_E16
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #1758 = PseudoVFDIV_VV_M2_E64_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #1757 = PseudoVFDIV_VV_M2_E64
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #1756 = PseudoVFDIV_VV_M2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #1755 = PseudoVFDIV_VV_M2_E32
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #1754 = PseudoVFDIV_VV_M2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #1753 = PseudoVFDIV_VV_M2_E16
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1752 = PseudoVFDIV_VV_M1_E64_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1751 = PseudoVFDIV_VV_M1_E64
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1750 = PseudoVFDIV_VV_M1_E32_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1749 = PseudoVFDIV_VV_M1_E32
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1748 = PseudoVFDIV_VV_M1_E16_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1747 = PseudoVFDIV_VV_M1_E16
  { 9, &RISCVDescs.OperandInfo[2077] },  // Inst #1746 = PseudoVFDIV_VFPR64_M8_E64_MASK
  { 8, &RISCVDescs.OperandInfo[2069] },  // Inst #1745 = PseudoVFDIV_VFPR64_M8_E64
  { 9, &RISCVDescs.OperandInfo[2060] },  // Inst #1744 = PseudoVFDIV_VFPR64_M4_E64_MASK
  { 8, &RISCVDescs.OperandInfo[2052] },  // Inst #1743 = PseudoVFDIV_VFPR64_M4_E64
  { 9, &RISCVDescs.OperandInfo[2043] },  // Inst #1742 = PseudoVFDIV_VFPR64_M2_E64_MASK
  { 8, &RISCVDescs.OperandInfo[2035] },  // Inst #1741 = PseudoVFDIV_VFPR64_M2_E64
  { 9, &RISCVDescs.OperandInfo[2026] },  // Inst #1740 = PseudoVFDIV_VFPR64_M1_E64_MASK
  { 8, &RISCVDescs.OperandInfo[2018] },  // Inst #1739 = PseudoVFDIV_VFPR64_M1_E64
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #1738 = PseudoVFDIV_VFPR32_MF2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #1737 = PseudoVFDIV_VFPR32_MF2_E32
  { 9, &RISCVDescs.OperandInfo[2009] },  // Inst #1736 = PseudoVFDIV_VFPR32_M8_E32_MASK
  { 8, &RISCVDescs.OperandInfo[2001] },  // Inst #1735 = PseudoVFDIV_VFPR32_M8_E32
  { 9, &RISCVDescs.OperandInfo[1992] },  // Inst #1734 = PseudoVFDIV_VFPR32_M4_E32_MASK
  { 8, &RISCVDescs.OperandInfo[1984] },  // Inst #1733 = PseudoVFDIV_VFPR32_M4_E32
  { 9, &RISCVDescs.OperandInfo[1975] },  // Inst #1732 = PseudoVFDIV_VFPR32_M2_E32_MASK
  { 8, &RISCVDescs.OperandInfo[1967] },  // Inst #1731 = PseudoVFDIV_VFPR32_M2_E32
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #1730 = PseudoVFDIV_VFPR32_M1_E32_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #1729 = PseudoVFDIV_VFPR32_M1_E32
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #1728 = PseudoVFDIV_VFPR16_MF4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #1727 = PseudoVFDIV_VFPR16_MF4_E16
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #1726 = PseudoVFDIV_VFPR16_MF2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #1725 = PseudoVFDIV_VFPR16_MF2_E16
  { 9, &RISCVDescs.OperandInfo[1941] },  // Inst #1724 = PseudoVFDIV_VFPR16_M8_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1933] },  // Inst #1723 = PseudoVFDIV_VFPR16_M8_E16
  { 9, &RISCVDescs.OperandInfo[1924] },  // Inst #1722 = PseudoVFDIV_VFPR16_M4_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1916] },  // Inst #1721 = PseudoVFDIV_VFPR16_M4_E16
  { 9, &RISCVDescs.OperandInfo[1907] },  // Inst #1720 = PseudoVFDIV_VFPR16_M2_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1899] },  // Inst #1719 = PseudoVFDIV_VFPR16_M2_E16
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #1718 = PseudoVFDIV_VFPR16_M1_E16_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #1717 = PseudoVFDIV_VFPR16_M1_E16
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1716 = PseudoVFCVT_X_F_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1715 = PseudoVFCVT_X_F_V_MF4
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1714 = PseudoVFCVT_X_F_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1713 = PseudoVFCVT_X_F_V_MF2
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #1712 = PseudoVFCVT_X_F_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #1711 = PseudoVFCVT_X_F_V_M8
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #1710 = PseudoVFCVT_X_F_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #1709 = PseudoVFCVT_X_F_V_M4
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #1708 = PseudoVFCVT_X_F_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #1707 = PseudoVFCVT_X_F_V_M2
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1706 = PseudoVFCVT_X_F_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1705 = PseudoVFCVT_X_F_V_M1
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1704 = PseudoVFCVT_XU_F_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1703 = PseudoVFCVT_XU_F_V_MF4
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1702 = PseudoVFCVT_XU_F_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1701 = PseudoVFCVT_XU_F_V_MF2
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #1700 = PseudoVFCVT_XU_F_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #1699 = PseudoVFCVT_XU_F_V_M8
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #1698 = PseudoVFCVT_XU_F_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #1697 = PseudoVFCVT_XU_F_V_M4
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #1696 = PseudoVFCVT_XU_F_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #1695 = PseudoVFCVT_XU_F_V_M2
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1694 = PseudoVFCVT_XU_F_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1693 = PseudoVFCVT_XU_F_V_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1692 = PseudoVFCVT_RTZ_X_F_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1691 = PseudoVFCVT_RTZ_X_F_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1690 = PseudoVFCVT_RTZ_X_F_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1689 = PseudoVFCVT_RTZ_X_F_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #1688 = PseudoVFCVT_RTZ_X_F_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #1687 = PseudoVFCVT_RTZ_X_F_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #1686 = PseudoVFCVT_RTZ_X_F_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #1685 = PseudoVFCVT_RTZ_X_F_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #1684 = PseudoVFCVT_RTZ_X_F_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #1683 = PseudoVFCVT_RTZ_X_F_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1682 = PseudoVFCVT_RTZ_X_F_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1681 = PseudoVFCVT_RTZ_X_F_V_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1680 = PseudoVFCVT_RTZ_XU_F_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1679 = PseudoVFCVT_RTZ_XU_F_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1678 = PseudoVFCVT_RTZ_XU_F_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1677 = PseudoVFCVT_RTZ_XU_F_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #1676 = PseudoVFCVT_RTZ_XU_F_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #1675 = PseudoVFCVT_RTZ_XU_F_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #1674 = PseudoVFCVT_RTZ_XU_F_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #1673 = PseudoVFCVT_RTZ_XU_F_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #1672 = PseudoVFCVT_RTZ_XU_F_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #1671 = PseudoVFCVT_RTZ_XU_F_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1670 = PseudoVFCVT_RTZ_XU_F_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1669 = PseudoVFCVT_RTZ_XU_F_V_M1
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1668 = PseudoVFCVT_RM_X_F_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1667 = PseudoVFCVT_RM_X_F_V_MF4
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1666 = PseudoVFCVT_RM_X_F_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1665 = PseudoVFCVT_RM_X_F_V_MF2
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #1664 = PseudoVFCVT_RM_X_F_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #1663 = PseudoVFCVT_RM_X_F_V_M8
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #1662 = PseudoVFCVT_RM_X_F_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #1661 = PseudoVFCVT_RM_X_F_V_M4
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #1660 = PseudoVFCVT_RM_X_F_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #1659 = PseudoVFCVT_RM_X_F_V_M2
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1658 = PseudoVFCVT_RM_X_F_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1657 = PseudoVFCVT_RM_X_F_V_M1
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1656 = PseudoVFCVT_RM_XU_F_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1655 = PseudoVFCVT_RM_XU_F_V_MF4
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1654 = PseudoVFCVT_RM_XU_F_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1653 = PseudoVFCVT_RM_XU_F_V_MF2
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #1652 = PseudoVFCVT_RM_XU_F_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #1651 = PseudoVFCVT_RM_XU_F_V_M8
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #1650 = PseudoVFCVT_RM_XU_F_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #1649 = PseudoVFCVT_RM_XU_F_V_M4
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #1648 = PseudoVFCVT_RM_XU_F_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #1647 = PseudoVFCVT_RM_XU_F_V_M2
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1646 = PseudoVFCVT_RM_XU_F_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1645 = PseudoVFCVT_RM_XU_F_V_M1
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1644 = PseudoVFCVT_RM_F_X_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1643 = PseudoVFCVT_RM_F_X_V_MF4
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1642 = PseudoVFCVT_RM_F_X_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1641 = PseudoVFCVT_RM_F_X_V_MF2
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #1640 = PseudoVFCVT_RM_F_X_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #1639 = PseudoVFCVT_RM_F_X_V_M8
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #1638 = PseudoVFCVT_RM_F_X_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #1637 = PseudoVFCVT_RM_F_X_V_M4
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #1636 = PseudoVFCVT_RM_F_X_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #1635 = PseudoVFCVT_RM_F_X_V_M2
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1634 = PseudoVFCVT_RM_F_X_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1633 = PseudoVFCVT_RM_F_X_V_M1
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1632 = PseudoVFCVT_RM_F_XU_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1631 = PseudoVFCVT_RM_F_XU_V_MF4
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1630 = PseudoVFCVT_RM_F_XU_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1629 = PseudoVFCVT_RM_F_XU_V_MF2
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #1628 = PseudoVFCVT_RM_F_XU_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #1627 = PseudoVFCVT_RM_F_XU_V_M8
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #1626 = PseudoVFCVT_RM_F_XU_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #1625 = PseudoVFCVT_RM_F_XU_V_M4
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #1624 = PseudoVFCVT_RM_F_XU_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #1623 = PseudoVFCVT_RM_F_XU_V_M2
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1622 = PseudoVFCVT_RM_F_XU_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1621 = PseudoVFCVT_RM_F_XU_V_M1
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1620 = PseudoVFCVT_F_X_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1619 = PseudoVFCVT_F_X_V_MF4
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1618 = PseudoVFCVT_F_X_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1617 = PseudoVFCVT_F_X_V_MF2
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #1616 = PseudoVFCVT_F_X_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #1615 = PseudoVFCVT_F_X_V_M8
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #1614 = PseudoVFCVT_F_X_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #1613 = PseudoVFCVT_F_X_V_M4
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #1612 = PseudoVFCVT_F_X_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #1611 = PseudoVFCVT_F_X_V_M2
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1610 = PseudoVFCVT_F_X_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1609 = PseudoVFCVT_F_X_V_M1
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1608 = PseudoVFCVT_F_XU_V_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1607 = PseudoVFCVT_F_XU_V_MF4
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1606 = PseudoVFCVT_F_XU_V_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1605 = PseudoVFCVT_F_XU_V_MF2
  { 8, &RISCVDescs.OperandInfo[2138] },  // Inst #1604 = PseudoVFCVT_F_XU_V_M8_MASK
  { 7, &RISCVDescs.OperandInfo[2131] },  // Inst #1603 = PseudoVFCVT_F_XU_V_M8
  { 8, &RISCVDescs.OperandInfo[2123] },  // Inst #1602 = PseudoVFCVT_F_XU_V_M4_MASK
  { 7, &RISCVDescs.OperandInfo[2116] },  // Inst #1601 = PseudoVFCVT_F_XU_V_M4
  { 8, &RISCVDescs.OperandInfo[2108] },  // Inst #1600 = PseudoVFCVT_F_XU_V_M2_MASK
  { 7, &RISCVDescs.OperandInfo[2101] },  // Inst #1599 = PseudoVFCVT_F_XU_V_M2
  { 8, &RISCVDescs.OperandInfo[2093] },  // Inst #1598 = PseudoVFCVT_F_XU_V_M1_MASK
  { 7, &RISCVDescs.OperandInfo[2086] },  // Inst #1597 = PseudoVFCVT_F_XU_V_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1596 = PseudoVFCLASS_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1595 = PseudoVFCLASS_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1594 = PseudoVFCLASS_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1593 = PseudoVFCLASS_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #1592 = PseudoVFCLASS_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #1591 = PseudoVFCLASS_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #1590 = PseudoVFCLASS_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #1589 = PseudoVFCLASS_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #1588 = PseudoVFCLASS_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #1587 = PseudoVFCLASS_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1586 = PseudoVFCLASS_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1585 = PseudoVFCLASS_V_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1584 = PseudoVFADD_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1583 = PseudoVFADD_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1582 = PseudoVFADD_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1581 = PseudoVFADD_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #1580 = PseudoVFADD_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #1579 = PseudoVFADD_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #1578 = PseudoVFADD_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #1577 = PseudoVFADD_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #1576 = PseudoVFADD_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #1575 = PseudoVFADD_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #1574 = PseudoVFADD_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #1573 = PseudoVFADD_VV_M1
  { 9, &RISCVDescs.OperandInfo[2077] },  // Inst #1572 = PseudoVFADD_VFPR64_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2069] },  // Inst #1571 = PseudoVFADD_VFPR64_M8
  { 9, &RISCVDescs.OperandInfo[2060] },  // Inst #1570 = PseudoVFADD_VFPR64_M4_MASK
  { 8, &RISCVDescs.OperandInfo[2052] },  // Inst #1569 = PseudoVFADD_VFPR64_M4
  { 9, &RISCVDescs.OperandInfo[2043] },  // Inst #1568 = PseudoVFADD_VFPR64_M2_MASK
  { 8, &RISCVDescs.OperandInfo[2035] },  // Inst #1567 = PseudoVFADD_VFPR64_M2
  { 9, &RISCVDescs.OperandInfo[2026] },  // Inst #1566 = PseudoVFADD_VFPR64_M1_MASK
  { 8, &RISCVDescs.OperandInfo[2018] },  // Inst #1565 = PseudoVFADD_VFPR64_M1
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #1564 = PseudoVFADD_VFPR32_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #1563 = PseudoVFADD_VFPR32_MF2
  { 9, &RISCVDescs.OperandInfo[2009] },  // Inst #1562 = PseudoVFADD_VFPR32_M8_MASK
  { 8, &RISCVDescs.OperandInfo[2001] },  // Inst #1561 = PseudoVFADD_VFPR32_M8
  { 9, &RISCVDescs.OperandInfo[1992] },  // Inst #1560 = PseudoVFADD_VFPR32_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1984] },  // Inst #1559 = PseudoVFADD_VFPR32_M4
  { 9, &RISCVDescs.OperandInfo[1975] },  // Inst #1558 = PseudoVFADD_VFPR32_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1967] },  // Inst #1557 = PseudoVFADD_VFPR32_M2
  { 9, &RISCVDescs.OperandInfo[1958] },  // Inst #1556 = PseudoVFADD_VFPR32_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1950] },  // Inst #1555 = PseudoVFADD_VFPR32_M1
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #1554 = PseudoVFADD_VFPR16_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #1553 = PseudoVFADD_VFPR16_MF4
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #1552 = PseudoVFADD_VFPR16_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #1551 = PseudoVFADD_VFPR16_MF2
  { 9, &RISCVDescs.OperandInfo[1941] },  // Inst #1550 = PseudoVFADD_VFPR16_M8_MASK
  { 8, &RISCVDescs.OperandInfo[1933] },  // Inst #1549 = PseudoVFADD_VFPR16_M8
  { 9, &RISCVDescs.OperandInfo[1924] },  // Inst #1548 = PseudoVFADD_VFPR16_M4_MASK
  { 8, &RISCVDescs.OperandInfo[1916] },  // Inst #1547 = PseudoVFADD_VFPR16_M4
  { 9, &RISCVDescs.OperandInfo[1907] },  // Inst #1546 = PseudoVFADD_VFPR16_M2_MASK
  { 8, &RISCVDescs.OperandInfo[1899] },  // Inst #1545 = PseudoVFADD_VFPR16_M2
  { 9, &RISCVDescs.OperandInfo[1890] },  // Inst #1544 = PseudoVFADD_VFPR16_M1_MASK
  { 8, &RISCVDescs.OperandInfo[1882] },  // Inst #1543 = PseudoVFADD_VFPR16_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1542 = PseudoVDIV_VX_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1541 = PseudoVDIV_VX_MF8_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1540 = PseudoVDIV_VX_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1539 = PseudoVDIV_VX_MF4_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1538 = PseudoVDIV_VX_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1537 = PseudoVDIV_VX_MF4_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1536 = PseudoVDIV_VX_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1535 = PseudoVDIV_VX_MF2_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1534 = PseudoVDIV_VX_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1533 = PseudoVDIV_VX_MF2_E32
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1532 = PseudoVDIV_VX_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1531 = PseudoVDIV_VX_MF2_E16
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #1530 = PseudoVDIV_VX_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #1529 = PseudoVDIV_VX_M8_E8
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #1528 = PseudoVDIV_VX_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #1527 = PseudoVDIV_VX_M8_E64
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #1526 = PseudoVDIV_VX_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #1525 = PseudoVDIV_VX_M8_E32
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #1524 = PseudoVDIV_VX_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #1523 = PseudoVDIV_VX_M8_E16
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #1522 = PseudoVDIV_VX_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #1521 = PseudoVDIV_VX_M4_E8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #1520 = PseudoVDIV_VX_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #1519 = PseudoVDIV_VX_M4_E64
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #1518 = PseudoVDIV_VX_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #1517 = PseudoVDIV_VX_M4_E32
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #1516 = PseudoVDIV_VX_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #1515 = PseudoVDIV_VX_M4_E16
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #1514 = PseudoVDIV_VX_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #1513 = PseudoVDIV_VX_M2_E8
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #1512 = PseudoVDIV_VX_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #1511 = PseudoVDIV_VX_M2_E64
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #1510 = PseudoVDIV_VX_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #1509 = PseudoVDIV_VX_M2_E32
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #1508 = PseudoVDIV_VX_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #1507 = PseudoVDIV_VX_M2_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1506 = PseudoVDIV_VX_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1505 = PseudoVDIV_VX_M1_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1504 = PseudoVDIV_VX_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1503 = PseudoVDIV_VX_M1_E64
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1502 = PseudoVDIV_VX_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1501 = PseudoVDIV_VX_M1_E32
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1500 = PseudoVDIV_VX_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1499 = PseudoVDIV_VX_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1498 = PseudoVDIV_VV_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1497 = PseudoVDIV_VV_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1496 = PseudoVDIV_VV_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1495 = PseudoVDIV_VV_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1494 = PseudoVDIV_VV_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1493 = PseudoVDIV_VV_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1492 = PseudoVDIV_VV_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1491 = PseudoVDIV_VV_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1490 = PseudoVDIV_VV_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1489 = PseudoVDIV_VV_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1488 = PseudoVDIV_VV_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1487 = PseudoVDIV_VV_MF2_E16
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1486 = PseudoVDIV_VV_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1485 = PseudoVDIV_VV_M8_E8
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1484 = PseudoVDIV_VV_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1483 = PseudoVDIV_VV_M8_E64
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1482 = PseudoVDIV_VV_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1481 = PseudoVDIV_VV_M8_E32
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1480 = PseudoVDIV_VV_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1479 = PseudoVDIV_VV_M8_E16
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1478 = PseudoVDIV_VV_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1477 = PseudoVDIV_VV_M4_E8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1476 = PseudoVDIV_VV_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1475 = PseudoVDIV_VV_M4_E64
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1474 = PseudoVDIV_VV_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1473 = PseudoVDIV_VV_M4_E32
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1472 = PseudoVDIV_VV_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1471 = PseudoVDIV_VV_M4_E16
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1470 = PseudoVDIV_VV_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1469 = PseudoVDIV_VV_M2_E8
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1468 = PseudoVDIV_VV_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1467 = PseudoVDIV_VV_M2_E64
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1466 = PseudoVDIV_VV_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1465 = PseudoVDIV_VV_M2_E32
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1464 = PseudoVDIV_VV_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1463 = PseudoVDIV_VV_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1462 = PseudoVDIV_VV_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1461 = PseudoVDIV_VV_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1460 = PseudoVDIV_VV_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1459 = PseudoVDIV_VV_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1458 = PseudoVDIV_VV_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1457 = PseudoVDIV_VV_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1456 = PseudoVDIV_VV_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1455 = PseudoVDIV_VV_M1_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1454 = PseudoVDIVU_VX_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1453 = PseudoVDIVU_VX_MF8_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1452 = PseudoVDIVU_VX_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1451 = PseudoVDIVU_VX_MF4_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1450 = PseudoVDIVU_VX_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1449 = PseudoVDIVU_VX_MF4_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1448 = PseudoVDIVU_VX_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1447 = PseudoVDIVU_VX_MF2_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1446 = PseudoVDIVU_VX_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1445 = PseudoVDIVU_VX_MF2_E32
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1444 = PseudoVDIVU_VX_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1443 = PseudoVDIVU_VX_MF2_E16
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #1442 = PseudoVDIVU_VX_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #1441 = PseudoVDIVU_VX_M8_E8
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #1440 = PseudoVDIVU_VX_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #1439 = PseudoVDIVU_VX_M8_E64
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #1438 = PseudoVDIVU_VX_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #1437 = PseudoVDIVU_VX_M8_E32
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #1436 = PseudoVDIVU_VX_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #1435 = PseudoVDIVU_VX_M8_E16
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #1434 = PseudoVDIVU_VX_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #1433 = PseudoVDIVU_VX_M4_E8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #1432 = PseudoVDIVU_VX_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #1431 = PseudoVDIVU_VX_M4_E64
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #1430 = PseudoVDIVU_VX_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #1429 = PseudoVDIVU_VX_M4_E32
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #1428 = PseudoVDIVU_VX_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #1427 = PseudoVDIVU_VX_M4_E16
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #1426 = PseudoVDIVU_VX_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #1425 = PseudoVDIVU_VX_M2_E8
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #1424 = PseudoVDIVU_VX_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #1423 = PseudoVDIVU_VX_M2_E64
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #1422 = PseudoVDIVU_VX_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #1421 = PseudoVDIVU_VX_M2_E32
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #1420 = PseudoVDIVU_VX_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #1419 = PseudoVDIVU_VX_M2_E16
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1418 = PseudoVDIVU_VX_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1417 = PseudoVDIVU_VX_M1_E8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1416 = PseudoVDIVU_VX_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1415 = PseudoVDIVU_VX_M1_E64
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1414 = PseudoVDIVU_VX_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1413 = PseudoVDIVU_VX_M1_E32
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #1412 = PseudoVDIVU_VX_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #1411 = PseudoVDIVU_VX_M1_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1410 = PseudoVDIVU_VV_MF8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1409 = PseudoVDIVU_VV_MF8_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1408 = PseudoVDIVU_VV_MF4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1407 = PseudoVDIVU_VV_MF4_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1406 = PseudoVDIVU_VV_MF4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1405 = PseudoVDIVU_VV_MF4_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1404 = PseudoVDIVU_VV_MF2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1403 = PseudoVDIVU_VV_MF2_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1402 = PseudoVDIVU_VV_MF2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1401 = PseudoVDIVU_VV_MF2_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1400 = PseudoVDIVU_VV_MF2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1399 = PseudoVDIVU_VV_MF2_E16
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1398 = PseudoVDIVU_VV_M8_E8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1397 = PseudoVDIVU_VV_M8_E8
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1396 = PseudoVDIVU_VV_M8_E64_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1395 = PseudoVDIVU_VV_M8_E64
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1394 = PseudoVDIVU_VV_M8_E32_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1393 = PseudoVDIVU_VV_M8_E32
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #1392 = PseudoVDIVU_VV_M8_E16_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #1391 = PseudoVDIVU_VV_M8_E16
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1390 = PseudoVDIVU_VV_M4_E8_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1389 = PseudoVDIVU_VV_M4_E8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1388 = PseudoVDIVU_VV_M4_E64_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1387 = PseudoVDIVU_VV_M4_E64
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1386 = PseudoVDIVU_VV_M4_E32_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1385 = PseudoVDIVU_VV_M4_E32
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #1384 = PseudoVDIVU_VV_M4_E16_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #1383 = PseudoVDIVU_VV_M4_E16
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1382 = PseudoVDIVU_VV_M2_E8_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1381 = PseudoVDIVU_VV_M2_E8
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1380 = PseudoVDIVU_VV_M2_E64_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1379 = PseudoVDIVU_VV_M2_E64
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1378 = PseudoVDIVU_VV_M2_E32_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1377 = PseudoVDIVU_VV_M2_E32
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #1376 = PseudoVDIVU_VV_M2_E16_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #1375 = PseudoVDIVU_VV_M2_E16
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1374 = PseudoVDIVU_VV_M1_E8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1373 = PseudoVDIVU_VV_M1_E8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1372 = PseudoVDIVU_VV_M1_E64_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1371 = PseudoVDIVU_VV_M1_E64
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1370 = PseudoVDIVU_VV_M1_E32_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1369 = PseudoVDIVU_VV_M1_E32
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #1368 = PseudoVDIVU_VV_M1_E16_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #1367 = PseudoVDIVU_VV_M1_E16
  { 6, &RISCVDescs.OperandInfo[1876] },  // Inst #1366 = PseudoVC_X_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1876] },  // Inst #1365 = PseudoVC_X_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1876] },  // Inst #1364 = PseudoVC_X_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1876] },  // Inst #1363 = PseudoVC_X_SE_M8
  { 6, &RISCVDescs.OperandInfo[1876] },  // Inst #1362 = PseudoVC_X_SE_M4
  { 6, &RISCVDescs.OperandInfo[1876] },  // Inst #1361 = PseudoVC_X_SE_M2
  { 6, &RISCVDescs.OperandInfo[1876] },  // Inst #1360 = PseudoVC_X_SE_M1
  { 6, &RISCVDescs.OperandInfo[1852] },  // Inst #1359 = PseudoVC_XV_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1852] },  // Inst #1358 = PseudoVC_XV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1852] },  // Inst #1357 = PseudoVC_XV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1870] },  // Inst #1356 = PseudoVC_XV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1864] },  // Inst #1355 = PseudoVC_XV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1858] },  // Inst #1354 = PseudoVC_XV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1852] },  // Inst #1353 = PseudoVC_XV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1810] },  // Inst #1352 = PseudoVC_XVW_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1810] },  // Inst #1351 = PseudoVC_XVW_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1810] },  // Inst #1350 = PseudoVC_XVW_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1846] },  // Inst #1349 = PseudoVC_XVW_SE_M4
  { 6, &RISCVDescs.OperandInfo[1840] },  // Inst #1348 = PseudoVC_XVW_SE_M2
  { 6, &RISCVDescs.OperandInfo[1834] },  // Inst #1347 = PseudoVC_XVW_SE_M1
  { 6, &RISCVDescs.OperandInfo[1810] },  // Inst #1346 = PseudoVC_XVV_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1810] },  // Inst #1345 = PseudoVC_XVV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1810] },  // Inst #1344 = PseudoVC_XVV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1828] },  // Inst #1343 = PseudoVC_XVV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1822] },  // Inst #1342 = PseudoVC_XVV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1816] },  // Inst #1341 = PseudoVC_XVV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1810] },  // Inst #1340 = PseudoVC_XVV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1786] },  // Inst #1339 = PseudoVC_V_X_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1786] },  // Inst #1338 = PseudoVC_V_X_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1786] },  // Inst #1337 = PseudoVC_V_X_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1804] },  // Inst #1336 = PseudoVC_V_X_SE_M8
  { 6, &RISCVDescs.OperandInfo[1798] },  // Inst #1335 = PseudoVC_V_X_SE_M4
  { 6, &RISCVDescs.OperandInfo[1792] },  // Inst #1334 = PseudoVC_V_X_SE_M2
  { 6, &RISCVDescs.OperandInfo[1786] },  // Inst #1333 = PseudoVC_V_X_SE_M1
  { 6, &RISCVDescs.OperandInfo[1786] },  // Inst #1332 = PseudoVC_V_X_MF8
  { 6, &RISCVDescs.OperandInfo[1786] },  // Inst #1331 = PseudoVC_V_X_MF4
  { 6, &RISCVDescs.OperandInfo[1786] },  // Inst #1330 = PseudoVC_V_X_MF2
  { 6, &RISCVDescs.OperandInfo[1804] },  // Inst #1329 = PseudoVC_V_X_M8
  { 6, &RISCVDescs.OperandInfo[1798] },  // Inst #1328 = PseudoVC_V_X_M4
  { 6, &RISCVDescs.OperandInfo[1792] },  // Inst #1327 = PseudoVC_V_X_M2
  { 6, &RISCVDescs.OperandInfo[1786] },  // Inst #1326 = PseudoVC_V_X_M1
  { 6, &RISCVDescs.OperandInfo[1762] },  // Inst #1325 = PseudoVC_V_XV_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1762] },  // Inst #1324 = PseudoVC_V_XV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1762] },  // Inst #1323 = PseudoVC_V_XV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1780] },  // Inst #1322 = PseudoVC_V_XV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1774] },  // Inst #1321 = PseudoVC_V_XV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1768] },  // Inst #1320 = PseudoVC_V_XV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1762] },  // Inst #1319 = PseudoVC_V_XV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1762] },  // Inst #1318 = PseudoVC_V_XV_MF8
  { 6, &RISCVDescs.OperandInfo[1762] },  // Inst #1317 = PseudoVC_V_XV_MF4
  { 6, &RISCVDescs.OperandInfo[1762] },  // Inst #1316 = PseudoVC_V_XV_MF2
  { 6, &RISCVDescs.OperandInfo[1780] },  // Inst #1315 = PseudoVC_V_XV_M8
  { 6, &RISCVDescs.OperandInfo[1774] },  // Inst #1314 = PseudoVC_V_XV_M4
  { 6, &RISCVDescs.OperandInfo[1768] },  // Inst #1313 = PseudoVC_V_XV_M2
  { 6, &RISCVDescs.OperandInfo[1762] },  // Inst #1312 = PseudoVC_V_XV_M1
  { 7, &RISCVDescs.OperandInfo[1755] },  // Inst #1311 = PseudoVC_V_XVW_SE_MF8
  { 7, &RISCVDescs.OperandInfo[1755] },  // Inst #1310 = PseudoVC_V_XVW_SE_MF4
  { 7, &RISCVDescs.OperandInfo[1755] },  // Inst #1309 = PseudoVC_V_XVW_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1748] },  // Inst #1308 = PseudoVC_V_XVW_SE_M4
  { 7, &RISCVDescs.OperandInfo[1741] },  // Inst #1307 = PseudoVC_V_XVW_SE_M2
  { 7, &RISCVDescs.OperandInfo[1734] },  // Inst #1306 = PseudoVC_V_XVW_SE_M1
  { 7, &RISCVDescs.OperandInfo[1755] },  // Inst #1305 = PseudoVC_V_XVW_MF8
  { 7, &RISCVDescs.OperandInfo[1755] },  // Inst #1304 = PseudoVC_V_XVW_MF4
  { 7, &RISCVDescs.OperandInfo[1755] },  // Inst #1303 = PseudoVC_V_XVW_MF2
  { 7, &RISCVDescs.OperandInfo[1748] },  // Inst #1302 = PseudoVC_V_XVW_M4
  { 7, &RISCVDescs.OperandInfo[1741] },  // Inst #1301 = PseudoVC_V_XVW_M2
  { 7, &RISCVDescs.OperandInfo[1734] },  // Inst #1300 = PseudoVC_V_XVW_M1
  { 7, &RISCVDescs.OperandInfo[1706] },  // Inst #1299 = PseudoVC_V_XVV_SE_MF8
  { 7, &RISCVDescs.OperandInfo[1706] },  // Inst #1298 = PseudoVC_V_XVV_SE_MF4
  { 7, &RISCVDescs.OperandInfo[1706] },  // Inst #1297 = PseudoVC_V_XVV_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1727] },  // Inst #1296 = PseudoVC_V_XVV_SE_M8
  { 7, &RISCVDescs.OperandInfo[1720] },  // Inst #1295 = PseudoVC_V_XVV_SE_M4
  { 7, &RISCVDescs.OperandInfo[1713] },  // Inst #1294 = PseudoVC_V_XVV_SE_M2
  { 7, &RISCVDescs.OperandInfo[1706] },  // Inst #1293 = PseudoVC_V_XVV_SE_M1
  { 7, &RISCVDescs.OperandInfo[1706] },  // Inst #1292 = PseudoVC_V_XVV_MF8
  { 7, &RISCVDescs.OperandInfo[1706] },  // Inst #1291 = PseudoVC_V_XVV_MF4
  { 7, &RISCVDescs.OperandInfo[1706] },  // Inst #1290 = PseudoVC_V_XVV_MF2
  { 7, &RISCVDescs.OperandInfo[1727] },  // Inst #1289 = PseudoVC_V_XVV_M8
  { 7, &RISCVDescs.OperandInfo[1720] },  // Inst #1288 = PseudoVC_V_XVV_M4
  { 7, &RISCVDescs.OperandInfo[1713] },  // Inst #1287 = PseudoVC_V_XVV_M2
  { 7, &RISCVDescs.OperandInfo[1706] },  // Inst #1286 = PseudoVC_V_XVV_M1
  { 6, &RISCVDescs.OperandInfo[1682] },  // Inst #1285 = PseudoVC_V_VV_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1682] },  // Inst #1284 = PseudoVC_V_VV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1682] },  // Inst #1283 = PseudoVC_V_VV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1700] },  // Inst #1282 = PseudoVC_V_VV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1694] },  // Inst #1281 = PseudoVC_V_VV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1688] },  // Inst #1280 = PseudoVC_V_VV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1682] },  // Inst #1279 = PseudoVC_V_VV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1682] },  // Inst #1278 = PseudoVC_V_VV_MF8
  { 6, &RISCVDescs.OperandInfo[1682] },  // Inst #1277 = PseudoVC_V_VV_MF4
  { 6, &RISCVDescs.OperandInfo[1682] },  // Inst #1276 = PseudoVC_V_VV_MF2
  { 6, &RISCVDescs.OperandInfo[1700] },  // Inst #1275 = PseudoVC_V_VV_M8
  { 6, &RISCVDescs.OperandInfo[1694] },  // Inst #1274 = PseudoVC_V_VV_M4
  { 6, &RISCVDescs.OperandInfo[1688] },  // Inst #1273 = PseudoVC_V_VV_M2
  { 6, &RISCVDescs.OperandInfo[1682] },  // Inst #1272 = PseudoVC_V_VV_M1
  { 7, &RISCVDescs.OperandInfo[1675] },  // Inst #1271 = PseudoVC_V_VVW_SE_MF8
  { 7, &RISCVDescs.OperandInfo[1675] },  // Inst #1270 = PseudoVC_V_VVW_SE_MF4
  { 7, &RISCVDescs.OperandInfo[1675] },  // Inst #1269 = PseudoVC_V_VVW_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1668] },  // Inst #1268 = PseudoVC_V_VVW_SE_M4
  { 7, &RISCVDescs.OperandInfo[1661] },  // Inst #1267 = PseudoVC_V_VVW_SE_M2
  { 7, &RISCVDescs.OperandInfo[1654] },  // Inst #1266 = PseudoVC_V_VVW_SE_M1
  { 7, &RISCVDescs.OperandInfo[1675] },  // Inst #1265 = PseudoVC_V_VVW_MF8
  { 7, &RISCVDescs.OperandInfo[1675] },  // Inst #1264 = PseudoVC_V_VVW_MF4
  { 7, &RISCVDescs.OperandInfo[1675] },  // Inst #1263 = PseudoVC_V_VVW_MF2
  { 7, &RISCVDescs.OperandInfo[1668] },  // Inst #1262 = PseudoVC_V_VVW_M4
  { 7, &RISCVDescs.OperandInfo[1661] },  // Inst #1261 = PseudoVC_V_VVW_M2
  { 7, &RISCVDescs.OperandInfo[1654] },  // Inst #1260 = PseudoVC_V_VVW_M1
  { 7, &RISCVDescs.OperandInfo[1626] },  // Inst #1259 = PseudoVC_V_VVV_SE_MF8
  { 7, &RISCVDescs.OperandInfo[1626] },  // Inst #1258 = PseudoVC_V_VVV_SE_MF4
  { 7, &RISCVDescs.OperandInfo[1626] },  // Inst #1257 = PseudoVC_V_VVV_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1647] },  // Inst #1256 = PseudoVC_V_VVV_SE_M8
  { 7, &RISCVDescs.OperandInfo[1640] },  // Inst #1255 = PseudoVC_V_VVV_SE_M4
  { 7, &RISCVDescs.OperandInfo[1633] },  // Inst #1254 = PseudoVC_V_VVV_SE_M2
  { 7, &RISCVDescs.OperandInfo[1626] },  // Inst #1253 = PseudoVC_V_VVV_SE_M1
  { 7, &RISCVDescs.OperandInfo[1626] },  // Inst #1252 = PseudoVC_V_VVV_MF8
  { 7, &RISCVDescs.OperandInfo[1626] },  // Inst #1251 = PseudoVC_V_VVV_MF4
  { 7, &RISCVDescs.OperandInfo[1626] },  // Inst #1250 = PseudoVC_V_VVV_MF2
  { 7, &RISCVDescs.OperandInfo[1647] },  // Inst #1249 = PseudoVC_V_VVV_M8
  { 7, &RISCVDescs.OperandInfo[1640] },  // Inst #1248 = PseudoVC_V_VVV_M4
  { 7, &RISCVDescs.OperandInfo[1633] },  // Inst #1247 = PseudoVC_V_VVV_M2
  { 7, &RISCVDescs.OperandInfo[1626] },  // Inst #1246 = PseudoVC_V_VVV_M1
  { 6, &RISCVDescs.OperandInfo[1602] },  // Inst #1245 = PseudoVC_V_I_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1602] },  // Inst #1244 = PseudoVC_V_I_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1602] },  // Inst #1243 = PseudoVC_V_I_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1620] },  // Inst #1242 = PseudoVC_V_I_SE_M8
  { 6, &RISCVDescs.OperandInfo[1614] },  // Inst #1241 = PseudoVC_V_I_SE_M4
  { 6, &RISCVDescs.OperandInfo[1608] },  // Inst #1240 = PseudoVC_V_I_SE_M2
  { 6, &RISCVDescs.OperandInfo[1602] },  // Inst #1239 = PseudoVC_V_I_SE_M1
  { 6, &RISCVDescs.OperandInfo[1602] },  // Inst #1238 = PseudoVC_V_I_MF8
  { 6, &RISCVDescs.OperandInfo[1602] },  // Inst #1237 = PseudoVC_V_I_MF4
  { 6, &RISCVDescs.OperandInfo[1602] },  // Inst #1236 = PseudoVC_V_I_MF2
  { 6, &RISCVDescs.OperandInfo[1620] },  // Inst #1235 = PseudoVC_V_I_M8
  { 6, &RISCVDescs.OperandInfo[1614] },  // Inst #1234 = PseudoVC_V_I_M4
  { 6, &RISCVDescs.OperandInfo[1608] },  // Inst #1233 = PseudoVC_V_I_M2
  { 6, &RISCVDescs.OperandInfo[1602] },  // Inst #1232 = PseudoVC_V_I_M1
  { 6, &RISCVDescs.OperandInfo[1578] },  // Inst #1231 = PseudoVC_V_IV_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1578] },  // Inst #1230 = PseudoVC_V_IV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1578] },  // Inst #1229 = PseudoVC_V_IV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1596] },  // Inst #1228 = PseudoVC_V_IV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1590] },  // Inst #1227 = PseudoVC_V_IV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1584] },  // Inst #1226 = PseudoVC_V_IV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1578] },  // Inst #1225 = PseudoVC_V_IV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1578] },  // Inst #1224 = PseudoVC_V_IV_MF8
  { 6, &RISCVDescs.OperandInfo[1578] },  // Inst #1223 = PseudoVC_V_IV_MF4
  { 6, &RISCVDescs.OperandInfo[1578] },  // Inst #1222 = PseudoVC_V_IV_MF2
  { 6, &RISCVDescs.OperandInfo[1596] },  // Inst #1221 = PseudoVC_V_IV_M8
  { 6, &RISCVDescs.OperandInfo[1590] },  // Inst #1220 = PseudoVC_V_IV_M4
  { 6, &RISCVDescs.OperandInfo[1584] },  // Inst #1219 = PseudoVC_V_IV_M2
  { 6, &RISCVDescs.OperandInfo[1578] },  // Inst #1218 = PseudoVC_V_IV_M1
  { 7, &RISCVDescs.OperandInfo[1571] },  // Inst #1217 = PseudoVC_V_IVW_SE_MF8
  { 7, &RISCVDescs.OperandInfo[1571] },  // Inst #1216 = PseudoVC_V_IVW_SE_MF4
  { 7, &RISCVDescs.OperandInfo[1571] },  // Inst #1215 = PseudoVC_V_IVW_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1564] },  // Inst #1214 = PseudoVC_V_IVW_SE_M4
  { 7, &RISCVDescs.OperandInfo[1557] },  // Inst #1213 = PseudoVC_V_IVW_SE_M2
  { 7, &RISCVDescs.OperandInfo[1550] },  // Inst #1212 = PseudoVC_V_IVW_SE_M1
  { 7, &RISCVDescs.OperandInfo[1571] },  // Inst #1211 = PseudoVC_V_IVW_MF8
  { 7, &RISCVDescs.OperandInfo[1571] },  // Inst #1210 = PseudoVC_V_IVW_MF4
  { 7, &RISCVDescs.OperandInfo[1571] },  // Inst #1209 = PseudoVC_V_IVW_MF2
  { 7, &RISCVDescs.OperandInfo[1564] },  // Inst #1208 = PseudoVC_V_IVW_M4
  { 7, &RISCVDescs.OperandInfo[1557] },  // Inst #1207 = PseudoVC_V_IVW_M2
  { 7, &RISCVDescs.OperandInfo[1550] },  // Inst #1206 = PseudoVC_V_IVW_M1
  { 7, &RISCVDescs.OperandInfo[1522] },  // Inst #1205 = PseudoVC_V_IVV_SE_MF8
  { 7, &RISCVDescs.OperandInfo[1522] },  // Inst #1204 = PseudoVC_V_IVV_SE_MF4
  { 7, &RISCVDescs.OperandInfo[1522] },  // Inst #1203 = PseudoVC_V_IVV_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1543] },  // Inst #1202 = PseudoVC_V_IVV_SE_M8
  { 7, &RISCVDescs.OperandInfo[1536] },  // Inst #1201 = PseudoVC_V_IVV_SE_M4
  { 7, &RISCVDescs.OperandInfo[1529] },  // Inst #1200 = PseudoVC_V_IVV_SE_M2
  { 7, &RISCVDescs.OperandInfo[1522] },  // Inst #1199 = PseudoVC_V_IVV_SE_M1
  { 7, &RISCVDescs.OperandInfo[1522] },  // Inst #1198 = PseudoVC_V_IVV_MF8
  { 7, &RISCVDescs.OperandInfo[1522] },  // Inst #1197 = PseudoVC_V_IVV_MF4
  { 7, &RISCVDescs.OperandInfo[1522] },  // Inst #1196 = PseudoVC_V_IVV_MF2
  { 7, &RISCVDescs.OperandInfo[1543] },  // Inst #1195 = PseudoVC_V_IVV_M8
  { 7, &RISCVDescs.OperandInfo[1536] },  // Inst #1194 = PseudoVC_V_IVV_M4
  { 7, &RISCVDescs.OperandInfo[1529] },  // Inst #1193 = PseudoVC_V_IVV_M2
  { 7, &RISCVDescs.OperandInfo[1522] },  // Inst #1192 = PseudoVC_V_IVV_M1
  { 6, &RISCVDescs.OperandInfo[1516] },  // Inst #1191 = PseudoVC_V_FPR64V_SE_M8
  { 6, &RISCVDescs.OperandInfo[1510] },  // Inst #1190 = PseudoVC_V_FPR64V_SE_M4
  { 6, &RISCVDescs.OperandInfo[1504] },  // Inst #1189 = PseudoVC_V_FPR64V_SE_M2
  { 6, &RISCVDescs.OperandInfo[1498] },  // Inst #1188 = PseudoVC_V_FPR64V_SE_M1
  { 6, &RISCVDescs.OperandInfo[1516] },  // Inst #1187 = PseudoVC_V_FPR64V_M8
  { 6, &RISCVDescs.OperandInfo[1510] },  // Inst #1186 = PseudoVC_V_FPR64V_M4
  { 6, &RISCVDescs.OperandInfo[1504] },  // Inst #1185 = PseudoVC_V_FPR64V_M2
  { 6, &RISCVDescs.OperandInfo[1498] },  // Inst #1184 = PseudoVC_V_FPR64V_M1
  { 7, &RISCVDescs.OperandInfo[1491] },  // Inst #1183 = PseudoVC_V_FPR64VV_SE_M8
  { 7, &RISCVDescs.OperandInfo[1484] },  // Inst #1182 = PseudoVC_V_FPR64VV_SE_M4
  { 7, &RISCVDescs.OperandInfo[1477] },  // Inst #1181 = PseudoVC_V_FPR64VV_SE_M2
  { 7, &RISCVDescs.OperandInfo[1470] },  // Inst #1180 = PseudoVC_V_FPR64VV_SE_M1
  { 7, &RISCVDescs.OperandInfo[1491] },  // Inst #1179 = PseudoVC_V_FPR64VV_M8
  { 7, &RISCVDescs.OperandInfo[1484] },  // Inst #1178 = PseudoVC_V_FPR64VV_M4
  { 7, &RISCVDescs.OperandInfo[1477] },  // Inst #1177 = PseudoVC_V_FPR64VV_M2
  { 7, &RISCVDescs.OperandInfo[1470] },  // Inst #1176 = PseudoVC_V_FPR64VV_M1
  { 6, &RISCVDescs.OperandInfo[1446] },  // Inst #1175 = PseudoVC_V_FPR32V_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1464] },  // Inst #1174 = PseudoVC_V_FPR32V_SE_M8
  { 6, &RISCVDescs.OperandInfo[1458] },  // Inst #1173 = PseudoVC_V_FPR32V_SE_M4
  { 6, &RISCVDescs.OperandInfo[1452] },  // Inst #1172 = PseudoVC_V_FPR32V_SE_M2
  { 6, &RISCVDescs.OperandInfo[1446] },  // Inst #1171 = PseudoVC_V_FPR32V_SE_M1
  { 6, &RISCVDescs.OperandInfo[1446] },  // Inst #1170 = PseudoVC_V_FPR32V_MF2
  { 6, &RISCVDescs.OperandInfo[1464] },  // Inst #1169 = PseudoVC_V_FPR32V_M8
  { 6, &RISCVDescs.OperandInfo[1458] },  // Inst #1168 = PseudoVC_V_FPR32V_M4
  { 6, &RISCVDescs.OperandInfo[1452] },  // Inst #1167 = PseudoVC_V_FPR32V_M2
  { 6, &RISCVDescs.OperandInfo[1446] },  // Inst #1166 = PseudoVC_V_FPR32V_M1
  { 7, &RISCVDescs.OperandInfo[1439] },  // Inst #1165 = PseudoVC_V_FPR32VW_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1432] },  // Inst #1164 = PseudoVC_V_FPR32VW_SE_M8
  { 7, &RISCVDescs.OperandInfo[1425] },  // Inst #1163 = PseudoVC_V_FPR32VW_SE_M4
  { 7, &RISCVDescs.OperandInfo[1418] },  // Inst #1162 = PseudoVC_V_FPR32VW_SE_M2
  { 7, &RISCVDescs.OperandInfo[1411] },  // Inst #1161 = PseudoVC_V_FPR32VW_SE_M1
  { 7, &RISCVDescs.OperandInfo[1439] },  // Inst #1160 = PseudoVC_V_FPR32VW_MF2
  { 7, &RISCVDescs.OperandInfo[1432] },  // Inst #1159 = PseudoVC_V_FPR32VW_M8
  { 7, &RISCVDescs.OperandInfo[1425] },  // Inst #1158 = PseudoVC_V_FPR32VW_M4
  { 7, &RISCVDescs.OperandInfo[1418] },  // Inst #1157 = PseudoVC_V_FPR32VW_M2
  { 7, &RISCVDescs.OperandInfo[1411] },  // Inst #1156 = PseudoVC_V_FPR32VW_M1
  { 7, &RISCVDescs.OperandInfo[1383] },  // Inst #1155 = PseudoVC_V_FPR32VV_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1404] },  // Inst #1154 = PseudoVC_V_FPR32VV_SE_M8
  { 7, &RISCVDescs.OperandInfo[1397] },  // Inst #1153 = PseudoVC_V_FPR32VV_SE_M4
  { 7, &RISCVDescs.OperandInfo[1390] },  // Inst #1152 = PseudoVC_V_FPR32VV_SE_M2
  { 7, &RISCVDescs.OperandInfo[1383] },  // Inst #1151 = PseudoVC_V_FPR32VV_SE_M1
  { 7, &RISCVDescs.OperandInfo[1383] },  // Inst #1150 = PseudoVC_V_FPR32VV_MF2
  { 7, &RISCVDescs.OperandInfo[1404] },  // Inst #1149 = PseudoVC_V_FPR32VV_M8
  { 7, &RISCVDescs.OperandInfo[1397] },  // Inst #1148 = PseudoVC_V_FPR32VV_M4
  { 7, &RISCVDescs.OperandInfo[1390] },  // Inst #1147 = PseudoVC_V_FPR32VV_M2
  { 7, &RISCVDescs.OperandInfo[1383] },  // Inst #1146 = PseudoVC_V_FPR32VV_M1
  { 6, &RISCVDescs.OperandInfo[1359] },  // Inst #1145 = PseudoVC_V_FPR16V_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1359] },  // Inst #1144 = PseudoVC_V_FPR16V_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1377] },  // Inst #1143 = PseudoVC_V_FPR16V_SE_M8
  { 6, &RISCVDescs.OperandInfo[1371] },  // Inst #1142 = PseudoVC_V_FPR16V_SE_M4
  { 6, &RISCVDescs.OperandInfo[1365] },  // Inst #1141 = PseudoVC_V_FPR16V_SE_M2
  { 6, &RISCVDescs.OperandInfo[1359] },  // Inst #1140 = PseudoVC_V_FPR16V_SE_M1
  { 6, &RISCVDescs.OperandInfo[1359] },  // Inst #1139 = PseudoVC_V_FPR16V_MF4
  { 6, &RISCVDescs.OperandInfo[1359] },  // Inst #1138 = PseudoVC_V_FPR16V_MF2
  { 6, &RISCVDescs.OperandInfo[1377] },  // Inst #1137 = PseudoVC_V_FPR16V_M8
  { 6, &RISCVDescs.OperandInfo[1371] },  // Inst #1136 = PseudoVC_V_FPR16V_M4
  { 6, &RISCVDescs.OperandInfo[1365] },  // Inst #1135 = PseudoVC_V_FPR16V_M2
  { 6, &RISCVDescs.OperandInfo[1359] },  // Inst #1134 = PseudoVC_V_FPR16V_M1
  { 7, &RISCVDescs.OperandInfo[1352] },  // Inst #1133 = PseudoVC_V_FPR16VW_SE_MF4
  { 7, &RISCVDescs.OperandInfo[1352] },  // Inst #1132 = PseudoVC_V_FPR16VW_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1345] },  // Inst #1131 = PseudoVC_V_FPR16VW_SE_M8
  { 7, &RISCVDescs.OperandInfo[1338] },  // Inst #1130 = PseudoVC_V_FPR16VW_SE_M4
  { 7, &RISCVDescs.OperandInfo[1331] },  // Inst #1129 = PseudoVC_V_FPR16VW_SE_M2
  { 7, &RISCVDescs.OperandInfo[1324] },  // Inst #1128 = PseudoVC_V_FPR16VW_SE_M1
  { 7, &RISCVDescs.OperandInfo[1352] },  // Inst #1127 = PseudoVC_V_FPR16VW_MF4
  { 7, &RISCVDescs.OperandInfo[1352] },  // Inst #1126 = PseudoVC_V_FPR16VW_MF2
  { 7, &RISCVDescs.OperandInfo[1345] },  // Inst #1125 = PseudoVC_V_FPR16VW_M8
  { 7, &RISCVDescs.OperandInfo[1338] },  // Inst #1124 = PseudoVC_V_FPR16VW_M4
  { 7, &RISCVDescs.OperandInfo[1331] },  // Inst #1123 = PseudoVC_V_FPR16VW_M2
  { 7, &RISCVDescs.OperandInfo[1324] },  // Inst #1122 = PseudoVC_V_FPR16VW_M1
  { 7, &RISCVDescs.OperandInfo[1296] },  // Inst #1121 = PseudoVC_V_FPR16VV_SE_MF4
  { 7, &RISCVDescs.OperandInfo[1296] },  // Inst #1120 = PseudoVC_V_FPR16VV_SE_MF2
  { 7, &RISCVDescs.OperandInfo[1317] },  // Inst #1119 = PseudoVC_V_FPR16VV_SE_M8
  { 7, &RISCVDescs.OperandInfo[1310] },  // Inst #1118 = PseudoVC_V_FPR16VV_SE_M4
  { 7, &RISCVDescs.OperandInfo[1303] },  // Inst #1117 = PseudoVC_V_FPR16VV_SE_M2
  { 7, &RISCVDescs.OperandInfo[1296] },  // Inst #1116 = PseudoVC_V_FPR16VV_SE_M1
  { 7, &RISCVDescs.OperandInfo[1296] },  // Inst #1115 = PseudoVC_V_FPR16VV_MF4
  { 7, &RISCVDescs.OperandInfo[1296] },  // Inst #1114 = PseudoVC_V_FPR16VV_MF2
  { 7, &RISCVDescs.OperandInfo[1317] },  // Inst #1113 = PseudoVC_V_FPR16VV_M8
  { 7, &RISCVDescs.OperandInfo[1310] },  // Inst #1112 = PseudoVC_V_FPR16VV_M4
  { 7, &RISCVDescs.OperandInfo[1303] },  // Inst #1111 = PseudoVC_V_FPR16VV_M2
  { 7, &RISCVDescs.OperandInfo[1296] },  // Inst #1110 = PseudoVC_V_FPR16VV_M1
  { 6, &RISCVDescs.OperandInfo[1272] },  // Inst #1109 = PseudoVC_VV_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1272] },  // Inst #1108 = PseudoVC_VV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1272] },  // Inst #1107 = PseudoVC_VV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1290] },  // Inst #1106 = PseudoVC_VV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1284] },  // Inst #1105 = PseudoVC_VV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1278] },  // Inst #1104 = PseudoVC_VV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1272] },  // Inst #1103 = PseudoVC_VV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1230] },  // Inst #1102 = PseudoVC_VVW_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1230] },  // Inst #1101 = PseudoVC_VVW_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1230] },  // Inst #1100 = PseudoVC_VVW_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1266] },  // Inst #1099 = PseudoVC_VVW_SE_M4
  { 6, &RISCVDescs.OperandInfo[1260] },  // Inst #1098 = PseudoVC_VVW_SE_M2
  { 6, &RISCVDescs.OperandInfo[1254] },  // Inst #1097 = PseudoVC_VVW_SE_M1
  { 6, &RISCVDescs.OperandInfo[1230] },  // Inst #1096 = PseudoVC_VVV_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1230] },  // Inst #1095 = PseudoVC_VVV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1230] },  // Inst #1094 = PseudoVC_VVV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1248] },  // Inst #1093 = PseudoVC_VVV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1242] },  // Inst #1092 = PseudoVC_VVV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1236] },  // Inst #1091 = PseudoVC_VVV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1230] },  // Inst #1090 = PseudoVC_VVV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1224] },  // Inst #1089 = PseudoVC_I_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1224] },  // Inst #1088 = PseudoVC_I_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1224] },  // Inst #1087 = PseudoVC_I_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1224] },  // Inst #1086 = PseudoVC_I_SE_M8
  { 6, &RISCVDescs.OperandInfo[1224] },  // Inst #1085 = PseudoVC_I_SE_M4
  { 6, &RISCVDescs.OperandInfo[1224] },  // Inst #1084 = PseudoVC_I_SE_M2
  { 6, &RISCVDescs.OperandInfo[1224] },  // Inst #1083 = PseudoVC_I_SE_M1
  { 6, &RISCVDescs.OperandInfo[1200] },  // Inst #1082 = PseudoVC_IV_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1200] },  // Inst #1081 = PseudoVC_IV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1200] },  // Inst #1080 = PseudoVC_IV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1218] },  // Inst #1079 = PseudoVC_IV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1212] },  // Inst #1078 = PseudoVC_IV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1206] },  // Inst #1077 = PseudoVC_IV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1200] },  // Inst #1076 = PseudoVC_IV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1158] },  // Inst #1075 = PseudoVC_IVW_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1158] },  // Inst #1074 = PseudoVC_IVW_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1158] },  // Inst #1073 = PseudoVC_IVW_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1194] },  // Inst #1072 = PseudoVC_IVW_SE_M4
  { 6, &RISCVDescs.OperandInfo[1188] },  // Inst #1071 = PseudoVC_IVW_SE_M2
  { 6, &RISCVDescs.OperandInfo[1182] },  // Inst #1070 = PseudoVC_IVW_SE_M1
  { 6, &RISCVDescs.OperandInfo[1158] },  // Inst #1069 = PseudoVC_IVV_SE_MF8
  { 6, &RISCVDescs.OperandInfo[1158] },  // Inst #1068 = PseudoVC_IVV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1158] },  // Inst #1067 = PseudoVC_IVV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1176] },  // Inst #1066 = PseudoVC_IVV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1170] },  // Inst #1065 = PseudoVC_IVV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1164] },  // Inst #1064 = PseudoVC_IVV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1158] },  // Inst #1063 = PseudoVC_IVV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1152] },  // Inst #1062 = PseudoVC_FPR64V_SE_M8
  { 6, &RISCVDescs.OperandInfo[1146] },  // Inst #1061 = PseudoVC_FPR64V_SE_M4
  { 6, &RISCVDescs.OperandInfo[1140] },  // Inst #1060 = PseudoVC_FPR64V_SE_M2
  { 6, &RISCVDescs.OperandInfo[1134] },  // Inst #1059 = PseudoVC_FPR64V_SE_M1
  { 6, &RISCVDescs.OperandInfo[1128] },  // Inst #1058 = PseudoVC_FPR64VV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1122] },  // Inst #1057 = PseudoVC_FPR64VV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1116] },  // Inst #1056 = PseudoVC_FPR64VV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1110] },  // Inst #1055 = PseudoVC_FPR64VV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1086] },  // Inst #1054 = PseudoVC_FPR32V_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1104] },  // Inst #1053 = PseudoVC_FPR32V_SE_M8
  { 6, &RISCVDescs.OperandInfo[1098] },  // Inst #1052 = PseudoVC_FPR32V_SE_M4
  { 6, &RISCVDescs.OperandInfo[1092] },  // Inst #1051 = PseudoVC_FPR32V_SE_M2
  { 6, &RISCVDescs.OperandInfo[1086] },  // Inst #1050 = PseudoVC_FPR32V_SE_M1
  { 6, &RISCVDescs.OperandInfo[1038] },  // Inst #1049 = PseudoVC_FPR32VW_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1080] },  // Inst #1048 = PseudoVC_FPR32VW_SE_M8
  { 6, &RISCVDescs.OperandInfo[1074] },  // Inst #1047 = PseudoVC_FPR32VW_SE_M4
  { 6, &RISCVDescs.OperandInfo[1068] },  // Inst #1046 = PseudoVC_FPR32VW_SE_M2
  { 6, &RISCVDescs.OperandInfo[1062] },  // Inst #1045 = PseudoVC_FPR32VW_SE_M1
  { 6, &RISCVDescs.OperandInfo[1038] },  // Inst #1044 = PseudoVC_FPR32VV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1056] },  // Inst #1043 = PseudoVC_FPR32VV_SE_M8
  { 6, &RISCVDescs.OperandInfo[1050] },  // Inst #1042 = PseudoVC_FPR32VV_SE_M4
  { 6, &RISCVDescs.OperandInfo[1044] },  // Inst #1041 = PseudoVC_FPR32VV_SE_M2
  { 6, &RISCVDescs.OperandInfo[1038] },  // Inst #1040 = PseudoVC_FPR32VV_SE_M1
  { 6, &RISCVDescs.OperandInfo[1014] },  // Inst #1039 = PseudoVC_FPR16V_SE_MF4
  { 6, &RISCVDescs.OperandInfo[1014] },  // Inst #1038 = PseudoVC_FPR16V_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1032] },  // Inst #1037 = PseudoVC_FPR16V_SE_M8
  { 6, &RISCVDescs.OperandInfo[1026] },  // Inst #1036 = PseudoVC_FPR16V_SE_M4
  { 6, &RISCVDescs.OperandInfo[1020] },  // Inst #1035 = PseudoVC_FPR16V_SE_M2
  { 6, &RISCVDescs.OperandInfo[1014] },  // Inst #1034 = PseudoVC_FPR16V_SE_M1
  { 6, &RISCVDescs.OperandInfo[966] },  // Inst #1033 = PseudoVC_FPR16VW_SE_MF4
  { 6, &RISCVDescs.OperandInfo[966] },  // Inst #1032 = PseudoVC_FPR16VW_SE_MF2
  { 6, &RISCVDescs.OperandInfo[1008] },  // Inst #1031 = PseudoVC_FPR16VW_SE_M8
  { 6, &RISCVDescs.OperandInfo[1002] },  // Inst #1030 = PseudoVC_FPR16VW_SE_M4
  { 6, &RISCVDescs.OperandInfo[996] },  // Inst #1029 = PseudoVC_FPR16VW_SE_M2
  { 6, &RISCVDescs.OperandInfo[990] },  // Inst #1028 = PseudoVC_FPR16VW_SE_M1
  { 6, &RISCVDescs.OperandInfo[966] },  // Inst #1027 = PseudoVC_FPR16VV_SE_MF4
  { 6, &RISCVDescs.OperandInfo[966] },  // Inst #1026 = PseudoVC_FPR16VV_SE_MF2
  { 6, &RISCVDescs.OperandInfo[984] },  // Inst #1025 = PseudoVC_FPR16VV_SE_M8
  { 6, &RISCVDescs.OperandInfo[978] },  // Inst #1024 = PseudoVC_FPR16VV_SE_M4
  { 6, &RISCVDescs.OperandInfo[972] },  // Inst #1023 = PseudoVC_FPR16VV_SE_M2
  { 6, &RISCVDescs.OperandInfo[966] },  // Inst #1022 = PseudoVC_FPR16VV_SE_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1021 = PseudoVCTZ_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1020 = PseudoVCTZ_V_MF8
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1019 = PseudoVCTZ_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1018 = PseudoVCTZ_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1017 = PseudoVCTZ_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1016 = PseudoVCTZ_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #1015 = PseudoVCTZ_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #1014 = PseudoVCTZ_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #1013 = PseudoVCTZ_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #1012 = PseudoVCTZ_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #1011 = PseudoVCTZ_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #1010 = PseudoVCTZ_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1009 = PseudoVCTZ_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1008 = PseudoVCTZ_V_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1007 = PseudoVCPOP_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1006 = PseudoVCPOP_V_MF8
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1005 = PseudoVCPOP_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1004 = PseudoVCPOP_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #1003 = PseudoVCPOP_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #1002 = PseudoVCPOP_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #1001 = PseudoVCPOP_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #1000 = PseudoVCPOP_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #999 = PseudoVCPOP_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #998 = PseudoVCPOP_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #997 = PseudoVCPOP_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #996 = PseudoVCPOP_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #995 = PseudoVCPOP_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #994 = PseudoVCPOP_V_M1
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #993 = PseudoVCPOP_M_B8_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #992 = PseudoVCPOP_M_B8
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #991 = PseudoVCPOP_M_B64_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #990 = PseudoVCPOP_M_B64
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #989 = PseudoVCPOP_M_B4_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #988 = PseudoVCPOP_M_B4
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #987 = PseudoVCPOP_M_B32_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #986 = PseudoVCPOP_M_B32
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #985 = PseudoVCPOP_M_B2_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #984 = PseudoVCPOP_M_B2
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #983 = PseudoVCPOP_M_B1_MASK
  { 5, &RISCVDescs.OperandInfo[961] },  // Inst #982 = PseudoVCPOP_M_B16_MASK
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #981 = PseudoVCPOP_M_B16
  { 4, &RISCVDescs.OperandInfo[957] },  // Inst #980 = PseudoVCPOP_M_B1
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #979 = PseudoVCOMPRESS_VM_MF8_E8
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #978 = PseudoVCOMPRESS_VM_MF4_E8
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #977 = PseudoVCOMPRESS_VM_MF4_E16
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #976 = PseudoVCOMPRESS_VM_MF2_E8
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #975 = PseudoVCOMPRESS_VM_MF2_E32
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #974 = PseudoVCOMPRESS_VM_MF2_E16
  { 6, &RISCVDescs.OperandInfo[951] },  // Inst #973 = PseudoVCOMPRESS_VM_M8_E8
  { 6, &RISCVDescs.OperandInfo[951] },  // Inst #972 = PseudoVCOMPRESS_VM_M8_E64
  { 6, &RISCVDescs.OperandInfo[951] },  // Inst #971 = PseudoVCOMPRESS_VM_M8_E32
  { 6, &RISCVDescs.OperandInfo[951] },  // Inst #970 = PseudoVCOMPRESS_VM_M8_E16
  { 6, &RISCVDescs.OperandInfo[945] },  // Inst #969 = PseudoVCOMPRESS_VM_M4_E8
  { 6, &RISCVDescs.OperandInfo[945] },  // Inst #968 = PseudoVCOMPRESS_VM_M4_E64
  { 6, &RISCVDescs.OperandInfo[945] },  // Inst #967 = PseudoVCOMPRESS_VM_M4_E32
  { 6, &RISCVDescs.OperandInfo[945] },  // Inst #966 = PseudoVCOMPRESS_VM_M4_E16
  { 6, &RISCVDescs.OperandInfo[939] },  // Inst #965 = PseudoVCOMPRESS_VM_M2_E8
  { 6, &RISCVDescs.OperandInfo[939] },  // Inst #964 = PseudoVCOMPRESS_VM_M2_E64
  { 6, &RISCVDescs.OperandInfo[939] },  // Inst #963 = PseudoVCOMPRESS_VM_M2_E32
  { 6, &RISCVDescs.OperandInfo[939] },  // Inst #962 = PseudoVCOMPRESS_VM_M2_E16
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #961 = PseudoVCOMPRESS_VM_M1_E8
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #960 = PseudoVCOMPRESS_VM_M1_E64
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #959 = PseudoVCOMPRESS_VM_M1_E32
  { 6, &RISCVDescs.OperandInfo[933] },  // Inst #958 = PseudoVCOMPRESS_VM_M1_E16
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #957 = PseudoVCLZ_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #956 = PseudoVCLZ_V_MF8
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #955 = PseudoVCLZ_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #954 = PseudoVCLZ_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #953 = PseudoVCLZ_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #952 = PseudoVCLZ_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #951 = PseudoVCLZ_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #950 = PseudoVCLZ_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #949 = PseudoVCLZ_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #948 = PseudoVCLZ_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #947 = PseudoVCLZ_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #946 = PseudoVCLZ_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #945 = PseudoVCLZ_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #944 = PseudoVCLZ_V_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #943 = PseudoVCLMUL_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #942 = PseudoVCLMUL_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #941 = PseudoVCLMUL_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #940 = PseudoVCLMUL_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #939 = PseudoVCLMUL_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #938 = PseudoVCLMUL_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #937 = PseudoVCLMUL_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #936 = PseudoVCLMUL_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #935 = PseudoVCLMUL_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #934 = PseudoVCLMUL_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #933 = PseudoVCLMUL_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #932 = PseudoVCLMUL_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #931 = PseudoVCLMUL_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #930 = PseudoVCLMUL_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #929 = PseudoVCLMUL_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #928 = PseudoVCLMUL_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #927 = PseudoVCLMUL_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #926 = PseudoVCLMUL_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #925 = PseudoVCLMUL_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #924 = PseudoVCLMUL_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #923 = PseudoVCLMUL_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #922 = PseudoVCLMUL_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #921 = PseudoVCLMUL_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #920 = PseudoVCLMUL_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #919 = PseudoVCLMUL_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #918 = PseudoVCLMUL_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #917 = PseudoVCLMUL_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #916 = PseudoVCLMUL_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #915 = PseudoVCLMULH_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #914 = PseudoVCLMULH_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #913 = PseudoVCLMULH_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #912 = PseudoVCLMULH_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #911 = PseudoVCLMULH_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #910 = PseudoVCLMULH_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #909 = PseudoVCLMULH_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #908 = PseudoVCLMULH_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #907 = PseudoVCLMULH_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #906 = PseudoVCLMULH_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #905 = PseudoVCLMULH_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #904 = PseudoVCLMULH_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #903 = PseudoVCLMULH_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #902 = PseudoVCLMULH_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #901 = PseudoVCLMULH_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #900 = PseudoVCLMULH_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #899 = PseudoVCLMULH_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #898 = PseudoVCLMULH_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #897 = PseudoVCLMULH_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #896 = PseudoVCLMULH_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #895 = PseudoVCLMULH_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #894 = PseudoVCLMULH_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #893 = PseudoVCLMULH_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #892 = PseudoVCLMULH_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #891 = PseudoVCLMULH_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #890 = PseudoVCLMULH_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #889 = PseudoVCLMULH_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #888 = PseudoVCLMULH_VV_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #887 = PseudoVBREV_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #886 = PseudoVBREV_V_MF8
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #885 = PseudoVBREV_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #884 = PseudoVBREV_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #883 = PseudoVBREV_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #882 = PseudoVBREV_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #881 = PseudoVBREV_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #880 = PseudoVBREV_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #879 = PseudoVBREV_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #878 = PseudoVBREV_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #877 = PseudoVBREV_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #876 = PseudoVBREV_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #875 = PseudoVBREV_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #874 = PseudoVBREV_V_M1
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #873 = PseudoVBREV8_V_MF8_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #872 = PseudoVBREV8_V_MF8
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #871 = PseudoVBREV8_V_MF4_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #870 = PseudoVBREV8_V_MF4
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #869 = PseudoVBREV8_V_MF2_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #868 = PseudoVBREV8_V_MF2
  { 7, &RISCVDescs.OperandInfo[926] },  // Inst #867 = PseudoVBREV8_V_M8_MASK
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #866 = PseudoVBREV8_V_M8
  { 7, &RISCVDescs.OperandInfo[919] },  // Inst #865 = PseudoVBREV8_V_M4_MASK
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #864 = PseudoVBREV8_V_M4
  { 7, &RISCVDescs.OperandInfo[912] },  // Inst #863 = PseudoVBREV8_V_M2_MASK
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #862 = PseudoVBREV8_V_M2
  { 7, &RISCVDescs.OperandInfo[905] },  // Inst #861 = PseudoVBREV8_V_M1_MASK
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #860 = PseudoVBREV8_V_M1
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #859 = PseudoVASUB_VX_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #858 = PseudoVASUB_VX_MF8
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #857 = PseudoVASUB_VX_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #856 = PseudoVASUB_VX_MF4
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #855 = PseudoVASUB_VX_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #854 = PseudoVASUB_VX_MF2
  { 9, &RISCVDescs.OperandInfo[544] },  // Inst #853 = PseudoVASUB_VX_M8_MASK
  { 8, &RISCVDescs.OperandInfo[536] },  // Inst #852 = PseudoVASUB_VX_M8
  { 9, &RISCVDescs.OperandInfo[527] },  // Inst #851 = PseudoVASUB_VX_M4_MASK
  { 8, &RISCVDescs.OperandInfo[519] },  // Inst #850 = PseudoVASUB_VX_M4
  { 9, &RISCVDescs.OperandInfo[510] },  // Inst #849 = PseudoVASUB_VX_M2_MASK
  { 8, &RISCVDescs.OperandInfo[502] },  // Inst #848 = PseudoVASUB_VX_M2
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #847 = PseudoVASUB_VX_M1_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #846 = PseudoVASUB_VX_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #845 = PseudoVASUB_VV_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #844 = PseudoVASUB_VV_MF8
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #843 = PseudoVASUB_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #842 = PseudoVASUB_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #841 = PseudoVASUB_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #840 = PseudoVASUB_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #839 = PseudoVASUB_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #838 = PseudoVASUB_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #837 = PseudoVASUB_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #836 = PseudoVASUB_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #835 = PseudoVASUB_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #834 = PseudoVASUB_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #833 = PseudoVASUB_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #832 = PseudoVASUB_VV_M1
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #831 = PseudoVASUBU_VX_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #830 = PseudoVASUBU_VX_MF8
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #829 = PseudoVASUBU_VX_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #828 = PseudoVASUBU_VX_MF4
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #827 = PseudoVASUBU_VX_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #826 = PseudoVASUBU_VX_MF2
  { 9, &RISCVDescs.OperandInfo[544] },  // Inst #825 = PseudoVASUBU_VX_M8_MASK
  { 8, &RISCVDescs.OperandInfo[536] },  // Inst #824 = PseudoVASUBU_VX_M8
  { 9, &RISCVDescs.OperandInfo[527] },  // Inst #823 = PseudoVASUBU_VX_M4_MASK
  { 8, &RISCVDescs.OperandInfo[519] },  // Inst #822 = PseudoVASUBU_VX_M4
  { 9, &RISCVDescs.OperandInfo[510] },  // Inst #821 = PseudoVASUBU_VX_M2_MASK
  { 8, &RISCVDescs.OperandInfo[502] },  // Inst #820 = PseudoVASUBU_VX_M2
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #819 = PseudoVASUBU_VX_M1_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #818 = PseudoVASUBU_VX_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #817 = PseudoVASUBU_VV_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #816 = PseudoVASUBU_VV_MF8
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #815 = PseudoVASUBU_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #814 = PseudoVASUBU_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #813 = PseudoVASUBU_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #812 = PseudoVASUBU_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #811 = PseudoVASUBU_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #810 = PseudoVASUBU_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #809 = PseudoVASUBU_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #808 = PseudoVASUBU_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #807 = PseudoVASUBU_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #806 = PseudoVASUBU_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #805 = PseudoVASUBU_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #804 = PseudoVASUBU_VV_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #803 = PseudoVAND_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #802 = PseudoVAND_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #801 = PseudoVAND_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #800 = PseudoVAND_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #799 = PseudoVAND_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #798 = PseudoVAND_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #797 = PseudoVAND_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #796 = PseudoVAND_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #795 = PseudoVAND_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #794 = PseudoVAND_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #793 = PseudoVAND_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #792 = PseudoVAND_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #791 = PseudoVAND_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #790 = PseudoVAND_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #789 = PseudoVAND_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #788 = PseudoVAND_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #787 = PseudoVAND_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #786 = PseudoVAND_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #785 = PseudoVAND_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #784 = PseudoVAND_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #783 = PseudoVAND_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #782 = PseudoVAND_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #781 = PseudoVAND_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #780 = PseudoVAND_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #779 = PseudoVAND_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #778 = PseudoVAND_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #777 = PseudoVAND_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #776 = PseudoVAND_VV_M1
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #775 = PseudoVAND_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #774 = PseudoVAND_VI_MF8
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #773 = PseudoVAND_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #772 = PseudoVAND_VI_MF4
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #771 = PseudoVAND_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #770 = PseudoVAND_VI_MF2
  { 8, &RISCVDescs.OperandInfo[689] },  // Inst #769 = PseudoVAND_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[682] },  // Inst #768 = PseudoVAND_VI_M8
  { 8, &RISCVDescs.OperandInfo[674] },  // Inst #767 = PseudoVAND_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[667] },  // Inst #766 = PseudoVAND_VI_M4
  { 8, &RISCVDescs.OperandInfo[659] },  // Inst #765 = PseudoVAND_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[652] },  // Inst #764 = PseudoVAND_VI_M2
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #763 = PseudoVAND_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #762 = PseudoVAND_VI_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #761 = PseudoVANDN_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #760 = PseudoVANDN_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #759 = PseudoVANDN_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #758 = PseudoVANDN_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #757 = PseudoVANDN_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #756 = PseudoVANDN_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #755 = PseudoVANDN_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #754 = PseudoVANDN_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #753 = PseudoVANDN_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #752 = PseudoVANDN_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #751 = PseudoVANDN_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #750 = PseudoVANDN_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #749 = PseudoVANDN_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #748 = PseudoVANDN_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #747 = PseudoVANDN_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #746 = PseudoVANDN_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #745 = PseudoVANDN_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #744 = PseudoVANDN_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #743 = PseudoVANDN_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #742 = PseudoVANDN_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #741 = PseudoVANDN_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #740 = PseudoVANDN_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #739 = PseudoVANDN_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #738 = PseudoVANDN_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #737 = PseudoVANDN_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #736 = PseudoVANDN_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #735 = PseudoVANDN_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #734 = PseudoVANDN_VV_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #733 = PseudoVAESZ_VS_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #732 = PseudoVAESZ_VS_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #731 = PseudoVAESZ_VS_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #730 = PseudoVAESZ_VS_M8_MF8
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #729 = PseudoVAESZ_VS_M8_MF4
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #728 = PseudoVAESZ_VS_M8_MF2
  { 6, &RISCVDescs.OperandInfo[865] },  // Inst #727 = PseudoVAESZ_VS_M8_M4
  { 6, &RISCVDescs.OperandInfo[859] },  // Inst #726 = PseudoVAESZ_VS_M8_M2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #725 = PseudoVAESZ_VS_M8_M1
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #724 = PseudoVAESZ_VS_M4_MF8
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #723 = PseudoVAESZ_VS_M4_MF4
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #722 = PseudoVAESZ_VS_M4_MF2
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #721 = PseudoVAESZ_VS_M4_M4
  { 6, &RISCVDescs.OperandInfo[841] },  // Inst #720 = PseudoVAESZ_VS_M4_M2
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #719 = PseudoVAESZ_VS_M4_M1
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #718 = PseudoVAESZ_VS_M2_MF8
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #717 = PseudoVAESZ_VS_M2_MF4
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #716 = PseudoVAESZ_VS_M2_MF2
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #715 = PseudoVAESZ_VS_M2_M2
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #714 = PseudoVAESZ_VS_M2_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #713 = PseudoVAESZ_VS_M1_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #712 = PseudoVAESZ_VS_M1_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #711 = PseudoVAESZ_VS_M1_MF2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #710 = PseudoVAESZ_VS_M1_M1
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #709 = PseudoVAESKF2_VI_MF2
  { 7, &RISCVDescs.OperandInfo[898] },  // Inst #708 = PseudoVAESKF2_VI_M8
  { 7, &RISCVDescs.OperandInfo[891] },  // Inst #707 = PseudoVAESKF2_VI_M4
  { 7, &RISCVDescs.OperandInfo[884] },  // Inst #706 = PseudoVAESKF2_VI_M2
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #705 = PseudoVAESKF2_VI_M1
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #704 = PseudoVAESKF1_VI_MF2
  { 7, &RISCVDescs.OperandInfo[898] },  // Inst #703 = PseudoVAESKF1_VI_M8
  { 7, &RISCVDescs.OperandInfo[891] },  // Inst #702 = PseudoVAESKF1_VI_M4
  { 7, &RISCVDescs.OperandInfo[884] },  // Inst #701 = PseudoVAESKF1_VI_M2
  { 7, &RISCVDescs.OperandInfo[877] },  // Inst #700 = PseudoVAESKF1_VI_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #699 = PseudoVAESEM_VV_MF2
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #698 = PseudoVAESEM_VV_M8
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #697 = PseudoVAESEM_VV_M4
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #696 = PseudoVAESEM_VV_M2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #695 = PseudoVAESEM_VV_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #694 = PseudoVAESEM_VS_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #693 = PseudoVAESEM_VS_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #692 = PseudoVAESEM_VS_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #691 = PseudoVAESEM_VS_M8_MF8
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #690 = PseudoVAESEM_VS_M8_MF4
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #689 = PseudoVAESEM_VS_M8_MF2
  { 6, &RISCVDescs.OperandInfo[865] },  // Inst #688 = PseudoVAESEM_VS_M8_M4
  { 6, &RISCVDescs.OperandInfo[859] },  // Inst #687 = PseudoVAESEM_VS_M8_M2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #686 = PseudoVAESEM_VS_M8_M1
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #685 = PseudoVAESEM_VS_M4_MF8
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #684 = PseudoVAESEM_VS_M4_MF4
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #683 = PseudoVAESEM_VS_M4_MF2
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #682 = PseudoVAESEM_VS_M4_M4
  { 6, &RISCVDescs.OperandInfo[841] },  // Inst #681 = PseudoVAESEM_VS_M4_M2
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #680 = PseudoVAESEM_VS_M4_M1
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #679 = PseudoVAESEM_VS_M2_MF8
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #678 = PseudoVAESEM_VS_M2_MF4
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #677 = PseudoVAESEM_VS_M2_MF2
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #676 = PseudoVAESEM_VS_M2_M2
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #675 = PseudoVAESEM_VS_M2_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #674 = PseudoVAESEM_VS_M1_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #673 = PseudoVAESEM_VS_M1_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #672 = PseudoVAESEM_VS_M1_MF2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #671 = PseudoVAESEM_VS_M1_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #670 = PseudoVAESEF_VV_MF2
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #669 = PseudoVAESEF_VV_M8
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #668 = PseudoVAESEF_VV_M4
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #667 = PseudoVAESEF_VV_M2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #666 = PseudoVAESEF_VV_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #665 = PseudoVAESEF_VS_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #664 = PseudoVAESEF_VS_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #663 = PseudoVAESEF_VS_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #662 = PseudoVAESEF_VS_M8_MF8
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #661 = PseudoVAESEF_VS_M8_MF4
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #660 = PseudoVAESEF_VS_M8_MF2
  { 6, &RISCVDescs.OperandInfo[865] },  // Inst #659 = PseudoVAESEF_VS_M8_M4
  { 6, &RISCVDescs.OperandInfo[859] },  // Inst #658 = PseudoVAESEF_VS_M8_M2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #657 = PseudoVAESEF_VS_M8_M1
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #656 = PseudoVAESEF_VS_M4_MF8
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #655 = PseudoVAESEF_VS_M4_MF4
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #654 = PseudoVAESEF_VS_M4_MF2
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #653 = PseudoVAESEF_VS_M4_M4
  { 6, &RISCVDescs.OperandInfo[841] },  // Inst #652 = PseudoVAESEF_VS_M4_M2
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #651 = PseudoVAESEF_VS_M4_M1
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #650 = PseudoVAESEF_VS_M2_MF8
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #649 = PseudoVAESEF_VS_M2_MF4
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #648 = PseudoVAESEF_VS_M2_MF2
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #647 = PseudoVAESEF_VS_M2_M2
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #646 = PseudoVAESEF_VS_M2_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #645 = PseudoVAESEF_VS_M1_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #644 = PseudoVAESEF_VS_M1_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #643 = PseudoVAESEF_VS_M1_MF2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #642 = PseudoVAESEF_VS_M1_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #641 = PseudoVAESDM_VV_MF2
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #640 = PseudoVAESDM_VV_M8
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #639 = PseudoVAESDM_VV_M4
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #638 = PseudoVAESDM_VV_M2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #637 = PseudoVAESDM_VV_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #636 = PseudoVAESDM_VS_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #635 = PseudoVAESDM_VS_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #634 = PseudoVAESDM_VS_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #633 = PseudoVAESDM_VS_M8_MF8
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #632 = PseudoVAESDM_VS_M8_MF4
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #631 = PseudoVAESDM_VS_M8_MF2
  { 6, &RISCVDescs.OperandInfo[865] },  // Inst #630 = PseudoVAESDM_VS_M8_M4
  { 6, &RISCVDescs.OperandInfo[859] },  // Inst #629 = PseudoVAESDM_VS_M8_M2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #628 = PseudoVAESDM_VS_M8_M1
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #627 = PseudoVAESDM_VS_M4_MF8
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #626 = PseudoVAESDM_VS_M4_MF4
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #625 = PseudoVAESDM_VS_M4_MF2
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #624 = PseudoVAESDM_VS_M4_M4
  { 6, &RISCVDescs.OperandInfo[841] },  // Inst #623 = PseudoVAESDM_VS_M4_M2
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #622 = PseudoVAESDM_VS_M4_M1
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #621 = PseudoVAESDM_VS_M2_MF8
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #620 = PseudoVAESDM_VS_M2_MF4
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #619 = PseudoVAESDM_VS_M2_MF2
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #618 = PseudoVAESDM_VS_M2_M2
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #617 = PseudoVAESDM_VS_M2_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #616 = PseudoVAESDM_VS_M1_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #615 = PseudoVAESDM_VS_M1_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #614 = PseudoVAESDM_VS_M1_MF2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #613 = PseudoVAESDM_VS_M1_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #612 = PseudoVAESDF_VV_MF2
  { 6, &RISCVDescs.OperandInfo[871] },  // Inst #611 = PseudoVAESDF_VV_M8
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #610 = PseudoVAESDF_VV_M4
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #609 = PseudoVAESDF_VV_M2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #608 = PseudoVAESDF_VV_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #607 = PseudoVAESDF_VS_MF2_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #606 = PseudoVAESDF_VS_MF2_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #605 = PseudoVAESDF_VS_MF2_MF2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #604 = PseudoVAESDF_VS_M8_MF8
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #603 = PseudoVAESDF_VS_M8_MF4
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #602 = PseudoVAESDF_VS_M8_MF2
  { 6, &RISCVDescs.OperandInfo[865] },  // Inst #601 = PseudoVAESDF_VS_M8_M4
  { 6, &RISCVDescs.OperandInfo[859] },  // Inst #600 = PseudoVAESDF_VS_M8_M2
  { 6, &RISCVDescs.OperandInfo[853] },  // Inst #599 = PseudoVAESDF_VS_M8_M1
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #598 = PseudoVAESDF_VS_M4_MF8
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #597 = PseudoVAESDF_VS_M4_MF4
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #596 = PseudoVAESDF_VS_M4_MF2
  { 6, &RISCVDescs.OperandInfo[847] },  // Inst #595 = PseudoVAESDF_VS_M4_M4
  { 6, &RISCVDescs.OperandInfo[841] },  // Inst #594 = PseudoVAESDF_VS_M4_M2
  { 6, &RISCVDescs.OperandInfo[835] },  // Inst #593 = PseudoVAESDF_VS_M4_M1
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #592 = PseudoVAESDF_VS_M2_MF8
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #591 = PseudoVAESDF_VS_M2_MF4
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #590 = PseudoVAESDF_VS_M2_MF2
  { 6, &RISCVDescs.OperandInfo[829] },  // Inst #589 = PseudoVAESDF_VS_M2_M2
  { 6, &RISCVDescs.OperandInfo[823] },  // Inst #588 = PseudoVAESDF_VS_M2_M1
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #587 = PseudoVAESDF_VS_M1_MF8
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #586 = PseudoVAESDF_VS_M1_MF4
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #585 = PseudoVAESDF_VS_M1_MF2
  { 6, &RISCVDescs.OperandInfo[817] },  // Inst #584 = PseudoVAESDF_VS_M1_M1
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #583 = PseudoVADD_VX_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #582 = PseudoVADD_VX_MF8
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #581 = PseudoVADD_VX_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #580 = PseudoVADD_VX_MF4
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #579 = PseudoVADD_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #578 = PseudoVADD_VX_MF2
  { 8, &RISCVDescs.OperandInfo[809] },  // Inst #577 = PseudoVADD_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[802] },  // Inst #576 = PseudoVADD_VX_M8
  { 8, &RISCVDescs.OperandInfo[794] },  // Inst #575 = PseudoVADD_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[787] },  // Inst #574 = PseudoVADD_VX_M4
  { 8, &RISCVDescs.OperandInfo[779] },  // Inst #573 = PseudoVADD_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[772] },  // Inst #572 = PseudoVADD_VX_M2
  { 8, &RISCVDescs.OperandInfo[764] },  // Inst #571 = PseudoVADD_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[757] },  // Inst #570 = PseudoVADD_VX_M1
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #569 = PseudoVADD_VV_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #568 = PseudoVADD_VV_MF8
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #567 = PseudoVADD_VV_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #566 = PseudoVADD_VV_MF4
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #565 = PseudoVADD_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #564 = PseudoVADD_VV_MF2
  { 8, &RISCVDescs.OperandInfo[749] },  // Inst #563 = PseudoVADD_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[742] },  // Inst #562 = PseudoVADD_VV_M8
  { 8, &RISCVDescs.OperandInfo[734] },  // Inst #561 = PseudoVADD_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[727] },  // Inst #560 = PseudoVADD_VV_M4
  { 8, &RISCVDescs.OperandInfo[719] },  // Inst #559 = PseudoVADD_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[712] },  // Inst #558 = PseudoVADD_VV_M2
  { 8, &RISCVDescs.OperandInfo[704] },  // Inst #557 = PseudoVADD_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[697] },  // Inst #556 = PseudoVADD_VV_M1
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #555 = PseudoVADD_VI_MF8_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #554 = PseudoVADD_VI_MF8
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #553 = PseudoVADD_VI_MF4_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #552 = PseudoVADD_VI_MF4
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #551 = PseudoVADD_VI_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #550 = PseudoVADD_VI_MF2
  { 8, &RISCVDescs.OperandInfo[689] },  // Inst #549 = PseudoVADD_VI_M8_MASK
  { 7, &RISCVDescs.OperandInfo[682] },  // Inst #548 = PseudoVADD_VI_M8
  { 8, &RISCVDescs.OperandInfo[674] },  // Inst #547 = PseudoVADD_VI_M4_MASK
  { 7, &RISCVDescs.OperandInfo[667] },  // Inst #546 = PseudoVADD_VI_M4
  { 8, &RISCVDescs.OperandInfo[659] },  // Inst #545 = PseudoVADD_VI_M2_MASK
  { 7, &RISCVDescs.OperandInfo[652] },  // Inst #544 = PseudoVADD_VI_M2
  { 8, &RISCVDescs.OperandInfo[644] },  // Inst #543 = PseudoVADD_VI_M1_MASK
  { 7, &RISCVDescs.OperandInfo[637] },  // Inst #542 = PseudoVADD_VI_M1
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #541 = PseudoVADC_VXM_MF8
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #540 = PseudoVADC_VXM_MF4
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #539 = PseudoVADC_VXM_MF2
  { 7, &RISCVDescs.OperandInfo[630] },  // Inst #538 = PseudoVADC_VXM_M8
  { 7, &RISCVDescs.OperandInfo[623] },  // Inst #537 = PseudoVADC_VXM_M4
  { 7, &RISCVDescs.OperandInfo[616] },  // Inst #536 = PseudoVADC_VXM_M2
  { 7, &RISCVDescs.OperandInfo[609] },  // Inst #535 = PseudoVADC_VXM_M1
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #534 = PseudoVADC_VVM_MF8
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #533 = PseudoVADC_VVM_MF4
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #532 = PseudoVADC_VVM_MF2
  { 7, &RISCVDescs.OperandInfo[602] },  // Inst #531 = PseudoVADC_VVM_M8
  { 7, &RISCVDescs.OperandInfo[595] },  // Inst #530 = PseudoVADC_VVM_M4
  { 7, &RISCVDescs.OperandInfo[588] },  // Inst #529 = PseudoVADC_VVM_M2
  { 7, &RISCVDescs.OperandInfo[581] },  // Inst #528 = PseudoVADC_VVM_M1
  { 7, &RISCVDescs.OperandInfo[553] },  // Inst #527 = PseudoVADC_VIM_MF8
  { 7, &RISCVDescs.OperandInfo[553] },  // Inst #526 = PseudoVADC_VIM_MF4
  { 7, &RISCVDescs.OperandInfo[553] },  // Inst #525 = PseudoVADC_VIM_MF2
  { 7, &RISCVDescs.OperandInfo[574] },  // Inst #524 = PseudoVADC_VIM_M8
  { 7, &RISCVDescs.OperandInfo[567] },  // Inst #523 = PseudoVADC_VIM_M4
  { 7, &RISCVDescs.OperandInfo[560] },  // Inst #522 = PseudoVADC_VIM_M2
  { 7, &RISCVDescs.OperandInfo[553] },  // Inst #521 = PseudoVADC_VIM_M1
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #520 = PseudoVAADD_VX_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #519 = PseudoVAADD_VX_MF8
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #518 = PseudoVAADD_VX_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #517 = PseudoVAADD_VX_MF4
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #516 = PseudoVAADD_VX_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #515 = PseudoVAADD_VX_MF2
  { 9, &RISCVDescs.OperandInfo[544] },  // Inst #514 = PseudoVAADD_VX_M8_MASK
  { 8, &RISCVDescs.OperandInfo[536] },  // Inst #513 = PseudoVAADD_VX_M8
  { 9, &RISCVDescs.OperandInfo[527] },  // Inst #512 = PseudoVAADD_VX_M4_MASK
  { 8, &RISCVDescs.OperandInfo[519] },  // Inst #511 = PseudoVAADD_VX_M4
  { 9, &RISCVDescs.OperandInfo[510] },  // Inst #510 = PseudoVAADD_VX_M2_MASK
  { 8, &RISCVDescs.OperandInfo[502] },  // Inst #509 = PseudoVAADD_VX_M2
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #508 = PseudoVAADD_VX_M1_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #507 = PseudoVAADD_VX_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #506 = PseudoVAADD_VV_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #505 = PseudoVAADD_VV_MF8
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #504 = PseudoVAADD_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #503 = PseudoVAADD_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #502 = PseudoVAADD_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #501 = PseudoVAADD_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #500 = PseudoVAADD_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #499 = PseudoVAADD_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #498 = PseudoVAADD_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #497 = PseudoVAADD_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #496 = PseudoVAADD_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #495 = PseudoVAADD_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #494 = PseudoVAADD_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #493 = PseudoVAADD_VV_M1
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #492 = PseudoVAADDU_VX_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #491 = PseudoVAADDU_VX_MF8
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #490 = PseudoVAADDU_VX_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #489 = PseudoVAADDU_VX_MF4
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #488 = PseudoVAADDU_VX_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #487 = PseudoVAADDU_VX_MF2
  { 9, &RISCVDescs.OperandInfo[544] },  // Inst #486 = PseudoVAADDU_VX_M8_MASK
  { 8, &RISCVDescs.OperandInfo[536] },  // Inst #485 = PseudoVAADDU_VX_M8
  { 9, &RISCVDescs.OperandInfo[527] },  // Inst #484 = PseudoVAADDU_VX_M4_MASK
  { 8, &RISCVDescs.OperandInfo[519] },  // Inst #483 = PseudoVAADDU_VX_M4
  { 9, &RISCVDescs.OperandInfo[510] },  // Inst #482 = PseudoVAADDU_VX_M2_MASK
  { 8, &RISCVDescs.OperandInfo[502] },  // Inst #481 = PseudoVAADDU_VX_M2
  { 9, &RISCVDescs.OperandInfo[493] },  // Inst #480 = PseudoVAADDU_VX_M1_MASK
  { 8, &RISCVDescs.OperandInfo[485] },  // Inst #479 = PseudoVAADDU_VX_M1
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #478 = PseudoVAADDU_VV_MF8_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #477 = PseudoVAADDU_VV_MF8
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #476 = PseudoVAADDU_VV_MF4_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #475 = PseudoVAADDU_VV_MF4
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #474 = PseudoVAADDU_VV_MF2_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #473 = PseudoVAADDU_VV_MF2
  { 9, &RISCVDescs.OperandInfo[476] },  // Inst #472 = PseudoVAADDU_VV_M8_MASK
  { 8, &RISCVDescs.OperandInfo[468] },  // Inst #471 = PseudoVAADDU_VV_M8
  { 9, &RISCVDescs.OperandInfo[459] },  // Inst #470 = PseudoVAADDU_VV_M4_MASK
  { 8, &RISCVDescs.OperandInfo[451] },  // Inst #469 = PseudoVAADDU_VV_M4
  { 9, &RISCVDescs.OperandInfo[442] },  // Inst #468 = PseudoVAADDU_VV_M2_MASK
  { 8, &RISCVDescs.OperandInfo[434] },  // Inst #467 = PseudoVAADDU_VV_M2
  { 9, &RISCVDescs.OperandInfo[425] },  // Inst #466 = PseudoVAADDU_VV_M1_MASK
  { 8, &RISCVDescs.OperandInfo[417] },  // Inst #465 = PseudoVAADDU_VV_M1
  { 4, &RISCVDescs.OperandInfo[413] },  // Inst #464 = PseudoTLSDESCCall
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #463 = PseudoTHVdotVMAQA_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #462 = PseudoTHVdotVMAQA_VX_MF2
  { 8, &RISCVDescs.OperandInfo[405] },  // Inst #461 = PseudoTHVdotVMAQA_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[398] },  // Inst #460 = PseudoTHVdotVMAQA_VX_M8
  { 8, &RISCVDescs.OperandInfo[390] },  // Inst #459 = PseudoTHVdotVMAQA_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[383] },  // Inst #458 = PseudoTHVdotVMAQA_VX_M4
  { 8, &RISCVDescs.OperandInfo[375] },  // Inst #457 = PseudoTHVdotVMAQA_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[368] },  // Inst #456 = PseudoTHVdotVMAQA_VX_M2
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #455 = PseudoTHVdotVMAQA_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #454 = PseudoTHVdotVMAQA_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #453 = PseudoTHVdotVMAQA_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #452 = PseudoTHVdotVMAQA_VV_MF2
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #451 = PseudoTHVdotVMAQA_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #450 = PseudoTHVdotVMAQA_VV_M8
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #449 = PseudoTHVdotVMAQA_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #448 = PseudoTHVdotVMAQA_VV_M4
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #447 = PseudoTHVdotVMAQA_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #446 = PseudoTHVdotVMAQA_VV_M2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #445 = PseudoTHVdotVMAQA_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #444 = PseudoTHVdotVMAQA_VV_M1
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #443 = PseudoTHVdotVMAQAU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #442 = PseudoTHVdotVMAQAU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[405] },  // Inst #441 = PseudoTHVdotVMAQAU_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[398] },  // Inst #440 = PseudoTHVdotVMAQAU_VX_M8
  { 8, &RISCVDescs.OperandInfo[390] },  // Inst #439 = PseudoTHVdotVMAQAU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[383] },  // Inst #438 = PseudoTHVdotVMAQAU_VX_M4
  { 8, &RISCVDescs.OperandInfo[375] },  // Inst #437 = PseudoTHVdotVMAQAU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[368] },  // Inst #436 = PseudoTHVdotVMAQAU_VX_M2
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #435 = PseudoTHVdotVMAQAU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #434 = PseudoTHVdotVMAQAU_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #433 = PseudoTHVdotVMAQAU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #432 = PseudoTHVdotVMAQAU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #431 = PseudoTHVdotVMAQAU_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #430 = PseudoTHVdotVMAQAU_VV_M8
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #429 = PseudoTHVdotVMAQAU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #428 = PseudoTHVdotVMAQAU_VV_M4
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #427 = PseudoTHVdotVMAQAU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #426 = PseudoTHVdotVMAQAU_VV_M2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #425 = PseudoTHVdotVMAQAU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #424 = PseudoTHVdotVMAQAU_VV_M1
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #423 = PseudoTHVdotVMAQAUS_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #422 = PseudoTHVdotVMAQAUS_VX_MF2
  { 8, &RISCVDescs.OperandInfo[405] },  // Inst #421 = PseudoTHVdotVMAQAUS_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[398] },  // Inst #420 = PseudoTHVdotVMAQAUS_VX_M8
  { 8, &RISCVDescs.OperandInfo[390] },  // Inst #419 = PseudoTHVdotVMAQAUS_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[383] },  // Inst #418 = PseudoTHVdotVMAQAUS_VX_M4
  { 8, &RISCVDescs.OperandInfo[375] },  // Inst #417 = PseudoTHVdotVMAQAUS_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[368] },  // Inst #416 = PseudoTHVdotVMAQAUS_VX_M2
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #415 = PseudoTHVdotVMAQAUS_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #414 = PseudoTHVdotVMAQAUS_VX_M1
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #413 = PseudoTHVdotVMAQASU_VX_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #412 = PseudoTHVdotVMAQASU_VX_MF2
  { 8, &RISCVDescs.OperandInfo[405] },  // Inst #411 = PseudoTHVdotVMAQASU_VX_M8_MASK
  { 7, &RISCVDescs.OperandInfo[398] },  // Inst #410 = PseudoTHVdotVMAQASU_VX_M8
  { 8, &RISCVDescs.OperandInfo[390] },  // Inst #409 = PseudoTHVdotVMAQASU_VX_M4_MASK
  { 7, &RISCVDescs.OperandInfo[383] },  // Inst #408 = PseudoTHVdotVMAQASU_VX_M4
  { 8, &RISCVDescs.OperandInfo[375] },  // Inst #407 = PseudoTHVdotVMAQASU_VX_M2_MASK
  { 7, &RISCVDescs.OperandInfo[368] },  // Inst #406 = PseudoTHVdotVMAQASU_VX_M2
  { 8, &RISCVDescs.OperandInfo[360] },  // Inst #405 = PseudoTHVdotVMAQASU_VX_M1_MASK
  { 7, &RISCVDescs.OperandInfo[353] },  // Inst #404 = PseudoTHVdotVMAQASU_VX_M1
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #403 = PseudoTHVdotVMAQASU_VV_MF2_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #402 = PseudoTHVdotVMAQASU_VV_MF2
  { 8, &RISCVDescs.OperandInfo[345] },  // Inst #401 = PseudoTHVdotVMAQASU_VV_M8_MASK
  { 7, &RISCVDescs.OperandInfo[338] },  // Inst #400 = PseudoTHVdotVMAQASU_VV_M8
  { 8, &RISCVDescs.OperandInfo[330] },  // Inst #399 = PseudoTHVdotVMAQASU_VV_M4_MASK
  { 7, &RISCVDescs.OperandInfo[323] },  // Inst #398 = PseudoTHVdotVMAQASU_VV_M4
  { 8, &RISCVDescs.OperandInfo[315] },  // Inst #397 = PseudoTHVdotVMAQASU_VV_M2_MASK
  { 7, &RISCVDescs.OperandInfo[308] },  // Inst #396 = PseudoTHVdotVMAQASU_VV_M2
  { 8, &RISCVDescs.OperandInfo[300] },  // Inst #395 = PseudoTHVdotVMAQASU_VV_M1_MASK
  { 7, &RISCVDescs.OperandInfo[293] },  // Inst #394 = PseudoTHVdotVMAQASU_VV_M1
  { 1, &RISCVDescs.OperandInfo[292] },  // Inst #393 = PseudoTAILIndirect
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #392 = PseudoTAIL
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #391 = PseudoSW
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #390 = PseudoSH
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #389 = PseudoSEXT_H
  { 2, &RISCVDescs.OperandInfo[290] },  // Inst #388 = PseudoSEXT_B
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #387 = PseudoSD
  { 3, &RISCVDescs.OperandInfo[287] },  // Inst #386 = PseudoSB
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #385 = PseudoReadVLENB
  { 1, &RISCVDescs.OperandInfo[286] },  // Inst #384 = PseudoReadVL
  { 1, &RISCVDescs.OperandInfo[285] },  // Inst #383 = PseudoRVVInitUndefM8
  { 1, &RISCVDescs.OperandInfo[284] },  // Inst #382 = PseudoRVVInitUndefM4
  { 1, &RISCVDescs.OperandInfo[283] },  // Inst #381 = PseudoRVVInitUndefM2
  { 1, &RISCVDescs.OperandInfo[282] },  // Inst #380 = PseudoRVVInitUndefM1
  { 3, &RISCVDescs.OperandInfo[279] },  // Inst #379 = PseudoRV32ZdinxSD
  { 3, &RISCVDescs.OperandInfo[276] },  // Inst #378 = PseudoRV32ZdinxLD
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #377 = PseudoRET
  { 3, &RISCVDescs.OperandInfo[273] },  // Inst #376 = PseudoQuietFLT_S_INX
  { 3, &RISCVDescs.OperandInfo[270] },  // Inst #375 = PseudoQuietFLT_S
  { 3, &RISCVDescs.OperandInfo[267] },  // Inst #374 = PseudoQuietFLT_H_INX
  { 3, &RISCVDescs.OperandInfo[264] },  // Inst #373 = PseudoQuietFLT_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #372 = PseudoQuietFLT_D_INX
  { 3, &RISCVDescs.OperandInfo[258] },  // Inst #371 = PseudoQuietFLT_D_IN32X
  { 3, &RISCVDescs.OperandInfo[255] },  // Inst #370 = PseudoQuietFLT_D
  { 3, &RISCVDescs.OperandInfo[273] },  // Inst #369 = PseudoQuietFLE_S_INX
  { 3, &RISCVDescs.OperandInfo[270] },  // Inst #368 = PseudoQuietFLE_S
  { 3, &RISCVDescs.OperandInfo[267] },  // Inst #367 = PseudoQuietFLE_H_INX
  { 3, &RISCVDescs.OperandInfo[264] },  // Inst #366 = PseudoQuietFLE_H
  { 3, &RISCVDescs.OperandInfo[261] },  // Inst #365 = PseudoQuietFLE_D_INX
  { 3, &RISCVDescs.OperandInfo[258] },  // Inst #364 = PseudoQuietFLE_D_IN32X
  { 3, &RISCVDescs.OperandInfo[255] },  // Inst #363 = PseudoQuietFLE_D
  { 2, &RISCVDescs.OperandInfo[253] },  // Inst #362 = PseudoMovImm
  { 7, &RISCVDescs.OperandInfo[246] },  // Inst #361 = PseudoMaskedCmpXchg32
  { 6, &RISCVDescs.OperandInfo[189] },  // Inst #360 = PseudoMaskedAtomicSwap32
  { 7, &RISCVDescs.OperandInfo[239] },  // Inst #359 = PseudoMaskedAtomicLoadUMin32
  { 7, &RISCVDescs.OperandInfo[239] },  // Inst #358 = PseudoMaskedAtomicLoadUMax32
  { 6, &RISCVDescs.OperandInfo[189] },  // Inst #357 = PseudoMaskedAtomicLoadSub32
  { 6, &RISCVDescs.OperandInfo[189] },  // Inst #356 = PseudoMaskedAtomicLoadNand32
  { 8, &RISCVDescs.OperandInfo[231] },  // Inst #355 = PseudoMaskedAtomicLoadMin32
  { 8, &RISCVDescs.OperandInfo[231] },  // Inst #354 = PseudoMaskedAtomicLoadMax32
  { 6, &RISCVDescs.OperandInfo[189] },  // Inst #353 = PseudoMaskedAtomicLoadAdd32
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #352 = PseudoLongBNE
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #351 = PseudoLongBLTU
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #350 = PseudoLongBLT
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #349 = PseudoLongBGEU
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #348 = PseudoLongBGE
  { 3, &RISCVDescs.OperandInfo[228] },  // Inst #347 = PseudoLongBEQ
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #346 = PseudoLWU
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #345 = PseudoLW
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #344 = PseudoLLAImm
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #343 = PseudoLLA
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #342 = PseudoLI
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #341 = PseudoLHU
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #340 = PseudoLH
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #339 = PseudoLGA
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #338 = PseudoLD
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #337 = PseudoLBU
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #336 = PseudoLB
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #335 = PseudoLA_TLS_IE
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #334 = PseudoLA_TLS_GD
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #333 = PseudoLA_TLSDESC
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #332 = PseudoLAImm
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #331 = PseudoLA
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #330 = PseudoJump
  { 3, &RISCVDescs.OperandInfo[201] },  // Inst #329 = PseudoFSW
  { 3, &RISCVDescs.OperandInfo[198] },  // Inst #328 = PseudoFSH
  { 3, &RISCVDescs.OperandInfo[195] },  // Inst #327 = PseudoFSD
  { 4, &RISCVDescs.OperandInfo[224] },  // Inst #326 = PseudoFROUND_S_INX
  { 4, &RISCVDescs.OperandInfo[220] },  // Inst #325 = PseudoFROUND_S
  { 4, &RISCVDescs.OperandInfo[216] },  // Inst #324 = PseudoFROUND_H_INX
  { 4, &RISCVDescs.OperandInfo[212] },  // Inst #323 = PseudoFROUND_H
  { 4, &RISCVDescs.OperandInfo[148] },  // Inst #322 = PseudoFROUND_D_INX
  { 4, &RISCVDescs.OperandInfo[208] },  // Inst #321 = PseudoFROUND_D_IN32X
  { 4, &RISCVDescs.OperandInfo[204] },  // Inst #320 = PseudoFROUND_D
  { 3, &RISCVDescs.OperandInfo[201] },  // Inst #319 = PseudoFLW
  { 3, &RISCVDescs.OperandInfo[198] },  // Inst #318 = PseudoFLH
  { 3, &RISCVDescs.OperandInfo[195] },  // Inst #317 = PseudoFLD
  { 6, &RISCVDescs.OperandInfo[189] },  // Inst #316 = PseudoCmpXchg64
  { 6, &RISCVDescs.OperandInfo[189] },  // Inst #315 = PseudoCmpXchg32
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #314 = PseudoCCXORI
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #313 = PseudoCCXOR
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #312 = PseudoCCXNOR
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #311 = PseudoCCSUBW
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #310 = PseudoCCSUB
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #309 = PseudoCCSRLW
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #308 = PseudoCCSRLIW
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #307 = PseudoCCSRLI
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #306 = PseudoCCSRL
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #305 = PseudoCCSRAW
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #304 = PseudoCCSRAIW
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #303 = PseudoCCSRAI
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #302 = PseudoCCSRA
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #301 = PseudoCCSLLW
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #300 = PseudoCCSLLIW
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #299 = PseudoCCSLLI
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #298 = PseudoCCSLL
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #297 = PseudoCCORN
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #296 = PseudoCCORI
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #295 = PseudoCCOR
  { 6, &RISCVDescs.OperandInfo[183] },  // Inst #294 = PseudoCCMOVGPRNoX0
  { 6, &RISCVDescs.OperandInfo[177] },  // Inst #293 = PseudoCCMOVGPR
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #292 = PseudoCCANDN
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #291 = PseudoCCANDI
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #290 = PseudoCCAND
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #289 = PseudoCCADDW
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #288 = PseudoCCADDIW
  { 7, &RISCVDescs.OperandInfo[170] },  // Inst #287 = PseudoCCADDI
  { 7, &RISCVDescs.OperandInfo[163] },  // Inst #286 = PseudoCCADD
  { 2, &RISCVDescs.OperandInfo[161] },  // Inst #285 = PseudoCALLReg
  { 1, &RISCVDescs.OperandInfo[160] },  // Inst #284 = PseudoCALLIndirect
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #283 = PseudoCALL
  { 2, &RISCVDescs.OperandInfo[158] },  // Inst #282 = PseudoBRIND
  { 1, &RISCVDescs.OperandInfo[157] },  // Inst #281 = PseudoBR
  { 5, &RISCVDescs.OperandInfo[152] },  // Inst #280 = PseudoAtomicLoadNand64
  { 5, &RISCVDescs.OperandInfo[152] },  // Inst #279 = PseudoAtomicLoadNand32
  { 4, &RISCVDescs.OperandInfo[148] },  // Inst #278 = PseudoAddTPRel
  { 2, &RISCVDescs.OperandInfo[146] },  // Inst #277 = KCFI_CHECK
  { 2, &RISCVDescs.OperandInfo[146] },  // Inst #276 = HWASAN_CHECK_MEMACCESS_SHORTGRANULES
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #275 = G_FCLASS
  { 3, &RISCVDescs.OperandInfo[143] },  // Inst #274 = BuildPairF64Pseudo_INX
  { 3, &RISCVDescs.OperandInfo[140] },  // Inst #273 = BuildPairF64Pseudo
  { 2, &RISCVDescs.OperandInfo[21] },  // Inst #272 = ADJCALLSTACKUP
  { 2, &RISCVDescs.OperandInfo[21] },  // Inst #271 = ADJCALLSTACKDOWN
  { 4, &RISCVDescs.OperandInfo[136] },  // Inst #270 = G_UBFX
  { 4, &RISCVDescs.OperandInfo[136] },  // Inst #269 = G_SBFX
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #268 = G_VECREDUCE_UMIN
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #267 = G_VECREDUCE_UMAX
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #266 = G_VECREDUCE_SMIN
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #265 = G_VECREDUCE_SMAX
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #264 = G_VECREDUCE_XOR
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #263 = G_VECREDUCE_OR
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #262 = G_VECREDUCE_AND
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #261 = G_VECREDUCE_MUL
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #260 = G_VECREDUCE_ADD
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #259 = G_VECREDUCE_FMINIMUM
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #258 = G_VECREDUCE_FMAXIMUM
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #257 = G_VECREDUCE_FMIN
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #256 = G_VECREDUCE_FMAX
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #255 = G_VECREDUCE_FMUL
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #254 = G_VECREDUCE_FADD
  { 3, &RISCVDescs.OperandInfo[123] },  // Inst #253 = G_VECREDUCE_SEQ_FMUL
  { 3, &RISCVDescs.OperandInfo[123] },  // Inst #252 = G_VECREDUCE_SEQ_FADD
  { 3, &RISCVDescs.OperandInfo[53] },  // Inst #251 = G_BZERO
  { 4, &RISCVDescs.OperandInfo[132] },  // Inst #250 = G_MEMSET
  { 4, &RISCVDescs.OperandInfo[132] },  // Inst #249 = G_MEMMOVE
  { 3, &RISCVDescs.OperandInfo[123] },  // Inst #248 = G_MEMCPY_INLINE
  { 4, &RISCVDescs.OperandInfo[132] },  // Inst #247 = G_MEMCPY
  { 2, &RISCVDescs.OperandInfo[130] },  // Inst #246 = G_WRITE_REGISTER
  { 2, &RISCVDescs.OperandInfo[51] },  // Inst #245 = G_READ_REGISTER
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #244 = G_STRICT_FLDEXP
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #243 = G_STRICT_FSQRT
  { 4, &RISCVDescs.OperandInfo[46] },  // Inst #242 = G_STRICT_FMA
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #241 = G_STRICT_FREM
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #240 = G_STRICT_FDIV
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #239 = G_STRICT_FMUL
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #238 = G_STRICT_FSUB
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #237 = G_STRICT_FADD
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #236 = G_STACKRESTORE
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #235 = G_STACKSAVE
  { 3, &RISCVDescs.OperandInfo[64] },  // Inst #234 = G_DYN_STACKALLOC
  { 2, &RISCVDescs.OperandInfo[51] },  // Inst #233 = G_JUMP_TABLE
  { 2, &RISCVDescs.OperandInfo[51] },  // Inst #232 = G_BLOCK_ADDR
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #231 = G_ADDRSPACE_CAST
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #230 = G_FNEARBYINT
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #229 = G_FRINT
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #228 = G_FFLOOR
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #227 = G_FSQRT
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #226 = G_FSIN
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #225 = G_FCOS
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #224 = G_FCEIL
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #223 = G_BITREVERSE
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #222 = G_BSWAP
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #221 = G_CTPOP
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #220 = G_CTLZ_ZERO_UNDEF
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #219 = G_CTLZ
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #218 = G_CTTZ_ZERO_UNDEF
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #217 = G_CTTZ
  { 4, &RISCVDescs.OperandInfo[126] },  // Inst #216 = G_SHUFFLE_VECTOR
  { 3, &RISCVDescs.OperandInfo[123] },  // Inst #215 = G_EXTRACT_VECTOR_ELT
  { 4, &RISCVDescs.OperandInfo[119] },  // Inst #214 = G_INSERT_VECTOR_ELT
  { 3, &RISCVDescs.OperandInfo[116] },  // Inst #213 = G_BRJT
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #212 = G_BR
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #211 = G_LLROUND
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #210 = G_LROUND
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #209 = G_ABS
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #208 = G_UMAX
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #207 = G_UMIN
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #206 = G_SMAX
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #205 = G_SMIN
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #204 = G_PTRMASK
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #203 = G_PTR_ADD
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #202 = G_RESET_FPMODE
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #201 = G_SET_FPMODE
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #200 = G_GET_FPMODE
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #199 = G_RESET_FPENV
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #198 = G_SET_FPENV
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #197 = G_GET_FPENV
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #196 = G_FMAXIMUM
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #195 = G_FMINIMUM
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #194 = G_FMAXNUM_IEEE
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #193 = G_FMINNUM_IEEE
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #192 = G_FMAXNUM
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #191 = G_FMINNUM
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #190 = G_FCANONICALIZE
  { 3, &RISCVDescs.OperandInfo[93] },  // Inst #189 = G_IS_FPCLASS
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #188 = G_FCOPYSIGN
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #187 = G_FABS
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #186 = G_UITOFP
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #185 = G_SITOFP
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #184 = G_FPTOUI
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #183 = G_FPTOSI
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #182 = G_FPTRUNC
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #181 = G_FPEXT
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #180 = G_FNEG
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #179 = G_FFREXP
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #178 = G_FLDEXP
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #177 = G_FLOG10
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #176 = G_FLOG2
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #175 = G_FLOG
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #174 = G_FEXP10
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #173 = G_FEXP2
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #172 = G_FEXP
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #171 = G_FPOWI
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #170 = G_FPOW
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #169 = G_FREM
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #168 = G_FDIV
  { 4, &RISCVDescs.OperandInfo[46] },  // Inst #167 = G_FMAD
  { 4, &RISCVDescs.OperandInfo[46] },  // Inst #166 = G_FMA
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #165 = G_FMUL
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #164 = G_FSUB
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #163 = G_FADD
  { 4, &RISCVDescs.OperandInfo[112] },  // Inst #162 = G_UDIVFIXSAT
  { 4, &RISCVDescs.OperandInfo[112] },  // Inst #161 = G_SDIVFIXSAT
  { 4, &RISCVDescs.OperandInfo[112] },  // Inst #160 = G_UDIVFIX
  { 4, &RISCVDescs.OperandInfo[112] },  // Inst #159 = G_SDIVFIX
  { 4, &RISCVDescs.OperandInfo[112] },  // Inst #158 = G_UMULFIXSAT
  { 4, &RISCVDescs.OperandInfo[112] },  // Inst #157 = G_SMULFIXSAT
  { 4, &RISCVDescs.OperandInfo[112] },  // Inst #156 = G_UMULFIX
  { 4, &RISCVDescs.OperandInfo[112] },  // Inst #155 = G_SMULFIX
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #154 = G_SSHLSAT
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #153 = G_USHLSAT
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #152 = G_SSUBSAT
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #151 = G_USUBSAT
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #150 = G_SADDSAT
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #149 = G_UADDSAT
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #148 = G_SMULH
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #147 = G_UMULH
  { 4, &RISCVDescs.OperandInfo[82] },  // Inst #146 = G_SMULO
  { 4, &RISCVDescs.OperandInfo[82] },  // Inst #145 = G_UMULO
  { 5, &RISCVDescs.OperandInfo[107] },  // Inst #144 = G_SSUBE
  { 4, &RISCVDescs.OperandInfo[82] },  // Inst #143 = G_SSUBO
  { 5, &RISCVDescs.OperandInfo[107] },  // Inst #142 = G_SADDE
  { 4, &RISCVDescs.OperandInfo[82] },  // Inst #141 = G_SADDO
  { 5, &RISCVDescs.OperandInfo[107] },  // Inst #140 = G_USUBE
  { 4, &RISCVDescs.OperandInfo[82] },  // Inst #139 = G_USUBO
  { 5, &RISCVDescs.OperandInfo[107] },  // Inst #138 = G_UADDE
  { 4, &RISCVDescs.OperandInfo[82] },  // Inst #137 = G_UADDO
  { 4, &RISCVDescs.OperandInfo[82] },  // Inst #136 = G_SELECT
  { 4, &RISCVDescs.OperandInfo[103] },  // Inst #135 = G_FCMP
  { 4, &RISCVDescs.OperandInfo[103] },  // Inst #134 = G_ICMP
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #133 = G_ROTL
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #132 = G_ROTR
  { 4, &RISCVDescs.OperandInfo[99] },  // Inst #131 = G_FSHR
  { 4, &RISCVDescs.OperandInfo[99] },  // Inst #130 = G_FSHL
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #129 = G_ASHR
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #128 = G_LSHR
  { 3, &RISCVDescs.OperandInfo[96] },  // Inst #127 = G_SHL
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #126 = G_ZEXT
  { 3, &RISCVDescs.OperandInfo[40] },  // Inst #125 = G_SEXT_INREG
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #124 = G_SEXT
  { 3, &RISCVDescs.OperandInfo[93] },  // Inst #123 = G_VAARG
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #122 = G_VASTART
  { 2, &RISCVDescs.OperandInfo[51] },  // Inst #121 = G_FCONSTANT
  { 2, &RISCVDescs.OperandInfo[51] },  // Inst #120 = G_CONSTANT
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #119 = G_TRUNC
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #118 = G_ANYEXT
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #117 = G_INTRINSIC_CONVERGENT_W_SIDE_EFFECTS
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #116 = G_INTRINSIC_CONVERGENT
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #115 = G_INTRINSIC_W_SIDE_EFFECTS
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #114 = G_INTRINSIC
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #113 = G_INVOKE_REGION_START
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #112 = G_BRINDIRECT
  { 2, &RISCVDescs.OperandInfo[51] },  // Inst #111 = G_BRCOND
  { 4, &RISCVDescs.OperandInfo[89] },  // Inst #110 = G_PREFETCH
  { 2, &RISCVDescs.OperandInfo[21] },  // Inst #109 = G_FENCE
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #108 = G_ATOMICRMW_UDEC_WRAP
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #107 = G_ATOMICRMW_UINC_WRAP
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #106 = G_ATOMICRMW_FMIN
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #105 = G_ATOMICRMW_FMAX
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #104 = G_ATOMICRMW_FSUB
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #103 = G_ATOMICRMW_FADD
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #102 = G_ATOMICRMW_UMIN
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #101 = G_ATOMICRMW_UMAX
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #100 = G_ATOMICRMW_MIN
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #99 = G_ATOMICRMW_MAX
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #98 = G_ATOMICRMW_XOR
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #97 = G_ATOMICRMW_OR
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #96 = G_ATOMICRMW_NAND
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #95 = G_ATOMICRMW_AND
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #94 = G_ATOMICRMW_SUB
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #93 = G_ATOMICRMW_ADD
  { 3, &RISCVDescs.OperandInfo[86] },  // Inst #92 = G_ATOMICRMW_XCHG
  { 4, &RISCVDescs.OperandInfo[82] },  // Inst #91 = G_ATOMIC_CMPXCHG
  { 5, &RISCVDescs.OperandInfo[77] },  // Inst #90 = G_ATOMIC_CMPXCHG_WITH_SUCCESS
  { 5, &RISCVDescs.OperandInfo[72] },  // Inst #89 = G_INDEXED_STORE
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #88 = G_STORE
  { 5, &RISCVDescs.OperandInfo[67] },  // Inst #87 = G_INDEXED_ZEXTLOAD
  { 5, &RISCVDescs.OperandInfo[67] },  // Inst #86 = G_INDEXED_SEXTLOAD
  { 5, &RISCVDescs.OperandInfo[67] },  // Inst #85 = G_INDEXED_LOAD
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #84 = G_ZEXTLOAD
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #83 = G_SEXTLOAD
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #82 = G_LOAD
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #81 = G_READCYCLECOUNTER
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #80 = G_INTRINSIC_ROUNDEVEN
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #79 = G_INTRINSIC_LRINT
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #78 = G_INTRINSIC_ROUND
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #77 = G_INTRINSIC_TRUNC
  { 3, &RISCVDescs.OperandInfo[64] },  // Inst #76 = G_INTRINSIC_FPTRUNC_ROUND
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #75 = G_CONSTANT_FOLD_BARRIER
  { 2, &RISCVDescs.OperandInfo[62] },  // Inst #74 = G_FREEZE
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #73 = G_BITCAST
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #72 = G_INTTOPTR
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #71 = G_PTRTOINT
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #70 = G_CONCAT_VECTORS
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #69 = G_BUILD_VECTOR_TRUNC
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #68 = G_BUILD_VECTOR
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #67 = G_MERGE_VALUES
  { 4, &RISCVDescs.OperandInfo[58] },  // Inst #66 = G_INSERT
  { 2, &RISCVDescs.OperandInfo[56] },  // Inst #65 = G_UNMERGE_VALUES
  { 3, &RISCVDescs.OperandInfo[53] },  // Inst #64 = G_EXTRACT
  { 2, &RISCVDescs.OperandInfo[51] },  // Inst #63 = G_CONSTANT_POOL
  { 2, &RISCVDescs.OperandInfo[51] },  // Inst #62 = G_GLOBAL_VALUE
  { 2, &RISCVDescs.OperandInfo[51] },  // Inst #61 = G_FRAME_INDEX
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #60 = G_PHI
  { 1, &RISCVDescs.OperandInfo[50] },  // Inst #59 = G_IMPLICIT_DEF
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #58 = G_XOR
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #57 = G_OR
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #56 = G_AND
  { 4, &RISCVDescs.OperandInfo[46] },  // Inst #55 = G_UDIVREM
  { 4, &RISCVDescs.OperandInfo[46] },  // Inst #54 = G_SDIVREM
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #53 = G_UREM
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #52 = G_SREM
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #51 = G_UDIV
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #50 = G_SDIV
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #49 = G_MUL
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #48 = G_SUB
  { 3, &RISCVDescs.OperandInfo[43] },  // Inst #47 = G_ADD
  { 3, &RISCVDescs.OperandInfo[40] },  // Inst #46 = G_ASSERT_ALIGN
  { 3, &RISCVDescs.OperandInfo[40] },  // Inst #45 = G_ASSERT_ZEXT
  { 3, &RISCVDescs.OperandInfo[40] },  // Inst #44 = G_ASSERT_SEXT
  { 1, &RISCVDescs.OperandInfo[1] },  // Inst #43 = JUMP_TABLE_DEBUG_INFO
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #42 = MEMBARRIER
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #41 = ICALL_BRANCH_FUNNEL
  { 3, &RISCVDescs.OperandInfo[37] },  // Inst #40 = PATCHABLE_TYPED_EVENT_CALL
  { 2, &RISCVDescs.OperandInfo[35] },  // Inst #39 = PATCHABLE_EVENT_CALL
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #38 = PATCHABLE_TAIL_CALL
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #37 = PATCHABLE_FUNCTION_EXIT
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #36 = PATCHABLE_RET
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #35 = PATCHABLE_FUNCTION_ENTER
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #34 = PATCHABLE_OP
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #33 = FAULTING_OP
  { 2, &RISCVDescs.OperandInfo[33] },  // Inst #32 = LOCAL_ESCAPE
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #31 = STATEPOINT
  { 3, &RISCVDescs.OperandInfo[30] },  // Inst #30 = PREALLOCATED_ARG
  { 1, &RISCVDescs.OperandInfo[1] },  // Inst #29 = PREALLOCATED_SETUP
  { 1, &RISCVDescs.OperandInfo[29] },  // Inst #28 = LOAD_STACK_GUARD
  { 6, &RISCVDescs.OperandInfo[23] },  // Inst #27 = PATCHPOINT
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #26 = FENTRY_CALL
  { 2, &RISCVDescs.OperandInfo[21] },  // Inst #25 = STACKMAP
  { 2, &RISCVDescs.OperandInfo[19] },  // Inst #24 = ARITH_FENCE
  { 4, &RISCVDescs.OperandInfo[15] },  // Inst #23 = PSEUDO_PROBE
  { 1, &RISCVDescs.OperandInfo[1] },  // Inst #22 = LIFETIME_END
  { 1, &RISCVDescs.OperandInfo[1] },  // Inst #21 = LIFETIME_START
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #20 = BUNDLE
  { 2, &RISCVDescs.OperandInfo[13] },  // Inst #19 = COPY
  { 2, &RISCVDescs.OperandInfo[13] },  // Inst #18 = REG_SEQUENCE
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #17 = DBG_LABEL
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #16 = DBG_PHI
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #15 = DBG_INSTR_REF
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #14 = DBG_VALUE_LIST
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #13 = DBG_VALUE
  { 3, &RISCVDescs.OperandInfo[2] },  // Inst #12 = COPY_TO_REGCLASS
  { 4, &RISCVDescs.OperandInfo[9] },  // Inst #11 = SUBREG_TO_REG
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #10 = IMPLICIT_DEF
  { 4, &RISCVDescs.OperandInfo[5] },  // Inst #9 = INSERT_SUBREG
  { 3, &RISCVDescs.OperandInfo[2] },  // Inst #8 = EXTRACT_SUBREG
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #7 = KILL
  { 1, &RISCVDescs.OperandInfo[1] },  // Inst #6 = ANNOTATION_LABEL
  { 1, &RISCVDescs.OperandInfo[1] },  // Inst #5 = GC_LABEL
  { 1, &RISCVDescs.OperandInfo[1] },  // Inst #4 = EH_LABEL
  { 1, &RISCVDescs.OperandInfo[1] },  // Inst #3 = CFI_INSTRUCTION
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #2 = INLINEASM_BR
  { 0, &RISCVDescs.OperandInfo[1] },  // Inst #1 = INLINEASM
  { 1, &RISCVDescs.OperandInfo[0] },  // Inst #0 = PHI
  }, {
    /* 0 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1 */
    /* 1 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 2 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 5 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 9 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 13 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 15 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 19 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 21 */ { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 23 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 29 */ { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 },
    /* 30 */ { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 33 */ { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 35 */ { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 37 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { 0, 0|(1<<MCOI_LookupPtrRegClass), MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 40 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 43 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 46 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 50 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 51 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 53 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 56 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 58 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 62 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 64 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 67 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 72 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 77 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 82 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 86 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 89 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 93 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 96 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 99 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 103 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 107 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 112 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 116 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 119 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 },
    /* 123 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 },
    /* 126 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 130 */ { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 },
    /* 132 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_2, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_IMM_0, 0 },
    /* 136 */ { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_0, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 }, { -1, 0, MCOI_OPERAND_GENERIC_1, 0 },
    /* 140 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 143 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 146 */ { RISCV_GPRJALRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 148 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 152 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 157 */ { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 158 */ { RISCV_GPRJALRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 160 */ { RISCV_GPRJALRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 161 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 163 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 170 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 177 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 183 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 189 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 195 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 198 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 201 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 204 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 208 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 212 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 216 */ { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 220 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 224 */ { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 228 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 231 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 239 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 246 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 253 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_IMMEDIATE, 0 },
    /* 255 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 258 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 261 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 264 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 267 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 270 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 273 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 276 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 279 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 282 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 283 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 284 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 285 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 286 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 287 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 290 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 292 */ { RISCV_GPRTCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 293 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 300 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 308 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 315 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 323 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 330 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 338 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 345 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 353 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 360 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 368 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 375 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 383 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 390 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 398 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 405 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 413 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 417 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 425 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 434 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 442 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 451 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 459 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 468 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 476 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 485 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 493 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 502 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 510 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 519 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 527 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 536 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 544 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 553 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 560 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 567 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 574 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 581 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 588 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 595 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 602 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 609 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 616 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 623 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 630 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 637 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 644 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 652 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 659 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 667 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 674 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 682 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 689 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 697 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 704 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 712 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 719 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 727 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 734 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 742 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 749 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 757 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 764 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 772 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 779 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 787 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 794 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 802 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 809 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 817 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 823 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 829 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 835 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 841 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 847 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 853 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 859 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 865 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 871 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 877 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 884 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 891 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 898 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 905 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 912 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 919 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 926 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 933 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 939 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 945 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 951 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 957 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 961 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 966 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 972 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 978 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 984 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 990 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 996 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1002 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1008 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1014 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1020 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1026 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1032 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1038 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1044 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1050 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1056 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1062 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1068 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1074 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1080 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1086 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1092 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1098 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1104 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1110 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1116 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1122 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1128 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1134 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1140 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1146 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1152 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1158 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1164 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1170 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1176 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1182 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1188 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1194 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1200 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1206 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1212 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1218 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1224 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1230 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1236 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1242 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1248 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1254 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1260 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1266 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1272 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1278 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1284 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1290 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1296 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1303 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1310 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1317 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1324 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1331 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1338 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1345 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1352 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1359 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1365 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1371 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1377 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1383 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1390 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1397 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1404 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1411 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1418 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1425 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1432 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1439 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1446 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1452 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1458 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1464 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1470 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1477 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1484 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1491 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1498 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1504 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1510 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1516 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1522 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1529 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1536 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1543 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1550 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1557 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1564 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1571 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1578 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1584 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1590 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1596 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1602 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1608 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1614 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1620 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1626 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1633 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1640 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1647 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1654 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1661 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1668 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1675 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1682 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1688 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1694 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1700 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1706 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1713 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1720 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1727 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1734 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1741 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1748 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1755 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1762 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1768 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1774 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1780 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1786 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1792 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1798 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1804 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1810 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1816 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1822 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1828 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1834 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1840 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1846 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1852 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1858 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1864 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1870 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1876 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1882 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1890 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1899 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1907 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1916 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1924 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1933 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1941 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1950 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1958 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1967 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1975 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1984 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 1992 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2001 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2009 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2018 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2026 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2035 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2043 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2052 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2060 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2069 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2077 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2086 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2093 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2101 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2108 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2116 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2123 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2131 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2138 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2146 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2154 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2163 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2171 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2180 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2188 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2197 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2205 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2214 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2222 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2231 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2239 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2248 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2256 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2265 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2273 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2282 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2290 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2299 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2307 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2316 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2324 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2333 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2341 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2350 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2357 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2365 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2372 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2380 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2387 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2395 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2402 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2410 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2417 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2425 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2432 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2440 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2447 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2455 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2462 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2470 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2477 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2485 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2492 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2500 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2507 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2515 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2522 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2530 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2537 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2544 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2551 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2558 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2565 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2572 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2579 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2586 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2593 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2600 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2607 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2614 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2617 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2620 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2623 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2626 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2629 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2632 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2635 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2638 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2641 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2644 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2647 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2650 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2655 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2660 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2665 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2670 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2675 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2680 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2685 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2690 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2695 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2700 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2705 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2710 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2716 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2722 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2728 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2734 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2740 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2746 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2752 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2758 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2764 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2770 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2776 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2782 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2789 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2797 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2804 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2812 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2819 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2827 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2834 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2842 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2848 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2855 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2861 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2868 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2874 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2881 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2887 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2894 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2902 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2911 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2919 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2928 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2936 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2945 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2953 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2962 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2969 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2977 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2984 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2992 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 2999 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3007 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3015 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3024 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3032 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3041 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3049 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3058 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3065 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3073 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3080 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3088 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3095 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3103 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3110 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3118 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3125 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3133 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3140 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3148 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3155 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3163 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3170 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3178 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3185 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3193 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3200 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3208 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3215 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3223 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3230 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3238 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3246 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3255 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3263 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3272 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3280 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3289 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3297 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3306 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3314 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3323 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3331 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3340 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3348 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3357 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3365 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3374 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3382 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3391 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3399 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3408 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3416 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3425 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3433 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3442 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3450 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3457 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3465 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3474 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3482 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3489 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3497 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3506 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3514 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3521 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3527 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3534 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3540 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3547 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3553 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3560 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3568 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3577 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3585 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3594 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3602 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3611 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3619 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3628 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3635 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3642 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3649 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3656 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3664 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3673 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3681 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3690 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3698 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3707 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3715 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3724 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3729 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3735 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3740 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3746 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3751 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3757 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3762 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3768 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3774 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3781 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3787 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3794 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3801 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3809 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3816 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3824 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3831 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3839 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3846 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3854 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3860 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3867 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3873 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3880 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3886 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3893 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3899 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3906 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3913 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3921 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3928 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3936 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3943 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3951 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3958 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3966 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3973 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3981 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3988 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 3996 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4003 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4011 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4018 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4026 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4033 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4041 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4048 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4056 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4063 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4071 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4078 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4086 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4093 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4101 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4108 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4116 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4123 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4131 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4138 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4146 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4153 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4161 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4168 */ { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4176 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4183 */ { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4191 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4198 */ { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4206 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4213 */ { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4221 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4228 */ { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4236 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4243 */ { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4251 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4258 */ { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4266 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4273 */ { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4281 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4288 */ { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4296 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4303 */ { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4311 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4318 */ { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4326 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4333 */ { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4341 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4348 */ { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4356 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4363 */ { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4371 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4378 */ { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4386 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4393 */ { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4401 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4408 */ { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4416 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4423 */ { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4431 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4438 */ { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4446 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4453 */ { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4461 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4468 */ { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4476 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4483 */ { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4491 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4498 */ { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4506 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4513 */ { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4521 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4528 */ { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4536 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4543 */ { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4551 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4558 */ { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4566 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4573 */ { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4581 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4588 */ { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4596 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4603 */ { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4611 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4618 */ { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4626 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4633 */ { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4641 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4648 */ { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4656 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4663 */ { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4671 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4678 */ { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4686 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4693 */ { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4701 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4708 */ { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4716 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4723 */ { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4731 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4738 */ { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4746 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4753 */ { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4761 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4768 */ { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4776 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4783 */ { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4791 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4798 */ { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4806 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4813 */ { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4821 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4828 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4836 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4843 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4851 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4858 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4866 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4873 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4881 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4888 */ { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4896 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4903 */ { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4911 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4918 */ { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4926 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4932 */ { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4939 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4945 */ { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4952 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4958 */ { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4965 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4972 */ { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4980 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4987 */ { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 4995 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5001 */ { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5008 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5014 */ { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5021 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5028 */ { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5036 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5043 */ { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5051 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5057 */ { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5064 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5070 */ { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5077 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5084 */ { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5092 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5098 */ { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5105 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5112 */ { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5120 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5126 */ { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5133 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5140 */ { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5148 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5154 */ { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5161 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5168 */ { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5176 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5182 */ { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5189 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5196 */ { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5204 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5211 */ { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5219 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5226 */ { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN2M4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5234 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5241 */ { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5249 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5256 */ { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN3M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5264 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5271 */ { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5279 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5286 */ { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN4M2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5294 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5301 */ { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN5M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5309 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5316 */ { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN6M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5324 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5331 */ { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN7M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5339 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5346 */ { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRN8M1NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5354 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5361 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5369 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5376 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5384 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5391 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5399 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5406 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5414 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5420 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5426 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5432 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5438 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5443 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5448 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5453 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5458 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5464 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5470 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5476 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5482 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5487 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5492 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5497 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5502 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5508 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5514 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5520 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5526 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5531 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5536 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5541 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5546 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5551 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5554 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5559 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5566 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5571 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5578 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5583 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5590 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5595 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5602 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5607 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5614 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5619 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5626 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5631 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5638 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5643 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5650 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5655 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5662 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5667 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5674 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5679 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5686 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5691 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5698 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5705 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5712 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5719 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5726 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5731 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5738 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5745 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5752 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5759 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5764 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5771 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5778 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5785 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5792 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5_PLUS1, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 5796 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 5799 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 5803 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 5808 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5813 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5819 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5825 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5831 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5837 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5843 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5849 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5855 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5861 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5864 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5872 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5881 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5889 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5898 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5906 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5915 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5923 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5932 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5940 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5949 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5957 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5966 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5974 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5983 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 5991 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6000 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6008 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6017 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6025 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6034 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6041 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6049 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6056 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6064 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6071 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6079 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6087 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6094 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6102 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6109 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6117 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6124 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6132 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6139 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6147 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6154 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6162 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6169 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6177 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6184 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6191 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6198 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6200 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6202 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6204 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6206 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6208 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6210 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6212 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6214 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6216 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6218 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 6220 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6227 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6235 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6242 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6250 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6257 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6265 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6272 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6280 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6287 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6295 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6302 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6310 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6317 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6325 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6332 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6340 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6347 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6355 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6362 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6370 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6377 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6385 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6392 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6400 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6407 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6415 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6422 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6430 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6437 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6445 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6452 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6460 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6467 */ { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRNoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6475 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6482 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6490 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6497 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6505 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6512 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6520 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6524 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6529 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6533 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6538 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6542 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6547 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6551 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6556 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_VTYPEI10, 0 },
    /* 6559 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_VTYPEI11, 0 },
    /* 6562 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_VTYPEI11, 0 },
    /* 6565 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6571 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6578 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6586 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6594 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6602 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6607 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6613 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6618 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6624 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6629 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6635 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6640 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6646 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6651 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6657 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6662 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6668 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6673 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6679 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6684 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6690 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6695 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6701 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6706 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6712 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6717 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6723 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6728 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6734 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6739 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6745 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6750 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6756 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6761 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6767 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6772 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6778 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6783 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6789 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6794 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6800 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6805 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6811 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6816 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6822 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6827 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6833 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6838 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6844 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6849 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6855 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6860 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6866 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6871 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6877 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6882 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6888 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6893 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6899 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6904 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6910 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6915 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6921 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6926 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6932 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6937 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6943 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6948 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6954 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6959 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6965 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6970 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6976 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6981 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6987 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6992 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 6998 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7003 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7009 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7014 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7020 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7025 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7031 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7036 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7042 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7047 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7053 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7058 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7064 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7069 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7075 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7080 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7086 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7091 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7097 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7102 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7108 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7113 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7119 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7124 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7130 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7135 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7141 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7146 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7152 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7157 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7163 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7168 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7174 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7179 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7185 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7190 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7196 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7201 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7207 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7212 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7218 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7223 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7229 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7234 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7240 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7245 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7251 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7256 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7262 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7267 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7273 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7278 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7284 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7289 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7295 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7300 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7306 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7310 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7315 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7319 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7324 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7328 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7333 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7337 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7342 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7346 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7351 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7355 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7360 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7364 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7369 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7373 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7378 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7382 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7387 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7391 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7396 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7400 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7405 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7413 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7422 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7430 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7439 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7447 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7456 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7461 */ { RISCV_VRN2M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7467 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7472 */ { RISCV_VRN2M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7478 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7483 */ { RISCV_VRN2M4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7489 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7494 */ { RISCV_VRN3M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7500 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7505 */ { RISCV_VRN3M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7511 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7516 */ { RISCV_VRN4M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7522 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7527 */ { RISCV_VRN4M2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7533 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7538 */ { RISCV_VRN5M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7544 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7549 */ { RISCV_VRN6M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7555 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7560 */ { RISCV_VRN7M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7566 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7571 */ { RISCV_VRN8M1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7577 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7584 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7592 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7599 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7607 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7614 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7622 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7629 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7637 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7644 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7652 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7659 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7667 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7674 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7682 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7689 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7697 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7704 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7712 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7719 */ { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM2NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7727 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7734 */ { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM4NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7742 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7749 */ { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRM8NoV0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, RISCVOp_OPERAND_AVL, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 7757 */ { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7763 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7769 */ { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7775 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7781 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7787 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7793 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7799 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7802 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7805 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 7807 */ { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 7808 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 7811 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 },
    /* 7815 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_RVKRNUM, 0 },
    /* 7818 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7821 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7825 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7829 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM20, 0 },
    /* 7831 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMMLOG2XLEN, 0 },
    /* 7834 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 },
    /* 7835 */ { -1, 0, RISCVOp_OPERAND_UIMM8_GE32, 0 },
    /* 7836 */ { RISCV_SR07RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_SR07RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7838 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM12, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7841 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM12, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 7844 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 7848 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7852 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM6, 0 },
    /* 7855 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 },
    /* 7858 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 7862 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 7865 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 7869 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 7872 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 7875 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 7880 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 },
    /* 7884 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 7888 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, CONSTRAINT_MCOI_TIED_TO(1) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7892 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 7897 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 7901 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7905 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM6, 0 },
    /* 7909 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 },
    /* 7912 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM4, 0 },
    /* 7915 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7918 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM6_NONZERO, 0 },
    /* 7921 */ { RISCV_SPRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_SPRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM10_LSB0000_NONZERO, 0 },
    /* 7924 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_SPRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM10_LSB00_NONZERO, 0 },
    /* 7927 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM6, 0 },
    /* 7930 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_ZERO, 0 },
    /* 7933 */ { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_ZERO, 0 },
    /* 7936 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7939 */ { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7942 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM6, 0 },
    /* 7945 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 7947 */ { RISCV_FPR64CRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM8_LSB000, 0 },
    /* 7950 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_SPRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM9_LSB000, 0 },
    /* 7953 */ { RISCV_FPR32CRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7_LSB00, 0 },
    /* 7956 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_SPRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM8_LSB00, 0 },
    /* 7959 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7960 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 },
    /* 7963 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM8_LSB000, 0 },
    /* 7966 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_SPRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM9_LSB000, 0 },
    /* 7969 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2_LSB0, 0 },
    /* 7972 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM6, 0 },
    /* 7974 */ { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM6, 0 },
    /* 7976 */ { RISCV_GPRNoX0X2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_CLUI_IMM, 0 },
    /* 7978 */ { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_CLUI_IMM, 0 },
    /* 7980 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7_LSB00, 0 },
    /* 7983 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_SPRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM8_LSB00, 0 },
    /* 7986 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7988 */ { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 7990 */ { -1, 0, RISCVOp_OPERAND_SIMM6_NONZERO, 0 },
    /* 7991 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 7993 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_SPRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM9_LSB000, 0 },
    /* 7996 */ { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRNoX0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_UIMMLOG2XLEN_NONZERO, 0 },
    /* 7999 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) },
    /* 8001 */ { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRX0RegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_UIMMLOG2XLEN_NONZERO, 0 },
    /* 8004 */ { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRCRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_UIMMLOG2XLEN_NONZERO, 0 },
    /* 8007 */ { RISCV_GPRX5RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8008 */ { RISCV_GPRX1RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8009 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_SPRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM8_LSB00, 0 },
    /* 8012 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8014 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8016 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8018 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8020 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8022 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8024 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8027 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8030 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8033 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8036 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8039 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8042 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8045 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8048 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8051 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8054 */ { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8057 */ { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8060 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8063 */ { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8066 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8069 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8072 */ { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8075 */ { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8078 */ { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8081 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8084 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8087 */ { -1, 0, RISCVOp_OPERAND_UIMM4, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM4, 0 },
    /* 8089 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 8092 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 8095 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8097 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8099 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8101 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 8104 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8109 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8114 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8119 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8124 */ { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8129 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8134 */ { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8139 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8142 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8145 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8148 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8151 */ { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8154 */ { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8157 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8159 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8161 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8163 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8166 */ { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8169 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8172 */ { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRPairRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8175 */ { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF16RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8178 */ { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRF32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_UNKNOWN, 0 },
    /* 8181 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 },
    /* 8183 */ { -1, 0, RISCVOp_OPERAND_UIMM7, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 8188 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8193 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 8197 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM6, 0 },
    /* 8201 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM8, 0 },
    /* 8205 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 8208 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 8213 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM4, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8217 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 8222 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 },
    /* 8226 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 8231 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 8234 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8240 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8247 */ { -1, 0, RISCVOp_OPERAND_UIMM7, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM3, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12, 0 },
    /* 8252 */ { -1, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM20, 0 },
    /* 8255 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, MCOI_OPERAND_PCREL, 0 },
    /* 8257 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM12_LSB00000, 0 },
    /* 8259 */ { RISCV_GPRX1X5RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8260 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8265 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8270 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMMLOG2XLEN, 0 }, { -1, 0, RISCVOp_OPERAND_UIMMLOG2XLEN, 0 },
    /* 8274 */ { RISCV_FPR64RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 },
    /* 8278 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 },
    /* 8282 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(1) }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 },
    /* 8287 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7, 0 },
    /* 8292 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 },
    /* 8297 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM7, 0 },
    /* 8302 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8306 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8310 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8314 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8316 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 },
    /* 8319 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8322 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8325 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8328 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8332 */ { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8336 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 },
    /* 8340 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 },
    /* 8344 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 },
    /* 8348 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8352 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8356 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8360 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM1, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8365 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 },
    /* 8369 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 },
    /* 8373 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 },
    /* 8378 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8382 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8387 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8391 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8395 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8400 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8404 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8408 */ { -1, 0, RISCVOp_OPERAND_UIMM2, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8412 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8416 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8421 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8426 */ { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8428 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8431 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8433 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8436 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8440 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8444 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_FPR32RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8449 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8452 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8454 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 },
    /* 8456 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 },
    /* 8458 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 },
    /* 8460 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 },
    /* 8462 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8465 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8469 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_MEMORY, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8473 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8478 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 },
    /* 8481 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8485 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8488 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8492 */ { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM2RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8494 */ { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM4RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8496 */ { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRM8RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8498 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_TIED_TO(0) }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8501 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_SIMM5, 0 },
    /* 8503 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8505 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8507 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, CONSTRAINT_MCOI_EARLY_CLOBBER }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8511 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM6, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
    /* 8515 */ { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_GPRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_VTYPEI11, 0 },
    /* 8518 */ { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { RISCV_VRRegClassID, 0, MCOI_OPERAND_REGISTER, 0 }, { -1, 0, RISCVOp_OPERAND_UIMM5, 0 }, { RISCV_VMV0RegClassID, 0, MCOI_OPERAND_REGISTER, 0 },
  }, {
    /* 0 */
    /* 0 */ RISCV_X2, RISCV_X2,
    /* 2 */ RISCV_X5, RISCV_X1, RISCV_X6, RISCV_X7, RISCV_X28, RISCV_X29, RISCV_X30, RISCV_X31,
    /* 10 */ RISCV_X6, RISCV_X7, RISCV_X28, RISCV_X29, RISCV_X30, RISCV_X31,
    /* 16 */ RISCV_X1,
    /* 17 */ RISCV_VL,
    /* 18 */ RISCV_X2,
    /* 19 */ RISCV_X10, RISCV_X10,
    /* 21 */ RISCV_VXSAT,
    /* 22 */ RISCV_VL, RISCV_VTYPE,
    /* 24 */ RISCV_FFLAGS,
    /* 25 */ RISCV_FRM,
    /* 26 */ RISCV_FRM, RISCV_FRM,
    /* 28 */ RISCV_VXRM,
    /* 29 */ RISCV_X10, RISCV_X11,
    /* 31 */ RISCV_SSP, RISCV_SSP,
    /* 33 */ RISCV_SSP,
    /* 34 */ RISCV_VTYPE, RISCV_VL,
  }
};

#endif // GET_INSTRINFO_MC_DESC

#ifdef GET_COMPUTE_FEATURES
#undef GET_COMPUTE_FEATURES

#endif // GET_COMPUTE_FEATURES

#ifdef GET_AVAILABLE_OPCODE_CHECKER
#undef GET_AVAILABLE_OPCODE_CHECKER

#endif // GET_AVAILABLE_OPCODE_CHECKER

